
/*********************************************************************
 *  made by : Kees Lemmens
 *  version : 0.1
 *  date    : May 1999
 * 
 *  class name  : JsepView
 *  function    : Main and toplevel part of the Sepran Viewer
 *
 * The toplevel window contains 3 regions : header, canvas and control,
 * each of them implemented in a separate class.
 * 
 * modified : July 1999 KL
 * Complete rewrite of scrolling and scaling classes
 *
 * modified : July 1999 KL 
 * Added Big Endian data reader
 *
 *********************************************************************/

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintStream;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.PrintJob;
import java.awt.Graphics;
import java.util.Properties;
import java.awt.event.*;

public class JsepView extends Frame     // use for normal applications
implements Runnable,WindowListener
{
   // Some predefined settings :

   static final String TITLE   = "Java Sepran Plotfile Viewer";
   static final String VERSION = "version : 1.3";
   static final String AUTHOR  = "(c) Kees Lemmens";
   static final String DATE    = "April 2000";

   String helppath = "file:/opt/java/";
   String helpfile = helppath + "jsepview.1";
   int    helpmode = FileViewer.USEURL;
   byte   recordType=RecordReader.AUTO;
   
   // And the class variables + methods :
   ViewHeader   header;
   ViewCanvas   canvas;
   ViewControl  control;
   ScrollCanvas viewport;
   Dimension    canvasSize;
   String       argv[];
   String       filelist[]; // storage for filename(s)
   int          current;
   Thread       thread=null;
   Player       player=null;
   boolean      playmode=false;
   static int nrOfThreads=0;
   
   public JsepView(String argv[]) // constructor
   {
      byte nrOpts=0;
      this.argv  = argv; // save it for i.e. clone 
      
      while(argv.length - nrOpts > 0 && argv[nrOpts].charAt(0) == '-')
      {
	 if     (argv[nrOpts].equals("-j"))
	   recordType=RecordReader.BINARY;
	 // JAVA BINARY format
	 
	 else if(argv[nrOpts].equals("-a"))
	   recordType=RecordReader.ASCII;
	 // ASCII format created by Sepran
	 
	 else if(argv[nrOpts].equals("-l"))
	   recordType=RecordReader.FORTRANLE;
	 // FORTRAN BINARY created by Sepran (little endian)
	 
	 else if(argv[nrOpts].equals("-n"))
	   recordType=RecordReader.NETBINARY;
	 // JAVA BINARY using a URL (http, ftp etc)

	 else if(argv[nrOpts].equals("-m"))
	    helpfile=argv[++nrOpts];
	 // Overrides the preset manual page path
	 
	 else  usage(); // default
	 
	 nrOpts ++;
      }
      
      //provide one extra string in case of no args :

      filelist = new String[argv.length - nrOpts];
      for(int n=0;n<argv.length - nrOpts;n++)
	filelist[n] = argv[n + nrOpts];
      
      init();
      
      if(argv.length - nrOpts > 0)
	start(filelist[current]);
      else
      {
	 filelist = new String[1];
	 start();
      }
   }
   
   public void usage()
   {
      System.err.print
	("\n" +
	 TITLE + ", " + VERSION + "; " + AUTHOR + ", " + DATE +
	 "\n\n" +
	 "Usage  : jsepview [-j|f|n] [plotfile(s)]\n" +
	 "Options: -l Force Fortran binary format (little endian)\n" +
	 "         -a Force ASCII file format from Sepran\n" +
	 "         -j Force Java portable binary format\n" +
	 "         -n Use Network access (only Java portable binaries)\n" +
	 "         -m <path> Override the precompiled manual path\n" +
	 "            (currently set to " + helpfile + ")\n" +
	 "\n" +
	 "Default format is autodetect which is somewhat slower.\n ");
      System.exit(0);
   }
   
   public void init()
   {
      current = 0;
      
      thread = new Thread(this); // necessary for cloning (new window)
      nrOfThreads++; // increase thread counter
      
      header   = new ViewHeader(this);

      canvasSize = new Dimension(400,400);
      viewport = new ScrollCanvas(this,canvasSize);
      canvas   = viewport.canvas;

      control  = new ViewControl(this);

      add(header,"North");
      add(viewport,"Center");
      add(control,"South");

      setSize(450,450); // need it on Solaris to get a proper window ?

      addWindowListener(this);
      
      setTitle(TITLE);
      pack(); // compute layout based on children size
      
   }

   public void run() // obligatory for Runnable
   {
      start();
   }
   
   public void start()
   {
      setVisible(true); // show the top widget 
      canvas.openPlotSelector();
   }

   public void start(String file)
   {
      setVisible(true); // show the top widget 
      header.setCurrentName(filelist[current]);
   }

   public void stop()
   {
      setVisible(false); // hide the top widget (window)
      
      nrOfThreads--; // decrease number of active threads
      
      if(nrOfThreads < 1)
	System.exit(0);
      else
	dispose();  // remove top widget from system joblist
   }
   
   public void newWindow()
   {
      JsepView s = new JsepView(argv);
   }
   
   public void setPrevious()
   {
      if(current>0)
      {
	 current--;
	 header.setCurrentName(filelist[current]);
	 canvas.repaint();
      }
   }

   public void togglePlay()
   {
      playmode = !playmode; // toggle
      
      if(playmode == true)
      {
	 control.but[control.PLAY].setBackground(Color.red);
	 player = new Player(this);
      }
      else if(player != null)
      {
	 control.but[control.PLAY].setBackground(Color.white);
	 player.stop();
      }
   }
   
   public void setNext()
   {
      if(current<filelist.length-1)
      {
	 current++;
	 header.setCurrentName(filelist[current]);
	 canvas.repaint();
      }
   }
   
   public Frame getParentFrame(Component component)
   {
      while(! (component instanceof Frame))
	component = component.getParent();
      return (Frame)component;
   }
   
   public String getCurrent()
   {
      return filelist[current];   
   }

   // Now some more general but very useful routines for printing, executing
   // commands etc. :
   
   public void printMainWindow(Properties printprefs)
   {
      printComponent( printprefs, this);
   }
   
   public void printComponent(Properties printprefs, Component printable)
   {
      PrintJob job = this.getToolkit().
	getPrintJob(this,"JsepView",printprefs);
      
      if(job != null)
      {
	 Graphics pg = job.getGraphics();

	 // Center the picture on the page :
	 Dimension tsize = printable.getSize();
	 Dimension psize = job.getPageDimension();

	 pg.translate((psize.width  - tsize.width)/2,
		      (psize.height - tsize.height)/2);

	 printable.printAll(pg);
	 pg.dispose();
	 job.end();
      }
      
      System.err.println("Printing window " + printable + " ....");
   }

   public void execCommand(String cmdArray[], PrintStream o)
   {
      if(Debug.LEVEL >= 1)
      {
	 System.err.print("Executing system command : ");
	 for(int n=0;n<cmdArray.length;n++)
	   System.err.print(cmdArray[n] + " ");
	 System.err.println();
      }
      
      try
      {
	 String s;
	 Process           p = Runtime.getRuntime().exec(cmdArray);
	 InputStreamReader i = new InputStreamReader(p.getInputStream());
	 BufferedReader pOut = new BufferedReader(i); // command output
	 
	 while ((s = pOut.readLine()) != null)
	   o.println(s);
	 
	 int exit_value = p.waitFor();
	 System.err.println("Exit status : " + exit_value);
      }
      catch (IOException io)
      {	 new DialogBox("IOException :" + io, this);}
      catch (InterruptedException in)
      {  new DialogBox("Interrupted Exception :" + in, this);}
   }

   // and the Windowevent listeners :
   
   private void removeWindow()
   {
      setVisible(false);
      System.exit(0);
   }

   public void windowClosing(WindowEvent we)     {removeWindow();}
   public void windowActivated(WindowEvent we)   {;}
   public void windowClosed(WindowEvent we)      {;}
   public void windowDeactivated(WindowEvent we) {;}
   public void windowDeiconified(WindowEvent we) {;}
   public void windowIconified(WindowEvent we)   {;}
   public void windowOpened(WindowEvent we)      {;}

   // And a very simplistic but vital main :
   public static void main(String argv[])
   {
      JsepView s = new JsepView(argv);
   }
}
