      subroutine ercomnor1 ( kmesh, norm, weight, coor, &
                             normaldr, kmeshn, kmeshs, isurnr )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.7  date 20-09-2006 Minor change format
!        version  2.6  date 05-03-2005 Fortran 90
!        version  2.5  date 05-05-2004 Debug statements
!        version  2.4  date 13-12-2003 Argument acos
!        version  2.3  date 22-01-2003 Correction empty surfaces
!        version  2.2  date 26-08-2002 Replace error message by warning
!        version  2.1  date 01-08-2002 Debug statements
!        version  2.0  date 18-03-2002 Remove elssurfs
!        version  1.1  date 10-10-2001 Adapt weights
!        version  1.0  date 06-10-2000
!
!   copyright (c) 2000-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the averaged normals in all points on the surface isurnr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     average
!     normal_vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision norm(3,*), weight(*), coor(3,*)
      integer kmesh(*), normaldr, kmeshn(*), kmeshs(8,*), isurnr

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     isurnr     i    Surface sequence number
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     kmeshn     i    KMESH part n, contains nodal point numbers of the surfaces
!     kmeshs     i    Kmesh part s2, contains information about the various
!                     surfaces. See the PG 13.1
!     norm       o    Array of length ndim * npoint containing the normal in
!                     the nodal points
!     normaldr   i    If 1 the outward directed normal is required, if -1 the
!                     inward directed normal
!     weight          Help array of length npoint to store the weights
!                     for the averaging
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, istart, nsurfs, inelem, &
              inpelm, nodp, nodprv, nodnxt, ielem, inboun, nodp1, &
              nodp2, iprev, inext, ishape
      double precision t1(3), t2(3), locnorm(3), h, lt1, lt2, alpha
      logical linear, odd, debug

!     alpha          multiplication factor
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     h              Help variable to compute the length of a vector
!     i              Counting variable
!     ielem          Element number
!     inboun         Number of nodes opn boundary of element
!     inelem         Number of elements in a specific element group
!     inext          relative node number in element of next node
!     inpelm         number of points per element
!     iprev          relative node number in element of previous node
!     ishape         Shape number of element
!     istart         First position used at a part
!     k              Counting variable
!     linear         If true the elements are linear
!     locnorm        Contains the norm in a node
!     lt1            Help variable to compute an angle
!     lt2            Help variable to compute an angle
!     nodnxt         Next nodal point at boundary
!     nodp           Nodal point sequence number
!     nodp1          Nodal point sequence number for computation of tang vector
!     nodp2          Nodal point sequence number for computation of tang vector
!     nodprv         Previous nodal point at boundary
!     nsurfs         Number of surfaces in mesh
!     odd            If true an odd node in the element has been found
!     t1             First tangential vector
!     t2             Second tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    2337   Element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays norms and weights
!     For isurnr := isrf1 (1) isrf2
!        For all elements ielem in surface isurnr
!           For all nodes nodp in ielem do
!              Compute the normal in the point using the right-handed screw
!              direction and the direction of the elements
!              Add normal to array norms
!              Add 1 to array weights
!     Divide all entries in norms by corresponding weights
!     Normalize the entries in norms
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ercomnor1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercomnor1'

      end if
      if ( ierror/=0 ) go to 1000

      nsurfs = kmesh(12)
      inelem = kmeshn(nsurfs+isurnr)
      inpelm = kmeshn(isurnr)
      istart = 2*nsurfs
      do i = 1, isurnr-1
         istart = istart+kmeshn(i)*kmeshn(nsurfs+i)
      end do
      ishape = kmeshs(2,isurnr)
      if ( debug ) &
         write(irefwr,*) 'nsurfs, inelem, inpelm, ishape ', &
                          nsurfs, inelem, inpelm, ishape

      linear = ishape==3 .or. ishape==5 .or. ishape>=9
      alpha = 1d0
      inboun = inpelm
      if ( ishape>=6 ) inboun = inboun-1
      if ( ishape==8 ) then

!     --- Shape number has not yet been implemented

         call eropen ( 'ercomnor1' )
         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         call erclos ( 'ercomnor1' )
         go to 1000

      end if

      if ( debug ) then

!     --- debug = true, debug statements

         call prinin1 ( kmeshn(istart+1), inelem, inpelm, 'kmeshn' )

      end if

!     --- For all elements ielem in surface isurnr

      do ielem = 1, inelem

!     --- For all nodes nodp in ielem do

         do i = 1, inpelm

            nodp = kmeshn(istart+i)
            if ( nodp<=0 ) go to 1000
            nodp1 = nodp
            nodp2 = nodp

!           --- Determine which nodes must be used for tangential vectors

            odd = mod(i,2)==1
            if ( i<=inboun .and. (linear.or.odd) ) then

!           --- Vertices

               inext = mod(i,inpelm)+1
               iprev = i-1
               if ( iprev==0 ) iprev = inboun

            else if ( i<=inboun ) then

!           --- Mid-side points

               inext = i+2
               iprev = i-2
               if ( inext>inboun ) inext = 2
               if ( iprev==0 ) iprev = inboun

            else if ( ishape==6 ) then

!           --- centroid of shape 6

               nodp1 = kmeshn(istart+4)
               iprev = 8
               inext = 6
               nodp2 = kmeshn(istart+2)

            else if ( ishape==7 ) then

!           --- centroid of shape 7

               iprev = 7
               nodp1 = kmeshn(istart+4)
               inext = 7
               nodp2 = kmeshn(istart+2)

            else if ( ishape==9 ) then

!           --- centroid of shape 9

               iprev = 1
               nodp1 = kmeshn(istart+3)
               inext = 4
               nodp2 = kmeshn(istart+2)

            else if ( ishape==10 ) then

!           --- centroid of shape 10

               iprev = 1
               nodp1 = kmeshn(istart+2)
               inext = 3
               nodp2 = kmeshn(istart+2)

            end if
            nodnxt = kmeshn(istart+inext)
            nodprv = kmeshn(istart+iprev)

!           --- Compute the normal in the point using the right-handed screw
!               direction and the direction of the elements

            do k = 1, 3
               t1(k) = coor(k,nodp1)-coor(k,nodprv)
               t2(k) = coor(k,nodnxt)-coor(k,nodp2)
            end do
            call eloutr ( t1, t2, locnorm )
            h = sqrt(locnorm(1)**2+locnorm(2)**2+locnorm(3)**2)
            if ( h<=0d0 ) then

!           --- h<=0

               call errint ( nodp, 1 )
               call errint ( ielem, 2 )
               call errint ( isurnr, 3 )
               call errwar ( 2337, 3, 0, 0 )
               h = 1d0

            end if
            h = normaldr/h
            do k = 1, 3
               locnorm(k) = locnorm(k)*h
            end do

!           --- Add normal to array norms
!               Add alpha to array weight

            if ( ishape==3 .or. ishape==5 ) then

!           --- Linear element, alpha is angle

               lt1 = sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
               lt2 = sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
               alpha = (-t1(1)*t2(1)-t1(2)*t2(2)-t1(3)*t2(3))/(lt1*lt2)

!              --- To avoid errors due to rounding we restrict alpha to
!                  the correct range (-1,1)

               alpha = max(alpha,-1d0)
               alpha = min(alpha,1d0)
               alpha = acos(alpha)

            end if

            do k = 1, 3
               norm(k,nodp) = norm(k,nodp)+locnorm(k)*alpha
            end do
            weight(nodp) = weight(nodp)+alpha
         end do

         istart = istart+inpelm

      end do

1000  call erclos ( 'ercomnor1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercomnor1'

      end if

      end
