      subroutine ernodssurfs ( kmesh, nsurfaces, isurfaces, kmeshn, &
                               nodes, nnodes, elemnodes )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 27-05-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find starting addresses of elements in free surface
!     corresponding to nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     node
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), nsurfaces, isurfaces(nsurfaces), kmeshn(*), &
              nnodes, nodes(nnodes), elemnodes(*)

!     elemnodes      o    Contains the elements in the surface corresponding
!                         to the nodes in array nodes
!                         Positions 1 to nnodes contain the starting addresses
!                         (istart) for each node
!                         position istart of node inode contains the number of
!                         elements connected to node inode
!                         The next positions contain the element numbers
!     isurfaces      i    Array containing the surfaces from which the
!                         normals must be computed
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers in the surface
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, inpelm, inelem, nsurfs, jsurnr, isurnr, i, inode, &
              numelms, npoint, error, ielem, j, jstart
      integer, allocatable :: iwork(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ielem          Element number
!     inelem         Number of elements in surface
!     inode          Node sequence number
!     inpelm         number of points per element
!     istart         Starting positiont
!     isurnr         Surface sequence number
!     iwork          integer work array
!     j              Counting variable
!     jstart         starting position
!     jsurnr         Surface sequence number
!     npoint         Number of nodal points
!     nsurfs         number of surfaces
!     numelms        Number of elements connected to node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork of length npoint
!     For all surfaces isurnr in free surface do
!        for all elements ielem in isurnr do
!           for all nodes inode in ielem do
!              iwork(inode) = iwork(inode)+1
!     Fill first part of array elemnodes
!     Set starting addresses in iwork
!     Fill second part of elemnodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ernodssurfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ernodssurfs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      npoint = kmesh(8)
      nsurfs = kmesh(12)

!     --- Allocate temporary space

      allocate( iwork(npoint), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( npoint, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- Clear array iwork of length npoint

      do i = 1, nnodes
         iwork(nodes(i)) = 0
      end do  ! i = 1, nnodes

!     --- For all surfaces isurnr in free surface do

      do jsurnr = 1, nsurfaces

         isurnr = isurfaces(jsurnr)
         call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                           isurnr )

!        --- for all elements ielem in isurnr do
!               for all nodes inode in ielem do
!                  iwork(inode) = iwork(inode)+1

         do i = istart+1, inelem*inpelm+istart

            inode = kmeshn(i)
            if ( inode<=0 ) go to 100
            iwork(inode) = iwork(inode)+1

         end do  ! i = istart+1, inelem*inpelm+istart

100   end do  ! jsurnr = 1, nsurfaces

!     --- Fill first part of array elemnodes

      elemnodes(1:nnodes) = 0
      istart = nnodes+1
      do i = 1, nnodes
         inode = nodes(i)
         numelms = iwork(inode)
         elemnodes(i) = istart
         elemnodes(istart) = numelms
         istart = istart+numelms+1
      end do  ! i = 1, nnodes

!     --- Set starting addresses in iwork

      do i = 1, nnodes
         iwork(nodes(i)) = elemnodes(i)+1
      end do  ! i = 1, nnodes

!     --- Fill second part of elemnodes
!         For all surfaces isurnr in free surface do

      do jsurnr = 1, nsurfaces

         isurnr = isurfaces(jsurnr)
         call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                           isurnr )

!        --- for all elements ielem in isurnr do
!               for all nodes inode in ielem do
!                  Fill start of element in elemnodes
!                  iwork(inode) = iwork(inode)+1

         do ielem = 1, inelem

            do j = 1, inpelm
               inode = kmeshn(istart+j)
               if ( inode<=0 ) go to 200
               jstart = iwork(inode)       ! position in elemnodes
               elemnodes(jstart) = istart  ! start of element in kmeshn
               iwork(inode) = jstart+1     ! next position in elemnodes
            end do  ! j = 1, inpelm
            istart = istart+inpelm

         end do  ! ielem = 1, inelem

200   end do  ! jsurnr = 1, nsurfaces

900   deallocate ( iwork, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  call erclos ( 'ernodssurfs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Contents of array elemnodes'
         call prinnmeshx ( nnodes, elemnodes )
         write(irefwr,*) 'End ernodssurfs'

      end if  ! ( debug )

      end
