      subroutine femeshbf ( ibuffr, buffer, kmesh1, kmesh2, iref, iren )
! ======================================================================
!
!        programmer    Luc Timmermans/Frans van de Vosse
!        version  1.7  date 29-05-2014 Extension of timebefore
!        version  1.6  date 21-11-2012 Long integers
!        version  1.5  date 04-11-2010 Extension parallel
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates a linear element mesh based on the nodes
!     of a spectral element mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cpackagef90'
      include 'SPcommon/cparallelf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1(*), kmesh2(*), iref, iren, ibuffr(*)
      double precision buffer(*)

!     buffer   i/o     Real buffer array in which all large data is stored
!     ibuffr   i/o     Integer buffer array in which all large data is stored
!     iref      i      reference number
!     iren      i      indicates whether to renumber the spectral element
!                      mesh according to the FE renumbering (used when
!                      preconditioning is performed)
!     kmesh1    i      standard sepran array, corresponding to the
!                      spectral element mesh
!     kmesh2    o      standard sepran array, corresponding to the
!                      linear element mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkm1c, ipkm2c, ipkm1i, ipkm2i
      integer leng, icount, ihelp, irenum, ndim, nelgrp, meshtp, &
              nelm, nnpe, nprev, idummy(10), lastps, ipkm2b
      double precision rnnpe, timebefore(2)
      character (len=100) filename
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     filename       Name of file
!     icount         counting variable
!     idummy         dummy array
!     ihelp          help variable
!     ipkm1c         pointer to kmesh1 part c
!     ipkm1i         pointer to kmesh1 part i
!     ipkm2b         pointer to kmesh2 part b
!     ipkm2c         pointer to kmesh2 part c
!     ipkm2i         pointer to kmesh2 part i
!     irenum         indicator for type of renumbering for kmesh2
!     lastps         Last non-blank position in filename
!     leng           help parameter to declare a length of an array
!     meshtp         type number for mesh generation
!     ndim           Dimension of the space.
!     nelgrp         Number of element groups
!     nelm           ?
!     nnpe           Number of points per element
!     nprev          ?
!     rnnpe          Help variable to make nnpe real
!     timebefore     Time at entrance of subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext

!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     FILLPC
!     FMSHTP
!     FNELM
!     FNNPE
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     MESHWRBF       fill the file meshoutput with the mesh
!     MSH071         Read mesh input from file meshoutput
!     MSHRDPARMESH   Read information parallel information about global mesh
!                    from file meshoutput_par.000
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       2   Array is too short
!      75   Array KMESH has not been filled
!    1696   Number of nodes per element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- check input

      call eropen ('femeshbf')
      if ( itime>0 ) call ersettime ( timebefore )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from femeshbf'
         call prinin ( kmesh1, kmesh1(3), 'kmesh1' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- check input

      if ( kmesh1(2)/=100 ) then
         call errint (kmesh1(2), 1)
         call errsub (75, 1, 0, 0)
      end if

      if ( ierror/=0 ) goto 1000

!     --- copy kmesh1 into kmesh2

      do icount = 1, kmesh1(1)
         kmesh2(icount) = kmesh1(icount)
      end do
      kmesh2(38) = 0
      kmesh2(22) = 0

!     --- adjust first part of kmesh2 for linear elements
!         first the maximal numbers of nodes per element

      nelgrp = kmesh1(5)
      ndim = kmesh1(6)
      if ( ndim==1 ) kmesh2(4) = 2
      if ( ndim==2 ) kmesh2(4) = 4
      if ( ndim==3 ) kmesh2(4) = 8
      kmesh2(17) = 0

!     --- fetch kmesh2 b

      ipkm2b = kmesh2(16)
      if( ipkm2b<=0 ) then
         ipkm2b = kmesh2(3) + 1
         kmesh2(3) = kmesh2(3) + nelgrp
         if ( kmesh2(3)>kmesh2(1) ) then
            call errchr ( 'kmesh2', 1 )
            call errint ( kmesh2(3), 1 )
            call errint ( kmesh2(1), 2 )
            call errsub ( 2, 2, 0, 1 )
         end if
      end if

!     --- next the number of elements
!         at this moment only spectral elements with the same number of
!         nodes in each direction are possible
!         also compute the length of kmesh2 part c and loads kmesh part a

      if ( ndim==1 ) then
         ihelp = kmesh1(4)
         kmesh2(9) = kmesh1(9)*(ihelp-1)
         leng = kmesh2(9)*2

!        --- load kmesh part b

         kmesh2(ipkm2b) = 2

      end if
      if ( ndim==2 ) then
         leng = 0
         kmesh2(9) = 0
         do icount = 1, nelgrp

!        --- fetch mesh type, # elements and # nodes per element

            call fmshtp(kmesh1, icount, meshtp)
            call fnelm(kmesh1, icount, nelm)
            call fnnpe(kmesh1, icount, nnpe)
            if ( meshtp==-10001 ) then

!           --- connection elements

               nprev = kmesh2(9)
               kmesh2(9) = kmesh2(9) + nelm
               leng = leng + (kmesh2(9)-nprev)*2

!              --- load kmesh part b

               kmesh2(ipkm2b+icount-1) = 2

            else if ( meshtp==1 ) then

!           --- line elements

               nprev = kmesh2(9)
               kmesh2(9) = kmesh2(9) + nelm*(nnpe-1)
               leng = leng + (kmesh2(9)-nprev)*2

!              --- load kmesh part b

               kmesh2(ipkm2b+icount-1) = 2

            else if ( meshtp==5 ) then

!           --- surface elements

               nprev = kmesh2(9)
               rnnpe = nnpe
               ihelp = int(sqrt(rnnpe))
               kmesh2(9) = kmesh2(9) + nelm*(ihelp - 1)*(ihelp - 1)
               leng = leng + (kmesh2(9)-nprev)*4

!              --- load kmesh part b

               kmesh2(ipkm2b+icount-1) = 4

            end if
         end do

      end if

      if ( ndim==3 ) then
         leng = 0
         kmesh2(9) = 0
         do icount = 1, nelgrp

!        --- fetch mesh type, # elements and # nodes per element

            call fmshtp(kmesh1, icount, meshtp)
            call fnelm(kmesh1, icount, nelm)
            call fnnpe(kmesh1, icount, nnpe)
            if ( meshtp==-10001 ) then

!           --- connection elements

               nprev = kmesh2(9)
               kmesh2(9) = kmesh2(9) + nelm
               leng = leng + (kmesh2(9)-nprev)*2

!              --- load kmesh part b

               kmesh2(ipkm2b+icount-1) = 2

            else if ( meshtp==1 ) then

!           --- line elements

               nprev = kmesh2(9)
               kmesh2(9) = kmesh2(9) + nelm*(nnpe-1)
               leng = leng + (kmesh2(9)-nprev)*2

!              --- load kmesh part b

               kmesh2(ipkm2b+icount-1) = 2

            else if ( meshtp==5 ) then

!           --- surface elements

               nprev = kmesh2(9)
               rnnpe = nnpe
               ihelp = int(sqrt(rnnpe))
               kmesh2(9) = kmesh2(9) + nelm*(ihelp - 1)*(ihelp - 1)
               leng = leng + (kmesh2(9)-nprev)*4

!              --- load kmesh part b

               kmesh2(ipkm2b+icount-1) = 4

            else if ( meshtp==13 .or. meshtp==14 ) then

!           --- volume elements

               nprev = kmesh2(9)
               rnnpe = nnpe
               ihelp = int(rnnpe**(1d0/3d0)+0.5d0)

               if ( ihelp**3/=nnpe ) then
                  call errint ( nnpe, 1 )
                  call errint ( ihelp, 2 )
                  call errint ( ihelp**3, 3 )
                  call errsub ( 1696, 3, 0, 0 )
               end if

               kmesh2(9) = kmesh2(9)+nelm*(ihelp-1)*(ihelp-1)*(ihelp-1)
               leng = leng + (kmesh2(9)-nprev)*8

!              --- load kmesh part b

               kmesh2(ipkm2b+icount-1) = 8

            end if

         end do

      end if

!     --- fetch kmesh1 part c

      call ini060text ( ibuffr, kmesh1(17), 'kmeshc-1'  )

!     --- create permanent space for kmesh2 part c

      call ini081 ( ibuffr, kmesh2(17), leng, 'kmesh2-c' )

!     --- fetch kmesh2 part c

      ipkm1c = inigettext ( kmesh1(17), 'kmeshc-1' )
      ipkm2c = inigettext ( kmesh2(17), 'kmeshc-2' )
      if ( ierror/=0 ) go to 1000

!     --- fill kmesh2 part c

      call fillpc ( ibuffr(ipkm1c), ibuffr(ipkm2c), kmesh1, kmesh2 )
      if ( ierror/=0 ) go to 1000

!     --- write kmesh2 to femesh.out

      if ( parallel ) then

!     --- parallel is true, hence parallel computing
!         Create file femesh_par.femeshbf

         filename = 'femesh_par.'
         write(filename(12:14),100) inodenr
100      format(i3.3)
         lastps = 14

      else

!     --- parallel is false
!         Create file femesh.out

         filename = 'femesh.out'
         lastps = 11

      end if  ! ( parallel )
      if ( debug ) call prinin ( kmesh2, kmesh2(3), 'kmesh2' )

      open ( iref, file=filename(1:lastps), status = 'unknown' )
      call meshwrbf ( ibuffr, buffer, iref, kmesh2 )
      if ( ierror/=0 ) go to 1000
      rewind(iref)

!     --- reset kmesh2

      do icount = 2, kmesh2(1)
         kmesh2(icount) = 0
      end do

!     --- read kmesh2 from femesh.out

      irenum = 0
      if( iren>0 ) irenum = 2
      call msh071 ( ibuffr, buffer, irenum, iref, kmesh2, idummy )
      if ( ierror/=0 ) go to 1000
      close(iref)

!     --- fetch kmesh1 and kmesh2 part i

      call ini060text ( ibuffr, kmesh1(23), 'coor-1' )
      call ini060text ( ibuffr, kmesh2(23), 'coor-2' )
      ipkm1i = inidgttext ( kmesh1(23), 'coor-1' )-1
      ipkm2i = inidgttext ( kmesh2(23), 'coor-2' )-1
      if ( ierror/=0 ) go to 1000

!     --- copy the original coordinates into kmesh2 part i

      do icount = 1, kmesh2(8)
         buffer(ipkm2i+icount) = buffer(ipkm1i+icount)
      end do

!     --- fetch kmesh1 and kmesh2 part j

      if ( iren>0 ) then

!     --- first check if a renumbering on kmesh2 is performed

         if ( kmesh2(24)/=0 ) then
            call ini060text ( ibuffr, kmesh2(24), 'kmesh2-j' )

!           --- reserve space for kmesh1 part j

            call ini081 ( ibuffr, kmesh1(24), kmesh1(8), 'kmesh1-j' )
            ipkm1i = inigettext ( kmesh1(24), 'kmesh1-j' )-1
            ipkm2i = inigettext ( kmesh2(24), 'kmesh2-j' )-1
            if ( ierror/=0 ) go to 1000

!           --- copy the renumbering into kmesh1 part j

            do icount = 1, kmesh1(8)
               ibuffr(ipkm1i+icount) = ibuffr(ipkm2i+icount)
            end do

         end if

      end if

      if ( parallel ) then

!     --- parallel is true, hence parallel computing
!         Read file meshoutput_par.000 and some information from meshoutput

         call mshrdparmesh ( ibuffr, buffer, kmesh2 )

      end if

1000  call erclmn('femeshbf', timebefore)
      kmesh2(38) = -kmesh1(38)

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End femeshbf'

      end if  ! ( debug )

      end
