/*
 * Copyright (c) 1999 Kees Lemmens, TU Delft
 *
 *
 * Date : June , 1999
 *
 * Modified :
 *
 * Function : Routines to read/write Java binary sepplot files in the way
 * Fortran would do it. This is also the reason why we use "call by
 * reference". It should be possible to link these routines directly into
 * a Fortran program.
 * 
 * NOTE: This program is supposed to be used on a little endian platform 
 *       such as Intel. A version for big endian would be straightforward :
 *       simply omit the byteswapping routines.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "plotfun.h"

/* Java portable binary format INPUT routines : */

static FILE* input;

float readBigEndian(void)
{
   char obytes[4],*nbytes;
   float be;

   fread(obytes,4,1,input);
   nbytes = (char *) &be;
   
#ifdef LITTLE_ENDIAN
   // Now swap the byte-order :
   
   nbytes[0] = obytes[3]; nbytes[1] = obytes[2];
   nbytes[2] = obytes[1]; nbytes[3] = obytes[0];
#else
   memcpy(nbytes,obytes,4);
#endif
   return be;
}

void jplrdcl(int *ios)
{
  *ios = fclose(input);
}

void jplrdop(char *fileid, int *fillen, int *ios)
{
   int len;
   char magic[8];
   char *name;
   
   name = malloc(*fillen + 1);
   strncpy(name,fileid,*fillen);
   name[*fillen] = '\0';  // provide a terminating zero 
   input = fopen(name,"r");
   free(name);
   if(input == NULL)
   {
      *ios = -1;
      return;
   }
   
   fgetc(input);       // discard first byte 
   len = fgetc(input); // max length is now 255 bytes
   fread(magic,7,1,input);
   if(strncmp(magic,MAGIC,7) != 0)
   {
      *ios = -1;
      fprintf(stderr,"Sorry: %s is not a Java binary file !\n",fileid);
      exit(1);
   }
   else
     *ios = 0;
}

void jplrd01(int *command, float *x, float *y,float *a,float *h,
	     int *nchar, int *ios)
{
   char work;
   
   fread(&work,1,1,input);
   *command = work;
   
   *x = readBigEndian();
   *y = readBigEndian();
   *a = readBigEndian();
   *h = readBigEndian();
   
   fread(&work,1,1,input);
   *nchar = work;

   if(feof(input) || ferror(input))
     *ios = -1;
   else
     *ios = 0;
   
  // fprintf(stderr,"%02d: %f %f %f %f , %d\n",*command,*x,*y,*a,*h,*nchar);
}

void jplrd06(int *nchar,char * string, int *ios)
{
   int len;
   
   len  = fgetc(input)* 0x100; // swap byte order
   len += fgetc(input);

   fread(string,len,1,input);

   if(feof(input) || ferror(input))
     *ios = -1;
   else
     *ios = 0;
}

void jplrd07(int *nchar,float polyg[], int *ios)
{
   int x;
   
   for(x=0;x<*nchar;x++)
   {
      polyg[2*x  ] = readBigEndian();
      polyg[2*x+1] = readBigEndian();
   }
   
   if(feof(input) || ferror(input))
     *ios = -1;
   else
     *ios = 0;
}

/* Java portable binary format OUTPUT routines : */

static FILE* output;

void writeBigEndian(float le)
{
   char *obytes,nbytes[4];

   obytes = (char *)&le;

#ifdef LITTLE_ENDIAN
   
   // Now swap the byte-order :
   nbytes[0] = obytes[3]; nbytes[1] = obytes[2];
   nbytes[2] = obytes[1]; nbytes[3] = obytes[0];
#else
   memcpy(nbytes,obytes,4); 
#endif
     
   fwrite(nbytes,4,1,output);
}

void jplwrcl(int *ios)
{
  *ios = fclose(output);
}

void jplwrop(char *fileid, int *fillen, int *ios)
{
   char *name;
   char *magic = MAGIC;
   
   name = malloc(*fillen + 1);
   strncpy(name,fileid,*fillen);
   name[*fillen] = '\0';  // provide a terminating zero 
   output = fopen(name,"w");
   free(name);

   if(output == NULL)
   {
      *ios = -1;
      return;
   }
   
   fputc((char)0x00,output);          // write length as 2 byte short
   fputc((char)strlen(magic),output); //         "            "
   
   fwrite(magic,strlen(magic),1,output);
   
   if(ferror(output))
     *ios = -1;
   else
     *ios = 0;
}

void jplwr01(int *command, float *x, float *y,float *a,float *h,
	     int *nchar, int *ios)
{
   char work;
   
   work = (char)*command;
   fwrite(&work,1,1,output);
   
   writeBigEndian(*x);
   writeBigEndian(*y);
   writeBigEndian(*a);
   writeBigEndian(*h);
    
   work = (char)*nchar;
   fwrite(&work,1,1,output);
   
   if(ferror(output))
     *ios = -1;
   else
     *ios = 0;
   
   // fprintf(stderr,"%02d: %f %f %f %f , %d\n",*command,*x,*y,*a,*h,*nchar);
}

void jplwr06(int *nchar, char * string, int *ios)
{
   int len=*nchar;
   
   fputc(len/0x100,output); // high byte
   fputc(len%0x100,output); // low byte
   
   fwrite(string,len,1,output);

   if(ferror(output))
     *ios = -1;
   else
     *ios = 0;
}

void jplwr07(int *nchar, float polyg[], int *ios)
{
   int x;
   
   for(x=0;x<*nchar;x++)
   {
      writeBigEndian(polyg[2*x  ]);
      writeBigEndian(polyg[2*x+1]);
   }
   
   if(ferror(output))
     *ios = -1;
   else
     *ios = 0;
}
