      subroutine iniactkmeshd ( ibuffr, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 12-06-2015 Use markelem
!        version  1.6  date 06-03-2015 Extra Debug
!        version  1.5  date 07-02-2014 Jump in case of errors
!
!   copyright (c) 2002-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kmesh part d
!     kmesh part d contains the elements corresponding to each node
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: ibuffr(*), kmesh(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) :: ipkmd1, ipkmsc, ipkmd2
      integer :: npoint, lengd2, nelem, error
      integer, allocatable :: markelem(:)
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     ipkmd1         Starting address of KMESH part d1 in array IBUFFR
!     ipkmd2         Starting address of KMESH part d2 in array IBUFFR
!     ipkmsc         Starting address of array KMESH part c in array
!                    IBUFFR
!     lengd2         length of second part of array KMESH part d
!     markelem       Work array to mark elements that must be used
!                    0: element is skipped
!                    1: element is used
!     nelem          Number of elements
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INI084         compute starting position of a new or already existing
!                    array
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     TOPLM1         Compute length of KMESH part D2, fill KMESH part D1
!     TOPLM2         Fill KMESH part D2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate kmeshc
!     Create work space for kmesh part d1
!     Activate arrays
!     Fill kmesh part d1 and compute length of d2
!     Create permanent space for kmesh part d and copy kmesh d part 1
!     Create work space
!     Activate arrays
!     Fill kmesh part d2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactkmeshd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniactkmeshd'
  1      format ( a, 1x, (10i10) )

      end if

!     --- activate kmeshc

      call ini060text ( ibuffr, -kmesh(17), 'kmeshc' ) ! kmesh part c (topology)

!     --- subroutine topelm compute elements corresponding to nodal points

      npoint = kmesh(8)   ! number of nodes
      nelem = kmesh(9)

      allocate ( markelem(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'markelem' )
      if ( ierror/=0 ) go to 1000

      markelem = 1

!     --- Create work space for kmesh part d1

      call ini081 ( ibuffr, kmesh(18), npoint+1, 'kmeshd' )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )   ! starting address of kmeshc
      ipkmd1 = inigettext ( kmesh(18), 'kmeshd' )

!     --- Fill kmesh part d1 and compute length of d2

      if ( debug ) write(irefwr,*) 'before toplm1'
      call toplm1 ( kmesh, ibuffr(ipkmd1), ibuffr(ipkmsc), markelem )
      lengd2 = ibuffr(ipkmd1+npoint)
      if ( debug ) &
         write(irefwr,1) 'npoint, lengd2', npoint, lengd2

!     --- Create permanent space for kmesh part d and copy kmesh d part 1

      call ini084 ( ibuffr, kmesh(18), lengd2+npoint+1, 'kmeshd' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )  ! kmesh part c
      ipkmd1 = inigettext ( kmesh(18), 'kmeshd' )  ! kmesh part d1
      ipkmd2 = ipkmd1+npoint+1     !  starting address of kmesh part d2
      if ( debug ) &
         write(irefwr,1) 'ipkmd1, ipkmd2', ipkmd1, ipkmd2

!     --- Fill kmesh part d2

      call toplm2 ( kmesh, ibuffr(ipkmd1), ibuffr(ipkmd2), &
                    ibuffr(ipkmsc), markelem )

      deallocate ( markelem, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'markelem' )

1000  call erclos ( 'iniactkmeshd' )
      if ( debug ) write(irefwr,*) 'End iniactkmeshd'

      end
