      subroutine inisurfcurvs ( surfsw0, numsurw0, surfsw1, numsurw1, &
                                curves, kmesh, kmeshs, numcurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find curves that is on the interface of two sets of surfaces
!     In this case one of the sets overlaps the other one
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     surface
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numsurw0, numsurw1, surfsw0(numsurw0), numcurves, &
              surfsw1(numsurw1), curves(*), kmesh(*), kmeshs(*)

!     curves         o    Curves at interface
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshs         i    array containing the user points corresponding to
!                         the curves in the following way:
!                         kmeshs(1)=ncurvs+1
!                         number of user points in curve i:
!                         kmeshs(i+1)-kmeshs(i)
!                         user points in curve i are stored in kmeshs from
!                         position kmeshs(i)+1 to kmeshs(i+1)
!     numcurves      o    Number of curves in array curves
!     numsurw0       i    Number of surfaces stored in array surfsw0
!     numsurw1       i    Number of surfaces stored in array surfsw1
!     surfsw0        i    Contains the surfaces on which w0 is defined
!     surfsw1        i    Contains the surfaces on which w1 is defined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurvs, i, numsurfs, isurnr, j, error, nsurfs
      integer, allocatable, dimension (:) :: iwork, setsurfs
      logical debug, negative

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     isurnr         Curve sequence number
!     iwork          integer work array
!     j              Counting variable
!     ncurvs         Number of curves in mesh
!     negative       If true all curve numbers are negative
!     nsurfs         number of surfaces
!     numsurfs       Number of surfaces in array setsurfs
!     setsurfs       Contains surface numbers of set w1 not in set w0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
!    3041   No common curve of set of surfaces corresponding to w0 and w1
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters
!     Allocate arrays
!     Separate sets of surfaces in two disjoint sets
!     Find common curves of set w0 and setsurfs
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inisurfcurvs' )
      debug = .false.
      numcurves = 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inisurfcurvs'
         call prinin ( surfsw0, numsurw0, 'surfaces for w0' )
         call prinin ( surfsw1, numsurw1, 'surfaces for w1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ncurvs = kmesh(11)
      nsurfs = kmesh(12)

!     --- Allocate arrays

      numsurfs = max(nsurfs, ncurvs)  ! help parameter
      allocate ( iwork(numsurfs), setsurfs(numsurfs), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( numsurfs+ncurvs, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- Separate sets of surfaces in two disjoint sets

      iwork = 0                    ! clear iwork
      do i = 1, numsurw0
         iwork(surfsw0(i)) = i     ! mark that surface belongs to set w0
      end do  ! i = 1, numsurw0
      if ( debug ) call prinin ( iwork, nsurfs, 'iwork' )

!     --- Store surfaces of set w1 not in w0 into setsurfs

      numsurfs = 0     ! number of surfaces in set

      do i = 1, numsurw1

         isurnr = surfsw1(i)

         if ( iwork(isurnr)==0 ) then

!        --- iwork(isurnr)=0, hence curve not in set w0

            numsurfs = numsurfs+1
            setsurfs(numsurfs) = isurnr

         end if  ! ( iwork(isurnr)==0 )

      end do  ! i = 1, numsurw1
      if ( debug ) call prinin ( setsurfs, numsurfs, 'setsurfs' )

!     --- Find common curves of set w0 and setsurfs

      iwork = 0
      do i = 1, numsurw0

         isurnr = surfsw0(i)

         if ( debug ) then
            write(irefwr,1) 'i, isurnr', i, isurnr
            call prinin ( kmeshs(kmeshs(isurnr)+1), &
                          kmeshs(isurnr+1)-kmeshs(isurnr), 'curves' )
         end if  ! ( debug )

         do j = kmeshs(isurnr)+1, kmeshs(isurnr+1)
            iwork(abs(kmeshs(j))) = 1    ! mark curve
         end do  ! j = kmeshs(isurnr)+1, kmeshs(isurnr+1)

      end do  ! i = 1, numsurw0
      if ( debug ) call prinin ( iwork, ncurvs, 'iwork/2' )

      do i = 1, numsurfs

         isurnr = setsurfs(i)

         if ( debug ) then
            write(irefwr,1) 'i, isurnr', i, isurnr
            call prinin ( kmeshs(kmeshs(isurnr)+1), &
                          kmeshs(isurnr+1)-kmeshs(isurnr), 'curves' )
         end if  ! ( debug )

         do j = kmeshs(isurnr)+1, kmeshs(isurnr+1)

            if ( iwork(abs(kmeshs(j)))==1 ) then

!           --- Common curve found, store

               numcurves = numcurves+1
               curves(numcurves) = kmeshs(j)

            end if  ! ( iwork(kmeshs(j))==0 )

         end do  ! j = kmeshs(isurnr)+1, kmeshs(isurnr+1)

      end do  ! i = 1, numsurfs

      if ( numcurves==0 ) then

!     --- Error, no curve has been found

         call errsub ( 3041, 0, 0, 0 )

      end if  ! ( numcurves==0 )

!     --- make an ordering of the curves such that they form
!         consecutive curves

      if ( numcurves==1 ) then

!     --- numcurves = 1, only 1 curve, make curve number positive

         curves(1) = abs(curves(1))

      else

!     --- numcurves > 1, more curves
!         if all curve numbers negative, reverse sequence and sign

         negative = .true.    ! all curves are negative
         do i = 1, numcurves
            if ( curves(i)>0 ) negative = .false.  ! positive curve found
         end do  ! i = 1, numcurves

         if ( negative ) then

!        --- All curves are negative, reverse sign and sequence

            do i = 1, numcurves
               iwork(numcurves+1-i) = -curves(i)
            end do  ! i = 1, numcurves
            curves(1:numcurves) = iwork(1:numcurves)

         end if   ! ( negative )

      end if  ! ( numcurves==1 )
900   deallocate ( iwork, setsurfs, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  call erclos ( 'inisurfcurvs' )
      if ( debug ) then

!     --- Debug information

         call prinin ( curves, numcurves, 'curves at interface' )
         write(irefwr,*) 'End inisurfcurvs'

      end if  ! ( debug )

      end
