module sepmodulesmf

   integer, parameter :: nbld=15  ! size of iinbld
   integer :: isubcall            ! Type of calling subroutine
                                  ! 1:
                                  ! 2: systm0
                                  ! 3: systm1
                                  ! 4: systm2
                                  ! 5: rest
   integer :: iinbld(nbld)        ! See below
   integer, pointer :: skipelgrps(:)  ! In skipelgrps(IELGRP)  (1<=IELGRP<=NELGRP+NUMNATBND)
!                           the user must store, whether elements with
!                           standard element sequence number IELGRP (or
!                           IELGRP = NELGRP + IBNGRP for boundary elements)
!                           are added to the large matrix and large vector
!                           or not.
!                           When skipelgrps(IELGRP) = 0 the elements with
!                           standard element sequence number IELGRP are
!                           skipped, when skipelgrps(IELGRP) = 1 these elements
!                           are added.

   type matsmf

!  --- Defines quantities related to the building of the
!      stiffness matrix, the mass matrix and the right-hand side

      integer, pointer :: irhsd(:)
      integer, pointer :: iusol
      integer, pointer :: islold(:)
      integer, pointer :: iuser(:)
      integer, pointer :: ioutvc
      double precision, pointer :: user(:)

   end type matsmf

   type (matsmf) :: SMF

!    Explanation of parameters
!    iinbld    In this array it is indicated how the matrix and right-hand
!              side, must be filled.
!              Contents
!               1 -
!               2 JCHOIS    general choice parameter, possibilities:
!                        1  Both the large matrix and large vector are built
!                        2  Only the large vector is built.
!                        3  The large matrix is built and the large vector
!                           is made equal to zero.
!                           If EFFBOUNCOND = 0 the large vector may be
!                           unequal to zero, due to
!                           non-homogeneous essential boundary conditions.
!                        4  No large matrix nor large vector is build
!                        5  The large matrix is not built;
!                           the large vector is made equal to zero.
!                           If EFFBOUNCOND = 0 the large vector may be
!                           unequal to zero, due to
!                           non-homogeneous essential boundary conditions.
!                        6  The large matrix is not built;
!                           The large vector is created by multiplication of
!                           the matrix consisting of all boundary elements
!                           by the solution vector
!                           In this way an extra term for the reaction force
!                           is created
!                       14  The large matrix is built;
!                           no right-hand-side vector is made.
!                           Default value: 1
!               3 EFFBOUNCOND   Indication whether the effect of essential
!                           boundary  conditions must be taken into
!                           account (0) or not (1).
!                           Default value: 0
!               4 IMAS      Indication if the mass matrix must be computed
!                           (>1) or not (0). The storage of the mass matrix
!                           depends on the parameter IMAS in the following
!                           way:
!                           0: No mass matrix
!                           1: The mass matrix is stored as a diagonal
!                              matrix
!                           2: The mass matrix is stored as a large matrix
!                              with exactly the same structure as the
!                              stiffness matrix
!                           3: The mass matrix is stored as a large matrix
!                              however, its storage scheme may differ
!                              from the one used for the stiffness matrix
!                              The structure of this matrix must be
!                              stored in INTMAT(.,2)
!                           Default value: 0
!               5 IPREC     Indication if the old solution is stored in
!                           ISLOLD (1) or not (0)
!                           IPREC is not used if IINBLD(10)>0
!                           Default value: 0
!               6 ALLELEMENTS  Indication if all standard element
!                           sequence numbers are used or not
!                           Possible values:
!                           -2: skip all internal elements
!                           -1: skip all boundary elements
!                            0: do not skip any element
!                           >0 skip some element groups
!                           In that case array skipelgrps has to be filled
!                           Default value: 0
!               7 ADDMATVEC   When ADDMATVEC = 0 the large matrix and vector
!                           are cleared before the assembling process.
!                           When ADDMATVEC = 1 the right-hand-side vector
!                           and the large matrix are not cleared
!                           but added to the already existing right-hand
!                           side vector and large matrix.
!                           Hence        RHSD := RHSD + DELTA  RHSD
!                                        MATRIX := MATRIX +  DELTA  MATRIX
!                           Default value: 0
!               8  CLEARLASTRHSD  When CLEARLASTRHSD = 0 the last part of
!                           the right-hand-side vector corresponding to the
!                           boundary conditions is cleared
!                           (standard situation)
!                           when CLEARLASTRHSD = 1 the standard assembling
!                           process is also applied to this last part of the
!                           right-hand-side vector. For most applications
!                           CLEARLASTRHSD = 0 suffices.
!                           Default value: 0
!               9  IPROB    Problem definition number
!                           Default value: defined by INTMAT
!              10  NUMOLD   Number of arrays that have been filled in array
!                           ISLOLD
!                           Default value: 0
!              11  ISPECTRAL Indication if a spectral element method is used
!                           needing extra information (1) or not (0)
!                           Default value: 0
!              12  NRIGHTHS Number of right-hand sides that must be computed
!                           by subroutine BUILD in this call. This parameter
!                           is only used if JCHOIS = 1 or 2.
!                           For all these vectors the other parameters in
!                           IINBLD are used in exactly the same way.
!                           The information about the right-hand sides is
!                           stored in the following way:
!                           f  refers to rhsd(.,i)  + = 1 (1) 5
!                            i
!                           Default value: 1
!              13  ISEQIN   Sequence number of old solution in array ISLOLD
!                           Default value: 1
!              14  IDEF_COR indicates if defect correction should be used
!                           0: no defect correction
!                           1: defect correction
!                           Default value: 0
!              15  ISUBDIVIDE indicates if quadratic elements must be
!                           treated as a conglomerate of linear elements (1)
!                           or as quadratic elements itself (0)
!                           Default value: 0
!    intmat    Standard SEPRAN array of length 5 containing information
!              concerning the structure of the matrix. See the Programmer's
!              Guide 13.5 for a complete description.
!              In the case of a non-diagonal mass matrix array intmat
!              must contain the information about the structure of
!              the mass matrix in intmat(1-5,2)
!    irhsd     Standard SEPRAN array of length 5 containing information
!              concerning the right-hand-side vector. See the Programmer's
!              Guide 13.4 for a complete description.
!              In case of defect correction the right-hand side consists of
!              two parts:
!              irhsd(.,i) refers to the right-hand side corresponding to
!                         matr(.,i)
!    islold    If an iterative procedure is used, for example for
!              non-linear problems the solution in a preceding iteration
!              may be necessary. In that case information concerning the
!              preceding solution must be stored in ISLOLD.
!              If the preceding solution is not necessary, a dummy parameter
!              may be used.
!              Another possibility is that coefficients depend on solution
!              arrays or arrays of special stricture that have been computed
!              before.
!              ISLOLD is used if either IPREC = 1 or NUMOLD>0
!              If NUMOLD=0 and IPREC=1 ISLOLD must be a solution vector
!              of exactly the same type and corresponding to the same
!              problem as ISOL
!              If NUMOLD>0, ISLOLD must be a two-dimensional array of size
!              (5,k), with k at least NUMOLD.
!              The first NUMOLD vectors ISLOLD(.,i) must have been filled
!              with solution vectors or vectors of special structure not
!              necessarily corresponding to problem IPROB
!    iuser     In this array together with array user information concerning
!              coefficients etc. for the differential equation must be
!              stored.
!              For standard problems that are contained in the library
!              the arrays IUSER and USER must be filled according to
!              fixed rules (see the manual Standard Problems).
!              When the user uses his own elements he may fill and use
!              IUSER and USER in his own way, since these arrays are only
!              used by the element subroutines.
!    iusol     Standard SEPRAN array of length 5 containing information
!              concerning the solution vector. See the Programmer's
!              Guide 13.3 for a complete description.
!              Essential boundary conditions must be filled in the
!              solution vector corresponding to ISOL if necessary.
!              (Depends on the value of ICHOIS).
!    kmesh     Standard SEPRAN array containing information of the mesh
!    kprob     Standard SEPRAN array containing information of the problem
!              definition. See the Programmer's Guide 13.2 for a complete
!              description
!    massmt    This array contains the Mass matrix at output, where the
!              mass matrix is considered to be a diagonal matrix
!              massmt is only filled when imas = 1
!    matr      Standard SEPRAN array of length 5 containing information
!              concerning the large matrix. See the Programmer's
!              Guide 13.6 for a complete description.
!              In case of defect correction the matrix consists of two parts
!              matr(.,1) refers to the matrix to be inverted for the defect
!                        correction, for example the upwind matrix
!              matr(.,2) refers to the matrix for the correction in the
!                        defect correction, for example the central
!                        difference matrix
!    user      See array iuser

end module sepmodulesmf
