      subroutine prevalveccexp1 ( work, iwork, ncount, result, text, &
                                  lenvec, irefwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read subexpression not containing any brackets and translate into a
!     vector. (complex variant)
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ncount, lenvec, irefwork(ncount)
      double complex work(lenvec,*), result(lenvec)
      character (len=*) text

!     irefwork      i/o   integer work array of length ncount
!                         if irefwork(i) = 0, it concerns a constant
!                         otherwise (1) a vector
!     iwork          i    integer work array of length lenvec x ncount
!                         iwork contains the operators that precede the numbers
!                         according to the following code:
!                          -4:  )
!                          -3:  (
!                          -2:  division
!                          -1:  subtraction
!                           1:  addition
!                           2:  multiplication
!                           3:  raise to the power
!                          10:  sqrt
!                          11:  sin
!                          12:  cos
!                          13:  exp
!                          14:  tan
!                          15:  asin
!                          16:  acos
!                          17:  atan
!     lenvec         i    Length of arrays work and result
!     ncount         i    Length of arrays iwork and irefwork
!     result         o    Result of the operation according to
!                         result = operator_i work(i) (summation convention)
!                         where operator_i is defined by iwork
!                         The priority rules of Fortran are applied
!     text           i    String variable containing some text for error message
!     work           i    work array of length lenvec x ncount
!                         work contains the vectors that must be used to apply
!                         the operation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALVECCEXPF Evaluate vector expression and fill actual values of
!                    vectors (complex version)
!     PREVALVECCEXPM Evaluate multiplications/subdivisions in vector expression
!                    (complex version)
!     PREVALVECCEXPP Evaluate powers in vector expression (complex version)
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := ncount (-1) 2 do
!        if ( iwork(i)=3 ) then
!           work(i-1) := work(i-1)**work(i)
!           shift work and iwork
!     For i := ncount (-1) 2 do
!        if ( iwork(i)=2 ) then
!           work(i-1) := work(i)*work(i-1)
!           ncount := ncount-1
!           shift work and iwork
!        else if ( iwork(i)=-2 ) then
!           work(i-1) := work(i-1)/work(i)
!           ncount := ncount-1
!           shift work and iwork
!
!     result := 0
!     For i := 1 (1) ncount do
!        result := result+iwork(i)*work(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalveccexp1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalveccexp1'
         call prinin ( iwork, ncount, 'iwork-in' )
         call prinin ( irefwork, ncount, 'irefwork-in' )
         call prinrl2 ( work, lenvec, ncount, 'work_in' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- First evaluate all functions

      call prevalveccexpf ( work, iwork, ncount, result, &
                            lenvec, irefwork  )

!     --- Next all powers

      call prevalveccexpp ( work, iwork, ncount, &
                            lenvec, irefwork  )

!     --- Next all multiplications and subdivisions

      call prevalveccexpm ( work, iwork, ncount, text, &
                            lenvec, irefwork  )

!     --- result := 0

      result = (0d0,0d0)

!     --- For i := 1 (1) ncount do
!           result := result+iwork(i)*work(i)

      do i = 1, ncount
         if ( iwork(i)==0 ) then
            result = result+work(1:lenvec,i)
         else
            result = result+iwork(i)*work(1:lenvec,i)
         end if
      end do

1000  call erclos ( 'prevalveccexp1' )
      if ( debug ) write(irefwr,*) 'End prevalveccexp1'

      end
