      subroutine prflgridnod2d ( kmeshac, nx, ny, nnodes, coor, nodes, &
                                 istart, xmin, xmax, ymin, ymax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-08-2012 Replace iwork by nodes
!        version  1.0  date 16-11-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information of a set of nodes in rectangular 2d grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     rectangle
!     grid
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshac(*), nx, ny, nnodes, nodes(nnodes), istart
      double precision coor(2,*), xmin, xmax, ymin, ymax

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     istart         i    First position used in this part of kmeshac
!     kmeshac        o    Integer array containing the information of the
!                         subdivision of the mesh in cells
!                         See calling subroutine for a description
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be used to fill
!                         kmeshac
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     xmax           i    Maximum x-value
!     xmin           i    Minimum x-value
!     ymax           i    Maximum y-value
!     ymin           i    Minimum y-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodp, indi, indj, indk, error
      integer, allocatable :: iwork(:,:)
      double precision distx, disty, x, y
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     indi           i-index
!     indj           j-index
!     indk           k-index
!     inodp          Nodal point number
!     iwork          integer work array of length 2*npoint
!                    The cell index (i,j) is stored iwork(1:2,.)
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array grid
!     For all nodes inode in grid do
!        Find index (ix,iy) in grid
!        raise grid(ix,iy,1) by 1
!        store ix, iy in iwork
!     Fill grid(ix,iy,2) with starting addresses
!     For all nodes inode in grid do
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prflgridnod2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prflgridnod2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(2,nnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nnodes, 'iwork' )
      if ( ierror/=0 ) go to 1000

      kmeshac(istart:istart+nx*ny+1) = 0

      distx = (xmax-xmin)/nx
      disty = (ymax-ymin)/ny

      do i = 1, nnodes

!     --- For all nodes in part do
!         Compute i,j index for node and store in iwork(2-3,.)
!         Adapt Pointer array pos k = (i-1)*nx+j

         inodp = nodes(i)
         x = coor(1,inodp)
         y = coor(2,inodp)
         indi = min(nx,int((x-xmin)/distx)+1)
         indj = min(ny,int((y-ymin)/disty)+1)
         indk = (indj-1)*nx+indi
         kmeshac(istart+indk) = kmeshac(istart+indk)+1
         iwork(1,i) = indi
         iwork(2,i) = indj

      end do

!     --- Accumulate pointer array

      kmeshac(istart) = istart+nx*ny+1
      do i = 1, nx*ny
         kmeshac(istart+i) = kmeshac(istart+i-1)+kmeshac(istart+i)
      end do

      do i = 1, nnodes

!     --- For all nodes in part do
!         Fill node in second part of kmeshac
!         Adapt pointer array

         inodp = nodes(i)
         indi = iwork(1,i)
         indj = iwork(2,i)
         indk = (indj-1)*nx+indi
         kmeshac(kmeshac(istart+indk-1)) = inodp
         kmeshac(istart+indk-1) = kmeshac(istart+indk-1)+1

      end do

!     --- Reset part 1

      do i = nx*ny, 1, -1
         kmeshac(istart+i) = kmeshac(istart+i-1)
      end do
      kmeshac(istart) = istart+nx*ny+1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'prflgridnod2d' )
      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshac(istart), nx*ny+1+nnodes, 'kmeshac' )
         write(irefwr,*) 'End prflgridnod2d'

      end if  ! ( debug )

      end
