      subroutine prveccombinat0 ( ibuffr, buffer, isolut, inpsep, kmesh, &
                                  kprob, numvct, iseqvec, iseqcoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 12-02-2015 New call to prstorevec
!        version  1.2  date 17-01-2015 Extra debug
!        version  1.1  date 11-03-2014 new call to prvecstrhs
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cpost1f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isolut(5,*), inpsep(*), kmesh(*), kprob(*), &
              iseqvec, iseqcoor, numvct
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
!     iseqcoor       i    Sequence number of coordinates vector in isolut
!     isolut        i/o   Standard SEPRAN array containing information of the
!                         solution array
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     numvct        i/o   Largest sequence number of vector that has been filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numvecrhs, numrealrhs, icount, istore(100), &
              vecs(100), indprf, indprp, nunkp, nphys, itypvc, &
              ivec, iprob, lenvec, nch
      double precision store(100)
      logical :: debug, completevec, complexvec

!     completevec    If true the complete vector must be filled
!     complexvec     If true the vector to be filled is complex
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icount         Computed length of array istore
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     iprob          Actual problem number
!     iseqvec        Sequence number of vector to be created
!     istore         Contains the expression as stored in common
!                    cexpression, where all scalars and reals
!                    have been computed and stored in store
!                    References to these values are given by
!                    their position in store plus 1000
!                    The values of the vectors are stored in
!                    array vectors
!                    References to these vectors are given by
!                    their position in vectors plus 10000
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     lenvec         Length of vector to be created
!     nch            Number of characters in a string
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     numrealrhs     Number of reals stored in store
!     numvecrhs      Number of vectors in right-hand side
!     nunkp          Maximum number of unknowns in a point
!     store          Contains values of reals and scalars, see istore
!     vecs           In this array of size numvecrhs the sequence numbers of
!                    the right-hand side vectors are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INI066TEXT     delete temporary array
!     INI086         compute pointers to degrees of freedom of vectors
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRCOPYCOORBCK  Copy coordinates stored in solution vector back
!     PRININ         print 1d integer array
!     PRSTOREVEC     Adapt vector sequence number and copy coor if necessary
!                    in new vector
!     PRVECCHECKLEN  Check if all vectors at right-hand side have the same
!                    length as the vector on the right-hand side
!     PRVECCOMBINAT1 Evaluate right-hand side of vector expression
!                    Complete real vector
!     PRVECCOMBINAT2 Evaluate right-hand side of vector expression
!                    Complete complex vector
!     PRVECCOMBINAT3 Evaluate right-hand side of vector expression
!                    Partial real vector
!     PRVECFIND      Find the type of vector to be created for the
!                    left-hand side of an expression
!     PRVECSTRHS     Translate expression to a set of reals and a new set of
!                    temporary vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3162   Option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( left-hand-side vector exists ) then
!        activate vector
!     else
!        create vector depending on right-hand side
!     Fill set of nodes where vector must be computed
!     Compute length of vector
!     Translate expression to a set of reals and a new set of
!     temporary vectors
!     Evaluate expression
!     Store result in vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat0' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat0'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Translate expression to a set of reals and a new set of
!         temporary vectors

      call prvecstrhs ( inpsep(8)-200000000, numvecrhs, numrealrhs, &
                        icount, istore, store, vecs )
      call inichkarlen ( icount, 100, 'istore' )
      call inichkarlen ( numrealrhs, 100, 'store' )
      call inichkarlen ( numvecrhs, 100, 'vecs' )
      if ( ierror/=0 ) go to 1000

!     --- Create left-hand side vector

      if ( iseqvec>maxvc1 ) then

!     --- iseqvec>maxvc1, hence special vector
!         Store temporarily into isolut

         call prstorevec ( ibuffr, buffer, isolut, kmesh, &
                           kprob, iseqcoor, iseqvec, iseqvec, numvct )

         if ( debug ) then
            write(irefwr,1) 'iseqvec, iseqcoor', iseqvec, iseqcoor
            call prinin ( isolut(1,iseqvec), 5, 'isolut_coor' )
         end if  ! ( debug )
         complexvec = .false.

      else

!     --- find vector depending on right-hand side

         call prvecfind ( ibuffr, kmesh, kprob, inpsep, iprob, ivec, &
                          itypvc, iseqvec, vecs, isolut, numvecrhs, &
                          complexvec )
         if ( debug ) write(irefwr,*) 'complexvec/2', complexvec

!        --- Create vector

         if ( complexvec ) isolut(3,iseqvec) = 1 ! Complex vector

         call ini086 ( ibuffr, isolut(1,iseqvec), itypvc, ivec, iprob, &
                       kmesh, kprob, indprf, indprp, nphys, &
                       names(iseqvec) )
         nunkp = nphys

         if ( debug ) then
            write(irefwr,1) 'nunkp, indprf, indprp', &
                             nunkp, indprf, indprp
            call prinin (isolut(1,iseqvec), 5,'isolut(iseqvec)' )
         end if  ! ( debug )

      end if  ! ( isolut(1,iseqvec)>0 .and. ( isolut(2,iseqvec)==110 ... )
      if ( ierror/=0 ) go to 1000

      numvct = max(numvct, iseqvec)
      numvec = max(numvec, iseqvec)

      if ( ioutp>=0 .and. itimloop<=1 ) then

         nch = index(names(iseqvec),' ')-1
         if ( nch<=0 ) nch = len(names(iseqvec))
         write ( irefwr, 100 ) names(iseqvec)(1:nch), iseqvec
100      format ( ' Create vector ', a, &
                  ' with sequence number ', i3, ' by algebraic', &
                  ' manipulations' )

      end if  ! ( ioutp>=0 .and. itimloop<=1 )

      if ( debug ) write(irefwr,*) 'complexvec/3', complexvec

      completevec = inpsep(14)==0 .and. inpsep(15)==0 .and. &
                    inpsep(5)==0

      if ( completevec ) then

!     --- Complete vector
!         Check if all vectors at the right-hand side have the same length
!         as the left-hand side

         call prvecchecklen ( isolut, iseqvec, vecs, numvecrhs, kmesh, &
                              nunkp, completevec )
         if ( ierror/=0 ) go to 1000

!        --- Compute length of vector

         lenvec = isolut(5,iseqvec)

         if ( complexvec ) then

!        --- Complex vectors

            call prveccombinat2 ( ibuffr, buffer, isolut, iseqvec, vecs, &
                                  numvecrhs, kmesh, lenvec, &
                                  istore, store, icount, numrealrhs )

         else

!        --- Real vectors

            call prveccombinat1 ( ibuffr, buffer, isolut, iseqvec, vecs, &
                                  numvecrhs, kmesh, lenvec, istore, &
                                  store, icount, numrealrhs, nunkp )

         end if  ! ( complexvec )

      else

!     --- Part of vector

         if ( complexvec ) then

!        --- Complex vectors

            call errsub ( 3162, 0, 0, 0 )
            go to 1000

         else

!        --- Real vectors

            call prveccombinat3 ( ibuffr, buffer, isolut, iseqvec, vecs, &
                                  numvecrhs, kmesh, kprob, &
                                  istore, store, icount, numrealrhs, &
                                  inpsep, indprf )

         end if  ! ( complexvec )

      end if  ! ( completevec )

      if ( iseqcoor>0 ) then

!     --- coor has been changed, copy back into kmesh

         call prcopycoorbck ( ibuffr, buffer, kmesh, isolut, kprob, &
                              iseqcoor, inpsep(3) )

!        --- Remove from isolut

         call ini066text ( ibuffr, isolut(1,iseqcoor), 'coor_copy' )
         if ( numvec==iseqcoor ) numvec = numvec-1

      end if  ! ( iseqcoor>0 )

1000  call erclos ( 'prveccombinat0' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numvec', numvec
         write(irefwr,*) 'End prveccombinat0'

      end if  ! ( debug )

      end
