      subroutine pzparglob ( n, rhsd, blockunkinfo, sendinfo, &
                             icontext )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-03-2013 Use inibsendmpi
!        version  1.0  date 24-03-2010
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send information to neighbouring processors in order to give all
!     local arrays the global value in the common points
!     Complex version of prparglob
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     global
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cparallelf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, blockunkinfo(*), sendinfo(*), icontext
      complex * 16 rhsd(n)

!     blockunkinfo   i    Array with global block information for surface
!                         number isurnr. In this case it is related to
!                         non-prescribed degrees of freedom and not to nodes
!                         Contents:
!                         Pos. 1  m ( number of neihgbour blocks)
!                         Pos. 2*i   block number of i-th neighbour
!                         Pos. 2*i+1 starting address of i-th neighbour in
!                                    array sendinfo
!     icontext       i    Tag for sending and receiving, is used as check
!     n              i    Size of the matrix
!     rhsd          i/o   Real vector of length n containing the global
!                         vector to be used in parallel computations
!                         At input it contains the local values
!                         At output it contains the global values
!     sendinfo       i    Array containing information of the transport of
!                         unknowns between neighbouring blocks
!                         The starting address of each neighbour block is
!                         stored in array blockuninfo
!                         For each neighbour block the following information
!                         is stored
!                         Pos. 1 number of common unknowns n on common side of
!                                both blocks
!                         Pos. 2 ... n+1  unknown numbers of the unknowns to be
!                                sent to or received from block i
!                         If a common point in one block contains more
!                         unknowns than in the other block only the minimum
!                         of both is transported since the other ones do
!                         not belong to both blocks
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, numneigh, nneigh, sendstart, iblock, &
              seqblocks(iacnodes)
      complex * 16 usend(n)
      logical debug
      integer, dimension(:), pointer :: send_buffer
      integer, save :: send_buffer_size

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iblock       Block sequence number
!     j            loop counter
!     k            loop counter
!     nneigh       Number of overlapping unknowns
!     numneigh     number of neighbouring blocks
!     send_buffer  send buffer
!     send_buffer_size  Length of the send buffer
!     sendstart    starting address in array sendinfo
!     seqblocks    Work array of length nblocks (=nsurfs) which
!                  is used to find the sequence of block numbers
!                  in array blockunkinfo
!                     seqblocks(iblock) = 0: block iblock not present
!                     seqblocks(iblock) = i: block iblock is the i-th block
!                  in blockinfo
!     usend        Work array of length n to store the double precision data
!                  to be send to another block
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pzparglob' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pzparglob'

      end if
      if ( ierror/=0 ) go to 1000

      numneigh = blockunkinfo(1)

!     --- Store sequence of block numbers in array seqblocks

      seqblocks = 0

      do j = 1, numneigh
         iblock =  blockunkinfo(2*j)
         seqblocks(iblock) = j
      end do

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

!     --- deallocate send_buffer

      if (associated(send_buffer)) then
         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)
      end if  ! (associated(send_buffer))

      send_buffer_size = 0
      do j = 1, numneigh
         sendstart = blockunkinfo(2*j+1)
         nneigh = sendinfo(sendstart)
         send_buffer_size = send_buffer_size + nneigh*16
      end do

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, numneigh )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

      do j = 1, numneigh
         iblock = blockunkinfo(2*j)
         sendstart = blockunkinfo(2*j+1)
         nneigh = sendinfo(sendstart)
         do k = 1, nneigh
            usend(k) = rhsd(sendinfo(sendstart+k))
         end do

!        --- Send array usend

         call inibsendmpi ( usend, nneigh*2, SPDOUBLE, iblock, &
                            icontext )

      end do

      call inibarriermpi

      do j = 1, numneigh

!     --- Receive next array and store in usend
!         Get corresponding block number

         nneigh = n
         call inireceivempi ( usend, nneigh*2, SPDOUBLE, iblock, &
                              icontext )

         sendstart = blockunkinfo(2*seqblocks(iblock)+1)

         do k = 1, nneigh
            rhsd(sendinfo(sendstart+k)) = &
                rhsd(sendinfo(sendstart+k)) + usend(k)
         end do

      end do

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

1000  call erclos ( 'pzparglob' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pzparglob'

      end if

      end
