      subroutine readexpr ( work, iwork, ncount, result, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 26-10-2015 Correction func with minus sign
!        version  2.0  date 14-11-2013 Extra parameter text
!        version  1.6  date 20-02-2008 Extra debug
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read subexpression not containing any brackets and translate into a real
!     or integer number. Set ityprd equal to 1 or 2 and fill realrd
!     If this subroutine is adapted also change readconstant
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision work(*), result
      integer iwork(*), ncount
      character (len=*) text

!     iwork          i    integer work array of length ncount
!                         iwork contains the operators that precede the numbers
!                         according to the following code:
!                          -4:  )
!                          -3:  (
!                          -2:  division
!                          -1:  subtraction
!                           1:  addition
!                           2:  multiplication
!                           3:  raise to the power
!                          10:  sqrt
!                          11:  sin
!                          12:  cos
!                          13:  exp
!                          14:  tan
!                          15:  asin
!                          16:  acos
!                          17:  atan
!     ncount         i    Length of arrays iwork and work
!                         Defines the number of numbers and operators
!     result         o    Result of the operation according to
!                         result = operator_i work(i) (summation convention)
!                         where operator_i is defined by iwork
!                         The priority rules of Fortran are applied
!     text           i    String variable containing some text for error message
!     work           i    work array of length ncount
!                         work contains the numbers that must be used to apply
!                         the operation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision funresult
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     funresult      Result of function evaluation
!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READEVALFUNC   Evaluate function defined by ifunc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1693   Division by 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := ncount (-1) 2 do
!        if ( iwork(i)=3 ) then
!           work(i-1) := work(i-1)**work(i)
!           shift work and iwork
!     For i := ncount (-1) 2 do
!        if ( iwork(i)=2 ) then
!           work(i-1) := work(i)*work(i-1)
!           ncount := ncount-1
!           shift work and iwork
!        else if ( iwork(i)=-2 ) then
!           work(i-1) := work(i-1)/work(i)
!           ncount := ncount-1
!           shift work and iwork
!
!     result := 0
!     For i := 1 (1) ncount do
!        result := result+iwork(i)*work(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexpr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexpr'
         call prinin ( iwork, ncount, 'iwork-in' )
         call prinrl ( work, ncount, 'work-in' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      i = 0

!     --- For i := 1(1)ncount do

100   do while ( i<ncount )

!     --- First all functions are evaluated in order to satisfy the
!         priority rules

         i = i+1
         if ( debug ) write(irefwr,1) 'i, iwork(i)', i, iwork(i)
         if ( iwork(i)>=10 ) then

!        --- iwork(i)>=10: function
!            work(i-1) := sqrt(work(i))
!            shift work and iwork

             if ( iwork(i+1)==-1 ) then

!            --- minus sign found, multiply work by -1
!                Remove position i+1 from iwork, work

                work(i) = -work(i+1)
                iwork(i+1:ncount-1) = iwork(i+2:ncount)
                work(i+1:ncount-1) = work(i+2:ncount)
                ncount = ncount-1

             end if  ! ( iwork(i+1)==-1 )
             call readevalfunc ( iwork(i), work(i), funresult )
             if ( ierror/=0 ) go to 1000
             if ( i==1 ) then
                iwork(1) = 1
                work(1) = funresult
             else
                work(i-1) = funresult
                ncount = ncount-1
                do j = i, ncount
                   work(j) = work(j+1)
                   iwork(j) = iwork(j+1)
                end do
                work(ncount+1) = 0d0
                iwork(ncount+1) = 0
                do while ( ncount>1 .and. iwork(ncount)==0 )
                   ncount = ncount-1
                   work(ncount) = work(ncount+1)
                end do  ! while ( ncount>1 .and. iwork(ncount)==0 )
             end if  ! ( i==1 )
             go to 100

         end if

      end do  ! while ( i<=ncount )
      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, ncount, 'after evaluation of functions' )
         call prinrl ( work, ncount, 'after evaluation of functions' )

      end if
150   do i = 1, ncount

!     --- Next all powers are evaluated in order to satisfy the
!         priority rules

         if ( iwork(i)==3 ) then

!        --- iwork(i)=3: power
!            work(i-1) := work(i-1)**work(i)
!            shift work and iwork

             work(i-1) = work(i-1)**work(i)
             ncount = ncount-1
             do j = i, ncount
                work(j) = work(j+1)
                iwork(j) = iwork(j+1)
             end do
             work(ncount+1) = 0d0
             iwork(ncount+1) = 0
             go to 150

         end if

      end do
      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, ncount, 'after evaluation of power' )
         call prinrl ( work, ncount, 'after evaluation of power' )

      end if

!     --- For i := ncount (-1) 2 do

200   do i = 2, ncount

!     --- Next all multiplications and subdivisions

         if  ( iwork(i)==2 ) then

!        --- iwork(i)=2: multiplication
!            work(i-1) := work(i)*work(i-1)
!            shift work and iwork

             work(i-1) = work(i)*work(i-1)
             ncount = ncount-1
             do j = i, ncount
                work(j) = work(j+1)
                iwork(j) = iwork(j+1)
             end do
             work(ncount+1) = 0d0
             iwork(ncount+1) = 0
             go to 200

         else if ( iwork(i)==-2 ) then

!        --- iwork(i)=-2: division
!            work(i-1) := work(i-1)/work(i)
!            shift work and iwork

             if ( abs(work(i))<=0d0 ) then

!            --- Division by zero

                call errchr ( text, 1 )
                call errsub ( 1693, 0, 0, 1 )

             else

!            --- Standard division

                work(i-1) = work(i-1)/work(i)

             end if
             ncount = ncount-1
             do j = i, ncount
                work(j) = work(j+1)
                iwork(j) = iwork(j+1)
             end do
             work(ncount+1) = 0d0
             iwork(ncount+1) = 0
             go to 200

         end if

      end do
      if ( debug ) then

!     --- Debug information

         call prinin ( iwork, ncount, 'after evaluation of multiply' )
         call prinrl ( work, ncount, 'after evaluation of multiply' )

      end if

!     --- result := 0

      result = 0d0

!     --- For i := 1 (1) ncount do
!           result := result+iwork(i)*work(i)

      do i = 1, ncount
         if ( iwork(i)==0 ) then
            result = result+work(i)
         else
            result = result+iwork(i)*work(i)
         end if
      end do

1000  call erclos ( 'readexpr' )
      if ( debug ) write(irefwr,*) 'End readexpr'

      end
