      subroutine readexpression ( text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2013 Extra parameter text
!        version  1.4  date 08-02-2008 Extra debug
!        version  1.3  date 28-12-2005 Replace part by subroutine
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read expression between { and } and translate into a real or
!     integer number. Set ityprd equal to 1 or 2 and fill realrd
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) text

!     text           i    String variable containing some text for error message
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxlevels, lenwork
      parameter ( maxlevels=10, lenwork=200 )
      integer iwork(lenwork), levels(maxlevels), nlevel, icount, &
              ncount, i, ispeclsav
      double precision work(lenwork)
      logical finished, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     ispeclsav      Help parameter to save the value of ispecl
!     iwork          integer work array of length ncount
!                    iwork contains the operators that precede the numbers
!                    according to the following code:
!                     -2:  division
!                     -1:  subtraction
!                      1:  addition
!                      2:  multiplication
!                      3:  raise to the power
!                     10:  sqrt
!                     11:  sin
!                     12:  cos
!                     13:  exp
!                     14:  tan
!     lenwork        Declared length of array work
!     levels         Array defining the starting addresses of each level
!                    Each time an opening bracket ( is found a new level is
!                    created, each time a closing bracket is found the
!                    level is decreased by one
!     maxlevels      Size of array levels
!     ncount         Counter
!     nlevel         Number of levels stored in levels
!     work           Work array of length ncount
!                    work contains the numbers that must be used to apply
!                    the operation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READCN         Read name of constant and substitute value
!     READCONSTANT   read the name of a constant behind the \ sign and
!                    substitute value
!     READEXPR       Read subexpression not containing any brackets and
!                    translate into a real or integer number
!     READEXPRCLOS   Interpret last part of an expression, &
!                    possibly closed by brackets
!     READNAME       Read name following %-sign
!     READNM         Read number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2069   Error opening bracket { found as last symbol in record
!    2385   Name found following the % is not a vector name
!    2386   % is not allowed in expression
!    2388   Number is not preceded by an operator
!    2389   Two succeeding operators found
!    2390   opening bracket in an expression is not preceded by an operator
!    2391   Closing bracket found in expression, but no corresponding
!           opening bracket
!    2392   Closing bracket } found in expression, but there is a
!           closing bracket ) missing
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays iwork and work
!     finished := false
!     nlevel := 1; levels(nlevel) := 1; icount = 1
!     while ( not finished ) do
!        Read next item
!        if ( item = number or $ or % or \ ) then
!           evaluate number
!           if ( iwork(icount) = 0 ) then
!              if ( levels(nlevel) = icount ) then
!                 iwork(icount) := 1
!              else ( levels(nlevel)<icount )
!                 error: no operator before number
!           work(icount) := number
!           icount := icount+1
!        else if ( item = operator ) then
!           if ( iwork(icount) = 0 ) then
!              Fill value of operator in iwork(icount)
!           else
!              error: two succeeding operators
!        else if ( item = ( ) then
!           if ( iwork(icount)=0 and levels(nlevel)#icount) then
!              error: Not an operator before (
!           nlevel := nlevel+1; icount := icount+1
!           levels(nlevel) := icount
!        else if ( item = ) ) then
!           if ( nlevel=1 ) then
!              error: no opening bracket
!           else
!              evaluate expression between ( and ) by subroutine readexpr
!              Clear end of iwork and work, &
!              Decrease icount and nlevel
!              Store result in work
!        else if ( item = } ) then
!           if ( nlevel>1 ) then
!              error: no closing bracket
!           else
!              evaluate final expression by subroutine readexpr
!              finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexpression' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexpression'
         call prinin ( typeitems, nitems, 'typeitems' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays iwork and work

      do i = 1, lenwork
         iwork(i) = 0
         work(i) = 0d0
      end do

!     --- finished := false
!         nlevel := 1; levels(nlevel) := 1; icount = 1

      finished = .false.
      nlevel = 1
      icount = 1
      levels(nlevel) = icount

!     --- while ( not finished ) do

      do while ( .not. finished )

!     --- Read next item

         lastitem = lastitem + 1
         if ( debug ) write(irefwr,*) 'lastitem, nlevel, icount ', &
                                       lastitem, nlevel, icount

         if ( lastitem>nitems ) then

!        --- Error opening bracket { found as last symbol in record

            call errsub ( 2069, 0, 0, 0 )
            call errcrd
            go to 1000

         end if
         if ( typeitems(lastitem)<=2 .or. &
              typeitems(lastitem)==9 .or. &
              typeitems(lastitem)==10 .or. &
              typeitems(lastitem)==14 ) then

!        --- item = number or $ or % or \
!            evaluate number

            ispeclsav = ispecl
            if ( typeitems(lastitem)<=2 ) then

!           --- Number found

               ispecl = 12
               call readnm

            else if ( typeitems(lastitem)==9 ) then

!           --- $ found, replace constant by number

               ispecl = 12
               call readcn

            else if ( typeitems(lastitem)==10 ) then

!           --- % found, replace vector name by sequence number
!               provided ispecl = 12

               if ( ispeclsav==13 ) then

!              --- ispecl = 13, read name of vector

                  ispecl = 13
                  call readname
                  if ( ityprd/=2 ) then

!                 --- Name found is not a vector

                     call errchr ( string(1:nchar), 1 )
                     call errsub ( 2385, 0, 0, 1 )
                     call errcrd

                  end if

               else

!              --- % found in a case that it is not allowed

                  call errsub ( 2386, 0, 0, 0 )
                  call errcrd

               end if

            else

!           --- \ found, replace constant by number or find special function

               ispecl = 12
               call readconstant ( iwork, icount )

            end if
            ispecl = ispeclsav
            if ( debug ) then
               write(irefwr,*) 'number found'
               call prinin ( iwork, icount, 'iwork' )
               call prinin ( levels, nlevel, 'levels' )
            end if  ! ( debug )

            if ( iwork(icount)==0 ) then

!           --- iwork(icount) = 0, hence no operator found yet

               if ( levels(nlevel)==icount ) then

!              --- levels(nlevel) = icount, so start of a new part between
!                  brackets or a minus sign found
!                  iwork(icount) := 1

                  iwork(icount) = 1

               else

!              --- levels(nlevel)<icount
!                  error: no operator before number

                  call erreal ( realrd, 1 )
                  call errsub ( 2388, 0, 1, 0 )
                  call errcrd
                  iwork(icount) = 1

               end if

            end if
            if ( debug ) then
               write(irefwr,*) 'possible operator added'
               call prinin ( iwork, icount, 'iwork' )
               call prinin ( levels, nlevel, 'levels' )
            end if  ! ( debug )

!           --- work(icount) := number
!               icount := icount+1

            if ( iwork(icount)<10 ) then

!           --- Not a function found

               work(icount) = realrd
               icount = icount+1

               if ( debug ) then
                  call prinin ( iwork, icount, 'iwork' )
                  call prinin ( levels, nlevel, 'levels' )
                  call prinrl ( work, icount, 'work' )
               end if  ! ( debug )

            end if  ! ( iwork(icount)<10 )

         else if ( typeitems(lastitem)==8 .or. &
                   typeitems(lastitem)==7 .or. &
                   typeitems(lastitem)==11 .or. &
                   typeitems(lastitem)==12 .or. &
                   typeitems(lastitem)==19 ) then

!        --- item = operator ( + * - / )
!            later on ** and ^

            if ( iwork(icount)==0 ) then

!           --- iwork(icount) = 0, so not an operator found yet
!               Fill value of operator in iwork(icount)

               if ( typeitems(lastitem)==8 ) then

!              --- + sign found

                  iwork(icount) = 1

               else if ( typeitems(lastitem)==7 ) then

!              --- * sign found

                  if ( lastitem<nitems .and. &
                       typeitems(lastitem+1)==7 ) then

!                 --- * is followed by another *, so ** = power

                     iwork(icount) = 3
                     lastitem = lastitem+1

                  else

!                 --- * is not followed by another * &

                     iwork(icount) = 2

                  end if

               else if ( typeitems(lastitem)==12 ) then

!              --- / sign found

                  iwork(icount) = -2

               else if ( typeitems(lastitem)==19 ) then

!              --- ^ sign found

                  iwork(icount) = 3

               else

!              --- - sign found

                  iwork(icount) = -1

               end if

            else

!           --- error: two succeeding operators

               call errsub ( 2389, 0, 0, 0 )

            end if

            if ( debug ) then
               write(irefwr,*) 'operator found'
               call prinin ( iwork, icount, 'iwork' )
               call prinin ( levels, nlevel, 'levels' )
               call prinrl ( work, icount, 'work' )
            end if  ! ( debug )

         else if ( typeitems(lastitem)==5 ) then

!        --- item = (, i.e. new level

            if ( iwork(icount)==0 .and. levels(nlevel)/=icount ) &
               then

!           --- iwork(icount)=0 and levels(nlevel)#icount
!               error: Not an operator before (

               call errsub ( 2390, 0, 0, 0 )
               call errcrd
               iwork(icount) = 1

            else if ( iwork(icount)==0 ) then

!           --- Start with ( instead of an operator, hence + &

               iwork(icount) = 1

            end if

!           --- nlevel := nlevel+1; icount := icount+1
!               levels(nlevel) := icount

            nlevel = nlevel+1
            icount = icount+1
            levels(nlevel) = icount

            if ( debug ) then
               write(irefwr,*) 'openings bracket'
               call prinin ( iwork, icount, 'iwork' )
               call prinin ( levels, nlevel, 'levels' )
               call prinrl ( work, icount, 'work' )
            end if  ! ( debug )

         else if ( typeitems(lastitem)==6 ) then

!        --- item = ), i.e. end of level

            if ( nlevel==1 ) then

!           --- nlevel=1
!               error: no opening bracket

               call errsub ( 2391, 0, 0, 0 )
               call errcrd

            else

!           --- evaluate expression between ( and ) by subroutine readexpr

               call readexprclos ( levels, nlevel, icount, iwork, work, &
                                   text )

            end if

            if ( debug ) then
               write(irefwr,*) 'Closing bracket'
               call prinin ( iwork, icount, 'iwork' )
               call prinin ( levels, nlevel, 'levels' )
               call prinrl ( work, icount, 'work' )
            end if  ! ( debug )

         else if ( typeitems(lastitem)==18 ) then

!        --- item = }, so end of expression

            if ( nlevel>1 ) then

!           --- nlevel>1
!               error: no closing bracket

               call errsub ( 2392, 0, 0, 0 )
               call errcrd

            else

!           --- evaluate final expression by subroutine readexpr

               ncount = icount-1
               if ( debug ) then
                  write(irefwr,*) 'final expression'
                  call prinin ( iwork, ncount, 'iwork' )
                  call prinrl ( work, ncount, 'work')
               end if

               call readexpr ( work, iwork, ncount, realrd, text )

               if ( debug ) &
                  write(irefwr,*) 'ityprd, realrd ', ityprd, realrd

!              --- Check if result is real or integer

               if ( abs(realrd)<=0d0 ) then

!              --- Result = 0, hence integer

                  ityprd = 2

               else if ( abs(realrd)>1d7 .or. &
                         abs(realrd)<1d0-1d-6 ) then

!              --- Result large or between 0 and 1, hence real

                  ityprd = 3

               else if ( abs(realrd-nint(realrd))<abs(realrd)*1d-10 ) &
                  then

!              --- nint(realrd) = realrd, hence integer

                  ityprd = 2

               else

!              --- Other case, hence real

                  ityprd = 3

               end if

            end if

!           --- finished := true

            finished = .true.

         end if

      end do
      if ( debug ) write(irefwr,*) 'ityprd, realrd ', ityprd, realrd

1000  call erclos ( 'readexpression' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readexpression'

      end if  ! ( debug )

      end
