      subroutine readmd ( iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.14 date 20-06-2013 New definition file ref numbers
!        version  1.13 date 21-01-2004 Fortran 90 change
!        version  1.12 date 11-01-2001 Reset irecnr
!
!   copyright (c) 1990-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Read default values from sepran environment file (sepran.env) and store
!  these quantities in some common blocks which
!  The SEPRAN environment file may be in the local directory of the
!  user or in the SEPRAN main directory (SPHOME)
!  First it is checked if a local environment file exists named SPRENV, &
!  if not the global environment file is used
! **********************************************************************
!
!                       KEYWORDS
!
!     default
!     environment_file
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cstandf90'
      include 'SPcommon/constaf90'
      include 'SPcommon/cmachnf90'
      include 'SPcommon/cmachtf90'
      include 'SPcommon/cfile3f90'
      include 'SPcommon/cfile2f90'
      include 'SPcommon/cmcdppf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/creffilesf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref

!     iref     i  Reference number with respect to the environment file
!                 This reference number is only used locally, the file
!                 is opened and closed in this subroutine. Afterwards the
!                 reference number may be reused.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenspr, lenhom, irefrt, ios, idum, i, ivers
      logical check
      character (len = 200) namefl

!    check     Indicates if a file exists (true) or not (false)
!    i         general loop variable
!    idum      Dummy parameter
!    ios       Indication if error has been found in reading or opening of
!              file (#0) or not (0)
!    irefrt    Help variable to store irefre temporarily
!    ivers     Version number of environment file
!    lenhom    Number of characters in the name of the home directory
!    lenspr    Number of characters in the name of the environment file
!    namefl    Help variable to store a name temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INISETREFS     Put reference number in creffiles
!     READM0         Read a name from the environment file
!     READM1         Read an integer from the environment file
!     READM2         Read a real from the environment file
!     READM3         Read whether a file referred to in the environment file is
!                    formatted (>0) or unformatted (<0)
!     READM4         Read the version number from the environment file
! **********************************************************************
!
!                       I/O
!
!      READMD reads the environment file, which must be available, &
!      either locally or globally
!      The enviroment file contains the following quantities (the sequence
!      is fixed)
!      SPHOME
!      version number
!      stoper
!      macnam
!      machin
!      spver
!      spexec
!      numchr
!      rinfin
!      epsmac
!      sqreps
!      irefre
!      irefwr
!      irefer
!      iref1
!      iref2
!      iref3
!      iref17
!      irefpl
!      iref10
!      iref73
!      iref74
!      indication of formatted or unformatted plot file
!      indication of formatted or unformatted mesh output file
!      indication of formatted or unformatted sepcomp.out file
!      spappn
!      namef1
!      namef4
!      namef3
!      namems
!      namere
!      namewr
!      name10
!      name73
!      name74
!      namef2
!      lenwor
!      nrec1
!      nrec2
!      nrec3
!      defplo
!      cm
!      small
!      wide
!      aleng
!      posdev
!      namepl
!      devdis
!      devhcp
!      icarcn
!      isite
!      9 dummies
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check if there is a local environment file
!     Open environmnet file
!     Read and check all quantities if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set some constants
!         The text that is read from the environment file will not be printed

      ispecl = 6
      lenspr = index ( sprenv, ' ' ) - 1
      lenhom = index ( sphome, ' ' ) - 1
      ioutp  = -1
      devhcp = 'p'
      namems = ' '
      iref17 = 0

!     --- Several characters, used to control terminals with

      null = char(0)
      escape = char(27)
      formfd = char(12)
      mgraph = char(29)
      malpha = char(31)
      mvt100 = char(24)
      iplotf = 0

!     --- Check for the presence of a local environment file

      inquire ( file = sprenv(1:lenspr), exist = check )

      if ( .not. check ) then

!     --- Local environment file does not exist, use global file
!         At this moment sphome does contain the name of the home directory
!         of the global environment file, including a possible / or \

         namefl(1:lenhom) = sphome(1:lenhom)
         namefl(lenhom+1:lenspr+lenhom) = sprenv(1:lenspr)
         lenspr = lenspr + lenhom
         sprenv(1:lenspr) = namefl(1:lenspr)

      end if

!     --- Open the environment file

      open ( unit = iref, file = sprenv(1:lenspr), status='old', &
             iostat = ios )
      if ( ios/=0 ) then

!     --- ios # 0, environment file does not exist, or is incorrect

         write(*,100) sprenv(1:lenspr), ios
100      format(//' The SEPRAN environment file ',a,' can not be', &
                  ' opened correctly'/ &
                  ' Either the file does not exist or is incorrect'// &
                  ' Execution will be terminated' /// &
                  ' Extra information: iostatus is',i4 )
         stop

      end if

!     --- Read SEPRAN home directory
!         The preceding file is overwritten
!         Letters are not translated into capitals
!         Next read operating system, machine name, version and name executable
!         Plus type of computer

      irefre = iref
      irefwr = 6
      namebm = ' '
      call readm0 ( macnam )
      sphome = sphome(1:lenhom-8)
      call readm4 ( ivers, stoper )
      if ( ivers>1 ) call readm0 ( stoper )
      call readm0 ( macnam )
      call readm1 ( machin )
      call readm0 ( spvers )
      call readm0 ( spexec )

!     --- Read numchr, rinfin, epsmac, sqreps

      call readm1 ( numchr )
      call readm2 ( rinfin )
      call readm2 ( epsmac )
      call readm2 ( sqreps )

!     --- Read irefre, irefwr, irefer, iref1, iref2, iref3, irefpl, iref10, &
!              iref73, iref74
!              irefre is temporarily stored in ireft

      call readm1 ( irefrt )
      call readm1 ( irefwr )
      call readm1 ( irefer )
      call readm1 ( iref1  )
      call readm1 ( iref2  )
      call readm1 ( iref3  )
      if ( ivers>1 ) call readm1 ( iref17 )
      call readm1 ( irefpl )
      call readm1 ( iref10 )
      call readm1 ( iref73 )
      call readm1 ( iref74 )

!     --- Read indication of formatted or unformatted plot file, &
!              indication of formatted or unformatted mesh output file, &
!              indication of formatted or unformatted sepcomp.out file

      call readm3 ( irefpl )
      call readm3 ( iref10 )
      call readm3 ( iref74 )

!     --- Read append mode output file

      call readm0 ( spappn )

!     --- Read names of files: namef1, namef4, namef3, namems, namere, &
!                            namewr, name10, name73, name74, namef2

      call readm0 ( namef1 )
      call readm0 ( namef4 )
      if ( ivers>1 ) call readm0 ( namems )
      call readm0 ( namef3 )
      call readm0 ( namere )
      call readm0 ( namewr )
      call readm0 ( name10 )
      call readm0 ( name73 )
      call readm0 ( name74 )
      call readm0 ( namef2 )

!     --- Read extra information about files: lenwor, nrec1, nrec2, nrec3

      call readm1 ( lenwor )
      call readm1 ( nrec1  )
      call readm1 ( nrec2  )
      call readm1 ( nrec3  )

!     --- Read plot information: defplo, cm, small, wide, aleng, posdev, &
!                                namepl, devdis

      call readm2 ( defplo )
      call readm2 ( cm     )
      call readm2 ( small  )
      call readm2 ( wide   )
      call readm2 ( alengm )
      call readm0 ( posdev )
      call readm0 ( namepl )
      call readm0 ( devdis )
      if ( ivers>1 ) call readm0 ( devhcp )
      if ( posdev=='fa' ) irefpl = abs(irefpl)
      if ( posdev=='fb' ) irefpl = -abs(irefpl)

!     --- Read carriadge control parameter

      call readm0 ( namefl )
      if ( namefl(1:3)=='yes' ) then
         icarcn = 1
      else
         icarcn = 0
      end if

!     --- Read site number isite

      call readm1 ( isite )

!     --- Read last 9 dummies

      do 200 i = 1, 9
         call readm1 ( idum  )
200   continue

!     --- Fill irefre

      close ( iref )
      irefre = irefrt
      ioutp = 0
      ispecl = 0
      ityprd = 1
      irecnr = 0

!     --- creffiles

      ireffiles = 0

      inextref = 1
      if ( abs(iref1)>0 ) call inisetrefs ( abs(iref1) )
      if ( abs(iref2)>0 ) call inisetrefs ( abs(iref2) )
      if ( abs(irefer)>0 ) call inisetrefs ( abs(irefer) )
      if ( abs(irefre)>0 ) call inisetrefs ( abs(irefre) )
      if ( abs(irefwr)>0 ) call inisetrefs ( abs(irefwr) )
      if ( abs(iref10)>0 ) call inisetrefs ( abs(iref10) )
      if ( abs(iref17)>0 ) call inisetrefs ( abs(iref17) )
      if ( abs(iref73)>0 ) call inisetrefs ( abs(iref73) )
      if ( abs(iref74)>0 ) call inisetrefs ( abs(iref74) )
      if ( abs(irefpl)>0 ) call inisetrefs ( abs(irefpl) )

      end
