      subroutine readslbf ( ibuffr, buffer, ivectr, kmesh, kprob, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 29-05-2014 remove call to time subroutine
!        version  1.2  date 15-11-2012 Long integers
!        version  1.1  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read real/complex  sepran solution array except the large matrix
!   only solution vectors of type 110, 115 and 116 are allowed
!   in case of formatted files the standard format 5d16.9 is used
!   This subroutine is meant to be used in combination with subroutine
!   prinsl.
!   prinsl writes a solution in character format to a file, &
!   readsl reads the solution
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kprob(*), ivectr(*), iref, ibuffr(*)
      double precision buffer(*)

!     buffer i/o  Real buffer array in which all large data is stored
!     ibuffr i/o  Integer buffer array in which all large data is stored
!     iref    i   Reference number of file to which solution must be written
!                 possibilities:  see user manual
!                 if iref<0, the file is read unformatted, otherwise it is
!                 supposed to be formatted
!     ivectr  o   integer array denoting the array to be readed
!     kmesh   i   array containing information of the mesh
!     kprob   i   array containing information of degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer (kind=8)  ipvect, ipkprh
      integer ihelp(5), ios, i, itypvc, ivec, nusol, nunkp, iprob, k, &
              nprob, ipkprb, indprf, indprh, indprp, length

!     compl          Indication if the solution vector is complex (true) or
!                    not (false)
!     i              General loop variable
!     ihelp          Help array to store the first 5 integers of the vector to
!                    be read
!     indprf         Sequence number of array KPROB part F with respect to infor
!     indprh         Sequence number of array KPROB part H with respect to infor
!     indprp         Sequence number of array KPROB part P with respect to infor
!     ios            IOSTAT variable
!     ipkprb         Starting address of present vector KPROB in KPROB
!     ipkprh         Starting address of KPROB part h in IBUFFR
!     iprob          Problem number
!     ipvect         Starting address of solution vector in IBUFFR
!     itypvc         Indicates type of vector 110 or 115
!     ivec           Sequence number of vector of special structure
!     k              General loop variable
!     length         Length of array
!     nprob          Number of problems
!     nunkp          Maximal number of degrees of freedom per point
!     nusol          Number of unknowns in vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget, inidgt
      integer inidln

!     ER0045         Actual reading and filling of real vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI087         compute pointers to degrees of freedom of vectors
!     INIDGT         Compute pointer of array in BUFFER
!     INIDLN         Compute length of array in BUFFER
!     INIGET         Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!    The solution vector or vector of special structure is read from the
!    file with reference number iref74
!    If iref74>0, the file is formatted, otherwise it is unformatted
!    The first record contain the 5 integers that must be stored into array
!    IVECTR, the second record contains the real values
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      60   Array to be read is of wrong type
!      75   Array KMESH has not been filled
!     699   IPROB has incorrect value
!     747   End of file reached
!     748   Error in reading from file
!    2023   The mesh does not correspond to the vectors read
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readslbf' )
      if ( kmesh(2)/=100 ) then
         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
      end if
      if ( kprob(2)/=101 ) then
         call errint ( kprob(2), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      if ( ierror/=0 )  go to 1000
      if ( iref>0 ) then
         read ( iref, 10, iostat=ios ) (ihelp(i),i=1,5)
 10      format(5i6)
      else
         read ( -iref, iostat=ios ) (ihelp(i),i=1,5)
      end if
      if ( ios<0 ) then

!     --- ios<0, end of file reached

         call errsub ( 747, 0, 0, 0 )
         go to 1000

      else if ( ios>0 ) then

!     --- ios>0, Read error

         call errint ( ios, 1 )
         call errsub ( 748, 1, 0, 0 )
         go to 1000
      end if

      itypvc = ihelp(2)
      ivec = mod(ihelp(4),1000)
      nusol = ihelp(5)
      if ( ivectr(2)/=itypvc ) ivectr(1) = 0
      if(mod(ihelp(3),10)==0) then
         compl = .false.
      else
         compl = .true.
      end if
      do i = 2, 5
         ivectr(i) = ihelp(i)
      end do
      if ( ivectr(2)/=110 .and. ivectr(2)/=115 &
                            .and. ivectr(2)/=116 ) then
         call errint ( ivectr(2), 1 )
         call errsub ( 60, 1, 0, 0 )
      end if
      iprob  = ivectr(4) / 1000 + 1
      nprob  = max ( 1, kprob(40) )
      if ( iprob<=0 .or. iprob>nprob ) then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0 )
      end if
      ipkprb = 1
      do k = 2, iprob
         ipkprb = ipkprb + kprob ( 2+ipkprb )
      end do
      call ini087 ( ibuffr, ivectr, itypvc, ivec, iprob, kmesh, &
                    kprob, indprf, indprh, indprp, nunkp, 'vector' )

      if ( ierror/=0 )  go to 1000
      ipvect = inidgt(ivectr(1))
      length = inidln(ivectr(1))
      if ( compl ) length = length/2
      if ( length/=nusol ) then

!     --- Error: incorrect length, probably incorrect mesh

         call errint ( nusol, 1 )
         call errint ( length, 2 )
         call errsub ( 2023, 2, 0, 0 )
         go to 1000

      end if

      if ( indprh==0 ) then
         ipkprh = 1
      else
         ipkprh = iniget(indprh)
      end if
      call er0045 ( buffer(ipvect), ibuffr(ipkprh), nusol, compl, &
                    indprh, iref )
1000  call erclos ( 'readslbf' )
      end
