      subroutine readsr ( dnumbr, maxnum, inum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 14-11-2013 New call to prevalexpr
!        version  1.7  date 11-10-2013 Extra debug
!        version  1.6  date 20-04-2012 Extra debug
!
!   copyright (c) 1990-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read a series of numbers
!     The numbers may be alternated with texts, which are skipped
!     The reading of numbers is stopped if
!     a ) sign is found
!     an end of record is found
!     maxnum numbers have been read
!     If a minus sign followed by text followed by a number is read, &
!     the number is made negative
!     After reading the numbers, icolum points to the next position to be read
!     If maxnum<0, texts are used as separater and no numbers are also allowed
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxnum, inum
      double precision dnumbr(*)

!     dnumbr  o  Array to be filled with the numbers
!     inum    o  Number of numbers that have been actually read
!     maxnum  i  |maxnum| defines the maximum number of numbers to be read
!                If maxnum>0, texts are skipped
!                If maxnum<0, texts are used as separater
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kcolsv
      logical debug, finished, minsgn, openfound, closefound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     kcolsv         Help parameter to save the old value of kcolum
!     minsgn         Sign that has been read. If no sign is read minsgn=1, &
!                    if a - has been found minsgn = -1
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READSREXPR     Interprete part of input as number or expression
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear array dnumbr
!   finished  := false
!   inum   := 0
!   ispecl := 4
!   minsgn := false
!   while ( not finished ) do
!      Read next item (ASKNXT)
!      if ( ityprd = 2 or 3 ) then
!         inum := inum+1
!         if ( minsgn ) then realrd := -realrd
!         minsgn := false
!      else if ( ityprd = 6 ) then
!         minsgn := true
!      else if ( ityprd = 7 or 0 or 1 ) then
!         finished := true
!      if ( inum = maxnum ) then
!         finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Make some initializations

      call eropen ( 'readsr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsr'
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array dnumbr

      dnumbr(1:abs(maxnum)) = 0

      finished = .false.
      inum   = 0
      ispecl = 4
      minsgn = .false.
      kcolsv = kcolum
      openfound = .false.
      closefound = .false.

      do while ( .not. finished )

!     --- While loop
!         Read next item, skip equals sign

         call readsrexpr ( openfound, closefound, .false., finished )
         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            write(irefwr,2) 'realrd', realrd
            write(irefwr,*) 'openfound, closefound', &
                             openfound, closefound
         end if  ! ( debug )
         if ( ierror/=0 ) go to 1000

         if ( ityprd==2 .or. ityprd==3 .or. ityprd==11 ) then

!        --- ityprd = 2 or 3, number found

            inum = inum + 1
            if ( minsgn .and. realrd>0 ) realrd = - realrd
            minsgn = .false.
            dnumbr(inum) = realrd

         else if ( ityprd==6 ) then

!        --- ityprd = 6, minus sign found

            minsgn = .true.

         else if ( openfound .and. .not. closefound ) then

!        --- open found but not close found, hence not finished

            if ( lastitem<nitems ) then

!           --- not yet last item

               lastitem = lastitem+1
               finished = .false.

            end if  ! ( lastitem<nitems )

         end if

         if ( inum>=abs(maxnum) .or. lastitem>=nitems ) then

!        --- inum = maxnum, stop reading

            finished = .true.

         end if

      end do  ! while ( .not. finished )

      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

1000  call erclos ( 'readsr' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( dnumbr, inum, 'dnumbr' )
         write(irefwr,*) 'End readsr'

      end if  ! ( debug )

      end
