      subroutine mesh3d ( nbound, jnew, n, nipnt, fix_pnt, &
                          curvenumbers, coorcurvs, surfnumbers )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  9.2  date 20-11-2013 Remove mshg3d
!        version  9.1  date 28-08-2013 renumber in case of disjoint parts
!        version  9.0  date 03-09-2010 Dynamic allocation help arrays
!
!   copyright (c) 1988-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to make a mesh of tetrahedrons for a volume which is
!     given by the triangles of kbndptlc. Elements are either linear or
!     quadratic. MESH3D is the driving subroutine.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: n, nbound, nipnt, curvenumbers(*), surfnumbers(*)
      double precision :: fix_pnt(*), coorcurvs(3,*)
      logical jnew

!     coorcurvs      i    Contains the coordinates of the extra curves
!     curvenumbers   i    Contains the extra curve numbers
!     fix_pnt        i    Real array of size 4 x nextrapnts containing the
!                         coordinates and coarseness of the fixed points
!                         in the mesh
!     jnew           i    indication whether the mesh is new (true) or old
!     n              i    Help parameter to compute length of arrays, see msh043
!     nbound         i    Number of elements in kbndptlc
!     nipnt          i    Number of points in boundary
!     surfnumbers    i    Array of size nextrasrfs containing the surface
!                         sequence numbers of each extra surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoh, nelemh

!     nelemh         Estimate for the number of elements in the mesh
!     npoh           Estimate of the number of nodes in the mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MESH3DPRININ   Print input of subroutine mesh3d
!     MSH313         Fill centroid for 9-point hexahedron and 5-point
!                    tetrahedron
!     MSH641         Fill elements and nodes for shape 15
!     MSHCOORCENTR   Create centre point in 15-point tetrahedrons
!     MSHG3K         Fill elements and nodes for shape 12
!     MSHGENESTIMATE Compute estimates for npointlc and nelemlc
!     MSHPHIL3D      Fill elements and nodes for shape 11
!     MSHRENDISJOIN  Renumber nodes in submesh in case the submesh consists of
!                    disjoint parts
!     MSHTRIANCENTR  Create centre point in 6-point triangles
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('mesh3d')
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mesh3d'

!        --- Print all input

         call mesh3dprinin ( coor3d, nbound, fix_pnt, &
                             curvenumbers, coorcurvs, surfnumbers )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- estimate number of points and elements

      call mshgenestimate ( n, npoh, nelemh, nbound )

      if ( ishape==11 .or. ishape==18 ) then

!     --- Linear tetrahedrons or Linear tetrahedrons plus extra point

         call mshphil3d ( kbndptlc, nbound, kmeshc, &
                          jnew, nipnt, fix_pnt, &
                          numnodextcurvs, curvenumbers, coorcurvs, &
                          surfnumbers  )

!        --- Second step: for test only repositioning:

         if ( .false. ) then
         write(irefwr,*) 'tussentest'
         call mshphil3d ( kbndptlc, nbound, kmeshc, &
                          .false., nipnt, fix_pnt, &
                          numnodextcurvs, curvenumbers, coorcurvs, &
                          surfnumbers  )

         end if  ! ( ivol==11 )

         inpelm = 4

         if ( debug ) then
            write(irefwr,*) 'Mesh3d: from mshphil3d: '
            write(irefwr,1) 'npointlc, nelemlc ', npointlc, nelemlc
         end if  ! ( debug )

         if ( ishape==18 ) then
            call msh313 ( kmeshc, coor3d, nelemlc, npointlc, 5 )
            inpelm = 5
         end if  ! ( ishape==18 )

      else if ( ishape==12 ) then

!     --- Quadratic tetrahedrons

         inpelm = 10
         call mshg3k ( nbound, jnew, nipnt, &
                       fix_pnt, curvenumbers, coorcurvs, surfnumbers  )

      else if ( ishape==15 .or. ishape==16 ) then

!     --- Quadratic tetrahedrons with extra points in centroids of surface
!         element

         call msh641 ( nbound, jnew, nipnt, &
                       fix_pnt, curvenumbers, coorcurvs, surfnumbers  )
         if ( ierror/=0 ) go to 1000
         if ( ishape==16 ) then

!        --- ishape = 16, create centroids of elements with 15 nodes

            call mshtriancentr ( kmeshc, npointlc, nelemlc, 15 )
            call mshcoorcentr ( kmeshc, coor3d, nelemlc, 3, 15 )
            inpelm = 15

         else

!        --- ishape = 15

            inpelm = 14

         end if
      else
         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
      end if

!     --- Check if mesh consists of subparts
!         If so renumber nodes

      npoint = npointlc
      nelem = nelemlc
      ndim = 3
      call mshrendisjoin ( kmeshc, coor3d, inpelm )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Write debug information: end mesh3d'
      end if
      call erclos('mesh3d')

      end
