      subroutine msh476 ( kstap , kstapl, coor , npoint, nelem, kelem, &
                          klaar, nlind, nipnt, ivolnr, xminvec )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 06-12-2013 Extra parameter ivolnr
!        version  4.0  date 03-09-2010 New release
!        version  3.4  date 08-02-2010 Compute reference for ia values (WS)
!        version  3.3  date 10-11-2009 Extra checks added (NP)
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine MSH476 constructs a grid of linear tetrahedrons if only
!     the boundary triangles and the nodes in a 3-D domain are given. Use
!     is made of a special TESSELATION property and of the advancing
!     boundary technique.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tesselation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(4,*), kstapl(3,*), kstap, nelem, nipnt, &
              npoint, klaar(npoint), nlind, ivolnr
      double precision coor(3,npoint), xminvec(4)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ivolnr     i    Volume sequence number
!     kelem     i/o   array containing the nodes of the tetrahedrons
!                     length of this 2d-array at output : (4,nelem)
!     klaar           work array of length npoint for detecting that
!                     all work is done
!     kstap     i/o   number of triangles constituting the actual
!                     boundary : at output this variable should be 0
!     kstapl     i    helparray for actual boundary of region, &
!                     at input containing all boundary triangles, &
!                     and during the process the actual boundary hence
!                     at output empty
!     nelem     i/o   number of tetrahedron elements created
!     nipnt      i    total number of initial points in boundary (important
!                     for quadratic case)
!     nlind      i    declared length array line
!     npoint     i    total number of points in mesh.
!     xminvec    i    Contains the vector xmint, ymint, zmint and tran
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ia, ib, ic, id, icheck, iex, iex1, iex2, iex3, &
              ifout, ja, jpoint, iloop, nwrel, k1, k2, k3, line(nlind)
      double precision angle1, angle2, angle3, radius, volume, volumt, &
                       e1, e2, e3, eps, dx, dy, dz, dis, d1, d2, d3, &
                       x1, x2, x3, y1, y2, y3, z1, z2, z3, &
                       xc, xm, yc, ym, zc, zm, refvol

!     angle1         angle between two faces of tetrahedron
!     angle2         angle between two faces of tetrahedron
!     angle3         angle between two faces of tetrahedron
!     d1             distance
!     d2             distance
!     d3             distance
!     dis            distance
!     dx             x-distance
!     dy             y-distance
!     dz             z-distance
!     e1             first component of inward pointing normal
!     e2             second component of inward pointing normal
!     e3             third component of inward pointing normal
!     eps            accuracy
!     i              loop-variable
!     ia             first node of triangle
!     ib             second node of triangle
!     ic             third node of triangle
!     icheck         indicator for visibility
!     id             help variable for nodes of tet
!     iex            "double" neighbor point of triangle
!     iex1           neighbor point of triangle's first boundary
!     iex2           neighbor point of triangle's second boundary
!     iex3           neighbor point of triangle's third boundary
!     ifout          counter for triangles
!     iloop          counter to make sure loop 240 is not executed endlessly
!     j              loop-variable
!     ja             indicator
!     jpoint         node considered as possible candidate for tetrahedron
!     k1             first node of triangle
!     k2             second node of triangle
!     k3             third node of triangle
!     line           work array of length nlind to store boundary lines
!     nwrel          Threshold value for printing elements
!     radius         radius of circumsphere of tetrahedron
!     refvol         reference volume to compute the accuracy
!     volume         help variable to store the volume of a tetrahedron
!     volumt         total volume of all tets
!     x1             x-coordinate first point triangle
!     x2             x-coordinate second point triangle
!     x3             x-coordinate third point triangle
!     xc             x-coordinate of centre of tetrahedron
!     xm             x-coordinate of barycentre of triangle
!     y1             y-coordinate first point triangle
!     y2             y-coordinate second point triangle
!     y3             y-coordinate third point triangle
!     yc             y-coordinate of centre of tetrahedron
!     ym             y-coordinate of barycentre of triangle
!     z1             z-coordinate first point triangle
!     z2             z-coordinate second point triangle
!     z3             z-coordinate third point triangle
!     zc             z-coordinate of centre of tetrahedron
!     zm             z-coordinate of barycentre of triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSH477         determine inward pointing normal and centre of
!                    circumcircle
!     MSH478         determine radius of circumsphere
!     MSH479         check whether point is visible
!     MSH480         check whether boundary does not intersect itself
!     MSH612         fill element in array and adjust
!     MSH615         compute special neighbour points of triangle
!     MSH632         compute volume of tetrahedron
!     MSH652         Check if all tetrahedrons of kmeshc are disjoint
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1089   Mesh generator in endless loop, skip rest of loop
!    2913   Error with barycenters
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start to check input boundary triangles      msh480
!     Clear array KLAAR
!     Fill array KLAAR
!     Compute reference value xm
!     Check whether the points are inside on the boundary or outside
!     Set actual number of triangles in boundary array
!     While boundary not empty , search for first boundary triangle
!        Check first whether tetrahedron can be formed with other
!        boundary triangles, compute the special neighbours
!        if point possible ready, jump over next loop
!        Run through all possible points and find first allowed point
!            Jump out of loop as soon as possible
!        No point visible : Error, according to the theory this
!        situation should never occur
!           Reorder and start again with next triangle
!        Point found
!        Determine inward pointing normal and centre of circumcircle
!        Determine radius of circumsphere if point jpoint is chosen
!           Check whether other points are
!              - inside the sphere
!              - give pos volume
!              - are visible
!           hence are "better", go on with this point
!        Create tetrahedron for triangle and point found
!        Fill element and adjust array kstapl and kstap
!        Go for next triangle
!     End while
!     Do some final checks
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh476' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh476'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nwrel = (nelem/100 + 1)*100

!     --- Start by checking the local boundary triangles

      call msh480 ( kstapl, kstap, line, nlind, coor, ivolnr, &
                    xminvec, 0 )
      if ( ierror/=0 ) go to 1000

      ifout = 0

!     --- Compute reference value for volume

      ia = min ( kstap, 25 )

      refvol = 0d0

      do i = 1, ia

         k1 = kstapl( 1, ia )
         k2 = kstapl( 2, ia )
         k3 = kstapl( 3, ia )

         x1 = coor( 1, k1 )
         y1 = coor( 2, k1 )
         z1 = coor( 3, k1 )

         x2 = coor( 1, k2 )
         y2 = coor( 2, k2 )
         z2 = coor( 3, k2 )

         x3 = coor( 1, k3 )
         y3 = coor( 2, k3 )
         z3 = coor( 3, k3 )

         d1 = (x2-x1) * (x2-x1) + (y2-y1)*(y2-y1) + (z2-z1)*(z2-z1)
         d2 = (x3-x2) * (x3-x2) + (y3-y2)*(y3-y2) + (z3-z2)*(z3-z2)
         d3 = (x1-x3) * (x1-x3) + (y1-y3)*(y1-y3) + (z1-z3)*(z1-z3)

         d1 = sqrt( d1 )
         d2 = sqrt( d2 )
         d3 = sqrt( d3 )

         refvol = refvol + d1 * d2 * d3

      end do

      refvol = refvol / ia

!     --- determine accuracy parameter

      eps = 1d-10 * refvol

!     --- Fill array KLAAR, 0 if in element, >0 if on boundary, <0 if not yet
!         used
!         Clear array KLAAR, assume all nodes not yet used

      do i = 1, nipnt
         klaar(i) = 0
      end do

      do i = nipnt+1, npoint
         klaar(i) = -1
      end do

!     --- set used in elements

      do i= 1,nelem
         do j=1,4
            ia = kelem(j,i)
            klaar(ia)=0
         end do
      end do

!     --- set used in boundary

      do i = 1 , kstap
         do j=1,3
            ia = kstapl( j,i )
            if (klaar(ia)==-1) klaar(ia)=0
            klaar( ia ) = klaar( ia ) + 1
         end do
      end do

!     --- Set actual number of triangles in boundary array

200   if ( kstap/=0 ) then

!     --- Get boundary triangle and find element
!         Determine nodes of boundary triangle to be considered

         ia = kstapl( 1,1 )
         ib = kstapl( 2,1 )
         ic = kstapl( 3,1 )

!        --- Check first whether tetrahedron can be formed with other
!            boundary triangles, compute the special neighbours

         call msh615( coor, kstapl, kstap, ia, ib, ic, iex1, iex2, iex3, &
                      angle1, angle2, angle3 )

         iex = 0

         if ( iex1==iex2 ) iex = iex1
         if ( iex2==iex3 ) iex = iex2
         if ( iex3==iex1 ) iex = iex3

         if ( iex>0 ) then
            call msh632( coor, ia, ib, ic, iex, volume )
            if ( volume>eps ) then

!           --- Volume is positive, check whether point
!               is "visible" or that there are other points inside

               call msh479( kstapl, kstap, line, nlind, coor, &
                            iex, klaar, npoint, icheck )

               if ( icheck==0 ) then

!              --- Visible point iex found, jump over to next loop

                  jpoint = iex
                  goto 260
               end if
            end if
         end if

!        --- Run through all points of array KLAAR and find point
!            that is allowed

         do i = 1 , npoint

!        --- Check whether point is a candidate
!            No candidate if point of basis

             if ( i==ia.or.i==ib.or.i==ic ) goto 250

!            --- No candidate if not inside or on boundary

             if ( klaar(i)==0 ) goto 250

!            --- Check what will remain

             ja = 0

             if ( i/=iex1.and.i/=iex2.and.i/=iex3 ) then
                call msh632( coor, ia, iex3, ib, i, volume )
                if ( volume<=eps ) ja = 1
                call msh632( coor, ib, iex1, ic, i, volume )
                if ( volume<=eps ) ja = 1
                call msh632( coor, ic, iex2, ia, i, volume )
                if ( volume<=eps ) ja = 1
             end if

!            --- If wrong volume then skip

             if ( ja==1 ) goto 250

!            --- Point is inside or on boundary, check whether volume is pos

             call msh632( coor, ia, ib, ic, i, volume )

!            --- No candidate if volume is negative or zero

             if ( volume<=eps ) goto 250

!            --- Volume is positive, check whether point
!                is visible or that there are points inside

             call msh479( kstapl, kstap, line, nlind, coor, &
                          i, klaar, npoint, icheck )

!            --- Jump out of loop as soon as icheck is not zero

             if ( icheck>0 ) then

!            --- There is a point inside, check whether that
!                point is allowed

                iloop = 0

240             iloop = iloop + 1
                if (iloop>100) then

!               --- This should be a warning as well, can still
!                   work, run check afterwards, a statement should
!                   be included
!                   Error stop!  Adjust later!

                   call errwar ( 1089, 0, 0, 0 )
                   goto 260
                end if

                jpoint = icheck

                call msh479( kstapl, kstap, line, nlind, coor, &
                             jpoint, klaar, npoint, icheck )

!               --- If icheck>0 there are still other points inside
!                   but if icheck = 0 go on with next loop

                if ( icheck>0 ) goto 240

!               --- No other points inside, hence ready for this loop

                if ( icheck==0 ) goto 260

             else if ( icheck==0 ) then

!            --- Determine radius and centre of circumsphere of the four
!                points ia, ib, ic, i

                jpoint = i

                goto 260

             end if

250      end do

!        --- No point visible : Error, according to the theory this
!            situation should only occur in very special cases. No division
!            is possible.

         ifout = ifout + 1

         if ( ifout> min(kstap+2,10) ) then

!        --- Try to divide in the classical way, go back to the
!            calling routine

            goto 1000

         else

!        --- Reorder and start again

            do i = 2 , kstap
               kstapl( 1, i-1 ) = kstapl ( 1, i )
               kstapl( 2, i-1 ) = kstapl ( 2, i )
               kstapl( 3, i-1 ) = kstapl ( 3, i )
            end do

            kstapl ( 1, kstap ) = ia
            kstapl ( 2, kstap ) = ib
            kstapl ( 3, kstap ) = ic

            goto 200

         end if

!        --- Determine inward pointing normal and centre of circumcircle

260      call msh477( coor, ia, ib, ic, e1, e2, e3, xm, ym, zm )

!        --- Determine radius of circumsphere if point jpoint is chosen

         call msh478( coor, ia, jpoint, xm, ym, zm, e1, e2, e3, &
                      xc, yc, zc, radius )

!        --- Check whether other points are
!            - inside the sphere
!            - give a positif volume
!            - are visible

         do i = 1, npoint

!        --- Check whether this point has to be considered

!            --- If outside region then not of interest

             if ( klaar(i)==0 ) goto 280

!            --- If node of basis then of no interest

             if ( i==ia.or.i==ib.or.i==ic.or.i==jpoint ) goto 280

!            --- Check whether point i gives a positive volume

             call msh632( coor, ia, ib, ic, i, volume )

!            --- If volume negative or zero then not of interest

             if ( volume<eps ) goto 280

!            --- Check distance point i to xc,yc,zc

             dx  = coor(1,i) - xc
             dy  = coor(2,i) - yc
             dz  = coor(3,i) - zc

             dis = dx*dx + dy*dy + dz*dz

             if ( dis<radius ) then

!            --- Check whether point i is allowed

                call msh479( kstapl, kstap, line, nlind, coor, &
                             i, klaar, npoint, icheck )
                if ( icheck==0 ) then

!               --- Point i is allowed, adjust jpoint

                    jpoint = i

!                   --- Compute new centre and radius

                    call msh478( coor, ia, jpoint, xm, ym, zm, &
                                 e1, e2, e3, xc, yc, zc, radius )
                end if
             end if
280      end do

!        --- Create tetrahedron for triangle ia-ib-ic and point jpoint

         ifout = 0

         klaar(ia) = klaar(ia) - 1
         klaar(ib) = klaar(ib) - 1
         klaar(ic) = klaar(ic) - 1

         if ( klaar(jpoint)==-1 ) klaar(jpoint) = 0

!        --- Fill element in array and adjust array kstapl and kstap

         call msh612( kelem, nelem, ia, ib, ic, jpoint, &
                      kstapl, kstap, klaar )

         if ( ioutp>=1 .and. nelem>=nwrel .and. debug ) then
            write(irefwr,1)'Number of elements', nelem
            write(irefwr,1)'Number of remaining boundary triangles', &
                            kstap
            write(irefwr,1)'Number of nodes', npoint
            nwrel = nwrel + 100
         end if

!        --- Go for next triangle

         goto 200
      end if
1000  call erclos( 'msh476' )

      if ( debug ) then
         write(irefwr,*) 'Temporary: Compute volume at end of 476'
         volumt = 0d0

         do i=1, nelem
            ia = kelem(1,i)
            ib = kelem(2,i)
            ic = kelem(3,i)
            id = kelem(4,i)
            call msh632( coor, ia, ib, ic, id, volume )
            if ( volume<0 ) then
               write(irefwr,*) 'error in element',ia,ib,ic,id
               write(irefwr,*) 'volume = ',volume
               !volume = - volume
            end if
            volumt = volumt + volume
         end do
         write(irefwr,*) 'volumt = ',volumt
      end if
      if ( debug .and. kstap>0 ) then

!     --- Debug information

         write(irefwr,*) 'Debug from end msh476'
         write(irefwr,*) 'kstap = ',kstap
         do i=1, kstap
            k1 = kstapl(1,i)
            k2 = kstapl(2,i)
            k3 = kstapl(3,i)
            write(irefwr,*) 'triangle',k1,k2,k3
         end do

         write(irefwr,*) 'Compute volume at end of 476'
         volumt = 0d0

         do i=1, nelem
            ia = kelem(1,i)
            ib = kelem(2,i)
            ic = kelem(3,i)
            id = kelem(4,i)
            call msh632( coor, ia, ib, ic, id, volume )
            if ( volume<0 ) then
               write(irefwr,*) 'error in element',ia,ib,ic,id
               write(irefwr,*) 'volume = ',volume
               !volume = - volume
            end if
            volumt = volumt + volume
         end do

         write(irefwr,*) 'Total volume = ',volumt

!        --- Eventually check all barycenters:

         if ( 0>1 ) then

!        --- Check all elements

            ia = 0
            call msh652( kelem, nelem, coor, npoint, ia )
            write(irefwr,*) 'After call to msh652'
            write(irefwr,*) 'Problems in element',ia
            if (ia>0) then
               call errint ( ia, 1 )
               call errsub ( 2913, 1, 0, 0 )
            end if
         end if
         write(irefwr,*) 'End debug info msh476'
      end if  ! ( debug )

      end
