      subroutine msh601 ( kbound, nbound, istart, ibuur, coarse, coor, &
                          npunt, ivolnr, lenibuur, shift )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.0  date 20-11-2013 Extra parameters
!        version  5.1  date 31-01-2011 Error message
!        version  5.0  date 03-09-2010 New release
!
!   copyright (c) 1987-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to check whether the boundary-elements of the volume
!     considered are correct and to fill for each point array coarse
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbound, kbound(3,nbound), istart(*), &
              lenibuur, ibuur(lenibuur), npunt, ivolnr
      double precision coarse(*), coor(3,npunt), shift(*)

!     coarse         o    Array containing the coarseness per point
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates
!                         of the nodal points with respect to the global
!                         numbering
!     ibuur          o    array with neighbours
!     istart         o    array containing the starting addresses of
!                         the neighbours in ibuur
!     ivolnr         i    Volume sequence number
!     kbound         i    array with boundary elements (triangles)
!     lenibuur       i    Declared length of array ibuur
!     nbound         i    number of triangles
!     npunt          i    number of points already created
!     shift          i    Contains information about the backtranformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, itotal, nbuur, ien, jerror, no, ni, ntal, &
              k, jknoop, jkn, j, node
      double precision afstan, afst, eps, x(3)
      logical found

!     afst           distance
!     afstan         help variable for distance
!     eps            Local accuracy
!     found          defines whether a conditions is satisfied
!     i              Counting variable
!     i1             first node number of triangle
!     i2             second nodenumber of triangle
!     i3             third node number of triangle
!     ien            help variable for length neighbour array
!     itotal         help variable to store length used of array
!     j              Counting variable
!     jerror         error number
!     jkn            node number
!     jknoop         node number
!     k              Counting variable
!     nbuur          number of neighbours
!     ni             temporary strataddress neighbours
!     no             old start address
!     node           Node number
!     ntal           neighbours of present point
!     x              co-ordinates of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH602         fill array ibuur of the neighbour points
!     MSH603         compute the distance between two points i and j
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    1683   The boundary of the volume is not closed
!    2995   Internal error
!    3068   Locally opposite boundary elements coincide
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh601' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh601'
         write(irefwr,1) 'npunt, nbound', npunt, nbound
         call prinin1 ( kbound, nbound, 3, 'kbound' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set accuracy

      eps = 10*epsmac

!     --- Fill istart

      istart(1:npunt) = 0

!     --- Run through boundary
!         The number of triangles connected to the nodes is stored

      do i = 1, nbound

!     --- Nodes

         i1 = kbound( 1, i )
         i2 = kbound( 2, i )
         i3 = kbound( 3, i )

!        --- Augment positions:

         istart(i1) = istart(i1) + 2
         istart(i2) = istart(i2) + 2
         istart(i3) = istart(i3) + 2

      end do

      if ( debug ) call prinin ( istart, npunt, 'istart/1' )

!     --- Check and rearrange

      itotal = 0

!     --- Number of neighbours

      do i = 1, npunt
         nbuur = istart(i) / 2
         itotal    =  itotal + nbuur
         istart(i) =  itotal
      end do

      if ( debug ) call prinin ( istart, npunt, 'istart/2' )

!     --- First clear and then fill array ibuur

      ien = istart(npunt)

!     --- Set

      ibuur(1:ien) = 0

!     --- Fill

      do i = 1, nbound

!     --- Nodes

         i1 = kbound( 1, i )
         i2 = kbound( 2, i )
         i3 = kbound( 3, i )

!        --- Each triangle six neighbours:

         call msh602(istart, ibuur, i1, i2)
         call msh602(istart, ibuur, i2, i1)
         call msh602(istart, ibuur, i2, i3)
         call msh602(istart, ibuur, i3, i2)
         call msh602(istart, ibuur, i3, i1)
         call msh602(istart, ibuur, i1, i3)

      end do

      if ( debug ) call prinin ( istart, npunt, 'istart/2' )

!     --- Check ibuur

      jerror = 0
      no     = 0

!     --- Run through all nodes

      do i = 1, npunt

!     --- Startposition

         ni = istart(i)

!        --- ntal is number of neighbours

         ntal   = ni - no
         afstan = 0d0
         if ( ntal>0 ) then

!        --- Check for error in input:

            if ( ntal<3 ) then

!           --- Each node should have at least three neighbours:

               call errint ( ivolnr, 1 )
               call errsub ( 3068, 1, 0, 0 )

               write(irefwr,100)
100            format ( 'Extra information'/ 'node with coordinates' )
               x(1:3) = (coor(1:3,i)-1d0)*shift(4)+shift(1:3)
               write(irefwr,110) i, x
110            format ( i8, ':', ' (',d16.7,',',d17.7,',',d17.7,')' )

!              --- Find corresponding triangles and print information

               write(irefwr,*) 'Node lies in the following triangles'

               do j = 1, nbound
                  found = kbound(1,j)==i .or. kbound(2,j)==i .or. &
                          kbound(3,j)==i
                  if ( found ) then

!                 --- Triangle found

                     write(irefwr,120) j, kbound(1:3,j)
120                  format ( 'triangle ', i8, '  with nodes:  ', 3i8 )
                     write(irefwr,*) 'coordinates of these nodes'
                     do k = 1, 3
                        node = kbound(k,j)
                        x(1:3) = (coor(1:3,node)-1d0)*shift(4)+ &
                                 shift(1:3)
                        write(irefwr,110) node, x
                     end do  ! k = 1, 3

                  end if  ! ( found )

               end do  ! j = 1, nbound
               go to 1000

            end if
            if ( no+ntal> 3 * lenibuur ) then

!           --- estimate of ibuur too short

               call errint ( no+ntal     , 1 )
               call errint ( 3 * lenibuur, 2 )
               call errchr ( 'ibuur'     , 1 )
               call errsub ( 550,   2,  0, 1 )
               go to 1000
            end if
            do k = no+1, no+ntal
               jknoop = ibuur(k)
               if ( jknoop<0 ) then
                    ibuur(k) = -jknoop
                    jkn      = -jknoop
                    call msh603(i, jkn, coor, afst)
                    afstan = afstan + afst
               else
                    jerror = jerror + 1
               end if
            end do
            if ( afstan<eps ) then

!           --- Unexpected error

               call erreal ( coor(1,i), 1 )
               call erreal ( coor(2,i), 2 )
               call erreal ( coor(3,i), 3 )
               call errsub ( 2995, 0, 3, 0 )
               go to 1000
            end if  ! ( afstan<eps )
            coarse(i) = afstan / float(ntal)
         else
            coarse(i) = 0
         end if
         no  =  ni
      end do
      if ( jerror/=0 ) then

!     --- Errors in kbound array

         call errint ( ivolnr, 1 )
         call errsub ( 1683, 1, 0, 0 )
      end if
1000  call erclos ( 'msh601' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh601'

      end if  ! ( debug )

      end
