      subroutine msh606 ( npoint, coor, dismax, xstart, ystart, zstart, &
                          nx, ny, nz, icube, coarse, cube, jcube, &
                          nfixpnt, fix_pnt )
! ======================================================================
!
!        programmer    niek praagman
!        version  5.0  date 03-09-2010 New release
!        version  4.1  date 20-08-2010 Update (dynamic allocations)
!        version  4.0  date 23-02-2005 Fixed points added
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine for each standard cube the coarseness
!     and for all the boundary points in which cube they are situated.
!     Also the type of each cube is determined
!       0 : cube has no points in common with volume given by kbound
!       1 : cube belongs partial to volume given by kbound (no specials)
!       2 : cube belongs totally to volume given by kbound (no specials)
!       3 : cube contains fixed points (is called has specials)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, icube(npoint), jcube(*), nx, ny, nz, nfixpnt
      double precision coor(3,*), coarse(*), cube(*), &
                       fix_pnt(4, nfixpnt), &
                       dismax, xstart, ystart, zstart

!     coarse     i    array with coarsenesses of each point
!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     cube       i    array with coarsenesses of each cube
!     dismax     i    largest distance of two neighbour points in the surface
!     fix_pnt    i    array containing coordinates and coarseness
!                     fixed points
!     icube      i    array with for each node cube the node belongs to
!     jcube      i    array with indication whether cube is
!                     - completely outside region (=0), &
!                     - completely inside region  (=2), &
!                     - boundary cube             (=1), &
!                     - cube with fixed point     (=3)
!     nfixpnt    i    number of prescribed fixed points
!     npoint     i    number of nodal points
!     nx         i    number of cubes in x direction
!     ny         i    number of cubes in y direction
!     nz         i    number of cubes in z direction
!     xstart     i    smallest x-coordinate of enveloping cube
!     ystart     i    as above for y
!     zstart     i    as above for z
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, alpha, coars, cox, coxmax, coxmin, &
                       coa, coy, coymax, coymin, coz, cozmax, cozmin, &
                       deel, xp, yp, zp
      integer i, ik, ikx, ikxmax, ikxmin, iky, ikymax, ikymin, ikz, &
              ikzmax, ikzmin, j, jblok, k, kubx, kuby, kubz, n1, n2, n3, &
              nc, nrkube, ntal

!     alpha    helpvariable for coarseness computation
!     coa      helpvariable to store the smallest coarse value
!     coarse   sun of local coarseness contributions in one point
!     cox      helpvariable for x-coarseness in cube
!     coxmax   helpvariable to store coarseness max in x-direction
!     coxmin   helpvariable to store coarseness min in x-direction
!     coy      helpvariable for y-coarseness in cube
!     coymax   helpvariable to store coarseness max in y-direction
!     coymin   helpvariable to store coarseness min in y-direction
!     coz      helpvariable for z-coarseness in cube
!     cozmax   helpvariable to store coarseness max in z-direction
!     cozmin   helpvariable to store coarseness min in z-direction
!     deel     number to indicate how many contributions have been found
!     eps      local accuracy
!     i        loop variable
!     ik       loop variable
!     ikx      loop variable
!     ikxmax   position in x
!     ikxmin   position in x
!     iky      loop variable
!     ikymax   position in y
!     ikymin   position in y
!     ikz      loop variable
!     ikzmax   position in z
!     ikzmin   position in z
!     j        loop variable
!     jblok    helpvariable internal position determination
!     k        loop variable
!     kubx     helpvariable cubenumber x-direction
!     kuby     helpvariable cubenumber y-direction
!     kubz     helpvariable cubenumber z-direction
!     n1       x-number cube
!     n2       y-number cube
!     n3       z-number cube
!     nc       total number of cubes
!     nrkube   number of cube considered
!     ntal     number of contributions
!     xp       x-coordinate
!     yp       y-coordinate
!     zp       z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     None
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all boundary points and determine for each the number
!     of the cube the point belongs to
!     Run through all cubes and check for each cube
!     whether a coarseness-value is given
!     Determine for all "empty" cubes which are inside the area
!     also a value for the coarseness by interpolation
!     Finally give all cubes a value for emergency
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
      eps = 10d0 * epsmac

      coa = rinfin

!     --- Determine for all boundary points the cube-numbers

      do i = 1, npoint

         xp = coor(1,i)
         yp = coor(2,i)
         zp = coor(3,i)

         n1 = int ( (xp-xstart)/dismax )
         n2 = int ( (yp-ystart)/dismax )
         n3 = int ( (zp-zstart)/dismax )

!        --- Determine smallest coarsenessvalue

         if ( coarse(i)>eps .and. coarse(i)<coa ) &
              coa = coarse(i)

!        --- This point belongs to cube-number

         nc = 1 + n1 + n2*nx + n3*nx*ny

         icube(i) = nc
      end do

!     --- Consider for each cube whether a coarseness-value is given

      do i = 1, nx * ny * nz

         jcube(i) = 0
         cube (i) = 0
         ntal     = 0
         coars    = 0d0

         do ik = 1, npoint

            if ( coarse(ik)>eps ) then

               nc = icube(ik)

               if ( nc==i ) then
                  ntal   = ntal + 1
                  coars  = coars  + coarse(ik)
               end if

            end if

         end do

!        --- Points found in cube i

         if ( ntal>0 ) then

!        --- Boundary points => type = 1

            coars    = coars /  ntal
            cube(i)  = coars
            jcube(i) = 1
         end if
      end do

!     --- Determine for all fixed points the cube-numbers

      do i = 1, nfixpnt

          xp = fix_pnt(1,i)
          yp = fix_pnt(2,i)
          zp = fix_pnt(3,i)

          coars = fix_pnt(4,i)

          n1 = int ( (xp-xstart)/dismax )
          n2 = int ( (yp-ystart)/dismax )
          n3 = int ( (zp-zstart)/dismax )

!         --- Determine smallest coarsenessvalue

          if ( coars>eps .and. coars<coa ) &
               coa = coars

!         --- This point belongs to cube-number

          nc = 1 + n1 + n2*nx + n3*nx*ny

          cube (nc) = coars

!         --- Fixedpoint cube (type = 3)

          jcube(nc) = 3
      end do

!     --- Determine for all empty cubes which are inside the volume
!            also a value for the coarseness

      do i = 1, nx
      do j = 1, ny
      do k = 1, nz

!     --- Compute number of cube

          nrkube = i + (j-1)*nx + (k-1)*nx*ny

!         --- Check contents

          if ( jcube(nrkube)==0 ) then

!         --- No boundary points in this cube
!             Determine cubes in i-direction

             kubx = 1 + (j-1)*nx + (k-1)*nx*ny

             coxmin = 0
             ikxmin = 0

             do ikx = kubx, nrkube - 1

                if ( jcube(ikx)>0 ) then
                   ikxmin = ikx
                   coxmin = cube(ikx)
                end if

             end do

             kubx = j*nx + (k-1)*nx*ny

             coxmax = 0
             ikxmax = 0

             do ikx = kubx,nrkube+1,-1

                if ( jcube(ikx)>0 ) then
                   ikxmax = ikx
                   coxmax = cube(ikx)
                end if

             end do

!            --- Determine cubes in j-direction

             kuby   = i + ( k-1 )*nx*ny

             coymin = 0
             ikymin = 0

             do iky = kuby, nrkube-nx, nx

                if ( jcube(iky)>0 ) then
                   ikymin = iky
                   coymin = cube(iky)
                end if

             end do

             kuby   = i  + (k-1)*nx*ny + (ny-1)*nx

             coymax = 0
             ikymax = 0

             do iky = kuby, nrkube + nx, - nx

                if ( jcube(iky)>0 ) then

                   ikymax = iky
                   coymax = cube(iky)

                end if

             end do

!            --- Determine cubes in k-direction

             kubz = i + (j-1)*nx

             cozmin = 0
             ikzmin = 0

             do ikz = kubz, nrkube - nx * ny, nx * ny

                if ( jcube(ikz)>0 ) then

                   ikzmin = ikz
                   cozmin = cube(ikz)

                end if

             end do

             kubz = i + (j-1)*nx + (nz-1)*nx*ny

             cozmax = 0
             ikzmax = 0

             do ikz = kubz, nrkube + nx * ny, - nx * ny

                if ( jcube(ikz)>0 ) then

                   ikzmax = ikz
                   cozmax = cube(ikz)

                end if

             end do

!            --- Determine weighted coarseness for this cube

             deel = 0.
             cox  = 0.
             coy  = 0.
             coz  = 0.

             jblok = ikxmax * ikxmin

             if ( jblok/=0 ) then
                alpha = (coxmax/coxmin)**(1d0/(ikxmax-ikxmin))
                cox   = coxmin * ( alpha ** (nrkube-ikxmin))
                deel  = deel + 1d0
             else if ( ikxmax/=0 ) then
                cox   = coxmax
                deel  = deel + 1d0
             else if ( ikxmin/=0 ) then
                cox   = coxmin
                deel  = deel + 1d0
             end if

             jblok = ikymax * ikymin

             if ( jblok/=0 ) then
                alpha = (coymax/coymin)**(1d0/((ikymax-ikymin)/nx))
                coy   = coymin * ( alpha ** ((nrkube-ikymin)/nx))
                deel  = deel + 1d0
             else if ( ikymax/=0 ) then
                coy   = coymax
                deel  = deel + 1d0
             else if ( ikymin/=0 ) then
                coy   = coymin
                deel  = deel + 1d0
             end if

             jblok = ikzmax * ikzmin

             if ( jblok/=0 ) then
                alpha = (cozmax/cozmin)**(1d0/((ikzmax-ikzmin)/(nx*ny)))
                coz   = cozmin * ( alpha ** ((nrkube-ikzmin)/(nx*ny)))
                deel  = deel + 1d0
             else if ( ikzmax/=0 ) then
                coz   = cozmax
                deel  = deel + 1d0
             else if ( ikzmin/=0 ) then
                coz   = cozmin
                deel  = deel + 1d0
             end if

             if ( deel>0.5d0 ) then
                cube( nrkube) = ( cox + coy + coz ) / deel
                jcube(nrkube) = 2
             end if

          end if

      end do
      end do
      end do

!     --- For emergency: Check all cubes and give if coarseness
!                        is too small a "global" (small-)value

      do i = 1, nx * ny * nz
         if ( cube(i)<eps ) then
              cube(i) = coa
         end if
      end do

      end
