      subroutine msh608 ( kstapl, kstap, acoor, xstart )
! ======================================================================
!
!        programmer    niek praagman
!        version  3.0  date 03-09-2010 New release
!        version  2.2  date 21-12-2009 Change accuracy point n
!        version  2.1  date 19-10-2005 Update code
!        version  2.0  date 09-09-1992 redesign
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether all elements are given in such a ordering that the three
!     corners and the inside pointing normal form a right-hand turning system.
!     If not the ordering of the nodes is changed.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstap, kstapl(3,kstap)
      double precision acoor(3,*), xstart

!     acoor      i       coordinate array
!     kstap      i       number of elements in array kstapl
!     kstapl    i/o      array with boundary elements
!     xstart     i       smallest x-value of help-cubes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, ja
      double precision dr, dx, dy, dz, e1, e2, e3, &
                       xa, xb, xc, xm, xn, xst, &
                       ya, yb, yc, ym, yn, &
                       za, zb, zc, zm, zn

!     debug  if true debug statements are carried out else not
!     dr     distance
!     dx     x-distance
!     dy     y-distance
!     dz     z-distance
!     e1     first  component normal vector
!     e2     second component normal vector
!     e3     third  component normal vector
!     i      loop variable
!     i1     first node triangle
!     i2     second node triangle
!     i3     third node triangle
!     ja     indicator
!     xa     x-coordinate first node
!     xb     x-coordinate second node
!     xc     x-coordinate third node
!     xm     x-coordinate barycenter
!     xn     x-coordinate helpnode
!     xst    x-coordinate of point (outside volume) most left
!     ya     y-coordinate first node
!     yb     y-coordinate second node
!     yc     y-coordinate third node
!     ym     y-coordinate barycenter
!     yn     y-coordinate helpnode
!     za     z-coordinate first node
!     zb     z-coordinate second node
!     zc     z-coordinate third node
!     zm     z-coordinate barycenter
!     zn     z-coordinate helpnode
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Resets old name of previous subroutine of higher level
!     EROPEN   Produces concatenated name of local subroutine
!     MSH607   Determines whether point belongs to volume
!     MSH609   Computes barycentre and normal of triangle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh608' )

      debug = .false.

      xst = xstart - 1d0

      do i = 1, kstap

          i1 = kstapl( 1,i )
          i2 = kstapl( 2,i )
          i3 = kstapl( 3,i )

!         --- Compute normal unity vector (e1,e2,e3) for triangle:

          call msh609(acoor,i1,i2,i3,e1,e2,e3,xm,ym,zm)

!         --- Find "characteristic" length of edge triangle:

          xa = acoor( 1,i1 )
          ya = acoor( 2,i1 )
          za = acoor( 3,i1 )

          xb = acoor( 1,i2 )
          yb = acoor( 2,i2 )
          zb = acoor( 3,i2 )

          xc = acoor( 1,i3 )
          yc = acoor( 2,i3 )
          zc = acoor( 3,i3 )

          dx = ( abs(xa-xb) + abs(xb-xc) + abs(xc-xa) ) / 3d0
          dy = ( abs(ya-yb) + abs(yb-yc) + abs(yc-ya) ) / 3d0
          dz = ( abs(za-zb) + abs(zb-zc) + abs(zc-za) ) / 3d0

          dr = sqrt( dx*dx + dy*dy + dz*dz )

!         --- Use barycentre for extra position:

          xm = ( xa + xb + xc ) / 3d0
          ym = ( ya + yb + yc ) / 3d0
          zm = ( za + zb + zc ) / 3d0

          xn = xm + 1d-5 * dr * e1
          yn = ym + 1d-5 * dr * e2
          zn = zm + 1d-5 * dr * e3

!         --- Check whether point 'n' belongs to volume

          ja = 0
          call msh607( xn, yn, zn, xst, acoor, kstapl, kstap, ja )
          if ( ja==0 ) then

!         --- Reorder triangle
!             Use in case of debugging:

              if ( debug ) then
                 write(irefwr,10) 'msh608:reorder trian',i,'-',i1,i2,i3
                 write(irefwr, *) 'crds',xn,yn,zn
                 write(irefwr, *) 'dr=',dr
              end if

              kstapl( 2,i ) = i3
              kstapl( 3,i ) = i2
          else
              if ( debug ) then
                 write(irefwr,10) 'No reordering of ',i,'-',i1,i2,i3
                 write(irefwr, *) 'crds',xn,yn,zn
                 write(irefwr, *) 'dr =',dr
              end if
          end if
      end do
10    format( a,(i4),a,(3(i6)) )
      call erclos( 'msh608' )
      end
