      subroutine msh650 ( kelem, nelem, nelmfix, kstap , kstapl, &
                          npoint, coor, itri )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  3.0  date 08-09-2010 New release
!        version  2.3  date 09-04-2009 Adjust value nelem
!        version  2.2  date 02-12-2008 Keep all nelmfix elements
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Cancel all elements with a small ratio or that have at least one
!     point in common with the present boundary kstapl. Keep the points.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, nelmfix, kelem(4,*), kstap, kstapl(3,*), npoint, &
              itri(*)
      double precision coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     itri                work array of length npoint for detecting that
!                         all work is done
!     kelem         i/o   topology of the tetrahedra
!     kstap         i/o   number of triangles constituting the actual
!                         boundary : at output this variable should be 0
!     kstapl        i/o   help array for actual boundary of region, &
!                         at input containing all boundary triangles, &
!                         and during the process the actual boundary hence
!                         at output empty
!     nelem          i    Number of elements
!     nelmfix        i    Number of fixed elements
!     npoint        i/o   Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, nel
      double precision ratio

!     i          loopvariable
!     i1         nodenumber
!     i2         nodenumber
!     i3         nodenumber
!     i4         nodenumber
!     nel        number of elements in the topology
!     ratio      parameter to store the ratio of the tetrahedron
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH646         Adds a new triangle to array KSTAPL or cancels
!                    the triangle if already present in KSTAPL.
!     MSH661         Compute ratio inner/outer sphere of tetrahedron
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear helparray itri
!     Mark in array itri all nodes of elements that have a "bad" ratio.
!     Determine which points are in the submitted boundary kstapl and
!     fill array itri accordingly
!     Run through all elements and cancel elements that have at least one
!     point in common with the given boundary (kstapl). Keep the nodal points.
!     Update array itri with respect to the new boundary
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     None
! ======================================================================
!
!     --- Clear array itri

      do i = 1 , npoint
         itri(i) = 0
      end do

!     --- Determine which points are in the boundary

      do i = 1 , kstap

!     --- Triangle i:

         i1 = kstapl(1,i)
         i2 = kstapl(2,i)
         i3 = kstapl(3,i)

!        --- Augment itri value:

         itri(i1) = itri(i1) + 1
         itri(i2) = itri(i2) + 1
         itri(i3) = itri(i3) + 1
      end do

!     --- Add to boundary all surroundings of elements that have
!         too small ratio's (not used!)

      if ( kstap<0 ) then
         do i= nelmfix+ 1 , nelem
            i1 = kelem(1,i)
            i2 = kelem(2,i)
            i3 = kelem(3,i)
            i4 = kelem(4,i)
            call msh661(coor,i1,i2,i3,i4,ratio)
            if ( ratio<0.15d0 ) then
               itri(i1) = itri(i1) + 1
               itri(i2) = itri(i2) + 1
               itri(i3) = itri(i3) + 1
               itri(i4) = itri(i4) + 1
            end if
         end do
      end if

!     --- Cancel all elements that have at least one point in common
!         with the given boundary (kstapl) but keep the nodal points
!         Keep the first nelmfix elements, hence start for nelmfix+1
!         and check each element:

      nel  = nelmfix

!     --- Run through all elements

      do i = nelmfix + 1 , nelem

!     --- Find nodes of element i:

         i1 = kelem(1,i)
         i2 = kelem(2,i)
         i3 = kelem(3,i)
         i4 = kelem(4,i)

!        --- Check the four nodes of the tetrahedron

         if ( itri(i1)+itri(i2)+itri(i3)+itri(i4)>0 ) then

!        --- Tetrahedron touches boundary:
!            Place the four "face" - triangles in array kstapl

            call msh646( kstapl, kstap, i1, i2, i3 )
            call msh646( kstapl, kstap, i1, i4, i2 )
            call msh646( kstapl, kstap, i2, i4, i3 )
            call msh646( kstapl, kstap, i3, i4, i1 )
         else

!        --- Tetrahedron not important for the situation, keep

            nel          = nel + 1

!           --- Fill nodenumbers in in kelem

            kelem(1,nel) = i1
            kelem(2,nel) = i2
            kelem(3,nel) = i3
            kelem(4,nel) = i4
         end if
      end do

!     --- Set actual number of elements:

      nelem = nel

!     --- Update itri with respect to the new boundary

      do i = 1 , npoint
         itri(i) = 0
      end do
      do i = 1 , kstap

!     --- Find nodes of triangle:

         i1 = kstapl(1,i)
         i2 = kstapl(2,i)
         i3 = kstapl(3,i)

!        --- Fill array itri:

         itri( i1 )    = itri(i1) + 1
         itri( i2 )    = itri(i2) + 1
         itri( i3 )    = itri(i3) + 1
      end do
      end
