      subroutine msh663 ( coor, nipnt, npoint, kelem, nelem, jelem )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.1  date 01-12-2010 New tuningfactor
!        version  2.0  date 09-09-2010 New release
!        version  1.1  date 19-10-2006 Extra debug facilities added
!        version  1.0  date 15-02-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Run through all elements and skip if possible bad tetrahedron
!     jelem by joining nodes.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, nipnt, npoint, jelem, kelem(4,*)
      double precision coor(3,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     jelem      i    element number of "bad" tetrahedron (to be skipped)
!     kelem     i/o   element array
!     nelem      i    number of elements in present mesh
!     nipnt      i    number of nodes in starting boundary
!     npoint     i    number of nodes in present mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, ia, ib, ic, id, &
              j1, j2, ja, jb, jc, jd, &
              ielem, jmin, jmax, lelem
      double precision distan12, distan13, distan14, &
                       distan23, distan24, distan34, &
                       dismin, dismax, &
                       minvol, volumc, volume, volumn, &
                       xmax, xmin, ymax, ymin, zmax, zmin

!     dismax         Maximal distance between two points
!     dismin         smallest distance
!     distan12       distance between the nodes 1 and 2
!     distan13       distance between the nodes 1 and 3
!     distan14       distance between the nodes 1 and 4
!     distan23       distance between the nodes 2 and 3
!     distan24       distance between the nodes 2 and 4
!     distan34       distance between the nodes 3 and 4
!     i              Counting variable
!     i1             node number of tetrahedron
!     i2             node number of tetrahedron
!     i3             node number of tetrahedron
!     i4             node number of tetrahedron
!     ia             help node number
!     ib             help node number
!     ic             help node number
!     id             help node number
!     ielem          loopvariable for element loop
!     j1             helpvariable for node recognition
!     j2             helpvariable for node recognition
!     ja             helpvariable
!     jb             help variable
!     jc             help variable
!     jd             help variable
!     jmax           help for max value
!     jmin           help for min value
!     lelem          help for new number of elements
!     minvol         smallest volumevalue
!     volume         helpvariable to store the volume of elmnt
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH603         compute the distance between two points i and j
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine smallest side of tetrahedron with no boundary node
!
!     If smallest side exists and is small enough join the two
!     nodes to one.
!     Skip all related elements, compute coordinates of new node
!     and reorder
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh663' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh663'
      end if
      if ( ierror/=0 ) go to 1000

!     --- Set the node numbers of element jelem

      i1 = kelem(1,jelem)
      i2 = kelem(2,jelem)
      i3 = kelem(3,jelem)
      i4 = kelem(4,jelem)

!     --- No possibility for joining nodes if only boundary points

      if ( max(i1,i2,i3,i4)<=nipnt ) go to 1000

!     --- Compute length of sides

      call msh603( i1, i2, coor, distan12 )
      call msh603( i1, i3, coor, distan13 )
      call msh603( i1, i4, coor, distan14 )
      call msh603( i2, i3, coor, distan23 )
      call msh603( i2, i4, coor, distan24 )
      call msh603( i3, i4, coor, distan34 )

!     --- Compute max and min distances:

      dismin = min(distan12, distan13, distan14, distan23, &
                   distan24, distan34 )
      dismax = max(distan12, distan13, distan14, distan23, &
                   distan24, distan34 )

!     --- Check ratio of max and min (0.5 seems optimal!):

      if ( dismin>0.5 * dismax ) then

!     --- All sides of element approximately same length do nothing, &
!         eventually goto msh665:

         goto 1000
      else if ( distan12<1.01 * dismin ) then

!     --- i1 - i2 is smallest distance, place together:

         j1 = i1
         j2 = i2
      else if ( distan13<1.01 * dismin ) then

!     --- i1 - i3 is smallest distance, place together:

         j1 = i1
         j2 = i3
      else if ( distan14<1.01 * dismin ) then

!     --- i1 - i4 is smallest distance, place together:

         j1 = i1
         j2 = i4
      else if ( distan23<1.01 * dismin ) then

!     --- i2 - i3 is smallest distance, place together:

         j1 = i2
         j2 = i3
      else if ( distan24<1.01 * dismin ) then

!     --- i2 - i4 is smallest distance, place together:

         j1 = i2
         j2 = i4
      else if ( distan34<1.01 * dismin ) then

!     --- i3 - i4 is smallest distance, place together:

         j1 = i3
         j2 = i4
      end if

!     --- Only effective if both points are internal

      if ( j1>nipnt .and. j2>nipnt ) then

!     --- Compute controle volume volumc

         volumc = 0d0
         minvol = 1d10

!        --- Add all volumes

         do ielem = 1, nelem
            ia = kelem(1, ielem)
            ib = kelem(2, ielem)
            ic = kelem(3, ielem)
            id = kelem(4, ielem)

!           --- Compute new ratio and volume

            call msh632( coor, ia, ib, ic, id, volume )
            if ( volume<minvol ) minvol=volume
            volumc = volumc + abs(volume)
         end do

!        --- determine ordering of nodes

         jmin = min(j1,j2)
         jmax = max(j1,j2)

!        --- Coordinates

         xmin = coor(1,jmin)
         ymin = coor(2,jmin)
         zmin = coor(3,jmin)

!        --- Coordinates

         xmax = coor(1,jmax)
         ymax = coor(2,jmax)
         zmax = coor(3,jmax)

!        --- Compute coords midpoint and place in positions j1 and j2:

         coor(1,j1) = (xmin+xmax)/2d0
         coor(2,j1) = (ymin+ymax)/2d0
         coor(3,j1) = (zmin+zmax)/2d0

!        --- Place these coordinates temporarily in positions j1 and j2:

         do i=1,3
            coor(i,j2) = coor(i,j1)
         end do

!        --- Check sum new volumes

         volumn = 0d0
         lelem  = 0
         minvol = 1d10

!        --- Run through all elements:

         do ielem = 1, nelem
            ia = kelem(1, ielem)
            ib = kelem(2, ielem)
            ic = kelem(3, ielem)
            id = kelem(4, ielem)

!           --- Compute new ratio and volume

            call msh632( coor, ia, ib, ic, id, volume )
            volumn = volumn + abs(volume)
            if ( debug .and. volume<10*epsmac ) then
               write(irefwr,*) 'el',ia,ib,ic,id
               write(irefwr,*) 'volume = ',volume
            end if
         end do

!        --- Finally check volumc and volumn

         if ( abs(volumc - volumn)>10 * epsmac ) then

!        --- No change possible, reset

            if ( j1==jmin ) then

!           --- First node j1:

               coor(1,j1) = xmin
               coor(2,j1) = ymin
               coor(3,j1) = zmin

!              --- Second node j2:

               coor(1,j2) = xmax
               coor(2,j2) = ymax
               coor(3,j2) = zmax
            else

!           --- First node j1:

               coor(1,j1) = xmax
               coor(2,j1) = ymax
               coor(3,j1) = zmax

!              --- Second node j2:

               coor(1,j2) = xmin
               coor(2,j2) = ymin
               coor(3,j2) = zmin
            end if
            goto 1000
         end if

!        --- Change: Run through all elements and adjust if necessary

         j1 = jmin
         j2 = jmax

!        --- Set number of new elements:

         lelem = 0

!        --- Run through old elements and adjust

         do ielem = 1, nelem

!        --- Find nodes:

            ia = kelem(1, ielem)
            ib = kelem(2, ielem)
            ic = kelem(3, ielem)
            id = kelem(4, ielem)

!           --- Set helpvariables:

            ja = 0
            jb = 0
            jc = 0
            jd = 0

!           --- Check whether line j1 - j2 belongs to element:

            if (ia==jmin .or. ia==jmax ) ja = 1
            if (ib==jmin .or. ib==jmax ) jb = 1
            if (ic==jmin .or. ic==jmax ) jc = 1
            if (id==jmin .or. id==jmax ) jd = 1

!           --- If so, sum==2 and hence skip element:

            if ( ja + jb + jc + jd<2 ) then

!           --- Keep element. If necessary give right node number:

               if ( ja==1 ) ia = jmin
               if ( jb==1 ) ib = jmin
               if ( jc==1 ) ic = jmin
               if ( jd==1 ) id = jmin

!              --- Keep element

               lelem = lelem + 1

!              --- Find new numbering

               if ( ia>jmax ) ia = ia - 1
               if ( ib>jmax ) ib = ib - 1
               if ( ic>jmax ) ic = ic - 1
               if ( id>jmax ) id = id - 1

!              --- Place new element

               kelem(1, lelem) = ia
               kelem(2, lelem) = ib
               kelem(3, lelem) = ic
               kelem(4, lelem) = id
            end if
         end do

!        --- New number of elements and nodes:

         nelem  = lelem
         npoint = npoint - 1

!        --- Adjust coor array by skipping node jmax:

         do i = jmax, npoint
            do ic = 1 , 3
               coor(ic,i) = coor(ic,i+1)
            end do
         end do
      end if
1000  call erclos ( 'msh663' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End debug info msh663'
         minvol = 1d10
         volumc = 0d0
         do ielem = 1, nelem
            ia = kelem(1, ielem)
            ib = kelem(2, ielem)
            ic = kelem(3, ielem)
            id = kelem(4, ielem)

!           --- Compute new ratio and volume

            call msh632( coor, ia, ib, ic, id, volume )
            volumc = volumc + abs(volume)
            if ( volume<minvol ) minvol=volume
         end do
         write(irefwr,*) 'End of msh663, minvol=',minvol
         write(irefwr,*) 'volumc = ',volumc
      end if

      end
