      subroutine msh672 ( j, coor, npoint, kstapl, kstap, &
                          xstart, iprob )
! ======================================================================
!
!        programmer    niek praagman
!        version  1.0  date 23-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check for boundary triangle j whether the distance to the nearest
!     opposite element is realistic in comparison to the coarseness of
!     element j.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iprob, j, kstap, npoint, kstapl(3,kstap)
      double precision coor(3,npoint), xstart

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect
!                         to the global numbering
!     iprob          o    indicator of result:
!                          0 no problems for triangle j
!                         >0 number of triangle that is to near to j
!     j              i    boundary triangle to be considered
!     kstap         i/o   number of triangles in present boundary
!     kstapl        i/o   all triangles in present boundary
!     npoint        i/o   number of nodes in present mesh
!     xstart         i    x-coordinate point far away
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, ia, ib, ic, ja, k
      double precision dis, dis1, dis2, dis3, &
                       e1, e2, e3, t, th, &
                       x1, x2, x3, xm, xn, &
                       y1, y2, y3, ym, yn, &
                       z1, z2, z3, zm, zn

!     dis            distance
!     dis1           distance
!     dis2           distance
!     dis3           distance
!     e1             first coordinate unity vector
!     e2             second coordinate unity vector
!     e3             third coordinate unity vector
!     i              Counting variable
!     i1             node number of triangle
!     i2             node number of triangle
!     i3             node number of triangle
!     ia             help node number
!     ib             help node number
!     ic             help node number
!     ja             help variable, indicator
!     k              loopvariable
!     t              coordinate intersectionpoint line
!     th             smallest coordinate intersections
!     x1             x-coordinate first node
!     x2             x-coordinate second node
!     x3             x-coordinate third node
!     xm             x-coordinate barycentre triangle
!     xn             x-coordinate "endpoint"
!     xstart         x-coordinate far away point
!     y1             x-coordinate first node
!     y2             x-coordinate second node
!     y3             x-coordinate third node
!     ym             x-coordinate barycentre triangle
!     yn             x-coordinate "endpoint"
!     z1             x-coordinate first node
!     z2             x-coordinate second node
!     z3             x-coordinate third node
!     zm             x-coordinate barycentre triangle
!     zn             x-coordinate "endpoint"
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH603         compute the distance between two points i and j
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine smallest side of tetrahedron (no boundary nodes!)
!     If smallest side exists and is small enough join the two
!     nodes to one
!     Skip all related elements in kelem and in kstapl, compute
!     coordinates of new node and reorder nodes in elements and
!     boundary.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh672' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh672'
      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize

      iprob = 0
      ja    = 0

!     --- Check by running through all boundary triangles
!         compute "distance" to "opposite" element
!         --- Consider triangle j

      i1 = kstapl(1,j)
      i2 = kstapl(2,j)
      i3 = kstapl(3,j)

!     --- Determine startpoint line = barycentre triangle i1 - i2 - i3

      call msh609 ( coor, i1, i2, i3, e1, e2, e3, xm, ym, zm )

!     --- Compute distances edges triangle

      call msh603 ( i1, i2, coor, dis1 )
      call msh603 ( i2, i3, coor, dis2 )
      call msh603 ( i3, i1, coor, dis3 )

!     --- Characteristic distance for basis element i1-i2-i3 is

      dis  = min(dis1,dis2,dis3)/3d0

!     --- "Startpoint"

      xm = (coor(1,i1)+coor(1,i2)+coor(1,i3))/3d0
      ym = (coor(2,i1)+coor(2,i2)+coor(2,i3))/3d0
      zm = (coor(3,i1)+coor(3,i2)+coor(3,i3))/3d0

!     --- Find "endpoint"

      xn = xm + dis * e1
      yn = ym + dis * e2
      zn = zm + dis * e3

!     --- Check whether this new point is inside volume:

      call msh607( xn, yn, zn, xstart, coor, kstapl, kstap, ja )
      if ( ja==1 ) then
         if ( debug ) then
            write(irefwr,*) 'triangle',j
            write(irefwr,*) 'dis value',dis,'inside'
         end if

!        --- Inside, so no problem: return

         goto 1000
      else

!     --- Not inside: determine distance to other side

         th = 1000d0

!        --- Run through all remaining boundary triangles, and find
!            "nearest" triangle:

         do i = 1, kstap

            if ( i/=j ) then
               ia = kstapl ( 1, i )
               ib = kstapl ( 2, i )
               ic = kstapl ( 3, i )

               x1 = coor(1,ia)
               x2 = coor(1,ib)
               x3 = coor(1,ic)

               y1 = coor(2,ia)
               y2 = coor(2,ib)
               y3 = coor(2,ic)

               z1 = coor(3,ia)
               z2 = coor(3,ib)
               z3 = coor(3,ic)

!              --- Common point with line?

               call msh610(xn, yn, zn, xm, ym, zm, x1, y1, z1, &
                           x2, y2, z2, x3, y3, z3, t)

               if ( t>0 .and. t<th ) then
                  th    = t
                  iprob = i
               end if
            end if
         end do

         if ( th>0 .and. th<1d0 ) then
            write(irefwr,*) 'Intersection for th = ',th
            write(irefwr,*) 'that means distance is',th*dis
            write(irefwr,*) 'distances ok'
         else if ( th==0 .or. th>1d0 ) then
            write(irefwr,*) 'Bad situation in msh672'
            write(irefwr,*) 'No intersection found'
            write(irefwr,*) 'Coordinates barycentre triangle:'
            write(irefwr,500) j,(coor(k,j),k=1,3)
 500        format ( 1x,'Trian',(i7),'Crds',(3f12.6) )
            write(irefwr,*) 'Please inform SEPRA'
         end if
      end if

1000  call erclos ( 'msh672' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End debug info msh672'
      end if

      end
