      subroutine mshg3k ( nbound, jnew, nipnt, fix_pnt, &
                          curvenumbers, coorcurvs, surfnumbers )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  7.1  date 20-11-2013 Remove mshg3d
!        version  7.0  date 20-08-2010 Dynamic allocation helparrays
!        version  6.3  date 15-07-2009 Name msh635 internal crvs routine
!
!   copyright (c) 1988-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to make a mesh of tetrahedrons for a volume which is
!     given by the quadratic triangles of KBOUND. First a linear mesh
!     is created using KBOND out of KBOUND,and the linear subroutine
!     mshphil3d. Middle points are placed afterwards.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbound, nipnt, &
              curvenumbers(*), surfnumbers(*)
      double precision fix_pnt(*), coorcurvs(3,*)
      logical jnew

!     coorcurvs      i    Contains the coordinates of the extra curves
!     curvenumbers   i    Contains the extra curve numbers
!     fix_pnt        i    array with coordinates and coarsenesses of
!                         fixed points
!     jnew           i    indication whether the mesh is new (true) or old
!     nbound         i    number of boundary elements
!     nipnt          i    number of points in boundary ; only needed if
!                         jnew = 'false'
!     surfnumbers    i    Array of size nextrasrfs containing the surface
!                         sequence numbers of each extra surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, iel, nextra, &
              istrtp, jstart, numnodes, &
              j, j1, j2, j3, j4, j5, j6, ia, ib, ic, ja, jb, jc
      integer, allocatable :: numnodextcurvsl(:), ibuur(:), &
                              istart(:), kbond(:)
      double precision coorsav(12*nbound), coorcurvsl(3,npointlc)
      double precision xp, yp, zp

!     coorcurvsl     ?
!     coorsav        array of length 12*nbound to store the coordinates
!                    of the boundary nodes
!     i              Counting variable
!     i1             Node number of basis triangle
!     i2             Node number of basis triangle
!     i3             Node number of basis triangle
!     i4             Node number of basis triangle
!     i5             Node number of basis triangle
!     i6             Node number of basis triangle
!     ia             helpvariable to store smallest nodenumber of triangle
!     ib             helpvariable to store middel nodenumber
!     ibuur          helparray for neighbours, length 10 * npointlc
!     ic             helpvariable to store largest nodenumber
!     iel            Loop variable
!     istart         Helparray for starting addresses, length npointlc
!     istrtp         helpvariable startposition
!     j              Counting variable
!     j1             Node number second triangle
!     j2             Node number second triangle
!     j3             Node number second triangle
!     j4             Node number second triangle
!     j5             Node number second triangle
!     j6             Node number second triangle
!     ja             helpvariable second triangle
!     jb             helpvariable second triangle
!     jc             helpvariable second triangle
!     jstart         Starting address
!     kbond          helparray for linear boundary triangles
!     nextra         array containing information about segment endings and
!     numnodes       Number of nodes on an internal curve
!     numnodextcurvsl Array with number of nodes on linear internal curves
!     xp             x-coordinate
!     yp             y-coordinate
!     zp             z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH622         determine the neighbours of each node with node
!                    number larger than its own node number
!     MSH625         fill the boundary points and neighbours in ibuur
!     MSH626         create new internal mid points
!     MSH628         refill array kmeshc. Linear elements are changed
!                    in quadratic elements
!     MSH635         insert the intermediate nodes of internal curves
!                    after the generation back into the quadratic boundary
!                    elements
!     MSHCOPYBOUNCR  Copy boundary coordinates into temporary array coorsav
!     MSHPHIL3D      Fill elements and nodes for shape 11
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('mshg3k')
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshg3k'
         call prinin1 ( kbndptlc, nbound, 6, 'kbndptlc' )
      end if
      if ( ierror/=0 ) go to 1000

!     --- Copy coordinates of boundary into coorsav

      call mshcopybouncr ( kbndptlc, nbound, coor3d, coorsav, 6 )

!     --- Allocate space for helparray kbond:

      allocate( kbond( 3 * nbound ), stat = error)
      if ( error/=0 ) call eralloc ( error, 3*nbound, 'kbond' )
      if ( ierror/=0 ) go to 1000

!     --- Start to fill array KBOND and call mshphil3d

      nextra = 0

      do i=1,nbound
          i1 = kbndptlc(6*i-5)
          i2 = kbndptlc(6*i-3)
          i3 = kbndptlc(6*i-1)
          i4 = kbndptlc(6*i-4)
          i5 = kbndptlc(6*i-2)
          i6 = kbndptlc(6*i  )

          kbond(3*i-2) = i1
          kbond(3*i-1) = i2
          kbond(3*i  ) = i3

          if  ( .not. jnew ) then
             if ( i1>nextra ) nextra = i1
             if ( i2>nextra ) nextra = i2
             if ( i3>nextra ) nextra = i3
             if ( i4>nextra ) nextra = i4
             if ( i5>nextra ) nextra = i5
             if ( i6>nextra ) nextra = i6
          end if

      end do
      if ( debug ) call prinin1 ( kbond, nbound, 3, 'kbond' )

!     --- Check for double triangles ("patatzak")

      if ( debug ) then
         write(irefwr,*) 'Check: double "plaxis" triangles? (patatzak)'

         do i=1, nbound - 1
            i1 = kbond(3*i-2)
            i2 = kbond(3*i-1)
            i3 = kbond(3*i  )

            ia = min(i1,i2,i3)
            ic = max(i1,i2,i3)

            ib = 0
            if ( ia==i1.and.ic==i2.or.ia==i2.and.ic==i1 ) then
               ib = i3
            else if ( ia==i2.and.ic==i3.or.ia==i3.and.ic==i2 ) then
               ib = i1
            else if ( ia==i3.and.ic==i1.or.ia==i1.and.ic==i3 ) then
               ib = i2
            end if

!           --- Second loop:

            do j=i+1, nbound
               j1 = kbond(3*j-2)
               j2 = kbond(3*j-1)
               j3 = kbond(3*j  )

!              --- Order the j-values:

               ja = min(j1,j2,j3)
               jc = max(j1,j2,j3)

               jb = 0
               if ( ja==j1.and.jc==j2.or.ja==j2.and.jc==j1 ) then
                  jb = j3
               else if ( ja==j2.and.jc==j3.or.ja==j3.and.jc==j2 ) then
                  jb = j1
               else if ( ja==j3.and.jc==j1.or.ja==j1.and.jc==j3 ) then
                  jb = j2
               end if

               if ( ja==ia.and.jb==ib.and.jc==ic ) then
                  write(irefwr,*) 'Double: triangle',i,':',ia,ib,ic
                  write(irefwr,*) 'same triangle as',j,':',ja,jb,jc
               end if

            end do

         end do
         write(irefwr,*) 'End check Plaxis (patatzak)'
      end if

!     --- Divide region given by KBOND in linear triangles

      if ( jnew ) then
           nipnt = npointlc
      else
           nipnt = nextra
           do iel = 1 , nelemlc
               i1 = kmeshc(10*iel - 9)
               i2 = kmeshc(10*iel - 7)
               i3 = kmeshc(10*iel - 5)
               i4 = kmeshc(10*iel    )
               kmeshc(4*iel-3) = i1
               kmeshc(4*iel-2) = i2
               kmeshc(4*iel-1) = i3
               kmeshc(4*iel  ) = i4
               if ( i1>nextra ) nextra = i1
               if ( i2>nextra ) nextra = i2
               if ( i3>nextra ) nextra = i3
               if ( i4>nextra ) nextra = i4
           end do
      end if

!     --- Internal curves and/or plates

      istrtp = 0
      jstart = 0

!     --- Allocate space for all helparrays:

      allocate( numnodextcurvsl( nextracrvs ), stat = error)
      if ( error/=0 ) &
         call eralloc ( error, nextracrvs, 'numnodextcurvsl' )
      if ( ierror/=0 ) go to 1000

      if ( nextracrvs>0 ) then
         do icurnr = 1, nextracrvs
            numnodes = numnodextcurvs(icurnr)
            numnodextcurvsl(icurnr) = ( numnodes + 1 ) / 2

            do iel = 1, ( numnodes + 1 ) / 2
               do i = 1, 3
                  coorcurvsl(i, jstart + 1) = coorcurvs(i, istrtp + 1)
               end do
               istrtp = istrtp + 2
               jstart = jstart + 1
            end do
         end do
      end if

      call mshphil3d ( kbond, nbound, kmeshc, &
                       jnew, nipnt, fix_pnt, &
                       numnodextcurvsl, curvenumbers, coorcurvsl, &
                       surfnumbers  )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin1 ( kmeshc, nelemlc, 4, 'kmeshc' )

      if ( .not. jnew ) npointlc = nipnt

!     --- Determine all neighbours for each point
!         and place extra nodes

      allocate( istart(npointlc), ibuur(10*npointlc ), stat = error )
      if ( error/=0 ) call eralloc ( error, 11*npointlc, 'istart' )
      if ( ierror/=0 ) go to 1000

      call msh622( kmeshc, nelemlc, istart, ibuur, npointlc )

!     --- Place "middle" points : (first old points)

      call msh625( kbndptlc, nbound, istart, ibuur, 6 )

!     --- New points

      if ( .not. jnew ) npointlc = nextra

      call msh626( istart, ibuur, npointlc, coor3d )

!     --- Internal points

      istrtp = nbound * 3
      jstart = nbound * 6

      if ( nextrapnts>0 ) then
         do i = 1, nextrapnts
            kbndptlc( jstart + i ) = kbond( istrtp + i )
         end do
         istrtp = istrtp + nextrapnts
         jstart = jstart + nextrapnts
      end if

!     --- Internal curves

      if ( nextracrvs>0 ) then
         call msh635( kbndptlc, kbond, nextracrvs, &
                      numnodextcurvsl, istart, ibuur, coor3d, &
                      coorcurvs, istrtp, jstart )
      end if

!     --- Refill kmeshc

      call msh628( kmeshc, nelemlc, istart, ibuur )

!     --- Add eventually special Plaxis elements

      do i=1, nbound - 1

         i1 = kbond(3*i-2)
         i2 = kbond(3*i-1)
         i3 = kbond(3*i  )

         ia = min(i1,i2,i3)
         ic = max(i1,i2,i3)

         ib = 0
         if ( ia==i1 .and. ic==i2 .or. ia==i2 .and. ic==i1 ) then
            ib = i3
         else if ( ia==i2 .and. ic==i3 .or. ia==i3 .and. ic==i2 ) then
            ib = i1
         else if ( ia==i3 .and. ic==i1 .or. ia==i1 .and. ic==i3 ) then
            ib = i2
         end if

!        --- Second loop:

         do j=i+1, nbound

            j1 = kbond(3*j-2)
            j2 = kbond(3*j-1)
            j3 = kbond(3*j  )

!           --- Order the j-values:

            ja = min(j1,j2,j3)
            jc = max(j1,j2,j3)

            jb = 0
            if ( ja==j1.and.jc==j2.or.ja==j2.and.jc==j1 ) then
              jb = j3
            else if ( ja==j2.and.jc==j3.or.ja==j3.and.jc==j2 ) then
              jb = j1
            else if ( ja==j3.and.jc==j1.or.ja==j1.and.jc==j3 ) then
              jb = j2
            end if

            if ( ja==ia.and.jb==ib.and.jc==ic ) then

               if ( debug ) then
                  write(irefwr,*) 'Double  ',i,':',ia,ib,ic
                  write(irefwr,*) 'Same as ',j,':',ja,jb,jc
                  write(irefwr,*) 'Add extra element'
               end if

               i1 = kbndptlc(6*i-5)
               i2 = kbndptlc(6*i-4)
               i3 = kbndptlc(6*i-3)
               i4 = kbndptlc(6*i-2)
               i5 = kbndptlc(6*i-1)
               i6 = kbndptlc(6*i  )

               if ( debug ) then
                  write(irefwr,*) 'First quadratic tri',i,'is'
                  write(irefwr,*) i1,i2,i3,i4,i5,i6
               end if

               nelemlc = nelemlc + 1

               kmeshc( 10 * nelemlc - 9 ) = i1
               kmeshc( 10 * nelemlc - 8 ) = i2
               kmeshc( 10 * nelemlc - 7 ) = i3
               kmeshc( 10 * nelemlc - 6 ) = i4
               kmeshc( 10 * nelemlc - 5 ) = i5
               kmeshc( 10 * nelemlc - 4 ) = i6
               kmeshc( 10 * nelemlc - 3 ) = -100
               kmeshc( 10 * nelemlc - 2 ) = -100
               kmeshc( 10 * nelemlc - 1 ) = -100
               kmeshc( 10 * nelemlc - 0 ) = -100

               kmeshc( 10 * nelemlc - 3 ) = i2
               kmeshc( 10 * nelemlc - 2 ) = i4
               kmeshc( 10 * nelemlc - 1 ) = i6
               kmeshc( 10 * nelemlc     ) = i1

               j1 = kbndptlc(6*j-5)
               j2 = kbndptlc(6*j-4)
               j3 = kbndptlc(6*j-3)
               j4 = kbndptlc(6*j-2)
               j5 = kbndptlc(6*j-1)
               j6 = kbndptlc(6*j  )

               if ( debug ) then
                  write(irefwr,*) 'Second quadratic tri',j,' is'
                  write(irefwr,*) j1,j2,j3,j4,j5,j6
               end if

!              --- Check j2

               if ( j1==i1 .and. j3==i3 .or. &
                    j3==i1 .and. j1==i3 ) kmeshc(10 * nelemlc - 3) = j2
               if ( j1==i3 .and. j3==i5 .or. &
                    j3==i3 .and. j1==i5 ) kmeshc(10 * nelemlc - 2) = j2
               if ( j1==i5 .and. j3==i1 .or. &
                    j3==i5 .and. j1==i1 ) kmeshc(10 * nelemlc - 1) = j2

!              --- Check j4

               if ( j3==i1 .and. j5==i3 .or. &
                    j5==i1 .and. j3==i3 ) kmeshc(10 * nelemlc - 3) = j4
               if ( j3==i3 .and. j5==i5 .or. &
                    j5==i3 .and. j3==i5 ) kmeshc(10 * nelemlc - 2) = j4
               if ( j3==i5 .and. j5==i1 .or. &
                    j5==i5 .and. j3==i1 ) kmeshc(10 * nelemlc - 1) = j4

!              --- Check j6

               if ( j5==i1 .and. j1==i3 .or. &
                    j1==i1 .and. j5==i3 ) kmeshc(10 * nelemlc - 3) = j6
               if ( j5==i3 .and. j1==i5 .or. &
                    j1==i3 .and. j5==i5 ) kmeshc(10 * nelemlc - 2) = j6
               if ( j5==i5 .and. j1==i1 .or. &
                    j1==i5 .and. j5==i1 ) kmeshc(10 * nelemlc - 1) = j6
            end if
         end do
      end do

!     --- End check

      if ( debug ) call prinin1 ( kmeshc, nelemlc, 10, 'kmeshc' )

!     --- Fill coordinates of boundary back into coor3d

      call mshcopybouncr ( kbndptlc, nbound, coorsav, coor3d, 6 )

      deallocate( numnodextcurvsl, istart, ibuur, kbond, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'numnodextcurvsl' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Write debug information: end mshg3k'
         write(irefwr,*) 'nipnt  = ',nipnt
         write(irefwr,*) 'npointlc = ',npointlc
         write(irefwr,*) 'nelemlc  = ',nelemlc
         write(irefwr,*) 'nbound = ',nbound

!        --- Coordinates?

         write(irefwr,*) 'List of all coordinates found'
         do i=1,npointlc
            write(irefwr,'(i6,(3d12.5))' ) i,(coor3d(ia,i),ia=1,3)
         end do

!        --- Check for position of nodes?

         write(irefwr,*) 'Check for all coordinates'
         do i= nipnt+1, npointlc
            xp = coor3d(1,i)
            yp = coor3d(2,i)
            zp = coor3d(3,i)
            if ( abs(xp      )<1d-10  .or. &
                 abs(xp-0.5d0)<1d-10  .or. &
                 abs(xp-1.0d0)<1d-10  .or. &
                 abs(yp      )<1d-10  .or. &
                 abs(yp-1.0d0)<1d-10  .or. &
                 abs(zp      )<1d-10  .or. &
                 abs(zp-1.0d0)<1d-10 ) then

!           --- Boundary point?

                 write(irefwr,'(i7,(3d12.5))') i,xp,yp,zp
            end if
         end do

!        --- Write all elements?

         write(irefwr,*) 'Elements created in mshg3k'
         do i=1, nelemlc
            j= 10 * (i-1) + 1
            write(irefwr,*) 'Element',i,' with nodes'
            write(irefwr,'(10i6)') (kmeshc(i1),i1=j,j+9)
         end do
         write(irefwr,*) 'End debug info mshg3k'
      end if
      call erclos('mshg3k')

      end
