      subroutine sepactsolbuf ( isol, kprob, indprf, indprp, nunkp, nphys, &
                                iprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Memory management subroutine which simplifies the computation of the starting
!  addresses of an existing SEPRAN solution array and KPROB parts f and p
!     If these arrays exist, it is checked if they are positioned in
!     in array IBUFFR (Blank common) or not.
!     If one or more of these arrays are not available
!     these array are read from file 3 and space is reserved in array IBUFFR.
!     The priority number is made equal to 10.
!     if no space is available in array IBUFFR, sepactsolbuf tries to
!     create space in the following ways:
!       -  by writing low priority arrays to backing storage
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
!     sepactsolbuf is exactly the same as ini075, but with the parameter nunkp as
!     extra output parameter
!     If array KPROB part f (or its corresponding counterpart) does exist, &
!     INDPRF gets the correct index number referring to array INFOR in
!     common block CARRAY.
!     Afterwards the starting address of KPROB part f is given by
!     INFOR(1,INDFOR)
!     In the same way INDPRP refers to KPROB parts p
!     sepactsolbuf is identical to subroutine INI055 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulebuf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol(5), kprob(*)
      integer, intent(out) :: indprf, indprp, nphys, nunkp, iprob

!     indprf         o    indication if kprob part f has been filled (>0)
!                         or not (0)
!     indprp         o    Indication if array kprob part p has been filled (1)
!                         or not (0)
!     iprob          o    Problem number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     nphys          o    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     nunkp          o    Maximum number of unknowns in a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
       integer :: i, itypvc, ipkprb, ivec, kpivec

!     i           Loop variable
!     ipkprb      Starting address of part of KPROB with respect to
!                 a complete problem in array KPROB (minus one)
!     itypvc      Type number of solution vector (110=solution vector, &
!                 115=vector of special structure)
!     ivec        Indication of the structure of a vector of special structure
!     kpivec      Starting address of information concerning arrays of
!                 special structure in KPROB
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIPRB         Get value of ipkprb as function of iprob
!     SEPACTIVATE    Reactivate existing array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     206   Input vector with illegal sequence number ivec
!     744   Only arrays of type 110 and 115 are permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute problem number (IPROB), type of solution vector (ITYPVC)
!     and for a vector of special structure the vector number (IVEC)
!     Read solution array in IBUFFR (INI070)
!     If ( ISOL is of type 110 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!        if ( KPROB part p has been filled ) then
!           Set INDPRP and read KPROB part p in IBUFFR
!     else if ( ISOL is of type 115 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!     else if ( ISOL is of type 116 or 126 ) then
!        Compute nphys
!     else
!        wrong type of vector found: error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepactsolbuf' )
      if ( ierror/=0 ) go to 1000

      indprf = 0
      indprp = 0
      itypvc = isol(2)

!     --- Put ISOL into IBUFFR

      call sepactivate ( isol(1), 'usol' )
      if ( ierror/=0 ) go to 1000

!     --- Compute IPROB and offset for array KPROB

      iprob  = 1 + isol(4)/1000
      ipkprb = iniprb ( iprob, kprob )
      if ( ierror/=0 ) go to 1000

      select case ( itypvc )

      case ( 110 )

!     --- ISOL is of type 110:  solution vector

         nphys = kprob(ipkprb+33)
         nunkp = kprob(ipkprb+4)
         indprf = kprob(ipkprb+19)
         indprp = kprob(ipkprb+35)
         if ( indprf>0 ) call sepactivate ( -indprf, 'kprobf' )
         if ( indprp>0 ) call sepactivate ( -indprp, 'kprobp' )

      case ( 115 )

!     --- ISOL is of type 115:  array of special structure

         ivec   = mod(isol(4),1000)
         if ( ivec<1 .or. ivec>kprob(ipkprb+6) ) then

!        --- ivec out of range

            call errint ( ivec, 1 )
            call errint ( kprob(ipkprb+6), 2 )
            call errsub ( 206, 2, 0, 0 )
            ivec = 1

         end if
         kpivec = kprob(ipkprb+18)-1+kprob(ipkprb+39)*kprob(ipkprb+6)+ &
                  ivec + ipkprb
         nphys  = kprob(kpivec)
         if ( nphys<0 ) then

!        --- nphys<0, constant number of degrees of freedom per point

            nphys = -nphys

         else

!        --- nphys>0, number of degrees of freedom per point not constant

            indprf = nphys
            kpivec = kprob(ipkprb+18)-1+(ivec-1)*kprob(ipkprb+39) + ipkprb
            nphys  = 0
            do i = 1, kprob(ipkprb+39)
               nphys = max ( nphys, kprob(kpivec+i) )
            end do
            call sepactivate ( -indprf, 'kprobf' )

         end if
         nunkp = nphys

      case ( 116, 122, 126, 129, 801 )

!     --- ISOL is of type 116:  array of special structure defined per element

         nphys = mod(isol(4),1000)
         nunkp = nphys

      case default

!     --- wrong type of vector

         call errint ( itypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end select  ! case ( itypvc )

1000  call erclos ( 'sepactsolbuf' )

      end
