      subroutine inineighmark ( nodes, nnodes, npoint, kmeshe, nset, &
                                set, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all marked neighbors of a set of nodes and store in array neighbors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), npoint, kmeshe(*), nset, &
              set(nset), iwork(npoint)

!     iwork          i    integer work array containing the marked nodes
!                         iwork(i) = 0: node is not marked
!                                   >0: node is marked
!     kmeshe         i    STANDARD SEPRAN array containing information on
!                         nodal points connected to nodal points
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Set of nodes from which neighbors must be found
!     npoint         i    Number of nodal points in mesh
!     nset          i/o   at output: Number of neighbors of nodes
!                         at input: declared length of set
!     set            o    Set of neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nneigh, ineigh, i
      integer, allocatable :: neighs(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ineigh         neighbor
!     neighs         set of neighbors
!     nneigh         Number of neighbors in neighs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ININEIGHBORSET Find all neighbors of a set of nodes and store in array
!                    neighbors
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inineighmark' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inineighmark'
         call prinin ( nodes, nnodes, 'nodes' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( neighs(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'neighs' )
      if ( ierror/=0 ) go to 1000

!     --- Find all neighbors of the set nodes and store in neighs

      nset = 0
      nneigh = npoint
      call inineighborset ( nodes, nnodes, npoint, kmeshe, nneigh, &
                            neighs )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( neighs, nneigh, 'neighs' )

!     --- Put all nodes of neighs satisfying iwork(node)>0 into set

      do i = 1, nneigh

         ineigh = neighs(i)
         if ( iwork(ineigh)==1 .or. iwork(ineigh)==10 ) then

!        --- node is marked, add to set

            nset = nset+1
            set(nset) = ineigh

         end if  ! ( iwork(ineigh)==1 )

      end do  ! i = 1, nneigh

      deallocate ( neighs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'neighs' )

1000  call erclos ( 'inineighmark' )
      if ( debug ) then

!     --- Debug information

         call prinin ( set, nset, 'set' )
         write(irefwr,*) 'End inineighmark'

      end if  ! ( debug )

      end
