      subroutine mshsortel2d ( lenkmeshac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-08-2012 Remove iwork form parameter list
!        version  1.1  date 19-05-2002  nsplit=0 possible (RvL)
!        version  1.0  date 25-02-2000
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part ac with information of in which part of the
!     region an element is positioned
!     Two-dimensional grid
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lenkmeshac

!     lenkmeshac     o    Length of array kmeshac
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, nx, ny, ifirstnode, ilastnode, isplit, &
              nnodes
      integer, allocatable :: iwork(:)
      double precision x, y, xmin, xmax, ymin, ymax

!     i              Counting variable
!     ifirstnode     First node in a part
!     ilastnode      Last node in a part
!     isplit         Sequence number of discontinuous part
!     istart         First position used at a part
!     iwork          integer work array to store nodes in a set
!     nnodes         Number of nodes
!     nx             number of points in x direction
!     ny             number of points in y direction
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFLGRIDNOD2D  Fill information of a set of nodes in rectangular 2d grid
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear pointer array
!     nsplit := kmesh(44)
!     For isplit := 1 (1) nsplit do
!        Compute xmin, xmax, ymin, ymax for region
!        Store all nodes of part into iwork(1,.)
!        nx := min(100,sqrt(number of nodes))
!        dx := (xmax-xmin)/dx
!        For all nodes in part do
!           Compute i,j index for node and store in iwork(2-3,.)
!           Adapt Pointer array pos k = (i-1)*nx+j
!        Accumulate pointer array
!        For all nodes in part do
!           Fill node in second part of kmeshac
!           Adapt pointer array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsortel2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsortel2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ifirstnode = 0
      istart = 4*nsplit

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear kmeshac part 1

      kmeshac(1:istart) = 0
      istart = istart+1

!     --- For isplit := 1 (1) nsplit do

      do isplit = 1, max(1,nsplit)

!     --- iwork(1,.) := all nodes in region isplit

         if ( nsplit<=1 ) then

!        --- The region consists of one part only

            nnodes = npoint
            do i = 1, nnodes
               iwork(i) = i
            end do

         else

!        --- The region consists of more parts

            ilastnode = kmeshk(nlevel+isplit)
            nnodes = ilastnode-ifirstnode
            if ( kelmj==0 ) then

!           --- No renumbering of nodes

               do i = 1, nnodes
                  iwork(i) = ifirstnode+i
               end do

            else

!           --- Nodes are renumbered

               do i = 1, nnodes
                  iwork(i) = kmeshj(ifirstnode+i)
               end do

            end if
            ifirstnode = ilastnode

         end if

!        --- Compute xmin, xmax, ymin and ymax

         xmin = rinfin
         ymin = rinfin
         xmax =-rinfin
         ymax =-rinfin
         do i = 1, nnodes
            x = coor(1,iwork(i))
            y = coor(2,iwork(i))
            xmin = min(xmin,x)
            ymin = min(ymin,y)
            xmax = max(xmax,x)
            ymax = max(ymax,y)
         end do
         rmeshac(1,isplit) = xmin
         rmeshac(2,isplit) = xmax
         rmeshac(3,isplit) = ymin
         rmeshac(4,isplit) = ymax

!        --- nx := min(100,sqrt(number of nodes))
!            dx := (xmax-xmin)/dx

         x = nnodes
         nx = max(1,nint(sqrt(x)))
         nx = min(100,nx)
         ny = nx
         kmeshac((isplit-1)*4+1) = nnodes
         kmeshac((isplit-1)*4+2) = nx
         kmeshac((isplit-1)*4+3) = ny
         kmeshac((isplit-1)*4+4) = istart

         call prflgridnod2d ( kmeshac, nx, ny, nnodes, coor, iwork, &
                              istart, xmin, xmax, ymin, ymax )

         istart = istart+nx*ny+1+nnodes

      end do
      lenkmeshac = istart-1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshac, lenkmeshac, 'kmeshac' )
         write(irefwr,*) 'End mshsortel2d'

      end if  ! ( debug )
      call erclos ( 'mshsortel2d' )

      end
