      subroutine prfindelm1 ( coor, inode, xgauss, igauss, xi, found, &
                              phi, ishapelam )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.1  date 07-04-2004 Update (RvL)
!        version  1.0  date 06-02-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine local coordinates of xgauss on curved line
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
!     obstacle
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode(*), igauss, ishapelam
      double precision  coor(2,*), xi, phi(3), xgauss(ishapelam,2)
      logical found

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     found          o    parameter that determines if jnode is in the
!                         element (true) or not (false)
!     igauss         i    Counter for gauss point
!     inode          i    Array of length 3 containing node numbers
!                         of element
!     ishapelam      i    Shape of lambda basis functions
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!     xi             o    local coordinate of jnode within the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  j, k, iter
      double precision sum, sum1, dphi(3), eps, delta, xcoor, ycoor, &
                       xmin, xmax, ymin, ymax, dx, dy, eps1

!     delta          jacobian of element
!     dphi           Array of length 9 to store the values of dphi/dxi
!                    according to:
!                               k
!                    dphi /dxi(x ) = dpdxi(i,k)
!                        i
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     eps            Local accuracy
!     eps1           Local accuracy
!     iter           Iteration sequence number
!     j              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
!     sum1           Parameter to compute a sum
!     xcoor          x coordinate
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     ycoor          y coordinate
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindelm1')
      debug=.false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindelm1'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- find local coordinates of intersection point using
!         a newton raphson iteration scheme

      found=.false.

      if ( debug ) then
         write(irefwr,*) &
            'coordinates  ',xgauss(igauss,1), xgauss(igauss, 2)
         write(irefwr,*) 'node numbers  ', &
                    inode(1), coor(1,inode(1)),coor(2,inode(1))
         write(irefwr,*) 'node numbers  ', &
                    inode(2), coor(1,inode(2)),coor(2,inode(2))
         write(irefwr,*) 'node numbers  ', &
                    inode(3), coor(1,inode(3)),coor(2,inode(3))
      end if

!     --- Determine if coordinate xgauss lies in square around
!         the line element. If not, go to end of file.

      eps1=2d-1
      xmin=min(min(coor(1,inode(1)), coor(1,inode(3))), &
                   coor(1,inode(2)))
      ymin=min(min(coor(2,inode(1)), coor(2,inode(3))), &
                   coor(2,inode(2)))
      xmax=max(max(coor(1,inode(1)), coor(1,inode(3))), &
                   coor(1,inode(2)))
      ymax=max(max(coor(2,inode(1)), coor(2,inode(3))), &
                   coor(2,inode(2)))
      dx=xmax-xmin
      dy=ymax-ymin
      dx=max(dx,dy)
      xmax=xmax+eps1*dx
      ymax=ymax+eps1*dx
      xmin=xmin-eps1*dx
      ymin=ymin-eps1*dx

      if ( ( xgauss(igauss,1).gt.xmax.or. &
             xgauss(igauss,1).lt.xmin ) .or. &
           ( xgauss(igauss,2).gt.ymax.or. &
             xgauss(igauss,2).lt.ymin ) ) then
         xi=2d0
         if ( debug ) write(irefwr,*)'xmax, xmin, ymax, ymin', &
                               xmax, xmin, ymax, ymin
         goto 1000
      end if

!     --- determine if x or y coordinate should be used for finding
!         the local coordinate xi based on the direction of the line
!         element

      if ( abs ( coor(1,inode(3))-coor(1,inode(1)) )/ &
           abs ( coor(2,inode(3))-coor(2,inode(1)) ).gt.1d0 ) then
         xcoor=xgauss(igauss,1)
         k=1
      else
         xcoor=xgauss(igauss,2)
         k=2
      end if

      xi=5d-1
      iter=0
      delta=1d0
      eps=1d-10
      do while ( abs(delta).gt.eps .and. iter.le.8 .and. &
                 abs(delta).lt.1d1 )
         iter = iter+1

!        --- compute some functions for local coordinate xi

         phi(1) =  2d0*(xi-0.5d0)*(xi-1d0)
         phi(2) = -4d0*xi*(xi-1d0)
         phi(3) =  2d0*xi*(xi-0.5d0)
         dphi(1) =  4d0*xi-3d0
         dphi(2) = -8d0*xi+4d0
         dphi(3) =  4d0*xi-1d0

!        --- compute error delta

         sum=0d0
         sum1=0d0
         do j=1,3
            sum = sum + dphi(j)*coor(k,inode(j))
            sum1 = sum1 + phi(j)* coor(k,inode(j))
         end do
         sum1=sum1-xcoor
         delta=-sum1/sum

!        --- update local coordinate

            xi=xi+delta

            if ( debug ) write(irefwr,*)'xi  ', xi, '  delta  ', delta

      end do

      phi(1) =  2d0*(xi-0.5d0)*(xi-1d0)
      phi(2) = -4d0*xi*(xi-1d0)
      phi(3) =  2d0*xi*(xi-0.5d0)

      if ( debug ) &
         write(irefwr,*) 'phiprfind ', phi(1), phi(2), phi(3)

      ycoor=0d0
      if (k.eq.1) then

         do j=1,3
            ycoor=ycoor+phi(j)*coor(2,inode(j))
         end do

         if ( debug ) &
            write(irefwr,*) 'ycoor-ygauss ', ycoor-xgauss(igauss,2)

      else

         do j=1,3
            ycoor=ycoor+phi(j)*coor(1,inode(j))
         end do

         if ( debug ) &
            write(irefwr,*) 'ycoor-ygauss ', ycoor-xgauss(igauss,1)

      end if

      if (abs(delta).ge.1d-6) xi=2d0

      if ( debug ) &
         write(irefwr,*) 'xcoor-xgauss ', xcoor- xgauss(igauss,k)

      if (xi.le.1d0-eps.and.xi.ge.eps) then

!     --- local coordinate is found

         found=.true.

      else if (xi.le.1d0-1d-2.and.xi.ge.-1d-2) then

         if (k.eq.1) then

            if (ycoor-xgauss(igauss,2).lt.1d-3) then

!           --- local coordinate is found

               found=.true.

            end if

         else if (k.eq.2) then

            if (ycoor-xgauss(igauss,1).lt.1d-3) then

!           --- local coordinate is found

               found=.true.
            end if

         end if

      end if

1000  call erclos ( 'prfindelm1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindelm1'

      end if

      end
