      subroutine prstorevec1 ( ivecin, iseqvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 04-09-2016 New parameter list
!        version  1.0  date 09-02-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new vector by computing a vector expression
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     create
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemnsub
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivecin
      integer, intent(out) :: iseqvec

!     iseqvec        o    Starting address of sequence numbers of vectors
!     ivecin         i    Sequence number of vector expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icount, istore(100), vecs(100), numvecrhs, numrealrhs, &
                 ivec, itypvc, lenvec
      double precision :: store(100)
      logical :: complexvec

!     complexvec     If true the vector to be filled is complex
!                    they are not
!     icount         Counter
!     istore         Contains information of the expression
!                    See subroutine READEXPR, array iwork
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     lenvec         Length of left-hand side vector in expression
!     numrealrhs     Number of reals stored in store
!     numvecrhs      Number of vectors in right-hand side
!                    Length of array vecs
!     store          Contains real information of the expression
!                    See subroutine READEXPR, array work
!     vecs           In this array of size numvecrhs the sequence numbers
!                    of the right-hand side vectors are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRVECCHECKLEN  Check if all vectors at right-hand side have the same
!                    length as the vector on the right-hand side
!     PRVECCOMBINAT1 Evaluate right-hand side of vector expression
!                    Complete real vector
!     PRVECFIND1     Find the type of vector to be created for the left-hand side
!                    This depends on the right-hand side
!     PRVECSTRHS     Translate expression to a set of reals and a new set of
!                    temporary vectors
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     222   input vector is complex:  not yet allowed.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstorevec1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstorevec1'
         write(irefwr,1) 'ivecin', ivecin
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Translate expression to a set of reals and a new set of
!         temporary vectors

      call prvecstrhs ( ivecin-200000000, numvecrhs, numrealrhs, &
                        icount, istore, store, vecs )
      call inichkarlen ( icount, 100, 'istore' )
      call inichkarlen ( numrealrhs, 100, 'store' )
      call inichkarlen ( numvecrhs, 100, 'vecs' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'numvecrhs, numrealrhs, icount', &
                          numvecrhs, numrealrhs, icount
         call prinin ( istore, icount, 'istore' )
         call prinrl ( store, numrealrhs, 'store' )
         call prinin ( vecs, numvecrhs, 'vecs' )
      end if  ! ( debug )

!     --- find vector depending on right-hand side

      numvecmn = numvecmn+1
      iseqvec = numvecmn  ! new temporary vector

      if ( debug ) write(irefwr,1) 'numvecmn, iseqvec', numvecmn, iseqvec

      call prvecfind1 ( ivec, itypvc, iseqvec, &
                        vecs, numvecrhs, complexvec )
      if ( debug ) write(irefwr,*) 'complexvec/2', complexvec

      if ( complexvec ) then

!     --- Error: complex vector

         call errsub ( 222, 0, 0, 0 )
         go to 1000

      end if  ! ( complexvec )

!     --- create temporary vector

      call sepcreasolbf1 ( isolut2d(iseqvec), itypvc, ivec, &
                           names(iseqvec), 0 )

!     --- Check if all vectors at the right-hand side have the same length
!         as the left-hand side

      call prvecchecklen ( iseqvec, vecs, numvecrhs, &
                           nphyso, .true. )

!     --- Compute length of vector

      lenvec = ks(isolut2d(iseqvec))%nusol

!     --- Compute vector

      call prveccombinat1 ( iseqvec, vecs, &
                            numvecrhs, lenvec, istore, &
                            store, icount, numrealrhs, nphyso )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( isolut2d(iseqvec), 5, 'isol_temp' )
         write(irefwr,*) 'End prstorevec1'

      end if  ! ( debug )
      call erclos ( 'prstorevec1' )

      end subroutine prstorevec1
