      subroutine prveccombinat5 ( iseqvec, vecs, numvecrhs, idegfd, &
                                  istore, store, icount, numrealrhs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate right-hand side of vector expression
!     Special case: a part of the real vector must be computed
!                   the number of unknowns per point is constant
!                   the whole region is copied, but only one unknown
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     expression
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvec, vecs(*), numvecrhs, &
              istore(*), icount, numrealrhs, idegfd
      double precision store(*)

!     icount         i    Computed length of array istore
!     idegfd         i    Degree of freedom sequence number
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     istore         i    Contains the expression as stored in common
!                         cexpression, where all scalars and reals
!                         have been computed and stored in store
!                         References to these values are given by
!                         their position in store plus 1000
!                         The values of the vectors are stored in
!                         array vectors
!                         References to these vectors are given by
!                         their position in vectors plus 10000
!     numrealrhs     i    Number of reals stored in store
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     store          i    Contains real information of the expression
!                         See subroutine READEXPR, array work
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenvec, nunkpvec
      double precision, allocatable :: outvec(:), vectors(:,:)

!     lenvec         Length of left-hand side vector in expression
!     nunkpvec       nunkp with respect to the output vector
!     outvec         Output vector; result of expression in the nodes of nodes
!     vectors        Array of size lenvec x numvecrhs to store the vectors
!                    of the right-hand side in the nodes required
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPYUS2VEC1  Copy user vector to sepran vector
!                    The user vector has 1 unknown per point in all nodes
!     PREVALVECEXPR  Evaluate vector expression and fill actual values of
!                    vectors
!     PRINRL         Print 1d real vector
!     PRVECFILL3     Fill all right-hand side vectors in expression where
!                    either idegfd>0 or nunkpi = 1 (whole region)
!     PRVECFILL4     Fill all right-hand side vectors in expression where
!                    a fixed number of degrees of freedom per point must
!                    be filled (whole region)
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prveccombinat5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prveccombinat5'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- activate array

      call sepactsolbf2 ( isolut2d(iseqvec) )
      nunkpvec = nunkpi

      if ( debug ) then
         write(irefwr,1) 'idegfd, nunkpi', idegfd, nunkpi
      end if  ! ( debug )

      if ( idegfd>0 .or. nunkpi==1 ) then

!     --- Only one degree of freedom must be filled

         lenvec = npoint       ! 1 degree of freedom per point

         allocate ( outvec(lenvec), vectors(lenvec,numvecrhs), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, lenvec, 'outvec' )
         if ( ierror/=0 ) go to 1000

!        --- Fill all right-hand side vectors

         call prvecfill3 ( isolut2d, iseqvec, vecs, &
                           numvecrhs, vectors, lenvec, idegfd )

!        --- Fill output vector by computing the expression

         call prevalvecexpr ( outvec, vectors, lenvec, &
                              numvecrhs, istore, store, icount, &
                              numrealrhs, names(iseqvec) )

         if ( debug ) call prinrl ( outvec, lenvec, 'outvec' )

!        --- Map output vector in isolut2d
!            Find starting address

         usoltemp => ks(isolut2d(iseqvec))%sol

!        --- copy

         nunkpi = nunkpvec
         call prcopyus2vec1 ( usoltemp, outvec, npoint, &
                              nunkpi, max(1,idegfd) )
         if ( debug ) call prinrl ( usoltemp, lenvec, 'outvec/2' )

      else

!     --- More degrees of freedom must be filled

         lenvec = npoint*nunkpi  ! nunkpi degrees of freedom per point
         allocate ( outvec(lenvec), vectors(lenvec,numvecrhs), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, lenvec, 'outvec' )
         if ( ierror/=0 ) go to 1000

!        --- Fill all right-hand side vectors

         call prvecfill4 ( isolut2d, iseqvec, vecs, &
                           numvecrhs, vectors, lenvec, nunkpi )

!        --- Fill output vector by computing the expression

         call prevalvecexpr ( outvec, vectors, lenvec, &
                              numvecrhs, istore, store, icount, &
                              numrealrhs, names(iseqvec) )

!        --- Map output vector in isolut2d
!            Find starting address

         usoltemp => ks(isolut2d(iseqvec))%sol

!        --- copy

         outvec(1:size(usoltemp)) = usoltemp

      end if  ! ( idegfd>0 )

      if ( allocated(outvec) ) deallocate ( outvec, stat = error )
      if ( allocated(vectors) ) deallocate ( vectors, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'outvec' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prveccombinat5'

      end if  ! ( debug )
      call erclos ( 'prveccombinat5' )

      end subroutine prveccombinat5
