      subroutine readfile ( filename, firstrec, nintread, &
                            nrealread, intarr, realarr, iformat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-06-2013 New definition file ref numbers
!        version  1.1  date 06-11-1999 Extension with iformat = 4
!        version  1.0  date 28-06-1998
!
!   copyright (c) 1998-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Open ASCII file with name filename
!    Check if file exists, if not give error message
!    Read information from ASCII file with reference number ireffile
!    Close file
! **********************************************************************
!
!                       KEYWORDS
!
!     file
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) filename
      logical firstrec
      integer nintread, nrealread, intarr(*), iformat
      double precision realarr(*)

!     filename   i    Name of file
!     firstrec   i    If true only the first record must be read
!     iformat    i    Type of format to be used in read statements
!                     Possible values:
!                     1: The first record contains n (=number of integers)
!                        The next n records contain one integer number
!                     2: The first record contains n (=number of integers)
!                        The next n records contain one integer and one real
!                        number
!                     3: The first record contains n (=number of integers)
!                        The next n records contain one integer and ndegfd real
!                        numbers
!                        The value of ndegfd is given at input in nintread
!                     4: The first record contains n (=number of integers)
!                        The next n records contain two integers and ndegfd real
!                        numbers
!                        The value of ndegfd is given at input in nintread
!     intarr     o    In this array the integers read are stored sequentially
!                     If more than one integer per record is needed the
!                     sequence is nintread times integer 1 followed by
!                     nintread  times integer 1 etc.
!     nintread  i/o   At output: number of integers read
!                     Depending on iformat nintread may be an input parameter
!                     See the description of nintread
!     nrealread  o    number of reals read
!     realarr    o    In this array the reals read are stored sequentially
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ireffile, ios, i, k, ndegfd
      logical check

!     check      Indicates if the file exist (true) or not
!     i          Counting variable
!     ios        IOSTAT variable, returns from reading or writing
!     ireffile   File reference nummber
!     k          Counting variable
!     ndegfd     Number of real numbers to be read per line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1318   Error in reading first record of file
!    1669   file does not exist
!    1670   First entry in the file non-positive
!    1671   Error in reading of file
!    2706   error in opening file
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Check if file exists
!    Open file
!    Read first record of file
!    if ( not firstrec ) then
!       Read rest of records
!    Close file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readfile' )

!     --- Check if file exists

      inquire ( file=filename, exist=check )
      if ( .not. check ) then

!     --- File does not exist

         call errchr ( filename, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if

!     --- open file

      ireffile = inisetnewref()
      open ( unit=ireffile, file=filename, iostat = ios )
      if ( ios/=0 ) then

!     --- error in opening file

         call errint ( ios, 1 )
         call errchr ( filename, 1 )
         call errsub ( 2706, 1, 0, 1 )
         go to 1000

      end if

!     --- Read first record

      if ( iformat>=3 ) ndegfd = nintread
      nintread = 0
      nrealread = 0
      if ( iformat<=4 ) then

!     --- iformat<=2, read number of items

         read(ireffile,*,iostat = ios) nintread
         if ( ios/=0 ) then

!        --- error in reading file

            call errint ( ios, 1 )
            call errchr ( filename, 1 )
            call errsub ( 1318, 1, 0, 1 )
            go to 1000

         else if ( nintread<=0 ) then

!        --- Number of points in file non-positive

            call errint ( nintread, 1 )
            call errchr ( filename, 1 )
            call errsub ( 1670, 1, 0, 1 )

         end if

      end if

      if ( .not. firstrec ) then

!     --- firstrec = false, read rest of records

         if ( iformat==1 ) then

!        --- iformat = 1, read nintread integers

            do i = 1, nintread
               read(ireffile,*,iostat = ios) intarr(i)
               if ( ios/=0 ) then

!              --- error in reading file

                  call errint ( i+1, 1 )
                  call errint ( ios, 2 )
                  call errchr ( filename, 1 )
                  call errsub ( 1671, 2, 0, 1 )
                  go to 1000

               end if

            end do

         else if ( iformat==2 ) then

!        --- iformat = 2, read nintread integers and reals

            do i = 1, nintread
               read(ireffile,*,iostat = ios) intarr(i), realarr(i)
               if ( ios/=0 ) then

!              --- error in reading file

                  call errint ( i+1, 1 )
                  call errint ( ios, 2 )
                  call errchr ( filename, 1 )
                  call errsub ( 1671, 2, 0, 1 )
                  go to 1000

               end if

            end do

         else if ( iformat==3 ) then

!        --- iformat = 3, read nintread integers and ndegfd*nintread reals

            do i = 1, nintread
               read(ireffile,*,iostat = ios) intarr(i), &
                   (realarr((i-1)*ndegfd+k), k = 1, ndegfd)
               if ( ios/=0 ) then

!              --- error in reading file

                  call errint ( i+1, 1 )
                  call errint ( ios, 2 )
                  call errchr ( filename, 1 )
                  call errsub ( 1671, 2, 0, 1 )
                  go to 1000

               end if

            end do

         else if ( iformat==4 ) then

!        --- iformat = 4, read 2*nintread integers and ndegfd*nintread reals

            do i = 1, nintread
               read(ireffile,*,iostat = ios) intarr(i), &
                   intarr(i+nintread), &
                   (realarr((i-1)*ndegfd+k), k = 1, ndegfd)
               if ( ios/=0 ) then

!              --- error in reading file

                  call errint ( i+1, 1 )
                  call errint ( ios, 2 )
                  call errchr ( filename, 1 )
                  call errsub ( 1671, 2, 0, 1 )
                  go to 1000

               end if

            end do

         end if

      end if
      close ( unit=ireffile )
      call inirmref ( ireffile )

1000  call erclos ( 'readfile' )
      end
