      subroutine seper0026 ( icrv1, icrv2, jstep, n, iwork, contiguous, &
                             kmeshmloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill nodal points of curves icrv1 to icrv2 in array iwork
!     Old subroutine er0026
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     fill
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icrv1, icrv2, jstep, n, iwork(*), kmeshmloc(*)
      logical contiguous

!     contiguous     i    If true the curves must be contiguous, otherwise
!                         gaps between curves are allowed
!     icrv1          i    First curve number to be stored in iwork
!     icrv2          i    Last curve number to be stored in iwork
!                         icrv1 to icrv2 must be subsequent curves
!     iwork          o    array containing the nodal point numbers of the
!                         curves subsequently
!                         If point does not belong to a node, the address of
!                         the point with respect to curves provided with a
!                         minus sign is stored
!     jstep          i    number of points per element minus one
!                         (curve elements)
!     kmeshmloc      i    KMESH part m
!                         In case of parallel might be kmeshglob
!     n              o    number of points on the curves icrv1 to icrv2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, ih, iw, k, k1, k2, ifirst, ilast, j, i, icurnr, &
              istorecurves(ncurvs), nodp, nsubcv, numcurvs, &
              idirec, jcrv1, jcrv2

!     i              Counting variable
!     icurnr         Curve sequence number
!     idirec         Defines direction of curve
!     ifirst         First curve (surface) number read
!     ih             Help variable to store a constant temporarily
!     ilast          Last curve (surface) number read
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istorecurves   Work array of length ncurvs to store the subcurves of a
!                    composite curve
!     iw             Address in array IWORK
!     j              Counting variable
!     jcrv1          |icrv1|
!     jcrv2          |icrv2|
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
!     nsubcv         Number of subcurves of a curve
!     numcurvs       Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      45   first point in part of boundary is not equal to last point
!     100   JSTAP has wrong value
!     185   first curve number (icrv1) is out of range
!     186   last curve number (icrv2) is out of range
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seper0026' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seper0026'
         write(irefwr,1) 'icrv1, icrv2, ncurvs, jstep', &
                          icrv1, icrv2, ncurvs, jstep
         call prinin ( kmeshmloc, ncurvs+6, 'kmeshmloc first part' )
  1      format ( a, 1x, (10i6) )

      end if

!     --- Perform some tests

      if ( icrv1==icrv2 .and. icrv1<0 ) then

!     --- Special case of one negative curve
!         Make curve number positive and store direction

         jcrv1 = -icrv1
         jcrv2 = jcrv1
         idirec = -1

      else

!     --- Standard case

         jcrv1 = icrv1
         jcrv2 = icrv2
         idirec = 1

      end if
      if ( jcrv1<=0 .or. jcrv1>ncurvs ) then
         call errint ( icrv1, 1 )
         call errint ( ncurvs, 2 )
         call errsub ( 185, 2, 0, 0 )
      end if
      if ( jcrv2<jcrv1 .or. jcrv2>ncurvs ) then
         call errint ( icrv2, 1 )
         call errint ( icrv1, 2 )
         call errint ( ncurvs, 3 )
         call errsub ( 186, 3, 0, 0 )
      end if

!     --- Extract parameters

      istar3 = kmeshmloc(2)-1
      inelem = 0
      if ( ierror/=0 ) go to 1000

!     --- Store curve numbers into istorecurves

      if ( kelmq==0 ) then

!     --- kelmq = 0, no composite curves

         do k = icrv1, icrv2
            istorecurves(k+1-icrv1) = k
         end do
         numcurvs = icrv2-icrv1+1

      else

!     --- kelmq > 0, there are composite curves

         if ( debug ) then

!        --- Debug information

            call prinin ( kmeshq, ncurvs+1, 'kmeshq-part 1' )

         end if

         numcurvs = 0
         do k = jcrv1, jcrv2
            nsubcv = kmeshq(k+1) - kmeshq(k)
            if ( nsubcv==0 ) then

!           --- No subcurves use curve itself

               numcurvs = numcurvs+1
               istorecurves(numcurvs) = idirec*k

            else

!           --- There are subcurves

               if ( idirec==1 ) then

!              --- Standard direction

                  do j = 1, nsubcv
                     numcurvs = numcurvs+1
                     istorecurves(numcurvs) = kmeshq( kmeshq(k)+j )
                  end do

               else

!              --- Reversed direction

                  do j = nsubcv, 1, -1
                     numcurvs = numcurvs+1
                     istorecurves(numcurvs) = -kmeshq( kmeshq(k)+j )
                  end do

               end if

            end if
         end do

      end if
      if ( numcurvs>ncurvs ) then

!     --- Error array length too short

         call errint ( numcurvs, 1 )
         call errint ( ncurvs, 2 )
         call errchr ( 'istorecurves', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      if ( debug ) &
         call prinin ( istorecurves, numcurvs, 'istorecurves' )

!     --- Loop over all curves

      iw = 1
      do i = 1, numcurvs

!     --- Find curve number

         icurnr = istorecurves(i)
         if ( icurnr>0 ) then

!        --- icurnr > 0: positive direction

            k1 = kmeshmloc(4+icurnr)
            k2 = kmeshmloc(5+icurnr)-1

         else

!        --- icurnr < 0: negative direction

            k1 = kmeshmloc(5-icurnr)-1
            k2 = kmeshmloc(4-icurnr)

         end if
         if ( debug ) &
            write(irefwr,*) 'i, icurnr, k1, k2 ',i, icurnr, k1, k2

         if ( i/=1 ) then

!        --- i # 1, hence not the first curve

            ifirst = kmeshmloc(istar3+k1)
            if ( ifirst/=ilast) then

!           --- first point of new curve not equal to last point of
!               previous one

               if ( contiguous ) then

!              --- Error the curves must be contiguous

                  call errint ( ifirst, 1 )
                  call errint ( ilast, 2 )
                  call errint ( k, 3 )
                  call errint ( 0, 4 )
                  call errsub ( 45, 4, 0, 0 )
                  if ( nerror>=10 ) go to 1000

               else

!              --- The curves do not have to be contiguous

                  iw = iw+1

               end if

            end if

         end if

!        --- Fill iwork

         if ( icurnr>0 ) then

!        --- Positive direction

            do j = k1, k2
               nodp = kmeshmloc(istar3+j)
               if ( nodp<=0 ) nodp = -j
               iwork(iw+j-k1) = nodp
            end do
            n = k2-k1

         else

!        --- Negative direction

            do j = k1, k2, -1
               nodp = kmeshmloc(istar3+j)
               if ( nodp<=0 ) nodp = -j
               iwork(iw+k1-j) = nodp
            end do
            n = k1-k2

         end if
         if ( debug ) write(irefwr,1) 'n, jstep, iw', n, jstep, iw
         ih = n/jstep
         if ( ih*jstep/=n ) then
            call errint ( ih*jstep, 1 )
            call errint ( n, 2 )
            call errsub ( 100, 2, 0, 0 )
         end if
         inelem = inelem+ih

         iw = iw+n
         ilast = kmeshmloc(istar3+k2)

      end do
      n = iw

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'n, inelem', n, inelem
         call prinin ( iwork, n, 'iwork' )
         write(irefwr,*) 'End seper0026'

      end if  ! ( debug )
      call erclos ( 'seper0026' )

      end subroutine seper0026
