      subroutine sepfillnmeshc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the arrays nmeshc and nmeshd with node numbers of edges and
!     edge numbers of elements
!     Old subroutine topfillnmeshc
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     topology
!     edge
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jkadersv, jmarksv, lpnmpc, &
                 lpnmpd, medpel, mndped, idum(1)
      integer, allocatable :: nmeshdtmp(:)

!     idum           General dummy variable
!     jkadersv       Help variable to store the value of jkader
!     jmarksv        Help variable to store the value of jmark
!     lpnmpc         length of array nmeshc
!     lpnmpd         length of array nmeshd
!     medpel         maximum number of edges in one element
!     mndped         maximum number of nodes in one edge
!     nmeshdtmp      Work array to store nmeshd temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PL0000         Compute length and width of the plot
!     PL0001         Compute the parameters xmin, xmax, ymin and ymax
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELM1         Plot elements in 2D mesh
!     PLOTEDGENR     Plot edge sequence numbers
!     PRININ         print 1d integer array
!     SEPEDG01       creates the edges part 1 pointers
!     SEPEDG02       creates the edges part 2 actual edge labels and lmeshc
!     SEPFILLEDGES   Fill array nmesh parts c and d
!     SEPFILLKMESHAA Fill kmesh part aa
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill kmesh part d if necessary
!     Create temporary space for nmeshd
!     Fill first part of nmeshd
!     Make the array permanent
!     Fetch kmesh part c,d and nmesh part d again
!     Create temporary space for nmeshc
!     Compute starting addresses
!     Fill nmeshd with actual data and create edge:nodes array nmeshc
!     Make nmeshc permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmeshc'
         write(irefwr,1) 'nelem, npoint, kelmd', nelem, npoint, kelmd
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( km(iseqkm)%kmesh(18)==0 ) call sepfillkmeshd
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprinkmesh ( 2 )
      if ( km(iseqkm)%kmesh(48)==0 ) call sepfillkmeshaa
      if ( ierror/=0 ) go to 1000

      nullify ( nmeshc, nmeshd )
      if ( associated(km(iseqkm)%nmeshc) ) &
         deallocate(km(iseqkm)%nmeshc, km(iseqkm)%nmeshd)

!     --- Create temporary space and fill first part of nmeshd
!         for each element pointers to the edges are created
!         and edge labels are set to zero. Moreover, the number
!         of connecting edges (ncone) is computed

      medpel = nmesh(22)      ! maximum number of edges per element
      lpnmpd = nelem*(2+medpel)  ! overestimation of length of nmeshd
      if ( debug ) &
         write(irefwr,1) 'kelmaa, medpel, nelem, lpnmpd', &
                          kelmaa, medpel, nelem, lpnmpd

      allocate ( nmeshd(lpnmpd), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'nmeshd' )
      if ( ierror/=0 ) go to 1000

!     --- Fill first part of nmeshd and compute length of array

      call sepedg01
      lpnmpd = length

!     --- Create final array nmeshd and copy first part

      allocate ( km(iseqkm)%nmeshd(lpnmpd) )
      km(iseqkm)%nmeshd = nmeshd(1:lpnmpd)

      deallocate ( nmeshd )
      nmeshd => km(iseqkm)%nmeshd

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lpnmpd', lpnmpd
         call prinin ( nmeshd, nelem, 'nmeshd part 1' )

      end if

!     --- Create temporary space for nmeshc + extra mndped

      mndped =  nmesh(23)       ! maximum number of nodes in one edge
      length = nelem*medpel*(2+mndped) ! overstimation of length of nmeshc

      allocate ( nmeshc(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'nmeshc' )
      if ( ierror/=0 ) go to 1000
      lpnmpc = length

!     --- Compute starting addresses

      if ( debug ) write(irefwr,1) 'nspec', nspec

!     --- Fill nmeshd with actual data and create edge:nodes array nmeshc
!         nmeshd is already initialized for each connection element and
!         will be filled with actual data, nmeshc is filled with nodes for
!         each edge

      if ( nspec==0 ) then

!     --- nspec = 0, new method

         call sepfilledges ( mndped )
         if ( debug ) then

!        --- Debug information

            call prinin ( nmeshd, lpnmpd, 'nmeshd' )
            call prinin ( nmeshc, lpnmpc, 'nmeshc' )

         end if

         if ( plotedges .and. ndim==2 ) then

!        --- Plot edge numbers

            jkadersv = jkader
            jkader = -1
            jtimes = 0
            jmarksv = jmark
            jmark = 3

!           --- compute xmin, xmax, ymin, ymax, aleng and width

            call pl0001 ( coor, 2, npoint, xmin, xmax, ymin, ymax )
            call pl0000 ( 15d0, 1d0 )

!           --- Start picture

            call plafep(1, aleng, width, 0d0, 0d0, 0)

!           --- Plot the elements

            call plelm1 ( idum, coor, .true., .true., .false. )

!           --- Plot the edge sequence numbers

            call plotedgenr ( coor, nmeshc, nedges )

!           --- Close plot dataset

            call plafep(2, 0d0, 0d0, 0d0, 0d0, 0)
            jmark = jmarksv
            jkader = jkadersv

         end if

      else

!     --- nspec>0, old method

         call sepedg02 ( mndped )
         lpnmpc = length

      end if

!     --- Make nmeshc permanent

      allocate ( km(iseqkm)%nmeshc(lpnmpc) )
      km(iseqkm)%nmeshc = nmeshc(1:lpnmpc)
      nmesh(4) = nedges
      nmesh(33) = lpnmpc
      nmesh(34) = lpnmpd

      deallocate ( nmeshc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshc' )

      nmeshc => km(iseqkm)%nmeshc

1000  if ( debug ) then

!     --- Debug = true, print nmeshc and nmeshd

         call sepprinkmesh ( 39 )
         call sepprinkmesh ( 40 )
         write(irefwr,*) 'End sepfillnmeshc'

      end if
      call erclos ( 'sepfillnmeshc' )

      end subroutine sepfillnmeshc
