      subroutine sepfillnodcurs ( inum, inpsep, nodes, nnodes, ipoint, &
                                  nparts, kfill )
! ======================================================================
!
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes with nodal points as defined in array inpsep
!     Special case: nodes in curve
!     Old subroutine prfillnodcurs
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nodes(*), nnodes, inum, &
              ipoint(*), nparts, kfill

!     inpsep         i    The integer input read is stored in array INPSEP
!                         The input depends in inum
!                         if inum>0, inpsep(1:inum) contains inum curve numbers
!                         if inum = 0 then
!                         Pos. 1 indicates how array inpsep is filled.
!                         Possible values:
!                          0   only one curve is used. The curve
!                              number is stored in inpsep(2)
!                         -1   The solution is printed along the curves
!                              icrv1 to icrv2. icrv1 is stored in
!                              inpsep(2) and icrv2 in inpsep(3).
!                         -2   See -1, however, the sequence of the printing is
!                              reversed. In case of curves this means that the
!                              curves are printed from the end point of icrv2 to
!                              the initial point of icrv2, then the end point of
!                              icrv2-1 to the begin point of icrv2-1 and so on.
!                         n>0  The curve numbers numbers are stored
!                              in inpsep(2) to inpsep(n+1).
!                              That sequence is used.
!                         if inum = -1 then
!                         Pos. 1 contains icurv1
!                              2 contains icurv2
!                              3 contains both iexclu and istep according to
!                                istep+100*iexlcu with
!                                istep defines step in curves
!                                iexclu  Indicates which points are excluded
!                                  0:  No points must be excluded
!                                  1:  Exclude initial point
!                                  2:  Exclude end point
!                                  3:  Exclude both initial and end point
!     inum           i    See inpsep
!     ipoint         o    Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the boundary consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     kfill          i    Defines how the curves must be filled
!                          2: curves must be filled depending on inum and inpsep
!                         22: inpsep(1) contains istep+100*iexlcu
!                             inpsep(2) contains number of curves
!                             inpsep(3... ) contain the curve numbers
!     nnodes         o    Number of nodes
!     nodes          o    Contains the node numbers that must be filled
!     nparts         o    number of parts the boundary consists of
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: num, iexclu, inpelm, istep
      integer, allocatable :: icurvs(:)

!                    they are not
!     icurvs         is used only if inum>0
!                    Pos. 1 contains inum, pos(2:inum+1) contains inpsep
!     iexclu         Indicates which points are excluded
!     inpelm         number of points per element
!     istep          Step in elements
!     num            Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLCURSTEP  Fill array nodes depending on the values of iexclu and
!                    istep
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     SEPCURVENODES  Store all nodes in one curves in array inodes and gaps in
!                    these curves in array ipoint
!     SEPOUTCURVNOD  Find all nodes in the outer curves and store in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     558   The parameter l for the definition of essential boundary conditions
!           or connected elements has an incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnodcurs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnodcurs'
         write(irefwr,1) 'inum, kfill', inum, kfill
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nparts = 1

      length = max(3,ncurvs+2)
      allocate ( icurvs(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'icurvs' )
      if ( ierror/=0 ) go to 1000

      if ( kfill==2 .and. inum==0 ) then

!     --- inum = 0, Use inpsep, which contains the curve numbers

         call sepcurvenodes ( inpsep, ipoint, nodes, nparts, kmeshm )
         nnodes = ipoint(nparts+1)-1

      else if ( inum==-1 .and. inpsep(1)==-1 ) then

!     --- Special case all outer curves

         call sepoutcurvnod ( nnodes, nodes )

      else if ( inum>0 ) then

!     --- inum>0, Fill icurvs

         call inichkarlen ( inum+1, length, 'icurvs' )
         if ( ierror/=0 ) go to 1000

         icurvs(1) = inum
         icurvs(2:inum+1) = inpsep(1:inum)

         call sepcurvenodes ( icurvs, ipoint, nodes, nparts, kmeshm )
         nnodes = ipoint(nparts+1)-1

      else

!     --- inum = -1, or kfill = 22 special case

         if ( kfill==22 ) then

!        --- kfill = 22, special storage

            istep  = inpsep(1)
            num = inpsep(2)
            call inichkarlen ( num+1, length, 'icurvs' )
            if ( ierror/=0 ) go to 1000

            icurvs(1) = num
            icurvs(2:num+1) = inpsep(3:num+2)

         else

!        --- kfill = 2, classical storage

            istep  = inpsep(3)

            icurvs(1) = -1
            icurvs(2:3) = inpsep(1:2)  ! icurv1 and icurv2

         end if  ! ( kfill=22 )

!        --- Fill nodes

         call sepcurvenodes ( icurvs, ipoint, nodes, nparts, kmeshm )
         nnodes = ipoint(nparts+1)-1

         if ( istep>0 ) then

!        --- istep>0,  compute iexclu

            iexclu = istep/100
            if ( iexclu>3 ) then
               call errint ( iexclu, 1 )
               call errint ( istep, 2 )
               call errsub ( 558, 2, 0, 0 )
            end if
            istep = istep-100*iexclu
         else
            iexclu = 0
         end if
         inpelm = abs(istep) + 1

!        --- Correct for initial and end point in the case that exclude>0

         call erfillcurstep ( istep, iexclu, nnodes, inpelm, nodes )

      end if  ! ( inum>0 )

      deallocate ( icurvs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'icurvs' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nparts', nparts
         call prinin ( ipoint, 2, 'ipoint' )
         write(irefwr,*) 'End sepfillnodcurs'

      end if  ! ( debug )
      call erclos ( 'sepfillnodcurs' )

      end
