      subroutine sepstmnew ( iinstr, maxmsh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-01-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    sepstmnew starts the SEPRAN program.
!    It must be called before all other subroutines.
!    The mesh is created by this subroutine and mesh input is read
!    file meshoutput is created if required
!    The input for subroutine PROBDF is stored in array iinput and written
!    to the file sepcomp.inf
!    The files meshoutput, sepcomp.inf and sepcomp.out are opened,
!    the file meshoutput is closed.
!    The machine dependent reference numbers iref10, iref73 and iref74 get
!    a value
!    sepstmnew replaces the calls of the following SEPRAN subroutines:
!           START:  General SEPRAN starting subroutine
!           MESH:   Create mesh.
!           MESHWR: Write mesh.
!           PROBDF: Reads problem definition.
!           incommat: Computes the structure of the large matrix.
!   sepstmnew is completely identical to SEPSTN, except that the mesh is created
!   and written instead of read from file meshoutput
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     start
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemeshinf
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleinput
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinstr(*)
      integer :: maxmsh

!     iinstr         i    Input array defined by the user
!                         With this array the user may indicate which actions
!                         he wants to be performed by this subroutine
!                         IINSTR must be filled as follows:
!                         Pos. 1: Last entry number that has been filled by the
!                                 user. If 0 or 1 only defaults are used
!                              2: IREAD Indicates if all SEPRAN input until end
!                                       of file or END_OF_SEPRAN_INPUT is read
!                                       (1) or that only the input as described
!                                       for SEPSTR is read plus mesh input (0)
!                                 Default value: 0
!                              3: -
!                              4: WRITE_MESH, defines information about how to
!                                             write the mesh. Possible values:
!                                 0:  Do not write the mesh, not open mesh file
!                                 1:  the mesh created is written to the
!                                     file meshoutput
!                                 2:  The mesh file is opened but no mesh is
!                                     written
!                                 Default value: 0
!                              5: ISUBR:  Type of subroutine,
!                                         see mainsubr in csepmain
!     maxmsh         i    Maximum number of meshes allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lniinp, lniincommat
      parameter ( lniinp=1000, lniincommat=25 )
      integer i, iinput(lniinp), iincommt(lniincommat,10), &
              iref, jstart, iseqnr, idummy, iread, meshw, &
              j, ihelp(10), ishift, isubr, maxcall, ipos, &
              manag(lenmanag), nextrecord, iseqprob
      double precision rinput(10)

!     i              Counting variable
!     idummy         Dummy parameter
!     ihelp          Work array to store input from the input file
!     iincommt       Input parameter for subroutine incommat.
!                    Defines the type of matrix to be used. (See User's Manual)
!                    iincommt is an array of length 10, which means that at
!                    most 10 problems may be solved at one time
!     iinput         Output array of subroutine PROBDF. A fixed length of 500
!                    has been declared. The input for subroutine PROBDF is
!                    written to array iinput and this array is written to the
!                    file sepcomp.out
!                    In this way the output program can call PROBDF again with
!                    the same input
!     ipos           Position of input arrays in sepmoduleinput
!     iread          Indicates if all SEPRAN input until end
!                    of file or END_OF_SEPRAN_INPUT is read (1)
!                    or that only the input as described for SEPSTR is read (0)
!     iref           Local reference number
!     iseqnr         Dummy parameter
!     iseqprob       Sequence number of problem input
!     ishift         Shift in mesh sequence number
!     isubr          Type of subroutine, see mainsubr in csepmain
!     j              General counting variable
!     jstart         Absolute value of first parameter in the call of START
!     lniincommat    Length of first index of iincommat
!     lniinp         Length of array iinput
!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTL
!                    See subroutine readsp
!     maxcall         Maximum number of coefficient inputs read
!     meshw          If 1 the mesh is written to the file meshoutput
!     nextrecord     Defines last record read by readsp
!                    Possible values:
!                    0: none
!                    1: PROBLEM
!                    2: POSTPROCESSING
!                    3: MESH
!                    4: READ MESH
!                    5: EXTRA_MESH_INPUT
!     rinput         Dummy parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCOPY         Copy one integer array into another
!     INIFIL         Open files
!     INIRMF         Remove file if it exists
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     MSH068         write information of the mesh to the file indicated by iref
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     MSHEXTRA       Read extra input for mesh and apply the actions required
!                    by this extra input
!     PROBDFEXT      Reads and stores problem definition
!     READ00         Reads input for subroutine COMMAT from standard input
!                    file.
!     READAL         Read all SEPRAN input for the computational program except
!                    problem definition and preceding information
!     READSP         Reads information for subroutine SEPSTL
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFEMESH      creates a linear element mesh based on the nodes
!                    of a spectral element mesh
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPMESHSUB     General mesh generator
!     SEPRELKMESH1   Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     SEPSTART       Standard SEPRAN starting subroutine
!     WRITPROBIINP   Write arrays iinput and rinput corresponding to kprob to
!                    file sepcomp.out
! **********************************************************************
!
!                       I/O
!
!    Input is read from the standard input file
!    See the various subroutines that perform this action
!    Furthermore the file meshoutput is read
!    File sepcomp.inf is created
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     719   Too much problems are to be created at one time
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( iinstr(1)>=2 ) then
         iread = iinstr(2)
      else
         iread = 0
      end if
      if ( iinstr(1)>=4 ) then
         meshw = iinstr(4)
      else
         meshw = 0
      end if
      if ( iinstr(1)>=5 ) then
         isubr = iinstr(5)
      else
         isubr = 0
      end if

!     --- Start the program

      iinput(1) = lniinp
      iinput(2:5) = 0
      rinput(1) = 10d0
      rinput(2:10) = 0d0

      call initcb
      call initmd

!     --- Open read and write files

      call inifil(5)
      call inifil(6)

!     --- Open standard elements file and error message file

      call inifil(1)
      call inifil(4)

!     --- Read array manag (defaults)

      manag(5) = -2
      call readsp ( manag, lenmanag, nextrecord )

!     --- Standard SEPRAN start subroutine

      jstart = abs(manag(1))
      if ( mod(jstart/100,10)==0 ) jstart = jstart + 100
      if ( manag(1)<0 ) then
         manag(1) = -jstart
      else
         manag(1) = jstart
      end if
      call sepstart ( manag(1), manag(2), manag(3), manag(4) )
      jkader = -1
      call eropen( 'sepstm' )
      call eropen( 'sepstmnew' )
      kmesh => kmesh2d(1)

      if ( kmesh<=0 .or. kmesh>lastmesh ) kmesh = 0
      iseqkm = kmesh
      if ( iseqkm==0 ) call sepcrmeshinfo ( kmesh )
      call seprelkmesh1

!     --- Open files meshoutput, sepcomp.inf and sepcomp.out and read mesh

      if ( manag(8)<=0 ) then

!     --- ndim = 0, the mesh must be read

         call inifil ( -10 )
         call msh071 ( manag(5), iref10, ihelp )
         iref = abs(iref10)
         close ( unit = iref )

      else

!     --- ndim>0, the mesh is generated

         call sepmeshsub ( 50+manag(8), iinput, rinput )

      end if
      if ( ierror/=0 ) go to 1000
      if ( meshw==1 ) then

!     --- Write mesh to file meshoutput

         call inifil ( 10 )
         call msh068 ( iref10 )
         iref = abs(iref10)
         close ( unit = iref )

      else if ( meshw==2 ) then

!     --- Open file meshoutput

         call inifil ( 10 )

      end if

!     --- Check if spectral mesh has been found
!         If so call femesh

      kmesh => kmesh2d(1)
      if ( km(iseqkm)%kmesh(38)/=0 ) then

!     --- Spectral mesh: fill fem mesh in kmesh2d(.,2)

         iref = 22
         call sepfemesh ( iref, 1 )
         close ( unit = iref )

      end if
      if ( ierror/=0 ) go to 1000

      kprob => kprob2d(1)

      if ( manag(7)==1 ) then

!     --- manag(7) = 1, fill file and 74 (sepcomp.out)

         call inirmf ( 73 )
         call inirmf ( 74 )
         call inifil ( 74 )

      end if

      if ( nextrecord==5 ) then

!     --- Read extra mesh input and perform corresponding actions

         call mshextra

      end if  ! ( nextrecord==5 )
      if ( ierror/=0 ) go to 1000

!     --- Read problem definition

      call probdfext ( manag(6), iseqprob )
      if ( ierror/=0 ) go to 1000

      if ( manag(7)==1 ) then

!     --- Write array iinput to file sepcomp.out

         call writprobiinp

      end if

      iseqkp = kprob
      nprob = kp(iseqkp)%nprob
      if ( nprob>10 ) then

!     --- nprob>10 not yet implemented

         call errint ( nprob, 1 )
         call errint ( 10, 2 )
         call errsub ( 719, 2, 0, 0 )
         go to 1000

      end if
      mainsubr = isubr

      if ( iread==1 ) then

!     --- isubr = 1, Read rest of the SEPRAN input

         call readal ( iseqprob, manag )
         if ( ierror/=0 ) go to 1000

!        --- Find first input of sepcommat and store in iincommt

         call sepgetinpfinf ( 3, 1, ipos, maxcall )
         if ( in(ipos)%used ) then

!        --- Standard case, input has been read

            call incopy ( in(ipos)%iinput, iincommt, nprob*lniincommat )

         else

!        --- No input found, store defaults

            iincommt = 0
            do i = 1, nprob
               iincommt(1,i) = lniincommat
               iincommt(2,i) = 2
               iincommt(5,i) = i
            end do  ! i = 1, nprob

         end if  ! ( iinput(1)>0 )

      else

!     --- Read iincommt and call commat

         iseqnr = 1
         call read00 ( iincommt, nprob, iseqnr, 0, lniincommat )

      end if
      if ( ierror/=0 ) go to 1000

      do iprob = 1, nprob
         ishift = iincommt(9,iprob)
         intmat => intmat1d(iprob)
         kmesh => kmesh2d(1+ishift)
         kprob => kprob2d(1)
         call sepcommat ( iincommt(1,iprob) )
         intmt(intmat)%spectral = ishift==1
         if ( ierror/=0 ) go to 1000
      end do

1000  call erclos ( 'sepstmnew' )
      call erclos ( 'sepstm' )
      end subroutine sepstmnew
