      subroutine sepstorecurvs ( ichois, nsubcv, istore, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array istore with (sub) curve numbers of the curves stored in
!     array icurvs using the parameter ichois
!     Old subroutine erstorecurvs
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     curve
!     compound_curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, nsubcv, istore(*), icurvs(*)

!     ichois         i    choice parameter.  Possibilities:
!                          0   only one curve is considered
!                         -1   curves icrv1 to icrv2 are considered
!                         -2   curves icrv1 to icrv2 are considered in
!                              reversed order
!                         n>0  n curves icrv1, icrv2, ... , icrvn are considered
!     icurvs         i    array containing the curve numbers (see ichois)
!     istore         o    Contains the set of single curves
!     nsubcv         o    Number of curves stored in istore
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, n, icrv1, icrv2, jcrv1, k, kdirec

!     i              Counting variable
!     icrv1          First curve number
!     icrv2          Last curve number
!     jcrv1          abs(icrv1)
!     k              Counting variable
!     kdirec         Defines direction of curve
!     n              number of nodes found on the curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number does not exist
!     185   first curve number (icrv1) is out of range
!     186   last curve number (icrv2) is out of range
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstorecurvs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstorecurvs'
         call ertrace
         write(irefwr,1) 'ichois', ichois
         call prinin ( kmeshq, size(kmeshq), 'kmeshq' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nsubcv = 0
      if ( ichois==0 ) then

!     --- ichois = 0, only one curve

         icrv1 = icurvs(1)
         jcrv1 = abs(icrv1)
         if ( jcrv1==0 .or. jcrv1>ncurvs ) then
            call errint ( icrv1, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 185, 2, 0, 0 )
            jcrv1 = 1
         end if
         if ( kelmq/=0 ) nsubcv = kmeshq(jcrv1+1)-kmeshq(jcrv1)
         if ( nsubcv<=1 ) then

!        --- Only one curve

            nsubcv = 1
            istore(1) = icrv1

         else

!        --- More subcurves

            do i = 1, nsubcv
               istore(i) = kmeshq(kmeshq(jcrv1)+i)
            end do

         end if

      else if ( ichois==-1 .or. ichois==-2) then

!     --- ichois = -1   curves icrv1 to icrv2 in that order
!         ichois = -2   curves icrv1 to icrv2 in reversed order

         icrv1 = icurvs(1)
         icrv2 = icurvs(2)
         if ( icrv1<=0 .or. icrv1>ncurvs ) then
            call errint ( icrv1, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 185, 2, 0, 0 )
            icrv1 = 1
         end if
         if ( icrv2<=0 .or. icrv2>ncurvs ) then
            call errint ( icrv2, 1 )
            call errint ( 1, 2 )
            call errint ( ncurvs, 3 )
            call errsub ( 186, 3, 0, 0 )
            icrv2 = icrv1
         end if
         if ( icrv1<=icrv2 ) then
            kdirec = 1
         else
            kdirec = -1
         end if

         n = 0
         nsubcv = 0
         do k = icrv1, icrv2, kdirec
            if ( kelmq/=0 ) nsubcv = kmeshq(k+1)-kmeshq(k)
            if ( nsubcv<=1 ) then

!           --- Only one curve

               n = n+1
               istore(n) = k

            else

!           --- More subcurves

               do i = 1, nsubcv
                  istore(n+i) = kmeshq(kmeshq(k)+i)
               end do
               n = n+nsubcv

            end if

         end do
         nsubcv = n

      else

!     --- ichois > 0;  ichois curves

         if ( debug ) call prinin ( icurvs, ichois, 'icurvs' )
         n = 0
         nsubcv = 0
         do k = 1, ichois
            icrv1 = icurvs(k)
            jcrv1 = abs(icrv1)
            if ( jcrv1==0 .or. jcrv1>ncurvs ) then
               call errint ( jcrv1, 1 )
               call errint ( ncurvs, 2 )
               call errsub ( 139, 2, 0, 0 )
               jcrv1 = 1
            end if
            if ( kelmq/=0 ) &
               nsubcv = kmeshq(jcrv1+1)-kmeshq(jcrv1)

            if ( nsubcv<=1 ) then

!           --- Only one curve

               n = n+1
               istore(n) = icrv1

            else if ( icrv1>0 ) then

!           --- More subcurves (positive direction)

               do i = 1, nsubcv
                  istore(n+i) = kmeshq(kmeshq(jcrv1)+i)
               end do
               n = n+nsubcv

            else

!           --- More subcurves (negative direction)

               do i = 1, nsubcv
                  istore(n+1+nsubcv-i) = -kmeshq(kmeshq(jcrv1)+i)
               end do
               n = n+nsubcv

            end if

         end do
         nsubcv = n

      end if

      if ( nsubcv>ncurvs ) then

!     --- Error too many subcurves

         call errint ( nsubcv, 1 )
         call errint ( ncurvs, 2 )
         call errchr ( 'istore', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstorecurvs'

      end if  ! ( debug )
      call erclos ( 'sepstorecurvs' )

      end subroutine sepstorecurvs
