      subroutine to0031 ( ichois, vec1, vec2, vec3, a, b, ipoint, &
                          kdegfd, nnodes )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version  8.3  date 29-04-2014 Extra debug
!        version  8.2  date 03-12-2013 Correction error message
!        version  8.1  date 19-10-2011 Debug
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     algebraic manipulations with real vectors
!     Only the possibilities 4, 5, 11, 12, 15 and 16 are treated
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vec1(*), vec2(*), vec3(*), a, b
      integer ichois, ipoint, kdegfd, nnodes

!   a       i/o input / output constant (see ichois)
!   b       i/o input / output constant (see ichois)
!   ichois   i  Choice parameter. Possible values
!               4  VEC1(IDEGFD) in IPOINT is stored in a
!               5  VEC1(IDEGFD) in IPOINT = a
!              11  Compute the minimum and maximum of vec1 and the
!                  corresponding co-ordinates.
!                  In this case only one degree of freedom per point is
!                  permitted. Furthermore some parameters have the following
!                  special meaning:
!                  vec1:    i  vector from which min and max must be computed
!                  vec2:    i  array containing the co-ordinates
!                  vec3:    o  Output array of length 6, containing the
!                              co-ordinates xmin, ymin, zmin, xmax, ymax, zmax
!                  ipoint:  i  Dimension of the space ( 1, 2 or 3)
!              12  See 11, however the absolute value of vec1 is used
!              15  a = || VEC1 - VEC 2 ||  (L1 norm)
!                  ipoint is point where maximum is reached
!              16  a= || VEC1 ||
!                  ipoint is point where maximum is reached
!   ipoint  i/o nusol for ichois = 2, 7, 10
!   kdegfd   i  Specific degree of freedom to be applied
!   nnodes   i  Number of nodes
!   vec1    i/o vector to be operated on (See ichois)
!   vec2    i/o vector to be operated on (See ichois)
!   vec3     o  output vector with result (See ichois)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, imin, imax, idegfd, inunkp, ldegfd, lunder, lupper
      double precision vv

!     i              Loop varibale
!     idegfd         Position of degree of freedom in vector
!     imax           Nodal point number for which the maximum value is reached
!     imin           Nodal point number for which the minimum value is reached
!     inunkp         Number of degrees of freedom in a point
!     ldegfd         Value of degree of freedom in point, (ldegfd=kdegfd, except
!                    in the case kdegfd=0, in which case ldegfd takes all values)
!     lunder         Lower value of ldegfd
!     lupper         Upper value of ldegfd
!     vv             Value of vector in a node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
!     219   Degree of freedom is not available in point
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0031' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0031'
         write(irefwr,1) 'ichois, ipoint, indprfi, kdegfd, indprpi', &
                          ichois, ipoint, indprfi, kdegfd, indprpi
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ichois==4 .or. ichois==5 ) then

!     --- ichois = 4 or 5, degree of freedom in vec1

         if ( indprfi==0 ) then

            call errint ( kdegfd, 1 )
            call errint ( ipoint, 2 )
            call errint ( nphysi, 3 )
            if ( kdegfd>nphysi ) call errsub ( 219, 3, 0, 0 )
            idegfd = (ipoint-1)*nphysi+kdegfd

         else

            if ( kdegfd>kprobfi(ipoint+1)-kprobfi(ipoint)) then
               call errint ( kdegfd, 1 )
               call errint ( ipoint, 2 )
               call errint ( kprobfi(ipoint+1)-kprobfi(ipoint), 3 )
               call errsub ( 219, 3, 0, 0 )
            end if
            idegfd = kprobfi(ipoint)+kdegfd

         end if
         if ( indprpi/=0 ) then
            idegfd = kprobpi(ipoint,kdegfd)
         end if
         if ( ichois==4 ) then
            a = vec1(idegfd)
         else
            vec1(idegfd) = a
         end if
      else

!     --- Ichois = 11, 12, compute minimum and maximum, plus corresponding
!                          co-ordinates
!         or ICHOIS = 15, 16  Compute maximum norm and corresponding
!                             nodal point for difference of IVEC1 and IVEC2 (15)
!                             or for IVEC1 alone (16)
!         First compute minimum, maximum and corresponding nodal points

         a =  rinfin
         b = -rinfin
         imin = 0
         imax = 0

         if (indprpi>0) then
            if (kdegfd>nphysi) then
               call errint ( kdegfd, 1 )
               call errint ( nphysi, 2 )
               call errsub ( 218, 2, 0, 0)
               go to 1000
            end if
            if ( kdegfd==0 ) then
               lunder = 1
               lupper = nphysi
            else
               lunder = kdegfd
               lupper = kdegfd
            end if
            do i = 1, nnodes
               do ldegfd = lunder, lupper
                  idegfd = kprobpi(i,ldegfd)
                  if ( idegfd>0 ) then
                     if ( ichois==11 ) then
                        vv = vec1(idegfd)
                     else if ( ichois==15 ) then
                        vv = abs( vec1(idegfd)-vec2(idegfd) )
                     else
                        vv = abs( vec1(idegfd) )
                     end if
                     if ( ichois<15 .and. vv<a ) then

!                    --- vv<a, new minimum found

                        a    = vv
                        imin = i
                     end if

                     if ( vv>b ) then

!                    --- vv>b, new maximum found

                        b    = vv
                        imax = i
                     end if
                  end if
               end do
            end do

         else if (indprfi==0) then
            if (kdegfd>nphysi) then
               call errint ( kdegfd, 1 )
               call errint ( nphysi, 2 )
               call errsub ( 218, 2, 0, 0)
               go to 1000
            end if
            if ( kdegfd==0 ) then
               lunder = 1
               lupper = nphysi
            else
               lunder = kdegfd
               lupper = kdegfd
            end if
            do i = 1, nnodes
               do ldegfd = lunder, lupper
                  idegfd = ldegfd + (i-1)*nphysi
                  if ( ichois==11 ) then
                     vv = vec1(idegfd)
                  else if ( ichois==15 ) then
                     vv = abs( vec1(idegfd)-vec2(idegfd) )
                  else
                     vv = abs( vec1(idegfd) )
                  end if
                  if ( ichois<15 .and. vv<a ) then

!                 --- vv<a, new minimum found

                     a    = vv
                     imin = i
                  end if

                  if ( vv>b ) then

!                 --- vv>b, new maximum found

                     b    = vv
                     imax = i
                  end if
            end do
         end do
         else
            do i = 1, nnodes
               inunkp = kprobfi(i+1) - kprobfi(i)
               if ( kdegfd==0 ) then
                  lunder = 1
                  lupper = inunkp
               else
                  lunder = kdegfd
                  lupper = kdegfd
               end if
               do ldegfd = lunder, lupper
                  if (ldegfd<=inunkp) then
                     idegfd = kprobfi(i)+ldegfd
                     if ( ichois==11 ) then
                        vv = vec1(idegfd)
                     else if ( ichois==15 ) then
                        vv = abs( vec1(idegfd)-vec2(idegfd) )
                     else
                        vv = abs( vec1(idegfd) )
                     end if
                     if ( ichois<15 .and. vv<a ) then

!                    --- vv<a, new minimum found

                        a    = vv
                        imin = i
                     end if

                     if ( vv>b ) then

!                    --- vv>b, new maximum found

                        b    = vv
                        imax = i
                     end if
                  end if
               end do
            end do
            if (imax==0 ) then
               call errint ( kdegfd, 1 )
               call errint ( nphysi, 2 )
               call errsub ( 218, 2, 0, 0)
            end if
         end if

!        --- Next store the corresponding co-ordinates

         if ( ichois<15 ) then
            do i = 1, ipoint
               vec3(i)   = vec2 ( (imin-1) * ipoint + i )
               vec3(i+3) = vec2 ( (imax-1) * ipoint + i )
            end do
         else
            ipoint = imax
            a = b
         end if

      end if
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0031'

      end if  ! ( debug )
      call erclos ( 'to0031' )

      end
