      subroutine askvec ( number, inpsep, rinsep, &
                          istrin, istart, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.16 date 28-03-2015 change in comment
!        version  3.15 date 12-02-2015 New call to askvec02
!        version  3.14 date 13-01-2015 Extension with compute_pressure
!
!   copyright (c) 1993-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword compute_vector i and store in
!     inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), number, istrin, istart, nprob
      double precision rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 20+nelgrp
!                         Array INPSEP is filled as follows:
!                          0  Type of function to be called
!                             6:  manvec
!                             8:  derivatives
!                          1  ISEQNR       Sequence number corresponding to
!                                          input.  If iseqnr = -1,
!                                          the next one will be used
!                          2  IPROB        Problem sequence number
!                                          If iprob = -1, the next one will be
!                                          used
!                          3  IVECTOR      Vector sequence number
!                                          (output vector)
!                                          If ivector = -1, the same value as
!                                          iprob will be used
!                          4: ICHOIS (See prmanv)
!                          5: IDEGFD (See manvec)
!                          6: IPOINT (See manvec)
!                          7: ISPEC  (See manvec)
!                          8: ICHNRM (See manvec)
!                          9: Vector sequence number first input vector
!                         10: Vector sequence number second input vector
!                         11: Scalar sequence number first scalar
!                         12: Scalar sequence number second scalar
!                         13: Sequence number first position in RINSEP
!                         14: iskip  defines if element groups must be skipped
!                             (>0) or not (0)
!                             If >0 position iskip defines how many element
!                             groups must be used and the following positions
!                             the corresponding element groups
!                         15: If >0, it defines that a selection of
!                             curves must be used
!                             inpsep(15) gives the starting address in
!                             inpsep with information
!                             In the first address the number of curves is
!                             given, followed by the curve numbers
!                         16: Defines what selection must be used
!                             iplace as defined in prfillnodes
!                         If this array is adapted, also adapt prvecfillnods
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of next positions in rinsep
!     nprob          i    Number of various problems
!     number         i    Vector sequence number
!     rinsep        i/o   Contains real input at output. The storage depends on
!                         the storage of inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=50, nparm=20, nextra=11
      integer ncode2(nncode2), ncodmx2(nncode2), &
              iscal1, nmcode2(nextra), isignvec(2), minussign
      character (len=22) code2(nncode2)
      character (len=26) codemn2(nextra)
      logical found, finished

!     code2          Table with keywords corresponding to options of
!                    COMPUTE_VECTOR
!     codemn2        Table with main keywords
!     finished       Indicates if the reading has finished (true) or not
!                    (false)
!     found          Indicates if one of the possibilities modulus, length, &
!                    phase or phase or subtract has been found (true) or not
!                    (false)
!     iscal1         First scalar
!     isignvec       Contains the signs of the two vectors in case of
!                    linear combination
!     minussign      Sign to be used for first scalar
!     ncode2         Table containing the number of significant characters in
!                    the keyword table code2
!     ncodmx2        Table containing the number of maximum characters in
!                    the keyword table code2
!     nextra         Number of extra keywords in codenm
!     nmcode2        Table containing the number of significant characters
!                    in the main keyword table codemn2, including those of
!                    the extra texts
!     nncode2        Number of keywords in code2
!     nparm          Number of parameters in inpsep before information
!                    concerning the element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKVEC01       Subroutine corresponding to askvec dealing with iretur<-10, &
!                    nseqkeyw==1 and iseqkeyw(1)<0
!     ASKVEC02       Read expression following vector = &
!     ASKVEC03       Store degree of freedom, curves and skip element groups
!                    in inpsep
!     ASKVEC24       Interpret part following mesh_velocity in subroutine askvec
!     ASKVECDERIV    Interpret input for derivatives following vi = derivatives
!     ASKVECGRAD     Interpret input following vi = grad
!     ASKVECLINCOM   Store coefficients for subroutine askvec
!     ASKVECNOTFOUND Interpret input of askvec if none of the standard options
!                    is found
!     ASKVECRHS      Interpret input following vi = right_hand_side
!     ASKVECSIGN     Interpret the options of the form v = sign(V1) * V2
!     ASKVECSTR      Interpret input following vi = stream_function
!     ASKVECSTSCAL   Store scalar numbers for subroutine askvec
!     ASKVECSTVEC    Store vector numbers for subroutine askvec
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!         askvec requires input from the standard input file
!         The following input is recognised:
!         STRUCTURE    (has been read already be readal)
!         start of input
!         the next records are interpreted in the sequence they are read
!         the program is carried out in the same sequence
!         [COMPUTE_]VECTOR i, options (all in one line)
!         where options may have the following contents
!         MODULUS VECTOR j             compute modulus of vector j
!         PHASE VECTOR j               compute phase of vector j
!         LENGTH VECTOR j              compute length of vector j
!         CONJUGATE VECTOR j           compute complex conjugate of
!                                      vector j
!         FUNC VECTOR j1 VECTOR j2     Vi = f ( Vj1, Vj2)
!         SIGN ( Vi ) or  SIGN ( Vi ) * Vj vector is replaced by sign
!                                      componentwise
!         Vi * Vj                      componentwise
!         MULTIPLY = m                 multiply result by m
!         SUBTRACT                     subtract constant from vector j
!         [LIN_COMBINATION] SCALAR1 VECTOR1 + SCALAR2 VECTOR2
!                                   Vi = S1 V1 + S2 V2
!           or:
!         [LIN_COMBINATION] COEF1 = i VECTOR1 COEF2 = j VECTOR2
!           or:
!         [LIN_COMBINATION] alpha VECTOR1 + beta VECTOR2
!         V1 + alpha
!         EXTRACT                      extract degree of freedom from vector
!         DEGFD j                      sequence number of degree of freedom
!         ELEMENT_GROUPS = (s1, s2, ...)           (Default all)
!         CURVES = (c1, c2, ...)                   (Default none)
!         SURFACES = (s1, s2, ...)                 (Default none)
!         SKIP_ELEMENT_GROUPS = (s1, s2, ... )     (Default none)
!         REAL VECTOR j                compute real part of vector j
!         IMAGINARY VECTOR j           compute imaginary part of vector j
!         CONSTANT = c                 this constant may be either a
!         SCALAR J                     constant or a scalar number
!         INNER_PRODUCT VECTOR j1 VECTOR j2, degfd = k Component wise inner
!                                      product for the first k components
!                                      Default: all components
!         MAP VECTOR i, PROBLEM = p, TYPE = t
!             t may be: 0, i.e. SOLUTION_VECTOR
!                       k, i.e. VECTOR_OF_SPECIAL_STRUCTURE k
!         MESH_VELOCITY ( VECTOR1, VECTOR2 )
!         INTEGRATED_TO_NODAL Vj [ZERO_LEVELSET i]
!         TRANSFORM_TO_NORMALDIR Vj
!         SMOOTH_END_POINTS
!         STRESS_VECTOR Vj
!         = ( V1, V2 )                Make vector from V1 and V2
!         DERIVATIVES ( V1, S1, icheld = i, problem =  p, seq_coef = s1, &
!                       seq_deriv  = s2, ix = j )
!         STREAM_FUNCTION ( V1, start_node = s, start_value = f )
!         POLAR (Vj)         transform to polar coordinates
!         GRADIENT (Vj)
!         FLUX (Vj)
!         COMPUTE_PRESSURE (Vj)
!         This program only interprets the part options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1903   From the possibilities: PHASE, MODULUS and LENGTH at least two are
!           found in the record
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'MODULUS', 'LENGTH', 'VECTOR', 'PHASE', 'MULTIPLY', &
                  'SUBTRACT', 'CONSTANT', 'SCALAR', 'LIN_COMBINATION', &
                  'EXTRACT', 'DEGFD', 'REAL', 'IMAGINARY', 'CONJUGATE', &
                  'FUNC', 'INNER_PRODUCT', 'MAP', 'PROBLEM', &
                  'TYPE', 'COEF', 'COEF', 'ELEMENT_GROUPS', &
                  'SKIP_ELEMENT_GROUPS', 'MESH_VELOCITY', &
                  'INTEGRATED_TO_NODAL', 'TRANSFORM_TO_NORMALDIR', &
                  'CURVES', 'SMOOTH_END_POINTS', 'STRESS_VECTOR', &
                  'ZERO_LEVEL_SET', 'ICHELD', 'SEQ_COEF', 'SEQ_DERIV', &
                  'DERIVATIVES', 'SIGN', 'IX', 'POLAR', 'TYPE_OUTPUT', &
                  'POINTS', 'SURFACES', 'GRADIENT', 'STREAM_FUNCTION', &
                  'START_NODE', 'START_VALUE', 'IVEC', 'STRESS_TENSOR', &
                  'RIGHT_HAND_SIDE', 'FLUX', 'COMPUTE_PRESSURE', 'CURL'/
      data ncode2 /  3, 3, 1, 3, 3,    3, 3, 3, 3, 3, &
                    3, 4, 4, 4, 3,    5, 3, 4, 3, 4, &
                    4, 4, 4, 8, 5,    5, 4, 5, 8, 8, &
                    4, 6, 7, 4, 4,    2, 5, 7, 4, 4, &
                    4, 6, 9, 9, 4,    8, 5, 4,12, 4 /
      data ncodmx2 / 7, 6, 6, 5, 8,    8, 8, 6,15, 7, &
                    5, 4, 9, 9, 4,   13, 3, 7, 4, 4, &
                    4,14,19,13,19,   22, 6,17,13,14, &
                    6, 8, 9,11, 4,    2, 5,11, 6, 8, &
                    8,15,10,12, 4,   13,15, 4,16, 4 /
      data codemn2 / 'C', 'P', 'S', 'SOLUTION', 'SPECIAL', 'ELEMENTWISE', &
                    'RIGHT_HAND_SIDE', 'INTERPOLATE', 'ELEMENT_NODES', &
                    'ELEMENT_INTEGRATION_POINTS', 'REACTION_FORCE' /
      data nmcode2 / 1, 1, 1, 3, 4,  4, 4, 5, 11, 11, &
                    5 /

!     code2   Table with keywords corresponding to options of COMPUTE_VECTOR
!             Values:
!              1:  MODULUS
!              2:  LENGTH
!              3:  VECTOR
!              4:  PHASE
!              5:  MULTIPLY
!              6:  SUBTRACT
!              7:  CONSTANT
!              8:  SCALARPOINTS
!              9:  LIN_COMBINATION
!             10:  EXTRACT
!             11:  DEGFD
!             12:  REAL
!             13:  IMAGINARY
!             14:  CONJUGATE
!             15:  FUNC
!             16:  INNER_PRODUCT
!             17:  MAP
!             18:  PROBLEM
!             19:  TYPE
!             20:  COEF1
!             21:  COEF2
!             22:  ELEMENT_GROUPS
!             23:  SKIP_ELEMENT_GROUPS
!             24:  MESH_VELOCITY
!             25:  INTEGRATED_TO_NODAL
!             26:  TRANSFORM_TO_NORMALDIR
!             27:  CURVES
!             28:  SMOOTH_END_POINTS
!             29:  STRESS_VECTOR
!             30:  ZERO_LEVEL_SET
!             31:  ICHELD
!             32:  SEQ_COEF
!             33:  SEQ_DERIV
!             34:  DERIVATIVES
!             35:  SIGN
!             36:  IX
!             37:  POLAR
!             38:  TYPE_OUTPUT
!             39:  POINTS
!             40:  SURFACES
!             41:  GRADIENT
!             42:  STREAM_FUNCTION
!             43:  START_NODE
!             44:  START_VALUE
!             45:  IVEC
!             46:  STRESS_TENSOR
!             47:  RIGHT_HAND_SIDE
!             48:  FLUX
!             49:  COMPUTE_PRESSURE
!             50:  CURL
!     codemn2  Table with sub keywords corresponding to subkeywords
!             Values:
!             1: C
!             2: P
!             3: S
!             4: SOLUTION
!             5: SPECIAL
!             6: ELEMENTWISE
!             7: RIGHT_HAND_SIDE
!             8: INTERPOLATE
!             9: ELEMENT_NODES
!            10: ELEMENT_INTEGRATION_POINTS
!            11: REACTION_FORCE
! ======================================================================
!
      call eropen ( 'askvec' )

      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from askvec'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'istart', istart
         write(irefwr,1) 'ityprd, lastitem, istrin, number', &
                          ityprd, lastitem, istrin, number
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopycodesub ( codemn2, nextra, nmcode2 )

!     --- Set parameters

      length = nparm

      inpsep(0) = 6          ! Use manvec
      inpsep(4) = 0          ! Use manvec

!     --- Initialize arrays

      table = -1d0
      tabmin = 0d0
      tabmax = 0d0
      itype = 2

      inpsep(1:nparm) = 0

!     --- MODULUS and LENGTH

      tabmin(1:2) = 1d0
      tabmax(1:2) = 1d0

!     --- VECTOR

      tabmin(3) = 2d0
      tabmax(3) = dble(maxvc1+5000)
      itype(3) = 15

!     --- PHASE

      tabmin(4) = 1d0
      tabmax(4) = 1d0

!     --- MULTIPLY

      tabmin(5) = -1d20
      tabmax(5) = 1d20
      itype(5) = 1
      table(5) = 0d0

!     --- CONSTANT

      tabmin(7) = -1d20
      tabmax(7) = 1d20
      itype(7) = 1
      table(7) = -1d30

!     --- SCALAR

      tabmin(8) = 2d0
      tabmax(8) = dble(maxcls+5000)
      itype(8) = 15

!     --- DEGFD

      tabmin(11) = 1d0
      tabmax(11) = 1d2
      itype(11) = 0

!     --- MAP, PROBLEM and TYPE

      tabmin(18) = 1d0
      tabmin(19) = -1d2
      tabmax(18:19) = 1d3
      itype(18:19) = 0
      table(17:19) = 0d0

!     --- COEF1

      tabmin(20) = 1d0
      tabmax(20) = 2d0
      itype(20) = 34

!     --- COEF2

      tabmin(21) = -1d25
      tabmax(21) = 1d25
      itype(21) = 1

!     --- ELEMENT_GROUPS and SKIP_ELEMENT_GROUPS

      tabmin(22:23) = nelgrp
      tabmax(22:23) = nelgrp
      itype(22:23) = 11

!     --- curves

      itype(27) = 18
      table(27) = -1d0  ! minus position in codemn2
      tabmin(27) = ncurvs
      tabmax(27) = ncurvs

!     --- zero_level_set

      itype(30) = 0  ! integer
      tabmax(30) = 1d0

!     --- ICHELD

      itype(31) = 0  ! integer
      tabmin(31) = 1d0
      tabmax(31) = 1d3

!     --- SEQ_COEF

      itype(32) = 0  ! integer
      tabmin(32) = 1d0
      tabmax(32) = 25d0
      table(32) = 0d0

!     --- SEQ_DERIV

      itype(33) = 0  ! integer
      tabmin(33) = 1d0
      tabmax(33) = 100d0

!     --- IX

      itype(36) = 0  ! integer
      tabmin(36) = 1d0
      tabmax(36) = ndim
      table(36) = 1d0

!     --- TYPE_OUTPUT

      itype(38) = 5       ! keyword = subkeyword
      table(38) = 1d0     ! default
      tabmin(38) = 4d0    ! start of subkeyword in codemn2
      tabmax(38) = 11d0   ! end of subkeyword in codemn2

!     --- points, surfaces

      itype(39:40) = 18
      table(39) = -2d0  ! minus position in codemn2
      table(40) = -3d0  ! minus position in codemn2
      tabmin(39) = nuspnt
      tabmin(40) = nsurfs
      tabmax(39:40) = tabmin(39:40)

      itype(43) = 0  ! start_node
      itype(44) = 1  ! start_value
      table(44) = -1d30

      itype(45) = 0  ! ivec
      tabmin(45) = 0d0
      tabmax(45) = 1d2

      if ( typeitems(lastitem+1)==13 ) then

!     --- vector = found, check if it is followed by an expression

         if ( debug ) write(irefwr,*) 'before askvec02'
         call askvec02 (  number, inpsep(1), istrin, rinsep, finished )
         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            if ( ityprd==2 .or. ityprd==3 ) &
               write(irefwr,2) 'realrd', realrd
            write(irefwr,*) 'finished', finished
         end if  ! ( debug )

         if ( finished ) then

!        --- vector = ( vec_expression, vec_expression ) found

            istart = istart+21
            inpsep(3) = number
            go to 1000

         end if  ! ( finished ) then
         if ( ityprd==19 .or. inpsep(4)==116 ) go to 800

      end if  ! ( typeitems(lastitem+1)==3 )

      iretur = -10
      ispecl = 4
      minussign = 1
      nmain = 0
      call sepreadsetkeyw
      ispecl = 0
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )
         write(irefwr,1) 'iretur', iretur
         call prinrl ( table, nncode2, 'table' )
         call prinrl ( tabmax, nncode2, 'tabmax' )
         call prinrl ( tabout, 4, 'tabout' )

      end if

      if ( iretur<-10 .and. nseqkeyw==1 .and. iseqkeyw(1)<0 ) then

!     --- iretur<-10 and no keyword found, hence number found
!         This must be the situation vi = c

         call askvec01 ( iretur, inpsep(1), tabout, rinsep, istrin, &
                         number )
         go to 800

      end if  ! ( iretur<-10 )

!     --- Check array length

      call inichkarlen ( nseqkeyw, 10, 'iseqkeyw' )
      if ( ierror/=0 ) go to 1000

!     --- First check the operation

      found = .false.

      if ( table(1)>0d0 ) then

!     --- modulus found

         inpsep(4) = 29
         found = .true.

      end if

      if ( table(2)>0d0 ) then

!     --- length found

         inpsep(4) = 33
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(4)>0d0 ) then

!     --- phase found

         inpsep(4) = 30
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(6)>0d0 ) then

!     --- subtract found

         inpsep(4) = 26
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(9)>0d0 ) then

!     --- linear combination found

         inpsep(4) = 27
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(10)>0d0 ) then

!     --- extract found

         inpsep(4) = 36
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(12)>0d0 ) then

!     --- real found

         inpsep(4) = 37
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(13)>0d0 ) then

!     --- imaginary found

         inpsep(4) = 38
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(14)>0d0 ) then

!     --- conjugate found

         inpsep(4) = 34
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(15)>0d0 ) then

!     --- FUNC found

         inpsep(4) = 31
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(16)>0d0 ) then

!     --- INNER_PRODUCT found

         inpsep(4) = 41
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(17)>0d0 ) then

!     --- MAP found

         inpsep(4) = 51
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.
         inpsep(2) = nint(table(18)) ! problem number
         inpsep(7)  = nint(table(19)) ! type of vector

      end if

      if ( table(18)>0d0 ) inpsep(2) = nint(table(18)) ! problem number

      if ( table(24)>0d0 ) & ! MESH_VELOCITY found
         call askvec24 ( inpsep(1), found, tabmax, 2, 101, code2(24) )

      if ( table(25)>0d0 ) then

!     --- INTEGRATED_TO_NODAL found

         call askvec24 ( inpsep(1), found, tabmax, 1, 103, code2(25) )
         if ( table(30)>0 ) then

!        --- table(30)>0, hence zero_level_set i found

            inpsep(7) = 1   ! zero_level set
            inpsep(8) = nint(table(30))  ! sequence number

         else

!        --- table(30)=0, use all outer boundaries

            inpsep(7) = 0   ! all outer boundaries

         end if  ! ( table(30)>0 )

      end if  ! ( table(25)>0d0 )

      if ( table(26)>0d0 ) & ! TRANSFORM_TO_NORMALDIR found
         call askvec24 ( inpsep(1), found, tabmax, 1, 104, code2(26) )
      if ( table(29)>0d0 ) & ! STRESS_VECTOR found
         call askvec24 ( inpsep(1), found, tabmax, 1, 105, code2(29) )

      if ( table(34)>0d0 ) then

!     --- DERIVATIVES found, special case

         call askvecderiv ( inpsep, istart, &
                            nprob, number, code2, ncodmx2, nelgrp )
         go to 1000

      end if   ! ( table(34)>0d0 )

      if ( table(41)>0d0 .or. table(46)>0d0 .or. table(48)>0d0 .or. &
           table(49)>0d0 .or. table(50)>0d0 ) then

!     --- GRADIENT, FLUX, STRESS, curl or COMPUTE_PRESSURE found, special case

         call askvecgrad ( inpsep, istart, nprob, number, nelgrp )
         go to 1000

      end if   ! ( table(34)>0d0 )

      if ( table(42)>0d0 ) then

!     --- STREAM_FUNCTION found, special case

         call askvecstr ( inpsep, istart, number, istrin, rinsep )
         go to 1000

      end if   ! ( table(34)>0d0 )

      if ( table(47)>0d0 ) then

!     --- RIGHT_HAND_SIDE found, special case

         call askvecrhs ( inpsep, istart, table, number )
         go to 1000

      end if   ! ( table(34)>0d0 )

      if ( table(35)>0d0 ) then

!     --- SIGN found

         call askvecsign ( inpsep(1), found )

      end if

      if ( table(37)>0d0 ) then

!     --- polar found

         inpsep(4) = 114  ! transform to other coordinate system
         inpsep(7) = 1    ! polar coordinates
         found = .true.

      end if

!     --- Store vector numbers

      call askvecstvec ( inpsep(1), isignvec, number, numvec, rinsep )
      if ( debug ) write(irefwr,*) 'found', found

      if ( .not. found ) then

!     --- None of the standard keywords is found
!         It might be possible that VECTOR is found, so that this case is
!         interpreted as LIN_COMBINATION

         call askvecnotfound ( inpsep(1), tabmax, table, nseqkeyw, &
                               iseqkeyw, minussign, -iretur-10, istrin, &
                               rinsep, tabout )

      end if  ! ( .not. found )
      if ( ierror/=0 ) go to 1000

!     --- Store scalar numbers

      call askvecstscal ( inpsep(1), minussign, iscal1 )

      if ( table(28)>0 ) inpsep(7) = 1   ! set ispec in case of smoothing

!     --- Store coefficients (only for LIN_COMBINATION)

      if ( inpsep(4)==27 ) &
         call askveclincom ( code2, ncodmx2, table, iscal1, iretur, &
                             tabout, inpsep(1), rinsep, istrin, &
                             nseqkeyw, iseqkeyw, isignvec )

      if ( inpsep(4)==26 .and. table(7)>-1d25 ) then

         rinsep(istrin) = table(7)
         rinsep(istrin+1) = 0d0
         inpsep(13) = istrin
         istrin = istrin + 2

      end if

!     --- Store degree of freedom, curves and skip element groups

800   call askvec03 ( inpsep(1), table, tabout, &
                      tabmax, code2, ncodmx2 )

!     --- raise istart

      istart = istart+1+length

1000  if ( debug ) then

!     --- debug information

         write(irefwr,1) 'istart, length', istart, length
         call prinin ( inpsep(1), length, 'inpsep' )
         if ( istrin>1 ) call prinrl ( rinsep, istrin-1, 'rinsep' )
         write(irefwr,*) 'End askvec'

      end if
      call erclos ( 'askvec' )

      end
