      subroutine askvecnotfound ( inpsep, tabmx2, table2, nseqkeyw, &
                                  iseqkeyw, minussign, numconsts, &
                                  istrin, rinsep, tabout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 16-06-2012 extra parameters
!        version  1.2  date 18-01-2012 extra debug
!        version  1.1  date 02-03-2011 Make vector from vectors
!
!   copyright (c) 2010-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret input of askvec if none of the standard options is found
!     It might be possible that VECTOR is found, so that this case is
!     interpreted as LIN_COMBINATION or product of two vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nseqkeyw, iseqkeyw(*), minussign, numconsts, &
              istrin
      double precision tabmx2(*), table2(*), rinsep(*), tabout(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     iseqkeyw       i    Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                       iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                      -isekeyw(i) refers to position in tabout
!     istrin        i/o   Starting address of next positions in rinsep
!     minussign      o    Sign to be used for first scalar
!     nseqkeyw       i    Number of entries in iseqkeyw
!     numconsts      i    Number of constants found
!     rinsep        i/o   Contains real input at output. The storage depends on
!                         the storage of inpsep
!     table2         i    Contains values of items read corresponding to code2
!     tabmx2         i    Table containing the maximum values of the quantities
!                         given in code2
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numvecs, numscalslc
      logical :: vectorfirst, vector, multiply_found, plus_found, &
                 divide_found

!     divide_found   If true a divide sign has been found
!     i              Counting variable
!     multiply_found If true a multiply sign has been found
!     numscalslc     Number of scalars in record
!     numvecs        Number of vectors in record
!     plus_found     If true a plus sign has been found
!     vector         If true a new vector is made from two others
!     vectorfirst    if true the combination contains firt a vector and then
!                    a scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1904   None of the possibilities: PHASE, MODULUS and LENGTH has been found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecnotfound' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecnotfound'
         write(irefwr,1) 'table2(3)', nint(table2(3))
         write(irefwr,1) 'nseqkeyw', nseqkeyw
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numvecs = nint(tabmx2(3))    ! number of vectors found
      numscalslc = nint(tabmx2(8))   ! number of scalars found

      if ( debug ) &
         write(irefwr,1) 'numvecs, numscalslc, numconsts', &
                          numvecs, numscalslc, numconsts

      multiply_found = .false.
      do i = 3, nitems
         if ( typeitems(i)==7 ) multiply_found = .true.
      end do  ! i = 3, nitems

      divide_found = .false.
      do i = 3, nitems
         if ( typeitems(i)==12 ) divide_found = .true.
      end do  ! i = 3, nitems

      plus_found = .false.
      do i = 3, nitems
         if ( typeitems(i)==8 .or. typeitems(i)==11 ) &
            plus_found = .true.
      end do  ! i = 3, nitems

      if ( debug ) then
         write(irefwr,*) 'multiply_found, plus_found ', &
                          multiply_found, plus_found
      end if  ! ( debug )

      vector = numvecs==2 .and. nitems>=6
      if ( vector ) &
         vector = typeitems(nitems)==6 .and.  &     ! last item = )
                  typeitems(nitems-2)==26 .and.  &  ! ,
                  typeitems(nitems-4)==5            !  (
      if ( debug ) write(irefwr,*) 'vector ', vector

      if ( vector ) then

!     --- Make a vector from two previous ones, by copying
!         The components of the first vector are the first ones, &
!         the components of the second vector are the last

         inpsep(4) = 106

      else if ( numvecs==2 .and. numscalslc==0 .and. multiply_found .and. &
                .not. plus_found ) then

!     --- two vectors, no scalars and multiply sign found
!         should be vector * vector

         inpsep(4) = 107
         inpsep(7) = 0

      else if ( numvecs==2 .and. numscalslc==0 .and. divide_found .and. &
                .not. plus_found ) then

!     --- two vectors, no scalars and divide sign found
!         should be vector / vector

         inpsep(4) = 108
         inpsep(7) = 0

      else if ( numvecs==1 .and. (numscalslc==1 .or. numconsts==1) ) then

!     --- one scalar/constant and one vector found
!         Check which one is first

         vectorfirst = .false.
         do i = 1, nseqkeyw

            if ( iseqkeyw(i)==3 ) then

!           --- First one is vector

               vectorfirst = .true.
               go to 100

            else if ( iseqkeyw(i)==8 .or. iseqkeyw(i)<0 ) then

!           --- First one is scalar or constant

               go to 100

            end if  ! ( iseqkeyw(i)==3 )

         end do  ! i = 1, nseqkeyw

100      if ( vectorfirst ) then

!        --- vector followed by scalar, i.e. subtract scalar from vector

            inpsep(4) = 26
            minussign = -1
            if ( numconsts==1 ) then

!           --- numconsts>1, store constant

               inpsep(13) = istrin
               do i = 1, nseqkeyw

                  if ( iseqkeyw(i)<0 ) then

!                 --- constant found

                     rinsep(istrin) = minussign*tabout(-iseqkeyw(i))
                     istrin = istrin+1
                     go to 1000

                  end if  ! ( iseqkeyw(i)<0

               end do  ! i = 1, nseqkeyw

            end if  ! ( numconsts==1 )

         else

!        --- two vectors, linear combination

            inpsep(4) = 27

         end if  ! ( iseqkeyw(1)==3 .and. iseqkeyw(2)==8 )

      else if ( table2(3)>0d0 ) then

!     --- Vector found, treat as LIN_COMBINATION

          inpsep(4) = 27

      else

!     --- Error, no known combination found

         call errsub ( 1904, 0, 0, 0 )

      end if  ! ( nseqkeyw==2 )

1000  call erclos ( 'askvecnotfound' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inpsep(4), minussign', inpsep(4), minussign
         write(irefwr,*) 'End askvecnotfound'

      end if  ! ( debug )

      end
