      subroutine erexpc ( iwork, work, nwork, nlparm, nelgrp, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 12-11-2014 Work arrays local
!        version  2.3  date 06-02-2012 Extension with indexed constants
!        version  2.2  date 15-08-2007 Allow coef %scalar
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Expand input for coefficients read in arrays IINCOF and RINCOF into
!     arrays IWORK, WORK and NPARM
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulenparm
      use sepmodulecomio
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nwork, iwork(4,nwork), nelgrp, nlparm(nelgrp)
      double precision work(2*nwork)
      logical compl

!     compl   i    indication whether the coefficients are real (false)
!                  or complex (true)
!     iwork   o    User array of length 4 x NWORK to store information of the
!                  coefficients, together with array work
!                  iwork(1,i) contains the parameter ifunc
!                  iwork(2,i) the parameter iref or the sequence number
!                             of the old solution vector
!                  iwork(3,i) the degree of freedom in the old sol. vector
!                  iwork(4,i) information of the function by which the old
!                             solution vector must be multiplied
!                  with
!                  ifunc        Indication of the type of coefficients.
!                               Possible values:
!                  0:           coef i is a constant, stored in work(i)
!                  >0<1000:     coef i is given by the subroutine FUNCCF
!                  >1000<2000:  coef i is given by the subroutine FUNCC1
!                  2001:        coef i is given by the user in each point
!                               The values are read from file iref
!                  2002:        coef i is given by the user in each element
!                               The values are read from file iref
!                  2003:        coef i is given by the k th degree of freedom
!                               in the j th solution vector
!                               The number j is stored in iwork(2,i)
!                               The number k is stored in iwork(3,i)
!                  2004:        coef i is given by the k th degree of freedom
!                               in the j th solution vector multiplied by a
!                               function or constant
!                               The number j is stored in iwork(2,i)
!                               The number k is stored in iwork(3,i)
!                               If muliplied by a constant then iwork(4,i) = 0
!                               and work(i) is the constant otherwise iwork(4,i)
!                               refers to the function subroutine FUNCCF
!                  >10000:      coef i is given by the subroutine FUNCC3
!                  <0:          Coefficient i is equal to coefficient j, &
!                               where j = -iwork(1,i) and j<i
!                  iref         When>0 it indicates the input reference number
!                               of the file
!                               when<0,  it indicates that the coefficients to
!                               be stored in array user are created by the user
!                               subroutine funcfl
!                               iref<0 should only be used in combination with
!                               iwork(1,k)  =  2001
!     nelgrp  i    Number of element groups + booundary groups + local groups
!     nlparm  i    Number of parameters for a specific element group
!     nwork   i    Total number of coefficients
!     work    o    Real work array of length NWORK*2 in which values of
!                  coefficients are stored. See iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, istart, itype, ipos, &
              jparm, ivar, iseq, ityp, scal, ihelp(nelgrp), &
              itest1(nwork), itest2(nelgrp)
      double precision sig
      logical :: finished, endelg

!                    they are not
!     endelg         Indicates if the end of the element group loop has been
!                    reached (true) or not (false)
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     ielgrp         element group number
!     ihelp          Work array containing the starting addresses (-1) of the
!                    element groups in array IWORK and WORK
!     ipos           Coefficient sequence number with respect to WORK and IWORK
!     iseq           Sequence number in reals or integers
!     istart         Starting address of coefficient in array IWORK
!     itest1         Work array in which for each coefficient it has been
!                    indicated if it has been filled (1) or not (0)
!     itest2         Work array in which for each element group it has been
!                    indicated if it has been filled (1) or not (0)
!     ityp           If 1: integer, if 2 real
!     itype          Type number (see IINCOF)
!     ivar           Sequence number of variable
!     jparm          Actual number of paramaters for element group
!     scal           value of scalar
!     sig            Sign of coefficient
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRININ1LARGE   print 2d long integer array
!     PRININLARGE    print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     646   Coeffient number read out of range
!    1864   Coefficients in element group have been filled more than once
!    1865   Coefficient has been filled more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Fill ihelp with starting addresses (-1) for each coefficient
!   Clear arrays ITEST1 and ITEST2
!   finished := false
!   iposin := 3+nelgrp+nbngrp
!   while ( not finished ) do
!      ielgrp := iincof(iposin+1)
!      if ( ielgrp<0 ) then
!         ibngrp := -ielgrp+nelgrp
!      if ( itest2(ielgrp)=0 ) then
!         itest2(ielgrp) = 1
!      else
!         warning element group filled more than once
!      istart := ihelp(ielgrp)
!      endelg := false
!      iposin := iposin + 2
!      while ( not endelg ) do
!         itype := iincof(iposin+1)
!         if ( itype=-2 ) then
!            finished := true
!            endelg := true
!         else if ( itype=-1 ) then
!            endelg := true
!         else ( itype>0 ) then
!            icoef := iincof(iposin+2)
!            ipos := icoef+istart
!            if ( itest1(ipos)=0 ) then
!               itest1(ipos) = 1
!            else
!               warning coefficient filled more than once
!            if ( itype=1 ) then
!               copy value from iincof into iwork
!            else if ( itype=2 ) then
!               copy value from rincof into work
!            else if ( itype=3 ) then
!               copy ifunc from iincof into iwork
!            else if ( itype = 4 or 5 ) then
!               copy iref from iincof into iwork
!            else if ( itype = 6 ) then
!               copy j and d from iincof into iwork
!            else if ( itype = 7 ) then
!               copy j and d from iincof into iwork
!               copy value from rincof into work
!            else if ( itype = 8 ) then
!               copy j, d and ifunc from iincof into iwork
!            else if ( itype = 9 ) then
!               copy j from iincof into iwork
!            else if ( itype = 10) then
!               copy j from iincof into iwork
!            Raise iposin
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erexpc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erexpc'
         write(irefwr,1) 'nelgrp', nelgrp
         call prininlarge ( iincof, 10, 'iincof' )
         call prinrl ( rincof, 2, 'rincof' )
         call prinin ( nlparm, nelgrp, 'nlparm' )
  1      format ( a, 1x, (10i9) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill ihelp with starting addresses (-1) for each coefficient

      ihelp(1) = 0
      do i = 2, nelgrp
         ihelp(i) = ihelp(i-1) + nlparm(i-1)
      end do

!     --- Clear arrays ITEST1 and ITEST2

      itest1 = 0
      itest2 = 0

      finished = .false.
      iposin = 3+nelgrp

!     --- while loop finished

200   if ( .not. finished ) then

         ielgrp = iincof(iposin+1)
         if ( debug ) write(irefwr,1) 'ielgrp, iposin', ielgrp, iposin

         if ( ielgrp>0 ) then

!        --- ielgrp>0, so input found

            iposin = iposin + 1

            jparm = nlparm(ielgrp)
            if ( itest2(ielgrp)==0 ) then

!           --- First time element group has been read

               itest2(ielgrp) = 1

            else

!           --- element group filled more than once

               call errint ( ielgrp, 1 )
               call errwar ( 1864, 1, 0, 0 )

            end if

            istart = ihelp(ielgrp)

         end if  ! ( ielgrp>0 )

         endelg = .false.
         if ( debug ) &
            write(irefwr,1) 'ielgrp, iposin, jparm, istart', &
                             ielgrp, iposin, jparm, istart

!        --- while loop endelg

250      if ( .not. endelg ) then

            itype = iincof(iposin+1)
            if ( debug ) write(irefwr,1) 'itype', itype
            if ( itype==-2 ) then

!           --- itype = -2, end of input

               finished = .true.
               endelg = .true.

            else if ( itype==-1 ) then

!           --- itype = -1, end of element group

               endelg = .true.
               iposin = iposin + 1

            else

               icoef = iincof(iposin+2)
               if ( icoef>jparm ) then
                  call errint ( icoef, 1 )
                  call errint ( jparm, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 646, 3, 0, 0 )
               end if
               ipos = icoef+istart
               if ( debug ) &
                  write(irefwr,1) 'icoef, ipos, istart', &
                                   icoef, ipos, istart
               if ( itest1(ipos)==0 ) then

!              --- First time coefficient has been read

                  itest1(ipos) = 1

               else

!              --- coefficient filled more than once

                  call errint ( icoef, 1 )
                  call errint ( ielgrp, 2 )
                  call errwar ( 1865, 2, 0, 0 )

               end if

               if ( itype==1 .or. itype==3 ) then

!              --- itype = 1, icoef = j or itype = 3, coef i = func j
!                  Take scalars into account

                  iwork(1,ipos) = iincof(iposin+3)
                  iposin = iposin + 3
                  if ( debug ) &
                     write(irefwr,1) 'iwork(1,ipos)', iwork(1,ipos)

               else if ( itype==12 ) then

!              --- itype = 12, icoef = text
!                  Take scalars into account

                  iwork(1,ipos) = iincof(iposin+3)+100100000
                  iposin = iposin + 3
                  if ( debug ) &
                     write(irefwr,1) 'iwork(1,ipos)', iwork(1,ipos)

               else if ( itype==2 ) then

!              --- itype = 2, coef = v

                  iwork(1,ipos) = 0
                  iposrn = iincof(iposin+3)
                  iposin = iposin + 3
                  if ( debug ) &
                     write(irefwr,1) 'iposrn, iposin, ipos', &
                                      iposrn, iposin, ipos
                  if ( compl ) then

!                 --- Complex case

                     work(2*ipos-1) = rincof(iposrn)
                     work(2*ipos) = rincof(iposrn+1)

                  else

!                 --- Real case

                     if ( abs(iposrn)>1000 ) then

!                    --- |iposrn|>1000, special case of indexed constant

                        if ( iposrn>0 ) then
                           sig = 1d0
                        else
                           sig = -1d0
                        end if  ! ( iposrn>0 )

                        iseq = iposrn/1000000     ! sequence number of constant
                        ivar = mod(iposrn,1000000)/1000  ! seq number of scalar
                        ityp = mod(iposrn,1000)   ! real or integer

                        if ( debug ) &
                           write(irefwr,1) 'iseq, ivar, ityp, ipos', &
                                            iseq, ivar, ityp, ipos

                        scal = nint(scalars(ivar)) ! value of scalar

                        if ( debug ) write(irefwr,1) 'scal', scal

                        if ( ityp==1 ) then

!                       --- Type integer

                           work(ipos) = sig*incons(iseq+scal-1)

                        else

!                       --- Type real

                           work(ipos) = sig*rlcons(iseq+scal-1)

                        end if  ! ( ityp==1 )

                     else

!                    --- Standard case

                        work(ipos) = rincof(iposrn)

                     end if  ! ( abs(iposrn)>1000 )

                  end if
                  if ( debug ) write(irefwr,2) 'coefficient', work(ipos)

               else if ( itype==4 .or. itype==5 ) then

!              --- itype = 4 or 5, elements or points

                  if ( itype==4 ) then
                     iwork(1,ipos) = 2001
                  else
                     iwork(1,ipos) = 2002
                  end if
                  iwork(2,ipos) = iincof(iposin+3)
                  iposin = iposin + 3

               else if ( itype==6 ) then

!              --- itype = 6, coef i = old solution j, degree of freedom d

                  iwork(1,ipos) = 2003
                  iwork(2,ipos) = iincof(iposin+3)
                  iwork(3,ipos) = iincof(iposin+4)
                  iposin = iposin + 4
                  if ( compl ) then

!                 --- Complex case

                     work(2*ipos-1) = 0d0
                     work(2*ipos) = 0d0

                  else

!                 --- Real case

                     work(ipos) = 0d0

                  end if

               else if ( itype==7 ) then

!              --- itype = 7, coef i = old solution j, degree of freedom d
!                           value = v

                  iwork(1,ipos) = 2004
                  iwork(2,ipos) = iincof(iposin+3)
                  iwork(3,ipos) = iincof(iposin+4)
                  iposrn        = iincof(iposin+5)
                  iposin = iposin + 5
                  if ( compl ) then

!                 --- Complex case

                     work(2*ipos-1) = rincof(iposrn)
                     work(2*ipos) = rincof(iposrn+1)

                  else

!                 --- Real case

                     work(ipos) = rincof(iposrn)

                  end if

               else if ( itype==8 ) then

!              --- itype = 8, coef i = old solution j, degree of freedom d
!                           func = ifunc

                  iwork(1,ipos) = 2004
                  iwork(2,ipos) = iincof(iposin+3)
                  iwork(3,ipos) = iincof(iposin+4)
                  iwork(4,ipos) = iincof(iposin+5)
                  iposin = iposin + 5

               else if ( itype==9 ) then

!              --- itype = 9, coefi = coefj

                  iwork(1,ipos) = -iincof(iposin+3)
                  iposin = iposin + 3

               else if ( itype==10 ) then

!              --- itype = 10, sol_func = k

                  iwork(1,ipos) = iincof(iposin+3)+10000
                  iposin = iposin + 3

               end if

            end if
            go to 250

         end if
         go to 200

      end if

1000  call erclos ( 'erexpc' )
      if ( debug ) then

!     --- Debug information

         call prinin1large ( iwork, ipos, 4, 'iwork' )
         write(irefwr,*) 'End erexpc'

      end if  ! ( debug )

      end
