      subroutine erfillcurstep ( istep, iexclu, n, inpelm, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes depending on the values of iexclu and istep
!     curves only
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istep, iexclu, n, inpelm, nodes(*)

!     iexclu         i    Indicates which points are excluded
!                         0:  No points must be excluded
!                         1:  Exclude initial point
!                         2:  Exclude end point
!                         3:  Exclude both initial and end point
!     inpelm         i    number of points per element
!     istep          i    Step in curves
!     iwork          i    Work array of length NPOINT.
!                         iwork has been filled with n nodes
!     n             i/o   at input: number of nodes in iwork
!                         at output: number of nodes in nodes
!     nodes          o    Contains the node numbers that must be used
!                         It is a copy of iwork but reduced by the values
!                         of istep and iexclu
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kstart, kend, i, j
      integer, allocatable :: iwork(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     j              Counting variable
!     kend           Last point in array iwork that should be taken into
!                    account
!     kstart         First point in array iwork that should be taken into
!                    account
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       6   no nodal points are created along the curve.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcurstep' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcurstep'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(n), stat = error )
      if ( error/=0 ) call eralloc ( error, n, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork(1:n) = nodes(1:n)

      if ( istep>=0 ) then

!     --- istep>=  0   end points of intervals

         if ( iexclu==0 ) then

!        --- iexclu = 0  No excluding of nodal points

            kstart = 1
            kend   = n

         else if ( iexclu==1 ) then

!        --- iexclu = 1 Exclude initial point

            kstart = 2
            kend   = n

         else if ( iexclu==2 ) then

!        --- iexclu = 2 Exclude end point

            kstart = 1
            kend   = n-1

         else if ( iexclu==3 ) then

!        --- iexclu = 3 Exclude initial and end point

            kstart = 2
            kend   = n-1

         end if

!        --- Fill array NODES

         n = 0
         do i = kstart, kend, inpelm
            n = n + 1
            nodes(n) = iwork(i)
            if ( nodes(n)==0 ) then

               call errint ( n, 1 )
               call errsub ( 6, 1, 0, 0 )

            else if ( nodes(n)<0 ) then

!           --- negative node: skip

               n = n-1

            end if
         end do

      else

!     --- istep<0

         kend = n-1
         n    = 0
         do i = 1, kend, inpelm
            do j = 1, -istep
               n = n + 1
               nodes(n) = iwork(i+j)
               if ( nodes(n)==0 ) then

                  call errint ( n, 1 )
                  call errsub ( 6, 1, 0, 0 )

               else if ( nodes(n)<0 ) then

!              --- negative node: skip

                  n = n-1

               end if
            end do
         end do

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'erfillcurstep' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcurstep'

      end if  ! ( debug )

      end
