      subroutine integs1 ( inpvol, outval, inpvec, iread, iseqvec, numveclc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.7  date 17-04-2013 Correction inpvec
!        version  2.6  date 22-22-2012 Extra debug
!        version  2.5  date 21-10-2011 Extension with ichoice negative
!
!   copyright (c) 1999-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Computes a volume integral over a region
!     Depending on the parameter iread input is given by inpvol or
!     read from the standard input file
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpvol(*), inpvec(5,*), iread, iseqvec, numveclc
      double precision outval(*)

!     inpvec         i    Contains information concerning the input vector
!     inpvol         i    Integer array in which the user indicates what
!                         computations must be performed by subroutine INTEGR
!                         The entries of INPVOL must be filled (by the user) as
!                         follows
!                         If the user sets a value equal to 0, the default value
!                         is used
!                         1  Number of entries of INPVOL that are filled by the
!                            user. If INPVOL(1) = 0, only defaults are used
!                         For the other positions see subroutine READIN.
!                         Pos. i in readin refers to position i+1 in this
!                         subroutine
!     iread          i    With this parameter it is indicated if array INPVOL
!                         is used or that input is read from the standard INPUT
!                         file
!                         Possible values:
!                         0   The input is read from the standard input file
!                         >0  The input as read from the standard input file by
!                             subroutine SEPSTN is used. The value of iread
!                             defines the sequence number
!                         -1  No input is read, all information from array
!                             INPVOL is used
!     iseqvec        i    Starting address of sequence numbers of vectors
!     numveclc         i    Number of vectors stored in isol
!     outval         o    In this array the computed integral is stored as well
!                         as the minimal and maximum values per element if
!                         MAXMIN>0:
!                         OUTVAL(1): integral
!                         OUTVAL(2): minimum value of the integrals per element
!                         OUTVAL(3): maximum value of the integrals per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nparm=8
      integer ihelp
      integer, allocatable :: inpcop(:)

!     ihelp          Help variable to compute the starting address of array
!                    IHELP
!     inpcop         Copy of input array extended with defaults
!     nparm          Number of entries in iinvol IHELP excepted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INTEGSINPUT    Read or copy input for subroutine integs into inpcop
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLSMF1    Fill part of input for subroutine SEPBUILD in SMF
!     TO0085         Performs the actual integration
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   initialize array inpcop
!   If ( iread=-1 ) then
!      Fill array inpcop by copying from array inpvol
!   else if ( iread=0 ) then
!      Read array inpcop from the standard input file
!   else (iread>0)
!      Copy array inpcop from IBUFFR
!   Compute integrals
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'integs1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from integs1'
         write(irefwr,1) 'iread, iseqvec, numveclc', &
                          iread, iseqvec, numveclc
         call prinin ( inpvol, inpvol(1), 'inpvol' )
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array inpcop

      allocate ( inpcop(nparm+nelgrp), stat = error )
      if ( error/=0 ) call eralloc ( error, nparm+nelgrp, 'inpcop' )
      if ( ierror/=0 ) go to 1000

      call integsinput ( inpcop, nparm, iread, &
                         nelgrp, numveclc, iseqvec, inpvol, ihelp )

!     --- Put information into SMF

      call sepfillsmf1 ( inpvec, max(1,numveclc) )

      call to0085 ( inpcop, outval, inpcop(ihelp) )
      if ( inpcop(4)>0 .and. abs(outval(2))<=0d0 .and. &
           abs(outval(2))<=0d0 .and. iread>0 ) then

!     --- if both the minimum and maximum value per element of the
!         integral are zero, one of the two is set equal to machine
!         accuracy, in order that integs recognizes that outval(2-3)
!         have been filled

         outval(3) = epsmac

      end if

      deallocate ( inpcop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inpcop' )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( outval, 2, 'outval' )
         write(irefwr,*) 'End integs1'

      end if  ! ( debug )
      call erclos ( 'integs1' )

      end
