      subroutine pradapintbn2d1 ( numendpoints, istore, ilev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find end nodes of interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     boundary
!     interface
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numendpoints, istore(2,*), ilev

!     ilev           i    Sequence number of level being considered
!     istore        i/o   Work array to store end points of interface
!                         istore(1,i) contains i-th end point
!                         istore(2,i) contains its neighbor
!     numendpoints   o    Number of end points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kstart, numedges, i, iedge, jstart, nnode_edge, j, &
              node
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iedge          edge sequence number
!     iwork          integer work array of  length npoint
!                    iwork(i) contains the number of times node i appears
!                    in the edges of the interface
!     j              Counting variable
!     jstart         starting position
!     kstart         starting position
!     nnode_edge     number of nodes corresponding to edge
!     node           Node number
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pradapintbn2d1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pradapintbn2d1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numendpoints = 0  ! number of end points

!     --- Find all edges with all points on boundary

      kstart = obstfaces(ilev)
      numedges = obstfaces(kstart)
      if ( numedges<=1 ) go to 1000

!     --- Define work space

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      if ( debug ) write(irefwr,1) 'ilev, numedges', ilev, numedges

!     --- Loop over all edges in interface

      do i = 1, numedges

         iedge = obstfaces(kstart+i)       ! edge number
         jstart = nmeshc(iedge)
         nnode_edge = nmeshc(jstart)        ! number of nodes in edge

         if ( debug ) &
            write(irefwr,1) 'i, iedge, nnode_edge', i, iedge, nnode_edge

!        --- Loop over end points of edge

         do j = 1, nnode_edge, nnode_edge-1

            node = nmeshc(jstart+j)         ! node on edge
            iwork(node) = iwork(node)+1     ! raise iwork
            if ( debug ) write(irefwr,1) 'j, node', j, node

         end do  ! j = 1, nnode_edge, nnode_edge

      end do  ! i = 1, numedges

!     --- Loop over all edges in interface

      do i = 1, numedges

         iedge = obstfaces(kstart+i)       ! edge number
         jstart = nmeshc(iedge)
         nnode_edge = nmeshc(jstart)        ! number of nodes in edge

         node = nmeshc(jstart+1)         ! start node on edge
         if ( debug ) &
            write(irefwr,1) 'i, node, iwork(node)', i, node, iwork(node)
         if ( iwork(node)==1 ) then

!        --- end point found, store node and neighbor

            numendpoints = numendpoints+1
            istore(1,numendpoints) = node
            istore(2,numendpoints) = nmeshc(jstart+2)
            iwork(node) = 0

            if ( debug ) &
               write(irefwr,1) 'numendpoints, istore', &
                                numendpoints, istore(:,numendpoints)

         end if  ! ( iwork(node)==1 )

         node = nmeshc(jstart+nnode_edge)         ! end node on edge
         if ( debug ) &
            write(irefwr,1) 'i, node, iwork(node)', i, node, iwork(node)

         if ( iwork(node)==1 ) then

!        --- end point found, store node and neighbor

            numendpoints = numendpoints+1
            istore(1,numendpoints) = node
            istore(2,numendpoints) = nmeshc(jstart+nnode_edge-1)
            iwork(node) = 0

         end if  ! ( iwork(node)==1 )

      end do  ! i = 1, numedges

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( istore, 2, numendpoints, 'istore' )
         write(irefwr,*) 'End pradapintbn2d1'

      end if  ! ( debug )
      call erclos ( 'pradapintbn2d1' )

      end
