      subroutine prcompmesh ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 31-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if two meshes are equal by comparing the coordinates
!     and the topology
!     The local numbering may be different
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compare
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    Contains the integer input
!                         See subroutine readsccompmesh for a description
!     istart        i/o   First position in inpsep with respect to the
!                         array inpsep in tosepcomsub
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mesh1, mesh2, npoint2, nelem2, &
              i, nelgrp2, iseqkm2, lenkmeshc, iseqkm1
      integer, allocatable :: inodes1(:), inodes2(:), iwork(:)
      logical same

!                    they are not
!     i              Counting variable
!     inodes1        work array to store the ordering of the nodes for mesh 1
!                    The nodes are ordered in increasing coordinates first
!                    with respect to the x-coordinate then to the y-coordinate
!                    etc.
!     inodes2        work array to store the ordering of the nodes for mesh 2
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     iwork          integer work array to be used for reordering
!     lenkmeshc      length of array kmesh2dc
!     mesh1          Sequence number of first mesh
!     mesh2          Sequence number of second mesh
!     nelem2         number of elements in mesh 2
!     nelgrp2        number of element groups in mesh 2
!     npoint2        number of points in second mesh
!     same           If true the arrays are the same
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOMPCOOR     Compare coordinates in two meshes
!     PRCOMPCURVE    Compare curves in two meshes
!     PRCOMPELEM     Compare elements in two meshes
!     PRCOMPUSERP    Compare user points in two meshes
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPSORTCOOR    Sort nodal points with respect to increasing co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
!    3105   parameters have the same value but should be different
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompmesh'

      end if  ! ( debug )
      same = .true.
      if ( ierror/=0 ) go to 1000

      mesh1 = inpsep(1)
      mesh2 = inpsep(2)

      if ( mesh1<0 .or. mesh1>5 ) then

!     --- mesh1 out of range

         call errchr ( 'mesh1', 1 )
         call errint ( mesh1, 1 )
         call errint ( 1, 2 )
         call errint ( 5, 3 )
         call errsub ( 1378, 3, 0, 1 )

      end if  ! ( mesh1<0 .or. mesh1>5 )

      if ( mesh2<0 .or. mesh2>5 ) then

!     --- mesh2 out of range

         call errchr ( 'mesh2', 1 )
         call errint ( mesh2, 1 )
         call errint ( 1, 2 )
         call errint ( 5, 3 )
         call errsub ( 1378, 3, 0, 1 )

      end if  ! ( mesh2<0 .or. mesh2>5 )

      if ( mesh1==mesh2 ) then

!     --- mesh1 = mesh2, but they should be different

         call errchr ( 'mesh1', 1 )
         call errchr ( 'mesh2', 2 )
         call errint ( mesh1, 1 )
         call errsub ( 3105, 1, 0, 2 )

      end if  ! ( mesh1==mesh2 )
      if ( ierror/=0 ) go to 1000

      kmesh => kmesh2d(mesh1)
      call sepfillkmesh(kmesh)
      iseqkm1 = iseqkm

      iseqkm2 = kmesh2d(mesh2)

      npoint2 = km(iseqkm2)%kmesh(8)

      if ( npoint/=npoint2) then

!     --- Number of nodes in meshes are different

         write(irefwr,*) 'The number of points in mesh ', mesh1, &
                         ' is equal to ', npoint
         write(irefwr,*) 'The number of points in mesh ', mesh2, &
                         ' is equal to ', npoint2
         write(irefwr,*) 'This means that both meshes are different'
         go to 1000

      end if  ! ( npoint/=npoint2)

      nelgrp2 = km(iseqkm2)%kmesh(5)
      if ( nelgrp/=nelgrp2 ) then

!     --- nelgrp#nelgrp2

         write(irefwr,*) 'The number of element groups in mesh ', mesh1, &
                         ' is equal to ', nelgrp
         write(irefwr,*) 'The number of element groups in mesh ', mesh2, &
                         ' is equal to ', nelgrp2
         write(irefwr,*) 'This means that both meshes are different'
         go to 1000

      end if  ! ( nelgrp/=nelgrp2 )

      if ( nelgrp>1 ) then

!     --- There are more element groups, check if the number of elements per
!         group are the same

         do i = 1, nelgrp

            if ( kmeshg(i)/=km(iseqkm2)%kmeshg(i) ) then

!           --- Number of elements different

              write(irefwr,*) 'The number of elements in element group', &
                              i, ' in mesh ', mesh1, &
                              ' is equal to ', kmeshg(i)

              write(irefwr,*) 'The number of elements in element group', &
                              i, ' in mesh ', mesh2, &
                              ' is equal to ', km(iseqkm2)%kmeshg(i)
              write(irefwr,*) &
                 'This means that both meshes are different'
              go to 1000

            end if  ! ( kmeshg(i)/=km(iseqkm2)%kmeshg(i) )

         end do  ! i = 1, nelgrp

      end if  ! ( nelgrp>1 )

      nelem2 = km(iseqkm2)%kmesh(9)

      if ( nelem/=nelem2) then

!     --- Number of nodes in meshes are different

         write(irefwr,*) 'The number of elements in mesh ', mesh1, &
                         ' is equal to ', nelem
         write(irefwr,*) 'The number of elements in mesh ', mesh2, &
                         ' is equal to ', nelem2
         write(irefwr,*) 'This means that both meshes are different'
         go to 1000

      end if  ! ( nelem/=nelem2)

!     --- sort coordinates of both meshes

      allocate ( inodes1(npoint), inodes2(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'inodes1' )
      if ( ierror/=0 ) go to 1000

!     --- sort

      do i = 1, npoint
         inodes1(i) = i   ! start numbering
      end do  ! i = 1, npoint

      inodes2 = inodes1
      call sepsortcoor ( 1, npoint, inodes1 )
      coor => km(iseqkm2)%coor
      call sepsortcoor ( 1, npoint, inodes2 )
      coor => km(iseqkm1)%coor

!     --- Compare coordinates

      call prcompcoor ( coor, km(iseqkm2)%coor, npoint, &
                        inodes1, inodes2, ndim, same )
      if ( .not. same ) go to 900

!     --- Store inverse of inodes1 and inodes2 in these arrays

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      do i = 1, npoint
         iwork(inodes1(i)) = i
      end do  ! i = 1, npoint
      inodes1 = iwork

      do i = 1, npoint
         iwork(inodes2(i)) = i
      end do  ! i = 1, npoint
      inodes2 = iwork

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      lenkmeshc = size(kmeshc)

!     --- Compare elements

      call prcompelem ( km(iseqkm2)%kmeshc, inodes1, &
                        inodes2, lenkmeshc, same )
      if ( .not. same ) go to 900

!     --- Compare kmesh2d part l (user points)

      call prcompuserp ( km(iseqkm2)%kmeshl, inodes1, inodes2, same )
      if ( .not. same ) go to 900

!     --- Compare kmesh2d part m (curves)

      call prcompcurve ( km(iseqkm2)%kmeshm, inodes1, inodes2, same )
      if ( .not. same ) go to 900

900   deallocate ( inodes1, inodes2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inodes1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompmesh'

      end if  ! ( debug )
      call erclos ( 'prcompmesh' )

      if ( .not. same ) ierror = 1
      istart = istart+3  ! raise istart

      end
