      subroutine prcopy ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 24-12-2014 New definition clinsl
!        version  1.3  date 06-02-2012 Allow indexed vectors
!        version  1.2  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 2000-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy complete vector into another vector or
!    copy one degree of freedom from one vector into another one
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmoduleuser
      use sepmodulemain
      use sepmodulemnsub
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents
!                         1: sequence number of input vector in isolut2d
!                         2: sequence number of output vector in isolut2d
!                         3: defines which component must be copied
!                            0: complete array
!                            >0: one component
!                         4: if inpsep(3)>0, the component of the output vector
!                            to which the degree of freedom must be copied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idegfd1, idegfd2, itypvc, ivec1, ivec2, nch1, nch2

!     idegfd1        First degree of freedom
!     idegfd2        Second degree of freedom
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec1          Sequence number of first vector
!     ivec2          Sequence number of second vector
!     nch1           Number of characters in first word
!     nch2           Number of characters in second word
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICREATVEC1   Create space for a vector with one degree of freedom
!                    per point
!     INIVECTR       Return with sequence number of vector
!     SEPCOPYDF      copies the contents of one degree of freedom in array ivec1
!                    into another degree of freedom in array ivec2
!     SEPCOPYVC      Copy one SEPRAN vector into another one
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2145   Vector to be copied has not been filled
!    2146   Vector in which a degree of freedom must be copied has not been
!           filled
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make some initializations
!     Check if vector to be copied has been filled
!     If ( inpsep(3) = 0 ) then
!        Copy complete vector
!     else
!        copy one degree of vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopy' )

!     --- Make some initializations

      ivec1 = inivectr(inpsep(1))
      ivec2 = inivectr(inpsep(2))
      nch1  = index(names(ivec1),' ')-1
      if ( nch1<=0 ) nch1 = len(names(ivec1))
      nch2  = index(names(ivec2),' ')-1
      if ( nch2<=0 ) nch1 = len(names(ivec2))

!     --- Check if vector to be copied has been filled

      itypvc = ks(isolut2d(ivec1))%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 .and. &
           itypvc/=126 .and. itypvc/=127 .and. itypvc/=801 ) then

!     --- vector to be copied has not been filled

         call errint ( ivec1, 1 )
         call errint ( itypvc, 2 )
         call errchr ( names(ivec1)(1:nch1), 1 )
         call errsub ( 2145, 2, 0, 1 )

      end if

      if ( inpsep(3)==0 ) then

!     --- inpsep(3) = 0, copy complete vector

         if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) then

!        --- print information about the copying

            write ( irefwr, 100 ) names(ivec1)(1:nch1), ivec1, &
                                  names(ivec2)(1:nch2), ivec2
100         format ( ' Copy vector ', a, &
                     ' with sequence number ', i3, &
                     ' into vector ', a, &
                     ' with sequence number ', i3 )

         end if
         call sepcopyvc ( isolut2d(ivec1), &
                         isolut2d(ivec2) )
         numvecmn = max(numvecmn,inpsep(2))

      else

!     --- inpsep(3)>0, copy component of vector 1 into vector 2
!         vector 2 has been filled

         idegfd1 = inpsep(3)
         idegfd2 = inpsep(4)

         if ( isolut2d(ivec2)==0 .and. idegfd2==1 ) then

!        --- Vector does not yet exist, create this vector

            call inicreatvec1 ( isolut2d(ivec2),'one_component' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( isolut2d(ivec2)==0 )

         if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) then

!        --- print information about the copying

            write ( irefwr, 110 ) idegfd1, names(ivec1)(1:nch1), &
               ivec1, idegfd2, names(ivec2)(1:nch2), ivec2
110         format ( ' Copy degree of freedom ', i3, ' of vector ', a, &
                     ' with sequence number ', i3, &
                     ' into degree of freedom ', i3, ' of vector ', a, &
                     ' with sequence number ', i3 )

         end if
         if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 ) then

!        --- vector to be copied has not been filled

            call errint ( ivec1, 1 )
            call errint ( itypvc, 2 )
            call errchr ( names(ivec1)(1:nch1), 1 )
            call errsub ( 2145, 2, 0, 1 )

         end if
         itypvc = ks(isolut2d(ivec2))%typevec
         if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 ) then

!        --- vector to which vec1 must copied has not been filled

            call errint ( ivec2, 1 )
            call errint ( itypvc, 2 )
            call errchr ( names(ivec2)(1:nch2), 1 )
            call errsub ( 2146, 2, 0, 1 )

         end if
         call sepcopydf ( isolut2d(ivec1), isolut2d(ivec2), kprob, kprob, &
                          idegfd1, idegfd2 )

      end if
      istartmn = istartmn + 5

1000  call erclos ( 'prcopy' )
      end subroutine prcopy
