      subroutine prcrealevvect ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 11-12-2017 New storage of intmat
!        version  2.0  date 05-11-2013 Extra parameter rinsep
!        version  1.0  date 01-10-2012
!
!   copyright (c) 2012-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create level set vector from given obstacle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     obstacle
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduledebug
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readsclevvect for a description
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readsclevvect for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leninpsep
      parameter ( leninpsep=9 )
      integer meshorig, meshobst, iinputusr(3), i, iseqad, &
              iphi, inpcop(leninpsep), iinsep(5), istartloc, iseqprob
      double precision rinputusr(2), epssav

!                    they are not
!     epssav         Hel parameter to store the value of accuracy_obstacle
!     i              Counting variable
!     iinputusr      input array for subroutine mshobstmsh
!                    For a description see mshobstmsh
!     iinsep         input array for subroutines
!     inpcop         Copy of integer input array extended with defaults
!     iphi           Sequence number of level set vector in isol
!     iseqad         Dummy
!     iseqprob       Sequence number of problem input
!     istartloc      Local parameter istart (dummy)
!     leninpsep      Length of array inpsep
!     meshobst       Sequence number of obstacle mesh to be used
!     meshorig       Sequence number of original mesh to be used
!     rinputusr      Is used to store the values of some parameters for
!                    the shift of nodes to the obstacle boundary
!                    1:  eps_move
!                    2:  eps_min
!                    See subroutine mshobstms for an extended description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     INICHINPSOL    Adapt input array with respect to variables
!     INIDELMAP      Delete contents of array map
!     MSHOBSTMARKELM Recompute array ielemobst in the sense that only elements
!                    inside or outside the obstacle are marked
!     MSHOBSTMS      Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh
!     PLOTOBSTACLE   Print and plot information about the obstacle
!     PRFILLLEVSET   Fill level set function phi as signed distance function
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINTERPOLATE  Interpolate solution from one mesh to the other
!     PROBDFEXT      Reads and stores problem definition
!     PROCOPY1       Copy important information of problem definition
!     PRRMLEVMESH    Delete the levelset mesh created for a zero levelset
!                    crossing the mesh
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPPRINIPOBST  Print the contents of array ipointobst
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcrealevvect' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcrealevvect'
         call prinin ( meshseqs, 5, 'meshseqs' )
         call prininlarge ( inpsep, 9, 'inpsep' )

  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get real values

      rinputusr(1) = getparm ( inpsep, rinsep, 7, 'eps_move' )
      rinputusr(2) = getparm ( inpsep, rinsep, 9, 'eps_min' )

!     --- Replace scalars by actual values

      inpcop(1:leninpsep) = inpsep(1:leninpsep)
      call inichinpsol ( inpsep, leninpsep-2 )

      if ( debug ) call prinin ( inpsep, leninpsep, 'inpsep' )

      if ( inpsep(1)<0 ) then

!     --- inpsep(1)<0, hence default mesh number

         meshorig = meshnr

!        --- Fill inpsep(1)

         do i = 1, 5
            if ( meshnr==meshseqs(i) ) then
               inpsep(1) = i
               exit
            end if  ! ( meshnr==meshseqs(i) )
         end do  ! i = 1, 5

      else

!     --- inpsep(1)>0, hence mesh number from inpsep(1)

         meshorig  = meshseqs(inpsep(1))

      end if  ! ( inpsep(1)<0 )

      if ( inpsep(2)<0 ) then

!     --- inpsep(2)<0, hence default mesh number

         meshobst = meshorig+1

!        --- Fill inpsep(2)

         do i = 1, 5
            if ( meshorig+1==meshseqs(i) ) then
               inpsep(2) = i
               exit
            end if  ! ( meshorig+1==meshseqs(i) )
         end do  ! i = 1, 5

      else

!     --- inpsep(2)>0, hence mesh number from inpsep(2)

         meshobst = meshseqs(inpsep(2))

      end if  ! ( inpsep(2)<0 )

      iprob = inpsep(3)
      iphi = inpsep(4)
      if ( debug ) &
         write(irefwr,1) 'meshorig, meshobst, iprob, iphi', &
                          meshorig, meshobst, iprob, iphi

!     --- Compute the intersections of the obstacle with the basis mesh

      epssav = accuracy_obstacle        ! save accuracy_obstacle
      accuracy_obstacle = rinputusr(2)  ! set accuracy_obstacle to eps_min

      if ( kelmad==0 ) call sepfillkmeshad(iseqad)
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3
      accuracy_obstacle = epssav        ! reset accuracy_obstacle
      if ( ierror/=0 ) go to 1000

      if ( kmeshad(4)==0 ) then

!     --- Special case submesh coincides with original mesh

         meshseqs(inpsep(2)) = -meshseqs(inpsep(2))

!        --- Refill array ielemobst: only elements inside the obstacle are
!            marked

         call mshobstmarkelm

!        --- Print and plot information depending on plotobst and printobst

         if ( plotobst .or. printobst ) then

!        --- debug = true, plot nodes and elements with a colour

            call plotobstacle ( debug_mshobstmesh, 0 )

         end if
         go to 1000

      end if

!     --- delete all arrays corresponding to map(.,meshobst)

      do i = 1, maxmesh
         if ( i/=meshorig ) then
            if ( associated(map(i,meshobst)%imap)) then
               deallocate ( map(i,meshobst)%imap, map(i,meshobst)%rmap )
               call sepclear ( map(i,meshobst) )
            end if  ! ( associated(map(i,meshobst)%imap))
         end if  ! ( i/=meshorig )
         if ( associated(map(meshobst,i)%imap)) then
            deallocate ( map(meshobst,i)%imap, map(meshobst,i)%rmap )
            call sepclear ( map(meshobst,i) )
         end if  ! ( associated(map(meshobst,i)%imap))
      end do  ! i = 1, maxmesh

      iinputusr(1) = 3
      iinputusr(2) = -2  ! ishift
      iinputusr(3) = 0  ! smooth
      call mshobstms ( meshorig ,meshobst, map(meshorig,meshobst), &
                       iinputusr, rinputusr )

!     --- Copy old kprob to new one

      call procopy1 ( kprob2d(meshorig), kprob2d(meshobst) )
      iseqprob = 1
      kmesh => kmesh2d(meshobst)
      call sepfillkmesh(kmesh)
      kprob => kprob2d(meshobst)
      call probdfext ( 3, iseqprob )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call sepprinipobst ( ipointobst )

!     --- Create level set function phi

      iprob = inpsep(3)
      call sepcreasol110 ( isolut3d(iphi,meshobst), 'phi', 0 )

      call prfilllevset ( inpsep, isolut3d(iphi,meshobst) )

!     --- reset inpsep

      inpsep(1:leninpsep) = inpcop(1:leninpsep)

!     --- Map levelset function phi back on old meshorig

      iinsep(1) = iphi
      iinsep(2) = meshobst
      iinsep(3) = meshorig
      istartloc = 0
      call printerpolate ( istartloc, iinsep )

!     --- Remove meshobst

      iinsep(1) = meshorig
      iinsep(2) = meshobst
      iinsep(4) = iphi
      iinsep(5) = -1
      call prrmlevmesh ( iinsep, istartloc )

!     --- reset inpsep

      inpcop(1:leninpsep) = inpcop(1:leninpsep)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iphi, meshobst, meshnr', &
                          iphi, meshobst, meshnr
         call sepprintv ( isolut3d(iphi,meshorig), 'phi' )
         write(irefwr,*) 'End prcrealevvect'

      end if  ! ( debug )
      call erclos ( 'prcrealevvect' )

      istartmn = istartmn+leninpsep+1

      end
