      integer function prfindclosnod ( x, y, grid, nx, ny, coor, &
                                       numnodes, xmin, ymin, hx, hy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-03-2012 Correction index
!        version  1.0  date 16-11-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find node in set of coordinates that has shortest distance
!     to point (x,y)
!     2d only
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     distance
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer grid(*), nx, ny, numnodes
      double precision x, y, coor(2,*), xmin, ymin, hx, hy

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     grid           i    In this array the nodes of the interface are
!                         coupled to a rectangular grid of size nx * ny
!                         The decription of this array is the same as that of
!                         kmesh part ac subparts 2 and 3
!     hx             i    x-direction step
!     hy             i    y-direction step
!     numnodes       i    Number of nodal points in part 2 of grid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     prfindclosnod  o    Sequence number of node in array grid that has
!                         shortest distance to (x,y)
!     x              i    x co-ordinate of point
!     xmin           i    Minimum x-value in grid
!     y              i    y co-ordinate of point
!     ymin           i    Minimum y-value in grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indi, indj, nactive, cells(2,400), &
              nnodes, istore(numnodes), nodefound
      double precision dist
      integer, allocatable :: markcells(:,:)
      logical found

!     cells          Contains (i,j)-indices of active cells
!     dist           Distance between two points
!     error          Return value of allocate or deallocate
!     found          If true a node has been found
!     indi           i-index
!     indj           j-index
!     istore         Contains the set of nodes to be considered
!     markcells      Array of size nx * ny to mark cells that have
!                    already been used
!     nactive        Number of active cells
!     nnodes         Number of nodes in istore
!     nodefound      Node number of node with shortest distance
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFINDCLOSNOD1 Fill array cells with neighbors of old cells which have
!                    not been treated yet
!     PRFINDCLOSNOD2 Find all nodes in the set of active cells and store in
!                    istore
!     PRFINDCLOSNOD3 Find node with shortest distance to (x,y) in istore
!                    corresponding to cell (i,j)
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2150   no element found containing this point
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find i,j index corresponding to x,y
!     Store in active cells
!     found := false
!     while ( not found ) do
!        Find all nodes in set of active cells and store in istore
!        If ( nodes found ) then
!           found := true
!        else
!           set of active cells := all neighbors of set of active cells
!     Find node with shortest distance in istore and corresponding (i,j)
!     Find all nodes in neighbor cells
!     Final node := node with shortest distance
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindclosnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindclosnod'
         write(irefwr,2) 'x, y', x, y
         write(irefwr,2) 'xmin, ymin, hx, hy', xmin, ymin, hx, hy
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( markcells(nx,ny), stat = error )
      if ( error/=0 ) call eralloc ( error, nx*ny, 'markcells' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array markcells

      markcells = 0
      nodefound = 0

!     --- Find i,j index corresponding to x,y

      indi = max(1,min(nx,int((x-xmin)/hx)+1))
      indj = max(1,min(ny,int((y-ymin)/hy)+1))
      if ( debug ) write(irefwr,1) 'indi, indj', indi, indj

!     --- Store in active cells

      nactive = 1
      cells(1,1) = indi
      cells(2,1) = indj

      markcells(indi,indj) = 1

!     --- found := false

      found = .false.

!     --- while ( not found ) do

      do while ( .not. found )

!     --- Find all nodes in set of active cells and store in istore

         call prfindclosnod2 ( grid, cells, nactive, nnodes, istore, &
                               nx )

!        --- If ( nodes found ) then
!              found := true
!            else
!              set of active cells := all neighbors of set of active cells

         if ( nnodes>0 ) then

!        --- Nodes found

            found = .true.
            if ( debug ) call prinin ( istore, nnodes, 'istore' )

         else

!        --- No nodes found, new active set

            call prfindclosnod1 ( cells, nactive, markcells, nx, ny )
            if ( nactive==0 ) then

!           --- error: no cell found

               call errint ( indi, 1 )
               call errint ( indj, 2 )
               call erreal ( x, 1 )
               call erreal ( y, 2 )
               call errsub ( 2150, 2, 2, 0 )
               go to 1000

            end if  ! ( nactive==0 )

         end if  ! ( nnodes>0 )

      end do  ! while ( .not. found )

!     --- Find node with shortest distance in istore and corresponding to (i,j)

      dist = rinfin
      call prfindclosnod3 ( nnodes, istore, dist, nodefound, &
                            coor, x, y )
      if ( debug ) write(irefwr,1) 'nodefound', nodefound

!     --- Find all nodes in non-marked neighbor cells

      call prfindclosnod1 ( cells, nactive, markcells, nx, ny )
      call prfindclosnod2 ( grid, cells, nactive, nnodes, istore, nx )

!     --- Final node := node with shortest distance

      call prfindclosnod3 ( nnodes, istore, dist, nodefound, &
                            coor, x, y )

      prfindclosnod = nodefound

      deallocate ( markcells, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'markcells' )

1000  call erclos ( 'prfindclosnod' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nodefound', nodefound
         write(irefwr,*) 'End prfindclosnod'

      end if  ! ( debug )

      end
