      subroutine prflgridnod3d ( kmeshac, nx, ny, nz, nnodes, coor, &
                                 nodes, istart, xmin, xmax, ymin, ymax, &
                                 zmin, zmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-08-2012 Replace iwork by nodes
!        version  1.0  date 29-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information of a set of nodes in rectangular 3d grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     rectangle
!     grid
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshac(*), nx, ny, nz, nnodes, nodes(nnodes), istart
      double precision coor(3,*), xmin, xmax, ymin, ymax, zmin, zmax

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     istart         i    First position used in this part of kmeshac
!     kmeshac        o    Integer array containing the information of the
!                         subdivision of the mesh in cells
!                         See calling subroutine for a description
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be used to fill
!                         kmeshac
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     xmax           i    Maximum x-value
!     xmin           i    Minimum x-value
!     ymax           i    Maximum y-value
!     ymin           i    Minimum y-value
!     zmax           i    Maximum z-value
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodp, indi, indj, indk, indl
      integer, allocatable :: iwork(:,:)
      double precision distx, disty, distz, x, y, z

!     distx          Distance in x-direction
!     disty          Distance in y-direction
!     distz          Distance in z-direction
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     indi           i-index
!     indj           j-index
!     indk           k-index
!     indl           l-index
!     inodp          Nodal point number
!     iwork          integer work array of length 3*npoint
!                    The cell index (i,j,k) is stored iwork(1:3,.)
!     x              x co-ordinate of point
!     y              y co-ordinate of point
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array grid
!     For all nodes inode in grid do
!        Find index (ix,iy,iz) in grid
!        raise grid(ix,iy,iz,1) by 1
!        store ix, iy, iz in iwork
!     Fill grid(ix,iy, iz,3) with starting addresses
!     For all nodes inode in grid do
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prflgridnod3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prflgridnod3d'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(3,nnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*nnodes, 'iwork' )
      if ( ierror/=0 ) go to 1000

      kmeshac(istart:istart+nx*ny*nz+1) = 0

      distx = (xmax-xmin)/nx
      disty = (ymax-ymin)/ny
      distz = (zmax-zmin)/nz

      do i = 1, nnodes

!     --- For all nodes in part do
!         Compute i,j index for node and store in iwork(2-3,.)
!         Adapt Pointer array pos k = (i-1)*nx+j

         inodp = nodes(i)
         x = coor(1,inodp)
         y = coor(2,inodp)
         z = coor(3,inodp)
         indi = min(nx,int((x-xmin)/distx)+1)
         indj = min(ny,int((y-ymin)/disty)+1)
         indl = min(nz,int((z-zmin)/distz)+1)
         indk = (indl-1)*nx*ny+(indj-1)*nx+indi
         kmeshac(istart+indk) = kmeshac(istart+indk)+1
         iwork(1,i) = indi
         iwork(2,i) = indj
         iwork(3,i) = indl
         if ( debug ) then
            write(irefwr,1) 'i, inodp', i, inodp
            write(irefwr,2) 'x, y, z', x, y, z
            write(irefwr,1) 'indi, indj, indl', indi, indj, indl
         end if  ! ( debug )

      end do

!     --- Accumulate pointer array

      kmeshac(istart) = istart+nx*ny*nz+1
      do i = 1, nx*ny*nz
         kmeshac(istart+i) = kmeshac(istart+i-1)+kmeshac(istart+i)
      end do

      do i = 1, nnodes

!     --- For all nodes in part do
!         Fill node in second part of kmeshac
!         Adapt pointer array

         inodp = nodes(i)
         indi = iwork(1,i)
         indj = iwork(2,i)
         indl = iwork(3,i)
         indk = (indl-1)*nx*ny+(indj-1)*nx+indi
         kmeshac(kmeshac(istart+indk-1)) = inodp
         kmeshac(istart+indk-1) = kmeshac(istart+indk-1)+1

      end do

!     --- Reset part 1

      do i = nx*ny*nz, 1, -1
         kmeshac(istart+i) = kmeshac(istart+i-1)
      end do
      kmeshac(istart) = istart+nx*ny*nz+1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'prflgridnod3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prflgridnod3d'

      end if  ! ( debug )

      end
