      subroutine printerpolate1 ( meshin, meshout, ivecseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate solution from one mesh to the other
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemeshinf
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: meshin, meshout, ivecseq

!     ivecseq        i    Sequence number of vector to be considered with
!                         respect to the solution array (isol or isolut3d)
!     meshin         i    Sequence number of first mesh to be used
!     meshout        i    Sequence number of last mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idummy(1), interp, iplane, itrans, &
              number, iseqkm1, iseqkm2
      double precision plane(4), trans(3,3)

!     idummy         Dummy parameter
!     interp         Indication of what variables must be interpolated
!                    See subroutine intp3d
!     iplane         Defines direction of plane
!                    1:  plane x = constant
!                    2:  plane y = constant
!                    3:  plane z = constant
!                    4:  general plane
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     itrans         Indicates if the mesh must be transformed from a
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     plane          Array of length 4 containing the parameters a, b, c
!                    and d describing the plane: ax=by+cz+d = 0
!                    plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     trans          array containing the inverse of the transformation
!                    dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTMAP2D       Map solution defined in 3d mesh onto 2d intersection
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPINTMAP3D    Map solution in 3d region onto regular 3d solution defined
!                    in all nodes of the mesh (except centroids)
!     SEPINTPOLMS    Interpolate solution from one mesh to another
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3119   input and output mesh have different dimensions
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printerpolate1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printerpolate1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iseqkm1 = kmesh2d(meshin)
      iseqkm2 = kmesh2d(meshout)
      if ( km(iseqkm1)%kmesh(6)==km(iseqkm2)%kmesh(6) ) then

!     --- Dimension of both meshes the same

         if ( debug ) write(irefwr,*) 'before intpolmbf'
         call sepintpolms ( meshin, meshout, iinmap, &
                            isolut3d(ivecseq,meshin), &
                            isolut3d(ivecseq,meshout), &
                            map(meshin,meshout), idummy )

      else if ( km(iseqkm1)%kmesh(6)==3 .and. km(iseqkm2)%kmesh(6)==2 ) then

!     --- Intersection of 3d mesh by plane
!         Make temporary solution array containing all unknowns in 3d mesh

         number = 1
         interp = 0
         call sepactsolbf2 ( isolut3d(ivecseq,meshin) )

!        --- Make temporary solution array usolt containing all unknowns
!            in 3d mesh

         call sepintmap3d ( isolut3d(ivecseq,meshin), &
                            number, nunkpi, interp )

!        --- map solution onto 2d mesh

         itrans = 0
         iplane = 0
         if ( debug ) write(irefwr,*) 'before intmap2d'
         kprob => kprob2d(meshout)
         call intmap2d ( isolut3d(ivecseq,meshout), &
                         nunkp, map(meshin,meshout), &
                         meshout, itrans, plane, trans, &
                         iplane )

!        --- Remove superfluous arrays

         deallocate ( usolt )

      else

!     --- Other options not yet available

         call errint ( km(iseqkm1)%kmesh(6), 1 )
         call errint ( km(iseqkm2)%kmesh(6), 2 )
         call errsub ( 3119, 2, 0, 0 )
         go to 1000

      end if  ! ( km(iseqkm1)%kmesh(6,meshin)==km(iseqkm1)%kmesh(6,meshout) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printerpolate1'

      end if  ! ( debug )
      call erclos ( 'printerpolate1' )

      end
