      subroutine printintersec ( inpsep, values, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of line with 2d mesh and print the results
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     vector
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: values(*), rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscprinint
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscprinint
!     values         i    Double precision or complex array in which the real
!                         or complex information concerning the line is stored
!                         See subroutine readscprinint (rinsep)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idegfd, ifile, ichois, &
                 iskip, number, numveclc, ioutvc(20), ihelp(1), &
                 irefwrsav, i, ivalue

!     i              Counting variable
!     ichois         Choice parameter, see subroutine intp2dbf
!     idegfd         Degree of freedom sequence number
!     ifile          Sequence number of file name in array texts
!     ihelp          dummy
!     ioutvc         Standard SEPRAN array containing information
!                    concerning the interpolated function on the line.
!                    ioutvc is an array of structure 119
!                    Both the one-dimensional co-ordinates as the function
!                    values are stored in the array corresponding to ioutvc
!     irefwrsav      Help variable to store the value of irefwr
!     iskip          Indicates if element groups must be skipped (>0)
!                    or not (0)
!     ivalue         Integer value
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     numveclc         Number of vectors stored in ioutvc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICLOSEFILE   Close file and reset irefwr
!     INIOPENFILE    Open serial file and set irefwr
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINVCBF119    print vector of type 119
!     PRREDUCINTERS  Check if end point of line is given and if so reduce array
!                    usol to part defined by this length if length smaller
!                    than original length
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPINTP2D      Interpolate 2D region with straight line
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printintersec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printintersec'
         call prinin ( inpsep(1), 4, 'inpsep' )
         call prinrl ( values, 3, 'values' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      idegfd = inpsep(2)
      ifile = inpsep(3)

!     --- Compute nunkpi and nphysi

      call sepactsolbf2 ( isolut )

!     --- Compute intersection

      ichois = 1   ! line is defined by origin and angle
      iskip = 0    ! do not skip element groups

      if ( idegfd>0 .or. nphysi==1 ) then

!     --- Only 1 degree of freedom

         number = max(1,idegfd)
         ioutvc(1) = 0
         call sepintp2d ( ichois, ivalue, values, number, &
                          ioutvc, iskip, ihelp )
         numveclc = 1

      else

!     --- More degrees of freedom

         ioutvc(1:nphysi) = 0
         do number = 1, nphysi

            call sepintp2d ( ichois, ivalue, values, number, &
                             ioutvc(number), iskip, ihelp )

         end do  ! number = 1, nphysi
         numveclc = nphysi

      end if  ! ( idegfd>0 .or. nphysi==1 )

!     --- Reduce line in case end point is given

      call prreducinters ( inpsep, ioutvc, numveclc, &
                           rinsep, values )

!     --- Print to file

      if ( ifile>0 ) &
         call iniopenfile ( ifile, irefwrsav, inpsep(4), ' ' )

      call prinvcbf119 ( ioutvc, numveclc )

      if ( ifile>0 ) call iniclosefile ( irefwrsav )

!     --- Remove arrays ioutvc

      do i = 1, numveclc
         call seprmsolinfo ( ioutvc(i) )
      end do  ! i = 1, numveclc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printintersec'

      end if  ! ( debug )
      call erclos ( 'printintersec' )

      end subroutine printintersec
