      subroutine prlevinterfcp3 ( coorold, interfaceold, nofaceold, &
                                  coornew, interfacenew, nofacenew, &
                                  oldnodes, meshvel, dt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-04-2013 New parameter list
!        version  1.2  date 18-09-2012 New computation of h
!        version  1.1  date 22-08-2012 use local grid iwork
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate the concentration at the new interface from
!     values at the old interface in case of a level set method
!     Compute the corresponding mesh velocity
!     3d
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interpolation
!     3d
!     interface
!     mesh_velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nofaceold, interfaceold(nofaceold), nofacenew, &
              interfacenew(nofacenew), oldnodes(*)
      double precision coorold(3,*), coornew(3,*), meshvel(3,*), dt

!     coornew        i    Array containing the new coordinates of the nodes
!     coorold        i    Array containing the coordinates in the old mesh
!     dt             i    Time step
!     interfacenew   i    Contains node numbers of new interface
!     interfaceold   i    Contains node numbers of old interface
!     meshvel        o    Contains mesh velocity
!     nofacenew      i    Number of nodes in interfacenew
!     nofaceold      i    Number of nodes in interfaceold
!     oldnodes       o    Contains the set of old nodes corresponding to
!                         the nodes in interfacenew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nx, ny, nz, nodeold, node, i
      double precision xmin, xmax, ymin, ymax, zmin, zmax, h, &
                       hx, hy, hz, x, y, z
      integer, allocatable :: grid(:)

!     error          Return value of allocate or deallocate
!     grid           In this array the nodes of the interface are
!                    coupled to a rectangular grid of size nx * ny
!                    The decription of this array is the same as that of
!                    kmesh part ac subparts 2 and 3
!     h              help variable
!     hx             x-direction step
!     hy             y-direction step
!     hz             z-direction step
!     i              Counting variable
!     node           Node number in new interface
!     nodeold        Node number in old interface
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer prfindclosn3d

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFINDCLOSN3D  Find node in set of coordinates that has shortest distance
!                    to point (x,y,z) (3d)
!     PRFLGRIDNOD3D  Fill information of a set of nodes in rectangular 3d grid
!     PRINGRID3D     Print part of kmesh ac corresponding to 1 subpart of mesh
!                    (3d)
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all nodes in the new interface
!        Find closest node in the old interface
!        Copy concentration
!        Compute mesh velocity
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlevinterfcp3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlevinterfcp3'
         call prinin ( interfaceold, nofaceold, 'interfaceold' )
         call prinin ( interfacenew, nofacenew, 'interfacenew' )
         write(irefwr,2) 'dt', dt
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Make a rectangular grid over the old interface and store for
!         each node in the old interface to which grid cell it belongs

      xmin = minval(coorold(1,interfaceold(1:nofaceold)))
      xmax = maxval(coorold(1,interfaceold(1:nofaceold)))
      ymin = minval(coorold(2,interfaceold(1:nofaceold)))
      ymax = maxval(coorold(2,interfaceold(1:nofaceold)))
      zmin = minval(coorold(3,interfaceold(1:nofaceold)))
      zmax = maxval(coorold(3,interfaceold(1:nofaceold)))

      h = sqrt((ymax-ymin+xmax-xmin+zmax-zmin)/nofaceold)
      nx = min(100,nint((xmax-xmin)/h)+1)
      ny = min(100,nint((ymax-ymin)/h)+1)
      nz = min(100,nint((zmax-zmin)/h)+1)
      if ( nx==1 ) xmax = max(xmax, xmin+h)
      if ( ny==1 ) ymax = max(ymax, ymin+h)
      if ( nz==1 ) zmax = max(zmax, zmin+h)

      if ( debug ) then
         write(irefwr,1) 'nx, ny, nz', nx, ny, nz
         write(irefwr,2) 'xmin, xmax, ymin, ymax, zmin, zmax, h', &
                          xmin, xmax, ymin, ymax, zmin, zmax, h
      end if  ! ( debug )

      allocate ( grid(1+nx*ny*nz+nofaceold), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 1+nx*ny*nz+nofaceold, 'grid' )
      if ( ierror/=0 ) go to 1000

!     --- Fill grid

      call prflgridnod3d ( grid, nx, ny, nz, nofaceold, coorold, &
                           interfaceold, 1, xmin, xmax, ymin, ymax, &
                           zmin, zmax )

      if ( debug ) &
         call pringrid3d ( nx, ny, nz, 1, grid, xmin, xmax, &
                           ymin, ymax, zmin, zmax )

      hx = (xmax-xmin)/nx
      hy = (ymax-ymin)/ny
      hz = (zmax-zmin)/nz

!     --- For all nodes in the new interface

      do i = 1, nofacenew

         node = interfacenew(i)
         x = coornew(1,node)
         y = coornew(2,node)
         z = coornew(3,node)

         if ( debug ) then
            write(irefwr,1) 'node', node
            write(irefwr,2) 'x, y, z', x, y, z
         end if  ! ( debug )

!        --- Find closest node in the old interface

         nodeold = prfindclosn3d ( x, y, z, grid, nx, ny, nz, coorold, &
                                   nofaceold, xmin, ymin, zmin, &
                                   hx, hy, hz )
         if ( debug ) write(irefwr,1) 'nodeold', nodeold
         if ( ierror/=0 ) go to 1000

!        --- Copy concentration
!            Compute mesh velocity

         oldnodes(i) = nodeold
         meshvel(:,node) = (coorold(:,nodeold)-coornew(:,node))/dt

      end do  ! i = 1, nofacenew

      deallocate ( grid, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'grid' )

1000  call erclos ( 'prlevinterfcp3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlevinterfcp3'

      end if  ! ( debug )

      end
