      subroutine prlincomb ( iinvec, rinvec, invec1, &
                             invec2, iresvc, ivectr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute linear combination of two vectors and store in resvc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinvec(*), invec1, invec2, iresvc, ivectr
      double precision rinvec(*)

!     iinvec         i    User input array for subroutine MANVEC
!                         For a description see subroutine MANVECBF or PRMANV
!     invec1         i    First vector to be used
!     invec2         i    Second vector to be used
!     iresvc        i/o   Standard SEPRAN array containing information of the
!                         output solution vector
!     ivectr         i    sequence number of output vector in isolut
!     rinvec         i    Real user input array for subroutine MANVEC
!                         The contents depend on iinvec
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec1, ivec2, nch1, nch2, indprf1, indprf2, &
              ivec, nphys1, nphys2, itypvc, idegfd, indprp1
      logical new
      character ( len=32) name1, name2

!     idegfd         Degree of freedom sequence number
!     indprf1        indication if kprob part f has been filled (>0)
!                    or not (0) (corresponds to vector 1)
!     indprf2        indication if kprob part f has been filled (>0)
!                    or not (0) (corresponds to vector 2)
!     indprp1        Indication if array kprob1 part p has been filled (1)
!                    or not (0)
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     ivec1          Sequence number of first vector
!     ivec2          Sequence number of second vector
!     len            length
!     name1          Name of first vector
!     name2          Name of second vector
!     nch1           Number of characters in name1
!     nch2           Number of characters in name2
!     new            indication whether the output vector is new (true) or not
!                    (false)
!     nphys1         Number of physical unknowns, or maximal degrees of
!                    freedom per point in vector 1
!     nphys2         Number of physical unknowns, or maximal degrees of
!                    freedom per point in vector 2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEP2DVEC       Associate 2d array usoltemp2d to 1d vector
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2103   Length of two input vectors is not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlincomb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlincomb'
         call prinin ( iinvec, iinvec(1), 'iinvec' )
         call prinrl ( rinvec, 2, 'rinvec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate vector invec1

      itypvc = ks(invec1)%typevec
      if ( itypvc ==116 ) then
         nphys1 = ks(invec1)%ivec
      else
         call sepactsolbf2 ( invec1 )
         nphys1 = nphysi
      end if
      indprp1 = indprpi
      indprf1 = indprfi

!     --- Activate vector invec2

      itypvc = ks(invec2)%typevec
      if ( itypvc ==116 ) then
         nphys2 = ks(invec2)%ivec
      else
         call sepactsolbf2 ( invec2 )
         nphys2 = nphysi
      end if
      indprf2 = indprfi
      if ( ierror/=0 ) go to 1000

      idegfd = iinvec(3)
      ivec1 = iinvec(12)
      ivec2 = iinvec(13)

      if ( debug ) then
          write(irefwr,1) 'idegfd, ivec1, ivec2, ivectr', &
                           idegfd, ivec1, ivec2, ivectr
          write(irefwr,1) 'indprf1, nphys1', indprf1, nphys1
          write(irefwr,1) 'indprf2, nphys2', indprf2, nphys2
      end if  ! ( debug )

      if ( ivectr==ivec1 ) then

!     --- Output vector is equal to vector 1

         nphyso = nphys1

      else if ( ivectr==ivec2 ) then

!     --- Output vector is equal to vector 2

         nphyso = nphys2

      else

!     --- Output vector differs from both vectors
!         Check if vector is new or already exists

         new = iresvc<=0 .or. iresvc>lastsol
         if ( .not. new ) new = solarr(iresvc)==0

         if ( debug ) write(irefwr,*) 'new', new

         if ( new ) then

!        --- Vector is new

            iresvc = 0
            if ( nphys2>nphys1 ) then

!           --- Vector 2 is larger than vector 1

               itypvc = ks(invec2)%typevec
               ivec = ks(invec2)%ivec
               iprob = ks(invec2)%iprob

            else

!           --- Vector 1 is larger than vector 2

               itypvc = ks(invec1)%typevec
               ivec = ks(invec1)%ivec
               iprob = ks(invec1)%iprob

            end if  ! ( nphys2>nphys1 )

            if ( debug ) then
               write(irefwr,1) 'nphys1, nphys2', nphys1, nphys2
               write(irefwr,1) 'itypvc, ivec, iprob', &
                                itypvc, ivec, iprob
            end if  ! ( debug )

            call sepcreasolbf1 ( iresvc, itypvc, ivec,'ivec3', &
                                 ks(invec1)%compl )
            usoltemp = 0d0

            if ( debug ) &
               write(irefwr,1) 'nphyso', nphyso

         else

!        --- Vector is old

            call sepcreasolbf1 ( iresvc, itypvc, ivec,'ivec3', &
                                 ks(invec1)%compl )
            nphyso = nphysi

         end if  ! ( new )

      end if  ! ( ivectr==ivec1 )
      if ( ierror/=0 ) go to 1000

      name1 = names(ivec1)
      name2 = names(ivec2)
      nch1 = index(name1,' ')-1
      if ( nch1<=0 ) nch1 = len(name1)
      nch2 = index(name2,' ')-1
      if ( nch2<=0 ) nch2 = len(name2)

      if ( debug ) &
         write(irefwr,*) 'name1 ', name1(1:nch1), &
                         ' name2 ', name2(1:nch2)

      vectin => ks(invec1)%sol
      vectin2 => ks(invec2)%sol
      usoltemp => ks(iresvc)%sol
      call sep2dvec ( vectout, nphyso, npoint )
      vectout2d => usoltemp2d

      if ( nphyso==nphys2 .and. nphys1==1 .and. nphyso>1 .and. &
           indprf1==0 .and. indprf2==0 ) then

!     --- Output vector and vector 2 has nphyso unknowns
!         Vector 1 has exactly 1 degree of freedom per point

         call sep2dvec ( vectin2, nphyso, npoint )
         vectout2d(idegfd,:) = rinvec(2)*usoltemp2d(idegfd,:) + &
                             rinvec(1)*vectin

      else if ( nphyso==nphys1 .and. nphys2==1 .and. nphyso>1 .and. &
                indprf2==0 .and. indprp1==0 ) then

!     --- Output vector and vector 1 has nphyso unknowns
!         Vector 2 has exactly 2 degree of freedom per point

         call sep2dvec ( vectin, nphyso, npoint )
         vectout2d(idegfd,:) = rinvec(1)*usoltemp2d(idegfd,:) + &
                             rinvec(2)*vectin2

      else

!     --- Situation not yet programmed

         call errchr ( name1(1:nch1), 1 )
         call errint ( ivec1, 1 )
         call errchr ( name2(1:nch2), 2 )
         call errint ( ivec2, 3 )
         call errint ( ks(invec1)%nusol, 2 )
         call errint ( ks(invec2)%nusol, 4 )
         call errsub ( 2103, 4, 0, 2 )

      end if  ! ( nphyso==nphys1 .and. nphys2==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlincomb'

      end if  ! ( debug )
      call erclos ( 'prlincomb' )

      end subroutine prlincomb
