      function prmeanarea ( ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 28-03-2015 Allow parallel
!        version  2.0  date 05-06-2014 Extra parameter ichoice
!        version  1.1  date 25-11-2012 Long integers
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the mean area of all elements of highest dimension in mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice
      double precision prmeanarea

!     ichoice        i    Defines which integral must be computed
!                         109:  mean area
!                         117:  volume
!     prmeanarea     o    Computed mean area
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icount, i
      double precision :: som, vsend(2), vrec(2)
      double precision, allocatable :: areas(:)

!     areas          Contains the areas of the elements
!     i              Counting variable
!     icount         Counter to compute number of elements with highest
!                    dimension
!     som            Help parameter to compute a sum
!     vrec           array to store quantities to be received from other nodes
!     vsend          array to store quantities to be sent to other nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRAREAS        Compute areas of all elements of highest dimension
!     PRPARGLSUM     This function performs a global sum of a quantitity
!                    or vector across all processes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmeanarea' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmeanarea'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- allocate work array

      allocate ( areas(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'areas' )

!     --- Compute areas of all elements of highest dimension

      call prareas ( areas )

!     --- Compute sum of these areas and number of elements of highest dimension

      icount = 0  ! counts number of elements
      som = 0d0   ! computes sum of areas

      do i = 1, nelem
         if ( areas(i)>0d0 ) then

!        --- area>0, hence element of highest dimension

            som = som+areas(i)
            icount = icount + 1

         end if  ! ( areas(i)>0d0 )

      end do  ! i = 1, nelem

      if ( ichoice==109 ) then

!     --- Compute mean value

         if ( parallel ) then
             vsend(1) = som
             vsend(2) = icount
             call prparglsum ( vsend, vrec, 2, SPDOUBLE )
             prmeanarea = vrec(1)/vrec(2)

         else
            prmeanarea = som/icount
         end if  ! ( parallel )

      else

!     --- Compute volume

         if ( parallel ) then
            call prparglsum ( som, prmeanarea, 1, SPDOUBLE )
         else
            prmeanarea = som
         end if  ! ( parallel )

      end if  ! ( ichoice==109 )

!     --- deallocate work array

      deallocate ( areas, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'areas' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmeanarea'

      end if  ! ( debug )
      call erclos ( 'prmeanarea' )

      end function prmeanarea
