      subroutine prmoveobstac2 ( iobst, uvel, uphi, rmeshx, coor, &
                                 npoint, rotation_dir, kmeshx, origin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-08-2004 Extension with origin
!        version  1.0  date 07-12-2001
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Move obstacle over a distance u in R^3
!    If necessary also the obstacle is rotated over an angle of uphi radians
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     displacement
!     rotation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, npoint, rotation_dir, kmeshx(12,*)
      double precision uvel(3), uphi, rmeshx(9,*), coor(3,*), origin(3)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iobst          i    Obstacle sequence number
!     kmeshx       i    Integer part of kmesh part x (i = obstacle number)
!                         kmeshx(1,i) = type of obstacle (1=curve, 2=surface)
!                         kmeshx(2,i) = curve/surface number of obstacle
!                         kmeshx(3,i) = nx
!                         kmeshx(4,i) = ny
!                         kmeshx(5,i) = nz
!                         kmeshx(6,i) = mm sequence number of iobgrid
!                         kmeshx(7,i) = starting address volumes corresponding
!                                       to obstacle
!                         kmeshx(8,i) = mm sequence number of kmeshc_iobst
!                         kmeshx(9,i) = mm sequence number of coor_iobst
!                         kmeshx(10,i) = inpelm
!                         kmeshx(11,i) = nelem
!                         kmeshx(12,i) = npoint
!     npoint         i    Number of nodal points
!     origin         i    Contains the coordinates of the origin of the axis of
!                         the obstacle
!     rmeshx      i/o   Real part of kmesh part x (i = obstacle number)
!                         rmeshx(1,i) = xmin
!                         rmeshx(2,i) = xmax
!                         rmeshx(3,i) = ymin
!                         rmeshx(4,i) = ymax
!                         rmeshx(5,i) = zmin
!                         rmeshx(6,i) = zmax
!                         rmeshx(7,i) = dx
!                         rmeshx(8,i) = dy
!                         rmeshx(9,i) = dz
!     rotation_dir   i    Rotation direction
!                         1: rotation in x-y plane
!                         2: rotation in y-z plane
!                         3: rotation in x-z plane
!     uphi           i    rotation angle in radians
!     uvel           i    Contains the velocity vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x, y, sinuphi, cosuphi, rsqmax, rmax, xmin, ymin, &
                       xmax, ymax, zmin, xmean, ymean

!     cosuphi        cosine of u_phi
!     i              Counting variable
!     rmax           maximum of the distance to rotation axis
!     rsqmax         (maximum of the distance to rotation axis)^2
!     sinuphi        sine of u_phi
!     x              x co-ordinate of point
!     xmax           Maximum x-value in plane z=zmin
!     xmean          Average x-value in plane z=zmin
!     xmin           Minimum x-value in plane z=zmin
!     y              y co-ordinate of point
!     ymax           Maximum y-value in plane z=zmin
!     ymean          Average y-value in plane z=zmin
!     ymin           Minimum y-value in plane z=zmin
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2403   The option rotation_plane has been found, but no rotation velocity
!    2663   Incorrect origin of rotation
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Update rmeshx
!     Change coordinates of surface
!     The rotation is performed with respect to a line in z-direction with
!     given origin
!     If the axis of the orientation is skewed we should perform the following
!     steps
!     Define the direction of the axis (t)
!     Make t a unit vector
!     Compute 2 unit vectors q_1 and q_2 perpendicular to t and each other
!     Shift the coordinates with velocity or displacement
!     Map the new coordinates to a system with base vectors t q_1 and q_2
!     The new coordinates (alpha, beta, gamma) are defined by
!     (alpha, beta, gamma) = (t, q_1, q_2 )^T (x-x_0)
!     Rotate in the new coordinate system
!     Map back:
!     x = x_0 + (t, q_1, q_2 ) * (alpha, beta, gamma)^T
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmoveobstac2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmoveobstac2'
         write(irefwr,*) 'iobst, uphi ', iobst, uphi
         call prinrl ( uvel, 3, 'uvel' )
         call prinrl ( origin, 3, 'origin' )
         call prinrl ( rmeshx(1,iobst), 6, 'rmeshx' )
         call prinrl1 ( coor, npoint, 3, 'coor_start' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Compute minimum z-coordinate

      zmin = rinfin
      do i = 1, npoint
         zmin = min ( coor(3,i), zmin )
      end do  ! i = 1, npoint

!     --- Compute minimum x and y-coordinates in plane z = zmin

      xmin = rinfin
      xmax = -xmin
      ymin = rinfin
      ymax = -ymin
      zmin = zmin+sqreps
      do i = 1, npoint
         if ( coor(3,i).lt. zmin ) then
            xmin = min ( coor(1,i), xmin )
            ymin = min ( coor(2,i), ymin )
            xmax = max ( coor(1,i), xmax )
            ymax = max ( coor(2,i), ymax )
         end if  ! ( coor(3,i).lt. zmin )
      end do  ! i = 1, npoint
      xmean = 0.5d0*(xmin+xmax)
      ymean = 0.5d0*(ymin+ymax)
      if ( debug ) write(irefwr,*) 'xmean, ymean ', xmean, ymean

!     --- Change coordinates of surface for displacement

      do i = 1, npoint
         coor(1,i) = coor(1,i)+uvel(1)
         coor(2,i) = coor(2,i)+uvel(2)
         coor(3,i) = coor(3,i)+uvel(3)
      end do  ! i = 1, npoint

!     --- Update rmeshx by a displacement

      rmeshx(1,iobst) = rmeshx(1,iobst)+uvel(1)
      rmeshx(2,iobst) = rmeshx(2,iobst)+uvel(1)
      rmeshx(3,iobst) = rmeshx(3,iobst)+uvel(2)
      rmeshx(4,iobst) = rmeshx(4,iobst)+uvel(2)
      rmeshx(5,iobst) = rmeshx(5,iobst)+uvel(3)
      rmeshx(6,iobst) = rmeshx(6,iobst)+uvel(3)

      if ( rotation_dir.ne.0 .and. abs(uphi).gt.0d0 ) then

!     --- Rotation

         if ( rotation_dir.ne.1 ) then

!        --- rotation_plane only in xy direction available

            call errsub ( 2403, 0, 0, 0 )
            go to 1000

         end if
         if ( abs(origin(1)-xmean).gt.sqreps .or. &
              abs(origin(2)-ymean).gt.sqreps ) then

!        --- origin of obstacle incorrect

            call errint ( iobst, 1 )
            call erreal ( origin(1), 1 )
            call erreal ( origin(2), 2 )
            call erreal ( xmean, 3 )
            call erreal ( ymean, 4 )
            call erreal ( zmin-sqreps, 5 )
            call errwar ( 2663, 1, 5, 0 )

         end if  ! ( abs(orig(1)-xmean).gt.sqreps ...

!        --- u_phi > 0, rotation in x-z plane

         sinuphi = sin(uphi)
         cosuphi = cos(uphi)

!        --- Update coor by a rotation over uphi radians
!            Use sum formula of the sinus

         rsqmax = 0.d00

         do i = 1, npoint
            x = coor(1,i)-origin(1)
            y = coor(2,i)-origin(2)
            rsqmax = max(rsqmax,x**2+y**2)
            coor(1,i) = x*cosuphi-y*sinuphi+origin(1)
            coor(2,i) = x*sinuphi+y*cosuphi+origin(2)
         end do  ! i = 1, npoint
         if ( debug ) write(irefwr,*) 'rmax ', sqrt(rsqmax)

!        --- Update rmeshx for the rotation
!            This code is only correct in case the obstacle is rotated around
!            the z-axis and the obstacle width in x and y direction are
!            approximately equal to the largest width in any xy direction.
!            We do this each time the obstacles are moved, but in this
!            particular case, once would do.
!            rmax is a bit larger then the largest width of the obtacle in
!            an xy direction

         rmax = sqrt(rsqmax)*1.01d0

!        --- xmin, ymin, xmax, ymax, dx, dy of the grid covering the obstacle

         rmeshx(1,iobst) = origin(1)-rmax
         rmeshx(3,iobst) = origin(2)-rmax
         rmeshx(2,iobst) = origin(1)+rmax
         rmeshx(4,iobst) = origin(2)+rmax
         rmeshx(7,iobst) = 2*rmax/kmeshx(3,iobst)
         rmeshx(8,iobst) = 2*rmax/kmeshx(4,iobst)

      end if

1000  call erclos ( 'prmoveobstac2' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( rmeshx(1,iobst), 6, 'rmeshx' )
         call prinrl1 ( coor, npoint, 3, 'coor_end' )
         write(irefwr,*) 'End prmoveobstac2'

      end if

      end
