      subroutine prmshlevms ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 05-11-2013 Extra parameter rinsep
!        version  3.1  date 19-04-2013 Adaptation in case of same mesh
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with a zero levelset crossing the mesh
!     Elements that are intersected are subdivided into several elements,
!     such that these new elements all belong to the levelset or are
!     outside the levelset
!     The problem definition is adapted as well as the matrix structure
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     level_set
!     intersection
!     subdivision
!     matrix_structure
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readsclevmesh for a description
!                         6..5+m:  sequence numbers of these vectors
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readsclevvect for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer meshorig, meshobst, numinterpol, inpcop(4), &
              iinputusr(3), ireadstruc, iseqlevel, i, iseqkm1, iseqkm2
      double precision rinputusr(2)

!                    they are not
!     i              General loop variable
!     iinputusr      input array for subroutine mshobstmsh
!                    For a desciption see mshobstmsh
!     inpcop         Copy of part of integer input inpsep
!     ireadstruc     Sequence number of input for matrix structure
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     iseqlevel      Sequence number of level set function in isol
!     meshobst       Sequence number of levelset mesh to be used
!     meshorig       Sequence number of original mesh to be used
!     numinterpol    number of vectors to be interpolated
!     rinputusr      Is used to store the values of some parameters for
!                    the shift of nodes to the obstacle boundary
!                    1:  eps_move
!                    2:  eps_min
!                    See subroutine mshobstms for an extended description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     INICHINPSOL    Adapt input array with respect to variables
!     MSHOBSTCOPMSH  Delete some parts of new mesh and copy part l from kmesh
!     MSHOBSTMS      Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh
!     PRININ         print 1d integer array
!     PRLEVCOPSOL    Copy solutions from old mesh onto new mesh if possible
!     PROBOBSTNEW    Fill kprob and intmat for new mesh in case of obstacle or
!                    level set
!     SEPCLEAR       clear array map
!     SEPFILLKMADL3  Fill array kmesh ad  part 3: icrossobst,rcrossobst
!                    in case of level set
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute the intersections of the zero value of the level set
!     with the basis mesh
!     If ( no intersections found ) then
!        Copy original mesh to new mesh
!     else
!        Create new mesh
!        Print and plot information if necessary
!        Check mesh if required
!        Copy old kprob to new one and extend this array
!        Fill matrix structure
!        Copy solution from old mesh onto new mesh if possible
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmshlevms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmshlevms'
         call prinin ( inpsep, 9, 'inpsep' )
         call prinin ( meshseqs, 3, 'meshseqs' )
         write(irefwr,1) 'istartmn', istartmn
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Replace scalars by actual values

      inpcop(1:4) = inpsep(1:4)
      call inichinpsol ( inpsep, 4 )
      if ( debug ) call prinin ( inpsep, 4, 'inpsep' )

      meshorig  = meshseqs(inpsep(1))
      meshobst = meshseqs(inpsep(2))
      ireadstruc = inpsep(3)
      iseqlevel = inpsep(4)
      rinputusr(1) = getparm ( inpsep, rinsep, 7, 'eps_move' )
      rinputusr(2) = getparm ( inpsep, rinsep, 9, 'eps_min' )

      numinterpol = 0
      if ( debug ) then
         write(irefwr,1) 'meshorig, meshobst', meshorig, meshobst
         write(irefwr,2) 'epsmove, epsmin', rinputusr
      end if  ! ( debug )

!     --- Compute the intersections of the zero value of the level set
!         with the basis mesh

      kmesh => kmesh2d(meshorig)
      kprob => kprob2d(meshorig)
      isolut => isolut3d(iseqlevel,meshorig)
      call sepfillkmesh ( kmesh )
      iseqkm1 = iseqkm
      call sepfillkmadl3
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'nlevelsets', nlevelsets
         write(irefwr,1) 'kelmad', kelmad
         call prinin ( kmeshad,7, 'kmeshad' )
      end if  ! ( debug )

!     --- Create mesh

      if ( kmeshad(4)==0 ) then

!     --- Special case submesh coincides with original mesh
!         Copy mesh
!         Delete some parts of new mesh and copy part l from kmesh2d

         call mshobstcopmsh ( meshorig, meshobst, map(meshorig,meshobst) )
         map(meshorig,meshobst)%equal = .true.
         iseqkm2 = iseqkm1

      else

!     --- Standard case submesh differs from original mesh

         iinputusr(1) = 3
         iinputusr(2) = inpsep(6)  ! ishift
         iinputusr(3) = 0  ! smooth

!        --- delete all arrays corresponding to map(.,meshobst)

         do i = 1, maxmesh
            if ( i/=meshorig ) then
               if ( associated(map(i,meshobst)%imap)) then
                  deallocate ( map(i,meshobst)%imap, map(i,meshobst)%rmap )
                  call sepclear ( map(i,meshobst) )
               end if  ! ( associated(map(i,meshobst)%imap))
            end if  ! ( i/=meshorig )
            if ( associated(map(meshobst,i)%imap)) then
               deallocate ( map(meshobst,i)%imap, map(meshobst,i)%rmap )
               call sepclear ( map(meshobst,i) )
            end if  ! ( associated(map(meshobst,i)%imap))
         end do  ! i = 1, maxmesh

         if ( debug ) &
            write(irefwr,1) 'meshorig, meshobst', meshorig, meshobst

         call mshobstms ( meshorig, meshobst, map(meshorig,meshobst), &
                          iinputusr, rinputusr )
         iseqkm2 = kmesh2d(meshobst)
         km(iseqkm2)%kmesh(64) = km(iseqkm1)%kmesh(64)

      end if  ! ( kmeshad(4)==0 )
      if ( ierror/=0 ) go to 1000

!     --- Copy old kprob2d to new one and compute new intmat2d

      removesol = .false.
      call probobstnew ( meshorig, meshobst, ireadstruc )
      removesol = .true.
      if ( ierror/=0 ) go to 1000

!     --- Copy solution from old mesh onto new mesh if possible

      call prlevcopsol ( meshorig, meshobst, map(meshorig,meshobst), &
                         iseqlevel, inpsep, numinterpol )

!     --- reset inpsep

      inpsep(1:4) = inpcop(1:4)

      km(iseqkm2)%kmesh(39) = 1  ! indicate that mesh has been adapted

!     --- raise istartmn, see readsclevmesh
!         if this is changed also change prrmlevmesh

1000  istartmn = istartmn+10+max(0,numinterpol)

      if ( debug ) then

!     --- Debug information

         call prinin ( isolut3d(1,meshorig), numvecmn, 'isol_orig' )
         call prinin ( isolut3d(1,meshobst), numvecmn, 'isol_obst' )
         write(irefwr,1) 'istartmn/end', istartmn
         call prinin ( meshseqs, 3, 'meshseqs' )
         write(irefwr,*) 'End prmshlevms'

      end if
      call erclos ( 'prmshlevms' )

      end
