      subroutine prmshobstms ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 15-12-2017 New storage of intmat
!        version  2.1  date 05-11-2013 New call to mshobstms
!        version  2.0  date 10-02-2012 Extra parameter maxmesh
!
!   copyright (c) 2001-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements,
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
!     The problem definition is adapted as well as the matrix structure
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
!     matrix_structure
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduledebug
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         1:  MESH_IN Sequence number of mesh to be refined
!                         2:  MESH_OUT Sequence number of refined mesh
!                         3:  SEQ_STRUCTURE Sequence number of input for
!                             structure of matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer meshorig, meshobst, i, iseqad, &
              iinputusr(3), ireadstruc, inpcop(3)
      double precision rinputusr(2)

!                    they are not
!     i              General loop variable
!     iinputusr      input array for subroutine mshobstmsh
!                    For a desciption see mshobstmsh
!     inpcop         Copy of part of integer input inpsep
!     ireadstruc     Sequence number of input for matrix structure
!     iseqad         Dummy
!     meshobst       Sequence number of obstacle mesh to be used
!     meshorig       Sequence number of original mesh to be used
!     rinputusr      Is used to store the values of some parameters for
!                    the shift of nodes to the obstacle boundary
!                    1:  eps_move
!                    2:  eps_min
!                    See subroutine mshobstms for an extended description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     INIDELMAP      Delete contents of array map
!     MSHOBSTMARKELM Recompute array ielemobst in the sense that only elements
!                    inside or outside the obstacle are marked
!     MSHOBSTMS      Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh
!     PLOTOBSTACLE   Print and plot information about the obstacle
!     PROBOBSTNEW    Fill kprob and intmat for new mesh in case of obstacle or
!                    level set
!     PROBSTCOPSOL   Copy solutions from old mesh onto new mesh if possible
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHAD Fill array kmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmshobstms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmshobstms'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Replace scalars by actual values

      inpcop(1:3) = inpsep(1:3)
      call inichinpsol ( inpsep, 3 )

      meshorig  = meshseqs(inpsep(1))
      meshobst = meshseqs(inpsep(2))
      ireadstruc = inpsep(3)
      call sepfillkmesh ( kmesh2d(meshorig) )

      rinputusr(1) = accuracy_obstacle
      rinputusr(2) = 0.01d0*accuracy_obstacle

!     --- Compute the intersections of the obstacle with the basis mesh

      if ( kelmad==0 ) call sepfillkmeshad(iseqad)
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3
      if ( ierror/=0 ) go to 1000

      if ( kmeshad(4)==0 ) then

!     --- Special case submesh coincides with original mesh

         meshseqs(inpsep(2)) = -meshseqs(inpsep(2))

!        --- Refill array ielemobst: only elements inside the obstacle are
!            marked

         call mshobstmarkelm

!        --- Print and plot information depending on plotobst and printobst

         if ( plotobst .or. printobst ) then

!        --- debug = true, plot nodes and elements with a colour

            call plotobstacle ( debug_mshobstmesh, 0 )

         end if
         go to 1000

      end if

!     --- delete all arrays corresponding to map(.,meshobst)

      do i = 1, maxmesh
         if ( i/=meshorig ) then
            if ( associated(map(i,meshobst)%imap)) then
               deallocate ( map(i,meshobst)%imap, map(i,meshobst)%rmap )
               call sepclear ( map(i,meshobst) )
            end if  ! ( associated(map(i,meshobst)%imap))
         end if  ! ( i/=meshorig )
         if ( associated(map(meshobst,i)%imap)) then
            deallocate ( map(meshobst,i)%imap, map(meshobst,i)%rmap )
            call sepclear ( map(meshobst,i) )
         end if  ! ( associated(map(meshobst,i)%imap))
      end do  ! i = 1, maxmesh

      iinputusr(1) = 3
      iinputusr(2) = 0   ! ishift
      iinputusr(3) = 0
      call mshobstms ( meshorig, meshobst, map(meshorig,meshobst), &
                       iinputusr, rinputusr )

!     --- Copy old kprob to new one and compute new intmat

      call probobstnew ( meshorig, meshobst, ireadstruc )
      if ( ierror/=0 ) go to 1000

!     --- Copy solution from old mesh onto new mesh if possible

      call probstcopsol ( meshorig, meshobst, map(meshorig,meshobst) )

!     --- reset inpsep

      inpsep(1:3) = inpcop(1:3)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmshobstms'

      end if  ! ( debug )
      call erclos ( 'prmshobstms' )

      end
