      subroutine prplotln ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 09-01-2015 Complete new subroutine
!        version  1.2  date 06-02-2012 Allow indexed vectors
!        version  1.1  date 03-05-2011 Jump for certain time steps
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot function along a straight intersection of a 2d mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     plot_function
!     driving_subroutine
!     plot
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotint
!     rinsep         i    The real input read is stored in array RINSEP
!                         See subroutine readscplotint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nnodes, numsubcurvs, input(2), ivectr, &
                 icross(5,20), numvecs
      double precision, allocatable :: funcx(:), funcy(:,:)
      integer, allocatable :: colours(:)
      character (len=80) :: textaxis(2), textleg(20)

!     colours        Contains the sequence numbers of the colors to be plotted
!     funcx          array containing the "x"-coordinates
!     funcy          array containing the function values
!                    for the numvec components
!     i              Counting variable
!     icross         Integer SEPRAN array of type 119 containing
!                    the information of the intersections
!     input          Work array to store input for subroutine plotfunc2
!     ivectr         sequence number in isolut2d, of vector to be plotted
!     nnodes         Number of nodes
!     numsubcurvs    Number of subcurves
!                    At this moment only 1 subcurve is allowed
!     numvecs        Number of vectors in icross
!     textaxis       Contains the text to be plotted along the axis
!     textleg        Contains the texts to be plotted in the legenda
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     PLOTFUNC2      Plot one or more functions in a function plot
!     PRFILLFUNCIN   Fill arrays funcx and funcy with values that are stored in
!                    array icross of type 119
!     PRFILLTEXTINT  Fill array textaxis with text to be plotted along the axis
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINTERSECT2D  Intersect a 2d mesh by a straight line and store the values
!                    in array icross of type 119
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotln' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotln'
         call prinin ( inpsep, 8, 'inpsep' )
         call prinin1 ( isolut2d, inpsep(1), 5, 'isolut2d' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( itimloop/=0 .and. ioutact/=1 ) go to 900 ! Only at certain time steps

!     --- Get parameters

      ivectr = inivectr(inpsep(1))   ! sequence number of vector in isolut2d

!     --- Compute intersection

      isolut => isolut2d(ivectr)
      call printersect2d ( inpsep, rinsep, icross, &
                           numsubcurvs, nnodes, numvecs )
      if ( ierror/=0 ) go to 1000

!     --- Allocate arrays funcx and funcy

      allocate ( funcx(nnodes), funcy(nnodes,numvecs), colours(numvecs), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nnodes*(numvecs+1), 'funcx' )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays funcx and funcy and remove contents of array icross

      call prfillfuncin ( icross, numsubcurvs, nnodes, numvecs, funcx, funcy )

!     --- Fill text along axis

      call prfilltextint ( textaxis, inpsep(6), ivectr )

!     --- Fill text legenda

      textleg(1:numvecs) = names(ivectr)

!     --- plot

      do i = 1, numvecs
         colours(i) = 1+5*i
      end do  ! i = 1, numvecs
      input(1) = numvecs  ! ncolors
      if ( numvecs==1 ) then
         input(2) = 0  ! nlegenda
      else
         input(2) = numvecs  ! nlegenda
      end if  ! ( numvecs==1 )

      call plotfunc2 ( nnodes, funcx, funcy, numvecs, &
                       textaxis, input, colours, textleg )

!     --- deallocate funcx

      deallocate ( funcx, funcy, colours, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'funcx' )

!     --- Raise istartmn

900   istartmn = istartmn+12+inpsep(10)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotln'

      end if  ! ( debug )
      call erclos ( 'prplotln' )

      end subroutine prplotln
