      subroutine prrefine ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.3  date 06-06-2014 Use refinenew
!        version  3.2  date 04-09-2013 Allow meshin = meshout for locrefine
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine an existing mesh, compute the corresponding problem definition
!     and create the corresponding matrix structure
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     matrix_structure
!     refine
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemeshinf
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         1:  N (Number of refinements)
!                         2:  MESH_IN Sequence number of mesh to be refined
!                         3:  MESH_OUT Sequence number of refined mesh
!                         4:  sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: niinpref=10, nrinpref=1
      integer meshin, meshout, nrefine, itypvc, &
              isolsav, idum(1), i, localrefine, iinputref(niinpref)
      integer, pointer :: kmeshout, kprobout, isolutout(:), &
                          kprobsav, kmeshsav
      double precision rdum(1), rinputref(nrinpref)
      type (mapping), pointer :: map1d

!                    they are not
!     i              Counting variable
!     idum           General dummy variable
!     iinputref      Actual input array created by merging input from
!                    the input block refine and the structure block
!     isolsav        Help array to store the contents of isol temporarily
!     isolutout      Reference to meshout for isol
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     kmeshout       Reference to meshout for kmesh
!     kmeshsav       Help array to store array kmesh temporarily
!     kprobout       Reference to meshout for kprob
!     kprobsav       Help array to store array kprob temporarily
!     localrefine    local refinement (0 = no, 1 = yes)
!     map1d          reference to part of map
!     meshin         Sequence number of first mesh to be used
!     meshout        Sequence number of last mesh to be used
!     niinpref       Declared length of array iinputref
!     nrefine        Number of refinements
!     nrinpref       Real input array
!     rdum           dummy variable
!     rinputref      Declared length of array rinputref
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOPY        Copy important information of mesh
!     MSHCOPYMESH    Copy mesh from one mesh array into the other and remove
!                    old mesh
!     PRCALLCONTCT   Call contact (prconct) for all input parts with respect
!                    to contact
!     PROBSTRUC      Create problem definition and matrix structure for new mesh
!     PROCOPY        Copy important information of problem definition
!     PRREFINEREAD   Combine input  read from structure block and refine block
!                    into one input array
!     REFINELOC      Refine an existing mesh locally
!     REFINENEW      Refine mesh
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPINTPOLMS    Interpolate solution from one mesh to another
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data isolsav / 0 /
! ======================================================================
!
      call eropen ( 'prrefine' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prrefine'
         write(irefwr,1) 'maxmesh', maxmesh
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
!     --- Get input from arrays inpsep and from the standard
!         input for refine

      call prrefineread ( iinputref, rinputref, &
                          niinpref, nrinpref, inpsep )

      nrefine = iinputref(1)
      meshin  = meshseqs(iinputref(2))
      meshout = meshseqs(iinputref(3))
      localrefine = iinputref(4)
      kmesh => kmesh2d(meshin)
      kmeshout => kmesh2d(meshout)
      kprob => kprob2d(meshin)
      kprobout => kprob2d(meshout)
      isolut2d => isolut3d(:,meshin)

      isolutout => isolut3d(:,meshout)
      map1d => map(meshin,meshout)
      kmeshsav => kmesh2d(maxmesh+1)
      kprobsav => kprob2d(maxmesh+1)

      if ( debug ) &
         write(irefwr,1) 'nrefine, meshin, meshout, localrefine', &
                          nrefine, meshin, meshout, localrefine

      if ( meshin/=meshout ) then

!     --- Input mesh and output mesh are not the same

         call mshcopy ( kmesh, kmeshout )
         call procopy ( kprob, kprobout )

      else

!     --- input mesh and output mesh are the same

         call mshcopy ( kmesh, kmeshsav )
         call procopy ( kprob, kprobsav )

         removesol = .false.

      end if
      map1d%iseqkm1 = kmesh

!     --- Refine mesh

      if ( debug ) &
         write(irefwr,1) 'localrefine, map1d%iseqkm1', &
                          localrefine, map1d%iseqkm1

      if ( localrefine==0 ) then

!     --- localrefine = 0, global refinement

         if ( ioutp>=0 ) write(irefwr,200) meshin, nrefine, meshout
200      format( ' Refine mesh ', i1, i2,' times and store in mesh', &
                 i2 )
         kmesh => kmesh2d(meshout)
         call sepfillkmesh(kmesh)
         call refinenew ( nrefine,  idum, rdum, .false., map1d, .true. )
         iinmap(2) = 2  ! array map has not been filled

      else

!     --- localrefine = 1, local refinement

         if ( ioutp>=0 ) write(irefwr,210) meshin, meshout
210      format( ' Refine mesh ', i1, ' locally and store in mesh', &
                 i2 )
         iinputref(4) = 3  ! temporary, number of elements filled

         if ( meshout==meshin ) then

!        --- meshin and meshout are the same
!            use temporary array kmeshout to store the new mesh

            kmesh2d(maxmesh+1) = 0
            call refineloc ( meshin, meshout, isolut2d, iinputref(4), map1d )

!           --- Copy kmeshnew into kmesh(.,meshin)
!               Old buffer entries are removed

            call mshcopymesh ( kmesh, kmesh2d(maxmesh+1) )

         else

!        --- meshin and meshout are different, standard case

            call refineloc ( meshin, meshout, isolut2d, iinputref(4), map1d )

         end if  ! ( meshout==meshin )

         iinputref(4) = localrefine  ! reset to original value
         iinmap(2) = 2  ! array map has been filled

      end if  ! ( localrefine==0 )

!     --- Fill arrays kprob and intmat for output mesh
!         meshnr is set to meshout


      removeintmat = .false.
      call probstruc ( meshin, meshout )
      kmesh => kmesh2d(meshin)
      kprob => kprob2d(meshin)
      call sepfillkmesh(kmesh)
      removesol = .true.
      removeintmat = .true.

      if ( meshin==meshout ) then

!     --- meshin = meshout, reset iseqkp for all vectors

         do i = 1, lastsol
            if ( solarr(i)==1 ) then
               if ( ks(i)%iseqkp==-1 ) ks(i)%iseqkp = kprob
            end if  ! ( solarr(i)==1 ) then
         end do  ! i = 1, lastsol

      end if  ! ( meshin==meshout )

!     --- Copy solution from old mesh onto new mesh if possible

      if ( meshin/=meshout ) then

!     --- Input mesh and output mesh are not the same

         do i = 1, numvecmn
            if ( isolut2d(i)>0 ) then
               itypvc = ks(isolut2d(i))%typevec
               if (itypvc==110 .or. itypvc==115) then
                  call sepintpolms ( meshin, meshout, iinmap,  &
                                     isolut2d(i), isolutout(i), &
                                     map1d, idum )
                  iinmap(2) = 2
                  iinmap(4) = meshin
                  iinmap(5) = meshout
               end if
            end if  ! ( isolut2d(i)>0 )
         end do

      else

!     --- input mesh and output mesh are the same

         map1d%iseqkm1 = kmeshsav

         do i = 1, numvecmn
            if ( isolut2d(i)>0 ) then
               itypvc = ks(isolut2d(i))%typevec
               if (itypvc==110 .or. itypvc==115) then
                  ks(isolut2d(i))%iseqkp = kprobsav
                  call sepcopyvc ( isolut2d(i), isolsav )
                  kmesh => kmesh2d(meshin)
                  kprob => kprob2d(meshin)
                  call sepintpolms ( maxmesh+1, meshin, iinmap, &
                                     isolsav, isolutout(i), map1d, idum )
                  iinmap(2) = 2
                  iinmap(4) = meshin
                  iinmap(5) = meshout
               end if
            end if  ! ( isolut2d(i)>0 )
         end do
         if ( isolsav>0 ) then
            call seprmmeshinfo ( kmeshsav )
            call seprmprobinfo ( kprobsav )
         end if  ! ( isolsav>0 )

      end if
      meshnr = meshout

!     --- Activate contact region if necessary

      kmesh => kmesh2d(meshout)
      kprob => kprob2d(meshout)
      iseqkp = kprob
      iprob = 1
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      intmat1d => intmat2d(:,meshout)
      isolut2d => isolut3d(:,meshout)

      call sepfillkmesh(kmesh)

      call prcallcontct

      istartmn = istartmn+5  ! raise istart

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prrefine'

      end if
      call erclos ( 'prrefine' )

      end subroutine prrefine
