      subroutine prrmobstmesh ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 13-12-2017 New storage of intmat
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 10-02-2012 Extra parameter maxmesh
!
!   copyright (c) 2001-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete the obstacle mesh created for an obstacle crossing the mesh
!     Also delete the corresponding problem definition arrays and
!     matrix structure arrays.
!     First the solution is copied from the obstacle mesh to the original mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     delete
!     matrix_structure
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemeshinf
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         1:  MESH_ORIG Sequence number of mesh to be adapted
!                         2:  MESH_OBST Sequence number of adapted mesh
!                         3:  SEQ_STRUCTURE (not used)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer meshorig, meshobst, i, npointlc, icurvs(1), inpcop(3), &
              iseqkm1, iseqkm2, itypvc

!                    they are not
!     i              Counting variable
!     icurvs         Dummy
!     inpcop         Copy of part of integer input inpsep
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     meshobst       Sequence number of obstacle mesh to be used
!     meshorig       Sequence number of original mesh to be used
!     npointlc       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     PRININ         print 1d integer array
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPINTPOLMS    Interpolate solution from one mesh to another
!     SEPRMINTMT     Remove array with sequence number iseq from intmt
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prrmobstmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prrmobstmesh'
         write(irefwr,1) 'meshnr, numvecmn', meshnr, numvecmn
         call prinin ( inpsep, 2, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Replace scalars by actual values

      inpcop(1:3) = inpsep(1:3)
      call inichinpsol ( inpsep, 3 )

!     --- Copy solution from new mesh onto old mesh if possible

      meshorig = meshseqs(inpsep(1))
      meshobst = meshseqs(inpsep(2))

      if ( debug ) write(irefwr,1) 'meshorig, meshobst', meshorig, meshobst

      if ( meshobst<0 ) then

!     --- Special case submesh coincides with original mesh

         meshseqs(inpsep(2)) = -meshseqs(inpsep(2))
         go to 1000

      end if

      iseqkm1 = kmesh2d(meshorig)
      iseqkm2 = kmesh2d(meshobst)

      if ( debug ) write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2

!     --- First array imap and rmap must be redefined

      imap => map(meshorig,meshobst)%imap
      npointlc = km(iseqkm1)%kmesh(8)
      do i = 1, npointlc
         imap(i,2) = i
      end do  ! i = 1, npointlc
      map(meshorig,meshobst)%iseqkm1 = iseqkm2
      map(meshorig,meshobst)%iseqkm2 = iseqkm1
      do i = 1, numvecmn
         if ( debug ) write(irefwr,*) 'Interpolate vector ', i
         if ( isolut3d(i,meshobst)>0 ) then
            itypvc = ks(isolut3d(i,meshobst))%typevec
            if (itypvc==110 .or. itypvc==115) then
               call sepintpolms ( meshobst, meshorig, iinmap, &
                                  isolut3d(i,meshobst), &
                                  isolut3d(i,meshorig), &
                                  map(meshorig,meshobst), icurvs )
               call seprmsolinfo ( isolut3d(i,meshobst) )
            end if
         end if  ! ( isolut3d(i,meshobst)>0 )
      end do
      meshnr = meshorig

!     --- Delete obstacle mesh

      call seprmmeshinfo ( kmesh2d(meshobst) )

!     --- Delete obstacle problem definition

      removesol = .false.
      call seprmprobinfo ( kprob2d(meshobst) )
      removesol = .true.
      kprob => kprob2d(meshorig)

!     --- Delete obstacle matrix structure

      do i = 1, nprob
         if ( intmat2d(i,meshobst)>0 .and. &
              intmat2d(i,meshobst)<=lastintmat ) &
            call seprmintmt ( intmat2d(i,meshobst) )
      end do ! i = 1, nprob

!     --- reset inpsep

      inpsep(1:3) = inpcop(1:3)

      kmesh => kmesh2d(meshorig)
      call sepfillkmesh ( kmesh )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prrmobstmesh'

      end if
      call erclos ( 'prrmobstmesh' )

      istartmn = istartmn + 3

      end
