      subroutine prsetlevsetzer ( kmesh, isol, ilevel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 03-01-2012 Use prfillnodes
!        version  1.0  date 25-07-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set levelset function equal to zero at interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulevecs
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh, isol, ilevel

!     ilevel         i    levelset sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nnodes, inpsep(1), npoint, i, nparts, ipoint(1), iseqkm2

!     i              Counting variable
!     inpsep         The integer input read is stored in array INPSEP
!     ipoint         Dummy
!     iseqkm2        Sequence number with respect to km of second mesh
!     nnodes         Number of nodes
!     nparts         dummy
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2645   The number of degrees of freedom corresponding to the level set
!           function is not equal to 1 in all points.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsetlevsetzer' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsetlevsetzer'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get constants

      iseqkm2 = kmesh
      npoint = km(iseqkm)%kmesh(8)

!     --- allocate work arrays

      allocate ( nodes(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

!     --- fill array nodes with nodes at interface

      inpsep(1) = ilevel
      call sepfillnodes ( 20, 0, inpsep, &
                          nodes, nnodes, ipoint, nparts )

!     --- activate solution array

      call sepactsolbf2 ( isol )

      if ( indprfi>0 .or. nunkpi/=1 .or. nphysi/=1 ) then

!     --- Either we have indprfi>0, i.e. number of unknowns per node
!         not constant or nunkpi # 1, nphysi#1, which means not one unknown per
!         node. This can not be a level set function

         call errint ( indprfi, 1 )
         call errint ( nunkpi, 2 )
         call errint ( nphysi, 3 )
         call errsub ( 2645, 3, 0, 0 )

      end if

!     --- make array equal to zero at interface

      usoltemp => ks(isol)%sol
      usoltemp(nodes(1:nnodes)) = 0d0
      if ( ierror/=0 ) go to 1000

      deallocate ( nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsetlevsetzer'

      end if  ! ( debug )
      call erclos ( 'prsetlevsetzer' )

      end subroutine prsetlevsetzer
