      subroutine prstoc ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve stochastic investment problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)
      double precision :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscstoc
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscstoc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idim, istrin, numvcs, iseqreac

!                    they are not
!     idim           dimension of the specific problem
!     iseqreac       Sequence number of reaction vector in array ISOL
!                    If <0 also the right-hand side contribution is added
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     numvcs         Number of vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRSTOC1        Solve 1d stochastic investment problem
!     PRSTOC2        Solve 2d stochastic investment problem
!     READSTOC       Read parameters for stochastic investment problem and store
!                    in common cstoc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3034   idim has wrong value
!    3035   numvec has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create structure for arry psi and initialize psi
!     Mark all nodes on start curve
!     While ( array psi has not been filled completely ) do
!        Find all neighbor elements in free surface that are not marked
!        Compute psi in these elements
!        Mark nodes in these elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstoc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstoc'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ioutpsav = ioutp    ! save ioutp
      ioutp = -1          ! suppress output

!     --- Get constants from inpsep

      numvcs  = inpsep(1)  ! Number of vectors
      idim    = inpsep(3)  ! dimension of problem
      istrin  = inpsep(4)  ! starting address in rinsep
      iseqreac = inpsep(5) ! reaction force
      if ( debug ) then
         write(irefwr,1) 'numvcs, idim, istrin', &
                          numvcs, idim, istrin
         call prinin ( inpsep(11), numvcs, 'seq numbers of vectors' )
         call prinin ( inpsep(11+numvcs), numvcs, 'seq boun conds' )
         call prinin ( inpsep(11+2*numvcs), numvcs, 'seq coeffs' )
         call prinin ( inpsep(11+3*numvcs), numvcs-1, 'user points' )
      end if  ! ( debug )

!     --- Read information of coefficients into common block cstoc

      call readstoc ( ndim )

      if ( idim==1 .and. numvcs==2 ) then

!     --- idim = 1, solve one-dimensional problem
!         numvcs = 2, one decision problem

         call prstoc1 ( inpsep, rinsep(istrin), &
                        inpsep(11), inpsep(11+numvcs), &
                        inpsep(11+2*numvcs), iseqreac )

      else if ( idim==2 .and. numvcs==2 ) then

!     --- idim = 2, solve two-dimensional problem
!         numvcs = 2, one decision problem

         call prstoc2 ( inpsep, rinsep(istrin), &
                        inpsep(11), inpsep(11+numvcs), &
                        inpsep(11+2*numvcs), iseqreac )

      else

!     --- error: idim # 1 or numvcs# 2

         if ( idim<1 .or. idim>1 ) then

!        --- idim#1

            call errint ( idim, 1 )
            call errint ( 1, 2 )
            call errint ( 1, 3 )
            call errsub ( 3034, 3, 0, 0 )

         end if  ! ( idim<1 .or. idim>1 )

         if ( numvcs<1 .or. numvcs>1 ) then

!        --- numvcs#1

            call errint ( numvcs, 1 )
            call errint ( 1, 2 )
            call errint ( 1, 3 )
            call errsub ( 3035, 3, 0, 0 )

         end if  ! ( numvcs<1 .or. numvcs>1 )

      end if  ! ( idim==1 .and. numvcs==2)

!     --- Raise istartmn

      istartmn = istartmn+11+numvcs*3

      ioutp = ioutpsav    ! reset ioutp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstoc'

      end if  ! ( debug )
      call erclos ( 'prstoc' )

      end
