      subroutine prstoc2 ( inpsep, rinsep, &
                           iseqvec, iseqboun, iseqcoeff, iseqreac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 03-01-2012 Use prfillnodes
!        version  1.0  date 12-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve 2d stochastic investment problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), iseqvec(2), &
              iseqboun(2), iseqcoeff(2), iseqreac
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscstoc
!     iseqboun       i    Array containing sequence numbers of input for
!                         boundary conditions
!     iseqcoeff      i    Array containing sequence numbers of input for
!                         coefficients
!     iseqreac       i    Sequence number of reaction vector in array ISOL
!                         If <0 also the right-hand side contribution is added
!     iseqvec        i    Array containing sequence numbers of input for
!                         vectors
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscstoc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxiter, iseqvel, iseqdiff, iprobw0, &
              iter, k, iprint, iprobw1, &
              numelgrpw0, numelgrpw1, isubr, &
              iseqw0, iseqw1, numsurw0, numsurw1,&
              numcurves, numnodes, nparts
      integer, allocatable :: elgrpw0(:), elgrpw1(:), &
                              surfsw0(:), surfsw1(:), curves(:), &
                              ipoint(:)
      double precision diff, eps, investment

!     curves         Contains curves of free boundary
!                    they are not
!     diff           Help parameter to compute the difference of two
!                    parameters
!     elgrpw0        Contains the element groups for which w0 is defined
!     elgrpw1        Contains the element groups for which w1 is defined
!     eps            Local accuracy
!     investment     Defines the investment
!     ipoint         dummy
!     iprint         If 1 information must be printed
!     iprobw0        problem number for w0
!     iprobw1        problem number for w1
!     iseqdiff       sequence number of difference vector in isolut2d
!     iseqvel        sequence number of velocity in isolut2d
!     iseqw0         sequence number of w0 in isolut2d
!     iseqw1         sequence number of w1 in isolut2d
!     isubr          Indicates the type of calling subroutine for er0034
!                    Possible values
!                    1:  CREATE
!                    2:  PRESDF/TM
!                    3:  CREATEN
!                    4:  presbc
!     iter           Iteration sequence number
!     k              Counting variable
!     maxiter        maximum number of iterations
!     nparts         dummy
!     numcurves      Number of curves in array curves
!     numelgrpw0     Number of element groups stored in array elgrpw0
!     numelgrpw1     Number of element groups stored in array elgrpw1
!     numnodes       Number of nodal points in array nodes
!     numsurw0       Number of surfaces stored in array surfsw0
!     numsurw1       Number of surfaces stored in array surfsw1
!     surfsw0        Contains the surfaces on which w0 is defined
!     surfsw1        Contains the surfaces on which w1 is defined
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0034         Performs the actual tasks for CREAT and PRESDF
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIELGRPPROB   Find all active element groups for problem iprob
!     INIELGRPSURFS  Find surface numbers corresponding to the element groups
!                    stored in ielgrps
!     INISURFCURVS   Find curves that is on the interface of two sets of
!                    surfaces
!     MSHADAPSURF    Adapt coordinates of mesh and coordinates of curves
!     PRININ         print 1d integer array
!     PRINTVCCURV    Print the values of a vector along the curve icurnr
!     PRSTOCCOMPVEL  Compute the velocity vector along the free surface
!     PRSTOCDIFFVEC  Compute difference vector v_diff = w1-w0-investment
!                    at free surface
!     PRSTOCREAC2    Compute reaction force for stochastic investment
!     PRSTOCSOLVE    Solve linear system for stochastic investment problem
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
!    3040   No convergence reached in maxiter iterations
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill essential boundary conditions in vectors
!     iter := 0
!     diff := 1
!     While ( diff>eps and iter<maxiter ) do
!        w1 := solution linear system
!        Compute flux in free boundary
!        w0 := solution linear system
!        diff_vec := value(w1)-value(w0) - investment at free curve
!        velocity := -(diff_ve/dwdx,diff_ve/dwdy)
!        Move free boundary in normal direction using velocity
!        Adapt mesh
!        diff := ||diff_vec||
!        iter := iter+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstoc2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstoc2'
         call prinin ( iseqvec, 2, 'iseqvec' )
         call prinin ( iseqboun, 2, 'iseqboun' )
         call prinin ( iseqcoeff, 2, 'iseqcoeff' )
         write(irefwr,1) 'iseqreac', iseqreac
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxiter = inpsep(2)    ! maximum number of iterations
      iprint = inpsep(6)     ! print level
      eps = rinsep(1)        ! accuracy
      investment = rinsep(2) ! investment
      if ( iprint>2 ) debug = .false.

      if ( debug ) then
         write(irefwr,1) 'maxiter, iprint', maxiter, iprint
         write(irefwr,2) 'eps, investment', eps, investment
      end if  ! ( debug )

!     --- allocate space

      allocate ( elgrpw0(nelgrp), elgrpw1(nelgrp), nodes(npoint), &
                 surfsw0(nsurfs), surfsw1(nsurfs), curves(ncurvs), &
                 ipoint(ncurvs+1), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 2*nelgrp+2*ncurvs, 1 )
         call errint ( error, 2 )
         call errchr ( 'elgrpw0', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- Fill essential boundary conditions in vectors

      do k = 1, 2
         isubr  = 6+100*(iseqvec(k)-1)
         call er0034 ( isubr, 0, isolut2d, iseqboun(k), isolut2d, 100 )
         numvecmn = max(numvecmn,isubr)
      end do  ! k = 1, 2
      iseqvel = numvecmn+1    ! sequence number of velocity vector (temporary)
      iseqdiff = numvecmn+2   ! sequence number of difference vector (temporary)

!     --- Find element groups corresponding to w0 and w1
!         Store in elgrpw0 and elgrpw1

      iseqw0 = iseqvec(2)
      iseqw1 = iseqvec(1)
      iprobw0 = ks(isolut2d(iseqw0))%iprob
      iprobw1 = ks(isolut2d(iseqw1))%iprob
      iprob = iprobw0
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobc => kprobpart%kprobc
      call sepfillkprob

      call inielgrpprob ( numelgrpw0, elgrpw0 )
      iprob = iprobw1
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkprob
      call inielgrpprob ( numelgrpw1, elgrpw1 )

      if ( debug ) then
         call prinin ( isolut2d(iseqw1), 5, 'isol_w1' )
         call prinin ( isolut2d(iseqw0), 5, 'isol_w0' )
         write(irefwr,1) 'iseqw0, iseqw1', iseqw0, iseqw1
         write(irefwr,1) 'iprobw0, iprobw1', iprobw0, iprobw1
         call prinin ( elgrpw0, numelgrpw0, 'elgrps for w0' )
         call prinin ( elgrpw1, numelgrpw1, 'elgrps for w1' )
      end if  ! ( debug )

!     --- Find surfaces corresponding to element groups

      call inielgrpsurfs ( iinputkm, elgrpw0, numelgrpw0, &
                           surfsw0, numsurw0 )
      call inielgrpsurfs ( iinputkm, elgrpw1, numelgrpw1, &
                           surfsw1, numsurw1 )
      if ( ierror/=0 ) go to 900

      if ( debug ) then
         call prinin ( surfsw0, numsurw0, 'surfaces for w0' )
         call prinin ( surfsw1, numsurw1, 'surfaces for w1' )
      end if  ! ( debug )

!     --- Find curve numbers on the interface

      call inisurfcurvs ( surfsw0, numsurw0, surfsw1, numsurw1, &
                          curves, numcurves )
      if ( debug ) &
         call prinin ( curves, numcurves, 'curves at interface' )
      if ( ierror/=0 ) go to 900

!     --- Store node numbers of free surface in array nodes

      call sepfillnodes ( 2, numcurves, curves, &
                          nodes, numnodes, ipoint, nparts )

!     --- iter := 0
!         diff := 1

      iter = 0
      diff = 1d0
      if ( iprint>=2 ) write(irefwr,100)
100   format ( '  iter     diff' )

!     --- While ( diff>eps and iter<maxiter ) do

      do while ( diff>eps .and. iter<maxiter )

!     --- w1 := solution linear system

         call prstocsolve ( iseqcoeff(1), iprobw1, iseqw1 )
         if ( debug ) then
            call printvccurv ( isolut2d(iseqw1), curves, 'w1' )
         end if  ! ( debug )
         if ( ierror/=0 ) go to 900

!        --- Compute flux in free boundary

         call prstocreac2 ( iseqw1, iseqcoeff(1), iseqreac, &
                            elgrpw0, numelgrpw0, curves, numcurves )
         if ( debug ) &
            call printvccurv ( isolut2d(iseqreac), curves, 'reac' )
         if ( ierror/=0 ) go to 900

!        --- w0 := solution linear system

         call prstocsolve ( iseqcoeff(2), iprobw0, iseqw0 )
         if ( debug ) &
            call printvccurv ( isolut2d(iseqw0), curves, 'w0' )
         if ( ierror/=0 ) go to 900

!        --- diff_vec := value(w1)-value(w0) - investment at free curve

         call prstocdiffvec ( iseqw1, iseqw0, iseqdiff, &
                              investment, nodes, numnodes )
         if ( debug ) &
            call printvccurv ( isolut2d(iseqdiff), curves, 'diff vector' )

!        --- velocity := -(diff_vec/dwdx,diff_vec/dwdy)

         call prstoccompvel ( iseqvel, iseqdiff, iseqw1, numnodes, curves )
         if ( debug ) &
            call printvccurv ( isolut2d(iseqvel), curves, &
                               'velocity vector' )

!        --- Move free boundary in normal direction using velocity
!            Adapt mesh

         call mshadapsurf ( curves, numcurves, iseqvel )

!        --- diff := ||diff_vec||
!            iter := iter+1

         usoltemp => ks(isolut2d(iseqdiff))%sol
         diff = maxval(abs(usoltemp))
         iter = iter+1
         if (iprint>=2 ) then

!        --- iprint = 2, maximal output

            write(irefwr,200) iter, diff
200         format(i6, d12.3)

         end if  ! (iprint>=2 )

      end do  ! while ( diff>eps .and. iter<maxiter )

!     --- Remove arrays diff_vec and velocity

      call seprmsolinfo ( isolut2d(iseqvel) )
      call seprmsolinfo ( isolut2d(iseqdiff) )

      if ( diff>eps ) then

!     --- Error, accuracy not reached, but number of iterations too large

         call errint ( 2, 1 )
         call errint ( iter, 2 )
         call erreal ( diff, 1 )
         call errsub ( 3040, 2, 1, 0 )
         go to 900

      end if  ! ( diff>eps )

900   deallocate ( elgrpw0, elgrpw1, surfsw0, surfsw1, curves, nodes, &
                   ipoint, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'elgrpw0', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstoc2'

      end if  ! ( debug )
      call erclos ( 'prstoc2' )

      end subroutine prstoc2
