      subroutine prtecplot0 ( iref, coor, ndim, npoint, nelem, &
                              npelm, kmeshc, usol, nunkp, ifirst, isol, &
                              maxvec1 )
! ======================================================================
!
!        programmer    Gert_jan Pieters
!        version  2.0  date 09-04-2014 Extra parameters
!        version  1.1  date 16-02-2005 Correction hexahedrons
!        version  1.0  date 01-02-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution array or array of special structure to file "iref"
!     This possibility is meant for a SEPRAN-Tecplot interface
!     The array should already have been mapped on a triangular mesh or a
!     mesh consisting of tetrahedrons only
!     It is assumed that the solution vector contains only scalars in each
!     point or vectors
!     There are no points to be skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     tecplot
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iref, ndim, npoint, nelem, nunkp, npelm, &
                             kmeshc(npelm,nelem), ifirst, isol(5,*), &
                             maxvec1
      double precision, intent(in) :: coor(ndim,npoint), &
                                      usol(npoint,nunkp)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ifirst         i    If ifirst = 0, the file is new and all information
!                         must be stored.
!                         If ifirst = 1, aready some information has been
!                         stored in the file and mesh information is not
!                         repeated
!     iref           i    File reference number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     maxvec1        i    Maximum number of vectors
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
!     npelm          i    Number of points per element
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, lenvar
      character ( len=1200) :: varline

!     i              Counting variable
!     j              Counting variable
!     lenvar         Length of text in varline
!     varline        Is sued to store the line with variables
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRTECPLOT1     Fill string varline with the containing variables
! **********************************************************************
!
!                       I/O
!
!    This subroutine writes the co-ordinates, the topology and the
!    solution to the file indicated by iref.
!    This file is in standard formatted Tecplot format
!    Shape of the file:                               meaning
!    VARIABLES="X" "Y" "VAR1" "VAR2"
!    ZONE N=npoint, E=nelem, F=FEPOINT, ET=TRIANGLE   1. dimensions, format
!    x1 y1 z1 u1,1 u1,2 u1,3                          2. co-ordinates and data
!    x2 y2 z2 u2,1 u2,2 u2,3l                            values
!          .
!          .
!          .
!    i1 i2 i3 ...                                     3. node connectivity list
!    i1 i2 i3 ...
!          .
!          .
!          .
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Write dimension parameters to file
!   Write co-ordinates and solution to file
!   Write topology to file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Start with header

      if ( ifirst==0 ) then

!     --- write header for starting time

         call prtecplot1 ( isol, ndim, varline, &
                           lenvar, maxvec1 )
         write(iref,*) varline(1:lenvar)

         if ( npelm==3 ) then

!        --- triangles

            write(iref,100) npoint, nelem
100         format ( 'ZONE N=',i8,', E=',i8,', F=FEPOINT, ET=TRIANGLE' )

         else if (npelm==4 ) then

!        --- tetrahedra

            write(iref,110) npoint, nelem
110         format ( 'ZONE N=',i8,', E=',i8, &
                     ', F=FEPOINT, ET=TETRAHEDRON' )

         else if ( npelm==8 ) then

!        --- hexahedra

            write(iref,120) npoint, nelem
120         format ( 'ZONE N=',i8,', E=',i8,', F=FEPOINT, ET=BRICKS' )

            end if

      else

!     --- write different header for subsequent times

         if ( ndim==2 ) then

!        --- 2D case

            if ( npelm==3 ) then

!           --- triangles

               write(iref,130) npoint, nelem
130            format ( 'ZONE N=',i8,', E=',i8,', F=FEPOINT,', &
                        'ET=TRIANGLE, D=(1,2,FECONNECT)' )

            else if ( npelm==4 ) then

!           --- tetrahedra

               write(iref,140) npoint, nelem
140            format ( 'ZONE N=',i8,', E=',i8,', F=FEPOINT,', &
                        'ET=TETRAHEDRON, D=(1,2,FECONNECT)' )

            end if

         else if ( ndim==3 ) then

!        --- 3D case

            if ( npelm==3 ) then

!           --- triangles

               write(iref,160) npoint, nelem
160            format ( 'ZONE N=',i8,', E=',i8,', F=FEPOINT,', &
                        'ET=TRIANGLE, D=(1,2,3,FECONNECT' )

            else if ( npelm==4 ) then

!           --- tetrahedra

               write(iref,170) npoint, nelem
170            format ( 'ZONE N=',i8,', E=',i8,', F=FEPOINT,', &
                        'ET=TETRAHEDRON, D=(1,2,3,FECONNECT' )

            else if ( npelm==8 ) then

!           --- hexahedra

               write(iref,180) npoint, nelem
180            format ( 'ZONE N=',i8,', E=',i8,', F=FEPOINT,', &
                        'ET=BRICKS, D=(1,2,3,FECONNECT' )

            end if

         end if

      end if

!     --- Write co-ordinates and solution to file

      if ( ndim==2 ) then

!     --- 2D case

         if ( nunkp==1 ) then

!        --- nunkp = 1, scalar

            write(iref,200) ( (coor(j,i), j=1,2), usol(i,1), &
                              i=1,npoint )
200         format ( 2e12.4 , e12.4 )

         else if ( nunkp==2) then

!        --- nunkp = 2, 2D-vector

            write(iref,201) ( (coor(j,i), j=1,2), (usol(i,j), j=1,2), &
                              i=1,npoint )
201         format ( 2e12.4 , 2e12.4 )

         else if ( nunkp==3) then

!        --- nunkp = 3, 3D-vector

            write(iref,202) ( (coor(j,i), j=1,2), (usol(i,j), j=1,3), &
                              i=1,npoint )
202         format ( 2e12.4 , 3e12.4 )

         else if ( nunkp==4 ) then

!        --- nunkp = 4, 4D-vector

            write(iref,203) ( (coor(j,i), j=1,2), (usol(i,j), j=1,4), &
                              i=1,npoint )
203         format ( 2e12.4 , 4e12.4 )

         else if ( nunkp==5) then

!        --- nunkp = 5, 5D-vector

            write(iref,204) ( (coor(j,i), j=1,2), (usol(i,j), j=1,5), &
                              i=1,npoint )
204         format ( 2e12.4 , 5e12.4 )

         else if ( nunkp==6) then

!        --- nunkp = 6, 6D-vector

            write(iref,205) ( (coor(j,i), j=1,2), (usol(i,j), j=1,6), &
                              i=1,npoint )
205         format ( 2e12.4 , 6e12.4 )

         else if ( nunkp==7) then

!        --- nunkp = 7, 7D-vector

            write(iref,206) ( (coor(j,i), j=1,2), (usol(i,j), j=1,7), &
                              i=1,npoint )
206         format ( 2e12.4 , 7e12.4 )

         else if ( nunkp==8) then

!        --- nunkp = 8, 8D-vector

            write(iref,207) ( (coor(j,i), j=1,2), (usol(i,j), j=1,8), &
                              i=1,npoint )
207         format ( 2e12.4 , 8e12.4 )

         else if ( nunkp==9) then

!        --- nunkp = 9, 9D-vector

            write(iref,208) ( (coor(j,i), j=1,2), (usol(i,j), j=1,9), &
                              i=1,npoint )
208         format ( 2e12.4 , 9e12.4 )

         else if ( nunkp==10) then

!        --- nunkp = 10, 10D-vector

            write(iref,209) ( (coor(j,i), j=1,2), (usol(i,j), j=1,10), &
                              i=1,npoint )
209         format ( 2e12.4 , 10e12.4 )

         end if

      else if ( ndim==3 ) then

!     --- 3D case

         if ( nunkp==1 ) then

!        --- nunkp = 1, scalar

            write(iref,210) ( (coor(j,i), j=1,3), usol(1,i), &
                              i=1,npoint )
210         format ( 3e12.4 , e12.4 )

         else if ( nunkp==2 ) then

!        --- nunkp = 2, 2D-vector

            write(iref,211) ( (coor(j,i), j=1,3), (usol(i,j), j=1,2), &
                              i=1,npoint )
211         format ( 3e12.4 , 2e12.4 )

         else if ( nunkp==3 ) then

!        --- nunkp = 3, 3D-vector

            write(iref,212) ( (coor(j,i), j=1,3), (usol(i,j), j=1,3), &
                              i=1,npoint )
212         format ( 3e12.4 , 3e12.4 )

         else if ( nunkp==4 ) then

!        --- nunkp = 4, 4D-vector

            write(iref,213) ( (coor(j,i), j=1,3), (usol(i,j), j=1,4), &
                              i=1,npoint )
213         format ( 3e12.4 , 4e12.4 )

         else if ( nunkp==5 ) then

!        --- nunkp = 5, 5D-vector

            write(iref,214) ( (coor(j,i), j=1,3), (usol(i,j), j=1,5), &
                              i=1,npoint )
214         format ( 3e12.4 , 5e12.4 )

         else if ( nunkp==6 ) then

!        --- nunkp = 6, 6D-vector

            write(iref,215) ( (coor(j,i), j=1,3), (usol(i,j), j=1,6), &
                              i=1,npoint )
215         format ( 3e12.4 , 6e12.4 )

         else if ( nunkp==7 ) then

!        --- nunkp = 7, 7D-vector

            write(iref,216) ( (coor(j,i), j=1,3), (usol(i,j), j=1,7), &
                              i=1,npoint )
216         format ( 3e12.4 , 7e12.4 )

         else if ( nunkp==8 ) then

!        --- nunkp = 8, 8D-vector

            write(iref,217) ( (coor(j,i), j=1,3), (usol(i,j), j=1,8), &
                              i=1,npoint )
217         format ( 3e12.4 , 8e12.4 )

         else if ( nunkp==9 ) then

!        --- nunkp = 9, 9D-vector

            write(iref,218) ( (coor(j,i), j=1,3), (usol(i,j), j=1,9), &
                              i=1,npoint )
218         format ( 3e12.4 , 9e12.4 )

         else if ( nunkp==10 ) then

!        --- nunkp = 10, 10D-vector

            write(iref,219) ( (coor(j,i), j=1,3), (usol(i,j), j=1,10), &
                              i=1,npoint )
219         format ( 3e12.4 , 10e12.4 )

         end if

      end if

!     --- Write topology to file

      if ( ifirst==0 ) then

         if ( npelm==3 ) then

!        --- triangles

            write(iref,300) ( (kmeshc(j,i), j=1,3), i=1,nelem )
300         format ( 3i8 )

         else if ( npelm==4 ) then

!        --- tetrahedra

            write(iref,310) ( (kmeshc(j,i), j=1,4), i=1,nelem )
310         format ( 4i8 )

         else

!        --- hexahedra

            write(iref,320) ( (kmeshc(j,i), j=1,8), i=1,nelem )
320         format ( 8i8 )

         end if

      end if

      end
