      subroutine prtransnorm ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-12-2013 New storage of information
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal vector along a set of curves and store
!     in a solution vector
!     All components outside the set of curves are set to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    integer input array,
!                         Contents See subroutine askvec
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqvect, iseqnormvect, istart, &
              itypvc, ivec, lennormvect, inorvect, ncurvslc

!     inorvect       Array of length 5 to store the information of
!                    output normal vector temporarily
!     iseqnormvect   Sequence number of array normvectin isol
!     iseqvect       Sequence number of array vect in isol
!     istart         Starting address in inpsep with information of curves
!                    and element groups
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     lennormvect    Length of array normvect
!     ncurvslc       Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRTRANSNORM2D  Transform a vector consisting of two components into the
!                    normal and tangential component along a set of curves
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASTRUC   find structure of vector
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1059   number of degrees of freedom incorrect
!    1653   Number of unknowns not constant in node
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtransnorm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtransnorm'
         call prinin ( inpsep, 20, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      iseqvect = inpsep(9)
      iseqnormvect = inpsep(3)
      if ( inpsep(15)>0 ) then
         istart = inpsep(15)
         ncurvslc = inpsep(istart)
         istart = istart+1
      else
         istart = 1
         ncurvslc = 0
      end if  ! ( inpsep(15)>0 )
      if ( ierror/=0 ) go to 1000
      if ( debug ) &
         write(irefwr,1) 'iseqvect, iseqnormvect, ndim, ncurvslc', &
                          iseqvect, iseqnormvect, ndim, ncurvslc

!     --- solution vector and corresponding arrays

      call sepactsolbf2 ( isolut2d(iseqvect) )
      iprob  = ks(isolut2d(iseqvect))%iprob
      if ( nphysi/=2 ) then

!     --- nphysi # 2

         call errint ( nphysi, 1 )
         call errint ( 2, 2 )
         call errchr ( names(iseqvect), 1 )
         call errsub ( 1059, 2, 0, 1)

      end if  ! ( nphysi/=6 )

      if ( indprfi/=0 ) then

!     --- indprfi # 0, not implemented

         call errsub ( 1653, 0, 0, 0 )

      end if  ! ( indprfi/=0 )
      if ( ierror/=0 ) go to 1000

!     --- Find structure of vector with exactly ndim parameters per node

      itypvc = 110
      call sepcreastruc ( ndim, itypvc, ivec )
      if ( ierror/=0 ) go to 1000

!     --- Create normal vector

      inorvect = 0
      call sepcreasolbf1 ( inorvect, itypvc, ivec, names(iseqnormvect), 0 )
      normalvec1d => ks(inorvect)%sol
      normalvec1d = 0
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      vectin => ks(isolut2d(iseqvect))%sol

!     --- Compute the inner product of vector and normals and store in
!         usol

      call prtransnorm2d ( kmeshm, normal, ncurvslc, inpsep(istart), &
                           vectin,  normalvec1d, inpsep(7), coor )
      if ( ierror/=0 ) go to 1000

!     --- Copy inorvect

      call sepcopyvc ( inorvect, isolut2d(iseqnormvect) )

!     --- Delete inorvect

      call  seprmsolinfo ( inorvect )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtransnorm'

      end if  ! ( debug )
      call erclos ( 'prtransnorm' )

      end subroutine prtransnorm
