      subroutine read03 ( inpsol, rinsol, isubr, iseqnr, iprob, nprob, &
                          ninsol, nrisol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.19 date 30-05-2015 Extension with parallel options
!        version  5.18 date 30-07-2014 Extension with options for petsc
!        version  5.17 date 08-06-2014 Use read03direct
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the solution subroutines, i.e. either the subroutine
!     SOLVE (direct method) or the iterative method
!     (at this moment only conjugate gradients)
!     The input is stored in array inpsol and the real eps
!     The input must be enclosed between SOLVE and END
!     If the length of the arrays inpsol or rinsol is changed, &
!     adapt the following subroutines: prsimpleiter, prsimplemat
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecadapt
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsol(*), isubr, iseqnr, iprob, nprob, ninsol, nrisol
      double precision rinsol(*)

!     inpsol         o    Contains integer information concerning the solution
!                         process.
!                         Array INPSOL must be filled as follows:
!                         Pos. 1  Last entry that has been filled by the user
!                                 hence if inpsol(1) = 0 or 1, only defaults
!                                 are used
!                         Pos. 2  IPOS  Is used only for direct methods
!                                       Indicates if the matrix to be solved is
!                                       positive definite, IPOS = 1
!                                       or not IPOS = 0.
!                                       Default: 0
!                         Pos. 3  METHOD  Indicates the type of solver used
!                                       Possible values:
!                                       0: A profile method is used
!                                          (direct solver)
!                                       1: CG (conjugate gradients) or BiCGstab
!                                          In case of a non-symmetric matrix
!                                          Bi-CGSTAB
!                                       2: CGS
!                                       3: GMRES
!                                       4: GMRESR
!                                       5: OVERRL or OVERCS
!                                       6: SIMPLE_GCR
!                                       7: SYMMLQ
!                                       8: SIMPLER_GCR
!                                       9: ML_multigrid
!                                      10: IDR
!                                      11: AL_GCR
!                                      12: HSIMPLER_GCR
!                                      13: MSIMPLER_GCR
!                                      14: LSC_GCR
!                                      15: QLSC_GCR
!                                      16: SCHUR
!                                      17: Bi_CGSTab(l)
!                                      18: BLOCK_TRIANGULAR
!                                      19: MUMPS
!                                      20: PETSC
!                                      21: RICHARDSON
!                                      22: CHEBYSHEV
!                                      23: FGMRES
!                                      24: DGMRES
!                                      25: BICG
!                                      26: TFQMR
!                                      27: TCQMR
!                                      28: CR
!                                      29: LSQR
!                                      30: PREONLY
!                                      31: GCR
!                                       Default: 0
!                         Pos. 4  IPRECO  Is used only for iterative methods
!                                       Control parameter concerning the type of
!                                       preconditioning
!                                       IPRECO consists of 2 parts IPRECO_orig
!                                       and IPRECO_par according to
!                                       IPRECO = IPRECO_orig + 100 * IPRECO_par
!                                       Possible values for IPRECO_orig
!                                       0: No preconditioning.
!                                       1: The matrix is premultiplied by a
!                                          diagonal matrix, such that all
!                                          diagonal elements become 1.
!                                          (Diagonal scaling)
!                                       2: Incomplete LU-decomposition using the
!                                          diagonal as only changed elements
!                                          Efficient Eisenstat implementation
!                                       3: Incomplete LU-decomposition
!                                       4: Symmetric Gauss-Seidel
!                                       5: Incomplete LU-decomposition using a
!                                          pre-assembled decomposition according
!                                          to a new method of Rik Kaasschieter
!                                       6: a standard (=default) preconditioner, &
!                                       7: See 2, however, MILU is applied
!                                       8: jacobi
!                                       9: bjacobi (block jacobi)
!                                      10: sor
!                                      11: icc     (incomplete cholesky)
!                                      12: asm     (additive schwarz)
!                                      13: amg     (algebraic multigrid)
!                                      14: mumps   (direct solver)
!                                      15: ml      (multigrid solver)
!                                      16: hypre   (multigrid solver)
!                                       Default value: 6
!                                       IPRECO_par = meant for parallel
!                                       computing only.
!                                       It defines what extra extensions must be
!                                       done to compute the parallel
!                                       preconditioner
!                                       Possible values
!                                       0: No further actions (NN coupling)
!                                       1: The diagonal of the preconditioning
!                                          matrix is assembled over the blocks
!                                          in order to get a kind of DD coupling
!                         Pos. 5  MAXITR  Maximum number of iterations to be
!                                       performed
!                                       In case of a direct solver iterative
!                                       improvement is applied if maxitr>0, &
!                                       with at most maxitr iterations
!                                       Default: 0 (implies number of unknowns)
!                         Pos. 6  IPRINT  Print level of subroutine
!                                       Possible values:
!                                      -1   No print output
!                                       0   Only error messages are printed
!                                       1   A little amount of information of
!                                           the iteration method is printed
!                                       2   A maximal amount of information of
!                                           the iteration method is printed
!                                       3   See 2, in this case also the matrix
!                                           and right-hand side are printed
!                                       Default: 0
!                         Pos. 7  MATSYM  Is used only when nonsymmetric storage
!                                       scheme jmetod=9,10 is used.
!                                       It tells that the matrix is symmetric, &
!                                       so the CG methods can be used.
!                                       Possible values:
!                                       0: The matrix is supposed to be
!                                          nonsymmetric
!                                       1: The matrix is symmetric.
!                         Pos. 8  MGMRES  Dimension of Krylov space for GMRES
!                                       Default: 20
!                                       In case of IDR parallel it indicates
!                                       if the shadow vectors must be computed
!                                       locally (0) or globally (1)
!                                       Default 0
!                                       In case of MUMPS it indicates the type
!                                       of renumbering scheme
!                                       Default: 7
!                         Pos. 9  ISTART  Control parameter concerning the start
!                                       of the conjugate process. (ISTART).
!                                       Possibilities:
!                                       0: The process starts with vector 0
!                                       1: The process starts with an explicitly
!                                              given start vector.
!                                       Default: 0
!                         Pos. 10 KEEP  With this parameter the user may
!                                       indicate whether the preconditioning
!                                       matrix must be destroyed, kept or
!                                       that an old preconditioning matrix
!                                       must be used. Possibilities:
!                                       0 The preconditioning matrix is
!                                         destroyed.
!                                       1 The preconditioning matrix is kept.
!                                         A reference is written in MATR(4).
!                                       2 The preconditioning matrix as computed
!                                         before is used.
!                                         It is supposed that the reference for
!                                         this matrix has been stored in MATR(4)
!                                       Default: 0
!                         Pos. 11 IRELER: Consists of three parts IRELER_orig
!                                       IABS and iaccuracy according to
!                                       IRELER = IRELER_orig + 100 x IABS + &
!                                                1000 x iaccuracy
!                                       Indicates which criterium should be
!                                       used for the termination of the process
!                                       Possible values for IRELER_orig:
!                                        0: The relative error is taken with
!                                           respect to ||x||.
!                                        1: The relative error is taken with
!                                           respect to ||res0||, where
!                                           res0=initial residual
!                                        2: The absolute error is taken
!                                        3: The relative error is taken with
!                                           respect to ||b||.
!                                       10: Use, default value, i.e. 1 except
!                                           for CG where 0 is used
!                                       Possible values for IABS:
!                                        0: No action
!                                        1: Except the error using IRELER_orig
!                                           it is also checked if the absolute
!                                           error is smaller than EPS2
!                                           (RINSOL(7))
!                                       Possible values for iaccuracy:
!                                        0: accuracy is stored in rinsol(1)
!                                       >0: sequence number of scalar in
!                                           which accuracy is stored
!                                       Default: 10
!                         Pos. 12 IRESID: Indicates whether the residual must be
!                                       computed and printed after computation
!                                       of the solution. Possible values:
!                                       0:  Do not compute the residual
!                                       1:  Compute and print the 2-norm of the
!                                           residual
!                                       2:  Compute and print the 2-norm of the
!                                           residual divided by the 2-norm of
!                                           the right-hand side
!                                       Default: 0
!                         Pos. 13 NTRUNK: This parameter is only used if GMRESR
!                                       is used. It indicates the maximum number
!                                       of search directions in the GMRESR
!                                       process. NTRUNK should be much smaller
!                                       than MGMRES
!                                       Default: 20
!                         Pos. 14 NINNER: This parameter is only used if GMRESR
!                                       is used. It indicates the maximum number
!                                       of iterations in the inner loop of the
!                                       GMRESR process.
!                                       Default: 5
!                                       In case of IDR, it contains the
!                                       parameter s of IDR(s)
!                                       and in case of BiCGSTAB(l): l
!                                       Default: 1
!                         Pos. 15 NITER1: This parameter is only used if
!                                       overrelaxation is applied.
!                                       NITER1 is the variable in common CITER.
!                                       If INPSOL(15) = -3, NITER1 is set equal
!                                       to 0, if INPSOL(15) = 0, NITER1 is not
!                                       changed
!                                       Default: 0
!                                       In the case of CG it indicates if a
!                                       minimum eigenvalue is given that is used
!                                       in the termination test (1) or not (0)
!                                       Default: 0
!                         Pos. 16 NITER2: This parameter is only used if
!                                       overrelaxation is applied.
!                                       NITER2 is the variable in common CITER.
!                                       If INPSOL(16) = 0, NITER2 is not changed
!                                       Default: 0
!                                       In the case of CG it indicates
!                                       PROJ_PRINTLEVEL: i.e. the amount of
!                                       output produced when computing the
!                                       projection vectors
!                                       Possible levels: -1, 0, 1, 2, where
!                                       -1 means no output, 0 standard output
!                                       and >0 extra output
!                                       Default: 0
!                         Pos. 17 ICONSTRAIN: This parameter is only used if
!                                       overrelaxation is applied. ICONTRAIN
!                                       indicates if constraints should be
!                                       applied.
!                                       Possible values:
!                                       0:  No constraints, OVERRL is applied
!                                       1:  The solution must satisfy u>=min(x)
!                                       2:  The solution must satisfy u<=max(x)
!                                       3:  The solution must satisfy
!                                           min(x)<=u<=max(x)
!                                       If ICONSTRAIN>0, OVERCS is applied
!                                       Default: 0
!                                       In the case of CG it indicates PROJ_KEEP
!                                       which indicates if projection vectors
!                                       must be kept, destroyed or reused.
!                                       Possible values:
!                                       0: destroy
!                                       1: keep
!                                       2: use old vectors
!                                       Default: 0
!                         Pos. 18 IMINIMUM: This parameter is only used if
!                                       overrelaxation is applied. It indicates
!                                       how the minimum must be computed.
!                                       Possible values:
!                                        0:  min(x) = 0
!                                       >0:  min(x) is a constant (rinsol(7))
!                                       <0:  min(x) is a function:
!                                            FUNC(-IMINIMUM,...)
!                                       Default: 0
!                                       In case of CG this position is used to
!                                       define the projection method
!                                       Possible values:
!                                       0: none
!                                       1: projection based on approximate
!                                          eigenvectors
!                                       Default: 0
!                         Pos. 19 IMAXIMUM: This parameter is only used if
!                                       overrelaxation is applied. It indicates
!                                       how the maximum must be computed.
!                                       Possible values:
!                                        0:  max(x) = 0
!                                       >0:  max(x) is a constant (rinsol(8))
!                                       <0:  max(x) is a function:
!                                            FUNC(-IMAXIMUM,...)
!                                       In the case of CG it indicates
!                                       PROJ_PLOTVECTORS which indicates if the
!                                       projection vectors must be plotted
!                                       Possible values:
!                                       0: do not plot
!                                       1: make a contour plot
!                                       Default: 0
!                         Pos. 20 IDEGFD: This parameter is only used if
!                                       overrelaxation is applied. It indicates
!                                       the degree of freedom that must be
!                                       restricted
!                                       If 0 all are restricted.
!                                       Default: 0
!                                       In the case of CG it indicates
!                                       PROJ_TYPE which indicates how the
!                                       projection vectors must be computed
!                                       Possible values:
!                                       1: by solving a Poisson equation for the
!                                          shale layers
!                                       2: by setting shale layers equal to 1
!                                       3: by using a 0/1 projection vector for
!                                          each layer
!                                       Default: 1
!                         Pos. 21 NUMDEF_CORR: Number of defect correction steps
!                                       to be performed
!                                       Default: 0
!                         Pos. 22 AT_ERROR: Defines what to do in case of an
!                                       error: Possible values:
!                                       0: stop
!                                       1: return
!                                       Default: 0
!                         Pos. 23 ISEQ_EXACT: Sequence number of exact solution
!                                       in ISOL
!                                       If zero the exact solution is not given
!                                       Default: 0
!                         Pos. 24 ISEQ_RES_EXACT: Sequence number of the
!                                       residual of exact solution in ISOL
!                                       If zero the residual is not computed.
!                                       This position can only be used in
!                                       combination with ISEQ_EXACT
!                                       Default: 0
!                         Pos. 25-27 -
!                         Pos. 28 ISEQ_START_RES Contains position of start
!                                       residual in array ISOL
!                                       Default: 0
!                         Pos. 29 ISEQ_END_RES Contains position of end residual
!                                       in array ISOL
!                                       Default: 0
!                         Pos. 30 ISEQ_DIAG Contains position of diagonal of
!                                       matrix in array ISOL
!                                       Default: 0
!                         Pos. 31 ISEQ_PREC DIAG Contains position of diagonal
!                                       of preconditionings matrix in array ISOL
!                                       Default: 0
!                         Pos. 32 ILIMIT defines if the solution must be limited
!                                       to the minimum and maximum values in the
!                                       input vector, including boundary
!                                       conditions. Possible values:
!                                       limit = 0, no limiting
!                                       limit = 1, limiting
!                                       Default: 0
!                         Pos. 33 PREC_LUMPING indicates if positive
!                                       off-diagonal elements must be lumped to
!                                       the diagonal with respect to the
!                                       preconditioning matrix (1) or not (0)
!                                       Default: 0
!                         Pos. 34 iscaling:  Choice of diagonal scaling.
!                                       Possible values:
!                                       0: no scaling
!                                       1: Only row scaling
!                                       2: Row and column scaling
!                                       3: Only column scaling
!                                       Default: 0
!                         Pos. 35       NUMSUBEQ: Number of Subequations
!                         Pos. 36       Type of deflation method
!                                       Possible values:
!                                       0: deflation
!                                       1: coarse grid correction
!                         Pos. 37 IPRINT_SPECTRAL  Print level of main
!                                       subroutine for spectral solve
!                                       Possible values:
!                                      -1   No print output
!                                       0   Only error messages are printed
!                                       1   A little amount of information of
!                                           the iteration method is printed
!                                       2   A maximal amount of information of
!                                           the iteration method is printed
!                         Pos. 38 MAX_ITER_SPECTRAL  maximum number of
!                                       iterations for spectral loop
!                         Pos. 39       isingular
!                                       If 0, standard
!                                       if 1: replace zero pivots by machine
!                                             accuracy
!                         Pos. 40       Sequence number of intmat in case
!                                       of ILU preconditioning with different
!                                       structure than that of the matrix
!                         Pos. 41       num_levels (used for ilu preconditioner)
!                         Pos. 42       Type of parallel solver
!                                       1: block jacobi  (default)
!                                       2: gcr
!                         Pos. 43       maximum number of parallel iterations
!                         Pos. 44       old_version
!                                       0: new version
!                                       1: old version
!                         If NUMSUBEQ>0:
!                         The next 50*NUMSUBEQ positions contain information
!                         about each subequation separately
!     iprob         i/o   Problem number
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!     isubr          i    Indicates by which subroutine READ03 is called
!                         Possible values:
!                         1:  TO0089,  read record with SOLVE
!                         2:  ERSOLV,  do not read record with SOLVE
!     ninsol        i/o   Length of array INPSOL
!     nprob          i    Number of problems
!     nrisol        i/o   Length of array RINSOL
!     rinsol         o    Array containing the real user input if iread=-1
!                         The storage of rinsol depends on inpsol.
!                         The following positions are used
!                         Pos. 1    Accuracy eps (iterative methods and
!                                                 iterative refinement)
!                                   Default value 1d-3
!                         Pos. 2    In case of iteration_method: adapt diagonal
!                                   In case of al_gcr: gamma
!                                   In case of parallel gcr:
!                                   eps w.r.t. subsolver
!                                   Default value 0
!                         Pos. 3.   in case of overrelaxation:
!                                      alpha    Parameter for overrelaxation.
!                                      If zero the value stored in CITER is used
!                                      Default value 0
!                                   in case of cg-type method:
!                                      factor for MILU process
!                                   In case of al_gcr: mu
!                                   Default value: 0.95
!                         Pos. 4.   in case of overrelaxation:
!                                      beta     Parameter for overrelaxation.
!                                      If zero the value stored in CITER is used
!                                   in case of cg:
!                                      minimum eigenvalue to be used in
!                                      termination criterion
!                                   Default value 0
!                         Pos. 5.   in case of overrelaxation:
!                                      omega    Parameter for overrelaxation.
!                                      If zero the value stored in CITER is used
!                                      Default value 0
!                                   in case of cg:
!                                      Accuracy with respect to the computation
!                                      of the projection vectors
!                                      Default value 10^-2
!                         Pos. 6.   in case of overrelaxation:
!                                      alam     Parameter for overrelaxation.
!                                      If zero the value stored in CITER is used
!                                      Default value 0
!                                   in case of cg:
!                                      Threshold value. All elements in the
!                                      projection vector with absolute value
!                                      less than this value are ignored
!                                      Default value 0
!                         Pos. 7.   amin     Parameter for overrelaxation.
!                                   Indicates the minimum value for the
!                                   constraint. See INPSOL(18)
!                                   Default value 0
!                                   in case of cg:
!                                      Value of EPS2
!                                   Default value 0
!                         Pos. 8.   amax     Parameter for overrelaxation.
!                                   Indicates the maximum value for the
!                                   constraint.  See INPSOL(19)
!                                   Default value 0
!                                   in case of spectral method:
!                                      Spectral accuracy (Default 1e-10)
!                                   in case of shifted laplace:
!                                      Real part of shift
!                         Pos  9.   eps_corr Required accuracy for the defect
!                                   correction process. Is only used if
!                                   inpsol(20)>1
!                                   Default value 1d-2
!                                   in case of shifted laplace:
!                                      Imaginary part of shift
!                         Pos 10.   eps_corr_sol Required accuracy for the
!                                   iteration method in the defect correction
!                                   steps (Not the first one). Is only used if
!                                   inpsol(20)>1 and method#0
!                                   Default value 1d-1
!                         Pos 11.   abs_accuray
!                         Pos 12.   rel_accuray
!                         Pos 13.   div_accuray
!                         Pos 14.   damping_factor
!                         pos 15.   emin
!                         pos 16.   emax
!                         pos 17.   drop_tolerance
!                         If NUMSUBEQ>0:
!                         The next 20*NUMSUBEQ positions contain information
!                         about each subequation separately
!                         Warning: if inpsol is extended, update
!                                  solvelbf, to0089 and to0091 !!!!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=18, nncod4=2, nncod5=9, nncod7=3
      integer ncode2(nncode2), ncode4(nncod4), itype4(nncod4), &
              ncode7(nncod7), ncode5(nncod5), itype5(nncod5), &
              ncod4m(nncod4), ncod5m(nncod5), ncod7m(nncod7), &
              ncodmx2(nncode2), isubequats(maxsubeq)
      integer i, maxmsh, iprint, iplot, &
              npairs, nread, numsubeq, isubeq
      double precision tabmn4(nncod4), tabmx4(nncod4), &
                       tabmn5(nncod5), tabmx5(nncod5)
      character (len=15) code4(nncod4), code7(nncod7)
      character (len=25) code2(nncode2), code5(nncod5)
      logical finished

!     code2           Table with the main keywords
!     code4          Table with the keywords in the SOLVE record
!     code5          Table with the keywords with the options of adaptive
!     code7          Table with the keywords with the options of residual
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     iplot          Plot parameter. Possible values:
!                    0:  no plot
!                    1:  only the first and last mesh are plotted
!                    2:  all meshes are plotted
!     iprint         Print level
!     isubeq         Sequence number of subequation
!     isubequats     Array to check if a subequation has been used
!                    (isubequats(.) = 1) or not (isubequats(.) = 0)
!     itype4         itype array for code4 (See intx02)
!     itype5         itype array for code5 (See intx02)
!     maxmsh         Maximum number of meshes
!     ncod4m         Table containing the number of maximum characters in
!                    the keyword table code4
!     ncod5m         Table containing the number of maximum characters in
!                    the keyword table code5
!     ncod7m         Table containing the number of maximum characters in
!                    the keyword table code7
!     ncode2         Table containing the number of significant characters in
!                    the keyword table code2
!     ncode4         See ncode, now for table code4
!     ncode5         See ncode, now for table code5
!     ncode7         See ncode, now for table code7
!     ncodmx2        Table containing the number of maximum characters in
!                    the keyword table code2
!     nncod4         Number of entries in table code4
!     nncod5         Number of entries in table code5
!     nncod7         Number of entries in table code7
!     nncode2        Number of entries in table code2
!     npairs         Number of pairs of nodes with the special boundary
!                    condition in the linear solver
!     nread          Number of items read
!     numsubeq       Number of subequations
!     tabmn4         Table with minimum values of numbers corresponding to code4
!     tabmn5         Table with minimum values of numbers corresponding to code5
!     tabmx4         Table with maximum values of numbers corresponding to code4
!     tabmx5         Table with maximum values of numbers corresponding to code5
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READ03DEFECT   Read all information corresponding to the sub-keyword
!                    DEFECT_CORRECTION
!     READ03DIRECT   Read all information corresponding to the sub-keyword
!                    DIRECT_SOLVER
!     READ03ITER     Read all information corresponding to the sub-keyword
!                    ITERATION_METHOD
!     READ03SPECTRAL Read all information corresponding to the sub-keyword
!                    SPECTRAL
!     READREALSCAL   Read integer, real or scalar
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     READ03 requires input from the standard input file
!     The following input is recognised:
!     SOLVE [, SEQUENCE_NUMBER = s][, PROBLEM = p] (mandatory):
!         start of input
!      optional records:
!       POSITIVE_DEFINITE
!       ALLOW_ZERO_ROWS
!       ITERATION_METHOD = ...,  see subroutine READ03ITER
!       SYMMETRIC          ... matrix is symmetric (when metod=11 or 12)
!       DIRECT_SOLVER = PROFILE, options
!                     = COMPACT, options
!                     = MUMPS, options
!          with options for profile empty
!          and for dir_compact empty or a combination of the following
!          possibilities:
!          droptolerance = d
!          num_pivot_rows= m
!          num_iter_ref  = n
!          accuracy      = eps
!          print_level   = p
!          lu_storage    = n
!       OLD_VERSION
!       RESIDUAL = none
!                  absolute
!                  relative
!       ADAPTIVE, options
!        with options empty or a combination of the following possibilities:
!                                    ( first two characters are significant )
!          relative        =  relative error in semi-energy norm
!                                        see inplrf(2) and rinlrf(1) above
!          absolute        =  absolute error in semi-energy norm
!          max_mesh        =  max number of meshes, >=1
!          plot            =  0=no plot, 1=first and last mesh, 2=all meshes
!          print_level     =  0=no print, 1=modest, 2=maximal print
!          smooth_iter     =  number of iterations in smoothing, 0..2
!       DEFECT_CORRECTION, options  see subroutine read03defect
!
!       ISEQ_EXACT = i
!       ISEQ_RES_EXACT = i
!       ISEQ_START_RES   = i
!       ISEQ_END_RES     = i
!       LIMIT_SOLUTION
!       SPECTRAL, options  see subroutine read03spectral
!       OVER_PRESSURES, options
!        with options:
!           USER_POINTS = ( i1, i2,  j1, j2 , ... )
!             or
!           NODES  = ( i1, i2,  j1, j2 , ... )
!           THRESHOLDS = ( t1, t2, ... )
!        If the number of subequations is larger than 1 the input may be
!        extended by:
!
!           sub_equation 1
!              standard input for sub equation 1
!           sub_equation 2
!              standard input for sub equation 2
!                .
!                .
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     574   Not an integer number found after the keyword
!     650   First record read by subroutine is not equal to SOLVE
!    1870   Integer number read out of range
!    2140   ISEQ_RES_EXACT has been given, but not ISEQ_EXACT
!    2448   Subequation number used more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (solve)
!    Read rest of first record (sequence number, problem number )
!    finished := false
!    While ( not finished ) do
!       Read next record
!       Analyse first item
!       If ( item # keyword ) then
!          error
!       else
!          analyse keyword
!          if ( keyword = END ) then
!             finished := true
!          else if ( keyword = ITERATION ) then
!             analyse rest of keyword
!          else if ( keyword = ADAPTIVE ) then
!             analyse rest of keyword
!          else if ( keyword = DIRECT_SOLVER ) then
!             analyse rest of keyword
!          else if ( keyword = RESIDUAL ) then
!             analyse rest of keyword
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'POSITIVE', 'SYMMETRIC', 'ITERATION_METHOD', &
                  'ADAPTIVE', &
                  'DIRECT_SOL', 'RESIDUAL', 'END', 'DEFECT_COR', &
                  'ISEQ_EXACT', 'ISEQ_RES_EXACT', 'XXX', &
                  'ISEQ_START_RES', 'ISEQ_END_RES', 'LIMIT_SOLUTION', &
                  'SUB_EQUATION', 'SPECTRAL', 'ALLOW_ZERO_ROWS', &
                  'OLD_VERSION' /
      data ncode2 / 3, 3, 4, 4, 3,   3, 3, 5, 8, 11, &
                  13, 8, 8, 5, 6,   5, 9, 7 /
      data ncodmx2/17, 9,16, 8,13,   8, 3,17,10, 14, &
                  14,14,12,14,12,   8,16,11 /
      data code4 / 'PROBLEM', 'SEQUENCE_NUMBER' /
      data ncode4 / 4, 3 /, itype4 / 0, 0 /, tabmn4 / 1d0, 1d0 /, &
           ncod4m / 7, 15 /
      data code5 / 'RELATIVE', 'ABSOLUTE', 'MAX_MESH', 'PLOT', &
                   'PRINT_LEVEL', 'SMOOTH_ITER', 'MESH', 'LEVEL', &
                   'ITER' /
      data ncode5 / 3, 3, 3, 4, 4, 3, 4, 3, 4 /
      data ncod5m / 8, 8, 8, 4,11,11, 4, 5, 4 /
      data itype5 / 1, 1, 3, 0, 3, 3, 0, 0, 0 /
      data tabmn5 / 0d0, 0d0,  1d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0 /
      data tabmx5 / 1d0, 1d20, 1d3, 2d0, 4d0, 2d0, 1d3, 2d0, 2d0 /
      data code7 / 'NONE', 'ABSOLUTE', 'RELATIVE' /
      data ncode7 / 3, 3, 3 /
      data ncod7m / 4, 8, 8 /

!     code2    Table with the main keywords
!             Values:
!             1:  positive_definite
!             2:  symmetric
!             3:  iteration_method
!             4:  adaptive
!             5:  direct_solver
!             6:  residual
!             7:  end
!             8:  defect_correction
!             9:  iseq_exact
!            10:  iseq_res_exact
!            11:  -
!            12:  iseq_start_res
!            13:  iseq_end_res
!            14:  limit_solution
!            15:  sub_equation
!            16:  spectral
!            17:  allow_zero_rows
!            18:  old_version
!     code4   Table with the keywords in the SOLVE record
!             Values:
!             1: problem
!             2: sequence
!     code5   Table with the keywords with the options of adaptive
!             Values:
!             1:  relative
!             2:  absolute
!             3:  max_mesh
!             4:  plot
!             5:  print_level
!             6:  smooth_iter
!             7:  mesh (in case max mesh is used)
!             8:  level (in case print level is used)
!             9:  iter (in case smooth iter is used)
!     code7   Table with the keywords with the options of residual
!             Values:
!             1:  none
!             2:  absolute
!             3:  relative
! ======================================================================
!
!     --- Initialize parameters

      call eropen ( 'read03' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read03'
         write(irefwr,1) 'ninsol, nrisol', ninsol, nrisol
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize inpsol and rinsol

      inpsol(2:ninsol) = 0
      rinsol(1:nrisol) = 0d0
      rinsol(1) = 1d-3
      rinsol(8) = 1d-8
      inpsol(4) = 6
      inpsol(11) = 10
      inpsol(37) = 1
      inpsol(38) = 100
      do i = 1, maxsubeq
         inpsol(i*ninpsol1+4) = 6
         inpsol(i*ninpsol1+11) = 10
         inpsol(i*ninpsol1+37) = 1
         inpsol(i*ninpsol1+38) = 100
         rinsol(i*nrinsol1+1) = 0.1d0
         rinsol(i*nrinsol1+8) = 1d-8
      end do

      inplrf(1) = 8
      inplrf(2) = 1
      inplrf(3) = 4
      inplrf(4) = 2
      inplrf(5) = 2
      inplrf(6) = 0
      inplrf(7) = 0
      inplrf(8) = 0
      rinlrf(1) = 0.05d0
      numsubeq = 0
      isubeq = 0
      isubequats(1:maxsubeq) = 0
      iadapt = 0

      if ( isubr==1 ) then

!     --- Call by TO0089, read first record  (SOLVE)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'SOLVE', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:4)/='SOLV' ) then
            call errchr ( 'SOLVE', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read rest of record

      table(1) = iprob
      table(2) = iseqnr
      tabmx4(1) = nprob
      tabmx4(2) = 25d0    ! change ini009 if this number must be extended
      iretur = 0
      nmain = 0
      call sepcopycode ( code4, nncod4, ncode4, ncod4m )
      call sepcopytype ( nncod4, itype4, tabmn4, tabmx4 )
      call sepreadsetkeyw
      iprob  = nint(table(1))
      iseqnr = nint(table(2))

      finished = .false.

!     --- While loop finished

200   do while ( .not. finished )

!     --- Read next record

         ityprd = 1
         call asknxt
         ierpri = 1
         call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
         icode = sepsubkeyw()
         select case (icode )

            case ( :0, 19: )

!           --- icode = 0, finished

               go to 1000

            case ( 1 )

!           --- POSITIVE_DEFINITE found

               inpsol(ninpsol1*isubeq+2) = 1

            case ( 2 )

!           --- SYMMETRIC found

               inpsol(ninpsol1*isubeq+7) = 1

            case ( 3 )

!           --- ITERATION_METHOD found, read rest of record

               call read03iter ( inpsol(ninpsol1*isubeq+1), &
                                 rinsol(nrinsol1*isubeq+1) )

            case ( 4 )

!           --- ADAPTIVE found, read rest of record (options)

               iadapt = 1
               table(1:nncod5) = 0d0
               table(5) = -1d0
               table(8) = -1d0
               table(4) = -1d0
               call sepcopytype ( nncod5, itype5, tabmn5, tabmx5 )
               iretur = 0
               nmain = 0
               call sepcopycode ( code5, nncod5, ncode5, ncod5m )
               call sepreadsetkeyw
               if ( iretur==-1 ) go to 200
               if ( table(2)>0d0 ) inplrf(2) = 2
               rinlrf(1) = max ( table(1), table(2) )
               if ( rinlrf(1)<=0d0 ) rinlrf(1) = 0.05d0
               maxmsh = max ( nint(table(3)), nint(table(7)) )
               if ( maxmsh>0 ) inplrf(3) = maxmsh
               iplot = nint(table(4))
               if ( iplot>-1 ) inplrf(4) = iplot
               iprint = max ( nint(table(5)), nint(table(8)) )
               if ( iprint>-1 ) inplrf(5) = iprint
               inplrf(7) = max ( nint(table(6)), nint(table(9)) )

            case ( 5 )

!           --- DIRECT_SOLVER found, read rest of record

               call read03direct ( isubeq, inpsol, rinsol )

            case ( 6 )

!           --- RESIDUAL found, read rest of record

               call asknxt
               ierpri = 1
               call sepcopycode ( code7, nncod7, ncode7, ncod7m )
               icode = sepsubkeyw()
               inpsol(ninpsol1*isubeq+12) = icode-1

            case ( 7 )

!           --- END found

               finished = .true.

            case ( 8 )

!           --- DEFECT_CORRECTION found, read rest of record

               call read03defect ( isubeq, inpsol, rinsol )

            case ( 9, 10, 12, 13 )

!           --- ISEQ_EXACT, ISEQ_RES_EXACT, ISEQ_START_RES or ISEQ_END_RES
!               found, read rest of record

               call asknxt

               if ( debug ) &
                  write(irefwr,1) 'icode, ityprd/1', icode, ityprd

               if ( ityprd==4 ) then

!              --- String found, check if it is a constant, scalar or vectpr

                  lastitem = lastitembef
                  call readrealscal
                  if ( ityprd==102 ) ityprd = 2  ! vector found
                  if ( debug ) write(irefwr,1) 'ityprd/2', ityprd

               end if  ! ( ityprd==4 )

               if ( ityprd/=2 ) then

!              --- Not an integer number found

                  call errchr ( code2(icode), 1 )
                  call errsub ( 574, 0, 0, 1 )
                  call errcrd

               else if ( nint(realrd)<1 .or. nint(realrd)>100 ) then

!              --- sequence number out of range

                  call errchr ( code2(icode), 1 )
                  call errint ( nint(realrd), 1 )
                  call errint ( 1, 2 )
                  call errint ( 100, 3 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               else if ( icode<12 ) then

!              --- ISEQ_EXACT or ISEQ_RES_EXACT

                  inpsol(ninpsol1*isubeq+icode+14) = nint(realrd)

               else

!              --- ISEQ_START_RES or ISEQ_END_RES

                  inpsol(ninpsol1*isubeq+icode+16) = nint(realrd)

               end if

            case ( 14 )

!           --- LIMIT-SOLUTION found

               inpsol(ninpsol1*isubeq+32) = 1

            case ( 15 )

!           --- Sub_equation found, read number

               call asknxt
               if ( ityprd/=2 ) then

!              --- No number found after subequation

                  call errchr ( code2(15)(1:ncodmx2(15)), 1 )
                  call errsub ( 574, 0, 0, 1 )
                  isubeq = 0

               end if
               isubeq = nint(realrd)
               if ( isubeq<=0 .or. isubeq>maxsubeq ) then

!              --- isubeq out of range

                  call errchr ( code2(15)(1:ncodmx2(15)), 1 )
                  call errint ( isubeq, 1 )
                  call errint ( 1, 2 )
                  call errint ( maxsubeq, 3 )
                  call errsub ( 1870, 3, 0, 1 )
                  isubeq = 0

               else

!              --- sub equation number ok, check if it is not in use

                  if ( isubequats(isubeq)/=0 ) then

!                 --- sub equation number already in use

                     call errint ( isubeq, 1 )
                     call errsub ( 2448, 1, 0, 0 )

                  end if
                  isubequats(isubeq) = 1
                  numsubeq = max(numsubeq,isubeq)

               end if

            case ( 16 )

!           --- SPECTRAL found, read rest of record

               call read03spectral ( isubeq, inpsol, rinsol )

            case ( 17 )

!           --- allow_zero_rows found, set inpsol(39)

               inpsol(39) = 1

            case ( 18 )

!           --- old_version found, set inpsol(44)

               inpsol(44) = 1

         end select  ! case (icode )

      end do  !  while ( .not. finished )

      if ( inpsol(ninpsol1*isubeq+24)>0 .and. &
           inpsol(ninpsol1*isubeq+23)==0 ) then

!     --- iseq_res_exact given, but not iseq_exact

         call errint ( inpsol(ninpsol1*isubeq+24), 1 )
         call errsub ( 2140, 1, 0, 0 )

      end if
      inpsol(35) = numsubeq

!     --- Check and set array length of inpsol and rinsol

      i = (numsubeq+1)*ninpsol1
      if ( i>ninsol ) then

!     --- Array inpsol too small

         call errint ( i, 1 )
         call errint ( ninsol, 2 )
         call errchr ( 'inpsol', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if
      ninsol = i
      i = (numsubeq+1)*nrinsol1
      if ( i>nrisol ) then

!     --- Array rinsol too small

         call errint ( i, 1 )
         call errint ( nrisol, 2 )
         call errchr ( 'rinsol', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if
      nrisol = i

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'numsubeq ', numsubeq
         do i = 0, numsubeq
            write(irefwr,*) 'sub equation ', i
            call prinin ( inpsol(1+ninpsol1*i), ninpsol1, 'inpsol')
            call prinrl ( rinsol(1+nrinsol1*i), nrinsol1, 'rinsol')
         end do
         write(irefwr,*) 'End read03'

      end if
      call erclos ( 'read03' )

      end subroutine read03
