      subroutine readloop ( iinput, rinput, isubr, iseqnr, niinp, nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 12-12-2007 Debug statements
!        version  1.2  date 08-05-1998 Check divergence
!        version  1.1  date 17-04-1998 Remove tabmax(1)=25
!        version  1.0  date 05-04-1998
!
!   copyright (c) 1998-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the loop subroutine toloop
!     The input is stored in the arrays iinput and rinput
!     The input must be enclosed between LOOP_INPUT and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     loop
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, nrinp, iinput(*), iseqnr, isubr
      double precision rinput(*)

!    iinput  o   The integer input read is stored in array IINPUT
!                This array must be filled as follows:
!                1:  MAXITER   Maximum number of iterations allowed
!                2:  IPRINT    Defines the amount of output produced
!                              Possible values:
!                             -1:  All output information is suppressed, &
!                                  even the message of no-convergence
!                              0:  No information about the iteration
!                                  process is printed except in the case
!                                  of an error
!                              1:  The number of iterations performed is
!                                  printed
!                              2:  Some extra information about the iteration
!                                  process is printed for each iteration
!                              3:  See 2, in this case also the ratio of two
!                                  succeeding iterations is printed
!                3:  IRETURN   Indicates if the subroutine must stop if
!                              no convergence has been found (0) or that
!                              control must be given back to the main
!                              program (1)
!                4:  MINITER   Minimum number of iterations allowed
!                5:  ICRITER   Type of criterion to be used for the
!                              termination
!                              Possible values:
!                              0:  Absolute difference between two succeeding
!                                  iterations
!                              1:  Relative difference between two succeeding
!                                  iterations
!                6:  IVECTR    Sequence number of vector to be used for
!                              comparison
!                7:  ICHKDIV   Indicates if divergence must be tested
!                              0: yes
!                              1: no
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine readloop is called
!                Possible values:
!                1:  ...      read record with LOOP_INPUT
!                2:  ERLOOP,  do not read record with LOOP_INPUT
!    niinp  i/o  Length of integer input array
!                At input available length, at output computed length
!    nrinp  i/o  Length of real input array
!                At input available length, at output computed length
!    rinput  o   Contains the real input for the stationary free boundary
!                subroutine
!                At this moment he input is defined as follows
!                1:  accuracy eps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=8, nncod2=6
      integer ncode1(nncode1), ncode2(nncod2), itype1(nncode1), ncodmx1(nncode1)
      integer k
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=16) code1(nncode1), code2(nncod2)

!     code1          Table with keywords corresponding to options
!     code2          Table with the keywords following the subkeywords
!     itype1         itype array for code1 (See intx02)
!     k              Counting variable
!     ncode1         Table containing the number of significant characters in
!                    the keyword table code1
!     ncode2         See ncode, now for table code2
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncod2         Number of entries in table code2
!     nncode1        Number of entries in table code1
!     tabmax1        Table with maximum values of numbers corresponding to code1
!     tabmin1        Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readloop requires input from the standard input file
!     The following input is recognized:
!     LOOP_INPUT [, SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!          maxiter = m             (Default 20)
!          miniter = m             (Default 2)
!          accuracy = eps          (Default 1d-3)
!          print_level = p         (Default 0)
!          criterion = c           (Default abs)
!          check_divergence = d    (Default yes)
!          at_error = e            (Default stop)
!          seq_vector = i          (Default 1)
!                      Possible values for c:   absolute
!                                               relative
!                      Possible values for d:   yes
!                                               no
!                      Possible values for e:   stop
!                                               return
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     650   First record read by subroutine is not equal to STATIONARY_FREE
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (LOOP_INPUT)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'MAXITER', 'MINITER', 'PRINT_LEVEL', 'ACCURACY', &
                  'CRITERION', 'AT_ERROR', 'SEQ_VECTOR', &
                  'CHECK_DIVERGENCE' /
      data ncode1  / 3, 3, 5, 3, 4, 5, 6,  9 /
      data ncodmx1 / 7, 7,11, 8, 9, 8,10, 16 /
      data itype1  / 0, 0, 0, 1, 5, 5, 0,  5 /
      data tabmin1 / 1d0, 1d0,-1d0, 0d0, 1d0, 3d0, 1d0, 5d0 /
      data tabmax1 / 1d3, 1d3, 3d0, 1d1, 2d0, 4d0, 1d2, 6d0 /
      data code2 / 'ABSOLUTE', 'RELATIVE', &
                   'STOP', 'RETURN', 'YES', 'NO' /
      data ncode2 / 3, 3, 3, 3, 3, 2 /

!     code    Table with keywords corresponding to options
!             Values:
!             1:  maxiter
!             2:  miniter
!             3:  print_level
!             4:  accuracy
!             5:  criterion
!             6:  at_error
!             7:  seq_vector
!             8:  check_divergence
!     code2   Table with the keywords following the subkeywords
!             Values:
!             1:  absolute
!             2:  relative
!             3:  stop
!             4:  return
!             5:  yes
!             6:  no
! ======================================================================
!
      call eropen ( 'readloop' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readloop'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize arrays and parameters

      if ( niinp<7 ) then

!     --- niinp too small

         call errint ( 7, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      if ( nrinp<1 ) then

!     --- nrinp too small

         call errint ( 1, 1 )
         call errint ( nrinp, 2 )
         call errsub ( 329, 2, 0, 0 )

      end if
      iinput(1:niinp) = 0
      iinput(1) = 20
      iinput(4) =  2
      iinput(6) =  1
      iinput(7) =  0
      rinput(1) =  1d-3

      if ( isubr==1 ) then

!     --- Call by TOLOOP, read first record  (LOOP_INPUT)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'LOOP_INPUT', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:4)/='LOOP' ) then
            call errchr ( 'LOOP_INPUT', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr (iseqnr  )   ! read sequence number
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend                  ! Fill END as main keyword

      table(1:nncode) = -1d20
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( table(1)>0d0 ) iinput(1) = nint(table(1))
      if ( table(2)>0d0 ) iinput(4) = nint(table(2))
      if ( table(3)>0d0 ) iinput(2) = nint(table(3))
      if ( table(4)>0d0 ) rinput(1) = table(4)
      if ( table(5)>0d0 ) iinput(5) = nint(table(5))-1
      if ( table(6)>0d0 ) iinput(3) = nint(table(6))-1
      if ( table(7)>0d0 ) iinput(6) = nint(table(7))
      if ( table(8)>0d0 ) iinput(7) = nint(table(8))-1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, 7, 'iinput' )
         write(irefwr,*) 'End readloop'

      end if  ! ( debug )
      call erclos ( 'readloop' )

      end subroutine readloop
