      subroutine readscbounint ( istart, inpsep, iscalinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 27-05-2014 Extension with irule
!        version  2.4  date 29-04-2014 Use local numbering in inpsep
!        version  2.3  date 23-01-2013 Extension of ichint
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     BOUNDARY_INTEGRAL
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), iscalinp

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          1: ISEQNR Sequence number boundary integrals
!                          2: IVECTOR  Sequence number input vector
!                          3: ISCALAR1 Sequence number first scalar of integral
!                          4: ISCALAR2 Sequence number second scalar of integral
!                          5: ISCALAR3 Sequence number third scalar of integral
!                          6  IPLACE   Position where boundary integrals must
!                                      be computed
!                                      1:  curves
!                                      2:  surfaces
!                          7  INUM     Number of curves
!                          8: ICHINT   Type of integral
!                          9: NUMBER   Degree of freedom
!                         10: IRULE    Type of integration
!                         11-14: -
!                         15-14+INUM    Curve numbers
!     iscalinp       i    Scalar number read before
!                         If 0, no scalar number has been read
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=15
      integer ncode1(nncode1), ncodmx1(nncode1), irule, &
              ivec, iscal, i, m, icodesav
      character (len=17) code1(nncode1)
      logical simpson, axisym

!     axisym         if true the keyword axi_symmetric has been found
!     code1          Table with keywords
!     i              Counting variable
!     icodesav       Help parameter to save the value of icode
!     irule          type of numerical integration.  possibilities:
!                     1  trapezoid rule,  Cartesian co-ordinates
!                     2  trapezoid rule,  axi-symmetric co-ordinates
!                     3  Simpsons rule   Cartesian co-ordinates
!                     4  Simpsons rule   axi-symmetric co-ordinates
!     iscal          Sequence number of scalar
!     ivec           Sequence number of vector with respect to array ISOL
!     m              number of items stored in tabout
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
!     simpson        if true simpson's rule is applied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       BOUNDARY_INTEGRAL, options (all in one line)
!           where options may have the following contents
!               SEQ_BOUN_INTEGRAL = s (Default the next one)
!               VECTOR = s           (Default 1)
!               SCALAR1 = s          (Default 1)
!               SCALAR2 = s          (Default 1)
!               SCALAR3 = s          (Default 1)
!               CURVES = (Ci, Cj, Ck )
!               SURFACES = (Si, Sj, Sk )
!               ICHINT = i           (Default 1)
!               DEGDFD i             (Default 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1886   Vector has not yet been filled
!    3183   No definition of where the boundary integral must be computed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQ_BOUN_INTEGRAL', 'VECTOR', 'SCALAR', 'SCALAR', &
                  'SCALAR', 'V', 'CURVES', 'ICHINT', 'SURFACES', &
                  'DEGFD', 'IRULE', 'AXI_SYMMETRIC', 'CARTESIAN', &
                  'TRAPEZOID_RULE', 'SIMPSONS_RULE'/
      data ncode1  /  6, 6, 1, 1, 1,   1, 3, 3, 4, 3, &
                     4, 3, 3, 4, 4 /
      data ncodmx1 / 17, 6, 6, 6, 6,   1, 6, 6, 8, 5, &
                     5,13, 9,14,13  /

!     code1   Table with sub keywords corresponding to keyword BOUNDARY_INTEGRAL
!             Values:
!             1:  SEQ_BOUN_INTEGRAL
!             2:  VECTOR
!             3:  SCALAR
!             4:  SCALAR
!             5:  SCALAR
!             6:  V
!             7:  CURVES
!             8:  ICHINT
!             9:  SURFACES
!            10:  DEGFD
!            11:  IRULE
!            12:  AXI_SYMMETRIC
!            13:  CARTESIAN
!            14:  TRAPEZOID_RULE
!            15:  SIMPSONS_RULE
! ======================================================================
!
      call eropen ( 'readscbounint' )
      debug = .false. .and. ioutp>=0
      icodesav = icode
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscbounint'
         write(irefwr,1) 'istart, icode', istart, icode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

      codesub(1) = 'C'
      codesub(2) = 'S'
      ncodesub(1:2) = 1

      inpsep(1:15) = 0

      itype(1:nncode) = 0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 0d0
      table(1:nncode) = -1d0

      tabmax(1) = 1d2                ! SEQ_BOUN_INTEGRAL
      table(1) = 1d0

      tabmax(2) = dble(maxvec)   ! VECTOR

      itype(3) = 12                  ! SCALAR
      tabmax(3) = 3d0
      table(3) = 1d0

      tabmax(4) = dble(maxcls)   ! SCALAR
      table(4) = 2d0

      table(5) = 3d0                 ! SCALAR

      tabmax(6) = tabmax(2)          ! VECTOR

      itype(7) = 18                  ! curves
      tabmin(7) = dble(ncurvs)
      tabmax(7) = tabmin(7)

      table(8) = 1d0                 ! ichint
      tabmax(8) = 1d2

      itype(9) = 18                  ! surfaces
      tabmin(9) = dble(nsurfs)
      tabmax(9) = tabmin(9)
      table(9) = -2d0                ! position in codemn (S)

      tabmax(10) = 1d1               ! degfd
      table(10) = 1d0

      tabmax(11) = 4d0

      itype(12:15) = 2               ! axi-symmetric/cartesian/trapezoid/simpson

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      icode = icodesav

      if ( table(2)>0d0 .and. table(6)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 2, 6 )  ! vector and v
      if ( table(7)>0d0 .and. table(9)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 7, 9 )  ! curves and surfaces

      table(2) = max(1d0,table(2),table(6))
      inpsep(0) = 16                 ! code for boundary integral
      inpsep(1) = nint(table(1))
      inpsep(2) = nint(table(2))
      inpsep(4) = mod(nint(table(4)),100000)
      inpsep(5) = mod(nint(table(5)),100000)

      if ( iscalinp==0 ) then

!     --- Standard case, scalar has not been read before

         inpsep(3) = mod(nint(table(3)),100000)

      else

!     --- Scalar number has been read before

         inpsep(3) = iscalinp

      end if  ! ( iscal==0 )
      inpsep(6:7) = 0

!     --- Check if not both curves and surfaces are used

      call errkeywexclus ( table, code, ncodmx, 7, 9 )

      if ( table(7)>0d0 ) then

!     --- Curves found

         iposut = nint(table(7))! Position of curve numbers in tabout
         m = nint(tabmax(7))    ! number of curves read

         inpsep(6) = 1   ! curves
         inpsep(7) = m   ! number of curves
         inpsep(15:14+m) = nint(tabout(iposut:iposut+m-1)) ! curves

      end if  ! ( table(7)>0d0 )

      if ( table(9)>0d0 ) then

!     --- Surfaces found

         iposut = nint(table(9))! Position of surface numbers in tabout
         m = nint(tabmax(9))    ! number of surfaces read

         inpsep(6) = 2   ! surfaces
         inpsep(7) = m   ! number of surfaces
         inpsep(15:14+m) = nint(tabout(iposut:iposut+m-1)) ! curves

      end if  ! ( table(7)>0d0 )

!     --- Check if the boundary is defined

      if ( inpsep(1)<=0 .and. inpsep(6)==0 ) &
         call errsub ( 3183, 0, 0, 0 )

      if ( icode==17 ) then

!     --- Boundary sum found

         inpsep(8) = 8                ! ichint

      else

!     --- Boundary integral found

         inpsep(8) = nint(table(8))   ! ichint

      end if  ! ( icode==17 )
      inpsep(9) = nint(table(10))   ! degfd

!     --- integration rule
!         First check if keywords are mutually exclusive

!         --- Check for keywords that exclude each other

      do i = 12, 15
         call errkeywexclus ( table, code, ncodmx, 11, i )
      end do  ! i = 12, 15
      call errkeywexclus ( table, code, ncodmx, 12, 13 )
      call errkeywexclus ( table, code, ncodmx, 14, 15 )

      irule = max(1,nint(table(11)))
      axisym = .false.
      if ( table(12)>0d0 ) axisym = .true.
      simpson = .false.
      if ( table(15)>0d0 ) simpson = .true.

      if ( irule==1 ) then

!     --- Standard case: adapt irule if necessary

         if ( simpson ) irule = irule+2
         if ( axisym ) irule = irule+1

      end if  ! ( irule==1 )

      inpsep(10) = irule

      ivec = nint(table(2))
      if ( ivec>0 ) then

         if ( ivectors(ivec)==0 .and. nwarn<mxwarn ) then

!        --- vector does not exist

            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd

         end if

      end if
      do i = 1, 3
         iscal = mod(nint(table(2+i)),100000)
         if ( iscal<=maxcls ) iscalars(iscal) = 1
      end do

      istart = istart + 15 + inpsep(7)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), 10+inpsep(7), 'inpsep' )
         write(irefwr,1) 'istart, icode', istart, icode
         write(irefwr,*) 'End readscbounint'

      end if  ! ( debug )
      call erclos ( 'readscbounint' )

      end subroutine readscbounint
