      subroutine readscderiv ( istart, inpsep, nprob, numscl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 13-01-2015 Extension with skip element groups
!        version  2.1  date 19-12-2013 Use type 39 for scalars
!        version  2.0  date 12-10-2012 Extra parameter kmesh
!
!   copyright (c) 2002-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     DERIVATIVES
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
!     Warning:  If this subroutine is changed also change ASKVECDERIV !!!!!!
!               Also READDR
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), nprob, numscl

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         0: ICODE
!                         1: Sequence number corresponding to input for
!                            coefficients
!                            If iseqcf = -1, the next one will be used
!                         2: IPROB   Problem sequence number
!                         3: IVECTOR Sequence number output vector
!                                    (first vector)
!                            If ivector = -1, the same value as iprob
!                            will be used
!                            If iseqdr = -1, the next one will be used
!                            If ivector = -2, no output vector is used
!                         4: ISEQDR Sequence number derivatives
!                            if -10: no block is read
!                         5: ISCALAR Sequence number scalar, is only used if
!                            boundary integrals are computed.
!                            Otherwise iscalar should be 0
!                            If iscalar>0, ivector must be -2
!                         6: ICHELD Choice parameter
!                         7: ISEQINPVEC Sequence number input vector
!                         8: IVEC Type of output vector
!                         9: IX, sequence number of derivative d/d_ix
!                        10: Type of output vector, &
!                            See readdr: iinder(1)
!                        11: typeboun, &
!                            The following values are available
!                            1:   the boundary consists of points
!                            2:   the boundary consists of curves
!                            3:   the boundary consists of surfaces
!                           20:   the boundary consists of the zero level set
!                        12: m, number of points, curves or surfaces
!                        13: start address point/curve or surface numbers
!                                 or levelset seq number
!                        14: ALLELEMENTS  Indication if all standard element
!                           sequence numbers are used (0) or not (>0)
!                           In that case the positions ALLELEMENTS to
!                           ALLELEMENTS+NELGRP-1
!                           (NELGRP is number of element groups)
!                           are considered as an array IELHLP to be
!                           filled by the user in the following way:
!                           In IELHLP(IELGRP)  ( 1<=IELGRP<=NELGRP)
!                           the user must store, whether elements with
!                           standard element sequence number IELGRP
!                           are added to the large matrix and large vector or
!                           not.
!                           When IELHLP(IELGRP) = 0 the elements with standard
!                           element sequence number IELGRP are skipped, when
!                           IELHLP(IELGRP) = 1 these elements are added.
!                         Followed by numbers indicated by positions 13-14
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
!     numscl         i    Number of scalars read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=17, nncod2=11
      integer :: ncode1(nncode1), ncodmx1(nncode1), &
                 ncode2(nncod2), ivec, iscal, iprev, m, &
                 itypeboun, ipos, i, maxtype, istartnum
      double precision :: maxtable
      character (len=19) :: code1(nncode1)
      character (len=26) :: code2(nncod2)

!     code1          Table with keywords
!     code2          Table with keywords
!     i              Counting variable
!     ipos           Position in array
!     iprev          Previous value of istart
!     iscal          Sequence number of scalar
!     istartnum      Starting address of a series of numbers in INPSEP
!     itypeboun      See inpsep(12)
!     ivec           Sequence number of vector with respect to array ISOL
!     m              Number of values stored in tabout
!     maxtable       Maximum value found in table
!     maxtype        Last position of subkeywords for type_output in code2
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncod2         Number of entries in code2
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READSCSKIPEL   Interpret information concerning skip element groups
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       DERIVATIVES, options (all in one line)
!           where options may have the following contents
!               SEQ_COEF = s         (Default none)
!               SEQ_DERIV = s        (Default the next one)
!               PROBLEM = s          (Default the next one)
!               VECTOR = s           (Default nprob+1)
!               SCALAR = s           (Default 0)
!               ICHELD = s           (Default 1)
!               INPUT_VECTOR = v
!               IX = j
!               TYPE_OUTPUT = j      (Default solution)
!               POINTS (P1, P2, ...)
!               CURVES (C1, C2, ...)
!               SURFACES (S1, S2, ...)
!               ZERO_LEVEL_SET i
!               ELEMENT_GROUPS = (s1, s2, ...)           (Default all)
!               SKIP_ELEMENT_GROUPS = (s1, s2, ... )     (Default none)
!           Remark: SCALAR and VECTOR are mutually exclusive
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1506   Both VECTOR and SCALAR given in DERIVATIVES
!    2056   Curves found, but not output vector is of type reaction force
!    2057   output vector is of type reaction force, but not curves found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQ_COEF', 'PROBLEM', 'VECTOR', 'SEQ_DERIV', &
                  'SCALAR', 'V', 'ICHELD', 'INPUT_VECTOR', 'IVEC', &
                  'IX', 'TYPE_OUTPUT', 'POINTS', &
                  'CURVES', 'SURFACES', 'ZERO_LEVEL_SET', &
                  'ELEMENT_GROUPS', 'SKIP_ELEMENT_GROUPS' /
      data ncode1  / 6, 4, 6, 6, 1,   1, 3, 4, 4, 2, &
                    7, 4, 4, 4, 7,   4, 4 /
      data ncodmx1 / 8, 7, 6, 9, 6,   1, 6,12, 4, 2, &
                   11, 6, 6, 8,14,  14,19 /
      data code2 / 'SOLUTION', 'SPECIAL', 'ELEMENTWISE', &
                   'RIGHT_HAND_SIDE', 'INTERPOLATE', 'ELEMENT_NODES', &
                   'ELEMENT_INTEGRATION_POINTS', 'REACTION_FORCE', &
                   'P', 'C', 'S' /
      data ncode2 / 3, 4, 4, 4, 5,  11, 11, 5, 1, 1, &
                    1 /

!     code1   Table with sub keywords corresponding to keyword DERIVATIVES
!             Values:
!             1:  SEQ_COEF
!             2:  PROBLEM
!             3:  VECTOR
!             4:  SEQ_DERIV
!             5:  SCALAR
!             6:  V
!             7:  ICHELD
!             8:  INPUT_VECTOR
!             9:  IVEC
!            10:  IX
!            11:  TYPE_OUTPUT
!            12:  POINTS
!            13:  CURVES
!            14:  SURFACES
!            15:  ZERO_LEVEL_SET
!            16:  ELEMENT_GROUPS
!            17:  SKIP_ELEMENT_GROUPS
!     code2  Table with sub keywords corresponding to subkeywords
!             Values:
!             1: SOLUTION
!             2: SPECIAL
!             3: ELEMENTWISE
!             4: RIGHT_HAND_SIDE
!             5: INTERPOLATE
!             6: ELEMENT_NODES
!             7: ELEMENT_INTEGRATION_POINTS
!             8: REACTION_FORCE
!             9: P
!            10: C
!            11: S
! ======================================================================
!
      call eropen ( 'readscderiv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscderiv'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopycodesub ( code2, nncod2, ncode2 )

      maxtype = 8

      itype(1:nncode) = 0
      itype(3) = 38  ! VECTOR
      itype(5) = 39  ! SCALAR
      itype(6) = 38  ! V
      itype(8) = 38  ! INPUT_VECTOR
      itype(11) = 5  ! TYPE_OUTPUT
      itype(12:14) = 18  ! CURVES/POINTS/SURFACES

      itype(16:17) = 11       ! ELEMENT_GROUPS / SKIP_ELEMENT_GROUPS
      tabmin(16:17) = nelgrp
      tabmax(16:17) = nelgrp

      table(1:nncode) = -1d0
      table(1) = 0d0 ! SEQ_COEF
      table(10) = 1d0 ! IX
      table(11) = 1d0 ! TYPE_OUTPUT
      do i = 1, 3
         table(11+i) = -maxtype-i
      end do  ! i = 1, 3

      tabmin(1:nncode) = 1d0
      tabmin(12) = nuspnt
      tabmin(13) = ncurvs
      tabmin(14) = nsurfs

      tabmax(1) = 25d0
      tabmax(2) = nprob
      tabmax(3) = dble(maxvec)
      tabmax(4) = 100d0
      tabmax(5) = dble(maxcls)
      tabmax(6) = tabmax(3)
      tabmax(7) = 1d3
      tabmax(8) = tabmax(3)
      tabmax(9) = 10d0
      tabmax(10) = 3d0
      tabmax(11) = maxtype
      tabmax(12:14) = tabmin(12:14)
      tabmax(15) = 10d0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( ierror/=0 ) go to 1000
      if ( table(3)>0d0 .and. table(6)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 3, 6 )

      table(3) = max(table(3),table(6))

      istartnum = 15             ! numbers are stored from this position

      inpsep(1:istartnum-1) = 0
      inpsep(1) = nint(table(1))
      inpsep(2) = nint(table(2))
      inpsep(4) = nint(table(4))
      ivec  = nint(table(3))
      iscal = mod(nint(table(5)),100000)

      if ( ivec==-1 ) then

!     --- Vector not given

         if ( iscal==-1 ) then
            ivec = nprob+1
         else
            ivec = -2
         end if

      else

!     --- Vector has been given, so scalar is not allowed

         if ( iscal>0 ) then
            call errsub ( 1506, 0, 0, 0 )
            call errcrd
         else
            iscal = 0
         end if
      end if
      inpsep(3) = ivec
      inpsep(5) = iscal

      maxtable = maxval(table(7:8))

      if ( inpsep(4)<0 .and. maxtable>=0d0 ) then

!     --- No sequence number found

         inpsep(4) = -10                    ! sequence number
         inpsep(6) = max(1,nint(table(7)))  ! icheld
         inpsep(7) = max(1,nint(table(8)))  ! input vector
         inpsep(8) = max(0,nint(table(9)))  ! ivec
         inpsep(9) = max(0,nint(table(10))) ! ix
         inpsep(10) = nint(table(11))-1     ! type output vector

!        --- Check on mutually exclusive keywords

         do i = 13, 15

            if ( table(12)>0d0 .and. table(i)>0d0 ) &
               call errkeywexcl ( code, ncodmx, 12, i ) ! points

         end do  ! i = 13, 15

         do i = 14, 15

            if ( table(12)>0d0 .and. table(i)>0d0 ) &
               call errkeywexcl ( code, ncodmx, 13, i ) ! curves

         end do  ! i = 14, 15

         if ( table(15)>0d0 .and. table(14)>0d0 ) &
            call errkeywexcl ( code, ncodmx, 15, 14 )  ! zero_levset, surfaces

         itypeboun = 0
         m = 0
         do i = 12, 14  ! points, curves, surfaces

            if ( table(i)>0d0 ) then

!           --- P, C or S found

               itypeboun = i-11
               m = nint(tabmax(i))
               ipos = nint(table(i))
               inpsep(13) = istartnum
               inpsep(istartnum:istartnum+m-1) = &
                  nint(tabout(ipos:ipos+m-1))
               istartnum = istartnum+m

            end if  ! ( table(i)>0d0 )

         end do  ! i = 12, 15
         if ( table(15)>0d0 ) then

!        --- zero level set

            itypeboun = 20
            m = 1
            inpsep(13) = istartnum
            inpsep(istartnum) = nint(table(15))
            istartnum = istartnum+1

         end if  ! ( table(15)>0d0 )

         if ( debug ) write(irefwr,1) 'itypeboun', itypeboun

         if ( itypeboun>0 ) then

!        --- Curves found, only allowed in combination with reaction force

            if ( inpsep(10)/=7 ) then

!           --- Not reaction force found

               call errchr ( code(11+itypeboun), 1 )
               call errsub ( 2056, 0, 0, 1 )

            end if

         else if ( inpsep(1)==7 .and. itypeboun==0 ) then

!        --- output vector is type reaction force, but not curves found

            call errsub ( 2057, 0, 0, 0 )

         end if

         inpsep(11) = itypeboun
         inpsep(12) = m

!        --- information about skip elements

         call readscskipel ( table(16), tabout, inpsep, istartnum, &
                             14, tabmax(16), nelgrp )

         istart = istart + istartnum  ! Change also askvecderiv

      else

!     --- Sequence number found

         istartnum = 6
         istart = istart + 6

      end if  ! ( inpsep(4)<0 .and. maxtable>=0d0 )

      numvec = min ( max(numvec,ivec), maxvec )
      if ( ivec>0 .and. ivec<=maxvec ) ivectors(ivec) = 1
      numscl = min ( max(numscl,iscal), maxcls )
      if ( iscal>0 .and. iscal<=maxcls ) iscalars(iscal) = 1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), istartnum, 'inpsep' )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscderiv'

      end if
      call erclos ( 'readscderiv' )

      end subroutine readscderiv
