      subroutine readsclevvect ( istart, inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 05-11-2013 Extra parameters
!        version  1.0  date 25-10-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keywords
!     CREATE_LEVELSET_VECTOR
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          0: icode
!                          1: MESH_ORIG Sequence number of mesh to be adapted
!                          2: MESH_SUBDIV Sequence number of adapted mesh
!                          3: IPROB problem number level set function
!                          4: SEQ_LEVELSET Sequence number of levelset vector
!                             in array solut
!                          5: isign
!                             -1: negative inside obstacle
!                              1: positive inside obstacle
!                          6: obstacle sequence number
!                          7: reference to eps_move
!                          8: - &
!                          9: reference to eps_min
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
!                         See inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=8
      integer ncode1(nncode1), ncodmx1(nncode1)
      character (len=15) code1(nncode1)

!     code1          Table with keywords
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININLARGE    Print 1d integer vector (large numbers)
!     READSEQVAR     Put the value of the variable read in rinput
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       CREATE_LEVELSET_VECTOR, options (all in one line)
!           where options may have the following contents
!           MESH_ORIG (Default present mesh)
!           MESH_SUBDIVIDE (Default MESH_ORIG+1)
!           VECTOR = i (Default phi or 2)
!           OBSTACLE i  (Default 1)
!           SIGN_INOBSTACLE = i (Default 1)
!           PROBLEM = i         (Default 2)
!           EPS_MOVE = e  (Default ACCURACY_OBSTACLE)
!           EPS_MIN = e  (Default 0.01 * ACCURACY_OBSTACLE)
!           Present mesh remains mesh_orig
!           mesh_subdiv is removed
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     996   mesh_in and mesh_out are equal
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'MESH_ORIG', 'MESH_SUBDIVIDE', 'PROBLEM', 'VECTOR', &
                  'SIGN_INOBSTACLE', 'OBSTACLE', 'EPS_MOVE', 'EPS_MIN' /
      data ncode1 /  7, 7, 4, 4, 4,   4, 6, 6 /
      data ncodmx1 / 9,14, 7, 6, 8,  15, 8, 7 /

!     code1   Table with sub keywords corresponding to keyword LEVELSET_MESH
!             Values:
!             1:   MESH_ORIG
!             2:   MESH_SUBDIV
!             3:   PROBLEM
!             4:   VECTOR
!             5:   SIGN_INOBSTACLE
!             6:   OBSTACLE
!             7:   EPS_MOVE
!             8:   EPS_MIN
! ======================================================================
!
      call eropen ( 'readsclevvect' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsclevvect'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode  ! code

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

      table(1:nncode) = -1d0
      itype(1:nncode) = 0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 5d0

      tabmin(3) = 1d0   ! PROBLEM
      tabmax(3) = 1d1
      table(3) = 2d0

      itype(4) = 38       ! VECTOR

      tabmin(5) =-1d0   ! SIGN_INOBSTACLE
      tabmax(5) = 1d0
      table(5) =  1d0

      tabmin(6) = 1d0   ! OBSTACLE
      tabmax(6) = 1d1
      table(6) = 1d0

      itype(7:8) = 1   ! EPS_MOVE, EPS_MIN
      tabmin(7:8) = 0d0
      tabmax(7:8) = 0.5d0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      inpsep(1) = nint(table(1))  !  MESH_ORIG
      inpsep(2) = nint(table(2))  !  MESH_SUBDIV

      if ( table(2)>0d0 ) then

!     --- MESH_SUBDIV found

         inpsep(2) = nint(table(2))

      else if ( table(1)>0d0 ) then

!     --- MESH_SUBDIV not found

         inpsep(2) = inpsep(1)+1

      end if  ! ( table(2)>0d0 )

      if ( inpsep(1)>0 .and. inpsep(1)==inpsep(2) ) then

!     --- mesh_orig and mesh_subdiv are the same

         call errint ( inpsep(1), 1 )
         call errsub ( 996, 1, 0, 0 )

      end if
      inpsep(3) = nint(table(3))  !  IPROB

      call readseqvector ( table(4), 'phi', inpsep(4), 2 )  ! vector
      inpsep(5:6) = nint(table(5:6))  ! isign, iobst

      icode = 7
      call readseqvar ( 'EPS_MOVE', rinsep, inpsep(7), &
                        istrin, accuracy_obstacle )
      icode = 8
      call readseqvar ( 'EPS_MIN', rinsep, inpsep(9), &
                        istrin, 1d-2*accuracy_obstacle )

!     --- raise istart
!         if this is changed also change prlevmesh and prrmlevmesh

      istart = istart+10

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart/end', istart
         call prininlarge ( inpsep(0), 10, 'inpsep' )
         write(irefwr,*) 'End readsclevvect'

      end if  ! ( debug )
      call erclos ( 'readsclevvect' )

      end subroutine readsclevvect
