      subroutine readsclinsol ( istart, inpsep, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.9  date 23-01-2014 extra call to readscnewvec
!        version  1.8  date 18-11-2013 extra option transform_bc
!        version  1.7  date 13-11-2010 error 1488 --> errkeywexcl
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     SOLVE_LINEAR_PROBLEM
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), nprob

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          1: contains the sequence number for the coefficients
!                          2  IPROB   Problem sequence number
!                                     If iprob = -1, the next one will be used
!                          3  IVECTOR Vector sequence number (first vector)
!                                     If ivector = -1, the same value as iprob
!                                     will be used
!                          4: contains the sequence number for the solver
!                          5: indicates if defect correction should be used:
!                             0: no defect correction
!                             1: defect correction
!                          6: indicates if quadratic elements should be
!                             treated as a cluster of linear subelements
!                             0: no subelements
!                             1: subelements
!                          7: IVEC_REACT indicates if reaction forces must be
!                             computed (>0) or not (0)
!                             If >0, IVEC_REACT defines the sequence number
!                             of the vector in which the reaction forces
!                             must be stored
!                          8: IVEC_FEM is only used in case of a spectral mesh
!                             It indicates if also the fem solution vector
!                             must be stored besides the sem solution vector
!                             if>0, IVEC_FEM defines the sequence number
!                             of the fem solution vector
!                          9: Indicates if a FEM preconditioning must be applied
!                             in case of a spectral mesh
!                             Possible values:
!                             0: no fem preconditioning
!                             1: fem preconditioning
!                         10: KEEP_MAT indicates if the matrix
!                             must be kept, destroyed or reused
!                             Possible values:
!                             0: new and destroy
!                             1: new and keep
!                             2: old and destroy
!                             3: old and keep
!                         11: KEEP_RHS indicates if the right-hand side
!                             must be kept, destroyed or reused
!                             Possible values:
!                             0: new and destroy
!                             1: new and keep
!                             2: old and destroy
!                             3: old and keep
!                         12: TEST_FORCE
!                         13: ISEQ_RHS If>0, the right-hand side
!                             vector already exists and is stored in isol
!                             with sequence number ISEQ_RHS
!                         14: TRANSFORM_BOUNDARY_CONDITIONS If>0, the boundary
!                             conditions are transformed to local coordinates
!                         If this number is extended adapt prbearmasscon
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=19
      integer ncode2(nncode2), ncodmx2(nncode2), ivec, i, iposv
      character (len=29) code2(nncode2)

!     code2          Table with keywords
!     i              Counting variable
!     iposv          position of keyword v in array code2
!     ivec           Sequence number of vector with respect to array ISOL
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncodmx2        Table containing the number of maximum characters in
!                    the keyword table code2
!     nncode2         Number of entries in code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSCNEWVEC   Check if text following main keyword is not a keyword
!                    or a known vector
!                    Make a new vector if necessary
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       SOLVE_LINEAR_SYSTEM, options (all in one line)
!           where options may have the following contents
!               SEQ_COEF = s         (Default the next one)
!               SEQ_SOLVE = s        (Default the next one)
!               PROBLEM = s          (Default the next one or if vector
!                                     is given and has been filled
!                                     before: the one corresponding to
!                                     vector)
!               VECTOR = s           (Default the same as problem)
!               DEFECT_CORRECTION
!               LINEAR_SUBELEMENTS
!               REACTION_FORCE = s   (Default none)
!               FEM_PRECONDITIONING  (Default not)
!               FEM_VECTOR = s       (Default the same as VECTOR)
!               KEEP_MATRIX, DESTROY_MATRIX  (Default DESTROY)
!               REUSE_MATRIX         (Default not)
!               KEEP_VECTOR, DESTROY_VECTOR (Default DESTROY)
!               REUSE_VECTOR         (Default not)
!               TEST_FORCE = s       (Default not)
!               ISEQ_RHS = s         (Default 0 )
!               TRANSFORM_BOUNDARY_CONDITIONS  (Default not)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2108   FEM preconditioning required, but there is no spectral mesh
!    2109   FEM solution vector required, but there is no FEM preconditioning
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'SEQ_COEF', 'PROBLEM', 'VECTOR', 'SEQ_SOLVE', &
                  'DEFECT_CORRECTION', 'LINEAR_SUBELEMENTS', &
                  'REACTION_FORCE', 'FEM_VECTOR', &
                  'FEM_PRECONDITIONING', &
                  'KEEP_MATRIX', 'DESTROY_MATRIX', 'REUSE_MATRIX', &
                  'KEEP_VECTOR', 'DESTROY_VECTOR', 'REUSE_VECTOR', &
                  'TEST_FORCE', 'ISEQ_RHS', &
                  'TRANSFORM_BOUNDARY_CONDITIONS', &
                  'V' /
      data ncode2 / 6, 4, 6, 6, 5,    5, 4, 7, 8, 8, &
                  11, 9, 8,11, 9,    8, 8, 8, 1 /
      data ncodmx2 / 8, 7, 6, 9,17,   18,14,10,19,11, &
                   14,12,11,14,12,   10, 8,18, 1 /

!     code2   Table with sub keywords corresponding to  keyword SOLVE_LINEAR
!             Values:
!             1:  SEQ_COEF
!             2:  PROBLEM
!             3:  VECTOR
!             4:  SEQ_SOLVE
!             5:  DEFECT_CORRECTION
!             6:  LINEAR_SUBELEMENTS
!             7:  REACTION_FORCE
!             8:  FEM_VECTOR
!             9:  FEM_PRECONDITIONING
!            10:  KEEP_MATRIX
!            11:  DESTROY_MATRIX
!            12:  REUSE_MATRIX
!            13:  KEEP_VECTOR
!            14:  DESTROY_VECTOR
!            15:  REUSE_VECTOR
!            16:  TEST_FORCE
!            17:  ISEQ_RHS
!            18:  TRANSFORM_BOUNDARY_CONDITIONS
!            19:  V
! ======================================================================
!
      call eropen ( 'readsclinsol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsclinsol'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

      inpsep(0) = icode  ! Fill icode into inpsep

!     --- First we check if the second item is an not yet defined vector
!         If so the vector is created

      call readscnewvec

!     --- Next we interpret the complete input

      iposv = 19         ! position of keyword v

      table(1:nncode) = 0d0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 0d0
      itype(1:nncode) = 2

      table(1:4) = -1d0    ! seq_coef ... seq_solve
      table(iposv) = -1d0  ! v

      tabmax(1) = 25d0     ! max number of coefficients
      itype(1:2) = 0
      tabmax(2) = nprob    ! max number of problems
      tabmax(3) = dble(maxvec)  ! vector
      itype(3) = 38
      tabmax(4) = 25d0              ! seq_solve
      itype(4) = 0

      tabmax(7) = dble(maxvec)  ! reaction_force
      tabmin(7) = -dble(maxvec)
      itype(7:8) = 0
      tabmax(8) = dble(maxvec)  ! fem_vector

      tabmax(16) = dble(maxvec) ! test_force
      itype(16:17) = 0
      tabmax(17) = dble(maxvec) ! iseq_rhs

      tabmax(iposv) = dble(maxvec)
      itype(iposv) = 38

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) call prinrl ( table, iposv, 'table' )
      if ( table(3)>0d0 .and. table(iposv)>0d0 ) &
         call errkeywexcl ( code2, ncodmx2, 3, iposv )

      table(3) = max(table(3),table(iposv))
      inpsep(1:9) = nint(table(1:9))

      if ( inpsep(9)==1 ) then

!     --- Finite element preconditioning required, check kmesh

         if ( nspec==0 ) then

!        --- warning no spectral mesh

            call errwar ( 2108, 0, 0, 0 )
            inpsep(9) = 0

         end if

      end if
      if ( inpsep(8)>0 .and. inpsep(9)==0 ) then

!     --- Finite element solution vector, but there is no spectral
!         preconditioning

         call errwar ( 2109, 0, 0, 0 )

      end if
      do i = 3, 7, 4
         ivec = abs(nint(table(i)))
         if ( ivec>0 ) then
            numvec = min ( max(numvec,ivec), maxvec )
            if ( ivec<=maxvec ) ivectors(ivec) = 1
         end if
      end do
      i = 8
      ivec = abs(nint(table(i)))
      if ( ivec>0 ) then
         numvec = min ( max(numvec,ivec), maxvec )
         if ( ivec<=maxvec ) ivectors(ivec) = 1
      end if

!     --- Keywords keep/destroy matrix

      if ( table(10)>0d0 ) then

!     --- keep matrix found

         if ( table(11)>0d0 ) &
            call errkeywexcl ( code2, ncodmx2, 10, 11 )

         if ( table(12)>0d0 ) then

!        --- Reuse found, hence old and keep

            inpsep(10) = 3

         else

!        --- Not reuse found, hence new and keep

            inpsep(10) = 1

         end if  ! ( table(13)>0d0 )

      else if ( table(11)>0d0 ) then

!     --- destroy matrix found

         if ( table(12)>0d0 ) then

!        --- Reuse found, hence old and destroy

            inpsep(10) = 2

         else

!        --- Not reuse found, hence new and destroy

            inpsep(10) = 0

         end if  ! ( table(13)>0d0 )

      else if ( table(12)>0d0 ) then

!     --- Reuse matrix found, keep

         inpsep(10) = 3

      else

!     --- Standard situation: new and destroy

         inpsep(10) = 0

      end if  ! ( table(10)>0 )

!     --- Keywords keep/destroy rhside

      if ( table(13)>0d0 ) then

!     --- keep rhs found

         if ( table(14)>0d0 ) &
            call errkeywexcl ( code2, ncodmx2, 13, 14 )

         if ( table(15)>0d0 ) then

!        --- Reuse found, hence old and keep

            inpsep(11) = 3

         else

!        --- Not reuse found, hence new and keep

            inpsep(11) = 1

         end if  ! ( table(15)>0d0 )

      else if ( table(14)>0d0 ) then

!     --- destroy rhs found

         if ( table(15)>0d0 ) then

!        --- Reuse found, hence old and destroy

            inpsep(11) = 2

         else

!        --- Not reuse found, hence new and destroy

            inpsep(11) = 0

         end if  ! ( table(15)>0d0 )

      else if ( table(15)>0d0 ) then

!     --- Reuse rhs found, keep

         inpsep(11) = 3

      else

!     --- Standard situation: new and destroy

         inpsep(11) = 0

      end if  ! ( table(10)>0 )

      inpsep(12:14) = nint(table(16:18))  ! test_force ... transform_bc

      istart = istart + 15  ! Remark: if this number is enlarged, also adapt
                            ! ini016

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         call prinin ( inpsep(1), 14, 'inpsep' )
         write(irefwr,*) 'End readsclinsol'

      end if  ! ( debug )
      call erclos ( 'readsclinsol' )

      end
