      subroutine readscmvobst ( istart, inpsep, istrin, rinsep, name_keyw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.4  date 22-11-2004 Fortran 90
!        version  1.3  date 06-08-2004 Extension with origin
!
!   copyright (c) 2001-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the keyword move_obstacle with respect to
!   subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     obstacle
!     displacement
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision :: rinsep(*)
      character (len=*) :: name_keyw

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-3 are used:
!                          1: ICODE
!                          2: IOBST
!                          3: Sequence number first position in RINSEP
!                          4: if>0 reference to scalar w.r.t. u_veloc
!                          5: if>0 reference to scalar w.r.t. v_veloc
!                          6: if>0 reference to scalar w.r.t. w_veloc
!                          7: if>0 reference to scalar w.r.t. phi_veloc
!                          8: Rotation plane 1: (xy) 2: (yz) 3: (xz)
!                          9: Indicates if positions 4 to 6 refer to a
!                             velocity (0) or displacement (1)
!                         10: if>0 reference to scalar w.r.t. x_orig
!                         11: if>0 reference to scalar w.r.t. y_orig
!                         12: if>0 reference to scalar w.r.t. z-orig
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     name_keyw      i    Name of keyword
!     rinsep        i/o   Array containing the real input for this subroutine
!                         1:  u_x  displacement in x-direction
!                         2:  u_y  displacement in y-direction
!                         3:  u_z  displacement in z-direction
!                         4:  u_phi rotation velocity
!                         5:  x_orig
!                         6:  y_orig
!                         7:  z-orig
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=5
      character (len=17) code1(nncode1)
      integer ncode1(nncode1), ncodmx1(nncode1), &
              num, i, iseq, istrinsav

!     code1          Table with keywords
!     i              Counting variable
!     iseq           Sequence number in table
!     istrinsav      Help parameter to save the value of istrin at input
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1       Maximum number of characters in a keyword
!     nncode1        Number of entries in code1
!     num            Number of numbers read between brackets
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READINTEGER    Read an integer from the input file
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       MOVE_OBSTACLE i, options (all in one line)
!           where options may have the following contents
!           velocity = ( u_1, u_2, u_3)
!           rotation_plane = xz
!           rotation_velocity = u_phi
!           displacement = ( u_1, u_2, u_3)
!           origin = (o_x, o_y, o_z)
!       The option displacement and velocity are mutually exclusive
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1134   keyword not available in combination with present keyword
!    2233   number of reals read incorrect
!    2402   The option rotation_velocity has only been implemented for ndim = 3
!    2403   The option rotation_plane has been found, but no rotation velocity
!    2404   The option rotation_plane has only been implemented for rotation in
!           the x-z plane
!    2660   origin found, but no rotation velocity
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initializations
!     read integer obstacle number
!     read rest of record
!     Fill inpsep and rinsep
!     Raise istart and istrin
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VELOCITY', 'ROTATION_VELOCITY', 'ROTATION_PLANE', &
                  'DISPLACEMENT', 'ORIGIN' /
      data ncode1   / 5, 12, 12,  5,  4 /
      data ncodmx1 / 8, 17, 14, 12,  6 /
! ======================================================================
!
      call eropen ( 'readscmvobst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscmvobst'
         write(irefwr,*) 'numobst, istrin, istart ', &
                          numobst, istrin, istart
         istrinsav = istrin

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      itype(1:nncode) = 19
      itype(3) = 5
      codesub(1) = 'XY'
      codesub(2) = 'YZ'
      codesub(3) = 'XZ'
      ncodesub(1:3) = 2

!     --- First read integer obstacle number

      inpsep(istart) = icode
      inpsep(istart+1:istart+11) = 0
      rinsep(istrin:istrin+6) = 0d0
      call readinteger ( inpsep(istart+1), 1, numobst, name_keyw )

!     --- Next read rest of record

      table(1:nncode) = -1d0
      tabmin(1) = ndim
      tabmax(1) = 1d10
      tabmin(2) = 1d0
      tabmax(2) = 2d0*pi
      tabmin(3) = 1d0
      tabmax(3) = 3d0
      tabmin(4) = ndim
      tabmax(4) = 1d10
      tabmin(5) = ndim
      tabmax(5) = 1d10
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

!     --- Fill inpsep and rinsep

      if ( table(1)<=0d0 .and. table(4)<=0d0 ) then

!     --- No velocity or displacement found, error

         call errchr ( code(1)(1:ncodmx(1)), 1 )
         call errchr ( name_keyw, 2 )
         call errsub ( 1134, 0, 0, 2 )

      else

!     --- Velocity or displacement found

         if ( table(1)>0 ) then

!        --- velocity found, check if not displacement found

            if ( table(4)>0 ) &
               call errkeywexcl ( code, ncodmx, 1, 4 )

            iseq = 1

         else

!        --- displacement found

            iseq = 4
            inpsep(istart+8) = 1

         end if
         inpsep(istart+2) = istrin
         iposut = nint(table(iseq))
         num = nint(tabmax(iseq))
         if ( num/=ndim ) then

!        --- Not the correct number of reals read

            call errchr ( code(iseq)(1:ncodmx(iseq)), 1 )
            call errint ( num, 1 )
            call errint ( ndim, 2 )
            call errsub ( 2233, 2, 0, 1 )

         end if
         do i = 0, num-1

            if ( infotabout(iposut+i)==11 ) then

!           --- scalar found

               inpsep(istart+3+i) = nint(tabout(iposut+i))-100000000
               rinsep(i+istrin) = scalars(inpsep(istart+3+i))

            else

!           --- real found

               rinsep(i+istrin) = tabout(iposut+i)

            end if

         end do

!        --- raise istrin

         istrin = istrin+num

      end if
      if ( debug ) write(irefwr,*) 'istrin ', istrin

!     --- Rotation velocity

      inpsep(istart+6) = 0
      if ( table(2)>0d0 ) then

!     --- Rotation velocity found

         iposut = nint(table(2))
         if ( infotabout(iposut)==11 ) then

!        --- scalar found

            inpsep(istart+6) = nint(tabout(iposut))-100000000
            rinsep(istrin) = scalars(inpsep(istart+6))

         else

!        --- real found

            rinsep(istrin) = tabout(iposut)
            inpsep(istart+6) = -1

         end if
         istrin = istrin+1

         if ( ndim/=3 ) then

!        --- Only implemented for ndim = 3

            call errint ( ndim, 1 )
            call errsub ( 2402, 1, 0 , 0 )

         end if

      end if

      inpsep(istart+7) = 0
      if ( table(3)>0d0 ) then

!     --- Rotation plane found

         inpsep(istart+7) = nint(table(3))
         if ( inpsep(istart+6)==0 ) then

!        --- Not rotation velocity found

            call errwar ( 2403, 0, 0, 0 )

         end if
         if ( inpsep(istart+7)/=1 ) then

!        --- Rotation velocity has only been implemented for rotation in xy
!            plane

            call errsub ( 2404, 0, 0, 0 )

         end if

      end if

      if ( table(5)>0d0 ) then

!     --- origin found

         if ( inpsep(istart+6)==0 ) then

!        --- Not rotation velocity found

            call errwar ( 2660, 0, 0, 0 )

         end if
         iposut = nint(table(5))
         num = nint(tabmax(5))
         if ( num/=ndim ) then

!        --- Not the correct number of reals read

            call errchr ( code(iseq)(1:ncodmx(iseq)), 1 )
            call errint ( num, 1 )
            call errint ( ndim, 2 )
            call errsub ( 2233, 2, 0, 1 )

         end if
         do i = 0, num-1

            if ( infotabout(iposut+i)==11 ) then

!           --- scalar found

               inpsep(istart+8+i) = nint(tabout(iposut+i))-100000000
               rinsep(i+istrin) = scalars(inpsep(istart+8+i))

            else

!           --- real found

               rinsep(i+istrin) = tabout(iposut+i)

            end if

         end do

      end if  ! ( table(5)>0d0 )

!     --- raise istrin

      istrin = istrin+num

!     --- raise istart

      istart = istart+12

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(istart-11), 11, 'inpsep' )
         write(irefwr,*) 'istrin, istrinsav ', istrin, istrinsav
         call prinrl ( rinsep(istrinsav), istrin-istrinsav, 'rinsep' )
         write(irefwr,*) 'End readscmvobst'

      end if
      call erclos ( 'readscmvobst' )

      end
