      subroutine readsctimhis ( istart, inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 16-04-2014 Extension with idegfd
!        version  1.1  date 14-10-2012 Extension with points
!        version  1.0  date 13-05-2010
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the option following the keyword TIME_HISTORY
!     with respect to subroutine readsc
!     The input is stored in array inpsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin
      double precision rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         Contents:
!                         0:  icode       Return code indicates time history
!                         1:  ivect       Sequence number of vector
!                         2:  itypenodes  Defines how coordinates are found
!                                         1: coordinates are given
!                                            the closest point is used
!                                         2: user points are given
!                                         3: zero_level set
!                         3:  istrin      Starting address of coordinates in
!                                         rintim (itypenodes=2)
!                                         or
!                                         starting address of user points in
!                                         inpsep
!                         4:  npoint_hist Number of points used
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep         i    Array containing the real input for this subroutine
!                         Contents from position istrin:
!                         coorindates of the npoins points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=4
      integer ncode1(nncode1), ncodmx1(nncode1), i, &
              npoint_hist, iprev
      character (len=14) code1(nncode1)
      character (len=12) namemain

!     code1          Table with keywords
!     i              Counting variable
!     iprev          Previous value if istart
!     namemain       Name of main keyword
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
!     npoint_hist    Number of points in ivecttim
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following text is recognized
!
!     TIME_HISTORY, vector = j, &
!         coordinates ((x1,y1,z1),(x2,y2,z2),...(xn,yn,zn))
!         points ( p1, p2, ... )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2227   Only one of a combination of keywords found
!    3118   zero level set only available for ndim = 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VECTOR', 'COORDINATES', 'POINTS', 'ZERO_LEVEL_SET' /
      data ncode1 /  4, 4, 3, 4 /
      data ncodmx1 /  6,11, 6,14 /
      data namemain  / 'TIME_HISTORY' /

!     code1   Table with sub keywords corresponding to keyword TIME_HISTORY
!             Values:
!             1:  VECTOR
!             2:  COORDINATES
!             3:  POINTS
!             4:  ZERO_LEVEL_SET
! ======================================================================
!
      call eropen ( 'readsctimhis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsctimhis'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode          ! code with respect to tosepcomsub

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

!     --- Get parameters

      iprev = istart

!     --- Fill tables for intx02
!         The keywords LINE and CURVES are just skipped

      table(1:nncode) = -1d20
      tabmin(1:nncode) = 1d0
      itype(1:nncode) = 0
      codesub(1) = 'P'
      ncodesub(1) = 1

!     --- Vector

      tabmax(1) = dble(maxvec)
      itype(1) = 38

!     --- Coordinates

      tabmin(2) = dble(ndim)       ! ndim coordinates per point
      tabmax(2) = 1d3              ! maximum number of points
      itype(2) = 17

!     --- Points

      table(3) = -1d0              ! position of P in code1
      itype(3) = 18                ! points = (pi, pj, ... )
      tabmin(3) = nuspnt
      tabmax(3) = nuspnt

!     --- Zero level set

      tabmax(4) = 1d0

!     --- Read input

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

!     --- Fill array inpsep

      if ( table(1)<-1003d0 .or. table(1)>-1d3 .and. table(1)<1d0 ) then

!     --- Error: no vector sequence number found

         call errchr ( namemain, 1 )
         call errchr ( code(1)(1:ncodmx(1)), 2 )
         call errsub ( 2227, 0, 0, 2 )

      else

!     --- vector sequence number found

         inpsep(1) = mod(nint(table(1)),100000000)

      end if  ! ( table(1)<1 )

      if ( table(2)>0d0 ) then

!     --- coordinates found

         iposut = nint(table(2))     ! start address in tabout
         npoint_hist = nint(tabmax(2))    ! number of points
         inpsep(2) = 1             ! coordinates
         inpsep(3) = istrin        ! start address in rinsep
         inpsep(4) = npoint_hist   ! number of points
         rinsep(istrin:istrin+npoint_hist*ndim-1) = &
            tabout(iposut:iposut+npoint_hist*ndim-1)
         istart = istart+5

         do i = 3, 4
            if ( table(i)>0d0 ) &
               call errkeywexcl ( code, ncodmx, 2, i )
         end do  ! i = 3, 4

      else if ( table(3)>0d0 ) then

!     --- points found

         iposut = nint(table(3))     ! start address in tabout
         npoint_hist = nint(tabmax(3))    ! number of points
         inpsep(2) = 2             ! points
         inpsep(3) = 4             ! starting address in inpsep
         inpsep(4) = npoint_hist   ! number of points
         inpsep(5:4+npoint_hist) = &
            nint(tabout(iposut:iposut+npoint_hist-1))  ! points
         istart = istart+5+npoint_hist
         if ( table(4)>0d0 ) &
            call errkeywexcl ( code, ncodmx, 3, 4 )

      else if ( table(4)>0d0 ) then

!     --- zero level set found

         inpsep(2) = 3               ! zero level set
         inpsep(3) = nint(table(4))  ! seq number
         inpsep(4) = 1               ! number of points
         if ( ndim/=1 ) then

!        --- ndim # 1

            call errint ( ndim, 1)
            call errsub ( 3118, 1, 0, 0 )

         end if  ! ( ndim/=1 )
         istart = istart+5

      else

!     --- Error: no coordinates found

         call errchr ( namemain, 1 )
         call errchr ( code(2)(1:ncodmx(2)), 2 )
         call errsub ( 2227, 0, 0, 2 )

      end if  ! ( table(1)<1 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), istart-iprev+1, 'inpsep' )
         if ( inpsep(2)==1 ) &
            call prinrl1 ( rinsep(istrin), npoint_hist, ndim, 'rinsep' )
         write(irefwr,*) 'End readsctimhis'

      end if
      call erclos ( 'readsctimhis' )

!     --- Raise parameters

      if ( inpsep(2)==1 ) istrin = istrin+npoint_hist*ndim

      end subroutine readsctimhis
