      subroutine to0089 ( ictime, iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.3  date 23-02-2016 Replace solve1 by seplinsol
!        version  5.2  date 29-05-2014 remove call to time subroutine
!        version  5.1  date 28-05-2014 New call to solvelbf
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   TO0089 constructs the matrix and right-hand side for a linear
!   problem and solves it.
!   Depending on the value of ictime coefficients are read and stored.
!   TO0089 replaces the calls of the SEPRAN subroutines:
!      FILCOF: Fill coefficients for differential equation and
!              natural boundary conditions
!      BUILD : Build matrix and right-hand side
!      SOLVE : Solve system of linear equations
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     matrix
!     right_hand_side
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulevecs
      use sepmodulesmf
      use sepmoduleelem
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulesolve
      use sepmodulekprob
      use sepmoduleplot
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ictime, iinsol(*)

!     ictime         i    Indication if information must be read from standard
!                         input file ( <=0 ) or that information read before is
!                         used ( 1 )
!                         Other possibilities
!                             2 : The same as 0, however, the matrix and
!                                 right-hand- side vector are not removed
!                             3 : The same as 1, however, the matrix and
!                                 right-hand- side vector are not removed
!                             4 : The same as 3, however, only the
!                                 right-hand-side vector is filled, the matrix
!                                 of a previous call is used.
!                             5 : The same as 1, however, only the
!                                 right-hand-side vector is filled, the matrix
!                                 of a previous call is used.
!                             6 : The same as 2, however, only the
!                                 right-hand-side vector is filled, the matrix
!                                 of a previous call is used.
!                             7 : The same as 0, however, only the
!                                 right-hand-side vector is filled, the matrix
!                                 of a previous call is used.
!     iinsol         i    Input array defined by the user
!                         With this array the user may indicate which actions
!                         he wants to be performed by LINPRB
!                         See subroutine linprbnew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rinsol1(1)
      integer iprecfem, inpsol1(1), i, iinbldlc(17), &
              iseqcf, iseqsl, ivectr, maxvct, idum, &
              ivecreac, ivecfem, iseqkmesh, iread

!     i              Counting variable
!     idum           Dummy
!     iinbldlc       Choice parameter for subroutine BUILD
!                    IINBLD must be filled as follows
!                     1 Number of entries of IINBLD that are filled by the user.
!                       If IINBLD(1) = 0, only defaults are used
!                     2 JCHOIS    general choice parameter, possibilities:
!                              1  Both the large matrix and large vector are
!                                 built.
!                              2  Only the large vector is built.
!                              3  The large matrix is built and the large vector
!                                 is made equal to zero.
!                                 If EFFBOUNCOND = 0 the large vector may be
!                                 unequal to zero, due to
!                                 non-homogeneous essential boundary conditions.
!                              5  The large matrix is not built;
!                                 the large vector is made equal to zero.
!                                 If EFFBOUNCOND = 0 the large vector may be
!                                 unequal to zero, due to
!                                 non-homogeneous essential boundary conditions.
!                             14  The large matrix is built;
!                                 no right-hand-side vector is made.
!                     3 EFFBOUNCOND   Indication whether the effect of essential
!                                 boundary  conditions must be taken into
!                                 account (0) or not (1).
!                     4 IMAS      Indication if the mass matrix must be computed
!                                 (>1) or not (0)
!                                 If IMAS=1, the mass matrix is stored as a
!                                 diagonal matrix, if IMAS=2 the mass matrix is
!                                 stored as a large matrix. The structure of
!                                 this matrix must be stored in intmat1d(.,2)
!                     5 IPREC     Indication if the old solution is stored in
!                                 ISLOLD (1) or not (0)
!                                 IPREC is not used if IINBLD(10)>0
!                     6 ALLELEMENTS  Indication if all standard element
!                                 sequence numbers are used (0) or not (1)
!                                 In that case the positions 16 to 15+NELGRP+ &
!                                 NUMNATBND   (NELGRP is number of element
!                                 groups and NUMNATBND the number of boundary
!                                 element groups) are considered as an array
!                                 IELHLP to be filled by the user in the
!                                 following way:
!                                 In IELHLP(IELGRP)
!                                 ( 1<=IELGRP<=NELGRP+NUMNATBND )
!                                 the user must store, whether elements with
!                                 standard element sequence number IELGRP (or
!                                 IELGRP = NELGRP + IBNGRP for boundary
!                                 elements) are added to the large matrix and
!                                 large vector or not.
!                                 When IELHLP(IELGRP) = 0 the elements with
!                                 standard element sequence number IELGRP are
!                                 skipped, when IELHLP(IELGRP) = 1 these
!                                 elements are added.
!                     7 ADDMATVEC    When ADDMATVEC = 0 the large matrix and
!                                 vector are cleared before the assembling
!                                 process.
!                                 When ADDMATVEC = 1 the right-hand-side vector
!                                 and the large matrix are not cleared, &
!                                 but added to the already existing right-hand
!                                 side vector and large matrix.
!                                 Hence        RHSD := RHSD + DELTA  RHSD
!                                              MATRIX := MATRIX +  DELTA  MATRIX
!                     8  CLEARLASTRHSD  When CLEARLASTRHSD = 0 the last part of
!                                 the right-hand-side vector corresponding to
!                                 the boundary conditions is cleared
!                                 (standard situation)
!                                 when CLEARLASTRHSD = 1 the standard assembling
!                                 process is also applied to this last part of
!                                 the right-hand-side vector. For most
!                                 applications CLEARLASTRHSD = 0 suffices.
!                     9  IPROB    Problem definition number
!                    10  NUMOLD   Number of arrays that have been filled in
!                                 array ISLOLD
!                    11  ISPECTRAL Indication if a spectral element method is
!                                 used needing extra information (1) or not (0)
!                    12  NRIGHTHS Number of right-hand sides that must be
!                                 computed by subroutine BUILD in this call.
!                                 This parameter is only used if JCHOIS = 1 or 2
!                                 For all these vectors the other parameters in
!                                 IINBLD are used in exactly the same way.
!                                 The information about the right-hand sides is
!                                 stored in the following way:
!                                 f  refers to rhsd(.,i)  . = 1 (1) 5
!                                  i
!                                 Default value: 1
!                    13  ISEQIN   Sequence number of old solution in array
!                                 ISLOLD
!                    14  IDEF_COR indicates if defect correction should be used
!                                 0: no defect correction
!                                 1: defect correction
!                    15  ISUBDIVIDE indicates if quadratic elements must be
!                                 treated as a conglomerate of linear elements
!                                 (1) or as quadratice elements itself (0)
!     inpsol1        Contains integer information concerning the solution
!                    process for subroutine SOLVEL.
!                    If iread = -2, inpsol1 is filled by SOLVEL, if iread = -1
!                    it is used
!     iprecfem       Indicates if a FEM preconditioning must be applied
!                    in case of a spectral mesh
!                    Possible values:
!                    0: no fem preconditioning
!                    1: fem preconditioning
!     iread          With this parameter it is indicated if array INPSOL is used
!                    in SOLVEL or that input is read from the standard INPUT
!                    file
!                    Possible values:
!                    0   The input is read from the standard input file
!                    >0  The input as read from the standard input file by
!                        subroutine SEPSTN is used. The value of iread
!                        defines the sequence number
!                    -1  No input is read, all information from array INPSOL
!                        is used
!                    -2  The input for SOLVEL is read from the standard input
!                        file. This input is stored in array INPSOL
!     iseqcf         Sequence number with respect to the storage in array ISEPIN
!                    for coefficients
!     iseqkmesh      Sequence number of mesh in kmesh
!     iseqsl         Sequence number with respect to the storage in array ISEPIN
!                    for linear solver
!     ivecfem        Sequence number of FEM solution vector
!     ivecreac       Sequence number of the reaction force vector with respect
!                    to array isolut (if >0)
!                    If (<0) the right-hand side contribution is added
!     ivectr         Sequence number of the solution vector with respect to
!                    array isolut
!     maxvct         Maximum number of vectors stored in isolut
!     rinsol1        Array containing the real user input if iread=-1
!                    The storage of rinsol1 depends on iinsol.
!                    The following positions are used
!                    1. eps      Required accuracy for the iteration process
!                    2. droptol  Drop tolerance with respect to Y12M
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPBUILD       Build matrix and right-hand side
!     SEPCOPYMT      Copy matrix
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPFILLCOEFFS  Fill coefficients at the end of the buffer array
!                    with starting addresses ipiusr and ipuser
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPLINSOL      Body of linear solver
!     SEPRMMATRIX    Remove array with sequence number iseq from realm
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLVEINPUT  Fill arrays inpsol and rinsol
!     TO0089CHECK    Perform some check with respect to subroutine to0089
!     TO0089CONSTS   Extract some constants from array iinsol
!     TO0089REAC     Compute the reaction forces in the linear system of
!                    equations
!     TO0089SEM      Compute SEM solution by FEM preconditioning
!     TO0089TEXT     Write some text information for subroutine to0089 (build)
!                    depending on output parameters
!     TO0089TEXT1    Write some text information for subroutine to0089 (solve)
!                    depending on output parameters
!     TOCOMPTSTFORCE Compute test force in outer boundary
! **********************************************************************
!
!                       I/O
!
!     All I/O is performed by subroutines ERLINS and READ03
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089'
         call prinin ( iinsol, 19, 'iinsol' )
         write(irefwr,1) 'ictime', ictime
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check input

      iseqkmesh = 1
      iseqintmat = max(1,iinsol(19))  ! sequence number intmat1d
      iinsol(20) = ictime

      intmat => intmat1d(iseqintmat)
      kmesh => kmesh2d(meshnr)
      stiffmat => stiffm

!     --- Perform some checks

      call to0089check ( ictime, iseqkmesh, iinsol )
      if ( ierror/=0 ) go to 1000
      iseqkmesh = iseqkmesh+meshnr-1

!     --- Clear arrays

      if ( iinsol(16)==1 .or. iinsol(16)==0 .and. &
           ( ictime==2 .or. ictime==3 .or. ictime<=0) ) then
         if ( debug ) write(irefwr,*) 'Set rhsd to 0'
         irhsd(1:2) = 0
      else if ( debug ) then
         write(irefwr,*) 'Old rhsd'
         call prinin ( irhsd(1), 5, 'irhsd' )
      end if

!     --- extract constants from iinsol and fill iinbld partly

      call to0089consts ( iinsol, isubr, iread, iseqcf, iseqsl, &
                          iprob, ivectr, maxvct, ivecreac, &
                          ivecfem, iprecfem, iinbldlc, ictime )

      inpsol1(1) = 0

!     --- Fill coefficients in IUSER and USER

      if ( debug ) write(irefwr,*) 'before '

      call sepfillcoeffs ( isubr, iseqcf, ictime )

      if ( ierror/=0 ) go to 1000

!     --- Write some text information for subroutine to0089
!         depending on output parameters

      if ( debug ) write(irefwr,*) 'before to0089text'
      call to0089text ( iinsol )

!     --- Actual building of system of equations
!         In case of local transformations both the matrix and
!         right-hand side correspond to the local coordinate system

      if ( debug ) write(irefwr,*) 'before sepbuild'
      isubcall = 5
      kmesh =>kmesh2d(iseqkmesh)
      call sepfillsmf ( iinbldlc, irhsd, isolut2d(ivectr), isolut2d )
      call sepbuild
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( stiffmat(1), 5, 'stiffmat/1' )

      if ( iinsol(18)>0 ) then

!     --- iinsol(18)>0, hence right-hand side is stored in isolut
!         we copy the vector into irhsd

         call sepcopyvc ( isolut2d(iinsol(18)), irhsd )

      end if  ! ( iinsol(18)>0 )

!     --- Print some information and fill iread for solver

      if ( debug ) write(irefwr,*) 'before to0089text1'
      call to0089text1 ( iinsol )

      if ( ivecfem<0 ) then

!     --- Special case: ivecfem<0, add extra rhsd to rhsd
!         Both correspond to the local coordinate system

         rhsdsp => ks(isolut2d(-ivecfem))%sol
         rhsd => ks(irhsd(1))%sol
         rhsd = rhsdsp + rhsd

      end if  ! ( ivecfem<0 )

!     --- Solve system of linear equations
!         In case of a spectral method, the solution of the finite element
!         system is computed which is used later on as start vector for the
!         spectral preconditioning
!         In case of local transformations, the output solution and reaction
!         force are in the Cartesian coordinate system, not in the local one

      if ( debug ) write(irefwr,*) 'before solvelbf'

      iseqsol = ivectr
      iseqreac = ivecreac
      keepmat = iinsol(14)
      iseqsolut = ivectr
      iseqmt = stiffmat(1)
      call sepsolveinput ( ireadinpsol, inpsol1, rinsol1 )
      call seplinsol ( stiffmat, isolut2d, irhsd )
      if ( ierror/=0 ) go to 1000

!     --- Fill reaction forces, depending on iinsol

      if ( debug ) write(irefwr,*) 'before to0089reac'
      call to0089reac ( iinsol, ivectr, iinbldlc, ivecreac, ivecfem )

      if ( ivecfem>0 ) then

!     --- Fill FEM vector

         write(irefwr,200) ivecfem
200      format( ' Compute vector ', i3, ' as FEM solution vector' )
         call sepcopyvc ( isolut2d(ivectr), isolut2d(ivecfem) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iprecfem>0 ) then

!     --- iprecfem>0, Compute SEM solution by FEM preconditioning

         call to0089sem ( ivectr, iseqsl )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinsol(17)>0 ) then

!     --- Special option to compute the test force
!         First copy solution array to temporary array and set all
!         boundary values to zero
!         Then replace the solution at the outer boundary by 0
!         Finally compute the test force

         call tocomptstforce ( iinsol, ivectr, iinbldlc, ivecfem )

      end if  ! ( iinsol(17)>0 )

!     --- Delete arrays matr and irhsd

      if ( iinsol(15)==2 .or. iinsol(15)==0 .and. &
           ( ictime<2 .or. ictime>4 .and. ictime/=6 ) ) then
         if ( stiffmat(1)/=0 ) call seprmmatrix ( stiffmat(1) )
         if ( debug ) write(irefwr,*) 'Destroy matrix'
      end if  ! ( iinsol(15)==2 .or. iinsol(15)==0 .. )

      if ( iinsol(16)==2 .or. iinsol(16)==0 .and. &
           ( ictime<2 .or. ictime>4 .and. ictime/=6 ) ) then
         if ( irhsd(1)>0 ) &
            call seprmsolinfo ( irhsd(1) )
         if ( debug ) write(irefwr,*) 'Destroy right-hand side'
      end if  ! ( iinsol(16)==2 .or. iinsol(16)==0 .. )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0089'

      end if  ! ( debug )
      call erclos ( 'to0089' )

      end subroutine to0089
