      subroutine toconct2 ( inpconct )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Make work arrays local
!        version  1.2  date 29-12-1997 Correction nodes in kmeshn
!        version  1.1  date 12-12-1997 Adaptation for the start
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the elements in the contact surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpconct(*)

!     inpconct   i    In this array it is indicated how the contact surface
!                     must be computed
!                     Contents
!                     1: sequence_number of contact
!                     2: type of contact
!                        1: curves
!                        2: surfaces
!                     3: contact method
!                        1: contact is made if contact_distance<=0
!                     4: contact disable method
!                        1: contact is disabled if CONTACT_FORCE<=0
!                     5: numvec
!                     6: sequence number of contact surface or curve
!                     7: sequence number of vector defining contact distance
!                     8: sequence number of vector defining contact force
!                     9: -
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, ielem, istart, isurnr, &
              iconctsr, nelconct, iseqlc, nconctct
      integer, allocatable :: iwork(:), iwork1(:)
      logical makecon

!     i              Counting variable
!     iconctsr       Surface number where contact is made
!     ielem          Element number
!     inpelm         number of points per element
!     iseqlc         Sequence number of contact
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          Work array of length npoint that is filled only for the
!                    contact points
!                    Let the i^th contact point have node number nodp
!                    then iwork(nodp) = i
!     iwork1         Work array of length npoint that is filled only for the
!                    contact points
!                    Let the i^th contact point have node number nodp
!                    then
!                    iwork1(nodp) =-1 point did make contact, but has been
!                                     disabled
!                    iwork1(nodp) = 0 point makes no contact
!                    iwork1(nodp) = 1 point makes contact
!     makecon        If true the element makes contact
!     nconctct       Number of points with contact
!     nelconct       Number of contact elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     iseqlc := inpconct(1)
!     nconctct := kprobvp1(iseqlc,1)
!     Set iwork for all nodes that make contact equal to 1
!     nelconct := 0
!     For all elements in the surface do
!        If ( all nodes in the element make contact ) then
!           nelconct := nelconct+1
!           set iwork1 for element equal to 1
!        else ( not all nodes make contact )
!           set iwork1 for element equal to 0
!     Fill kprobw
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toconct2' )
      iseqlc = inpconct(1)
      iconctsr = inpconct(6)
      nconctct = kprobvp1(iseqlc,1)

      nelconct = 0
      inelem = kmeshn(iconctsr+nsurfs)
      inpelm = kmeshn(iconctsr)

      allocate ( iwork(npoint), iwork1(inelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+inelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- Set points with contact equal to 1

      istart = kprobvp1(iseqlc,3)
      do i = istart, nconctct+istart-1
         iwork(kprobvp2(i)) = 1
      end do

      istart = 2*nsurfs
      do isurnr = 1, iconctsr-1
         istart = istart+kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
      end do

      if ( nconctct>0 ) then

!     --- For all elements in the surface do

         do ielem = 1, inelem

            makecon = .true.
            do i = 1, inpelm
               if ( iwork(kmeshn(istart+i))==0 ) then
                  makecon = .false.
                  go to 100
               end if
            end do

100         if ( makecon ) then

!           --- all nodes in the element make contact

               nelconct = nelconct+1
               iwork1(ielem) = 1

            else

!           --- Not all nodes make contact

               iwork1(ielem) = 0

            end if
            istart = istart + inpelm

         end do

      else

!     --- none of the elements make contact

         do ielem = 1, inelem
            iwork1(ielem) = 0
         end do

      end if

!     --- Fill array kprobw

      istart = kprobwp1(iseqlc,4)
      if ( istart==0 ) then

!     --- istart = 0, hence this surface has not yet been filled
!         Compute istart and store in kprobwp1

         do i = 1, nsurfs
            if ( kprobwp1(i,4)>istart ) &
               istart = istart+kprobwp1(i,1)+kprobwp1(i,2)+kprobwp1(i,4)
         end do
         istart = max(istart,1)
         kprobwp1(iseqlc,4) = istart

      end if
      kprobwp1(iseqlc,1) = nelconct
      kprobwp1(iseqlc,2) = inelem-nelconct
      kprobwp1(iseqlc,3) = iconctsr

!     --- Fill kprobw part 2

      nelconct = nelconct+istart
      do ielem = 1, inelem
         if ( iwork1(ielem)==1 ) then

!        --- contact element

            kprobwp2(istart) = ielem
            istart = istart+1

         else

!        --- nocontact element

            kprobwp2(nelconct) = ielem
            nelconct = nelconct+1

         end if

      end do

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'toconct2' )

      end
