      subroutine toconvergcheck ( niter, minitr, finished, diffsl, &
                                  nparm, ncopeq, residu, minchk, diver, &
                                  iprint, iretur, maxitr, icrit )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-06-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if an iteration process has converged or diverged
!     Print information of the convergence depending on iprint
!     Copy diffsl(.,2) into diffsl(.,3) and diffsl(.,1) into diffsl(.,2)
! **********************************************************************
!
!                       KEYWORDS
!
!     iteration
!     non_linear
!     norm
!     check
!     convergence
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niter, minitr, nparm, ncopeq, minchk, iprint, iretur, &
              maxitr, icrit
      double precision diffsl(nparm,*)
      logical finished, residu, diver

!     diffsl        i/o   In this array of size nparm x 7
!                         information about the convergence of the
!                         iteration process is stored for each coupled equation
!                         DIFFSL is filled as follows:
!                         diffsl(i,1)   maximum difference between present and
!                                       prior iteration, i.e. ||u(n)-u(n-1)||
!                                       or  || u(n) - u(n-1) || / || u(n) ||
!                         diffsl(i,2)   value of diffsl(i,1) in the prior
!                                       iteration, i.e. || u(n-1) - u(n-2) ||
!                         diffsl(i,3)   value of diffsl(i,2) in the prior
!                                       iteration, i.e. || u(n-2) - u(n-3) ||
!                         diffsl(i,4)   required accuracy
!                         diffsl(i,5)   ||residual||, i.e. ||A(n)u(n-1)-f(n)||
!                         diffsl(i,6)   ||residual|| in the prior iteration
!                         diffsl(i,7)   required residual accuracy
!                         The parameter i indicates the sequence number of the
!                         equation
!     diver          o    Indicates if the iteration process diverges (true) or
!                         not
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icrit          i    Type of criterion to be used for the termination
!                         See input array iinsol in readnl
!     iprint         i    Defines the amount of output produced
!                          Possible values:
!                         -1:  All output information is suppressed
!                              even the message of no-convergence
!                          0:  No information about the iteration
!                              process is printed except in the
!                              case of an error
!                          1:  The number of iterations performed is printed
!                          2:  Some extra information about the iteration
!                              process is printed for each iteration
!                          3:  See 2, in this case also the ratio of two
!                              succeeding iterations is printed
!     iretur         i    Indicates if the subroutine must stop if
!                         no convergence has been found (0) or that
!                         control must be given back to the main program (1)
!     maxitr         i    Maximum number of iterations
!     minchk         i    Minimum number of iterations before divergence is
!                         checked
!     minitr         i    Minimum number of iterations
!     ncopeq         i    Number of coupled equations
!     niter          i    Iteration sequence number
!     nparm          i    Length of first index of array diffsl
!                         Contains maximum number of coupled equations
!     residu         i    If true the residual must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision speed, speed1

!     i              Counting variable
!     speed          Convergence speed
!     speed1         Convergence speed of residual
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1906   Divergence of iteration process
!    1907   No convergence before maximum number of iterations has been reached
!    1938   Divergence of iteration process
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toconvergcheck' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toconvergcheck'
         write(irefwr,1) 'niter, minitr, icrit, iprint', &
                          niter, minitr, icrit, iprint
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Test divergence or convergence of iteration process

      if ( niter>=minitr ) then

!     --- number of iterations is at least equal to minimum value

         finished = .true.
         if ( icrit<6 ) then
            do i = 1, ncopeq
               if ( diffsl(i,1)>diffsl(i,4) ) finished = .false.
            end do
         else if ( niter==1 ) then
            finished = .false.
         end if  ! ( icrit<6 )
         if ( residu .and. niter>1 ) then
            do i = 1, ncopeq
               if ( diffsl(i,5)>diffsl(i,7) ) finished = .false.
            end do
         end if  ! ( residu .and. niter>1 )

      end if

      if ( debug ) write(irefwr,*) 'finished', finished

      if ( .not. finished .and. niter>1 .and. niter>=minchk ) then

!     --- Check divergence

         diver = .false.
         if ( icrit<6 ) then
            do i = 1, ncopeq
               if ( diffsl(i,2)<diffsl(i,1) .and. &
                    diffsl(i,3)<diffsl(i,2) ) diver = .true.
            end do
         end if  ! ( icrit<6 )

         if ( residu .and. niter>2 ) then
            do i = 1, ncopeq
               if ( diffsl(i,6)<diffsl(i,5) ) diver = .true.
            end do
         end if

         if ( debug ) write(irefwr,*) 'diver', diver
         if ( diver .and. residu ) then

!        --- divergence in combination with residual computation

            call errint ( niter, 1 )
            if ( iretur==0 ) then
               call errsub ( 1938, 1, 0, 0 )
            else
               call errwar ( 1938, 1, 0, 0 )
            end if
            do i = 1, ncopeq
               write ( irefwr, 340 ) i, diffsl(i,1), diffsl(i,2), &
                                     diffsl(i,5), diffsl(i,6)
340            format ( i5, 10x, ss, 1pe13.2e2, 10x, 1pe13.2e2, &
                        1pe12.1e2, 1pe12.1e2 )
            end do

         else if ( diver ) then

!        --- divergence, without residual computation

            call errint ( niter, 1 )
            if ( iretur==0 ) then
               call errsub ( 1906, 1, 0, 0 )
            else
               call errwar ( 1906, 1, 0, 0 )
            end if
            do i = 1, ncopeq
               write ( irefwr, 350 ) i, diffsl(i,1), diffsl(i,2)
350            format ( i5, 10x, ss, 1pe13.2e2, 10x, 1pe13.2e2 )
            end do

         end if

      end if

      if ( debug .or. abs(iprint)>=1 ) then

!     --- Print output required

         if ( debug .or.  abs(iprint)>=2 ) then

!        --- iprint>=2, print extra information

            if ( residu ) then

               if ( niter==1 ) write(irefwr,380)
380            format ( /' Information about the iteration process'/ &
                         ' Iter   Eqn  ', &
                         ' ||u(n)-u(n-1)||  conv. speed  ', &
                         ' residual  res-quotient' )
               speed = 1d0
               if ( niter>1 .and. diffsl(1,2)>0d0 ) &
                   speed = diffsl(1,1)/diffsl(1,2)
               speed1 = 1d0
               if ( niter>1 .and. diffsl(1,6)>0d0 ) &
                   speed1 = diffsl(1,5)/diffsl(1,6)
               write (irefwr,390) niter, 1, diffsl(1,1), speed, &
                                  diffsl(1,5), speed1
390            format ( i4, 2x, i5, 3x, ss, 4(1pe13.3e2) )
               do i = 2, ncopeq
                  speed = 1d0
                  if ( niter>1 .and. diffsl(i,2)>0d0 ) &
                      speed = diffsl(i,1)/diffsl(i,2)
                  speed1 = 1d0
                  if ( niter>1 .and. diffsl(i,6)>0d0 ) &
                      speed1 = diffsl(i,5)/diffsl(i,6)
                  write (irefwr,400) i, diffsl(i,1), speed, &
                                     diffsl(i,5), speed1
400               format ( 6x, i5, 3x, ss, 4(1pe13.3e2) )
               end do

            else

               if ( niter==1 ) write(irefwr,410)
410            format ( /' Information about the iteration process'/ &
                         ' Iteration  Equation   ', &
                         ' ||u(n)-u(n-1)|| conv. speed')
               speed = 1d0
               if ( niter>1 .and. diffsl(1,2)>0d0 ) &
                   speed = diffsl(1,1)/diffsl(1,2)
               write (irefwr,420) niter, 1, diffsl(1,1), speed
420            format ( 2x, i5, 5x, i5, 4x, ss, 2(1pe14.2e2) )
               do i = 2, ncopeq
                  speed = 1d0
                  if ( niter>1 .and. diffsl(i,2)>0d0 ) &
                      speed = diffsl(i,1)/diffsl(i,2)
                  write (irefwr,430) i, diffsl(i,1), speed
430               format ( 12x, i5, 4x, ss, 2(1pe14.2e2) )
               end do

            end if

         end if

         if ( finished ) then
            write(irefwr,440) niter
440         format ( ' Convergence has been reached after ', i3, &
                     ' steps' )
         end if

      end if

!     --- Copy diffsl(1) into 2

      do i = 1, ncopeq
         diffsl(i,3) = diffsl(i,2)
         diffsl(i,2) = diffsl(i,1)
      end do

      if ( .not. finished .and. niter>=maxitr ) then
         call errint ( niter, 1 )
         if ( iretur==0 ) then
            call errsub ( 1907, 1, 0, 0 )
         else
            call errwar ( 1907, 1, 0, 0 )
         end if
         finished = .true.
      end if
      if ( diver ) finished = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toconvergcheck'

      end if
      call erclos ( 'toconvergcheck' )

      end
