      subroutine msh004 ( tau, ishape, nelm, n, iratio, factor, isubd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 12-01-2009 Remove array work
!        version  5.3  date 27-04-2001 Allow iratio = 5, 6
!        version  5.2  date 06-06-1996 Layout
!        version  5.1  date 10-02-1991 Error correction tau for ishape>1
!        version  5.0  date 30-12-1990 Extra parameters isubd and work
!        version  4.0  date 08-08-1988 Negative values for iratio
!        version  3.0  date 05-05-1987
!
!   copyright (c) 1982-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill array tau,  depending on the values of nelm,  ishape,  iratio
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, nelm, n, iratio, isubd
      double precision tau(*), factor

!    factor     i   is used when iratio>0,   see programmer's guide
!    iratio     i   array tau is computed depending on the value of iratio
!                   possibilties:
!                   0   equidistand mesh size
!                   1   last element is factor times the first one
!                   2   each element is factor times the preceding one
!    ishape     i   number of nodal points in element minus one
!    isubd      i   Defines the type of division of the nodes along the side
!                   in case of hihger order elements.
!                   0  equidistant step size
!                   1  Gauss-Legendre
!    n          o   number of points in tau
!    nelm       i   number of elements on curve
!    tau        o   array of length ( nelm * ishape + 1 )
!                   contents  tau(1) = 0  . . .  tau(nelm+1) = 1
!                   tau(i-1)<tau(i)<tau(i+1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx, dt, f, sum, h, dtau, work(ishape+1)
      integer i, j

!     dt         relative length of first element
!     dtau       Difference in tau values in tow succeeding elements
!     dx         length of vector in x-direction
!     f          Length of next element subdivided by present one
!     h          Help variable to store a constant temporarily
!     i          Counting variable
!     j          Counting variable
!     sum        Help parameter to compute a sum
!    work        work array of length ishape+1 to store the subdivision along
!                the elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELLGLP    compute the Legendre Gauss-Lobatto points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Formula
!
!    Let f be the length of the next element divided by the present one
!    Then the length of the elements is:
!
!    alpha, alpha f, alpha f^2, alpha f^3, ...
!
!    Setting the total length to 1, this gives:
!
!    1 = alpha ( 1 + f + f^2 + f^3 + ... )
!
!    Hence:
!
!    alpha = 1 / ( 1 + f + f^2 + f^3 + ... + f^n-1 ) = (1-f)/(1-f^n)
!
!
!               Actual PSEUDO CODE
!
!    if ( ishape>1 ) then
!       Make a relative subdivision per element and store in array work
!    tau(1) = 0
!    tau(n) = 1
!    Fill tau for vertices
!    if ( ishape>1 ) then
!       Fill tau for mid points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ishape>1 ) then

!     --- ishape>1

         if ( isubd==0 ) then

!        --- isubd = 0, equidistant division

            dx = 1d0 / ishape
            do i = 2, ishape
               work(i) = dx * (i-1)
            end do

         else

!        --- isubd = 1, Gauss-Legendre

            call ellglp ( work, ishape+1 )
            do i = 2, ishape
               work(i) = 0.5d0 * ( work(i) + 1d0 )
            end do

         end if

      end if

      tau(1) = 0d0
      n = nelm*ishape+1
      tau(n) = 1d0

      if ( nelm==1 .or. abs(factor-1d0)<sqreps .or. &
           iratio==0 .or. iratio==-1 ) then

!     --- iratio = 0  equidistant mesh size

         dt = 1d0/(n-1)
         do i = 2, n
            tau(i) = (i-1d0)*dt
         end do

      else if ( iratio<=4 ) then

!     --- 1<=iratio<=4, or -5<=iratio<=-2  changing mesh size

         if ( iratio==1 .or. iratio==-2 ) then

!        --- iratio = 1 or -2  last element is factor times the first one

            f = factor**(1d0/(nelm-1d0))

         else

!        --- iratio = 2 or -3  each element is factor times the preceding one

            f = factor

         end if

!        --- subdivide

         dt = (1d0-f)/(1d0-f**nelm)
         sum = 0d0
         h = dt
         do i = ishape+1, n, ishape
            sum = sum+h
            tau(i) = sum
            h = h*f
         end do

      else

!     --- iratio>4  changing mesh size, factor to middle 1-factor to end

         if ( iratio==5 ) then

!        --- iratio = 5  last element is factor times the first one

            f = factor**(1d0/(nelm/2-1d0))

         else

!        --- iratio = 6  each element is factor times the preceding one

            f = factor

         end if

!        --- subdivide

         if ( mod(nelm,2)==0 ) then

!        --- nelm is even, even number of elements
!            The subdivision is symmetric

            dt = 0.5d0*(1d0-f)/(1d0-f**(nelm/2))
            sum = 0d0
            h = dt
            do i = ishape+1, n/2, ishape
               sum = sum+h
               tau(i) = sum
               tau(n-i+1) = 1d0-sum
               h = h*f
            end do
            tau(n/2+1) = 0.5d0

         else

!        --- nelm is odd, odd number of elements
!            The subdivision is symmetric, but the middle element is extra

            h =f**(nelm/2)
            dt = (1d0-f)/(2d0*(1d0-h)+(1d0-f)*h)
            sum = 0d0
            h = dt
            do i = ishape+1, n/2, ishape
               sum = sum+h
               tau(i) = sum
               tau(n-i+1) = 1d0-sum
               h = h*f
            end do
         end if

      end if

      if ( ishape>1 ) then

!     --- ishape>1, Fill points in between according to isubd
!         The relative division is stored in work

           do i = 2 , n - 1 , ishape
              dtau   = tau(i+ishape-1)-tau(i-1)
              do j = 1, ishape-1
                 tau(i+j-1) = tau(i-1) + dtau * work(j+1)
              end do
           end do

      end if

      end
