      subroutine msh006 ( coarse, ndim, curves, iinput, rinput, tau, &
                          inpncv, nuspnt, curstr, icurnr, icurve )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.4  date 11-12-2014 Extra debug
!        version  6.3  date 11-06-2008 Extra debug
!        version  6.2  date 05-12-2005 More info in error message
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill part of array curves for a circle segment
!   n positions are filled
!   the spacing of the nodal points is given by array tau
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical coarse
      integer iinput(*), ndim, inpncv, nuspnt, icurnr, icurve
      double precision curves(ndim,*), tau(*), rinput(ndim,*), curstr(*)

!     coarse         i    indicates if array curstr has been filled already
!                         (true) or not (false)
!     curstr         i    is only used when coarse is true
!                         in that case it must contain the following information
!                         1:   length of the circle part
!                         2:   phi arc of the circle segment
!                         3:   r   radius of the circle
!                         ndim=2 4,5   :   xz,yz     mid points of the circle
!                         ndim=3 4,5,6 :   xz,yz,zz  centre of the circle
!     curves         o    array containing the coordinates of the curves
!     icurnr         i    curve number
!     icurve         i    Type of curve to be generated
!     iinput         i    the first two positions of this array contain the
!                         point numbers of the initial and end points
!     inpncv         i    number of points on the line
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rinput         i    array containing the coordinates of the points
!     tau            i    array containing the spacing of the nodal points
!                         output of subroutine msh004 or msh031
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iuserp1, iuserp2, iuserp3, i, ikeuze
      double precision x1, y1, x2, y2, phi, xz, yz, xt, yt, dx, dy, c2, &
                       alpha, r, phih, convex, ch, sh, z1, z2, zz, zt, &
                       dz, cs, a1, a2, b1, b2, b3, d1, d2, d3, dd, det, &
                       hedet, beta, a3, tang1(3), tang2(3)

!     a1
!     a2
!     a3
!     alpha
!     b1
!     b2
!     b3
!     beta
!     c2
!     ch
!     convex
!     cs
!     d1
!     d2
!     d3
!     dd
!     det
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     hedet
!     i              Counting variable
!     ikeuze
!     iuserp1        First user point
!     iuserp2        Second user point
!     iuserp3        third user point
!     phi
!     phih
!     r
!     sh
!     tang1          First tangential vector
!     tang2          Second tangential vector
!     x1             x-coordinate of first point in curve
!     x2             x-coordinate of last point in curve
!     xt
!     xz
!     y1             y-coordinate of first point in curve
!     y2             y-coordinate of last point in curve
!     yt
!     yz
!     z1             z-coordinate of first point in curve
!     z2             z-coordinate of last point in curve
!     zt
!     zz
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCIRC3D      Fill coordinates on circle in 3d
!     MSHMAP3D2D     Map plane defined by 3 points onto xi,eta system
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     127   user point number out of range
!     422   arc or carc is only permitted in two or three dimensions
!     524   Coinciding initial and end point of arc in 3-D. Curve number $
!    1919   The three nodal points defining a circle in R3 are all lying in
!           one line
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh006' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh006'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( icurve/=17 ) then

!     --- arc, standard situation

         iuserp1 = iinput(1)
         iuserp2 = iinput(2)
         iuserp3 = iinput(3)

      else

!     --- circle

         iuserp3 = iinput(1)
         iuserp1 = iinput(2)
         if ( ndim==2 ) then
            iuserp2 = iuserp1
         else
            iuserp2 = iinput(3)
         end if

      end if
      if ( debug ) &
         write(irefwr,1) 'icurve, iuserp1, iuserp2, iuserp3, ndim', &
                          icurve, iuserp1, iuserp2, iuserp3, ndim
      if ( iuserp1<=0 .or. iuserp1>nuspnt ) then
         call errint ( iuserp1, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( iuserp2<=0 .or. iuserp2>nuspnt ) then
         call errint ( iuserp2, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( iuserp3==0 .or. abs(iuserp3)>nuspnt ) then
         call errint ( iuserp3, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( ndim<2 .or. ndim>3 ) then
         call errsub ( 422, 0, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000
      if ( ndim==2 ) then

         x1 = rinput(1,iuserp1)
         y1 = rinput(2,iuserp1)
         x2 = rinput(1,iuserp2)
         y2 = rinput(2,iuserp2)
         xt = rinput(1,abs(iuserp3))
         yt = rinput(2,abs(iuserp3))
         dx = x1-x2
         dy = y1-y2
         if ( debug ) &
            write(irefwr,2) 'x1, y1, x2, y2, xt, yt', &
                             x1, y1, x2, y2, xt, yt
         if ( icurve==17 .or. &
              abs(dx)<=sqreps*abs(x1+x2) .and. &
              abs(dy)<=sqreps*abs(y1+y2) ) then

!        --- coinciding initial and end point

            phi = 2d0*pi
            if(iuserp3<0) phi = -phi
            xz = xt
            yz = yt

         else

            c2 = dx**2+dy**2
            alpha=((0.5d0*(x1+x2)-xt)*dx+(0.5d0*(y1+y2)-yt)*dy)/c2
            xz = xt+alpha*dx
            yz = yt+alpha*dy
            r = sqrt((xz-x1)**2+(yz-y1)**2)
            phih = min(1d0,sqrt(c2)/(2d0*r))
            phih = max(-1d0,phih)
            phi = 2d0*asin(phih)
            if ( iuserp3>0 ) then
               convex = x1*(yz-y2)-xz*dy+x2*(y1-yz)
            else
               convex = x2*(yz-y1)+xz*dy+x1*(y2-yz)
            end if
            if(convex>0d0) phi = 2d0*pi-phi
            if(iuserp3<0) phi = -phi

         end if

         do i = 2, inpncv-1
             ch = cos(tau(i)*phi)
             sh = sin(tau(i)*phi)
             curves(1,i) = xz+ch*(x1-xz)-sh*(y1-yz)
             curves(2,i) = yz+ch*(y1-yz)+sh*(x1-xz)
         end do
         curves(1,1) = x1
         curves(2,1) = y1
         curves(1,inpncv) = x2
         curves(2,inpncv) = y2

      else if ( icurve==17 .and. ndim==3 ) then

!     --- icurve = 17: complete circle in R3
!         First map the three points onto a plane

         call  mshmap3d2d ( iuserp3, iuserp1, iuserp2, rinput,  tang1, &
                            tang2 )

!        --- Then define the circle

         call mshcirc3d ( iuserp3, iuserp1, rinput, curves, tau, &
                          inpncv, tang1, tang2 )

      else if ( ndim==3 ) then

!     --- ndim = 3

         x1 = rinput(1,iuserp1)
         y1 = rinput(2,iuserp1)
         z1 = rinput(3,iuserp1)
         x2 = rinput(1,iuserp2)
         y2 = rinput(2,iuserp2)
         z2 = rinput(3,iuserp2)

         if ( debug ) then
            write(irefwr,2) 'x1,y1,z1, x2,y2,z2', &
                             x1,y1,z1, x2,y2,z2
            write(irefwr,*) 'coarse', coarse
         end if  ! ( debug )

         if ( coarse ) then

!        --- coarse is true phi etc have already been computed

            phi = curstr(2)
            r = curstr(3)
            xz = curstr(4)
            yz = curstr(5)
            zz = curstr(6)
            if ( debug ) &
               write(irefwr,2) 'phi, r, xz, yz, zz', phi, r, xz, yz, zz

         else

            xt = rinput(1,abs(iuserp3))
            yt = rinput(2,abs(iuserp3))
            zt = rinput(3,abs(iuserp3))
            dx = x1-x2
            dy = y1-y2
            dz = z1-z2
            if(abs(dx)<=sqreps*abs(x1+x2) .and. abs(dy)<=sqreps* &
               abs(y1+y2) .and. abs(dz)<=sqreps*abs(z1+z2)) then

!           --- coinciding initial and end point

               call errint ( icurnr, 1 )
               call errsub ( 524, 1, 0, 0 )
               go to 1000

            else

               c2 = dx**2+dy**2+dz**2
               alpha = ((0.5d0*(x1+x2)-xt)*dx+(0.5d0*(y1+y2)-yt)*dy+ &
                        (0.5d0*(z1+z2)-zt)*dz)/c2
               xz = xt+alpha*dx
               yz = yt+alpha*dy
               zz = zt+alpha*dz
               r = sqrt((xz-x1)**2+(yz-y1)**2+(zz-z1)**2)
               cs = 1d0-c2/r/r/2d0
               phi = acos(cs)

            end if

         end if

!        --- determine local coordinates

         a1 = (x1-xz)/r
         a2 = (y1-yz)/r
         a3 = (z1-zz)/r

         b1 = (x2-xz)/r
         b2 = (y2-yz)/r
         b3 = (z2-zz)/r

         d1 = a1*(a2*b2+a3*b3)-b1*(a2*a2+a3*a3)
         d2 = a2*(a1*b1+a3*b3)-b2*(a1*a1+a3*a3)
         d3 = a3*(a1*b1+a2*b2)-b3*(a1*a1+a2*a2)

!        --- Normalize d

         dd = sqrt ( d1*d1 + d2*d2 + d3*d3 )
         if ( dd<epsmac*r ) then
            call errint ( icurnr, 1 )
            call errint ( iuserp1, 2 )
            call errint ( iuserp2, 3 )
            call errint ( iuserp3, 4 )
            call erreal ( x1, 1 )
            call erreal ( y1, 2 )
            call erreal ( z1, 3 )
            call erreal ( x2, 4 )
            call erreal ( y2, 5 )
            call erreal ( z2, 6 )
            call erreal ( rinput(1,iuserp3), 7 )
            call erreal ( rinput(2,iuserp3), 8 )
            call erreal ( rinput(3,iuserp3), 9 )
            call errsub ( 1919, 4, 9, 0 )
            go to 1000
         end if

         d1 = d1 / dd
         d2 = d2 / dd
         d3 = d3 / dd

         ikeuze = 1
         det = a1*d2-a2*d1

         hedet = a2*d3-a3*d2
         if ( abs(hedet)>abs(det) ) then
              ikeuze = 2
              det = hedet
         end if

         hedet = a3*d1-a1*d3
         if ( abs(hedet)>abs(det) ) then
            ikeuze = 3
            det = hedet
         end if

         if ( ikeuze==1 ) then
              beta = (a1*(y2-yz)-a2*(x2-xz))/det
         else if ( ikeuze==2 ) then
              beta = (a2*(z2-zz)-a3*(y2-yz))/det
         else if ( ikeuze==3 ) then
              beta = (a3*(x2-xz)-a1*(z2-zz))/det
         end if

         if ( beta<0 ) then
              phi = -phi
         end if

         a1 = a1*r
         a2 = a2*r
         a3 = a3*r

         d1 = d1*r
         d2 = d2*r
         d3 = d3*r

         do i = 2, inpncv-1
             ch = cos(tau(i)*phi)
             sh = sin(tau(i)*phi)
             curves(1,i) = xz+ch*a1+sh*d1
             curves(2,i) = yz+ch*a2+sh*d2
             curves(3,i) = zz+ch*a3+sh*d3
         end do

         curves(1,1) = x1
         curves(2,1) = y1
         curves(3,1) = z1
         curves(1,inpncv) = x2
         curves(2,inpncv) = y2
         curves(3,inpncv) = z2

      end if

1000  call erclos ( 'msh006' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( curves, inpncv, ndim, 'curves' )
         write(irefwr,*) 'End msh006'

      end if  ! ( debug )

      end
