      subroutine msh013 ( iinput, irelcurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.2  date 10-02-2015 Extension with circ with extra user pnts
!        version 14.1  date 11-12-2014 Extension with lines with extra user pnts
!        version 14.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1982-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    initialize kmesh part m
!    part (iv) gets length ncurvs and is set equal to zero
!           positions corresponding to user points get the negative value
!           of these points
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irelcurves(iinp2)

!     iinput         i    integer input array with respect to the curves
!     irelcurves      i    Array with the same length as the first part of
!                         iinput (until surfaces)
!                         Relative node numbers of the user points in
!                         framecurv are filled at the same positions as
!                         the user points in iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istar2, iinp, icurve, nelm, ifirst, ilast, nh, &
                 icount, jstart, i, jcurnr,  n, &
                 ndefcv, iwork(ncurvs), istartrel, j, itype

!     i              Counting variable
!     icount         Counter
!     icurve         Type of curve
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinput
!     ilast          Last curve (surface) number read
!     istar2         Starting address of kmeshm part 2 in kmeshm
!     istartrel      starting position in irelcurves
!     itype          type of spline
!     iwork          integer work array of length ncurvs to store all the
!                    subcurve numbers of a composite curve
!     j              Counting variable
!     jcurnr         Curve sequence number
!     jstart         starting position of curve i in kmeshm
!     n              parameter for mesh generation
!     ndefcv         Number of points at a spline curve
!     nelm           number of elements along a curve
!     nh             number of nodal points in curve i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     MSH013_10      initialize kmesh part m, Special case icurve = 10, 11
!     MSH013_7       initialize kmesh part m, Special case icurve = 7
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     124   code for generation of curves (icurve) incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh013' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh013'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if

      nbndpt = icurvsmsh(ncurvs)
      kmeshm(1) = 0
      kmeshm(2) = 6+ncurvs
      kmeshm(3) = kmeshm(2)+nbndpt
      kmeshm(4) = 0
      kmeshm(5) = 1

      kmeshm(6:ncurvs+5) = 1+icurvsmsh(1:ncurvs)
      istar2 = kmeshm(2)
      kmeshm(istar2:kmeshm(3)-1) = 0

      if ( debug ) write(irefwr,1) 'istar2', istar2

      istar2 = istar2-1

      do jcurnr = 1, ncurvs

         icurnr = iseqcurvs(jcurnr)
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)

         istartrel = irelcurves(icurnr)

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'jcurnr, icurnr, icurve, istartrel', &
                             jcurnr, icurnr, icurve, istartrel

         end if

         if ( icurve==0 ) go to 500
         if ( icurve>=10000 .and. icurve<=10003 ) then

!        --- Special curve on free surface

            ifirst = iinput(iinp+3)
            ilast = iinput(iinp+4)
            kmeshm(istar2+kmeshm(4+icurnr)) = -ifirst
            kmeshm(istar2+kmeshm(5+icurnr)-1) = -ilast
            go to 500

         end if
         ishape = iinput(iinp+1)
         nelm = iinput(iinp+2)

!        --- In case icurve = 5 and more than 2 user points we
!            temporarily replace icurve by 99
!            in order to distinguish between the old and new case

         if ( icurve==5 .and. iinput(iinp+3)>2 ) icurve = 99

         select case ( icurve )

         case(1, 3, 5, 6, 8, 9, 14, 20, 22, 23)

!        --- icurve = 1, 3, 6, 8, 9, 14, 20, 22, 23

            ifirst = iinput(iinp+4)
            ilast = iinput(iinp+5)
            kmeshm(istar2+kmeshm(4+icurnr)) = -ifirst
            kmeshm(istar2+kmeshm(5+icurnr)-1) = -ilast

         case(2)

!        --- icurve = 2

            n = nelm+1
            nh = istar2+kmeshm(4+icurnr)-ishape
            do j = 1, n
               kmeshm(nh+j*ishape) = -iinput(iinp+j+3)
            end do

         case(4, 12)

!        --- icurve = 4, 12: spline

            ifirst = iinput(iinp+6)
            itype  = mod(iinput(iinp+4),10)
            ndefcv = iinput(iinp+5)
            if ( itype==4 ) ndefcv = ndefcv-2
            ilast  = iinput(iinp+5+ndefcv)

!           --- iinput(iinp+5) points are expected, only the first and the last
!               are always user points

            kmeshm(istar2+kmeshm(4+icurnr))   = -ifirst
            kmeshm(istar2+kmeshm(5+icurnr)-1) = -ilast

         case(99)

!        --- icurve = 5, with more than 2 user points

            ndefcv = iinput(iinp+3)

            if ( debug ) then
               write(irefwr,1) 'ndefcv, istartrel, istar2', &
                                ndefcv, istartrel, istar2
               call prinin ( irelcurves(istartrel), ndefcv, &
                             'irelcurves' )
               call prinin ( iinput(iinp+4), ndefcv, 'iinput' )
            end if  ! ( debug )

            icount = istar2+kmeshm(4+icurnr)
            do i = 1, ndefcv
               kmeshm(icount+irelcurves(istartrel+i-1)) = &
                 -iinput(iinp+3+i)
            end do  ! i = 1, ndefcv

         case(7)

!        --- icurve = 7   curve consisting of curves
!            positions corresponding to user points are filled in the
!            usual way.
!            Check if curve is a complete curve

            call msh013_7 ( iinput, iinp, istar2, icurve, iwork )

         case(10,11)

!        --- icurve = 10 or 11   curve constructed by translate or rotate
!            user points are filled in the usual way.
!            The distance between user points is found by considering the
!            preceding curve iprec.

            call msh013_10 ( iinput, iinp, kelmq, istar2, kmeshm, &
                             icurnr, kmeshq, ipointcurv )

         case(13)

!        --- icurve = 13   curve constructed by reflect
!            User points as for reflected curve are given for this curve

            n = iinput(iinp+3)

!           --- Set only begin and end value

            jstart = kmeshm(2) + kmeshm(4+icurnr) - 1
            kmeshm(jstart) = -iinput(iinp+5+ndim)
            nh = kmeshm(5+icurnr) - kmeshm(4+icurnr)
            kmeshm(jstart+nh-1) = -iinput(iinp+4+n)

         case(15, 16)

!        --- icurve = 15, 16

            ifirst = iinput(iinp+5)
            ilast = iinput(iinp+6)
            kmeshm(istar2+kmeshm(4+icurnr)) = -ifirst
            kmeshm(istar2+kmeshm(5+icurnr)-1) = -ilast

         case(17)

!        --- icurve = 17

            ifirst = iinput(iinp+5)
            ilast = iinput(iinp+5)
            kmeshm(istar2+kmeshm(4+icurnr)) = -ifirst
            kmeshm(istar2+kmeshm(5+icurnr)-1) = -ilast

         case(18)

!        --- icurve = 18

            ifirst = iinput(iinp+5)
            ilast = iinput(iinp+6)
            kmeshm(istar2+kmeshm(4+icurnr)) = -ifirst
            kmeshm(istar2+kmeshm(5+icurnr)-1) = -ilast

         case(19)

!        --- icurve = 19

            ifirst = iinput(iinp+6)
            ilast = iinput(iinp+7)
            kmeshm(istar2+kmeshm(4+icurnr)) = -ifirst
            kmeshm(istar2+kmeshm(5+icurnr)-1) = -ilast

         case(21)

!        --- icurve = 21: framecurv

            ifirst = iinput(iinp+6)

!           --- itype  = mod(iinput(iinp+4),10)

            ndefcv = iinput(iinp+5)

!           --- if ( itype==4 ) ndefcv = ndefcv-2

            ilast  = iinput(iinp+5+ndefcv)

            if ( debug ) &
               write(irefwr,1) 'ifirst, ilast, ndefcv', &
                                ifirst, ilast, ndefcv

!           --- iinput(iinp+5) points are expected

            nh = istar2+kmeshm(4+icurnr)-1
            do j = 1, ndefcv
               kmeshm(nh+irelcurves(istartrel+j-1)) = -iinput(iinp+5+j)
            end do  ! j = 1, ndefcv

            if ( debug ) then
               write(irefwr,1) 'nh', nh
               call prinin ( iinput(iinp+6), ndefcv, 'iinput' )
               call prinin ( irelcurves(istartrel), ndefcv, &
                             'irelcurves' )
            end if  ! ( debug )

         case(24)

!        --- icurve = 24, ccircle

            ndefcv = iinput(iinp+3)-1

            if ( debug ) then
               write(irefwr,1) 'ndefcv, istartrel, istar2', &
                                ndefcv, istartrel, istar2
               call prinin ( irelcurves(istartrel), ndefcv, &
                             'irelcurves' )
               call prinin ( iinput(iinp+4), ndefcv, 'iinput' )
            end if  ! ( debug )

            icount = istar2+kmeshm(4+icurnr)
            do i = 1, ndefcv
               kmeshm(icount+irelcurves(istartrel+i-1)) = &
                 -iinput(iinp+4+i)
            end do  ! i = 1, ndefcv

!           --- end node is begin node

            kmeshm(istar2+kmeshm(5+icurnr)-1) = &
               kmeshm(istar2+kmeshm(4+icurnr))

         case default

!        --- This option can not occur

            call errint ( icurve, 1 )
            call errint ( 20, 2 )
            call errsub ( 124, 2, 0, 0 )
            go to 1000

         end select  ! case ( icurve )

500   end do

1000  if ( debug ) then

!     --- debug = true, perform debugging

          call pr0008 ( kmeshm, ncurvs )

      end if
      call erclos ( 'msh013' )

      end
