      subroutine msh030ccircle ( coorusp, iinput, rincor, nelm, ndim, &
                                 icurnr, edge, iinp, np, curstr, &
                                 istartcur, irelcurves, istartrel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     circle with more than ndim points
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*), iinp, icurnr, ndim
      integer, intent(out) :: nelm, np, irelcurves(*)
      integer, intent(inout) :: istartcur, istartrel
      double precision, intent(in) :: coorusp(ndim,*), rincor(*), &
                                      edge
      double precision, intent(inout) :: curstr(*)

!     coorusp        i    array containing on the first nuspnt*ndim positions
!                         coordinates , on position nuspnt+1 the unit edge
!                         length and on the next nuspnt positions the defined
!                         coarseness of the user points
!     curstr        i/o   Contains real information of the curves
!                         See subroutine mshcrtcurv
!     edge           i    Unit length defined for coarseness
!     icurnr         i    Curve sequence number
!     iinp           i    starting position of curve input in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     irelcurves     o    Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     istartcur     i/o   next free position in curstr
!     istartrel     i/o   next free position in irelcurves
!     ndim           i    Dimension of the space.
!     nelm           i    number of elements along a curve
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     rincor         i    Part of array coorusp refering to coarseness
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: p1, p2, numuserp, isegment, nelmloc, ncurvs
      double precision :: dx(ndim), radius, x1(ndim), &
                          x2(ndim), c2, xz(ndim), sleng, phi

!     c2             Helping constant (dx,dx)
!     dx             length of vector (x2-x1)
!     isegment       sequence number of segment
!     ncurvs         Number of curves in mesh
!     nelmloc        Number of elements in segment
!     numuserp       Number of user points along curve
!     p1             user point number of first point
!     p2             user point number of second point
!     phi            angle from begin point to end point
!     radius         Radius of sphere or circle
!     sleng          length of the curve
!     x1             coordinates of first point
!     x2             coordinates of last point
!     xz             coordinates of centroid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH030ARC1     Fill arrays x2, xz, dx and c2 for a circle
!     MSH030ARCLENG  Compute length of arc (part of circle)
!     MSHCOMPNELM    Compute number of the number of elements given the length
!                    of the curve
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030ccircle' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030ccircle'
         write(irefwr,1) 'istartcur, istartrel', istartcur, istartrel
         write(irefwr,2) 'edge', edge
         call prinin ( iinput(iinp), 10, 'iinput' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ncurvs = iinput(3)

      numuserp = iinput(iinp+3)-1
      p1 = iinput(iinp+4)           ! centre of circle
      p2 = iinput(iinp+5)           ! point on circle
      x1 = coorusp(:,p1)
      x2 = coorusp(:,p2)
      dx = x2-x1
      radius = sqrt(dot_product(dx,dx))

      np = iinput(iinp+numuserp+5)
      nelm = 0

      if ( debug ) then
         write(irefwr,1) 'p1, p2, numuserp, np', p1, p2, numuserp, np
         write(irefwr,2) 'radius', radius
      end if  ! ( debug )

!     --- Consider each part of the circle, from initial point to end point

      irelcurves(istartrel) = 0
      do isegment = 2, numuserp

!     --- Fill user points

         p1 = p2
         p2 = iinput(iinp+4+isegment)

!        --- Fill x1, x2, xz, dx, c2

         call msh030arc1 ( ndim, p1, p2, x1, x2, xz, dx, c2, coorusp )

!        --- Compute  sleng and phi

         call  msh030arcleng ( ndim, c2, radius, phi, sleng, 1, &
                               x1, x2, xz, dx )
         curstr(isegment) = phi  ! store phi in curstr

         if ( debug ) write(irefwr,2) 'phi, sleng', phi, sleng

!        --- Compute nelm for this part

         call mshcompnelm ( p1, p2, rincor, edge, sleng, nelmloc )
         irelcurves(istartrel+isegment-1) = nelmloc
         nelm = nelm+nelmloc

         if ( debug ) write(irefwr,1) 'nelmloc', nelmloc

      end do  ! i = 1, numuserp-1

!     --- Consider last part of the circle, from end point to initial point

      p1 = p2
      p2 = iinput(iinp+5)

!     --- Fill x1, x2, xz, dx, c2

      call msh030arc1 ( ndim, p1, p2, x1, x2, xz, dx, c2, coorusp )

!     --- Compute  sleng and phi

      call  msh030arcleng ( ndim, c2, radius, phi, sleng, 1, &
                            x1, x2, xz, dx )

!     --- Compute nelm for this part

      call mshcompnelm ( p1, p2, rincor, edge, sleng, nelmloc )
      irelcurves(istartrel+numuserp) = nelmloc

      irelcurves(icurnr) = istartrel
      irelcurves(icurnr+ncurvs) = istartcur

      curstr(1) = radius
      curstr(numuserp+1) = phi  ! store phi in curstr
      irelcurves(istartrel+numuserp) = nelmloc
      nelm = nelm+nelmloc

      if ( debug ) then
         write(irefwr,1) 'nelmloc, nelm', nelmloc, nelm
         write(irefwr,2) 'phi, sleng', phi, sleng
         call prinrl ( curstr, numuserp+1, 'curstr' )
         call prinin ( irelcurves(istartrel), numuserp+1, 'irelcurves' )
      end if  ! ( debug )

      istartcur = istartcur+numuserp+1  ! raise istartcur
      istartrel = istartrel+numuserp+1  ! raise istartrel

1000  call erclos ( 'msh030ccircle' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartcur, istartrel', istartcur, istartrel
         write(irefwr,*) 'End msh030ccircle'

      end if  ! ( debug )

      end subroutine msh030ccircle
