      subroutine msh055surfc7 ( kmeshsp2, isurnr, kmeshm, iprec, &
                                kmeshs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if outer boundary of surface to be copied from surface of surfaces
!     has the same structure as the surface to be copied
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshsp2(8,*), isurnr, kmeshm(*), iprec, kmeshs(*)

!     iprec          i    Surface number of surface to be copied etc.
!     isurnr         i    Surface sequence number
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshs        i/o   Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2      i/o   Second part of kmeshs, referring to surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nc, kcur, firstpnt, lastpnt, nouter, nholes, nholescp, &
              kfirst, istart, istar3

!                    they are not
!     firstpnt       First point of a curve
!     i              Counting variable
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     kcur           curve sequence number
!     kfirst         First user point of a curve
!     lastpnt        last point of a curve
!     nc             number of points along the curves
!     nholes         Number of holes in the boundary of surface to be copied
!     nholescp       Number of holes in the boundary of surface
!     nouter         Number of outer boundaries of surface to be copied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1223   The boundary of the surface is not closed
!    1782   The first user point of the curve is different from the first
!           user point of the preceding curve
!    2622   The number of holes in surface and surface to be copied is
!           different
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nholes := number of holes in surface to be copied
!     nholescp := number of holes in new surface
!     if ( nholes # nholescp ) then
!        error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh055surfc7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh055surfc7'
         write(irefwr,1) 'isurnr, iprec', isurnr, iprec
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror.ne.0 ) go to 1000

      istar3 = kmeshm(2)-1

!     --- nholes := number of holes in surface to be copied
!         The outer curves are stored in array kmeshsp2

      nouter = kmeshsp2(3,iprec)

      if (debug ) write(irefwr,1) 'nouter', nouter

      if ( kmeshsp2(5,iprec)==nouter ) then

!     --- No holes in mesh, because kmeshsp2(5,iprec) = kmeshsp2(3,iprec)

         nholes = 0

      else

!     --- kmeshsp2(5,iprec) < kmeshsp2(3,iprec), so there are holes
!         Compute number of holes

         istart = kmeshs(iprec)
         nholes = -1
         kfirst = 0

!        --- Loop over outer boundaries

         do i = 1, nouter
            kcur = kmeshs(istart+i)
            if ( debug ) &
               write(irefwr,*) 'iprec, i, kcur ', iprec, i, kcur

            if ( kcur.gt.0 ) then

!           --- kcur > 0, positive direction

               firstpnt = abs ( kmeshm(istar3+kmeshm(4+kcur)) )

            else

!           --- kcur < 0, negative direction

               firstpnt = abs ( kmeshm(istar3+kmeshm(5-kcur)-1) )

            end if
            if ( debug ) &
               write(irefwr,*) 'firstpnt, kfirst ',firstpnt, kfirst

            if ( kfirst==0 ) then

!           --- firstpnt = 0, First curve of closed curve

               kfirst = firstpnt

            end if  ! ( kfirst==0 )

!           --- Find last user point of curve kcur

            if ( kcur.gt.0 ) then

!           --- kcur > 0, positive direction

               lastpnt = abs ( kmeshm(istar3+kmeshm(5+kcur)-1) )

            else

!           --- kcur < 0, negative direction

               lastpnt = abs ( kmeshm(istar3+kmeshm(4-kcur)) )

            end if

            if ( lastpnt==kfirst ) then

!           --- End of closed curve found

               kfirst = 0
               nholes = nholes+1

            end if  ! ( lastpnt==kfirst )

         end do  ! i = 1, nouter

      end if  ! ( kmeshsp2(5,iprec)==nouter )
      if ( debug ) write(irefwr,*) 'nholes ', nholes

!     --- nholescp := number of holes in new surface

      istart = kmeshs(isurnr)
      kfirst = 0
      nholescp = -1
      do i = 1, kmeshs(isurnr+1) - istart
         kcur = kmeshs(istart+i)
         if ( debug ) &
            write(irefwr,*) 'isurnr, i, kcur ', isurnr, i, kcur

!        --- Find first user point of curve kcur

         if ( kcur.gt.0 ) then

!        --- kcur > 0, positive direction

            firstpnt = abs ( kmeshm(istar3+kmeshm(4+kcur)) )

         else

!        --- kcur < 0, negative direction

            firstpnt = abs ( kmeshm(istar3+kmeshm(5-kcur)-1) )

         end if
         if ( debug ) &
            write(irefwr,*) 'firstpnt, kfirst ',firstpnt, kfirst

         if ( kfirst==0 ) then

!        --- firstpnt = 0, First curve of closed curve

            kfirst = firstpnt

         else

!        --- Not the first curve, so check if first node is equal to
!            last node of preceding curve

            if ( debug ) &
               write(irefwr,*) 'lastpnt, kfirst ',lastpnt, kfirst
            if ( firstpnt==0 .or. lastpnt==0 ) then

!           --- One of the nodes is 0

               call errint ( isurnr, 1 )
               call errint ( kcur, 2 )
               call errint ( firstpnt, 3 )
               call errint ( lastpnt, 4 )
               call errwar ( 1782, 4, 0, 0 )

            else if ( lastpnt.ne.firstpnt ) then

!           --- Last point of preceding curve different from first point

               call errint ( isurnr, 1 )
               call errint ( nc, 2 )
               call errint ( kfirst, 3 )
               call errint ( lastpnt, 4 )
               call errsub ( 1223, 4, 0, 0 )

            end if  ! ( firstpnt==0 .or. lastpnt==0 )

         end if  ! ( kfirst==0 )

!        --- Find last user point of curve kcur

         if ( kcur.gt.0 ) then

!        --- kcur > 0, positive direction

            lastpnt = abs ( kmeshm(istar3+kmeshm(5+kcur)-1) )

         else

!        --- kcur < 0, negative direction

            lastpnt = abs ( kmeshm(istar3+kmeshm(4-kcur)) )

         end if

         if ( lastpnt==kfirst ) then

!        --- End of closed curve found

            kfirst = 0
            nholescp = nholescp+1

         end if  ! ( lastpnt==kfirst )

      end do
      if ( debug ) write(irefwr,*) 'nholescp ', nholescp

      if ( nholes.ne.nholescp ) then

!     --- Number of holes in original surface and new surface different

         call errint ( isurnr, 1 )
         call errint ( nholescp, 2 )
         call errint ( iprec, 3 )
         call errint ( nholes, 4 )
         call errsub ( 2622, 4, 0, 0 )

      end if  ! ( nholes.ne.nholescp )

1000  call erclos ( 'msh055surfc7' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh055surfc7'

      end if

      end
