      subroutine msh063 ( iinput, rpin, iseqsurfs, ipointsurf, maxncursur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 30-11-2014 Change isurf 23 in 99
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.1  date 20-10-2008 Allow isurf = 24
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array KMESHS with boundaries of surfaces
!    Help subroutine for subroutine MSH060
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iseqsurfs(*), ipointsurf(*), maxncursur
      double precision rpin(4,*)

!     iinput        i/o   Array containing information of the surfaces
!                         See Prog. Guide 3.1.2
!     ipointsurf     i    Array containing the relative starting addresses
!                         of all surfaces in the part of iinput referring
!                         to surfaces
!     iseqsurfs      i    Contains new sequence of surface numbers
!     maxncursur     i    Estimate of the length of kmeshscopy
!     rpin          i/o   Array containing the thicknesses and factors
!                         of certain curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, lstart, istart, i, ndefsf, kstart, &
              ncrvs, jsurnr, jstart, isurfc, iprec, n, l, lengav
      integer :: iworkcopy(nsurfs+1), iwork1(nsurfs)
      integer :: kmeshscopy(ncurvs+maxncursur+1)

!     i              Counting variable
!     iinp           last position used in array iinput
!     iprec          Surface number of surface to be copied etc.
!     istart         First position used at a part
!     isurfc         Type of the surface to be copied
!     iwork1         integer work array of length nsurfs containing
!                    the inverse of iseqsurfs
!     iworkcopy      Work array of length nsurfs+1 containing a copy
!                    of the first part of kmeshs in the sequence of
!                    iseqsurfs
!     jstart
!     jsurnr         Surface sequence number
!     kmeshscopy     Work array of the same length as kmeshs part 2
!                    containing a copy of kmeshs part 2 in the sequence
!                    of iseqsurfs
!     kstart         Start value of parameter
!     l              General loop variable
!     lengav         Available length for an array
!     lstart         ?
!     n
!     ncrvs          Number of curves
!     ndefsf         Number of curves in surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH06311       Check if all subsurfaces are of the same type
!     MSH06312       Fill kmeshs part curves for surfaces of type 12
!     MSH0637        Fill kmeshs part curves for surfaces of types 7 or 11
!     MSH063GEN      Fill kmeshs part curves for single surfaces
!     PR0012         Print accumulated array
!     PRININ1        print 2d integer array
!     SEPFILLKMESHS  Make relation between kmeshs and kmeshs2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     377   The computed length of array KMESH part Q is more than estimated
!     379   Incorrect type of surface generator found
!    2095   Surface number out of range
!    2096   Surface corresponds to empty surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh063' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh063'
  1      format ( a, 1x, (10i6) )

      end if

      lengav = ncurvs+maxncursur+1+10*nsurfs

      if ( associated(kmeshs) ) nullify(kmeshs)
      allocate ( kmeshs(lengav), stat = error )
      if ( error/=0 ) call eralloc ( error, lengav, 'kmeshs' )
      if ( ierror/=0 ) go to 1000

      n = 0
      iinp = 1
      istart = nsurfs+1 + 8*nsurfs

      kmeshs(1) = istart
      kmeshs(2:istart) = 0

      istart = 0
      iworkcopy(1:nsurfs) = 0
      do i = 1, nsurfs
         iwork1(iseqsurfs(i)) = i
      end do

!     --- Loop over surfaces

      do l = 1, nsurfs
         isurnr = iseqsurfs(l)
         iinp = ipointsurf(isurnr)
         isurf  = iinput(iinp)
         if ( isurf>0 ) then
            ndefsf = mod(iinput(iinp+2),10000)
         else
            ndefsf = 0
         end if

         if ( debug ) &
            write(irefwr,1) 'l, isurnr, isurf, ndefsf', &
                             l, isurnr, isurf, ndefsf

         isurfc = 0
         lstart = nsurfs + 1 + (isurnr-1)*8
         kmeshs(lstart+1) = isurf
         if ( isurf>0 ) kmeshs(lstart+2) = iinput(iinp+1)

         if ( isurf<0 .or. isurf==6 .or. isurf>24 .and. isurf/=99 ) then

!        --- Error 379:  Type of surface incorrect

            call errint ( isurf, 1 )
            call errint ( isurnr, 2 )
            call errsub ( 379, 2, 0, 0)
            go to 1000

         end if

         if ( isurf==1 .or. isurf==5 .or. isurf==6 .or. &
              isurf==17 ) then

            kmeshs(lstart+3) = iinput(iinp+3+ndefsf)
            kmeshs(lstart+4) = iinput(iinp+4+ndefsf)
            kmeshs(lstart+5) = iinput(iinp+5+ndefsf)

         else if ( isurf==8 .or. isurf==9 .or. isurf==14 .or. &
                   isurf==16 .or. isurf==21 ) then

!        --- isurf = 8, 9, 14, 16, 21: i.e. copied surface
!            Store surface number of surface to be copied as well as surface
!            number of originating surface

            iprec = iinput(iinp+3+ndefsf)
            if ( iprec<=0 .or. iprec>nsurfs ) then

!           --- surface number out of range

               call errint ( iprec, 1 )
               call errint ( nsurfs, 2 )
               call errint ( isurnr, 3 )
               call errsub ( 2095, 3, 0, 0 )

            else if ( kmeshs(nsurfs+2+(iprec-1)*8)==0 ) then

!           --- surface empty

               call errint ( iprec, 1 )
               call errint ( isurnr, 2 )
               call errsub ( 2096, 2, 0, 0 )

            end if
            kmeshs(lstart+3) = iprec
            kstart = nsurfs+1+(iprec-1)*8
            isurfc = kmeshs(kstart+1)
            if ( isurfc==8 .or. isurfc==9 .or. isurfc==14 .or. &
                 isurfc==16 ) iprec = kmeshs(kstart+4)
            kmeshs(lstart+4) = iprec

         else if ( isurf==20 ) then

!        --- isurf = 20   sphere

            kmeshs(lstart+3) = iinput(iinp+6)
            kmeshs(lstart+4) = iinput(iinp+7)
            kmeshs(lstart+5) = iinput(iinp+8)
            kmeshs(lstart+6) = iinput(iinp+9)

         else if ( isurf==11 ) then

!        --- isurf = 11   ordered surface of surfaces
!            Check if all subsurfaces are of the same type
!            (rectangle or pipe surface)

            call msh06311 ( kmeshr, kmeshs, isurnr, nsurfs, iinput, &
                            iinp, lstart, isurf )

         end if

         select case ( isurf )

            case (7, 11, 24)

!           --- isurf = 7 or 11    surface consisting of surfaces

               call msh0637 ( iinput, isurf, iinp, ncurvs, ndefsf, &
                              nsurfs, isurnr, kmeshs, iworkcopy, iwork1, &
                              kmeshscopy, lstart, maxncursur, istart )

            case (12)

!           --- isurf = 12  Ordered surface of pipe surfaces
!                           Each pipe surface contains three or four curves

               call msh06312 ( iinput, iinp, ncurvs, nsurfs, &
                               kmeshs, iworkcopy, iwork1, &
                               kmeshscopy, lstart, maxncursur, istart )

            case default

!           --- isurf # 7, 11 or 12   standard surface

               call msh063gen ( iinput, iinp, ndefsf, &
                                kmeshscopy, lstart, maxncursur, &
                                istart, isurfc, rpin )

         end select  ! case ( isurf )

         if ( isurfc==10 ) lstart = lstart - 2
         if ( istart+max(1,n)>lengav ) then

!        --- Error 377:  Not enough space available

            call errint ( istart+n, 1 )
            call errint ( lengav, 2 )
            call errsub ( 377, 2, 0, 0 )

         end if
         iworkcopy(1+l) = istart

      end do  ! l = 1, nsurfs

!     --- Finally copy parts i and iii of kmeshs from kmeshscopy and iworkcopy
!         in the natural sequence

      istart = kmeshs(1)
      do isurnr = 1, nsurfs
         jsurnr = iwork1(isurnr)
         jstart = iworkcopy(jsurnr)
         ncrvs = iworkcopy(jsurnr+1)-jstart
         do i = 1, ncrvs
            kmeshs(istart+i) = kmeshscopy(jstart+i)
         end do
         istart = istart+ncrvs
         kmeshs(isurnr+1) = istart
      end do

      kelms = kmeshs(nsurfs+1)
      allocate ( km(iseqkm)%kmeshs(kelms) )
      km(iseqkm)%kmeshs = kmeshs(1:kelms)

      deallocate ( kmeshs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshs' )

      kmeshs => km(iseqkm)%kmeshs
      call sepfillkmeshs ( km(iseqkm)%kmeshs(nsurfs+2) )

      if ( debug ) then

!     --- Debugging statements

         write (irefwr,*) 'curves corresponding to surfaces'
         call pr0012 ( nsurfs, kmeshs, kmeshs )
         call prinin1 ( kmeshs(nsurfs+2), nsurfs, 8, &
                        'Rest of information per surface' )

      end if

1000  if ( debug ) write(irefwr,*) 'End msh063'
      call erclos ( 'msh063' )

      end
