      subroutine msh354 ( kmscbt, kmscps, kbndbt, nelbot, weddir, &
                          found, klp, nlp )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  3.0  date 24-08-2010 Removal of work arrays
!        version  2.1  date 13-05-2010 Fortran 90
!        version  2.0  date 29-04-1999 Extra param's for hollow pipes (BvR)
!        version  1.2  date 29-04-1998 compute random nr's differently
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Fill the array distri with the directions of the diagonals
!       on the sides of the wedges resulting from the translation
!       of a triangle in pipe surface.  These diagonals, when going
!       around the bottom triangle, can be bottom-up or top-down.
!       A wedge can be filled with 3 tetrahedrons, instead of 8 with
!       an additional node, if for a wedge the diagonals run not
!       all bottom-up or all top-down.  Thus the diagonals that are
!       stored in distri are chosen such that all wedges contain
!       diagonals in both directions.  If this is achieved found
!       is set to 1 else to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelbot,kmscbt(3,nelbot), kbndbt(*), weddir(7,nelbot), &
              found, kmscps(3,*), klp(2,*), nlp

!     found   o    Boolean indicating succes 1, or failure 0
!     kbndbt  i    Array KBNDPT for bottom surface
!                  Contains the local nodal point numbers of the boundary of
!                  the bottom surface
!     klp     i    In this array the relative node numbers with respect to
!                  array kbndbt of first and last point of a consecutive part
!                  are stored. The number of parts the boundary consists of
!                  is nlp
!                  Hence klp(1,1) = 1, klp(2,1) denotes the end point of the
!                  first part of the boundary
!                  klp(1,2) contains the node number of the first point of
!                  the second part. This must be klp(2,1)+1
!                  klp(2,nlp) contains the length of kbndbt
!     kmscbt  i    Kmesh part c corresponding to bottom surface
!                  Contains the local nodal point numbers of the bottom surface
!     kmscps  i    Kmesh part c corresponding to pipe surface
!                  This array is required to check the direction of diagonals
!                  in the pipe surface
!     nelbot  i    Number of elements in bottom surface
!     nlp     i    Number of disjoint parts the boundary of the bottom
!                  surface consists of
!     weddir  o    Direction of the diagonals per wedge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iel, side(4), isid, jel, jsid, chk, trouble, ready, &
              dir, inbnd1, inbnd2, iupper, iunder, ielpsf, &
              j1, j2, nbndel, side1(4), dir1, ilp, jlp, ii, &
              conedge(6,nelbot), leftover
      double precision randomnr

!     chk        indicates whether condition in loop was met (1) or not (0)
!     conedge
!     dir        direction of a side
!     dir1       random direction of a side
!     i          counting variable
!     iel        counting variable for elements
!     ielpsf     element number related to pipesurface (?)
!     ii         Counting variable
!     ilp        Counting variable
!     inbnd1     first node of iel on the boundary of the domain
!     inbnd2     second node of iel on the boundary of the domain
!     isid       counting variable for sides
!     iunder     lower bound
!     iupper     upper bound
!     j1         node of boundary element related to inbnd1
!     j2         node of boundary element related to inbnd2
!     jel        counting variable for elements
!     jlp
!     jsid       counting variable for sides
!     leftover   side that has not been filled
!     nbndel     Number of boundary elements
!     randomnr   Random number between 0 and 1
!     ready      indicates whether all sides have been filled (1) or not (0)
!     side       array containing the three edges sequentially
!                nodes of edge i are stored in side(i) and side(i+1)
!     side1      array containing the three edges sequentially
!                nodes of edge i are stored in side(i) and side(i+1)
!     trouble    indicates wether a diagonilisation was found (0) or not (1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer mshpavin
      double precision :: prandm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     MSH355         assigns direction to one side of element
!     MSHPAVIN       determines whether a list is contained by another list
!                    e.g. whether a node occurs in an element
!     PRANDM         returns random number between [0,1)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1056   Old method is applied for tetrahedrons
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh354' )

!     --- initialize random generator

      if ( testversion ) then
         randomnr = prandm(nelbot**3)
      else
         call random_number ( randomnr )
      end if  ! ( testversion )

!     --- set weddir and conedge to zero

      weddir = 0
      conedge = 0

!     --- fill conedge

      do iel = 1, nelbot
         side(1:3) = kmscbt(1:3,iel)
         side(4) = kmscbt(1,iel)

         do isid=1,3
            do jel = iel+1, nelbot
               side1(1:3) = kmscbt(1:3,jel)
               side1(4) = kmscbt(1,jel)
               if (mshpavin(side(isid),2,side1(1),3)==1) then
                  do jsid=1,3
                     if ( side(isid+1)==side1(jsid)  .and. &
                          side(isid)  ==side1(jsid+1) ) then
                        conedge(isid,iel)   = jel
                        conedge(isid+3,iel) = jsid
                        conedge(jsid,jel)   = iel
                        conedge(jsid+3,jel) = isid
                     else if ( side(isid)==side1(jsid)  .and. &
                              side(isid+1)==side1(jsid+1) ) then
                        conedge(isid,iel)   = jel
                        conedge(isid+3,iel) = -jsid
                        conedge(jsid,jel)   = iel
                        conedge(jsid+3,jel) = -isid
                     end if
                  end do
               end if
            end do
         end do
      end do

!     --- fill weddir

      nbndel = 0

      do iel = 1, nelbot

         do isid=1,3
            if (conedge(isid,iel)==0) then

            nbndel = nbndel + 1

!           --- this is the boundary of the bottom surface, &
!               determine direction of diagonal from kmscps
!               these are the two nodes on the boundary

               inbnd1 = kmscbt(isid,iel)
               inbnd2 = kmscbt(mod(isid+1-1,3)+1,iel)

!              --- find the corresponding boundary element

               j1 = 0
               j2 = 0
               ii = 0
               do ilp=1,nlp
                  if (j1==0 .and. j2==0) then
                     do i=klp(1,ilp),klp(2,ilp)
                        ii = ii+1
                        if (inbnd1==kbndbt(i)) then
                           j1 = ii
                           jlp = ilp
                        else if (inbnd2==kbndbt(i)) then
                           j2 = ii
                           jlp = ilp
                        end if
                     end do
                  end if
               end do

!              --- this is from msh352, it seems this won't always work
!                  but there are apparently restrictions on how
!                  kmscps will be filled

               iunder = min ( j1,j2 )
               iupper = max ( j1,j2 )
               if ( iupper==iunder+1 ) then
                  ielpsf = j1
               else if ( iupper==klp(2,jlp)-jlp+1 .and. &
                         iunder==klp(1,jlp)-jlp+1 ) then
                  ielpsf = iupper
                  j2 = iupper+1
               else
                  ielpsf = j2
                  j2 = j2+1
               end if

               if ( kmscps(1,2*ielpsf)==j2 ) then

!              --- positive orientation of surface

                  if (kmscps(2,2*ielpsf-1)==j1) then

!                 --- diagonal runs from j1 to across

                     weddir(isid,iel)    = 1
                  else

!                 --- diagonal runs from j2 to across

                     weddir(isid,iel)    = -1
                  end if
               else

!              --- negative orientation of surface

                  if (kmscps(2,2*ielpsf)==j2) then

!                 --- diagonal runs from j2 to across

                     weddir(isid,iel)    = -1
                  else

!                 --- diagonal runs from j1 to across

                     weddir(isid,iel)    = 1
                  end if
               end if

!              --- number of prescribed diagonals is increased

               weddir(4,iel)      = weddir(4,iel)+1

!              --- indicate this one  cannot be changed

               weddir(isid+4,iel) = 1
            end if
         end do
      end do

      ready = 0

 10   if (ready==0) then

!     --- find the weddir with 2 defined sides with
!         the same direction on those sides

         do iel=1,nelbot
            if (weddir(4,iel)==2) then
               dir = 0
               chk = 0
               do isid=1,3
                  if (dir==0 .and. weddir(isid,iel)/=0) then
                     dir = weddir(isid,iel)
                  else if(weddir(isid,iel)==dir .and. &
                         weddir(isid,iel)/=0 .and. dir/=0) then
                     chk=1
                  else
                     leftover = isid
                  end if
               end do
               if (chk==1) then
                  call msh355(weddir,conedge,iel,leftover,-dir,1)
               end if
            end if
         end do

!        --- find the weddir with 2 defined sides with
!            the different direction on those sides

         do iel=1,nelbot
            if (weddir(4,iel)==2) then
               dir = 0
               chk = 0
               do isid=1,3
                  if (dir==0 .and. weddir(isid,iel)/=0) then
                     dir = weddir(isid,iel)
                  else if(weddir(isid,iel)/=dir .and. &
                         weddir(isid,iel)/=0 .and. dir/=0) then
                     chk=1
                  else
                     leftover = isid
                  end if
               end do
               if (chk==1) then
                  if ( testversion ) then
                     randomnr = prandm(0)
                  else
                     call random_number ( randomnr )
                  end if  ! ( testversion )
                  if ( randomnr>0.5d0 ) then
                     dir1 = 1
                  else
                     dir1 = -1
                  end if
                  call msh355(weddir,conedge,iel,leftover,dir1,0)
               end if
            end if
         end do

!        --- set one side belonging to a wedge with one fixed
!            diagonal

         chk = 0
         iel = 1
 20      if (chk==0 .and. iel<=nelbot) then
            if (weddir(4,iel)==1) then
               chk = 1
               do isid=1,3
                  if (weddir(isid,iel)/=0) then
                     dir = weddir(isid,iel)
                     leftover = mod(isid+1,3)+1
                  end if
               end do
               if ( testversion ) then
                  randomnr = prandm(0)
               else
                  call random_number ( randomnr )
               end if  ! ( testversion )
               if ( randomnr>0.5d0 ) then
                  dir1 = 1
               else
                  dir1 = -1
               end if
               call msh355(weddir,conedge,iel,leftover,dir1,0)
            end if
            iel = iel+1
            goto 20
         end if

!        --- see if there are empty weddir left

         ready = 1
         iel = 1
 30      if (ready==1 .and. iel<=nelbot) then
            if (weddir(4,iel)/=3) then
               ready = 0
            end if
            iel = iel + 1
            goto 30
         end if

         goto 10
      end if

!     --- now all the wedge diagonals are set.  They may, however, not
!         all be correct.  Check whether they are all correct and,
!         if one is not, correct this by flipping one diagonal to one
!         element

      ready = 0
      trouble = 0

 40   if (ready==0 .and. trouble==0) then
         ready=1
         do iel=1,nelbot
            dir = weddir(1,iel)
            chk=0
            do isid=2,3
               if (weddir(isid,iel)/=dir) then
                  chk=1
               end if
            end do

            if (chk==0) then

!           --- This wedge is wrong

               ready=0

!              --- Flip one of the diagonals of the wedge
!                  only if it is flippable.  Each diagonal is flippable once

               trouble = 1
               do isid = 1,3
                  if (weddir(isid+4,iel)==0 .and. trouble==1) then
                     trouble=0

!                    --- do not flip again

                     weddir(isid+4,iel) = 1
                     weddir(isid,iel) = -dir
                     jel = conedge(isid,iel)
                     jsid = conedge(isid+3,iel)
                     weddir(abs(jsid),jel) = dir*sign(1,jsid)
                  end if
               end do
            end if
         end do
         goto 40
      end if

      if (trouble==0) then

!     --- New method is used

         found = 1

      else

!     --- Old method is used

         call errwar ( 1056, 0, 0, 0 )
         found = 0

      end if
      call erclos ( 'msh354' )

      end
