      subroutine msh465 ( ifile, coorh, npoint, nbndpt, &
                          kelemh, nelemh, funkh )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 17-04-1994 introduction of klasse
!        version  1.1  date 19-04-1994 small adjustments
!        version  1.0  date 20-03-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     MSH465 reads points from file with number ifile and makes a
!     triangular grid such that all points read are incorporated.
!     It is necessary that the first (NBNDPT) points constitute the
!     boundary, that all remaining points are inside the region defined
!     by this boundary and that the boundary points are given anti-clock- &
!     wise.
! **********************************************************************
!
!                       KEYWORDS
!
!     data
!     read
!     tesselation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifile, npoint, nbndpt, kelemh(*), nelemh
      double precision coorh( 2, npoint ), funkh( npoint )

!     coorh   o    coordinate array to be used for TESSELATION mesh
!     funkh   o    array with function values in all nodes of mesh
!     ifile   i    unit number of the file where the information of the
!                  measurements has been stored
!     kelemh  o    element array of TESSELATION mesh
!     nbndpt  i    number of boundary points in array
!     nelemh  o    number of elements in kelemh
!     npoint  i    number of points in coorh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ncl, nx, ny, klaar(npoint), kstapl(2*npoint), &
              klasse(2,npoint)
      double precision xp, yp, funk, xmax, xmin, ymax, ymin, &
                       dstepx, dstepy

!     dstepx  step in x-direction
!     dstepy  step in y-direction
!     funk    function value
!     i       counting variable
!     klaar   helparray for TESSA
!     klasse  helparray for all points to be used to make a subdivision
!             in classes for method = 1
!     kstapl  helparray for TESSA
!     ncl     number of classes
!     nx      number of classes in x-direction
!     ny      number of classes in y-direction
!     xmax    max x-value encountered
!     xmin    min x-value encountered
!     xp      x-coordinate of point
!     ymax    max y-value encountered
!     ymin    min y-value encountered
!     yp      y-coordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     MSH468  Make element grid for points given
!     MSH473  Divide region in a number of subclasses
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read coordinates of all the points and the corresponding function
!     values from the indicated file.
!     Divide region in a number of classes (MSH473)
!     Make grid for the points (make use of the fact that only points
!     from adjacent classes are important) and store the grid in
!     KELEMH  (MSH468)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh465' )

!     --- Read all the points and function values

      xmax = -rinfin
      xmin =  rinfin
      ymax = -rinfin
      ymin =  rinfin

      do 100 i = 1 , npoint

          read( ifile, * ) xp, yp, funk

          coorh( 1,i ) =    xp
          coorh( 2,i ) =    yp

          if ( xp.gt.xmax ) xmax = xp
          if ( xp.lt.xmin ) xmin = xp
          if ( yp.gt.ymax ) ymax = yp
          if ( yp.lt.ymin ) ymin = yp

          funkh(  i  ) =    funk

100   continue

      xmin = xmin - 10 * epsmac
      ymin = ymin - 10 * epsmac

      xmax = xmax + 10 * epsmac
      ymax = ymax + 10 * epsmac

!     --- Determine number of classes and for each point its class number
!         (Tuning indicates that 8 points in a class is a good choice, &
!         smaller numbers are better, but if points are not too regular
!         problems may be encountered if classes turn out to be empty)

      ncl = npoint / 8 + 1

      ny  = nint( sqrt( ncl * (ymax - ymin) / (xmax - xmin) ) )
      if ( ny.lt.1 ) ny = 1
      nx  = nint( float( ncl / ny ) )

!     --- Stepsize

      dstepx = ( xmax - xmin ) / ( 1d0 * nx )
      dstepy = ( ymax - ymin ) / ( 1d0 * ny )

      call msh473( coorh, npoint, xmin, ymin, klasse, dstepx, dstepy )

!     --- Number of elements expected

      nelemh = 2 * ( npoint - 1 ) - nbndpt

!     --- Make grid for these points and store the elements in kelemh

      call msh468 ( nbndpt, coorh, npoint, kelemh, nelemh, &
                    kstapl, klaar, klasse )

      call erclos( 'msh465' )

      end
