      subroutine mshadap ( inpada, rinada, inpmsh, rinmsh, icurvs, &
                           iread, numvecs, iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 11-12-2017 New storage of intmat
!        version  5.0  date 20-08-2016 use sepmoduleinput
!        version  4.2  date 15-11-2012 Long integers
!
!   copyright (c) 1997-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Body of subroutine adapmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     change_boundary
!     change_mesh
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleinput
      use sepmoduletime
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  inpada(*), inpmsh(*), icurvs(*), iread, numvecs, iseqvecs(*)
      double precision rinada(*), rinmsh(*)

!     icurvs         i    Integer array containing the curve numbers for the
!                         adaptation of the boundary. A negative curve
!                         number indicates that the curve must be computed in
!                         the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array inpada
!                         icurvs is only used if iread = -1
!                         If the boundary must be adapted in a series of calls, &
!                         i.e. NCALL>1, then the curves must be stored in the
!                         sequence of the various calls. Hence first all ncurves
!                         curves for the first call, then for the second one and
!                         so on
!     inpada         i    Integer array to be filled by the user, indicating how
!                         the boundary must be adapted. inpada is only used if
!                         iread = -1
!                         inpada contains the input arrays inpada for the
!                         various calls of adapboun sequentially.
!                         Hence:
!                         inpada(1) contains the last entry for call 1 (ilast1)
!                         inpada(1-ilast1) contains the complete array inpada
!                         for the first call of adapboun
!                         inpada(1+ilast1) contains the last entry for call 2
!                         (ilast2), however, shifted with ilast1.
!                         So the actual last entry is ilast1+ilast2
!                         inpada(ilast1+1-ilast1+ilast2) contains the complete
!                         array inpada for the second call of adapboun.
!                         This process is repeated.
!                         The easiest way to implement this in the calling
!                         program is to define a more dimensional array
!                         inpada(LENADA,NCALL), and to give
!                         inpada(1,i) the value LENADA
!     inpmsh         i    Integer array to be filled by the user, indicating how
!                         the mesh must be adapted.
!                         INPMSH is only used if iread = -1
!                         INPMSH must be filled as follows:
!                         Pos. 1   Position of last entry that has been filled
!                                  by the user. If inpmsh(1) = 0 or 1, only
!                                  defaults are used.
!                         Pos. 2   NCALL: Number of calls of adapboun that must
!                                  be used to update the complete boundary
!                                  before updating the mesh.
!                                  In this way it is possible to update parts
!                                  of the boundary separately
!                                  Default value: 1
!                         Pos. 3   CHANGE_TOPOLOGY: defines whether the topology
!                                  must be adapted or not
!                                  Possible values:
!                                  0: The topology remains unchanged
!                                  1: The topology is always adapted
!                                  2: The topology is adapted if necessary
!                                  3: The adaptation of the topology is
!                                     performed in two steps
!                                     First the mesh is updated without changing
!                                     the topology
!                                     Then the arrays kmesh and kprob are copied
!                                     for interpolation purposes
!                                     Next remeshing is applied
!                                     This option is an extension of 1 and makes
!                                     only sense in combination with inpmsh(5)>0
!                                  Default value: 0
!                         Pos. 4   PLOT_MESH: defines whether the new mesh
!                                  must be plotted or not
!                                  Possible values:
!                                  0: No plot
!                                  1: Plot
!                                  Default value: 0
!                         Pos. 5   INTERPOLATE_SOL: defines the number of
!                                  solution vectors that must be interpolated
!                                  Default value: 0
!                         Pos. 6   INDEX_SOL: defines the starting address in
!                                  INPMSH of the vectors to be interpolated
!                                  In positions INPMSH(INDEX_SOL) to
!                                  INPMSH(INDEX_SOL+INTERPOLATE_SOL-1) the
!                                  sequence numbers of the solution vectors
!                                  to be interpolated are stored.
!                                  If INDEX_SOL = 0, the vectors 1 to
!                                  INTERPOLATE_SOL are interpolated
!                                  If INDEX_SOL<0, the vectors
!                                  -INDEX_SOL to -INDEX_SOL+INTERPOLATE_SOL-1
!                                  are interpolated
!                                  Default value: 0
!     iread          i    Defines if parameters should be read or
!                         are given in the arrays INPADA and INPMSH through the
!                         parameter list.
!                         Possible values:
!                         0   The input is read from the standard input file
!                         >0  The input as read from the standard input file by
!                             subroutine SEPSTM is used. The value of iread
!                             defines the sequence number
!                         -1  No input is read, all information from the input
!                             arrays is used
!                         iread with respect to inpada and inpmsh is the same
!     iseqvecs       i    Contains sequence numbers of vectors with respect to
!                         isol
!     numvecs        i    Number of vectors in iseqvecs
!     rinada        i/o   See inpada
!     rinmsh        i/o   See inpmsh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nadapmsh, nrladapm, ncall
      parameter ( nadapmsh=40, nrladapm=1 )
      integer icopmsh(nadapmsh), itypvc, i, iseqnr, iinmsh, irlmsh, &
              istart, istart1, istart2, iinput(1), idummy(1), &
              iinplt(1), n1, ireadbn, ivectr, maxcal, ipos, iseqprob
      double precision rlcopmsh(nrladapm), rinput(5), rinplt(2)
      integer, pointer :: readbn(:)
      logical changed, changeinput

!     changed        If true the number of nodes along the boundary has been
!                    changed
!     changeinput    If true the input arrays iinput and rinput have been
!                    changed
!     i              Counting variable
!     icopmsh        Copy of array inpmsh extended with default values
!     idummy         Dummy parameter
!     iinmsh         Memory management sequence number corresponding to inpmsh
!     iinplt         Input array for subroutine plotms
!     iinput         integer input array (used as dummy)
!     ipos           Position of input arrays in sepmoduleinput
!     ireadbn        Parameter iread for adapboun
!     irlmsh         Memory management sequence number corresponding to rinmsh
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     iseqprob       Sequence number of problem input
!     istart         First position used at a part
!     istart1        First position used at a part
!     istart2        First position used at a part
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivectr         sequence number of first vector referred to in iseqvecs
!     maxcal         Maximum number of coefficient inputs read
!     n1             Number of items filled in inpmsh
!     nadapmsh       Length of array inpmsh
!     ncall          Number of calls to a specific under subroutine
!     nrladapm       Length of array rinmsh
!     readbn         contains the sequence numbers
!                    with respect to adapt boundary
!     rinplt         Input array for subroutine plotms:
!                    1:    plotfm
!                    2:    yfact
!     rinput         Real input array (used as dummy)
!     rlcopmsh       Copy of array rinmsh extended with defaults
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADAPBNDB       Adapt the boundary of a mesh
!     CHECKRANGE     Check range of integer parameter
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCOMGEN       Compute matrix structure for free boundary problem
!     INICHINPSOL    Adapt input array with respect to variables
!     MSHREMSH       Perform remeshing
!     PRININ         print 1d integer array
!     PROBDFEXT      Reads and stores problem definition
!     READADMS       Read input for subroutine ADAPMESH from the standard input
!                    file
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPMESHSUB     General mesh generator
!     SEPPLOTMS      plot mesh
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutines READADBN and READADMS
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      60   Input solution array has not been filled or is of the wrong type
!     419   NDIM # 2
!    1378   Parameter has wrong value
!    1948   No input with the main keyword has been read
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform standard tests
!    if ( iread=-1 ) then
!       Fill icopmsh with default values
!       Copy contents from inpmsh into icopmsh
!    else if ( iread=0 ) then
!       Read input from the standard input file
!    else ( iread>0 )
!       Copy input read before from ibuffr into icopmsh
!    Check values
!    ncall := Number of calls to adapboun
!    For i := 1 (1) ncall do
!       call adapboun to adapt the boundary
!    Change mesh
!    if ( topology has been changed ) then
!       Call probdf to change the problem description
!       Call commat to change the matrix structure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadap' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadap'
         write(irefwr,1) 'nelem, npoint', nelem, npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Standard tests

      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
      end if
      if ( numvecs<1 .or. numvecs>100 ) then

!     --- numvecs out of range

         call errint ( numvecs, 1 )
         call errint ( 100, 2 )
         call errint ( 1, 3 )
         call errchr ( 'numvecs', 1 )
         call errsub ( 1378, 3, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000
      ivectr = iseqvecs(1)

      call checkrange ( ivectr, 1, 100, 1, 'ivectr', 'iseqvecs' )
      if ( ierror/=0 ) go to 1000

      itypvc = ks(isolut2d(ivectr))%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 ) then
         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0)
      end if
      ichangeobst = 0
      ichanmesh = 0
      if ( ierror/=0 ) go to 1000

!     --- Fill array icopmsh

      if ( debug ) write(irefwr,1) 'iread', iread

      if ( iread==-1 ) then

!     --- iread = -1, fill icopmsh from inpmsh

         icopmsh(1) = nadapmsh
         icopmsh(2) = 1
         icopmsh(3) = 1
         do i = 4, nadapmsh
            icopmsh(i) = 0
         end do
         do i = 1, min(inpmsh(1),nadapmsh)
            icopmsh(i) = inpmsh(i)
         end do
         do i = 1, nrladapm
            rlcopmsh(i) = rinmsh(i)
         end do
         ncall = icopmsh(2)

         allocate ( readbn(ncall), stat = error )
         if ( error/=0 ) call eralloc ( error, ncall, 'readbn' )
         if ( ierror/=0 ) go to 1000

         readbn = -1

      else if ( iread==0 ) then

!     --- iread = 0, fill icopmsh by reading from the standard input file

         ncall = 25

         allocate ( readbn(ncall), stat = error )
         if ( error/=0 ) call eralloc ( error, ncall, 'readbn' )
         if ( ierror/=0 ) go to 1000
         iseqnr = 1
         call readadms ( icopmsh, rlcopmsh, 1, iseqnr, readbn, &
                         nadapmsh )

!        --- Adapt variables

         call inichinpsol ( icopmsh, icopmsh(1) )
         call inichinpsol ( readbn, icopmsh(2) )

      else

!     --- iread>0, fill icopmsh by copying from ibuffr

         iseqnr = iread
         call sepgetinpfinf ( 17, iseqnr, ipos, maxcal )
         if ( .not. in(ipos)%used ) then
            call errchr ( 'ADAPT_MESH', 1 )
            call errsub ( 1948, 0, 0, 1 )
            go to 1000
         end if
         n1 = in(ipos)%iinput(1)
         icopmsh(1:n1) = in(ipos)%iinput(1:n1)
         rlcopmsh = re(ipos)%rinput(1:nrladapm)
         ncall = icopmsh(2)

         allocate ( readbn(ncall), stat = error )
         if ( error/=0 ) call eralloc ( error, ncall, 'readbn' )
         if ( ierror/=0 ) go to 1000

         readbn = in(ipos)%iinput(n1+1:n1+ncall)

!        --- Adapt variables

         call inichinpsol ( icopmsh, n1 )
         call inichinpsol ( readbn, ncall )

      end if
      if ( ierror/=0 ) go to 1000

      ncall = icopmsh(2)
      if ( debug ) write(irefwr,1) 'ncall', ncall
      do i = 1, ncall

!     --- call adapboun to adapt the boundary

         istart = 1
         istart1 = 1
         istart2 = 1
         if ( i>1 .and. iread==-1 ) then
            istart2 = istart2+inpada(istart+1)
            istart = istart+inpada(1)*(i-1)
            istart1 = istart1+4*(i-1)
         end if
         ireadbn = readbn(i)+1000*ncall
         call adapbndb ( inpada(istart), rinada(istart1), &
                         icurvs(istart2), ireadbn, changed, &
                         changeinput, numvecs, iseqvecs )

      end do
      if ( ierror/=0 ) go to 1000

!     --- Change the mesh

      if ( debug ) then
         write(irefwr,*) 'changed, changeinput',changed, changeinput
         call prinin ( icopmsh, 5, 'icopmsh' )
      end if  ! ( debug )

      if ( .not. changed .and. icopmsh(3)==0 ) then

!     --- The topology is not adapted

         call sepmeshsub ( 11, iinput, rinput )
         if ( debug ) write(irefwr,1) 'nelem, npoint/1', nelem, npoint
         if ( ierror/=0 ) go to 1000

         if ( ichangeobst==1 ) then

!        --- An obstacle has been activated, probdf and so on must be called

            iseqprob = 1
            call probdfext ( 3, iseqprob )
            if ( ierror/=0 ) go to 1000
            call incomgen ( iread, intmat1d )
            if ( ierror/=0 ) go to 1000

         end if

      else if ( .not. changeinput ) then

!     --- New topology

         if ( icopmsh(3)==3 .and. icopmsh(5)>0 ) &
            call sepmeshsub ( 11, iinput, rinput )
         if ( ierror/=0 ) go to 1000
         call mshremsh ( isolut2d, iread, icopmsh, 2 )

      else

!     --- Complete new topology, update of mesh is not possible
!         A new mesh is created and the solution is interpolated
!         The mesh velocity must be set to 0, so in fact we are using an Euler
!         step

         ichanmesh = 1
         call mshremsh ( isolut2d, iread, inpmsh, 3 )

      end if
      if ( ierror/=0 ) go to 1000
      if ( icopmsh(4)==1 .and. &
          (itimloop==0 .or. ioutact==1) ) then

!     --- Plot the mesh

         iinplt(1) = 0
         rinplt(1) = 15d0
         rinplt(2) = 1d0
         call sepplotms ( iinplt, rinplt, idummy )

      end if

      deallocate ( readbn, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'readbn' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadap'

      end if  ! ( debug )
      call erclos ( 'mshadap' )

      end subroutine mshadap
