      subroutine mshcheck ( inpchk, rinchk, jcheck )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.2  date 15-11-2012 Long integers
!        version  5.1  date 02-10-2012 New call to mshobst
!        version  5.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Check if the mesh generated satisfies the conditions as described
!   in the arrays inpchk and rinchk.
!   Return with a value for jcheck indicating if remeshing should be applied
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpchk(*), jcheck
      double precision rinchk(*)

!     inpchk     i    Indicates how the checking must be applied
!                     inpchk must be filled as follows:
!                     1:  indication of how the boundary must be checked
!                         Possible values:
!                         0   No checking of the boundary, except in the
!                             case that general is applied
!                             If general is applied all boundaries of the
!                             surfaces generated by general are checked
!                             with respect to the general requirements
!                         1   The first and last element of each curve
!                             in array icurves is compared with the
!                             original length stored in array kmesh p
!                             If 1/a<dx/dx_orig<a the curve is
!                             accepted, otherwise jcheck is set equal to 1
!                             dx is the actual length of the element
!                             dx_orig is the original length of the element
!                             a must be stored in rinchk(1)
!                         As soon as one of the curves returns with jcheck = 1, &
!                         checking is stopped
!                     2:  indication of how the internal elements must be
!                         checked. This is only applied if the boundary
!                         check returns with jcheck = 0
!                         Possible values:
!                         0   No checking of the internal elements
!                         1   All angles of the elements are checked
!                             If the smallest angle of an element<rinchk(2)
!                             or the largest angle of an element>rinchk(3)
!                             jcheck is set equal to 2 and checking is stopped
!                         2   See 1, but now jcheck is set to 1, meaning that
!                             the number of nodes along the boundary may be
!                             changed
!     jcheck     o    Indicates if remeshing must be applied or not
!                     Possible values:
!                     0  No remeshing necessary
!                     1  The boundary does not satisfy the requirements
!                     2  The internal elements do not satisfy the requirements
!     rinchk     i    See inpchk
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lencurvs, lengcv, iobst, i
      logical changed
      integer, allocatable :: icurvscp(:)
      double precision, pointer :: curvscp(:,:), curvefactscp(:)

!     changed        If true the number of nodes along at least one curve may
!                    change
!     curvefactscp   Copy of array curvefacts
!     curvscp        copy of array curves
!     i              Counting variable
!     icurvscp       Copy of array icurvs
!     iobst          Obstacle sequence number
!     lencurvs       Overestimate of length of new array curves
!     lengcv         Length of old array curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH074         Update IINPUT for curves of which the number of nodes may
!                    be changed
!     MSHCHKCV       Check quality of curves
!     MSHCHKSF       Check quality of surfaces
!     MSHCURFR       Compute new arrays icurvs and curves in case the number of
!                    nodes at the boundary may be changed
!     MSHOBST        Restrict new boundary to region outside the obstacles
!     MSHSEQCURVS    Compute sequence in which curves must be created
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     jcheck = 0
!     Check distances along curves
!     If ( jcheck=0 ) then
!        Check angles in elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheck' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcheck'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcheck = 0

!     --- Check distances along curves

      call mshchkcv ( inpchk, rinchk, jcheck )
      if ( jcheck==0 ) then

!     --- jcheck = 0, Check angles in elements

         call mshchksf ( inpchk, rinchk, jcheck )

      end if
      if ( jcheck==1 ) then

!     --- jcheck = 1, indicate that number of nodes along boundaries may be
!                     adapted

         call msh074 ( kmeshz, ncurvs, iinputkm, changed )
         if ( changed ) then

!        --- Update arrays curves and icurvs
!            First copy old array curves to curvescp and icurvs to icurvscp

            if ( kmeshh(1,1)==0 ) then

!           --- No obstacles have yet been activated

               numobst = 0

            else

!           --- Obstacles have been activated at least once
!               Now check if any of the obstacles are still active

               if ( kmeshh(4,1)>0 ) then
                  do i = 1, ncurvs
                     if ( iobscurv(4)>0 ) go to 100
                  end do
               end if
               numobst = 0

            end if
100         lengcv = size ( curvesmsh,2 )
            lencurvs = lengcv*5

            alloccurv = .true.

            if ( debug ) write(irefwr,*) 'before associated(curvefacts)'
            if ( associated(curvefacts) ) then

!           --- Copy old curvefacts temporarily to curvefactscp

               length = size(curvefacts)
               allocate ( curvefactscp(length) )
               curvefactscp = curvefacts
               nullify ( curvefacts )

            end if  ! ( associated(curvefacts) )

            if ( debug ) write(irefwr,*) 'before allocate ipointcurv'
            allocate ( ipointcurv(ncurvs), iseqcurvs(ncurvs), &
                       curvefacts(lencurvs), &
                       stat = error )
            if ( error/=0 ) call eralloc ( error, ncurvs, 'ipointcurv' )
            if ( ierror/=0 ) go to 1000

            call mshseqcurvs ( iinputkm )

            allocate ( icurvscp(ncurvs+1), curvscp(ndim,lengcv), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, ncurvs+lencurvs, 'icurvscp' )
            if ( ierror/=0 ) go to 1000

            curvscp(1:ndim,1:lengcv) = curvesmsh
            icurvscp(2:1+ncurvs) = icurvsmsh(1:ncurvs)
            icurvscp(1) = 0

            nullify(curvesmsh)
            allocate ( curvesmsh(ndim,lencurvs) )
            jcoars = iinputkm(11)
            call mshcurfr ( icurvscp, curvscp, lencurvs, &
                            iinputkm, rinputkm, changed )

            if ( debug ) write(irefwr,*) 'before deallocate icurvscp'
            deallocate ( icurvscp, curvscp, iseqcurvs, ipointcurv, &
                         curvefacts, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'icurvscp' )
            if ( ierror/=0 ) go to 1000

            if ( debug ) write(irefwr,*) 'before associated(curvefactscp)'
            if ( associated(curvefactscp) ) then

!           --- Copy old curvefacts back

               allocate ( km(iseqkm)%curvefacts(size(curvefactscp)) )
               curvefacts => km(iseqkm)%curvefacts
               curvefacts = curvefactscp
               deallocate ( curvefactscp )

            end if  ! ( associated(curvefacts) )
            if ( debug ) write(irefwr,*) 'after associated(curvefactscp)'

            alloccurv = .false.

!           --- Copy new arrays icurvs and curves to old ones

            allocate ( km(iseqkm)%curvesmsh(ndim,lencurvs) )
            km(iseqkm)%curvesmsh = curvesmsh(1:ndim,1:lencurvs)
            deallocate(curvesmsh)
            curvesmsh => km(iseqkm)%curvesmsh
            if ( numobst>0 ) then

!           --- obstacles, make a complete new array icurvsobs

               if ( associated(km(iseqkm)%icurvsobst2d) ) &
                  deallocate (km(iseqkm)%icurvsobst2d)
               allocate ( km(iseqkm)%icurvsobst2d(2,lencurvs/2) )
               icurvsobst2d => km(iseqkm)%icurvsobst2d
               icurvsobst2d = 0
               do iobst = 1, numobst

                  if ( iactobst(iobst)>0 ) then

!                 --- Active obstacle

                     nx       = kmeshx(3,iobst)
                     ny       = kmeshx(4,iobst)
                     iobgrid2d => kmobst(iobst)%iobgrid2d
                     iobgrid2d1 => kmobst(iobst)%iobgrid2d1
                     call mshobst ( rinputkm(3), nx, ny, iobst, lengcv )

                  end if

               end do

            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcheck'

      end if  ! ( debug )
      call erclos ( 'mshcheck' )

      end subroutine mshcheck
