      subroutine mshchkcv ( inpchk, rinchk, icheck )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date  12-12-1997 Correction for non-positive end points
!        version 1.0    date  11-08-1996
!
!
!   copyright (c) 1996-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Check if the curves of the mesh generated satisfies the conditions
!   as described in the arrays inpchk and rinchk.
!   Return with a value for icheck indicating if remeshing should be applied
! **********************************************************************
!
!                       KEYWORDS
!
!   check
!   curve
!   mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpchk(*), icheck
      double precision rinchk(*)

!     icheck     o    Indicates if remeshing must be applied or not
!                     Possible values:
!                     0  No remeshing necessary
!                     1  The boundary does not satisfy the requirements
!                     2  The internal elements do not satisfy the requirements
!     inpchk     i    Indicates how the checking must be applied
!                     inpchk must be filled as follows:
!                     1:  indication of how the boundary must be checked
!                         Possible values:
!                         0   No checking of the boundary, except in the
!                             case that general is applied
!                             If general is applied all boundaries of the
!                             surfaces generated by general are checked
!                             with respect to the general requirements
!                         1   The first and last element of each curve
!                             in array icurves is compared with the
!                             original length stored in array kmesh p
!                             If 1/a < dx/dx_orig < a the curve is
!                             accepted, otherwise icheck is set equal to 1
!                             dx is the actual length of the element
!                             dx_orig is the original length of the element
!                             a must be stored in rinchk(1)
!                         As soon as one of the curves returns with icheck = 1, &
!                         checking is stopped
!                     2:  indication of how the internal elements must be
!                         checked. This is only applied if the boundary
!                         check returns with icheck = 0
!                         Possible values:
!                         0   No checking of the internal elements
!                         1   All angles of the elements are checked
!                             If the smallest angle of an element < rinchk(2)
!                             or the largest angle of an element > rinchk(3)
!                             icheck is set equal to 2 and checking is stopped
!     rinchk     i    See inpchk
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  istar3, icurnr, inodp1, inodp2, isurnr, isurf, i, ncurvslc
      double precision dx, dx1, dx2, factor

!     dx         length of vector
!     dx1        length of vector
!     dx2        length of vector
!     factor     Multiplication factor
!     i          Counting variable
!     icurnr     Curve sequence number
!     inodp1     First node in element
!     inodp2     Second node in element
!     istar3     Starting address of KMESHM part 3 in KMESH part M
!     isurf      Type of surface with respect to mesh generator
!     isurnr     Surface sequence number
!     ncurvslc   Number of local curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First check if general is called and if so check if the elements
!     along the sides are admissable
!     Next check the relation between element sizes and original element
!     sizes
!
!     for isurnr := 1 (1) nsurfs do
!        isurf := kmeshs(nsurfs+1+8*(isurnr-1)+1)
!        if ( isurf=2 ) then
!           For all curves along the surface do
!              Check if last element of curve and first element of
!              next curve ave a ratio between 0.3 and 3.3
!
!     For icurnr := 1 (1) numcrv do
!        Check relation between original length of first and last element
!        of icurnr with present length
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- First check if general is called and if so check if the elements
!         along the sides are admissable

      istar3 = kmeshm(2) - 1
      do isurnr = 1, nsurfs
         isurf = kmeshs(nsurfs+1+8*(isurnr-1)+1)
         if ( isurf==2 ) then

!        --- Surface is of type general
!            Check the sizes of the elements along the surface

            ncurvslc = kmeshs(isurnr+1)-kmeshs(isurnr)

!           --- First curve, first element

            icurnr = kmeshs(kmeshs(isurnr)+1)
            if ( icurnr>0 ) then

!           --- curve number is positive, standard direction

               inodp1 = kmeshm(istar3+kmeshm(4+icurnr))
               inodp2 = kmeshm(istar3+kmeshm(4+icurnr)+1)

            else

!           --- curve number is negative, reversed direction

               inodp1 = kmeshm(istar3+kmeshm(5-icurnr)-1)
               inodp2 = kmeshm(istar3+kmeshm(5-icurnr)-2)

            end if
            dx1 = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                         (coor(2,inodp1) - coor(2,inodp2))**2 )

!           --- All curves except last one

            do i = 1, ncurvslc-1

               icurnr = kmeshs(kmeshs(isurnr)+i)

!              --- last element of curve

               if ( icurnr>0 ) then

!              --- curve number is positive, standard direction

                  inodp1 = kmeshm(istar3+kmeshm(5+icurnr)-2)
                  inodp2 = kmeshm(istar3+kmeshm(5+icurnr)-1)

               else

!              --- curve number is negative, reversed direction

                  inodp1 = kmeshm(istar3+kmeshm(4-icurnr))
                  inodp2 = kmeshm(istar3+kmeshm(4-icurnr)+1)

               end if

               dx2 = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                            (coor(2,inodp1) - coor(2,inodp2))**2 )

               icurnr = kmeshs(kmeshs(isurnr)+i+1)

!              --- first element of next curve

               if ( icurnr>0 ) then

!              --- curve number is positive, standard direction

                  inodp1 = kmeshm(istar3+kmeshm(4+icurnr))
                  inodp2 = kmeshm(istar3+kmeshm(4+icurnr)+1)

               else

!              --- curve number is negative, reversed direction

                  inodp1 = kmeshm(istar3+kmeshm(5-icurnr)-2)
                  inodp2 = kmeshm(istar3+kmeshm(5-icurnr)-1)

               end if
               dx = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                           (coor(2,inodp1) - coor(2,inodp2))**2 )
               factor = dx/dx2
               if ( factor<0.3d0 .or. factor>3.3d0 ) then

!              --- Difference in spacing too large, return with icheck = 1

                  icheck = 1
                  go to 1000

               end if

            end do

!           --- last curve, last element

            icurnr = kmeshs(kmeshs(isurnr)+ncurvslc)

            if ( icurnr>0 ) then

!           --- curve number is positive, standard direction

               inodp1 = kmeshm(istar3+kmeshm(5+icurnr)-2)
               inodp2 = kmeshm(istar3+kmeshm(5+icurnr)-1)

            else

!           --- curve number is negative, reversed direction

               inodp1 = kmeshm(istar3+kmeshm(4-icurnr))
               inodp2 = kmeshm(istar3+kmeshm(4-icurnr)+1)

            end if
            dx2 = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                         (coor(2,inodp1) - coor(2,inodp2))**2 )
            factor = dx1/dx2
            if ( factor<0.3d0 .or. factor>3.3d0 ) then

!           --- Difference in spacing too large, return with icheck = 1

               icheck = 1
               go to 1000

            end if

         end if

      end do

      if ( inpchk(1)==1 ) then

!     --- Next check the relation between element sizes and original element
!         sizes

         factor = rinchk(1)
         do icurnr = 1, ncurvs

            if ( kmeshm(5+icurnr)-kmeshm(4+icurnr)>0 ) then

!           --- curve is non-empty

               inodp1 = kmeshm(istar3+kmeshm(4+icurnr))
               inodp2 = kmeshm(istar3+kmeshm(4+icurnr)+1)
               if ( inodp1>0 .and. inodp2>0 ) then

!              --- Compare first element with original size

                  dx = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                              (coor(2,inodp1) - coor(2,inodp2))**2 )
                  if ( dx*factor< elsize(1,icurnr) .or. &
                       dx/factor> elsize(1,icurnr) ) then

!                 --- Difference in spacing too large, return with icheck = 1

                     icheck = 1
                     go to 1000

                  end if

               end if

!              --- Compare last element with original size

               inodp1 = kmeshm(istar3+kmeshm(5+icurnr)-2)
               inodp2 = kmeshm(istar3+kmeshm(5+icurnr)-1)
               if ( inodp1>0 .and. inodp2>0 ) then

                  dx = sqrt ( (coor(1,inodp1) - coor(1,inodp2))**2 + &
                              (coor(2,inodp1) - coor(2,inodp2))**2 )
                  if ( dx*factor<elsize(2,icurnr) .or. &
                       dx/factor>elsize(2,icurnr) ) then

!                 --- Difference in spacing too large, return with icheck = 1

                     icheck = 1
                     go to 1000

                  end if

               end if

            end if

         end do

      end if

1000  continue

      end
