      subroutine mshcrosssurf ( xp1, xp2, x1, x2, x3, fact1, fact2, &
                                eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-11-2013 Debug statements
!        version  1.0  date 29-06-2001
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find the intersection of the lines xp1-xp2 and the triangle x1,x2,x3
!    The result are the parameters fact1 and fact2 where the intersection point
!    is defined as xp1 + fact1*(xp2-xp1) and
!    x = x_1 + fact2(1)*(x_2-x_1) + fact2(2)*(x_3-x_1)
!    If no intersection point is found fact gets the value -infinity
!    Three-dimenions only
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     intersection
!     surface
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(3), x2(3), x3(3), xp1(3), xp2(3), fact1, &
                       fact2(2), eps

!     eps            i    local accuracy
!     fact1          o    Defines the factor for the intersection point
!                         The intersection point on the edge is defined by
!                         xp1 + fact1 * (xp2-xp1)
!     fact2          o    Defines the factors for the intersection point
!                         on the surface
!                         The intersection point on the face is defined by
!                         x = x_1 + fact2(1)*(x_2-x_1) + fact2(2)*(x_3-x_1)
!     x1             i    Contains the coordinates of the point x1
!     x2             i    Contains the coordinates of the point x2
!     x3             i    Contains the coordinates of the point x3
!     xp1            i    Contains the coordinates of the point xp1
!     xp2            i    Contains the coordinates of the point xp2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision det, v1(3), v2(3), v3(3), f(3), som

!     det            determinant of system of equations to be solved
!     f              Right-hand-side vector of system of equations
!     som            Help parameter to compute a sum
!     v1             First column of matrix
!     v2             Second column of matrix
!     v3             Third column of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ercdet

!     ERCDET         Compute a 3x3 determinant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The line through xp1-xp2 is defined as
!     x = xp1 + fact1 * (xp2-xp1)
!     The plane through (x1,x2,x3) is defined as
!     x = x1 + fact2(1) * (x2-x1) + fact2(2) * (x3-x1)
!     If an intersection is present then
!        xp1 + fact * (xp2-xp1) = x1 + fact2(1) * (x2-x1) + fact2(2) * (x3-x1)
!        Hence:
!        [(x2-x1) (x3-x1) (xp1-xp2)] [fact2_1 fact2_2 fact1]^T = xp1-x1
!     The system of equations is solved by Kramers rule
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrosssurf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcrosssurf'
         write(irefwr,2) 'xp1', xp1
         write(irefwr,2) 'xp2', xp2
         write(irefwr,2) 'x1', x1
         write(irefwr,2) 'x2', x2
         write(irefwr,2) 'x3', x3
         write(irefwr,2) 'eps', eps
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      v1 = x2-x1
      v2 = x3-x1
      v3 = xp1-xp2
      f  = xp1-x1
      det = ercdet ( v1, v2, v3 )
      if ( debug ) then
         write(irefwr,2) 'v1', v1
         write(irefwr,2) 'v2', v2
         write(irefwr,2) 'v3', v3
         write(irefwr,2) 'determinant', det
      end if  ! ( debug )
      if ( abs(det)<=1d-50 ) then
         fact1 = -rinfin
         fact2(1) = -rinfin
         fact2(2) = -rinfin
      else
         det = 1d0/det
         fact2(1) = det*ercdet ( f, v2, v3 )
         fact2(2) = det*ercdet ( v1, f, v3 )
         som = fact2(1)+fact2(2)
         if ( fact2(1)>=-eps .and. fact2(1)<=1d0+eps .and. &
              fact2(2)>=-eps .and. fact2(2)<=1d0+eps .and. &
              som>=-eps .and. som<=1d0+eps ) then
            fact1 = det*ercdet ( v1, v2, f )
         else
            fact1 = -rinfin
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'fact1, fact2', fact1, fact2
         write(irefwr,*) 'End mshcrosssurf'

      end if  ! ( debug )
      call erclos ( 'mshcrosssurf' )

      end
