      subroutine mshcrtsp ( iinput, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 06-12-2013 New call to msh476/670
!        version  2.3  date 25-11-2012 Long integers
!        version  2.2  date 18-03-2002 Extend first part of kmesh to lenkmeshfp
!
!   copyright (c) 1996-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Create mesh in the case ispecial = 1
!    Hence the user has given all nodal points with the option NODAL_POINTS
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     create
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)
      double precision rinput(*)

!     iinput     i    integer input array with the following contents:
!                     1: ndim
!                     2: istruct
!                        0=unstructured
!                        1=structured
!                        2=rectangular
!                     3-5: information of the mesh:
!                        unstructured:
!                        3= number of boundary points
!                        4= number of internal points
!                        structured and rectangular
!                        3= nx
!                        4= ny
!                        5= nz
!                     6: Type of elements (See table 2.2.1 in users manual)
!                     7-10: information of the plots:
!                        7= iplot (0=no plot, 1=plot)
!                        8= jmark
!                        9= orientation (3D mesh)
!                       10= view point given (1) or not (0)
!     rinput     i    real input array with the following contents:
!                     1: plotfm
!                     2: yfact
!                     3-5: eye point
!                     6-end: information of the co-ordinates:
!                       (un)structured:
!                         co-ordinates in sequence x_1, y_1, z_1, x_2, ..
!                       rectangular:
!                       6: O_1
!                       7: O_2
!                       8: O_3
!                       9: dx
!                      10: dy
!                      11: dz
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinplt(14), iskip(1)
      integer nbound, ninter, i, nx, ny, nz, lenmsh, nexpnt, ntotal, &
              kstap, lngbnd, lngklr, &
              lnglin, nipnt, nelm, nh
      double precision xminvec(4)
      integer, allocatable :: kbound(:), klaar(:)
      double precision, allocatable :: coortmp(:,:)

!     coortmp        array of length ndim x npoint containing the
!                    co-ordinates of the nodal points with respect to the
!                    global numbering
!     i              Loop variable
!     iinplt         Input array for subroutine plotms, see PG 16.3
!     iskip          Dummy array
!     kbound         array with the boundary triangles
!     klaar          ?
!     kstap          Number of triangles in boundary
!     lenmsh         Last position used in array KMESH
!     lngbnd         Declared length of boundary array
!     lngklr         Declared length of "klaar" array
!     lnglin         Declared length of lines array
!     nbound         Number of essential boundary conditions
!     nelm           Helpvariable for number of elements found
!     nexpnt         Number of extra nodes needed to make a correct mesh
!     nh             Helpvariable for estimation of number of points
!     ninter         Number of boundary curves that remains to be considered
!     nipnt          Number of points in initial boundary
!     ntotal         Number of nodes in total mesh
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     xminvec            Contains the vector xmint, ymint, zmint and tran
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH476         Construct a tetrahedronal mesh given the points in 3D
!     MSHO70         compute the coordinates of the nodes in case of
!                    a structured mesh in 2d or 3d
!     MSHO71         determine whether line i - k is permitted
!     MSHO72         Construct a tetrahedronal or hexahedronal mesh in 3d in
!                    case of a structured set of nodal points
!     MSHO76         Fill kmesh part m in case of an unstructured mesh where the
!                    nbound boundary points have been given as a starting
!                    sequence
!     MSHO77         Fill kmesh part m in case of a structured mesh in 2D where
!                    nx and ny are the number of points in the both coordinate
!                    directions
!     MSHO78         Fill coordinate array in case of an unstructured mesh
!                    without a given boundary
!     MSHO79         Remove the extra coordinates and elements in case of an
!                    unstructured mesh with artificial constructed boundary
!     MSHO80         Fill coordinate array in case of an unstructured mesh while
!                    the boundary nodes have been given
!     MSHO82         compute the coordinates of the nodes in case of
!                    a structured mesh in 2d or 3d
!     MSHO83         fill KMESH part M in case of a structured mesh in 3D
!     MSHO84         fill array with boundary triangles in case of an
!                    unstructured mesh in 3D
!     MSHO85         drop (extra) help nodes and elements in case of an
!                    unstructured mesh in 3D
!     MSHO86         check whether division of multihedron is possible
!                    with extra nodes
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPLOTMS      plot mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      62   dimension of space (ndim) has wrong value
!     157   The actual generated number of elements is more than the
!           estimated number
!     975   Only triangles and tetrahedra allowed for unstructured grids
!     977   Internal error, no adequate unstructured grid can be found
!     978   Boundary in 3D given by the user not yet supported
!    1644   The number of extra points needed is too large
!    1645   Only linear triangles or quadrilaterals allowed for structured
!           meshes in 2d
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( unstructured set of points ) then
!        if ( boundary is prescribed and ndim = 2 ) then
!           copy coordinates of nodes to kmesh part i
!           determine and fill elements in kmesh part c
!           fill kmesh part m (only one curve)
!        else if ( no boundary and ndim = 2 ) then
!           use help nodes (4) to make a convex mesh
!           drop helpnodes and helpelements and
!           fill kmesh part i, part c and part m
!        else if ( no boundary and ndim = 3 ) then
!           use help nodes (8) and make convex mesh
!           drop helpnodes and helpelements and
!           fill kmesh part i, part c and part m
!        else if ( boundary given and ndim = 3 ) then
!           not yet implemented situation
!        end if
!     else if ( structured set of points ) then
!        determine number of nodes
!        if ( coordinates are given ) then
!           fill kmesh part i
!        else if ( origin and steps are given ) then
!           fill kmesh part i
!        end if
!        fill (regular) kmesh part c with elements
!        if ( ndim = 2 ) then
!           fill kmesh part m with the four standard curves
!        else if ( ndim = 3 ) then
!           fill kmesh part m with the twelve standard curves
!           fill kmesh part n with the six standard surfaces
!           ( not yet implemented )
!        end if
!     end if
!     Fill array kmesh
!     Plot the mesh if required
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrtsp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcrtsp'
         call prinin ( iinput, 10, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define one user point

      nuspnt =  1
      nexpnt =  0
      lenmsh = 80

      xminvec = 1d0
      xminvec(4) = 0d0

!     --- Fill arrays kmeshc and coor

      if ( iinput(2)==0 ) then

!     --- Unstructured grid, only nodes are given

         ndim     =   iinput(1)
         nbound   =   iinput(3)
         ninter   =   iinput(4)
         npoint   =   nbound + ninter
         if ( ndim<2 .or. ndim>3 ) then

            call errint ( ndim, 1 )
            call errsub ( 62, 1, 0, 0 )
            go to 1000

         end if

         if ( nbound>0 .and. ndim==2 ) then

!        --- Unstructured but user did give boundary
!            Check type of element

            if ( iinput(6)/=3 ) then

               call errint ( iinput(6), 1 )
               call errint ( ndim, 2 )
               call errsub ( 975, 2, 0, 0 )
               go to 1000

            end if

!           --- Fill coordinates array

            length   =    2 * npoint

            if ( kelmi>0 ) deallocate ( km(iseqkm)%coor )
            allocate ( km(iseqkm)%coor(ndim,npoint) )
            coor => km(iseqkm)%coor
            kelmi = length

!           --- Copy coordinates to array coor (KMESH part I)

            call msho80( ndim, coor, npoint, rinput(6) )

!           --- Compute number of elements (triangles)

            nelem = 2*(npoint-1)-nbound
            npelm = 3

            allocate ( kmeshctmp(npelm*nelem), stat = error )
            if ( error/=0 ) call eralloc ( error, npelm*nelem, 'kmeshctmp' )
            if ( ierror/=0 ) go to 1000

            call msho70 ( nbound, coor, npoint, nelem, kmeshctmp )

!           --- Make array kmeshctmp permanent

            if ( kelmc>0 ) deallocate ( km(iseqkm)%kmeshc )
            kelmc =nelem*npelm
            allocate ( km(iseqkm)%kmeshc(kelmc) )
            kmeshc => km(iseqkm)%kmeshc
            kmeshc = kmeshctmp(1:kelmc)

            deallocate ( kmeshctmp, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'kmeshctmp' )
            if ( ierror/=0 ) go to 1000

!           --- Fill array kmeshm

            length =  6 + nbound
            if ( kelmm/=0 ) deallocate ( km(iseqkm)%kmeshm )
            allocate ( km(iseqkm)%kmeshm(length) )
            kmeshm => km(iseqkm)%kmeshm
            kelmm = length

            call msho76( kmeshm, nbound )

!           --- Set variables for kmesh

            ncurvs   =  1
            nsurfs   =  1
            nvolms   =  0
            ishape   =  3

         else if ( nbound==0 .and. ndim==2 ) then

!        --- Check element type given by user

            if ( iinput(6)/=3 ) then

               call errint ( iinput(6), 1 )
               call errint ( ndim, 2 )
               call errsub ( 975, 2, 0, 0 )
               go to 1000

            end if

!           --- No boundary points given by user
!               make convex mesh of nodes given

            npoint = npoint + 4

!           --- Compute number of elements (triangles)

            nbound        =   4
            nelem         =   2  *  ( npoint - 1 ) - nbound

            npelm  = 3

            allocate ( kmeshctmp(3*nelem), coortmp(2,npoint), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, 3*nelem+2*npoint, 'kmeshctmp' )
            if ( ierror/=0 ) go to 1000

!           --- Copy coordinates to array coortmp (KMESH part I)

            call msho78 ( ndim, coortmp, npoint, rinput(6) )

!           --- Fill elements array

            call msho70 ( nbound, coortmp, npoint, nelem, kmeshctmp )

!           --- Rearrange : drop extra nodes and extra elements

            call msho79 ( coortmp, npoint, kmeshctmp, nelem, nbound )

!           --- Make arrays coortmp and kmeshctmp permanent

            length = npoint*2
            if ( kelmi>0 ) deallocate ( km(iseqkm)%coor )
            allocate ( km(iseqkm)%coor(ndim,npoint) )
            coor => km(iseqkm)%coor
            coor = coortmp(:,1:npoint)
            kelmi = length
            if ( kelmc>0 ) deallocate ( km(iseqkm)%kmeshc )
            kelmc =nelem*3
            allocate ( km(iseqkm)%kmeshc(kelmc) )
            kmeshc => km(iseqkm)%kmeshc
            kmeshc = kmeshctmp(1:kelmc)

            deallocate ( kmeshctmp, coortmp, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'kmeshctmp' )
            if ( ierror/=0 ) go to 1000

!           --- Fill array kmeshm
!               (temporary only line 1-2)

            length =  6 + nbound
            if ( kelmm/=0 ) deallocate ( km(iseqkm)%kmeshm )
            allocate ( km(iseqkm)%kmeshm(length) )
            kmeshm => km(iseqkm)%kmeshm
            kelmm = length

            call msho76 ( kmeshm, nbound )

            npelm    =  3
            ncurvs   =  1
            nsurfs   =  1
            nvolms   =  0
            ishape   =  3

         else if ( nbound==0 .and. ndim==3 ) then

            nexpnt   =  0

!           --- No boundary points given in 3D, make convex mesh

            npoint = npoint + 8

!           --- Estimate number of elements (tetrahedrons)

            nh            = nint( sqrt( npoint + 5.0 ) ) - 1

            nelem         = nh * nh * nh + 100

!           --- Several help arrays are needed
!               kbound array for the boundary triangles

            lngbnd = 60 * nh * nh

!           --- klaar array for detecting whether ready, length = npoint+10

            lngklr = ( npoint + 10 )

!           --- line workarray to store boundary lines, length lnglin

            lnglin =  60 * nh * nh

            allocate ( kmeshctmp(4*nelem), coortmp(3,(npoint+10)), &
                       kbound(lngbnd), klaar(npoint+10), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, 4*nelem+3*npoint+lngbnd, 'kmeshctmp' )
            if ( ierror/=0 ) go to 1000

!           --- Copy coordinates to array coortmp (KMESH part I)

            call msho78 ( ndim, coortmp, npoint, rinput(6) )

            kstap  = 12
            nipnt  =  8

!           --- Fill array with boundary triangles

            call msho84( kbound )

!           --- Set number of elements created

            nelm   =  0

!           --- Make division as long as kstap is not zero

150         if ( kstap/=0 ) then

               if ( nexpnt>5 ) then

!              --- Point added but special situation, make
!                  full restart

                  kstap  =  12
                  call msho84( kbound )
                  nelm   =   0

               end if

               ntotal = npoint + nexpnt

               call msh476 ( kstap, kbound, coortmp, &
                             ntotal, nelm, kmeshctmp, &
                             klaar, lnglin, nipnt, 1, xminvec )
               if ( ierror/=0 ) go to 1000

               if ( kstap/=0 ) then

                  if ( nexpnt==10 ) then

!                 --- Error, even with ten points added, no
!                     solution possible

                     call errint ( nexpnt, 1 )
                     call errsub ( 1644, 1, 0, 0 )
                     go to 1000

                  end if

                  call msho86( kstap, kbound, coortmp, &
                               npoint, nexpnt, klaar, 1 )

               end if

               goto 150

            end if

            npoint = npoint + nexpnt

            if ( nelm>nelem ) then

               call errint ( 1, 1 )
               call errint ( nelm, 2 )
               call errint ( nelem, 3 )
               call errsub ( 157, 3, 0, 0 )
               go to 1000

            end if

            nelem = nelm

!           --- Rearrange : drop extra nodes and extra elements

            call msho85 ( coortmp, npoint, kmeshctmp, nelem, &
                          kbound, nbound )
            if ( nelem==0 ) then
               call errsub ( 977, 0, 0, 0 )
               go to 1000
            end if

!           --- Make arrays coortmp and kmeshctmp permanent

            length = npoint*3
            if ( kelmi>0 ) deallocate ( km(iseqkm)%coor )
            allocate ( km(iseqkm)%coor(ndim,npoint) )
            coor => km(iseqkm)%coor
            coor = coortmp(:,1:npoint)
            kelmi = length
            if ( kelmc>0 ) deallocate ( km(iseqkm)%kmeshc )
            kelmc =nelem*4
            allocate ( km(iseqkm)%kmeshc(kelmc) )
            kmeshc => km(iseqkm)%kmeshc
            kmeshc = kmeshctmp(1:kelmc)

            deallocate ( kmeshctmp, coortmp, kbound, klaar, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'kmeshctmp' )
            if ( ierror/=0 ) go to 1000

!           --- Fill array kmeshm
!               (temporary only line 1-2)

            length =  6 + 2
            if ( kelmm/=0 ) deallocate ( km(iseqkm)%kmeshm )
            allocate ( km(iseqkm)%kmeshm(length) )
            kmeshm => km(iseqkm)%kmeshm
            kelmm = length

            call msho76 ( kmeshm, 2 )

!           --- Fill array kmeshn
!               ( not yet implemented )
!               Fill values for array KMESH

            npelm    =  4
            ncurvs   =  1
            nsurfs   =  0
            nvolms   =  1
            ishape   = 11

         else

!        --- Not allowed situation, give message

            call errsub ( 978, 0, 0, 0 )
            go to 1000

         end if

      else if ( iinput(2)==1 .or. iinput(2)==2 ) then

!     --- Structured mesh

         ndim = iinput( 1 )

         nx   = iinput( 3 )
         ny   = iinput( 4 )
         nz   = iinput( 5 )

         if ( debug ) write(irefwr,1) 'ndim, nx, ny, nz', ndim, nx, ny, nz

!        --- Determine number of nodes

         if ( ndim==2 ) then

            npoint = nx * ny
            nz     = 1

         else if ( ndim==3 ) then

            npoint = nx * ny * nz

         end if

!        --- Fill coordinates array

         length   =    ndim * npoint
         if ( kelmi>0 ) deallocate ( km(iseqkm)%coor )
         allocate ( km(iseqkm)%coor(ndim,npoint) )
         coor => km(iseqkm)%coor
         km(iseqkm)%kmesh(23) = length

         if ( debug ) write(irefwr,1) 'npoint, length', npoint, length

         if ( iinput( 2 )==1 ) then

!        --- Copy coordinates to array coor (KMESH part I)

            call msho80 ( ndim, coor, npoint, rinput(6) )

         else if ( iinput(2)==2 ) then

!        --- Compute coordinates of rectangular grid

            call msho82 ( ndim, coor, nx, ny, nz, rinput(6) )

         end if

         if ( debug ) call prinrl1 ( coor, npoint, ndim, 'coor' )

!        --- Fill elements array

         if ( ndim==2 ) then

!        --- Determine type of elements

            ishape = iinput(6)

            if ( ishape==3 ) then

               npelm = 3

            else if ( ishape==5 ) then

               npelm = 4

            else

               call errsub ( 1645, 0, 0, 0 )
               go to 1000

            end if

!           --- Compute number of elements (quadrilaterals)

            if ( npelm==3 ) then

               nelem  = 2 * ( nx - 1 ) * ( ny - 1 )

            else

               nelem  =  ( nx - 1 ) * ( ny - 1 )

            end if

            length = npelm  *  nelem
            if ( kelmc>0 ) deallocate ( km(iseqkm)%kmeshc )
            kelmc = length
            allocate ( km(iseqkm)%kmeshc(kelmc) )
            kmeshc => km(iseqkm)%kmeshc

!           --- Compute elements

            call msho71 ( nx, ny, kmeshc, nelem, npelm )

!           --- Fill array kmeshm

            length =  6 + 2 * ( nx + ny )
            if ( kelmm/=0 ) deallocate ( km(iseqkm)%kmeshm )
            allocate ( km(iseqkm)%kmeshm(length) )
            kmeshm => km(iseqkm)%kmeshm
            kelmm = length

!           --- Fill part m

            nbound = 2 * ( nx + ny ) - 4

            call msho77 ( nx, ny, kmeshm )

!           --- Fill values for kmesh

            ncurvs = 1
            nsurfs = 1
            nvolms = 0

         else if ( ndim==3 ) then

!        --- Determine type of elements

            ishape = iinput( 6 )

!           --- Compute number of elements

            if ( ishape==11 ) then

!           --- Tetrahedrons

               nelem  = 6 * ( nx - 1 ) * ( ny - 1 ) * ( nz - 1 )
               npelm  = 4
               length = npelm * nelem

            else if ( ishape==13 ) then

!           --- Hexahedrons

               nelem  = ( nx - 1 ) * ( ny - 1 ) * ( nz - 1 )
               npelm  = 8
               length = npelm * nelem

            else

               call errsub ( 1645, 0, 0, 0 )
               go to 1000

            end if

            if ( kelmc>0 ) deallocate ( km(iseqkm)%kmeshc )
            kelmc = length
            allocate ( km(iseqkm)%kmeshc(kelmc) )
            kmeshc => km(iseqkm)%kmeshc

!           --- Compute elements

            call msho72( nx, ny, nz, kmeshc, npelm )

!           --- Fill array kmeshm

            length =  17 + 4 * ( nx + ny + nz )
            if ( kelmm/=0 ) deallocate ( km(iseqkm)%kmeshm )
            allocate ( km(iseqkm)%kmeshm(length) )
            kmeshm => km(iseqkm)%kmeshm
            kelmm = length

            call msho83( nx, ny, nz, kmeshm )

!           --- Fill kmesh part n
!               Fill values for kmesh

            ncurvs = 12
            nsurfs =  0
            nvolms =  1

         end if

      end if

!     --- Fill all values required in array kmesh

      km(iseqkm)%kmesh( 2 ) = 100

!     --- only one group of elements

      nelgrp = 1
      km(iseqkm)%kmesh(5) = nelgrp

!     --- only 1 standardelement

      km(iseqkm)%kmesh(14) = npelm

!     --- kelma is not used
!         kelmb is not used
!         kelmc : the nodal points of the elements. This array
!                 has been filled already
!         kelmd is not used
!         kelme is not used
!         kelmf is always used

      allocate ( km(iseqkm)%kmeshf(1), Topgrp(nelgrp) )
      kmeshf => km(iseqkm)%kmeshf
      kmeshf(1) = ishape
      km(iseqkm)%kmesh(20) = 1

!     --- kelmg is not used
!         kelmh is not used
!         kelmi : the coordinates of the nodes. This array has been filled
!                 already
!         kelmj : no renumbering
!         kelmk : no levels have been computed
!         kelml : for security reasons always one point i.e.
!                 nodal point one is stored as a user point

      length      =    1
      allocate ( km(iseqkm)%kmeshl(length) )
      kmeshl => km(iseqkm)%kmeshl
      kmeshl(1) = 1
      km(iseqkm)%kmesh(26) = 1

      allocate ( km(iseqkm)%kmesht(2,1) )
      kmesht => km(iseqkm)%kmesht
      km(iseqkm)%kmesh(36) = 1

      kmesht(1,1) = 1
      kmesht(2,1) = nelem

      km(iseqkm)%kmesh(4) = npelm
      km(iseqkm)%kmesh(6) = ndim
      km(iseqkm)%kmesh(8) = npoint
      km(iseqkm)%kmesh(9) = nelem

      call sepfillkmesh ( kmesh )

!     --- kelmm : part m has been stored already
!         kelmn : part n is only used in case of 3D. It is stored already.
!         all other parts of kmesh are not used yet
!         Plot the mesh if required

      if ( iinput(7)==1 ) then

!     --- Mesh must be plotted

         iinplt(1) = 14
         do i = 2, 10
            iinplt(i) = 0
         end do
         jmark = iinput(8)
         jtimes = 0
         if ( jmark<3 ) iinplt(2) = 1
         if ( jmark==0 .or. jmark==3 ) iinplt(3) = 1
         if ( jmark==1 .or. jmark==4 ) iinplt(4) = 1
         iinplt(11) = (1-iinput(10))*10
         iinplt(12) = 1
         iinplt(13) =16
         iinplt(14) = iinput(9)
         call sepplotms ( iinplt, rinput, iskip )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor, npoint, ndim, 'coor' )
         write(irefwr,*) 'End mshcrtsp'

      end if  ! ( debug )
      call erclos ( 'mshcrtsp' )

      end subroutine mshcrtsp
