      subroutine mshg ( jnew, coorlc, kmeshclc1, nbndptlc, kbndpt, &
                        bcord, nhelp, nheen )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  8.1  date 29-05-2014 Extension of timebefore
!        version  8.0  date 22-09-2010 Remove coarse
!        version  7.13 date 08-12-2003 Correct centroid for ishape=6 (BW)
!
!   copyright (c) 1983-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  driving subroutine for the division of an arbitrary polygon
!  into linear or quadratic elements of triangular or quadrilateral shape
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical jnew
      double precision coorlc(*), bcord(*)
      integer kmeshclc1(*), kbndpt(*), nheen(*), nbndptlc, nhelp

!     bcord         o    array containing the coordinates of the boundaries
!     coorlc       o    array of length ndim x npointlc containing the
!                        co-ordinates of the nodal points
!     jnew          i    indication whether the mesh is new (true) or old
!     kbndpt        o    array containing the nodal point numbers of
!                        the boundary
!     kmeshclc1     o    kmesh part c, contains the nodal points corresponding
!                        to the elements
!     nbndptlc      o    Number of nodal points in boundary,
!                        length of array kbndpt
!     nheen         i    ?
!     nhelp         i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, j, i1, i2, i3, i4, npti, npt, ntot, l3, l4, istep, &
              ipos1, ipos2, ipos3, ipos4, ipos5, ipos6, nh, n, np, ikn
      integer :: izwaar(leniwk)
      double precision xs, ys, afst1, afst2, afst, rim, rh, rst, r, &
                       factor
      double precision, allocatable :: coarse(:)
      double precision :: timebefore(2)

!     afst
!     afst1
!     afst2
!     coarse         Array containing the coarseness per point
!     factor         Multiplication factor
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     i3
!     i4
!     ih             Help variable to store a constant temporarily
!     ikn
!     ipos1
!     ipos2
!     ipos3
!     ipos4
!     ipos5
!     ipos6
!     istep          Step size of S in elemmt
!     izwaar         Integer work array
!     j              Counting variable
!     l3
!     l4
!     n              number of basis functions for each component
!     nh
!     np             Help variable to store the maximum value of nphys and nunkp
!     npt            total number of points in the mesh
!     npti           number of boundary points + 1
!     ntot
!     r
!     rh
!     rim            length of smallest edge along the boundary
!     rst
!     xs
!     ys
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     INSTOP         Stop the program
!     MSHD06         Determine neighbours and adjacent elements
!     MSHD08         Replace coordinates
!     MSHD09         Make linear elements and determine max corner node
!                    (3-angles)
!     MSHD10         Make linear elements and determine max corner node
!                    (4-angles)
!     MSHD3          Create triangles in an arbitrary polygon
!     MSHD6          Places extra numbers for quadratic case (3-angles)
!     MSHD7          Places coorindates of centroids of 7-point triangles
!                    in array coor
!     MSHG04
!     MSHG05
!     MSHG06
!     MSHG07
!     MSHG08
!     MSHG09
!     MSHTRIANCENTR  Create centre point in 6-point triangles
!     MSHV10         correct centroid postition for biquadratic quadrilaterals
!     MSHV4          Subdivide region into quadrilaterals
!     MSHV8          Places extra numbers for quadratic case (4-angles)
!     MSHV9          Places extra centre nodes (4-angles)
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     141   The actual generated number of nodal points is more than the
!           estimated number
!     358   The number of nodal points per element is incorrect
!     359   Number of vertices of boundaries for a submesh odd
!     362   ratio of subsequent intervals on the boundary too large
!     900   Estimated number of points too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshg' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshg'
         write(irefwr,*) 'jnew ', jnew
         write(irefwr,1) 'npointlc, nelemlc', npointlc, nelemlc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( coarse(npointlc), stat = error )
      if ( error/=0 ) call eralloc ( error, npointlc, 'coarse' )
      if ( ierror/=0 ) go to 1000

!     --- use or fill nheen depending on jnew

      if ( jnew ) then
         do i=1, nhelp
            nheen(i)=0
         end do
      else
         call mshg05(izwaar, nheen, npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, izwaar)
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, nheen)
         if ( ierror/=0 ) go to 1000
      end if

!     --- fill coorlc with the boundary points
!         and refill kbndpt

      do i=1, nbndptlc
         kbndpt(i)=i
      end do

      kbndpt(nbndptlc+1)=1
      ih=2*nbndptlc

      do i=1, ih
         coorlc(i)=bcord(i)
      end do

!     --- fill array coarse

      j=2*nbndptlc-1
      xs=coorlc(j)-coorlc(1)
      ys=coorlc(j+1)-coorlc(2)
      afst1=sqrt(xs*xs+ys*ys)
      xs=coorlc(3)-coorlc(1)
      ys=coorlc(4)-coorlc(2)
      afst2=sqrt(xs*xs+ys*ys)
      afst=(afst1+afst2)/2d0
      coarse(1)=afst

      do i=2, nbndptlc-1
         afst1=afst2
         xs=coorlc(2*i+1)-coorlc(2*i-1)
         ys=coorlc(2*i+2)-coorlc(2*i)
         afst2=sqrt(xs*xs+ys*ys)
         afst=(afst1+afst2)/2d0
         if ( inpelm<=4) then
            coarse(i)=afst
         else
            coarse(i)=2*afst
         end if
      end do

      afst1=afst2
      xs=coorlc(2*nbndptlc-1)-coorlc(1)
      ys=coorlc(2*nbndptlc)-coorlc(2)
      afst2=sqrt(xs*xs+ys*ys)
      afst=(afst1+afst2)/2d0
      coarse(nbndptlc)=afst

!     --- compute minimum length rim
!         (and check different element lengths)

      j=2*nbndptlc-1
      xs=coorlc(j)-coorlc(1)
      ys=coorlc(j+1)-coorlc(2)
      rim=sqrt(xs*xs+ys*ys)
      rh=rim
      rst=rim

!     --- Check ratio of boundary for new grids to be created
!         and compute rim

      do i = 2, nbndptlc

         j = 2*i
         xs = coorlc(j-1)-coorlc(j-3)
         ys = coorlc(j)-coorlc(j-2)
         r = sqrt(xs*xs+ys*ys)
         rim = min(rim,r)
         if ( jnew ) then

!        --- Check

            factor=r/rh
            if ( factor<0.25d0 .or. factor>4d0 ) then
               call errint ( i, 1 )
               call errint ( isurf, 2 )
               call erreal ( coorlc(j-1), 1 )
               call erreal ( coorlc(j), 2 )
               call erreal ( coorlc(j-3), 3 )
               call erreal ( coorlc(j-2), 4 )
               call erreal ( r , 5 )
               call erreal ( rh, 6 )
               call erreal ( factor, 7 )
               call errsub ( 362, 2, 7, 0 )
            end if
            if ( nerror>=10 ) go to 1000
            rh=r
         end if

      end do

      if ( jnew ) then

!     --- Check

         factor=r/rst
         if ( factor<0.25d0 .or. factor>4d0 ) then
            call errint ( 1, 1 )
            call errint ( isurf, 2 )
            call erreal ( coorlc(1), 1 )
            call erreal ( coorlc(2), 2 )
            call erreal ( coorlc(nbndptlc*2-1), 3 )
            call erreal ( coorlc(nbndptlc*2), 4 )
            call erreal ( rst , 5 )
            call erreal ( r, 6 )
            call erreal ( factor, 7 )
            call errsub ( 362, 2, 7, 0 )
         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute length arrays

      i1=2*nhelp+1
      i2=4*nhelp+1
      i3=6*nhelp+1
      i4=8*nhelp+1
      izwaar(i3)=nbndptlc

      do i=1, nbndptlc
         izwaar(i3+i)=i
      end do

      izwaar(i3+nbndptlc+1)=0

!     --- arrange to call subroutines

      go to ( 100, 100, 110, 140, 100, 120, 120, 150, 150 ), inpelm

!     --- if inpelm # 3, 4, 6, 7, 8 or 9 error

100   call errint ( inpelm, 1 )
      call errsub ( 358, 1, 0, 0)
      go to 1000

!     --- inpelm = 3 : mshd3

110   npti=nbndptlc+1
      if ( jnew ) then
         nelemlc=1
         npt=nbndptlc
         call mshd3(coorlc, kmeshclc1, izwaar(i1), izwaar(i2), izwaar(i3), &
                    izwaar(i4), izwaar, nelemlc, npt, nhelp, &
                    coarse, rim)
         nelemlc=nelemlc-1
         npointlc=npt
      end if
      if ( ierror/=0 ) go to 1000

!     --- repositioning

      ntot=14*nhelp

      do i=1, ntot
          izwaar(i)=0
      end do

!     --- Check whether nhelp is still correct

      if ( npointlc>nhelp ) then
         call errint ( 0, 1 )
         call errint ( npointlc, 2 )
         call errint ( nhelp, 3 )
         call errsub ( 141, 3, 0, 0)
         go to 1000
      end if
      i1=1
      i2=1+npointlc
      i3=1+2*npointlc
      i4=1+8*nhelp
      l3=i4-i3
      l4=6*nhelp
      istep=1

      call mshd06 ( kmeshclc1, nelemlc, npointlc, nbndptlc, 3, izwaar(i1), &
                    izwaar(i2), izwaar(i3), izwaar(i4), l3, l4, istep )
      if ( ierror/=0 ) go to 1000
      call mshd08(coorlc, npointlc, npti, izwaar(i1), izwaar(i3), rim)
      if ( ierror/=0 ) go to 1000

!     --- renumber

      if ( jnew ) then
         call mshg09(izwaar, ipos1, ipos2, ipos3, ipos4, ipos5, ipos6, &
                     npointlc, npointlc, nhelp, nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg04(izwaar(ipos2), izwaar(ipos1), izwaar(ipos4), &
                     izwaar(ipos3), npointlc, npointlc, kmeshclc1, inpelm, &
                     izwaar(ipos6), nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg05(izwaar(ipos5), izwaar(ipos4), npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar(ipos4))
         if ( ierror/=0 ) go to 1000

!        --- fill array nheen

         do i=1, npointlc
             nheen(i)=izwaar(ipos5+i-1)
         end do
      else
         call mshg05(izwaar, nheen, npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar)
         if ( ierror/=0 ) go to 1000
      end if

      goto 200

!     --- inpelm = 6 or 7 : mshd6 and mshd7

120   nh=nbndptlc/2
      nh=2*nh-nbndptlc
      if ( nh/=0 ) then

!     --- Error: number of vertices is odd

         call errint ( nbndptlc, 1 )
         call errsub ( 359, 1, 0, 0 )
         go to 1000

      end if

      n=nbndptlc/2
      izwaar(i3)=n

      do i=1, n
         izwaar(i3+i)=2*i-1
      end do

      izwaar(i3+n+1)=0
      rim=rim*2d0
      npti=nbndptlc+1
      if ( jnew ) then
         nelemlc=1
         npt=nbndptlc
         call mshd3(coorlc, kmeshclc1, izwaar(i1), izwaar(i2), izwaar(i3), &
                    izwaar(i4), izwaar, nelemlc, npt, nhelp, &
                    coarse, rim)
         nelemlc=nelemlc-1
         npointlc=npt
         np=npointlc
      else
         call mshd09(kmeshclc1, npt, nelemlc, inpelm)
         if ( npt<nbndptlc) npt=nbndptlc
         npointlc=npt
         np=npointlc
      end if
      if ( ierror/=0 ) go to 1000

!     --- determine neighbours and adjacent elements

      ntot=14*nhelp

      do i=1, ntot
         izwaar(i)=0
      end do

      if ( npointlc>nhelp ) then
         call errint ( npointlc, 1 )
         call errint ( nhelp, 2 )
         call errsub ( 900, 2, 0, 0)
         go to 1000
      end if

      i1=1
      i2=1+npointlc
      i3=1+2*npointlc
      i4=1+8*nhelp
      l3=i4-i3
      l4=6*nhelp
      istep=2
      call mshd06 ( kmeshclc1, nelemlc, npointlc, nbndptlc, 3, izwaar(i1), &
                    izwaar(i2), izwaar(i3), izwaar(i4), l3, l4, istep )
      if ( ierror/=0 ) go to 1000
      if (npt>=npti) then
         call mshd08(coorlc, npt, npti, izwaar(i1), izwaar(i3), rim)
         if ( ierror/=0 ) go to 1000
      end if
      call mshd6 ( coorlc, kmeshclc1, nelemlc, npointlc, npt, npti, &
                   izwaar(i1), izwaar(i2), izwaar(i3), izwaar(i4) )

      if ( ierror/=0 ) go to 1000

      if ( inpelm==7 ) then

!     --- inpelm = 7, also fill centroid

          call mshtriancentr ( kmeshclc1, npt, nelemlc, inpelm )
          call mshd7 ( kmeshclc1, coorlc, nelemlc, 2 )

      end if
      if ( ierror/=0 ) go to 1000
      npointlc=npt

!     --- renumbering

      if ( jnew ) then
         call mshg09(izwaar, ipos1, ipos2, ipos3, ipos4, ipos5, ipos6, &
                     np, npointlc, nhelp, nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg04(izwaar(ipos2), izwaar(ipos1), izwaar(ipos4), &
                     izwaar(ipos3), np, npointlc, kmeshclc1, inpelm, &
                     izwaar(ipos6), nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg05(izwaar(ipos5), izwaar(ipos4), npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar(ipos4))
         if ( ierror/=0 ) go to 1000

!        --- fill array nheen

         do i=1, npointlc
            nheen(i)=izwaar(ipos5+i-1)
         end do
      else
         call mshg05(izwaar, nheen, npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar)
         if ( ierror/=0 ) go to 1000
      end if

      goto 200

!     --- inpelm = 4 :  mshv4

140   npti=nbndptlc+1
      if ( jnew ) then
         nelemlc=1
         npt=nbndptlc
         call mshv4(coorlc, kmeshclc1, izwaar(i2), izwaar(i3), izwaar(i1), &
                    izwaar, nelemlc, npt, coarse, rim)
         nelemlc=nelemlc-1
         npointlc=npt
      end if
      if ( ierror/=0 ) go to 1000

!     --- repositioning

      ntot=14*nhelp
      do i=1, ntot
         izwaar(i)=0
      end do

      i1=1
      i2=1+npointlc
      i3=1+2*npointlc
      i4=1+8*nhelp
      l3=i4-i3
      l4=6*nhelp
      istep=1
      call mshd06 ( kmeshclc1, nelemlc, npointlc, nbndptlc, 4, izwaar(i1), &
                    izwaar(i2), izwaar(i3), izwaar(i4), l3, l4, istep )
      if ( ierror/=0 ) go to 1000
      call mshd08(coorlc, npointlc, npti, izwaar(i1), izwaar(i3), rim)
      if ( ierror/=0 ) go to 1000
      if ( jnew ) then

!     --- renumbering

         call mshg09(izwaar, ipos1, ipos2, ipos3, ipos4, ipos5, ipos6, &
                     npointlc, npointlc, nhelp, nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg04(izwaar(ipos2), izwaar(ipos1), izwaar(ipos4), &
                     izwaar(ipos3), npointlc, npointlc, kmeshclc1, inpelm, &
                     izwaar(ipos6), nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg05(izwaar(ipos5), izwaar(ipos4), npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar(ipos4))

!        --- fill array nheen

         do i=1, npointlc
            nheen(i)=izwaar(ipos5+i-1)
         end do
      else
         call mshg05(izwaar, nheen, npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar)
      end if
      if ( ierror/=0 ) go to 1000

      goto 200

!     --- inpelm = 8 0r 9 : mshv8 and mshv9

150   nh=nbndptlc/4
      nh=4*nh-nbndptlc
      if ( nh/=0 ) then

!     --- Error: number of vertices is odd

         call errint ( nbndptlc, 1 )
         call errsub ( 359, 1, 0, 0 )
         go to 1000

      end if

      n=nbndptlc/2
      izwaar(i3)=n

      do i=1, n
         izwaar(i3+i)=2*i-1
      end do

      izwaar(i3+n+1)=0
      rim=rim*2d0
      npti=nbndptlc+1
      if ( jnew ) then
         nelemlc=1
         npt=nbndptlc
         call mshv4 ( coorlc, kmeshclc1, izwaar(i2), izwaar(i3), izwaar(i1), &
                      izwaar, nelemlc, npt, coarse, rim )
         nelemlc=nelemlc-1
         npointlc=npt
         np=npointlc
      else
         call mshd10(kmeshclc1, npt, nelemlc, inpelm)
         if ( npt<nbndptlc) npt=nbndptlc
         npointlc=npt
         np=npointlc
      end if
      if ( ierror/=0 ) go to 1000

!     --- determine neighbours and adjacent elements

      ntot=16*nhelp

      do i=1, ntot
         izwaar(i)=0
      end do

      i1=1
      i2=1+npointlc
      i3=1+2*npointlc
      i4=1+8*nhelp
      l3=i4-i3
      l4=6*nhelp
      istep=2
      call mshd06 ( kmeshclc1, nelemlc, npointlc, nbndptlc, 4, izwaar(i1), &
                    izwaar(i2), izwaar(i3), izwaar(i4), l3, l4, istep )
      if ( ierror/=0 ) go to 1000
      if (npt>=npti) then
         call mshd08(coorlc, npt, npti, izwaar(i1), izwaar(i3), rim)
         if ( ierror/=0 ) go to 1000
      end if
      call mshv8(coorlc, kmeshclc1, nelemlc, npointlc, npt, npti, nbndptlc, &
                 izwaar(i1), izwaar(i2), izwaar(i3), izwaar(i4))
      if ( ierror/=0 ) go to 1000

      if ( inpelm==9 ) call mshv9(coorlc, kmeshclc1, nelemlc, npt)
      if ( ierror/=0 ) go to 1000

      npointlc=npt
      if ( jnew ) then

!     --- renumbering

         call mshg09(izwaar, ipos1, ipos2, ipos3, ipos4, ipos5, ipos6, &
                     np, npointlc, nhelp, nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg04(izwaar(ipos2), izwaar(ipos1), izwaar(ipos4), &
                     izwaar(ipos3), np, npointlc, kmeshclc1, inpelm, &
                     izwaar(ipos6), nelemlc)
         if ( ierror/=0 ) go to 1000
         call mshg05(izwaar(ipos5), izwaar(ipos4), npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, izwaar(ipos5))
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar(ipos4))

!        --- fill array nheen

         do i=1, npointlc
            nheen(i)=izwaar(ipos5+i-1)
         end do
      else
         call mshg05(izwaar, nheen, npointlc)
         if ( ierror/=0 ) go to 1000
         call mshg06(kbndpt, nbndptlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg07(kmeshclc1, inpelm, nelemlc, nheen)
         if ( ierror/=0 ) go to 1000
         call mshg08 ( coorlc, npointlc, izwaar)
      end if

200   if ( inpelm<=4 ) goto 1000

!     --- Put boundary coordinates into coorlc again

      do i = 1, nbndptlc
         if ( jnew ) then
             ikn=izwaar(ipos5+i-1)
         else
             ikn=nheen(i)
         end if
         coorlc(2*ikn-1)=bcord(2*i-1)
         coorlc(2*ikn)=bcord(2*i)
      end do

!     --- Put centroids of elements in the correct place in case of bilinear
!         quadrilaterals

      if ( inpelm==9 ) call mshv10 ( coorlc, kmeshclc1, nelemlc )

1000  if ( itime==1) call printtime('general')

      deallocate ( coarse, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coarse' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshg'

      end if  ! ( debug )
      call erclos ( 'mshg' )

      end subroutine mshg
