      subroutine mshline1segm ( icurnr, maxratio, rinput, nelm, ishape, &
                                nuspnt, n, ndim, tau, work, j1, j2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array tau and compute sleng for a line with 1 segment
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     curve
!     point_distribution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: maxratio, rinput(*), work(*)
      double precision, intent(out) :: tau(*)
      integer, intent(in) :: icurnr,  nelm, ishape, &
                             nuspnt, ndim, j1, j2
      integer, intent(out) :: n

!     icurnr         i    Curve sequence number
!     ishape         i    Shape number of elements
!     j1             i    pointer in array rinput
!     j2             i    pointer in array rinput
!     maxratio       i    max allowed coarseness ratio
!     n              o    number of points in curve
!     ndim           i    Dimension of the space.
!     nelm           i    number of elements along a curve
!     nuspnt         i    Number of user points
!     rinput         i    Real input array
!     tau            o    Work array of length n to store the relative distances
!                         between points
!     work           i    work array of length ishape
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: ri1, ri2, alpha, dtau, s, som
      integer :: j, ih, newpoints, i

!     alpha          helpvariable for computation of new internal points
!     dtau           difference of tau values
!     i              Counting variable
!     ih             pointer in rinput array
!     j              Counting variable
!     newpoints      number of new points to be created
!     ri1            coarseness starting point
!     ri2            coarseness end point
!     s              local independent curve variable
!     som            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshline1segm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshline1segm'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ih    = ndim * nuspnt + 1
      ri1   = rinput(ih+j1)
      ri2   = rinput(ih+j2)
      newpoints = ishape * nelm - 1
      n     = newpoints + 2

      tau(1) = 0d0
      tau(n) = 1d0
      if ( debug ) write(irefwr,2) 'ri1, ri2', ri1, ri2

      if ( newpoints>0 ) then

         if ( nelm>1 ) then

!        --- nelm>1

            alpha = ( ri2 / ri1 ) ** ( 1d0 / (nelm-1) )

            if ( maxratio> 1d0 ) then

!           --- This check only if user has submitted a value

              if ( alpha>maxratio .or. alpha<(1d0/maxratio) ) then
                 write(irefwr,*) &
                  'MSH031: curve',icurnr,'coarsenessproblems?!'
                 write(irefwr,*) 'Coarseness: userp',j1,' = ',ri1
                 write(irefwr,*) 'and         userp',j2,' = ',ri2
              end if
              if ( alpha<1d0/maxratio ) then

!             --- Compute new coarseness in point: j1

                 s = ri2 / ( (1d0/maxratio) ** (nelm-1) )

                 write(irefwr,*) 'Warning, coarseness',j1,'too large'
                 write(irefwr,*) 'Advise:Use',s,'in userpoint',j1

              else if ( alpha>maxratio ) then

!             --- Compute new coarseness in point: j2

                 s = ri1 * ( maxratio ** (nelm-1) )
                 write(irefwr,*) 'Warning, coarseness',j2,'too large'
                 write(irefwr,*) 'Advise:Use c-value',s,'in node',j2
              end if

            end if

!           --- determine factors

            som = 1d0

            do i = 1 , nelm-1
               som  = alpha * som + 1d0
            end do

            s = 1d0

            do i = 1+ishape, newpoints+1, ishape
               tau(i) = s/som
               s = alpha * s + 1d0
            end do

         end if

         if ( ishape>1 ) then

!        --- ishape>1, Fill points in between according to isubd
!            The relative division is stored in work

            do i = 2, newpoints + 1 , ishape
               dtau   = tau(i+ishape-1)-tau(i-1)
               do j = 1, ishape-1
                  tau(i+j-1) = tau(i-1) + dtau * work(j+1)
               end do
            end do

         end if

      end if

1000  call erclos ( 'mshline1segm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshline1segm'

      end if  ! ( debug )

      end subroutine mshline1segm
