      subroutine mshmap3d2d ( iuserp1, iuserp2, iuserp3, rinput, &
                              tang1, tang2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-1999
!
!   copyright (c) 1999-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Define the mapping of a 3d plane to 2d using the 3 user points iuserp?
!    The mapping is defined by x = x1 + alpha t1 + beta t2
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     2d
!     3d
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuserp1, iuserp2, iuserp3
      double precision rinput(3,*), tang1(3), tang2(3)

!     iuserp1        i    First user point
!     iuserp2        i    Second user point
!     iuserp3        i    third user point
!     rinput         i    Real input array
!     tang1          o    First tangential vector
!     tang2          o    Second tangential vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision normal(3), dist, t1, t2, t3, eps, &
                       t11, t12, t13, t21, t22, t23, lengthvec

!     dist           Distance between two points
!     eps            Local accuracy
!     i              Counting variable
!     lengthvec      length of a vector (l2 norm)
!     normal         normal vector
!     t1             dummy
!     t11            dummy
!     t12            dummy
!     t13            dummy
!     t2             dummy
!     t21            dummy
!     t22            dummy
!     t23            dummy
!     t3             dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision normvec, innerproduct
      normvec(t1,t2,t3) = sqrt(t1**2+t2**2+t3**2)
      innerproduct(t11,t12,t13,t21,t22,t23) = t11*t21+t12*t22+t13*t23

!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ININORMOUTPROD Compute the normalized outer product of vectors v1 and v2
!                    and store in v3
!     INNERPRODUCT   Computes inner product of two vectors
!     NORMVEC        Computes length of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2083   Points too close to each other to create a tangential vector
!    2084   3 points nearly on a line, so no plane can be constructed
! **********************************************************************
!
!                       PSEUDO CODE
!
!      t1 = p2-p1/||p2-p1||
!      t2 = p3-p1/||p3-p1||
!      if ( (t1,t2)=1 ) then
!         error three points on a line
!      else
!         n = t1xt2 / ||t1xt2||
!         t2 = t1 x n / ||t1 x n||
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmap3d2d' )

!     --- Compute accuracy parameter

      eps = max ( abs(rinput(1,iuserp1)), abs(rinput(2,iuserp1)), &
                  abs(rinput(3,iuserp1)), abs(rinput(1,iuserp2)), &
                  abs(rinput(2,iuserp2)), abs(rinput(3,iuserp2)), &
                  abs(rinput(1,iuserp3)), abs(rinput(2,iuserp3)), &
                  abs(rinput(3,iuserp3)) ) * sqreps

!     --- t1 = p2-p1/||p2-p1||
!         t2 = p3-p1/||p3-p1||

      do i = 1, 3
         tang1(i) = rinput(i,iuserp2) - rinput(i,iuserp1)
         tang2(i) = rinput(i,iuserp3) - rinput(i,iuserp2)
      end do
      lengthvec = normvec(tang1(1),tang1(2),tang1(3))
      if ( lengthvec<=eps ) then

!     --- First two points too close to each other

         call errint ( iuserp1, 1 )
         call errint ( iuserp2, 2 )
         call erreal ( rinput(1,iuserp1), 1 )
         call erreal ( rinput(2,iuserp1), 2 )
         call erreal ( rinput(3,iuserp1), 3 )
         call erreal ( rinput(1,iuserp2), 4 )
         call erreal ( rinput(2,iuserp2), 5 )
         call erreal ( rinput(3,iuserp2), 6 )
         call errsub ( 2083, 2, 6, 0 )
         go to 1000

      end if
      lengthvec = 1d0/lengthvec
      do i = 1, 3
         tang1(i) = tang1(i)*lengthvec
      end do
      lengthvec = normvec(tang2(1),tang2(2),tang2(3))
      if ( lengthvec<=eps ) then

!     --- First and third points too close to each other

         call errint ( iuserp1, 1 )
         call errint ( iuserp3, 2 )
         call erreal ( rinput(1,iuserp1), 1 )
         call erreal ( rinput(2,iuserp1), 2 )
         call erreal ( rinput(3,iuserp1), 3 )
         call erreal ( rinput(1,iuserp3), 4 )
         call erreal ( rinput(2,iuserp3), 5 )
         call erreal ( rinput(3,iuserp3), 6 )
         call errsub ( 2083, 2, 6, 0 )
         go to 1000

      end if
      lengthvec = 1d0/lengthvec
      do i = 1, 3
         tang2(i) = tang2(i)*lengthvec
      end do
      dist = innerproduct(tang1(1),tang1(2),tang1(3), &
                          tang2(1),tang2(2),tang2(3))

      if ( abs(dist-1d0)<0.95d0 ) then

!     --- (t1,t2)=1, three points on a line

         call errint ( iuserp1, 1 )
         call errint ( iuserp2, 2 )
         call errint ( iuserp3, 3 )
         call errsub ( 2084, 3, 0, 0 )
         go to 1000

      else

!     --- n = t1xt2 / ||t1xt2||
!         t2 = t1 x n / ||t1 x n||

         call ininormoutprod ( tang1, tang2, normal )

!        --- Since n and t1 are perpendicular t2 has length 1

         call eloutr ( normal, tang1, tang2 )

      end if

1000  call erclos ( 'mshmap3d2d' )
      end
