      subroutine mshob3dcross ( xp, yp, zp, ipcell, jpcell, kpcell, &
                                onobstacle, iobgrid, nx, ny, nz, &
                                ixcross, iycross, izcross, eps, coor, &
                                kmeshc, inpelm, leniobgrid, xi, nxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 07-01-2002 Extra parameters xi and nxi
!        version  2.0  date 14-11-2001 Extra parameter leniobgrid
!        version  1.0  date 29-06-2001
!
!   copyright (c) 2001-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute number of cross sections of the line y=yp,z=zp with the obstacle in
!    cell (i,j,k) left and right of x=xp and of the line z=zp,x=xp with the
!    obstacle in cell (i,j,k) left and right of y=yp and also of the line
!    y=yp,x=xp with the obstacle in cell (i,j,k) left and right of z=zp
!    Store the results in ixcross,izcross and iycross
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     node
!     obstacle
!     crossing
!     cross_section
!     crossing_point
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical onobstacle
      double precision xp, yp, zp, coor(3,*), eps, xi(*)
      integer ipcell, jpcell, kpcell, nx, ny, nz, nxi, &
              iobgrid(0:nx-1,0:ny-1,0:nz-1,*), &
              ixcross(2), iycross(2), izcross(2), inpelm, &
              kmeshc(inpelm,*), leniobgrid

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         obstacle
!     eps            i    Accuracy
!                         Points that are at a distance less than eps of the
!                         obstacle are defined as part of the obstacle
!     inpelm         i    number of points per element in obstacle
!     iobgrid        i    In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     ipcell         i    i-index of cell(i,j,k)
!     ixcross        o    Contains the number of cross sections of the line
!                         y=xp,z=zp with the obstacle within cell(i,j.k)
!                         ixcross(1) contains the number of cross sections
!                         below xp
!                         ixcross(2) contains the number of cross sections
!                         above xp
!     iycross        o    Contains the number of cross sections of the line
!                         x=xp,z=zp with the obstacle within cell(i,j,k)
!                         iycross(1) contains the number of cross sections
!                         at the left-hand side of yp
!                         iycross(2) contains the number of cross sections
!                         at the right-hand side of yp
!     izcross        o    Contains the number of cross sections of the line
!                         x=xp,y=yp with the obstacle within cell(i,j,k)
!                         izcross(1) contains the number of cross sections
!                         at the left-hand side of zp
!                         izcross(2) contains the number of cross sections
!                         at the right-hand side of zp
!     jpcell         i    j-index of cell(i,j,k)
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     kpcell         i    k-index of cell(i,j,k)
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     nxi            o    Number of points that are intersected in x-direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     onobstacle     o    If true the point is positioned on the obstacle
!     xi             o    Array containing the x-coordinates of the intersection
!                         points in the cell
!     xp             i    x-coordinate of point
!     yp             i    y-coordinate of point
!     zp             i    z-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, icell(leniobgrid), ncell, i, nyi, nzi, l, &
              inode
      double precision x(3), y(3), z(3), xcross, ycross, zcross, &
                       yi(leniobgrid), zi(leniobgrid), &
                       alpha1, beta1, det

!     alpha1         multiplication factor
!     beta1          multiplication factor
!     det            determinant
!     i              Counting variable
!     icell          Cell number of cell under consideration
!     ielem          Element number
!     inode          Node sequence number
!     l              General loop variable
!     ncell          Number of cells that must be used to compute
!                    cross sections
!     nyi            Number of points that are intersected in y-direction
!     nzi            Number of points that are intersected in z-direction
!     x              x co-ordinate of points of triangle
!     xcross         x co-ordinate of intersection point
!     y              y co-ordinate of point of points of triangle
!     ycross         y co-ordinate of intersection point
!     yi             Array containing the y-coordinates of the intersection
!                    points in the cell
!     z              z co-ordinate of point of points of triangle
!     zcross         z co-ordinate of intersection point
!     zi             Array containing the z-coordinates of the intersection
!                    points in the cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!    A triangle (x_1, x_2, x_3) is defined by
!       x = x_1 + alpha (x_2-x_1) + beta (x_3-x_1)
!
!    0 <= alpha <=1; 0 <= beta <=1; 0 <= alpha+beta <=1
!    The cross section with the line x=xp, y=yp is defined as
!
!       xp = xp_1 + alpha (xp_2-xp_1) + beta (xp_3-xp_1)
!       yp = yp_1 + alpha (yp_2-yp_1) + beta (yp_3-yp_1)
!
!    Hence
!
!       | alpha | = 1/det  | yp_3-yp_1  xp_1-xp_3 | | xp-xp_1 |
!       | beta  |          | yp_1-yp_2  xp_2-xp_1 | | yp-yp_1 |
!
!    with det =  (xp_2-xp_1) (yp_3-yp_1) - (xp_3-xp_1) (yp_2-yp_1)
!
!
!    onobstacle := false
!    ixcross := 0;  iycross := 0;  izcross := 0
!    Store triangles in cell into icell (1 to ncell)
!    For i := 1 (1) ncell do
!       ielem := element number icell(i)
!       Store coordinates of triangle in x, y and z
!       Find intersection with line x=xp, y=yp
!       if ( det#0 ) then
!          Compute alpha, beta
!          if ( 0 <= alpha <=1; 0 <= beta <=1; 0 <= alpha+beta <=1 ) then
!             zp = zp_1 + alpha (zp_2-zp_1) + beta (zp_3-zp_1)
!             Store zp in zi if not double
!       Find intersection with line y=yp, z=zp
!       if ( det#0 ) then
!          Compute alpha, beta
!          if ( 0 <= alpha <=1; 0 <= beta <=1; 0 <= alpha+beta <=1 ) then
!             xp = xp_1 + alpha (xp_2-xp_1) + beta (xp_3-xp_1)
!             Store xp in xi if not double
!       Find intersection with line z=zp, x=xp
!       if ( det#0 ) then
!          Compute alpha, beta
!          if ( 0 <= alpha <=1; 0 <= beta <=1; 0 <= alpha+beta <=1 ) then
!             yp = yp_1 + alpha (yp_2-yp_1) + beta (yp_3-yp_1)
!             Store yp in yi if not double
!       For all points x in xi do
!          if ( x<xp ) then
!             ixcross(1) := ixcross(1)+1
!          else if ( x>xp ) then
!             ixcross(2) := ixcross(2)+1
!          else
!             onobstacle := true
!       For all points y in yi do
!          if ( y<yp ) then
!             iycross(1) := iycross(1)+1
!          else if ( y>yp ) then
!             iycross(2) := iycross(2)+1
!          else
!             onobstacle := true
!       For all points z in zi do
!          if ( z<zp ) then
!             izcross(1) := izcross(1)+1
!          else if ( z>zp ) then
!             izcross(2) := izcross(2)+1
!          else
!             onobstacle := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshob3dcross' )

      if ( inpelm.ne.3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- onobstacle := false
!         ixcross := 0;  iycross := 0;  izcross := 0

      onobstacle = .false.
      ixcross(1) = 0
      ixcross(2) = 0
      iycross(1) = 0
      iycross(2) = 0
      izcross(1) = 0
      izcross(2) = 0

!     --- Store triangles in cell into icell

      ncell = 1
      icell(1) = iobgrid(ipcell,jpcell,kpcell,1)
      do l = 2, leniobgrid
         if ( iobgrid(ipcell,jpcell,kpcell,l).eq.0 ) go to 100
         ncell = ncell+1
         icell(ncell) = iobgrid(ipcell,jpcell,kpcell,l)
      end do

!     --- For i := 1 (1) ncell do

100   nxi = 0
      nyi = 0
      nzi = 0
      do i = 1, ncell

!     --- Store coordinates in x, y and z

         ielem = icell(i)
         do l = 1, inpelm
            inode = kmeshc(l,ielem)
            x(l) = coor(1,inode)
            y(l) = coor(2,inode)
            z(l) = coor(3,inode)
         end do

!        --- First find intersection with the line x=xp and y=yp

         det = (x(2)-x(1))*(y(3)-y(1))-(x(3)-x(1))*(y(2)-y(1))
         if ( abs(det).gt.eps*1d-4 ) then

!        --- determinant not zero, intersection possible

            det = 1d0/det
            alpha1 = det*((y(3)-y(1))*(xp-x(1))+(x(1)-x(3))*(yp-y(1)))
            beta1  = det*((y(1)-y(2))*(xp-x(1))+(x(2)-x(1))*(yp-y(1)))
            if ( alpha1.ge.-1d-6 .and. alpha1.le.1.000001d0 .and. &
                 beta1.ge.-1d-6 .and. beta1.le.1.000001d0 .and. &
                 alpha1+beta1.le.1.000001d0 ) then

!           --- intersection found

               zcross = z(1)+alpha1*(z(2)-z(1))+beta1*(z(3)-z(1))
               if ( abs(zp-zcross).le.eps ) then

!              --- Point on obstacle

                  onobstacle = .true.
                  go to 1000

               else

!              --- store z in zi if point not yet stored

                  do l = 1, nzi
                     if ( abs(zi(l)-zcross ).le.1d-5 ) go to 110
                  end do
                  nzi = nzi+1
                  zi(nzi) = zcross

               end if

            end if

         end if

!        --- Next find intersection with the line y=yp and z=zp

110      det = (y(2)-y(1))*(z(3)-z(1))-(y(3)-y(1))*(z(2)-z(1))
         if ( abs(det).gt.eps*1d-4 ) then

!        --- determinant not zero, intersection possible

            det = 1d0/det
            alpha1 = det*((z(3)-z(1))*(yp-y(1))+(y(1)-y(3))*(zp-z(1)))
            beta1  = det*((z(1)-z(2))*(yp-y(1))+(y(2)-y(1))*(zp-z(1)))
            if ( alpha1.ge.-1d-6 .and. alpha1.le.1.000001d0 .and. &
                 beta1.ge.-1d-6 .and. beta1.le.1.000001d0 .and. &
                 alpha1+beta1.le.1.000001d0 ) then

!           --- intersection found

               xcross = x(1)+alpha1*(x(2)-x(1))+beta1*(x(3)-x(1))
               if ( abs(xp-xcross).le.eps ) then

!              --- Point on obstacle

                  onobstacle = .true.
                  go to 1000

               else

!              --- store x in xi if point not yet stored

                  do l = 1, nxi
                     if ( abs(xi(l)-xcross ).le.1d-5 ) go to 120
                  end do
                  nxi = nxi+1
                  xi(nxi) = xcross

               end if

            end if

         end if

!        --- Finally find intersection with the line z=zp and x=xp

120      det = (z(2)-z(1))*(x(3)-x(1))-(z(3)-z(1))*(x(2)-x(1))
         if ( abs(det).gt.eps*1d-4 ) then

!        --- determinant not zero, intersection possible

            det = 1d0/det
            alpha1 = det*((x(3)-x(1))*(zp-z(1))+(z(1)-z(3))*(xp-x(1)))
            beta1  = det*((x(1)-x(2))*(zp-z(1))+(z(2)-z(1))*(xp-x(1)))
            if ( alpha1.ge.-1d-6 .and. alpha1.le.1.000001d0 .and. &
                 beta1.ge.-1d-6 .and. beta1.le.1.000001d0 .and. &
                 alpha1+beta1.le.1.000001d0 ) then

!           --- intersection found

               ycross = y(1)+alpha1*(y(2)-y(1))+beta1*(y(3)-y(1))
               if ( abs(yp-ycross).le.eps ) then

!              --- Point on obstacle

                  onobstacle = .true.
                  go to 1000

               else

!              --- store y in yi if point not yet stored

                  do l = 1, nyi
                     if ( abs(yi(l)-ycross ).le.1d-5 ) go to 200
                  end do
                  nyi = nyi+1
                  yi(nyi) = ycross

               end if

            end if

         end if

200   end do

!     --- Fill ixcross, iycross and izcross

      do i = 1, nxi

         if ( xi(i).lt.xp ) then

!        --- Cross section below xp

            ixcross(1) = ixcross(1)+1

         else

!        --- Cross section above xp

            ixcross(2) = ixcross(2)+1

         end if

      end do

      do i = 1, nyi

         if ( yi(i).lt.yp ) then

!        --- Cross section below yp

            iycross(1) = iycross(1)+1

         else

!        --- Cross section above yp

            iycross(2) = iycross(2)+1

         end if

      end do

      do i = 1, nzi

         if ( zi(i).lt.zp ) then

!        --- Cross section below zp

            izcross(1) = izcross(1)+1

         else

!        --- Cross section above zp

            izcross(2) = izcross(2)+1

         end if

      end do

1000  call erclos ( 'mshob3dcross' )
      end
