      subroutine mshobsfl3in ( nx, ny, nz, iobgrid, kmeshc, coor, &
                               inpelm, xmin, ymin, zmin, dx, dy, dz, &
                               leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 13-09-2004 Minor adaptation moving xmid,ymid
!        version  2.1  date 27-09-2002 Remove error message
!        version  2.0  date 15-11-2001 Correction for triangles parallel to
!                                      z-axis; Extra parameter leniobgrid
!        version  1.0  date 15-06-2001
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid with information about which cells are internal
!     R^3 only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, inpelm, iobgrid(0:nx-1,0:ny-1,0:nz-1,*), &
              kmeshc(inpelm,*), leniobgrid
      double precision coor(3,*), xmin, ymin, zmin, dx, dy, dz

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         obstacle
!     dx             i    length of cell in x-direction
!     dy             i    length of cell in y-direction
!     dz             i    length of cell in z-direction
!     inpelm         i    number of points per element
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     nx             i    number of cells in x direction
!     ny             i    number of cells in y direction
!     nz             i    number of cells in z direction
!     xmin           i    Minimum x-value of region around obstacle
!     ymin           i    Minimum y-value of region around obstacle
!     zmin           i    Minimum z-value of region around obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, icount, ielem, l1, inodp, numcross, l, jcount
      double precision xmid, ymid, x(3), y(3), z(3), xsi, eta, yint, &
                       detinv, delta, e1(2), e2(2), dfx, dfy, phi(3), &
                       zmid, zmin1, zmax1, xsimax, xsimin
      logical finished, outobst, warning

!     delta          jacobian of element
!     detinv         inverse of determinant
!     dfx            xmid - x1
!     dfy            ymid - y1
!     e1             contains x2-x1; x3-x1
!     e2             contains y2-y1; y3-y1
!     eta            second local coordinate
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     inodp          Nodal point number
!     j              Counting variable
!     jcount         Counter
!     k              Counting variable
!     l              General loop variable
!     l1             general loop variable
!     numcross       Number of cross sections
!     outobst        if true, the cell is outside the obstacle
!     phi            Contains values of linear basis functions corresponding
!                    to triangle in (xmid,ymid)
!     warning        If true already a warning has been printed
!     x              x co-ordinate of point
!     xmid           x co-ordinate of mid point
!     xsi            first local coordinate
!     xsimax         maximum value of xsi
!     xsimin         minimum value of xsi
!     y              y co-ordinate of point
!     yint           y coordinate for interpolation
!     ymid           y co-ordinate of mid point
!     z              z co-ordinate of point
!     zmax1          maximum z-value in cell
!     zmid           value of z in triangle for point (xmid,ymid)
!     zmin1          minimum z-value in cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2254   inpelm # 3
!    2255   internal error: cell is internal and external
!    2257   internal error: more than 10 attempts to compute cross-section
!    2258   internal error: last cell is internal
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For each column in z-direction do
!        Compute xmid, ymid
!        finished := false
!        while ( not finished ) do
!           outobst = true
!           do k = 0, nz
!              if ( iobgrid(i,j,k,1) = 0 and not outobst ) then
!                 iobgrid(i,j,k,1) = -1
!              else if ( iobgrid(i,j,k,1) = -1 and outobst ) then
!                 error
!              else ( iobgrid(i,j,k,1) > 0 ) then
!                 Compute intersection with line x=xmid, y=ymid with all
!                 triangles in cell
!                 if ( intersection too close to boundary of triangle ) then
!                    adapt xmid and ymid
!                    repeat the process
!                 else ( standard case )
!                    if ( number of cross sections computed is odd ) then
!                       outobst = not outobst
!           if ( not repeat ) then
!              finished := true
!
!    Computation of intersection with line x=xmid, y=ymid:
!
!    Map triangle in (x,y,z) space onto unity triangle in (xsi,eta) space
!    with coordinates: (0,0), (1,0) and (0,1)
!
!    Basis functions in unit triangle:  phi_1 = 1-xsi-eta
!                                       phi_2 = xsi
!                                       phi_3 = eta
!
!    Mapping:  x = sum_j phi_j x_j
!
!    So:  x = (1-xsi-eta) x_1 + xsi x_2 + eta x_3 (all vectors)
!
!    Solving gives
!
!    ( x_2-x_1  x_3-x_1 ) xsi = x-x_1  (all vectors)
!
!    This system can be solved using the x and y component only if the
!    determinant of the matrix (i.e. without z components) is unequal to 0
!    Otherwise the triangle is parallel to the z-axis
!
!    Solution gives:
!
!    | x2-x1  x3-x1 | | xsi |   | x-x1 |
!    | y2-y1  y3-y1 | | eta | = | y-y1 |
!
!    with e1(1) = x2-x1, e1(2) = x3-x1, e2(1) = y2-y1, e2(2) = y3-y1
!
!    | e1(1) e1(2) | | xsi |   | x-x1 |
!    | e2(1) e2(2) | | eta | = | y-y1 |
!
!    we get
!
!    delta = e1(1) e2(2) - e1(2) e2(1)
!
!    | xsi |           |  e2(2) -e1(2) | | x-x1 |
!    | eta | = 1/delta | -e2(1)  e1(1) | | y-y1 |
!
!    There is an intersection if all phi_i in the interval (0,1)
!
!    If the triangle is parallel to the z-axis and the line x=xmid, y=ymid is
!    close to the triangle move the line a little bit in order to avoid
!    difficult situations
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl3in' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfl3in'
         write(irefwr,*) 'nx, ny, nz ', nx, ny, nz
         write(irefwr,*) 'xmin, ymin, zmin ', xmin, ymin, zmin
         write(irefwr,*) 'dx, dy, dz ', dx, dy, dz

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      if ( inpelm.ne.3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if
      warning = .false.

!     --- For each column in z-direction do

      do i = 0, nx-1
         do j = 0, ny-1

!        --- Compute xmid, ymid

            xmid = xmin+(i+0.5d0)*dx
            ymid = ymin+(j+0.5d0)*dy

!           --- finished := false

            finished = .false.
            icount = 0
            jcount = 0
            do while ( .not. finished )

!           --- outobst = true

               outobst = .true.
               jcount = jcount+1
               finished = .true.

               if ( jcount.gt.20 ) then

!              --- error: too much retries

                  call errint ( jcount, 1 )
                  call errint ( 20, 2 )
                  call errsub ( 2257, 2, 0, 0 )
                  go to 1000

               end if  ! ( jcount.gt.20 )

!              --- Loop over all cells in column

               do k = 0, nz-1

                  zmin1 = zmin+k*dz
                  zmax1 = zmin+(k+1)*dz
                  if ( iobgrid(i,j,k,1).eq.0 .and. .not. outobst ) then

!                 --- iobgrid(i,j,k,1) = 0 and not outobst, hence internal cell

                     iobgrid(i,j,k,1) = -1

                  else if ( iobgrid(i,j,k,1).eq.-1 .and. outobst ) then

!                 --- iobgrid(i,j,k,1) = -1 and outobst; error

                     call errsub ( 2255, 0, 0, 0 )
                     go to 1000

                  else

!                 --- iobgrid(i,j,k,1) > 0, hence cross-section with triangles
!                     Compute intersection with line x=xmid, y=ymid with all
!                     triangles in cell

                      numcross = 0
                      do l = 1, leniobgrid

                         ielem = iobgrid(i,j,k,l)
                         if ( ielem.eq.0 ) go to 100
                         do l1 = 1, inpelm

!                        --- Get node, Compute x1,y1,z1

                            inodp = kmeshc(l1,ielem)
                            x(l1) = coor(1,inodp)
                            y(l1) = coor(2,inodp)
                            z(l1) = coor(3,inodp)

                         end do
                         e1(1) = x(2)-x(1)
                         e1(2) = x(3)-x(1)
                         e2(1) = y(2)-y(1)
                         e2(2) = y(3)-y(1)
                         delta = e1(1)*e2(2)-e2(1)*e1(2)
                         if ( abs(delta).lt.epsmac ) then

!                        --- element is parallel to z-axis
!                            This is a special case

                            if ( abs(x(2)-x(1)).lt.sqreps .and. &
                                 abs(x(3)-x(1)).lt.sqreps ) then

!                           --- x constant, check xmid

                               if ( abs(xmid-x(1)).lt.sqreps ) then

!                              --- xmid close to x, repeat

                                  icount = icount+1
                                  xmid = xmid+0.01d0*dx
                                  ymid = ymid+0.01d0* &
                                         (1d0+icount*0.2d0)*dy
                                  if ( icount.gt.10 ) then

!                                 --- error: too much retries

                                     call errint ( icount, 1 )
                                     call errint ( 10, 2 )
                                     call errsub ( 2257, 2, 0, 0 )
                                     go to 1000

                                  else

!                                 --- repeat

                                     finished = .false.
                                     go to 200

                                  end if

                               end if

                            else if ( abs(y(2)-y(1)).lt.sqreps .and. &
                                      abs(y(3)-y(1)).lt.sqreps ) then

!                           --- y constant, check ymid

                               if ( abs(ymid-y(1)).lt.sqreps ) then

!                              --- xmid close to x, repeat

                                  icount = icount+1
                                  xmid = xmid+0.01d0*dx
                                  ymid = ymid+0.01d0* &
                                         (1d0+icount*0.2d0)*dy
                                  if ( icount.gt.10 ) then

!                                 --- error: too much retries

                                     call errint ( icount, 1 )
                                     call errint ( 10, 2 )
                                     call errsub ( 2257, 2, 0, 0 )
                                     go to 1000

                                  else

!                                 --- repeat

                                     finished = .false.
                                     go to 200

                                  end if

                               end if

                            else

!                           --- x nor y constant, a little bit more difficult
!                               The line in the plane z=constant is defined as
!                               x = x_1 + xsi(x_2-x_1)
!                               Find extreme values of xsi by substituting x_3

                               if ( abs(e1(1)).gt.sqreps ) then

!                              --- e1 # 0

                                  xsi = (x(3)-x(1))/e1(1)
                                  xsimin = min (xsi,0d0)-sqreps
                                  xsimax = max (xsi,1d0)+sqreps

!                                 --- compute xsi for x = xmid

                                  xsi = (xsi-x(1))/e1(1)
                                  if ( xsi.ge.xsimin .and. &
                                       eta.le. xsimax ) then

!                                 --- cross section possible, compute y

                                     yint = y(1)+xsi*e2(1)
                                     if ( abs(yint-ymid).lt.sqreps ) then

!                                    --- xmid,ymid close to line, repeat

                                        icount = icount+1
                                        xmid = xmid+0.01d0*dx
                                        ymid = ymid+0.01d0* &
                                            (1d0+icount*0.2d0)*dy
                                        if ( icount.gt.10 ) then

!                                       --- error: too much retries

                                           call errint ( icount, 1 )
                                           call errint ( 10, 2 )
                                           call errsub ( 2257, 2, 0, 0 )
                                           go to 1000

                                        else

!                                       --- repeat

                                           finished = .false.
                                           go to 200

                                        end if

                                     end if

                                  end if
                               else

!                              --- e2 # 0, replace 2 by 3 and vice versa

                                  xsi = (x(2)-x(1))/e1(2)
                                  xsimin = min (xsi,0d0)-sqreps
                                  xsimax = max (xsi,1d0)+sqreps

!                                 --- compute xsi for x = xmid

                                  xsi = (xsi-x(1))/e1(2)
                                  if ( xsi.ge.xsimin .and. &
                                       eta.le. xsimax ) then

!                                 --- cross section possible, compute y

                                     yint = y(1)+xsi*e2(2)
                                     if ( abs(yint-ymid).lt.sqreps ) then

!                                    --- xmid,ymid close to line, repeat

                                        icount = icount+1
                                        xmid = xmid+0.01d0*dx
                                        ymid = ymid+0.01d0* &
                                            (1d0+icount*0.2d0)*dy
                                        if ( icount.gt.10 ) then

!                                       --- error: too much retries

                                           call errint ( icount, 1 )
                                           call errint ( 10, 2 )
                                           call errsub ( 2257, 2, 0, 0 )
                                           go to 1000

                                        else

!                                       --- repeat

                                           finished = .false.
                                           go to 200

                                        end if

                                     end if

                                  end if

                               end if

                            end if

                         else

!                        --- Standard case, intersection can be computed

                            detinv = 1d0/delta
                            dfx = xmid-x(1)
                            dfy = ymid-y(1)
                            xsi = detinv * ( e2(2)*dfx-e1(2)*dfy)
                            eta = detinv * (-e2(1)*dfx+e1(1)*dfy)
                            phi(1) = 1d0-xsi-eta
                            phi(2) = xsi
                            phi(3) = eta
                            zmid = phi(1)*z(1)+phi(2)*z(2)+phi(3)*z(3)
                            if ( phi(1).gt.sqreps .and. &
                                 phi(2).gt.sqreps .and. &
                                 phi(3).gt.sqreps .and. &
                                 phi(1).lt.1d0-sqreps .and. &
                                 phi(2).lt.1d0-sqreps .and. &
                                 phi(3).lt.1d0-sqreps ) then

!                           --- intersection found, raise numcross

                               if (  zmid.ge.zmin1 .and. zmid.le.zmax1 ) &
                                   numcross = numcross+1

                            else if ( phi(1).gt.-sqreps .and. &
                                      phi(2).gt.-sqreps .and. &
                                      phi(3).gt.-sqreps .and. &
                                      phi(1).lt.1d0+sqreps .and. &
                                      phi(2).lt.1d0+sqreps .and. &
                                      phi(3).lt.1d0+sqreps ) then

!                           --- intersection close to boundary of triangle

                               icount = icount+1
                               xmid = xmid+0.01d0*dx
                               ymid = ymid+0.01d0* &
                                   (1d0+icount*0.2d0)*dy
                               if ( icount.gt.10 ) then

!                              --- error: too much retries

                                  call errint ( icount, 1 )
                                  call errint ( 10, 2 )
                                  call errsub ( 2257, 2, 0, 0 )
                                  go to 1000

                               else

!                              --- repeat

                                  finished = .false.
                                  go to 200

                               end if

                            end if

                         end if

                      end do
100                   if ( mod(numcross,2).eq.1 ) then

!                     --- number of cross sections computed is odd

                         outobst = .not. outobst

                      end if

                  end if

               end do

200         end do

!           --- final check, the last cell may never be internal

            if ( iobgrid(i,j,nz-1,1).eq.-1 ) then

!           --- last cell is internal

               if ( .not. warning ) then

!              --- The message is printed only once

                  call errwar ( 2258, 0, 0, 0 )
                  warning = .true.

               end if

            end if

         end do

      end do

1000  call erclos ( 'mshobsfl3in' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsfl3in'

      end if

      end
