      subroutine mshobsfl3yc ( nx, ny, nz, iobgrid, ielem, x, z, &
                               i1, i2, i3, inpelm, xmin, zmin, dx, dz, &
                               leniobgrid, leniobgridact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2001 Extra parameters leniobgrid,leniobgridact
!        version  1.0  date 15-06-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid for all edges of an obstacle element in case
!     the variation in j-direction of all elements is zero
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, iobgrid(0:nx-1,0:ny-1,0:nz-1,*), ielem, &
              i1(*), i2(*), i3(*), inpelm, leniobgrid, leniobgridact
      double precision x(*), z(*), xmin, zmin, dx, dz

!     dx             i    length of vector in x-direction
!     dz             i    length of vector in z-direction
!     i1             i    Array containing the "i" values of the vertices
!     i2             i    Array containing the "j" values of the vertices
!     i3             i    Array containing the "k" values of the vertices
!     ielem          i    Element number
!     inpelm         i    number of points per element
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     leniobgridact i/o   Actual length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     x              i    Contains the x-coordinates of the vertices
!     xmin           i    Minimum x-value
!     z              i    Contains the z-coordinates of the vertices
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision alpha, alphaf, alphal, gamma, gammaf, gammal, &
                       ratio
      integer i, j, k, l, kprev, lf, ll

!     alpha          Factor in x-direction
!     alphaf         first multiplication factor
!     alphal         last multiplication factor
!     gamma          Factor in z-direction
!     gammaf         first multiplication factor
!     gammal         last multiplication factor
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     kprev          Previous value of k
!     l              General loop variable
!     lf             index of first node on edge
!     ll             index of first node on edge
!     ratio          z-length/x-length in units dz/dx
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFL1      Fills array iobsgrid for one cell
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all edges do
!        Find information of one edge
!        If ( variation in x-direction is 0 ) then
!           Fill iobgrid for all z cells
!        else If ( variation in z-direction is 0 ) then
!           Fill iobgrid for all x cells
!        else ( variation in x nor z-direction is 0 ) then
!           Fill iobgrid for all cells in z-direction
!           Fill iobgrid for all cells in x-direction
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl3yc' )

!     --- First consider all edges

      do l = 1, inpelm

!     --- Mark all cells at the intersection line as boundary cells

         if ( i1(l).eq.i1(l+1) ) then

!        --- i1(l) = i1(l+1)

            if ( abs(i3(l+1)-i3(l)).gt.1 ) then

!           --- i3(l) # i3(l+1)

               do k = min(i3(l),i3(l+1)), max(i3(l),i3(l+1))
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i1(l), i2(1), k, &
                                   ielem, leniobgrid, leniobgridact )
               end do

            end if

         else

!        --- i1(l) # i1(l+1)

            if ( i3(l).eq.i3(l+1) .and. abs(i1(l+1)-i1(l)).gt.1 ) then

!           --- i3(l) = i3(l+1) and i1(l) # i1(l+1)

               do i = min(i1(l),i1(l+1)), max(i1(l),i1(l+1))
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i, i2(1), i3(l), &
                                   ielem, leniobgrid, leniobgridact )
               end do

            else if ( i3(l).ne.i3(l+1) ) then

!           --- i3(l) # i3(l+1) and i1(l) # i1(l+1)
!               Compute all cross sections of the edge with the grid
!               in the i-direction
!               Start with the end node of edge for which i1 is minimal
!               end stop at the other end node

               if ( i1(l+1).gt.i1(l) ) then
                  lf = l
                  ll = l+1
               else
                  lf = l+1
                  ll = l
               end if

               kprev  = i3(lf)
               alphaf = (x(lf)-xmin)/dx
               alphal = (x(ll)-xmin)/dx
               gammaf = (z(lf)-zmin)/dz
               gammal = (z(ll)-zmin)/dz
               ratio = (gammal-gammaf)/(alphal-alphaf)

               do i = i1(lf), i1(ll)
                  if( i.eq.i1(ll)) then

!                 --- the last point has to be treated differently

                     gamma = gammal

                  else

                     alpha = i+1
                     gamma = gammaf +(alpha-alphaf) * ratio

                  end if

                  k = min(int(max(gamma,0d0)),nz-1)
                  do j = min(k,kprev), max(k,kprev)
                     call mshobsfl1 ( nx, ny, nz, iobgrid, i, i2(1), &
                                      j, ielem, leniobgrid, &
                                      leniobgridact )
                  end do
                  kprev = k

               end do

            end if

         end if

      end do
      call erclos ( 'mshobsfl3yc' )
      end
