      subroutine mshobsfl3zc ( nx, ny, nz, iobgrid, ielem, x, y, &
                               i1, i2, i3, inpelm, xmin, ymin, dx, dy, &
                               leniobgrid, leniobgridact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 06-11-2012 Extra debug
!        version  2.0  date 14-11-2001 Extra parameters leniobgrid,leniobgridact
!        version  1.0  date 15-06-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid for all edges of an obstacle element in case
!     the variation in j-direction of all elements is zero
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, leniobgrid, inpelm, &
              iobgrid(0:nx-1,0:ny-1,0:nz-1,leniobgrid), ielem, &
              leniobgridact, i1(inpelm+1), i2(inpelm+1), i3(inpelm+1)
      double precision x(*), y(*), xmin, ymin, dx, dy

!     dx             i    length of vector in x-direction
!     dy             i    length of vector in y-direction
!     i1             i    Array containing the "i" values of the vertices
!     i2             i    Array containing the "j" values of the vertices
!     i3             i    Array containing the "k" values of the vertices
!     ielem          i    Element number
!     inpelm         i    number of points per element
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, ymin, zmax
!                         The cell size is dx x dy x dy
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,ymin+(k-1)dy) x
!                         (xmin+idx,ymin+jdy,ymin+kdy)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     leniobgridact i/o   Actual length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     x              i    Contains the x-coordinates of the vertices
!     xmin           i    Minimum x-value
!     y              i    Contains the y-coordinates of the vertices
!     ymin           i    Minimum y-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision beta, betaf, betal, alpha, alphaf, alphal, &
                       ratio
      integer i, j, k, l, kprev, lf, ll

!     alpha          Factor in z-direction
!     alphaf         first multiplication factor
!     alphal         last multiplication factor
!     beta           Factor in y-direction
!     betaf          first multiplication factor
!     betal          last multiplication factor
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     kprev          Previous value of k
!     l              General loop variable
!     lf             index of first node on edge
!     ll             index of first node on edge
!     ratio          y-length/x-length in units dy/dx
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFL1      Fills array iobsgrid for one cell
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all edges do
!        Find information of one edge
!        If ( variation in x-direction is 0 ) then
!           Fill iobgrid for all y cells
!        else If ( variation in y-direction is 0 ) then
!           Fill iobgrid for all x cells
!        else ( variation in x nor y-direction is 0 ) then
!           Fill iobgrid for all cells in y-direction
!           Fill iobgrid for all cells in x-direction
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl3zc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfl3zc'
         write(irefwr,1) 'inpelm', inpelm
         call prinin ( i1, inpelm+1, 'i1' )
         call prinin ( i2, inpelm+1, 'i2' )
         call prinin ( i3, inpelm+1, 'i3' )
         write(irefwr,1) 'leniobgrid', leniobgrid
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


!     --- First consider all edges

      do l = 1, inpelm

!     --- Mark all cells at the intersection line as boundary cells

         if ( i1(l).eq.i1(l+1) ) then

!        --- i1(l) = i1(l+1)

            if ( abs(i2(l+1)-i2(l)).gt.1 ) then

!           --- i2(l) # i2(l+1)

               do j = min(i2(l),i2(l+1)), max(i2(l),i2(l+1))
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i1(l), j, i3(1), &
                                   ielem, leniobgrid, leniobgridact )
               end do

            end if

         else

!        --- i1(l) # i1(l+1)

            if ( i2(l).eq.i2(l+1) .and. abs(i1(l+1)-i1(l)).gt.1 ) then

!           --- i2(l) = i2(l+1) and i1(l) # i1(l+1)

               do i = min(i1(l),i1(l+1)), max(i1(l),i1(l+1))
                  call mshobsfl1 ( nx, ny, nz, iobgrid, i, i2(l), i3(1), &
                                   ielem, leniobgrid, leniobgridact )
               end do

            else if ( i2(l).ne.i2(l+1) ) then

!           --- i2(l) # i2(l+1) and i1(l) # i1(l+1)
!               Compute all cross sections of the edge with the grid
!               in the i-direction
!               Start with the end node of edge for which i1 is minimal
!               end stop at the other end node

               if ( i1(l+1).gt.i1(l) ) then
                  lf = l
                  ll = l+1
               else
                  lf = l+1
                  ll = l
               end if

               kprev  = i2(lf)
               alphaf = (x(lf)-xmin)/dx
               alphal = (x(ll)-xmin)/dx
               betaf = (y(lf)-ymin)/dy
               betal = (y(ll)-ymin)/dy
               ratio = (betal-betaf)/(alphal-alphaf)

               do i = i1(lf),i1(ll)
                  if( i.eq.i1(ll)) then

!                 --- the last point has to be treated differently

                     beta = betal

                  else

                     alpha = i+1
                     beta = betaf +(alpha-alphaf) * ratio

                  end if

                  k = min(int(max(beta,0d0)),ny-1)

                  do j = min(k,kprev), max(k,kprev)
                     call mshobsfl1 ( nx, ny, nz, iobgrid, i, j, &
                                      i3(l), ielem, leniobgrid, &
                                      leniobgridact )
                  end do
                  kprev = k

               end do

            end if

         end if

      end do

1000  call erclos ( 'mshobsfl3zc' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsfl3zc'

      end if  ! ( debug )

      end
