      subroutine mshobstintlin1 ( x1, x2, xmin, ymin, nx, ny, iobgrid, &
                                  icell, ncell, dx, dy, nnodes, &
                                  maxcells, iobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-10-2012 Extra parameters
!        version  1.0  date 07-03-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array icell with indices of curve parts of the obstacle that is
!     possibily intersected by the line x1-x2
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     line
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), icell(*), ncell, nnodes, &
              maxcells,  iobgrid1(*)
      double precision x1(2), x2(2), xmin, ymin, dx, dy

!     dx             i    length of vector in x-direction
!     dy             i    length of vector in y-direction
!     icell          o    Array containing the curve parts that must be
!                         considered for computing the cross sections
!     iobgrid        i    array of length nx x ny that is filled as follows:
!                         The region covered is defined by xmin, xmax, ymin,
!                         ymax
!                         The cell size is dx x dy
!                         Cell i,j refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy) x (xmin+idx,ymin+jdy)
!                         iobgrid(i,j) = 0: cell outside obstacle
!                         iobgrid(i,j) = k>0: boundary part k crosses cell
!                         iobgrid(i,j) = -1: cell completely in obstacle
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     maxcells       i    Declared length of array icell
!     ncell          o    Number of curve parts that must be used to compute
!                         cross sections
!     nnodes         i    Number of nodes
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     x1             i    x-coordinate of first point
!     x2             i    x-coordinate of last point
!     xmin           i    Minimum x-value
!     ymin           i    Minimum y-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipcell, jpcell, ip1, ip2, jp1, jp2, istart
      double precision x, y, alpha1, beta1

!     alpha1         multiplication factor
!     beta1          multiplication factor
!     ip1            i-index of cell(i,j)
!     ip2            i-index of cell(i,j)
!     ipcell         i-index of cell(i,j)
!     istart         Starting address
!     jp1            j-index of cell(i,j)
!     jp2            j-index of cell(i,j)
!     jpcell         j-index of cell(i,j)
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFILLCELL    Fill array icell for one grid cell
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstintlin1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstintlin1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ncell = 0

!     --- For both points x1 and x2 do

      x = x1(1)
      y = x1(2)
      alpha1 = (x-xmin)/dx
      beta1  = (y-ymin)/dy
      ip1 = int(alpha1)
      jp1 = int(beta1)
      x = x2(1)
      y = x2(2)
      alpha1 = (x-xmin)/dx
      beta1  = (y-ymin)/dy
      ip2 = int(alpha1)
      jp2 = int(beta1)
      if ( ip1<0 .and. ip2<0 .or. &
           ip1>=nx .and. ip2>=nx .or. &
           jp1<0 .and. jp2<0 .or. &
           jp1>=ny .and. jp2>=ny ) then

!     --- No intersection possible

         ip1 = -1
         ip2 = -2
         jp1 = -1
         jp2 = -2

      else

!     --- intersection possible, set ip1 etc

         ip1 = min(max(ip1,0),nx-1)
         ip2 = min(max(ip2,0),nx-1)
         jp1 = min(max(jp1,0),ny-1)
         jp2 = min(max(jp2,0),ny-1)
         if ( ip1>ip2 ) then
            ipcell = ip1
            ip1 = ip2
            ip2 = ipcell
         end if
         if ( jp1>jp2 ) then
            jpcell = jp1
            jp1 = jp2
            jp2 = jpcell
         end if

      end if
      do ipcell = ip1, ip2

         do jpcell = jp1, jp2

!        --- ind := the corresponding part of the obstacle curve

            istart = iobgrid(ipcell,jpcell)
            if ( istart>0 ) then

               if ( debug ) &
                  write(irefwr,1) 'ipcell, jpcell, istart, iobgrid1(istart)', &
                                   ipcell, jpcell, istart, iobgrid1(istart)
               call mshfillcell ( istart, iobgrid1, ncell, maxcells, &
                                  icell, nnodes )
               if ( ierror/=0 ) go to 1000

            end if  ! ( istart>0 )

         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstintlin1'

      end if  ! ( debug )
      call erclos ( 'mshobstintlin1' )

      end
