      subroutine mshp04 ( curv1, curv2, curv3, curv4, coor, &
                          linear, m, n, npoint, interpol )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.0  date 20-09-2010 Remove work arrays
!        version  5.0  date 24-10-2003 Extension with interpolation
!                                      Extra parameter interpol
!        version  4.2  date 28-12-1999 Complete change in case of 3
!                                      generating curves
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill co-ordinates in array coor for pipe surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), curv1(3,*), curv2(3,*), curv3(3,*), &
                       curv4(3,*)
      integer linear, n, m, npoint, interpol

!     coor      i/o   array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     curv1      i    contains the coordinates of the first generating curve
!     curv2      i    contains the coordinates of the second generating curve
!     curv3      i    contains the coordinates of the third generating curve
!     curv4      i    contains the coordinates of the fourth generating curve
!     interpol   i    Defines the type of interpolation
!                     0: default
!                     1: hor_straight
!                     2: vert_straight
!     linear     i    Indicates whether the element are linear (1)
!                     or quadratic (2)
!     m          i    Number of elements in "horizontal" direction
!     n          i    Number of elements in "vertical" direction
!     npoint     o    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n2, m2, k, i, j, ih, ikn
      double precision amax, ds, x1, y1, z1, x3, y3, &
                       z3, x5, y5, z5, x7, y7, z7, s1, s2, t1, t2, s, &
                       t, x2, y2, z2, x6, y6, z6, x8, y8, z8, x4, y4, &
                       z4, coef1, coef2, coef3, coef4, coef5, coef6, &
                       coef7, coef8, x, y, z, leng3, tau1(m*linear+1), &
                       tau2(m*linear+1), tau3(n*linear+1), &
                       tau4(n*linear+1)

!     amax
!     coef1
!     coef2
!     coef3
!     coef4
!     coef5
!     coef6
!     coef7
!     coef8
!     ds         Minimum distance between two points along a curve
!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     ikn
!     j          Counting variable
!     k          Counting variable
!     leng3      Distance between two extreme points on third generating curve
!                generating curve compared to leng3
!     m2
!     n2
!     s
!     s1
!     s2
!     t          actual time
!     t1         First component to compute the tangential vector
!     t2         Second component to compute the tangential vector
!     tau1       Work array of length m*linear+1 in which the relative
!                arc length of the first generating curve is stored
!     tau2       Work array of length m*linear+1 in which the relative
!                arc length of the second generating curve is stored
!     tau3       Work array of length n*linear+1 in which the relative
!                arc length of the third generating curve is stored
!     tau4       Work array of length n*linear+1 in which the relative
!                arc length of the fourth generating curve is stored
!     x          x co-ordinate of point
!     x1         x-coordinate of first point in curve
!     x2         x-coordinate of last point in curve
!     x3         Third component of co-ordinate vector
!     x4
!     x5
!     x6
!     x7
!     x8
!     y          y-coordinate
!     y1         y-coordinate of first point in curve
!     y2         y-coordinate of last point in curve
!     y3
!     y4
!     y5
!     y6
!     y7
!     y8
!     z          z co-ordinate of point
!     z1         z-coordinate of first point in curve
!     z2         z-coordinate of last point in curve
!     z3
!     z4
!     z5
!     z6
!     z7
!     z8
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHO51    Map boundary points on 2-d curve with length one
!     MSHO54
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set constants

      n2 = linear*n
      m2 = linear*m+1

!     --- Check which situation has to be handled

      ds = 0d0
      amax = 0d0

      do i = 1, n2 + 1
         do k = 1, 3
            amax = max( amax, abs(curv3(k,i)), abs(curv4(k,i)) )
            ds   =  ds + abs( curv3(k,i) - curv4(k,i) )
         end do
      end do

      if ( ds<sqreps * amax ) then

!     --- There are only three generating curves
!         First the first row of co-ordinates

         do i = 1, m2
            do k = 1, 3
               coor(k,i)=curv1(k,i)
            end do
         end do

!        --- Next n-1 rows
!            Compute distance between extreme points on third generating curve

         leng3 = sqrt((curv2(1,1)-curv1(1,1))**2+ &
                      (curv2(2,1)-curv1(2,1))**2+ &
                      (curv2(3,1)-curv1(3,1))**2)

!        --- For all elements in vertical direction

         do i = 2, n2

!        --- Compute step along third generating curve and multiply by
!            relative length

            ds = sqrt((curv3(1,i)-curv3(1,1))**2+ &
                      (curv3(2,i)-curv3(2,1))**2+ &
                      (curv3(3,i)-curv3(3,1))**2)/leng3

!           --- For all points in horizontal direction

            ih = (i-1)*m2
            do k = 1, m2

               coor(1,ih+k) = (1d0-ds)*curv1(1,k) + ds*curv2(1,k)
               coor(2,ih+k) = (1d0-ds)*curv1(2,k) + ds*curv2(2,k)
               coor(3,ih+k) = (1d0-ds)*curv1(3,k) + ds*curv2(3,k)

            end do

         end do

!        --- Last row

         ih = n2*m2
         do k = 1, m2
            coor(1,ih+k)=curv2(1,k)
            coor(2,ih+k)=curv2(2,k)
            coor(3,ih+k)=curv2(3,k)
         end do
         npoint=m2*(n2+1)

      else

!     --- Four generating curves, use Coons formulas, &
!         start filling the local tau arrays

         npoint = m2 * ( n2 + 1 )

         call msho51(curv1,m2,tau1)

         call msho51(curv2,m2,tau2)

         call msho51(curv3,n2+1,tau3)

         call msho51(curv4,n2+1,tau4)

!        --- Start to copy first curve in array coor

         do i = 1, m2
            coor(1,i) = curv1(1,i)
            coor(2,i) = curv1(2,i)
            coor(3,i) = curv1(3,i)
         end do

!        --- Essential corner points

         x1 = curv1(1,1)
         y1 = curv1(2,1)
         z1 = curv1(3,1)

         x3 = curv4(1,1)
         y3 = curv4(2,1)
         z3 = curv4(3,1)

         x5 = curv2(1,m2)
         y5 = curv2(2,m2)
         z5 = curv2(3,m2)

         x7 = curv2(1,1)
         y7 = curv2(2,1)
         z7 = curv2(3,1)
         ikn = m2

!        --- Run through all lines sequentially and place coordinates

         do i = 2, n2

            ikn = ( i-1 ) * m2 + 1

            coor(1,ikn) = curv3(1,i)
            coor(2,ikn) = curv3(2,i)
            coor(3,ikn) = curv3(3,i)

            if ( interpol==0 ) then

!           --- Standard Coons interpolation

               t1 = tau3(i)
               t2 = tau4(i)
               do j = 2, m2 - 1

                  s1 = tau1(j)
                  s2 = tau2(j)

                  s = (s1 + s2*t1 - s1*t1)/(1d0 - (s2-s1)*(t2-t1))
                  t = (t1 + t2*s1 - t1*s1)/(1d0 - (s2-s1)*(t2-t1))

                  call msho54(curv1,tau1,s,x2,y2,z2,m2)
                  call msho54(curv2,tau2,s,x6,y6,z6,m2)
                  call msho54(curv3,tau3,t,x8,y8,z8,n2+1)
                  call msho54(curv4,tau4,t,x4,y4,z4,n2+1)

                  coef1 = 1d0 - s
                  coef2 = s
                  coef3 = 1d0 - t
                  coef4 = t
                  coef5 = coef1 * coef3
                  coef6 = coef1 * coef4
                  coef7 = coef2 * coef3
                  coef8 = coef2 * coef4

                  x = coef1*x8 + coef2*x4 + coef3*x2 + coef4*x6 &
                    -(coef5*x1 + coef6*x7 + coef7*x3 + coef8*x5)
                  y = coef1*y8 + coef2*y4 + coef3*y2 + coef4*y6 &
                    -(coef5*y1 + coef6*y7 + coef7*y3 + coef8*y5)
                  z = coef1*z8 + coef2*z4 + coef3*z2 + coef4*z6 &
                    -(coef5*z1 + coef6*z7 + coef7*z3 + coef8*z5)

                  ikn = ikn + 1

                  coor(1,ikn) = x
                  coor(2,ikn) = y
                  coor(3,ikn) = z

               end do  ! j = 2, m2 - 1

            else if ( interpol==1 ) then

!           --- Interpolation straight in horizontal direction

               t1 = tau3(i)
               t2 = tau4(i)

               do j = 2, m2 - 1

                  s1 = tau1(j)
                  s2 = tau2(j)

                  t = (t1 + t2*s1 - t1*s1)/(1d0 - (s2-s1)*(t2-t1))

                  coef3 = 1d0 - t
                  coef4 = t

                  x2 = curv1(1,j)
                  y2 = curv1(2,j)
                  z2 = curv1(3,j)
                  x6 = curv2(1,j)
                  y6 = curv2(2,j)
                  z6 = curv2(3,j)

                  x = coef3*x2 + coef4*x6
                  y = coef3*y2 + coef4*y6
                  z = coef3*z2 + coef4*z6

                  ikn = ikn + 1

                  coor(1,ikn) = x
                  coor(2,ikn) = y
                  coor(3,ikn) = z

               end do  ! j = 2, m2 - 1

            else  ! ( interpol==2 )

!           --- Interpolation straight in vertical direction

               x8 = curv3(1,i)
               y8 = curv3(2,i)
               z8 = curv3(3,i)
               x4 = curv4(1,i)
               y4 = curv4(2,i)
               z4 = curv4(3,i)
               t1 = tau3(i)
               t2 = tau4(i)

               do j = 2, m2 - 1

                  s1 = tau1(j)
                  s2 = tau2(j)

                  s = (s1 + s2*t1 - s1*t1)/(1d0 - (s2-s1)*(t2-t1))

                  coef1 = 1d0 - s
                  coef2 = s

                  x = coef1*x8 + coef2*x4
                  y = coef1*y8 + coef2*y4
                  z = coef1*z8 + coef2*z4

                  ikn = ikn + 1

                  coor(1,ikn) = x
                  coor(2,ikn) = y
                  coor(3,ikn) = z

               end do  ! j = 2, m2 - 1

            end if  ! ( interpol==0 )
            ikn = ikn + 1

            coor(1,ikn) = curv4(1,i)
            coor(2,ikn) = curv4(2,i)
            coor(3,ikn) = curv4(3,i)

         end do  ! i = 2, n2

!        --- Finally copy curv2

         do i = 1, m2

            ikn = ikn + 1

            coor(1,ikn) = curv2(1,i)
            coor(2,ikn) = curv2(2,i)
            coor(3,ikn) = curv2(3,i)

         end do

      end if

      end
