      subroutine mshr01 ( icurvs, ncurvs, isurf, isurnr, maxsrf, &
                          nuspnt, lenipoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 30-11-2014 Extra parameter lenipoint
!        version  2.4  date 01-07-2008 Extension with internal curves
!        version  2.3  date 10-03-2008 Extra debug
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret information in surface record after the part
!     Si =  rectangle j ( ...    etc.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     read
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, lenipoint, icurvs(lenipoint), isurf, isurnr, &
              maxsrf, nuspnt

!     icurvs         i    Array indicating if curve i has been filled
!                         (icurvs(i)>0) or not (icurvs(i)=0)
!     isurf          i    Type of surface to be treated.
!                         See subroutine mshrd2: clcode
!     isurnr         i    Surface number
!     lenipoint      i    Length of array ipoint
!     maxsrf         i    Maximal number of surfaces allowed
!     ncurvs         i    Number of curves in mesh
!     nuspnt         i    Number of user points
!     work           o    Array containing the output of this subroutine
!                         work is filled as follows
!                            1:    ncur:  Number of curves in surface
!                            2:    n
!                            3:    m
!                            4:    nelem
!                            5:    npoint
!                            6:    iblend
!                            7:    icurva
!                            8:    umin
!                            9:    umax
!                           10:    vmin
!                           11:    vmax
!                           12:    ismooth
!                           13:    diagonal_dir
!                           14:    mapping
!                           15:    centre
!                           16:    type_surface
!                           17:    number of subsurfaces
!                           18:    iregular
!                           19:    interpolation
!                           20:    max_length
!                           21:    - &
!                           22:    nintercurves: Number of internal curves
!                           23:    ninterpoints: Number of internal points
!                           24:    number of layers
!                           25-24+n:  Curve numbers (At most 975)
!                         1001-1000+nsubsurfaces:  Surface numbers (At most 500)
!                                  or in case of internal points
!                         1001-1000+ninterpoints:  User point sequence numbers
!                                  (At most 250)
!                         1251-1250+nintercurves:  Extra curve sequence numbers
!                                  (At most 250)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=27, lencodemn=12
      character (len=15) code2(nncode2)
      character (len=17) codemn2(lencodemn)
      integer ncur, icurnr, nsur, numpoints, numcurves, itype2(nncode2), &
              ncode2(nncode2), ncodmx2(nncode2), nmcode2(lencodemn), i
      double precision :: tabmn2(nncode2), tabmx2(nncode2)

!     code2          Table containing the keywords
!     codemn2        Table with subkeywords
!     i              Counting variable
!     icurnr         Curve sequence number
!     itype2         Indicates the type of action corresponding to keywords
!                    in code2
!     lencodemn      Length of array codemn2
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2         Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncur           Number of curves read
!     nmcode2        Table containing the number of significant characters
!                    in the subkeyword table codemn2
!     nncode2        Number of entries in code2
!     nsur           Number of surfaces read
!     numcurves      Number of curves
!     numpoints      Number of points
!     tabmn2         Table containing the minimum values of the quantities
!                    given in code2
!     tabmx2         Table containing the maximum values of the quantities
!                    given in code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHR01PAV      Fill information about curves in array work
!                    Special case for paver
!     MSHR01RECT     Fill n, m and curve numbers for rectangle in case no
!                    keywords are given
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following combinations are recognized:
!
!     curvature = c
!     centre = Pi
!     nelem = n
!     npoint = n
!     blend = b
!     n = i
!     mapping = default/sphere
!     umin = u
!     umax = u
!     vmin = u
!     vmax = u
!     smooth = i
!     max_length = m
!     layers = l
!     type = sphere/upper_half_sphere/lower_half_sphere
!     f = c
!     diagonal_dir = i
!     subdivision = default/regular
!     interpolation = default/hor_straight/vert_straight
!     m = i
!     c or number
!     t = c
!     subsurfaces = (s1,s2)
!     internal_points = (pi, pj, .. )
!     internal_curves = (ci, cj, .. )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1219   Curve number has not been defined and may therefore not be used
!    2227   Only one of a combination of keywords found
!    2753   Number of user points found after keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'CURVATURE', 'CENTRE', 'NELEM', 'NPOINT', 'BLEND', &
                  'N', 'MAPPING', 'UMIN', 'UMAX', 'VMIN', 'VMAX', &
                  'SMOOTH', 'TYPE', 'F', 'DIAGONAL_DIR', 'MAX_LENGTH', &
                  'C', 'T', 'SUBSURFACES', 'SUBDIVISION', 'S', &
                  'M', 'INTERPOLATION', 'LAYERS', 'INTERNAL_POINTS', &
                  'P', 'INTERNAL_CURVES' /
      data ncode2 / 2, 3, 2, 2, 2,   1, 3, 3, 3, 3, &
                   3, 3, 3, 1, 3,   7, 1, 1, 6, 6, &
                   1, 1, 8, 4,11,   1,11 /
      data ncodmx2/ 9, 6, 5, 6, 5,   1, 7, 4, 4, 4, &
                   4, 6, 5, 1,12,  10, 1, 1,11,11, &
                   1, 1,13, 6,15,   1, 15 /
      data itype2 / 0, 2, 0, 0, 0,   0, 5, 1, 1, 1, &
                   1, 0, 5, 8, 0,   1, 9, 8, 2, 5, &
                  14, 0, 5, 0, 2,  14,18 /
      data tabmn2 / 0d0, 0d0, 1d0, 1d0, 1d0,    1d0, 0d0,-1d5,-1d5,-1d5, &
                   -1d5, 0d0, 0d0,-1d3, 1d0, 1d-10,975d0, 0d0, 0d0, 0d0, &
                    5d2, 1d0, 0d0, 1d0, 5d2,    0d0, 5d2 /
      data tabmx2 / 2d0, 0d0, 1d7, 1d7, 3d0,    1d5, 0d0, 1d5, 1d5, 1d5, &
                    1d5,15d0, 0d0, 1d3, 2d0,  1d10,  1d3, 1d3, 0d0, 0d0, &
                    1d5, 1d5, 0d0, 1d4, 0d0,    0d0, 0d0 /
      data codemn2 / 'DEFAULT', 'SPHERE', 'OLD_SPHERE', 'SPHERE', &
                    'UPPER_HALF_SPHERE', 'LOWER_HALF_SPHERE', 'DEFAULT', &
                    'REGULAR', 'DEFAULT', 'HOR_STRAIGHT', &
                    'VERT_STRAIGHT', 'C' /
      data nmcode2 / 4, 10, 4, 4, 5,   5, 4, 4, 4, 3, &
                    4, 1 /

!     code2      Table containing the various codes to be recognized, &
!                1:  curvature
!                2:  centre
!                3:  nelem
!                4:  npoint
!                5:  blend
!                6:  n
!                7:  mapping
!                8:  umin
!                9:  umax
!               10:  vmin
!               11:  vmax
!               12:  smooth
!               13:  type
!               14:  f
!               15:  diagonal_dir
!               16:  max_length
!               17:  c
!               18:  t
!               19:  subsurfaces
!               20:  subdivision
!               21:  s
!               22:  m
!               23:  interpolation
!               24:  layers
!               25:  internal_points
!               26:  p
!               27:  internal_curves
!     codemn2    Table containing the subkeywords
!                1:  DEFAULT
!                2:  SPHERE_OLD
!                3:  SPHERE
!                4:  SPHERE
!                5:  UPPER_HALF_SPHERE
!                6:  LOWER_HALF_SPHERE
!                7:  DEFAULT
!                8:  REGULAR
!                9:  DEFAULT
!               10:  HOR_STRAIGHT
!               11:  VERT_STRAIGHT
!               12:  C
! ======================================================================
!
      call eropen ( 'mshr01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill defaults

      work(1:nncode2) = 0d0
      table(1:nncode2) = 0d0

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )
      call sepcopycodesub ( codemn2, lencodemn, nmcode2 )

!     --- Set position in tabmin and tabmax with respect to subkeywords

      tabmin(7) = 1d0  ! starting address of mapping in codemn
      tabmax(7) = tabmin(7)+2d0    ! end address of mapping in codemn
      tabmin(13) = tabmax(7)+1d0   ! starting address of type in codemn
      tabmax(13) = tabmin(13)+2d0  ! end address of type in codemn
      tabmin(20) = tabmax(13)+1d0  ! starting address of subdivision in codemn
      tabmax(20) = tabmin(20)+1d0  ! end address of subdivision in codemn
      tabmin(23) = tabmax(20)+1d0  ! starting address of interpolation in codemn
      tabmax(23) = tabmin(23)+2d0  ! end address of interpolation in codemn
      tabmax(14) = 1d3     ! maximum value f
      if ( isurf/=23 ) tabmax(17) = ncurvs  ! maximum curve number
      tabmax(18) = 1d3     ! maximum value t
      tabmax(21) = maxsrf  ! maximum surface number
      tabmin(26) = nuspnt  ! number of user points
      tabmax(26) = nuspnt  ! maximum user point number
      table( 5) = 2d0  ! iblend
      table( 9) = 1d0  ! umax
      table(11) = 1d0  ! vmax
      table(13) = 1d0  ! type of surface
      table(23) = 1d0  ! interpol
      table(27) = -12d0  ! refers to C in codemn
      tabmin(27) = ncurvs  ! number of curves
      tabmax(27) = ncurvs  ! maximum curve number
      ispecl = 4       ! mark appearance of minus sign
      nmain = 0
      iretur = -10
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
         call prinrl ( tabout, 10, 'tabout' )
         call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )

      end if  ! ( debug )

!     --- Interpret the information read

      work( 7) = table(1)     ! curvature
      work( 4) = table(3)     ! nelem
      work( 5) = table(4)     ! npoint
      work( 6) = table(5)     ! iblend
      work( 2) = table(6)     ! n
      if ( table(7)>0d0 ) work(14) = table(7)-1d0 ! mapping
      work( 8) = table(8)     ! umin
      work( 9) = table(9)     ! umax
      work(10) = table(10)    ! vmin
      work(11) = table(11)    ! vmax
      work(12) = table(12)    ! smooth
      work(16) = table(13)    ! type of surface
      work(13) = table(15)    ! diagonal_dir
      work(20) = table(16)    ! max_length
      if ( table(20)>0d0 ) work(18) = table(20)-1d0    ! subdivision
      work( 3) = table(22)    ! m
      work(19) = table(23)-1d0! interpol
      work(24) = table(24)    ! nlayers
      if ( table(2)>0d0 ) then

!     --- centre found, pi required

         iposut = nint(table(26))  ! starting address of user points in tabout
         numpoints = nint(tabmax(26)) ! number of points read
         if ( numpoints<1 .or. numpoints>1 ) then

!        --- Number of user points found is not 1

            call errint ( numpoints, 1 )
            call errint ( 1, 2 )
            call errint ( 1, 3 )
            call errchr ( code(2)(1:ncodmx(2)), 1 )
            call errsub ( 2753, 3, 0, 1 )

         else

!        --- Store user point number

            work(15) = tabout(iposut)

         end if  ! ( numpoints<1 .or. numpoints>1 )

      end if  ! ( table(2)>0d0 )

!     --- Fill curve numbers depending on type of surface

      ncur = 0
      if ( table(17)>0d0 ) then

!     --- table(17)>0, curve numbers found

         iposut = nint(table(17))
         ncur = nint(tabmax(17))
         if ( isurf==19 ) then

!        --- surface is of type paver, we need 5 positions per curve
!            furthermore f and t may have been read

            call mshr01pav ( nseqkeyw, iseqkeyw, table, tabmax, tabout, &
                             work, isurnr, icurvs )

         else

!        --- surface is not of type paver, we need 1 position per curve
!            copy curve numbers from tabout into work from position 25

            do i = 0, ncur-1

               icurnr = nint(tabout(iposut+i))
               if ( isurf/=23 .and. icurvs(abs(icurnr))==0 ) then

!              --- Error 1219:  Curve has not been filled

                  call errint ( icurnr, 1 )
                  call errint ( isurnr, 2 )
                  call errsub ( 1219, 2, 0, 0 )
                  call errcrd

               end if  ! ( icurvs(abs(icurnr))==0 )
               work(25+i) = tabout(iposut+i)
            end do  ! i = 1, ncur

         end if  ! ( isurf==19 )

      else if ( isurf/=1 ) then

!     --- No curve numbers read, check if there are numbers read without
!         precedence of c
!         The case of rectangle is treated separately

         do i = 1, nseqkeyw
            if ( iseqkeyw(i)<0 ) then

!           --- Number found, store

              ncur = ncur+1
              work(24+ncur) = tabout(-iseqkeyw(i))

            end if  ! ( iseqkeyw(i)<0 )
         end do  ! i = 1, nseqkeyw

      end if  ! ( table(17)>0d0 )

      nsur = 0
      if ( table(19)>0d0 ) then

!     --- subsurfaces found, store surface numbers

         iposut = nint(table(21))
         nsur = nint(tabmax(21))
         work(17) = nsur
         do i = 0, nsur-1
            work(1001+i) = tabout(iposut+i)
         end do  ! i = 1, nsur
         if ( nsur==0 ) then

!        --- keyword  subsurface found, but nosurfaces

            call errchr ( code(21)(1:ncodmx(21)), 1 )
            call errchr ( code(19)(1:ncodmx(19)), 1 )
            call errsub ( 2227, 0, 0, 2 )

         end if  ! ( nsur==0 )

      else if ( table(21)>0d0 ) then

!     --- surface numbers found, but not the keyword subsurfaces

         call errchr ( code(21)(1:ncodmx(21)), 1 )
         call errchr ( code(19)(1:ncodmx(19)), 1 )
         call errsub ( 2227, 0, 0, 2 )

      end if  ! ( table(19)>0d0 ) then

      numpoints = 0
      if ( table(26)>0d0 ) then

!     --- internal_points found, store user points

         iposut = nint(table(26))
         numpoints = nint(tabmax(26))
         do i = 0, numpoints-1
            work(1001+i) = tabout(iposut+i)
         end do  ! i = 1, numpoints
         work(23) = numpoints
         if ( numpoints<1 .or. numpoints>250 ) then

!        --- Number of user points found<1

            call errint ( numpoints, 1 )
            call errint ( 1, 2 )
            call errint ( 250, 3 )
            call errchr ( code(25)(1:ncodmx(25)), 1 )
            call errsub ( 2753, 3, 0, 1 )

         end if  ! ( numpoints<1 .or. numpoints>250 )

      end if  ! ( table(26)>0d0 )

      numcurves = 0
      if ( table(27)>0d0 ) then

!     --- curves found, store curve numbers

         iposut = nint(table(27))
         numcurves = nint(tabmax(27))
         do i = 0, numcurves-1
            work(1251+i) = tabout(iposut+i)
         end do  ! i = 1, numpoints
         work(22) = numcurves
         if ( numcurves<1 .or. numcurves>250 ) then

!        --- Number of user points found<1

            call errint ( numcurves, 1 )
            call errint ( 1, 2 )
            call errint ( 250, 3 )
            call errchr ( code(27)(1:ncodmx(27)), 1 )
            call errsub ( 2753, 3, 0, 1 )

         end if  ! ( numcurves<1 .or. numcurves>250 )

      end if  ! ( table(25)>0d0 )

      if ( isurf==1 ) then

!     --- Special case of rectangle

         call mshr01rect ( work, iseqkeyw, nseqkeyw, ncur, tabout )

      end if  ! ( isurf==1)
      work(1) = ncur
      ispecl = 0

1000  if ( ierror==0 .and. debug ) then

!     --- Debug information

         call prinrl ( work, 24, 'work' )
         if ( ncur>0 ) call prinrl ( work(25), ncur, 'curves' )
         if ( nsur>0 ) call prinrl ( work(1001), nsur, 'surfaces' )
         if ( numpoints>0 ) call prinrl ( work(1001), numpoints, &
            'points' )
         write(irefwr,*) 'End mshr01'

      end if
      call erclos ( 'mshr01' )

      end subroutine mshr01
