      subroutine mshr02 ( isurfs, nsurfs, ivol, ivolnr, nuspnt, ncurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 16-07-2008 Extra parameter
!        version  2.0  date 04-05-2005 Complete revision
!        version  1.4  date 20-11-2002 Increase length strings
!
!   copyright (c) 1990-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret information in volume record after the part
!     Vi =  rectangle j ( ...    etc.
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     mesh_generation
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs, isurfs(nsurfs), ivol, ivolnr, nuspnt, ncurvs

!     isurfs         i    Array indicating if surface i has been filled
!                         (isurfs(i)>0) or not (isurfs(i)=0)
!     ivol           i    Type of volume to be treated. Possibilities:
!                          1:   brick
!                          2:   user
!                          3:   input
!                          4:   pipe
!                          5:   general
!                          6:   translate
!                          7:   rotate
!                          8:   reflect
!                          9:   channel
!                         10:   general with internal points
!     ivolnr         i    Volume sequence number
!     ncurvs         i    Number of curves in mesh
!     nsurfs         i    number of surfaces
!     nuspnt         i    Number of user points
!     work           o    Array containing the output of this subroutine
!                            work is filled as follows
!                            1:    nsur:  Number of surfes in volume
!                            2:    n
!                            3:    m
!                            4:    l
!                            5:    nelem
!                            6:    npoint
!                            7:    iorien
!                            8:    ninterpoints: Number of internal points
!                            9:    nintercurves: Number of internal curves
!                           10:    nintersurfs: Number of internal surfaces
!                           11-24: Not yet used
!                           25-24+n:  Surface numbers (At most 975)
!                         1001-1000+ninterpoints:  User point sequence numbers
!                                  (At most 100)
!                         1101-1100+nintercurves:  Curve sequence numbers
!                                  (At most 100)
!                         1201-1200+nintersurfs:  Surface sequence numbers
!                                  (At most 300)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=12
      character (len=17) code2(nncode2)
      integer nsur, iorien, i, j, numpoints, isurnr, &
              ncode2(nncode2), ncodmx2(nncode2), numcurvs, numsurfs, &
              itype2(nncode2)
      double precision :: tabmn2(nncode2), tabmx2(nncode2)

!     code2          Table with keywords
!     i              Counting variable
!     iorien         Defines orientation of the base vectors in 3D plots
!                    1 x-y-z
!                    2 z-x-y
!                    3 y-z-x
!     isurnr         Surface sequence number
!     itype2          Table containing information about the numbers behind
!                    the keywords in the table code2
!     j              Counting variable
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     nsur           Number of surfaces read
!     numcurvs       Number of internal curves
!     numpoints      Number of internal points
!     numsurfs       number of internal surfaces
!     tabmn2         Contains minimum values allowed in table
!     tabmx2         Contains maximum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHR02BRICK    Fill n, m, l and surface numbers for brick in case no
!                    keywords are given
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       The following texts are recognized:
!       Si   or number
!       n =
!       m =
!       l =
!       nelem =
!       npoint =
!       orientation =
!       internal_points = (pi, pj, .. )
!       internal_curves = (ci, cj, .. )
!       internal_surfaces = (si, sj, .. )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1020   Digit in orientation incorrect
!    1220   Surface number is not defined
!    1488   Number of curves found after keyword is incorrect
!    2753   Number of user points found after keyword is incorrect
!    2970   Number of surfaces found after keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'S', 'NELEM', 'NPOINT', 'L', 'N', 'M', &
                  'ORIENTATION', 'INTERNAL_POINTS', 'P', &
                  'INTERNAL_CURVES', 'INTERNAL_SURFACES', 'C' /
      data ncode2 / 1, 2, 2, 1, 1,   1, 2,11, 1, 11, &
                  11, 1 /
      data ncodmx2/ 1, 5, 6, 1, 1,   1,11,15, 1, 15, &
                  17, 1 /
      data itype2 / 9, 0, 0, 0, 0,   0, 0, 2,14,  2, &
                  18,14  /
      data tabmn2 / 1d0, 1d0, 1d0, 1d0, 1d0,  1d0, 1d0, 5d2, 0d0, 0d0, &
                    0d0, 0d0  /
      data tabmx2 / 0d0, 1d7, 1d7, 1d5, 1d5,  1d5, 888888d0, 0d0, 0d0, &
                    0d0, 0d0, 0d0 /

!     code2     Table containing the various codes to be recognized, &
!                1:  s
!                2:  nelem
!                3:  npoint
!                4:  l
!                5:  n
!                6:  m
!                7:  orientation
!                8:  internal_points
!                9:  p
!               10:  internal_curves
!               11:  internal_surfaces
!               12:  c
! ======================================================================
!
      call eropen ( 'mshr02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr02'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill defaults

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )
      codesub(1) = code(1)
      ncodesub(1) = ncode(1)

      work(1:nncode) = 0d0
      table(1:nncode) = 0d0

!     --- Set position in tabmin and tabmax with respect to subkeywords

      tabmax(1) = nsurfs    ! maximum surface number
      tabmin(9) = nuspnt    ! number of user points
      tabmax(9) = nuspnt    ! maximum user point number
      tabmin(11) = nsurfs   ! number of surface number
      tabmax(11) = nsurfs   ! maximum surface number
      tabmin(12) = ncurvs   ! number of curves
      tabmax(12) = ncurvs   ! maximum curve number
      table(11)  =-1d0      ! minus start address of S in codemn
                            ! since S is also the first keyword in code we
                            ! use code as codemn

      ispecl = 4       ! mark appearance of minus sign
      nmain = 0        ! number of keywords in codemn (which is code now)
      iretur = -10
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
         call prinrl ( tabout, 10, 'tabout' )
         call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )

      end if  ! ( debug )

!     --- Interpret the information read

      iposut = nint(table(1)) ! Position of first surface
      nsur = nint(tabmax(1))  ! number of surfaces
      work(1) = tabmax(1)    ! nsur
      work(2) = table(5)     ! n
      work(3) = table(6)     ! m
      work(4) = table(4)     ! l
      work(5) = table(2)     ! nelem
      work(6) = table(3)     ! npoint
      work(7) = table(7)     ! iorien

!     --- copy surface numbers from tabout into work from position 25

      if ( nsur>0 ) then

!     --- nsur>0, surface numbers found

         do i = 0, nsur-1

            isurnr = nint(tabout(iposut+i))
            if ( isurfs(abs(isurnr))==0 ) then

!           --- Error 1220:  Surface has not been filled

               call errint ( isurnr, 1 )
               call errint ( ivolnr, 2 )
               call errsub ( 1220, 2, 0, 0 )
               call errcrd

            end if  ! ( isurfs(abs(isurnr))==0 )
            work(25+i) = tabout(iposut+i)

         end do  ! 0, nsur-1

      else if ( ivol/=1 ) then

!     --- No surfacee numbers read, check if there are numbers read without
!         precedence of s
!         The case of brick is treated separately

         do i = 1, nseqkeyw
            if ( iseqkeyw(i)<0 ) then

!           --- Number found, store

              nsur = nsur+1
              work(24+nsur) = tabout(-iseqkeyw(i))

            end if  ! ( iseqkeyw(i)<0 )
         end do  ! i = 1, nseqkeyw

      end if  ! ( nsur>0 )

!     --- internal points

      numpoints = 0
      if ( table(8)>0d0 ) then

!     --- internal_points found, store user points

         iposut = nint(table(9))
         numpoints = nint(tabmax(9))
         work(1001:1000+numpoints) = tabout(iposut:iposut+numpoints-1)
         work(8) = numpoints
         if ( numpoints<1 .or. numpoints>100 ) then

!        --- Number of user points found<1 or more than 100

            call errint ( numpoints, 1 )
            call errint ( 1, 2 )
            call errint ( 100, 3 )
            call errchr ( code(8)(1:ncodmx2(8)), 1 )
            call errsub ( 2753, 3, 0, 1 )

         end if  ! ( numpoints<1 .or. numpoints>100 )

      end if  ! ( table(8)>0d0 )

!     --- internal curves

      numcurvs = 0
      if ( table(10)>0d0 ) then

!     --- internal_curves found, store curve numbers

         iposut = nint(table(12))
         numcurvs = nint(tabmax(12))
         work(1101:1100+numcurvs) = tabout(iposut:iposut+numcurvs-1)
         work(9) = numcurvs
         if ( numcurvs<1 .or. numcurvs>100 ) then

!        --- Number of curves found<1 or more than 100

            call errint ( numcurvs, 1 )
            call errint ( 1, 2 )
            call errint ( 100, 3 )
            call errchr ( code(10)(1:ncodmx2(10)), 1 )
            call errsub ( 1488, 3, 0, 1 )

         end if  ! ( numcurvs<1 .or. numcurvs>100 )

      end if  ! ( table(10)>0d0 )

!     --- internal surfaces

      numsurfs = 0
      if ( table(11)>0d0 ) then

!     --- internal_surfaces found, store surface numbers

         iposut = nint(table(11))
         numsurfs = nint(tabmax(11))
         work(1201:1200+numsurfs) = tabout(iposut:iposut+numsurfs-1)
         work(10) = numsurfs
         if ( numsurfs<1 .or. numsurfs>300 ) then

!        --- Number of surfaces found<1 or more than 300

            call errint ( numsurfs, 1 )
            call errint ( 1, 2 )
            call errint ( 300, 3 )
            call errchr ( code(11)(1:ncodmx2(11)), 1 )
            call errsub ( 2970, 3, 0, 1 )

         end if  ! ( numsurfs<1 .or. numsurfs>300 )

      end if  ! ( table(11)>0d0 )

      if ( table(7)>0d0 ) then

!     --- Orientation found, check values

         iorien = nint(table(7))     ! iorien
         do i = 1, 6
            j = mod(iorien,10)
            if ( j<=0 .or. j>8 ) then

!           --- Error 1020:  digit out of range

               call errint ( 7-i, 1 )
               call errint ( j, 2 )
               call errint ( 1, 3 )
               call errint ( 8, 4 )
               call errsub ( 1020, 4, 0, 0 )
               call errcrd

            end if
            iorien = iorien/10
         end do

      end if  ! ( table(7)>0d0 )

      if ( ivol==1 ) then

!     --- Special case of brick

         call mshr02brick ( work, iseqkeyw, nseqkeyw, nsur, tabout )

      end if  ! ( isurf==1)
      work(1) = nsur
      ispecl = 0

1000  if ( ierror==0 .and. debug ) then

!     --- Debug information

         call prinrl ( work, 24, 'work' )
         if ( nsur>0 ) call prinrl ( work(25), nsur, 'surfaces' )
         if ( numpoints>0 ) call prinrl ( work(1001), numpoints, &
            'internal_points' )
         if ( numcurvs>0 ) call prinrl ( work(1101), numcurvs, &
            'internal_curves' )
         if ( numsurfs>0 ) call prinrl ( work(1201), numsurfs, &
            'internal_surfaces' )
         write(irefwr,*) 'End mshr02'

      end if
      call erclos ( 'mshr02' )

      end
