      subroutine mshrd0 ( rinput, ndim, maxpnt, rincor, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.0  date 21-11-2014 New parameter list
!        version 13.0  date 30-07-2012 Remove iwork from parameter list
!        version 12.4  date 19-04-2008 Error message in case of too few coors
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill arrays rinput and rincor with information of the points
!    in the following way:
!      rinput(k,i)  k th co-ordinate of node i
!      rincor(i)  coarseness corresponding to node i
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleconsts
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ndim, maxpnt
      integer, intent(inout) :: iinput(*)
      double precision, intent(out) :: rinput(ndim,maxpnt), &
                                       rincor(maxpnt)

!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     maxpnt         i    Maximum number of nodal points allowed
!     ndim           i    dimension of space
!     rincor         o    coarseness corresponding to node i
!     rinput         o    array to be filled   see description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=6
      integer :: ncodelc(nncodelc), ncodmxlc(nncodelc)
      character (len=8) :: codelc(nncodelc)
      integer :: i, inum, ipoint, itrans, maxnum, ifirst, iwar97, &
                 iwar98, iwork(2,maxpnt), nuspnt, jcoars
      double precision :: r, phi, coarse, work(5)
      logical ::  finished

!     coarse         Local coarseness value
!     codelc         Table with keywords
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Loop variable
!     ifirst         Number of numbers that have been read in record
!     inum           Number of numbers read in card by ASK000
!     ipoint         Point sequence number
!     itrans         Indication of the type of transformation to be applied for
!                    the co-ordinates. Possibilities:
!                       0:  no tranformation
!                    >0:  transformation from polar to cartesian co-ordinates
!                       1: phi in radials
!                       2: phi in degrees
!     iwar97         Number of occurences of warning 1797
!     iwar98         Number of occurences of warning 1798
!     iwork          work array of length 2*maxpnt
!                    iwork(1,i) contains the i-th point where warning 1797
!                               is given
!                    iwork(2,i) contains the i-th point where warning 1798
!                               is given
!     jcoars         Indication if coarseness is used (1) or not (0)
!     maxnum         Maximum number of numbers to be read in a record
!     ncodelc        Table containing the number of significant characters
!                    in the keyword table codelc
!     ncodmxlc       Table containing the maximal number of  characters in
!                    the keyword table codelc!
!     nncodelc         Number of entries in codelc
!     nuspnt         Number of user points
!     phi            Angle phi for the polar transormation
!     r              Radius r for the polar tranformation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     READSR         Read a series of numbers from the input (texts are
!                    neglected)
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!       This subroutine reads records form the input file until the record
!       CURVES has been found
!       The following records are recognised:
!       Pi  = ( x, y, z, c )
!       PDi = ( x, y, z, c )
!       PRi = ( x, y, z, c )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     120   The number of nodal points or the nodal point number read is
!           incorrect
!     411   wrong character in record containing co-ordinates of points
!    1797   Too few input in co-ordinates record
!    1798   Too much input in co-ordinates record
!    2957   Too few coordinates read in reord
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear arrays rinput and rincor
!   finished := false
!   maxnum := ndim+1+jcoars
!   while ( not finished ) do
!      itrans := 0
!      ifirst := 1
!      read next record
!      Determin first item
!      If ( item = 'CURVES' ) then
!         finished := true
!      else if ( item = 'PD' ) then
!         itrans := 0
!      else if ( item = 'PR' ) then
!         itrans := 2
!      else if ( item = number ) then
!         ifirst := 1
!         work(1) := realrd
!      else if ( item # 'P' ) then
!         Error: text not recognised
!      Read maxnum-ifirst numbers
!      Transform numbers if necessary
!      Put numbers in arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc   / 'CURVES', 'SURFACES', 'VOLUMES', 'PD', 'PR', 'P' /
      data ncodelc  /  3, 3, 3, 2, 2,   1 /
      data ncodmxlc /  6, 8, 7, 2, 2,   1 /

!     codelc         Table with keywords
!                    The following keywords are available:
!                     1: CURVES
!                     2: SURFACES
!                     3: VOLUMES
!                     4: PD
!                     5: PR
!                     6: P
! ======================================================================
!
      call eropen ( 'mshrd0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd0'
         write(irefwr,1) 'ndim, maxpnt', ndim, maxpnt
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      jcoars = iinput(11)

      call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )

!     --- clear arrays rinput and rincor

      rinput = 0d0
      rincor = 1d0

!     --- read point cards, until cards with curves is found
!         at most maxpnt points are permitted

      nuspnt = 0
      iwar97 = 0
      iwar98 = 0
      finished = .false.
      maxnum = ndim + 1 + jcoars

      do while ( .not. finished )

!     --- while loop
!         Read new record

         itrans = 0
         ifirst = 0
         ityprd = 1
         call asknxt
         if ( ityprd==4 ) then

            ierpri = 0
            icode = sepsubkeyw ()
            if ( debug ) write(irefwr,1) 'icode', icode

            select case (icode)

            case(1:3)

!           --- CURVES, SURFACES or VOLUMES found, finished

               finished = .true.
               go to 500

            case(4)

!           --- PD found, transformation in degrees

               itrans = 1

            case(5)

!           --- PR found, transformation in radials

               itrans = 2

            case(6)

!           --- P found no action

            case default

!           --- wrong character found error

               call errsub ( 411, 0, 0, 0 )
               call errcrd
               go to 1000

            end select  ! case (icode)

         else if ( ityprd==2 ) then

!        --- integer number found, i.e. point number

            ifirst = 1
            work(1) = realrd

         end if  ! ( ityprd==4 )

!        --- Read maxnum - ifirst numbers

         call readsr ( work(ifirst+1), maxnum-ifirst+1, inum )
         if ( debug ) call prinrl ( work, inum+ifirst, 'work' )

!        --- ipoint is stored in work(1)
!            the coordinates in work(2) . . . work(4)

         ipoint = nint(work(1))
         if ( ipoint<=0 .or. ipoint>maxpnt ) then

!        --- Error: ipoint out of range

            call errint ( ipoint, 1 )
            call errint ( maxpnt, 2 )
            call errsub ( 120, 2, 0, 0 )
            call errcrd
            go to 1000

         end if
         if ( inum<ndim+1-ifirst ) then

!        --- Error too few coordinates read

            call errint ( ipoint, 1 )
            call errint ( inum-1+ifirst, 2 )
            call errint ( ndim, 3 )
            call errsub ( 2957, 3, 0, 0 )
            call errcrd
            go to 1000

         end if  ! ( inum<ndim )

         if ( inum+ifirst-1<ndim+jcoars ) then
            iwar97 = iwar97+1
            iwork(1,iwar97) = ipoint
         else if ( inum+ifirst-1>ndim+jcoars ) then
            iwar98 = iwar98+1
            iwork(2,iwar98) = ipoint
         end if
         nuspnt = max ( ipoint, nuspnt )

!        --- Check if co-ordinates must be transformed

         if ( itrans==1 .or. itrans==2 ) then

!        --- itrans>0, tranformation from polar co-ordinates

            r   = work(2)
            if ( itrans==1 ) then

!           --- itrans = 1, phi in degrees

               phi = work(3)*pi/1.8d2

            else

!           --- itrans = 2, phi in radials

               phi = work(3)

            end if
            work(2) = r * cos(phi)
            work(3) = r * sin(phi)

         end if

         do i = 1, ndim
            rinput(i,ipoint) = work(i+1)
         end do
         if ( jcoars==1 ) then
            coarse = work(ndim+2)
            if ( coarse<=0d0 .and. coarse>=0d0 ) coarse = 1d0
            rincor(ipoint) = coarse
         end if

      end do  ! while ( .not. finished )

500   if ( ndim==3 ) then

!     --- ndim = 3
!         Compute xmin, xmax, ymin, ymax, zmin and zmax for plotting

         xmin = rinfin
         ymin = xmin
         zmin = xmin
         xmax = -xmin
         ymax = xmax
         zmax = xmax
         do i = 1, nuspnt
            xmin = min ( xmin, rinput(1,i) )
            ymin = min ( ymin, rinput(2,i) )
            zmin = min ( zmin, rinput(3,i) )
            xmax = max ( xmax, rinput(1,i) )
            ymax = max ( ymax, rinput(2,i) )
            zmax = max ( zmax, rinput(3,i) )
         end do

      end if
      if ( iwar97>0 ) then

!     --- Warning 1797 apears at least one time

         iwar97 = min(1000,iwar97)
         do i = 1, iwar97
            call errint ( iwork(1,i), i )
         end do
         call errwar ( 1797, iwar97, 0, 0 )

      end if
      if ( iwar98>0 ) then

!     --- Warning 1798 apears at least one time

         iwar98 = min(1000,iwar98)
         do i = 1, iwar98
            call errint ( iwork(2,i), i )
         end do
         call errwar ( 1798, iwar98, 0, 0 )

      end if

!     --- Fill parameters

      iinput(1) = ndim
      iinput(2) = nuspnt

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( rinput, nuspnt, ndim, 'rinput' )
         call prinrl ( rincor, nuspnt, 'rincor' )
         write(irefwr,*) 'End mshrd0'

      end if  ! ( debug )
      call erclos ( 'mshrd0' )

      icode = icode-2  ! necessary to be in correspondence
                           ! with previous choice
      end
