      subroutine mshrd2surnr ( iinp, inpmax,  nsurfs, ksurfs, ipoint, &
                               maxsrf, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-01-2015 New call to readreal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read first part of surface record and detect surface number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: inpmax, maxsrf
      integer,intent(inout) :: iinp, ipoint(maxsrf), isurnr, ksurfs, &
                               nsurfs

!     iinp          i/o   last position used in array iinput
!     inpmax         i    Available space for array iinput
!     ipoint        i/o   pointer array: contains the starting positions of the
!                         surfaces in array iinput
!     isurnr        i/o   Surface sequence number
!     ksurfs        i/o   Actual number of surfaces
!     maxsrf         i    Maximal number of surfaces allowed
!     nsurfs        i/o   number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: kcode
      logical ::  openfound

!     kcode          if <0, a main keyword has been found
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     142   Internal error, too few space for integers
!     144   Surface number has already been read before
!     146   surface number has incorrect value
!     648   string could not be recognized
!    1348   Record does not start with a character
!    1349   Not a text found after Si =.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2surnr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2surnr'

  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kcode = icode

      ityprd = 1   ! end of record
      call asknxt

      if ( iinp>=inpmax ) then

!     --- iinp>=  inpmax

          call errint ( inpmax, 1 )
          call errint ( iinp, 2 )
          call errsub ( 142, 2, 0, 0 )
          go to 1000

      end if
      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( ityprd/=2 .and. ityprd/=4 ) then

!     --- error 1348: Record does not start with a character or integer

         call errsub ( 1348, 0, 0, 0 )
         call errcrd
         go to 1000

      else if ( ityprd==4 .and. &
                .not. (nchar==1 .and. string(1:1)=='S') ) then

!     --- Detect the type of record
!         Skip this part if the keyword = S
!         If a main keyword has been found, return to calling subroutine

         ierpri = 0
         kcode = sepmainkeyw () - 1
         if ( debug ) write(irefwr,1) 'kcode', kcode
         if ( kcode>=0 ) go to 1000

      end if  ! ( ityprd/=2 .and. ityprd/=4 )

      if ( ityprd==4 ) then

!     --- Record starts with a string, this must be S

         if ( string(1:1)/='S' ) then
            call errchr ( string(1:1), 1 )
            call errchr ( 'S', 2 )
            call errsub ( 648, 0, 0, 2 )
            call errcrd
            go to 1000
         end if
         if ( debug ) write(irefwr,*) string(1:nchar)

!        --- detect the surface record
!            first the surface number is read

         openfound = .false.
         call readreal ( openfound, .false. )

      end if
      if ( debug ) write(irefwr,1) 'ityprd/2', ityprd

      if ( ityprd/=2 ) then

!     --- Next item is not an integer

         call errint ( 0, 1 )
         call errsub ( 1349, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

      isurnr = nint(realrd)
      if ( debug ) write(irefwr,1) 'isurnr', isurnr
      nsurfs = max ( nsurfs, isurnr )
      ksurfs = ksurfs + 1
      if ( isurnr<=0 .or. isurnr>maxsrf ) then

!     --- isurnr has wrong value

         call errint ( isurnr, 1 )
         call errint ( maxsrf, 2 )
         call errsub ( 146, 2, 0, 0 )
         call errcrd
         go to 1000

      else if ( ipoint(isurnr)/=0 ) then

!     --- Surface isurnr has already been filled

         call errint ( isurnr, 1 )
         call errsub ( 144, 1, 0, 0 )
         call errcrd

      end if

      ipoint(isurnr) = iinp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd2surnr'

      end if  ! ( debug )
      call erclos ( 'mshrd2surnr' )

      icode = kcode

      end subroutine mshrd2surnr
