      subroutine mshrenkmeshc ( kmeshc, inpelm, nelem, isplit, nsplit, &
                                npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber elements in case elements belong to disjoint sets
!     elements are renumbered per set
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     renumbering
!     disjoint
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, nelem, nsplit, npoint, kmeshc(inpelm,nelem), &
              isplit(nsplit)

!     inpelm         i    number of points per element
!     isplit         i    contains the number of nodes per disjoint set
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
!     nsplit         i    Number of disjoint parts the mesh consists of
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih1, ih2, ielem, ilevel
      integer, allocatable :: iwork(:), ihelp(:), kmeshccop(:,:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ielem          Element number
!     ih1            Help variable to store a constant temporarily
!     ih2            Help variable to store a constant temporarily
!     ihelp          integer work array, see method
!     ilevel         sequence number of disjoint set
!     iwork          integer work array, see method
!     kmeshccop      renumbered copy of kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill iwork(1:npoint) with level for each node
!     ihelp(1:nsplit+1) := 0
!     For i := 1 (1) nelem do
!        ilevel := iwork(kmeshc(1,i))+1
!        ihelp(ilevel) := ihelp(ilevel)+1
!     For i := 1 (1) nelem do
!        ilevel := iwork(kmeshc(1,i))
!        ihelp(ilevel) := ihelp(ilevel)+1
!        ielem := ihelp(ilevel)
!        kmeshccop(:,ielem) := kmeshc(:,i)
!     kmeshc := kmeshccop
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrenkmeshc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrenkmeshc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), ihelp(nsplit+1), &
                 kmeshccop(inpelm,nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill iwork(1:npoint) with level for each node

      ih1 = 1
      do i = 1, nsplit
         ih2 = isplit(i)
         iwork(ih1:ih2) = i
         ih1 = ih2+1
      end do  ! i = 1, nsplit

!     --- ihelp(1:nsplit+1) := 0

      ihelp = 0

!     --- For i := 1 (1) nelem do
!            ilevel := iwork(kmeshc(1,i))+1
!            ihelp(ilevel) := ihelp(ilevel)+1

      do i = 1, nelem
         ilevel = iwork(kmeshc(1,i))+1
         ihelp(ilevel) = ihelp(ilevel)+1
      end do  ! i = 1, nelem

!     --- For i := 1 (1) nelem do
!            ilevel := iwork(kmeshc(1,i))
!            ihelp(ilevel) := ihelp(ilevel)+1
!            ielem := ihelp(ilevel)
!            kmeshccop(:,ielem) := kmeshc(:,i)

      do i = 1, nelem
         ilevel = iwork(kmeshc(1,i))
         ihelp(ilevel) = ihelp(ilevel)+1
         ielem = ihelp(ilevel)
         kmeshccop(:,ielem) = kmeshc(:,i)
      end do  ! i = 1, nelem

!     --- kmeshc := kmeshccop

      kmeshc = kmeshccop

      deallocate ( iwork, ihelp, kmeshccop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshrenkmeshc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrenkmeshc'

      end if  ! ( debug )

      end
