      subroutine mshseqcurvs ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 28-10-2013 allocate isubcurvs
!        version  2.0  date 08-09-2010 New parameter list
!        version  1.2  date 24-06-2004 Extension with own curve
!
!   copyright (c) 1999-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Make a reordering of curves such that curves of curves or curves
!   to be copied are created once the subcurves are created
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     reordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)

!     iinput         i    integer input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, iseq, icurve, ih, ndefsf, minloop, istart, &
              i, jcurnr, iprec, j, icount, jstart, jinp, iwork(ncurvs), &
              iwork1(2,ncurvs)
      integer, allocatable :: isubcurvs(:)
      logical check

!     check          if true a curve has got a new sequence number
!     i              Counting variable
!     icount         Counter
!     icurve         Type of curve
!     ih             Help variable to store a constant temporarily
!     iinp           last position used in array iinput
!     iprec          curve number of curve to be copied etc.
!     iseq           Sequence number
!     istart         First position used at a part
!     isubcurvs      integer work array of length 40*ncurvs in which
!                    the subcurves of the curves are stored
!     iwork          integer work array of length ncurvs
!                    In this array the new sequence numbers of the
!                    curves are stored
!     iwork1         integer work array of length 2*ncurvs
!                    iwork1(1,i) contains the starting address of curve
!                    i in isubcurvs
!                    iwork1(2,i) contains the number of subcurves of
!                    curve i
!     j              Counting variable
!     jcurnr         curve sequence number
!     jinp           position used in array iinput
!     jstart         starting position of curve i in kmeshm
!     minloop        Minimum bound for loop
!     ndefsf         Number of curves in curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH046         Raise iinp as function of icurve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2101   curve numbers can not be ordered because of unsolvable recursion
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear work arrays
!     iseq := 0
!     for icurnr := 1 (1) ncurvs do
!        icurve := type of curve
!        if ( icurve is composite curve ) then
!           Check if all subcurves have lower sequence number
!           If ( there are subcurve with seq. number higher than icurnr ) then
!              Store these sequence numbers into isubcurvs and adapt iwork1
!           else ( all subcurve sequence numbers<icurnr )
!              iseq := iseq+1
!              iwork(icurnr) := iseq
!        else if ( icurve is copy of curve ) then
!           If ( original curves has lower sequence number ) then
!              iseq := iseq+1
!              iwork(icurnr) := iseq
!           else ( original curves has higher sequence number )
!              Store sequence number into isubcurvs and adapt iwork1
!        else ( standard curve )
!           iseq := iseq+1
!           iwork(icurnr) := iseq
!           If ( there are any of the previous numbers that can get a sequence
!                number ) then
!              Raise iseq
!              Store these curves into iwork
!              Adapt iwork1 and isubcurvs
!     Check if all curves have got a number
!     Store inverse of iwork into iseqcurvs
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshseqcurvs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshseqcurvs'

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( isubcurvs(40*ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, 40*ncurvs, 'isubcurvs' )
      if ( ierror/=0 ) go to 1000

!     --- Clear work arrays

      do icurnr = 1, ncurvs
         iwork(icurnr) = 0
         iwork1(1,icurnr) = 0
         iwork1(2,icurnr) = 0
      end do
      iinp = 16
      iseq = 0
      minloop = ncurvs+1
      istart = 0

!     --- Loop over curves

      do icurnr = 1, ncurvs
         icurve = iinput(iinp)
         check = .false.
         ipointcurv(icurnr) = iinp

         if ( debug ) write(irefwr,*) 'icurnr, icurve, iinp ', &
            icurnr, icurve, iinp

         if ( icurve==7 .or. icurve==14 ) then

!        --- curve of curves, check if there are curves with higher
!            number

            if ( icurve==7 ) then
               iinput(iinp+1) = 1
               ndefsf = iinput(iinp+3)
               jinp = iinp+3
            else
               ndefsf = iinput(iinp+6)
               jinp = iinp+6
            end if
            icount = 0
            do i = 1, ndefsf
               jcurnr = abs(iinput(jinp+i))
               if ( jcurnr>icurnr .or. iwork(jcurnr)<=0 ) then
                  minloop = min(minloop,icurnr)
                  icount = icount+1
                  isubcurvs(istart+icount) = jcurnr
               end if
            end do

            if ( icount>0 ) then

!           --- There are curves with sequence number higher than icurnr

               iwork1(1,icurnr) = istart+1
               iwork1(2,icurnr) = icount
               istart = istart+icount

            else

!           --- Standard case, store next sequence number into iwork

               iseq = iseq + 1
               iwork(icurnr) = iseq
               check = .true.

            end if

         else if ( icurve==10 .or. icurve==11 .or. icurve==13 ) then

!        --- curve is a copy of a previous curve

            iprec = iinput(iinp+4)
            if ( iprec>icurnr .or. iwork(iprec)<=0 ) then

!           --- curve number of curve to be copied has a higher number
!               or has not been created yet

               minloop = min(minloop,icurnr)
               istart = istart+1
               iwork1(1,icurnr) = istart
               iwork1(2,icurnr) = 1
               isubcurvs(istart) = iprec

            else

!           --- Standard case

               iseq = iseq + 1
               iwork(icurnr) = iseq
               check = .true.

            end if

         else

!        --- Standard curve

            iseq = iseq + 1
            iwork(icurnr) = iseq
            check = .true.

         end if
100      if ( check .and. minloop<=icurnr ) then

!        --- Check if there are any curves that can be numbered now

            ih = ncurvs+1
            check = .false.
            do i = minloop, icurnr
               if ( iwork(i)<=0 ) then

!              --- curve i has not been numbered yet

                  icount = 0
                  jstart = iwork1(1,i)-1
                  do j = 1, iwork1(2,i)
                     if ( iwork(isubcurvs(jstart+j))<=0 ) then
                        icount = icount+1
                        isubcurvs(jstart+icount) = isubcurvs(jstart+j)
                     end if
                  end do
                  iwork1(2,i) = icount
                  if ( icount==0 ) then
                     iseq = iseq + 1
                     iwork(i) = iseq
                     check = .true.
                  else
                     ih = min(ih,i)
                  end if
               end if
            end do
            if ( ih<=ncurvs ) minloop = ih
            go to 100
         end if
         if ( istart>40*ncurvs ) then

!        --- Error: estimated length of isubcurvs too small

            call errchr ( 'isubcurvs', 1 )
            call errint ( istart, 1 )
            call errint ( 40*ncurvs, 2 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if
         call msh046 ( iinput, iinp, icurve )

      end do

!     --- Check if all curves have got a number

      do icurnr = 1, ncurvs
         if ( iwork(icurnr)<=0 ) then

!        --- curve has not yet been numbered

            call errint ( icurnr, 1 )
            call errsub ( 2101, 1, 0, 0 )

         end if

      end do

!     --- Store inverse of iwork into iseqcurvs

      do icurnr = 1, ncurvs
         iseqcurvs(iwork(icurnr)) = icurnr
      end do

      deallocate ( isubcurvs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'isubcurvs' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshseqcurvs'

      end if
      call erclos ( 'mshseqcurvs' )

      end subroutine mshseqcurvs
