      subroutine mshsim ( kmeshc, nelem , inpelm, bcoor , nbndpt, ndim, &
                          acoor , npoint, isurf , kbndpt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.1  date 29-05-2014 Extension of timebefore
!        version  2.0  date 25-09-2010 Remove iwork
!        version  1.6  date 12-08-1999 Replace askclk by printtime
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array acoor by a similarity-transformation with respect to the
!    topology of an already created surface. It is supposed that all
!    boundary points of the new mesh are in a plane.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     similarity
!     repositioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), nelem, inpelm, nbndpt, ndim, npoint, isurf
      double precision acoor(ndim,*), bcoor(ndim,*)
      integer kbndpt(*)

!     acoor   o   array containing the coordinates of the points
!     bcoor   i   array containing the coordinates of the new boundary
!     inpelm  i   number of points in each element
!     isurf   i   surface number
!     kbndpt  i   node numbers
!     kmeshc  i   array containing the topology of the mesh to be copied
!     nbndpt  i   number of points in bcoor
!     ndim    i   dimension of space
!     nelem   i   number of elements in mesh
!     npoint  i   number of points in acoor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iplane, linear, lengnb, klp(2,1), nlp
      integer, allocatable :: istart(:), ibuurp(:)
      double precision angle, d1, d2, d3, eps, gamma, sum, &
             t1(3), t2(3), t3(3), xo(3), x1, x2, x3
      double precision :: timebefore(2)

!     angle          angle between proposed second basis vector and first
!                    vector
!     d1             distance
!     d2             distance
!     d3             distance
!     eps            accuracy
!     gamma          third coordinate in local basis of boundary points of
!                    surface
!     i              loop variable
!     ibuurp         Help array to store neighbors
!     iplane         helpvariable if all points are in a plane iplane=0 else =1
!     istart         Contains starting information in ibuurp
!     klp            In this array the relative node numbers with respect to
!                    array kbndpt of first and last point of a consecutive part
!                    are stored. The number of parts the boundary consists of
!                    is nlp
!                    Hence klp(1,1) = 1, klp(2,1) denotes the end point of the
!                    first part of the boundary
!                    klp(1,2) contains the node number of the first point of
!                    the second part. This must be klp(2,1)+1
!                    klp(2,nlp) contains the length of kbndbt
!     lengnb         length of neighbour array
!     linear         variable to store linearity of elements considered
!     nlp            Number of disjoint parts the boundary consists of
!     sum            local sum
!     t1             first local basisvector
!     t2             second local basisvector
!     t3             third local basisvector
!     x1             first coordinate of point considered
!     x2             second coordinate of point considered
!     x3             third coordinate of point considered
!     xo             origin of local basis
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Places integer value for message in array
!     ERRSUB         Error messages
!     ERRWAR         Error warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH356         Computes all neighbour points in mesh
!     MSH357         Performs repositioning for all points
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      914     Dimension not correct, only ndim = 3 allowed
!      915     Not enough points in boundary
!      916     All points are on one line
!      947     Points not in one plane
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Do some checks
!     Check whether all points are in one plane
!     Compute all neighbour points
!     Perform repositioning
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsim' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- SIMILAR only used for ndim = 3:

      if ( ndim/=3 ) then
         call errint(ndim,1)
         call errsub ( 914, 1, 0, 0)
         go to 1000
      end if

      if ( nbndpt<3 ) then
         call errint(isurf,1)
         call errsub ( 915, 1, 0, 0)
         go to 1000
      end if

!     --- Set iplane value:

      iplane = 0

!     --- Check whether all boundary points are in one plane

      eps = sqreps

!     --- Compute basis:

      xo(3) = 0d0

      xo(1:ndim) = bcoor(1:ndim,1)

!     --- Determine basevectors T1, T2 and T3:

      t1(3) = 0d0

      t1(1:ndim) = bcoor(1:ndim,2)-xo(1:ndim)

!     --- Normalize:

      sum = sqrt(t1(1)*t1(1)+t1(2)*t1(2)+t1(3)*t1(3))

      t1(1:ndim) = t1(1:ndim)/sum

!     --- Determine second vector:

      do i = 3, nbndpt

          t2(3) = 0d0

          t2(1:ndim) = bcoor(1:ndim,i)-bcoor(1:ndim,i-1)

!         --- Normalize:

          sum = sqrt(t2(1)*t2(1)+t2(2)*t2(2)+t2(3)*t2(3))

          t2(1:ndim) = t2(1:ndim)/sum

          angle = abs(t1(1)*t2(1)+t1(2)*t2(2)+t1(3)*t2(3))

          if ( angle<0.9d0 ) goto 200

      end do

!     --- Angle never large enough: Error

      call errint ( isurf, 1 )
      call errsub ( 916, 1, 0, 0 )
      go to 1000

!     --- Make basis orthogonal

200   d1=t1(1)*(t1(2)*t2(2)+t1(3)*t2(3))-t2(1)*(t1(2)*t1(2)+t1(3)*t1(3))
      d2=t1(2)*(t1(1)*t2(1)+t1(3)*t2(3))-t2(2)*(t1(1)*t1(1)+t1(3)*t1(3))
      d3=t1(3)*(t1(1)*t2(1)+t1(2)*t2(2))-t2(3)*(t1(1)*t1(1)+t1(2)*t1(2))

      sum = sqrt( d1*d1 + d2*d2 + d3*d3 )

      t2(1) = d1 / sum
      t2(2) = d2 / sum
      t2(3) = d3 / sum

!     --- Compute third basis vector:

      t3(1) = t1(2)*t2(3) - t2(2)*t1(3)
      t3(2) = t1(3)*t2(1) - t2(3)*t1(1)
      t3(3) = t1(1)*t2(2) - t2(1)*t1(2)

!     --- Check all boundary points:

      do i = 1 , nbndpt

         x1 = bcoor(1,i)-bcoor(1,1)
         x2 = bcoor(2,i)-bcoor(2,1)

         if ( ndim==3 ) then
            x3 = bcoor(3,i)-bcoor(3,1)
         else
            x3 = 0d0
         end if

         gamma = x1*t3(1) + x2*t3(2) + x3*t3(3)

         if ( abs( gamma)>eps ) iplane = 1

      end do

      if ( iplane==1 ) then

!     --- Points are not in one plane, results may be erroneous:

         call errint( isurf, 1 )
         call errwar( 947, 1, 0, 0 )

      end if

!     --- Compute neighbour nodes for repositioning:

      if ( inpelm<5 ) then
         linear = 1
      else
         linear = 2
      end if

      lengnb = 6 * npoint

      allocate ( istart(npoint), ibuurp(lengnb), stat = error )
      if ( error/=0 ) call eralloc ( error, 7*npoint, 'istart' )
      if ( ierror/=0 ) go to 1000

      nlp = 1
      klp(1,1) = 1
      klp(2,1) = nbndpt
      call msh356 ( kmeshc, nelem, npoint, inpelm, istart, &
                    ibuurp, lengnb, linear, kbndpt, klp, nlp )

!     --- Compute new coordinates of points:

      call msh357 ( bcoor, acoor, npoint, ibuurp, &
                    istart, inpelm, kmeshc, nelem, kbndpt, 0d0, klp, &
                    nlp )

      if ( itime==1 ) call printtime ( 'mshsim')

      deallocate ( istart, ibuurp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istart' )

1000  call erclos ( 'mshsim')

      end
