      subroutine sepfillcrsobs
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays icrossobst and rcrossobst (2d)
!     Old subroutine mshfillcrsobs
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduledebug
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iobst, lencurvs, &
                 lenicrossobst, lenrcrossobst, nx, ny, iseqlc
      integer, allocatable :: iwork(:), iwork1(:)

!     iobst          Obstacle sequence number
!     iseqlc           Sequence number
!     iwork          integer work array of length nelem to be used by
!                    mshobstcrossel
!     iwork1         integer work array of length npoint to be used by
!                    mshobstcrossel
!     lencurvs       Overestimate of the number of points along the new
!                    curves
!     lenicrossobst  Computed length of array icrossobst
!     lenrcrossobst  Computed length of array rcrossobst
!     nx             number of cells in x direction
!     ny             number of cells in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPOBSTCROSSEL Compute all information concerning kmesh ad parts 3 and 4
!     SEPPLOTINTERS  plot and print intersection points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2206   KMESH part x has not been filled
!    2207   There are no obstacles
!    2212   Previous part of kmesh ad has not been filled
!    2213   nmeshc has not been filled
!    2214   kmesh aa has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillcrsobs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillcrsobs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      if (debug)  write(irefwr,1) 'numobst', numobst
      if ( numobst==0 ) then

!     --- No obstacles

         call errwar ( 2207, 0, 0, 0 )
         go to 1000

      end if

      if ( kelmad==0 ) then

!     --- kmesh part ad has not been initialized

         call errint ( 3, 1)
         call errint ( 2, 2)
         call errsub ( 2212, 2, 0, 0 )
         go to 1000

      else if ( kmeshad(2)==0 ) then

!     --- kmesh part ad -2 has not been filled

         call errint ( 3, 1)
         call errint ( 2, 2)
         call errsub ( 2212, 2, 0, 0 )
         go to 1000

      end if

      iseqlc = kmeshad(5)
      if ( kelmx==0 ) then

!     --- error, kmesh part x has not been filled

         call errsub ( 2206, 0, 0, 0 )
         go to 1000

      end if
      if ( kelmaa==0 ) then

!     --- error, kmesh part aa has not been filled

         call errsub ( 2214, 0, 0, 0 )
         go to 1000

      end if

      if ( nmesh(33)==0 ) then

!     --- error, nmesh part c has not been filled

         call errsub ( 2213, 0, 0, 0 )
         go to 1000

      end if

!     --- define work space to store rcrossobst, icrossobst and iedges
!         Length:
!         rcrossobst  at most nedges reals
!         icrossobst  at most 3*nedges+2*numobst
!         iedges      nedges

      length = nedges+nedges+2*numobst+3*nedges+9*nelem+npoint

      allocate ( iwork(nelem), iwork1(npoint), rcrossobst(nedges), &
                 icrossobst(2*numobst+3*nedges+4*nelem), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- compute starting addresses

      lenicrossobst = 0
      lenrcrossobst = 0
      lencurvs = size ( curvesmsh )
      iwork = 0
      iwork1 = 0
      do iobst = 1, numobst
         nx = kmeshx(3, iobst)
         ny = kmeshx(4, iobst)
         iobgrid2d => kmobst(iobst)%iobgrid2d
         iobgrid2d1 => kmobst(iobst)%iobgrid2d1
         call sepobstcrossel ( nx, ny, lenicrossobst, lenrcrossobst, &
                               iobst, iwork, iwork1, lencurvs )

      end do

!     --- Make the arrays rcrossobst, icrossobst permanent

      if ( lenrcrossobst>0 ) then

!     --- Array may be created

         if ( associated(km(iseqkm)%rcrossobst) ) &
            deallocate ( km(iseqkm)%rcrossobst )
         allocate ( km(iseqkm)%rcrossobst(lenrcrossobst) )
         km(iseqkm)%rcrossobst = rcrossobst(1:lenrcrossobst)
         deallocate(rcrossobst)
         rcrossobst => km(iseqkm)%rcrossobst
         kmeshad(4) = lenrcrossobst

      end if
      if ( associated(km(iseqkm)%icrossobst) ) &
         deallocate ( km(iseqkm)%icrossobst )
      allocate ( km(iseqkm)%icrossobst(lenicrossobst) )
      km(iseqkm)%icrossobst = icrossobst(1:lenicrossobst)
      deallocate(icrossobst)
      icrossobst => km(iseqkm)%icrossobst
      kmeshad(3) = lenicrossobst
      kmeshad(8) = iseqlc

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( lenrcrossobst>0 .and. &
           (debug .or. plotobst .or. printobst) ) then

!     --- debug = true, plot and print intersection points

         call sepplotinters

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillcrsobs'

      end if  ! ( debug )
      call erclos ( 'sepfillcrsobs' )

      end subroutine sepfillcrsobs
