      subroutine sepob3dmarkel4 ( iobst, ielemobstlc, ielemobstcp, &
                                  nx, ny, nz, inpelm, leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if there are any elements with all vertices in the obstacle, &
!     that still crosses the obstacle
!     Copy adapted array ielemobst in ielemobstcp
!     3d case
!     Old subroutine mshob3dmarkel4
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     element
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielemobstlc(*), ielemobstcp(*), &
              iobst, nx, ny, nz, inpelm, leniobgrid

!     ielemobstcp    o    Copy of ielemobst rordered
!     ielemobstlc    i    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                               the obstacle
!                         2. The number of elements that are not completely
!                               inside the element but with all vertices inside
!                               the element
!                         3. The number of elements that are partly inside
!                               the obstacle
!                               elements that contain only boundary points of
!                               the obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     inpelm         i    number of points per element
!     iobst          i    Obstacle sequence number
!     iwork               integer work array of size nelem
!                         if iwork(ielem) = 1 element ielem intersects the
!                         obstacle, otherwise iwork(ielem) = 0, not
!     leniobgrid     i    Declared length of last dimension of iobgrid3d
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ncross, numinelmobst, numbnelmobst, k, j, l, &
              ielem, ipstart, nedge_elem, iedge, jpstart, nnode_edge, &
              inode1, inode2, icell(8*leniobgrid), ncell, inode, &
              istart1, istart2, ielemobs
      double precision xmin, ymin, zmin, dx, dy, dz, eps, &
                       x1(3), x2(3), x3(3), xp1(3), xp2(3), &
                       fact1, fact2(2)

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     eps            Local accuracy
!     fact1          multiplication factor
!     fact2          multiplication factors for triangle
!     i              Counting variable
!     icell          Array containing the surface parts of the obstacle that
!                    must be considered for computing the cross sections
!     iedge          edge sequence number
!     ielem          Element number
!     ielemobs       Element number in obstacle
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     istart1        First position used at a part
!     istart2        First position used at a part
!     j              Counting variable
!     jpstart        Starting address in array
!     k              Counting variable
!     l              Counting variable
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     ncross         Number of extra cross sections found
!     nedge_elem     number of edges corresponding to element
!     nnode_edge     number of nodes corresponding to edge
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     x1             coordinates of first point in triangle
!     x2             coordinates of second point in triangle
!     x3             coordinates of third point in triangle
!     xmin           Minimum x-value
!     xp1            coordinates of first point of edge
!     xp2            coordinates of second point of edge
!     ymin           Minimum y-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     MSHCROSSSURF   Find the intersection of the lines xp1-xp2 and the
!                    triangle x1,x2,x3
!     MSHOB3DINTLIN1 Fill array icell with indices of surface parts of the
!                    obstacle that are possibly intersected by the line x1-x2
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Extract xmin,ymin,zmin from obstacle array
!     ncross := 0
!     For all internal elements ielem do
!        Find edges corresponding to element
!        For all edges of element do
!           if ( element intersects obstacle ) then
!              ncross := nross+1
!              iwork(ielem) := 1
!     if ( ncross=0 ) then
!        Copy ielemobst in ielemobstcp
!     else ( ncross>0 )
!        Put elements in ielemobstcp in new sequence
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepob3dmarkel4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepob3dmarkel4'
         call ertrace
         write(irefwr,1) 'nelem, leniobgrid', nelem, leniobgrid
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpelm/=3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- Clear array iwork

      inelem = nelem
      iwork(1:nelem) = 0

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      zmin = rmeshx(5,iobst)
      dx   = rmeshx(7,iobst)
      dy   = rmeshx(8,iobst)
      dz   = rmeshx(9,iobst)
      eps  = accuracy_obstacle

      if ( debug ) then
         write(irefwr,2) 'xmin, ymin, zmin', xmin, ymin, zmin
         write(irefwr,2) 'dx, dy, dz, eps', dx, dy, dz, eps
      end if  ! ( debug )

!     --- ncross := 0

      ncross = 0
      numinelmobst = ielemobstlc(1)
      numbnelmobst = ielemobstlc(3)

      if ( debug ) &
         write(irefwr,1) 'numinelmobst, numbnelmobst', &
                          numinelmobst, numbnelmobst

!     --- For all internal elements ielem do

      do k = 1, numinelmobst

         ielem = ielemobstlc(3+k)

!        --- Find edges corresponding to element

         ipstart = nmeshd(ielem)
         nedge_elem = nmeshd(ipstart)
         if ( debug ) &
            write(irefwr,1) 'k, ielem, nedge_elem', k, ielem, nedge_elem

!        --- Loop over the edges of the element

         do j = 1, nedge_elem

!        --- edge iedge

            iedge = abs(nmeshd(ipstart+j))
            jpstart = nmeshc(iedge)
            nnode_edge = nmeshc(jpstart)

!           --- store initial and end node in inode1/2

            inode1 = nmeshc(jpstart+1)
            inode2 = nmeshc(jpstart+nnode_edge)
            if ( debug ) then
               write(irefwr,1) 'i, iedge, nnode_edge', i, iedge, nnode_edge
               write(irefwr,1) 'inode1, inode2', inode1, inode2
            end if  ! ( debug )
            xp1(1) = coor(1,inode1)
            xp1(2) = coor(2,inode1)
            xp1(3) = coor(3,inode1)
            xp2(1) = coor(1,inode2)
            xp2(2) = coor(2,inode2)
            xp2(3) = coor(3,inode2)
            if ( debug ) then
               call prinrl ( xp1, 3, 'xp1')
               call prinrl ( xp2, 3, 'xp2')
            end if  ! ( debug )

!           --- Fill array icell with indices of surface parts of the obstacle
!               that are possibily intersected by the line x1-x2

            call mshob3dintlin1 ( xp1, xp2, xmin, ymin, zmin, &
                                  nx, ny, nz, iobgrid3d, icell, ncell, &
                                  dx, dy, dz, leniobgrid )

            if ( ncell>8*leniobgrid ) then

!           --- ncell too large

               call errint ( ncell, 1 )
               call errint ( 8*leniobgrid, 1 )
               call errchr ( 'icell', 1 )
               call errsub ( 550, 2, 0 , 1 )
               go to 1000

            end if

!           --- For i := 1 (1) ncell do

            do i = 1, ncell

!           --- Store coordinates in x, y and z

               ielemobs = icell(i)
               inode = kmeshcobst(1,ielemobs)
               do l = 1, 3
                  x1(l) = coorobst(l,inode)
               end do
               inode = kmeshcobst(2,ielemobs)
               do l = 1, 3
                  x2(l) = coorobst(l,inode)
               end do
               inode = kmeshcobst(3,ielemobs)
               do l = 1, 3
                  x3(l) = coorobst(l,inode)
               end do

!              --- Check if line xp1-xp2 intersects triangle and
!                  compute intersection

               call mshcrosssurf ( xp1, xp2, x1, x2, x3, fact1, fact2, &
                                   eps )

               if ( fact1>-eps .and. fact1<1d0+eps ) then

!              --- the line x1-x2 intersects the triangle (x1,x2,x3)
!                  ncross := nross+1
!                  iwork(ielem) := 1

                  ncross = ncross+1
                  iwork(ielem) = 1
                  go to 200

               end if

            end do

         end do

200   end do

      if ( ncross==0 ) then

!     --- ncross=0; Copy ielemobst in ielemobstcp

         ielemobstcp(1) = numinelmobst
         ielemobstcp(2) = 0
         ielemobstcp(3) = numbnelmobst
         do i = 1, numinelmobst+numbnelmobst
            ielemobstcp(3+i) = ielemobstlc(3+i)
         end do

      else

!     --- ncross>0; Put elements in ielemobstcp in new sequence

         ielemobstcp(1) = numinelmobst-ncross
         ielemobstcp(2) = ncross
         ielemobstcp(3) = numbnelmobst
         istart1 = 3
         istart2 = 3+numinelmobst-ncross
         do i = 1, numinelmobst
            ielem = ielemobstlc(3+i)
            if ( iwork(ielem)==0 ) then

!           --- internal element

               istart1 = istart1+1
               ielemobstcp(istart1) = ielem

            else

!           --- boundary element

               istart2 = istart2+1
               ielemobstcp(istart2) = ielem
               if ( debug ) write(irefwr,*) 'boundary element ', ielem

            end if
         end do
         do i = 1, numbnelmobst
            ielemobstcp(3+numinelmobst+i) = ielemobstlc(3+numinelmobst+i)
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepob3dmarkel4'

      end if
      call erclos ( 'sepob3dmarkel4' )

      end
