      subroutine msh351 ( bcord1, bcord2, bcord3, m, n, ivolnr, kbndbt, &
                          klp, nlp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.2  date 22-05-2014 Extra debug
!        version  3.1  date 13-11-2012 Debug statements
!        version  3.0  date 29-04-1999 Extra param's for hollow pipes (BvR)
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check co-ordinates of the three surfaces
!     Volume mesh generator PIPE
! **********************************************************************
!
!                       KEYWORDS
!
!     volume
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord1(3,*), bcord2(3,*), bcord3(3,*)
      integer m, n, ivolnr, kbndbt(*), klp(2,*), nlp

!     bcord1  i    Co-ordinates of the bottom surface
!     bcord2  i    Co-ordinates of the top surface
!     bcord3  i    Co-ordinates of the envelope
!     ivolnr  i    Volume number for error messages
!     kbndbt  i    Array containing the nodal point numbers of the boundary
!                  of bottom surface
!     klp     i    In this array the relative node numbers with respect to array
!                  kbndbt of first and last point of a consecutive part are
!                  stored. The number of parts the boundary consists of is nlp
!                  Hence klp(1,1) = 1, klp(2,1) denotes the end point of the
!                  first part of the boundary
!                  klp(1,2) contains the node number of the first point of the
!                  second part. This must be klp(2,1)+1
!                  klp(2,nlp) contains the length of kbndbt
!     m       i    Number of boundary nodes in the circles
!     n       i    Number of intermediate circles
!     nlp     i    Number of disjoint parts the boundary of the bottom surface
!                  consists of
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dist, epsdis
      integer i, j, ih, ilp, ii

!     dist           Distance between two points
!     epsdis         relative accuracy multiplied by a local distance
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ii             Counting variable
!     ilp            Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     590   Surface of pipe is non-consistent with definition
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh351' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh351'
         write(irefwr,1) 'nlp, m, n', nlp, m, n
         call prinin1 ( klp, nlp, 2, 'klp' )
         call prinrl1 ( bcord1, m, 3, 'bcord1' )
         call prinrl1 ( bcord2, m, 3, 'bcord2' )
         call prinrl1 ( bcord3, n, 3, 'bcord3' )
         call prinin ( kbndbt, klp(2,nlp), 'kbndbt' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Determine value for epsdis

      dist = sqrt ( (bcord1(1,1)-bcord2(1,1))**2 + &
                    (bcord1(2,1)-bcord2(2,1))**2 + &
                    (bcord1(3,1)-bcord2(3,1))**2 )
      if ( dist>0d0 ) then
         epsdis = 10d0*sqreps*dist
      else
         epsdis = 10d0*sqreps*sqrt((bcord1(1,2)-bcord1(1,1))**2 + &
                                   (bcord1(2,2)-bcord1(2,1))**2 + &
                                   (bcord1(3,2)-bcord1(3,1))**2 )
      end if

!     --- Compare S1 and S3

      ii = 0
      do ilp = 1, nlp
         do i = klp(1,ilp), klp(2,ilp)
            ii = ii+1
            j = kbndbt(i)
            dist = sqrt ( (bcord1(1,j)-bcord3(1,ii))**2 + &
                          (bcord1(2,j)-bcord3(2,ii))**2 + &
                          (bcord1(3,j)-bcord3(3,ii))**2 )
            if(dist>epsdis) then
               call errint ( ivolnr, 1 )
               call errint ( j, 2 )
               call errchr ( 'bottom', 1 )
               call errchr ( 'pipe', 2 )
               call erreal ( bcord1(1,j), 1 )
               call erreal ( bcord1(2,j), 2 )
               call erreal ( bcord1(3,j), 3 )
               call erreal ( bcord3(1,i), 4 )
               call erreal ( bcord3(2,i), 5 )
               call erreal ( bcord3(3,i), 6 )
               call errsub ( 590, 2, 6, 2 )
            end if
         end do
      end do

!     --- Compare S2 and S3

      ii = 0
      do ilp = 1, nlp
         do i = klp(1,ilp), klp(2,ilp)
            j = kbndbt(i)
            ii = ii+1
            ih = ii+(n+1)*(m+1)
            dist = sqrt ( (bcord2(1,j)-bcord3(1,ih))**2 + &
                          (bcord2(2,j)-bcord3(2,ih))**2 + &
                          (bcord2(3,j)-bcord3(3,ih))**2 )
            if ( dist>epsdis ) then
               call errint ( ivolnr, 1 )
               call errint ( j, 2 )
               call errchr ( 'top', 1 )
               call errchr ( 'pipe', 2 )
               call erreal ( bcord2(1,j), 1 )
               call erreal ( bcord2(2,j), 2 )
               call erreal ( bcord2(3,j), 3 )
               call erreal ( bcord1(1,ih), 4 )
               call erreal ( bcord1(2,ih), 5 )
               call erreal ( bcord1(3,ih), 6 )
               call errsub ( 590, 2, 6, 2 )
            end if
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh351'

      end if  ! ( debug )
      call erclos ( 'msh351' )

      end
