      subroutine msh352 ( kmeshc, inpelm, nelz, npobot, nelbot, &
                          kmscbt, nelem, kbndbt, kmscps, weddir, &
                          found, klp, nlp, ishape, inpelmbot )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.2  date 22-05-2014 Extension with shape 22
!        version  5.1  date 20-06-2013 Correction call to mshpipquadtets
!        version  5.0  date 24-08-2010 Removal of iwork
!
!   copyright (c) 1987-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshc with nodal point numbers in elements
!     Volume mesh generator PIPE
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     element
!     topology
!     pipe
!     tetrahedron
!     hexahedron
!     prism
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), kmscbt(*), kbndbt(*), kmscps(*), &
              inpelm, nelz, npobot, nelbot, nelem, weddir(7,*), &
              found, klp(2,*), nlp, inpelmbot, ishape

!     found          i    indication that 3 tets can be formed out of one wedge
!     inpelm         i    number of points per element
!     inpelmbot      i    Number of nodes per element of bottom surface and
!                         top surface of pipe
!     ishape         i    Shape number of elements
!     iwork          i    integer work array of length npobot
!     kbndbt         i    Array KBNDPT for bottom surface
!                         Contains the local nodal point numbers of the
!                         boundary of the bottom surface
!     klp            i    In this array the relative node numbers with respect
!                         to array kbndbt of first and last point of a
!                         consecutive part are stored. The number
!                         of parts the boundary consists of is nlp
!                         Hence klp(1,1) = 1, klp(2,1) denotes the end point of
!                         the first part of the boundary
!                         klp(1,2) contains the node number of the first point
!                         of the second part. This must be klp(2,1)+1
!                         klp(2,nlp) contains the length of kbndbt
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     kmscps         i    Kmesh part c corresponding to pipe surface
!                         This array is required to check the direction of
!                         diagonals in the pipe surface
!     nelbot         i    Number of elements in bottom surface
!     nelem          o    Number of elements
!     nelz           i    Number of line elements in vertical direction
!     nlp            i    Number of disjoint parts the boundary of the bottom
!                         surface consists of
!     npobot         i    Number of points of bottom surface
!     weddir         i    Direction of the diagonals per wedge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ikelm, istep, ikelbt, iielem, k, nh, kstep, ilp, &
              iwork(npobot)

!     i              Counting variable
!     iielem         Relative element number in element group
!     ikelbt         Last position used in kmscbt
!     ikelm          Last position used in kmeshc
!     ilp            Counting variable
!     istep          Step size of nodes in one row to the new row of
!                    elements
!     j              Counting variable
!     k              Counting variable
!     kstep
!     nh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHPIPLINPRISM Fill bottom layer of elements in case of a pipe consisting
!                    of linear prisms
!     MSHPIPLINTETS3 Fill bottom layer of elements in case of a pipe consisting
!                    of linear tetrahedrons (3 tets per wedge)
!     MSHPIPLINTETS8 Fill bottom layer of elements in case of a pipe consisting
!                    of linear tetrahedrons (8 tets per wedge)
!     MSHPIPQUADTETS Fill bottom layer of elements in case of a pipe consisting
!                    of quadratic tetrahedrons (8 tets per wedge)
!     MSHPIPQUAPRISM Fill bottom layer of elements in case of a pipe consisting
!                    of quadratic prisms
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1025   shape number has not yet been implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh352' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh352'
         write(irefwr,*) 'npobot, nelbot, inpelm, nlp ', &
                          npobot, nelbot, inpelm, nlp

      end if

!     --- Mark each point in bottom surface to be internal (=0)
!         or boundary point(>0)
!         When iwork(i)>0 the value of iwork(i) refers to the position of
!         node i in kbndbt

      iwork = 0
      i  =  0
      do ilp = 1,nlp
         do j = klp(1,ilp),klp(2,ilp)
            i = i+1
            iwork(kbndbt(j)) = i
         end do
      end do

!     --- Fill elements

      ikelm=0
      if ( (ishape==11 .or. ishape==18) .and. found==0 ) then

!     --- ishape = 11, 18  Linear tetrahedron 8 tets per wedge

         call mshpiplintets8 ( npobot, nelbot, kmeshc, ikelm, iwork, &
                               inpelm, kmscbt, klp, nlp, kmscps, istep )

      else if ( (ishape==11 .or. ishape==18) .and. found==1 ) then

!     --- ishape = 11, 18 Linear tetrahedron with 3 tets per wedge

         call mshpiplintets3 ( npobot, nelbot, kmeshc, ikelm, &
                               inpelm, kmscbt, weddir, istep )

      else if ( ishape==13 .or. ishape==17 ) then

!     --- inpelm = 13, 17  Trilinear hexahedron (plus extra point)

         istep = npobot
         ikelbt = 0
         do iielem = 1, nelbot

!        --- Loop over elements in bottom cross-section

            do k = 1, 4
               kmeshc(ikelm+k) = kmscbt(ikelbt+k)
               kmeshc(ikelm+k+4) = kmscbt(ikelbt+k)+istep
            end do
            ikelm = ikelm+inpelm
            ikelbt = ikelbt+4
         end do

      else if ( ishape==12 ) then

!     --- ishape = 12  Quadratic tetrahedron

         call mshpipquadtets ( npobot, nelbot, kmeshc(ikelm+1), ikelm, &
                               iwork, kmscbt, klp, nlp, kmscps, istep )

      else if ( ishape==14 ) then

!     --- inpelm = 27  Quadratic quadrilateral

         istep = npobot
         ikelbt = 0
         do iielem = 1,nelbot

!        --- Loop over elements in bottom cross-section

            do k = 1, 3
               kmeshc(ikelm+k) = kmscbt(ikelbt+k)
            end do
            kmeshc(ikelm+4) = kmscbt(ikelbt+8)
            kmeshc(ikelm+5) = kmscbt(ikelbt+9)
            kmeshc(ikelm+6) = kmscbt(ikelbt+4)
            kmeshc(ikelm+7) = kmscbt(ikelbt+7)
            kmeshc(ikelm+8) = kmscbt(ikelbt+6)
            kmeshc(ikelm+9) = kmscbt(ikelbt+5)

            do k = 1, 9
               kmeshc(ikelm+k+9) = kmeshc(ikelm+k)+istep
               kmeshc(ikelm+k+18) = kmeshc(ikelm+k)+istep*2
            end do
            ikelm = ikelm+27
            ikelbt = ikelbt+9
         end do
         istep = 2*npobot

      else if ( ishape==21 ) then

!     --- ishape = 21  Prism with linear subdivision in "z-direction"

         call mshpiplinprism ( npobot, nelbot, kmeshc, ikelm, &
                               inpelm, kmscbt, istep, inpelmbot )

      else if ( ishape==22 ) then

!     --- ishape = 22  Prism with quadratic subdivision in "z-direction"

         call mshpipquaprism ( npobot, nelbot, kmeshc, ikelm, &
                               inpelm, kmscbt, istep, inpelmbot )
         istep = 2*istep

      else

!     --- ishape not implemented

         call errint ( ishape, 1 )
         call errchr ( 'msh352', 1 )
         call errsub ( 1025, 1, 0, 1 )
         go to 1000

      end if

!     --- Copy the last nelz-1 cross-sections from the first one

      nh = ikelm
      do i = 1, nelz-1
         kstep = i*istep
         do j = 1, nh
            kmeshc(ikelm+j) = kmeshc(j)+kstep
         end do
         ikelm = ikelm+nh
      end do

      nelem = ikelm/inpelm

1000  if ( debug ) write(irefwr,*) 'End msh352'
      call erclos ( 'msh352' )

      end
