      subroutine msh450 ( coornew, kmeshcnew, kmeshmnew, &
                          kmeshnnew, lencoor, lkmeshc, lenkmeshm, &
                          lenkmeshn, ibrp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.1  date 18-12-2002 Check of shapes
!        version  3.0  date 14-10-2002 Extra parameter kmeshq
!        version  2.2  date 28-02-1999 Include commons
!
!   copyright (c) 1992-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform an existing mesh of linear elements (triangles, quadri-
!     laterals, tetrahedrons or hexahedrons) into a quadratic mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     quadratic
!     element
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coornew(ndim,*)
      integer kmeshmnew(*), kmeshnnew(*), kmeshcnew(*), lencoor, lkmeshc, &
              lenkmeshm, lenkmeshn, ibrp

!     coornew        o    Array containing the new coordinates of the nodes
!     ibrp           i    Estimated number of neighbors
!     kmeshcnew      o    New kmesh part c
!     kmeshmnew      o    New array kmeshm
!     kmeshnnew      i    New array kmeshn
!     lencoor       i/o   Length of array COORNEW
!                         at input : declared length
!                         at output : used length
!     lenkmeshm     i/o   Length of array kmeshmnew
!                         at input : declared length
!                         at output : used length
!     lenkmeshn     i/o   Length of array kmeshnnew
!                         at input : declared length
!                         at output : used length
!     lkmeshc       i/o   Length of array kmeshcnew
!                         at input : declared length
!                         at output : used length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iklemh, ikmeshc, nelhel, neltal, inpelo, ielgrp, &
              inpelm, ipnt, nelmlc, inod, leused, ipos, is, &
              nel, ip(9), ia, ib, &
              jstart, jstend, npnold, kcoor, ih, npt
      integer, allocatable :: ibrpnt(:), istart(:)

!     i              Counting variable
!     ia             selected node
!     ib             selected node
!     ibrpnt         help array for neighbor array and new points
!     ielgrp         loop variable for groups
!     ih             help variable to store node number
!     iklemh         start position kmeshcnew array in buffer
!     ikmeshc        start position kmeshc array in buffer
!     inod           start position of node values
!     inpelm         number of nodes in element
!     inpelo         number of points in element
!     ip             node numbers
!     ipnt           node number
!     ipos           starting position
!     is             temporary node number
!     istart         help array for starting positions in ibrpnt
!     jstart         help variable where to start with neighbors
!     jstend         help variable where to end with neighbors
!     kcoor          number of positions actually used
!     leused         length used to store surface array (KMESH part n)
!     nel            local number of elements
!     nelhel         variable to store actual number of elements
!     nelmlc         number of elements in local element group
!     neltal         variable to store number of elements
!     npnold         number of nodes in original mesh
!     npt            node number of new created node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH401         Run through group of elements to determine neighbors
!     MSH403         Determine number refinement point in line piece
!     MSH406         Place quadratic triangle in element array
!     MSH407         Place quadratic quadrilateral in element array
!     MSH416         Determine preferred diagonal in quadrilateral
!     MSH451         Create new quadratic triangulair elements
!     MSH453         Create new quadratic quadrilateral elements
!     MSH454         Create new quadratic tetrahedronal elements
!     MSH457         Create new quadratic hexahedronal elements
!     MSH462         Determine new nodes
!     MSHTRAKMESHM   Fill kmeshm after transformation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!     940   Not enough positions reserved for helparray
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the important parameters
!     Clear neighbor arrays ISTART and IBRPNT
!     Copy array coor to coornew
!     Run through each group of elements
!        place, if not yet done, points in the middle
!        of all sides, and in the barycenters, &
!        compute new coordinates and store
!     Endrun
!     Rearrange arrays ibrpnt and istart and place the new nodes
!     Fill coornew
!     Next loop (place internal points and) create new elements
!     Fill new KMESH part M (curves)
!     If 3d then fill new KMESH part N (surfaces)
!     Check used length and declared length
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh450' )

!     --- Set variables

      ikmeshc  =   1

!     --- Run through all groups of elements, check each side and place new
!         points

      neltal = 0
      kelgrp = 0

      allocate ( ibrpnt(ibrp*(npoint+1)), istart(npoint+1), stat = error )
      if ( error/=0 ) call eralloc ( error, ibrp*(npoint+1), 'ibrpnt' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array istart (array for the starting addresses)

      istart = 0

!     --- Clear array ibrpnt (array for the neighbors)

      ibrpnt = 0

      kmeshcnew(1) = 0

      if ( kelma==0 ) then

!     --- Only one type of standard element

           ishape = kmeshf(1)

           if ( ishape==3 ) then

!          --- New points only along lines

                niedge = 1
                nisurf = 0
                nivolm = 0

           else if ( ishape==5 ) then

!          --- New points along lines and in barycenter of quadrilateral

                niedge = 1
                nisurf = 1
                nivolm = 0

           else if ( ishape==11 ) then

!          --- New points only along lines

                niedge = 1
                nisurf = 0
                nivolm = 0

           else if ( ishape==13 ) then

!          --- New points "everywhere"

                niedge = 1
                nisurf = 1
                nivolm = 1

           else

!          --- Shape number not implemented

              call errint ( ishape, 1 )
              call errsub ( 800, 1, 0, 0 )

           end if

           call msh401( npoint, kmeshc , nelem, istart, ibrp, ibrpnt, &
                        kmeshcnew, ishape, npelm, niedge, nisurf, &
                        nivolm )

      else

!     --- Several standard elements
!         Start address in kmeshc

           ikmeshc = 1

           do ielgrp = 1, nelgrp

               inpelm = kmesha(ielgrp+1) - kmesha(ielgrp)
               ishape = kmeshf(ielgrp)

               if ( ishape==3 ) then

!              --- New points only along lines

                  niedge = 1
                  nisurf = 0
                  nivolm = 0

               else if ( ishape==5 ) then

!              --- New points along lines and in barycenter of
!                  quadrilateral

                  niedge = 1
                  nisurf = 1
                  nivolm = 0

               else if ( ishape==11 ) then

!              --- New points only along lines

                  niedge = 1
                  nisurf = 0
                  nivolm = 0

               else if ( ishape==13 ) then

!              --- New points "everywhere"

                  niedge = 1
                  nisurf = 1
                  nivolm = 1

               else

!              --- Shape number not implemented

                  call errint ( ishape, 1 )
                  call errsub ( 800, 1, 0, 0 )

               end if

!              --- Number of elements of this type

               inelem = kmeshg(ielgrp)

               call msh401( npoint, kmeshc(ikmeshc), inelem, istart, &
                            ibrp  , ibrpnt, kmeshcnew, ishape, inpelm, &
                            niedge, nisurf, nivolm )

!              --- Increase ikmeshc

               ikmeshc = ikmeshc + inpelm * inelem

           end do

      end if

      npnold = npoint

!     --- Arrays ibrpnt and istart have been filled ; Rearrange and place
!         the new nodes

      call msh462 ( ibrp, ibrpnt, istart, npoint, kmeshcnew, niedge, &
                    nisurf, nivolm )

!     --- Save new number of nodes

      kcoor = ndim * npoint

      if (  kcoor>lencoor ) then

            call errint ( kcoor, 1 )
            call errint ( lencoor, 2 )
            call errsub ( 940, 2, 0, 0 )
            go to 1000

      end if

!     --- Fill coornew (for quadrilaterals and  hexahedrons only
!                     a first estimate)

      do i = 1, npnold

          jstart  =  istart( i   )
          jstend  =  istart( i+1 ) - 2

!         --- Copy old coordinates to new array

          coornew(:,i) = coor(:,i)

          do ipnt = jstart, jstend, 2

              ih  = abs( ibrpnt( ipnt   ) )
              npt = ibrpnt( ipnt+1 )

              if ( ih<npnold ) &
                 coornew(:,npt) = (coor(:,ih)+coor(:,i))*0.5d0

          end do

      end do

!     --- Next loop (place internal points and) create new elements

      nelhel    = 0
      ikmeshc    = 1
      iklemh    = 1
      if ( ishape==5 ) then
         if ( km(iseqkm)%kmesh(16)>0 ) deallocate(km(iseqkm)%kmeshb)
         allocate(km(iseqkm)%kmeshb(nelgrp))
         kmeshb => km(iseqkm)%kmeshb
         kelmb = nelgrp
      end if

      do ielgrp = 1, nelgrp

!     --- determine meshtype

          ishape = kmeshf(ielgrp)

!         --- number of elements of this type

          if ( kelmg==0 ) then
               inelem = nelem
          else
               inelem = kmeshg(ielgrp)
          end if

          if ( ishape==3 ) then

               inpelo = 3
               inpelm = 6

!              --- Create new quadratic elements for all triangles

               call msh451( kmeshc(ikmeshc), inelem, kmeshcnew(iklemh), &
                            ibrpnt, istart, npnold )

!              --- Submesh has been transformed

               ishape = 4

          else if ( ishape== 5 ) then

               inpelo  = 4
               inpelm  = 9
               nelhel  = nelhel + inelem

!              --- linear quadrilaterals : create quadratic elements

               call msh453( kmeshc(ikmeshc), inelem, kmeshcnew(iklemh), &
                            ibrpnt, istart, npnold, coornew )

               kmeshb(ielgrp) = 8

               ishape =  6

          else if ( ishape==11 ) then

               inpelo =  4
               inpelm = 10

!              --- linear tetrahedrons : points have been placed in the
!                  middles of all edges, create quadratic elements

               call msh454( kmeshc(ikmeshc), inelem, kmeshcnew(iklemh), &
                            ibrpnt, istart, npnold )

               ishape = 12

          else if ( ishape==13 ) then

               inpelo =  8
               inpelm = 27
               nelhel = nelhel + inelem

!              --- linear hexahedrons : create new quadratic elements

               call msh457( kmeshc(ikmeshc), inelem, kmeshcnew(iklemh), &
                            coornew , ibrpnt, istart, npnold )

               ishape = 14

          end if

!         --- Increase ikmeshc and iklemh

          ikmeshc = ikmeshc + inelem * inpelo
          iklemh = iklemh + inelem * inpelm
          kelgrp = kelgrp + inpelm

          kmeshf(ielgrp) = ishape

          neltal = neltal + inelem

      end do

!     --- mesh has been changed

      npelm  = inpelm
      nelem  = neltal

      if ( (ndim * npoint)>lencoor ) then

!     --- Not enough positions reserved

           call errint ( ndim * npoint, 1 )
           call errint ( lencoor, 2 )
           call errsub ( 940, 2, 0, 0)
           go to 1000

      end if

      lencoor = ndim * npoint

!     --- Compute number of positions needed

      if ( ishape== 4 ) then
         leused =  6 * nelem
      else if ( ishape== 6 ) then
         leused =  9 * nelem
      else if ( ishape==12 ) then
         leused = 10 * nelem
      else if ( ishape==14 ) then
         leused = 27 * nelem
      else

!     --- Shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end if

      if ( leused>lkmeshc ) then

!     --- Not enough positions reserved

           call errint ( leused, 1 )
           call errint ( lkmeshc, 2 )
           call errsub ( 940, 2, 0, 0)
           go to 1000

      end if

!     --- Make final reservation

      lkmeshc = leused

!     --- Fill kmeshmnew (new KMESH part M)

      call mshtrakmeshm ( kmeshmnew, istart, ibrpnt, npnold, lenkmeshm )

      if ( ndim==3 .and. kelmn/=0 ) then

!     --- Fill kmeshnnew (KMESH part N)

           inod   = 2 * nsurfs
           ipos   = inod + 1
           nelhel = 0

!          --- Triangles or quadrilaterals

           if ( ishape==12 ) then

!          --- tetrahedrons and hence triangles

              do is = 1, nsurfs

!             --- Value of npelm was 3 store new value 6

                  kmeshnnew( is ) = 6

!                 --- determine number of elements (number does not change)

                  nelmlc = kmeshn( is + nsurfs )
                  kmeshnnew( is + nsurfs ) = nelmlc

!                 --- Run through all triangles and make quadratic

                  do nel = 1, nelmlc

                    ip(1) = kmeshn( inod + 1 )
                    ip(2) = kmeshn( inod + 2 )
                    ip(3) = kmeshn( inod + 3 )

                    inod = inod + 3

!                   --- determine the other three points

                    call msh403( npnold, ip(1), ip(2), ibrpnt, istart, &
                                 ip(4) )
                    call msh403( npnold, ip(2), ip(3), ibrpnt, istart, &
                                 ip(5) )
                    call msh403( npnold, ip(3), ip(1), ibrpnt, istart, &
                                 ip(6) )

                    nelhel = nelhel + 1

                    call msh406 ( kmeshnnew(ipos), nelhel, &
                                  ip(1), ip(4), ip(2), &
                                  ip(5), ip(3), ip(6) )

                  end do

              end do

              leused = ipos + 6 * nelhel - 1

              if ( leused>lenkmeshn ) then

!             --- Not enough positions reserved

                 call errint ( leused, 1 )
                 call errint ( lenkmeshn, 2 )
                 call errsub ( 940, 2, 0, 0)
                 go to 1000

              end if

              lenkmeshn  = leused

           else if ( ishape==14 ) then

!          --- hexaheders and hence quadrilaterals

              do is = 1, nsurfs

!             --- Value of npelm was 4 store new value 9

                  kmeshnnew( is ) = 9

!                 --- determine number of elements (number does not change)

                  nelmlc = kmeshn( is + nsurfs )
                  kmeshnnew( is + nsurfs ) = nelmlc

!                 --- Run through all quadrilaterals and make quadratic

                  do nel = 1, nelmlc

                      ip(1) = kmeshn( inod + 1 )
                      ip(2) = kmeshn( inod + 2 )
                      ip(3) = kmeshn( inod + 3 )
                      ip(4) = kmeshn( inod + 4 )

                      inod  = inod + 4

!                     --- determine the other five points

                      call msh403( npnold, ip(1), ip(2), ibrpnt, istart, &
                                   ip(5) )
                      call msh403( npnold, ip(2), ip(3), ibrpnt, istart, &
                                   ip(6) )
                      call msh403( npnold, ip(3), ip(4), ibrpnt, istart, &
                                   ip(7) )
                      call msh403( npnold, ip(4), ip(1), ibrpnt, istart, &
                                   ip(8) )

!                     --- barycenter

                      call msh416( ip(1), ip(2), ip(3), ip(4), &
                                   ia, ib )
                      ib = ib * npnold
                      call msh403( npnold, ia, ib, ibrpnt, istart, &
                                   ip(9) )

                      nelhel = nelhel + 1

                      call msh407( kmeshnnew(ipos), nelhel, &
                                   ip(1) , ip(5) , ip(2) , &
                                   ip(6) , ip(3) , ip(7) , &
                                   ip(4) , ip(8) , ip(9) )

                  end do

              end do

              leused = ipos + 9 * nelhel - 1

              if ( leused>lenkmeshn ) then

!             --- Not enough positions reserved

                 call errint ( leused, 1 )
                 call errint ( lenkmeshn, 2 )
                 call errsub ( 940, 2, 0, 0)
                 go to 1000

              end if

              lenkmeshn  = leused

           end if

      end if

      deallocate ( ibrpnt, istart, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ibrpnt' )

1000  call erclos ( 'msh450' )

      end subroutine msh450
