      subroutine mshanalvol ( volusp, nvolms, nuspnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-11-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if all volumes are coupled because they have common user points
!     If not issue a warning
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     mesh
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer volusp(*), nvolms, nuspnt

!     nuspnt         i    Number of user points
!     nvolms         i    number of volumes
!     volusp         i    array containing the user points corresponding to
!                         the volumes in the following way:
!                         volusp(1)=nvolms+1
!                         number of user points in volume i:
!                         volusp(i+1)-volusp(i)
!                         user points in volume i are stored in volusp from
!                         position volusp(i)+1 to volusp(i+1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivolume(nvolms), iwork(nuspnt), ivolnr, iseq, nvolleft, i, &
              j, icount
      logical found, foundloc

!     found          If true already a possibility has been found
!     foundloc       If true already a possibility has been found
!     i              Counting variable
!     icount         Counter
!     iseq           Sequence number
!     ivolnr         Volume sequence number
!     ivolume        Work array to be filled with the sequence numbers of the
!                    coupled volumes
!     iwork          integer work array
!     j              Counting variable
!     nvolleft       Number of volumes not treated yet
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1753   Not all volumes are coupled
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array ivolume(1:nvolms)
!     iseq := 0
!     While ( there are still volumes to be treated ) do
!        ivolnr := first free volume
!        iseq := iseq+1;  volusp(ivolnr) := iseq
!        Clear array iwork
!        Make iwork equal to 1 for all user points in ivolnr
!        found := true
!        While ( found ) do
!           found := false
!           for all volumes ivolnr not treated yet do
!              if ( one of the user points is stored in iwork ) then
!                 Make iwork 1 for all user points in ivolnr
!                 found := true
!                 ivolume(ivolnr) := iseq
!     If ( iseq>1 ) then
!        print warning that some volumes are not coupled
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( nvolms.le.1 ) go to 1000

      call eropen ( 'mshanalvol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshanalvol'

      end if

!     --- Clear array ivolume(1:nvolms)
!         iseq := 0

      do ivolnr = 1, nvolms
         ivolume(ivolnr) = 0
      end do
      iseq = 0
      nvolleft = nvolms
      do while ( nvolleft.gt.0 )

!     --- there are still volumes to be treated
!         ivolnr := first free volume

         do i = 1, nvolms

            if ( ivolume(i).eq.0 ) then

!           --- ivolume(i) = 0, free volume found

               ivolnr = i
               go to 100

            end if  ! ivolume(i).eq.0

         end do  ! i = 1, nvolms

!        --- iseq := iseq+1;  volusp(ivolnr) := iseq

100      nvolleft = nvolleft-1
         iseq = iseq+1
         ivolume(ivolnr) = iseq
         if ( debug ) write(irefwr,*) 'iseq, ivolnr ', iseq, ivolnr

!        --- Clear array iwork

         do i = 1, nuspnt
            iwork(i) = 0
         end do

!        --- Make iwork equal to 1 for all user points in ivolnr

         do i = volusp(ivolnr)+1, volusp(ivolnr+1)
            iwork(volusp(i)) = 1
         end do
         if ( debug ) call prinin ( iwork, nuspnt, 'iwork' )

!        --- found := true

         found = .true.
         do while ( found )

!        --- While ( found ) do
!            found := false

            found = .false.

!           --- for all volumes ivolnr not treated yet do

            do ivolnr = 1, nvolms

               if ( ivolume(ivolnr).eq.0 ) then

!              --- Volume has not been treated yet
!                  Check if one of the user points of volume is in iwork

                  do i = volusp(ivolnr)+1, volusp(ivolnr+1)

                     foundloc = .false.
                     if ( iwork(volusp(i)).eq.1 ) then

!                    --- At least one of the user points is stored in iwork
!                        found := true

                        found = .true.
                        foundloc = .true.
                        go to 200

                     end if  ! iwork(volusp(i)).eq.1

                  end do  ! i = volusp(ivolnr)+1, volusp(ivolnr+1)

200               if ( foundloc ) then

!                 --- one of the user points is stored in iwork
!                     Make iwork 1 for all user points in ivolnr
!                     ivolume(ivolnr) := iseq

                     do i = volusp(ivolnr)+1, volusp(ivolnr+1)
                        iwork(volusp(i)) = 1
                     end do
                     if ( debug ) then

!                    --- Debug information

                        write(irefwr,*) 'ivolnr, iseq ',ivolnr, iseq
                        call prinin ( iwork, nuspnt, 'iwork' )

                     end if
                     ivolume(ivolnr) = iseq
                     nvolleft = nvolleft-1

                  end if  ! foundloc

               end if  ! ivoume(ivolnr).eq.0

            end do  ! ivolnr = 1, nvolms

         end do  ! while ( found )

      end do  ! while ( nvolleft.gt.0 )

      if ( iseq.gt.1 ) then

!     --- iseq > 1
!          print warning that some volumes are not coupled

         call errint ( iseq, 1 )
         call errwar ( 1753, 1, 0, 0 )
         write(irefwr,*) 'The following sets of volumes are coupled to', &
                         ' each other'
         do i = 1, iseq

            icount = 0
            do j = 1, nvolms

               if ( ivolume(j).eq.i ) then

!              --- Next volume of the iseq-th set found

                  icount = icount+1
                  iwork(icount) = j

               end if  ! ivolume(j).eq.iseq

            end do  ! j = 1, nvolms

            write(irefwr,300) i, (iwork(j), j = 1, icount)
300         format ( 'Set ', i2, ':    Volumes: ', (40i4) )

         end do  ! i = 1, iseq

      end if  ! iseq.gt.1

      call erclos ( 'mshanalvol' )
      if ( debug ) write(irefwr,*) 'End mshanalvol'

1000  end
