      subroutine mshcr03 ( coorol, coornw, usol, anorm, npoint, &
                           inpada, rinada )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  2.1  date 29-12-2008 Fortran 90
!        version  2.0  date 14-03-2003 Complete revision
!        version  1.3  date 28-02-1999 Include commons
!        version  1.2  date 29-05-1996 improve accuracy of constants
!        version  1.1  date 08-05-1996 correct u0,v0 of 'first' point;
!                                      correct test on curved element
!        version  1.0  date 18-04-1996
!
!   copyright (c) 1996-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the co-ordinates of the new boundary in msh047 in the case
!   iadapt = 3 i.e. x_new is computed by the so-called film method
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, inpada(*)
      double precision coorol(2,*), coornw(2,*), usol(npoint,2), &
                       anorm(2,*), rinada(*)

!     anorm          i    In this array the normal vector is stored
!                         anorm(j,i) contains the j th component of the normal
!                         vector in the i-th integration point
!     coornw         o    Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     inpada         i    User input array for subroutine ADAPBOUN, &
!                         See subroutine ADAPBOUN
!     npoint         i    Number of nodal points along the boundary
!     rinada         i    User input array for subroutine ADAPBOUN
!                         See subroutine ADAPBOUN
!     usol           i    Solution vector of size npoint x 2
!                         Contains the velcoity vector in all boundary points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer linear, i, number, jcart, imult, &
              ithres, idirec
      double precision alpha, un, ut, ut1, ut2, &
                       u1, v1, thresh, amult, psi(npoint)

!     alpha          Parameter (see inpada(4))
!     amult          Multiplication factor
!     i              Counting variable
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     imult          Parameter imult as described in inpada(11)
!     ithres         Parameter ithres as described in inpada(12)
!     jcart          Parameter jcart as described in inpada(8)
!     linear         Parameter linear as described in inpada(7)
!     number         The number-th unknown in each nodal point of the array
!                    is considered.
!     psi            Array of length npoint containing the stream function
!                    values in the nodes of the boundary
!     thresh         Threshold value
!     u1             First component of velocity vector
!     un             normal component of velocity
!     ut             tangential component of velocity
!     ut1            first component of tangential vector
!     ut2            second component of tangential vector
!     v1             second component of velocity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERPSIBOUN      Compute the stream function along a boundary
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The co-ordinates along the boundary are updated according to the
!    formula:
!    x_new(i) = x_old(i) + alpha(i) u(i) * n(i)
!    alpha(i) = factor (psi(i)-psi(0)) / |u(i).t(i)|
!                       x(i)
!    psi(i) - psi(0) = /     u.n dx
!                      x(0)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcr03' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcr03'
         call prinin ( inpada, 12, 'inpada' )
         call prinrl1 ( coorol, npoint, 2, 'coorol' )

      end if

      idirec = inpada(5)
      number = inpada(6)
      linear = inpada(7)
      jcart  = inpada(8)
      imult  = inpada(11)
      ithres = inpada(12)
      if ( ithres==0 ) then
         thresh = epsmac
      else
         thresh = max ( epsmac, rinada(2) )
      end if
      if ( imult==0 ) then
         amult = 1d0
      else if ( imult==1 ) then
         amult = tstep
      else
         amult = rinada(1)
      end if

      if ( debug ) then
         write(irefwr,*) 'idirec, number, linear, jcart ', &
                          idirec, number, linear, jcart
         write(irefwr,*) 'imult, ithres ', imult, ithres
         write(irefwr,*) 'thresh, amult ', thresh, amult
      end if

!     --- First fill array psi

      call erpsiboun ( npoint, usol, psi, coorol, linear, jcart, &
                       idirec, anorm )

!     --- film method is applied; (re)set ibegin, iend, istep

      if ( idirec==1 ) then

!     --- Positive direction

         coornw(1,1) = coorol(1,1)
         coornw(2,1) = coorol(2,1)
         do i = 2, npoint
            u1 = usol(i,1)
            v1 = usol(i,2)
            un = u1*anorm(1,i)+v1*anorm(2,i)
            ut1 = u1 - un*anorm(1,i)
            ut2 = v1 - un*anorm(2,i)
            ut = sqrt(ut1**2+ut2**2)
            if ( ut>thresh ) then

!           --- tangential velocity not zero

               if ( jcart==1 ) then
                  alpha = psi(i)/ut
               else
                  alpha = psi(i)/(ut*2*pi*coorol(1,i))
               end if
               if ( imult>0 ) alpha = amult*alpha
               coornw(1,i) = coorol(1,i)+alpha*anorm(1,i)
               coornw(2,i) = coorol(2,i)+alpha*anorm(2,i)

            else

!           --- tangential velocity zero; set new coordinates equal to old

               coornw(1,i) = coorol(1,i)
               coornw(2,i) = coorol(2,i)

            end if  ! ( ut>thresh )

         end do

      else

!     --- negative direction

         coornw(1,npoint) = coorol(1,npoint)
         coornw(2,npoint) = coorol(2,npoint)
         do i = npoint, 1, -1
            u1 = usol(i,1)
            v1 = usol(i,2)
            un= u1*anorm(1,i)+v1*anorm(2,i)
            ut1 = u1 - un*anorm(1,i)
            ut2 = v1 - un*anorm(2,i)
            ut = sqrt(ut1**2+ut2**2)
            if ( ut>thresh ) then

!           --- tangential velocity not zero

               if ( jcart==1 ) then
                  alpha = -psi(i)/ut
               else
                  alpha = -psi(i)/(ut*2*pi*coorol(1,i))
               end if
               if ( imult>0 ) alpha = amult*alpha
               coornw(1,i) = coorol(1,i)+alpha*anorm(1,i)
               coornw(2,i) = coorol(2,i)+alpha*anorm(2,i)

            else

!           --- tangential velocity zero; set new coordinates equal to old

               coornw(1,i) = coorol(1,i)
               coornw(2,i) = coorol(2,i)

            end if  ! ( ut>thresh )

         end do

      end if  ! ( idirec==1 )

      call erclos ( 'mshcr03' )
      if ( debug ) then
         call prinrl1 ( coornw, npoint, 2, 'coornw' )
         write(irefwr,*) 'End mshcr03'
      end if  ! ( debug )

      end
