      subroutine mshcrobstelm ( kmeshcnew, fluidnd, ielem, coornew  )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.1  date 07-04-2004 Extension for quadratic elements (RvL)
!        version  1.0  date 06-02-2003
!
!   copyright (c) 2000-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Determine coupling elements and store and store the information
!    in kmeshcnew
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     fluid_structure
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  kmeshcnew(*), fluidnd(*), ielem
      double precision coornew(*)

!     coornew        i    Array containing the coordinates of the nodes
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     ielem          i    Element number
!     kmeshcnew     i/o   KMESH part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, iobst, inode(9), jnode(3), n, n1, icurv, &
              icurnr, ishapelam, ishape_curve, istart1, igauss, &
              iwork
      double precision xi, xgauss(4), philam(4), w(2), phi(3)
      logical found

!     found          If true already a possibility has been found
!     i              Counting variable
!     icurnr         Curve sequence number
!     icurv          Curve number with direction
!     igauss         Counter for loop over lagrange multipliers
!     inode          Node sequence number
!     iobst          Obstacle sequence number
!     ishape_curve   Shape of fluid basis functions
!     ishapelam      Shape of lambda basis functions
!     istart1        First position used at a part
!     iwork          integer work array
!     j              Counting variable
!     jnode          Array containing the node numbers of an element
!     n              Number of obstacle parts
!     n1             Help variable
!     phi            Array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!                    array phi is only filled when gauss integration is
!                    applied
!     philam         Shape functions of lagrange multipliers
!     w              array of length m containing the weights for
!                    integration
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!     xi             Local coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMFICTLAM     Compute the integration points and basis functions for the
!                    Lagrangian multipliers
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRFINDELM1     Finds local coordinate of a coordinate (x,y) on
!                    a (curved) line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('mshcrobstelm')

      debug=.false.

!     --- clear array

      do i=1,9
         inode(i)=0
      end do

!     --- do loop over obstacles or boundary elements (not always the same)

      do iobst = 1, 1

!     --- iobst := obstacle number

!         --- find obstacle curve

         icurnr =  kmeshx(2,iobst)
         if (debug) write(irefwr,*) 'obstacle curve', icurnr

         if ( icurnr.gt.0 ) then

!        --- find start of icurnr in kmeshm

            istart1 = kmeshm(2)
            n1=0
            do icurv=1,icurnr
               istart1=istart1+n1
               n1=kmeshm(5+icurv)-kmeshm(5+icurv-1)
            end do
            n=(n1-1)/2

            if (debug) write(irefwr,*) 'number of solid elements  ', n

         end if

!        --- find fluidnodes corresponding to element

         do i=1,3
            inode(i)=fluidnd((ielem-1)*3+i)
            kmeshcnew(i)=inode(i)
            if (debug) write(irefwr,*)'fluid nodes  ',inode(i)
         end do

!        --- ishape of fluid

         ishape_curve=2

!        --- ishape of lagrange multiplier

         ishapelam=2
         ndim=2
         call elmfictlam ( coornew, inode, ishape_curve, ndim, xgauss, &
                           philam, w, ishapelam, .false. )

!        --- loop over lambda's

         do igauss=1,ishapelam
            found=.false.

!           --- loop over obstacle elements in curve

            do i = 1, n
               if (.not.found) then

!              --- find node numbers

                  do j=1,3
                     jnode(j)=kmeshm(istart1+(i-1)*2+j-1)
                  end do

!                 --- print the obstacle nodes and the gauss coordinates

                  if (debug) write(irefwr,*)'structure node  ', &
                          jnode(1), jnode(2), jnode(3)
                  if (debug) write(irefwr,*)'xgauss',xgauss

!                 --- find obstacle node in element

                  call prfindelm1(coornew, jnode, xgauss, igauss, &
                                  xi, found, phi, ishapelam)

                  if (debug) &
                     write(irefwr,*) 'local coordinate  ', found, xi
                  if (abs(xi-2d0).gt.0d0) then
                     iwork=i
                  end if
               end if
            end do !end loop over obstacle elements

            if (found) then
               do i=1,3
                  kmeshcnew(i+igauss*3)=jnode(i)
               end do
            else
                do i=1,3
                  kmeshcnew(i+igauss*3)=kmeshm(istart1+(iwork-1)*2+i-1)
                end do
            end if

         end do !end loop over lagrange multipliers

         if (debug) then
            write(irefwr,*) 'element  ', ielem
            write(irefwr,*) 'fluid nodes  ', &
                      inode(1), inode(2), inode(3)
            write(irefwr,*) 'structure nodes  ', &
              kmeshcnew(1+1*3), kmeshcnew(2+1*3), kmeshcnew(3+1*3)
            write(irefwr,*) 'structure nodes  ', &
              kmeshcnew(1+2*3), kmeshcnew(2+2*3), kmeshcnew(3+2*3)
         end if

      end do !end obstacle loop

      call erclos('mshcrobstelm')
      end
