      subroutine mshedgfaces ( nelem, npoint, kmeshc, nmeshd, &
                               lengnmeshd, nedges, nmesha, lengnmesha, &
                               nfaces, nmeshb, lengnmeshb, &
                               nmeshc, lengnmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nmesha-d for the specific case of a linear
!     triangle mesh corresponding to one volume only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, npoint, kmeshc(*), nmeshd(*), lengnmeshd, nedges, &
              nmesha(*), lengnmesha, nfaces, nmeshb(*), lengnmeshb, &
              nmeshc(*), lengnmeshc

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the linear elements
!     lengnmesha     i    Declared length of nmesha
!     lengnmeshb     i    Declared length of nmeshb
!     lengnmeshc     i    Declared length of nmeshc
!     lengnmeshd     i    Declared length of nmeshd
!     nedges         o    number of edges in volume
!     nelem          i    Number of elements in volume
!     nfaces         o    Number of faces in volume
!     nmesha         o    nmesh part a in volume
!     nmeshb         o    nmesh part b in volume
!     nmeshc         o    nmesh part c in volume
!     nmeshd         o    nmesh part d in volume
!     npoint         i    Number of nodal points in volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer length, kmshd1(npoint+1), kmshd2(20*npoint), inpelm, &
              nedpel, mnoped, ishape, nfapel, nmeshh(12*nelem), &
              melped, medpfa

!     inpelm         number of points per element
!     ishape         Shape number of elements
!     kmshd1         first part of kmesh part d, contains the number of
!                    elements corresponding to each nodal point accumulated
!                    kmshd1(1) = 0.
!                    Length: npoint+1
!     kmshd2         second part of kmesh part d
!                    Contains all the element numbers the nodal points are
!                    lying in sequentially
!     length         Help parameter to indicate the length of arrays
!     medpfa         maximum number of edges per face
!     melped         maximum number of elements connected to one edges
!     mnoped         maximum number of nodes per edge
!     nedpel         number of edges in one element
!     nfapel         number of faces in one element
!     nmeshh         nmesh part h
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOPEDG05       creates the nmesh part h
!     TOPEDGES01     creates the edges part 1 pointers for one element group
!     TOPEDGES02     creates the edges part 2 actual edge labels and nmeshc
!                    for one element group
!     TOPELNODES1    fill kmesh part d1 with pointers to kmesh part d2
!                    for one element group
!     TOPELNODES2    fill second part of kmesh part d
!                    for one element group
!     TOPFACES01     creates the faces part 1 pointers
!                    for one element group
!     TOPFACES02     creates the faces part 2 actual face labels
!                    for one element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create array with elements corresponding to nodes in volume
!     Create edges for all elements in volume
!     Create faces for all elements in volume
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshedgfaces' )

!     --- Create array with elements corresponding to nodes in volume
!         First part 1 pointers

      ishape = 11
      inpelm = 4
      nedpel = 6
      mnoped = 2
      nfapel = 4
      medpfa = 3
      call topelnodes1 ( inpelm, kmshd1, kmeshc, length, npoint, &
                         nelem )
      if ( length.gt.20*npoint ) then

!     --- Length of array kmshd2 too long

         call errint ( length, 1 )
         call errint ( 20*npoint, 2 )
         call errchr ( 'kmshd2', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Fill rest of kmeshd (kmshd2)

      call topelnodes2 ( npoint, inpelm, nelem, kmshd1, kmshd2, &
                         kmeshc )

!     --- Create edges for all elements in volume
!         First part 1 consisting of pointers

      call topedges01 ( nmeshd, nelem, nedpel, length )
      if ( length.gt.lengnmeshd ) then

!     --- Length of array nmeshd too long

         call errint ( length, 1 )
         call errint ( lengnmeshd, 2 )
         call errchr ( 'nmeshd', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Fill rest of nmeshd

      call topedges02 ( kmeshc, kmshd1, kmshd2, nmeshd, nmeshc, mnoped, &
                        nedges, nelem, nedpel, inpelm, ishape, length )
      if ( length.gt.lengnmeshc ) then

!     --- Length of array nmeshc too long

         call errint ( length, 1 )
         call errint ( lengnmeshc, 2 )
         call errchr ( 'nmeshc', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Fill nmeshh with element sequence numbers of elements connected to
!         edges

      call topedg05 ( nmeshd, nmeshh, nedges, melped, length, &
                      nmeshc, kmshd1, kmshd2 )
      if ( length.gt.12*nelem ) then

!     --- Length of array nmeshd too long

         call errint ( length, 1 )
         call errint ( 12*nelem, 2 )
         call errchr ( 'nmeshh', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Create faces for all elements in volume
!         First part with pointers

      call topfaces01 ( nmesha, length, nelem, nfapel )
      if ( length.gt.lengnmesha ) then

!     --- Length of array nmesha too long

         call errint ( length, 1 )
         call errint ( lengnmesha, 2 )
         call errchr ( 'nmesha', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Fill rest of nmesha as well as nmeshb

      call topfaces02 ( nmeshd, nmesha, nmeshb, nmeshh, medpfa, &
                        nfaces, length, nelem, nfapel, ishape )
      if ( length.gt.lengnmeshb ) then

!     --- Length of array nmeshb too long

         call errint ( length, 1 )
         call errint ( lengnmeshb, 2 )
         call errchr ( 'nmeshb', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

1000  call erclos ( 'mshedgfaces' )
      end
