      subroutine mshnormaldir ( itriangle, coor, parallel_elm, normal1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal on a triangle and check whether the normal is
!     perpendicular to the z-axis, upwards or downwards
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     triangle
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itriangle(*)
      double precision coor(3,*), normal1(3)
      logical parallel_elm

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     itriangle      i    Contains the node numbers of a triangle
!     normal1        o    First normal vector
!     parallel_elm   o    If true the element is parallel to the line
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node
      double precision x(3), y(3), z(3), tang2(3), tang1(3), &
                       locnorm, t2(3), t1(3)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     locnorm        Contains the norm in a node
!     node           Node number
!     t1             First component to compute the tangential vector
!     t2             Second component to compute the tangential vector
!     tang1          First tangential vector
!     tang2          Second tangential vector
!     x              x co-ordinate of point
!     y              y co-ordinate of point
!     z              z co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store coordinates of triangle in arrays x, y and z
!     t2 := x_1 - x_2; t1 := x_2 - x_3
!     n := t2 x t1 / ||t2 x t1||
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnormaldir' )
      debug = .false.

!     --- Store coordinates of triangle in arrays x, y and z

      do i = 1, 3
         node = itriangle(i)
         x(i) = coor(1,node)
         y(i) = coor(2,node)
         z(i) = coor(3,node)
      end do

      if ( debug ) then

!     --- debug information

         call prinin ( itriangle, 3, 'nodes' )
         call prinrl ( x, 3, 'x' )
         call prinrl ( y, 3, 'y' )
         call prinrl ( z, 3, 'z' )

      end if

!     --- t2 := x_2 - x_1; t1 := x_1 - x_3
!         n := t2 x t1 / ||t2 x t1||

      tang1(1) = x(1)-x(3)
      tang1(2) = y(1)-y(3)
      tang1(3) = z(1)-z(3)
      tang2(1) = x(2)-x(1)
      tang2(2) = y(2)-y(1)
      tang2(3) = z(2)-z(1)
      locnorm = 1d0/sqrt(tang1(1)**2+tang1(2)**2+tang1(3)**2)
      do i = 1, 3
         t1(i) = locnorm*tang1(i)
      end do
      locnorm = 1d0/sqrt(tang2(1)**2+tang2(2)**2+tang2(3)**2)
      do i = 1, 3
         t2(i) = locnorm*tang2(i)
      end do
      call eloutr ( t1, t2, normal1 )
      locnorm = 1d0/sqrt(normal1(1)**2+normal1(2)**2+normal1(3)**2)
      do i = 1, 3
         normal1(i) = locnorm*normal1(i)
      end do

      if ( debug ) then

!     --- debug information

         call prinrl ( t1, 3, 't1' )
         call prinrl ( t2, 3, 't2' )
         call prinrl ( normal1, 3, 'n' )

      end if

      if ( abs(normal1(3)).lt.1d-4 ) then

!     --- |n(3)| < 10^-4; element = parallel

         parallel_elm = .true.

      else

         parallel_elm = .false.

      end if

      call erclos ( 'mshnormaldir' )
      end
