      subroutine mshobstboun3d2 ( ibounelems, iworkedge, npointnew, &
                                  coornew, iwork, nmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 29-11-2004 Correction for more obstacles
!        version  2.1  date 21-05-2004 Extension for more obstacles
!        version  2.0  date 07-01-2004 Complete revision
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new points in mesh by subdividing some edges of
!     elements that have all points on the boundary of the mesh and on the
!     obstacle
!     At least one must be on the obstacle
!     If an element has 4 nodes on the boundary of an obstacle, this element
!     is still considered to be completely inside the obstacle.
!     No subdivision takes place in order to avoid problems with
!     obstacles with sharp points
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     subdivision
!     edge
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
     use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibounelems(*), iworkedge(*), npointnew, &
              iwork(*), nmap
      double precision coornew(3,*)

!     coornew        o    Array containing the new coordinates of the nodes
!     ibounelems     i    Array of length 3+numbounelms (<nelem) containing
!                         the element numbers of the elements that have all
!                         points on the boundary or on the obstacle
!                         Pos. 1 contains the number of elements that have all
!                                points on the obstacle
!                         Pos. 2 contains the number of elements that have all
!                                points on the boundary
!                         Pos. 3 contains the number of elements that have some
!                                points on the boundary and the others on the
!                                obstacle
!                         The other positions contain the element numbers in the
!                         sequence: first all elements corresponding to pos. 1
!                         then to pos 2 and finally to pos 3
!     iwork          i    integer work array of length npoint
!                         iwork(i) is filled as follows
!                          0 for all points outside the obstacle and
!                            the boundaries
!                         -1 for all points inside the obstacle
!                          1 for all points on the boundary of the obstacle
!                          2 for all points on the outer surfaces but not on
!                            an obstacle
!                          3 for all points on the outer surfaces and on an
!                            obstacle
!     iworkedge      o    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     nmap          i/o   length of second parameter of imap and rmap
!                         At input: declared length
!                         At output: computed length
!     npointnew      o    Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, numobstelms, numbounelms, numbnobelms, &
              istart, i, istartedge, j, iedge, &
              istartnode, node1, node2, nedge_elem, &
              npointmap, nmapmax
      double precision xc, yc, zc
      logical different

!     different      if true nodes of edge are on the boundary and an obstacle
!                    or on two different obstacles
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     istart         First position used at a part
!     istartedge     Starting position of edge in array NMESHD
!     istartnode     Starting address in nodes array
!     j              Counting variable
!     nedge_elem     number of edges corresponding to element
!     nmapmax        Actual value of nmap
!     node1          Node of face
!     node2          Node of face
!     npointmap      Declared value of npointnew
!     numbnobelms    number of elements that have some points on the boundary
!                    and the others on the obstacle
!     numbounelms    Number of elements that have all points on the
!                    boundary of the obstacle or a set of curves
!     numobstelms    number of elements that have all points on the obstacle
!     xc             x-coordinate of the midth-point
!     yc             y-coordinate of the midth-point
!     zc             z-coordinate of the midth-point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMAP2D   Fill arrays imap and rmap for a new point inode which is
!                    the mean value of two already existing nodes node1 and
!                    node2 stored in imap/rmap
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iworkedge
!     Copy old arrays imap and rmap into imap2 and rmap2
!     npointnew := npoint
!     For all elements ielem with some nodes on obstacle and all others on
!         boundary do
!        For all edges iedge corresponding to ielem do
!           if ( one endpoint on boundary and other on obstacle ) then
!              create new point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun3d2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstboun3d2'

      end if
      if ( ierror.ne.0 ) go to 1000

      numobstelms = ibounelems(1)
      numbounelms = ibounelems(2)
      numbnobelms = ibounelems(3)
      nmapmax = 2
      npointmap = npointnew

!     --- Clear array iworkedge

      do i = 1, nedges
         iworkedge(i) = 0
      end do

!     --- Copy old arrays imap and rmap into imap2 and rmap2

      do i = 1, npoint
         imap2(i,1) = imap(i,1)
         imap2(i,2) = imap(i,2)
         imap2(i,3) = imap(i,3)
         rmap2(i,1) = rmap(i,1)
         rmap2(i,2) = rmap(i,2)
      end do

!     --- npointnew := npoint

      npointnew = npoint
      istart = 3+numobstelms+numbounelms

!     --- For all elements ielem with some nodes on obstacle and all others on
!            boundary do

      do i = 1, numbnobelms
         ielem = ibounelems(istart+i)
         istartedge = nmeshd(ielem)
         nedge_elem = nmeshd(istartedge)

!        --- For all edges iedge corresponding to ielem do

         do j = 1, nedge_elem
            iedge = abs(nmeshd(istartedge+j))
            if ( iworkedge(iedge).eq.0 ) then

               istartnode = nmeshc(iedge)
               node1 = nmeshc(istartnode+1)
               node2 = nmeshc(istartnode+2)

!              --- if ( one end point on boundary and other on obstacle ) then
!                     create new point

               different = iwork(node1).ne.iwork(node2)
               if ( different ) then

!              --- iwork for both nodes different
!                  it is possible that both are on the obstacle or
!                  both on the boundary

                   if ( iwork(node1).ge.1001 .and. &
                        iwork(node2).ge.1001 ) then

!                  --- Both nodes on boundary

                       different = .false.

                   else if ( iwork(node1).gt.1001 .and. &
                             iwork(node2).lt.1001 ) then

!                  --- Both nodes on obstacle, is it the same one

                      different = iwork(node1)-1001.ne.iwork(node2)

                   else if ( iwork(node2).gt.1001 .and. &
                             iwork(node1).lt.1001 ) then

!                  --- Both nodes on obstacle, is it the same one

                      different = iwork(node1).ne.iwork(node2)-1001

                   end if  ! ( iwork(node1).ge.1001 .... )

               end if  ! ( different )

               if ( different ) then

!              --- One point on obstacle the other one on boundary

                  npointnew = npointnew+1
                  xc = 0.5d0*(coor(1,node1)+coor(1,node2))
                  yc = 0.5d0*(coor(2,node1)+coor(2,node2))
                  zc = 0.5d0*(coor(3,node1)+coor(3,node2))
                  coornew(1,npointnew) = xc
                  coornew(2,npointnew) = yc
                  coornew(3,npointnew) = zc
                  iworkedge(iedge) = npointnew
                  call mshobstmap2d ( npoint, npointmap, nmap, nmapmax, &
                                      node1, node2, npointnew )
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Node ', npointnew,' has been ', &
                        'created between nodes ', node1, ' and ', &
                        node2, ' which are on the boundary', &
                        ' and obstacle'
                     write(irefwr,*) 'iwork ', iwork(node1),iwork(node2)
                     write(irefwr,*) 'ielem, iedge ', ielem, iedge

                  end if  ! ( debug )

               end if  ! ( different )

            end if

         end do

      end do
      nmap = nmapmax

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstboun3d2'

      end if
      call erclos ( 'mshobstboun3d2' )

      end
