      subroutine mshobstpoint ( iobst, nx, ny, iobgrid, x, y, &
                                inside, lencurvs, iobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-10-2012 Extra parameter iobgrid1
!        version  2.0  date 14-10-2002 Extra parameters kmeshq, lencurvs
!        version  1.1  date 07-01-2002 New calls to mshobstcross/1
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if point with coordinates x,y is inside or outside the obstacle
!     Points on the boundary are considered as inside the obstacle
!     Remark:  if this subroutine is changed also change MSHOBSTMARKPT
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduledebug
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, nx, ny, iobgrid(0:nx-1,0:ny-1), &
              lencurvs, iobgrid1(*)
      double precision x, y
      logical inside

!     inside         o    If true the point with coordinates (x,y) is inside
!                         the obstacle
!     iobgrid        i    array of length nx x ny that is filled as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax
!                         The cell size is dx x dy
!                         Cell i,j refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy) x (xmin+idx,ymin+jdy)
!                         iobgrid(i,j) = 0: cell outside obstacle
!                         iobgrid(i,j) = k>0: boundary part k crosses cell
!                         iobgrid(i,j) = -1: cell completely in obstacle
!     iobgrid1       i    Must be used in connection with iobgrid
!                         iobgrid contains the starting addresses for each cell
!                         i,j
!                         From this starting address iobgrid1 contains
!                         the number of curve parts of the obstacle
!                         intersecting the cell followed by the sequence
!                         numbers of these curve parts
!     iobst          i    Obstacle sequence number
!     lencurvs       i    Overestimate of the number of points along the new
!                         curves
!     nx             i    number of cells in x direction corresponding to
!                         obstacle
!     ny             i    number of cells in y direction corresponding to
!                         obstacle
!     x              i    x co-ordinate of point
!     y              i    y co-ordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical onobstacle, found
      double precision xmin, xmax, ymin, ymax, dx, dy, alpha1, &
                       beta1, eps, xminorig, yminorig, xi(6), &
                       curvescp(2,lencurvs)
      integer ipcell, jpcell, nnodes, ixcross(2), iycross(2), i, &
              icurnrobs, nintersect, nxi

!     alpha1         multiplication factor
!     beta1          multiplication factor
!     curvescp       Array to store all coordinates of one curve
!     dx             step in x-direction
!     dy             step in y-direction
!     eps            Accuracy
!                    Points that are at a distance less than eps of the
!                    obstacle are defined as part of the obstacle
!     found          If true already the point has been determined
!     i              Counter
!     icurnrobs      Curve sequence number of obstacle
!     ipcell         i-index of cell(i,j)
!     ixcross        Contains the number of cross sections of the line x=xp
!                    with the obstacle within cell(i,j)
!                    ixcross(1) contains the number of cross sections
!                    below yp
!                    ixcross(2) contains the number of cross sections
!                    above yp
!     iycross        Contains the number of cross sections of the line y=yp
!                    with the obstacle within cell(i,j)
!                    iycross(1) contains the number of cross sections
!                    at the left-hand side of xp
!                    iycross(2) contains the number of cross sections
!                    at the right-hand side of xp
!     jpcell         j-index of cell(i,j)
!     nintersect     Number of intersections found
!     nnodes         Number of nodes
!     nxi            Number of points that are intersected in x-direction
!     onobstacle     If true the point is positioned on the obstacle
!     xi             Array containing the x-coordinates of the intersection
!                    points in the cell
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xminorig       Minimum x-value to compute ipcell, jpcell
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     yminorig       Minimum y-value to compute ipcell, jpcell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTCROSS   Compute cross section of obstacle with lines x=xp and y=yp
!                    in one cell
!     MSHOBSTCROSS1  Compute cross section of obstacle with line y=yp
!                    in one cell
!     SEPCOPCURVCOOR Copy coordinates of curve icurnr from array curves into
!                    array curvescp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract xmin,xmax,ymin,ymax from obstacle array
!    inside := false
!    If ( (x,y) inside (xmin,xmax;ymin,ymax) )
!       icell := (i,j) index of obstacle cell in which (x,y) is lying
!       if ( iobgrid(i,j) = -1 ) then
!          inside := true
!       else if ( iobgrid(i,j)>0 ) then
!          compute number of cross sections of line x = xp left and right
!          of xp in rectangle as well as number of cross sections of
!          line y = yp below and above yp
!          found := false
!          if ( no cross sections left of x = xp ) then
!             if ( cell at left-hand side is in obstacle ) then
!                inside := true
!                found := true
!             else if ( cell at left-hand side is outside obstacle ) then
!                found := true
!          else if ( no cross sections right of x = xp ) then
!             if ( cell at right-hand side is in obstacle ) then
!                inside := true
!                found := true
!             else if ( cell at right-hand side is outside obstacle ) then
!                found := true
!          if ( no cross sections below y = yp ) then
!             if ( cell below is in obstacle ) then
!                inside := true
!                found := true
!             else if ( cell below is outside obstacle ) then
!                found := true
!          else if ( no cross sections above y = yp ) then
!             if ( cell above is in obstacle ) then
!                inside := true
!                found := true
!             else if ( cell above is outside obstacle ) then
!                found := true
!          else
!             use line x=xp to detect the number of cross sections at
!             the left-hand side
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = debug_mshobstpoint
      call eropen ( 'mshobstpoint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstpoint'

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters

      icurnrobs = kmeshx(2,iobst)
      nxi = 0

!     --- Store all coordinates of the curve in curvescp
!         and compute number of nodes

      call sepcopcurvcoor ( icurnrobs, curvescp, nnodes )

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xminorig = rmeshx(1,iobst)
      xmax     = rmeshx(2,iobst)
      yminorig = rmeshx(3,iobst)
      ymax     = rmeshx(4,iobst)
      dx   = rmeshx(5,iobst)
      dy   = rmeshx(6,iobst)
      eps = rinfin
      do i = 1, ncurvs
         eps = min(eps,elsize(1,i),elsize(2,i))
      end do
      eps = eps*accuracy_obstacle
      xmin = xminorig-eps
      xmax = xmax    +eps
      ymin = yminorig-eps
      ymax = ymax    +eps
      inside = .false.

      if ( debug ) then

!     --- debug is true, print info

         print *,'Data with respect to obstacle'
         print *,'xmin, xmax, ymin, ymax ', xmin, xmax, ymin, ymax
         print *,'accuracy ', eps
         print *,'nx, ny, dx, dy ', nx, ny, dx, dy

      end if

      if ( x>=xmin .and. x<=xmax .and. &
           y>=ymin .and. y<=ymax ) then

!     --- (x,y) inside (xmin,xmax;ymin,ymax)
!         icell := (i,j) index of obstacle cell in which (x,y) is lying

         alpha1 = (x-xminorig)/dx
         beta1  = (y-yminorig)/dy
         ipcell = int(alpha1)
         jpcell = int(beta1)
         ipcell = min(max(ipcell,0),nx-1)
         jpcell = min(max(jpcell,0),ny-1)

         if ( debug ) then

!        --- debug is true, print info

            print *,'New node is in rectangle around obstacle'
            print *,'x, y ', x, y
            print *,'alpha1, beta1 ',alpha1, beta1
            print *,'ipcell, jpcell ',ipcell, jpcell
            print *,'iobgrid info for cell ',iobgrid(ipcell,jpcell)

         end if

         if ( iobgrid(ipcell,jpcell)==-1 ) then

!        --- iobgrid(i,j,1) = iobst

            inside = .true.

         else if ( iobgrid(ipcell,jpcell)>0 ) then

!        --- iobgrid(i,j)>0
!            compute number of cross sections of line x = xp left and right
!            of xp in rectangle as well as number of cross sections of
!            line y = yp below and above yp

            call mshobstcross ( x, y, ipcell, jpcell, onobstacle, &
                                rmeshx(1,iobst), iobgrid, nx, ny, &
                                curvescp, nnodes, ixcross, iycross, &
                                eps, xi, nxi, iobgrid1 )

               if ( onobstacle ) then

!              --- Point on obstacle

                  inside = .true.
                  go to 1000

               end if

            found = .false.

            if ( ixcross(1)==0 ) then

!           --- no cross sections below y = yp

               if ( jpcell<=0 ) then

!              --- Below there is no obstacle

                  found = .true.

               else if ( iobgrid(ipcell,jpcell-1)==-1 ) then

!              --- cell below is in obstacle
!                  ipointobst(inode) := iobst
!                  found := true

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell,jpcell-1)==0 ) then

!              --- cell below is outside obstacle
!                  found := true

                  found = .true.

               end if

            else if ( ixcross(1)==1 ) then

               if ( jpcell==0 ) then

!              --- Below there is no obstacle

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell,jpcell-1)==-1 ) then

!              --- cell below is inside obstacle
!                  found := true

                  found = .true.

               else if ( iobgrid(ipcell,jpcell-1)==0 ) then

!              --- cell below is outside obstacle
!                  found := true

                  inside = .true.
                  found = .true.

               end if

            end if

            if ( .not. found .and. ixcross(2)==0 ) then

!           --- no cross sections above y = yp

               if ( jpcell>=ny-1 ) then

!              --- Above there is no obstacle

                  found = .true.

               else if ( iobgrid(ipcell,jpcell+1)==-1 ) then

!              --- cell above is in obstacle
!                  found := true

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell,jpcell+1)==0 ) then

!              --- cell above is outside obstacle
!                  ipointobst(inode) := 0
!                  found := true

                  found = .true.

               end if

            else if ( .not. found .and. ixcross(2)==1 ) then

!           --- One cross section above

               if ( jpcell>=ny-1 ) then

!              --- Above there is no obstacle

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell,jpcell+1)==-1 ) then

!              --- cell above is in obstacle
!                  found := true

                  found = .true.

               else if ( iobgrid(ipcell,jpcell+1)==0 ) then

!              --- cell above is outside obstacle
!                  found := true

                  inside = .true.
                  found = .true.

               end if

            end if

            if ( .not. found .and. iycross(1)==0 ) then

!           --- no cross sections left of x = xp

               if ( ipcell<=0 ) then

!              --- At the left-hand side there is no obstacle

                  found = .true.

               else if ( iobgrid(ipcell-1,jpcell)==-1 ) then

!              --- cell at left-hand side is in obstacle
!                  found := true

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell-1,jpcell)==0 ) then

!              --- cell at left-hand side is outside obstacle
!                  found := true

                  found = .true.

               end if

            else if ( .not. found .and.  iycross(1)==1 ) then

!           --- one cross section at the left-hand side

               if ( ipcell==0 ) then

!              --- At the left-hand side there is no obstacle

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell-1,jpcell)==-1 ) then

!              --- cell at left-hand side is in obstacle
!                  found := true

                  found = .true.

               else if ( iobgrid(ipcell-1,jpcell)==0 ) then

!              --- cell at left-hand side is outside obstacle
!                  found := true

                  inside = .true.
                  found = .true.

               end if

            end if

            if ( .not. found .and. iycross(2)==0 ) then

!           --- no cross sections right of x = xp

               if ( ipcell>=nx-1 ) then

!              --- At the right-hand side there is no obstacle

                  found = .true.

               else if ( iobgrid(ipcell+1,jpcell)==-1 ) then

!              --- cell at right-hand side is in obstacle
!                  found := true

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell+1,jpcell)==0 ) then

!              --- cell at right-hand side is outside obstacle
!                  found := true

                  found = .true.

               end if

            else if ( .not. found .and. iycross(2)==1 ) then

!           --- one cross section at right-hand side

               if ( ipcell>=nx-1 ) then

!              --- At the right-hand side there is no obstacle

                  inside = .true.
                  found = .true.

               else if ( iobgrid(ipcell+1,jpcell)==-1 ) then

!              --- cell at right-hand side is in obstacle
!                  found := true

                  found = .true.

               else if ( iobgrid(ipcell+1,jpcell)==0 ) then

!              --- cell at right-hand side is outside obstacle
!                  found := true

                  inside = .true.
                  found = .true.

               end if

            end if

            if ( .not. found ) then

!           --- Complicated situation, type not yet detected
!               use line y=yp to detect the number of cross sections at
!               the left-hand side

               nintersect = 0
               do i = 1, nxi

                  if ( xi(i)<x ) then

!                 --- Cross section left of x

                     nintersect = nintersect+1
                     xi(nintersect) = xi(i)

                 end if

               end do
               do while ( .not. found )

!              --- Find number of intersections in cell at left-hand side

                  ipcell = ipcell-1
                  call mshobstcross1 ( y, ipcell, jpcell, &
                                       rmeshx(1,iobst), nintersect, &
                                       iobgrid, nx, ny, &
                                       curvescp, nnodes, xi, iobgrid1 )

                  if ( ipcell<=0 ) then

!                 --- At the left-hand side there is no obstacle

                     if ( mod(nintersect,2)/=0 ) then

!                    --- odd number of cross sections

                        inside = .true.

                     end if
                     found = .true.

                  else if ( iobgrid(ipcell-1,jpcell)==-1 ) then

!                 --- cell at left-hand side is in obstacle
!                     found := true

                     if ( mod(nintersect,2)==0 ) then

!                    --- even number of cross sections

                        inside = .true.

                     end if
                     found = .true.

                  else if ( iobgrid(ipcell-1,jpcell)==0 ) then

!                 --- cell at left-hand side is outside obstacle
!                     found := true

                     if ( mod(nintersect,2)/=0 ) then

!                    --- odd number of cross sections

                        inside = .true.

                     end if
                     found = .true.

                  end if

               end do

            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstpoint'

      end if
      call erclos ( 'mshobstpoint' )

      end
