      subroutine mshobstvol3d ( volumes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 25-01-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute volume of part of region inside, outside and near obstacle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     mesh
!     3d
!     volume
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision volumes(*)

!     volumes        o    Array of length 3 containing the volume of the
!                         following subparts:
!                         1: all elements completely inside the obstacle
!                         2: all elements partly inside the obstacle
!                         3: all elements outside the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenwork
      parameter (lenwork=100)
      integer istart, iobst, numinelmobst, numbnelmobst, ikelmc, &
              iuser(1), islold(1), i
      integer iworkelm(nelem)
      double precision volobst, volnearobst, voloutobst, vol, &
                       work(lenwork), user(1)

!     i              Counting variable
!     ikelmc         position in kmesh part c
!     iobst          Obstacle sequence number
!     islold         dummy
!     istart         First position used at a part
!     iuser          dummy
!     iworkelm       integer work array of length nelem
!                    This array is filled as follows:
!                    iworkelm(ielem) = 0 if ielem outside obstacle
!                    iworkelm(ielem) = 1 if ielem inside obstacle
!                    iworkelm(ielem) =-1 if ielem partly inside obstacle
!     lenwork        Length of array work
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     user           dummy
!     vol            Volume of one element
!     volnearobst    Volume of elements partly inside the obstacle
!     volobst        Volume of elements completely inside the obstacle
!     voloutobst     Volume of elements outside the obstacle
!     work           Work array to be used in eli800
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision eli800

!     ELI800         Element subroutine for second order elliptic equation
!                    (General)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2292   More than one obstacle
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill iworkelm with information about elements in and outside obstacle
!     volobst := 0, volnearobst := 0, voloutobst := 0
!     ielem := 0
!     for ielgrp := 1 (1) nelgrp do
!        ishape := shape number element group
!        inpelm := number of nodes per element in element group
!        inelem := number of elements in element group
!        if ( ishape=11 ) then
!           for irelem := 1 (1) inelem do
!              ielem := ielem + 1
!              vol := volume of element ielem
!              if ( element inside obstacle ) then
!                 volobst := volobst + vol
!              else if ( element outside obstacle ) then
!                 voloutobst := voloutobst + vol
!              else ( element near obstacle )
!                 volnearobst := volnearobst + vol
!        else ( ishape#11 )
!           raise ielem
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstvol3d' )

      if ( numobst>1 ) then

!     --- More than one obstacle

         call errint ( numobst, 1 )
         call errsub ( 2292, 1, 0, 0 )
         go to 1000

      end if
      iobst = 1

!     --- Activate arrays

      call sepflielemobst

!     --- Fill iworkelm with information about elements in and outside obstacle

      iworkelm = 0
      istart = ielemobst(iobst)
      numinelmobst = ielemobst(istart)+ielemobst(istart+1)
      numbnelmobst = ielemobst(istart+2)
      istart = istart+2
      do i = 1, numinelmobst
         iworkelm(ielemobst(istart+i)) = 1
      end do
      istart = istart+numinelmobst
      do i = 1, numbnelmobst
         iworkelm(ielemobst(istart+i)) = -1
      end do

!     --- Fill parameters for eli800

      icheli = 7
      numold = 1
      jdegfd = 1
      itype  = 800
      ifirst = 0
      icount = inpelm
      nelemcac  = inelem
      isubr  = 2

!     --- volobst := 0, volnearobst := 0, voloutobst := 0
!         ielem := 0

      volobst = 0
      volnearobst = 0
      voloutobst = 0
      ikelmc = 1
      ielem = 0

!     --- for ielgrp := 1 (1) nelgrp do

      do ielgrp = 1, nelgrp

!     --- ishape := shape number element group
!         inpelm := number of nodes per element in element group
!         inelem := number of elements in element group

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)

         if ( ishape==11 ) then

!        --- ishape=11, tetrahedrons
!            for irelem := 1 (1) inelem do
!              ielem := ielem + 1
!              vol := volume of element ielem
!              if ( element inside obstacle ) then
!                 volobst := volobst + vol
!              else if ( element outside obstacle ) then
!                 voloutobst := voloutobst + vol
!              else ( element near obstacle )
!                 volnearobst := volnearobst + vol

            do irelem = 1, inelem
               ielem = ielem + 1
               vol = eli800 ( coor, iuser, user, kmeshc(ikelmc), &
                              islold, work )
               if ( iworkelm(ielem)==1 ) then

!              --- inside obstacle

                  volobst = volobst + vol

               else if ( iworkelm(ielem)==-1 ) then

!              --- near obstacle

                  volnearobst = volnearobst + vol

               else

!              --- outside obstacle

                  voloutobst = voloutobst + vol

               end if
               ikelmc = ikelmc+inpelm

            end do

         else

!        --- ishape#11, other type of elements
!            raise ielem

            ielem = ielem + inelem
            ikelmc = ikelmc + inelem*inpelm

         end if

      end do
      volumes(1) = volobst
      volumes(2) = volnearobst
      volumes(3) = voloutobst

1000  call erclos ( 'mshobstvol3d' )

      end
