      subroutine mshouterchk2d ( kmeshm, kmeshmtmp, ncurvs, npoint, ncurvstmp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-10-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the outer boundary of a 2d mesh from the single edges in the mesh
!    and compare with the curves stored in ioutercurvs
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     curve
!     check
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm(*), kmeshmtmp(*), ncurvs, npoint, ncurvstmp

!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshmtmp      i    Temporary array kmeshm created from the mesh
!     ncurvs         i    Number of curves in mesh
!     ncurvstmp      i    Number of curves kmeshmtmp
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(npoint), i, ninner, istart, &
              icurnr, inode, istar3
      integer, allocatable :: ioutcurvs(:)

!     i              Counting variable
!     icurnr         Curve sequence number
!     inode          Node sequence number
!     ioutcurvs      Work array of length ncurvs with information about
!                    outer curves
!                    if ioutcurvs(i) = 0 then curve i is outer curve
!                    if ioutcurvs(i) = 1 then curve i is inner curve
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     iwork          integer work array of size npoint
!                    iwork(i) = 0, means node i is inner node, else
!                    node is on boundary
!     ninner         Number of inner curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR0008         Print information of curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2466   Node on single edge but not on outer boundary
!    2467   Node on outer boundary but not on single edge
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:npoint) and ioutcurvs(1:ncurvs)
!     Set ioutcurvs(icurnr) = 1, for all inner curves
!     For all curves icurnr in ioutercurves do
!        For all nodes inode in curve icurnr do
!           iwork(inode) := 1
!     For all outer curves icurnr in kmeshmtmp do
!        For all nodes inode in icurnr in kmeshmtmp do
!           if ( iwork(inode) = 0 ) then
!              error inode not on boundary but part of a single edge
!           else ( iwork(inode) # 0 )
!              iwork(inode) := -1
!     For inode := 1 (1) npoint do
!        if ( iwork(inode) = 1 ) then
!           error: node on outer boundary but not in single edge
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterchk2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshouterchk2d'
         write(irefwr,*) 'kmeshm_orig'
         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,*) 'kmeshm_tmp'
         call pr0008 ( kmeshmtmp, ncurvstmp )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( ioutcurvs(max(ncurvs,ncurvstmp)), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'ioutcurvs' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork(1:npoint) and ioutcurvs(1:ncurvs)

      iwork(1:npoint) = 0
      ioutcurvs(1:ncurvs) = 0

!     --- Set ioutcurvs(icurnr) = 1, for all inner curves

      ninner = kmeshm(1)
      istar3 = kmeshm(2)-1
      istart = kmeshm(3)
      do i = 0, ninner-1
         ioutcurvs(abs(kmeshm(istart+i))) = 1
      end do  ! i = 1, ninner

!     --- For all outer curves icurnr do

      do icurnr = 1, ncurvs

         if ( ioutcurvs(icurnr)==0 ) then

!        --- Curve is outer curve
!            For all nodes inode in curve icurnr do
!               iwork(inode) := 1

            do i = istar3+kmeshm(4+icurnr), istar3+kmeshm(5+icurnr)-1
               inode = kmeshm(i)
               if ( inode>0 ) iwork(inode) = 1
            end do  ! i = istar3+kmeshm(4+icurnr), istar3+kmeshm(5+icurnr)-1

         end if  ! ( ioutcurvs(icurnr)==0 )

      end do  ! icurnr = 1, ncurvs

      do i = 1, ncurvstmp
         ioutcurvs(i) = 0
      end do

!     --- Set ioutcurvs(icurnr) = 1, for all inner curves

      ninner = kmeshmtmp(1)
      istar3 = kmeshmtmp(2)-1
      istart = kmeshmtmp(3)
      do i = 0, ninner-1
         ioutcurvs(kmeshmtmp(istart+i)) = 1
      end do  ! i = 1, ninner

!     --- For all outer curves icurnr in kmeshmtmp do

      do icurnr = 1, ncurvstmp

         if ( ioutcurvs(icurnr)==0 ) then

!        --- Curve is outer curve
!            For all nodes inode in icurnr in kmeshmtmp do

            do i = istar3+kmeshmtmp(4+icurnr), &
                   istar3+kmeshmtmp(5+icurnr)-1
               inode = kmeshmtmp(i)
               if ( iwork(inode)==0 ) then

!              --- iwork(inode) = 0
!                  error inode not on boundary but part of a single edge

                  call errint ( inode, 1 )
                  call errsub ( 2466, 1, 0, 0 )

               else  ! ( iwork(inode)/=0 )

!              --- iwork(inode) # 0
!                  iwork(inode) := -1

                  iwork(inode) = -1

               end if  ! ( iwork(inode)==0 )

            end do  ! i = istar3+kmeshm(4+icurnr), istar3+kmeshm(5+icurnr)-1

         end if  ! ( ioutcurvs(icurnr)==0 )

      end do  ! icurnr = 1, ncurvs

!     --- For inode := 1 (1) npoint do

      do inode = 1, npoint

         if ( iwork(inode)==1 ) then

!        --- iwork(inode) = 1 )
!            error: node on outer boundary but not in single edge

            call errint ( inode, 1 )
            call errsub ( 2467, 1, 0, 0 )

         end if  ! ( iwork(inode)==1 )

      end do  ! inode = 1, npoint

      deallocate ( ioutcurvs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ioutcurvs' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshouterchk2d'

      end if
      call erclos ( 'mshouterchk2d' )

      end
