      subroutine mshouterord2d ( nparts, lenkmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 08-12-2011 Extra debug
!        version  1.1  date 26-10-2002 Debug statements
!        version  1.0  date 04-02-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Order array outerboun containing the outer curves of the region such
!    that all curves are ordered counter clockwise
!    In this way the outer normal is uniquely defined
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nparts, lenkmeshm

!     lenkmeshm      i    Length of array kmeshm ( only part 3 )
!                         Is used to dimension some local arrays
!     nparts         i    number of parts the boundary consists of
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer imarkpart(nparts), ipart, istart, nouter, i, icurnr, &
              istar3, j, nodp, nodp1, nodp2, nintersect, &
              iwork(lenkmeshm), isequence(lenkmeshm), iseq, jcurnr, &
              icount
      double precision ymin, ymax, eps, y, yminpart, ymaxpart, ymid, &
                       minmaxpart(2,nparts), xfirst, yfirst, xlast, &
                       ylast, x, xvalues(lenkmeshm,3), alpha
      logical finished, downwards

!     alpha          help variable to compute an intersection
!     downwards      If true the boundary part is directed "down wards"
!     eps            Local accuracy
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     icount         Counter
!     icurnr         Curve sequence number
!     imarkpart      Array to mark boundary parts
!                    imarkpart(i) =-1, part has been in negative direction
!                    imarkpart(i) = 0, part has not been marked yet
!                    imarkpart(i) = 1, part has been in positive direction
!     ipart          Sequence number of part
!     iseq           Sequence number
!     isequence      Array containing the sequence of the x-values after sorting
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     iwork          integer work array containing curve numbers
!     j              Counting variable
!     jcurnr         Curve sequence number
!     minmaxpart     Contains minimum and maximum y-values in the various parts
!                    minmaxpart(1,ipart) = ymin
!                    minmaxpart(2,ipart) = ymax
!     nintersect     Number of intersections found
!     nodp           Nodal point sequence number
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     nouter         Number of points outside the obstacle
!     x              x co-ordinate of point
!     xfirst         x co-ordinate of first point of curve
!     xlast          x co-ordinate of last point of curve
!     xvalues        Contains the x values of the intersections
!     y              y co-ordinate of point
!     yfirst         y co-ordinate of first point of curve
!     ylast          y co-ordinate of last point of curve
!     ymax           Maximum y-value
!     ymaxpart       Maximum y-value in a part
!     ymid           y co-ordinate of point in the middle
!     ymin           Minimum y-value
!     yminpart       Minimum y-value in a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Warnings
!     PRININ         print 1d integer array
!     PRINOUTCRVS    Print the contents of array outerboun
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2411   Inconsistency in outer boundary
!    3089   Error in outer boundary
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute eps = 1d-10 * ( |ymax|+|ymin|)
!     Mark all boundary parts as not oriented
!     For all boundary parts  do
!        Find the minimum and maximum y-values (ymin,ymax) of the first
!        boundary part
!        ymid := (ymax+ymin)/2
!        finished := false
!        While (not finished) do
!           Intersect all boundary intervals with y = ymid
!           Store all intersected curve parts and corresponding x-values
!           if (one of the curve parts satisfies |y-ymid|<eps) then
!              ymid := ymid-10*eps
!           else
!              finished := true
!        Sort all intersected intervals in increasing x-sequence
!        downwards := true
!        For i := 1 (1) number_of_intersections do
!           if ( downwards ) then
!              if ( y_last<y_first ) then
!                 Mark boundary as oriented
!              else ( y_last>y_first )
!                 if ( boundary not oriented ) then
!                    reverse sequence of boundary
!                    Mark boundary as oriented
!                 else ( boundary already oriented )
!                    Error: inconsistency
!           else ( upwards )
!              if ( y_last>y_first ) then
!                 Mark boundary as oriented
!              else ( y_last<y_first )
!                 if ( boundary not oriented ) then
!                    reverse sequence of boundary
!                    Mark boundary as oriented
!                 else ( boundary already oriented )
!                    Error: inconsistency
!           downwards := not downwards
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterord2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshouterord2d'
         write(irefwr,1) 'nparts', nparts
         write(irefwr,1) 'size(kmeshm'
         call prinin ( kmeshm, size(kmeshm), 'kmeshm' )
         call prinin ( outerboun, size(outerboun), 'outerboun' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

!     --- Compute eps = 1d-10 * ( |ymax|+|ymin|)

      ymin = rinfin
      ymax = -ymin
      istar3 = kmeshm(2)-1
      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         nouter = outerboun(istart)
         yminpart = rinfin
         ymaxpart = -yminpart
         do i = 1, nouter

            icurnr = abs(outerboun(istart+i))
            do j = kmeshm(4+icurnr), kmeshm(5+icurnr)-1

               nodp = kmeshm(istar3+j)
               if ( nodp>0 ) then

!              --- Nodal point on boundary

                  y = coor(2,nodp)
                  yminpart = min(y,yminpart)
                  ymaxpart = max(y,ymaxpart)

               end if

            end do
            minmaxpart(1,ipart) = yminpart
            minmaxpart(2,ipart) = ymaxpart

         end do
         ymin = min(ymin,yminpart)
         ymax = max(ymax,ymaxpart)

      end do
      eps = 1d-10*(abs(ymin)+abs(ymax))

      if ( debug ) then

!     --- Debugging information

         write(irefwr,1) 'nparts', nparts
         write(irefwr,2) 'ymin, ymax, eps', ymin, ymax, eps
         call prinrl1 ( minmaxpart, nparts, 2, 'minmaxpart' )

      end if

!     --- Mark all boundary parts as not oriented

      imarkpart = 0

!     --- For all parts do

      do ipart = 1, nparts

!     --- Find the minimum and maximum y-values (ymin,ymax) of the first
!         boundary part
!         ymid := (ymax+ymin)/2
!         finished := false

         ymin = minmaxpart(1,ipart)
         ymax = minmaxpart(2,ipart)
         ymid = 0.5d0*(ymin+ymax)
         finished = .false.

         if ( debug ) then

!        --- Debugging information

            write(irefwr,2) 'ymin, ymax, ymid', ymin, ymax, ymid

         end if

!        --- While (not finished) do

         do while ( .not. finished )

!        --- Intersect all boundary intervals with y = ymid
!            Store all intersected curve parts and corresponding x-values
!            if (one of the curve parts satisfies |y-ymid|<eps) then
!               ymid := ymid-10*eps
!            else
!               finished := true

            nintersect = 0
            istart = outerboun(ipart+1)
            nouter = outerboun(istart)
            do i = 1, nouter

               jcurnr = outerboun(istart+i)
               icurnr = abs(jcurnr)

               if ( debug ) write(irefwr,1) 'i, icurnr',i, icurnr
               do j = kmeshm(4+icurnr), kmeshm(5+icurnr)-2

                  nodp1 = kmeshm(istar3+j)
                  nodp2 = kmeshm(istar3+j+1)
                  if ( nodp1>0 .and. nodp2>0 ) then

!                 --- Both nodal points on boundary

                     yfirst = coor(2,nodp1)
                     ylast = coor(2,nodp2)

                     if ( abs(ymid-yfirst)<eps .or. &
                          abs(ymid-ylast)<eps ) then

!                    --- Boundary almost parallel to y=ymid, &
!                        or ymid close to begin or end point of interval
!                        move ymid

                        ymid = ymid - 10d0*eps
                        go to 100

                     else

                        ymin = min(yfirst,ylast)
                        ymax = max(yfirst,ylast)

                        if ( ymid>=min(yfirst,ylast) .and. &
                             ymid<=max(yfirst,ylast) ) then

!                       --- This interval intersects the line y = ymid
!                           Equation for line is
!                            x = x_first + alpha * (x_last-x_first)
!                           Hence alpha = (ymid-y_first)/(y_last-y_first)

                           alpha = (ymid-yfirst)/(ylast-yfirst)
                           xfirst = coor(1,nodp1)
                           xlast = coor(1,nodp2)
                           x = xfirst + alpha * (xlast-xfirst)
                           nintersect = nintersect+1
                           xvalues(nintersect,1) = x
                           xvalues(nintersect,2) = yfirst
                           xvalues(nintersect,3) = ylast
                           iwork(nintersect) = jcurnr
                           if ( debug ) then
                              write(irefwr,1) 'nintersect', nintersect
                              write(irefwr,2) &
                                 'ymin, ymax, ymid, yfirst, ylast', &
                                  ymin, ymax, ymid, yfirst, ylast
                           end if  ! ( debug )

                        end if

                     end if

                  end if

               end do

            end do
            finished = .true.

100      end do

!        --- Sort all intersected intervals in increasing x-sequence

         call prsort ( xvalues, isequence, nintersect )

!        --- Check if there are x-values with the same coordinates
!            If so order them such that they are as close as possible to
!            the previous one

         icount = 1
         xfirst = xvalues(isequence(1),1)
         do j = 2, nintersect
            x = xvalues(isequence(j),1)

            if ( abs(x-xfirst)>eps ) then

!           --- Both x values are different, raise values

               icount = icount+1
               isequence(icount) = isequence(j)
               xfirst = x

            else

!           --- Both x values are the same, delete both

               icount = icount-1
               if ( icount<0 ) then

!              --- icount < 0, boundary can not be correct

                  call errsub ( 3089, 0, 0, 0 )
                  go to 1000

               end if  ! ( icount<0 )

            end if

         end do

         if ( debug ) then

!        --- debug information

            write ( irefwr,* ) 'Intersections'
            call prinin ( iwork, nintersect, ' iwork(icurnr)' )
            call prinrl ( xvalues(1,1), nintersect, 'x-values' )
            call prinrl ( xvalues(1,2), nintersect, 'yfirst' )
            call prinrl ( xvalues(1,3), nintersect, 'ylast' )
            call prinin ( isequence, nintersect, 'sequence' )
            write ( irefwr,* ) 'icount = ', icount

         end if
         nintersect = icount

!        --- downwards := true
!            For i := 1 (1) number_of_intersections do

         downwards = .true.
         do i = 1, nintersect
            iseq = isequence(i)
            x = xvalues(iseq,1)
            icurnr = iwork(iseq)
            if ( icurnr>0 ) then
               yfirst = xvalues(iseq,2)
               ylast  = xvalues(iseq,3)
            else
               yfirst = xvalues(iseq,3)
               ylast  = xvalues(iseq,2)
            end if

            if ( downwards ) then

!           --- downwards = true (odd intersection)

               if ( ylast<yfirst ) then

!              --- y_last<y_first
!                  Mark boundary as oriented

                  if ( imarkpart(ipart)<0 ) then

!                 --- Error part has already marked in other direction

                     call errint ( ipart, 1 )
                     call errint ( icurnr, 2 )
                     call errsub ( 2411, 2, 0, 0 )

                  else if ( imarkpart(ipart)==0 ) then

!                 --- Positive orientation

                     imarkpart(ipart) = 1

                  end if

               else

!              --- y_last>y_first

                  if ( imarkpart(ipart)>0 ) then

!                 --- Error part has already marked in other direction

                     call errint ( ipart, 1 )
                     call errint ( icurnr, 2 )
                     call errsub ( 2411, 2, 0, 0 )

                  else if ( imarkpart(ipart)==0 ) then

!                 --- Negative orientation

                     imarkpart(ipart) = -1

                  end if

               end if

            else

!           --- upwards (even intersection)

               if ( ylast>yfirst ) then

!              --- y_last>y_first
!                  Mark boundary as oriented

                  if ( imarkpart(ipart)<0 ) then

!                 --- Error part has already marked in other direction

                     call errint ( ipart, 1 )
                     call errint ( icurnr, 2 )
                     call errsub ( 2411, 2, 0, 0 )

                  else if ( imarkpart(ipart)==0 ) then

!                 --- Positive orientation

                     imarkpart(ipart) = 1

                  end if

               else

!              --- y_last<y_first

                  if ( imarkpart(ipart)>0 ) then

!                 --- Error part has already marked in other direction

                     call errint ( ipart, 1 )
                     call errint ( icurnr, 2 )
                     call errwar ( 2411, 2, 0, 0 )
                     lenkmeshm = 0
                     go to 1000

                  else if ( imarkpart(ipart)==0 ) then

!                 --- Negative orientation

                     imarkpart(ipart) = -1

                  end if

               end if

            end if

!           --- downwards := not downwards

            downwards = .not. downwards

         end do

      end do
      if ( debug ) write(irefwr,1) 'nintersect', nintersect

!     --- Reorder sequence if necessary

      do ipart = 1, nparts

         if ( imarkpart(ipart)<0 ) then

!        --- imarkipart(ipart)<0, reverse direction

            istart = outerboun(ipart+1)
            nouter = outerboun(istart)
            do i = 1, nouter
               isequence(i) = outerboun(istart+i)
            end do
            do i = 1, nouter
               outerboun(istart+i) = -isequence(nouter+1-i)
            end do
         end if

      end do

      if ( debug ) then

!     --- debugging information

         call prinin ( imarkpart, nparts, 'imarkpart' )
         call prinoutcrvs ( outerboun )

      end if

1000  call erclos ( 'mshouterord2d' )

      end
