      subroutine mshparbf ( iinput, kmeshlocal )
! ======================================================================
!
!        programmer    Patrick Anderson/Guus Segal
!        version  4.7  date 30-01-2016 Replace meshwrbf by meshwr
!        version  4.6  date 28-03-2013 Extra debug
!        version  4.5  date 15-11-2012 Long integers
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide global array kmesh in a set of local arrays kmeshlocal,
!     corresponding to all surfaces (2d) or volumes (3d)
!     This subroutine is to subdivide the mesh for parallel computing
!     At present it is restricted to R^2
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
!     surface
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulekmesh
      use sepmodulemesh
      use sepmodulemach
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer:: iinput(*)
      integer, target :: kmeshlocal(maxmesh)

!     iinput         i    integer input array with information concerning the
!                         parallelization
!                         Contents
!                         Pos. 1 Type of subdivision
!                                1:  If the mesh contains volumes the number of
!                                       subparts is equal to the number of
!                                       volumes otherwise equal
!                                       to the number of surfaces
!                                2:  the mesh is subdivided in layers
!                                3:  the mesh is subdivided in blocks
!                                4:  specially meant for testpurposes:
!                                    the number of elements in each region is
!                                    approximately the same elements are
!                                    not renumbered and are subdivided
!                                    in natural sequence
!                                5:  specially meant for testpurposes:
!                                    a 3d block of equal number of elements
!                                    (hexahedrons) in each direction is
!                                    subdivided into subblocks of equal size
!                                    ( block subdivision)
!                                    The number of blocks must be a power of 3
!                         Pos. 2 If iinput(1) >1: number of processors
!                         Pos. 3 If>0, plot mesh
!     kmeshlocal      i/o   Subdivision of array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer methodpar, nprocessors, i, irefsav, lastps, &
              lentransinfo, lenneighblock, lenblockinfo, &
              npointloc
      character (len=100)  filename, filenm
      character (len=80) text
      integer, allocatable :: neighblocks(:)
      integer, pointer :: points(:)

!     filename       Name of file
!     filenm         Name of file
!     i              Counting variable
!     irefsav        Help parameter to save the value of iref
!     lastps         Last non-blank position in namefl
!     lenblockinfo   Length of array blockinfo
!     lenneighblock  Length of array neighblocks
!     lentransinfo   Length of array transinfo
!     methodpar      Defines the type of subdivision of the mesh to be made
!                    for parallel computing
!                    Possible values:
!                    1:  If the mesh contains volumes the number of
!                           subparts is equal to the number of volumes
!                           otherwise equal to the number of surfaces
!                    2:  the mesh is subdivided in layers
!                    3:  the mesh is subdivided in blocks
!                    4:  specially meant for testpurposes:
!                        the number of elements in each region is
!                        approximately the same
!                        elements are not renumbered and are subdivided
!                        in natural sequence
!                    5:  specially meant for testpurposes:
!                        a 3d block of equal number of elements
!                        (hexahedrons) in each direction is subdivided into
!                        subblocks of equal size ( block subdivision)
!                        The number of blocks must be a power of 3
!     neighblocks    Array containing of each block all neighbor blocks
!                    in the following way:
!                    Pos. 1 to nprocesors starting addresses of each block
!                    For each block from start position:
!                    Number of neighbor blocks, followed by
!                    neighbor block sequence numbers
!     npointloc      Number of points in local mesh
!     nprocessors    Number of processors that is to be used for
!                    parallel processing
!                    Defines also the number of submeshes
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIFIL         Open files
!     MSHPARELEMS    Subdivide the region into nprocessors blocks depending on
!                    the parameter methodpar
!     MSHPARKMESHLOC Fill array kmeshloc with the complete topology of all
!                    submeshes that are used in the parallel computation
!     MSHPARNEIGHBL  Fill array neighblocks with information of all blocks
!                    coupled to each block in a parallel environment
!     MSHPARNUMPROC  Compute number of parallel processors to be used and hence
!                    the number of subparts the mesh is subdivided into
!     MSHWRTRANINFO  Write the contents of the arrays blockinfo and transinfo to
!                    the file meshoutput_par.000
!     PLOTM2PAR      Plot two-dimensional mesh and give each parallel block a
!                    different color
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPMESHWR      fill the file meshoutput with the mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract parameters from input array
!     Compute number of processors
!     For all processors do
!        Create an elements and a nodes array per processor
!     Create local kmesh arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparbf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparbf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extract parameters from input array and array kmesh

      methodpar = iinput(1)

      if ( debug ) &
         write(irefwr,1) 'methodpar, npoint, nelem', &
                          methodpar, npoint, nelem

!     --- Compute number of processors

      call mshparnumproc ( iinput, nprocessors, maxmesh )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,1) 'nprocessors', nprocessors

      allocate ( kmhpar(nprocessors) )
      do i = 1, nprocessors
         nullify(kmhpar(i)%nodepointers)
         nullify(kmhpar(i)%elempointers)
      end do  ! i = 1, nprocessors

!     --- For all processors do
!           Create an elements and a nodes array per processor

      call mshparelems ( methodpar, nprocessors )
      if ( ierror/=0 ) go to 1000

!     --- Make neighbors array containing all neighbors of all blocks
!         First reserve space for array neighblocks at the end of ibuffr

      lenneighblock = 3*nprocessors+2*nprocessors**2

      allocate ( neighblocks(lenneighblock), stat = error )
      if ( error/=0 ) call eralloc ( error, lenneighblock, 'neighblocks' )
      if ( ierror/=0 ) go to 1000

!     --- Actual creation

      call mshparneighbl ( nprocessors, npoint, &
                           neighblocks, lenneighblock )

!     --- Create local kmesh arrays

      call mshparkmeshloc ( nprocessors, kmeshlocal, neighblocks )

      deallocate ( neighblocks, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'neighblocks' )
      if ( ierror/=0 ) go to 1000

!     --- write output to files

      filenm = name10
      irefsav = iref10
      if ( iref10>0 ) then
         iref10 = 24
      else
         iref10 = -24
      end if
      lastps = index ( name10, ' ' )-1
      filename = name10

      do i = 1, nprocessors

         write(filename(lastps+1:lastps+8),100) i
100      format('_par.',i3.3)
         name10 = filename
         call inifil(10)
         kmesh => kmeshlocal(i)
         call sepfillkmesh(kmesh)
         call sepmeshwr ( iref10 )
         close (unit = 24)

      end do

      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)

      if ( ierror>0 ) go to 1000

!     --- Write arrays transinfo and blockinfo for each block to file
!         meshoutput_par.000

      filename(lastps+1:lastps+8) = '_par.000'
      name10 = filename
      call inifil(10)

      text = '//number of blocks'
      if ( iref10>0 ) then

!     --- formatted

         write(iref10,190) text
190      format ( a )
         write(iref10,200) nprocessors
200      format ( i6 )

      else

!     --- unformatted

         write(-iref10) text
         write(-iref10) nprocessors

      end if

      do i = 1, nprocessors
         iseqkm = kmeshlocal(i)
         points => kmhpar(i)%nodepointers
         lenblockinfo = size(km(iseqkm)%blockinfo)
         lentransinfo = size(km(iseqkm)%transinfo)
         npointloc = size(points)
         call mshwrtraninfo ( i, km(iseqkm)%blockinfo, lenblockinfo, &
                              km(iseqkm)%transinfo, lentransinfo, &
                              points, npointloc )

      end do  ! i = 1, nprocessors
      close (unit = 24)

!     --- reset name10 and iref10

      name10 = filenm
      iref10 = irefsav

      if ( ndim==2 .and. iinput(3)/=0 ) then

!     --- ndim = 2 and iplot>0, plot mesh with each processor a different
!         color

         call plotm2par ( 15d0, 1d0, nprocessors )

      end if

      do i = 1, nprocessors
         if ( associated(kmhpar(i)%nodepointers) ) &
            deallocate(kmhpar(i)%nodepointers)
         if ( associated(kmhpar(i)%elempointers) ) &
            deallocate(kmhpar(i)%elempointers)
      end do  ! i = 1, nprocessors
      deallocate ( kmhpar )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparbf'

      end if  ! ( debug )
      call erclos ( 'mshparbf' )

      end subroutine mshparbf
