      subroutine mshparelems1 ( methodpar, nprocessors, ifirst, &
                                ilast, nelemproc, npointproc, nelements )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 30-04-2015 extension with test
!        version  1.2  date 20-03-2015 New calls of subroutines
!        version  1.1  date 18-12-2007 Debug statements
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the region into nprocessors blocks depending on the parameter
!     methodpar
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer methodpar, nprocessors, ifirst, &
              ilast, nelemproc, npointproc, nelements(*)

!     ifirst        i/o   First node in next block
!     ilast         i/o   Last node in previous block
!     methodpar      i    Defines the type of subdivision of the mesh to be made
!                         for parallel computing
!                         Possible values:
!                         1:  If the mesh contains volumes the number of
!                                subparts is equal to the number of volumes
!                                otherwise equal to the number of surfaces
!                         2:  the mesh is subdivided in layers
!                         3:  the mesh is subdivided in blocks
!                         4:  specially meant for testpurposes:
!                             the number of elements in each region is
!                             approximately the same
!                             elements are not renumbered and are subdivided
!                             in natural sequence
!     nelements      i    Array of size nprocessors containing for each block
!                         the end pointer in array elements
!                         Hence the elements stored in array elements
!                         pos. 1 to nelements(1) belong to block 1, &
!                         the elements from pos.nelements(1)+1 to nelements(2)
!                         belong to block 2 etc
!     nelemproc      i    Number of elements per processor (approximately)
!     npointproc     i    Number of nodes per processor (approximately)
!     nprocessors    o    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprocessor, numelms, numnodes
      integer, allocatable :: iwork(:), elementsact(:), pointsact(:)
      integer, allocatable :: elements(:), points(:)

!     elements       Array of size nelemloc containing the element
!                    numbers corresponding to the local mesh
!     elementsact    Array of size nelem indicating which elements are
!                    already part of a submesh (>0) or not yet (=0)
!     iprocessor     Processor sequence number
!     iwork          integer work array of size npoint
!     numelms        Number of elements in array elements
!     numnodes       Number of nodal points in array points
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
!     pointsact      Array of size npoint indicating which nodes are
!                    already used in a submesh (>0) or not yet (=0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ININODEELMS1   ?
!     MSHMETIS3      Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points
!                    with the point numbers
!                    The elements are subdivided by metis
!     MSHPARELEMS3   Store arrays elements and points into ibuffr and store the
!                    corresponding memory management sequence numbers into
!                    elempointers and nodepointers
!     MSHPARELROWS   Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points with the
!                    point numbers in the case that a layered ordering is chosen
!     MSHPARELSORT   Fill array elements with the elements corresponding to one
!                    layer. The subdivision is in the sequence of the elements
!                    without ordening.
!                    This option is only meant for testing
!     MSHPARELSURFS  Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points with the
!                    point numbers
!     MSHPARELUSER   ?
!     MSHPARELVOLMS  Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points with the
!                    point numbers
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
!    2432   The number of elements in block of the parallel mesh is 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelems1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelems1'
         write(irefwr,1) 'methodpar', methodpar
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      allocate ( elements(nelem), points(npoint) )

      allocate ( iwork(npoint), elementsact(nelem), &
                 pointsact(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      elementsact = 0
      pointsact = 0

      if ( methodpar==7 ) then

!     --- Users subdivision

         call mshpareluser ( nprocessors, elementsact, nelem )
         go to 500  ! skip next part

      end if  ! ( methodpar==7 )

!     --- Loop over the processors

      do iprocessor = 1, nprocessors

!     --- Compute starting addresses

         if ( methodpar==1 ) then

!        --- Old method, the number of processors is equal to the number of
!            volumes (3d) or surfaces (2d)

            if ( nvolms>0 ) then

!           --- nvolms>0, number of processors is number of volumes

               call mshparelvolms ( iprocessor, elementsact, numelms )
               if ( debug ) &
                  write(irefwr,1) 'iprocessor, numelms', &
                                   iprocessor, numelms
               if ( ierror/=0 ) go to 1000

            else

!           --- nvolms = 0, number of processors is number of surfaces
!               Find all elements and nodes corresponding to the processor

               call mshparelsurfs ( iprocessor, elementsact, numelms )
               if ( debug ) &
                  write(irefwr,1) 'iprocessor, numelms', &
                                   iprocessor, numelms
               if ( ierror/=0 ) go to 1000

            end if  ! ( nvolms>0 ) then

         else if ( methodpar==2 ) then

!        --- New method subdivide region in layers of equal size
!            Create work space of length npoint

            call mshparelrows ( ifirst, ilast, iprocessor, numelms, &
                                nprocessors, pointsact, elementsact, &
                                npointproc, iwork, .false. )
            if ( ierror/=0 ) go to 1000

         else if ( methodpar==3 ) then

!        --- Subdivide region in blocks of approximately equal size
!            First compute starting address of processor in array elements
!            and the number of elements

            if ( iprocessor==1 ) then
               numelms = nelements(iprocessor)
            else
               numelms = nelements(iprocessor)-nelements(iprocessor-1)
            end if
            if ( debug ) &
               write(irefwr,1) 'nelem, numelms', nelem, numelms

!           --- Next fill array elementsact

            call ininodeelms1 ( elements, numelms, &
                                elementsact, iprocessor )
            if ( debug ) call prinin ( points, numnodes, 'points' )
            if ( ierror/=0 ) go to 1000

         else if ( methodpar==4 ) then

!        --- Subdivide region in layers with almost the same number of elements
!            Is only meant for test purposes

            call mshparelsort ( ifirst, ilast, iprocessor, &
                                nprocessors, elements, &
                                numelms, nelemproc, elementsact )

         else

!        --- Other values of methodpar have not yet been implemented

            call errint ( methodpar, 1 )
            call errint ( 1, 2 )
            call errint ( 5, 3 )
            call errchr ( 'methodpar', 1 )
            call errsub ( 2147, 3, 0, 1 )
            go to 1000

         end if
         if ( numelms==0 ) then

!        --- No elements in block

            call errint ( iprocessor, 1 )
            call errsub ( 2432, 1, 0, 0 )
            go to 1000

         end if

      end do  ! iprocessor = 1, nprocessors

!     --- Loop over the processors in reversed order

500   do iprocessor = nprocessors, 1, -1

!     --- subdivide region in layers of equal size

         call mshmetis3 ( iprocessor, nprocessors,elements, points, &
                          numelms, numnodes, elementsact )
         if ( ierror/=0 ) go to 1000

!        --- Copy arrays elements and ipoints in ibuffr

         call mshparelems3 ( elements, iprocessor, numelms, points, &
                             numnodes )
         if ( ierror/=0 ) go to 1000

      end do  ! nprocessors, 1, -1

      deallocate ( iwork, elementsact, pointsact, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      deallocate ( elements, points )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( elementsact, nelem, 'elementsact' )
         call prinin ( pointsact, npoint, 'pointsact' )
         write(irefwr,*) 'End mshparelems1'

      end if
      call erclos ( 'mshparelems1' )

      end
