      subroutine mshparkmesht ( kmeshtloc, nelemloc, &
                                elements, lenkmesht )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-03-2007 Correction for empty surfaces
!        version  1.0  date 07-01-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesht for a local mesh from array kmesht of the global mesh
!     using the information stored in array elements
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshtloc(2,*), nelemloc, elements(*), lenkmesht

!     elements       i    Array of size nelemloc containing the element
!                         numbers corresponding to the local mesh
!     kmeshtloc      o    Local array kmesh t
!     lenkmesht      o    Length of second dimension of array kmesht
!     nelemloc       i    Number of elements in local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ielem1, ielem2, nel, nelprev
      integer, allocatable :: elemmap(:)

!     elemmap        Array of size nelem containing the local element numbers of
!                    the elements in the global mesh
!                    if element i is not present in local mesh elemmap(i) = 0
!     i              Counting variable
!     ielem1         First element number in surface or volume
!     ielem2         Last element number in surface or volume
!     j              Counting variable
!     nel            Number of elements in surfaces/volumes
!     nelprev        Number of elements treated before
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array elemmap
!     Fill array elemmap from elements
!     Fill array kmeshtloc from kmesht and elemmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparkmesht' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparkmesht'
         call ertrace
         write(irefwr,1) 'nelem, nelemloc', nelem, nelemloc
         call prinin ( elements, nelemloc, 'elements' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( elemmap(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'elemmap' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array elemmap

      elemmap = 0

!     --- Fill array elemmap from elements

      do i = 1, nelemloc
         elemmap(elements(i)) = i
      end do

!     --- Fill array kmeshtloc from kmesht and elemmap

      if ( nvolms>0 ) then

!     --- There are volumes

         lenkmesht = nvolms

      else

!     --- There are no volumes only surfaces

         lenkmesht = nsurfs

      end if
      if ( debug ) &
         call prinin1 ( kmesht, lenkmesht, 2, 'kmesht_glob' )

!     --- Find number of elements corresponding to lower dimensional space

      ielem1 = kmesht(1,1)
      nel = 0
      do i = 1, ielem1-1
         if ( elemmap(i)>0 ) nel = nel+1
      end do
      if ( debug ) then
         write(irefwr,1) 'ielem1, nel', ielem1, nel
      end if  ! ( debug )

!     --- Fill array

      nelprev = nel
      do j = 1, lenkmesht
         ielem1 = kmesht(1,j)
         ielem2 = kmesht(2,j)
         do i = ielem1, ielem2
            if ( elemmap(i)>0 ) nel = nel+1
         end do
         if ( nel>nelprev ) then

!        --- nel>nelprev, there are elements in this group

            kmeshtloc(1,j) = nelprev+1
            kmeshtloc(2,j) = nel

         else

!        --- no elements in this group

            kmeshtloc(1,j) = 0
            kmeshtloc(2,j) = 0

         end if  ! ( nel>nelprev )
         nelprev = nel

         if ( debug ) &
            write(irefwr,1) 'j, ielem1, ielem2, nel', &
                             j, ielem1, ielem2, nel
      end do  ! j = 1, lenkmesht

      deallocate ( elemmap, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'elemmap' )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshtloc, lenkmesht, 2, 'kmeshtloc' )
         write(irefwr,*) 'End mshparkmesht'

      end if  ! ( debug )
      call erclos ( 'mshparkmesht' )

      end
