      subroutine mshparncorr5 ( kmeshn, inpelm, list, kmeshncopy, &
                                icount, inelem, nodesurf, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 27-11-2012 Long integers
!        version  1.1  date 18-02-2012 Extra debug
!        version  1.0  date 05-04-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a list of elements in one surface that are not equal to zero
!     The elements are copied in kmeshncopy in sorted sequence
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, inelem, kmeshn(inpelm,inelem), list(*), &
              kmeshncopy(inpelm,*), icount, npoint, nodesurf(npoint)

!     icount         o    Number of non-zero elements in kmeshncopy
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmeshncopy     o    Sorted copy of array kmeshn with all zero elements
!                         removed
!     list           o    List of element numbers in kmeshncopy with respect to
!                         kmeshn
!     nodesurf      i/o   Array of length npoint containing the number
!                         of times a node in a surface appears
!                         If a node is cleared the number is decreased
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, i, ihelp(inpelm), ipointer(inpelm), j, &
              iloc
      integer, allocatable :: istore1(:,:), iseq(:)

!     i              Counting variable
!     ihelp          integer work array to store node numbers of element
!     iloc           Sequence number of local node in element
!     inode          Node sequence number
!     ipointer       work array for sorting algorithm
!                    Contains sequence of sorted set
!     iseq           work array for sorting algorithm
!                    Contains sequence of sorted set
!     istore1        Help array to store sorted array istore
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPARNCORR4   Sort nodes in istore in increasing sequence
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparncorr5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparncorr5'
         call prinin1 ( kmeshn, inelem, inpelm, 'kmeshn' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store number of times a node in kmeshn appears into nodesurf

      nodesurf = 0

      do i = 1, inelem

         do j = 1, inpelm
            inode = kmeshn(j,i)
            if ( inode>0 ) nodesurf(inode) = nodesurf(inode)+1
         end do  ! j = 1, inpelm

      end do  ! i = 1, inelem

!     --- Make a list of all non-empty surface elements in kmeshn
!         Store these elements in kmeshncopy

      icount = 0

      do i = 1, inelem

         inode = minval(kmeshn(1:inpelm,i))

         if ( inode>0 ) then
            icount = icount+1
            list(icount) = i
            ihelp(1:inpelm) = kmeshn(1:inpelm,i)
            call chsort ( ihelp, ipointer, inpelm )
            kmeshncopy(1:inpelm,icount) = ihelp(ipointer(1:inpelm))
         end if  ! ( inode>0 )

      end do  ! i = 1, inelem

      if ( icount==0 ) go to 1000   ! jump  surface is empty

      if ( debug ) then
         call prinin ( list, icount, 'list' )
         call prinin1 ( kmeshncopy, icount, inpelm, 'kmeshncopy' )
      end if  ! ( debug )

      allocate ( istore1(inpelm,icount), iseq(icount), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, icount*(inpelm+1), 'istore1' )

!     --- Sort arrays list and kmeshncopy

      iloc = 1
      call mshparncorr4 ( kmeshncopy, istore1, iloc, &
                          icount, inpelm, iseq, list )

      deallocate ( istore1, iseq, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istore1' )

1000  call erclos ( 'mshparncorr5' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount', icount
         if ( icount>0 ) then
            call prinin ( list, icount, 'list' )
            call prinin1 ( kmeshncopy, icount, inpelm, 'kmeshncopy' )
         end if  ! ( icount>0 )

         write(irefwr,*) 'End mshparncorr5'

      end if  ! ( debug )

      end
