      subroutine mshpiplintets8 ( npobot, nelbot, kmeshc, ikelm, iwork, &
                                  inpelm, kmscbt, klp, nlp, kmscps, &
                                  istep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-12-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill bottom layer of elements in case of a pipe consisting of
!     linear tetrahedrons
!     Each wedge is subdivided into 8 tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     element
!     topology
!     pipe
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npobot, nelbot, kmeshc(*), ikelm, iwork(*), inpelm, &
              kmscbt(*), klp(2,*), nlp, istep, kmscps(*)

!     ikelm         i/o   Last position used in kmeshc
!     inpelm         i    number of points per element
!     istep          i    Step size
!     iwork               integer work array
!     klp            i    In this array the relative node numbers with respect
!                         to array kbndbt of first and last point of a
!                         consecutive part are stored. The number
!                         of parts the boundary consists of is nlp
!                         Hence klp(1,1) = 1, klp(2,1) denotes the end point of
!                         the first part of the boundary
!                         klp(1,2) contains the node number of the first point
!                         of the second part. This must be klp(2,1)+1
!                         klp(2,nlp) contains the length of kbndbt
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmscbt         i    Kmesh part c corresponding to bottom surface
!                         Contains the local nodal point numbers of the bottom
!                         surface
!     kmscps         i    Kmesh part c corresponding to pipe surface
!                         This array is required to check the direction of
!                         diagonals in the pipe surface
!     nelbot         i    Number of elements in bottom surface
!     nlp            i    Number of disjoint parts the boundary of the bottom
!                         surface consists of
!     npobot         i    Number of points of bottom surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mcentr, ikelbt, iielem, k, i1(3), i, j1, j2, &
              iunder, iupper, ind(3), klpinf(3), ielpsf, ikelsf, ilp
      logical bound(3), small

!     bound          Indicates if boundary elements are present (true) or not
!                    (false)
!     i              Counting variable
!     i1             Array to store the nodes of one bottom triangle
!     ielpsf
!     iielem         Relative element number in element group
!     ikelbt         Last position used in kmscbt
!     ikelsf
!     ilp            Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     iunder         Lower bound of array
!     iupper         Upper bound of array
!     j1             Lower index number along second direction of a face
!     j2             Upper index number along second direction of a face
!     k              Counting variable
!     klpinf
!     mcentr         Counter for centroids of wedges
!     small          Parameter small indicates the direction:
!                    small=true:  from i1(1) to i1(2)+istep
!                    small=false: from i1(2) to i1(1)+istep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind /2,3,1/
! ======================================================================
!
      call eropen ( 'mshpiplintets8' )

      istep = npobot+nelbot
      mcentr = npobot
      ikelbt = 0
      do iielem = 1, nelbot

!     --- Loop over elements in bottom cross-section

         mcentr = mcentr+1
         do k = 1, 3
            i1(k) = kmscbt(ikelbt+k)
         end do
         ikelbt = ikelbt+3

!        --- mcentr contains the node number of the centroid point
!            i1 contains the node numbers of the corresponding triangle
!            in the bottom surface
!            Check if sides belong to outer boundary or not

         do k = 1,  3
            bound(k) = iwork(i1(k)).gt.0 .and. iwork(i1(ind(k))).gt.0
         end do

!        --- Check if nodes on a side are subsequent boundary nodes
!            Only then we have an actual boundary

         do k = 1, 3
            if ( bound(k) ) then

!           --- bound(k) = true, hence side k contains two boundary points

               j1 = iwork( i1(k) )
               j2 = iwork( i1(ind(k)) )
               iunder = min ( j1,j2 )
               iupper = max ( j1,j2 )
               if ( iupper.ne.iunder+1 ) then

!              --- iupper # iunder+1, hence the points are not subsequent.
!                  Check if both points are begin and end points

                  bound(k) = .false.
                  do ilp = 1, nlp
                     if (iunder.eq.klp(1,ilp)-ilp+1 .and. &
                         iupper.eq.klp(2,ilp)-ilp+1) then

!                    --- store loop for this element

                        bound(k) = .true.
                        klpinf(k) = ilp

                     end if
                  end do
               end if
            end if
         end do

!        --- Fill elements
!            All elements have the centroid point mcentr as starting point

         do k = 1, 8
            kmeshc(ikelm+1+(k-1)*inpelm) = mcentr
         end do

!        --- Under and upper element

         do k = 1, 3
            kmeshc(ikelm+1+k) = i1(4-k)
            kmeshc(ikelm+inpelm+1+k) = i1(k)+istep
         end do
         ikelm = ikelm+2*inpelm

!        --- Side elements

         do i = 1,3

!        --- Test direction of diagonal
!            Parameter small indicates the direction:
!            small = true:  from i1(1) to i1(2)+istep
!            small = false: from i1(2) to i1(1)+istep

            if ( bound(i) ) then

!           --- bound(i) true, hence boundary i is outer boundary
!               in that case the sequence in kbndbt gives the boundary
!               direction
!               ielpsf is minimum node number at pipe surface
!               ikelsf is start adress in kmscps minus one

               j1 = iwork( i1(i) )
               j2 = iwork( i1(ind(i)) )
               iunder = min ( j1,j2 )
               iupper = max ( j1,j2 )

!              --- what loop ?

               ilp = klpinf(i)
               if ( iupper.eq.iunder+1 ) then
                  ielpsf = j1
               else if ( iupper.eq.klp(2,ilp)-ilp+1 .and. &
                         iunder.eq.klp(1,ilp)-ilp+1 ) then
                  ielpsf = iupper
                  j2 = iupper+1
               else
                  ielpsf = j2
                  j2 = j2+1
               end if
               ikelsf = 6 * (ielpsf-1)
               if ( kmscps(ikelsf+4).eq.j2 ) then
                  small = .true.
               else
                  small = .false.
               end if

            else

!           --- internal boundary; check smallest node

               small = i1(i).lt.i1(ind(i))

            end if

            if ( small ) then

!           --- small = true

               kmeshc(ikelm+2) = i1(i)
               kmeshc(ikelm+3) = i1(ind(i))+istep
               kmeshc(ikelm+4) = i1(i)+istep
               kmeshc(ikelm+inpelm+2) = i1(ind(i))
               kmeshc(ikelm+inpelm+3) = kmeshc(ikelm+3)
               kmeshc(ikelm+inpelm+4) = i1(i)

            else

!           --- small = false

               kmeshc(ikelm+2) = i1(i)
               kmeshc(ikelm+3) = i1(ind(i))
               kmeshc(ikelm+4) = i1(i)+istep
               kmeshc(ikelm+inpelm+2) = kmeshc(ikelm+3)
               kmeshc(ikelm+inpelm+3) = kmeshc(ikelm+3)+istep
               kmeshc(ikelm+inpelm+4) = i1(i)+istep

            end if
            ikelm = ikelm+2*inpelm
         end do

      end do

      call erclos ( 'mshpiplintets8' )

      end
