      subroutine mshshiftnod4 ( iworkpoint, coornew, iwork, &
                                istarticross, skipnod, eps, shifted, &
                                indelm, map, fluidnd, inelem1 )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  3.1  date 22-11-2011 Debug information
!        version  3.0  date 22-06-2004 Extra parameters
!        version  2.1  date 23-04-2003 Small adjustments
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Shift node of intersecting curve onto boundary if
!     node is close (depending on eps) to boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), iworkpoint(*), &
              istarticross, skipnod(*), &
              shifted(*), indelm(*), map(*), fluidnd(*), inelem1
      double precision coornew(ndim,*), eps

!     coornew       i/o   Array containing the new coordinates of the nodes
!     eps            i    Local accuracy
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     indelm         o    Array of length nelem in which elements are marked
!                         of which at least one node is shifted
!     inelem1        i    ?
!     istarticross   i    Starting address of obstacle in array icrossobst
!     iwork          i    integer work array of size nelem
!                         iwork(ielem) contains the starting address of element
!                         ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!     iworkpoint    i/o   Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 3 if node i on obstacle
!     map            i    Mapping array that is filled if ichoice = 2 or must
!                         be filled if ichoice = 3
!                         map is an integer array of length 5 with the following
!                         contents:
!                         1: mmimap  memory management reference to array imap
!                         2: 130 to indicate the type of array
!                         3: mmrmap  memory management reference to array rmap
!                         4: mmcoor1  memory management reference to array coor1
!                         5: mmcoor2  memory management reference to array coor2
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
!     skipnod        i    Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  istart, icross, nodebegin, nodeend, ipstart, i, iedge, &
               ireledge, nintersect, ielgrp, inpelm, ielem, iielem
      double precision fact

!     fact           Interpolation factor
!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     iedge          Edge sequence number
!     ielem          Sequence number of element
!     ielgrp         Sequence number of element group
!     iielem         Sequence number of element per specific element group
!     inpelm         Number of nodes per element
!     ipstart        Starting address in array
!     ireledge       Relative edge sequence number
!     istart         First position used at a part
!     nintersect     Number of intersections found
!     nodebegin      Node at the begin of a crossed edge
!     nodeend        Node at the end of a crossed edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHGETELM      Check for coupling elements (2D)
!     MSHGETELM3D    Check for coupling elements (3D)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     eps := 0.499
!     do i = 1,number of intersections
!        iedge := edge which is intersected
!        fact := interpolation factor
!        if ( cross section with edge close (depending on eps)
!             to begin point of edge ) then
!           shift corresponding node and mark node as on boundary
!        else if ( cross section with edge close (depending on eps)
!                  to end point of edge ) then
!           shift corresponding node and mark node as on boundary
!        end if
!     end do
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshiftnod4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshiftnod4'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem=0
      istart=0
      nintersect=0

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)

         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm, ishape, inelem', &
                             ielgrp, inpelm, ishape, inelem

!        --- loop over elements

         if (ishape==3 .or. ishape==7 .or. &
             ishape==11.or. ishape==16)  then
            do iielem=1, inelem

               ielem=ielem+1

               if ( debug ) &
                  write(irefwr,1) 'ielem, iwork(ielem)', &
                                   ielem, iwork(ielem)
               if (iwork(ielem)/=0) then
                  istart = iwork(ielem)
                  do i = 1,icrossobst(istart)

                     ireledge = icrossobst(istart+i)
                     icross  = istarticross+1+2*(ireledge-1)
                     iedge = icrossobst(icross)
                     fact  = rcrossobst(icrossobst(icross+1))
                     ipstart = nmeshc(iedge)

                     if ( debug ) then
                        write(irefwr,1) 'i, ireledge, icross, iedge', &
                                         i, ireledge, icross, iedge
                        write(irefwr,2) 'fact', fact
                        write(irefwr,1) 'nodes', &
                           nmeshc(ipstart+1:ipstart+2)
                     end if  ! ( debug )

                     if ( fact>1d0-eps ) then

                        nodeend   = nmeshc(ipstart+1)
                        if (inpelm==3.or. inpelm==4) then
                           nodebegin = nmeshc(ipstart+2)
                        else
                           nodebegin = nmeshc(ipstart+3)
                        end if
                        if (debug) then
                           write(irefwr,1)'nodebegin, nodeend', &
                                           nodebegin, nodeend
                           write(irefwr,1)'skipnod', &
                              skipnod(nodebegin), skipnod(nodeend)
                        end if  ! (debug)

!                       --- if one of the two nodes of the edge has already been
!                           shifted then don't shift the other node

                        if (  (skipnod(nodebegin)==0 .and. &
                             (skipnod(nodeend)==0 .or. &
                              skipnod(nodeend)==1)).or. &
                             (skipnod(nodebegin)==1.and. &
                              skipnod(nodeend)==1) ) then

                           if (debug) then
                              write(irefwr,2)'coors begin', &
                                         coornew(1,nodebegin), &
                                         coornew(2,nodebegin)
                              write(irefwr,2)'coors end', &
                                         coornew(1,nodeend), &
                                         coornew(2,nodeend)
                           end if

                           coornew(1,nodebegin) = (1d0-fact)* &
                                          coornew(1,nodeend) &
                                          +fact* &
                                           coornew(1,nodebegin)

                           coornew(2,nodebegin) = (1d0-fact)* &
                                            coornew(2,nodeend)+fact* &
                                            coornew(2,nodebegin)
                           if (debug) then
                              write(irefwr,2)'coors begin (new)', &
                                         coornew(1,nodebegin), &
                                         coornew(2,nodebegin)
                           end if

                           if (ndim==3) then
                              coornew(3,nodebegin) = &
                                    (1d0-fact)*coornew(3,nodeend)+ &
                                          fact*coornew(3,nodebegin)
                           end if

                           iworkpoint(nodebegin) = 3
                           skipnod(nodebegin)=3
                           shifted(nodebegin)=1
                           map(nodebegin)=-1

!                          --- loop over the elements

                           if (ndim==2) then

                              call mshgetelm ( nmeshe, kmeshd, fluidnd, &
                                               inelem1, npoint, &
                                               nodebegin, indelm, &
                                               shifted )

                           else if (ndim==3)then

                              call mshgetelm3d( nmeshe, kmeshd, fluidnd, &
                                               inelem1, npoint, &
                                               nodebegin, indelm, &
                                               shifted)

                           end if

                        end if

                     else if ( fact<=eps ) then

                        nodeend   = nmeshc(ipstart+1)
                        if (inpelm==3.or. inpelm==4) then
                           nodebegin = nmeshc(ipstart+2)
                        else
                           nodebegin = nmeshc(ipstart+3)
                        end if
                        if (debug) then
                           write(irefwr,1)'nodebegin, nodeend', &
                                           nodebegin, nodeend
                           write(irefwr,1)'skipnod', &
                              skipnod(nodebegin), skipnod(nodeend)
                        end if  ! (debug)

!                       --- if one of the two nodes of the edge has already been
!                           shifted then don't shift the other node

                        if (((skipnod(nodebegin)==0 .or. &
                             skipnod(nodebegin)==1) .and. &
                             skipnod(nodeend)==0) .or. &
                             (skipnod(nodebegin)==1.and. &
                              skipnod(nodeend)==1) ) then

                           if (debug) then
                              write(irefwr,2)'coors begin', &
                                         coornew(1,nodebegin), &
                                         coornew(2,nodebegin)
                              write(irefwr,2)'coors end', &
                                         coornew(1,nodeend), &
                                         coornew(2,nodeend)
                           end if

                           coornew(1,nodeend) =((1d0-fact)* &
                                             coornew(1,nodeend) &
                                             +fact* &
                                              coornew(1,nodebegin))

                           coornew(2,nodeend) = ((1d0-fact)* &
                                             coornew(2,nodeend) &
                                            +fact* &
                                             coornew(2,nodebegin))
                           if (ndim==3) then
                              coornew(3,nodeend) = (1d0-fact)* &
                                             coornew(3,nodeend) &
                                             +fact* &
                                              coornew(3,nodebegin)
                           end if

                           if (debug) then
                              write(irefwr,2)'coors end (new)', &
                                         coornew(1,nodeend), &
                                         coornew(2,nodeend)
                           end if

                           iworkpoint(nodeend) = 3
                           skipnod(nodeend)=3
                           shifted(nodeend)=1
                           map(nodeend)=-1

!                          --- loop over the elements

                           if (ndim==2) then

                              call mshgetelm ( nmeshe, kmeshd, fluidnd, &
                                               inelem1, npoint, nodeend, &
                                               indelm, shifted )

                           else if (ndim==3)then

                              call mshgetelm3d( nmeshe, kmeshd, fluidnd, &
                                               inelem1, npoint, &
                                               nodeend, indelm, &
                                               shifted)

                           end if

                        end if

                     end if

                  end do

                  nintersect=nintersect+icrossobst(istart)

               end if

            end do

         else

            ielem=ielem+inelem

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshiftnod4'

      end if  ! ( debug )
      call erclos ( 'mshshiftnod4' )

      end
