      subroutine mshshiftnod6a ( shifted, kmeshe, npoint, bndnod, &
                                 fluidnd, ilast )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 23-04-2003
!
!   copyright (c) 2002-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Find consequent order of vertex nodes on 2D obstacle
!       (fluidnd is partly filled)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshe(*), npoint, shifted(*), bndnod, ilast, fluidnd(*)

!     bndnod         i    Node that lies on boundary of the fluid domain and
!                         on the obstacle boundary
!     fluidnd        o    Array that contains the nodal numbers of the coupling
!                         elements in case of fluid-structure computations
!     ilast          o    Number of coupling elements in case of fluid-structure
!                         computations
!     kmeshe         i    STANDARD SEPRAN array containing information on
!                         nodal points connected to nodal points
!     npoint         i    Number of nodal points
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, j, k, inode, jnode(npoint,4), bndnodes(npoint), &
               nrnodes, oldnod, nrshft, startnode
      logical debug

!     bndnodes       Array containing the node numbers of the nodes that
!                    are shifted onto the obstacle
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     inode          Node sequence number
!     j              Counting variable
!     jnode          Array containing the node numbers of an element
!     k              Counting variable
!     nrnodes        Number of nodes connected to a node
!     nrshft         Number of nodes that are shifted onto the obstacle
!     oldnod         Help variable
!     startnode      Starting node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshiftnod6a' )

      debug=.false.

!     --- Get nodes that are shifted onto the obstacle

      nrshft=0
      k=1
      do i=1,npoint
         if (shifted(i).eq.1) then
            nrshft=nrshft+1
            bndnodes(k)=i
            k=k+1
         end if
      end do

      if (debug) write(*,*)'nrshft', nrshft

!     --- Compute the number of nodes that lie on the
!         obstacle and initialize arrays
!         (nrshft is the number of vertex nodes on obstacle)

      ilast=2*nrshft-1
      do i=1,ilast
         fluidnd(i)=0
         do j=1,4
           jnode(i,j)=0
         end do
      end do

!     --- Determine which nodes are connected to which nodes

      if (debug) write(*,*)'Fill array jnode'

!     --- Loop over shifted nodes

      do i=1,nrshft
         k=0
         if (debug) write(*,*)'bndnodes  ',bndnodes(i)

!        --- Number of nodes connected to node i

         nrnodes=kmeshe(bndnodes(i)+1)-kmeshe(bndnodes(i))
         do j=1,nrnodes
            inode=kmeshe(npoint+1+kmeshe(bndnodes(i))+j)

!           --- If connecting node is also shifted

            if (shifted(inode).eq.1) then
               k=k+1

!              --- store node

               jnode(i,k+1)=inode

               if (debug) &
               write(*,*)'node ',bndnodes(i), '  connected to ', inode
            end if
         end do

!        --- Store number of nodes that are connected to i and are also shifted

         jnode(i,1)=k

!        --- Node only connected to one node (end node found)

         if (k.eq.1) startnode=i

      end do

!     --- find the nodes on the obstacle in consequent order

      if (debug) write(*,*)'Find consequent order of nodes on obstacle'
      j=startnode
      fluidnd(1)=bndnodes(j)
      do i=3,ilast,2

         if (jnode(j,1).eq.1) then

            if (debug) write(*,*)'one node found', jnode(j,2)

!           --- store node in fluidnd

            fluidnd(i)=jnode(j,2)

         else if (jnode(j,1).eq.2) then

            if (debug) then
               write(*,*)'two nodes found', jnode(j,2), jnode(j,3)
            end if

!           --- store node in fluidnd

            if (jnode(j,2).eq.oldnod) then
               fluidnd(i)=jnode(j,3)
            else if (jnode(j,3).eq.oldnod) then
               fluidnd(i)=jnode(j,2)
            end if

         else if (jnode(j,1).eq.3) then

            if (debug) then
               write(*,*)'three nodes found', &
                          jnode(j,2), jnode(j,3), jnode(j,4)
            end if

!           --- store node in fluidnd

            do k=2,4
               if ( bndnodes(jnode(j,k)).ne.bndnod .and. &
                    bndnodes(jnode(j,k)).ne.oldnod ) then
                  fluidnd(i)=jnode(j,k)
               end if
            end do

         end if

         oldnod=fluidnd(i-2)
         do k=1,nrshft
            if (bndnodes(k).eq.fluidnd(i)) j=k
         end do

      end do

      if (debug) then
         do i=1,ilast
            write(*,*)'nodes  ',i, fluidnd(i)
         end do
      end if

      call erclos ( 'mshshiftnod6a' )

      end
