      subroutine refinenew ( ichoice, iinput, &
                             rinput, rinputavail, maploc, fillmap )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 05-06-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     refine an existing Sepran mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, iinput(*)
      double precision rinput(*)
      logical rinputavail, fillmap
      type (mapping) :: maploc

!     fillmap        i    If true array imap must be filled
!     ichoice        i    choice parameter   possibilities:
!                         ichoice = ntimes + 100 * itopol + 1000 * jrenum
!                         with:
!                         ntimes   Indicates how many times the mesh must be
!                                  refined
!                         itopol   Indicates if subroutine topol must be called
!                                  0  subroutine topol is called
!                                     (standard situation)
!                                  1  subroutine topol is skipped (must only be
!                                     used when the mesh will be written by
!                                     meshwr)
!                         jrenum   Indicates the type of renumbering to be
!                                  performed
!                                  jrenum + 1, is equal to the parameter
!                                  jrenum in TOPOL
!     iinput         i    Output array of subroutine MESH
!     maploc         o    contains information of the mapping
!     rinput         i    Real input array
!     rinputavail    i    If true, the input array rinput is available
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leniinp, lenrinp, itopol, jrenum(1), i, ntimes, &
              numrowkmesht
      double precision :: timebefore(2)

!     i              General loop variable
!     itopol         Indicates if array TOPOL must be called, see ichois
!     jrenum         Parameter jrenum for TOPOL
!     leniinp        Length of array iinput
!     lenrinp        Length of array rinput
!     ntimes         number of subdivisions
!     numrowkmesht   number of rows in kmesh part t
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHREFINE      Refine an existing mesh one times by putting new nodes on
!                    all edges in the middle of existing nodes
!     PRININ1        print 2d integer array
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     543   NTIMES has wrong value (<1 or>99)
!    1182   ITOPOL has wrong value (<0 or>1)
!    1277   Array IINPUT has not been filled correctly
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'refinenew' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from refinenew'
         write(irefwr,1) 'ichoice', ichoice
         write(irefwr,1) 'kelmh, kmiinput', kelmh, kmiinput
  1      format ( a, 1x, (10i6) )

      end if

      jrenum(1) = ichoice / 1000
      itopol = ( ichoice - 1000 * jrenum(1) ) /100
      ntimes = mod ( ichoice, 100 )

      if ( jrenum(1)==0 ) then
         jrenum(1) = 9
      else
         jrenum(1) = jrenum(1) + 1
      end if

      if ( itopol<0 .or. itopol>1 ) then
         call errint ( itopol, 1 )
         call errsub ( 1182, 1, 0, 0 )
      end if

      if ( ntimes<=0 ) then
         call errint ( ntimes, 1 )
         call errsub ( 543, 1, 0, 0 )
      end if
      if ( debug ) then

         write(irefwr,1) 'itopol, ntimes, jrenum', &
                          itopol, ntimes, jrenum(1)
         numrowkmesht = nvolms
         if ( numrowkmesht==0 ) numrowkmesht = nsurfs
         if ( numrowkmesht==0 ) numrowkmesht = ncurvs
         call prinin1 ( kmesht, numrowkmesht, 2, 'kmesht' )

      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

!     --- Check whether array IINPUT has to be checked and if so check

      if ( kelmh/=0 .and. kmiinput==0 ) then

         if ( iinput(1)<21 .or. iinput(1)>10000 .or. &
              iinput(3)<21 .or. iinput(3)>iinput(1) .or. &
              iinput(6)< 1 .or. iinput(6)>3  ) then
              call errsub ( 1277, 0, 0, 0 )
              go to 1000
         end if

         do i = 1, ntimes

            call mshrefine ( iinput(6), &
                             rinput(6), rinputavail, maploc, fillmap )
            if ( ierror/=0 ) go to 1000

            if ( debug ) then

               write(irefwr,1) 'refinement', i
               call prinin1 ( kmesht, numrowkmesht, 2, 'kmesht' )

            end if  ! ( debug )

         end do  ! i = 1, ntimes

      else

!     --- Arrays iinput and rinput are extracted from kmesh

         leniinp = size ( iinputkm )
         lenrinp = size ( rinputkm )

         allocate ( iinputmesh(leniinp), rinputmesh(lenrinp), stat = error )
         if ( error/=0 ) call eralloc ( error, lenrinp+leniinp, 'iinputmesh' )
         if ( ierror/=0 ) go to 1000

         iinputmesh = iinputkm
         rinputmesh = rinputkm
         if ( ierror/=0 ) go to 1000

         do i = 1, ntimes

            call mshrefine ( iinputmesh, rinputmesh, .true., maploc, fillmap )
            if ( ierror/=0 ) go to 1000

            if ( debug ) then

               write(irefwr,1) 'refinement', i
               call prinin1 ( kmesht, numrowkmesht, 2, 'kmesht' )

            end if  ! ( debug )

         end do  ! i = 1, ntimes

         deallocate ( iinputmesh, rinputmesh, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iinputmesh' )
         if ( ierror/=0 ) go to 1000

      end if
      call seprelkmesh(kmesh)

!     --- Renumber if necessary

      if ( itopol==0 ) call topol ( jrenum )

1000  if ( debug ) write(irefwr,*) 'End refinenew'
      call erclmn ( 'refinenew', timebefore )

      end subroutine refinenew
