      subroutine sepob3dmarkpt1 ( iobst, inpelm )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reduced version of subroutine mshob3dmarkpt in case of open
!     obstacles lying in x, y or z direction
!     Old subroutine mshob3dmarkpt1
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, inpelm

!     inpelm         i    number of points per element in obstacle
!     iobst          i    Obstacle sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: inode, numpntobst
      integer, allocatable :: iwork(:)
      double precision xmin, xmax, ymin, ymax, zmin, zmax, x, y, z, &
                       dx, dy, dz, eps, xminorig, yminorig, zminorig

!     dx             step in x-direction
!     dy             step in y-direction
!     dz             length of vector in z-direction
!     eps            Accuracy
!                    Points that are at a distance less than eps of the
!                    obstacle are defined as part of the obstacle
!     inode          Node sequence number
!     iwork          Work array of length npoint to store the boundary
!                    nodes temporarily
!     numpntobst     Number of nodal points inside the obstacle
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     xminorig       Minimum x-value to compute ipcell, jpcell
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     yminorig       Minimum y-value to compute ipcell, jpcell
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
!     zminorig       Minimum z-value to compute ipcell, jpcell, kpcell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract xmin,xmax,ymin,ymax, zmin, zmax from obstacle array
!    numpntobst := 0
!    For inode := 1 (1) npoint do
!       (x,y,z) := coordinates of node
!       If ( (x,y,z) inside (xmin,xmax;ymin,ymax;zmin,zmax) )
!             numpntobst := numpntobst+1
!             ipointobstpart(numpntobst) := inode
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepob3dmarkpt1' )
      if ( inpelm/=3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- initialize parameters

      numpntobst = 0

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Extract xmin,xmax,ymin,ymax, zmin, zmax from obstacle array

      xminorig = rmeshx(1,iobst)
      xmax     = rmeshx(2,iobst)
      yminorig = rmeshx(3,iobst)
      ymax     = rmeshx(4,iobst)
      zminorig = rmeshx(5,iobst)
      zmax     = rmeshx(6,iobst)
      dx   = rmeshx(7,iobst)
      dy   = rmeshx(8,iobst)
      dz   = rmeshx(9,iobst)

      eps = min(dx,dy,dz)*0.01d0
      xmin = xminorig-eps
      xmax = xmax    +eps
      ymin = yminorig-eps
      ymax = ymax    +eps
      zmin = zminorig-eps
      zmax = zmax    +eps

!     --- For inode := 1 (1) npoint do

      do inode = 1, npoint

!     --- (x,y) := coordinates of node

         x = coor(1,inode)
         y = coor(2,inode)
         z = coor(3,inode)
         if ( x>=xmin .and. x<=xmax .and. &
              y>=ymin .and. y<=ymax .and. &
              z>=zmin .and. z<=zmax ) then
                  numpntobst = numpntobst+1
                  iwork(numpntobst) = inode
         end if
       end do

!      --- Fill three positions of ipointobstpart
!          At this moment position 2 is set to zero, it is filled in subroutine
!          mshobstmarkel

      ipointobstpart(1) = numpntobst
      ipointobstpart(2) = 0
      ipointobstpart(3) = 0
      ipointobstpart(4) = 0
      ipointobstpart(5:4+numpntobst) = iwork(1:numpntobst)

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'sepob3dmarkpt1' )
      end
