      subroutine seprefloc ( jcrit, user, intarray )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     refine an existing Sepran mesh locally by a criterion as given
!     in either array ICRIT, USER or as prescribed in subroutine REFUSE
!     Old subroutine refloc
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     2d
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jcrit, intarray(*)
      double precision user(*)

!     intarray   i    indicator array, with the following para-
!                     meters
!                     pos. 1 : length of array intarray used in present
!                              computation. If intarray(1) = 0 then all
!                              default values are used.
!                     pos. 2 : Indicator whether the local criterion is
!                              - given in array   JCRIT         (=1)
!                              - given in subroutine REFUSE     (=2)
!                              REFUSE is a user-written subroutine to
!                              fill the criterion-array. See example.
!                              Default value : intarray(2) = 2
!                     pos. 3 : Indicator for criterion.
!                              Relative means halving the edge sizes
!                              round(CRIT(i)) times.
!                              Example: for CRIT(*)=3 the length of new edge
!                                       is 1/8 of the original length
!                              Halving  : intarray(3) = 1
!                              Absolute means refine as long as local length
!                              between nodes i,j is>(CRIT(i)+CRIT(j))/2
!                              Example: for CRIT(*)=3 all edges must become
!                              shorter or equal to 3.
!                              Absolute : intarray(3) = 2
!                              Default value : intarray(3) = 2
!                     pos. 4 : Maximal number of refine-steps,
!                              Default value : intarray(4) = 10
!     jcrit      i    criterion array, having the structure of
!                     a Sepran solution array
!     user       i    helparray for user to pass information
!                     for his own crit array  (see REFUSE)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lncoor, lnkmsc, lnkmsm, irefto, &
              nelemo, jrefin, ncrit, &
              npold, irefus, irelat, k, irenum(1)
      integer, allocatable :: iwork(:), kmeshcnew(:), kmeshmnew(:)
      double precision, allocatable :: crit(:)
      double precision :: timebefore(2)

!     crit           criterion array, if irefus = 1 then an INPUT
!                    if irefus = 2 array to be filled by the user
!     irefto         Max number of refinements allowed
!     irefus         Indicator which type of refinement will be used
!     irelat         Indicator for relative or absolute criterion
!     irenum         Indication of the type of renumbering to be used
!     iwork          integer work array
!     jrefin         Number of refinements
!     k              general loop variable
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     lncoor         length coor array
!     lnkmsc         length kelem array
!     lnkmsm         length curve array
!     ncrit          number of items in the criterion array jcrit
!     nelemo         old number of elements (before refinement)
!     npold          new number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0007         copy array of type 110/115
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH414         Determine whether curves are filled in
!     MSH430         perform local refinement
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
!     SEPACTSOLBF2  compute pointers to degrees of freedom of vectors
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1207   array intarray position 1 not filled correctly
!     1208   value in intarray(2) not correct
!     1209   local refinement only available for ndim=2
!     1210   arrays not of supportrd structure
!     1295   wrong value for renumbering method
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check contents of array intarray and KMESH
!     Be sure that coordinates, curves and elements arrays are in IBUFFR
!     If criterion array is already existent copy to present
!     positions
!     determine length of coordinates array : ( kmesh(23) )
!     determine length elements array : ( kmesh(17) )
!     determine length curves array : ( kmesh(27) )
!     Set maximal number of Refinements
!     Make refinement loop
!         determine startpositions for help-arrays
!         Set starting addresses COOR, KELEM and CURVES
!         Set all starting addresses of new arrays in help array
!         Set extra length for new arrays
!         Copy criterion array
!         If possible copy criterion from last step
!         Refine call msh430
!         Store new arrays for elements, coordinates and curves
!         on a permanent position
!         Store criterion array temporary on a permanent position
!         Check whether refinement is ready
!         If ready goto end of routine
!     end refinement loop
!     Ready with Refinement
!     Delete Criterium array
!     If needed renumber
!     Decrease all priorities
!     Close routine
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprefloc' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprefloc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( intarray(1)<0 .or. intarray(1)>5 ) then

           call errint ( intarray(1), 1 )
           call errwar ( 1207, 1, 0, 0 )

      end if

      if ( intarray(1)<2 ) then

           irefus = 2

      else

           irefus = intarray(2)
           if ( irefus<1 .or. irefus>2 ) then

                call errint ( irefus, 1 )
                call errsub ( 1208, 1, 0, 0 )
                go to 1000

           end if

      end if

      if ( intarray(1)<3 ) then

           irelat = 2

      else

           irelat = intarray(3)

      end if

      if ( ndim/=2 ) then
           call errsub ( 1209, 0, 0, 0 )
           go to 1000
      end if

!     --- If criterion array is already existent copy to present
!         positions

      if ( irefus==1 ) then

!     --- Fill positions corresponding to solution array

           ncrit = ks(jcrit)%nusol

           if ( ks(jcrit)%typevec/=110 .and. ks(jcrit)%typevec/=115 ) &
                call errsub ( 1210, 0, 0, 0 )

!          --- Fill indprfi and indprpi

           call sepactsolbf2 ( jcrit )

      else
         indprfi = 0
         indprpi = 0
      end if

!     --- determine length of arrays

      lncoor  = size (coor,2)
      lnkmsc = size ( kmeshc )
      lnkmsm  = size ( kmeshm )

!     --- Set maximal number of Refinements

      if ( intarray(1)<4 ) then

           irefto = 10

      else

           irefto = intarray(4)

      end if

      allocate ( crit(4*npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 4*npoint, 'crit' )
      if ( ierror/=0 ) go to 1000

      do jrefin = 1, irefto

          nelemo = nelem

          lncoor = 4 * lncoor
          lnkmsc = 4 * lnkmsc
          lnkmsm = 2 * lnkmsm

          allocate ( iwork(ncurvs), coornew(2,lncoor), kmeshcnew(lnkmsc), &
                     kmeshmnew(lnkmsm), stat = error )
          if ( error/=0 ) &
             call eralloc ( error, ncurvs+lncoor+lnkmsc+lnkmsm, &
                            'iwork' )
          if ( ierror/=0 ) go to 1000

!         --- Set starting addresses COOR, KELEM and CURVES

          if ( irefus==1 ) usoltemp => ks(jcrit)%sol

!         --- Set extra length for new arrays

          call msh414 ( kmeshm, ncurvs, iwork )

!         --- Copy criterion array from ibuffr(ipusol) to crit

          if ( irefus==1 .and. jrefin==1 ) &
             call er0007 ( ncrit, 1, usoltemp, crit )

          call msh430( crit, coornew, kmeshcnew, kmeshmnew, &
                       lncoor, lnkmsc, lnkmsm, npold, user, &
                       iwork, irefus, irelat, jrefin )

!         --- KMESH part c (elements)
!             Store this array on a permanent position

          deallocate ( km(iseqkm)%kmeshc )
          allocate ( km(iseqkm)%kmeshc(lnkmsc) )
          kmeshc => km(iseqkm)%kmeshc
          kmeshc = kmeshcnew(1:lnkmsc)
          km(iseqkm)%kmesh(17) = lnkmsc
          call sepfillkmeshc   ! Make relation between kmeshc and Topgrp

!         --- KMESH part i (coordinates)
!             Store on a permanent position

          deallocate ( km(iseqkm)%coor )
          allocate ( km(iseqkm)%coor(ndim,npoint) )
          coor => km(iseqkm)%coor
          coor = coornew(:,1:npoint)
          km(iseqkm)%kmesh(23) = lncoor

!         --- KMESH part m (curves)
!             Store on a permanent position

          deallocate ( km(iseqkm)%kmeshm )
          allocate ( km(iseqkm)%kmeshm(lnkmsm) )
          kmeshm => km(iseqkm)%kmeshm
          kmeshm = kmeshmnew(1:lnkmsm)
          km(iseqkm)%kmesh(27) = lnkmsm

!         --- Check whether refinement is finished

          deallocate ( iwork, coornew, kmeshcnew, kmeshmnew, stat = error )
          if ( error/=0 ) call erdealloc ( error, 'iwork' )
          if ( ierror/=0 ) go to 1000

          if ( nelem==nelemo ) goto 500

      end do

      deallocate ( crit, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'crit' )
      if ( ierror/=0 ) go to 1000

500   if ( intarray(1)<5 ) then
         irenum(1) = 2
      else
         irenum(1) = intarray(5)
         if ( irenum(1)<0 .or. irenum(1)>8 ) then
            call errint ( irenum(1), 1 )
            call errsub ( 1295, 1, 0, 0 )
         end if
      end if

!     --- Remove some subarrays of kmesh that are no longer valid

      call mshdelsel ( kmesh )

      call topol ( irenum )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprefloc'

      end if  ! ( debug )
      call erclmn ( 'seprefloc', timebefore )

      end subroutine seprefloc
