      subroutine transfbflin
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 15-11-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Transform an existing Sepran mesh of triangles and tetrahedrons
!    into a mesh of quadrilaterals and hexahedrons
!    All elements must be linear
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     mesh
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lncoor, lnkmsc, lnkmsm, lnkmsn, ibrp
      integer, allocatable :: kmeshnnew(:), kmeshmnew(:), kmeshcnew(:)
      double precision, allocatable :: coornew(:)

!     coornew        Array containing the new coordinates of the nodes
!     ibrp           Estimated number of neighbours of one point
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     kmeshnnew      New array kmeshn
!     lncoor         declared length for coor, resp coorh
!     lnkmsc         length of kelem array (kmesh part c)
!     lnkmsm         length of curves array (kmesh part m)
!     lnkmsn         length of surfaces array (kmesh part n)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH440         Perform the actual transformation
!     SEPTRANSF1     Copy new kmeshc, coor, kmeshm and kmeshn to old arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       To transform Coordinates, Elements and Curves are needed
!       Be sure that all are in IBUFFR - array
!       Determine present length of the arrays
!       Determine startpositions for help-arrays , first the total
!       length needed has to be computed. Next divide over the arrays.
!       Set check variables length for new arrays
!       Perform actual transformation into quadrilaterals  (MSH440)
!       Store new arrays on a permanent position
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'transfbflin' )
      if ( ierror/=0 ) go to 1000

!     --- Leave routine if no surfaces are given

      if ( nsurfs==0 ) goto 1000

!     --- To transform Coordinates, Elements, Surfaces and Curves are needed
!          determine present length elements

      lnkmsc = size ( kmeshc )
      lnkmsm = size ( kmeshm )
      lnkmsn = max(1,size ( kmeshn ))

!     --- determine startpositions for help-arrays , first the total
!         length needed has to be computed

      if ( ndim==2 ) then

!     --- array kelemh
!         nelem triangles are transformed into 3*nelem quadrilaterals
!         length = 4 * length kelem integer positions

         lnkmsc = 4 * lnkmsc

!        --- array coorh, new length is determined
!            extra points on all lines and in all barycenters, hence <= &
!            ndim * ( npoint + nelem + 0.5 * ( 3 * nelem + npoint ) ) reals

         lncoor = 5 * nelem + 3 * npoint

!        --- array kromh
!            length<=2 * length kromme integer positions

         lnkmsm = 2 * lnkmsm

!        --- arrays ibrpnt and istart, to store temporarily the nodenumbers
!            of the extra points on the lines (for each line a position)

         ibrp   = 10

!        --- no space for triangles and surfaces needed

         lnkmsn = 0

      else if ( ndim==3 ) then

!     --- array coorh, new length is determined
!         extra points on all lines, in all barycenters and in all elements,
!         hence <= ndim * ( 7 * npoint + 4 * nelem  ) reals

         lncoor = 3 * ( 4 * nelem + 7 * npoint )

!        --- array kelemh
!            triangles are transformed into 3 * nelem quadrilaterals and
!            nelem tetrahedrons are transformed into 4 hexahedrons
!            length<=8 * length kelem integer positions

         lnkmsc = 8 * lnkmsc

!        --- array kromh
!            length<=2 * length kromme integer positions

         lnkmsm = 2 * lnkmsm

!        --- array surfh
!            length<=4 * length surf integer positions

         lnkmsn = 4 * lnkmsn

!        --- arrays ibrpnt and istart, to store temporarily the nodenumbers of
!            the new points on the lines (for each line one position)
!            length =  41 * npoint integers

         ibrp   =  40

      end if

!     --- Make reservation

      allocate ( kmeshnnew(lnkmsn), kmeshmnew(lnkmsm), coornew(lncoor), &
                 kmeshcnew(lnkmsc), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lnkmsn+lnkmsm+lncoor+lnkmsc, 'kmeshnnew' )
      if ( ierror/=0 ) go to 1000

!     --- Perform actual transformation into quadrilaterals

      call msh440 ( coornew, kmeshcnew, kmeshmnew, kmeshnnew, &
                    lncoor, lnkmsc, lnkmsm, lnkmsn, ibrp   )

!     --- Copy to old arrays and recreate these arrays

      call septransf1 ( lnkmsc, lncoor, lnkmsm, lnkmsn, kmeshcnew, &
                        kmeshmnew, kmeshnnew, coornew )

      deallocate ( kmeshnnew, kmeshmnew, kmeshcnew, coornew, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshnnew' )

1000  call erclos ( 'transfbflin' )

      end subroutine transfbflin
