      subroutine transftria7
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 15-11-2012 Long integers
!        version  1.1  date 17-05-2010 New call to ini082
!        version  1.0  date 15-11-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend mesh consisting of triangles with extra node in center
!
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenkmeshcext, lencoorext, lenkmeshc, lencoor, &
              inpelm, istart, ipkmscold, ipkmscnew, ielgrp
      integer, allocatable :: kmeshcold(:)
      double precision, allocatable :: coorold(:,:)

!     coorold        Array containing the coordinates in the old mesh
!     ielgrp         element group number
!     inpelm         number of points per element
!     ipkmscnew      Starting address of array KMESH part c (new mesh)
!     ipkmscold      Starting address of array KMESH part c (old mesh)
!     istart         First position used at a part
!     kmeshcold      Old kmesh part c
!     lencoor        Length of array COOR (old mesh) (in doubles)
!     lencoorext     Extra length of new array coor (in doubles)
!     lenkmeshc      length of array kmeshc
!     lenkmeshcext   Extra length of new array kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHD7          Places coordinates of centroids of 7-point triangles
!                    in array coor
!     MSHTRIANCENT2  Copy contents of array kmeshcold into kmeshcnew
!                    Extend with center
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'transftria7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from transftria7'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)     ! shape number of element
         if ( ishape>2 .and. ishape/=4 ) then

!        --- Error: surface element not of shape 4

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )

         end if  ! ( ishape>2 .and. ishape/=4 )

      end do  ! ielgrp = 1, nelgrp
      if ( ierror/=0 ) go to 1000

!     --- Compute number of extra nodes in mesh
!         Compute new length of coor and kmeshc

      if ( nelgrp==1 ) then

!     --- Only one element group

         if ( ishape/=4 ) then

!        --- Error: no surface elements

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if  ! ( ishape/=4 )

         lenkmeshcext = nelem
         lencoorext = nelem*ndim

      else

         lenkmeshcext = 0
         lencoorext = 0
         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)     ! shape number of element
            if ( ishape==4 ) then
               inelem = kmeshg(ielgrp)     ! number of elements in group
               lenkmeshcext = lenkmeshcext+inelem
               lencoorext = lencoorext+inelem*ndim
            end if  ! ( ishape==4 )

         end do  ! ielgrp = 1, nelgrp

         if ( lenkmeshcext==0 ) then

!        --- Error: no surface elements

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if  ! ( ishape/=4 )

      end if  ! ( nelgrp==1 )

!     --- Extend the arrays kmeshc and coor and copy old parts
!         Copy old array kmeshc into kmeshcold

      lencoor = size(coor,2)
      lenkmeshc = size(kmeshc)
      allocate ( kmeshcold(lenkmeshc), coorold(ndim,lencoor),  stat = error )
      if ( error/=0 ) call eralloc ( error, lenkmeshc+lencoor, 'kmeshcold' )
      if ( ierror/=0 ) go to 1000

      kmeshcold = kmeshc
      coorold = coor

!     --- Create new arrays kmeshc and coor

      kelmm = lenkmeshc+lenkmeshcext
      deallocate ( km(iseqkm)%kmeshc )
      allocate ( km(iseqkm)%kmeshc(kelmm) )
      kmeshc => km(iseqkm)%kmeshc
      km(iseqkm)%kmesh(17) = kelmm

      kelmi = lencoor+lencoorext/ndim
      deallocate ( km(iseqkm)%coor )
      allocate ( km(iseqkm)%coor(ndim,kelmi) )
      coor => km(iseqkm)%coor
      kelmi = kelmi*ndim

      coor(:,1:lencoor) = coorold

!     --- Fill kmeshc for centers
!         Extend coor with centers

      inelem = nelem
      inpelm = npelm
      ipkmscold =1
      ipkmscnew = 1
      do ielgrp = 1, nelgrp
         if ( kelma/=0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
         if ( kelmg>0 ) inelem = kmeshg(ielgrp)
         ishape = kmeshf(ielgrp)
         if ( ishape/=4 ) then

!        --- Not an element of shape 4, just copy kmeshc

            length = inelem*inpelm
            kmeshc(ipkmscnew:ipkmscnew+length-1) = &
               kmeshcold(ipkmscold:ipkmscold+length-1)

         else

!        --- element of shape 4, extend part of kmeshc

            call mshtriancent2 ( kmeshc(ipkmscnew), kmeshcold(ipkmscold), &
                                 inpelm, inelem, npoint )

         end if  ! ( ishape/=4 )
         kmeshf(ielgrp) =  7
         if ( kelmb/=0 ) kmeshb(ielgrp) = 6

         call mshd7 ( kmeshc(ipkmscnew), coor, inelem, ndim )
         ipkmscnew = ipkmscnew+inelem*7
         ipkmscold = ipkmscold+inelem*inpelm

      end do  ! ielgrp = 1, nelgrp

      deallocate ( kmeshcold, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshcold' )
      if ( ierror/=0 ) go to 1000

      if ( kelmb<=0 ) then

!     --- kelmb<=0, extend kmesh with new kmesh part b

         allocate (km(iseqkm)%kmeshb(nelgrp))
         kmeshb => km(iseqkm)%kmeshb
         km(iseqkm)%kmesh(16) = nelgrp
         if ( kelma==0 ) then

!        --- kelma = 0, fill kmesh part b with 6 (number of boundary points)

            kmeshb(:) = 6

         else

!        --- kelma>0, fill kmesh part b with number of boundary points

            do ielgrp = 1, nelgrp
               inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
               kmeshb(ielgrp) = inpelm
            end do  ! ielgrp = 1, nelgrp

         end if  ! ( kelma<=0 )

      end if  ! ( kelmb<=0 )

      if ( kelma/=0 ) then

!     --- kelma>0, adapt kmesha

         istart = kmesha(1)
         do ielgrp = 1, nelgrp

            inpelm = kmesha(ielgrp+1)-istart
            istart = istart+inpelm
            ishape = kmeshf(ielgrp)
            if ( ishape==7 ) then
               inpelm = 7
               kmeshb(ielgrp) = 6
            end if  ! ( ishape==7 )
            kmesha(ielgrp+1) = kmesha(ielgrp)+inpelm

         end do  ! ielgrp = 1, nelgrp

      end if  ! ( kelma/=0 )

!     --- Put npoint into kmesh

      npelm = max(npelm,7)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End transftria7'

      end if  ! ( debug )
      call erclos ( 'transftria7' )

      end subroutine transftria7
