      subroutine mshadapcurv ( listcurvs, ncurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-09-2010 Remove work arrays
!        version  1.0  date 07-11-2009
!
!   copyright (c) 2009-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of curves when coordinates of user points
!     have been changed
!     The number of nodes and the relative subdivision is kept
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     change
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurves, listcurvs(ncurves)

!     listcurvs      i    List of curves that must be adapted
!     ncurves        i    Number of curves that must be adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurve, irnp, iposc, inpncv, &
              irelnodes(1), iinp, istartrel
      double precision, allocatable :: curstr(:)

!     curstr         Work array for subroutine mshcurcr
!     i              Counting variable
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     inpncv         Number of points in curve
!     iposc          last position used in array curves
!     irelnodes      Array with the same length as the first part of
!                    iinput (until surfaces)
!                    Relative node numbers of the user points in
!                    framecurv are filled at the same positions as
!                    the user points in iinput
!     irnp           position in array rinput
!     istartrel      dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHADAPCOORCRV Copy coordinates from curve stored in array curves into
!                    array coor
!     MSHCURCR       Create curve
!     MSHSEQCURVS    Compute sequence in which curves must be created
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3038   Array curstr has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
!     --- Find starting positions of arrays
!     --- Fill new coordinates in array curves
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadapcurv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapcurv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some parameters

      jcoars = iinputkm(11)
      irnp1 = 3
      irnp = irnp1+(ndim+jcoars)*nuspnt+2*jcoars-1

      istartrel = 0

!     --- Allocate arrays

      allocate ( iseqcurvs(ncurvs), ipointcurv(ncurvs), &
                 curstr(10*ncurvs), stat = error )
      if ( error/=0 ) call eralloc (  error, 12*ncurvs, 'iseqcurvs' )
      if ( ierror/=0 ) go to 1000

      alloccurv = .true.

!     --- Fill array ipointcurv with starting addresses of curves
!         in array iinputkm

      call mshseqcurvs ( iinputkm )

!     --- For all curves to be adapted do

      do i = 1, ncurves

         icurnr = listcurvs(i)

!        --- adapt curve icurnr

         iinp = ipointcurv(icurnr) ! starting address in array iinputkm
         icurve = iinputkm(iinp)     ! type of curve
         if ( icurve==9 .or. icurve==16 .or. icurve==23 ) then

!        --- Array curstr has not been filled

            call errint ( icurve, 1 )
            call errsub ( 3038, 1, 0, 0 )
            go to 1000

         end if  ! ( icurve==9 .or. icurve==16 .or. icurve==23 )
         if ( icurnr==1 ) then
            iposc = 1
         else
            iposc = icurvsmsh(icurnr-1)+1
         end if

         call mshcurcr ( iinputkm, rinputkm, curstr, &
                         icurve, iinp, irnp, &
                         iposc, icurnr, inpncv, jcoars, &
                         ipointcurv, irelnodes, irnp1, istartrel )

!        --- Copy coordinates from curves into coor

         call mshadapcoorcrv ( kmeshm, coor, ndim, icurnr, &
                               curvesmsh(1,iposc) )

      end do  ! i = 1, ncurves

      deallocate ( iseqcurvs, ipointcurv, curstr, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iseqcurvs' )

      alloccurv = .false.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapcurv'

      end if  ! ( debug )
      call erclos ( 'mshadapcurv' )

      end subroutine mshadapcurv
