      subroutine mshadapmids4 ( coor, kmeshc, iwork, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of mid-side points of quadratic triangle
!     such that they are in the centre of the side
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     smoothing
!     free_surface
!     triangle
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), iwork(*), ishape
      double precision coor(3,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ishape         i    Shape number of elements
!     iwork          i    integer work array of length npoint
!                         only nodes i with iwork(i)=0 may be moved
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind(4), k, nodp, nodp1, nodp2
      double precision t1(3), t2(3), t3(3), norm(3), prod
      logical curved

!     curved         indication if the element is curved (true) or not
!                    (false)
!     ind            Contains the relativ e node numbers of the vertices
!                    (cyclic )
!     k              Counting variable
!     nodp           Nodal point sequence number of mid-point node
!     nodp1          Node sequence number of vertex
!     nodp2          Node sequence number of vertex
!     norm           normal vector
!     prod           help parameter to store a value
!     t1             First tangential vector
!     t2             Second tangential vector
!     t3             Help vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMPNORMVEC  Compute the normal vector with respect to a plane, given
!                    two orthogonal tangential vectors in this plane
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCENTQUADR   Move middle point on quadratic curve to centre
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 1, 3, 5, 1 /
! ======================================================================
!
      call eropen ( 'mshadapmids4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapmids4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      curved = .false.

!     --- Check if element is in a plane or not

      t1 = coor(1:3,kmeshc(3))-coor(1:3,kmeshc(1))  ! first tangential vector
      t1 = t1/sqrt(dot_product(t1,t1))              ! normalized
      t2 = coor(1:3,kmeshc(7))-coor(1:3,kmeshc(1))  ! second tangential vector
      t2 = t2 - dot_product(t1,t2)*t1               ! perpendicular to t1
      t2 = t2/sqrt(dot_product(t2,t2))              ! normalized

!     --- normal vector

      call ercompnormvec ( t1, t2, norm )
      if ( debug ) then
         call prinrl ( t1, 3, 't1' )
         call prinrl ( t2, 3, 't2' )
         call prinrl ( norm, 3, 'norm' )
      end if  ! ( debug )

!     --- vector from 5 to 1

      t3 = coor(1:3,kmeshc(5))-coor(1:3,kmeshc(1))
      prod = abs(dot_product(norm,t3))
      if ( prod>sqreps * dot_product(t3,t3) ) curved = .true.

      if ( .not. curved ) then

!     --- vector from 2 to 1

         t3 = coor(1:3,kmeshc(2))-coor(1:3,kmeshc(1))
         prod = abs(dot_product(norm,t3))
         if ( prod>sqreps * dot_product(t3,t3) ) curved = .true.

      end if  ! ( .not. curved )
      if ( debug ) write(irefwr,*) 'curved ', curved

      if ( curved ) then

!     --- Element is not in a plane, complicated situation

         do k = 1, 3

            nodp = kmeshc(k*2)             ! mid_side node
            if ( iwork(nodp)==0 ) then

!           --- Node not on curve, check length of both parts

               nodp1 = kmeshc(ind(k))   ! prior node
               nodp2 = kmeshc(ind(k+1)) ! next node
               call mshcentquadr ( coor, nodp, nodp1, nodp2 )
               iwork(nodp) = 1

            end if  ! ( iwork(nodp)==0 )

         end do  ! k = 1, 3
         if ( ishape==7 ) then

!        --- ishape = 7, centroid

            nodp = kmeshc(7)
            nodp1 = kmeshc(1)
            nodp2 = kmeshc(5)
            t1 = coor(1:3,nodp)   ! save old values of coor in centroid
            call mshcentquadr ( coor, nodp, nodp1, nodp2 )
            t2 = coor(1:3,nodp)   ! save new values of coor in centroid
            nodp1 = kmeshc(3)
            nodp2 = kmeshc(1)
            coor(1:3,nodp) = t1   ! reset old values
            call mshcentquadr ( coor, nodp, nodp1, nodp2 )
            coor(1:3,nodp) = 0.5d0*(coor(1:3,nodp)+t2)
            if ( debug ) call prinrl ( coor(1,nodp), 3, 'x_nodp' )

         end if  ! ( ishape==7 )

      else

!     --- Element is in a plane, simple situation

         do k = 1, 3

            nodp = kmeshc(k*2)             ! mid_side node
            if ( iwork(nodp)==0 ) coor(1:3,nodp) = 0.5d0* &
               (coor(1:3,kmeshc(ind(k)))+coor(1:3,kmeshc(ind(k+1))))

         end do  ! k = 1, 3

         if ( ishape==7 ) then

!        --- ishape = 7, centroid

            nodp = kmeshc(7)             ! centre node
            coor(1:3,nodp) = &
                          (coor(1:3,kmeshc(2))+coor(1:3,kmeshc(4))+ &
                           coor(1:3,kmeshc(6)))/3d0
         end if  ! ( ishape==7 )

      end if  ! ( curved )

1000  call erclos ( 'mshadapmids4' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapmids4'

      end if  ! ( debug )

      end
