      subroutine mshbackgrid ( nx, origin, dx, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-04-2005 Extension with 3D
!        version  1.0  date 28-06-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create mesh for background rectangular grid
!     Arrays kmesh, kmeshc and coor are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     1d
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx(*), ndim
      double precision origin(*), dx(*)

!     dx             i    contains the step size in each coordinate direction
!     ndim           i    Dimension of the space.
!     nx             i    number of points in each coordinate direction
!     origin         i    Contains the coordinates of the origin
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelem, npelm, npoint, ishape, ncurvs
      double precision dummy(1)
      double precision, pointer :: coor(:,:)
      integer, pointer :: kmeshc(:), kmeshm(:)

!     coor           co-ordinates array stored as one-dimensional array
!     dummy          Dummy parameter
!     ishape         Shape number of elements
!     kmeshc         kmesh part c, contains the nodal points
!                    corresponding to the elements
!     kmeshm         KMESH part m, contains nodal point numbers of the
!                    curves
!     ncurvs         Number of curves in mesh
!     nelem          Number of elements
!     npelm          Number of points per element
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHBACKGRID2D  Create mesh for background rectangular grid in R^2
!     MSHBACKGRID3D  Create mesh for background rectangular grid in R^3
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill part of kmesh
!     Create space for arrays coor and kmeshc
!     Compute starting addresses
!     Fill arrays coor and kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshbackgrid' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshbackgrid'
         write(irefwr,*) 'ndim ', ndim
         call prinin ( nx, 3, 'nx' )
         call prinrl ( origin, 3, 'origin' )
         call prinrl ( dx, 3, 'dx' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill part of kmesh

      km(iseqkm)%kmesh(5) = 1              ! nelgrp
      km(iseqkm)%kmesh(20) = 1             ! kmeshf
      km(iseqkm)%kmesh(36) = 2             ! kmesht
      km(iseqkm)%kmesh(44) = 0             ! no subparts of mesh

      select case(ndim )

         case ( :1, 4: )

!        --- ndim # 2, not yet implemented

            call errint ( ndim, 1 )
            call errint ( 2, 2 )
            call errint ( 3, 3 )
            call errsub ( 419, 3, 0, 0 )

         case (2)

!        --- ndim = 2

         npelm = 4
         npoint = nx(1)*nx(2)
         nelem = (nx(1)-1)*(nx(2)-1)
         ncurvs = 1
         ishape = 5

         case (3)

!        --- ndim = 3

         npelm = 8
         npoint = nx(1)*nx(2)*nx(3)
         nelem = (nx(1)-1)*(nx(2)-1)*(nx(3)-1)
         ncurvs = 1
         ishape = 13

      end select  ! case(ndim )

      km(iseqkm)%kmesh(4) = npelm
      km(iseqkm)%kmesh(6) = ndim
      km(iseqkm)%kmesh(8) = npoint
      km(iseqkm)%kmesh(9) = nelem
      km(iseqkm)%kmesh(11) = ncurvs
      allocate ( km(iseqkm)%kmeshf(1) )
      km(iseqkm)%kmeshf(1) = ishape
      allocate ( km(iseqkm)%kmesht(2,1) )
      km(iseqkm)%kmesht(1,1) = 1
      km(iseqkm)%kmesht(2,1) = nelem
      if ( debug ) &
         write(irefwr,*) 'npelm, ndim, npoint, nelem ', &
                          npelm, ndim, npoint, nelem

!     --- Create space for arrays coor and kmeshc, kmeshm

      allocate ( km(iseqkm)%coor(ndim,npoint) )
      coor => km(iseqkm)%coor
      km(iseqkm)%kmesh(23) = npoint*ndim
      allocate ( km(iseqkm)%kmeshc(npelm*nelem) )
      km(iseqkm)%kmesh(17) = npelm*nelem
      kmeshc => km(iseqkm)%kmeshc
      if ( ndim==2 ) then
         length = 2*(nx(1)+nx(2))+3
      else  ! ( ndim=3)
         length = 10
      end if  ! ( ndim==2 )
      allocate ( km(iseqkm)%kmeshm(length) )
      km(iseqkm)%kmesh(27) = length
      kmeshm => km(iseqkm)%kmeshm

!     --- Fill arrays coor and kmeshc

      select case(ndim )

         case ( :1, 4: )

!        --- ndim # 2, 3, not yet implemented

            call errint ( ndim, 1 )
            call errint ( 2, 2 )
            call errint ( 3, 3 )
            call errsub ( 419, 3, 0, 0 )

         case (2)

!        --- ndim = 2

            call mshbackgrid2d ( nx(1), nx(2), origin(1), origin(2), &
                                 dx(1), dx(2), coor, kmeshc, kmeshm )

         case (3)

!        --- ndim = 3

            call mshbackgrid3d ( nx(1), nx(2), nx(3), &
                                 origin(1), origin(2), origin(3), &
                                 dx(1), dx(2), dx(3), &
                                 coor, kmeshc, kmeshm )

      end select  ! case(ndim )

1000  if ( debug ) then

!     --- Debug information

         call sepprinkmesh ( 0 )
         write(irefwr,*) 'End mshbackgrid'

      end if
      call erclos ( 'mshbackgrid' )

      end
