      subroutine mshcentquadr ( coor, nodp, nodp1, nodp2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if nodp is in the centre of nodp1 and nodp2 along
!     the quadratic curve formed by these nodes
!     If not move this point to the center (approximately)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     smoothing
!     free_surface
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodp, nodp1, nodp2
      double precision coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     nodp           i    Nodal point sequence number of centre point
!     nodp1          i    Node sequence number of first point
!     nodp2          i    Node sequence number of last point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t1(3), t2(3), h1, h2, s, s1, s2

!     h1             Length of vector t1
!     h2             Length of vector t2
!     s              Parameter s along curve corresponding to centroid
!     s1             Help parameter to compute new position
!     s2             Help parameter to compute new position
!     t1             Contains vector from node1 to nodp
!     t2             Contains vector from node to nodp2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2996   Distance netween new and old coordinates too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcentquadr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcentquadr'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      t1 = coor(1:3,nodp) - coor(1:3,nodp1)
      h1 = dot_product(t1,t1)
      t2 = coor(1:3,nodp2) - coor(1:3,nodp)
      h2 = dot_product(t2,t2)

      if ( abs(h2-h1)>sqreps*h1 ) then

!     --- Length of both sides not equal

         if ( debug ) then
            write(irefwr,1) 'nodp, nodp1, nodp2', &
                             nodp, nodp1, nodp2
            write(irefwr,2) 'h1, h2', h1, h2
            call prinrl ( coor(1,nodp), 3, 'x_nodp' )
            call prinrl ( coor(1,nodp1), 3, 'x_nodp1' )
            call prinrl ( coor(1,nodp2), 3, 'x_nodp2' )
         end if  ! ( debug )

         h1 = sqrt(h1)
         h2 = sqrt(h2)
         s = 0.5d0*(h1+h2)
         s1 = h1
         s2 = h1+h2
         t1 = coor(1:3,nodp)  ! store old values of nodp
         coor(1:3,nodp) = &
            coor(1:3,nodp1)*(s-s1)*(s-s2)/(s1*s2) + &
            coor(1:3,nodp)*s*(s-s2)/(s1*(s1-s2)) + &
            coor(1:3,nodp2)*s*(s-s1)/(s2*(s2-s1))

         if ( debug ) then
            write(irefwr,2) 's, s1, s2', s, s1, s2
            call prinrl ( coor(1,nodp), 3, 'x_nodp' )
         end if  ! ( debug )

         t2 = coor(1:3,nodp)-t1  ! new value of nodp
         s = dot_product(t2,t2)

         if ( s>sqreps ) then

!        --- s may be too large, give warning

            call erreal ( sqrt(s), 1 )
            call errwar ( 2996, 0, 1, 0 )

         end if

      end if  ! ( abs(h2-h1)>sqreps*h1 )

1000  call erclos ( 'mshcentquadr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcentquadr'

      end if  ! ( debug )

      end
