      subroutine mshcentrepartc ( coor, curvenodes, numpoints, origin, &
                                  radius, isurnr, startpoints, &
                                  endpoints, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute centroid of part of a circle as well as radius
!     Check if all nodes not on the straight lines are positioned on circle
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     plane
!     surface
!     centre
!     radius
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), origin(3), radius, eps
      integer curvenodes(*), numpoints, isurnr, startpoints(*), &
              endpoints(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curvenodes     i    Contains the nodes of the boundary curve
!     endpoints      i    Array containing the relative end points of the
!                         straight lines on the part of the circle (if any)
!                         relative with respect to array curvenodes
!     eps           i/o   Local accuracy
!     isurnr         i    Surface sequence number
!     numpoints      i    Number of points
!     origin         o    Contains the coordinates of the origin of the circle
!     radius         o    Radius of sphere or circle
!     startpoints    i    Array containing the relative start points of the
!                         straight lines on the part of the circle (if any)
!                         relative with respect to array curvenodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode1, inode2

!     i              Counting variable
!     inode1         First node
!     inode2         Second node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHECKCIRCLE Check if node is lying on circle defined by origin
!                    and radius
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute radius and centroid of circle as common point of striaght lines
!     Check if other nodes are on circle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcentrepartc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcentrepartc'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Compute radius and centroid of circle as midpoint of straight line

      if ( endpoints(1).eq.startpoints(2) ) then

!     --- end point of first straight line is starting point of next one
!         This must be the centroid of the circle

         inode1 = curvenodes(endpoints(1))    ! centroid
         inode2 = curvenodes(startpoints(1))  ! end point of line

      else

!     --- end point of second straight line is starting point of first one
!         This must be the centroid of the circle

         inode1 = curvenodes(startpoints(1))    ! centroid
         inode2 = curvenodes(startpoints(2))  ! end point of line

      end if  ! ( endpoints(1).eq.startpoints(2) ) then

      do i = 1, 3
         origin(i) = coor(i,inode1)
      end do  ! i = 1, 3

      radius = 0d0  ! square of radius
      do i = 1, 3
         radius = radius + (coor(i,inode2)-origin(i))**2
      end do  ! i = 1, 3
      eps = radius*eps

      if ( debug ) &
         write(irefwr,*) 'origin, radius ', origin, sqrt(radius)

!     --- Check if other nodes are on circle

      if ( startpoints(1).gt.1 .and. &
           startpoints(1).lt.endpoints(2) ) then

!     --- startpoints1(1) > 1 and startpoints(1)<endpoints(2)
!         circle from 1 to startpoints(1)-1 and from
!         endpoints(2)+1 to numpoints

         do i = 1, startpoints(1)-1
            if ( debug ) write(irefwr,*) 'check node ', i
            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )
         end do  ! i = startpoints(1)-1

         do i = endpoints(2)+1, numpoints
            if ( debug ) write(irefwr,*) 'check node ', i
            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )
         end do  ! i = startpoints(1)-1

      else if ( startpoints(1).eq.1 .and. &
                endpoints(2).eq.numpoints ) then

!     --- startpoints1(1) = 1 and endpoints(2) = numpoints
!         circle from endpoints(1)+1 to startpoints(2)-1

         do i = endpoints(1)+1, startpoints(2)-1
            if ( debug ) write(irefwr,*) 'check node ', i
            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )
         end do  ! i = endpoints(1)+1, startpoints(2)-1

      else if ( startpoints(1).eq.1 ) then

!     --- startpoints1(1) = 1 and endpoints(2) < numpoints
!         circle from endpoints(2)+1 to numpoints

         do i = endpoints(2)+1, numpoints
            if ( debug ) write(irefwr,*) 'check node ', i
            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )
         end do  ! i = endpoints(2)+1, numpoints

      else

!     --- startpoints1(1) > endpoints(2)
!         circle from endpoints(2)+1 to startpoints(1)-1

         do i = endpoints(2)+1, startpoints(1)-1
            if ( debug ) write(irefwr,*) 'check node ', i
            call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                                  coor, isurnr )
         end do  ! i = endpoints(2)+1, startpoints(1)-1

      end if  ! ( startpoints(1).gt.1 ...

      radius = sqrt(radius)
      if ( debug ) write(irefwr,*) 'radius ', radius

1000  call erclos ( 'mshcentrepartc' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcentrepartc'

      end if

      end
