      subroutine mshcompdispl3d ( coornw, inpada, rinada, iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-12-2008
!
!   copyright (c) 2008-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute new coordinates of boundary (3d)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), iseqvecs(*)
      double precision coornw(3,*), rinada(*)

!     coornw         o    Array containing the new co-ordinates
!     inpada         i    User input array for subroutine ADAPBOUN
!     iseqvecs       i    Contains sequence numbers of vectors with respect to
!                         isol
!     rinada         i    User input array for subroutine ADAPBOUN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivectr, iadapt, numsurfs, i, indprfpsi, ipsi
      integer, allocatable :: marksurf(:), marknodes(:), markcurv(:), &
                              adapsurfs(:)
      integer, pointer :: kprobfpsi(:)
      double precision, pointer :: psi(:)

!     adapsurfs      Array of size numsurfs containing the surfaces that
!                    must be adapted
!     i              Counting variable
!     iadapt         Indicates how the boundary must be adapted, see INPADA
!     indprfpsi      ndication if kprob part f has been filled (>0)
!                    or not (0) (in this case with respect to psi)
!     ipsi           Sequence number of array psi in solution array
!     ivectr         sequence number of velocity vector in isol
!     kprobfpsi      kprobf corresponding to psi
!     markcurv       Array of size ncurvs that marks all curves that
!                    have been adapted in the moving boundary
!                    markcurv(icurnr) = 0: curve has not been moved
!                    markcurv(icurnr) = 1: curve has been moved
!     marknodes      Array of size npoint that marks all nodes that
!                    have been adapted in the moving boundary
!                    marknodes(node) = 0: node has not been moved
!                    marknodes(node) = 1: node has been moved
!     marksurf       Array of size nsurfs that marks all surfaces that
!                    have been adapted in the moving boundary
!                    marksurf(icurnr) = 0: surface has not been moved
!                    marksurf(icurnr) = 1: surface has been moved
!     numsurfs       number of surfaces in adapsurfs
!     psi            pseudo stream function
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCOMPDISPL21 Adapt coordinates of curves of which at least one point has
!                    been moved in a changing boundary
!     MSHCOMPDISPL30 Compute new coordinates of boundary (3d)
!     MSHFINDADSURF  Find all surfaces that have at least one node that has
!                    been moved during a free boundary problem
!     MSHSMOOTHQUAD  Adapt displacement of nodes of quadratic elements
!                    such that mid-side points remain in the middle
!     MSHSMOOTHSURF  Compute displacement of surface nodes, given the
!                    displacement of the boundary by Laplacian smoothing
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1393   Not constant number of degrees of freedom not available in intmsh.
!    2910   Not enough space
!    2911   Not possible to deallocate
!    2998   No psi vector whereas film method must be used
! **********************************************************************
!
!                       PSEUDO CODE
!
!     coornew := coorold
!     Store nodes of curves to be changed in iwork and coordinates in curves
!     Compute new coordinates of curves
!     Adapt coornew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompdispl3d' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompdispl3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      iadapt = inpada(4)        ! seq number input boundary adaptation

      ivectr = iseqvecs(1)      ! sequence number of velocity vector
      ipsi = iseqvecs(2)        ! sequence number of pseudo stream function

      if ( iadapt==3 .and. ipsi<=0 ) then

!     --- Error the film method must be applied, but no psi vector given

         call errsub ( 2998, 0, 0, 0 )
         go to 1000

      end if  ! ( iadapt==3 .and. ipsi<=0 )

!     --- allocate arrays

      allocate ( marksurf(nsurfs), marknodes(npoint), &
                 markcurv(ncurvs), adapsurfs(nsurfs), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( npoint+ncurvs+nsurfs, 1 )
         call errint ( error, 2 )
         call errchr ( 'marksurf', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

      marksurf = 0
      marknodes = 0
      markcurv = 0

      if ( iadapt==3 ) then
         call sepactsolbf2 ( isolut2d(ipsi) )
         indprfpsi = indprfi
         kprobfpsi => kprobfi
      else
         indprfpsi = 0
         kprobfpsi => kprobf
      end if  ! ( iadapt==3 )

      call sepactsolbf2 ( isolut2d(ivectr) )

      if (ks(isolut2d(ivectr))%typevec==110) then
         nunkpi = nunkp
      else if ( ks(isolut2d(ivectr))%typevec==115 .and. nphysi>0 ) then
         nunkpi=nphysi
      else if ( ks(isolut2d(ivectr))%typevec==116 ) then
         nunkpi = 2
      else if (ks(isolut2d(ivectr))%typevec/=110) then
         call errint ( ks(isolut2d(ivectr))%typevec, 1 )
         call errchr ( 'mshcompdispl3d', 1 )
         call errsub ( 1393, 1, 0, 1 )
         nunkpi = 0
      end if

!     --- Compute starting addresses

      usolt => ks(isolut2d(ivectr))%sol
      if ( indprfpsi>0 ) psi => ks(isolut2d(ipsi))%sol

!     --- Compute new coordinates

      call mshcompdispl30 ( coornw, inpada, rinada, nunkpi, usolt, &
                            indprfi, kprobfi, &
                            marksurf, marknodes, psi, kprobfpsi, markcurv )

!     --- Put nodes in center of elements in case of quadratic elements

      do i = 1, nsurfs
         if ( marksurf(i)>0 ) &
            call  mshsmoothquad ( coornw, i )
      end do  ! i = 1, nsurfs

!     --- Adapt curves of which at least one point has been changed

      call mshcompdispl21 ( markcurv, marknodes, coornw )

!     --- Find all surfaces of which at least one curve has been moved

      call mshfindadsurf ( marksurf, marknodes, &
                           numsurfs, adapsurfs, markcurv )
      if ( debug ) call prinin ( adapsurfs, numsurfs, 'adapsurfs' )

!     --- Adapt surfaces of which at least one curve has been changed

      do i = 1, numsurfs

         call mshsmoothsurf ( adapsurfs(i), coornw )

      end do  ! i = 1, numsurfs

900   deallocate ( marksurf, marknodes, markcurv, adapsurfs, &
                   stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'marksurf', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcompdispl3d'

      end if  ! ( debug )
      call erclos ( 'mshcompdispl3d' )

      end
