      subroutine mshdoublecurvs ( kmeshc, nelem, inpelm, coor, ndim, &
                                  npoint, kbndpt, nbndpt, ncurvs, &
                                  boundary, numcurvboun, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-06-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map nodes of double curves onto each other
!     Decrease npoint and adapt coor, kbndpt and kmeshc
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, ncurvs, numcurvboun, boundary(2,numcurvboun), &
              ndim, nelem, inpelm, kmeshc(nelem*inpelm), nbndpt, &
              kbndpt(nbndpt), isurnr
      double precision coor(ndim,npoint)

!     boundary       i    Array of size 2 x numcurvboun containing
!                         information of the boundary of the surface
!                         (1,i) contains the i-th curve number (including sign)
!                         (2,i) contains the starting address in kbndpt
!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     isurnr         i    Surface sequence number
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     ncurvs         i    Number of curves in mesh
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
!     npoint        i/o   Number of nodal points
!                         is decreased at output due to removal of double nodes
!     numcurvboun    i    Number of curves in the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(npoint), icurvs(ncurvs), iseq, icurnr, jcurnr, &
              istart, iend, jseq, jend, numnodes, i, icount, jstart
      logical changecoor

!     changecoor     If true coordinates must be adapted
!     i              Counting variable
!     icount         Counter for nodal points
!     icurnr         Curve sequence number
!     icurvs         work array of length ncurvs
!                    Contains the sequence number of the curve in the boundary
!     iend           Last position of curve in kbndpt
!     iseq           Sequence number of curve
!     istart         First position of curve in kbndpt
!     iwork          integer work array to store the new numbering of
!                    the nodes (length: npoint)
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     jend           end position of curve in kbndpt (first appearance)
!     jseq           Sequence number of curve (first appearance)
!     jstart         First position of curve in kbndpt (first appearance)
!     numnodes       Number of nodal points along curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIMAPINTARR   Map integer array onto new numbering using numbering
!                    stored in irenum
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:npoint)
!     Clear array icurvs(1:ncurvs)
!     iseq := 0
!     For all curves icurnr in boundary do
!        iseq := iseq+1
!        if ( icurvs(icurnr)=0 ) then
!           icurvs(icurnr) := iseq
!        else
!           Mark positions in iwork corresponding to these nodes
!           with reference to positions in curve iseq (negative number)
!     icount := 0
!     for i := 1 (1) npoint do
!        if ( iwork(i)=0 ) then
!           icount := icount+1;  iwork(i) := icount
!     Reorder array coor by removing double nodes (i.e. iwork(i)<0 )
!     Fill negative positions in iwork with new values
!     Renumber arrays kmeshc and kbndpt
!     npoint := icount
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdoublecurvs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshdoublecurvs'
         write(irefwr,1) 'isurnr', isurnr
         call prinin ( kbndpt, nbndpt, 'kbndpt' )
         call prinin1 ( boundary, numcurvboun, 2, 'boundary' )
         call prinin1 ( kmeshc, nelem, inpelm, 'kmeshc' )
         call prinrl1 ( coor, npoint, ndim, 'coor' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork(1:npoint)
!         Clear array icurvs(1:ncurvs)
!         iseq := 0

      iwork = 0
      icurvs = 0
      iseq = 0

!     --- For all curves icurnr in boundary do
!         iseq := iseq+1

      do iseq = 1, numcurvboun

         icurnr = boundary(1,iseq)
         jcurnr = abs(icurnr)
         if ( debug ) write(irefwr,1) 'iseq, icurnr', iseq, icurnr

         if ( icurvs(jcurnr)==0 ) then

!        --- icurvs(icurnr)=0
!            icurvs(icurnr) := iseq

            icurvs(jcurnr) = iseq

         else

!        --- icurvs(icurnr)>0
!            Mark positions in iwork corresponding to these nodes
!            with reference to positions in curve iseq (negative number)

            istart = boundary(2,iseq)     ! starting address in kbndpt
            if ( iseq<numcurvboun ) then
               iend = boundary(2,iseq+1)  ! end address in kbndpt
            else
               iend = nbndpt
            end if  ! ( iseq<numcurvboun )
            jseq = icurvs(jcurnr)         ! sequence number of first curve
            jstart = boundary(2,jseq)     ! starting address in kbndpt
            if ( debug ) &
                write(irefwr,1) 'istart, iend, jseq', istart, iend, jseq

            if ( icurnr*boundary(1,jseq)<0 ) then

!           --- Curve numbers have different sign
!               This is the case of a crack

               if ( jseq<numcurvboun ) then
                  jend = boundary(2,jseq+1)  ! end address in kbndpt for
                                             ! other curve
               else
                  jend = nbndpt
               end if  ! ( iseq<numcurvboun )

!              --- Compute number of nodes along curve (-1)
!                  Since jumps are possible in the boundary we have to take
!                  the minimum value

               numnodes = min(iend-istart,jend-jstart)

               if ( debug ) &
                  write(irefwr,1) 'jstart, jend, numnodes', &
                                   jstart,jend, numnodes

               do i = 0, numnodes
                  if ( kbndpt(istart+i)/=kbndpt(jend-i) ) then
                     if ( iwork(kbndpt(jend-i))/=-kbndpt(istart+i) ) &
                        iwork(kbndpt(istart+i)) = -kbndpt(jend-i)
                  end if  ! ( kbndpt(istart+i)/=kbndpt(jend-i) )
               end do  ! i = 1, numnodes

            end if  ! ( icurnr*boundary(1,jseq)<0 )

         end if  ! ( icurvs(jcurnr)==0 )

      end do  ! i = 1, numcurvboun

      if ( debug ) call prinin ( iwork, npoint, 'iwork/1' )

!     --- icount := 0

      icount = 0

!     --- for i := 1 (1) npoint do

      do i = 1, npoint

         if ( iwork(i)==0 ) then

!        --- iwork(i)=0
!            icount := icount+1;  iwork(i) := icount

            icount = icount+1
            iwork(i) = icount

         end if  ! ( iwork(i)==0 ) then

      end do  ! i = 1, npoint

!     --- Reorder array coor by removing double nodes (i.e. iwork(i)<0 )
!         Fill negative positions in iwork with new values

      changecoor = .false.
      do i = 1, npoint

         if ( iwork(i)<0 ) then

!        --- iwork(i)<0, hence double point
!            Fill node number in iwork

            iwork(i) = iwork(-iwork(i))
            changecoor = .true.

         else

!        --- iwork(i)>0, hence new point
!            Fill coor

            if ( changecoor ) then

!           --- New node number less than old one

               coor(1:ndim,iwork(i)) = coor(1:ndim,i)

            end if  ! ( changecoor )

         end if  ! ( iwork(i)<0 )

      end do  ! i = 1, npoint
      if ( debug ) call prinin ( iwork, npoint, 'iwork/2' )

!     --- Renumber arrays kmeshc and kbndpt

      call inimapintarr ( kmeshc, iwork, nelem*inpelm )
      call inimapintarr ( kbndpt, iwork, nbndpt )

!     --- npoint := icount

      npoint = icount

1000  call erclos ( 'mshdoublecurvs' )
      if ( debug ) then

!     --- Debug information

         call prinin ( kbndpt, nbndpt, 'kbndpt/end' )
         call prinin1 ( kmeshc, nelem, inpelm, 'kmeshc/end' )
         call prinrl1 ( coor, npoint, ndim, 'coor/end' )
         write(irefwr,*) 'End mshdoublecurvs'

      end if  ! ( debug )

      end
