      subroutine mshellarc ( coarse, ndim, curves, iinput, rinput, tau, &
                             inpncv, nuspnt, curstr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 19-02-2011 New call to mshellparms
!        version  1.1  date 19-01-2011 Extra debug
!        version  1.0  date 15-08-2009
!
!   copyright (c) 2009-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill part of array curves for an ellips segment
!    n positions are filled
!    the spacing of the nodal points is given by array tau
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
!     ellips
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical coarse
      integer iinput(*), ndim, inpncv, nuspnt
      double precision curves(ndim,*), tau(inpncv), rinput(ndim,*), &
                       curstr(*)

!     coarse         i    indicates if array curstr has been filled already
!                         (true) or not (false)
!     curstr         i    is only used when coarse is true
!                         in that case it must contain the following information
!                         1:   length of the circle part
!                         2:   phi arc of the circle segment
!                         3:   r   radius of the circle
!                         ndim=2 4,5   :   xz,yz     mid points of the circle
!                         ndim=3 4,5,6 :   xz,yz,zz  centre of the circle
!     curves         o    array containing the coordinates of the curves
!     iinput         i    the first two positions of this array contain the
!                         point numbers of the initial and end points
!     inpncv         i    number of points on the line
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rinput         i    array containing the coordinates of the points
!     tau            i    array containing the spacing of the nodal points
!                         output of subroutine msh004 or msh031
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iuserp1, iuserp2, iuserp3, i
      double precision x1, y1, x2, y2, phi, xt, yt, a, b, t0

!     a              Parameter defining the ellips (length of first direction)
!     b              Parameter defining the ellips (length of second direction)
!     i              Counting variable
!     iuserp1        First user point
!     iuserp2        Second user point
!     iuserp3        third user point
!     phi            Difference in parameter t from begin to end
!     t0             Value of t corresponding to x1
!     x1             x-coordinate of first point in curve
!     x2             x-coordinate of last point in curve
!     xt             x-coordinate of centre
!     y1             y-coordinate of first point in curve
!     y2             y-coordinate of last point in curve
!     yt             y-coordinate of centre
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHELLPARMS    Compute defining parameters for ellips
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     127   user point number out of range
!     422   arc or carc is only permitted in two or three dimensions
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshellarc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshellarc'
         write(irefwr,*) 'coarse', coarse
         call prinrl ( tau, inpncv, 'tau' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get user point numbers

      iuserp1 = iinput(1)
      iuserp2 = iinput(2)
      iuserp3 = iinput(3)

      if ( debug ) &
         write(irefwr,1) 'iuserp1, iuserp2, iuserp3, ndim', &
                          iuserp1, iuserp2, iuserp3, ndim
      if ( iuserp1<=0 .or. iuserp1>nuspnt ) then
         call errint ( iuserp1, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( iuserp2<=0 .or. iuserp2>nuspnt ) then
         call errint ( iuserp2, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( iuserp3==0 .or. abs(iuserp3)>nuspnt ) then
         call errint ( iuserp3, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( ndim<2 .or. ndim>2 ) then
         call errsub ( 422, 0, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- get coordinates

      x1 = rinput(1,iuserp1)
      y1 = rinput(2,iuserp1)
      x2 = rinput(1,iuserp2)
      y2 = rinput(2,iuserp2)
      xt = rinput(1,abs(iuserp3))
      yt = rinput(2,abs(iuserp3))
      if ( debug ) &
         write(irefwr,2) 'x1, y1, x2, y2, xt, yt', &
                          x1, y1, x2, y2, xt, yt

!     --- Store begin and points in curves

      curves(1,1) = x1
      curves(2,1) = y1
      curves(1,inpncv) = x2
      curves(2,inpncv) = y2

!     --- Compute parameters defining the ellips part (a, b, t0, phi)

      if ( coarse ) then

!     --- coarse = true, parameters have already been computed

         phi = curstr(2)
         a = curstr(3)
         b = curstr(4)
         t0 = curstr(5)

      else

!     --- coarse = false, compute parameters

         call mshellparms ( x1, y1, x2, y2, xt, yt, a, b, t0, phi, &
                            iuserp3 )

      end if  ! ( coarse )
      if ( debug ) write(irefwr,2) 'a, b, t0, phi', a, b, t0, phi

!     --- Store all nodes between in curves

      do i = 2, inpncv-1
         curves(1,i) = xt+a*cos(tau(i)*phi+t0)
         curves(2,i) = yt+b*sin(tau(i)*phi+t0)
      end do

1000  call erclos ( 'mshellarc' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( curves, inpncv, ndim, 'curves' )
         write(irefwr,*) 'End mshellarc'

      end if  ! ( debug )

      end
