      subroutine mshmakeoutersb ( iinput, iseqinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 15-11-2012 Long integers
!        version  3.2  date 17-05-2010 New call to msho84
!        version  3.1  date 08-09-2005 New length of iwork
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute outer boundary (curves or surfaces) for 2d or 3d mesh
!     and store the curve or surface numbers in kmesh part AL
!     Outer boundary is coupled to a set of element groups
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     boundary
!     curve
!     surface
!     2d
!     3d
!     outer_boundary
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iseqinp

!     iinput         i    Integer input array describing which element
!                         groups must be coupled to a sequence number
!                         iinput must be filled as follows:
!                         Pos. 1  numelgrps: number of element groups
!                              2-numelgrps+1  element group sequence numbers
!     iseqinp        i    Sequence number of input (used for error messages)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nseq, lenoutercurvs, lencop
      integer, pointer :: ioutercurvs(:), listgrp(:), iouternew(:)

!     ioutercurvs    Contains information about the curves in the outer
!                    boundary (2D) in the following way
!                    Pos. 1  nparts  Number of parts the outer boundary
!                                    consists of
!                    Pos. 2-1+nparts Starting addresses of each part
!                    Per part: (first address: ioutercurvs(1+i)
!                    Pos. 1: nouter  Number of curves in part
!                    followed by the curve numbers (provided with a sign)
!                    These curve numbers are consecutive, which means
!                    the end of one curve is the start of the new one
!                    The part itself is closed
!     iouternew      temporary array iouterboun
!     lencop         Length of part to be copied
!     lenoutercurvs  Length of array ioutercurvs
!     listgrp        Array of size nelgrp indicating if an element
!                    group must be used (1) or not (0)
!     nseq           Number of cases stored in kmeshal
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHLISTGRP     Fill array listgrp with a 0 if an element group must be
!                    skipped and a 1 if it has to be taken into account
!     MSHOUTERCURVES Compute the outer boundary of the set of 2d elements
!                    defined by a set of element groups in array iinput
!     MSHOUTERSURF   Make alist of all outer surfaces of the mesh under the
!                    condition that array kmesh part n has been filled but
!                    kmesh part s not
!     PRININ         print 1d integer array
!     PRINIOUTERCS   Print array ioutercurvs or ioutersurfs,
!                    depending on the dimension
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHF  Fill array nmeshf
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2488   Mesh contains surface nor volume elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create array listgrp at the end of ibuffr
!     Fill array listgrp with a 0 for element groups to be skipped
!     and a 1 for all other element groups and check array iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmakeoutersb' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmakeoutersb'
         call prinin ( iinput, iinput(1)+1, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if !  ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array iouterbounseq if it already exists

      if ( kelmal>0 ) then

!     --- iouterbounseq exists, activate and extract nseq

         nseq = iouterbounseq(1)

      else

!     --- iouterbounseq does not exist, set nseq 0

         nseq = 0

      end if !  ( kelmal>0 )

!     --- Create temporary array ioutercurvs at the end of ibuffr
!         and store nseq in position 1

      if ( nvolms==0 ) then

!     --- nvolms = 0, no volume elements

         if ( nsurfs==0 ) then

!        --- there are no surface nor volume elements: not allowed

            call errsub ( 2488, 0, 0, 0 )
            go to 1000

         end if
         length = 4*ncurvs+2

      else

!     --- There are volume elements

         length = 4*nsurfs+2

      end if

      allocate ( ioutercurvs(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'ioutercurvs' )
      if ( ierror/=0 ) go to 1000

      ioutercurvs(1) = nseq

      if ( kelmaa==0 ) call sepfillkmeshaa

      if ( nvolms==0 ) then

!     --- nvolms = 0, create outer curves
!         Activate arrays

         if ( nmesh(33)==0 ) call sepfillnmeshc
         if ( nmesh(38)==0 ) call sepfillnmeshh
         if ( nmesh(42)==0 ) call sepfillnmeshl
         if ( .false. .and. debug ) then
            call sepprinkmesh ( 40 )
            call sepprinkmesh ( 43 )
            call sepprinkmesh ( 41 )
         end if

         call mshoutercurves ( iinput, ioutercurvs(3), lenoutercurvs, iseqinp )

      else

!     --- nvolms>0, create outer surfaces
!         Create array listgrp at the end of ibuffr

         allocate ( listgrp(nelgrp), stat = error )
         if ( error/=0 ) call eralloc ( error, nelgrp, 'listgrp' )
         if ( ierror/=0 ) go to 1000

!        --- Fill array listgrp with a 0 for element groups to be skipped
!            and a 1 for all other element groups and check array iinput

         call mshlistgrp ( listgrp, iinput, ioutercurvs, iseqinp )
         if ( ierror/=0 ) go to 1000

!        --- Activate arrays

         if ( nmesh(31)==0 ) call sepfillnmesha
         if ( nmesh(36)==0 ) call sepfillnmeshf
         if ( nmesh(37)==0 ) call sepfillnmeshg
         if ( nmesh(41)==0 ) call sepfillnmeshk
         if ( .false. .and. debug ) then
            call sepprinkmesh ( 44 )
            call sepprinkmesh ( 46 )
            call sepprinkmesh ( 48 )
         end if

!        --- Fill array ioutercurvs

         outerboun => ioutercurvs(3:size(ioutercurvs))
         call mshoutersurf ( lenoutercurvs, listgrp  )
         if ( debug ) then
            call prinioutercs
            write(irefwr,*) 'lenoutercurvs ', lenoutercurvs
         end if  ! ( debug )

         deallocate ( listgrp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'listgrp' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( nvolms==0 )

!     --- Create or extend array iouterbounseq

      if ( debug ) write(irefwr,1) 'kelmal', kelmal
      if ( kelmal==0 ) then

!     --- kelmal = 0, array iouterbounseq is new
!         create and copy contents of ioutercurvs

         length = lenoutercurvs+2
         allocate ( km(iseqkm)%iouterbounseq(length) )
         km(iseqkm)%kmesh(63) = length
         iouterbounseq => km(iseqkm)%iouterbounseq
         kelmal => km(iseqkm)%kmesh(63)

!        --- Store end address in position 2

         iouterbounseq = ioutercurvs(1:length)
         iouterbounseq(1) = 1
         iouterbounseq(2) = length
         if ( debug ) then
            write(irefwr,*) 'nseq, lenoutercurvs ', nseq, lenoutercurvs
            call prinin ( iouterbounseq(1), length, &
                          'kmeshal' )
         end if  ! ( debug )

      else

!     --- kelmal>0, array iouterbounseq is old
!         extend array and copy old part to new one

         lencop = size ( iouterbounseq )
         length = lencop+lenoutercurvs+1
         allocate ( iouternew(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'iouternew' )
         if ( ierror/=0 ) go to 1000

!        --- Store end address in position lencop+1

         iouternew(1:lencop) = iouterbounseq
         iouternew(1) = nseq+1
         iouternew(1+lencop) = length

!        --- Copy ioutercurvs after position lencop+1

         iouternew(2+lencop:1+lencop+lenoutercurvs) = &
            ioutercurvs(3:lenoutercurvs+2)
         if ( debug ) then
            write(irefwr,*) 'nseq, length ', nseq, length
            call prinin ( iouternew, length, 'kmeshal' )
         end if  ! ( debug )

         allocate ( km(iseqkm)%iouterbounseq(length) )
         km(iseqkm)%kmesh(63) = length
         iouterbounseq => km(iseqkm)%iouterbounseq
         kelmal => km(iseqkm)%kmesh(63)
         iouterbounseq = iouternew

         deallocate ( iouternew, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iouternew' )
         if ( ierror/=0 ) go to 1000

      end if

!     --- Remove temporary arrays

      deallocate ( ioutercurvs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ioutercurvs' )
      if ( ierror/=0 ) go to 1000

      outerboun => km(iseqkm)%outerboun

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmakeoutersb'

      end if
      call erclos ( 'mshmakeoutersb' )

      end subroutine mshmakeoutersb
