      subroutine mshmaptoplane ( kmeshlc, isol3d, isol2d, &
                                 mapped, isurfaces, nsurfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 23-12-2011 new call to ersurfsnod
!        version  1.1  date 28-12-2010 new call to ersurfsnod
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map the solution defined on a plane in a 3d mesh onto a plane
!     in a 2d mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     solution_vector
!     mesh
!     plane
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshlc, isol3d(*), isol2d(*), isurfaces(*), nsurfaces
      logical mapped

!     isol2d        i/o   Standard SEPRAN array containing information of the
!                         2d velocity vector stored as solution vector
!     isol3d         i    Standard SEPRAN array containing information of the
!                         3d velocity vector stored as solution vector
!     isurfaces      i    Array containing the surfaces from which information
!                         must be found
!     kmeshlc        o    Standard SEPRAN array containing information of the
!                         2d mesh
!     mapped         o    if true the mapping has been carried out
!     nsurfaces      i    Number of surfaces in array isurfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nnodes, isurnr
      integer, allocatable :: iwork(:), nodes(:)
      double precision normallc(3)

!     isurnr         Surface sequence number
!     iwork          integer work array
!     nnodes         Number of nodes in surfaces
!     nodes          Contains the node numbers that must be filled
!     normallc       Array to store the normal to a plane
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERMARKSURNOD   Fill array marknodes for all nodes in array nodes
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHPLANE     Check if all nodes in surface are lying in a plane
!     MSHCOPSOL2D    Copy solution from3d mesh onto 2d mesh given the node
!                    numbers of the surfaces to which the solution
!                    must be copied
!     MSHFILLKMESH2d Fill kmesh for a temporary 2d mesh given a 3d surface
!     MSHNORMALSURF  Compute the normal onto the first element of
!                    surface isurnr
!     PRININ         print 1d integer array
!     PROBMAP3D2D    Map kprob defined on 3d grid onto 2d surface
!     SEPSURFSNOD    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     mapped := all surfaces are in a plane
!     if ( mapped ) then
!        create 2d mesh
!        Fill kprob2d corresponding to 2d mesh
!        Fill isol2d as far as possible
!        Copy solution from 3d mesh onto 2d mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmaptoplane' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmaptoplane'
         write(irefwr,100) nsurfaces
         if ( nsurfaces>0 ) &
            call prinin ( isurfaces, nsurfaces, 'surfaces' )
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- mapped := all surfaces are in a plane

      mapped = nsurfaces>0
      if ( .not. mapped ) go to 1000

!     --- Compute some constants

      allocate ( iwork(npoint), nodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Store node numbers of nodes in surfaces in array nodes
!         of length nnodes
!         Mark nodes if array nodes in array iwork

      call sepsurfsnod ( nsurfaces, nodes, nnodes, isurfaces )
      if ( ierror/=0 ) go to 1000

!     --- Fill array iwork

      call ermarksurnod ( nodes, nnodes, iwork, npoint )

!     --- Compute normal on plane

      isurnr = isurfaces(1)
      call mshnormalsurf ( kmeshn, nsurfs, isurnr, coor, normallc )
      if ( ierror/=0 ) go to 1000

!     --- Check if all surfaces are in a plane

      call mshchplane ( nnodes, nodes, coor, isurnr, &
                        normallc, mapped )
      if ( ierror/=0 ) go to 1000

      if ( mapped ) then

!     --- mapped = true
!         create 2d mesh

         call mshfillkmesh2d ( kmeshlc, iwork, nnodes, isurfaces, &
                               nsurfaces, normallc )

!        --- Fill kprob2d corresponding to 2d mesh

         call probmap3d2d ( iwork, isol3d, nnodes )

!        --- Fill isol2d as far as possible
!            Copy solution from 3d mesh onto 2d mesh

         call mshcopsol2d ( isol3d, isol2d, iwork, npoint )

      end if  ! ( mapped )

      deallocate ( iwork, nodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmaptoplane'

      end if
      call erclos ( 'mshmaptoplane' )

      end
