      subroutine mshmarkcuredge ( markbnedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all edges on the curves with the corresponding curve number
!     At this moment we assume that all elements are linear
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     curve
!     mark
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer markbnedges(*)

!     markbnedges   i/o   Array to mark all edges at the curves
!                         if markbnedges(e) = 0 the edge e is not on a curve
!                         if markbnedges(e)>0 the edge e is on curve
!                         with sequence number markbnedges(e)
!                         At input markbnedges must be zero at output the
!                         curve numbers have been filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, inpelm, kstep, i, &
              node1, node2, inode, iedge, icurv, istar4
      integer, allocatable :: iwork(:), icurmark(:)

!     i              Counting variable
!     icurmark       Work array of length ncurvs to mark composite curves (1)
!                    All other curves are marked 0
!     icurv          Curve number
!     iedge          edge sequence number
!     inode          Node sequence number
!     inpelm         number of points per element
!     istar4         Starting address of KMESHM part 4 in KMESH part M
!     istart         Starting address of curves in kmeshm
!     iwork          integer work array of length nedges for subroutine
!                    iniedge
!                    Must be made 0 at start of the subroutine
!     kstep          Number of nodes along edge minus 1
!     node1          First node of edge
!     node2          Second node of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIEDGE        Returns with edge number corresponding to 2 nodes
!     PRININ         print 1d integer array
!     PRINNMESHL     print nmesh part l
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2460   No edge found corresponding to two nodes
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmarkcuredge' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmarkcuredge'
         call prinnmeshl ( npoint, nmeshl )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      istart = kmeshm(2)-1 ! starting address first curve
      inpelm = 0           ! number of nodes along curve
      kstep = 1            ! number of nodes per edge (-1)

      if ( debug ) write(irefwr,1) 'ncurvs, nedges', ncurvs, nedges

      allocate ( iwork(nedges), icurmark(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges+ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork = 0

!     --- Mark curves to be composite (1) or not

      icurmark = 0
      ninner=kmeshm(1)   ! number of special curves (inner or composite)

      if ( ninner>0 ) then

!     --- There are special curves

         istar4=kmeshm(3)    ! starting address of kmesh part 4

         do i = 0, ninner-1
            icurv = kmeshm(istar4+i)        ! curve number
            if ( icurv<0 ) icurmark(-icurv) = 1  ! composite curve
         end do  ! i = 0, ninner-1

      end if  ! ( ninner>0 )

!     --- loop over curves

      do icurv = 1, ncurvs

         istart = istart+inpelm
         inpelm = kmeshm(5+icurv)-kmeshm(5+icurv-1)

         if ( icurmark(icurv)==0 ) then

!        --- curve is not composite
!            loop over nodes

            do inode = 1, inpelm-1, kstep

               node1 = kmeshm(istart+inode)       ! first node
               node2 = kmeshm(istart+inode+kstep) ! last node
               if ( node1>0 .and. node2>0 ) then

!              --- Nodes exist

                  iedge = iniedge ( node1, node2, nmeshl, iwork )

                  if ( iedge==0 ) then

!                 --- No edge found

                     call errint ( node1, 1 )
                     call errint ( node2, 2 )
                     call errsub ( 2460, 2, 0, 0 )
                     go to 1000

                  end if  ! ( iedge==0 )

                  markbnedges(iedge) = icurv

               end if  ! ( nodenr>0 ) then

            end do  ! inode = 1, inpelm

         end if  ! ( icurmark(icurv)==0 )

      end do  ! icurv = 1, ncurvs

      deallocate ( iwork, icurmark, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( markbnedges, nedges, 'markbnedges' )
         write(irefwr,*) 'End mshmarkcuredge'

      end if  ! ( debug )
      call erclos ( 'mshmarkcuredge' )

      end
