      subroutine mshobstboun2d6 ( ibounelems, iworkedge, &
                                  npointnew, iworkshape, &
                                  nmeshc, nmeshd, nmeshh, coor, coornew, &
                                  iwork, npoint, nmap, nmapmax, npointmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-05-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new points in mesh by subdividing some edges of
!     elements that have all points on the boundary of the mesh and/or
!     the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     edge
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibounelems(*), iworkedge(*), npointmap, &
              npointnew, iworkshape(*), nmeshc(*), nmeshd(*), nmeshh(*), &
              iwork(*), npoint, nmapmax, nmap
      double precision coor(2,*), coornew(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coornew        o    Array containing the new coordinates of the nodes
!     ibounelems     i    Array of length 3+numbounelms (<nelem) containing
!                         the element numbers of the elements that have all
!                         points on the boundary or on the obstacle
!                         Pos. 1 contains the number of elements that have all
!                                points on one obstacle
!                         Pos. 2 contains the number of elements that have all
!                                points on the boundary
!                         Pos. 3 contains the number of elements that have some
!                                points on the boundary and the others on an
!                                obstacle or on different obstacles
!                         The other positions contain the element numbers in the
!                         sequence: first all elements corresponding to pos. 1
!                         then to pos 2 and finally to pos 3
!     iwork          i    integer work array of length npoint
!                         iwork(i) is filled as follows (see pseudo code)
!                          0 for all points outside the obstacle and
!                            the boundaries
!                         -1 for all points inside the obstacle
!                          i for all points on the boundary of the obstacle i
!                         1001 for all points on the outer curves
!                              but not on an obstacle
!                         1002 for all points on the outer curves
!                              and on an obstacle
!     iworkedge      o    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     iworkshape          Work array of length nelem that is filled as follows
!                         iworkshape(ielem) contains the shape number of the
!                         element ielem
!     nmap          i/o   length of second parameter of imap and rmap
!                         At input: declared length
!                         At output: computed length
!     nmapmax       i/o   Actual value of nmap
!     nmeshc         i    nmesh part c
!     nmeshd         i    nmesh part d
!     nmeshh         i    nmesh part h
!     npoint         i    Number of nodal points
!     npointmap      i    Declared value of npointnew
!     npointnew      o    Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, numobstelms, numbounelms, numbnobelms, istart, i, &
              nneigh, ishape, istartelem, istartedge, j, k, iedge, &
              istartnode, node1, node2, nedge_elem, nelem_edge
      double precision xc, yc

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ishape         Shape number of elements
!     istart         First position used at a part
!     istartedge     Starting position of edge in array NMESHD
!     istartelem     Starting address in element array
!     istartnode     ?
!     j              Counting variable
!     k              Counting variable
!     nedge_elem     number of edges corresponding to element
!     nelem_edge     number of elements corresponding to edge
!     nneigh         Number of neighbours of node
!     node1          Node of face
!     node2          Node of face
!     numbnobelms    number of elements that have some points on the boundary
!                    and the others on the obstacle
!     numbounelms    Number of elements that have all points on the
!                    boundary of the obstacle or a set of curves
!     numobstelms    number of elements that have all points on the obstacle
!     xc             array with the coordinates of the centre point of the
!                    element
!     yc             y-coordinate of the midth-point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMAP2D   Fill arrays imap and rmap for a new point inode which is
!                    the mean value of two already existing nodes node1 and
!                    node2 stored in imap/rmap
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iworkshape with the shape numbers of the elements
!     Clear array iworkedge
!     Copy old arrays imap and rmap into imap2 and rmap2
!     npointnew := npoint
!     For all elements ielem with all nodes on boundary do
!        For all edges iedge corresponding to ielem do
!           nneigh := number of triangles corresponding to iedge
!           if ( nneigh=2 ) then
!              create new point
!     For all elements ielem with all nodes on obstacle do
!        For all edges iedge corresponding to ielem do
!           compute coordinates of centroid of edge
!           if ( centroid outside obstacle ) then
!              create new point
!     For all elements ielem with some nodes on obstacle and all others on
!         boundary do
!        For all edges iedge corresponding to ielem do
!           if ( one endpoint on boundary and other on obstacle ) then
!              create new point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun2d6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstboun2d6'

      end if
      if ( ierror/=0 ) go to 1000

      numobstelms = ibounelems(1)
      numbounelms = ibounelems(2)
      numbnobelms = ibounelems(3)

      istart = 3+numobstelms

!     --- For all elements ielem with all nodes on outer boundary do

      do i = 1, numbounelms
         ielem = ibounelems(istart+i)
         istartedge = nmeshd(ielem)
         nedge_elem = nmeshd(istartedge)

!        --- For all edges iedge corresponding to ielem do

         do j = 1, nedge_elem
            iedge = abs(nmeshd(istartedge+j))
            if ( iworkedge(iedge)==0 ) then

!           --- nneigh := number of triangles corresponding to iedge
!               if ( nneigh=2 ) then
!                  create new point

               istartelem = nmeshh(iedge)
               nelem_edge = nmeshh(istartelem)
               nneigh = 0
               do k = 1, nelem_edge
                  ishape = iworkshape(nmeshh(istartelem+k))
                  if ( ishape==3 ) nneigh = nneigh+1
               end do
               if ( nneigh==2 ) then

!              --- nneigh = 2, create new point

                  istartnode = nmeshc(iedge)
                  node1 = nmeshc(istartnode+1)
                  node2 = nmeshc(istartnode+2)
                  xc = 0.5d0*(coor(1,node1)+coor(1,node2))
                  yc = 0.5d0*(coor(2,node1)+coor(2,node2))
                  npointnew = npointnew+1
                  coornew(1,npointnew) = xc
                  coornew(2,npointnew) = yc
                  iworkedge(iedge) = npointnew

                  call mshobstmap2d ( npoint, npointmap, nmap, nmapmax, &
                                      node1, node2, npointnew )
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Node ', npointnew, ' has been ', &
                        'created between nodes ', node1, ' and ', node2, &
                        ' which are on the boundary'

                  end if

               end if

            end if

         end do

      end do
      istart = istart+numbounelms

!     --- For all elements ielem with some nodes on obstacle and all others on
!            boundary or nodes on various obstacles do

      do i = 1, numbnobelms
         ielem = ibounelems(istart+i)
         istartedge = nmeshd(ielem)
         nedge_elem = nmeshd(istartedge)

!        --- For all edges iedge corresponding to ielem do

         do j = 1, nedge_elem
            iedge = abs(nmeshd(istartedge+j))
            if ( iworkedge(iedge)==0 ) then

               istartnode = nmeshc(iedge)
               node1 = nmeshc(istartnode+1)
               node2 = nmeshc(istartnode+2)

!              --- if ( one endpoint on boundary and other on obstacle ) then
!                     create new point

               if ( iwork(node1)/=iwork(node2) ) then

!              --- One point on obstacle the other one on boundary or
!                  other obstacle

                  npointnew = npointnew+1
                  xc = 0.5d0*(coor(1,node1)+coor(1,node2))
                  yc = 0.5d0*(coor(2,node1)+coor(2,node2))
                  coornew(1,npointnew) = xc
                  coornew(2,npointnew) = yc
                  iworkedge(iedge) = npointnew
                  call mshobstmap2d ( npoint, npointmap, nmap, nmapmax, &
                                      node1, node2, npointnew )
                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'Node ', npointnew, ' has been ', &
                        'created between nodes ', node1, ' and ', node2, &
                        ' which are on the boundary and the obstacle', &
                        ' or are on different obstacles'

                  end if

               end if

            end if

         end do

      end do
      nmap = nmapmax

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstboun2d6'

      end if
      call erclos ( 'mshobstboun2d6' )

      end
