      subroutine mshobstms021 ( iwork, nelem, numobst, icrossobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-12-2011 Remove iedges
!        version  1.1  date 08-11-2004 check on intersection of element
!                                      by several obstacles
!        version  1.0  date 20-05-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear array iwork (1:nelem) and set iwork(i) = starting address in
!     iobstcross if element i intersects obstacle iobst
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, iwork(nelem,2), numobst, icrossobst(*)

!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iwork          o    integer work array of length nelem
!                         iwork(ielem) contains the starting address of element
!                         ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!                         iwork(ielem,2) contains the corresponding obstacle
!                         sequence number iobst
!     nelem          i    Number of elements
!     numobst        i    number of obstacles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, istarticross, numintersect, istart, numelms, &
              ielem, nintersect

!     i              Counting variable
!     ielem          Element number
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     nintersect     Number of intersections found
!     numelms        Number of elements
!     numintersect   Number of intersections
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2734   Element is intersected by at least two obstacles
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:nelem)
!     For iobst := 1 (1) numobst do
!        For all elements ielem in icrossobst(iobst) do
!           iwork(ielem) := starting address
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms021' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms021'
         write(irefwr,1) 'nelem', nelem
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork(1:nelem)

      do i = 1, nelem
         iwork(i,1) = 0
         iwork(i,2) = 0
      end do  ! i = 1, nelem

!     --- For iobst := 1 (1) numobst do

      do iobst = 1, numobst

         istarticross = icrossobst(iobst)        ! starting address of obstacle
         if ( debug ) write(irefwr,1) 'istarticross', istarticross
         numintersect = icrossobst(istarticross) ! number of intersections
         istart = istarticross+2*numintersect+1  ! starting address of elements
         numelms = icrossobst(istart)            ! number of elements
         if ( debug ) &
            write(irefwr,1) 'numintersect, istart, numelms', &
                             numintersect, istart, numelms

!        --- For all elements ielem in icrossobst(iobst) do
!               iwork(ielem) := starting address

          do i = 1, numelms
             ielem = icrossobst(istart+1)        ! element number
             if ( iwork(ielem,2)/=0 ) then

!            --- Element in more than one obstacle (not permitted)

                call errint ( ielem, 1 )
                call errint ( iwork(ielem,2), 2 )
                call errint ( iobst, 3 )
                call errsub ( 2734, 3, 0, 0 )

             end if  ! ( iwork(ielem,2)/=0 )
             iwork(ielem,1) = istart+2           ! starting address element
             iwork(ielem,2) = iobst              ! obstacle number
             nintersect = icrossobst(istart+2)   ! number of intersections
             istart = istart+2+nintersect
          end do  ! i = 1, numelms

      end do  ! iobst = 1, numobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms021'

      end if
      call erclos ( 'mshobstms021' )

      end
