      subroutine mshobstms3d ( kmeshcnew, &
                               coornew, kmeshnew, ipointobstnew, &
                               ielemobstnew, iinput, nmap, &
                               kmeshnnew, kmeshmnew, rinputusr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 05-11-2013 Extra parameter rinputusr
!        version  3.0  date 19-09-2013 New parameter list
!        version  2.0  date 24-02-2012 Extra parameter shft
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new mesh from a fixed mesh with an obstacle crossing the mesh
!     Elements that are intersected are subdivided into several elements,
!     such that these new elements all belong to the obstacle or are
!     outside the obstacle
!     In this subroutine points are not moved to the obstacle
!     3d
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     subdivision
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), nmap, &
              ipointobstnew(*), ielemobstnew(*), kmeshnew, iinput(*), &
              kmeshnnew(*), kmeshmnew(*)
      double precision coornew(2,*), rinputusr(*)

!     coornew        o    Array containing the new coordinates of the nodes
!     ielemobstnew   o    New array ielemobst
!     iinput         o    Array to pass some constants to mshobstms
!                         See subroutine mshobstms23d
!     ipointobstnew  o    New array ipointobst
!     kmeshcnew      o    New kmesh part c
!     kmeshmnew      i    New array kmeshm
!     kmeshnew      i/o   New array kmesh
!     kmeshnnew      i    New array kmeshn
!     nmap           i    length of first parameter of imap and rmap
!     rinputusr      i    See subroutine mshobstms
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer shft, ielemnew, npointnew, i, ikelmcnew, &
              mmicrossobst, mmipointobst, iseqkm1,  iseqkm2
      integer, allocatable :: iwork(:,:), iworkpoint(:,:), iedges(:,:), &
                              skipnod(:), edgemoved(:)
      double precision epssav

!     edgemoved      Array of size nedges indicating if nodes on an edge
!                    have been moved (1) or not (0)
!     epssav         Help parameter to save the value of accuracy_obstacle
!     i              Counting variable
!     iedges         Work array of size nedges
!                    If iedges(i) = 0, no point has been created in edge i
!                    If iedges(i)>0, a point has been created in edge i, &
!                    with sequence number iedges(i)
!     ielemnew       Sequence number of element in new mesh
!     ikelmcnew      Starting address of next element in kmeshcnew
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     iwork          integer work array of length nelem*2
!                    iwork(ielem,1) contains the starting address of
!                    element ielem in array icrossobst
!                    If 0, the element is not present in icrossobst
!                    iwork(ielem,2) contains the corresponding obstacle
!                    sequence number iobst
!     iworkpoint     Work array of length npointnew*2
!                    This array is filled as follows:
!                    iworkpoint(i,1) = 0 if node i outside obstacle
!                    iworkpoint(i,1) = 1 if node i inside obstacle
!                    iworkpoint(i,1) = 2 if node i inside obstacle (near
!                                        boundary)
!                    iworkpoint(i,1) = 3 if node i on obstacle
!                    iworkpoint(i,2) contains the corresponding obstacle
!                    sequence number iobst
!     mmicrossobst   memory management sequence number of array icrossobst
!     mmipointobst   memory management sequence number of array ipointobst
!     npointnew      Sequence number of last generated node
!     shft           Parameter that determines if nodes must be shifted (1) or
!                    not (0)
!     skipnod        Array of npoint long in which the nodes are marked
!                    which are not allowed to be moved
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBST3DNM    Create a new mesh from a fixed mesh with an obstacle
!                    crossing the mesh (3d)
!     MSHOBSTCURV    Create new array kmeshm in the case that extra points at
!                    the surface are created when a new mesh is created due to
!                    intersection with an obstacle (2D only)
!     MSHOBSTMS021   Clear array iwork (1:nelem) and set iwork(i) = starting
!                    address in iobstcross if element i intersects obstacle
!     MSHOBSTMS022   Copy part of kmesh to kmeshnew n case of obstacles
!     MSHOBSTMS023   Fill iworkpoint with information about nodal points in and
!                    outside obstacle and corresponding obstacle sequence
!                    numbers
!     MSHOBSTMS024   Adapt the arrays ipointobstnew and ielemobstnew for new
!                    mesh
!     MSHOBSTSURF3   Create new array kmeshn in the case that extra points at
!                    the surface  are created when a new mesh is created due to
!                    intersection with an obstacle (3D, linear triangles)
!     MSHSHIFT2D     Shift node of intersecting curve onto boundary if
!                    node is close (depending on eps) to boundary
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize some parameters
!     Copy part of kmesh to kmeshnew
!     Copy coordinates from coor to coornew
!     Clear array iwork
!     Set iwork to 1 for elements stored in icrossobst
!     Fill iworkpoint with information about nodal points in and outside obst
!     Fill iworkelem with information about elements in and outside obstacle
!     npointnew := npoint
!     ielem := 0
!     ielemnew := 0
!     For ielgrp := 1 (1) nelgrp do
!        inelem := number of elements in group ielgrp
!        inpelm := number of nodes per element in group ielgrp
!        For irelem := 1 (1) inelem do
!           ielem := ielem+1
!           ielemnew := ielemnew+1
!           if ( ielem completely in obstacle or completely outside obstacle )
!              then
!              copy element from kmeshc to kmeshcnew
!           else ( ielem is intersected by obstacle )
!              create new elements
!              copy these elements to kmeshcnew
!              Raise ielemnew
!     adapt ipointobstnew and ielemobstnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms3d' )
      debug = .false.
      shft = iinput(5)
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms3d'
         write(irefwr,1) 'nmap, shft', nmap, shft
         write(irefwr,*) 'ipointobst_old'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

      allocate ( iwork(nelem,2), iworkpoint(nmap,2), iedges(3,nedges), &
                 skipnod(npoint), edgemoved(nedges), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*nelem+2*nmap+3*nedges, 'coornew' )
      if ( ierror/=0 ) go to 1000

!     --- Copy part of kmesh to kmeshnew

      call mshobstms022 ( kmeshnew, iseqkm2, coornew )
      iseqkm = iseqkm1

!     --- Fill first part of imap and rmap with 1

      do i = 1, npoint
         imap(i,2) = i
      end do
      imap(1:npoint,1) = 1
      imap(1:npoint,3) = 0
      rmap(1:npoint,1) = 1d0
      rmap(1:npoint,2) = 0d0
      if ( debug ) write(irefwr,1) 'npoint', npoint

      skipnod = 0  ! initialization
      edgemoved = 0
      iedges = 0

!     --- Clear array iwork and array iedges
!         Fill iwork with starting address for elements stored in icrossobst
!         as well as obstacle sequence numbers

      call sepflpointobst
      call sepflielemobst
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3
      call mshobstms021 ( iwork, nelem, numobst, icrossobst )

!     --- Fill iworkpoint with information about nodal points in and outside
!         obstacle and corresponding obstacle sequence numbers

      if ( debug ) call sepprinipobst ( ipointobst )

      call mshobstms023 ( npoint, iworkpoint, nmap, ipointobst, numobst )

!     --- npointnew := npoint
!         ielem := 0
!         ielemnew := 0

      npointnew = npoint
      ielemnew = 0

!     --- For ielgrp := 1 (1) nelgrp do

      ikelmcnew = 0

      if ( shft==-2 ) then

!     --- Shift nodes that are close to the new boundary

         if ( debug ) write(irefwr,*) 'before mshshift2d'
         call mshshift2d ( skipnod, iworkpoint, coornew, &
                           nmap, rinputusr(1), iwork, edgemoved )
         if ( ierror/=0 ) go to 1000

      end if  ! ( shft==-2 )

!     --- Create new mesh

      epssav = accuracy_obstacle        ! save value of accuracy_obstacle
      accuracy_obstacle = rinputusr(2)  ! define new value of accuracy_obstacle
      call mshobst3dnm ( kmeshcnew, coornew, kmeshnew, nmap, &
                         iwork, iworkpoint, iedges, edgemoved, &
                         ikelmcnew, npointnew, ielemnew )
      accuracy_obstacle = epssav        ! reset value of accuracy_obstacle

!     --- adapt ipointobstnew and ielemobstnew

      call mshobstms024 ( nmap, iworkpoint, &
                          ipointobstnew, npointnew, kmeshnew, &
                          ielemnew, kmeshcnew, iinput, ielemobstnew )

!     --- Update kmeshm if necessary

      call mshobstcurv ( kmeshmnew, iinput(7), iedges, iinput(6), 3 )

!     --- Update kmeshn if necessary

      call mshobstsurf3 ( kmeshnnew, iinput(8), iedges, 3, 3 )

!     --- Fill iinput

      iinput(1) = ikelmcnew

!     --- Remove temporary arrays

      deallocate ( iworkpoint, iwork, iedges, skipnod, edgemoved, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworkpoint' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'ipointobst_new'
         iseqkm = iseqkm2
         call sepprinipobst ( ipointobstnew )
         call sepprinielobst ( ielemobstnew )
         iseqkm = iseqkm1
         write(irefwr,*) 'End mshobstms3d'

      end if
      call erclos ( 'mshobstms3d' )

      end
