      subroutine mshobstms3dq1 ( tetnode, tetnodenew, coornew, &
                                  nextranodes, nelemnew, edgenode, &
                                  inpelm, centroid, midfacenodes, &
                                  facenode, fluidnd, inelem1 )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 06-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subdivide each tetrahedron into the right number of subtetrahedrons
!    Check volumes of subtetrahedrons if necessary, quadratic case
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer edgenode(5,6), inpelm, tetnode(inpelm), tetnodenew(*), &
              nextranodes, nelemnew, facenode(4,4), midfacenodes(4), &
              centroid(*), fluidnd(*), inelem1
      double precision coornew(3,*)

!     centroid       i    Array containing the node numbers of the centroids
!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     facenode       i    Array containing the node numbers of the faces
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     inelem1        i    ?
!     inpelm         i    number of points per element
!     midfacenodes   i    Array containing the node numbers of the centroids
!                         of the faces
!     nelemnew       o    Number of elements in new mesh
!     nextranodes    i    Number of extra nodes in element
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, count(4), isort(4), isorttemp, ledgenode(2,6)
      double precision tetvolold, tetvolnew(1:8), tetvolnewtot

!     count          count(i) is the number of crossed edges on which
!                    tetnode(i) is located
!     i              Counting variable
!     idx            Index
!     isort          if isort(i)>isort(j) then tetnode(i) is at least on
!                    the same number of crossed edges as tetnode(j)
!     isorttemp      temporary storage for an isort element
!     j              Counting variable
!     k              Counting variable
!     ledgenode      for i=1,2 and 1<j< number of crossed edges
!                    ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                    in other words:
!                    edgnode(i,j) = tetnode(ledgenode(i,j))
!     tetvolnew      array of volumes of the new tetrahedrons
!     tetvolnewtot   total volume of new tetrahedrons
!     tetvolold      volume of original tetrahedron
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer idx(4)
      double precision tetvolume

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     MSHOBSTMS3DQ4  Subdivide a quadratic tetrahedron with only one crossed
!                    edge into 2 tetrahedrons
!     MSHOBSTMS3DQ5A Subdivide a quadratic tetrahedron, with only two crossed
!                    edges which share a node, into three tetrahedrons
!     MSHOBSTMS3DQ6A Subdivide a quadratic tetrahedron with three crossed edges
!                    which have one single node in common into four tetrahedrons
!     MSHOBSTMS3DQ6C Subdivide a quadratic tetrahedron, with three crossed edges
!                    which form a curve that is not closed, into four or five
!                    tetrahedrons
!     MSHOBSTMS3DQ7B Subdivide a quadratic tetrahedron with four crossed edges
!                    which form a closed curve
!     MSHOBSTUPDCOOR position intermediate nodes based on the coordinates of the
!                    vertex nodes. Straight triangle/tetrahedron are formed.
!     TETVOLUME      Compute the volume of a tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2268   The volume of the original tetraeder is not equal to the sum of
!           the volumes of the new tetraeder
!    2269   Old volume has different orientation than new volume
!    2270   integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( nextranodes/=0 ) then
!        count(i) := number of intersected edges which contain tetnode(i)
!        ledgenode(1:2,j) are the local numbers of the corner nodes which lay
!                        on the same edge as intersection node j.
!        isort := the sort index of count, in other words
!                 if ( isort(i)>isort(j) then count(i)>count(j)
!     end if
!     subdivide tetrahedron into the right number of subtetrahedrons, &
!     mark each subtetrahedron as inside or outside
!     if ( checksubtetrahedrons) then
!        check volumes of subtetrahedrons
!     end if
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx /1, 3, 5, 10 /
! ======================================================================
!
      call eropen ( 'mshobstms3dq1' )

!     --- Determine some topological properties of the new nodes

      if ( nextranodes/=0 ) then

!     --- There are extra nodes

         do i = 1,4
            count(i) = 0
         end do
         do j = 1,nextranodes
            k = 1
            do i = 1,4
               if ( edgenode(1,j)==tetnode(idx(i)) .or. &
                    edgenode(2,j)==tetnode(idx(i)) ) then
                  count(i) = count(i)+1
                  ledgenode(k,j) = i
                  k = k+1
               end if
            end do
         end do

!        --- sort count(i)

         do i = 1, 4
            isort(i) = i
         end do
         if ( count(1)>count(2)) then
            isort(1) = 2
            isort(2) = 1
         end if
         if ( count(3)>count(4)) then
            isort(3) = 4
            isort(4) = 3
         end if
         if ( count(isort(1))>count(isort(3)) ) then
            isorttemp = isort(1)
            isort(1) = isort(3)
            isort(3) = isorttemp
         end if
         if ( count(isort(2))>count(isort(4))) then
            isorttemp = isort(2)
            isort(2) = isort(4)
            isort(4) = isorttemp
         end if
         if ( count(isort(2))>count(isort(3))) then
            isorttemp = isort(2)
            isort(2) = isort(3)
            isort(3) = isorttemp
         end if

      end if

      if ( nextranodes==0 ) then

!     --- subtetrahedron = original tetrahedron

         do i = 1, inpelm
            tetnodenew(i) = tetnode(i)
         end do
         nelemnew = 1

      else if ( nextranodes==1 ) then

!     --- nextranodes = 1

            write(*,*)'mshobstms3dq4'
         call mshobstms3dq4 ( tetnode, tetnodenew, edgenode, &
                              isort, nelemnew, centroid, &
                              midfacenodes, facenode, fluidnd )

         inelem1=inelem1+1

      else if ( nextranodes==2 ) then

!     --- nextranodes = 2

         if ( count(isort(4))==2 ) then

!        --- the two cut edges share local node isort(4)

            write(*,*)'mshobstms3dq5a'
            call mshobstms3dq5a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, centroid, &
                                  midfacenodes, facenode, fluidnd )

            inelem1=inelem1+1

         else

!        --- the two cut edges do not share a node

            write(*,*)'mshobstms3dq5b not yet implemented'
            call instop

         end if

      else if ( nextranodes==3 ) then

!     --- nextranodes = 3

         if ( count(isort(4))==3 ) then

!        --- all 3 cut edges share local node isort(4)

            write(*,*)'mshobstms3dq6a'
            call mshobstms3dq6a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, centroid, &
                                  midfacenodes, facenode, fluidnd )

            inelem1=inelem1+1

         else if ( count(isort(1))==0 ) then

!        --- node isort(1) is not on any of the 3 cut edges

            write(*,*)'mshobstms3dq6b not yet implemented'
            call instop

         else

!        --- the cut edges form a curve
!            edge [isort(3),isort(4)] is the middle segment of the line

            write(*,*)'mshobstms3dq6c'
            call mshobstms3dq6c ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, centroid, &
                                  midfacenodes, facenode, fluidnd )

            if (nelemnew==5) inelem1=inelem1+1

         end if

      else if ( nextranodes==4 ) then

!     --- nextranodes = 4

         if ( count(isort(4))==3 ) then

!        --- 3 of the 4 cut edges share local node isort(4)

            write(*,*)'mshobstms3dq7a not yet implemented'
            call instop

         else

!        --- the 4 cut edges form an closed curve

            write(*,*)'mshobstms3dq7b'
            call mshobstms3dq7b ( tetnode, tetnodenew, edgenode, &
                                  ledgenode, nelemnew, centroid, &
                                  facenode, midfacenodes, fluidnd  )

            inelem1=inelem1+2

         end if

      else if ( nextranodes==5 ) then

!     --- nextranodes = 5

         write(*,*)'mshobstms3dq8 not yet implemented'
         call instop

      else if ( nextranodes==6 ) then

!     --- nextranodes = 6

         write(*,*)'mshobstms3dq9 not yet implemented'
         call instop

      else

!     --- nextranodes has wrong value

         call errint ( nextranodes, 1 )
         call errchr ( 'nextranodes', 1 )
         call errsub ( 2270, 1, 0, 1 )

      end if

      do i=1,nelemnew
         write(*,*)'elm', i, (tetnodenew((i-1)*inpelm+j), j=1,inpelm)
      end do

!     --- reposition the nodes in the middle of the edges, faces and the
!         centroid based on the position of the vertices

      do i=1,nelemnew
         call mshobstupdcoor(coornew, tetnodenew((i-1)*inpelm+1), 3)
      end do

      if ( checksubtetrahedrons ) then

!     --- check volumes of tetrahedrons

         tetvolold = tetvolume(coornew(1,tetnode(1)), &
                               coornew(1,tetnode(3)), &
                               coornew(1,tetnode(5)), &
                               coornew(1,tetnode(10)) )

         tetvolnewtot = 0.
         do j = 1, nelemnew
            tetvolnew(j) = tetvolume(coornew(1,tetnodenew(1+(j-1)*15)), &
                                     coornew(1,tetnodenew(3+(j-1)*15)), &
                                     coornew(1,tetnodenew(5+(j-1)*15)), &
                                     coornew(1,tetnodenew(10+(j-1)*15)))

!           --- Test orientation of new tetraeder

            if ( tetvolold*tetvolnew(j)<=0d0 ) then

!           --- New sub tetraeder has different orientation than old one

               call errint ( j, 1 )
               call erreal ( tetvolold, 1 )
               call erreal ( tetvolnew(j), 2 )
               call errsub ( 2269, 1, 2, 0 )

            end if
            tetvolnewtot = tetvolnewtot+ tetvolnew(j)
         end do

!        --- test

         if ( abs((tetvolold-tetvolnewtot)/tetvolold)>1d-12 ) then

!        --- Old volume not equal to sum of new volumes

            call erreal ( tetvolold, 1 )
            call erreal ( tetvolnewtot, 2 )
            call errsub ( 2268, 0, 2, 0 )
         end if
      end if

      call erclos ( 'mshobstms3dq1' )
      end
