      subroutine mshpavbnquad2 ( iedge1, inode1, inode2, nmeshl, &
                                 nmeshc, markedges, iwork, nmeshh, &
                                 nmeshd, iworkedge, iworkelem, nquads, &
                                 iworkquads, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find quadrilateral with all 4 nodes on the boundary
!     The first boundary edge iedge1 is given with nodes inode1 and inode2
!     Both from inode1 and inode2 there are more than one edges connecting
!     the opposite boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     node
!     boundary
!     quadrilateral
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedge1, inode1, inode2, nmeshl(*), nmeshc(*), &
              markedges(*), iwork(*), nmeshh(*), nmeshd(*), &
              iworkedge(*), iworkelem(*), nquads, iworkquads(2,*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iedge1         i    edge sequence number
!     inode1         i    First node in edge
!     inode2         i    Second node in edge
!     iwork         i/o   integer work array of length nedges
!                         At start iwork must be set to zero
!                         After each call iwork is reset to 0
!     iworkedge     i/o   work array of length nedges to be used to mark
!                         edges
!                          0:  not marked yet
!                         -1:  boundary edge that has been used already
!                         -2:  diagonal of quadrilateral that may be removed
!     iworkelem     i/o   Work array of length nelem to be used to mark
!                         elements
!                         0:  element is not special
!                         1:  element is triangle with all nodes on the boundary
!                         2:  elements is triangle, that is part of a
!                             quadrilateral with all 4 points on the boundary
!     iworkquads    i/o   work array of size nelem, containing sets of
!                         triangles that may be considered as quads
!                         (1,.) first triangle
!                         (2,.) second triangle
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     nquads        i/o   number of quads stored in iworkquads
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenopposite
      parameter ( lenopposite=10 )
      integer nodeopposite1(lenopposite), nodeopposite2(lenopposite), &
              i, j, inode, numnodes1, numnodes2, itrian1, itrian2, &
              iedge3, iedge5, inode4, ipstart, iedge3sav
      logical acceptquad

!     acceptquad     Check if quadrilateral is acceptable (true) or not
!                    (false)
!     i              Counting variable
!     iedge3         edge sequence number
!     iedge3sav      Contains old value of iedge3
!     iedge5         edge sequence number
!     inode          Node sequence number
!     inode4         Second node in second edge
!     ipstart        Starting address in array
!     itrian1        Sequence number of first triangle
!     itrian2        Sequence number of second triangle
!     j              Counting variable
!     lenopposite    Length of array nodeoopsitei
!     nodeopposite1  Work array to store nodes in opposite boundary connected
!                    to node 1
!     nodeopposite2  Work array to store nodes in opposite boundary connected
!                    to node 2
!     numnodes1      Number of nodes stored innodeopposite1
!     numnodes2      Number of nodes stored innodeopposite2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVBNOPP    Store all nodes on opposite boundary connected to node
!                    in array nodopposite
!     MSHPAVBNQUAD4  Given the nodes inode1 and inode find edge iedge5
!                    connecting both and the 2 triangles itrian1 and itrian2
!                    connected to edge5
!     MSHPAVBNQUAD5  Checks if a better quadrilateral can be found
!     MSHPAVCHKQUAD  Check if quadrilateral to be subdivided is acceptable
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The situation is as follows
!     Situation 1:      Situation 2:
!
!      4  e3 3            4 e3 3
!     -x-----x-          -x----x- &
!      |   / |\           |\   |
!      |  /  |          e4| \  |e2
!   e4 | /   | e2         |  \ |
!      |/    |/          \|   \|/
!     -x-----x-          -x----x- &
!      1  e1 2            1 e1 2
!
!     Store all nodes on opposite boundary connected to node 1
!     Store all nodes on opposite boundary connected to node 2
!     The common node defines the diagonal e5 (1-3 or 2-4)
!     itrian1 := triangle spanned by e1 and e5
!     itrian2 := other triangle connected to edge 5
!     e3 := boundary edge in itrian2
!     Mark e3, e5 and itrian1/2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbnquad2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbnquad2'
         write(irefwr,*) 'iedge1, inode1, inode2 ', &
                          iedge1, inode1, inode2

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Store all nodes on opposite boundary connected to node 1

      call mshpavbnopp ( inode1, nmeshl, numnodes1, lenopposite, &
                         nodeopposite1, markedges, nmeshc )
      if ( ierror.ne.0 ) go to 1000
      if ( debug ) &
         call prinin ( nodeopposite1, numnodes1, 'nodeopposite1' )

!     --- Store all nodes on opposite boundary connected to node 2

      call mshpavbnopp ( inode2, nmeshl, numnodes2, lenopposite, &
                         nodeopposite2, markedges, nmeshc )
      if ( ierror.ne.0 ) go to 1000
      if ( debug ) &
         call prinin ( nodeopposite2, numnodes2, 'nodeopposite2' )

!     --- The common node defines the diagonal e5 (1-3 or 2-4)

      do i = 1, numnodes1

         inode = nodeopposite1(i)
         do j = 1, numnodes2
            if ( nodeopposite2(j).eq.inode ) go to 100
         end do  ! j = 1, numnodes2

      end do  ! i = 1, numnodes1

!     --- If the program reaches this position, no node has been found
!         This means that no quadrilateral can be constructed and we jump

      go to 1000

!     --- itrian1 := triangle spanned by e1 and e5
!         itrian2 := other triangle connected to edge 5
!         e3 := boundary edge in itrian2
!         Since we do not know which of the edges (1-3 or 2-4) we are dealing
!         with, we have to test them both
!         First we start with 1-3

100   call mshpavbnquad4 ( inode1, inode, nmeshl, iwork, iedge5, &
                           itrian1, itrian2, iedge1, iedge3, &
                           nmeshh, nmeshd, markedges, iworkedge, &
                           numnodes1, nodeopposite1, nmeshc, coor )
      if ( ierror.ne.0 ) go to 1000
      if ( debug ) &
         write(irefwr,*) 'inode, iedge3, iedge5, itrian1, itrian2 ', &
                          inode, iedge3, iedge5, itrian1, itrian2

      if ( iedge3.eq.0 ) then

!     --- No edge found, so it must be 2-4, otherwise something is wrong

         call mshpavbnquad4 ( inode2, inode, nmeshl, iwork, iedge5, &
                              itrian1, itrian2, iedge1, iedge3, &
                              nmeshh, nmeshd, markedges, iworkedge, &
                              numnodes2, nodeopposite2, nmeshc, coor )
         if ( ierror.ne.0 ) go to 1000
         if ( debug ) &
            write(irefwr,*) 'inode, iedge3, iedge5, itrian1, itrian2 ', &
                             inode, iedge3, iedge5, itrian1, itrian2

      end if  ! ( iedge3.eq.0 )

      if ( iedge3.eq.0 ) then

!     --- No edge found
!         This means that no quadrilateral can be constructed and we jump

         go to 1000

      end if  ! ( iedge3.eq.0 )

!     --- A quadrilateral has been found, check if this quad is acceptable

      ipstart = nmeshc(iedge3)
      if ( nmeshc(ipstart+1).eq.inode ) then

!     --- inode4 is next node

         inode4 = nmeshc(ipstart+2)

      else

!     --- inode4 is this node

         inode4 = nmeshc(ipstart+1)

      end if  ! ( nmeshc(ipstart+1).eq.inode )
      call mshpavchkquad ( inode1, inode2, inode, inode4, &
                           acceptquad, coor )
      if ( debug ) &
         write(irefwr,*) 'inode1, inode2, inode, inode4, acceptquad ', &
                          inode1, inode2, inode, inode4, acceptquad

      if ( .not. acceptquad ) then

!     --- A quadrilateral has been found but it is not acceptable, &
!         check if a better quadrilateral can be found

         iedge3sav = iedge3
         call mshpavbnquad5 ( inode2, inode, itrian1, itrian2, &
                              iedge3, iedge5, nmeshl, nmeshd, &
                              iwork, nmeshh, iworkedge, markedges )
         if ( ierror.ne.0 ) go to 1000
         if ( iedge3.eq.iedge3sav ) then

!        --- Not a new quadrilateral found

            iedge3 = 0

         else

!        --- A new quadrilateral found, check if it is acceptable

            ipstart = nmeshc(iedge3)
            if ( nmeshc(ipstart+1).eq.inode ) then

!           --- inode4 is next node

               inode4 = nmeshc(ipstart+2)

            else

!           --- inode4 is this node

               inode4 = nmeshc(ipstart+1)

            end if  ! ( nmeshc(ipstart+1).eq.inode )
            call mshpavchkquad ( inode1, inode2, inode, inode4, &
                                 acceptquad, coor )
            if ( debug ) &
               write(irefwr,*) 'inode1, inode2, inode, inode4, ', &
                  'acceptquad ', &
                  inode1, inode2, inode, inode4, acceptquad

            if ( .not. acceptquad ) iedge3 = 0

         end if  ! ( iedge.eq.iedge3sav )

         if ( debug ) &
            write(irefwr,*) 'inode, iedge3, iedge5, itrian1, itrian2 ', &
                             inode, iedge3, iedge5, itrian1, itrian2

      end if  ! ( .not. acceptquad )


      if ( iedge3.eq.0 ) then

!     --- No edge found
!         This means that no quadrilateral can be constructed and we jump

         go to 1000

      end if  ! ( iedge3.eq.0 )

!     --- Mark e3, e5 and itrian1/2

      iworkedge(iedge3) = -1
      iworkedge(iedge5) = -2

      iworkelem(itrian1) = 2
      iworkelem(itrian2) = 2
      nquads = nquads+1
      iworkquads(1,nquads) = itrian1
      iworkquads(2,nquads) = itrian2

1000  call erclos ( 'mshpavbnquad2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavbnquad2'

      end if

      end
