      subroutine mshpavmarkedge ( nquads, iworkquads, nmeshd, markedges, &
                                  nmeshc, thickness, coor, iworkedge, &
                                  nedges, nelem, iworkelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all boundary connecting edges by +1 if they must be subdivided
!     and by -2 if they must be removed
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     mesh_generation
!     edge
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nquads, iworkquads(2,*), nmeshd(*), markedges(*), &
              nmeshc(*), iworkedge(*), nedges, nelem, iworkelem(*)
      double precision coor(2,*), thickness(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iworkedge      o    work array of length nedges to be used to mark
!                         edges
!                          0:  not marked
!                          1:  edge must be subdivided
!                         -2:  diagonal of quadrilateral that may be removed
!     iworkelem      i    Work array of length nelem to be used to mark
!                         elements
!                         0:  element is not special
!                         1:  element is triangle with all nodes on the boundary
!                         2:  elements is triangle, that is part of a
!                             quadrilateral with all 4 points on the boundary
!     iworkquads     i    work array of size nelem, containing sets of
!                         triangles that may be considered as quads
!                         (1,.) first triangle
!                         (2,.) second triangle
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nedges         i    number of edges
!     nelem          i    Number of elements
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     nquads         i    number of quads stored in iworkquads
!     thickness      i    Array containing the thickness in the nodes in exactly
!                         the same sequence as in coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iquad, itrian1, itrian2, iedge5, iedge1, iedge3, i, &
              iedge2, iedge4, itrian
      double precision h1, h2, dist, h

!     dist           Distance
!     h              mean thickness
!     h1             thickness in first edge
!     h2             thickness in second edge
!     i              Counting variable
!     iedge1         edge sequence number (boundary edge)
!     iedge2         edge sequence number (connecting edge)
!     iedge3         edge sequence number (boundary edge)
!     iedge4         edge sequence number (connecting edge)
!     iedge5         edge sequence number (diagonal)
!     iquad          Sequence number of quadrilateral
!     itrian         Sequence number of triangle
!     itrian1        Sequence number of first triangle
!     itrian2        Sequence number of second triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVQUADINF  Extract information about a quadrilateral consisting of
!                    2 triangles
!     MSHPAVTRIANINF Extract information about a triangle
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all quadrilaterals in iworkquads do
!        Compute distance between opposite boundaries
!        Compute average thickness
!        if ( distance > 1.8 * thickness ) then
!           mark connecting edges by 1 and diagonal by -2
!     For all triangles with iworkelem(.) = 1 do
!        Compute distance between top and boundary edge
!        Compute average thickness
!        if ( distance > 1.8 * thickness ) then
!           mark connecting edges by 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmarkedge' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmarkedge'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Clear iworkedge

      do i = 1, nedges
         iworkedge(i) = 0
      end do  ! i = 1, nedges

!     --- For all quadrilaterals in iworkquads do

      do iquad = 1, nquads

         itrian1 = iworkquads(1,iquad)
         itrian2 = iworkquads(2,iquad)

!        --- Compute distance between opposite boundaries
!            Compute average thickness

         call mshpavquadinf ( itrian1, itrian2, iedge5, iedge1, &
                              iedge3, iedge2, iedge4,  nmeshd, &
                              markedges, nmeshc, thickness, coor, &
                              h1, h2, dist )
         if ( debug ) then
            write(irefwr,*) 'iquad, itrian1, itrian2, h1, h2, dist ', &
                             iquad, itrian1, itrian2, h1, h2, dist
            write(irefwr,*) 'iedge5, iedge2, iedge4 ', &
                             iedge5, iedge2, iedge4
         end if  ! ( debug )

         h = max(h1,h2)
         if ( h.gt.0d0 .and. dist.gt.1.8*h ) then

!        --- distance > 1.8 * thickness
!            mark connecting edges by 1 and diagonal by -2

            iworkedge(iedge2) = 1
            iworkedge(iedge4) = 1
            iworkedge(iedge5) = -2

         end if  ! ( h.gt.0d0 .and. dist.gt.1.8*h )

      end do  ! iquad = 1, nquads

!     --- For all triangles with iworkelem(.) = 1 do

      do itrian = 1, nelem

         if ( iworkelem(itrian).eq.1 ) then

!        --- iworkelem = 1, triangle connecting boundaries found
!            Compute distance between top and boundary edge
!            Compute average thickness

            call mshpavtrianinf ( itrian, iedge1, iedge2, iedge3, h, &
                                  dist, nmeshc, thickness, coor, &
                                  markedges, nmeshd )
            if ( debug ) then
               write(irefwr,*) 'itrian, h, dist ', &
                                itrian, h, dist
               write(irefwr,*) 'iedge1, iedge2, iedge3 ', &
                                iedge1, iedge2, iedge3
            end if  ! ( debug )

            if ( h.gt.0d0 .and. dist.gt.1.8*h ) then

!           --- distance > 1.8 * thickness
!               mark connecting edges by 1

               iworkedge(iedge2) = 1
               iworkedge(iedge3) = 1

            end if  ! ( h.gt.0d0 .and. dist.gt.1.8*h )

         end if  ! ( iworkelem(i).eq.1 )

      end do  ! i = 1, nelem

1000  call erclos ( 'mshpavmarkedge' )

      if ( debug ) then

!     --- Debug information

         call prinin ( iworkedge, nedges, 'iworkedge' )
         write(irefwr,*) 'End mshpavmarkedge'

      end if

      end
