      subroutine mshplaxread1 ( nnodes, ntria, cord, itria, isurnr, &
                                irefplax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-06-2013 Extra parameter irefplax
!        version  1.0  date 18-07-2007
!
!   copyright (c) 2007-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read set of triangles that defines the frame for the
!     submesh generator framesurf
!     The file to be read is framesurf_x, where x is the surface number
!     No leading zeros in x!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, ntria, itria(3, ntria), isurnr, irefplax
      double precision cord(3,nnodes)

!     cord           o    Contains the coordinates of the nodes in the
!                         triangle mesh (frame)
!     irefplax       i    File reference number for plaxis
!     isurnr         i    Surface sequence number
!     itria          o    Contains the topology of thetriangle mesh
!     nnodes         i    Number of nodes in triangle mesh
!     ntria          i    number of trianglesin triangle mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ios, itest(nnodes), i1, i2, i3, iref
      double precision x1, y1, z1

!     i              Counting variable
!     i1             First point in triangle
!     i2             Second point in triangle
!     i3             Third point in triangle
!     ios            IOSTAT variable, returns from reading or writing
!     iref           file reference number
!     itest          Work array to mark nodes that have been read
!                    0: not read, 1: read
!     x1             x-coordinate of point
!     y1             y-coordinate of point
!     z1             z-coordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCHKINT       Check if integer is between two values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     This subroutine reads a file with name framesurf_x
!     where x is the surface number
!     The file must be organized as follows:
!     Line 1: n (number of points), m  ( number of triangles)
!     Line 2 ... n+1:  node number followed by 3 coordinates
!     Line n+2, n+m+1: 3 node numbers that form a triangle
!
!     The file has already been opened
!     The first line has been read
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2878   End of file framesur_... while reading coordinates
!    2879   Error in reading of file framesur_... while reading coordinates
!    2880   Node number has been read before
!    2881   End of file framesur_... while reading triangles
!    2882   Error in reading of file framesur_... while reading triangles
!    2883   Double node numbers found in triangles
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read contents of file
!     Check contents
!     Close file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshplaxread1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshplaxread1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iref = irefplax  ! reference number of file

!     --- Clear array itest

      do i = 1, nnodes
         itest(i) = 0
      end do  ! i = 1, nnodes

!     --- Read contents of file
!         Check contents
!         First nodes with coordinates

      do i = 1, nnodes

         read(iref,*,iostat=ios) i1, x1, y1, z1
         if ( ios<0 ) then

!        --- error: end of file found

            call errint ( isurnr, 1 )
            call errint ( i+1, 2 )
            call errsub ( 2878, 2, 0, 0 )
            go to 1000

         else if ( ios>0 ) then

!        --- error in reading of file

            call errint ( isurnr, 1 )
            call errint ( i+1, 2 )
            call errint ( ios, 3 )
            call errsub ( 2879, 3, 0, 0 )
            go to 1000

         end if  ! ( ios<0 )

!        --- Check range of i1

         call erchkint ( i1, 1, nnodes, 'node number' )
         if ( ierror/=0 ) go to 1000

         if ( itest(i1)/=0 ) then

!        --- itest(i1)#0, hence node number has been read before

            call errint ( isurnr, 1 )
            call errint ( i1, 2 )
            call errint ( i+1, 3 )
            call errsub ( 2880, 3, 0, 0 )

         end if  ! ( itest(i1)/=0 )
         itest(i1) = 1
         cord(1,i1) = x1
         cord(2,i1) = y1
         cord(3,i1) = z1

      end do  ! i = 1, nnodes

!     --- Next triangles

      do i = 1, ntria
         read(iref,*,iostat=ios) i1, i2, i3
         if ( ios<0 ) then

!        --- error: end of file found

            call errint ( isurnr, 1 )
            call errint ( i+1, 2 )
            call errsub ( 2881, 2, 0, 0 )
            go to 1000

         else if ( ios>0 ) then

!        --- error in reading of file

            call errint ( isurnr, 1 )
            call errint ( i+1+nnodes, 2 )
            call errint ( ios, 3 )
            call errsub ( 2882, 3, 0, 0 )
            go to 1000

         end if  ! ( ios<0 )

!        --- Check range of i1, i2, i3

         call erchkint ( i1, 1, nnodes, 'node number' )
         call erchkint ( i2, 1, nnodes, 'node number' )
         call erchkint ( i3, 1, nnodes, 'node number' )
         if ( ierror/=0 ) go to 1000
         if ( i1==i2 .or. i1==i3 .or. i2==i3 ) then

!        --- At least 2 node numbers are the same

            call errint ( isurnr, 1 )
            call errint ( i+1+nnodes, 2 )
            call errint ( i, 3 )
            call errint ( i1, 4 )
            call errint ( i2, 5 )
            call errint ( i3, 6 )
            call errsub ( 2883, 6, 0, 0 )

         end if  ! ( i1==i2. or. i1==i3 .or. i2==i3 )

         itria(1,i) = i1
         itria(2,i) = i2
         itria(3,i) = i3

      end do  ! i = 1, ntria

!     --- Close file

      close ( unit=iref)
      call inirmref ( iref )

1000  call erclos ( 'mshplaxread1' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( cord, nnodes, 3, 'coordinates' )
         call prinin1 ( itria, ntria, 3, 'triangles' )
         write(irefwr,*) 'End mshplaxread1'

      end if  ! ( debug )

      end
