      subroutine mshrediscur ( numcurvs, adapcurvs, coornew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Redistribute nodes along set of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcurvs, adapcurvs(numcurvs)
      double precision coornew(*)

!     adapcurvs      i    Array of size numcurvs containing the curve numbers
!                         of the curves to be adapted
!     coornew       i/o   Array containing the new coordinates of the nodes
!     numcurvs       i    Number of curves in adapcurvs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, istar3, k1, k2, nnodes, &
              lentau, numnodes
      double precision , allocatable :: tau(:)

!     i              Counting variable
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     lentau         Overestimate of size of array tau
!     nnodes         Number of nodes
!     numnodes       Number of nodal points that can be used to define the
!                    spline
!     tau            Array to store the relative distances
!                    between points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INILEN         Compute length of array in IBUFFR
!     MSHCURSPLINE   Create a spline through an existing curve
!     MSHREDISCUR1   Find number of nodes of the old curve that must be used to
!                    define the spline
!     MSHTAUCURV     Compute distribution of nodes along curve
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all curves icurnr to be redistributed do
!        Compute distribution of nodes along curve from old coordinates
!        Compute new coordinates using spline interpolation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrediscur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrediscur'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      istar3 = kmeshm(2)-1   ! starting address of node numbers
      lentau = min(npoint, size(kmeshm)-ncurvs)  ! length of array tau
      if ( debug ) &
         write(irefwr,1) 'ndim, npoint, lentau', ndim, npoint, lentau

      allocate ( tau(lentau), stat = error )
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( lentau, 1 )
         call errint ( error, 2 )
         call errchr ( 'tau', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- For all curves icurnr to be redistributed do

      do i = 1, numcurvs

         icurnr = abs(adapcurvs(i))      ! curve number
         k1 = kmeshm(4+icurnr)
         k2 = kmeshm(5+icurnr)-1
         nnodes = k2-k1+1

!        --- Compute distribution of nodes along curve from old coordinates

         call mshtaucurv ( nnodes, kmeshm(istar3+k1), coor, ndim, &
                           tau )
         if ( debug ) call prinrl ( tau, nnodes, 'tau' )

!        --- Find number of nodes of the old curve that must be used to
!            define the spline

         icurnr = adapcurvs(i)    ! curve number with sign
         call mshrediscur1 ( nnodes, numnodes, icurnr, coornew, &
                             coor, kmeshm(istar3+k1), ndim )

!        --- Compute new coordinates using spline interpolation

         call mshcurspline ( nnodes, kmeshm(istar3+k1), coor, ndim, &
                             tau, icurnr, numnodes, coornew )

      end do  ! i = 1, numcurvs

900   deallocate ( tau, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'tau', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrediscur'

      end if  ! ( debug )
      call erclos ( 'mshrediscur' )

      end
