      subroutine mshshift2dset ( iwork, nodesb, &
                                 numnodes, imarknodes, imarkelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the set B with the nodes on the intersected edges
!     that are closest to the interface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(nelem,*), nodesb(*), &
              numnodes, imarknodes(npoint), imarkelem(nelem)

!     imarkelem      o    Array of length nelem to mark all elements as being
!                         inside or outside the obstacle
!                         0: element is either inside the obstacle or
!                            contains at least one node of the boundary
!                         1: element is completely outside the obstacle
!     imarknodes     o    Work array of length npoint to mark nodes
!                         -1  node inside obstacle
!                          0  node outside obstacle
!                          1  node belongs to set B
!     iwork          i    integer work array of length nelem*2
!                         See subroutine mshobstms2d
!     nodesb         o    Set of nodes B of nodes on edges that are intersected
!                         where the node closest to the boundary is used
!     numnodes      o    Number of nodal points in array nodesb
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, inpelm, jump, iielem, &
              ielem, istart, iobst, node, i, ireledge, icross, iedge, &
              ipstart, istarticross, numbnelmobst, numbndobst, &
              numinelmobst, numinnobst, numpntobst
      double precision fact
      logical linear

!     fact           factor defining the intersection of the edge
!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     jump           if 2 the elements are linear, &
!                    if 3 quadratic
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     node           Node number
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2dset' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2dset'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3

!     --- Actual call

!     --- Select the nodes of the edges that cross the interface with minimum
!         distance to the interface

      ielem = 0

      numnodes = 0
      imarknodes = 0

!     --- Mark all elements completely outside the obstacle (1)

      imarkelem = 1

!     --- elements at the boundary and inside are marked 0

      iobst = 1
      istart = ielemobst(iobst)
      numinelmobst = ielemobst(istart)+ielemobst(istart+1)
      numbnelmobst = ielemobst(istart+2)
      istart = istart+3
      imarkelem(ielemobst(istart:istart+numinelmobst+numbnelmobst-1)) = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         linear = ishape==3 .or. ishape==11
         if ( linear ) then
            jump = 2
         else
            jump = 3
         end if  ! ( linear )

         if ( debug ) write(irefwr,1) 'inpelm, jump', inpelm, jump

!        --- loop over elements

         do iielem = 1, inelem

            ielem=ielem+1

            if ( iwork(ielem,1)/=0 ) then

!           --- iwork(ielem,1)#0, hence element is intersected

               if ( debug ) &
                  write(irefwr,1) 'ielem, iwork(ielem,1)', &
                                   ielem, iwork(ielem,1)
               istart = iwork(ielem,1)  ! starting address in icrossobst
               iobst = iwork(ielem,2)
               istarticross = icrossobst(iobst)

!              --- loop over crossed edges

               do i = 1, icrossobst(istart)

                  ireledge = icrossobst(istart+i)
                  icross  = istarticross+1+2*(ireledge-1)
                  iedge = icrossobst(icross)
                  fact  = rcrossobst(icrossobst(icross+1))
                  ipstart = nmeshc(iedge)

                  if ( debug ) then
                     write(irefwr,1) 'i, nodes', &
                         i, nmeshc(ipstart+1),nmeshc(ipstart+jump)
                     write(irefwr,2) 'fact', fact
                  end if  ! ( debug )

                  if ( fact<=0.5d0 ) then

!                 --- first node is in set B

                     node = nmeshc(ipstart+1)

                  else

!                 --- last node is in set B

                     node = nmeshc(ipstart+jump)

                  end if  ! ( fact<=0.5d0 )

                  if ( imarknodes(node)==0 ) then

!                 --- New node found, store node in nodesb

                     imarknodes(node) = 1
                     numnodes = numnodes+1
                     nodesb(numnodes) = node

                     if ( debug ) write(irefwr,1) 'node', node

                  end if  ! ( imarknodes(node)==0 )

               end do  ! i = 1, icrossobst(istart)

            end if  ! ( iwork(ielem)/=0 )

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

!     --- Add nodes on the interface to nodesb
!         Mark all nodes inside the obstacle with -1

      iobst = 1
      istart = ipointobst(iobst)
      numpntobst = ipointobst(istart)   ! Number of nodes inside the obstacle
      numinnobst = ipointobst(istart+1) ! Number of nodes near the boundary
      numbndobst = ipointobst(istart+2) ! Number of nodes on the boundary
      istart = istart+4

      do i = 0, numpntobst+numinnobst-1
         node = ipointobst(istart+i)
         if ( imarknodes(node)==0 ) imarknodes(node)=-1
      end do  ! i = 0, numpntobst+numinnobst-1
      istart = istart+numpntobst+numinnobst

      nodesb(numnodes+1:numnodes+numbndobst) = &
         ipointobst(istart:istart+numbndobst-1)
      imarknodes(nodesb(numnodes+1:numnodes+numbndobst)) = 1
      numnodes = numnodes+numbndobst

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2dset'

      end if  ! ( debug )
      call erclos ( 'mshshift2dset' )

      end
