      subroutine mshsortcurv ( istore, kmeshm, ncurvsistore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sort closed set of curves, such that curves are stored sequentially
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     outer_boundary
!     sort
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), kmeshm(*), ncurvsistore

!     istore        i/o   Contains the set of curves with signs to be sorted
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     ncurvsistore   i    Number of curves in istore
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icopy(ncurvsistore), i, istar3, icurnr, k1, k2, &
              ilast, icount, nleft, jcount
      logical found

!     found          If true a possibility has been found
!     i              Counting variable
!     icopy          Number of volume to be copied
!     icount         Counter
!     icurnr         Curve sequence number
!     ilast          Last node of curve
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     jcount         Counter
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nleft          Number of curves left
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2641   The curves in the set of closed curves are not stored in the right
!           direction
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy array istore(2:ncurvsistore) into icopy
!     icurnr := first curve in istore
!     istart := first node of icurnr
!     ilast := last node of icurnr
!     icount := 1; nleft := ncurvsistore-1
!     while ( nleft > 0 ) do
!        icurnr := curve in icopy that starts with ilast
!        ilast := last node of icurnr
!        icount := icount+1; istore(icount) := icurnr
!        Remove icurnr from icopy; nleft := nleft-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsortcurv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsortcurv'

      end if
      if ( ierror.ne.0 ) go to 1000
      istar3 = kmeshm(2)-1 ! starting address of kmeshm part 3 minus 1

!     --- Copy array istore(2:ncurvsistore) into icopy

      do i = 1, ncurvsistore-1
         icopy(i) = istore(i+1)
      end do

!     --- icurnr := first curve in istore
!         istart := first node of icurnr
!         ilast := last node of icurnr
!         icount := 1; nleft := ncurvsistore-1

      icurnr = istore(1)
      if ( icurnr.gt.0 ) then

!     --- Curve is positive, standard direction

         k1 = kmeshm(4+icurnr)+istar3
         k2 = kmeshm(5+icurnr)-1+istar3

      else

!     --- Curve is negative, reversed direction

         k2 = kmeshm(4-icurnr)+istar3
         k1 = kmeshm(5-icurnr)-1+istar3

      end if  ! ( icurnr.gt.0 )

      ilast = kmeshm(k2)
      icount = 1
      nleft = ncurvsistore-1
      if ( debug ) write(irefwr,*) 'icurnr, ifirst, ilast ', &
          icurnr, kmeshm(k1), ilast

!     --- while ( nleft > 0 ) do

      do while ( nleft.gt.0 )

!     --- icurnr := curve in icopy that starts with ilast

         found = .false.
         jcount = 0
         do while ( .not. found )

!        --- Curve not yet found

            jcount = jcount + 1
            icurnr = icopy(jcount)
            if ( icurnr.gt.0 ) then

!           --- Curve is positive, standard direction

               k1 = kmeshm(4+icurnr)+istar3
               k2 = kmeshm(5+icurnr)-1+istar3

            else

!           --- Curve is negative, reversed direction

               k2 = kmeshm(4-icurnr)+istar3
               k1 = kmeshm(5-icurnr)-1+istar3

            end if  ! ( icurnr.gt.0 )
            if ( debug ) write(irefwr,*) 'icurnr, ifirst, ilast ', &
               icurnr, kmeshm(k1), kmeshm(k2)

            if ( kmeshm(k1).eq.ilast ) then

!           --- Next curve found

               ilast = kmeshm(k2)
               found = .true.

            else if ( kmeshm(k2).eq.ilast ) then

!           --- Next curve found in reversed direction

               icurnr = -icurnr
               ilast = kmeshm(k1)
               found = .true.

            else if ( jcount.ge.nleft ) then

!           --- End of set reached, error

               call errsub ( 2641, 0, 0, 0 )
               go to 1000

            end if  ! ( kmeshm(k1).eq.ilast )

         end do  ! while ( .not. found )

!        --- ilast := last node of icurnr
!            icount := icount+1; istore(icount) := icurnr
!            Remove icurnr from icopy; nleft := nleft-1

         icount = icount+1
         istore(icount) = icurnr
         if ( debug ) write(irefwr,*) 'icurnr, ilast ', icurnr, ilast
         nleft = nleft-1
         do i = jcount, nleft
            icopy(i) = icopy(i+1)
         end do  ! i = jcount, nleft

      end do  ! while ( nleft.gt.0 )

1000  call erclos ( 'mshsortcurv' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsortcurv'

      end if

      end
