      subroutine mshtframecur ( icurnr, cunit, work, &
                                tau, ishape, nelm )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 20-06-2013 New definition file ref numbers
!        version  1.0  date 06-05-2009
!
!   copyright (c) 2009-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine tau array for framecurve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     spline
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurnr, ishape, nelm
      double precision tau(*),work(*), cunit

!     cunit      i    global coarseness of mesh submitted by user
!     icurnr     i    sequencenumber of curve
!     ishape     i    indicator
!                     1  :  linear elements
!                     2  :  quadratic elements
!                     3  :  cubic elements
!     nelm       i    number of elements on curve
!     tau             work array to store the relative arc lengths of
!                     the elements
!     work       i    array containing the relative division
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenname, ios, iref
      character (len=13) filename
      integer, allocatable          :: helpusp(:)
      double precision, allocatable :: helppoints(:,:)
      integer i, itau, iusp, j, ndegree, nh, nhelm, &
              nnodes, npoint, ntype
      double precision coarse, dist, dtau, dx, dy, dz, &
              srest, tauh, x1, y1, z1
      logical check

!     check          indicator if file exist (true) or not
!     coarse         coarseness
!     debug          if true debug statements are carried out otherwise
!     dist           Euclidian distance
!     dtau           difference of tau values
!     dx             difdistance in x
!     dy             difdistance in y
!     dz             difdistance in z
!     error          Return value of allocate or deallocate
!     filename       Name of file
!     helppoints     ?
!     helpusp        ?
!     i              counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     iref           file reference number
!     itau           local position indicator in tau array
!     iusp           helpvariable for userpointnumber
!     j              counting variable
!     lenname        Length of name
!     ndegree        degree of spline
!     nh             local number of elements to be created
!     nhelm          total number of elements to be created
!     nnodes         number of points in framecurve array
!     npoint         number of points on the final framecurve
!     ntype          type of spline curve
!     srest          restdistance of previous part
!     tauh           element length to be used to end up in userpoint
!     x1             x-coordinate first point
!     y1             y-coordinate first point
!     z1             z-coordinate first point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCHKINT       Check if integer is between two values
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number has incorrect value
!    1669   file does not exist
!    2706   error in opening file
!    2910   Not enough space
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Create file name and open file

      filename = 'framecurve_'
      if ( icurnr<0 .or. icurnr>=10000 ) then

!     --- Curve number larger than 9999, not yet implemented

         call errint ( icurnr, 1 )
         call errint ( 10000, 2 )
         call errsub ( 139, 2, 0, 0 )
         go to 1000

      else if ( icurnr<10 ) then

!     --- Curve number between 1 and 9

         lenname = 11
         write(filename(11:lenname),100) icurnr
100      format(i1)

      else if ( icurnr<100 ) then

!     --- Curve number between 10 and 99

         lenname = 12
         write(filename(11:lenname),110) icurnr
110      format(i2)

      else if ( icurnr<1000 ) then

!     --- Curve number between 100 and 999

         lenname = 13
         write(filename(11:lenname),120) icurnr
120      format(i3)

      else

!     --- Curve number between 1000 and 9999

         lenname = 14
         write(filename(11:lenname),130) icurnr
130      format(i4)

      end if  ! ( icurnr<0 .or. icurnr>=10000 )

!     --- Check if file exists

      inquire ( file=filename, exist = check )
      if ( .not. check ) then

!     --- Error: file does not exist

         call errchr ( filename, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if  ! ( .not. fileexist )
      iref = inisetnewref()  ! reference number of file
      open ( unit = iref, file = filename, iostat = ios )
      if ( ios/=0 ) then

!     --- error in opening file

         call errint ( ios, 1 )
         call errchr ( filename, 1 )
         call errsub ( 2706, 1, 0, 1 )
         go to 1000

      end if

!     --- Read number of nodes, type of curve and degree of helpspline
!         and check whether values are ok

      read(iref,*) nnodes, ntype, ndegree

!     --- Check usersubmitted values

      call erchkint ( nnodes,  1, 10000, 'number of nodes' )
      call erchkint ( ntype,   0,     1, 'type of curve'   )
      call erchkint ( ndegree, 0,     0, 'degree of fitting curve')

!     --- Second step: determine all values to be used for the construction
!         of the curve

      allocate ( helpusp(nnodes),      stat = error )
      allocate ( helppoints(4,nnodes), stat = error )

      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 4*nnodes, 1 )
         call errint (  error, 2 )
         call errchr ( 'helppoints', 1 )
         call errsub ( 2910, 2, 0, 1 )
         goto 1000

      end if  ! ( error/= 0 )

!     --- Run through file and read all helpnodes of framecurve

      do i=1, nnodes

         read(iref,*) j, x1, y1, z1, iusp, coarse

         if ( j/=i ) then
            write(irefwr,*) 'wrong order in file, stop'
            go to 1000
         end if

         helppoints(1,i) = x1
         helppoints(2,i) = y1
         helppoints(3,i) = z1
         helppoints(4,i) = coarse

         helpusp(i) = iusp

      end do

      debug = .false.

      if ( debug ) then
         do i = 1 , nnodes
            write(irefwr,*) (helppoints(j,i),j=1,4),helpusp(i)
         end do
      end if

!     --- Determine total length of this curve, determine number
!         of elements and determine tau array

      srest  = 0d0

!     --- First node has tau value zero:

      tau(1) = 0d0

      itau = 1

!     --- Find number of elements (initialize)

      nhelm = 0

      do i = 1 , nnodes - 1

         x1 = helppoints(1,i)
         y1 = helppoints(2,i)
         z1 = helppoints(3,i)

         dx = helppoints(1,i+1) - x1
         dy = helppoints(2,i+1) - y1
         dz = helppoints(3,i+1) - z1

         dist = sqrt( dx*dx + dy*dy +dz*dz )

!        --- Avarage coarseness value:

         coarse = cunit * (helppoints(4,i) + helppoints(4,i+1))/2d0

!        --- Add and check number of elements on this part:

         if ( helpusp(i+1)==0 ) then

!        --- Local node is only helpnode place nodes between:

             nh    = int((srest+dist)/coarse)
             nhelm = nhelm + nh

!            --- Place tau values:

             do j = 1, nh

                tau(itau+ishape) = tau(itau) + coarse
                itau = itau + ishape

             end do

             srest = srest+dist - nh * coarse

         else if ( helpusp(i+1)/= 0 ) then

!        --- End node is userpoint, hence point has to be used:

            nh = int((srest+dist)/coarse+0.5d0)
            nhelm = nhelm + nh

!           --- Determine tau-step

            tauh = (srest+dist)/nh

            srest = 0

            do j = 1, nh
               tau(itau+ishape) = tau(itau) + tauh
               itau = itau + ishape
            end do

         end if

      end do

      if ( ishape/=((itau - 1 ) / nelm) ) then
         write(irefwr,*) 'Error: wrong number of elemnts (mshtframecur)'
         write(irefwr,*) 'Execution stopped. Inform SEPRA'
         go to 1000
      end if

!     --- Number of elements, total length and array tau determined

      npoint    = ishape * nelm + 1

      if ( ishape>1 ) then

!     --- ishape>1, Fill points in between according to isubd
!         The relative division to be used is given in work

         do 380 i = 1 , npoint-1 , ishape
            dtau   = tau(i+ishape)-tau(i)
            do 370 j = 1, ishape-1
               tau(i+j) = tau(i) + dtau * work(j+1)
370         continue
380      continue

      end if

      if ( debug ) then
         write(irefwr,*) 'Values of tauarray found for curve',icurnr
         write(irefwr,*) 'Result of routine mshtframecur'

         do i=1, npoint
            write(irefwr,*) 'node',i,'tau = ',tau(i)
         end do
      end if

!     --- Close file and deallocate helparrays:

      close ( unit = iref )
      call inirmref ( iref )
      if ( error==0 ) deallocate ( helpusp,    stat = error )
      if ( error==0 ) deallocate ( helppoints, stat = error )

1000  continue
      if ( error/=0 ) then

!     --- error#0, errors in closing and/or deallocating

         write(irefwr,*) 'Error in mshnframecur'
         write(irefwr,*) 'Stop execution'
         go to 1000

      end if  ! ( error/= 0 )

      end
