      subroutine mshtrans3d2d ( normal, trans )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2005
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the transformation from 3d to 2d for a plane given the
!     normal
! **********************************************************************
!
!                       KEYWORDS
!
!     plane
!     2d
!     3d
!     transformation
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision normal(3), trans(3,3)

!     normal         i    normal vector on the plane (normalised)
!     trans          o    array containing the inverse of the transformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t1(3), t2(3)

!     t1             first tangential vectors in plane (normalized)
!     t2             second tangential vectors in plane (normalized
!                    and orthogonal  to t1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ININORMOUTPROD Compute the normalized outer product of vectors v1 and v2
!                    and store in v3
!     MSHTRANSMAT    Compute the inverse transformation matrix to be used to
!                    map a 3d plane onto 2d coordinates, given 2 tangential
!                    vectors and the normal
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Let the normal n be given
!     if ( n = (1,0,0) ) then
!        t1 = (0,1,0);  t2 = (0,0,1)
!     else if ( n = (0,1,0) ) then
!        t1 = (1,0,0);  t2 = (0,0,1)
!     else if ( n = (0,0,1) ) then
!        t1 = (1,0,0);  t2 = (0,1,0)
!     else if ( n_1 not close to 1 ) then
!        t1 := (1,0,0);  t2 := t1 x n; t1 := t2xn
!     else ( n_1 close to 1 ) then
!        t1 := (0,1,0);  t2 := t1 x n; t1 := t2xn
!     T := (t1 t2 n),  trans := T^-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtrans3d2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtrans3d2d'

      end if
      if ( ierror/=0 ) go to 1000

      if ( normal(1)>=1d0-1d-10 ) then

!     --- n = (1,0,0)
!         t1 = (0,1,0);  t2 = (0,0,1)

         t1(1) = 0d0
         t1(2) = 1d0
         t1(3) = 0d0

         t2(1) = 0d0
         t2(2) = 0d0
         t2(3) = 1d0

      else if ( normal(2)>=1d0-1d-10 ) then

!     --- n = (0,1,0)
!         t1 = (1,0,0);  t2 = (0,0,1)

         t1(1) = 1d0
         t1(2) = 0d0
         t1(3) = 0d0

         t2(1) = 0d0
         t2(2) = 0d0
         t2(3) = 1d0

      else if ( normal(3)>=1d0-1d-10 ) then

!     --- n = (0,0,1)
!         t1 = (1,0,0);  t2 = (0,1,0)

         t1(1) = 1d0
         t1(2) = 0d0
         t1(3) = 0d0

         t2(1) = 0d0
         t2(2) = 1d0
         t2(3) = 0d0

      else if ( normal(1)<=0.9d0 ) then

!     --- n_1 not close to 1
!         t1 := (1,0,0);  t2 := t1 x n; t1 := t2xn

         t1(1) = 1d0
         t1(2) = 0d0
         t1(3) = 0d0
         call ininormoutprod ( t1, normal, t2 )
         call eloutr ( t2, normal, t1 )

      else

!     --- n_1 close to 1
!         t1 := (0,1,0);  t2 := t1 x n; t1 := t2xn

         t1(1) = 0d0
         t1(2) = 1d0
         t1(3) = 0d0
         call ininormoutprod ( t1, normal, t2 )
         call eloutr ( t2, normal, t1 )

      end if  ! ( normal(1)>=1d0-1d-10 )

!     --- T := (t1 t2 n),  trans := T^-1

      call mshtransmat ( t1, t2, normal, trans )

1000  call erclos ( 'mshtrans3d2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshtrans3d2d'

      end if

      end
