      subroutine refineloc ( meshin, meshout, isol, iinputrefloc, maploc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 19-09-2013 New call to mshobstms07
!        version  2.2  date 23-04-2012 Extension for parallel
!        version  2.1  date 05-03-2012 Clear kmesh part h
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine an existing mesh locally
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: meshin, meshout, isol(*), iinputrefloc(*)
      type (mapping) :: maploc

!     iinputrefloc   i    Integer input array for subroutine refloc
!                         For a description see subroutine mshreflocinp
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     maploc         i    Mapping information
!     meshin         i    Sequence number of first mesh to be used
!     meshout        i    Sequence number of last mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelemnew, npointnew, typerefine, irefarr, lencoornew, &
              ielmfaces(1), lenkmscnew, leng1, leng2, lennewtrans, &
              lenkmsnnew, iseqkm1, iseqkm2, &
              linear, idimedges, lenkmsmnew, lenkmscnewsv, lenkmsmnewsv, &
              lenkmsnnewsv, npoint_glob, iinput(20)
      integer, allocatable :: kmeshcnew(:), kmeshnnew(:), &
                              iedges(:), refarray(:), &
                              kmeshmnew(:), elements(:), &
                              newblockinfo(:), newtransinfo(:), &
                              newmapping(:), istorest(:,:)
      integer, pointer :: kmeshnew

!     elements       Array of length levels(nlevels) (<=nelem)
!                    containing the reordered element sequence numbers
!                    So elements(1) contains the first element number to
!                    be used, elements(2) the second one and so on
!     idimedges      Length of first dimension of array iedges
!                    (linear:1, quadratic: 3)
!     iedges         Work array of size nedges
!     ielmfaces      Contains faces of elements
!     iinput         integer input array
!                    See subroutine mshobstms23d
!     irefarr        Sequence number of marker array in array isol
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     istorest       array of length (4,iacnodes) containing information
!                    for the various blocks
!                    istorest(1,iblock):  starting address of block iblock
!                    istorest(2,iblock):  number of edges of block iblock
!                    istorest(3,iblock):  starting address in transportinfo
!                    istorest(4,iblock):  sequence number of neighbor
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     kmeshnew       New array kmesh
!     kmeshnnew      New array kmeshn
!     lencoornew     Estimated length of array coornew1d
!     leng1          Length of new array transinfo
!     leng2          Length of new array blockinfo
!     lenkmscnew     Length of array kmshcnew
!     lenkmscnewsv   Help parameter to store original value of lenkmscnew
!     lenkmsmnew     Length of array kmshmnew
!     lenkmsmnewsv   Help parameter to store original value of lenkmsmnew
!     lenkmsnnew     Length of array kmshnnew
!     lenkmsnnewsv   Help parameter to store original value of kmshnnew
!     lennewtrans    Length of new array transinfo
!     linear         Indicates if linear elements are used (1) or
!                    quadratic (2)
!     nelemnew       Number of elements in new mesh
!     newblockinfo   New array blockinfo
!     newmapping     contains global node numbers for refined mesh
!     newtransinfo   New array transportinfo
!     npoint_glob    Number of nodes in global mesh
!     npointnew      Sequence number of last generated node
!     refarray       Array of size nelem that is filled with a 0
!                    if the element ielem must not be subdivided and a 1
!                    if it must
!     typerefine     Type of criterion to be used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHDELKMESHH   Delete kmesh part h and subarrays
!     MSHOBSTMS05    Update kmeshm and kmeshn if necessary
!     MSHOBSTMS06    Remove some subarrays of kmeshnew that are no longer valid
!                    Recompute topology
!     MSHOBSTMS07    Make new arrays permanent for subroutines mshobstms
!                    and refineloc
!     MSHOBSTMS08    Delete some parts of kmeshnew and copy parts l and m
!                    from kmesh
!     MSHREFESTIMATE Compute some parameters to estimate array length in case of
!                    local refinement
!     MSHREFLOCAL    Refine an existing mesh locally and store the result in a
!                    new array kmesh
!     MSHREFLOCCHK   Put control integers around the temporary arrays
!                    in refineloc
!     MSHREFLOCCHK1  Check control integers around the temporary arrays
!                    in refineloc
!     MSHREFLOCINP   Extract parameters from input array for subroutine
!                    refineloc
!     MSHREFPAR      Refill arrays transinfo, blockinfo and kmeshpar
!     PRININ         print 1d integer array
!     PRINMAP1       Actual printing of array imap
!     PRINRL         Print 1d real vector
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Perform some tests
!     Activate arrays
!     Compute some parameters
!     Compute starting addresses
!     Reserve temporary space
!     Put control integers around these arrays
!     next permanent arrays
!     Replace real starting addresses by their "buffer" variant
!     Refine mesh locally
!     Update kmeshm and kmeshn if necessary
!     Copy contents of imap and rmap to final positions
!     Check control integers
!     Make arrays permanent and copy to these permanent positions
!     Remove temporary arrays from the end of ibuffr
!     Remove some subarrays of kmeshnew that are no longer valid
!     Recompute topology
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'refineloc' )
      debug = .false.

      kmesh => kmesh2d(meshin)
      call sepfillkmesh(kmesh)
      iseqkm1 = iseqkm
      if ( meshout==meshin ) then
         kmeshnew => kmesh2d(maxmesh+1)
      else
         kmeshnew => kmesh2d(meshout)
      end if  ! ( meshout==meshin )
      iseqkm2 = kmeshnew
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from refineloc'
         call prinin ( iinputrefloc, 3, 'iinputrefloc' )
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npoint_glob = npoint
      if ( ndim<1 .or. ndim>3 ) then

!     --- ndim # 1, 2, 3

         call errint ( ndim, 1 )
         call errint ( 1, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim<1 .or. ndim>3 )

!     --- Delete some parts of kmeshnew and copy parts l from kmesh

      call mshobstms08 ( kmeshnew )

!     --- Extract parameters from input array

      call mshreflocinp ( iinputrefloc, typerefine, irefarr )
      if ( debug ) &
         write(irefwr,1) 'typerefine, irefarr', typerefine, irefarr

!     --- Activate arrays

      if ( ks(isol(irefarr))%typevec==116 ) then
      else
      end if   ! ( ks(isol(irefarr))%typevec==110 )

      call sepfillkmesh(kmesh)

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( nmesh(42)==0 ) call sepfillnmeshl

      if ( parallel ) then

!     --- Parallel case activate arrays transinfo and blockinfo

         if ( nmesh(38)==0 ) call sepfillnmeshh

         npoint_glob = kmeshpar(1)

         leng1 = size ( transinfo )*2
         leng2 = size ( blockinfo )
         length = leng1+leng2
         lennewtrans = leng1
         if ( debug ) &
            write(irefwr,1) 'lennewtrans, leng2', lennewtrans, leng2
         allocate ( newtransinfo(leng1), newblockinfo(leng2), &
                    istorest(4,iacnodes), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'newtransinfo' )
         if ( ierror/=0 ) go to 1000

         istorest = 0  ! starting addresses in istore

      end if  ! ( parallel )
      if ( ierror/=0 ) go to 1000

      if ( ndim==3 ) then

!     --- ndim = 3, kmeshn and nmeshk

         if ( nmesh(41)==0 ) call sepfillnmeshk

      end if

      call mshrefestimate ( nelemnew, npointnew, &
                            lenkmscnew, linear, idimedges, &
                            lenkmsnnew, lencoornew, lenkmsmnew )
      length = lenkmscnew+4*nedges+2*nelem+lenkmsnnew+ &
               lenkmsmnew+lencoornew+7*2

      if ( debug ) then
         write(irefwr,1) 'length, lenkmscnew, nedges, nelem', &
                          length, lenkmscnew, nedges, nelem
         write(irefwr,1) 'lenkmsnnew, lenkmsmnew, lencoornew', &
                          lenkmsnnew, lenkmsmnew, lencoornew
         write(irefwr,1) 'ndim', ndim
      end if  ! ( debug )

!     --- Reserve space for temporary arrays

      allocate ( coornew1d(0:lencoornew+1), rmap(npointnew,2), &
                 kmeshcnew(0:lenkmscnew+1), &
                 kmeshnnew(0:lenkmsnnew+1), iedges(nedges), &
                 imap(npointnew,3), kmeshmnew(0:lenkmsmnew), &
                 elements(nelem), refarray(nelem), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'coornew1d' )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Parallel case, allocate newmapping

         allocate ( newmapping(npointnew), stat = error )
         if ( error/=0 ) call eralloc ( error, npointnew, 'newmapping' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( parallel )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lencoornew', lencoornew
         write(irefwr,1) 'lenkmscnew, nedges, nelem', &
                          lenkmscnew, nedges, nelem
         write(irefwr,1) 'lenkmsnnew, lenkmsmnew, npointnew', &
                          lenkmsnnew, lenkmsmnew, npointnew
         write(irefwr,1) 'length', length

      end if  ! ( debug )

!     --- Put control integers around these arrays

      call mshreflocchk ( kmeshcnew, kmeshnnew, &
                          coornew1d, kmeshmnew, lencoornew, &
                          lenkmscnew,  lenkmsnnew, lenkmsmnew )
      lenkmscnewsv = lenkmscnew
      lenkmsmnewsv = lenkmsmnew
      lenkmsnnewsv = lenkmsnnew

!     --- next permanent arrays

      if ( parallel ) then

!     --- Parallel case use arrays transinfo and blockinfo
!         Copy old blockinfo into new one

         newblockinfo(1:leng2) = blockinfo(1:leng2)

      end if  ! ( parallel )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( isol(irefarr), 5, 'sol_irefarr' )
      if ( ks(isol(irefarr))%typevec==116 ) then

!     --- Array is real array defined per element
!         Store as integer in refarray

         usoltemp => ks(isol(irefarr))%sol
         if ( debug ) call prinrl ( usoltemp, nelem, 'refarray' )

         refarray(1:nelem) = nint(usoltemp(1:nelem))

!        --- Refine mesh locally

         call mshreflocal ( kmeshcnew(1), coornew1d(1), iedges, &
                            kmeshnew, elements, refarray, lenkmscnew, &
                            newblockinfo, newtransinfo, lennewtrans, &
                            istorest, leng2 )

      else

!     --- Array is special integer type

         isoltemp => ks(isol(irefarr))%isol
         call mshreflocal ( kmeshcnew(1), coornew1d(1), iedges, &
                            kmeshnew, elements, isoltemp, lenkmscnew, &
                            newblockinfo, newtransinfo, lennewtrans, &
                            istorest, leng2 )

      end if  ! ( ks(isol(irefarr))%typevec==116 ) then
      if ( ierror/=0 ) go to 1000

      if ( parallel ) &
         call inichkarlen ( lennewtrans, leng1, 'transinfo' )
      if ( ierror/=0 ) go to 1000

!     --- Update kmeshm and kmeshn if necessary
!         Copy contents of imap and rmap to final positions

      call mshobstms05 ( kmeshnnew(1), lenkmsnnew, iedges, linear, &
                         ielmfaces, npointnew, kmeshnew, &
                         idimedges, kmeshmnew(1), lenkmsmnew )

!     --- Check control integers

      call mshreflocchk1 ( kmeshcnew, kmeshnnew, &
                           coornew1d, kmeshmnew, lencoornew, &
                           lenkmscnewsv,  lenkmsnnewsv, lenkmsmnewsv )

!     --- Make new arrays permanent
!         It concerns kmesh parts c, i, m, n, t

      nelemnew = km(iseqkm2)%kmesh(9)
      iinput(4) = npointnew
      iinput(7) = lenkmsmnew
      iinput(8) = lenkmsnnew
      iinput(12) = lenkmscnew
      call mshobstms07 ( kmeshnew, coornew1d(1), kmeshcnew(1), &
                         kmeshnnew(1), kmeshmnew(1), iinput )

!     --- Make arrays permanent and copy to these permanent positions

      if ( associated(maploc%imap) ) deallocate ( maploc%imap, maploc%rmap )
      allocate ( maploc%imap(npointnew,3), maploc%rmap(npointnew,2) )
      maploc%imap = imap(1:npointnew,1:3)
      maploc%rmap = rmap(1:npointnew,1:2)
      maploc%iseqkm1 = iseqkm1
      maploc%iseqkm2 = iseqkm2
      maploc%copy = .false.
      maploc%equal = .false.

!     --- Copy kmeshl if it exists

      if ( kelml/=0 ) then

!     --- kmeshl exists

         length = size(kmeshl)
         if ( km(iseqkm2)%kmesh(26)>0 ) deallocate ( km(iseqkm2)%kmeshl)
         allocate ( km(iseqkm2)%kmeshl(length) )
         km(iseqkm2)%kmeshl = kmeshl
         km(iseqkm2)%kmesh(26) = length

      end if  ! ( kelml/=0 )

!     --- Remove some subarrays of kmeshnew that are no longer valid
!         Recompute topology

      call mshobstms06 ( meshin, meshout )

!     --- Remove kmesh part h

      call mshdelkmeshh

      if ( parallel ) then

!     --- parallel computation, the arrays transinfo blockinfo and kmeshpar
!         must be refilled

         call mshrefpar ( kmeshnew, newtransinfo, &
                          lennewtrans, newblockinfo, leng2, &
                          npoint_glob, newmapping, istorest )

      end if  ! ( parallel )

      if ( debug ) then

!     --- debug true, print map

         call prinmap1 ( npointnew )

      end if

!     --- Remove temporary arrays

      deallocate ( kmeshcnew, kmeshnnew, elements, &
                   iedges, imap, coornew1d, rmap, kmeshmnew, &
                   refarray, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coornew1d' )

      if ( parallel ) then

!     --- Parallel case, deallocate parallel arrays

         deallocate ( newblockinfo, newtransinfo, newmapping, &
                      istorest, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'newblockinfo' )

      end if  ! ( parallel )
      call sepfillkmesh(kmeshnew)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End refineloc'

      end if
      call erclos ( 'refineloc' )

      end subroutine refineloc
