      subroutine sepnormalsurf ( kmeshn, coor, normal, isurnr, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal onto the first element of surface
!     The surface must be stored in kmeshn
! **********************************************************************
!
!                       KEYWORDS
!
!     normal_vector
!     surface
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmeshn(*), isurnr, inpelm
      double precision, intent(in) :: coor(3,*)
      double precision, intent(out) :: normal(3)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inpelm         i    number of points per element
!     isurnr         i    Surface sequence number
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     normal         o    Infor entry with respect to the array containing the
!                         normal vectors along the boundaries
!                         This array is only used in case of curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nodp1, nodp2, nodp3
      double precision :: t1(3), t2(3), h

!     h              Help variable to store the length of a vector
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     nodp3          Node sequence number
!     t1             First tangential vector
!     t2             Second tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute cross product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2337   Element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     t1 := first tangential vector
!     t2 := second tangential vector
!     n := t1 x t2
! ======================================================================
!
      call eropen ( 'sepnormalsurf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnormalsurf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if element is linear or quadratic

      if ( inpelm<=5 ) then

!     --- element is linear, use nodes 1, 2 and 3

         nodp1 = kmeshn(1)
         nodp2 = kmeshn(2)
         nodp3 = kmeshn(3)

      else

!     --- element is quadratic, use nodes 1, 3 and 5

         nodp1 = kmeshn(1)
         nodp2 = kmeshn(3)
         nodp3 = kmeshn(5)

      end if  ! ( inpelm==3 .or. inpelm.eq.4 .or. inpelm.eq.5 )

!     --- Compute tangential vectors

      t1 = coor(:,nodp2)-coor(:,nodp1)
      t2 = coor(:,nodp3)-coor(:,nodp2)

!     --- Compute normal vector

      call eloutr ( t1, t2, normal )
      h = sqrt(normal(1)**2+normal(2)**2+normal(3)**2)
      if ( h<=0d0 ) then

!     --- h<=0

         call errint ( nodp2, 1 )
         call errint ( 1, 2 )
         call errint ( isurnr, 3 )
         call errwar ( 2337, 3, 0, 0 )
         h = 1d0

      end if
      h = 1d0/h
      normal = normal*h

1000  call erclos ( 'sepnormalsurf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnormalsurf'

      end if  ! ( debug )

      end subroutine sepnormalsurf
