      subroutine er00773 ( ichint, irule, ndim, n, usolbn, &
                           coorbn, value, mcomp, mvalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-03-2013 Correction dimension dx
!        version  1.0  date 06-10-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual computation of boundary integrals over curves
!     irule = 3, 4: simpsons rule   (complex)
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichint, irule, ndim, n, mcomp, mvalue
      double precision coorbn(ndim,n)
      complex * 16 usolbn(mcomp,n), value(mvalue)

!     coorbn     i    array containing the co-ordinates of the curves
!     ichint     i    indicates the kind of integral to be computed.
!                     possibilities:
!                      1  /  f u  ds
!                      2  /  f  u.n  ds
!                      3  /  f  u.t  ds
!                      4  /  f  n. sigma .n  ds
!                      5  /  f  n. sigma .t  ds
!                      6  /  f  n. sigma     ds
!                      7  /  f  u  n  ds
!                      8  sum f(i) u(i)
!                      9  /  f ds
!     irule      i    type of numerical integration.  possibilities:
!                      1  trapezoid rule,  cartesian co-ordinates
!                      2  trapezoid rule,  axisymmetric co-ordinates
!                      3  simpson's rule   cartesian co-ordinates
!                      4  simpson's rule   axisymmetric co-ordinates
!     mcomp      i    number of components
!     mvalue     i    Number of components used in integral
!     n          i    number of points along the boundary
!     ndim       i    Dimension of the space.
!     usolbn     i    array containing the solution along the curves
!     value     i/o   Value of the integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision h, anorm(2,3), x3(ndim), xs(ndim,3), &
                       dx(3), x1(ndim), x2(ndim)
      complex * 16 v(3,mcomp),  u(3,mcomp)

!     anorm          In this array the normal vector is stored
!                    anorm(j,i) contains the j th component of the normal
!                    vector in the i-th integration point
!     dx             length of vector in x-direction
!     h              Step size
!     i              Counting variable
!     j              Counting variable
!     u              contains function values in interval
!     v              contains multiplication of tensor with normal
!     x1             coordinates of first point
!     x2             coordinates of second point
!     x3             coordinates of third point
!     xs             contains factors for curved integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3083   Number of nodes in boundary is even for Simpson's rule
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The simpsons rule for / f ds is equal to
!
!     sum_elements h/6(f(1)+4f(2)+f(3))
!
!     with f(1) and f(3) the values in begin and end point
!
!     In a curved case we need an adaptation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er00773' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er00773'
         write(irefwr,1) 'ichint, irule, ndim, n, mcomp, mvalue', &
                          ichint, irule, ndim, n, mcomp, mvalue
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d15.7) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check if the number of points is odd

      if ( mod(n,2)==0 ) then

!     --- Number of points is even, give error message

         call errsub ( 3083, 0, 0, 0 )
         go to 1000

      end if  ! ( mod(n,2)==0 )

!     --- Initialize the arrays

      u = (0d0,0d0)

!     --- first point
!         Fill co-ordinates

      x1(:)=coorbn(:,1)

      select case ( ichint )

      case(1, 9)

!     --- ichint = 1, 9:  /  f u  ds or / f ds

         u(1,1) = usolbn(1,1)  ! u in first point

         do i = 1, n-1, 2   ! 3 points per interval

            x2(:)=coorbn(:,i+1) ! x in mid point
            x3(:)=coorbn(:,i+2) ! x in end point
            u(2,1)=usolbn(1,i+1)          ! u in mid point
            u(3,1)=usolbn(1,i+2)          ! u in end point

!           --- Compute factors for integration in array xs

            xs(:,1)=-1.5d0*x1(:)+2d0*x2(:) &
                         -0.5d0*x3(:)
            xs(:,2)=.5d0*(x3(:)-x1(:))
            xs(:,3)=0.5d0*x1(:)-2d0*x2(:) &
                         +1.5d0*x3(:)

!           --- Compute Jacobian and store in array dx

            do j = 1, 3
               dx(j)=sqrt(dot_product(xs(:,j),xs(:,j)))
            end do

            value(1) = value(1)+u(1,1)*dx(1)+ &
                       4d0*u(2,1)*dx(2)+u(3,1)*dx(3)

!           --- Copy values of end point in new begin point

            u(1,1) = u(3,1)
            x1 = x3

         end do  ! i = 1, n-1

      case(2)

!     --- ichint = 2:  /  f u.n  ds

         u(1,:) = usolbn(:,1)  ! u in first point

         do i = 1, n-1, 2   ! 3 points per interval

            x2(:)=coorbn(:,i+1) ! x in mid point
            x3(:)=coorbn(:,i+2) ! x in end point
            u(2,:)=usolbn(:,i+1)          ! u in mid point
            u(3,:)=usolbn(:,i+2)          ! u in end point

!           --- Compute factors for integration in array xs

            xs(:,1)=-1.5d0*x1(:)+2d0*x2(:) &
                         -0.5d0*x3(:)
            xs(:,2)=.5d0*(x3(:)-x1(:))
            xs(:,3)=0.5d0*x1(:)-2d0*x2(:) &
                         +1.5d0*x3(:)

            value(1)=value(1)+xs(2,1)*u(1,1)+4d0*xs(2,2)*u(2,1) &
                     +xs(2,3)*u(3,1)-xs(1,1)*u(1,2)- &
                     4d0*xs(1,2)*u(2,2)-xs(1,3)*u(3,2)

!           --- Copy values of end point in new begin point

            u(1,:) = u(3,:)
            x1 = x3

         end do  ! i = 1, n-1

      case(3)

!     --- ichint = 3:  /  u.t  ds

         u(1,:) = usolbn(:,1)  ! u in first point

         do i = 1, n-1, 2   ! 3 points per interval

            x2(:)=coorbn(:,i+1)  ! x in mid point
            x3(:)=coorbn(:,i+2)  ! x in end point
            u(2,:)=usolbn(:,i+1) ! u in mid point
            u(3,:)=usolbn(:,i+2) ! u in end point

!           --- Compute factors for integration in array xs

            xs(:,1)=-1.5d0*x1(:)+2d0*x2(:) &
                         -0.5d0*x3(:)
            xs(:,2)=.5d0*(x3(:)-x1(:))
            xs(:,3)=0.5d0*x1(:)-2d0*x2(:) &
                         +1.5d0*x3(:)

            do j = 1, ndim
               value(1)=value(1)+xs(j,1)*u(1,j)+4d0*xs(j,2)*u(2,j) &
                            +xs(j,3)*u(3,j)
            end do

!           --- Copy values of end point in new begin point

            u(1,:) = u(3,:)
            x1 = x3

         end do  ! i = 1, n-1

      case(4)

!     --- ichint = 4:  /  n. sigma .n  ds

         u(1,:) = usolbn(:,1)  ! u in first point

         do i = 1, n-1, 2   ! 3 points per interval

            x2(:)=coorbn(:,i+1)  ! x in mid point
            x3(:)=coorbn(:,i+2)  ! x in end point
            u(2,:)=usolbn(:,i+1) ! u in mid point
            u(3,:)=usolbn(:,i+2) ! u in end point

!           --- Compute factors for integration in array xs

            xs(:,1)=-1.5d0*x1(:)+2d0*x2(:) &
                         -0.5d0*x3(:)
            xs(:,2)=.5d0*(x3(:)-x1(:))
            xs(:,3)=0.5d0*x1(:)-2d0*x2(:) &
                         +1.5d0*x3(:)

!           --- Compute normal vector

            do j = 1, 3
               h=1d0/sqrt(xs(1,j)**2+xs(2,j)**2)
               anorm(1,j) = xs(2,j)*h
               anorm(2,j) =-xs(1,j)*h
            end do

!           --- Store n . sigma in v

            v(1,1)=anorm(1,1)*u(1,1)+anorm(2,1)*u(1,2)
            v(1,2)=anorm(1,1)*u(1,2)+anorm(2,1)*u(1,3)
            v(2,1)=anorm(1,2)*u(2,1)+anorm(2,2)*u(2,2)
            v(2,2)=anorm(1,2)*u(2,2)+anorm(2,2)*u(2,3)
            v(3,1)=anorm(1,3)*u(3,1)+anorm(2,3)*u(3,2)
            v(3,2)=anorm(1,3)*u(3,2)+anorm(2,3)*u(3,3)

            value(1)=value(1)+xs(2,1)*v(1,1)+4d0*xs(2,2)*v(2,1) &
                     +xs(2,3)*v(3,1)-xs(1,1)*v(1,2)- &
                     4d0*xs(1,2)*v(2,2)-xs(1,3)*v(3,2)

!           --- Copy values of end point in new begin point

            u(1,:) = u(3,:)
            x1 = x3

         end do  ! i = 1, n-1

      case(5)

!     --- ichint = 5:  /  n . sigma . t  ds

         u(1,:) = usolbn(:,1)  ! u in first point

         do i = 1, n-1, 2   ! 3 points per interval

            x2(:)=coorbn(:,i+1)  ! x in mid point
            x3(:)=coorbn(:,i+2)  ! x in end point
            u(2,:)=usolbn(:,i+1) ! u in mid point
            u(3,:)=usolbn(:,i+2) ! u in end point

!           --- Compute factors for integration in array xs

            xs(:,1)=-1.5d0*x1(:)+2d0*x2(:) &
                         -0.5d0*x3(:)
            xs(:,2)=.5d0*(x3(:)-x1(:))
            xs(:,3)=0.5d0*x1(:)-2d0*x2(:) &
                         +1.5d0*x3(:)
            dx = x2-x1                    ! difference vector
            h = sqrt(dot_product(dx,dx))  ! length of interval

!           --- Compute normal vector

            do j = 1, 3
               h=1d0/sqrt(xs(1,j)**2+xs(2,j)**2)
               anorm(1,j) = xs(2,j)*h
               anorm(2,j) =-xs(1,j)*h
            end do

!           --- Store n. sigma in v

            v(1,1)=anorm(1,1)*u(1,1)+anorm(2,1)*u(1,2)
            v(1,2)=anorm(1,1)*u(1,2)+anorm(2,1)*u(1,3)
            v(2,1)=anorm(1,2)*u(2,1)+anorm(2,2)*u(2,2)
            v(2,2)=anorm(1,2)*u(2,2)+anorm(2,2)*u(2,3)
            v(3,1)=anorm(1,3)*u(3,1)+anorm(2,3)*u(3,2)
            v(3,2)=anorm(1,3)*u(3,2)+anorm(2,3)*u(3,3)

            do j = 1, ndim
               value(1)=value(1)+xs(j,1)*v(1,j)+4d0*xs(j,2)*v(2,j) &
                                +xs(j,3)*v(3,j)
            end do

!           --- Copy values of end point in new begin point

            u(1,:) = u(3,:)
            x1 = x3

         end do  ! i = 1, n-1

      case(6)

!     --- ichint = 6:  /  n . sigma  ds

         u(1,:) = usolbn(:,1)  ! u in first point

         do i = 1, n-1, 2   ! 3 points per interval

            x2(:)=coorbn(:,i+1)  ! x in mid point
            x3(:)=coorbn(:,i+2)  ! x in end point
            u(2,:)=usolbn(:,i+1) ! u in mid point
            u(3,:)=usolbn(:,i+2) ! u in end point

!           --- Compute factors for integration in array xs

            xs(:,1)=-1.5d0*x1(:)+2d0*x2(:) &
                         -0.5d0*x3(:)
            xs(:,2)=.5d0*(x3(:)-x1(:))
            xs(:,3)=0.5d0*x1(:)-2d0*x2(:) &
                         +1.5d0*x3(:)

!           --- Compute Jacobian ans store in array dx

            do j = 1, 3
               dx(j)=sqrt(dot_product(xs(:,j),xs(:,j)))
            end do

!           --- Compute normal vector

            do j = 1, 3
               h=1d0/sqrt(xs(1,j)**2+xs(2,j)**2)
               anorm(1,j) = xs(2,j)*h
               anorm(2,j) =-xs(1,j)*h
            end do

!           --- Store n. sigma .n in v

            v(1,1)=anorm(1,1)*u(1,1)+anorm(2,1)*u(1,2)
            v(1,2)=anorm(1,1)*u(1,2)+anorm(2,1)*u(1,3)
            v(2,1)=anorm(1,2)*u(2,1)+anorm(2,2)*u(2,2)
            v(2,2)=anorm(1,2)*u(2,2)+anorm(2,2)*u(2,3)
            v(3,1)=anorm(1,3)*u(3,1)+anorm(2,3)*u(3,2)
            v(3,2)=anorm(1,3)*u(3,2)+anorm(2,3)*u(3,3)

            value(1)=value(1)+v(1,1)*dx(1)+4d0*v(2,1)*dx(2)+ &
                              v(3,1)*dx(3)
            value(2)=value(2)+v(1,2)*dx(1)+4d0*v(2,2)*dx(2)+ &
                              v(3,2)*dx(3)

!           --- Copy values of end point in new begin point

            u(1,:) = u(3,:)
            x1 = x3

         end do  ! i = 1, n-1

      case(7)

!     --- ichint = 7:  / u n  ds

         u(1,1) = usolbn(1,1)  ! u in first point

         do i = 1, n-1, 2   ! 3 points per interval

            x2(:)=coorbn(:,i+1)  ! x in mid point
            x3(:)=coorbn(:,i+2)  ! x in end point
            u(2,:)=usolbn(:,i+1) ! u in mid point
            u(3,:)=usolbn(:,i+2) ! u in end point

!           --- Compute factors for integration in array xs

            xs(:,1)=-1.5d0*x1(:)+2d0*x2(:) &
                         -0.5d0*x3(:)
            xs(:,2)=.5d0*(x3(:)-x1(:))
            xs(:,3)=0.5d0*x1(:)-2d0*x2(:) &
                         +1.5d0*x3(:)

!           --- Compute Jacobian ans store in array dx

            do j = 1, 3
               dx(j)=sqrt(dot_product(xs(:,j),xs(:,j)))
            end do

!           --- Compute normal vector

            do j = 1, 3
               h=1d0/sqrt(xs(1,j)**2+xs(2,j)**2)
               anorm(1,j) = xs(2,j)*h
               anorm(2,j) =-xs(1,j)*h
            end do

            value(1)=value(1)+u(1,1)*anorm(1,1)*dx(1)+ &
                          4d0*u(2,1)*anorm(1,2)*dx(2)+ &
                              u(3,1)*anorm(1,3)*dx(3)
            value(2)=value(2)+u(1,1)*anorm(2,1)*dx(1)+ &
                          4d0*u(2,1)*anorm(2,2)*dx(2)+ &
                              u(3,1)*anorm(2,3)*dx(3)

!           --- Copy values of end point in new begin point

            u(1,1) = u(3,1)
            x1 = x3

         end do  ! i = 1, n-1

      end select  ! case ( ichint )

!     --- Mulitply by correct factor depending on irule

      if ( irule==3 ) then

!     --- Cartesian coordinates

         value = value/3d0

      else

!     --- Axisymmetric coordinates

         value = 2d0*pi*value/3d0

      end if  ! ( irule==1 )

1000  call erclos ( 'er00773' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'value', value(1)
         write(irefwr,*) 'End er00773'

      end if

      end
