      subroutine er0079 ( ichint, irule, numcrv, vector, number, &
                          ichfun, value, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 05-10-2011 Complete revision
!        version  4.0  date 16-06-2005 Extra parameetr lniwrk
!        version  3.0  date 14-10-2002 Extra parameters kmesh, kmeshq
!
!   copyright (c) 1991-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute a boundary integral over curves
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichint, irule, numcrv, number, ichfun, icurvs(*)
      double complex :: vector(*), value(*)

!     ichfun         i    choice parameter for the computation of f
!                         Possibible values:
!                         0         f = 1
!                         >0        f = func ( ichfun, x, y, z ) (real case)
!                                       or
!                                   f = cfunc ( ichfun, x, y, z ) (complex case)
!                                   at this moment only the combination f#1
!                                   and ichint=1 is permitted
!     ichint         i    indicates the kind of integral to be computed.
!                         possibilities:
!                          1  /  f u  ds
!                          2  /  f  u.n  ds
!                          3  /  f  u.t  ds
!                          4  /  f  n. sigma .n  ds
!                          5  /  f  n. sigma .t  ds
!                          6  /  f  n. sigma     ds
!                          7  /  f  u  n  ds
!                          8  sum f(i) u(i)
!                          9  /  f ds
!     icurvs         i    Integer array containing the curve numbers for the
!                         computation of the boundary integral. A negative curve
!                         number indicates that the curve must be computed in
!                         the reversed direction.
!     irule          i    type of numerical integration.  possibilities:
!                          1  trapezoid rule,  cartesian co-ordinates
!                          2  trapezoid rule,  axisymmetric co-ordinates
!                          3  simpson's rule   cartesian co-ordinates
!                          4  simpson's rule   axisymmetric co-ordinates
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered.
!     numcrv         i    Number of curves in icurvs
!     value          o    array containing the computed value of the integral
!     vector         i    vector to be integrated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurvscop(0:numcrv), istart, lnnodes, &
              numnodes, numb, j, inode, nparts, iparts, mvalue, &
              ipoint(10*numcrv), mcomp
      double complex :: valueloc(2)
      integer, allocatable :: inodes(:)
      double precision, allocatable :: coorbn(:,:)
      double complex, allocatable :: usolbn(:,:)

!     coorbn         work array to store the co-ordinates of the nodal
!                    points of the curves sequentially
!     icurvscop      Contains copy of array icurvs extended by position 0
!     inode          Node sequence number
!     inodes         work array to store the nodal points of the
!                    curves sequentially
!     iparts         Loop parameter
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     istart         First position used at a part
!     j              Counting variable
!     lnnodes        Length of array inodes
!     mcomp          number of components
!     mvalue         Number of components used in integral
!     nparts         number of parts the boundary consists of
!     numb           highest degree of freedom in a node that must be used
!                    If number = 0 then numb = 1 else numb = number
!     numnodes       Number of nodal points in array nodes
!     usolbn         work array to store the solution in the nodal points
!                    of the curves sequentially
!     valueloc       Local value of integrals (for one part)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0077         actual computation of boundary integrals over curves
!                    Complex case
!     ER0078         Fill array usolbn with the solution corresponding to the
!                    points at the boundary as stored in iwork, Complex case
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCURVENODESP  Store all nodes of one curve in array inodes and gaps in
!                    these curves in array ipoint
!     EROPEN         Produces concatenated name of local subroutine
!     ERRMCURNODS    delete points in inoces where degrees of freedom number
!                    is not available
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINCOMPL1     Print 2d complex vector
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0079' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0079'
         call prinin ( icurvs, numcrv, 'icurvs' )
         write(irefwr,1) 'ichint', ichint
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d15.7) )

      end if
      if ( ierror/=0 ) go to 1000

      lnnodes = kmeshm(3)-kmeshm(2)
      allocate ( inodes(lnnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, lnnodes, 'inodes' )
      if ( ierror/=0 ) go to 1000

!     --- initialize array value

      mvalue = 1
      if ( ichint>5 .and. ichint<8 ) mvalue = 2
      value(1:mvalue) = (0d0,0d0)

!     --- Copy icurvs in icurvscop extended with position 0

      icurvscop(0) = numcrv
      icurvscop(1:numcrv) = icurvs(1:numcrv)

!     --- Store nodes along curves in inodes

      call ercurvenodesp ( icurvscop(0), ipoint, inodes, nparts )

      numnodes = ipoint(nparts+1)-ipoint(1)
      if ( numnodes<2 ) go to 1000  ! too few points

!     --- Compute parameter numb indicating the highest degree of freedom
!         in a node that must be used

      numb = number
      if ( ichint==2 .or. ichint==3 ) numb = number+ndim-1
      if ( ichint>=4 .and. ichint<=6 ) numb = number+3*(ndim-1)-1
      mcomp = numb-number+1 ! number of unknwons per point in usolbn

!     --- delete nodal points without degree of freedom in inodes

      if ( indprfi>0 ) then

!     --- indprfi>0, hence variable number of unknowns per point

         call  errmcurnods ( inodes, numnodes, number, kprobfi, ipoint, &
                             nparts )
         if ( numnodes<2 ) go to 1000  ! too few points
         call inichkarlen ( nparts+1, 10*numcrv, 'ipoint' )

      end if  ! ( indprfi>0 )
      if ( ierror/=0 ) go to 1000

      if ( nparts ==1 .and. inodes(1)==inodes(numnodes) .and. &
           ichint==8 ) then

!     --- First and last point coincide

         numnodes = numnodes-1
         ipoint(2) = ipoint(2)-1

      end if  ! ( nparts ==1 .and. inodes(1)==inodes(numnodes) .and. .. )

      allocate ( coorbn(ndim,numnodes), usolbn(mcomp,numnodes), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, (ndim+mcomp)*numnodes, 'coorbn' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array coorbn with co-ordinates of the curves

      do j = 1, numnodes
         inode = inodes(j)
         coorbn(1:ndim,j) = coor(1:ndim,inode)
      end do  ! j = 1, numnodes
      if ( debug ) call prinrl1 ( coorbn, numnodes, ndim, 'coorbn' )

      if ( ichint/=9 ) then

!     --- Fill array usolbn with solution

         numb = numb-number+1
         call er0078 ( numb, number, inodes, numnodes, vector, usolbn )
         if ( debug ) &
            call princompl1 ( usolbn, numnodes, numb, 'usolbn' )

      end if  !( ichint/=9 )

!     --- compute boundary integrals per part

      do iparts = 1, nparts

         istart = ipoint(iparts)  ! starting position in part
         numnodes = ipoint(iparts+1)-istart ! number of nodes in part

         if ( debug ) &
            write(irefwr,1) 'iparts, istart, numnodes, mvalue', &
                             iparts, istart, numnodes, mvalue

         if ( numnodes>1 ) then

!        --- More than one node; compute integral

            valueloc(1:mvalue) = (0d0,0d0)
            call er0077 ( ichint, irule, ichfun, ndim, numnodes, &
                          usolbn(1,istart), coorbn(1,istart), &
                          valueloc, mcomp, mvalue )
            value(1:mvalue) = value(1:mvalue)+valueloc(1:mvalue)
            if ( debug ) &
              write(irefwr,2) 'value, valueloc', value(1), valueloc(1)

         end if  ! ( numnodes>1 )

      end do  ! iparts = 1, nparts

      deallocate ( inodes, usolbn, coorbn, stat = error )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'value', value(1)
         write(irefwr,*) 'End er0079'

      end if
      call erclos ( 'er0079' )

      end
