      subroutine ercurvenodesp ( icurvs, ipoint, inodes, nparts )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes of one curve in array inodes and gaps in
!     these curves in array ipoint
!     In case of parallel computation nodes may be absent and more gaps
!     may be present
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*), ipoint(*), inodes(*), nparts

!     icurvs         i    Integer array containing the curve or surface
!                         numbers that must be printed.
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one curve is used. The curve is stored in
!                              icurvs(2)
!                         -1   The  curves icrv1 to icrv2 are used.
!                              icrv1 is stored in icurvs(2) and icrv2
!                              in icurvs(3).
!                         -2   See -1, however, the sequence is reversed.
!                              This means that the curves are stored from
!                              the end point of icrv2 to the initial point
!                              of icrv2, then the end point of icrv2-1 to the
!                              begin point of icrv2-1 and so on.
!                         n>0  The curve numbers numbers are stored
!                              in icurvs(2) to icurvs(n+1).
!                              That sequence is used.
!                         Remark: a negative curve number means that the curve
!                              is printed from end point to initial point, hence
!                              in reversed order.
!     inodes         o    Array containing the nodal point numbers along curve.
!                         The nodes along the curves are ordered in
!                         the sequence indicated by icurnr.
!                         Starting addresses of all parts are stored
!                         in array ipoint.
!     ipoint         o    Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the curve consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     nparts         o    number of parts the curves are subdivided in
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubcv, istore(ncurvs), istar3, numpoints, k, icrv1, &
              jcrv1, ifirst, i1, i2, ilast, i, istart, numnodes
      integer, allocatable :: icurnods(:)
      logical gapfound

!     gapfound       If true a gap has been found
!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icrv1          First curve number
!     icurnods       Contains the nodes of a sub curve
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     istore         work array of length ncurvs to store all subcurves
!     jcrv1          abs(icrv1)
!     k              Counting variable
!     nsubcv         Number of subcurves of a curve
!     numnodes       Number of points along curve
!     numpoints      Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPSTORECURVS  Fill array istore with (sub) curve numbers of the curves
!                    stored in array icurvs using the parameter ichois
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercurvenodesp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercurvenodesp'
         if ( icurvs(1)<0 ) then
            k = 2
         else
            k = max(1,icurvs(1))
         end if
         call prinin ( icurvs(2), k, 'icurvs' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First store all (sub) curves in array istore

      call sepstorecurvs ( icurvs(1), nsubcv, istore, icurvs(2) )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nsubcv, ncurvs', nsubcv, ncurvs
         call prinin ( istore, nsubcv, 'istore' )

      end if

      numpoints = kmeshm(3)-kmeshm(2)
      allocate ( icurnods(numpoints), stat = error )
      if ( error/=0 ) call eralloc ( error, numpoints, 'icurnods' )

!     --- fill ipoint and inodes

      istar3 = kmeshm(2)-1  ! starting address in kmesh part m
      numpoints = 0         ! number of points found
      ilast = 0             ! last node found
      nparts = 0            ! number of parts
      gapfound = .false.  ! no gaps found

      do k = 1, nsubcv

!     --- Store nodes of subcurve in icurnods

         icrv1 = istore(k)   ! curve number with sign
         jcrv1 = abs(icrv1)  ! curve number without sign
         if ( debug ) &
            write(irefwr,1) 'k, icrv1, jcrv1', k, icrv1, jcrv1

         if ( icrv1>0 ) then

!        --- curve number positive

            i1 = kmeshm(4+jcrv1)    ! start of curve
            i2 = kmeshm(5+jcrv1)-1  ! end of curve
            numnodes = i2-i1+1      ! number of nodes along curve
            icurnods(1:numnodes) = kmeshm(istar3+i1:istar3+i2)

         else

!        --- curve number negative

            i1 = kmeshm(5+jcrv1)-1    ! start of curve
            i2 = kmeshm(4+jcrv1)      ! end of curve
            numnodes = i1-i2+1        ! number of nodes along curve
            do i = 1, numnodes
               icurnods(i) = kmeshm(istar3+i1-i+1)
            end do  ! i = 1, numnodes

         end if  ! ( icrv1>0 )

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'k, icrv1, i1, i2', &
                             k, icrv1, i1, i2
            call prinin ( icurnods, numnodes, 'icurnods' )
            write(irefwr,1) 'numpoints, nparts, numnodes', &
                             numpoints, nparts, numnodes

         end if  ! ( debug )

!        --- Find first non-zero node in icurnods

         istart = 0
         do i = 1, numnodes

            if ( icurnods(i)>0 ) then

!           --- Non-zero node found

               istart = i  ! position of first non-zero node
               exit

            end if  ! ( icurnods(i)>0 )

         end do  ! i = 1, numnodes

         if ( istart>0 ) then

!        --- istart>0, hence curve is not empty

            if ( .not. gapfound ) then

!           --- New part found

               nparts = nparts+1
               ipoint(nparts) = numpoints+1

            end if  ! ( .not. gapfound )

            ifirst = icurnods(istart)
            if ( ilast==ifirst ) then

!           --- First point is equal to prior last point, join parts

               istart = istart+1
               nparts = nparts-1

            end if  ! ( ilast==ifirst )

            do i = istart, numnodes

               if ( icurnods(i)>0 ) then

!              --- New node found, raise numpoints and store node

                  numpoints = numpoints+1
                  inodes(numpoints) = icurnods(i)
                  gapfound = .false.

               else if ( .not. gapfound ) then

!              --- New gap

                  nparts = nparts+1
                  ipoint(nparts) = numpoints+1
                  gapfound = .true.   ! gap found

               end if  ! ( icurnods(i)>0 )

            end do  ! i = istart, numnodes
            ilast = inodes(numpoints)  ! last node in curve

         end if  ! ( istart>0 )

      end do  ! k = 1, nsubcv

      if ( .not. gapfound ) then
         ipoint(nparts+1) = numpoints+1
      else
         nparts = nparts-1
      end if  ! ( .not. gapfound )

      if ( numpoints==0 ) nparts = 0
      deallocate ( icurnods, stat = error )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nparts', nparts
         call prinin ( ipoint, nparts+1, 'ipoint' )
         call prinin ( inodes, numpoints, 'inodes' )
         write(irefwr,*) 'End ercurvenodesp'

      end if  ! ( debug )
      call erclos ( 'ercurvenodesp' )

      end
