      subroutine erintobst ( iintbn, usol, iobsts, value, compl, lenkmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-01-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    computes the boundary integral over a set of subsequent obstacles
!    erintobst is a help subroutine for subroutine INTBOU
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintbn(*), iobsts(*), lenkmeshc
      double precision usol(*), value(*)
      logical compl

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     iintbn         i    Integer input array in which the user indicates what
!                         type of integral must be computed.
!                         IINTBN must be filled as follows:
!                         IINTBN(1) Type of calling subroutine
!                                   1:  BOUNIN
!                                   2:  BOUNIV
!                                   3:  INTBOU
!                                   4:  INTBND
!                         IINTBN(2) (ICHINT) indicates what type of integral has
!                                   to be computed. Possibible values:
!                                   1          /  f u  ds
!                                   2          /  f  u.n  ds
!                                   3          /  f  u.t  ds
!                                   4          /  f  n. sigma .n  ds
!                                   5          /  f  n. sigma .t  ds
!                                   6          /  f  n. sigma     ds
!                                   7          /  f  u  n  ds
!                                   8          sum f u
!                                   9          /  f ds
!                                   The possibilities 3 to 6 have only been
!                                   implemented for NDIM = 2, i.e.
!                                   two-dimensional regions
!                                   Default value: 1
!                         IINTBN(3) (ICHFUN) indication of the value of f.
!                                   Possibible values:
!                                   0         f = 1
!                                   >0        f = func(ichfun,x,y,z) (real case)
!                                             or
!                                             f = cfunc(ichfun,x,y,z) (complex
!                                                 case)
!                                             at this moment only the
!                                             combination f#1 and ichint=1
!                                             is permitted
!                                   Default value: 0
!                         IINTBN(4) (IRULE) The numerical integration rule to be
!                                   carried out depends on IRULE.
!                                   Possibible values:
!                                   For curves only (i.e. 2D):
!                                   1  trapezoid rule cartesian co-ordinates
!                                   2  trapezoid rule axisymmetric co-ordinates
!                                   3  simpson's rule cartesian co-ordinates
!                                   4  simpson's rule axisymmetric co-ordinates
!                                   For surfaces only (i.e. 3D):
!                                   1  Newton cotes rule, i.e. integration rule
!                                      based on the type of approximation used
!                                      in the element
!                                   Default value: 1
!                         IINTBN(5) (IDIM) This parameter indicates what type of
!                                   boundary integral must be computed
!                                   Possible values:
!                                   1: curves
!                                   2: surfaces
!                                   3: contact
!                                   4: no_contact
!                                   5: obstacles
!                         IINTBN(6) (NUMBER) This parameter indicates which
!                                   unknown (physical degree of freedom) must
!                                   be used from the (solution) vector u
!                                   (IVECTR)
!                                   If ICHINT = 2, 3 or 7, it is supposed that
!                                   the degrees of freedom are equal to NUMBER, &
!                                   NUMBER+1 and NUMBER+2 (3D)
!                                   If ICHINT = 4, 5 or 6, it is supposed that
!                                   the degrees of freedom are equal to NUMBER, &
!                                   NUMBER+1 and NUMBER+2 (2D) or NUMBER to
!                                   NUMBER+5 (3D)
!                                   Default value: 1
!                         IINTBN(7) (NCURVES) Indicates the number of curves, &
!                                   surfaces or obstacles in array ICURVS that
!                                   must be used for the integration
!                                   Default value: 1
!     iobsts         i    array containing the obstacle numbers for the
!                         computation of the boundary integral
!     lenkmeshc      i    length of array kmeshc
!     usol           i    Solution vector
!     value          o    array containing the computed value of the integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, inpelm, i, idim, ipstart, &
              k, numobstacs, ichint, ichfun, number, ikelmc, j, &
              iface, istart, numfaces, idum
      integer, allocatable :: kmeshclc(:)

!     i              Counting variable
!     ichfun         indication of the value of f. Possibilities:
!                    0   f = 1
!                    >0  f = func ( ichfun, x, y, z )
!                        at this moment only the combination f#1 and
!                        ichint=1 is permitted
!     ichint         indicates the kind of integral to be computed.
!                    possibilities:
!                     1  /  f u  ds
!                     2  /  f  u.n  ds
!                     3  /  f  u.t  ds
!                     4  /  f  n. sigma .n  ds
!                     5  /  f  n. sigma .t  ds
!                     6  /  f  n. sigma     ds
!                     7  /  f  u  n  ds
!                     8  sum f(i) u(i)
!     idim           This parameter indicates what type of boundary
!                    integral must be computed
!                    Possible values:
!                    1: curves
!                    2: surfaces
!                    3: contact
!                    4: no_contact
!     iface          Face sequence number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kmeshclc         work array to store nodal points
!                    corresponding to the elements
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     numfaces       number of faces
!     numobstacs     Number of obstacles in array iobsts, see iintbn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0071         Actual computation of boundary integral for one surface
!                    Real case
!     ER0072         Actual computation of boundary integral for one surface
!                    Complex case
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!     451   ichois has incorrect value
!     550   array length too short
!    1513   Too much surfaces in isurfs
!    2209   Obstacle number out of range
!    2407   Number of nodes in faces of obstacle nnot constant
! **********************************************************************
!
!                       PSEUDO CODE
!
!     check the parameters
!     For i := 1 (1) number_of_obstacles do
!        Fill surface of obstacle in temporary array kmeshclc
!        Add integral over surface
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erintobst' )

!     --- check the parameters

      ichint = iintbn(2)
      ichfun = iintbn(3)
      idim   = iintbn(5)
      number = iintbn(6)
      numobstacs = iintbn(7)
      if ( ichint/=1 .and. ichint/=9 ) then
         call errint ( ichint, 1 )
         call errint ( ichfun, 2 )
         call errint ( ichint, 3 )
         call errsub ( 451, 3, 0, 0 )
      end if
      if ( ichfun<0 .or. ichfun>1000 ) then
         call errint ( ichfun, 1 )
         call errint ( ichfun, 2 )
         call errint ( ichint, 3 )
         call errsub ( 451, 3, 0, 0 )
      end if
      if ( numobstacs>numobst ) then
         call errint ( numobstacs, 1 )
         call errint ( numobst, 2 )
         call errsub ( 1513, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      allocate ( kmeshclc(lenkmeshc), stat = error )
      if ( error/=0 ) call eralloc ( error, lenkmeshc, 'kmeshclc' )
      if ( ierror/=0 ) go to 1000

      value(1) = 0d0
      if ( compl ) value(2) = 0d0

!     --- Loop over obstacles

      do i = 1, numobstacs

         iobst = iobsts(i)
         if ( iobst<=0 .or. iobst>numobst ) then

!        --- obstacle number out of range

            call errint ( iobst, 1 )
            call errint ( numobst, 3 )
            call errsub ( 2209, 2, 0, 0 )
            go to 1000

         end if
         istart = obstfaces(iobst)

         if ( ndim/=3 ) then

!        --- Curves, not yet implemented

            call errint ( ndim, 1 )
            call errint ( 3, 2 )
            call errint ( 3, 3 )
            call errsub ( 419, 3, 0, 0 )

         else

!        --- ndim = 3, surfaces

            numfaces = obstfaces(istart)
            ikelmc = 0
            do k = 1, numfaces
               iface = obstfaces(istart+k)
               ipstart = nmeshg(iface)
               if ( k==1 ) then

!              --- First face, set inpelm

                  inpelm = nmeshg(ipstart)

               else

!              --- Next face, check inpelm

                  if ( inpelm/=nmeshg(ipstart) ) then

!                 --- inpelm not constant

                     call errint ( iobst, 1 )
                     call errint ( inpelm, 2 )
                     call errint ( k, 3 )
                     call errint ( iface, 4 )
                     call errint ( nmeshg(ipstart), 5 )
                     call errsub ( 2407, 5, 0, 0 )

                  end if

               end if

!              --- Store nodes in kmeshclc

               do j = 1, inpelm
                  kmeshclc(ikelmc+j) = nmeshg(ipstart+j)
               end do
               ikelmc = ikelmc + inpelm
               if ( ikelmc>lenkmeshc ) then

!              --- lenkmeshc too small

                  call errint ( ikelmc, 1 )
                  call errint ( lenkmeshc, 2 )
                  call errchr ( 'kmeshclc', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  go to 1000

               end if

            end do

         end if
         if ( compl ) then

!        --- Complex case

            call er0072 ( kmeshclc, inpelm, numfaces, usol, ichint, ichfun, &
                          number, value, iobst, idim, idum )
         else

!        --- Real case

            call er0071 ( kmeshclc, inpelm, numfaces, usol, ichint, ichfun, &
                          number, value, iobst, idim, idum )

         end if

      end do

      deallocate ( kmeshclc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshclc' )

1000  call erclos ( 'erintobst' )
      end
