      subroutine intcopun ( usol1, usol2, nunkp, minnode, &
                            maxnode, npointnw, minmax, nunks )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.4  date 14-10-2011 Extra debug
!        version  3.3  date 18-06-2004 Minor change
!        version  3.2  date 11-04-2003 Debug statements
!
!   copyright (c) 1997-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Interpolate usol1 into usol2 using the information stored in the arrays
!   imap and rmap
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointnw, nunkp, minnode, maxnode, &
              minmax, nunks
      double precision usol1(*), usol2(*)

!     maxnode    i    Maximum nodal point number in part
!     minmax     i    If 1 the interpolated solution is restricted to the
!                     range uold_iun(min)<=unew_iun<=uold_iun(max)
!                     for each physical unknown iun, if 0 no restriction
!     minnode    i    Minimum nodal point number in part
!     npointnw   i    Number of nodes in new mesh
!     nunkp      i    Maximum number of unknowns in a point
!     nunks      i    Number of unknowns per point in usol2
!     usol1      i    Array to be interpolated
!     usol2      o    Result of interpolation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iunkp, nodp, k
      double precision sum, u, minval, maxval

!     iunkp          Sequence number of unknown
!     k              Counting variable
!     maxval         Maximum value of unknown
!     minval         Minimum value of unknown
!     nodp           Nodal point sequence number
!     sum            Help parameter to compute a sum
!     u              Help parameter to store the value of the solution
!                    in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For iunkp := 1 (1) nunks do
!       For nodp := minnode (1) maxnode do
!          sum := 0d0
!          For k := 1 (1) imap(nodp,1) do
!             sum := sum + rmap(nodp,k)*usol1((imap(nodp,k+1)-1)*nunkp+iunkp)
!          usol2((nodp-1)*nunks+iunkp) := sum
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intcopun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intcopun'
         call ertrace
         write(irefwr,1) 'nunkp, minmax, npointnw', &
                          nunkp, minmax, npointnw
         call prinin2 ( imap, min(5,npointnw), 3, 'imap' )
  1      format ( a, 1x, (10i6) )

      end if

      if ( nunkp==1 ) then

!     --- nunkp = 1

         if ( minmax==1 ) then

!        --- minmax = 1, restrict values

            minval = rinfin
            maxval = -rinfin
            do nodp = minnode, maxnode
               do k = 1, imap(nodp,1)
                  u = usol1(imap(nodp,k+1))
                  minval = min(u,minval)
                  maxval = max(u,maxval)
               end do
            end do

            do nodp = minnode, maxnode
               sum = 0d0
               do k = 1, imap(nodp,1)
                  sum = sum + rmap(nodp,k)*usol1(imap(nodp,k+1))
               end do
               usol2(nodp) = max(minval,min(sum,maxval))
            end do

         else

!        --- No need to restrict

            do nodp = minnode, maxnode

               sum = 0d0
               do k = 1, imap(nodp,1)
                  sum = sum + rmap(nodp,k)*usol1(imap(nodp,k+1))
               end do
               usol2(nodp) = sum

            end do

         end if

      else

!     --- nunkp>1

         if ( minmax==1 ) then

!        --- minmax = 1, restrict values

            do iunkp = 1, nunks

               minval = rinfin
               maxval = -rinfin
               do nodp = minnode, maxnode
                  do k = 1, imap(nodp,1)
                     u = usol1((imap(nodp,k+1)-1)*nunkp+iunkp)
                     minval = min(u,minval)
                     maxval = max(u,maxval)
                  end do
               end do

               do nodp = minnode, maxnode

                  sum = 0d0
                  do k = 1, imap(nodp,1)
                     sum = sum + rmap(nodp,k) * &
                           usol1((imap(nodp,k+1)-1)*nunkp+iunkp)
                  end do
                  usol2((nodp-1)*nunks+iunkp) = &
                     max(minval,min(sum,maxval))

               end do

            end do

         else

!        --- No need to restrict

            do iunkp = 1, nunks
               do nodp = minnode, maxnode
                  sum = 0d0
                  do k = 1, imap(nodp,1)
                     sum = sum + rmap(nodp,k) * &
                           usol1((imap(nodp,k+1)-1)*nunkp+iunkp)
                  end do
                  usol2((nodp-1)*nunks+iunkp) = sum

               end do

            end do

         end if

      end if

      if ( debug ) write(irefwr,*) 'End intcopun'
      call erclos ( 'intcopun' )

      end
