      subroutine intmap2dc ( isol2, nunkploc, maplc, imesh, &
                             itrans, plane, trans, iplane )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 04-12-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map solution defined in 3d mesh onto 2d intersection
!     The solution in the 3d mesh has a constant number of degrees
!     of freedom per point
!     The mapping of nodes in the intersection is already computed
!     complex version
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekprob
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol2, nunkploc, imesh, &
              itrans, iplane
      double precision plane(4), trans(3,3)
      type ( mapping) :: maplc

!     imesh          i    Sequence number of 2d mesh
!     iplane         i    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     isol2         o    Standard SEPRAN array containing information of the
!                         2d velocity vector stored as solution vector
!     itrans         i    indicates if the quantity to be interpolated
!                         must be transformed or not.
!                         Possible values:
!                         0   No special transformation
!                         1   The first two unknowns are projected in the
!                             plane in the direction of the tangential
!                             vectors, the third one perpendicular to the
!                             plane
!     maplc          o    Mapping array that is filled
!     nunkploc       i    Number of unknowns in a point
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     trans          i    In this array the Transformation matrix from x,y,z
!                         co-ordinates to ksi-eta co-ordinates is stored as
!                         well as the origin in trans(.,3)
!                         The transformation of x,y,z to ksi,eta is defined by
!                         ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                         eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nnodes

!     nnodes         Number of nodes in 2d intersection
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTMAP2D1C     Map solution defined in 3d mesh onto 2d intersection
!                    Complex version
!     PRININ         print 1d integer array
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPCRPROBINFO  create a new array in kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays
!     Create output array
!     Compute starting addresses
!     Map solution
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap2dc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap2dc'
         write(irefwr,1) 'itrans, nunkploc, iplane', itrans, nunkploc, iplane
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprob => kprob2d(imesh)

!     --- Create output array

      nnodes = size ( maplc%rmap )

      call sepcrprobinfo ( kprob, 1 )
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc
      allocate ( kp(iseqkp)%kprobsub(iprob)%kprobc(1) )
      kprobc => kp(iseqkp)%kprobsub(iprob)%kprobc

      kprobloc(4) = nunkploc
      kprobloc(5) = nunkploc*nnodes
      kprobloc(7) = 1
      kprobloc(8) = 1
      kprobloc(31) = 2
      kprobloc(33) = nunkploc
      kprobc(1) = 1

      call sepcreasol110 ( isol2, 'usol2d', 1 )
      usol => usoltemp

!     --- Compute starting addresses

      imap => maplc%imap
      rmap => maplc%rmap

!     --- Map solution

      call intmap2d1c ( nnodes, nunkploc, usol, &
                        usolt, itrans, plane, trans, iplane )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( isol2, 5, 'isol2' )
         write(irefwr,*) 'End intmap2dc'

      end if  ! ( debug )
      call erclos ( 'intmap2dc' )

      end subroutine intmap2dc
