      subroutine intmap3d04 ( kmeshc, inelem, nunkploc, &
                              number, usol, usolcop, ielgrp, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The solution to be copied is not present in all nodes
!     Center nodes of elements and faces are skipped
!     Elements of shape 4 or 7
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     2d
!     triangle
!     quadratic
!     vertex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm,*), inelem, nunkploc, number, &
              ielgrp
      double precision usol(*), usolcop(nunkploc,*)

!     ielgrp         i    element group number
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    Number of nodes in elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         kmeshc is only related to this element group
!     number         i    Thefirst unknown to be copied has sequence number
!                         number
!     nunkploc       i    Number of unknowns in a point that must be copied
!     usol           i    Solution vector
!     usolcop       i/o   Solution vector to which the solution must be
!                         copied. usolcop has nunkploc unknowns per node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ielem, node, node2, node3, imidside(3,3), k1, k2, &
              k3, ih, ivertex(3)
      logical present

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant
!     imidside       Array of size 6 * 3 containing:
!                    (i,1):  i^th mid side point
!                    (i,2) and (i,3) neighbor vertices of the midside point
!     ivertex        Contains the relative positions of the vertices per element
!     j              Counting variable
!     k1             Relative node number of midside point
!     k2             Relative node number of first neighbor
!     k3             Relative node number of second neighbor
!     node           Node number
!     node2          Node number of first neighbor
!     node3          Node number of second neighbor
!     present        If true the unknowns are present in the midside points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     583   The required degree of freedom is not available in the vertices
!           of the element
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivertex / 1, 3, 5 /
! ======================================================================
!
      call eropen ( 'intmap3d04' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap3d04'
         write(irefwr,1) 'nunkploc, number', nunkploc, number
         call prinin ( ivertex, 3, 'ivertex' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if unknown is present in all vertices

      do i = 1, 3

         node = kmeshc(ivertex(i),1)

         do j = 1, nunkploc

            if ( kprobpi(node,j+number-1)==0 ) then

!           --- Unknown not present in vertex

               call errint ( j, 1 )
               call errint ( nunkploc+number-1, 2 )
               call errint ( ielgrp, 3 )
               call errwar ( 583, 3, 0, 0 )
               go to 1000

            end if  ! ( kprobpi(node,j+number-1)==0 ) then

         end do  ! j = 1, nunkploc

      end do  ! i = 1, 3

!     --- Check if unknown is present in the midside points
!         First fill array imidside

      do i = 1, 3
         imidside(i,1) = 2*i
         imidside(i,2) = 2*i-1
         imidside(i,3) = 2*i+1
      end do  ! i = 1, 3
      imidside(3,3) = 1

      present = .true.
      node = kmeshc(2,1)

      do j = 1, nunkploc

         if ( kprobpi(node,j+number-1)==0 ) present = .false.

      end do  ! j = 1, nunkploc

      if ( debug ) write(irefwr,*) 'present ', present

      if ( present ) then

!     --- Copy solution in vertices and midside points

         do ielem = 1, inelem

            do i = 1, 6

               node = kmeshc(i,ielem)

               do j = 1, nunkploc
                  usolcop(j,node) = usol(kprobpi(node,number-1+j))
               end do  ! j = 1, nunkploc

            end do  ! i = 1, 6

         end do  ! ielem = 1, inelem

      else

!     --- Copy solution in vertices

         do ielem = 1, inelem

            do i = 1, 3

               node = kmeshc(ivertex(i),ielem)

               do j = 1, nunkploc
                  usolcop(j,node) = usol(kprobpi(node,number-1+j))
               end do  ! j = 1, nunkploc

            end do  ! i = 1, 3

         end do  ! ielem = 1, inelem

!        --- Interpolate solution in midside points

         do i = 1, 3

            k1 = imidside(i,1)
            k2 = imidside(i,2)
            k3 = imidside(i,3)
            do ielem = 1, inelem

               node = kmeshc(k1,ielem)
               node2 = kmeshc(k2,ielem)
               node3 = kmeshc(k3,ielem)

               do j = 1, nunkploc
                  ih = kprobpi(node,number-1+j)
                  if ( ih>0 ) then
                     usolcop(j,node) = usol(ih)
                  else
                     usolcop(j,node) = 0.5d0* &
                        (usolcop(j,node2)+usolcop(j,node3))
                  end if  ! ( kprobpi(node,number-1+j)>0 )
               end do  ! j = 1, nunkploc

            end do  ! ielem = 1, inelem

         end do  ! i = 1, 3

      end if ! ( present )

1000  call erclos ( 'intmap3d04' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmap3d04'

      end if  ! ( debug )

      end
