      subroutine intmshm2 ( kmesh1, kmesh2, map, iinput, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 30-11-2012 Long integers
!        version  4.1  date 13-09-2006 Correction nodnew/old
!        version  4.0  date 18-04-2005 Extra parameters iinput, icurvs
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a formal interpolation from one mesh to another.
!     In fact the mapping between the meshes is computed,
!     without knowing the function (2D version)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmeshv
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1, kmesh2, iinput(*), icurvs(*)
      type (mapping) :: map

!     icurvs         i    Integer array containing the user points, curve or
!                         surface numbers that must be printed.
!                         Pos. 1 indicates how array icurvs is filled.
!                         Possible values:
!                          0   only one curve is used. The curve number is
!                              stored in icurvs(2)
!                         -1   The solution is printed along the curves
!                              icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                              icrv2 in icurvs(3).
!                         n>0  The curve numbers numbers are stored in
!                              icurvs(2) to icurvs(n+1). That sequence is used.
!     iinput         i    Integer input array with teh following contents
!                         1:  ichoice, Choice parameter
!                             Possible values:
!                             0,1: All points are interpolated; Points outside
!                                  the mesh are extrapolated
!                             2:   Only points inside the old mesh or very close
!                                  to the old mesh are interpolated, the other
!                                  points are left unchanged
!                         2:  idim, defines if all surfaces are used (0), or
!                             only a restricted set (1)
!                             If idim = 1, array icurvs is used
!     kmesh1         i    Standard SEPRAN array containing information of mesh 1
!     kmesh2         i    Standard SEPRAN array containing information of mesh 2
!     map           i/o   contains information of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical check
      integer nsplit, minnodol, minnodnw, iseqkm1, iseqkm2, &
              maxnodol, maxnodnw, ipart, nx, ny, lenkmtri, nodnew, &
              nodold, i, ncurvs, ichoice, idim, npoint
      double precision xmin, xmax, ymin, ymax, ds, dsnw, eps, xminnw, &
                       xmaxnw, yminnw, ymaxnw, dx, dy, dxnw, dynw
      integer, allocatable :: ijcrol(:), ijcrnw(:), npntij(:), pntcij(:)
      integer, pointer :: kmstri(:)

!     check          Indicates if the file exist (true) or not
!     ds             Minimum distance between two points along a curve
!     dsnw           Minimum distance between two points along a curve
!                    (new mesh)
!     dx             Distance in the cells in x-direction
!     dxnw           Distance in the cells in x-direction (new mesh)
!     dy             Distance in the cells in y-direction
!     dynw           Distance in the cells in x-direction (new mesh)
!     eps            Local accuracy
!     i              Counting variable
!     ichoice        Integer choice parameter, see iinput(1)
!     idim           See iinput(2)
!     ijcrnw         ij cell number coornw
!     ijcrol         ij cell number coorol
!     ipart          Sequence number of part
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmstri         kmeshc subdivided
!     lenkmtri       Length of array kmeshtri
!                    If 0, no subdivision is required
!     maxnodnw       Largest node number in part of new mesh
!     maxnodol       Largest node number in part of old mesh
!     minnodnw       Smallest node number in part of new mesh
!     minnodol       Smallest node number in part of old mesh
!     ncurvs         Number of curves in mesh
!     nodnew         Number of nodes in part of new mesh
!     nodold         Number of nodes in part of old mesh
!     npntij         Contains number of coordinates in ij-cell accumulated
!                    npntij(1) = 0
!                    npntij((i-1)*nx+j+1) - npntij((i-1)*nx+j) is number
!                    of points in cell ij
!     npoint         Number of nodal points
!     nsplit         Number of parts into which the mesh has been splitted
!     nx             number of points in x direction
!     ny             number of points in y direction
!     pntcij         Node numbers corresponding to cells
!     xmax           Maximum x-value
!     xmaxnw         Maximum x-value (new mesh)
!     xmin           Minimum x-value
!     xminnw         Minimum x-value (new mesh)
!     ymax           Maximum y-value
!     ymaxnw         Maximum y-value (new mesh)
!     ymin           Minimum y-value
!     yminnw         Minimum y-value (new mesh)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR2021         Compute xmin, xmax, ymin, ymax from array coor
!     PR2022         Compute mapping for all new points that are present in
!                    this part of the mesh
!     PR2023         Fill array ijcoor with the ij-cell number of the
!                    co-ordinates
!     PR2024         compute number of nodes for each ij combination and
!                    store in array npntij
!     PR2025         compute number of nodes for each ij combination and
!                    store in array npntij
!     PR2026         Compute length of array kmeshtri
!     PR2026RES      Compute length of array kmeshtri, which is used to
!                    subdivide an existing 2D mesh into three-point triangles
!                    Special case where we restrict ourselves to some surfaces
!     PR2027         Subdivide a mesh consisting of triangles and/or
!                    quadrilaterals into three-point triangles
!     PR2027RES      Subdivide a mesh consisting of triangles and/or
!                    quadrilaterals into three-point triangles
!                    Special case where we restrict ourselves to some surfaces
!     PR2028         Find all points in imap that have not been treated yet and
!                    interpolate those points
!     PR2029         Compute xmin, xmax, ymin, ymax from array coor
!     PRININ1        print 2d integer array
!     PRINMAP        Print the contents of the mapping array map
!     SEPCREATEIBUF  Create permanent space in array IBUFFR
!     SEPCREATERBUF  Create permanent space in array BUFFER
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPRINKMESHC  print kmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     429   The number of disjoint parts in both meshes are unequal
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Create arrays imap, rmap
!   Clear arrays imap, rmap
!   Detect the number of sub parts the mesh consists of
!   Check if the number of sub parts of both meshes are identical
!   For each sub part ipart do
!      Compute xmin, xmax, ymin, ymax and ds from kmeshm_old
!      Correct xmin, xmax, ymin, ymax and ds from kmeshm_new
!      Compute nx and ny
!      Define new rectangular grid by
!           x_i = xmin+(i-1)*ds   (i=1(1)nx+1)
!           y_j = ymin+(j-1)*ds   (j=1(1)ny+1)
!           cell_ij = (x_i, x_i+1) * (y_j, y_j+1)
!      For all nodes in coornw compute cell number i,j
!      For all cells i,j detect all nodes in coornw
!      For all nodes in coorol compute cell number i,j
!      Create subelement mesh consisting of triangles only
!      For all elements in subelement mesh
!          Compute mapping
!   For all points that have not been treated
!     Find points in old mesh that are the most close to new points
!     Interpolate
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmshm2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmshm2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepfillkmesh ( kmesh1 )

!     --- extract information from iinput

      ichoice = iinput(1)
      idim = iinput(2)


!     --- Create arrays imap, rmap

      iseqkm1 = iseqkm
      iseqkm2 = kmesh2

      if ( debug ) write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2

      npoint = km(iseqkm2)%kmesh(8)
      if ( debug ) &
         write(irefwr,1) 'npoint, ichoice, idim', &
                          npoint, ichoice, idim
      if ( associated(map%imap) ) deallocate ( map%imap, map%rmap )
      call sepclear ( map )
      allocate ( map%imap(npoint,5), map%rmap(npoint,3) )
      imap => map%imap
      rmap => map%rmap
      imap2 => imap
      rmap2 => rmap

!     --- Clear arrays imap, rmap

      imap = 0
      rmap = 0d0
      map%iseqkm1 = iseqkm1
      map%iseqkm2 = iseqkm2

!     --- Detect the number of sub parts the mesh consists of

      nsplit = max(1,km(iseqkm1)%kmesh(44))
      if ( max(1,km(iseqkm2)%kmesh(44))/=nsplit ) then

!     --- Mesh1 and mesh2 do not consist of the same number of sub parts

         call errint ( nsplit, 1 )
         call errint ( max(1,km(iseqkm2)%kmesh(44)), 2 )
         call errsub ( 429, 2, 0, 0 )

      end if

!     --- For each sub part ipart do

      minnodol = 1
      minnodnw = 1
      maxnodol = km(iseqkm1)%kmesh(8)
      maxnodnw = km(iseqkm2)%kmesh(8)
      if ( debug ) &
         write(irefwr,*) 'nsplit, maxnodol, maxnodnw ', &
                          nsplit, maxnodol, maxnodnw
      do ipart = 1, nsplit
         if ( nsplit>1 ) then
            maxnodol = km(iseqkm1)%kmeshk(km(iseqkm1)%kmesh(7)+ipart)
            maxnodnw = km(iseqkm2)%kmeshk(km(iseqkm2)%kmesh(7)+ipart)
         end if

!        --- Compute xmin, xmax, ymin, ymax and ds from kmeshm_old

         ncurvs = km(iseqkm1)%kmesh(11)

         call pr2021 ( km(iseqkm1)%coor, xmin, xmax, ymin, ymax, ds, &
                       minnodol, maxnodol, km(iseqkm1)%kmeshm, dx, dy, &
                       ncurvs )
         if ( debug ) &
            write(irefwr,*) 'xmin, xmax, ymin, ymax ', &
                             xmin, xmax, ymin, ymax

!        --- Correct xmin, xmax, ymin, ymax and ds from kmeshm_new

         ncurvs = km(iseqkm2)%kmesh(11)
         if ( km(iseqkm2)%kmesh(27)/=0 ) then

!        --- Standard case

            call pr2021 ( km(iseqkm2)%coor, xminnw, xmaxnw, yminnw, &
                          ymaxnw, dsnw, minnodnw, maxnodnw, &
                          km(iseqkm2)%kmeshm, dxnw, dynw, ncurvs )

         else

!        --- intmshm2 is called by intcoor

            call pr2029 ( km(iseqkm2)%coor, xminnw, xmaxnw, yminnw, &
                          ymaxnw, dsnw, minnodnw, maxnodnw )
            dxnw = dx
            dynw = dy

         end if
         if ( debug ) &
            write(irefwr,*) 'xminnw, xmaxnw, yminnw, ymaxnw ', &
                             xminnw, xmaxnw, yminnw, ymaxnw
         xmin = min(xmin,xminnw)
         ymin = min(ymin,yminnw)
         xmax = max(xmax,xmaxnw)
         ymax = max(ymax,ymaxnw)
         xmin = xmin - 0.1d-5*(xmax-xmin)
         ymin = ymin - 0.1d-5*(ymax-ymin)
         xmax = xmax + 0.1d-5*(xmax-xmin)
         ymax = ymax + 0.1d-5*(ymax-ymin)
         ds = min ( ds, dsnw )
         dx = min ( dx, dxnw )
         dy = min ( dy, dynw )
         eps = 1d-5*ds
         nx = min(int((xmax-xmin)/dx)+1,100)
         ny = min(int((ymax-ymin)/dy)+1,100)
         if ( idim==0 ) then

!        --- idim = 0, use all surfaces

            call pr2026 ( lenkmtri )

         else

!        --- idim = 1, use some surfaces

            call pr2026res ( lenkmtri, icurvs )

         end if  ! ( idim==0 )

!        --- Define new rectangular grid by
!              x_i = xmin+(i-1)*ds   (i=1(1)nx+1)
!              y_j = ymin+(j-1)*ds   (j=1(1)ny+1)
!              cell_ij = (x_i, x_i+1) * (y_j, y_j+1)

         nodnew = maxnodnw+1
         nodold = maxnodol+1

         allocate ( ijcrol(2*nodold), ijcrnw(2*nodnew), &
                    npntij(nx*ny+1), pntcij(max(nodold,nodnew)), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, 2*nodold+2*nodnew, 'ijcrol' )
         if ( ierror/=0 ) go to 1000

         if ( lenkmtri==0 ) then

!        --- lenkmtri = 0, use old kmeshc

            kmstri => km(iseqkm1)%kmeshc

         else

!        --- lenkmtri > 0, new array

            allocate ( kmstri(lenkmtri), stat = error )
            if ( error/=0 ) call eralloc ( error, lenkmtri, 'kmstri' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( lenkmtri==0 )

         if ( debug ) call sepprinkmeshc

!        --- For all nodes in coornw compute cell number i,j

         call pr2023 ( km(iseqkm2)%coor, xmin, ymin, dx, dy, &
                       ijcrnw, minnodnw, maxnodnw, nx, ny )

!        --- For all cells i,j detect all nodes in coornw

         call pr2024 ( ijcrnw, minnodnw, maxnodnw, &
                       npntij, nx, ny )
         call pr2025 ( ijcrnw, minnodnw, maxnodnw, &
                       npntij, nx, ny, pntcij )

!        --- For all nodes in coorol compute cell number i,j

         call pr2023 ( km(iseqkm1)%coor, xmin, ymin, dx, dy, &
                       ijcrol, minnodol, maxnodol, nx, ny )
         if ( debug ) write(irefwr,*) 'lenkmtri ', lenkmtri

         if ( idim==0 .and. lenkmtri>0 ) then

!        --- lenkmtri>0, Create subelement mesh consisting of triangles only

            call pr2027 ( kmstri, minnodol, maxnodol, inelem )

         else if ( lenkmtri>0 ) then

!        --- lenkmtri>0, Create subelement mesh consisting of triangles only
!            also idim#0, hence restrict to surfaces stored in icurvs

            call pr2027res ( kmstri, minnodol, maxnodol, icurvs, inelem )

         else

            inelem = km(iseqkm1)%kmesh(9)

         end if
         if ( debug ) then
            write(irefwr,1) 'inelem', inelem
            call prinin1 ( kmstri, inelem, 3, 'kmeshtri' )
         end if  ! ( debug )

!        --- Compute mapping for all elements in subelement mesh

         call pr2022 ( kmstri, km(iseqkm1)%coor, km(iseqkm2)%coor, &
                       inelem, ijcrol, npntij, &
                       pntcij, nx, ny, npoint, eps )

!        --- Check if there are any points in new mesh that have not been
!            treated yet

         check = .false.
         if ( ichoice<=1 ) then

!        --- ichoice = 1, all points must get a value

            do i = minnodnw, maxnodnw
               if ( imap(i,1)==0 ) then
                  check = .true.
                  go to 200
               end if
            end do

         end if
         if ( debug ) write(irefwr,*) 'check, ichoice ', check, ichoice

200      if ( check ) then

!        --- For all cells i,j detect all nodes in coornw

            call pr2024 ( ijcrol, minnodol, maxnodol, &
                          npntij, nx, ny )
            call pr2025 ( ijcrol, minnodol, maxnodol, &
                          npntij, nx, ny, pntcij )

!           --- For all points that have not been treated
!               Find points in old mesh that are the most close to new points
!               Interpolate

            call pr2028 ( npoint, &
                          ijcrnw, npntij, pntcij, minnodnw, maxnodnw, &
                          km(iseqkm1)%coor, km(iseqkm2)%coor, nx, ny, &
                          km(iseqkm1)%kmesh(8) )
         end if
         minnodol = maxnodol+1
         minnodnw = maxnodnw+1

         deallocate ( ijcrol, ijcrnw, npntij, pntcij, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ijcrol' )

         if ( lenkmtri>0 ) then
            deallocate ( kmstri, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'ijcrol' )
         end if  ! ( lenkmtri>0 )

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinmap ( map, kmesh2 )
         write(irefwr,*) 'End intmshm2'

      end if
      call erclos ( 'intmshm2' )

      end subroutine intmshm2
