      subroutine pr2022 ( kmeshtri, coorol, coornw, nelem, &
                          ijcrol, npntij, pntclij, nx, ny, &
                          npointnw, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 14-02-2012 Use prgeomtriangle and printerpolnod2
!        version  1.4  date 20-04-2005 Debug statements
!        version  1.3  date 02-08-2004 Debug statements
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute mapping for all new points that are present in this part of
!     the mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kmeshtri(3,nelem), nx, ny, ijcrol(2,*), npointnw, &
              npntij(*), pntclij(*)
      double precision coorol(2,*), coornw(2,*), eps

!     coornw     i    Array containing the co-ordinates of mesh 2
!     coorol     i    Array containing the co-ordinates of mesh 1
!     eps        i    margin parameter
!     ijcrol     i    Contains the ij cell number of the co-ordinates of mesh2
!                     ijcrol(1,k) contains the i number of coorol_k
!                     ijcrol(2,k) contains the j number of coorol_k
!     kmeshtri   i    Kmesh part c for triangular mesh
!     nelem      i    Number of elements
!     npntij     i    Contains number of coordinates in ij-cell accumulated
!                     npntij(1) = 0
!                     npntij((i-1)*nx+j+1) - npntij((i-1)*nx+j) is number
!                     of points in cell ij
!     npointnw   i    Number of nodes in new mesh
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     pntclij    i    Contains the node numbers per cell
!                     The sequence is defined by npntij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xmin, xmax, ymin, ymax, &
                       xh(3), yh(3), dphidx(3), dphidy(3), x, y
      integer i, j, k, ielem, inode, imin, imax, jmin, jmax, ih, ij, &
              nnodes
      logical check, geom

!     check          Indicates if trher are new nodes in the triangle
!     dphidx         Contains the x-derivatives of the basis functions
!     dphidy         Contains the y-derivatives of the basis functions
!     geom           If true the geometrical quantities of the element must be
!                    compute
!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     ij             help parameter
!     imax           Maximum i-value of cells
!     imin           Minimum i-value of cells
!     inode          Node sequence number
!     j              Counting variable
!     jmax           Maximum j-value of cells
!     jmin           Minimum j-value of cells
!     k              Counting variable
!     nnodes         Number of nodes in isolnw
!     x              x co-ordinate of point
!     xh             x-coordinates of vertices
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     yh             y-coordinates of vertices
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRGEOMTRIANGLE Compute geometrical quantities of triangle
!     PRININ2        Print an integer two-dimensional array
!     PRINRL2        Print a real two-dimensional array
!     PRINTERPOLNOD2 Check if node is in element, if so interpolate (2d, trian)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all triangles in sub mesh do
!        Compute minimum and maximum cell sequence numbers
!        If ( there are new nodes in this triangle ) then
!           Compute xmin, xmax, ymin, ymax of triangle
!           geom := false
!           For all nodes inode in the cells corresponding to the triangle do
!             if ( imap(inode,1)=0 ) then
!                If ( node in rectangle (xmin,xmax) x (ymin,ymax) ) then
!                   if ( not geom ) then
!                      Compute geometrical quantities
!                      geom := true
!                   compute baricentric co-ordinates of inode
!                   If ( all baricentric co-ordinates in [0,1] ) then
!                      Compute interpolation
!                      Fill imap and rmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2022' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2022'
         write(irefwr,100) 'nelem', nelem
100      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For all triangles in sub mesh do

      do ielem = 1, nelem

!     --- Compute minimum and maximum cell sequence numbers

         imin = nx+1
         imax = 1
         jmin = ny+1
         jmax = 1
         do i = 1, 3
            inode = kmeshtri(i,ielem)
            imin = min(imin,ijcrol(1,inode))
            imax = max(imax,ijcrol(1,inode))
            jmin = min(jmin,ijcrol(2,inode))
            jmax = max(jmax,ijcrol(2,inode))
         end do

!        --- Check if there are new nodes in this triangle

         check = .false.
         do i = imin, imax
            ih = (i-1)*ny
            do j = jmin, jmax
               ij = ih+j
               if ( npntij(ij+1)-npntij(ij)>0 ) then

!              --- There is at least one node in the range

                  check = .true.
                  go to 150
               end if
            end do
         end do
150      if ( check ) then

!        --- Compute xmin, xmax, ymin, ymax of triangle

            xmin = minval(coorol(1,kmeshtri(1:3,ielem)))
            ymin = minval(coorol(2,kmeshtri(1:3,ielem)))
            xmax = maxval(coorol(1,kmeshtri(1:3,ielem)))
            ymax = maxval(coorol(2,kmeshtri(1:3,ielem)))
            geom = .false.
            xmin = xmin-eps
            xmax = xmax+eps
            ymin = ymin-eps
            ymax = ymax+eps

!           --- For all nodes inode in the cells corresponding to the triangle
!               do

            do i = imin, imax
               ih = (i-1)*ny
               do j = jmin, jmax
                  ij = ih+j
                  nnodes = npntij(ij+1)-npntij(ij)
                  do k = 1, nnodes
                     inode = pntclij(npntij(ij)+k)
                     if ( imap(inode,1)==0 ) then

!                    --- Node has not yet been treated

                         x = coornw(1,inode)
                         y = coornw(2,inode)
                         if ( x>=xmin .and. x<=xmax .and. &
                              y>=ymin .and. y<=ymax ) then

!                        --- node in rectangle (xmin,xmax) x (ymin,ymax)

                           if ( .not. geom ) then

!                          --- Compute geometrical quantities

                              call prgeomtriangle ( coorol, xh, yh, &
                                                    kmeshtri(1,ielem), &
                                                    dphidx, dphidy )
                              geom = .true.

                           end if  ! ( .not. geom )

!                          --- compute baricentric co-ordinates of inode
!                              if node in triangle interpolate

                           call printerpolnod2 ( inode, coornw, dphidx, &
                                                 dphidy, npointnw, xh, yh, &
                                                 ielem, kmeshtri(1,ielem) )

                        end if ! x>=xmin .and. x<=xmax .and. ... )

                     end if  ! ( imap(inode,1)==0 )

                  end do  ! k = 1, nnodes

               end do

            end do

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin2 ( imap, npointnw, 5, 'imap' )
         call prinrl2 ( rmap, npointnw, 3, 'rmap' )
         write(irefwr,*) 'End pr2022'

      end if
      call erclos ( 'pr2022' )

      end
