      subroutine pr2031 ( xmin, xmax, ymin, ymax, zmin, zmax, ds, &
                          minnode, maxnode, dx, dy, dz )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 05-04-2004 Jump in case of double nodes
!        version  2.2  date 06-03-2004 New computation of xmin, xmax, ...
!        version  2.1  date 23-04-2003 Debug statements
!
!   copyright (c) 1998-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute xmin, xmax, ymin, ymax, zmin, zmax from array coor
!    Compute ds from coor and kmeshm
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xmin, xmax, ymin, ymax, zmin, zmax, &
                       ds, dx, dy, dz
      integer minnode, maxnode

!     ds             o    Minimum distance between two points along a curve
!     dx             o    Distance in the cells in x-direction
!     dy             o    Distance in the cells in y-direction
!     dz             o    Distance in the cells in z-direction
!     maxnode        i    Maximum nodal point number in part
!     minnode        i    Minimum nodal point number in part
!     xmax           o    Maximum x-value
!     xmin           o    Minimum x-value
!     ymax           o    Maximum y-value
!     ymin           o    Minimum y-value
!     zmax           o    Maximum z-value
!     zmin           o    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, np, i1, i2, icurnr, ipos, nsubcv
      double precision x1, y1, z1, x2, y2, z2

!     i              Counting variable
!     i1             Point at curve
!     i2             Next point at curve
!     icurnr         Curve sequence number
!     ipos           position in kmeshm
!     istart         First position used at a part
!     np             number of nodes along curve
!     nsubcv         Number of subcurves of a curve
!     x1             x-coordinate of point i1
!     x2             x-coordinate of point i2
!     y1             y-coordinate of point i1
!     y2             y-coordinate of point i2
!     z1             z-coordinate of point i1
!     z2             z-coordinate of point i2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINKMESHM  Print the contents of array kmeshm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   initialize xmin, xmax, ymin, ymax, zmin, zmax, dx
!   For all surfaces do
!      For all nodes along the surface do
!         Compute xmin, xmax, ymin, ymax, zmin, zmax
!         Compute ds
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2031' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2031'
         call sepprinkmeshm

      end if

!     --- initialize xmin, xmax, ymin, ymax, zmin, zmax, dx

      xmin = coor(1,minnode)
      xmax = xmin
      ymin = coor(2,minnode)
      ymax = ymin
      zmin = coor(3,minnode)
      zmax = zmin

!     --- Compute xmin, ymin, .. zmin

      do i1 = minnode+1, maxnode

         x1 = coor(1,i1)
         y1 = coor(2,i1)
         z1 = coor(3,i1)

         xmin = min(xmin,x1)
         xmax = max(xmax,x1)
         ymin = min(ymin,y1)
         ymax = max(ymax,y1)
         zmin = min(zmin,z1)
         zmax = max(zmax,z1)

      end do  ! i1 = minnode, maxnode

      ds = rinfin
      dx = rinfin
      dy = rinfin
      dz = rinfin
      istart = kmeshm(2)
      ncurvs = istart-6
      nsubcv = 0

!     --- For all curves do

      do icurnr = 1, ncurvs
         if ( kelmq/=0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         if ( debug ) write(irefwr,*) 'icurnr, nsubcv ', icurnr, nsubcv
         if ( nsubcv==0 ) then

!        --- nsubcv = 0, hence curve is a single curve

            np = kmeshm(5+icurnr)-kmeshm(4+icurnr)
            ipos = istart+kmeshm(4+icurnr)-1
            i1 = kmeshm(ipos)
            if ( i1<minnode .or. i1>maxnode ) go to 100
            x1 = coor(1,i1)
            y1 = coor(2,i1)
            z1 = coor(3,i1)

!           --- For all nodes along the curve do

            do i = 1, np-1

!           --- Compute ds

               i2 = kmeshm(ipos+i)
               if ( i1==i2 .or. i2<minnode .or. i2>maxnode ) &
                  go to 100
               x2 = coor(1,i2)
               y2 = coor(2,i2)
               z2 = coor(3,i2)
               ds = min(ds,sqrt((x2-x1)**2+(y2-y1)**2+(z2-z1)**2))
               if ( ds>0 ) then

!              --- Length of part > 0

                  if ( abs(x2-x1)>0.5d0*ds ) dx = min(dx,abs(x2-x1))
                  if ( abs(y2-y1)>0.5d0*ds ) dy = min(dy,abs(y2-y1))
                  if ( abs(z2-z1)>0.5d0*ds ) dz = min(dz,abs(z2-z1))
                  x1 = x2
                  y1 = y2
                  z1 = z2
                  i1 = i2

               end if

            end do

         end if

100   end do
      if ( dx>xmax-xmin ) dx = ds
      if ( dy>ymax-ymin ) dy = ds
      if ( dz>zmax-zmin ) dz = ds

      if ( debug ) then
         write(irefwr,*) 'dx, dy, dz ', dx, dy, dz
         write(irefwr,*) 'End pr2031'
      end if
      call erclos ( 'pr2031' )

      end
