      subroutine prcpcoef1 ( npoint, presnode, cpcoef, ipost, rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.0  date 15-06-1999
!
!   copyright (c) 1998-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Computes pressure-coefficient in all vertices
!
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          npoint, ipost(*)
      double precision presnode(npoint), rpost(*), cpcoef(npoint)

!     cpcoef     o    Array containing the pressure coefficient in all nodes
!     ipost      i    Integer array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
!     npoint     i    Number of nodal points
!     presnode   i    Pressure at the nodes of the mesh
!     rpost      i    Real array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer          i, idimchoice, mcom
      double precision const, spgamma, machref, momlinf, enthinf, &
                       presinf, densinf, dynpres

!     const      Constant, required in the computation of the
!                pressure coefficient
!     densinf    Density at infinity
!     dynpres    Dynamic pressure: 0.5*density*velocity*velocity
!     enthinf    Enthalpy at infinity
!     i          Counting variable
!     idimchoice Dimensionless (0) or dimensionful computation (1)
!     machref    Reference Mach number
!     mcom       If 0 the flow is incompressible; if 1 then the flow is
!                compressible and the pressure-correction method is
!                used;
!                if 2 then we are dealing with multi-gas flow, and
!                if 3 then the flow is compressible and the fully
!                compressible method is used.
!     momlinf    Length of momentum vector at infinity
!     presinf    Pressure at infinity
!     spgamma    Specific heat ratio gamma: for ideal gas gamma = 1.4
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The pressure coefficient is computed from:
!
!     c_p = const*(p - p_inf)
!
!     where const=1/dyn.pres for dimensionful computations, and
!     const=1/(gamma*machref^2*dyn.pres) for dimensionless computations.
!     Since one usually plots -c_p (minus c_p), we put a minus-sign in
!     front of c_p.
!
!     The dynamic pressure is equal to 0.5*momentum_inf^2/rho_inf.
!     rho_inf follows from the equation of state, and is equal  to
!     p_inf/h_inf in the dimensionless case, and is equal to
!     gamma/(gamma-1)*p_inf/h_inf in the dimensionful case.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      idimchoice = ipost(6)
      mcom       = ipost(7)

!     --- Some relevant constants from rpost

      spgamma = rpost(1)
      presinf = rpost(3)
      enthinf = rpost(4)
      momlinf = rpost(5)

!     --- Dimensionless quantities

      if ( idimchoice.eq.0 ) then

         machref = rpost(2)
         densinf = presinf/enthinf
         dynpres = 0.5d0*(momlinf*momlinf/densinf)
         const   = 1.0d0/(spgamma*machref*machref*dynpres)

!        --- Dimensionful quantities

      else

         densinf = (spgamma/(spgamma-1.0d0))*(presinf/enthinf)
         dynpres = 0.5d0*(momlinf*momlinf/densinf)
         const   = 1.0d0/dynpres

      end if

      if ( mcom.eq.1 ) then

!     --- Mach-uniform pressure-correction formulation

         densinf = (1.0d0 + spgamma*machref*machref*presinf)/enthinf
         dynpres = 0.5d0*(momlinf*momlinf/densinf)
         const = 1.0d0/dynpres
         if ( abs(presinf).gt.1d-5 ) then
            write(*,*) ' Dimensionless pressure at outlet not equal '
            write(*,*) ' to zero. This is not allowed in fvpslift01.f '
            stop
         end if
      end if

!     --- Compute pressure coefficient in all nodes

      do i = 1, npoint
         cpcoef(i) = -const*( presnode(i) - presinf )
      end do

      end
