      subroutine pringrid ( kmeshlc, kproblc, isol, rinput, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 03-01-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate the solution stored in array isol to a rectangular grid
!     with bounds (xmin,xmax,ymin,ymax,zmin,zmax) and step size (dx,dy,dz)
!     Print the interpolated values
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     interpolation
!     regular
!     grid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshlc, kproblc, isol(5)
      double precision rinput(3)
      character *(*) text

!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kmeshlc        i    Standard SEPRAN array containing information of the
!                         mesh
!     kproblc        i    Standard SEPRAN array, containing information of
!                         the problem
!     rinput         i    Real input array containing:
!                         1: step size dx
!                         2: step size dy
!                         3: step size dz
!     text           i    String variable containing the text for the heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nstepx1, nstepy1, nstepz1, ncoor
      double precision dx, dy, dz
      double precision, allocatable :: coorloc(:), solint(:)

!     coorloc        Array with the coordinates of the local mesh
!     dx             step size in x-direction
!     dy             step size in y-direction
!     dz             step size in z-direction
!     ncoor          Number of points in rectangular grid
!     nstepx1        Number of steps in x-direction
!     nstepy1        Number of steps in y-direction
!     nstepz1        Number of steps in z-direction
!     solint         Interpolated solution
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFILLGRID1    Fill coordinates in rectangular grid (1D)
!     PRFILLGRID2    Fill coordinates in rectangular grid (2D)
!     PRFILLGRID3    Fill coordinates in rectangular grid (3D)
!     PRINGRID01     print solution in rectangular grid
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPINTCOOR     Interpolate the solution as stored in array isol
!                    from nodal points in array kmesh onto points with
!                    coordinates stored in coor
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2228   (dx,dy,dz) not all positive
!    2229   (nx,ny,nz) too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make initializations
!     Check parameters
!     Create temporary space at the end of buffer to store coor and solint
!     Fill array coor
!     Interpolate solution to solint
!     Print solint
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pringrid' )

      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)
      kprob2d(1) = kproblc
      kprob => kprob2d(1)

!     --- Make initializations

      call sepactsolbf2 ( isol )

!     --- Check parameters

      dx = rinput(1)
      dy = dx
      dz = dx
      if ( ndim>1 ) dy = rinput(2)
      if ( ndim>2 ) dz = rinput(3)
      if ( dx<=0d0 .or. dy<=0d0 .or. dz<=0d0 ) then

!     --- dx,dy,dz not all positive

         call erreal ( dx, 1 )
         call erreal ( dy, 2 )
         call erreal ( dz, 3 )
         call errsub ( 2228, 0, 3, 0 )
         go to 1000

      end if
      nstepx1 = max(nint((xmax-xmin)/dx),0)+1
      nstepy1 = max(nint((ymax-ymin)/dy),0)+1
      nstepz1 = max(nint((zmax-zmin)/dz),0)+1
      if ( nstepx1>100 .or. nstepy1>100 .or. nstepz1>100 ) then

!     --- nxtepx, nstep, nstepz1 out of range

         call errint ( nstepx1, 1 )
         call errint ( nstepy1, 2 )
         call errint ( nstepz1, 3 )
         call errsub ( 2229, 3, 0, 0 )
         go to 1000

      end if

!     --- Create temporary space at the end of buffer to store coor and solint

      ncoor = nstepx1*nstepy1*nstepz1

      allocate ( coorloc(ncoor*ndim), solint(nphysi*ncoor), stat = error )
      if ( error/=0 ) call eralloc ( error, ncoor*(ndim+nphysi), 'coorloc' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array coor

      if ( ndim==1 ) then

!     --- ndim = 1

         call prfillgrid1 ( coorloc, xmin, dx, nstepx1 )

      else if ( ndim==2 ) then

!     --- ndim = 1

         call prfillgrid2 ( coorloc, xmin, ymin, dx, dy, nstepx1, &
                            nstepy1 )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call prfillgrid3 ( coorloc, xmin, ymin, zmin, dx, dy, &
                            dz, nstepx1, nstepy1, nstepz1 )

      end if

!     --- Interpolate solution to solint

      iinmap(1) = 0
      call sepintcoor ( isol, solint, &
                        coorloc, nphysi, ncoor, iinmap, map )

!     --- Print solint

      call pringrid01 ( coorloc, solint, ndim, ncoor, nphysi, text )

!     --- Release end of buffer

      deallocate ( coorloc, solint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorloc' )

1000  call erclos ( 'pringrid' )
      end subroutine pringrid
