      subroutine printerpol3d ( kmesh2d, kprob2d, &
                                isol2d, isol3d, map, kmesh, numveclc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 07-07-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate all solutions to 2d mesh, defined by intersection
!     of 3d mesh by plane
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulevecs
      use sepmodulemap
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh2d, kprob2d(*), isol2d(*), &
              isol3d(*), kmesh, numveclc
      type (mapping) :: map

!     isol2d         o    Standard SEPRAN array containing information of the
!                         2d solution vectors
!     isol3d         i    Standard SEPRAN array containing information of the
!                         3d solution vectors
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmesh2d        o    Standard SEPRAN array containing information of the
!                         2d mesh
!     kprob2d        o    Standard SEPRAN array containing information of the
!                         2d problem description
!     map            i    Contains mapping information
!     numveclc       o    Number of vectors stored in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, npoint, nunkp, nnodes, iseqkm1, iseqkm2, itypvc, ivec

!     i              Counting variable
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     nnodes         Number of nodes
!     npoint         Number of nodal points
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFILLISOL2D   Map 3d solution onto 2d mesh and fill
!                    corresponding array isol2d
!     PRFILLKPROB2D  Fill kprob2d for a 3d mesh intersected by a plane
!     PRININ1        print 2d integer array
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1653   Number of unknowns not constant in node
!    2831   This number of unknowns per point not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create array kprob2d
!     For all solutions in isol3d do
!        map solution 2d 2d mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printerpol3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printerpol3d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      iseqkm1 = kmesh
      iseqkm2 = kmesh2d
      npoint = km(iseqkm1)%kmesh(8)
      nnodes = km(iseqkm2)%kmesh(8)
      if ( debug ) write(irefwr,1) 'npoint, nnodes', npoint, nnodes

!     --- Create array kprob2d and iinputkprob

      call prfillkprob2d ( kprob2d, kmesh2d )

!     --- Activate arrays


!     --- For all solutions in isol3d do
!            map solution on 2d mesh

      do i = 1, 100

         if ( isol3d(i)==0 ) go to 1000
         numveclc = i

!        --- Detect type of array

         nunkp = ks(isol3d(i))%nusol/npoint

         if ( debug ) &
            write(irefwr,1) 'i, nunkp, ks(isol3d(i))%nusol ', &
                             i, nunkp, ks(isol3d(i))%nusol

         if ( nunkp*npoint/=ks(isol3d(i))%nusol ) then

!        --- number of unknowns per point not constant

            call errsub ( 1653, 0, 0, 0 )
            go to 1000

         end if  ! ( nunkp*npoint/=ks(isol3d(i))%nusol )

         itypvc = 115   ! vector of special structure

         if ( nunkp==1 ) then

!        --- nunkp = 1, solution vector

            itypvc = 110   ! solution vector
            ivec = 0     ! solution vector

         else if ( nunkp==3 ) then

!        --- nunkp = 3, vector of special structure 1

            ivec = 1     ! vector of special structure 1

         else if ( nunkp==4 ) then

!        --- nunkp = 4, vector of special structure 2

            ivec = 2     ! vector of special structure 2

         else if ( nunkp==6 ) then

!        --- nunkp = 6, vector of special structure 3

            ivec = 3     ! vector of special structure 3

         else

!        --- nunkp has other value, not implemented

            call errint ( nunkp, 1 )
            call errsub ( 2831, 1, 0, 0 )
            go to 1000

         end if  ! ( nunkp==1 )

!        --- Create vector

         isol2d(i) = 0
         call sepcreasolbf1 ( isol2d(i), itypvc, ivec, 'isol2d', 0 )
         usolt => usoltemp

         usol => ks(isol3d(i))%sol
         if ( ierror/=0 ) go to 1000

         call prfillisol2d ( usol, usolt, map%imap, map%rmap, nunkp, &
                             nnodes )

      end do  ! i = 1, 100

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( isol2d, numveclc, 5, 'isol2d' )
         write(irefwr,*) 'End printerpol3d'

      end if  ! ( debug )
      call erclos ( 'printerpol3d' )

      end subroutine printerpol3d
