      function prlengthline ( inpsep, rinsep, values )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of line with origin (o_x,o_y), angle and
!     end point(e_x,e_y)
! **********************************************************************
!
!                       KEYWORDS
!
!     length
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision :: prlengthline
      double precision, intent(in) :: rinsep(*), values(3)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Only positions 8-10 are used in this subroutine
!                         See subroutine readscintersin for a description
!     prlengthline   o    Computed length
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscintersin for a description
!     values         i    Double precision array with the following contents
!                         pos. 1:2  coordinates of origin of line
!                         pos. 3    angle in degrees with respect to x-axis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lenline, ex, ey, sinangle, cosangle, eps, s1, &
                          s2

!     cosangle       cosine of angle
!     eps            Local accuracy
!     ex             x-coordinate of end point
!     ey             y-coordinate of end point
!     lenline        length of line
!     s1             factor to compute length (ex-ox)/cos(angle)
!     s2             factor to compute length (ey-oy)/sin(angle)
!     sinangle       sine of angle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     GETPARM        Return with real defined by inpsep/rinsep
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3184   End point not on line
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlengthline' )
      debug = .true. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlengthline'
         call prinin ( inpsep(8), 3, 'inpsep' )
         call prinrl ( values, 3, 'values' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpsep(10)/=0 ) then

!     --- lenline is given

         lenline = getparm ( inpsep, rinsep, 10, 'length' )

      else

!     --- Length is not given

         ex = values(1)
         ey = values(2)
         if ( inpsep(8)/=0 ) ex = getparm ( inpsep, rinsep, 8, 'E_x' )
         if ( inpsep(9)/=0 ) ey = getparm ( inpsep, rinsep, 9, 'E_y' )

         sinangle = sin(values(3)/180d0*pi)
         cosangle = cos(values(3)/180d0*pi)
         eps = 1d-6

         if ( debug ) &
            write(irefwr,2) 'ex, ey, sinangle, cosangle', &
                             ex, ey, sinangle, cosangle

         if ( abs(sinangle)>eps ) then

!        --- sin(angle)>eps

            s2 = (ey-values(2))/sinangle

            if ( abs(cosangle)>eps ) then

!           --- cos(angle)>eps

               s1 = (ex-values(1))/cosangle

               if ( debug ) write(irefwr,2) 's1, s2', s1, s2

               if ( abs(s1-s2)<eps ) then

!              --- s1 = s2, hence lenline

                  lenline = s1

               else

!              --- end point not on line

                  call erreal ( ex, 1 )
                  call erreal ( ey, 2 )
                  call erreal ( values(1), 3 )
                  call erreal ( values(2), 4 )
                  call erreal ( values(3), 5 )
                  call errsub ( 3184, 0, 5, 0 )
                  lenline = 0d0

               end if  ! ( abs(s1-s2)<eps ) then

            else

!           --- cos(angle)<eps

               lenline = s2

            end if  ! ( abs(cosangle)>eps )

         else

!        --- sin(angle)<eps

            lenline = (ex-values(1))/cosangle

         end if  ! ( abs(sinangle)>eps )

      end if  ! ( inpsep(10)/=0 )

      prlengthline = lenline

1000  call erclos ( 'prlengthline' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlengthline'

      end if  ! ( debug )

      end
