      subroutine prtoavsext0 ( isol, usollc, maxvec1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-04-2014 New call to prtoavsext1
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 13-04-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy sepran solutions to usollc organized per unknown
!     The first parameter refers to the nodes, the second one to the
!     unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     output
!     paraview
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), maxvec1
      double precision :: usollc(*)

!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     maxvec1        i    Maximum number of vectors stored in isol
!     usollc              temporary solution array organized per node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqlc, isoltmp, i, itypvc, &
                 interp, inunkp, lenname, ivec, nphysorig

!     i              Counting variable
!     interp         Dummy parameter in intmap3d
!     inunkp         Number of degrees of freedom in point
!     iseqlc         Sequence number of component in output vector
!     isoltmp        Help array to store the solution temporarily
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     lenname        Length of name
!     nphysorig      Number of physical degrees of freedom before reduction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INISTREXT      Find the structure of the vector with exactly
!                    NUMBER unknowns per point
!     PRTOAVSEXT1    Copy solution array with nphysi degrees of freedom in each
!                    node into temporary solution array stored in buffer from
!                    position ipusol
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPINTMAP3D    Map solution in 3d region onto regular 3d solution defined
!                    in all nodes of the mesh (except centroids)
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtoavsext0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtoavsext0'
         write(irefwr,1) 'maxvec1', maxvec1
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqlc = 1

!     --- For all vectors 1 to maxvec1 do

      do i = 1, maxvec1

         if ( ks(isol(i))%typevec==110 .or. ks(isol(i))%typevec==115 ) then

!        --- Vector has been filled and is of type 110 or 115
!            Determine type of vector, inunkp and number of vectors

            call sepactsolbf2 ( isol(i) )
            lenname = index(names(i),' ')-1
            if ( lenname<0 ) lenname = len(names(i))

            if ( debug ) write(irefwr,1) 'indprfi, nphysi', indprfi, nphysi

            if ( indprfi==0 ) then

!           --- Constant number of degrees of freedom per point

               call prtoavsext1 ( isol(i), usollc, &
                                  npoint, names(i)(1:lenname), &
                                  iseqlc, nphysi, nphysi )

            else

!           --- Non-constant number of degrees of freedom per point
!               First copy array into array usolt with nphysi degrees of
!               freedom per point

               nphysorig = nphysi
               if ( nphysi==7 ) nphysi = 3  ! In case nphysi=7, only the
                                            ! velocity is copied
               interp = 0
               call sepintmap3d ( isol(i), 1, nphysi, interp )
               call inistrext ( nphysi, itypvc, ivec )
               if ( ierror/=0 ) go to 1000

               call sepcreasolbf1 ( isoltmp, itypvc, ivec, 'isoltmp', 0 )
               usoltemp = usolt

!              --- Copy to buffer

               call prtoavsext1 ( isoltmp, usollc, &
                                  npoint, names(i)(1:lenname), &
                                  iseqlc, nphysi, nphysorig )

!              --- Delete temporary array

               call seprmsolinfo ( isoltmp )
               deallocate ( usolt )

            end if  ! ( indprfi==0 )

         end if  ! ( isol(i)>0 )

      end do  ! i = 1, maxvec1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtoavsext0'

      end if  ! ( debug )
      call erclos ( 'prtoavsext0' )

      end subroutine prtoavsext0
