      subroutine sep2paraview ( isol, itimelc, iseq, maxvec1, nameparaview )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 09-04-2014 New call to prtoavsdim
!        version  1.3  date 09-10-2013 Minor correction nchar
!        version  1.2  date 20-06-2013 New definition file ref numbers
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution arrays and mesh to a file in avs format
!     such that it can be read by paraview
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     vector
!     write
!     paraview
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemach
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5,*), itimelc, iseq, maxvec1, nameparaview

!     iseq           i    Sequence number for array to be written
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     itimelc        i    Integer time sequence number
!     maxvec1        i    Maximum number of vectors per time-step
!     nameparaview   i    Sequence number of name of paraview file in array
!                         file_names
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref, nchar, npointsav, &
              numveclc, nunkp, numveccomps(100), i, kmeshtmp
      character (len=100) :: filename, filenm
      logical new_mesh
      double precision, allocatable :: usol(:)

!     filename       Name of file
!     filenm         Name of file
!     i              Counting variable
!     iref           File reference number
!     kmeshtmp       Contains information of the old mesh
!     nchar          Number of characters in a word
!     new_mesh       If true a new mesh is created
!     npointsav      Help parameter to save the old value of npoint
!     numveccomps    Array to store the number of degrees of freedom per
!                    point for each vector to be written
!     numveclc         Number of vectors to be written
!     nunkp          Maximum number of unknowns to be written
!     usol           temporary solution array organized per node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRAVSMESH      Write mesh information to file iref in AVS format
!     PRININ         print 1d integer array
!     PRTOAVS1       Write solution stored in columns per degree of freedom
!                    to AVS input file
!     PRTOAVSDIM     Define number of components and number of vectors for
!                    paraview output
!     PRTOAVSEXT0    Create array of length npoint x nunkp
!                    Copy sepran solutions to this array organized per unknown
!     PRTOAVSHEAD    Write heading for paraview output
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     SEPRMMESHINFO  Remove mesh information
!     TOTRANSMESH    Transform quadratic mesh to linear mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sep2paraview' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sep2paraview'
         write(irefwr,1) 'iref74', iref74
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=3 ) then

!     --- ndim# 3, not implemented

         call errint ( ndim, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errwar ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim/=3 )

!     --- Open file sepparaview_xxx.inp

      iref = inisetnewref()
      if ( debug ) &
         write(irefwr,1) 'ndim, iref, nameparaview', &
                          ndim, iref, nameparaview
      if ( nameparaview==0 ) then

!     --- nameparaview = 0, make file name default

         filename = 'sepparaview_'
         nchar = 12

      else

!     --- nameparaview>0, use file name from cfilenames

         filename = file_names(nameparaview)
         nchar = index ( filename, ' ' )
         filename(nchar:nchar) = '_'

      end if
      write ( filenm, 100 ) filename(1:nchar), itimelc+iseq
100   format ( a,i3.3,'.','inp' )
      open ( unit=iref, file=filenm )
      if ( debug ) write(irefwr,*) 'file = ', filenm

!     --- Transform mesh into linear mesh

      call totransmesh ( kmeshtmp, new_mesh )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'new_mesh ', new_mesh

!     --- The number of unknowns depends on what is to be written
!         Adapt nunkp and numveclc
!         Fill numveccomps

      call prtoavsdim ( isol, nunkp, numveclc, numveccomps, &
                        maxvec1, ndim )

      if ( debug ) then
         write(irefwr,1) 'nunkp, numveclc, maxvec1', nunkp, numveclc, maxvec1
         call prinin ( numveccomps, numveclc, 'numveccomps' )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- write mesh and dimension parameters

      call pravsmesh ( iref, nunkp )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'after pravsmesh'

!     --- write information about solution
!         This must be followed by for each vector
!         name of vector, dimension

      call prtoavshead ( isol, iref, numveclc, &
                         numveccomps, maxvec1 )
      if ( debug ) write(irefwr,*) 'after prtoavshead'

!     --- write solution
!         The solution must be written in the form:
!         node number, components
!         So it is necessary to create an array with nunkp components per point
!         This array is put at the back of ibuffr temporarily

      if ( new_mesh ) then

!     --- New mesh is used

         length = nunkp*npoint
         if ( debug ) write(irefwr,1) 'npoint, length', npoint, length

         allocate ( usol(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'usol' )
         if ( ierror/=0 ) go to 1000

         call prtoavsext0 ( isol, usol, maxvec1 )

      else

!     --- Old mesh is used

         length = nunkp*npoint
         if ( debug ) write(irefwr,1) 'npoint, length', npoint, length

         allocate ( usol(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'usol' )
         if ( ierror/=0 ) go to 1000

         call prtoavsext0 ( isol, usol, maxvec1 )

      end if  ! ( new_mesh )
      if ( debug ) write(irefwr,*) 'after prtoavsext0'

!     --- write array

      call prtoavs1 ( usol, npoint, nunkp, iref )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'after prtoavs1'

!     --- Close file

      close ( unit = iref )
      call inirmref ( iref )

      if ( new_mesh ) then

!     --- Remove new mesh and velocity

         call seprmmeshinfo ( kmesh )

!        --- Copy old mesh from kmeshtmp into kmesh

         kmesh = kmeshtmp
         call seprelkmesh(kmesh)

      end if  ! ( new_mesh )

      deallocate ( usol, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'usol' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sep2paraview'

      end if  ! ( debug )
      call erclos ( 'sep2paraview' )

      end subroutine sep2paraview
