      subroutine sep2parelgrp ( isol, maxvec1, iref, inunkp, &
                                numveccomps, numveclc, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution arrays and mesh to a file in avs format
!     such that it can be read by paraview
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     vector
!     write
!     paraview
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iref, numveclc, &
              maxvec1, inunkp, numveccomps(*), ielgrp

!     ielgrp         i    element group number
!     inunkp         i    Maximum number of unknowns in a point
!     iref           i    File reference number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     maxvec1        i    Maximum number of vectors per time-step
!     numveccomps    i    Array to store the number of degrees of freedom per
!                         point for each vector to be written
!     numveclc       i    Number of vectors stored in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i, nnodes, iseqlc, lenname
      integer, allocatable :: kmeshcloc(:,:), map(:)
      double precision, allocatable :: coorloc(:,:), usolloc(:,:)
      logical :: filled

!     coorloc        Array with the coordinates of the local mesh
!                    corresponding to the element group
!     i              Counting variable
!     inpelm         number of points per element
!     iseqlc           Sequence number in array usolloc
!     kmeshcloc      Local array kmeshc containing the node numbers of one
!                    element group
!     lenname        Length of name
!     map            Mapping array of length npoint to make an
!                    inverse mapping of array nodes
!     nnodes         Number of nodes
!     usolloc        Contains all solution vectors for one element group only
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIMAPCOOR     Map coordinates of global mesh into local mesh using an
!                    array with global node numbers corresponding to the
!                    local mesh
!     INIMAPSOL      Map usol of global mesh into local mesh using an array with
!                  global node numbers corresponding to the local mesh
!     ININODEELGRP   Fill all nodes of element group ielgrp in array nodes
!     MSHCREATELGRP  Create mesh corresponding to one element group
!     PRAVS3         Write mesh information to file iref in AVS format
!     PRTOAVS1       Write solution stored in columns per degree of freedom
!                    to AVS input file
!     PRTOAVSHEAD    Write heading for paraview output
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1653   Number of unknowns not constant in node
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sep2parelgrp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sep2parelgrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create mesh consisting of 1 element group

      inpelm = Topgrp(ielgrp)%inpelm
      inelem = Topgrp(ielgrp)%inelem

      allocate ( kmeshcloc(inpelm,inelem), nodes(npoint), map(npoint), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, inelem*inpelm+2*npoint, 'kmeshcloc' )
      if ( ierror/=0 ) go to 1000

!     --- First find all nodes in element group

      nnodes = npoint
      call ininodeelgrp ( ielgrp, nodes, nnodes, map )
      if ( ierror/=0 ) go to 1000

!     --- Next fill array kmeshc with local numbers

      call mshcreatelgrp ( map, ielgrp, kmeshcloc )

      allocate ( coorloc(3,nnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*nnodes, 'coorloc' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array coorloc with local coordinates

      call inimapcoor ( coor, coorloc, nodes, 3, nnodes )

!     --- write mesh and dimension parameters

      write(iref,200) nnodes, inelem, inunkp
200   format ( 3i8, ' 0 0' )

      call pravs3 ( iref, coorloc, 3, nnodes, inelem, inpelm, &
                    kmeshcloc )
      if ( debug ) write(irefwr,*) 'after pravs3'
      if ( ierror/=0 ) go to 1000

      deallocate ( coorloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorloc' )
      if ( ierror/=0 ) go to 1000

!     --- write information about solution
!         This must be followed by for each vector
!         name of vector, dimension

      call prtoavshead ( isol, iref, numveclc, &
                         numveccomps, maxvec1 )
      if ( debug ) write(irefwr,*) 'after prtoavshead'

!     --- write solution
!         The solution must be written in the form:
!         node number, components
!         So it is necessary to create an array with inunkp components per point

      allocate ( usolloc(nnodes,inunkp), stat = error )
      if ( error/=0 ) call eralloc ( error, inunkp*nnodes, 'usolloc' )
      if ( ierror/=0 ) go to 1000

!     --- For all vectors 1 to maxvec1 do

      iseqlc = 1
      do i = 1, maxvec1

         filled = isol(i)>0 .and. isol(i)<=lastsol
         if ( filled ) filled = solarr(isol(i))>0
         if ( filled ) &
            filled = ks(isol(i))%typevec==110 .or. ks(isol(i))%typevec==115
         if ( filled ) then

!        --- Vector has been filled and is of type 110 or 115
!            Determine type of vector, iinunkp and number of vectors

            call sepactsolbf2 ( isol(i) )
            lenname = index(names(i),' ')-1
            if ( lenname<0 ) lenname = len(names(i))

            if ( indprfi==0 ) then

!           --- Constant number of degrees of freedom per point

               usoltemp => ks(isol(i) )%sol
               call inimapsol ( usoltemp, usolloc(1,iseqlc), nodes, &
                                nphysi, nnodes )
               iseqlc = iseqlc+nphysi

            else

!           --- Non-constant number of degrees of freedom per point
!               Not yet implemented

               call errsub ( 1653, 0, 0, 0 )
               go to 1000

            end if  ! ( indprfi==0 )

         end if  ! ( filled )

      end do  ! i = 1, maxvec1

      if ( debug ) write(irefwr,*) 'after filling usolloc'

!     --- write array

      call prtoavs1 ( usolloc, nnodes, inunkp, iref )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'after prtoavs1'

      deallocate ( kmeshcloc, map, nodes, usolloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshcloc' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sep2parelgrp'

      end if  ! ( debug )
      call erclos ( 'sep2parelgrp' )

      end subroutine sep2parelgrp
