      subroutine sepcopyus ( iincop, isol, routcp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 29-05-2014 remove call to time subroutine
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine COPYUS is used to find the solution from a SEPRAN
!     array and store it in a user array
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     user_vector
!     sepran_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iincop(*), isol
      double precision routcp(*)

!     iincop     i    User input array
!                     In this array the user defines which output the user
!                     requires from the solution.
!                     The output may be restricted to a user
!                     point, a curve, a surface or to the complete mesh etc.
!                     IINCOP must be filled as follows:
!                     Pos. 1   Number of entries that are filled by the user
!                              All other positions are defined by default values
!                              So if IINCOP(1) = 0 or 1 only defaults are used
!                     Pos. 2   ICHOIS: Defines what type of nodes must be used.
!                              Possible values:
!                              0: copy contents of isol into ROUTCP
!                              1: copy contents of ROUTCP into isol
!                                 In this case the user must give IPROB
!                                 (IINCOP(5)) a value
!                                 The array ISOL is supposed to be a solution
!                                 vector
!                              2: Copy the values of the unknowns in user point
!                                 IPOINT into ROUTCP
!                              10: See 0, COPYUS has been called by COPYUV, i.e.
!                                  KMESH has not been filled
!                              11: See 1, COPYUS has been called by COPYUV, i.e.
!                                  KMESH has not been filled
!                              Default value: 0
!                     Pos. 3   IDEGFD: Defines the specific unknowns in each
!                              point that must be copied into ROUTCP
!                              If IDEGFD = 0, all unknowns are copied
!                              Default value: 0
!                     Pos. 4   INODE:  Defines the user point in which the
!                              unknown must be selected (ICHOIS=2)
!                              Default value: 1
!                     Pos. 5   IPROB Problem number, must only be filled
!                              for ichois = 1
!                              Default value: 1
!                     Pos. 6   ICOMPL Indication if the vector is real (0) or
!                              complex, must only be filled for ichois = 1
!                              Default value: 0
!                     Pos. 7   ISTART, Starting position in array ROUTCP, with
!                              respect to the information.
!                              Default value: 6
!     isol       i    Standard SEPRAN solution array of type 110, 115 or 116
!     routcp    i/o   User array in which the computed value(s) is (are) stored
!                     ROUTCP is a variable length array which implies that the
!                     first position must be filled by the user with the
!                     declared length
!                     It is advised to set the third position equal to 0
!                     The output is always stored from position 6!
!                     If ICHOIS = 1, ROUTCP is an intput array to be filled
!                     from position 6.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ichois, idegfd, inode, icompl, itypvc, &
              ivec, nfill, istart

!    i               General loop variable
!    ichois          Choice parameter, see IINCOP
!    icompl          Indicates if vector is real (0) or complex (1), see IINCOP
!    idegfd          Degree of freedom, see IINCOP
!    inode           User point number, see IINCOP
!    istart          Starting position in ROUTCP, see IINCOP
!    itypvc          Type number of solution vector (110=solution vector, &
!                    115=vector of special structure)
!    ivec            Indication of the structure of a vector of special
!                    structure
!    nfill           Number of positions that have been filled in array routcp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0035         Put solution in user array or vice versa (real case)
!     ER0051         Put solution in user array or vice versa (complex case)
!     ER0067         Put value of unknowns of user point into array (real case)
!     ER0068         Put value of unknowns of user point into array
!                    (complex case)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCPUS         Check input for COPYUS and set some parameters
!     PRINLC         Print and checks length of complex user arrays
!     PRINLG         Print and checks length of user arrays
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1217   IDEGFD>Nphys
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopyus' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopyus'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Test input

      call incpus ( iincop, isol, routcp, ichois, &
                    inode, idegfd, icompl, istart )
      if ( debug )  &
         write(irefwr,1) 'ichois, inode, idegfd, icompl, istart', &
                          ichois, inode, idegfd, icompl, istart

      if ( ichois/=1 ) then

!     --- Array ISOL has been filled

         call sepactsolbf2 ( isol )
         if ( idegfd>nphysi ) then

!        --- error 1217:IDEGFD (IINCOP(3))>nphys

            call errint ( idegfd, 1 )
            call errint ( nphys, 2 )
            call errsub ( 1217, 2, 0, 0 )
         end if

      else

!     --- Array ISOL must be filled

         call sepcreasol110 ( isol, 'usol',icompl  )
         indprfi = indprfo
         indprpi = indprpo
         kprobfi => kprobfo
         kprobpi => kprobpo
         nphysi = nphyso

      end if

      if ( ierror/=0 ) go to 1000

!     --- Set starting addresses

      usoltemp => ks(isol)%sol

!     --- Call actual computing subroutine, depending on ichois

      if ( ichois==2 ) then

!     --- ichois = 2, Fill array in user point

         if ( icompl==0 ) then

!        --- Real case

            call er0067 ( idegfd, inode, usoltemp, &
                          nfill, routcp(istart) )

            call prinlg ( 'routcp', -istart+1-nfill, routcp, isol )
         else

!        --- Complex case

            call er0068 ( idegfd, inode, usoltemp, &
                          nfill, routcp(istart*2-1) )
            call prinlc ( 'routcp', istart-1+nfill, routcp )

         end if

      else

!     --- ichois = 0 or 1

         nfill = ks(isol)%nusol
         if ( icompl==0 ) then

!        --- Real case

            call er0035 ( ichois, nfill, idegfd, usoltemp, &
                          routcp(istart) )
            call prinlg ( 'routcp', -istart+1-nfill, routcp, isol )

         else

!        --- Complex case

            call er0051 ( ichois, nfill, idegfd, usoltemp, &
                          routcp(2*istart-1) )
            call prinlc ( 'routcp', istart-1+nfill, routcp )

         end if

      end if
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopyus'

      end if  ! ( debug )
      call erclos ( 'sepcopyus' )

      end subroutine sepcopyus
