      subroutine sepgather ( isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute global vector in case of parallel computations by
!     combining parts of the vectors from the nodes
!     The global vector is stored on the host (node 1)
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     solution_vector
!     vector
!     global
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulesol
      use sepmodulecpack
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol

!     isol           i    Sequence number of vector in array ks
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icontext, leng, iblock, icount, npointsav
      integer, allocatable :: ibuffer(:)

!     i              General loop variable
!     iblock         Block sequence number
!     ibuffer        Work array to store quantities to be sent or received
!     icontext       Tag for sending and receiving, is used as check
!     icount         Size of arrsy to be sent or received
!     leng           Help variable to compute the length of some arrays
!     npointsav      Help parameter to save the old value of npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: seplenglobvec

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOPY         Copy one integer array into another
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRINRL         Print 1d real vector
!     SEPLENGLOBVEC  Return with length of global vector corresponding to isol
!     SEPMAPGLOBVEC  Map local vector referred to by isol into global vector
!                    Only for host
!     SEPMAPGLOBVECN Map local vector received from nodes into global vector
!                    on host
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( vector isol has changed ) then
!        If ( nodenr==1 ) then
!           compute length of vector
!           allocate vector if it does not exist
!           copy local vector into global vector
!           For i := 2 (1) iacnodes do
!              receive local vector from node i
!              copy local vector into global vector
!        else
!           send local vector to host
!        mark isol as not changed
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgather' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgather'
         write(irefwr,1) 'isol', isol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprob = ks(isol)%iprob
      iseqkp = ks(isol)%iseqkp
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobpar => kprobpart%kprobpar
      nusolglob = kprobpar(1)
      iseqkm = kp(iseqkp)%iseqkm
      kmeshpar => km(iseqkm)%kmeshpar
      npointglob = kmeshpar(1)
      allocate ( ibuffer(npointglob+2*nusolglob+2) )
      npointsav = km(iseqkm)%kmesh(8)
      if ( ks(isol)%changed ) then

!     --- vector isol has changed

         if ( inodenr==1 ) then

!        --- nodenr==1, hence host
!            compute length of vector
!            allocate vector if it does not exist

            leng = seplenglobvec ( isol )

            if ( debug ) write(irefwr,1) 'leng', leng

            allocate(usolt(leng))

            if ( associated(ks(isol)%solglob) ) then

!           --- ks(isol)%solglob exists

               if ( size(ks(isol)%solglob)/=leng ) then
                  deallocate(ks(isol)%solglob)
                  allocate(ks(isol)%solglob(leng))
               end if  ! ( size(ks(isol)%solglob)/=leng )

            else

!           --- ks(isol)%solglob does not exist

               allocate(ks(isol)%solglob(leng))

            end if  ! ( associated(ks(isol)%solglob) )

!           --- copy local vector into global vector

            call sepmapglobvec ( isol )

!           --- For i := 2 (1) iacnodes do

            do i = 2, iacnodes

!           --- receive local vector from node i

               icontext = 44
               allocate ( nodes(npointglob) )
               icount = npointglob+2*nusolglob+2
               call inireceivempi ( ibuffer, icount, SPINT, iblock, &
                                    icontext )
               length = ibuffer(1)
               npoint = ibuffer(2)
               call incopy ( ibuffer(3), usolt, length*2 )
               call incopy ( ibuffer(3+length*2), nodes, npoint )

!              --- copy local vector into global vector

               call sepmapglobvecn ( isol, iblock )
               npoint = npointsav
               deallocate ( nodes )

            end do  ! i = 2, iacnodes

            deallocate ( usolt )

            if ( debug ) call prinrl ( ks(isol)%solglob, leng, 'usolglob' )

         else

!        --- send local vector to host

            icontext = 44

!           --- Store usol and mappingparcp in ibuffer

            usol => ks(isol)%sol
            length = 2*size(usol)
            ibuffer(1) = size(usol)
            ibuffer(2) = npoint
            call incopy ( usol, ibuffer(3), length )
            call incopy ( mappingparcp, ibuffer(length+3), npoint )
            call inisendmpi ( ibuffer, length+npoint+2, SPINT, 1, icontext )

         end if  ! ( inodenr==1 )

!        --- mark isol as not changed

         ks(isol)%changed = .false.

!        --- Synchronize communication by calling mpi_barrier
!            This avoids that something is written to other nodes,
!            before reading by those other nodes has been done

         call inibarriermpi

      end if  ! ( ks(isol)%changed )

      deallocate ( ibuffer )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgather'

      end if  ! ( debug )
      call erclos ( 'sepgather' )

      end subroutine sepgather
