      subroutine sepintmap3dc ( isol, number, nunkploc, interp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map solution in 3d region onto regular 3d solution defined in
!     all nodes of the mesh (except centroids)
!     complex version
!     Old subroutine intmap3dc
!     The result is stored in usolt
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), number, nunkploc, interp

!     interp         i    Indication of what variables must be interpolated
!                         See subroutine intp3dbg
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     number         i    indicates which is the first degree of freedom
!                         that must be interpolated in each point
!     nunkploc       i    indicates the number of unknowns that must be
!                         interpolated in each point containing this
!                         number of unknowns counted from number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical interpol

!     interpol       If true interpolation may be necessary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTMAP3D1      Check if interpolation of unknowns is necessary or that
!                    copying suffices
!     INTMAP3D2C     Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array with constant number of
!                    unknowns
!                    The unknowns are present in all points of mesh
!                    Complex version
!     PRCOPY2D2C     Copy two-dimensional array of size ndim1 x npoint into
!                    two-dimensional array of size ndim2 x npoint.
!                    Complex version
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPINTMAP3D3C  Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array with constant number of
!                    unknowns
!                    The solution to be copied is not present in all nodes
!                    Complex version
!     SEPINTMAP3D4C  Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array with constant number of
!                    unknowns
!                    The solution to be copied is not present in all nodes
!                    Center nodes of elements and faces are skipped
!                    Complex version
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1211   INTERP has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create output array
!     If ( all unknowns present in input array ) then
!        Copy unknowns
!     else
!        Copy and map unknowns
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmap3dc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmap3dc'
         write(irefwr,1) 'number, nunkploc, interp', number, nunkploc, interp
         call prinin ( isol, 5, 'isol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate input arrays

      call sepactsolbf2 ( isol )

      if ( debug ) &
         write(irefwr,1) 'indprfi, indprpi, nphysi', &
                          indprfi, indprpi, nphysi

      if ( number-1+nunkploc>nphysi ) then

!     --- More unknowns required than available in solution array

         call errint ( interp, 1 )
         call errint ( nunkploc+number-1, 2 )
         call errint ( nphysi , 3 )
         call errsub ( 1211, 3, 0, 0 )
         go to 1000

      end if  ! ( number-1+nunkploc>nphysi )

!     --- Create output array sol3dtemp

      allocate ( usolt(nunkploc*npoint*2) )
      usol => ks(isol(1))%sol

      if ( indprfi==0 ) then

!     --- all unknowns present in input array

         call prcopy2d2c ( npoint, usol, nphysi, usolt, nunkploc, number )

      else if ( indprpi/=0 ) then

!     --- Complicated situation, the number of unknowns is not constant
!         Check if interpolation might be necessary

         call intmap3d1 ( number+nunkploc-1, interpol )
         if ( debug ) write(irefwr,*) 'interpol', interpol

         if ( interpol ) then

!        --- interpol true, so interpolation may be necessary

            call sepintmap3d3c ( usol, usolt, nunkploc, number )

         else

!        --- interpol false, no interpolation necessary
!            Copying is sufficient

            call intmap3d2c ( usol, usolt, nunkploc, number )

         end if  ! ( interpol )

      else

!     --- indprfi>0, 1 unknown per point
!         interpolation is necessary

         call sepintmap3d4c ( usol, usolt, kprobfi )

      end if  ! ( indprfi==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintmap3dc'

      end if  ! ( debug )
      call erclos ( 'sepintmap3dc' )

      end subroutine sepintmap3dc
