      subroutine sepintp3dchk ( isol, ichoice, ichoicearr, &
                                icompl, interp, itrans, number, nunkp, &
                                plane, values )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some checks for sepintp3d
!     Fill array plane
!     Old subroutine intp3dchk
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, ichoice, ichoicearr(*), &
              icompl, interp, itrans, number, nunkp
      double precision plane(4), values(*)

!     ichoice        i    Choice parameter consisting of three parts
!                         See subroutine intp3dbf
!     ichoicearr     o    Array of size 3 containing the three parts of ichoice
!                         1:  ichoice_plane
!                         2:  ichoice_tang
!                         3:  ichoice_orig
!     icompl         o    Multiplication factor for complex numbers
!                         Real: icompl=1,   Complex: icompl=2
!     interp         i    Indication of what variables must be interpolated
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!                         See subroutine intp3dbf
!     itrans         i    indicates if the quantity to be interpolated
!                         must be transformed or not.
!                         See subroutine intp3dbf
!     number         o    The unknowns from position number must be interpolated
!     nunkp          o    Maximum number of unknowns in a point
!     plane          o    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     values         i    Array containing information concerning the plane, &
!                         tangential vectors and origin
!                         See subroutine intp3dbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, i

!     i              Counting variable
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!      60   Input solution array has not been filled or is of the wrong type
!     581   The mesh is not three-dimensional
!     582   INTERP has incorrect value
!     696   itrans=1, whereas nunkp<3
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintp3dchk' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintp3dchk'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=3 ) then
         call errint ( ndim, 1 )
         call errsub ( 581, 1, 0, 0 )
      end if
      ichoicearr(1) = mod(ichoice,10)
      ichoicearr(2) = mod(ichoice/10,10)
      ichoicearr(3) = ichoice/100

      if ( ichoicearr(1)<1 .or. ichoicearr(1)>4 )  then
         call errint ( ichoice, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( ichoicearr(2)<0 .or. ichoicearr(2)>7 )  then
         call errint ( ichoice, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( ichoicearr(3)<0 .or. ichoicearr(3)>1 )  then
         call errint ( ichoice, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      itypvc = ks(isol)%typevec
      if ( itypvc/=110 .and. itypvc/=115 )  then

!     --- Type of vector not allowed

         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )

      end if
      icompl = ks(isol)%compl+1

      itrans = interp/10000
      number = (interp - 10000*itrans)/100
      nunkp = interp - 100 * number - 10000*itrans
      if ( number<0 .or. nunkp<1 ) then
         call errint ( interp, 1 )
         call errsub ( 582, 1, 0, 0 )
      end if
      if ( itrans<0 .or. itrans>1 ) then
         call errint ( interp, 1 )
         call errsub ( 582, 1, 0, 0 )
      end if
      if ( itrans==1 .and. nunkp<3 ) then
         call errint ( itrans, 1 )
         call errint ( nunkp, 2 )
         call errint ( 3, 3 )
         call errsub ( 696, 3, 0, 0 )
      end if
      number = max(1, number)

      if ( ichoicearr(1)<4 ) then

!     --- ichoice< 4, fill array plane

         plane(4) = values(1)

      else

         do i = 1, 4
            plane(i) = values(i)
         end do

      end if  ! ( ichoice_tang==0 .and. ichoice<4 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintp3dchk'

      end if  ! ( debug )
      call erclos ( 'sepintp3dchk' )

      end
