      subroutine sepprecal5
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy matrix augmented lagrangian matrix into matrix premat,
!     where premat may have a different structure.
!     Both matrices are stored as compact matrix
!     If an entry of amat is not present in premat this entry is skipped
!     This concerns the lower and upper part of the matrix
!     Old subroutine preccopy2
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     preconditioning
!     compact
!     augmented_lagrange
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(nu), i, istarta, istartpre, irow, numcola, &
              numcolpre, j, icolumn, jpos, icol, jcol, l, numcold, k
      double precision sum
      logical :: extradebug

!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     icol           Column sequence number
!     icolumn        Column sequence number
!     irow           Row sequence number
!     istarta        Last entry used in array Q%S
!     istartpre      Last entry used in aray Q%MD
!     iwork          integer work array of length nu
!                    This array is used to mark non-zero columns in each row
!                    of the preconditioning matrix
!                    iwork(i) = 0: entry not in premat
!                    iwork(i)>0 then column i is in premat in position iwork(i)
!     j              Counting variable
!     jcol           Column sequence number
!     jpos           Position in array
!     k              Counting variable
!     l              General loop variable
!     numcola        Number of non-zero columns in row of matrix amata
!     numcold        Number of non-zero columns in row of matrix d
!     numcolpre      Number of non-zero columns in row of matrix amatpre
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINDMAT       Print the D matrix in the Simple method
!     PRINRL         Print 1d real vector
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iwork(1-nu) := 0
!     istarta := Q%rows(1);  istartpre := Q%Mrows(1)
!     For irow := 1 (1) nu do
!        numcola := Q%rows(irow+1)-istarta
!        numcolpre := Q%Mrows(irow+1)-istartpre
!        Mark positions of columns in premat in array iwork:
!        For j := 1, numcolpre
!           iwork(Q%Mcols(istartpre+j)) := istartpre+j
!        For j := 1, numcola
!           istarta := istarta+1
!           icolumn := icolsQ(istarta), jpos := iwork(icolumn)
!           if ( jpos>0 ) then
!              Q%MD(jpos) := Q%S(istarta)
!        Clear array iwork
!        For j := 1, numcolpre
!           iwork(Q%Mcols(istartpre+j)) := 0
!        istartpre := istartpre+numcolpre
!     Add algamma G M^-1 D to matrix, i.e.
!     s_ij := s_ij + algamma G_ik m_kk D_kj = s_ij + algamma G^T_ki m_kk D_kj, &
!           where a summation over k is carried out
!     for k := 1 (1) n_p do
!        numcold := number of columns in d-row
!        for j := 1, numcold do
!           icol := column number d_ik
!           Mark positions of columns in premat_i in iwork
!           for i := 1, numcold do
!              if ( i=j ) then
!                 add element to diagonal element icol of matrix
!              else
!                 jcol := column number g^t_jk
!                 if ( iwork(jcol)>0 ) then
!                    add element to matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecal5' )
      debug = .false.
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecal5'
         if ( extradebug ) then
            write(irefwr,*) 'q-matrix'
            call sepprinmat6 ( Q )
            write(irefwr,*) 'Matrix D'
            call prindmat ( D%DIV, D%rows, D%cols, np )
            call prinrl ( S_PMD, np, 'S_PMD' )
         end if  ! ( extradebug )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      Q%MD(Q%Mdiag) = Q%S(Q%diag)

!     --- First clear array iwork

      iwork = 0

!     --- istarta := Q%rows(1);  istartpre := Q%Mrows(1)

      istarta = Q%rows(1)
      istartpre = Q%Mrows(1)

!     --- For irow := 1 (1) nu

      do irow = 1, nu

!     --- numcola := Q%rows(irow+1)-istarta
!         numcolpre := Q%Mrows(irow+1)-istartpre

         istarta = Q%rows(irow)+1
         istartpre = Q%Mrows(irow)+1
         numcola = Q%rows(irow+1)-istarta
         numcolpre = Q%Mrows(irow+1)-istartpre

!        --- Mark positions of columns in premat in array iwork:
!            For j := 1, numcolpre
!               iwork(Q%Mcols(istartpre+j)) := istartpre+j

         do j = 1, numcolpre
            iwork(Q%Mcols(istartpre+j)) = istartpre+j
         end do  ! j = 1, numcolpre

!        --- For j := 1, numcola

         do j = 1, numcola

!        --- istarta := istarta+1
!            icolumn := icolsQ(istarta), jpos := iwork(icolumn)

            istarta = Q%rows(irow)+j+1
            icolumn = icolsQ(istarta)
            jpos = iwork(icolumn)
            if ( jpos>0 ) then

!           --- jpos>0, hence entry in both matrices

               Q%MD(jpos) = Q%S(istarta)

            end if  ! ( jpos>0 )

         end do  ! j = 1, numcola

!        --- Clear array iwork
!            For j := 1, numcolpre
!               iwork(Q%Mcols(istartpre+j)) := 0

         do j = 1, numcolpre
            iwork(Q%Mcols(istartpre+j)) = 0
         end do  ! j = 1, numcolpre

      end do  ! irow = 1, nu

!     --- Add algamma G M^-1 D to matrix
!         for k := 1 (1) n_p do

      do k = 1, np

!     --- numcold := number of columns in d-row

         istarta = D%rows(k)  ! last position used in D%DIV
         numcold = D%rows(k+1)-istarta

!        --- for j := 1, numcold do

         do j = 1, numcold

!        --- icol := column number d_ik

            icol = D%cols(istarta+j)

!           --- Mark positions of columns in premat in array iwork:

            do l = Q%Mrows(icol)+1, Q%Mrows(icol+1)
               iwork(Q%Mcols(l)) = l
            end do  ! l = Q%Mrows(icol)+1, Q%Mrows(icol+1)

!           --- for i := 1, numcold do

            do i = 1, numcold

!              --- jcol := column number g^t_jk

               jcol = D%cols(istarta+i)
               jpos = iwork(jcol)
               if ( jpos>0 ) then

!              --- iwork(jcol)>0
!                  add element to matrix

                  sum = algamma*G%grad(istarta+j)* &
                        S_PMD(k)*D%DIV(istarta+i)
                  Q%MD(jpos) = Q%MD(jpos)+sum

               end if  ! ( iwork(jcol)>0 )

            end do  ! i = 1, numcold

!           --- Clear iwork:

            do l = Q%Mrows(icol)+1, Q%Mrows(icol+1)
               iwork(Q%Mcols(l)) = 0
            end do  ! l = Q%Mrows(icol)+1, Q%Mrows(icol+1)

         end do  ! j = 1, numcold

      end do  ! k = 1, np

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'adapted preconditioning matrix'
         call sepprinmat6p ( Q )
         write(irefwr,*) 'End sepprecal5'

      end if  ! ( debug )
      call erclos ( 'sepprecal5' )

      end
