      subroutine to0069 ( vecold, startp, npart, &
                          tbegin, tend, plotfm, yfact, ichois, user, &
                          iuser, icount, maxsteps )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  3.0  date 02-09-2010 Remove work array
!        version  2.1  date 06-09-1997 New call to pl0003
!        version  2.0  date 06-04-1996 Extra parameters
!
!   copyright (c) 1990-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Perform particle tracking from (xstart,ystart)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mixed_element
!     post_processing
!     track
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois(5), icount, iuser(*), maxsteps, npart
      double precision plotfm, startp(*), tbegin, tend, &
                       user(*), vecold(*), yfact

!     ichois     i    Choice parameter array:
!                     ichois(1)  =  length of array ichois to be used
!                     ichois(2)  =  type of velocity field given
!                          3 : Fluxes through the edges are given (elmtyp=650)
!                     ichois(3)  =  print indicator
!                          0 : Make no print of trajectories       (default)
!                         >0 : Make print of particle trajectories on file with
!                              referencenumber ichois(3)
!                     ichois(4)  =  plot indicator
!                          0 : Make no plots
!                          1 : Make plot of particle trajectories
!                          2 : Make plot of mesh and trajectories  (default)
!                     ichois(5) = direction indicator for parallel projection
!                               (only used for 3-D)
!                          1 : Projection on (x,z) plane from y is +inf(default)
!                          2 : Projection on (y,z) plane from x is +inf
!                          3 : Projection on (x,z) plane from y is -inf
!                          4 : Projection on (y,z) plane from x is -inf
!     icount     i    the number of unknowns per element
!     iuser      i    Integer array in which information concerning the
!                     coefficients is stored
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     npart      i    Number of particles to be followed
!     plotfm     i    Plotting factor. When PLOTFM<0, then the length of
!                     the plot is computed by - (xmax-xmin) * PLOTFM
!                     and the width by - (ymax-ymin) * YFACT * PLOTFM
!                     else the length of the plot is equal to PLOTFM cm
!                     When by some mistake the user has given | PLOTFM | a value
!                     smaller than 1 cm, then the length is chosen equal to
!                       COMMON BLOCKS
!     startp     i    User array of length NPART * NDIM in which the
!                     user is supposed to have stored the starting points of
!                     the particles to be followed
!     tbegin     i    Begin time for tracking
!     tend       i    End time for tracking
!     user       i    Real array in which information concerning the
!                     coefficients is stored.
!     vecold     i    real array, containing the fluxes through the edges
!     yfact      i    Multiplication for the y-coordinates. With this
!                     parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision b(3), binv(3,3), bmat(3,3), deltat, fluxe(4), &
                       rjacob, por, t, x(30), xc(3), xloc(3), xstart, &
                       ystart, zstart, partout(ndim+1,maxsteps+1)
      integer icolor, id, idoubl, ielem, igroup, iinner, inpelm, &
              ikader, ikeus, iloc, index1(14), index2(4), &
              istart, nrofel, nview, jkadsv, nsteps, idummy(1)
      logical eoarea, eotime, herh, ncontr, part

!     b           real vector, the translation vector of the mapping
!     binv        real matrix, the inverse of bmat
!     bmat        real matrix, the mapping to a reference element
!     deltat      real, the calculated elapsed time since the last write
!     eoarea      boolean, end_of_area tells whether the particle has reached
!                 the boundary of the area
!     eotime      boolean, end_of_time tells whether there is still time left
!     fluxe       real array, containing the outward fluxes through the edges
!                 of one element
!     herh        boolean, while loop variable
!     icolor      integer, the color of the track
!     id          loop variable
!     idoubl      integer, plot not with double line (0)
!     idummy         Dummy parameter
!     ielem       integer, the element number
!     igroup      groupe number of triangle considered
!     iinner      integer, plot inner boundaries (3)
!     ikader      integer, temporary variable for jkader
!     ikeus       integer, choise parameter
!                 0 in call to PL0003, tells to compute and start the picture
!     iloc        integer, the local edge number
!     index1      integer array, containing the nodal point numbers of one
!                 element
!     index2      integer array, containing the index numbers of vecold of
!                 one element
!     inpelm      the number of nodal points per element
!     istart      help variable for porosity determination
!     jkadsv      Saves the value of jkader at input
!     ncontr      boolean, see INITS5
!     nrofel      count variable for element group determination:
!                 number of elements
!     nsteps      Contains the number of positions stored in partout
!     nview       see ichois(5)
!     part        boolean, see INITS5
!     partout     Array containing the actual information concerning the
!                 particle trajectories
!                 partout(1,i) contains t_i
!                 partout(2-4,i) contains x_i, y_i and z_i
!     por         real, the (constant) porosity of the element
!     rjacob      real, the determinant of bmat
!     t           real, the calculated total elapsed time
!     x           real array, containing the coordinates of all nodal points
!                 of one element
!     xc          real array, the coordinates of the centre of the element
!     xloc        real array, the local coordinates of (xstart, ystart)
!     xstart      real, the x-coordinate of the point
!     ystart      real, the y-coordinate of the point
!     zstart      real, the z-coordinate of the point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision funcpo

!     ELM651         fills x
!     ELM659         computes the inverse of tensor A
!     ELM660         computes the transformation matrix B
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCPO         compute the porosity in this element ( not used yet )
!     INSTOP         Stop the program
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELM1         Plot elements in 2D mesh
!     PLTR16         Plot reference cadre in case of 3-D
!     SEPPLOTBN      Plot boundary of region
!     TO0068         Particle tracking routine for the case elmtyp = 650
!     TO0071         find the element in which (xstart,ystart) lies
!     TO0072         fill array INDEX1
!     TO0073         fill array INDEX2
!     TO0074         determine the translation vector of the mapping
!     TO0075         compute the translation vector
!     TO0076         determine the local coordinates
!     TO0077         compute the coordinates of the centre of the element
!     TO0078         compute the track in the reference element
!     TO0079         get the entered element
!     TO0080         compute the global coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     937   value of porosity is equal to 0
!    1638   Particle array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!    ----------------------------------------------------------------
!    | initialise track                                             |
!    |--------------------------------------------------------------|
!    | while not end_of_time and not end_of_area                    |
!    |   -----------------------------------------------------------|
!    |   | map point to reference element                           |
!    |   |----------------------------------------------------------|
!    |   | compute track in reference element                       |
!    |   |----------------------------------------------------------|
!    |   | map track to global coordinates                          |
!    |   |----------------------------------------------------------|
!    |   | report track                                             |
!    |--------------------------------------------------------------|
!    | close track                                                  |
!    ----------------------------------------------------------------
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0069' )

      inpelm = npelm

!     --- set colour

      icolor = 2
      dcolor = 0
      jmark  = 5
      jkadsv = jkader
      jkader = -3

!     --- initialise plot

      textx = 'x'
      texty = 'y'
      if ( ichois(4)/=0 ) then
         ikader = 0
         ikeus = 0
         if ( ndim==2 ) then
            call pl0003 ( ikader, 1, plotfm, yfact, ikeus )
         end if

         if ( ichois(4)==2 ) then

!        --- plot mesh

            if ( ndim==2 ) then
               ncontr = .true.
               part   = .false.
               call plelm1 ( iuser, coor, ncontr, part, .false. )
            else

!           --- plot surrounding box

               if ( ichois(5)>4  .or. ichois(5)<1 ) then
                  nview = 1
               else
                  nview = ichois(5)
               end if
               call pltr16 ( plotfm, yfact, nview )

            end if

         else

            if ( ndim==2 ) then

!           --- plot boundaries

               idoubl = 0
               iinner = 3
               call sepplotbn ( idoubl, iinner, .false. )

            else

!           --- plot surrounding box

               call pltr16 ( plotfm, yfact, ichois(5) )

            end if

         end if

      end if

!     --- loop over starting points

      do 30 id = 1, npart

         if ( icolor==8 ) then
            icolor = 3
         else
            icolor = icolor + 1
         end if
         dcolor = icolor
         t = tbegin
         if ( ndim==2 ) then
            xstart = startp(2*(id-1)+1)
            ystart = startp(2*(id-1)+2)
            zstart = 0
         else
            xstart = startp(3*(id-1)+1)
            ystart = startp(3*(id-1)+2)
            zstart = startp(3*(id-1)+3)
         end if
         eotime = ( t>=tend )
         eoarea = .false.

!        --- initialise track

         nsteps = 1
         partout(1,nsteps) = tbegin
         partout(2,nsteps) = xstart
         partout(3,nsteps) = ystart
         if ( ndim==3 ) partout(4,nsteps) = zstart

!        --- find element in which (xstart,ystart) lies

         call to0071 ( xstart, ystart, zstart, coor, kmeshc, inpelm, &
                       ielem, nelem, ndim )

!        --- if (xstart,ystart) lies in area then fill index1 and index2
!            corresponding to the element

         if ( ielem==0 ) then
            eoarea = .true.
         else
            call to0072 ( index1, kmeshc, inpelm, ielem )
            call to0073 ( index2, index1, ndim )
         end if

!        --- while not end_of_time nor end_of_area

10       if ( ( .not. eotime ) .and. (.not. eoarea ) ) then

!        --- get the information of this element

            call elm651 ( coor, inpelm, index1, x, ndim )
            call to0074 ( inpelm, index1, index2, fluxe, vecold, ndim )
            call to0077 ( icount, x, xc, ndim )

!           --- Determine elementgroup for this triangle

            igroup = 1
            if ( kelmg/=0 ) then
               nrofel = 0
               herh = .true.
20             if ( herh ) then
                  nrofel = nrofel + kmeshg(igroup)
                  if ( ielem>nrofel ) then
                     igroup = igroup + 1
                  else
                     herh = .false.
                  end if
                  goto 20
               end if
            end if

!           --- Determine porosity for this element

            istart = iuser(5+igroup)

            if ( iuser(istart)==0 ) then

!           --- Error, zero porosity

               call errsub ( 937, 0, 0, 0 )
               call instop

            else if ( iuser(istart)<-5 ) then

!           --- Porosity is constant

               por = user(-iuser(istart))

            else

               write(irefwr,*) ' Porosity not constant in elementgroup '
               write(irefwr,*) ' Not yet supported possibility '
               call instop
               por = funcpo ( user(istart), xc(1), xc(2), xc(3) )

            end if

!           --- map (xstart,ystart) to reference element

            call elm660 ( x, bmat, rjacob, ndim )
            call elm659 ( bmat, binv, ndim )
            call to0075 ( inpelm, x, b )
            call to0076 ( binv, xstart, ystart, zstart, b, xloc, iloc, &
                          ndim )
            if ( ndim==2 ) xloc(3) = 0d0

!           --- compute the track for a reference element

            call to0078 ( rjacob, por, fluxe, xloc(1), xloc(2), xloc(3), &
                          iloc, deltat, ndim, t, tend )

!           --- find the entered element

            if (iloc==10) then
               eotime = .true.
            else
               call to0079 ( iloc, index1, index2, kmeshc, &
                             nelem, inpelm, ielem, ndim )
            end if

!           --- compute the global coordinates

            call to0080 ( xloc, bmat, b, xstart, ystart, zstart, ndim )

!           --- update total elapsed time

            t = t + deltat

!           --- report track

            nsteps = nsteps + 1
            if ( nsteps>maxsteps ) then
               call errint ( nsteps, 1 )
               call errsub ( 1638, 1, 0, 0 )
               go to 1000
            end if
            partout(1,nsteps) = t
            partout(2,nsteps) = xstart
            partout(3,nsteps) = ystart
            if ( ndim==3 ) partout(4,nsteps) = zstart

!           --- check end_of_time and end_of_area

            if ( iloc==0 ) eoarea = .true.
            if ( .not. eotime ) then
               eotime = ( t>=tend )
            end if
            goto 10
         end if
         call to0068 ( ndim, ichois, partout, nsteps )

30    continue

!     --- close plot

1000  call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      jkader = jkadsv
      call erclos ( 'to0069' )

      end
