      subroutine to0078 ( det, por, flux, x, y, z, i, deltat, ndim, &
                          t, tend )
!
! ====================================================================
!
!
!        programmer    Arno van Duin
!        version 1.2   date   92-03-30 ending track in middle of element added
!        version 1.1   date   91-03-21 adjusted for NDIM = 3
!        version 1.0   date   90-11-14
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!                DESCRIPTION
!
!   The track is computed for a reference element
!
! ********************************************************************
!
!                KEYWORDS
!
!     mixed_element
!     post_processing
!     track
!
! ********************************************************************
!
!                INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision deltat, det, flux(4), por, tend, t, x, y, z
      integer i, ndim
!
!     deltat   o  real, the elapsed time for going from the point of entrance
!                 to the point of exit
!
!     det      i  real, the determinant of the mapping to a reference element
!
!     flux     i  real array, containing the outward fluxes through the edges
!                 of the global element
!
!     i        o  integer, the number of the edge through which the element is
!                 left. If i=10 then track ended in the middle of an element
!                 because of out_of_time
!
!     ndim     i  the dimension of space
!
!     por      i  real, the (element-wise constant) porosity of this element
!
!     t        i  elapsed simulation time thusfar for this track
!
!     tend     i  end time for tracking
!
!     x        i  real, the local x-coordinate of the point of entrance
!              o  the local x-coordinate of the point of exit
!
!     y        i  real, the local y-coordinate of the point of entrance
!              o  the local y-coordinate of the point of exit
!
!     z        i  real, the local z-coordinate of the point of entrance
!              o  the local z-coordinate of the point of exit
!
! *********************************************************************
!
!               LOCAL PARAMETERS
!
      integer iside
      double precision a, b, c, d, xin, yin, zin
      logical oke

!
!     a           real, constant part of x-component of velocity field
!                 see chapter 9 of E.F. Kaasschieter:
!                 "Preconditioned Conjugate Gradients And Mixed-Hybrid Finite
!                  Elements For The Solution Of Potential Flow Problems"
!
!     b           real, constant part of y-component of velocity field
!                 see chapter 9 of E.F. Kaasschieter:
!                 "Preconditioned Conjugate Gradients And Mixed-Hybrid Finite
!                  Elements For The Solution Of Potential Flow Problems"
!
!     c           real, constant part of z-component of velocity field
!                 see chapter 9 of E.F. Kaasschieter:
!                 "Preconditioned Conjugate Gradients And Mixed-Hybrid Finite
!                  Elements For The Solution Of Potential Flow Problems"
!
!     d           real, divergence of velocity field
!                 see chapter 9 of E.F. Kaasschieter:
!                 "Preconditioned Conjugate Gradients And Mixed-Hybrid Finite
!                  Elements For The Solution Of Potential Flow Problems"
!
!     iside       side number through which the element can be left
!
!     oke         boolean, checks if there is an inward flux at the point
!                 of entrance
!
!     xin         real, the local x-coordinate
!
!     yin         real, the local y-coordinate
!
!     zin         real, the local y-coordinate
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
      double precision to0083, to0084
!
!     ERCLOS  delete this routte from call tree
!     EROPEN  add this routte to call tree
!     TO0083  computes ( exp(x) - 1 ) / x
!     TO0084  computes ( ln(1 + x) / x
!
! ********************************************************************
!
!                              METHOD
!
!    see E.F. Kaasschieter:
!       "Preconditioned Conjugate Gradients And Mixed-Hybrid Finite
!        Elements For The Solution Of Potential Flow Problems"
!    Chapter 9
!
! ====================================================================

      call eropen ( 'to0078' )

      oke = .true.
      deltat = 0
      iside = 0
      if ( i.ne.0 ) then
         if ( flux(i).ge.0 ) then
            deltat = 0
            oke = .false.
         end if
      end if
      if ( ndim.eq.2 ) then
         if ( oke ) then
            i = 0
            xin = x
            yin = y
            if ( ( flux(1).le.0 ) .and. &
                 ( flux(2).le.0 ) .and. &
                 ( flux(3).le.0 )  )    then
               deltat = 0
               i = 0
            else
               if ( xin.lt.0 ) xin = 0
               if ( yin.lt.0 ) yin = 0
               if ( xin + yin.gt.1d0 ) then
                  if ( xin.ge.yin ) xin = 1d0 - yin
                  if ( yin.ge.xin ) yin = 1d0 - xin
               end if
               a = -flux(2) / ( por * det )
               b = -flux(3) / ( por * det )
               d =  flux(1) / ( por * det ) - a - b
               if ((a+b+d.gt.0) .and. (a+b+d*(xin+yin).gt.0)) then
                  deltat = (1d0-xin-yin) / (a+b+d*(xin+yin)) &
                     * to0084 ( d*(1d0-xin-yin)/(a+b+d*(xin+yin)) )
                  x = xin*exp(d*deltat) + a*deltat*to0083(d*deltat)
                  y = 1d0 - x
                  iside = 1
                  if ( ( x.gt.0 ) .and. ( x.lt.1d0 ) ) then
                     i = 1
                     if ( ( t + deltat ).gt.tend ) then
                        deltat = tend - t
                        i = 10
                        x = xin*exp(d*deltat) +a*deltat*to0083(d*deltat)
                        y = yin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                     end if
                     call erclos ( 'to0078' )
                     return
                  end if
               end if
               if ( ( a.lt.0 ) .and. ( a + d * xin.lt.0 ) ) then
                  deltat = -xin / (a+d*xin) * to0084(-d*xin/(a+d*xin))
                  x = 0
                  y = yin * exp(d*deltat)+b*deltat*to0083(d*deltat)
                  iside = 2
                  if ( ( y.gt.0 ) .and. ( y.lt.1d0 ) ) then
                     i = 2
                     if ( ( t + deltat ).gt.tend ) then
                        deltat = tend - t
                        i = 10
                        x = xin*exp(d*deltat) +a*deltat*to0083(d*deltat)
                        y = yin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                     end if
                     call erclos ( 'to0078' )
                     return
                  end if
               end if
               if ( ( b.lt.0 ) .and. ( b + d * yin.lt.0 ) ) then
                  deltat = -yin/(b+d*yin)*to0084(-d*yin/(b+d*yin))
                  x = xin*exp(d*deltat)+a*deltat*to0083(d*deltat)
                  y = 0
                  iside = 3
                  if ( ( x.gt.0 ) .and. ( x.lt.1d0 ) ) then
                     i = 3
                     if ( ( t + deltat ).gt.tend ) then
                        deltat = tend - t
                        i = 10
                        x = xin*exp(d*deltat) +a*deltat*to0083(d*deltat)
                        y = yin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                     end if
                     call erclos ( 'to0078' )
                     return
                  end if
               end if
            end if
         end if
      else if ( ndim.eq.3 ) then
         if ( oke ) then
            i = 0
            xin = x
            yin = y
            zin = z
            if ( ( flux(1).le.0 ) .and. &
                 ( flux(2).le.0 ) .and. &
                 ( flux(3).le.0 ) .and. &
                 ( flux(4).le.0 )  )    then
               deltat = 0
               i = 0
            else
               if ( xin.lt.0d0 ) xin = 0d0
               if ( yin.lt.0d0 ) yin = 0d0
               if ( zin.lt.0d0 ) zin = 0d0
               if ( xin + yin + zin.gt.1d0 ) then
                  if ( ( xin.ge.yin ) .and. ( xin.ge.zin ) ) &
                      xin = 1d0 - yin - zin
                  if ( ( yin.ge.xin ) .and. ( yin.ge.zin ) ) &
                      yin = 1d0 - xin - zin
                  if ( ( zin.ge.xin ) .and. ( zin.ge.yin ) ) &
                      zin = 1d0 - xin - yin
               end if
               a = -2d0 * flux(2) / ( por * det )
               b = -2d0 * flux(3) / ( por * det )
               c = -2d0 * flux(4) / ( por * det )
               d =  2d0 * flux(1) / ( por * det ) - a - b - c
               if ( ( a+b+c+d.gt.0d0 ) .and. &
                    ( a+b+c+d*(xin+yin+zin).gt.0d0 ) ) then
                  deltat = (1d0-xin-yin-zin)/(a+b+c+d*(xin+yin+zin))* &
                     to0084(d*(1d0-xin-yin-zin)/(a+b+c+d*(xin+yin+zin)))
                  x = xin*exp(d*deltat) + a*deltat*to0083(d*deltat)
                  y = yin*exp(d*deltat) + b*deltat*to0083(d*deltat)
                  z = 1d0 - x - y
                  iside = 1
                  if ( ( x.gt.0d0 ) .and. ( y.gt.0d0 ) .and. &
                       ( x + y.lt.1d0 ) ) then
                     i = 1
                     if ( ( t + deltat ).gt.tend ) then
                        deltat = tend - t
                        i = 10
                        x = xin*exp(d*deltat) +a*deltat*to0083(d*deltat)
                        y = yin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                        z = zin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                     end if
                     call erclos('to0078')
                     return
                  end if
               end if
               if ( ( a.lt.0d0 ) .and. ( a + d * xin.lt.0 ) ) then
                  deltat = -xin/(a+d*xin)*to0084(-d*xin/(a+d*xin))
                  x = 0
                  y = yin*exp(d*deltat) + b*deltat*to0083(d*deltat)
                  z = zin*exp(d*deltat) + b*deltat*to0083(d*deltat)
                  iside = 2
                  if ( ( y.gt.0d0 ) .and. ( z.gt.0d0 ) .and. &
                       ( y + z.lt.1d0 ) ) then
                     i = 2
                     if ( ( t + deltat ).gt.tend ) then
                        deltat = tend - t
                        i = 10
                        x = xin*exp(d*deltat) +a*deltat*to0083(d*deltat)
                        y = yin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                        z = zin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                     end if
                     call erclos('to0078')
                     return
                  end if
               end if
               if ( ( b.lt.0d0 ) .and. ( b + d * yin.lt.0d0 ) ) then
                  deltat = -yin/(b+d*yin)*to0084(-d*yin/(b+d*yin))
                  x = xin*exp(d*deltat) + b*deltat*to0083(d*deltat)
                  y = 0
                  z = zin*exp(d*deltat) + b*deltat*to0083(d*deltat)
                  iside = 3
                  if ( ( x.gt.0d0 ) .and. ( z.gt.0d0 ) .and. &
                       ( x + z.lt.1d0 ) ) then
                     i = 3
                     if ( ( t + deltat ).gt.tend ) then
                        deltat = tend - t
                        i = 10
                        x = xin*exp(d*deltat) +a*deltat*to0083(d*deltat)
                        y = yin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                        z = zin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                     end if
                     call erclos('to0078')
                     return
                  end if
               end if
               if ( ( c.lt.0d0 ) .and. ( c + d * zin.lt.0d0 ) ) then
                  deltat = -zin/(c+d*zin)*to0084(-d*zin/(c+d*zin))
                  x = xin*exp(d*deltat) + b*deltat*to0083(d*deltat)
                  y = yin*exp(d*deltat) + b*deltat*to0083(d*deltat)
                  z = 0
                  iside = 4
                  if ( ( x.gt.0d0 ) .and. ( y.gt.0d0 ) .and. &
                       ( x + y.lt.1d0 ) ) then
                     i = 4
                     if ( ( t + deltat ).gt.tend ) then
                        deltat = tend - t
                        i = 10
                        x = xin*exp(d*deltat) +a*deltat*to0083(d*deltat)
                        y = yin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                        z = zin*exp(d*deltat) +b*deltat*to0083(d*deltat)
                     end if
                     call erclos('to0078')
                     return
                  end if
               end if
            end if
         end if
      end if

      if ( i.eq.0 ) then
         i = iside
         if ( ( t + deltat ).gt.tend ) then
            deltat = tend-t
            i = 10
            x = xin*exp(d*deltat) + a*deltat*to0083(d*deltat)
            y = yin*exp(d*deltat) + b*deltat*to0083(d*deltat)
            if ( ndim.eq.3 ) then
               z = zin*exp(d*deltat) + c*deltat*to0083(d*deltat)
            end if
         end if
      end if

      call erclos ( 'to0078' )

      end
