      subroutine to0101 ( list, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Make list of element groups and elements at the subsequent curves
!    icrv1 to icrv2 with nodal points in nodes exist
!    Restriction: in the present version the elements are restricted
!                 to linear elements in R2 only
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, list(3,nnodes-1)

!     list    o    Array of length 3*nnodes-1, containing the list of all
!                  admissible elements that have two points with the
!                  curves to be considered in common. For each two points
!                  only one element is found.
!                  The sequence of the elements is list is the same as the
!                  one stored in skipelgrps at output.
!                  list(1,i) contains the i-th element sequence number
!                  list(2,i) contains the first node at the curve for this
!                            element
!                  list(3,i) contains the second node at the curve for this
!                            element
!     nnodes  i    Number of nodes in nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iielem, node1, istart, nneigh, nbr1, ineigh, ielm, &
              node2, nbr2, icommn, j, nelm, maxnbr
      integer, allocatable :: ireflg(:), istore(:,:), iarelm(:,:)

!     i            Counting variable
!     iarelm       Work array of length 2 times maximum number of elements
!                  one node is lying in
!                  iarelm(1,i) contains all admissable elements where the
!                  first node of a curve element is lying in
!                  iarelm(2,i) contains all admissable elements where the
!                  second node of a curve element is lying in
!     icommn       Element number of common element of two points
!     ielm         (relative) element number
!     iielem       Relative element number in element group
!     ineigh       Element that contains nodal point number that is considered
!     ireflg       Work array of length nelem
!                  ireflg(ielem) contains the element group sequence number
!                  of element ielem
!     istart       Starting address in array kmshd2
!     istore       Work array of length 2 times nnodes-1
!                  istore(1,ielm) contains the admissable element that contains
!                  both nodes of the "curve element" ielm
!                  istore(2,ielm) contains the corresponding element group
!     j            Counting variable
!     maxnbr       Maximum number of neighbors for all nodes
!     nbr1         Number of admissable elements found for node1
!     nbr2         Number of admissable elements found for node2
!     nelm         Number of admissable elements found for one element group
!     nneigh       Number of elements a nodal point is lying in
!     node1        First node in "curve" element
!     node2        Second node in "curve" element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1493   Only linear 2D elements allowed
!    1494   No common element found for neighbouring nodes
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Fill array ref_element_groups with element numbers corresponding
!    to elements
!    Check if all element groups allowed are linear
!    Adapt array skipelgrps if necessary
!    Find all allowed elements the first node of nodes is lying in
!    For all "elements" along the curve do
!       Find all admissable elements for the second node in the element
!       Compute intersection of elements of first and second node
!       Store the first common element in and corresponding element group
!       in istore
!    Sort array istore and put the result in array list
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0101' )

      maxnbr = maxval ( kmeshd(1:npoint+1) )

      allocate ( ireflg(nelem), istore(2,nnodes), iarelm(2,maxnbr), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'ireflg' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array ref_element_groups with element numbers corresponding
!         to elements

      if ( nelgrp==1 ) then

!     --- only one element group

         ireflg = 1

      else

!     --- More than one element group

         iielem = 0
         do ielgrp = 1, nelgrp
            inelem = sepgetinelem ( ielgrp )
               ireflg(1+iielem:inelem+iielem) = ielgrp
            iielem = iielem + inelem
         end do
      end if

!     --- Check if all element groups allowed are linear
!         Adapt array skipelgrps if necessary

      do ielgrp = 1, nelgrp
         if ( skipelgrps(ielgrp)>0 ) then

!        --- element group is allowed

            ishape = sepgetshape ( ielgrp )
            if ( ishape<3 ) skipelgrps(ielgrp) = 0
            if ( ishape==4 .or. ishape>5 ) then

!           --- error only linear elements are permitted

               call errint ( ishape, 1 )
               call errsub ( 1493, 1, 0, 0 )
               skipelgrps(ielgrp) = 0
            end if
         end if
      end do

!     --- Find all allowed elements the first node of nodes is lying in

      node1  = nodes(1)
      istart = kmshd1(node1)
      nneigh = kmshd1(node1+1) - istart
      nbr1   = 0
      do i = 1, nneigh
         ineigh = kmshd2(istart+i)
         ielgrp = ireflg(ineigh)
         if ( skipelgrps(ielgrp)>0 ) then

!        --- element is allowed

            nbr1 = nbr1 + 1
            iarelm(1,nbr1) = ineigh
         end if
      end do

!     --- For all "elements" along the curve do

      do ielm = 1, nnodes-1

!     --- Find all admissable elements for the second node in the element

         node2 = nodes(ielm+1)
         istart = kmshd1(node2)
         nneigh = kmshd1(node2+1) - istart
         nbr2   = 0
         do i = 1, nneigh
            ineigh = kmshd2(istart+i)
            ielgrp = ireflg(ineigh)
            if ( skipelgrps(ielgrp)>0 ) then

!           --- element is allowed

               nbr2 = nbr2 + 1
               iarelm(2,nbr2) = ineigh
            end if
         end do

!        --- Compute intersection of elements of first and second node

         icommn = 1
         do i = 1, nbr1
            do j = 1, nbr2
               if ( iarelm(1,i)==iarelm(2,j) ) then

!              --- common element found

                  icommn = iarelm(1,i)
                  go to 260
               end if
            end do
         end do

!        --- Error: no common element found

         call errint ( node1, 1 )
         call errint ( node2, 2 )
         call errsub ( 1494, 2, 0, 0 )

!        --- Store the first common element in istore

260      istore(1,ielm) = icommn
         istore(2,ielm) = ireflg(icommn)

!        --- node1 := node2

         node1 = node2
         do i = 1, nbr2
            iarelm(1,i) = iarelm(2,i)
         end do
         nbr1 = nbr2
      end do

!     --- Sort array istore and put the result in array list

      ipos = 0
      do ielgrp = 1, nelgrp
         if ( skipelgrps(ielgrp)>0 ) then

!        --- skipelgrps(ielgrp)>0, admissible element group

            nelm = 0
            do ielm = 1, nnodes-1
               if ( istore(2,ielm)==ielgrp ) then

!              --- element found

                  nelm = nelm + 1
                  ipos = ipos + 1
                  list(1,ipos) = istore(1,ielm)
                  list(2,ipos) = nodes(ielm)
                  list(3,ipos) = nodes(ielm+1)
               end if
            end do
            skipelgrps(ielgrp) = nelm
         end if
      end do

      deallocate ( ireflg, istore, iarelm, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ireflg' )

1000  call erclos ( 'to0101' )
      end
