      subroutine pltjbp ( acnseg, ackseg, nlines, xcint, fcint, ncint, &
                          nextra, kextra, ncomp, ymin, dy, mode )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.2  date 09-03-2013 Correction upperbound
!        version  1.1  date 21-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   pltjbp is the basic plotting algorithm of pltj3d. It takes the
!   output from prlins (see there) & pltjan and  per intersection segment
!   puts it in a list,  adds boundary points, checks the visibility (twice
!   in SHEET-mode), makes a new skyline (twice in SHEET-mode) and plots it.
!   Some care has to be taken in adding the boundary points. The segments
!   that are plotted at one time, must be such, that the x-coordinates in
!   the transformed plane do not overlap. For this reason there is a choice
!   algorithm that chooses candidates to be plotted. A segment may be
!   plotted only if ALL ancestors have been plotted, an ancestor being
!   1. A segment in the same component with lower intersection number
!   2. If this segment has a high left point:
!         the highest upper component of the segment left to this segment
!         (BUT only if it has a low right point)
!   3. If this segment has a high right point:
!         the right successor of the highest upper component of this segment
!         (BUT only if it has a low left point)
!   This step is repeated until all lines are exhausted.
!   Visualization parameters are in common block /ctrans/.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlines, ncint, mode
      integer nextra (2,*), ncomp (2,*), acnseg (0:nlines)
      integer ackseg (0:*), kextra (2, 2, *)
      double precision xcint (ncint), fcint (ncint), dy, xcoor (1000), &
                       ymin

!     ackseg  i    number of points in this segment (array (0:5*nlines))
!     acnseg  i    number of segments in line i (array (0:nlines+1))
!     dy      i    space between successive lines
!     fcint   i    function values along the intersection line
!     kextra  i    information of anlsnc
!     mode    i    BLOCK (=0) or SHEET (=1) mode
!     ncint   i    dimension of fcint & xcint
!     ncomp   i    information of anlsnc
!     nextra  i    information of anlsnc
!     nlines  i    dimension of acnseg
!     xcint   i    coordinates along the intersection line
!     ymin    i    minimal value of (transformed) y coordinate
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision zcoor (1000)
      integer segstr, segst0, skyhi, skylo, point (1000), high, low
      integer skyl1, skyh0, segst1, sline, fline, ilines, inc
      integer cmplst (0:100), pltlst (0:100)
      integer cmpinx, cntlst

!     cmpinx  ?
!     cmplst  array containing segment numbers and line numbers of
!             candidates to be plotted.
!     cntlst  ?
!     fline   ?
!     high    ?
!     ilines  Counting variable
!     inc     1 = forward loading, -1 = backward loading (reverse = true)
!     low     ?
!     pltlst  ?
!     point   array containing the visibility property of the cells
!     segst0  ?
!     segst1  ?
!     segstr  ?
!     skyh0   ?
!     skyhi   ?
!     skyl1   ?
!     skylo   ?
!     sline   ?
!     xcoor   array of x coordinates in the listpack.
!     zcoor   ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJIL  ?
!     PLTJLI  ?
!     PLTJLN  ?
!     PLTJMR  ?
!     PLTJNX  ?
!     PLTJR1  ?
!     PLTJRL  ?
!     PLTJSE  ?
!     PLTJSL  ?
!     PLTJSS  ?
!     PLTJVI  ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      high = 0
      low = 1

!     --- Initialize ListPack & segment count

      call eropen ('pltjbp')
      call pltjil

!     --- If the angle of view is such, that in the actual plot the rear side is
!         up front, we should plot the figure starting from the rear side. That
!         is, We should load the lines from the rear segment, down to the first
!         segment. We also should start at ymax and go down with the
!         (transformed)
!         y-coordinate.

      if (cth - scfi * sth<0) then
         fline = 1
         sline = nlines
         inc = -1
      else
         sline = 1
         fline = nlines
         inc = 1
      end if

!     --- Initialize high & low sky

      skyhi = nil
      skylo = nil

!     --- Initialize cmplst, pltlst

      cmplst (0) = 0
      pltlst (0) = 0

!     --- Now for the main algorithm

      ilines = sline
100   if (inc * (fline - ilines)<0) then
        call erclos ('pltjbp')
        return
      end if

!     --- initialize cmplst for new component

      cmpinx = 1
      if ( ilines-inc<nlines ) then
         do cntlst = acnseg (ilines - inc) + inc,acnseg (ilines),inc
            cmplst (cmpinx) =  cntlst
            cmplst (cmpinx + 1) = ilines
            cmpinx = cmpinx + 2
         end do
      end if
      cmplst (0) = cmpinx - 1
120   if (cmplst (0)==0) then

!     --- empty cmplist: new component or ready.

         if (pltlst (0) ==0) then
            ilines = ilines + inc
         else
            ilines = pltlst (2)
            do 125 cntlst = 4, pltlst (0), 2
            if (inc * (pltlst (cntlst) - ilines)>0) then
               ilines = pltlst (cntlst)
            end if
125         continue
            ilines = ilines + inc
            pltlst(0) = 0
         end if
         goto 100
      else

!     --- Main algorithm

         call pltjnx (cmplst, pltlst, acnseg, ncomp, nextra, kextra, &
              inc)

!        --- First  load & remove isolated points
!            (in sheet mode they only must be removed)

         if (mode==0) then
            call pltjli (xcint, ymin, dy, fcint, ackseg, &
                  ncomp, pltlst, segstr, xcoor, zcoor, inc)
            if ( ierror/=0 ) goto 99999
         end if

!        --- plot isolated points if any (segstr <> nil)
!            first check visibility
!            BLOCK mode only

        if (mode ==0 .and. segstr/=nil) then
            call pltjvi (skyhi, segstr, xcoor, zcoor, point, high)
            call pltjss (segstr, point)
            call pltjsl (segstr, segst0, point)
            call pltjrl (segst0)
            if ( ierror/=0 ) goto 99999
            call pltjse (segstr, xcoor, zcoor)
            call pltjrl (segstr)
         end if

!        --- Load the coordinates in projection into a list starting at
!            segstr.

        call pltjln (xcint, ymin, dy, fcint, ackseg, segstr, &
          xcoor, zcoor, nextra, kextra, pltlst, inc, mode)

!       --- check visibility against the high sky

         call pltjvi (skyhi, segstr, xcoor, zcoor, point, high)
         if ( ierror/=0 ) goto 99999
         call pltjss (segstr, point)
         call pltjsl (segstr, segst0, point)

!        --- The visible part is in segstr, the invisible part in segst0
!            Plot visible part and make new hi sky
!            We remove the 1 point segments so as not to clobber the plotfile

         call pltjr1 (segstr, xcoor, zcoor)
         call pltjse (segstr, xcoor, zcoor)
         call pltjss (skyhi, point)
         call pltjsl (skyhi, skyh0, point)

!        --- Invisible part is in skyh0. Remove this and merge the visible parts

         call pltjr1 (skyhi, xcoor, zcoor)
         call pltjmr (skyhi, segstr, xcoor, zcoor)
         if ( ierror/=0 ) goto 99999
         if (mode==1) then

!        --- SHEET mode
!            Low sky has to be checked against union of the invisible parts of
!            the line and the high sky

            call pltjmr (segst0, skyh0, xcoor, zcoor)
            call pltjvi (skylo, segst0, xcoor, zcoor, point, low)
            if ( ierror/=0 ) goto 99999
            call pltjss (skylo, point)
            call pltjsl (skylo, skyl1, point)
            call pltjss (segst0, point)
            call pltjsl (segst0, segst1, point)

!           --- The visible parts are in skylo and segst0 respectively
!               Remove 1 point segments

            call pltjr1 (segst0, xcoor, zcoor)
            call pltjse (segst0, xcoor, zcoor)

!           --- Make new low sky and discard the invisible parts

            call pltjr1 (skylo, xcoor, zcoor)
            call pltjmr (skylo, segst0, xcoor, zcoor)
            call pltjrl (skyl1)
            call pltjrl (segst1)
            if ( ierror/=0 ) goto 99999
         else

!        --- BLOCK mode. We just throw away the invisible part of the line.
!            and the high sky

            call pltjrl (segst0)
            call pltjrl (skyh0)
            if ( ierror/=0 ) goto 99999
         end if
      end if
      go to 120
99999 continue
      call erclos ('pltjbp')
      end
