      subroutine pltjis ( xcoor, ycoor, outsp0, outsp1, insp0, insp1, &
                          xc, yc )
! ======================================================================
!
!        programmer     Jos van Kan
!        version 3.0    date 04-01-1999 New order of singular cases
!        version 2.3    date 21-12-1998 Error in intersection criterion
!                                       corrected.
!        version 2.2    date 27-10-1998 Now correctly handles coinciding
!                                       right points
!
!   copyright (c) 1995-1998 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjis calculates intersection point of two line segments, one
!       with endpoints (xcoor (outsp(0), ycoor (outsp0)) and (xcoor (outsp1), &
!       ycoor (outsp1)), the other with endpoints (xcoor (insp0), ycoor (insp0))
!       and (xcoor (insp1), ycoor (insp1)).
!       If the linesegments are degenerate, but *do* intersect, that is, &
!       they are almost collinear or have been reduced to a single point, &
!       the result will the right point of both intervals if they coincide
!       and otherwise  xcoor (insp1), ycoor (insp1), since this will be
!       a common point to both segments. If the two segments do not
!       intersect  an error is signalled.
!       The result is put in (xc, yc).
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer insp0, insp1, outsp0, outsp1
      double precision ycoor(*), xcoor (*), xc, yc

!     insp0   i    indices of endpoints of the other segment
!     insp1   i    indices of endpoints of the other segment
!     outsp0  i    indices of endpoints of one line segment
!     outsp1  i    indices of endpoints of one line segment
!     xc      o    value of x coordinate at intersection point
!     xcoor   i    array of x coordinates in the listpack.
!     yc      o    value of y coordinate at intersection point
!     ycoor   i    array of y coordinates in the listpack.
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx01, dx23, dx02, dy01, dy23, dy02, alfa
      double precision e012, e013, eps
      double precision rctngl, denom, numer
      double precision l01, l23
      parameter (eps =1d-8)

!     alfa       interpolation weight
!     denom      denominator of alfa
!     dx01       differences of x coordinates between points i and j i,j = 0...3
!     dx02       differences of x coordinates between points i and j i,j = 0...3
!     dx23       differences of x coordinates between points i and j i,j = 0...3
!     dy01       differences of y coordinates between points i and j i,j = 0...3
!     dy02       differences of y coordinates between points i and j i,j = 0...3
!     dy23       differences of y coordinates between points i and j i,j = 0...3
!     e012       d01 x d02
!     e013       d01 x d03
!     eps        local accuracy
!     l01
!     l23
!     numer      numerator of alfa
!     rctngl     measure of degeneracy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERREAL  Put real in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1269   pltjis called with non intersecting segments
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjis')
      dx01 = xcoor (outsp0) - xcoor (outsp1)
      dy01 = ycoor (outsp0) - ycoor (outsp1)
      dx23 = xcoor (insp0) - xcoor (insp1)
      dy23 = ycoor (insp0) - ycoor (insp1)
      dx02 = xcoor (outsp0) - xcoor (insp0)
      dy02 = ycoor (outsp0) - ycoor (insp0)
      denom = -dx01 * dy23 + dy01 * dx23
      l01 = sqrt (dx01**2 + dy01**2)
      l23 = sqrt (dx23**2 + dy23**2)
      rctngl = l01*l23

!     --- we test exception conditions in the following order
!         1. No intersection (fatal)
!         2. Singularity of equation set (not fatal)

      e012 = dx01*dy02 - dy01*dx02
      e013 = e012 + dx01*dy23 - dy01*dx23

      if (e012*e013.gt.eps) then

!     --- Uh oh, no intersection

          call erreal (e012*e013, 1)
          call errsub (1269, 0, 1, 0)
          goto 99999
      end if

      if (abs(denom).le.rctngl*eps ) then

!     --- Uh oh, singular system
!         Since we know the lines intersect however, they must be collinear
!         We take the farthest possible intersection point, since that is best
!         for the integrity of the data.

          if (xcoor (insp1).lt.xcoor (outsp1)) then
             xc = xcoor (insp1)
             yc = ycoor (insp1)
          else
             xc = xcoor (outsp1)
             yc = ycoor (outsp1)
          end if
      else

!     --- regular system

        numer = -dx02*dy23+dy02*dx23
        alfa = numer/denom
        xc = (1d0-alfa) * xcoor (outsp0) + alfa * xcoor (outsp1)
        yc = (1d0-alfa) * ycoor (outsp0) + alfa * ycoor (outsp1)
      end if

99999 call erclos ('pltjis')
      end
