      subroutine pltjln ( xcint, ymin, dy, fcint, ackseg, stadr, &
                          xcoor, zcoor, nextra, kextra, pltlst, inc, &
                          mode )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 20-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjln takes all segments from the plotting list, takes interpolated
!       function values, transforms them to the projection plane and puts them
!       in a list as required by the list pack.
!       Extra points are added to correctly implement the boundary.
!       Information concerning this is stored in the arrays kextra and nextra, &
!       which have to be produced by the subroutine anlsnc.
!       In block mode one or two extra points per segment are added: the
!       groundpoints. Start address of the list is in stadr. xcoor and zcoor are
!       arrays in which the coordinates are stored.
!       If inc = -1 the line is stored backwards.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ackseg(0:*), stadr, mode, nextra(2,*), kextra(2,2,*), &
              pltlst(0:*)
      double precision xcint(*), fcint(*), xcoor(*), zcoor(*), dy, ymin

!     ackseg  i    array with accumulated number of points in ith segment.
!     dy      i    increment for y
!     fcint   i    array with interpolated z coordinates
!     inc     i    +1 or -1 according to whether the loading is in
!                  normal (+1) or reversed (-1) order
!     kextra  i    array containing information to add boundary
!                  points ( see analys)
!     mode    i    0 = BLOCK mode, 1 = SHEET mode
!     nextra  i    array concerning information to add boundary
!                  points (see analys)
!     pltlst  i    list with segments to be plotted and their line
!                  values (odd & even positions respectively). pltlst (0)
!                  contains the length of the list.
!     stadr   o    startaddress of list
!     xcint   i    array with interpolated x coordinates
!     xcoor   o    array with x coordinates (transformed to
!                  projection plane)
!     ymin    i    minimum value of y coordinate
!     zcoor   o    array with z coordinates (transformed to
!                   projection plane)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer stadr0, seghpt, iseg, inc, seginx, corinx, atr, atl, &
              statmp, cell0, modsgl, modsgr, celcnt, cornxh, ilines, &
              left, right
      double precision y, xproj, zproj

!     atl     ?
!     atr     ?
!     celcnt  ?
!     cell0   cell number
!     corinx  ?
!     cornxh  ?
!     ilines  Counting variable
!     iseg    counter to number of segments
!     left    ?
!     modsgl  ?
!     modsgr  ?
!     right   ?
!     seghpt  first cell (header) of a segment
!     seginx  ?
!     stadr0  start addres of the new list that is created for each segment
!     statmp  ?
!     xproj   ?
!     y       y-coordinate
!     zproj   ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer pltjcd

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJCD  To find cellpointer to next cell
!     PLTJMS  to append a created segment to an existing list
!     PLTJNE  to create a new list
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       While {there is another segment do}
!         Create new list with as many cells as there are points in this segment
!          taking care for the extra points in block mode
!         Fill x & z coordinates
!         Append this to the existing list.
!       End While
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjln')
      left = 1
      right = 2

!     --- Create empty list at stadr

      stadr = nil
      seghpt = nil

!     --- Main algorithm
!         for all segments in the plot list we load the points in
!         plottable lists.

      do 200 iseg = 1, pltlst (0), 2
         seginx = pltlst (iseg)
         ilines = pltlst (iseg + 1)
         modsgl = mod (nextra (left, seginx), 10)
         modsgr = mod (nextra (right, seginx) , 10)
         if (ilines.gt.0) then

!        --- generic case. no forked ending of segment or high part of a forked
!            segment. we count the number of extra cells needed.
!            (Remember that plnxlr conventionally makes ilines negative for
!            a forked segment that is encountered first time)

            if  (modsgl.gt.0) then
               if (modsgl.eq.1) then
                  atl = kextra (left, 2, seginx)
               else if (modsgl.eq.2) then
                  atl = 0
               else
                  atl = 1
               end if

!              --- at denotes high (1), neutral (0) or low (-1) extra point
!                  a high point is grounded in block mode

               if ((mode.eq.0) .and. (atl.eq.1)) then
                  celcnt = 2
               else
                  celcnt = 1
               end if
            else
               celcnt = 0
            end if
            if (modsgr.gt.0) then
               if (modsgr.eq.1) then
                  atr = kextra (right, 2, seginx)
               else if (modsgr.eq.2) then
                  atr = 0
               else
                  atr = 1
               end if

!              --- at denotes high (1), neutral (0) or low (-1) extra point
!                  a high point is grounded in block mode

               if ((mode.eq.0) .and. (atr.eq.1)) then
                  celcnt = celcnt + 2
               else
                  celcnt = celcnt + 1
               end if
            end if
            celcnt = celcnt + abs (ackseg (seginx) - ackseg (seginx - &
               inc))
            call pltjne (stadr0, celcnt)
            if ( ierror/=0 ) goto 99999
            cell0 = stadr0
            if (modsgl.gt.0) then

!           --- here comes an extra point

               corinx = kextra (left, 1, seginx)
               y = ymin + (ilines - 1 + atl * inc) * dy
               xproj = xcint (corinx) * (cth - scfi * sth) + &
                  y * (sth + scfi * cth)
               zproj = -ssfi * sth * xcint (corinx) + y * ssfi * cth
               if ((mode.eq.0) .and. (atl.eq.1)) then

!              --- Here comes a groundpoint

                  xcoor (cell0) = xproj
                  zcoor (cell0) = zproj + zscale * fminh
                  cell0 = pltjcd (cell0)
               end if
               xcoor (cell0) = xproj
               zcoor (cell0) = zproj + zscale * fcint (corinx)
               cell0 = pltjcd (cell0)
            end if
            y = ymin + (ilines - 1) * dy
            do 20 corinx = ackseg (seginx - inc) + inc, ackseg &
               (seginx), inc
               xcoor (cell0) = xcint (corinx) * (cth - scfi * sth) + &
                  y * (sth + scfi * cth)
               zcoor (cell0) = zscale * fcint (corinx) - ssfi * sth &
                  * xcint(corinx) + y * ssfi * cth
               cell0 = pltjcd (cell0)
20            continue

!             --- Now for the extra points to the right ...

            if (modsgr.gt.0) then

!           --- here comes an extra point

               corinx = kextra (right, 1, seginx)
               y = ymin + (ilines - 1 + atr * inc) * dy
               xproj = xcint (corinx) * (cth - scfi * sth) + &
                  y * (sth + scfi * cth)
               zproj = -ssfi * sth * xcint (corinx) + y * ssfi * cth
               xcoor (cell0) = xproj
               zcoor (cell0) = zproj + zscale * fcint (corinx)
               cell0 = pltjcd (cell0)
               if ((mode.eq.0) .and. (atr.eq.1)) then

!              --- Here comes a groundpoint

                  xcoor (cell0) = xproj
                  zcoor (cell0) = zproj + zscale * fminh
                  cell0 = pltjcd (cell0)
               end if
            end if

!           --- move this segment to the existing list
!               save stadr0, to place it in seghpt later

            statmp = stadr0
            call pltjms (stadr0, nil, stadr, seghpt)
            seghpt = statmp
         else

!        --- a forked segment of which only the low extra segments have to
!            be plotted. There is never a ground point.
!            First we repair the sign of ilines before accidents happen.

               ilines = -ilines
               modsgl = mod (nextra (left, seginx), 10)
               if ((modsgl.eq.1) .and. (kextra (left, 2, seginx) &
              .lt.1)) then
                  atl = kextra (left, 2, seginx)
                  corinx = kextra (left, 1, seginx)
               else if (modsgl.eq.6) then
                  atl = 0
                  corinx = kextra (left, 2, seginx)
               else if ((modsgl.eq.2) .or. (modsgl.eq.3)) then
                  atl = -1
                  corinx = kextra (left, 2, seginx)
               else
                  goto 100

!                 --- no extra point. break to right side or out

               end if
               call pltjne (stadr0, 2)
               if ( ierror/=0 ) goto 99999
               cell0 = stadr0
               y = ymin + (ilines - 1 + atl * inc) * dy
               xcoor (cell0) = xcint (corinx) * (cth - scfi * sth) + &
                  y * (sth + scfi * cth)
               zcoor (cell0) = zscale * fcint (corinx) - ssfi * sth * &
                  xcint (corinx) + y * ssfi * cth
               cell0 = pltjcd (cell0)
               y = ymin + (ilines - 1) * dy
               corinx = ackseg (seginx - inc) + inc
               xcoor (cell0) = xcint (corinx) * (cth - scfi * sth) + &
                  y * (sth + scfi * cth)
               zcoor (cell0) = zscale * fcint (corinx) - ssfi * sth * &
                  xcint (corinx) + y * ssfi * cth

!              --- move this segment to the existing list
!                  save stadr0, to place it in seghpt later

            statmp = stadr0
            call pltjms (stadr0, nil, stadr, seghpt)
            seghpt = statmp

!           --- now for the right side

100         continue
            modsgr = mod (nextra (right, seginx), 10)
            if ((modsgr.eq.1) .and. (kextra (right, 2, seginx) &
           .lt.1)) then
               atr = kextra (right, 2, seginx)
               corinx = kextra (right, 1, seginx)
            else if (modsgr.eq.6) then
               atr = 0
               corinx = kextra (right, 2, seginx)
            else if ((modsgr.eq.2) .or. (modsgr.eq.3)) then
               atr = -1
               corinx = kextra (right, 2, seginx)
            else
               goto 200

!              --- no extra point. break out

            end if
            call pltjne (stadr0, 2)
            if ( ierror/=0 ) goto 99999
            cell0 = stadr0
            cornxh = corinx
            corinx = ackseg (seginx)
            y = ymin + (ilines - 1) * dy
            xcoor (cell0) = xcint (corinx) * (cth - scfi * sth) + &
               y * (sth + scfi * cth)
            zcoor (cell0) = zscale * fcint (corinx) - ssfi * sth * &
               xcint (corinx) + y * ssfi * cth
            corinx = cornxh
            cell0 = pltjcd (cell0)
            y = ymin + (ilines - 1 + atr * inc) * dy
            xcoor (cell0) = xcint (corinx) * (cth - scfi * sth) + &
               y * (sth + scfi * cth)
            zcoor (cell0) = zscale * fcint (corinx) - ssfi * sth * &
               xcint (corinx) + y * ssfi * cth

!           --- move this segment to the existing list
!               save stadr0, to place it in seghpt later

            statmp = stadr0
            call pltjms (stadr0, nil, stadr, seghpt)
            seghpt = statmp
         end if
200   continue
99999 continue
      call erclos ('pltjln')
      end
