      subroutine pltjpn ( insp0, insp1, outsp0, outsp1, xcoor, ycoor, &
                          point, chckmd )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 20-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjpn checks the visibility of insp1.
!       insp and outsp are mnemonics for inside point and outside point
!       respectively.
!       generically xcoor (outsp0) < xcoor (insp1) < xcoor (outsp1).
!       There are several possible cases
!       1. outsp0 = nil, denoting the start of a segment.
!          insp1 is visible.
!       2. outsp0, outsp1 <> nil, but insp0 = nil. If insp1 is visible an
!          intersection point has to be added to outs.
!       3. outsp0, outsp1 <> nil, insp0 <> nil. The generic case. If
!          visibility changes between insp0 and insp1 an intersection point has
!          to be added to both segments.
!       4. outsp1 = nil, insp1, insp0 <> nil, the end of a segment.
!          xcoor (insp0) < xcoor(outsp0) <xcoor (insp1). If outsp0 is
!          visible an intersection point must be added.
!       5. insp0 = nil, outsp1 = nil, a degenerate case. insp1 is
!          visible.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer insp0, insp1, outsp0, outsp1, point(*), chckmd
      double precision xcoor(*), ycoor(*)

!     chckmd  i    high (=0) : check against high sky
!                  low (=1) : check against low sky
!     insp0   i    index of the inside points
!     insp1   i    index of the inside points
!     outsp0  i    index of the outside points
!     outsp1  i    index of the outside points
!     point   o    array containing the visibility property of the cells
!     xcoor  i/o   array of x coordinates in the listpack.
!     ycoor  i/o   array of y coordinates in the listpack.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nwins0, nwins1, nwout0, nwout1, high
      double precision xc1, yc1, yint

!     high    ?
!     nwins0  index of appended cells to segment to which insp0 belongs
!     nwins1  index of appended cells to segment to which insp1 belongs
!     nwout0  index of appended cells to segment to which outsp0 belongs
!     nwout1  index of appended cells to segment to which outsp1 belongs
!     xc1
!     yc1      y-coordinate
!     yint    ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer pltjcd

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJAP  to append the tail of n-1 cells after the header.
!     PLTJCD  To find cellpointer to next cell
!     PLTJIP  to interpolate between the outside points
!     PLTJIS  to calculate an intersection point if any
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjpn')
      high = 0
      if (outsp0.eq.nil) then

!     --- Case 1: There is no outside point in this segment to the left
!         of insp1.  Hence insp1 is visible AND SO IS INSP0. So after marking
!         point (insp1) visible we may leave.

        point (insp1) = visbl
        go to 1000

!       --- 1000 = End-of-Routine

      end if

!     --- Case 2: There is no outside point in  this segment to the right of
!         insp1. In this case insp0 < outsp0 < insp1. If outsp0 is visible we
!         have
!         to insert an intersection point. Otherwise we just do nothing.

      if (outsp1.eq.nil) then
         if (insp0.eq.nil) then
         goto 1000

!        --- 1000 = EoR

         else if (point (outsp0).eq.visbl) then
         call pltjip (xcoor, ycoor, insp0, insp1, outsp0, yint)
         call pltjap (insp0, nwins1, 2)
         if ( ierror/=0 ) goto 99999
         nwins0 = pltjcd (insp0)

!        --- index of first appended cell

         xcoor (nwins0) = xcoor (outsp0)
         ycoor (nwins0) = yint
         point (nwins0) = invsbl
         xcoor (nwins1) = xcoor (outsp0)
         ycoor (nwins1) = yint
         point (nwins1) = visbl

!        --- move pointer

        insp0 = nwins1
         end if
         goto 1000

!        --- 1000 = EoR

      end if

!     --- Case 3: insp1 falls between two outside points. We interpolate
!         and compare.

      call pltjip (xcoor, ycoor, outsp0, outsp1, insp1, yint)
      if (ycoor (insp1).ge.yint .eqv. chckmd.eq.high) then
         point (insp1) = visbl
      else
         point (insp1) = invsbl
      end if

!     --- We are ready if
!          1. insp1 is the first point of the segment (insp0 = nil)
!             AND insp1 is invisible. (this will happen 99% of the time
!             in BLOCK mode, but not necessarily in SHEET mode.
!          2. insp1 has same visibility as insp0

      if (insp0.eq.nil) then
         if (point (insp1).eq.visbl) then

!        --- We add an intersection point to the other segment at xcoor
!            (insp1) with function value yint.

            call pltjap (outsp0, nwout1, 2)
            if ( ierror/=0 ) goto 99999
            nwout0 = pltjcd (outsp0)
            xcoor (nwout0) = xcoor (insp1)
            xcoor (nwout1) = xcoor (insp1)
            ycoor (nwout0) = yint
            ycoor (nwout1) = yint
            point (nwout0) = visbl
            point (nwout1) = invsbl

!           --- move pointer

            outsp0 = nwout1
         end if
         goto 1000

!        --- 1000 = EoR

      end if
      if (point (insp1).eq.point (insp0)) then
         goto 1000

!        --- 1000 = EoR

      end if

!     --- We are left with the case that there is an intersection point.
!         we calculate the x- and y-coordinates of the intersection and
!         append TWO cells to each segment, with opposite visibility.
!         Finally we move the index of outsp0 to the last of the appended
!         cells, since this is now the true predecessor in the remaining
!         visibility tests. (In principle we should also move the index of
!         insp0, but this is not necessary, since in the sequel insp0 is not
!         inspected. In a later version this may be necessary however.

      call pltjis (xcoor, ycoor, outsp0, outsp1, insp0, insp1, xc1, yc1)
      if ( ierror/=0 ) goto 99999
      call pltjap (insp0, nwins1, 2)
      if ( ierror/=0 ) goto 99999
      nwins0 = pltjcd (insp0)

!     --- index of first appended cell

      xcoor (nwins0) = xc1
      ycoor (nwins0) = yc1
      point (nwins0) = point (insp0)
      xcoor (nwins1) = xc1
      ycoor (nwins1) = yc1
      point (nwins1) = point (insp1)
      call pltjap (outsp0, nwout1, 2)
      if ( ierror/=0 ) goto 99999
      nwout0 = pltjcd (outsp0)

!     --- index of first appended cell

      xcoor (nwout0) = xc1
      ycoor (nwout0) = yc1
      point (nwout0) = point (outsp0)
      xcoor (nwout1) = xc1
      ycoor (nwout1) = yc1
      point (nwout1) = point (insp0)
      outsp0 = nwout1
1000  continue
99999 continue
      call erclos ('pltjpn')
      end
