      subroutine pltr10 ( icase, kmeshc, coor, velo, x, y, z, nelem, t, &
                          tend, itel, user, iuser, ihulp, &
                          ielold, nsteps, maxsteps, partout, icallfn )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 06-04-1996 New parameter list
!        version  2.2  date 02-04-1993 original element array added
!        version  2.1  date 09-02-1993 new calls PLTR routines
!        version  2.0  date 23-11-1992 redesign
!        version  1.1  date 07-10-1992 adjustment for porosity
!        version  1.0  date 31-05-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to compute the track of a particle in a 3-D
!    velocity field.
!    the porosity for each element group may be specified utilizing
!    arrays user, iuser and ihulp.
! **********************************************************************
!
!                       KEYWORDS
!
!     particle_tracking
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, z, coor(3,*), velo(3,*), t, tend, user(*), &
                       partout(4,*)
      integer nelem, kmeshc(4,*), itel(*), ielold(*), nsteps, maxsteps, &
              icase, iuser(*), icallfn, ihulp(*)

!     coor       i    array containing the coordinates of the
!                     nodal points sequentially .
!     icallfn    i    Indicates if for each position in the particle trace
!                     a user subroutine FNPARTIC must be called (1) or not (0)
!     icase      i    case indicator : 1 and 3 - no porosity
!                                      2 and 4 -    porosity
!                                      1 and 2 - velocity on elements
!                                      3 and 4 - velocity in nodes
!     ielold     i    array in which is stored from which element each
!                     tetrahedron is derived
!     ihulp      o    helparray to store in which element group
!                     each tetrahedron is situated originally
!     itel       i    array to count how many times an element
!                     appears in the track of one particle
!     iuser      i    helparray in which integer variables are stored
!     kmeshc     i    array containing the nodal points of the
!                     elements sequentially
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     nelem      i    number of elements
!     nsteps    i/o   Contains the number of positions stored in partout
!     partout   i/o   Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-4,i) contains x_i, y_i and z_i
!     t         i/o   at input starting time , at output final time
!     tend       i    endtime of integration
!     user       i    helparray in which real variables are stored
!     velo       i    velocity array
!     x          i    starting x-position in global coordinates
!                o    end x-position in global coordinates
!     y          i    start y-position in global coordinates
!                o    end y-position in global coordinates
!     z          i    start z-position in global coordinates
!                o    end z-position in global coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, i1, i2, i3, i4, iresul, io1, io2, io3, io4, &
              ital, igroep, istart, itrap, ilne, ielemo, ielemv, &
              istag, ivel, ivelol
      double precision a1, a2, a3, b1, b2, b3, c1, c2, c3, &
                       dx, dy, dref, xl(4), yl(4), zl(4), &
                       u, v, w,  ul(4), vl(4), wl(4), up, vp, wp, &
                       uo, vo, wo, upl(4), vpl(4), wpl(4), &
                       ski, eta, zta, det, porost
      logical repeat, herh

!     a1           - &
!     a2           - helpcoefficients
!     a3           - &
!     b1           - &
!     b2           - &
!     b3           - &
!     c1           - &
!     c2           - &
!     c3           - &
!     det          determinant
!     dref         reference distance for plotting
!     dx           x-distance
!     dy           y-distance
!     eta          local coordinate
!     herh         iteration variable
!     i            loop variable
!     i1           first node number of tetrahedron
!     i2           second node number
!     i3           third node number
!     i4           fourth node number of tetrahedron
!     ielem        element number
!     ielemo       previous element number
!     ielemv       pre-previous element number
!     igroep       group number of tetrahedron considered
!     ilne         interface between two elements indicator
!     io1          (old) first node
!     io2          (old) second node
!     io3          (old) third node
!     io4          (old) fourth node
!     iresul       indicator for point belonging to element or not
!     istag        indicator for stagnation
!     istart       helpvariable for porositydetermination
!     ital         local counter
!     itrap        number of visits to element allowed in one run
!     ivel         element number for velocity
!     ivelol       previous element number for velocity
!     porost       local porosity
!     repeat       while variable
!     ski          local coordinate
!     u            u-velocity component
!     ul           local u-velocity array
!     uo           old value of up
!     up           first component of local element velocity
!     upl          local up-velocity array for the four nodes
!     v            v-velocity component
!     vl           local v-velocity array
!     vo           old value of vp
!     vp           second component of local element velocity
!     vpl          local vp-velocity array
!     w            w-velocity component
!     wl           local w-velocity array
!     wo           old value of wp
!     wp           third component of local element velocity
!     wpl          local wp-velocity array
!     xl           local x-coordinates
!     yl           loval y-coordinates
!     zl           local z-coordinates
!     zta          local coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate name from sequence of names
!     EROPEN    concatenate name to sequence of names
!     ERRINT    Put integer in error message
!     ERRSUB    give error message
!     ERRWAR    give warning
!     FNPARTIC  User function subroutine that is called in eah step of the
!               particle tracer
!     INSTOP    stop SEPRAN execution
!     PLTR11    determine in which element particle is situated
!     PLTR12    compute track for case 2 in tetrahedron
!     PLTR13    compute track in tetrahedron for case 3
!     PLTR14    transform to local coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     937   Porosity is zero
!     953   Particle reached stagnation point, error ?
!    1638   Particle array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine elementnumber of tetrahedron
!     While not outside region of interest and time not too large
!        Compute track in tetrahedron, determine next tetrahedron
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      itrap = 20
      call eropen( 'pltr10' )

!     --- Determine reference length for plotting and printing

      dx = xmax - xmin
      dy = ymax - ymin
      dref = sqrt( dx * dx + dy * dy ) / 2d2

!     --- Set counterarray to zero

      do 50 i = 1 , nelem
          itel( i ) = 0
50    continue
      io1 = 0
      io2 = 0
      io3 = 0
      io4 = 0
      ielem  = 0
      ielemo = 0
      istag  = 0
      repeat = .true.
100   if ( repeat ) then

!     --- determine elementnumber of tetrahedron

         ielemv = ielemo
         ielemo = ielem
         ielem = 0
         do 150 i = 1 , nelem
            i1 = kmeshc(1,i)
            i2 = kmeshc(2,i)
            i3 = kmeshc(3,i)
            i4 = kmeshc(4,i)

!           --- Check whether element is neighbour element

            ital  = 0
            if ( i1.eq.io1 .or. i1.eq.io2 .or. i1.eq.io3 &
                           .or. i1.eq.io4 ) ital=ital+1
            if ( i2.eq.io1 .or. i2.eq.io2 .or. i2.eq.io3 &
                           .or. i2.eq.io4 ) ital=ital+1
            if ( i3.eq.io1 .or. i3.eq.io2 .or. i3.eq.io3 &
                           .or. i3.eq.io4 ) ital=ital+1
            if ( i4.eq.io1 .or. i4.eq.io2 .or. i4.eq.io3 &
                           .or. i4.eq.io4 ) ital=ital+1
            if ( ital.ne.3 ) goto 150
            xl(1) = coor(1,i1)
            xl(2) = coor(1,i2)
            xl(3) = coor(1,i3)
            xl(4) = coor(1,i4)
            yl(1) = coor(2,i1)
            yl(2) = coor(2,i2)
            yl(3) = coor(2,i3)
            yl(4) = coor(2,i4)
            zl(1) = coor(3,i1)
            zl(2) = coor(3,i2)
            zl(3) = coor(3,i3)
            zl(4) = coor(3,i4)
            iresul=0
            call pltr11( x, y, z, iresul, xl, yl, zl, &
                         ski, eta, zta, det )
            if ( iresul.eq.1 ) then
               ielem  = i
               io1    = i1
               io2    = i2
               io3    = i3
               io4    = i4

!              --- Element found, jump to next section

               goto 300
            end if
150      continue

!        --- Element not yet found, &
!            Second action, run through all elements

         ielem = 0
         do 200 i = 1 , nelem
            i1 = kmeshc(1,i)
            i2 = kmeshc(2,i)
            i3 = kmeshc(3,i)
            i4 = kmeshc(4,i)
            xl(1) = coor(1,i1)
            xl(2) = coor(1,i2)
            xl(3) = coor(1,i3)
            xl(4) = coor(1,i4)
            yl(1) = coor(2,i1)
            yl(2) = coor(2,i2)
            yl(3) = coor(2,i3)
            yl(4) = coor(2,i4)
            zl(1) = coor(3,i1)
            zl(2) = coor(3,i2)
            zl(3) = coor(3,i3)
            zl(4) = coor(3,i4)
            iresul = 0
            call pltr11( x, y, z, iresul, xl, yl, zl, &
                         ski, eta, zta, det )
            if ( iresul.eq.1 ) then
               ielem   = i
               io1     = i1
               io2     = i2
               io3     = i3
               io4     = i4

!              --- Element found, jump to next section

               goto 300
            end if
200      continue

!        --- No element found, point outside, track ready

         repeat = .false.
         goto 100
300      continue

!        --- Set face indicator to zero

         ilne = 0

!        --- Check whether particle has been in element before for case
!            that velocities are constant over one element, in order to
!            circumvent the situation that a particle will travel zig-zag
!            along the common interface

         if ( ielem.eq.ielemv .and. icase.lt.3 ) then
            ilne = 0

!           --- Element has been visited before, check whether there is a
!               common face between the last two elements

            if ( i1.eq.io1 .or. i1.eq.io2 .or. i1.eq.io3 .or. &
                 i1.eq.io4 ) then
                 ilne = ilne + 1
            end if
            if ( i2.eq.io1 .or. i2.eq.io2 .or. i2.eq.io3 .or. &
                 i2.eq.io4 ) then
                 ilne = ilne + 2
            end if
            if ( i3.eq.io1 .or. i3.eq.io2 .or. i3.eq.io3 .or. &
                 i3.eq.io4 ) then
                 ilne = ilne + 4
            end if
            if ( i4.eq.io1 .or. i4.eq.io2 .or. i4.eq.io3 .or. &
                 i4.eq.io4 ) then
                 ilne = ilne + 8
            end if
         end if

!        --- Check whether particle is trapped in element

         itel( ielem ) = itel( ielem ) + 1
         if ( itel(ielem).gt.itrap .or. istag.eq.1 ) then

!        --- Particle trapped in element, stagnation point

              call errwar( 953, 0, 0, 0 )
              repeat = .false.
              goto 100
         end if

!        --- Element found, determine elementgroup for this triangle

         igroep = 0
         herh = .true.
400      if ( herh ) then

!        --- Find element group number

             if ( ielem.gt.ihulp(igroep+1) ) then
                  igroep = igroep + 1
             else
                  herh   = .false.
             end if
             goto 400
         end if
         if ( icase.eq.1 .or. icase.eq.3 ) then

!        --- No porosity

            porost = 1d0
         else if ( icase.eq.2 .or. icase.eq.4 ) then

!        --- Determine porosity for this element

            istart = iuser( 5+igroep )
            if ( iuser(istart).eq.0 ) then

!           --- Error, zero porosity

               call errsub ( 937, 0, 0, 0 )
               call instop
            else if ( iuser(istart).lt.-5 ) then

!           --- Porosity is constant

               porost = user(-iuser(istart))
            else
               write(irefwr,*) ' Porosity not constant in element '
               write(irefwr,*) ' Not yet supported posibility in '
               write(irefwr,*) ' tracking PLTRAC/PLTR10 '
               write(irefwr,*) ' Execution of tracking stopped '
               goto 1000
            end if
         end if

!        --- Determine global velocities

         if ( icase.eq.1 .or. icase.eq.2 ) then

!        --- Velocity has been given per element

            ivel = ielold( ielem )
            u  = velo( 1,ivel ) / porost
            v  = velo( 2,ivel ) / porost
            w  = velo( 3,ivel ) / porost
         else if ( icase.eq.3 .or. icase.eq.4 ) then

!        --- Velocity has been given per point

            ul(1) = velo( 1,i1 ) / porost
            ul(2) = velo( 1,i2 ) / porost
            ul(3) = velo( 1,i3 ) / porost
            ul(4) = velo( 1,i4 ) / porost
            vl(1) = velo( 2,i1 ) / porost
            vl(2) = velo( 2,i2 ) / porost
            vl(3) = velo( 2,i3 ) / porost
            vl(4) = velo( 2,i4 ) / porost
            wl(1) = velo( 3,i1 ) / porost
            wl(2) = velo( 3,i2 ) / porost
            wl(3) = velo( 3,i3 ) / porost
            wl(4) = velo( 3,i4 ) / porost
         end if

!        --- Determine track

         a1 = xl(2) - xl(1)
         a2 = yl(2) - yl(1)
         a3 = zl(2) - zl(1)
         b1 = xl(3) - xl(1)
         b2 = yl(3) - yl(1)
         b3 = zl(3) - zl(1)
         c1 = xl(4) - xl(1)
         c2 = yl(4) - yl(1)
         c3 = zl(4) - zl(1)
         if ( icase.eq.1 .or. icase.eq.2 ) then

!        --- Transform to local velocities

            call pltr14( u , v, w, b1,b2,b3, c1,c2,c3, up )
            call pltr14( a1,a2,a3, u , v, w, c1,c2,c3, vp )
            call pltr14( a1,a2,a3, b1,b2,b3, u , v, w, wp )
            up  =  up / det
            vp  =  vp / det
            wp  =  wp / det
            if ( ilne.ne.0 ) then

!           --- Particle should travel in interface

               ivelol = ielold( ielemo )
               u = velo( 1,ivelol ) / porost
               v = velo( 2,ivelol ) / porost
               w = velo( 3,ivelol ) / porost
               call pltr14( u, v, w, b1,b2,b3,c1,c2,c3, uo )
               call pltr14( a1,a2,a3,u ,v ,w ,c1,c2,c3, vo )
               call pltr14( a1,a2,a3,b1,b2,b3,u ,v ,w , wo )
               uo = uo / det
               vo = vo / det
               wo = wo / det

!              --- Check in which interface particle should travel

               if ( ilne.eq.7 ) then

!              --- In face zeta = 0

                  up = ( up + uo ) / 2d0
                  vp = ( vp + vo ) / 2d0
                  wp =   0d0
               else if ( ilne.eq.11 ) then

!              --- In face eta = 0

                  up = ( up + uo ) / 2d0
                  vp =   0d0
                  wp = ( wp + wo ) / 2d0
               else if ( ilne.eq.13 ) then

!              --- In face ski = 0

                  up  =    0d0
                  vp  =  ( vp + vo ) / 2d0
                  wp  =  ( wp + wo ) / 2d0
               else if ( ilne.eq.14 ) then

!              --- In face ski+eta+zeta = 1

                  uo  =  ( up + uo ) / 2d0
                  vo  =  ( vp + vo ) / 2d0
                  wo  =  ( wp + wo ) / 2d0
                  up  =  (   2 * uo - vo - wo ) / 3d0
                  vp  =  ( - uo + 2 * vo - wo ) / 3d0
                  wp  =  ( - uo - vo + 2 * wo ) / 3d0
               end if
            end if

!           --- determine track of particle in this element

            istag = 0
            call pltr13 ( up, vp, wp, ski, eta, zta, t, tend, x, y, z, &
                          xl, yl, zl, dref, nsteps, partout, &
                          maxsteps, icallfn, repeat )
         else if ( icase.eq.3 .or. icase.eq.4 ) then

!        --- Compute local velocities

            call pltr14( ul(1),vl(1),wl(1), b1,b2,b3, c1,c2,c3, upl(1) )
            call pltr14( a1,a2,a3, ul(1),vl(1),wl(1), c1,c2,c3, vpl(1) )
            call pltr14( a1,a2,a3, b1,b2,b3, ul(1),vl(1),wl(1), wpl(1) )
            upl(1)  =  upl(1) / det
            vpl(1)  =  vpl(1) / det
            wpl(1)  =  wpl(1) / det
            call pltr14( ul(2),vl(2),wl(2), b1,b2,b3, c1,c2,c3, upl(2) )
            call pltr14( a1,a2,a3, ul(2),vl(2),wl(2), c1,c2,c3, vpl(2) )
            call pltr14( a1,a2,a3, b1,b2,b3, ul(2),vl(2),wl(2), wpl(2) )
            upl(2)  =  upl(2) / det
            vpl(2)  =  vpl(2) / det
            wpl(2)  =  wpl(2) / det
            call pltr14( ul(3),vl(3),wl(3), b1,b2,b3, c1,c2,c3, upl(3) )
            call pltr14( a1,a2,a3, ul(3),vl(3),wl(3), c1,c2,c3, vpl(3) )
            call pltr14( a1,a2,a3, b1,b2,b3, ul(3),vl(3),wl(3), wpl(3) )
            upl(3)  =  upl(3) / det
            vpl(3)  =  vpl(3) / det
            wpl(3)  =  wpl(3) / det
            call pltr14( ul(4),vl(4),wl(4), b1,b2,b3, c1,c2,c3, upl(4) )
            call pltr14( a1,a2,a3, ul(4),vl(4),wl(4), c1,c2,c3, vpl(4) )
            call pltr14( a1,a2,a3, b1,b2,b3, ul(4),vl(4),wl(4), wpl(4) )
            upl(4)  =  upl(4) / det
            vpl(4)  =  vpl(4) / det
            wpl(4)  =  wpl(4) / det

!           --- determine track of particle in this element

            call pltr12 ( upl, vpl, wpl, ski, eta, zta, t, tend, &
                          x, y, z, xl, yl, zl, dref, istag, &
                          nsteps, partout, maxsteps, icallfn, repeat )
         end if
         if ( t.ge.tend ) then
            nsteps = nsteps + 1
            if ( nsteps.gt.maxsteps ) then
               call errint ( nsteps, 1 )
               call errsub ( 1638, 1, 0, 0 )
               repeat = .false.
               go to 100
            end if
            partout(1,nsteps) = t
            partout(2,nsteps) = x
            partout(3,nsteps) = y
            partout(4,nsteps) = z
            if ( icallfn.gt.0 ) &
                 call fnpartic ( t, x, y, z, .false., repeat )
            repeat = .false.
         end if
         goto 100
      end if

!     --- Return to pltrac

1000  call erclos( 'pltr10' )
      end
