      subroutine pltr12 ( up, vp, wp, ski,eta,zta, t,tend, x,y,z, &
                          xl, yl, zl, dref, istag, nsteps, &
                          partout, maxsteps, icallfn, repeat )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 06-04-1996 Extra parameters
!        version  2.0  date 10-02-1993 new criterium for step size
!        version  1.1  date 18-12-1991 correction for time writing
!        version  1.0  date 31-05-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine for particle-tracking in the unit-triangle
!     velocity has been given in the three corner points
! **********************************************************************
!
!                       KEYWORDS
!
!     particle_tracking
!     3d
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision up(4), vp(4), wp(4), ski, eta, zta, t, tend, &
                       x, y, z, xl(4), yl(4), zl(4), dref, partout(4,*)
      integer istag, nsteps, maxsteps, icallfn
      logical repeat

!     dref       i    reference distance for plotting
!     eta        i    local coordinate
!     icallfn    i    Indicates if for each position in the particle trace
!                     a user subroutine FNPARTIC must be called (1) or not (0)
!     istag      o    indicator whether track ended in element (stagnation)
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     nsteps    i/o   Contains the number of positions stored in partout
!     partout   i/o   Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-3,i) contains x_i and y_i
!     repeat    i/o   If true the process has not yet been finished
!     ski        i    local coordinate
!     t         i/o   actual time
!     tend       i    endtime
!     up         i    x-velocities in four nodes of tetrahedron considered
!     vp         i    y-velocities in four nodes of tetrahedron considered
!     wp         i    z-velocities in four nodes of tetrahedron considered
!     x         i/o   x-coordinate of actual position
!     xl         i    x-coordinates of four nodes
!     y         i/o   y-coordinate of actual position
!     yl         i    y-coordinates of four nodes
!     z         i/o   z co-ordinate of point
!     zl         i    z-coordinates of four nodes
!     zta        i    local coordinate
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision afst, difeps, diff, dx, dy, dz, dt, eps, etadub, &
                       etaold, skidub, skiold, releps, ztadub, &
                       ztaold, term1, term2, uex, uhulp, ustart, vex, &
                       vhulp, vstart, wex, whulp, wstart, xold, yold, &
                       zold, uvw, t0, t1, t2, t3, t4
      integer istap, istep, ja

!     afst    distance
!     difeps  distance difference in two substeps
!     diff    help parameter to compute the ratio of made error and allowed
!     dt      time step
!     dx      difference in x-direction
!     dy      difference in y-direction
!     dz      difference in z-direction
!     eps     local accuracy
!     etadub  second guess of eta
!     etaold  old value of eta
!     istap   counter of steps in element
!     istep   number of failures in one step
!     ja      indicator
!     releps  relative error
!     skidub  second guess of ski
!     skiold  old value of ski
!     t0      t-variable of crossing point
!     t1      t-variable of face ski = 0
!     t2      t-variable of face eta = 0
!     t3      t-variable of face zta = 0
!     t4      t-variable of face ski+eta+zta = 1
!     term1   help variable to store sum
!     term2   help variable to store another sum
!     uex     difference of two computed points in x-direction
!     uhulp   helpvalue for velocity
!     ustart  start velocity value
!     uvw     norm of velocity
!     vex     difference of two computed points in y-direction
!     vhulp   helpvalue for velocity
!     vstart  start velocity value
!     wex     difference of two computed points in z-direction
!     whulp   helpvalue for velocity
!     wstart  start velocity value
!     xold    old x-position particle
!     yold    old y-position particle
!     zold    old z-position particle
!     ztadub  second guess of zta
!     ztaold  old value of zta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate name from sequence of names
!     EROPEN    concatenate name to sequence of names
!     ERRINT    Put integer in error message
!     ERRSUB    give error message
!     ERRWAR    give warning
!     FNPARTIC  User function subroutine that is called in eah step of the
!               particle tracer
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     953        Particle trapped in element, stagnation
!    1638   Particle array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine step size
!     Compute new position using the modified Euler method
!     Check accuracy, determine factor
!     Check whether particle is outside element and if so
!       then compute point where elementboundary has been crossed
!       else repeat whole procedure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltr12' )
      eps    = 1d-4
      difeps = 1d-2
      releps = 10 * epsmac
      xold   = x
      yold   = y
      zold   = z
      istap  = 0
      istag  = 0
100   continue

!     --- determine step size

      istap  =  istap + 1
      ustart =  ( 1d0 - ski - eta - zta ) * up(1) + &
                  ski * up(2) + eta * up(3) + zta * up(4)
      vstart =  ( 1d0 - ski - eta - zta ) * vp(1) + &
                  ski * vp(2) + eta * vp(3) + zta * vp(4)
      wstart =  ( 1d0 - ski - eta - zta ) * wp(1) + &
                  ski * wp(2) + eta * wp(3) + zta * wp(4)
      uvw = sqrt( ustart*ustart + vstart*vstart + wstart*wstart )
      if ( istap.eq.1 ) then

!     --- First step compute step size (conservative approach)

         dt   = 0.08 / uvw
         diff = 1d0
      else

!     --- Compute step size in accordance with last result

         if ( diff.lt.1d-4 ) diff = 1d-4
         if ( diff.gt.1d-1 ) diff = 1d-1
         dt = 0.8 * dt * sqrt( difeps / diff )
      end if
      if ( t+dt.gt.tend ) dt = ( 1d0 + releps ) * ( tend - t )
      istep = 0
      skiold = ski
      etaold = eta
      ztaold = zta
120   continue
      ski  = skiold + dt * ustart
      eta  = etaold + dt * vstart
      zta  = ztaold + dt * wstart

!     --- Compute helpvelocities

      uhulp = ( 1d0 - ski - eta - zta ) * up(1) + &
                ski * up(2) + eta * up(3) + zta * up(4)
      vhulp = ( 1d0 - ski - eta - zta ) * vp(1) + &
                ski * vp(2) + eta * vp(3) + zta * vp(4)
      whulp = ( 1d0 - ski - eta - zta ) * wp(1) + &
                ski * wp(2) + eta * wp(3) + zta * wp(4)
      skidub = skiold + dt * ( ustart + uhulp ) / 2d0
      etadub = etaold + dt * ( vstart + vhulp ) / 2d0
      ztadub = ztaold + dt * ( wstart + whulp ) / 2d0

!     --- Check whether timestep dt is too large

      uex = ski - skidub
      vex = eta - etadub
      wex = zta - ztadub
      diff = sqrt( uex * uex + vex * vex + wex * wex )
      if ( diff.gt.difeps .and. istep.lt.5 ) then

!     --- Difference too large take smaller step and do new calculation

         dt = dt * 0.8 * sqrt( difeps / diff )
         istep = istep + 1
         goto 120
      end if

!     --- New position determined

      ski    = skidub
      eta    = etadub
      zta    = ztadub

!     --- Determine new global position

      x = xl(1) + ( xl(2)-xl(1) ) * ski + &
                  ( xl(3)-xl(1) ) * eta + ( xl(4)-xl(1) ) * zta
      y = yl(1) + ( yl(2)-yl(1) ) * ski + &
                  ( yl(3)-yl(1) ) * eta + ( yl(4)-yl(1) ) * zta
      z = zl(1) + ( zl(2)-zl(1) ) * ski + &
                  ( zl(3)-zl(1) ) * eta + ( zl(4)-zl(1) ) * zta
      dx = x - xold
      dy = y - yold
      dz = z - zold
      afst = sqrt( dx * dx + dy * dy + dz * dz )
      if ( afst.gt.dref ) then
         ja   = 1
         xold = x
         yold = y
         zold = z
      else
         ja   = 0
      end if
      if ( ( -eps.lt.ski .and. 1d0+eps.gt.ski )  .and. &
           ( -eps.lt.eta .and. 1d0+eps.gt.eta )  .and. &
           ( -eps.lt.zta .and. 1d0+eps.gt.zta )  .and. &
           ( -eps.lt.1d0-ski-eta-zta ) )  then

!     --- New time

         t = t + dt
         if ( ja.eq.1 ) then

!        --- New point

            nsteps = nsteps + 1
            if ( nsteps.gt.maxsteps ) then
               call errint ( nsteps, 1 )
               call errsub ( 1638, 1, 0, 0 )
               repeat = .false.
               go to 1000
            end if
            partout(1,nsteps) = t
            partout(2,nsteps) = x
            partout(3,nsteps) = y
            partout(4,nsteps) = z
            if ( icallfn.gt.0 ) &
                 call fnpartic ( t, x, y, z, .false., repeat )
            if ( .not.repeat ) go to 1000
         end if
         if ( istap.gt.100 .or. istep.gt.4 ) then

!        --- Particle trapped in element : Stagnation point

            call errwar ( 953, 0, 0, 0 )
            istag = 1
            go to 1000
         end if
         if ( t.ge.tend ) goto 1000
         goto 100
      end if

!     --- Outside tetrahedron, interpolate to obtain crossing point
!         ( ski,eta,zta is the new point, skinew,etanew,ztanew the old point ! )

      if ( ski.gt.0 ) then
           t1 = 2d0
      else
         if ( abs( skiold - ski ).lt.eps ) then
            t1 = 2d0
         else
            t1 = skiold / ( skiold - ski )
         end if
      end if
      if ( eta.gt.0 ) then
         t2 = 2d0
      else
         if ( abs( etaold - eta ).lt.eps ) then
            t2 = 2d0
         else
            t2 = etaold / ( etaold - eta )
         end if
      end if
      if ( zta.gt.0d0 ) then
         t3 = 2d0
      else
         if ( abs( ztaold - zta ).lt.eps ) then
            t3 = 2d0
         else
            t3 = ztaold / ( ztaold - zta )
         end if
      end if
      if ( ski+eta+zta.lt.1d0 ) then
         t4 = 2d0
      else
         term1 = skiold + etaold + ztaold
         term2 = ski + eta + zta
         if ( abs( term2 - term1 ).lt.eps ) then
            t4 = 2d0
         else
            t4 = ( 1d0 - term1 ) / ( term2 - term1 )
         end if
      end if
      t0 = min( t1, t2, t3, t4 )
      t0 = t0 + 1d-5
      ski = skiold + t0 * ( ski - skiold )
      eta = etaold + t0 * ( eta - etaold )
      zta = ztaold + t0 * ( zta - ztaold )

!     --- Adjust time in accordance with the interpolation

      t = t + t0 * dt

!     --- determine global position

      x = xl(1) + (xl(2)-xl(1))*ski + &
                  (xl(3)-xl(1))*eta + (xl(4)-xl(1))*zta
      y = yl(1) + (yl(2)-yl(1))*ski + &
                  (yl(3)-yl(1))*eta + (yl(4)-yl(1))*zta
      z = zl(1) + (zl(2)-zl(1))*ski + &
                  (zl(3)-zl(1))*eta + (zl(4)-zl(1))*zta
      nsteps = nsteps + 1
      if ( nsteps.gt.maxsteps ) then
         call errint ( nsteps, 1 )
         call errsub ( 1638, 1, 0, 0 )
         repeat = .false.
         go to 1000
      end if
      partout(1,nsteps) = t
      partout(2,nsteps) = x
      partout(3,nsteps) = y
      partout(4,nsteps) = z
      if ( icallfn.gt.0 ) &
           call fnpartic ( t, x, y, z, .false., repeat )
1000  continue
      call erclos( 'pltr12' )
      end
