      subroutine prcontourcomp ( func, value, numbubbles, area, lenarea, &
                                 iplots, rplots )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a set of contours corresponding to one function value
!     and compute the area occupied by each contour for the part
!     larger than this function
!     The mesh must consist of linear triangles only
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     area
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecons
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numbubbles, lenarea, iplots(*)
      double precision func(npoint), value, area(lenarea), rplots(*)

!     area           o    Array containing the areas of all the bubbles
!                         Length: lenarea (must be at least numbubbles)
!     func          i/o   Contains the function values
!                         At output these are scaled and shifted
!     iplots         i    Integer control array in which the plot options
!                         are defined
!                         Positions:
!                         1:  if 0, no plot is made, if >0 a plot is made
!                         2:  icolor    Color for plotting
!     lenarea        i    Declared length of array area
!     numbubbles     o    Number of bubbles found
!     rplots         i    Double precision input array that must be filled as
!                         follows:
!                         1:    plotfm
!                         2:    yfact
!     value          i    The value which defines the threshold for the
!                         bubbles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nelremain, ielem, nodp, ikader, &
              nelmin, nelfirst, icolor, ktimes
      double precision valmax, valmaxinv, eps, areabub
      integer, allocatable :: marknodes(:), markelems(:)
      logical plotpos

!     areabub        Area of one bubble
!     eps            Local accuracy
!     i              Counting variable
!     icolor         Sequence number of colour
!     ielem          Element number
!     ikader         Help variable to store jkader temporarily
!     ktimes         mod(itimes,10)
!     markelems      Element marker
!                    markelems(i) may have one of the following values:
!                    -2:   element i is in the active set
!                    -1:   element is below bubble
!                     0:   element i has not been treated yet
!                    >0:   element i is (partly) in bubble markelems(i)
!     marknodes      Array of size npoint
!                    If func(i)>0 marknodes(i)=1, else if func(i)<0, &
!                    marknodes(i)=-1 else 0
!     nelfirst       First element to be used
!     nelmin         Next free element
!     nelremain      Number of elements remaining
!     nodp           Nodal point sequence number
!     plotpos        If true the bubbles must be plotted
!     valmax         Maximum value of function
!     valmaxinv      Inverse of valmax
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLMORE         User subroutine to plot extra information in the plot
!     PRCONTOURCOMP0 Find all neighboring elements of element corresponding to
!                    the same region satisfying function value>specified value
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     SEPPLOTBN      Plot boundary of region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     471   function array is empty
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     marknodes := 0
!     For all nodes i do
!       if ( value>val ) then
!          marknodes(i) := 1
!       else if ( value<val ) then
!          marknodes(i) := -1
!     markelems := 0
!     nelem_remaining := nelem
!     For all elements i do
!        if ( marknodes(nodp) = -1 for all nodes in element ) then
!        markelems(i) := -1;  nelem_remaining := nelem_remaining-1
!     num_bubbles := 0
!     while nelem_remaining>0 do
!        Find first element marked 0
!        num_bubbles := num_bubbles+1
!        Mark element and all its non-marked neighbors with num_bubbles
!        Compute area
!        Plot element if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcontourcomp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcontourcomp'
         write(irefwr,1) 'npoint, nelem', npoint, nelem
         write(irefwr,2) 'value', value
         call prinrl ( func, npoint, 'func' )
         call prinin1 ( kmeshc, nelem, 3, 'kmeshc' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )

      allocate(marknodes(npoint), markelems(nelem), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( npoint+nelem, 1 )
         call errint ( error, 2 )
         call errchr ( 'marknodes', 1 )
         call errsub ( 2910, 2, 0, 1 )

      end if  ! ( error/= 0 )
      if ( ierror/=0 ) go to 1000

      plotpos = iplots(1)>0
      icolor = iplots(2)

      if ( plotpos ) then

!     --- Start plot if necessary (ad-hoc)

         ktimes = mod(jtimes,10)
         ikader = jkader
         if ( debug ) write(irefwr,1) 'ktimes', ktimes
         if ( ktimes<2 ) then

!        --- ktimes<2, hence open plot

            call pl0003 ( ikader, 1, rplots(1), rplots(2), 0 )

         end if  ! ( ktimes<2 )

!        --- Plot boundary

         call sepplotbn ( 0, 0, .false. )

      end if  ! ( plotpos )

!     --- Find maximum value of function array

      valmax = 0d0  ! maximum function value
      do i = 1, npoint
         valmax = max ( valmax, abs(func(i)) )
      end do  ! i = 1, npoint

      if ( debug ) write(irefwr,2) 'valmax', valmax

      if ( valmax<=0d0 ) then

!     --- Error the function is completely zero

         call errsub ( 471, 0, 0, 0 )
         go to 1000

      end if

!     --- Scale func and val with 1/valmax

      valmaxinv =1d0/valmax  ! 1/valmax
      func = (func-value)*valmaxinv  ! scaled func

!     --- Mark all nodes with value>val with a 1, all nodes with
!         value<val with a -1 and all other nodes with a zero

      marknodes = 0  ! clear array

!     --- For all nodes i do
!           if ( value>val ) then
!              marknodes(i) := 1
!           else if ( value<val ) then
!              marknodes(i) := -1

      eps = 1d-6  ! accuracy
      do i = 1, npoint

         if ( func(i)+eps>0d0 ) then

!        --- func>contval

            marknodes(i) = 1

         else if ( func(i)-eps<0d0 ) then

!        --- func<contval

            marknodes(i) = -1

         end if  ! ( func(i)+eps>0d0 )

      end do  ! i = 1, npoint

      markelems = 0  ! clear array

!     --- nelem_remaining := nelem
!         For all elements i do
!           if ( marknodes(nodp) = -1 for all nodes in element ) then
!              markelems(i) := -1;  nelem_remaining := nelem_remaining-1

      nelremain = nelem
      nelmin = 0
      do ielem = 1, nelem

         do i = 1, 3
            nodp = kmeshc(3*(ielem-1)+i)
            if ( marknodes(nodp)>0 ) go to 100  ! jump if node is positive
         end do  ! i = 1, 3

!        --- Element found

         markelems(ielem) = -1
         nelremain = nelremain-1
         if ( ielem==nelmin+1 ) nelmin = nelmin+1

100   end do   ! ielem = 1, nelem

      if ( debug ) then

!     --- debug information

         call prinin ( marknodes, npoint, 'marknodes' )
         call prinin ( markelems, nelem, 'markelems' )

      end if  ! ( debug )

!     --- num_bubbles := 0

      numbubbles = 0
      if ( nelmin==0 ) nelmin = 1

!     --- while nelem_remaining>0 do

      do while ( nelremain> 0 )

!     --- Find first element marked 0

         if ( debug ) &
            write(irefwr,1) 'nelremain, nelmin, nelem', &
                             nelremain, nelmin, nelem

         do ielem = nelmin, nelem

            if ( markelems(ielem)==0 ) then

!           --- element found

               nelmin = ielem+1
               nelfirst = ielem
               go to 200

            end if  ! ( markelems(ielem)==0 )

         end do  ! ielem = nelmin, nelem
200      if ( debug ) write(irefwr,1) 'nelfirst', nelfirst

!        --- num_bubbles := num_bubbles+1

         numbubbles = numbubbles+1
         call inichkarlen ( numbubbles, lenarea, 'area' )
         if ( ierror/=0 ) go to 1000

!        --- Mark element and all its non-marked neighbors with num_bubbles
!            Compute area
!            Plot element if necessary

         call prcontourcomp0 ( markelems, nelfirst, numbubbles, &
                               nelremain, nelem, kmeshc, coor, &
                               kmesho(2), marknodes, npoint, areabub, &
                               icolor, plotpos, func )
         area(numbubbles) = areabub
         if ( ierror/=0 ) go to 1000

      end do  ! while ( nelremain> 0 )

      if ( plotpos ) then

!     --- Close plot

         if ( ktimes<1 .or. ktimes>2 ) then
            call plmore
            call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
         end if  ! ( ktimes<1 .or. ktimes>2 )

         jkader = ikader

      end if  ! ( plotpos )

1000  deallocate ( marknodes, markelems, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'marknodes', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcontourcomp'

      end if  ! ( debug )
      call erclos ( 'prcontourcomp' )

      end
