      subroutine prcontourcomp0 ( markelems, nelfirst, numbubbles, &
                                  nelremain, nelem, kmeshc, coor, &
                                  kmesho, marknodes, npoint, area, &
                                  icolor, plotpos, func )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all neighboring elements of element corresponding to the
!     same region satisfying function value>specified value
!     Compute area and plot elements if necessary
!     The mesh must consist of linear trangles only
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     area
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, markelems(nelem), nelfirst, numbubbles, nelremain, &
              kmeshc(3,nelem), kmesho(3,nelem), npoint, icolor, &
              marknodes(npoint)
      double precision func(npoint), area, coor(2,*)
      logical plotpos

!     area          i/o   Area of bubble
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     func           i    Contains the scaled and shifted function values
!     icolor         i    Sequence number of colour
!     kmeshc         i    kmesh part c, contains the nodal point numers of the
!                         element
!     kmesho         i    kmesh part o, contains the neighbor elements of the
!                         element
!     markelems     i/o   See subroutine prcontourcomp
!     marknodes     i/o   See subroutine prcontourcomp
!     nelem          i    Number of elements
!     nelfirst       i    First element to be used
!     nelremain     i/o   Number of elements remaining
!     npoint         i    Number of nodal points
!     numbubbles     i    Number of bubbles
!     plotpos        i    If true the bubbles must be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, ielem, icount, numelms
      integer, allocatable :: activeelems(:)
      logical zeroval

!     activeelems    Contains the list of still active elements
!                    These elements are neighbors of the elements in the
!                    bubble
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     node           Node number
!     numelms        Number of elements in activeelems
!     zeroval        If true one of the points has value 0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCONTOURCOMP1 Find all neighboring elements of element corresponding to
!                    the same region satisfying function value>specified value
!                    Special case of 1 node completely in region
!     PRCONTOURCOMP2 Find all neighboring elements of element corresponding to
!                    the same region satisfying function value>specified value
!                    Special case of 2 nodes completely in region
!     PRCONTOURCOMP3 Find all neighboring elements of element corresponding to
!                    the same region satisfying function value>specified value
!                    Special case of 3 nodes completely in region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark element and all its non-marked neighbors with num_bubbles
!     Compute area
!     Plot element if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcontourcomp0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcontourcomp0'
         write(irefwr,1) 'nelem, nelfirst', nelem, nelfirst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      allocate(activeelems(nelem), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( nelem, 1 )
         call errint ( error, 2 )
         call errchr ( 'activeelems', 1 )
         call errsub ( 2910, 2, 0, 1 )

      end if  ! ( error/= 0 )
      if ( ierror/=0 ) go to 1000

!     --- mark element and add to set of active elements

      numelms = 1
      activeelems(1) = nelfirst
      area = 0d0

      do while ( numelms>0 )

         ielem = activeelems(numelms)
         numelms = numelms-1
         nelremain = nelremain-1
         markelems(ielem) = numbubbles
         if ( debug ) &
            write(irefwr,1) 'ielem, numelms, nelremain', &
                             ielem, numelms, nelremain

!        --- Find number of nodes that belong to bubble

         icount = 0
         zeroval = .false.
         do i = 1, 3
            node = kmeshc(i,ielem)
            if ( marknodes(node)>0 ) then

!           --- node in bubble found

               icount = icount+1

            else  if ( marknodes(node)==0 ) then

!           --- node in bubble found

               icount = icount+1
               zeroval = .true.

            end if  ! ( marknodes(node)>0 )

         end do  ! i = 1, 3

         if ( debug ) then
            write(irefwr,1) 'icount', icount
            write(irefwr,*) 'zeroval ', zeroval
         end if  ! ( debug )

         select case ( icount )

         case(1)

!        --- Two nodes belong to bubble

            call prcontourcomp1 ( markelems, area, nelem, &
                                  kmeshc(1,ielem), coor, &
                                  kmesho(1,ielem), plotpos, numelms, &
                                  activeelems, icolor, func, &
                                  marknodes )

         case(2)

!        --- Two nodes belong to bubble

            call prcontourcomp2 ( markelems, area, nelem, &
                                  kmeshc(1,ielem), coor, &
                                  kmesho(1,ielem), plotpos, numelms, &
                                  activeelems, icolor, zeroval, func, &
                                  marknodes )

         case(3)

!        --- All three nodes belong to bubble

            call prcontourcomp3 ( markelems, area, nelem, &
                                  kmeshc(1,ielem), coor, &
                                  kmesho(1,ielem), plotpos, numelms, &
                                  activeelems, icolor )

         end select  ! case ( icount )

      end do  ! while ( numelms>1 )

1000  deallocate ( activeelems, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'activeelems', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      call erclos ( 'prcontourcomp0' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcontourcomp0'

      end if  ! ( debug )

      end
