      subroutine prjest ( coor, kmeshc, nelem, npoint, &
                          nlines, elkey, diael, kgrad, &
                          ndimw, ncint, nibuf, nrbuf )
! ======================================================================
!
!        programmer     Jos van Kan
!        version 1.0    date 21-12-1998
!
!
!
!   copyright (c) 1998-1998 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   prjest is an auxiliary routine to the plotpackage pltj3d. It
!   estimates the sizes of the workarrays needed in that package.
!   It does so, by faking the motions of prjlin to get ndimwork
!   and estimating the size of the workarrays using a conservative
!   strategy as to maximum number of segments (10*nlines)
!   Estimate of real buffer in nwork, of integer buffer in niwork.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nelem, npoint,  kmeshc(3,nelem), nlines, &
              ncint, kgrad(nelem), nibuf, nrbuf, ndimw
      double precision coor(2, npoint), elkey(nelem), &
                       diael(nelem), mxdia

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     diael   o    array containing maximum difference between y -coordinates
!                  per element
!     elkey   o    array containing lowest y-coordinates per element
!     kgrad   o    integer array containing permutation that orders elkey
!                  in ascending order
!     kmeshc  i    nodal points of elements
!     ncint   o    upperbound for total number of intersecion points
!     ndimw   o    dimension of array work needed for prjlin
!     nelem   i    Number of elements
!     nibuf   o    Estimated size of integer buffer
!     nlines  i    number of parallel lines of intersection
!     npoint  i    Number of nodal points
!     nrbuf   o    Estimated size of real buffer
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision slevel, stsize, ymax, ymin
      integer acciout, maxiout, ilines, niout

!     acciout
!     ilines    Counting variable
!     maxiout
!     mxdia     maximal difference in y -coordinate over all elements
!     niout     number of elements intersecting a specific line.
!     slevel    actual value of y-coordinate of interscetion
!     stsize    increment of y-coordinate between lines.
!     ymax      maximal value of y coordinate over all elements.
!               (After rotation)
!     ymin      minimal value of y coordinate over all elements.
!               (After rotation)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prgtmx, prgtmn

!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PRCALK    calculate elkey
!     PRELIN0   which elements intersect this line?
!     PRGTMN    minimum value of y-coordinate in a grid
!     PRGTMX    maximum value of y-coordinate in a grid
!     PRSORT    Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1259   nlines too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!       The coordinate system is first rotated over angdeg degrees.
!       The lowest y coordinate per element is calculated together
!       with the maximal y distance per element. (stored in elkey
!       and diael respectively.)
!       The elements are sorted by this sortkey and the permutation
!       that would order them in ascending order is stored in kgrad.
!       The y-icrement is calculated and starting from ymin + .5 * stsize
!       there are nlines steps in which the following happens per line:
!
!       1. The number of elements that intersect the given
!          line is counted and accumulated.
!          At the end 10*nlines is added to give ncint.
!       2. The maximum number of elements that intersect one single line
!          is updated if necssary. At the end this is multiplied by 2
!          to give nwork.
!       The coordinate system is rotated back to clean up the mess.
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('prjest')

!     --- calculate minmal y-coordinates per element
!         calculate element "diameters"
!         calculate maximal "diameter" over all elements

      call prcalk (elkey, diael, coor, kmeshc, mxdia, nelem)

!     --- sort using elkey as sortkey. Sorting permutation in kgrad.

      call prsort (elkey, kgrad, nelem )

!     --- get minimal and maximal y-values

      ymin = prgtmn (elkey, kgrad, nelem)

!     --- Shift it a bit, to get generic cases

      ymin = ymin + abs (ymin) * 1d-4 * (exp (1d0) - 2*atan (1d0))
      ymax = prgtmx (elkey, kgrad, diael, nelem)

!     --- Shift a bit, but over different interval

      ymax = ymax - abs (ymax) * 1d-4 * (exp (.5d0) + atan (1d0))
      if (nlines.lt.2) then

!     --- ERROR: nlines too small

         call errint (nlines, 1)
         call errsub (1259, 1, 0, 0)

      end if

!     --- calculate stepsize & initialize lineloop

      stsize = (ymax - ymin)/ (nlines - 1)
      slevel = ymin
      maxiout = 0
      acciout = 0

!     --- Now for all lines ...

      do ilines = 1, nlines

!     --- calculate how many elements intersect this line

         call prelin0 ( slevel, elkey, diael, kgrad, mxdia, nelem, &
                        niout)
         if ( niout.gt.maxiout ) then
            maxiout = niout
         end if
         acciout = acciout + niout

!        --- get the new intersection line

         slevel = slevel + stsize

!        --- and start all over again

      end do

!     --- Now add some slack to acciout to allow for several segments per
!         intersection line

      acciout = acciout + 10*nlines

      ndimw = 2*maxiout
      ncint = acciout
      nibuf = nelem + 2*maxiout + 30*nlines
      nrbuf = 2*(nelem+acciout+maxiout)

      call erclos ('prjest')
      end
