      subroutine prjlin ( fval, coor, kmeshc, nelem, npoint, angdeg, &
                          orgx, orgy, nlines, nseg, kseg, xcint, &
                          fcint, ncint, ymin, ymax, elkey, diael, kgrad, &
                          work, iwork, nwork )
! ======================================================================
!
!        programmer     Jos van Kan
!        version  1.3  date 14-10-2016 Newcall to prgath
!        version  1.2  date 30-11-1998 Max number of intersected
!                                      elements per line is now unlimited
!        version  1.1  date 23-01-1995 New call to prsort
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-2016 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!       prjlin is a SEPRAN subroutine to do the following:
!       Given a mesh consisting of triangles with only tree nodal points
!       and function values (one only) in those nodal points, to calcu- &
!       late a set of interpolated values along a set of equidistant spaced
!       lines. The angle of those lines with respect to the x-axis is given
!       by angdeg in degrees. The new origin by (orgx, orgy).
!       The output is organized as follows:
!       any given line may consist of a number of separate SEGMENTS, for
!       example if an L-shaped region is cut like this:
!                  ***\*****
!                  *   \   * &
!                  *    \  * &
!                  *     \ * &
!                  *      \* &
!                  *       \
!                  *       *\
!                  *       * \
!                  *       ***\*****
!                  *           \   * &
!                  *            \  * &
!                  **************\**
!       the line consists of two segments. For every line i this number of
!       segments is stored in nseg (i). The number of interpolated points
!       per segment is stored in kseg (j), in which j is increased by one
!       for each new segment. The coordinate system is rotated over angdeg
!       degrees, so that the y-coordinate is parallel to the intersection lines.
!       The (rotated and shifted) x-coordinates of interpolation are stored in
!       xcint, the corresponding interpolated function values in fcint.
!       the subroutine performs identically to prjlin, but for the
!       workarrays work & iwork of length 2*ncint & npoint respectively
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nelem, npoint,  kmeshc(3,nelem), nlines, nseg(nlines), &
              kseg(5*nlines), ncint, kgrad(nelem), nwork, iwork(nelem)
      double precision fval(npoint), coor(2, npoint), xcint(ncint), &
                       fcint(ncint), ymax, ymin, elkey(nelem), &
                       diael(nelem), orgx, orgy, work(2*nwork), angdeg

!     angdeg  i    angle of rotation in degrees
!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     diael   o    array containing maximum difference between y -coordinates
!                  per element
!     elkey   o    array containing lowest y-coordinates per element
!     fcint   o    array of interpolated function values
!     fval    i    function values in nodes
!     iwork        work array of nelem elements
!     kgrad   o    integer array containing permutation that orders elkey
!                  in ascending order
!     kmeshc  i    nodal points of elements
!     kseg    o    number of points per segment
!     ncint   o    declared length of fcint and xcint (must be
!                  larger than sum kseg (j))
!                               j
!     nelem   i    Number of elements
!     nlines  i    number of parallel lines of intersection
!     npoint  i    Number of nodal points
!     nseg    o    number of segments per line
!     nwork   i    half the declared dimension of work. Should be
!                  greater than or equal to double the number of intersection
!                  points on any line. A safe estimate will be
!                  SHAPE * 2 * SQRT (nelem), where SHAPE is
!                  the ratio of the largest and smallest diameter of the region.
!     orgx    i    coordinates of new origin
!     orgy    i    coordinates of new origin
!     work         work array (double) of 2 * nwork elements
!     xcint   o    array of interpolated x- values
!     ymax    i    maximal value of y coordinate over all elements.
!                  (After rotation)
!     ymin    i    minimal value of y coordinate over all elements.
!                  (After rotation)
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision slevel, stsize, mxdia
      integer mintnd, mksgnd, ilines, i, niout, nchelp

!     i       Counting variable
!     ilines  Counting variable
!     mintnd  starting address to xcint and fcint for a specific line
!     mksgnd  starting address to kseg for a specific line.
!     mxdia   maximal difference in y -coordinate over all elements
!     nchelp  ?
!     niout   number of elements intersecting a specific line.
!     slevel  actual value of y-coordinate of interscetion
!     stsize  increment of y-coordinate between lines.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prgtmx, prgtmn

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     PRCALK  calculate elkey
!     PRELIN  which elements intersect this line?
!     PRGATH  removes doubles and sorts.
!     PRGTMN  get minimal y coordinate
!     PRGTMX  get maximal y coordinate
!     PRROTA  rotate coordinates
!     PRSEGI  what are the intersection points?
!     PRSORT  Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1257   nwork too small
!    1259   nlines too small
!    1260   Insufficient reserved space for work
! **********************************************************************
!
!                       PSEUDO CODE
!
!       The coordinate system is first rotated over angdeg degrees.
!       The lowest y coordinate per element is calculated together
!       with the maximal y distance per element. (stored in elkey
!       and diael respectively.)
!       The elements are sorted by this sortkey and the permutation
!       that would order them in ascending order is stored in kgrad.
!       The y-icrement is calculated and starting from ymin + .5 * stsize
!       there are nlines steps in which the following happens per line:
!
!       1. The elementnumbers of the elements that intersect the given
!          line are calculated and stored in iout .
!       2. The intersection points of those elements with the given
!          line are calculated, together with the interpolated function
!          values. All internal points now will be double, boundary
!          points single.
!       3. These points are sorted and the double points removed. The number
!          of segments per line and points per segment are stored in nseg and
!          kseg respectively.
!
!       The coordinate system is rotated back to clean up the mess.
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('prjlin')

!     --- rotate over angdeg degrees
!         Shift origin

      do 50 i = 1, npoint
         coor ( 1,i ) = coor ( 1,i ) - orgx
 50      coor ( 2,i ) = coor ( 2,i ) - orgy
      call prrota (coor, angdeg, npoint)

!     --- calculate minmal y-coordinates per element
!         calculate element "diameters"
!         calculate maximal "diameter" over all elements

      call prcalk (elkey, diael, coor, kmeshc, mxdia, nelem)

!     --- sort using elkey as sortkey. Sorting permutation in kgrad.

      call prsort (elkey, kgrad, nelem )

!     --- get minimal and maximal y-values

      ymin = prgtmn (elkey, kgrad, nelem)

!     --- Shift it a bit, to get generic cases

      ymin = ymin + abs (ymin) * 1d-4 * (exp (1d0) - 2*atan (1d0))
      ymax = prgtmx (elkey, kgrad, diael, nelem)

!     --- Shift a bit, but over different interval

      ymax = ymax - abs (ymax) * 1d-4 * (exp (.5d0) + atan (1d0))
      if (nlines.lt.2) then

!     --- ERROR: nlines too small

         call errint (nlines, 1)
         call errsub (1259, 1, 0, 0)
      end if

!     --- calculate stepsize & initialize lineloop

      stsize = (ymax - ymin)/ (nlines - 1)
      slevel = ymin
      mksgnd = 1
      mintnd = 1

!     --- Now for all lines ...

      do 200 ilines = 1, nlines

!     --- calculate which elements intersect this line

        call prelin (slevel, elkey, diael, kgrad, mxdia, nelem, iwork, &
                     niout)

      if (nwork.lt.2*niout) then

!     --- ERROR: nwork too small

         call errint (2 * niout - nwork, 1)
         call errint (0, 2)
         call errchr ('iwork', 1)
         call errsub (1257, 2, 0, 1)
         call errsub (1260, 0, 0, 0)
      end if

!     --- calculate in which points this line intersects and interpolate
!         function values

      call prsegi (slevel, coor, fval, kmeshc, iwork, niout, &
            xcint (mintnd), fcint (mintnd))

!     --- sort the points, remove the doubles and partition into segments

      nchelp = 2*niout
      call prgath (xcint (mintnd), fcint (mintnd), nseg (ilines), &
                   kseg (mksgnd), nchelp )

!     --- update the actual start addresses of *cint and kseg

      do 100 i = 1, nseg (ilines)
        mintnd = mintnd + kseg (mksgnd + i - 1)
100   continue
      mksgnd = mksgnd + nseg (ilines)
      if (mintnd.gt.ncint) then
         call errint (mintnd - ncint, 1)
         call errint (0, 2)
         call errchr ('xcint' , 1)
         call errsub (1257, 2, 0, 1)
      end if

!     --- get the new intersection line

      slevel = slevel + stsize

!     --- ... and start all over again

200   continue

!     --- end do

!         now rotate back the coordinate system

      call prrota (coor, -angdeg, npoint)

!     --- And backshift the origin

      do 210 i = 1, npoint
      coor (1, i) = coor (1,i) + orgx
210   coor (2, i) = coor (2,i) + orgy
      call erclos ('prjlin')
      end
