      subroutine sepplot3c ( iplots, rplots )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 29-05-2014 Extension of timebefore
!        version  2.3  date 25-11-2012 Long integers
!        version  2.2  date 18-05-2005 Fortran 90
!
!   copyright (c) 1989-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a three-dimensional plot of a function defined on a two-dimensional
!     mesh, using a display that is able to fill polygons with colors
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     color
!     function
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(*)
      double precision rplots(*)

!     iplots  i   Integer array in which the following quantities must be
!                 stored:
!                 1:  number
!                 2:  choice parameter for type of projection
!                      0 : parallel projection plot
!                      1 : perspective projection plot
!                     10 : parallel projection plot (View point automatic)
!                     11 : perspective projection plot (View point automatic)
!                 3:  color number of farthest triangle
!                 4:  color number of nearest triangle
!                 5:  Indication if boundaries must be plotted (1) or not (0)
!                 6:  iskip
!                 7...6+nelgrplc:  ihelp
!     rplots  i   Double precision input array that must be filled as follows:
!                 1:    plotfm
!                 2:    yfact
!                 3-5:  eye-point
!                 6-8:  projection point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelemt, isave, idum(1), jv, nelgrplc, &
              ichois, iresul(5,3), nomber(2), neleml, special, ilevel, &
              nelgrpsav, nelemsav
      integer :: kmeshalc(1), kmeshglc(1)
      double precision dum(3), xmean(3), eps
      logical perspc, compl
      double precision, allocatable :: coornw(:), coorlc(:)
      double precision :: timebefore(2)

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     coorlc         array of length ndim x npoint containing the
!                    co-ordinates of the nodal points with respect to the
!                    global numbering
!     coornw         coordinate array for projected points
!                    (1,i) contains the x-coordinate of the projected point i
!                    (2,i) contains the y-coordinate of the projected point i
!                    (3,i) contains the distance of point i to
!                          the view-point
!     dum            General double precision dummy variable
!     eps            Local accuracy
!     ichois         Choice parameter. With this parameter it is defined which
!                    actions will be performed. Possible values:
!                    1:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex only the real or
!                        imaginary part is used
!                    2:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex the copied solution
!                        vector is also complex
!                    3:  Create a submesh consisting of the surface elements
!                        only of each element only the vertices are stored, &
!                        hence the final grid consists of a grid of 3-point
!                        triangles or 4-point quadrilaterals only
!                        This possibility only uses KMESH
!                        It has only been implemented for 3D
!                    4:  Combination of 1 and 3, hence possibly 2 meshes may be
!                        created
!                    5:  Combination of 2 and 3, hence possibly 2 meshes may be
!                        created
!     idum           General dummy variable
!     ilevel         dummy
!     iresul         array of length 5,1 if ichois=2 and otherwise of
!                    length 5,3
!                    The array is filled as follows:
!                    Pos .,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    Pos .,2 corresponds to the "triangle" mesh array
!                            (ichois=1,3)
!                    Pos .,3 corresponds to the solution array (ichois=1,3)
!                    Pos. 1,1  INDVER sequence number with respect to INFOR
!                                     for the "vertex" mesh array
!                    Pos. 2,1  NPELM  number of points per element for this mesh
!                    Pos. 3,1  NELEM  number of points per element for this mesh
!                    Pos. 4,1  NEWVER indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0)
!                    Pos. 5,1  INDLIN sequence number with respect to INFOR
!                                     for the "line"-element array
!                    Pos. 1,2  INDTRI sequence number with respect to INFOR
!                                     for the "triangle" mesh array
!                    Pos. 2,2  NPELM  number of points per element for this mesh
!                    Pos. 3,2  NELEM  number of points per element for this mesh
!                    Pos. 4,2  NEWTRI indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0) or the mesh
!                                     created by the "vertex" mesh
!                    Pos. 5,2  NPHYS  Max. number of degrees of freedom
!                    Pos. 1,3  INDSOL sequence number with respect to INFOR
!                                     for the new solution array
!                    Pos. 2,3  NEWSOL indicates if the solution array is new
!                                     (1) or is in fact the standard solution
!                                     array (0)
!                    Pos. 3,3  INDPRF sequence number of KPROBF
!                    Pos. 4,3  INDPRH sequence number of KPROBH
!                    Pos. 5,3  INPPRP sequence number of KPROBP
!     isave          Help variable to save a quantity
!     jv             Indicates if the view point has been filled (1) or not (0)
!     kmeshalc       kmesh part a
!     kmeshglc       kmesh part g
!     neleml         Number of elements in "line"-element array
!     nelemsav       Help parameter to save the old value of nelem
!     nelemt         Number of elements in the new mesh
!     nelgrplc       Local nelgrp
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     nomber         Helparray to store relative number of unknown considered
!     perspc         indicates if a perspective (true) or a parallel projection
!                    (false) must be applied
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a colored
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a colored
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
!     xmean          Array containing the mean values of the co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0014         Compute projection of points onto projection plane
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLMORE         User subroutine to plot extra information in the plot
!     PLOTSF         Make a three-dimensional hidden line plot of a 3D surface,
!                    using colored surfaces. (Display only)
!     PLTOP0         Store array with co-ordinates and array SOLUT into a new
!                    3-D co-ordinates array for use in PLOTSF
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     702   The mesh is not two-dimensional.
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Perform standard checks
!       Verify if the mesh consists of linear triangles or not
!       verify if unknown is present in all points and if elements
!       must be subdivided into triangles
!       If (mesh must be subdivided) then
!          Compute new KMESH part c
!       Copy unknown NUMBER into a new array SOLUT that is node oriented
!       Make a 3-D color plot based on the new arrays KMESHC and SOLUT
!       END
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Perform some tests

      call eropen ( 'sepplot3c' )
      if ( itime>0 ) call ersettime ( timebefore )
      nelgrpsav = nelgrp
      nelemsav = nelem
      ichois = 1
      compl = .false.
      nomber(1) = 1
      nomber(2) = iplots(1)
      special = 0
      ilevel = 1
      call topqu0 ( ichois, nomber, iplots(6), iplots(7), iresul, &
                    compl, 0, special, ilevel )
      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errsub ( 702, 1, 0, 0 )
      end if
      if ( ierror/=0 ) goto 1000

!     --- Reserve temporary space
!         Array itrian  (iptria) is used to store information of the element
!         groups

      nelemt = iresul(3,2)

!     --- Check if mesh must be subdivided into subtriangles or not
!         Store solution and co-ordinates in new array coor

      allocate ( coornw(3*npoint), coorlc(3*npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'coornw' )
      if ( ierror/=0 ) go to 1000

      call pltop0 ( coor, usolt, npoint, coorlc )

      kmeshalc(1) = 3
      kmeshglc(1) = nelemt

!     --- Call actual plotting subroutine

      nelgrplc = 1

      eps = 10 * epsmac

      if ( abs( rplots(2))<=eps ) rplots(2) = 1d0

      isave = iplots(1)
      iplots(1) = 1
      idum(1) = 0

      jv = 1 - iplots(2)/10
      perspc = mod(iplots(2),10)==1

      call pl0014 ( coorlc, rplots(3), jv, coornw, &
                    npoint, xmean, perspc, 3, dum, 1, rplots(1), &
                    rplots(2), 1 )

      if ( iresul(5,1)/=0 ) then

!     --- line elements

         neleml = size ( kmeshcline ) / 2

      else

!     --- no line elements

         neleml = 0

      end if

      call plotsf ( coornw, npoint, kmeshalc, &
                    kmeshglc, nelgrplc, kmeshctmp, iplots, &
                    dum, dum, idum(1), idum, 0, 0, neleml, kmeshcline )

      iplots(1) = isave
      if ( mod(jtimes,10)<1 .or. mod(jtimes,10)>2 ) then
         call plmore
         call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      end if

!     --- Delete temporary arrays

      deallocate ( coornw, coorlc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coornw' )

      if ( iresul(2,3)==1 ) call seprmsolinfo ( iresul(1,3) )
      if ( iresul(2,2)==1 ) deallocate ( kmeshctmp )
      if ( iresul(5,1)/=0 ) deallocate ( kmeshcline )
      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem

1000  call erclmn ( 'sepplot3c', timebefore )
      ierror = 0
      end subroutine sepplot3c
