      subroutine topsubdiv2d ( isol, idegfd, skipelgrps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a 2d region in linear triangles if necessary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol, idegfd
      integer, intent(out) :: skipelgrps(*)

!     idegfd         i    Degree of freedom sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     skipelgrps     o    array of size 2 x nelgrp containing information
!                         for the subdivision of the elements in triangles
!                         (1,i) mult, i.e. number of triangles each element
!                               of element group i must be subdivided
!                         (2,i) typesubdiv, defines special subdivisions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenkmeshc

!     lenkmeshc      length of array kmeshc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREATEIBUF  Create permanent space in array IBUFFR
!     TOPSUBDIV2DCH  Check if a mesh must be subdivided into triangles or not
!     TOPSUBDIVTRI   Subdivide mesh into triangles depending on skipelgrps
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If (all elements are linear triangles) then
!        newmesh := false
!     else
!        newmesh := true
!     If (newmesh) then
!        vertex_present := unknown present in all vertices
!        mid_point_present := unknown present in all midpoints
!        centroid_present := unknown present in all centers
!        for each element group do
!           shape := shape of element
!           subdivide elements depending on shape
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubdiv2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubdiv2d'
         write(irefwr,1) 'idegfd', idegfd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=2 ) then

!     --- Error: ndim/= 2

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim/=2 )

      iprob = ks(isol)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kproba => kprobpart%kproba
      indpra = kprobloc(14)

!     --- Activate solution array and other relevant arrays

      call sepactsolbf2 ( isol )

!     --- Check if mesh must be subdivided

      call topsubdiv2dch ( idegfd, isol, lenkmeshc, skipelgrps )

      if ( debug ) write(irefwr,1) 'lenkmeshc', lenkmeshc

      nullify(kmeshctmp)
      if ( lenkmeshc>0 ) then

!     --- The mesh must be sudivided
!         First reserve space for a new temporary array kmeshc
!         This array should be removed in the calling subroutine

         allocate ( kmeshctmp(lenkmeshc) )

         call topsubdivtri ( skipelgrps )

         if ( debug ) &
            call prinin1 ( kmeshctmp, lenkmeshc/3, 3, 'kmsctmp')

      end if  ! ( lenkmeshc>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topsubdiv2d'

      end if  ! ( debug )
      call erclos ( 'topsubdiv2d' )

      end subroutine topsubdiv2d
