      subroutine topsubdiv2dch ( idegfd, isol, lenkmeshc, skipelgrps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if a mesh must be subdivided into triangles or not
!     Compute length of new array kmeshc
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol, idegfd
      integer, intent(out) :: lenkmeshc, skipelgrps(2,*)

!     idegfd         i    Degree of freedom sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     lenkmeshc      o    length of temporary array kmeshc
!     skipelgrps     o    array of size 2 x nelgrp containing information
!                         for the subdivision of the elements in triangles
!                         (1,i) mult, i.e. number of triangles each element
!                               of element group i must be subdivided
!                         (2,i) typesubdiv, defines special subdivisions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, inpelm, ivec, mult, typesubdiv, indpralc

!     ielgrp         element group number
!     inpelm         number of points per element
!     ivec           Sequence number of vector of special structure
!     mult           Number of triangles an element is subdivided in
!     typesubdiv     Defines type of subdivision and interpolation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getkproba

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETKPROBA      Return with starting address in indpralc or kprobe for
!                    vector of type ivec and element group ielgrp
!     PRININ1        print 2d integer array
!     TOPSUBDIV2D1   Check if element must be subdivided into triangles
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubdiv2dch' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubdiv2dch'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indprflc = kprobloc(19)

      lenkmeshc = 0        ! no new mesh yet
      skipelgrps(1:2,1:nelgrp) = 0
      ivec = ks(isol)%ivec  ! vector sequence number

      if ( debug ) &
         write(irefwr,1) 'nelgrp, indprflc, ivec', nelgrp, indprflc, ivec

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)   ! shape of element
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inpelm, inelem', &
                             ielgrp, ishape, inpelm, inelem

         if ( ishape==3 ) then

!        --- Linear triangle

            if ( indprflc==0 ) then

!           --- unknown present in all nodes

               lenkmeshc = lenkmeshc+3*inelem
               skipelgrps(1,ielgrp) = 1

            else

!           --- unknown not present in all nodes
!               Check if element group must be skipped

               indpralc = getkproba ( ivec, ielgrp )
               if ( kprobatmp(1)>=idegfd ) then
                  skipelgrps(1,ielgrp) = 1
                  lenkmeshc = lenkmeshc+3*inelem
               end if  ! ( kprobloc(indpralc)>=idegfd )

            end if  ! ( indprflc==0 )

         else if ( ishape>3 ) then

!        --- Other type of element

            indpralc = getkproba ( ivec, ielgrp )
            if ( indpralc==0 ) then

!           --- unknown present in all nodes

               select case ( ishape )
               case(4, 7)
                  mult = 4
               case(5, 9)
                  mult = 2
               case(6)
                  mult = 8
               case(10)
                  mult = 1
               end select ! case ( ishape )
               lenkmeshc = lenkmeshc+3*inelem*mult
               skipelgrps(1,ielgrp) = mult

            else

!           --- unknown not present in all nodes
!               Check if element group must be skipped

               call topsubdiv2d1 ( ishape, mult, typesubdiv, idegfd )
               skipelgrps(1,ielgrp) = mult
               skipelgrps(2,ielgrp) = typesubdiv
               lenkmeshc = lenkmeshc+3*inelem*mult

               if ( debug ) &
                  write(irefwr,1) 'mult, typesubdiv, lenkmeshc', &
                                   mult, typesubdiv, lenkmeshc

            end if  ! ( indpralc==0 )

         end if  ! ( ishape<3 )

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( skipelgrps, nelgrp, 2, 'skipelgrps' )
         write(irefwr,*) 'End topsubdiv2dch'

      end if  ! ( debug )
      call erclos ( 'topsubdiv2dch' )

      end subroutine topsubdiv2dch
