      subroutine topsubint2d ( isol, indisol, idegfd, skipelgrps, umin, umax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate solution from original mesh to triangular mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol, idegfd, skipelgrps(2,*)
      integer, intent(inout) :: indisol
      double precision, intent(out) :: umin, umax

!     idegfd         i    Degree of freedom sequence number
!     indisol       i/o   memory management reference to solution array
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     skipelgrps     i    array of size 2 x nelgrp containing information
!                         for the subdivision of the elements in triangles
!                         (1,i) mult, i.e. number of triangles each element
!                               of element group i must be subdivided
!                         (2,i) typesubdiv, defines special subdivisions
!     umax           o    largest value of function
!     umin           o    smallest value of function
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: typesubdiv, ipos, jdegfd, i
      integer, allocatable :: iwork(:)
      double precision, allocatable :: usoltmp(:), weight(:)

!     iwork          integer work array to mark nodes that have been
!                    treated completely (iwork(node)=1)
!                    For those nodes no interpolation is required, but
!                    the value is copied directly
!     typesubdiv     Defines type of subdivision and interpolation
!     usoltmp        Temporary array usol
!     weight         array containing the weights for those values that
!                    must be averaged
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPSEPVEC4   Copy one degree of freedom of sepran vector in user vector
!                    All nodes in the mesh are used
!                    The number of degrees of freedom per point is variable
!                    If a node is not filled the value 0 is substituted
!     PRCOPSEPVEC5   Copy one degree of freedom of sepran vector in user vector
!                    All nodes in the mesh are used
!                    The number of degrees of freedom per point is variable
!                    If a node is not filled the value 0 is substituted
!                    Array kprobf is used
!     PRCOPY2D2      Copy two-dimensional array of size ndim1 x npoint into
!                    two-dimensional array of size ndim2 x npoint.
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     TOPSUBINT2D1   Interpolate solution from original mesh to triangular mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubint2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubint2d'
         write(irefwr,1) 'isol', isol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      typesubdiv = maxval(skipelgrps(2,1:nelgrp))  ! maximum of typesubdiv

      if ( debug ) &
         write(irefwr,1) 'nelgrp, npoint, typesubdiv', &
                          nelgrp, npoint, typesubdiv

!     --- Activate solution array and other relevant arrays

      call sepactsolbf2 ( isol )
      usol => ks(isol)%sol

      if ( typesubdiv==0 ) then

!     --- typesubdiv = 0, no interpolation necessary

         if ( indprfi==0 .and. nunkpi==1 ) then

!        --- Only one unknown  in each node, use isol

            indisol = isol
            usolt => usol

            umin = minval(usolt)
            umax = maxval(usolt)

         else

!        --- Create array usol of length npoint and extract idegfd

            allocate ( usolt(npoint) )
            usolt = 0d0

            if ( indprpi>0 ) then

!           --- indprpi>0, kprob part p has been filled

               umin = 1d70
               umax = -1d70
               do i = 1, npoint
                  ipos = kprobpi(i,idegfd)
                  if ( ipos>0 ) then
                     usolt(i) = usol(ipos)
                     umin = min(umin,usol(ipos))
                     umax = max(umax,usol(ipos))
                  end if  ! ( ipos>0 )
               end do  ! i = 1, npoint
               
            else if ( indprfi>0 ) then

!           --- use kprobfi

               umin = 1d70
               umax = -1d70
               ipos = kprobfi(1)
               do i = 1, npoint
                  jdegfd = kprobfi(i+1)-ipos
                  if ( idegfd<=jdegfd ) then
                     usolt(i) = usol(ipos+jdegfd)
                     umin = min(umin,usol(ipos+jdegfd))
                     umax = max(umax,usol(ipos+jdegfd))
                  end if  ! (  idegfd<=jdegfd )
                  ipos = kprobfi(i+1)
               end do  ! i = 1, npoint

            else

!           --- indprpi = 0, kprob part p has not been filled

               call prcopy2d2 ( npoint, usol, nunkpi, usolt, 1, idegfd )

               umin = minval(usolt)
               umax = maxval(usolt)

            end if  ! ( indprpi>0 )

         end if  ! ( indprfi==0 .and. nunkpi==1 )

      else

!     --- typesubdiv>0, interpolation necessary
!         Create array usol of length npoint and extract idegfd

         allocate ( usoltmp(npoint), weight(npoint), iwork(npoint), &
                    usolt(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, 3*npoint, 'usoltmp' )
         if ( ierror/=0 ) go to 1000
         
         usoltmp = 0d0

         if ( indprpi>0 ) then

!        --- indprpi>0, hence kprobp has been filled

            umin = 1d70
            umax = -1d70
            do i = 1, npoint
               ipos = kprobpi(i,idegfd)
               if ( ipos>0 ) then
                  usoltmp(i) = usol(ipos)
                  umin = min(umin,usol(ipos))
                  umax = max(umax,usol(ipos))
               end if  ! ( ipos>0 )
            end do  ! i = 1, npoint
 
         else

!        --- indprpi = 0, use kprobf

            umin = 1d70
            umax = -1d70
            ipos = kprobfi(1)
            do i = 1, npoint
               jdegfd = kprobfi(i+1)-ipos
               if ( idegfd<=jdegfd ) then
                  usoltmp(i) = usol(ipos+jdegfd)
                  umin = min(umin,usol(ipos+jdegfd))
                  umax = max(umax,usol(ipos+jdegfd))
               end if  ! (  idegfd<=jdegfd )
               ipos = kprobfi(i+1)
            end do  ! i = 1, npoint

         end if  ! ( indprpi>0 )

         call topsubint2d1 ( usoltmp, usolt, &
                             weight, skipelgrps, iwork )

         deallocate ( usoltmp, weight, iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'usoltmp' )

      end if  ! ( typesubdiv==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topsubint2d'

      end if  ! ( debug )
      call erclos ( 'topsubint2d' )

      end subroutine topsubint2d
