      subroutine creatvspec ( kfun, usol, icompl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 09-03-2013 Correction declaration
!        version  2.1  date 10-02-2012 use getnpelmelgrp
!        version  2.0  date 27-12-2011 New parameter list
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array usol with values in the case that 1000<fkun<2000
!     This means that a loop over the elements is performed and that
!     the subroutines func1b or cfun1b are called
!     This subroutine is only implemented for kfill = 0 (all nodes)
!     usol must be of type 110 or 115
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kfun, icompl
      double precision usol(*)

!     icompl         i    Indication if the vector to be created is real (0)
!                         or complex
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     usol          i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichoice, ielem, ielgrp, ikelmc, inpelm, inunkp
      integer, allocatable :: index1(:), index2(:)

!     ichoice        Integer choice parameter
!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     index1         Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     index2         Array of length icount containing the positions of the
!                    degrees of freedom in the element with respect to USOL
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CFUN1B         User subroutine to fill boundary conditions
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     FUNC1B         User subroutine to fill boundary conditions
!     PRFILLINDEX1   Fill arrays index1 and index2 with information about
!                    nodes and unknowns per element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initializations
!    For all element groups do
!       For all elements in element group do
!          Fill index1 and index2
!          call user subroutine
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatvspec' )

!     --- Initializations

      inunkp = nunkp
      ikelmc = 0
      ichoice = kfun-1000

!     --- allocate work arrays

      allocate ( index1(npelm), index2(npelm*inunkp), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npelm*(inunkp+1), 'index1' )
      if ( ierror/=0 ) go to 1000

!     --- For all element groups do

      do ielgrp = 1, nelgrp

         inpelm = sepgetinpelm ( ielgrp )
         inelem = sepgetinelem ( ielgrp )

!        --- For all elements in element group do

         do ielem = 1, inelem

!        --- Fill index1 and index2

            call prfillindex1 ( kmeshc(ikelmc+1), index1, index2, &
                                inpelm, inunkp )

!           --- call user subroutine

            if ( icompl==1 ) then

!           --- complex case

               call cfun1b ( ichoice, index1, index2, usol, coor )

            else

!           --- real case

               call func1b ( ichoice, index1, index2, usol, coor )

            end if

            ikelmc = ikelmc+inpelm

         end do

      end do

      deallocate ( index1, index2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index1' )

1000  call erclos ( 'creatvspec' )
      end
