      subroutine eldsubs
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 06-01-2016 Use sepmoduleelem
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.6  date 11-11-2014 Extension with type 700
!
!   copyright (c) 1998-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call specific element subroutine, depending on itype
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element
!     element_group
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipcoef, ipcoefnodes, ipphi, ippsi, ipphixi, ippsixi
      integer, pointer :: iuser(:), islold(:)
      double precision vecloc(numold*maxunk*3*inpelm)
      double precision, pointer :: user(:)
      logical :: compts
      save ipcoef, ipcoefnodes, ipphi, ippsi, ipphixi, ippsixi

!     compts         Indicates if element is real (false) or complex (true)
!     ipcoef         Starting address of array coeffintpnts
!     ipcoefnodes    Starting address of array coeffnodes
!     ipphi          Starting address of array phi
!     ipphixi        Starting address of array phixi
!     ippsi          Starting address of psi
!     ippsixi        Starting address of array psixi
!     islold         User input array in which the user puts information
!                    of all preceding solutions
!     iuser          Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user           Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     vecloc         Work array in which all old solution vectors for the
!                    integration points are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELD200         Compute derived quantities for general nonlinear solids
!     ELD203         Compute derived for 4-phase solids
!     ELD204         Fill element derivative vector and weights for nonlinear
!                    solid elements
!     ELD250         Compute derivatives for general linear stress/strain
!                    element
!     ELD260         Compute derivatives for rubber element
!     ELD400         Compute pressure and other derived quantities
!     ELD406         Compute pressure and other derived quantities for
!                    Navier-Stokes
!                    2d extended quadratic triangle including swirl
!     ELD420         Compute derivatives for 2d Boussinesq
!     ELD600         element derivative vector for second order elliptic
!                    equation 2D spectral elements
!     ELD603         element derivative vector for inhomogeneous Neumann bc
!                    spectral elements
!     ELD607         computes the convection residual in the case of an
!                    explicitly performed convection spectral elements
!     ELD608         element derivative vector for inhomogeneous Neumann bc
!                    2D surface in R3  spectral elements
!     ELD650         Mixed finite element method (Ground water flow)
!     ELD800         element matrix and vector for second order elliptic
!                    equation gen
!     ELD900         Compute pressure and other derived quantities for
!                    Navier-Stokes laminar flow
!     ELD903         Compute pressure and other derived quantities for
!                    Navier-Stokes laminar flow (Taylor-Hood)
!     ELD905         Compute derived quantities for pressure correction
!                    method applied to Taylor-Hood
!     ELDER2         element routine
!     ELDERV         element routine
!     ELM900PARMS    Compute parameters irule, m and n for type elements of
!                    type 900
!     ELM903PARMS    Compute parameters irule, m and n for type elements of
!                    type 903
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     SEPCOPYCELVEC  Copy information of complex elements in type element
!     SEPELD150      element derivative vector ans weight vector for general
!                    elliptic equations with one unknown (complex)
!     SEPELD705      element derivative vector ans weight vector for general
!                    elliptic equations with one unknown
!     SEPELDEFAULT   Set default values for parameters in module sepmodulebasefn
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
!      49   complex arithmetic with real element
!      50   Real arithmetic with complex element
! **********************************************************************
!
!                       PSEUDO CODE
!
!  ielrou = 50  eld400 ;  itype=400,402,404
!                                 (2-d Cartesian, polar, axi-symm)
!         = 53  eld406 ;  itype=406
!                                 (axi-symm with swirl)
!         = 54  eld420 ;  itype=420-422
!                                 (Boussinesq;   2-d,polar,axi-symm)
!         = 55  eld430 ;  itype=430-431
!                                 (turbulent, Prandtl, 2-d,axi-symm)
!         = 56  eld429 ;  itype=429
!                                 (Boussinesq, axi-symm with swirl)
!         = 57  eld433 ;  itype=433
!                                 (turbulent, Prandtl, axi-symm with swirl)
!         = 58  eld440 ;  itype=414-416, 440-442
!                                 (decoupled, (v-p), penalty)
!         = 59  eld900 ;  itype=900
!                                 (3D laminar, isothermal flow)
!         = 60  eld903 ;  itype=903  Taylor-Hood
!                                 (3D laminar, isothermal flow)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldsubs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldsubs'
         write(irefwr,1) 'itype, ifirst, icouno', itype, ifirst, icouno
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      compts = .false.
      iuser => E%iuser
      user => E%user
      islold => E%islold

      if ( ifirst==0 ) then

!     --- ifirst = 0, set pointers in coeffarr

         ipphi = 1
         ipphixi = ipphi+27*27
         ippsi = ipphixi+27*27*3
         ippsixi = ippsi+27*27
         ipcoefnodes = ippsixi+27*27*3
         ipcoef = ipcoefnodes+32*27

         call inichkarlen ( ipcoef-1, 7425, 'coeffarr' )

!        --- Set default values

         call sepeldefault

      end if  ! ( ifirst==0 )

      select case ( itype )

         case (0)

!        --- Type 0, skip

         case (1:99)

!        --- Type 1-99, user elements

            if ( numold<=1 .or. numol0==0 ) then

!           --- numold<=0 or numol0=0 subroutine elderv

               call elderv ( icheld, ix, jdegfd, coor, elemvc1, elemwg, &
                             iuser, user, vecold, vectin2, index1, &
                             index2, icouno )

            else

!           --- elder2

               call elder2 ( icheld, ix, jdegfd, coor, elemvc1, elemwg, &
                             iuser, user, vecold, vectin2, index1, &
                             index2, icouno )

            end if
            compts = complo

         case (150)

!        --- Type 150
!            Helmholtz equation (Two-dimensional)

            call sepcopycelvec ( elemvc1 )
            call sepeld150
            compts = .true.

         case (152)

!        --- Type 152
!            complex boundary conditions for helmholtz equation )

            compts = .true.

         case (200:202)

!        --- Type 200, 201, 202: general nonlinear solid element

            call eld200 ( coor, iuser, user, islold, vecloc, coeffarr )

         case (203)

!        --- Type  203: 2-phase nonlinear solid element

            call eld203 ( coor, iuser, user, islold, vecloc, coeffarr )

         case (204, 205)

!        --- Type  204, 205: 4-phase nonlinear solid element

            call eld204 ( coor, iuser, user, islold, vecloc, coeffarr )

         case (250)

!        --- Type 250, linear elasticity element

            call eld250 ( iuser, user, coor, islold, vecloc, &
                          coeffarr(ipcoef), coeffarr(ipphi), coeffarr(ipphixi), &
                          coeffarr(ippsi), coeffarr(ippsixi) )

         case (252)

!        --- Type 250, linear elasticity element

            call eld250 ( iuser, user, coor, islold, vecloc, &
                          coeffarr(ipcoef), coeffarr(ipphi), coeffarr(ipphixi), &
                          coeffarr(ippsi), coeffarr(ippsixi) )

         case (208:212, 230, 232, 251, 255:256)

!        --- Type 210: boundary element for element 200-204

         case (260, 261)

!        --- Type 260, 261, rubber element

            call eld260 ( coor, iuser, user, islold, vecloc, &
                          coeffarr(ipcoef), numold, &
                          coeffarr(ipphi), coeffarr(ipphixi), &
                          coeffarr(ippsi), coeffarr(ippsixi) )

         case (302:304)

!        --- Type 302
!            elasticty-flow interaction (incompressible bearing )
!            (linear line element)

         case (400, 402, 404, 450)

!        --- Type 400, 402, 404, 450
!            navier-stokes (penalty equation) extended quadratic element

            call eld400 ( iuser, user, vectin2, islold, coeffarr )

         case (401, 403, 405, 407, 423, 424, 425, 437)

!        --- Type 401, 403, 405, 407, 423, 424, 425, 437
!            line element for navier-stokes quadratic element

         case (406)

!        --- Type 406,  navier-stokes equation axi-symmetric with swirl

            call eld406 ( iuser, user, vectin2, islold, coeffarr )

         case (408)

!        --- Type 408  line element for navier-stokes  (Free boundary)
!                      quadratic element


         case (420:422)

!        --- Type 420:422  boussinesq-stokes equation

            call eld420 ( coor, iuser, user, vecold, vectin2, &
                          coeffarr, vecloc )

         case (501)

!        --- Type 501 mass matrix  ( linear triangle  )

         case (600)

!        --- Type 600
!            second order elliptic equation General using spectral elements

            call eld600 ( coor, iuser, user,islold, coeffarr )

         case (601)

!        --- Type 601

         case (603)

!        --- Type 603
!            Spectral elements, boundary element for a scalar

            call eld603 ( coor, iuser, user, coeffarr )

         case (606)

!        --- Type 606
!            Navier-Stokes equation (2D) with spectral elements

         case (607)

!        --- Type 607
!            Spectral elements, Convection-diffusion of a scalar; 3-D case

            call eld607 ( coor, iuser, user, coeffarr )

         case (608)

!        --- Type 608
!            Spectral elements, boundary element for a scalar; 3-D case

            call eld608 ( icheld, coor, elemvc1, iuser, user, index1, &
                          coeffarr )
            notvec = 1

         case (609, 612, 615)

!        --- Type 609

         case (650:652)

!        --- Type 650,651,652 mixed element
!            Special ground water flow element, using mixed finite elements

            call eld650 ( coor, elemvc1, elemwg, iuser, user, vecold, &
                          index1, index2 )

         case (661, 665)

!        --- Type 665, 661 mixed boundary element

         case ( 300, 325, 700:703, 705, 800, 803, 850 )

!        --- Type 700-705 : General second order elliptic equation

            call sepeld705

         case ( 810, 813, 815 )

!        --- Type 800 : General second order elliptic equation or
!                 803 : Special nonlinear second order elliptic equation
!                 300, 325 : Reynolds equation (in)compressible 1D/2D

            call eld800 ( vecloc, &
                          coeffarr(ipcoef), &
                          coeffarr(ipphi), coeffarr(ipphixi), &
                          coeffarr(ippsi), coeffarr(ippsixi) )

         case (801:802, 804:806)

!        --- Type 801 : Boundary conditions for general second order elliptic
!                       equation

         case (410,900:902)

!        --- Type 410, 900, 901, 902
!            laminar, isothermal flow, penalty method and integrated method

            if ( ifirst==0 ) call elm900parms ( iuser )

            call eld900 ( coor, elemvc1, iuser, user, index1, islold, vecloc, &
                          coeffarr(ipcoefnodes), coeffarr(ipcoef), &
                          coeffarr(ipphi), coeffarr(ippsi), coeffarr(ipphixi), &
                          coeffarr(ippsixi) )

         case (903)

!        --- Type 903  laminar, isothermal flow, integrated method
!                      Taylor Hood

            if ( ifirst==0 ) call elm903parms ( iuser )
            call eld903 ( coor, elemvc1, iuser, user, index1,islold, vecloc, &
                          coeffarr(ipcoefnodes), coeffarr(ipcoef), &
                          coeffarr(ipphi), coeffarr(ippsi), coeffarr(ipphixi), &
                          coeffarr(ippsixi) )

         case (905)

!        --- Type 905  laminar, isothermal flow, pressure correction method
!                      Taylor Hood

            call eld905 ( coor, iuser, user, islold, vecloc, &
                          coeffarr(ipcoef), coeffarr(ipphi), &
                          coeffarr(ipphixi), &
                          coeffarr(ippsi), coeffarr(ippsixi) )

         case (910:912, 914, 915, 922, 924, 928, 997, 998 )

!        --- Type 910, 911  laminar, isothermal flow, boundary conditions

         case default

!        --- Type<0 or>999
!            These types are not available

            call errint ( itype, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 43, 2, 0, 0)

      end select  ! case ( itype )

!     --- Test complots

      if ( compts .and. .not. complo ) then
         call errint ( itype, 1 )
         call errint ( ielgrp, 2 )
         call errint ( ielem, 3 )
         call errsub ( 50, 3, 0, 0 )
      end if
      if ( .not. compts .and. complo ) then
         call errint ( itype, 1 )
         call errint ( ielgrp, 2 )
         call errint ( ielem, 3 )
         call errsub ( 49, 3, 0, 0 )
      end if

      ifirst = 1
1000  if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc1, icouno, 'elemvc1' )
         write(irefwr,*) 'End eldsubs'

      end if  ! ( debug )
      call erclos ( 'eldsubs' )

      end subroutine eldsubs
