      subroutine elmsubs
! ======================================================================
!
!        programmer    Guus Segal
!        version 14.0  date 24-07-2015 Use sepmodulebuild
!        version 13.4  date 03-02-2015 extension for type 704
!        version 13.3  date 13-11-2014 extension with type 816
!
!   copyright (c) 1998-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call specific element subroutine, depending on itype
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_group
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipcoef, ipcoefnodes, ipphi, ippsi, ipphixi, ippsixi, &
              ipphixicp
      double precision vecloc(numold*maxunk*3*inpelm)
      double precision, pointer :: vecoldlc(:)
      integer, pointer :: iuser(:), islold(:)
      double precision, pointer :: user(:)
      logical compts
      save ipcoef, ipcoefnodes, ipphi, ippsi, ipphixi, ippsixi, &
           ipphixicp

!     compts         Indicates if element is real (false) or complex (true)
!     ipcoef         Starting address of array coeffintpnts
!     ipcoefnodes    Starting address of array coeffnodes
!     ipphi          Starting address of array phi
!     ipphixi        Starting address of array phixi
!     ipphixicp      Starting address of array phixicp
!     ippsi          Starting address of psi
!     ippsixi        Starting address of array psixi
!     islold         User input array in which the user puts information
!                    of all preceding solutions
!     iuser          Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user           Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     vecloc         Work array in which all old solution vectors for the
!                    integration points are stored
!     vecoldlc       Local old vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELEM           Standard user element subroutine
!     ELEM1          Standard user element subroutine including mass matrix
!     ELEM2          Standard user element subroutine including mass matrix and
!                    extra old vectors
!     ELM200         Fill element matrix and vector for general nonlinear solids
!     ELM203         Fill element matrix and vector for general nonlinear solids
!     ELM204         Fill element matrix and vector for general nonlinear solids
!     ELM210         Fill element matrix and vector for boundary conditions for
!                    general nonlinear solids
!     ELM211         Fill element matrix and vector for contact conditions for
!                    general nonlinear solids
!     ELM303         elasticity-flow interaction (incompressible bearing )
!     ELM304         Point boundary condition for Hydrostatic bearing
!     ELM400         Navier-stokes (penalty equation) extended quadratic element
!     ELM401         Natural boundary conditions for 2-dim flow
!     ELM406         Navier-stokes equation axi-symmetric with swirl
!     ELM408         line element for Navier-stokes  (Free boundary)
!     ELM420         Boussinesq-stokes equation
!     ELM426         line element for Navier-stokes  Temperature part
!                    quadratic element
!     ELM501         Mass matrix for Parabolic equation
!     ELM600         second order elliptic equation General using spectral
!                    elements
!     ELM601         Boundary conditions for ELM600
!     ELM603         Spectral elements, boundary element for a scalar
!     ELM606         Navier-Stokes equation (2D) with spectral elements
!     ELM607         Spectral elements, Convection-diffusion of a scalar;
!                    3-D case
!     ELM608         Spectral elements, boundary element for a scalar; 3-D case
!     ELM609         Spectral elements, second order equations, two unknowns per
!                    point
!     ELM612         Spectral elements, viscoelastic element (2D)
!     ELM615         Spectral elements, second order equations, 2 unknowns per
!                    point
!     ELM650         Mixed finite element method (Ground water flow)
!     ELM665         Ground water flow element, using mixed finite elements (3D)
!     ELM807         Fill element matrix and vector for enthalpy equation
!                    (Stefan problem) solved by the Newton method of
!                    Fachinotti et al
!     ELM808         element matrix and vector for second order elliptic
!                    equation
!                    two unknowns per point, general equation
!     ELM810         Fill element matrix and vector for enthalpy equation
!                    Nedjar quasi-newton linearization
!     ELM811         Fill element matrix and vector for computation of pseudo
!                    stream function
!     ELM812         Fill element matrix for smoothing operator
!                    on surface
!     ELM813         element matrix and vector for second order
!                    elliptic equation with special rhs
!     ELM814         Fill element matrix and vector for minimal surface problem
!     ELM815         element matrix and vector for equation
!                    to make phi a distance function
!     ELM815PARMS    Compute parameters irule, m and n for type elements of
!                    type 815
!     ELM816         Fill element vector for special right-hand side
!     ELM850         Fill element matrix and vector for special
!                    stochastic investement equation
!     ELM903         Compute element matrix and vector for the incompressible
!                    Navier-Stokes equations (Taylor-Hood)
!     ELM903PARMS    Compute parameters irule, m and n for type elements of
!                    type 903
!     ELM905         Compute element matrix and vector for the incompressible
!                    Navier-Stokes equations (pressure correction, Taylor-Hood)
!     ELM906         Compute element matrix for the pressure matix
!                    (Navier-Stokes equations, pressure correction, Taylor-Hood)
!     ELM907         Compute element matrix and vector natural boundary
!                    conditions for the pressure matix
!                    (Navier-Stokes equations, pressure correction, Taylor-Hood)
!     ELM910         Compute element vector for the boundary conditions of the
!                    Navier-Stokes equations
!     ELM912         Compute element matrix and vector for a large line element
!                    with given mass flow
!     ELM914         Compute element vector for the discontinuous boundary
!                    conditions of the Navier-Stokes equations
!     ELM915         Compute element matrix and vector for the free surface
!                    boundary conditions of the Navier-Stokes equations
!                    based on the method of Kruyt at all
!     ELM922         Compute element matrix and element vector for special
!                    fictitious domain boundary element, treated as
!                    cross-section of mesh with obstacle
!     ELM924         Fill element matrix and element vector for coupling
!                    elements of type 109.
!     ELM995         Compute element vector for copying reaction force
!     ELM996         Compute element vector for copying reaction force
!     ELM997         Compute element matrix and element vector for special
!                    contact element
!     ELM998         Compute element matrix and element vector for special
!                    friction element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPCOPYCELEM   Copy information of complex elements in type element
!     SEPELDEFAULT   Set default values for parameters in module sepmodulebasefn
!     SEPELM150      Fill element matrix and vector for general complex elliptic
!                    equation
!     SEPELM152      Boundary conditions for SEPELM150
!     SEPELM220      Fill element matrix and vector for beam element
!     SEPELM250      Fill element matrix and vector for linear elasticity
!                    element
!     SEPELM251      Fill element matrix and vector for boundary conditions
!                    for linear elasticity element
!     SEPELM255      Compute element matrix and vector for shell element
!     SEPELM256      Compute element matrix and vector for plate element
!     SEPELM260      ?
!     SEPELM300      Fill element matrix and vector for Reynolds equation
!     SEPELM302      Elasticity Flow interaction
!     SEPELM700      Fill element matrix and vector for laplace equation
!     SEPELM701      Fill element matrix and vector for Poisson equation
!     SEPELM702      Fill element matrix and vector for Diffusion equation
!     SEPELM703      Fill element matrix and vector for Convection-diffusion
!                    equation
!     SEPELM705      Fill element matrix and vector for general elliptic
!                    equation
!     SEPELM801      boundary condition for type 800 (general)
!     SEPELM802      Delta function for right-hand side
!     SEPELM803      element matrix and vector for special second order
!                    elliptic equation
!     SEPELM804      boundary condition for type 800 (discontinuous)
!     SEPELM805      element matrix and vector for second order elliptic
!                    equation
!                    two unknowns per point
!     SEPELM806      boundary condition for type 805 (general)
!     SEPELM900      element matrix and vector for the incompressible
!                    Navier-Stokes equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
!      49   Complex arithmetic with real element
!      50   Real arithmetic with complex element
! **********************************************************************
!
!                       PSEUDO CODE
!
!   compute element matrix and element vector depending on the values
!   of the parameters
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmsubs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmsubs'
         write(irefwr,1) 'itype, ielem', itype, ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      compts = .false.
      transform = .true.
      icheld = 0    ! Call from build
      E%symm = symm

      iuser => E%iuser
      user => E%user
      islold => E%islold
      vecoldlc => oldsols(1)%oldvecs
      if ( ifirst==0 ) then

!     --- ifirst = 0, set pointers in coeffarr

         ipphi = 1
         ipphixi = ipphi+27*27
         ippsi = ipphixi+27*27*3
         ippsixi = ippsi+27*27
         ipphixicp = ippsixi+27*27
         ipcoefnodes = ipphixicp+27*27*3
         ipcoef = ipcoefnodes+32*27

         call inichkarlen ( 7425, ipcoef-1, 'coeffarr' )

!        --- Set default values

         call sepeldefault

      end if  ! ( ifirst==0 )

      select case ( itype )

         case (0)

!        --- Type 0, skip

         case (1:99)

!        --- Type 1-99, user elements

            if ( isubcall==4 .or. isubcall==5 .and. iinbld(10)>1 ) then

!           --- isubcall = 4  (SYSTM2) or  BUILD and numold>0  elem2

               call elem2 ( iuser, user )
               compts = compl

            else if ( isubcall==3 .or. isubcall==5 .and. imas>0 ) then

!           --- isubcall = 3  (SYSTM1) or  BUILD and imas>0  elem1

               call elem1 ( coor, elemmt, elemvc2d, elemms, iuser ,user, &
                            vecoldlc, index1, index2 )
               compts = compl

            else

!           --- isubcall<3  (SYSTM0) or  BUILD and imas = 0  elem

               call elem ( coor, elemmt, elemvc2d, iuser ,user, &
                           vecoldlc, index1, index2 )
               compts = compl

            end if

         case (150)

!        --- Type 150
!            Helmholtz equation

            call sepcopycelem ( elemmt, elemvc, elemmt2 )
            call sepelm150
            compts = .true.

         case (152)

!        --- Type 152
!            complex boundary conditions for Helmholtz equation )

            call sepcopycelem ( elemmt, elemvc, elemmt2 )
            call sepelm152
            compts = .true.

         case (200:202)

!        --- Type 200, 201, 202: general nonlinear solid element

            call elm200

         case (203)

!        --- Type  203: 2-phase nonlinear solid element

            call elm203

         case (204, 205)

!        --- Type  204, 205: 4-phase nonlinear solid element

            call elm204

         case (250, 252)

!        --- Type 250, linear elasticity element

            call sepelm250
            symm = .true.

         case (251, 253)

!        --- Type 251, boundary conditions for (non)-linear elasticity element

            call sepelm251

         case (210)

!        --- Type 210: boundary element for element 200-204

            call elm210 ( vecloc, coeffarr )

         case (211, 212)

!        --- Type 211, 212: contact element for element 200

            call elm211 ( coeffarr )

         case (220)

!        --- Type 220: simple beam element

            call sepelm220

         case (255)

!        --- Type 255: plate element

            call sepelm255

         case (256)

!        --- Type 256: shell element

            call sepelm256

         case (260, 261)

!        --- Type 260, 261, rubber element

            call sepelm260 ( vecloc, coeffarr(ipcoef) )

         case (300, 325)

!        --- Type 300, 325
!            Reynolds equation (in)compressible 1D/2D

            call sepelm300

         case (302)

!        --- Type 302
!            elasticty-flow interaction (incompressible bearing )
!            (linear line element)

            call sepelm302

         case (303)

!        --- Type 303
!            elasticity-flow interaction (incompressible bearing )
!            point element

            call elm303

         case (304)

!        --- Type 304
!            Point boundary condition for Hydrostatic bearing

            call elm304 ( vecloc )

         case (400, 402, 404, 450)

!        --- Type 400, 402, 404, 450
!            navier-stokes (penalty equation) extended quadratic element

            call elm400 ( coeffarr, vecloc, elemmt )

         case (401, 403, 405, 407, 423, 424, 425, 437)

!        --- Type 401, 403, 405, 407, 423, 424, 425, 437
!            line element for navier-stokes quadratic element

            call elm401

         case (406)

!        --- Type 406,  navier-stokes equation axi-symmetric with swirl

            call elm406 ( coeffarr, vecloc, elemmt )

         case (408)

!        --- Type 408  line element for navier-stokes  (Free boundary)
!                      quadratic element

            call elm408 ( coeffarr )

         case (420:422)

!        --- Type 420:422  boussinesq-stokes equation

            call elm420 ( vecloc, coeffarr, elemmt )

         case (426:428, 438)

!        --- Type 426:428, 438
!            line element for navier-stokes  Temperature part quadratic element

            call elm426 ( vecloc, coeffarr )

         case (501)

!        --- Type 501 mass matrix  ( linear triangle  )

            call elm501 ( coeffarr )

         case (600)

!        --- Type 600
!            second order elliptic equation General using spectral elements

            call elm600 ( coeffarr )

         case (601)

!        --- Type 601

            call elm601 ( coeffarr )

         case (603)

!        --- Type 603
!            Spectral elements, boundary element for a scalar

            call elm603 ( coeffarr )

         case (606)

!        --- Type 606
!            Navier-Stokes equation (2D) with spectral elements

            call elm606 ( coeffarr )

         case (607)

!        --- Type 607
!            Spectral elements, Convection-diffusion of a scalar; 3-D case

            call elm607 ( coeffarr )

         case (608)

!        --- Type 608
!            Spectral elements, boundary element for a scalar; 3-D case

            call elm608 ( coeffarr )

         case (609)

!        --- Type 609

            call elm609 ( coeffarr )

         case (612)

!        --- Type 612

            call elm612 ( coeffarr )
            compts = .false.

         case (615)

!        --- Type 615

            call elm615 ( coeffarr )

         case (650:652)

!        --- Type 650,651652 mixed element
!            Special ground water flow element, using mixed finite elements

            call elm650

         case (661, 665)

!        --- Type 665, 661 mixed boundary element

            call elm665

         case (700)

!        --- Type 700 : Laplace equation

            call sepelm700

         case (701)

!        --- Type 701 : Poisson equation

            call sepelm701

         case (702)

!        --- Type 702 : Diffusion equation

            call sepelm702

         case (703)

!        --- Type 703 : Convection-diffusion equation

            call sepelm703
            symm = E%symm

         case (705, 800)

!        --- Type 705 : General second order elliptic equation 1 unknown

            call sepelm705
            symm = E%symm

         case (801)

!        --- Type 801 : Boundary conditions for general second order elliptic
!                       equation

            call sepelm801

         case (802)

!        --- Type 802 : Delta function

            call sepelm802

         case (803)

!        --- Type 803 : Special nonlinear second order elliptic equation

            call sepelm803

         case (804)

!        --- Type 804 : Boundary conditions for general second order elliptic
!                       equation discontinuous

            call sepelm804

         case (805)

!        --- Type 805 : General second order elliptic equation with two unknowns
!                       per point

            call sepelm805

         case (806)

!        --- Type 806 : Boundary conditions for general second order elliptic
!                       equation with two unknowns per point

            call sepelm806

         case (807)

!        --- Type 807 : Special element for enthalpy equation

            call elm807 ( vecloc, coeffarr(ipcoef) )

         case (808)

!        --- Type 808 : General second order elliptic equation with two unknowns
!                       per point

            call elm808 ( vecloc, coeffarr )

         case (810)

!        --- Type 810 : Special element for enthalpy equation
!                       Nedjar approach

            call elm810 ( coeffarr, vecloc )
            symm = .true.

         case (811)

!        --- Type 811 : Compute psi at surface

            call elm811
            symm = .false.

         case (812)

!        --- Type 812 : Smoothing of surface

            call elm812
            symm = .true.

         case (813)

!        --- Type 813 : second order elliptic equation with special rhs

            call elm813 ( vecloc, coeffarr(ipcoef), E%iflipflop )

         case (814)

!        --- Type 814 : Minimal surface problem

            call elm814

         case (815)

!        --- Type 815 : make phi a distance function

            if ( ifirst==0 ) call elm815parms ( iuser )
            call elm815 ( vecloc, coeffarr(ipcoef), E%iflipflop )

         case (816)

!        --- Type 816: special right-hand side

            call elm816 ( vecloc, coeffarr(ipcoef) )

         case (850)

!        --- Type 850 : Stochastic investment

            call elm850 ( vecloc, coeffarr(ipcoef) )
            symm = .false.

         case (410, 900:902, 904)

!        --- Type 410, 900, 901, 902
!            laminar, isothermal flow, penalty method and integrated method

            call sepelm900

         case (903)

!        --- Type 903  laminar, isothermal flow, integrated method
!                      Taylor Hood

            if ( ifirst==0 ) call elm903parms ( iuser )
            call elm903 ( vecloc, coeffarr(ipcoefnodes), coeffarr(ipcoef) )

         case (905)

!        --- Type 905  laminar, isothermal flow, pressure correction method
!                      Taylor Hood

            call elm905 ( vecloc, coeffarr(ipcoef) )

         case (906)

!        --- Type 906  laminar, isothermal flow, pressure correction method
!                      Taylor Hood (pressure matrix)

            call elm906

         case (907)

!        --- Type 907  Natural bounday condition for type 906

            call elm907 ( vecloc, coeffarr )

         case (910, 911)

!        --- Type 910, 911  laminar, isothermal flow, boundary conditions

            call elm910 ( elemmt, vecloc, &
                          coeffarr, coortr, max(1,maxtrn) )

         case (912)

!        --- Type 912  Special line element for given mass flow

            call elm912 ( coeffarr )

         case (914)

!        --- Type 914  laminar, isothermal flow, discontinuous boundary
!                      conditions

            call elm914 ( vecloc, coeffarr )
            transform = .false.

         case (915)

!        --- Type 915  sepcial element for free surface computation
!                      restricted to die-swell, 2d, quadratic

            call elm915 ( vecloc, coeffarr )

         case (922)

!        --- Type 922  laminar, isothermal flow, discontinuous boundary
!                      conditions

            call elm922 ( rprobz )

         case (924, 928)

!        --- Type 924, 928  special fluid-structure coupling element

            call elm924

         case (995)

!        --- Type 995  special element to copy reaction force (1 unknown1)

            call elm995

         case (996)

!        --- Type 996  special element to copy reaction force (ndim unknowns)

            call elm996

         case (997)

!        --- Type 997  special contact element

            call elm997

         case (998)

!        --- Type 998  special friction element

            call elm998

         case default

!        --- Type<0 or>999
!            These types are not available

            call errint ( itype, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 43, 2, 0, 0)

      end select  ! case ( itype )

!     --- Test complts

      if ( compts .and. .not. compl ) then
         call errint ( itype, 1 )
         call errint ( ielgrp, 2 )
         call errint ( ielem, 3 )
         call errsub ( 50, 3, 0, 0 )
      end if
      if ( .not. compts .and. compl ) then
         call errint ( itype, 1 )
         call errint ( ielgrp, 2 )
         call errint ( ielem, 3 )
         call errsub ( 49, 3, 0, 0 )
      end if

      notmat = max(notmat,E%notmat)
      notvec = max(notvec,E%notvec)

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemmt, icount, icount, 'elemmt' )
         call prinrl ( elemvc, icount, 'elemvc' )
         write(irefwr,*) 'End elmsubs'

      end if  ! ( debug )
      call erclos ( 'elmsubs' )

      end subroutine elmsubs
