      subroutine ercreatquad ( vector, nodes, n, kdeg, iposmax, rincre, &
                               kfun, kfill, icompl, inpcre )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 12-04-2012 Change of parameter list (inpcre)
!        version  2.5  date 23-12-2011 Replace er0074 by ercurvenodes
!        version  2.4  date 14-07-2011 Extension with kfill = 22
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill boundary conditions that are defined by a quadratic polynomial
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
!     quadratic
!     polynomial
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, nodes(n), kdeg, iposmax, inpcre(*), &
              kfun, kfill, icompl
      double precision vector(*), rincre(*)

!     icompl         i    Multiplication factor for complex numbers
!                         Real: icompl=1,   Complex: icompl=2
!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!                         The first position must refer to this subroutine
!     iposmax        i    Position in array rincre containing the first real
!                         corresponding to this boundary condition
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         Possible values:
!                         2:  Fill in curves
!                         3:  Fill in surfaces
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         9001:  Fill quadratic boundary condition along curve
!                         9002:  Fill half-quadratic boundary condition along
!                                curve
!                         9003:  Fill quadratic boundary condition with
!                                boundary layer along circular surface
!                         9004:  Fill quadratic boundary condition with
!                                boundary layer and temperature difference
!                                along circular surface
!     n              i    number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be filled
!     rincre         i    Double precision array containing the values of the
!                         constants corresponding to the boundary condition
!     vector        i/o   Array to be filled with the boundary condition
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcurves
      parameter ( maxcurves=10 )
      integer ncurves, numpoints, curvenodes(npoint), isurnr, &
              pointers(maxcurves+1), icurvs(maxcurves+1), i, nparts, &
              nsurfaces

!     curvenodes     Contains the nodes of the boundary curve
!     i              Counting variable
!     icurvs         array containing the curve numbers of the boundary of the
!                    surface
!     isurnr         Surface sequence number
!     maxcurves      Maximum number of curves allowed, &
!                    dimension of icurvs
!     ncurves        Number of curves
!     nparts         number of parts the boundary consists of
!     nsurfaces      Number of surfaces
!     numpoints      Number of points in curves
!     pointers       Gives start pointers of curves in array icurvs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCURVENODES   Store all nodes in one curves in array inodes and gaps in
!                    these curves in array ipoint
!     ERFILLQUADSUR  Fill boundary condition of the shape quadratic function
!                    with given mean value and boundary layer width along a
!                    surface
!     ERFILQUA       Fill quadratic profile along curve (real case)
!     ERFILQUC       Fill quadratic profile along curve (complex case)
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINKMESHCUR   Print the contents of array kmeshm for one curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2334   Part of outer boundary not closed
!    2633   The possibility to give the mean value for a quadratic function
!           along the boundary has not yet been implemented in the case of
!           curves.
!    2634   The possibility to give a quadratic function along a surface
!           has not yet been implemented in the complex case.
!    2635   The possibility to give a quadratic function has only been
!           implemented for a curve or circular surface.
!    2636   The possibility to give a quadratic function with given maximum
!           value along a circular surface has not yet been implemented.
!    2780   parallel version not yet available
!    3099   nsurfaces>1, not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatquad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatquad'
         write(irefwr,1) 'iposmax', iposmax
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( parallel ) then

!     --- Parallel implementation not yet available

         call errsub ( 2780, 0, 0, 0 )
         go to 1000

      end if  ! ( parallel )

      if ( icompl==0 ) then

!     --- Real case

         if ( kfill==2 .or. kfill==22 ) then

!        --- kfill = 2, curves

            if ( kfun>9002 ) then

               call errint ( kfun, 1 )
               call errsub ( 2633, 1, 0, 0 )
               go to 1000

            end if  ! ( kfun>9002 )

            call erfilqua ( vector, nodes, n, kdeg, iposmax, rincre, kfun )

         else if ( kfill==3 .or. kfill==24 ) then

!        --- kfill = 3, 24:  surfaces

            if ( kfun<9003 ) then

               call errint ( kfun, 1 )
               call errsub ( 2636, 1, 0, 0 )
               go to 1000

            end if  ! ( kfun<9003 )

!           --- Find boundary nodes corresponding to surface
!               This must be a surface

            nsurfaces = inpcre(1)
            if ( nsurfaces>1 ) then

!           --- Error: nsurfaces>1 not yet implemented

               call errsub ( 3099, 0, 0, 0 )
               go to 1000

            end if  ! ( nsurfaces>1 )
            isurnr = inpcre(3)
            if ( debug ) write(irefwr,1) 'isurnr', isurnr
            ncurves = kmeshs(isurnr+1)-kmeshs(isurnr) ! number of curves
                                                      ! corresponding to isurnr
            call inichkarlen ( ncurves, maxcurves, 'icurvs' )
            if ( ierror/=0 ) go to 1000

!           --- Store all nodes of the curve in curvenodes and the pointers
!               in array pointers
!               First store curve numbers in icurvs from position 2

            icurvs(1) = ncurves
            icurvs(2:ncurves+1) = &
               kmeshs(kmeshs(isurnr)+1:kmeshs(isurnr)+ncurves)

            if ( debug ) then

               call prinin ( icurvs(2), ncurves, 'icurvs' )
               do i = 1, ncurves
                  call prinkmeshcur ( abs(icurvs(i+1)), kmeshm )
               end do  ! i = 1, ncurves

            end if  ! ( debug )

!           --- Next store node numbers

            nparts = 1
            call sepcurvenodes ( icurvs, pointers, curvenodes, nparts, kmeshm )
            if ( ierror/=0 ) go to 1000

            if ( debug ) then
               write(irefwr,*) 'nparts, ncurvs ', nparts, ncurvs
               call prinin ( pointers, nparts+1, 'pointers' )
               call prinin ( curvenodes, pointers(nparts+1)-1, &
                             'curvenodes' )
            end if  ! ( debug )

            if ( ierror/=0 ) go to 1000
            if ( nparts>1 ) then

!           --- Error: curve is not closed

               call errint ( isurnr, 1 )
               call errint ( kmeshs(kmeshs(isurnr)+1), 2 )
               call errsub ( 2334, 2, 0, 0 )
               go to 1000

            end if  ! ( nparts>1 )
            numpoints = pointers(nparts+1)-pointers(nparts) ! number of points

            if ( curvenodes(numpoints)==curvenodes(1) ) &
               numpoints = numpoints-1

            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'numpoints ', numpoints
               call prinin ( curvenodes, numpoints, 'curvenodes' )

            end if  ! ( debug )

!           --- Fill quadratic boundary function in vector

            call erfillquadsur ( n, nodes, rincre(iposmax), kfun, &
                                 kdeg, vector, numpoints, &
                                 curvenodes, isurnr )
            if ( ierror/=0 ) go to 1000

         else

!        --- Only curves or surfaces are allowed in this case

            call errint ( kfun, 1 )
            call errsub ( 2635, 1, 0, 0 )
            go to 1000

         end if  ! ( kfill/=2 )

      else

!     --- Complex case

         if ( kfill/=2 ) then

!        --- Only curves are allowed in this case

            call errint ( kfun, 1 )
            call errsub ( 2634, 1, 0, 0 )
            go to 1000

         end if  ! ( kfill/=2 )

         if ( kfun>9002 ) then

            call errint ( kfun, 1 )
            call errsub ( 2633, 1, 0, 0 )
            go to 1000

         end if  ! ( kfun>9002 )

         call erfilquc ( vector, nodes, n, kdeg, iposmax, rincre, kfun )

      end if  ! ( icompl==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatquad'

      end if
      call erclos ( 'ercreatquad' )

      end
