      subroutine ercreatsub1 ( isolut, inpcre, ifirst, ipstin, ivecsq, &
                               nv, kvec, isubr, ichvc, iprob, ivec, &
                               icompl, ifill, itypvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-13-2013 Extension with change_vector
!        version  1.0  date 27-12-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters for one loop in create vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isolut(*), inpcre(*), ifirst, ipstin, ivecsq, nv, kvec, &
              isubr, ichvc, iprob, ivec, icompl, ifill, itypvc

!     ichvc          o    Indication of the type of vector to be created.
!                         Possible values
!                         0: Vector either exists or is a solution vector
!                         1: Vector is a solution vector
!                         2: Vector is an array of special structure
!                         3: Vector is an array of special structure defined per
!                            element
!                         4: Vector is an array of special structure defined per node
!                            per element
!                         5: Vector with capacities
!                         6: Vector exists and must not be cleared
!     icompl         o    Indication if the vector to be created is real (0) or
!                         complex
!     ifill          o    Indicates how the vector must be filled.
!                         Possible values:
!                         -1  The corresponding vector is not filled, &
!                             only created
!                          0  The corresponding vector is created and made
!                             equal to zero
!                         m>0 The filling of the vector is defined by m commands
!     ifirst         i    Value of first position of array INPCRE
!     inpcre         i    The integer input read is stored in array INPCRE from
!                         position 1. See subroutine creatv01
!     iprob          o    Actual problem number
!     ipstin        i/o   Next address in array INPCRE
!     isolut         i    Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     isubr          i    Indicates by what subroutine creatv01 has been called.
!                         Possible values:
!                         1:  By the user or CREATE
!                         2:  By PRESDF or PRESTM
!     itypvc         o    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     ivec           o    Gives the sequence number of the array of special
!                         structure if ICHVC = 2, &
!                         or the number of degrees of freedom per element if
!                         ICHVC = 3 or the number of degrees of freedom per node if
!                         ICHVC = 4
!     ivecsq         i    Sequence number of vector in isolut
!     kvec           i    Sequence number over the vectors
!                         Loop parameter in call ing subroutine
!     nv             i    number of calls to create vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nch

!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1114   Parameter ICHVC indicating the type of vector to be created out
!           of range
!    2400   Addition of sequence numbers may cause unwanted effects
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub1'
         write(irefwr,1) 'isubr, ifirst', isubr, ifirst
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute ICHVC, IPROB, IVEC, ICOMPL and IFILL

      ichvc  = 0
      iprob  = 1
      ivec   = 1
      icompl = 0
      ifill  = 0
      if ( ifirst>=ipstin+1 ) ichvc  = inpcre(ipstin+1)
      if ( ifirst>=ipstin+2 ) iprob  = inpcre(ipstin+2)
      if ( ifirst>=ipstin+3 ) ivec   = inpcre(ipstin+3)
      if ( ifirst>=ipstin+4 ) icompl = inpcre(ipstin+4)
      if ( ifirst>=ipstin+5 ) ifill  = inpcre(ipstin+5)

      if ( debug ) &
         write(irefwr,1) 'ipstin, ichvc, iprob, ivec, icompl, ifill', &
                          ipstin, ichvc, iprob, ivec, icompl, ifill
      ipstin = ipstin + 5

      if ( ichvc<0 .or. ichvc>5 ) then

!     --- Error: ichvc out of range

         call errint ( ichvc, 1 )
         call errsub ( 1114, 1, 0, 0 )

      end if  ! ( ichvc<0 .or. ichvc>5 )

      if ( ivecsq>0 .and. nv>1 .and. ifill==-1 ) then

!     --- Possibly wrong vector to be filled

         nch = index(names(kvec),' ')-1
         if ( nch<=0 ) nch = len(names(kvec))
         call errchr ( names(kvec)(1:nch), 1 )
         call errint ( kvec, 1 )
         nch = index(names(ivecsq+1),' ')-1
         if ( nch<=0 ) nch = len(names(ivecsq+1))
         call errchr ( names(ivecsq+1)(1:nch), 2 )
         call errint ( ivecsq+1, 2 )
         call errint ( nv, 3 )
         call errwar ( 2400, 3, 0, 2 )

      end if  ! ( ivecsq>0 .and. nv>1 .and. ifill==-1 )

      itypvc = 0
      if ( isolut(kvec)>0 .and. isolut(kvec)<=lastsol ) &
         itypvc = ks(isolut(kvec))%typevec
      if ( isubr==8 .and. itypvc==115 .and. ichvc==1 ) then

         ichvc = 2

      else if ( isubr==9 ) then

!     --- vector exists and must not be cleared

         ichvc = 6

      end if  ! ( isubr==8 .and. itypvc==115 .and. ichvc==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'itypvc, kvec, iprob, ichvc', &
                          itypvc, kvec, iprob, ichvc
         write(irefwr,*) 'End ercreatsub1'

      end if  ! ( debug )
      call erclos ( 'ercreatsub1' )

      end
