      subroutine prov00check ( isol, irhsd, ichois, ikeuze, eps, iwork, &
                               jcart, n, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-12-2017 New storage of intmat
!        version  1.1  date 04-06-2014 Allow -102 in intmat(2)
!        version  1.0  date 02-07-2004
!
!   copyright (c) 2004-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Test the input for subroutine prov00 and set some parameters
!
! **********************************************************************
!
!                       KEYWORDS
!
!     constraint
!     linear_solver
!     overrelaxation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulesolve
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekmesh
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, irhsd, ichois, ikeuze, &
              iwork(*), jcart, n, number
      double precision eps

!     eps            i    Local accuracy, see prov00
!     ichois         i    Choice parameter, see prov00
!     ikeuze         o    ?
!     irhsd          i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     iwork          i    integer work array, see prov00
!     jcart          o    indication whether the element is a triangle with
!                         Cartesian coordinates (1)
!                         or a circular triangle with cylinder symmetric
!                         coordinates (2)
!     n              o    number of degrees of freedom
!     number         o    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, ivec
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!      94   jmethod has incorrect value
!     183   idegfd incorrect
!     240   Parameter eps is not positive
!     483   nmax has wrong value
!     484   omega has wrong value
!     485   alpha has wrong value
!     486   beta has wrong value
!     487   niter1 has wrong value
!     488   niter2 has wrong value
!     489   alam has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prov00check' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prov00check'

      end if
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      iprint = ichois/100
      iprint = (ichois-100*iprint)/10
      ikeuze = mod(ichois,10)
      if ( ikeuze<0 .or. ikeuze>1 ) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( iprint<0 .or. iprint>1 ) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( nmax<=0 ) then
         call errint ( nmax, 1 )
         call errsub ( 483, 1, 0, 0 )
      end if
      if ( abs(eps)<=0d0 ) then
         call erreal ( eps, 1 )
         call errsub ( 240, 0, 1, 0 )
      end if
      if ( omega<=0d0 .or. omega>=2d0 ) then
         call erreal ( omega, 1 )
         call errsub ( 484, 0, 1, 0 )
      end if
      if ( alpha<=-1d0 .or. alpha>=1d0 ) then
         call erreal ( alpha, 1 )
         call errsub ( 485, 0, 1, 0 )
      end if
      if ( beta<=-1d0 .or. beta>=1d0 ) then
         call erreal ( beta, 1 )
         call errsub ( 486, 0, 1, 0 )
      end if
      if ( niter1<-2 ) then
         call errint ( niter1, 1 )
         call errsub ( 487, 1, 0, 0 )
      end if
      if ( niter2<-2 ) then
         call errint ( niter2, 1 )
         call errsub ( 488, 1, 0, 0 )
      end if
      if ( niter1==-2 .and. alam<=0d0 .or. alam>=1d0 ) then
         call erreal ( alam, 1 )
         call errsub ( 489, 0, 1, 0 )
      end if

      call sepfillmatstr

!     --- initializations;   computing starting positions


      n = nusol-nbound-npboun
      if ( IM%symmetric ) then
         call errint ( 5, 1 )
         call errsub ( 94, 1, 0, 0 )
      end if
      jcart = ks(irhsd)%jcart
      call sepcreasol110 ( isol, 'usol', ks(irhsd)%compl )
      ks(isol)%jcart = ks(irhsd)%jcart
      number = iwork(1)
      if ( number<0 .or. number>nphys ) then
         call errint ( number, 1 )
         call errint ( nphys, 2 )
         call errsub ( 183, 2, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prov00check'

      end if
      call erclos ( 'prov00check' )

      end subroutine prov00check
