      subroutine sepblockgcr ( normrhsd, A, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using the
!     Block Triangular preconditioner.
!     The method is accelerated by the GCR process
!     Old subroutine prblockgcr
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpu
      use sepmodulesimple
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision normrhsd
      type (rmatrix) :: A
      double precision :: usol(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     normrhsd       i    Norm of right-hand side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, iter, niteru, niterp, &
              itimesav, niteruacc, niterpacc, &
              niteracc, niterutot, niterptot, ngcr
      double precision, allocatable :: svec(:,:), vvec(:,:)
      double precision rnorm, rferr, normprec
      logical converged
      double precision :: timebefore(2)

!     converged      If true the iteration process has been converged
!     iter           Iteration sequence number
!     itimesav       Help parameter to save the parameter itime
!     k              Counting variable
!     ngcr           Maximum number of GCR iterations
!     niteracc       Accumulated number of GCR iterations
!     niterp         Number of iterations to solve the p-equations
!     niterpacc      Accumulated number of GCR iterations to solve p-equations
!     niterptot      Total number of p iterations in all GCR steps
!     niteru         Number of iterations to solve the u-equations
!     niteruacc      Accumulated number of iterations to solve the u-equations
!     niterutot      Total number of u iterations in all GCR steps
!     normprec       Norm of preconditioned residual
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rnorm          Norm of residual
!     svec           Work array of size ngcr x nu to store the vectors
!                    s during the iteration
!     vvec           Work array of size ngcr x nu to store the vectors
!                    v during the iteration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prpardnrm2

!     ASKCLK         print elapsed time
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRINRL         Print 1d real vector
!     PRPARDNRM2     Parallel Blas routine to compute the 2-norm of a vector
!     SEPGCRCYCLE    Perform one step of the gcr process
!     SEPSIMPLECONVG Compute norm of residual and adapt accuracy if necessary
!     SEPSIMPLESTART Compute initial norm, adapted eps and print start
!     SEPTRIITER     Perform a block triangular iteration as part of
!                    some outer loop
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data niteruacc /0/, niterpacc /0/, niteracc /0/
! ======================================================================
!
      call eropen ( 'sepblockgcr' )
      call ersettime ( timebefore )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepblockgcr'
         write(irefwr,1) 'nu, np ', nu, np
         call prinrl ( resup, nu+np, 'resup' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ngcr = inpsol(13,1)   ! (maximum) number of gcr iterations

      allocate(svec(nu+np,0:ngcr+1) , vvec(nu+np,0:ngcr+1), &
               stat = error)
      if ( error/=0 ) call eralloc ( error, 2*(nu+np)*(ngcr+2), 'svec' )
      if ( ierror/=0 ) go to 1000

      converged = .false.
      itimesav = itime
      itime = 0
      niterutot = 0
      niterptot = 0

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) then
         write ( irefwr, 100 )
100      format(//' Output for the BLOCK_TRIANGULAR preconditioner'//)
      end if  ! ( A%iprint>0 ) then

!     --- Compute initial norm, adapted eps and print start
!         Produce matlab output if required

      call sepsimplestart ( rnorm, rferr, normrhsd, A )

      if ( A%irelerr/=0 .and. rnorm<=A%eps ) then

!     --- Initial residual already small

         go to 1000

      end if

!     --- For k := 0 (1) A%nmax-1 do

      do k = 0, A%nmax-1

!     --- Perform one step of simple to compute du and dp
!         s^(k+1) := (du,dp), r = (ru,r_p)

         iter = k
         call septriiter ( svec(1,0), A )
         if ( ierror/=0 ) go to 1000

         niteru = Q%numiter
         niterp = R%numiter
         niterutot = niterutot+niteru
         niterptot = niterptot+niterp

         normprec = norm ( svec(1,0), A )

         if ( iter==0 .and. A%irelerr==1 ) then

!        --- A%irelerr = 1 and iter=0, we have to adapt the accuracy

            A%eps = A%eps*normprec

         end if  ! ( iter==0 .and. A%irelerr==1 )

!        --- Perform one step of the gcr process.

         vvec(:,0) = A*svec(:,0)
         call sepgcrcycle ( svec, vvec, ngcr, k, rnorm, resup, usol, A )
         if ( ierror/=0 ) go to 1000

         call sepsimpleconvg ( iter, rferr, converged, niteru, niterp, &
                               rnorm, normprec, A )

         if ( converged ) go to 500

      end do  ! k = 0, A%nmax-1

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

      call errint ( iter, 1 )
      if ( inpsol(22,1)==1) then
         inpsol(1,1) = iter
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

500   niteracc = niteracc + iter
      niteruacc = niteruacc + niterutot
      niterpacc = niterpacc + niterptot

      if ( A%iprint>=1 ) then

!     --- A%iprint = 1  Print information

         write ( irefwr, 510 ) iter, rnorm, niterutot, niterptot, &
                               niteracc, niteruacc, niterpacc
510      format('The number of executed iterations in the', &
                ' GCR-process is', i5/ &
                'The 2-norm of the residual is:', ss,1pe12.2e2/ &
                'The number of u iterations is ', i6/ &
                'The number of p iterations is ', i6/ &
                'The accumulated number of GCR-iterations is', i8/ &
                'The accumulated number of u-iterations is', i8/ &
                'The accumulated number of p-iterations is', i8 )

      end if

!     --- Fill information in citerinfo
!         This is only used for performance checking

      niterout = niteracc
      niteruout = niteruacc
      niterpout = niterpacc
      call askclk
      timesubr = tim3-timebefore(1)

1000  deallocate ( svec, vvec, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'svec' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepblockgcr'

      end if  ! ( debug )
      call erclos ( 'sepblockgcr' )
      itime = itimesav

      end
