      subroutine sepcbicgstab ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!     Apply the Bi-CGSTAB preconditioned method of conjugate gradients to solve
!     the linear system of equations AMAT x = RHSD      ( A x = b )
!     where the matrix A and the preconditioning matrix M may be unsymmetric
!        ( see Report of Henk A. Van der Vorst, 8 July 1990 )
!                                                    T
!     matrix A and the preconditioning matrix K = L U  may be unsymmetric.
!     This subroutine contains the following algorithms:
!     -  Standard Bi-CGSTAB method
!     -  Right-preconditioned Bi-CGSTAB method
!     -  Eisenstat version of the left-right-preconditioned Bi-CGSTAB method
!     Both in the case of the preconditioned Bi-CGSTAB method
!     as in the case of the Eisenstat implementation, the matrix and right-hand
!     side are supposed to be premultiplied by the inverse of the diagonal D
!     of the triangular factors from the preconditioning matrix in the
!     following way:  ( A x = b )
!               -1        -1
!          A = D   A     D
!                   orig
!               -1
!          b = D   b
!                   orig
!     If the initial estimate x is given it must also be premultiplied by D, &
!     before PRCGST is called:
!          x = D  x
!                  orig
!                                                        -1
!     At output the solution x must be premultiplied by D   in order to get
!     the solution of the original problem, after go to 1000 from PRCGST
!                  -1
!         x     = D   x
!          orig
!     In the case of Eisenstat the matrix amat is supposed to contain the
!     matrix H = D  - 2I in the first n positions, where D  is the diagonal
!                 A                                       A
!     of A after the premultiplication
!        algorithm:  Bi-CGSTAB  ( Henk A. Van der Vorst, Report July 8, 1990 )
!                    left-right-preconditioning for Eisenstat version, &
!                    right-preconditioning      for other cases
!    adapted from the CGS-subroutine prcgs of
!    Guus Segal, Rik Kaasschieter, Henk van der Vorst
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: n, nglobal, nmax, ifiter, countres, &
                 imin, iter, j, ireturn, priter(10)
      double precision :: rnorm, rmin, rferr, bnorm, residu(10)
      double complex :: rho, rhoold, beta, omega, sigma, ss, tt, alpha
      double complex, allocatable :: res(:), p(:), v(:), r0bar(:), &
                                       t(:), xmin(:), pbar(:), diag(:)
      logical :: repeat1, restart, finished, extradebug

!     alpha          Factor alpha in the CG-process ( (r,r) / (p,Ap) )
!     beta           Factor beta in the CG-process ( (r ,r ) / (r   ,r   ) )
!                                                      i  i      i-1  i-1
!     bnorm          Norm of the right-hand side (b,b)
!     countres       Counter for number of restarts
!                    they are not
!     diag           Diagonal of the matrix
!     extradebug     If true extra debugging statements are carried out
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     ifiter         Starting value of parameter iter
!     imin           Iteration with minimal norm
!     ireturn        Return code:
!                    0: continue process
!                    1: restart process
!                    2: end of process
!     iter           Iteration sequence number
!     j              Counting variable
!     n              size of matrix and vectors
!     nglobal        Global number of unknowns. This is an overestimate, &
!                    because unknowns on interfaces are summed up.
!     nmax           The maximum number of iterations that may be performed
!     omega          ( t, s ) / ( t, t )
!     p              Work array of length n to store the direction vector
!     pbar           Work array of length n to store the vector pbar, &
!                    needed only for iprec = 2, 3 or 5, &
!                    the vector sbar(*) shares this memory
!     priter         Output array to print residuals
!     r0bar          Work array of length n to store the vector r0bar.
!     repeat1        Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    cnorm of the solution could not be computed
!     res            Work array of length n to store the residual vector r, &
!                    the vector s(*) shares this memory
!     residu         Output array to print residuals
!     restart        If true the recomputation of rho has not been started
!     rferr          Maximal accuracy possible for this matrix, due to round-off
!     rho            Inner product ( r0bar, r )
!     rhoold         Inner product ( r0bar, r ) from the previous iteration
!     rmin           minimal cnorm of residual
!     rnorm          cnorm of residual
!     sigma          Factor sigma in the CG-process (inner product of p and Ap)
!     ss             ( s, s )
!     t              Work array of length n to store the vector t
!     tt             ( t, t )
!     v              Work array of length n to store the vector v
!     xmin           Work array of length n to store the vector x with
!                    the smallest residual norm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINCOMPL      Print 1d complex vector
!     PRINTITER      Fill arrays priter and residu
!     SEPCCGESTEPS   Print first line of iteration output
!                    Compute first estimate of accuracy  eps
!     SEPCCGSTART    Fill starting value in array x and compute initial residual
!     SEPCCHECKACCUR Check on accuracy reached in step iter-1 of bi-cgstab
!     SEPCMAXACC     Compute maximum accuracy possible in iterative solver
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!    1400   Bi-CGSTAB process has terminated because sigma = 0
!    1401   Bi-CGSTAB process has terminated because cnorm of vector t = 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If no preconditioning is used then K = I in the right-preconditioned
!     process.
!     The right-preconditioned Bi-CGSTAB process
!                       ( see Henk A. Van der Vorst, Report July 8, 1990 )
!     and the Eisenstat version of left-right-preconditioned Bi-CGSTAB
!     process can be written as:
!     first approximation x is given
!                                             -1
!     r = b - A x ,  but for Eisenstat:  r = L  * ( b - A x )
!     r0bar = r
!     Iteration: k = 1 (1) nmax  while ||r||>eps
!       rho = ( r0bar, r )
!       If ( k = multiple of sqrt(n) )
!          check error level
!       End
!       If ( k=1 )
!          p = r
!       Else ( k>1 )
!          beta = ( rho / rhoold ) * ( alpha / omega )
!          p = r + beta * ( p - omega * v )
!       End
!               -1                      T                           -T
!       pbar = K  * p ,  where K = L * U ,   but Eisenstat: pbar = U   * p
!                                                  -1
!       v = A * pbar,   but Eisenstat: v = pbar + L  * ( H * pbar + p )
!       sigma = ( r0bar, v )
!       alpha = rho / sigma
!       First update of x:  x = x + alpha * pbar
!       s = r - alpha * v
!               -1                      T                           -T
!       sbar = K  * s ,  where K = L * U ,   but Eisenstat: sbar = U   * s
!                                                  -1
!       t = A * sbar,   but Eisenstat: t = sbar + L  * ( H * sbar + s )
!       omega = ( t, s ) / (||t|| ||s||)
!       omega = sign(omega)max(|omega|,0.7) ||s||/||t||
!       Second update of x:  x = x + omega * sbar
!       r = s - omega * t
!       rhoold = rho
!    End
!    Check error level:
!                         2
!            2       (k/n)                2
!    If ||r||<exp       || Ax -b -r ||   ,  but Eisenstat:
!                               -1              2
!         ...      ...      || L  ( Ax -b) -r ||  ,
!    then maximal accuracy has been reached.
!    Notice:  s           can share memory with r,
!             sbar        can share memory with pbar,
!             sbar, pbar  are not needed when K = 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcbicgstab' )
      debug = .false.! .and. ioutp>=0
      extradebug = .false.

      n = A%neq

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcbicgstab'
         beta = (0d0,0d0)
         if ( extradebug ) call princompl ( b, n, 'rhsd' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      bnorm = cnorm(b,n)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x = 0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      nglobal = n                    ! number of global unknowns
      nmax = A%nmax                  ! maximum number of iterations
      if ( nmax==0 ) nmax=10*nglobal

      if ( debug ) &
         write(irefwr,1) 'iprec, iprint', A%type_prec, A%iprint

      allocate ( res(n), p(n), v(n), r0bar(n), t(n), xmin(n), pbar(n), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, n*7, 'res' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'nmax, iprint', nmax, A%iprint
         write(irefwr,*) 'eistat, precon ', A%eistat, A%precon
      end if  ! ( debug )

!     --- Compute maximum accuracy possible

      call sepcmaxacc ( A, b, t, bnorm, rferr )
      if ( debug ) write(irefwr,2) 'rferr', rferr

      if ( A%eistat ) then

!     --- Eisenstat preconditioning, fill diag

         allocate ( diag(A%neq) )

         do j = 1, A%neq
            diag(j) = A%S(A%diag(j))
         end do  ! j = 1, A%neq

      end if  ! ( A%eistat )

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr,100 )
100   format(//' Output for the Bi-CGSTAB method', &
               ' (complex non-symmetric matrix)'//)

      ifiter = 1
      countres = 0

      restart = .true.

!     --- Set residual res = b, it will be updated when x # 0, label 110
!         is used when the method is restarted

110   call sepccgstart ( A, x, b, res, r0bar )
      if ( extradebug ) call princompl ( res, n, 'residual' )
      countres = countres+1

      if ( ifiter==1 ) xmin = x  ! store first iteration in xmin

      r0bar = res ! copy of residual

!     --- Set constants
!         Norm of the initial residual res

      rnorm = cnorm ( res, n )
      rmin  = rnorm
      imin  = ifiter
      if ( debug ) write(irefwr,2) 'rnorm', rnorm

!     --- Print first line of iteration output
!         Compute first estimate of accuracy eps

      iter = ifiter
      call sepccgesteps ( A, b, t, rnorm, rferr, repeat1, iter )
      if ( debug ) &
         write(irefwr,2) 'rnorm, eps, eps1, eps2', &
                          rnorm, A%eps, A%eps1, A%eps2

      finished = .false.

      if ( ifiter==1 .and. rnorm<A%eps ) then

!     --- first iteration, but already accuracy reached
!         process finished

         nmax = 1
         A%numiter = 1
         if ( A%iprint>=1 ) then

!        --- iprint>=1  Print information

            write ( irefwr,150 ) nmax, rnorm

         end if  ! ( ifiter==1 .and. rnorm<A%eps )
         finished = .true.

      end if  ! ( ifiter==1 .and. rnorm<A%eps )

      do while ( .not. finished .and. iter<=nmax+1 )

         if ( debug ) write(irefwr,1) 'iter', iter

!        --- Check on accuracy reached in iter-1

         call sepccheckaccur ( A, x, b, res, iter, ifiter, &
                               rnorm, repeat1, rferr, &
                               imin, ireturn, nmax, rmin, priter, &
                               residu, xmin, x, x )
         if ( ireturn==1 ) then
            go to 110  ! restart method
         else if ( ireturn==2 ) then
            finished = .true.
            exit  ! method has finished
         end if  ! ( ireturn==1 )

         if ( iter>nmax ) then
            finished = .true.
            exit  ! do not start (nmax+1)-th iteration
         end if  ! ( iter>nmax )

!        --- Compute rho = ( r0bar, res )

         rho = dot_product(r0bar, res)
         if ( debug ) write(irefwr,2) 'rho', rho

         if ( iter>1 ) then

!        --- rhoold is available
!            Correction similar to correction in PRCGS of June 23, 1997

            if ( restart .and. abs(rhoold)*epsmac*1d2>abs(rho) ) then

!           --- rho too small, restart the process

               ifiter = iter

               restart = .false.
               go to 110

            end if

         end if

         if ( iter==ifiter ) then

!        --- the method is (re)started,  copy res into p

            p = res

         else

!        --- k>1
!            beta = ( rho / rhoold ) * ( alpha / omega )
!            p = res + beta * ( p - omega * v )

            beta = ( rho / rhoold ) * ( alpha / omega )
            if ( debug ) write(irefwr,2) 'beta', beta

            p = res + beta * ( p - omega * v )

         end if
         if ( debug ) write(irefwr,2) 'beta', beta

         if ( extradebug ) call princompl ( p, n, 'p' )

!        --- pbar = K^(-T) * p ,  but Eisenstat:  pbar U^(-T) * p
!            v = A * pbar  ,  but Eisenstat:  v = pbar + L^(-1) * ( H*pbar*p )

         if ( A%eistat ) then
            pbar = p/A
            v = p + diag * pbar
            v = A/v
            v = v+pbar
         else if ( A%precon ) then
            pbar = p/A
            v = A*pbar
         else
            pbar = p
            v = A*p
         end if  ! ( A%eistat )

         iter = iter+1
         if ( extradebug ) call princompl ( v, n, 'v' )

!        --- Compute sigma = ( r0bar, v ),  alpha = rho / sigma

         sigma = dot_product(r0bar, v)
         if ( abs(sigma)<=0d0 ) then

!        --- Sigma = 0, Process can not be continued

            if ( countres<=10 ) then

!           --- countres<=10 restart the process

               ifiter = iter
               go to 110

            else

!           --- countres>10 terminate

               call errsub ( 1400, 0, 0, 0 )

               A%numiter = iter
               A%itererr = 2
               finished = .true.
               exit

            end if  ! ( countres>=10 )

         end if

         alpha = rho / sigma
         if ( debug ) write(irefwr,2) 'alpha, sigma', alpha, sigma

!        --- First update of x:  x = x + alpha * pbar , to release pbar

         x = x + alpha*pbar

         if ( extradebug ) call princompl ( x, n, 'x' )

!        --- Compute s = r - alpha * v  ,
!            r  can be overwritten by s(*)

         res = res - alpha*v
         ss = dot_product ( res, res )
         if ( debug ) write(irefwr,2) 'ss', ss

!        --- sbar = K^(-1) * s ,  but Eisenstat:  sbar = U^(-T) * s
!            t = A * sbar  ,  but Eisenstat:  t = sbar + L^(-1) * ( H*sbar + s )
!            sbar is stored in pbar, s is stored in res

         pbar = res/A

         if ( A%eistat ) then
            t = res + diag * pbar
            t = A/t
            t = t+pbar
         else
            t = A*pbar
         end if  ! ( A%eistat )

         iter = iter+1

!        --- Compute omega  = ( t, s ) / ( t, t ),  s is stored in r(*),

         tt = dot_product ( t, t )
         if ( debug ) write(irefwr,2) 'tt', tt
         if ( abs(tt)<=0d0 ) then

!        --- t = 0,  process terminates

            rnorm  = cnorm ( res, n )
            if ( rnorm>epsmac ) then

!           --- rnorm>epsmac, hence not the special case of zero norm

               call errsub ( 1401, 0, 0, 0 )

               A%numiter = iter-1
               A%itererr = 2
               finished = .true.
               exit

            else

!           --- rnorm = 0, skip stabilization

               go to 300

            end if  ! ( rnorm>epsmac )

         end if  ! ( abs(tt)<=0d0 )

!        --- Stabilization as described in Sleijpen and v.d. Vorst (1995)

         omega = dot_product ( t, res ) / sqrt(tt*ss)
         omega = omega/abs(omega) * max(abs(omega),0.7d0) * sqrt(ss/tt)
         if ( debug ) write(irefwr,2) 'omega', omega

!        --- Second update of x:  x_k = x_k + omega_k * sbar ,
!            sbar is stored in pbar,  s is stored in res

         x = x + omega*pbar
         if ( extradebug ) call princompl ( x, n, 'x' )

!        --- Update r_k = s - omega_k * t,   s is stored in r(*)

         res = res - omega*t
         if ( extradebug ) call princompl ( res, n, 'res' )

!        --- Compute cnorm of residual

         rnorm = cnorm ( res, n )
         if ( debug ) write(irefwr,2) 'rnorm', rnorm

300      if ( rnorm<rmin ) then
            xmin = x
            rmin = rnorm
            imin = iter+1
         end if

         if ( A%iprint>1 ) &
            call printiter ( A%iprint, iter, rnorm, priter, residu, 2 )

         rhoold = rho

      end do  ! while ( .not. finished .and. iter<=nmax+1 )

      if ( .not. finished ) then

!     --- If the subroutine reaches this stage, too much  iterations have
!         been carried out

         if ( A%iprint==2 .and. iter>20 ) then

!        --- iprint = 2 Print final residuals

            do j = max(1,33-iter), 10
               write ( irefwr, 450 ) priter(j), residu(j)
450            format( i10, ss,1pe15.2e2 )
            end do

         end if

         if ( A%iprint>=1 ) then

!        --- iprint>=1  Print information

            write ( irefwr,150 ) nmax, rmin
            write ( irefwr,160 ) imin-1
150         format(' The number of executed iterations in the', &
                   ' Bi-CGSTAB process is', i5/ &
                   ' The 2-norm of the residual is:', ss,1pe12.2e2 )
160         format (' The minimal residual is obtained at', &
                    ' iteration', i5 )

         end if

         call errint ( nmax, 1 )

         A%numiter = nmax
         A%itererr = 1

         if ( A%aterror==1 ) then

            call errwar ( 676, 1, 0, 0 )

            if ( A%aterror>=1 .and. imin>1 ) then

               x = xmin

            end if

         else

            call errsub ( 676, 1, 0, 0 )

         end if

      end if  ! ( .not. finished )

      deallocate ( res, p, v, r0bar, t, xmin, pbar, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'res' )
      if ( A%eistat ) deallocate ( diag )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcbicgstab'

      end if  ! ( debug )
      call erclos ( 'sepcbicgstab' )

      end subroutine sepcbicgstab
