      subroutine sepccg ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Apply the preconditioned method of conjugate gradients to solve the
!     linear system of equations A x = y     ( A x = b )
!     where the matrix A and the preconditioning matrix M should be symmetric
!     and positive definite (Complex case)
!     This subroutine contains the following algorithms:
!     -  Standard conjugate gradient method
!     -  Preconditioned conjugate gradient method
!     -  Eisenstat implementation of the preconditioned conjugate gradient
!        method
!     Both in the case of the preconditioned conjugate gradients method, &
!     as in the case of the Eisenstat implementation, the matrix and right-hand
!     side are supposed to be premultiplied by the inverse of the diagonal D
!     of the preconditioning matrix in the following way:  ( A x = b )
!               -1        -1
!          A = D   A     D
!                   orig
!               -1
!          b = D   b
!                   orig
!     If the initial estimate x is given it must also be premultiplied by D:
!          x = D  x
!                  orig
!                                                        -1
!     At output the solution x must be premultiplied by D   in order to get
!     the solution of the original problem:
!                  -1
!         x     = D   x
!          orig
!     In the case of Eisenstat the matrix A is supposed to contain the
!     matrix K = D  - 2I in the first n positions, where D  is the diagonal
!                 A                                       A
!     of A after the premultiplication
!     Original subroutine: Rik Kaasschieter and Henk van der Vorst
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nglobal, isqrn, nmax, iter, n
      double precision :: bnorm, rhotest, r0norm, sqrn, &
                          rferr, bsnorm, xnorm
      double complex sigma, alpha, beta, rho, rhosta
      double complex, allocatable :: res(:), p(:), work(:), pbar(:)
      logical :: repeat1, finished, extradebug

!     alpha          Factor alpha in the CG-process ( (r,r) / (p,Ap) )
!     beta           Factor beta in the CG-process ( (r_i,r_i) / (r_i-1,r_i-1) )
!     bnorm          Norm of the right-hand side (b,b)
!     bsnorm         Preconditioned norm of the right-hand side
!                    (M^(-1)b,M^(-1)b)
!                    they are not
!     extradebug     If true extra debugging statements are carried out
!     finished       if true the iteration has finished
!     isqrn          Nearest integer to sqrt(n)
!     iter           Loop variable
!     n              Numer of equations
!     nglobal        Global number of unknowns. This is an overestimate,
!                    because unknowns on interfaces are summed up.
!     nmax           maximum number of iterations
!     p              Work array of length n to store the direction vector
!     pbar           Work array of length n to store the vector p^_. This
!                    vector is only used for the Eisenstat preconditioning
!     r0norm         norm of start residual
!     repeat1        Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    norm of the solution could not be computed
!     res            Work array of length n to store the residual vector
!     rferr          Maximal accuracy possible for this matrix, due to round-off
!     rho            Inner product between M^(-1) r and r at old level, i.e.
!                    M^(-1) norm of residual
!     rhosta         rho at new level
!     rhotest        norm of residual used to terminate CG
!     sigma          Factor sigma in the CG-process (inner product of p and Ap)
!     sqrn           Help variable to compute sqrt(n)
!     work           Work array of length n to store the help vectors
!     xnorm          Norm of the estimated solution
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex :: zcjdot

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     PRINCOMPL      Print 1d complex vector
!     SEPCCGACCUR    Fill starting value in array x and compute initial residual
!                    Perform extended print for subroutine sepcg
!     SEPCCGCHECKEPS rhotest is smaller than maximal accuracy possible
!     SEPCCGEPS      It is useless to continue the process, print
!     SEPCCGNORMS    Check if eps must be recomputed
!                    Compute eps as function of eps1 eps2 and norms
!     SEPCCGPRINT    ?
!     SEPCCGPRINT1   Compute some norms necessary for the termination criterion
!                    of the cg proces
!     SEPCCGPRINT2   Perform extended print for subroutine sepcg
!                    Perform extended print for subroutine prcg during iteration
!     SEPCCGRFERR    Perform printing CG process when iteration has not
!     SEPCCGSTART    converged
!     ZCJDOT         Conjugate complex inner product (BLAS routine)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     675   The CG-process has terminated because the matrix is not positive
!           definite
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccg' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
 !     debug = .true.

!     --- Get parameters

      n = A%neq
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccg'
         write(irefwr,1) 'n', n
         write(irefwr,2) 'eps, eps1, eps2', A%eps, A%eps1, A%eps2
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      nglobal = n                ! number of global unknowns
      nmax = A%nmax                  ! maximum number of iterations
      if ( nmax==0 ) nmax=10*nglobal

      if ( debug ) &
         write(irefwr,1) 'iprec, iprint, nproj', &
                          A%type_prec, A%iprint, A%nproj

!     --- Allocate arrays

      allocate ( res(n), p(n), work(n), pbar(n), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 4*n, 'res' )
      if ( ierror/=0 ) go to 1000

!     --- Compute norm of right-hand side (squared)
!         Some other norms with respect to the termination criterion are
!         computed as well

      call sepccgnorms ( x, b, A, bnorm, bsnorm, xnorm, work )

      if ( bnorm<=0d0 ) go to 900

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr, 100 )

100   format(//' Output for the complex CG method (symmetric matrix)'//)

!     --- Set start vector and initial residu

      call sepccgstart ( A, x, b, res, work )

!     --- Compute the direction vector p and the square of the norm of the
!         residual (rho)

      p = res
      if ( A%precon ) p = res/A

      rho = zcjdot ( n, p, 1, res, 1 )

      if ( debug ) write(irefwr,2) 'rho', rho

      if ( A%iaccuracy==0 ) then
         rhotest = abs(dot_product(p,p))       ! termination based on x
      else
         rhotest = abs(dot_product(res,res))   ! termination based on res
      end if
      r0norm = rhotest

      if ( A%iprint>=1 ) then

!     --- iprint>=1,  Print information

         call sepccgprint ( res, bnorm, r0norm, A )

      end if

      if ( debug ) write(irefwr,*) 'after sepccgprint'

!     --- Set constants

      sqrn = n

      isqrn  = nint ( sqrt ( sqrn ) )
      rferr  = bnorm * epsmac **2

!     --- First estimate of eps  ( eps = eps1 + ||b|| eps2 )

      call sepccgeps ( A, repeat1, bnorm, bsnorm, r0norm )
      if ( debug ) then
         write(irefwr,2) 'bnorm, epsmac, rferr', bnorm, epsmac, rferr
         write(irefwr,2) 'bsnorm, r0norm', bsnorm, r0norm
         write(irefwr,2) 'eps, rho', A%eps, rho
      end if  ! ( debug )

!     --- While loop check on accuracy ( nmax+1 is used because
!         the residual of the previous iteration is always checked )

      do iter = 1, nmax+1

!     --- Check on accuracy reached in iter-1

         if ( debug ) write(irefwr,1) 'iter', iter

         finished = iter>1 .and. rhotest<=A%eps .or. rhotest<=0d0

         if ( finished ) then

!        --- The accuracy of the process seems to be reached
!            Check if eps must be recomputed

            call sepccgcheckeps ( x, repeat1, xnorm, res, &
                                  iter, finished, &
                                  rhotest, .false., A, b )

          if ( debug ) then
               write(irefwr,*) 'finished', finished
               write(irefwr,2) 'eps', A%eps
            end if  ! ( debug )

            if ( finished ) go to 900  ! accuracy has been reached

         end if  ! ( finished )

!        --- Accuracy not yet reached
!            First check error level if iter is multiple of sqrt(n)

         if ( mod(iter,isqrn )==0 ) then

!        --- iter is multiple of sqrt(n)
!            Check error level, i.e. compute new value for rferr

            call sepccgrferr ( A, x, b, res, rferr, iter, nglobal, &
                               work )

         end if  ! ( mod(iter,isqrn )==0 )
         if ( debug ) write(irefwr,2) 'rhotest, rferr', rhotest, rferr

         if ( rhotest<rferr ) then

!        --- rhotest is smaller than maximal accuracy possible
!            It is useless to continue the process

            call sepccgaccur ( res, rferr, A, iter, .false. )
            go to 900

         end if  ! ( rhotest<rferr )

!        --- do not start (nmax+1)-th iteration

         if ( iter>nmax ) exit

!        --- Compute Ap, store in work

         if ( A%eistat ) then

!        --- In case of Eisenstat Ap = E^(-T) p + E^(-1) p + K E^(-T) p
!            If necessary projection is carried out

            pbar = p/A
            work = p + A%S(A%diag) * pbar
            work = A/work
            work = work+pbar

         else

           work = A*p

         end if  ! ( A%eistat )

         if ( A%nproj>0 .and. A%deflation ) work = A .PROJT. work

         if ( extradebug ) call princompl ( work, n, 'Ap' )

!        --- Compute sigma

         sigma = zcjdot ( n, p, 1, work, 1 )

         if ( abs(sigma)<=0d0 ) then

!        --- Sigma<=0, matrix not positive definite

            call erreal ( sigma, 1 )
            call errsub ( 675, 0, 1, 0 )

            A%numiter = iter
            A%itererr = 2

            go to 900

         end if  ! ( abs(sigma)<=0d0 )

         alpha = rho / sigma
         if ( debug ) write(irefwr,2) 'alpha, sigma', alpha, sigma

!        --- Update x and r

         if ( A%eistat ) then

!        --- Eisenstat

            x = x + alpha*pbar

         else

!        --- Normal case

            x = x + alpha*p

         end if  ! ( A%eistat )

         res = res - alpha*work

         if ( extradebug ) then
            call princompl ( x, n, 'x' )
            call princompl ( res, n, 'res' )
         end if  ! ( extradebug )

!        --- Compute new inner product (r,r), store in rhostr

         if ( A%precon ) then

!        --- Preconditioning, store M^(-1) r into work

            work = res/A
            rhosta = zcjdot ( n, res, 1, work, 1 )

         else

            rhosta = zcjdot ( n, res, 1, res, 1 )

         end if

         beta = rhosta/rho
         rho  = rhosta
         if ( debug ) write(irefwr,2) 'beta, rho', beta, rho

         if ( A%iaccuracy==0 .and. A%precon ) then
            rhotest =  abs(dot_product(work,work))
         else
            rhotest = abs(dot_product(res,res))
         end if
         if ( debug ) write(irefwr,2) 'rhotest', rhotest

!        --- Update p:  p_(k+1) = M^(-1) r_(k+1) + beta p_k

         if ( A%precon ) then

!        --- Preconditioning,  M^(-1) r has been stored in work

            p = work + beta * p

         else

!        --- no preconditioning

            p = res + beta * p

         end if
         if ( debug ) write(irefwr,*) 'after update p'

         if ( extradebug ) call princompl ( p, n, 'p' )

         if ( A%iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            call sepccgprint1 ( res, iter, A, bnorm, r0norm )

         end if
      end do

!     --- If the subroutine reaches this stage, too much  iterations have
!         been carried out

      call sepccgprint2 ( res, .false., A, nmax )

900   if ( A%nproj>0 .and. A%deflation ) then

!     --- Project vector x
!         First x := Proj(x)

         x = A .PROJ. x

!        --- Next x := x + V E^(-1) V^T y

          work = A .CPROJ. b
          x = x+work

      end if

!     --- Deallocate arrays

      deallocate ( res, p, work, pbar, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'res' )

1000  if ( debug ) then

!     --- Debug information

         if ( extradebug ) call princompl ( x, n, 'x' )
         write(irefwr,*) 'End sepccg'

      end if  ! ( debug )
      call erclos ( 'sepccg' )

      end subroutine sepccg
