      subroutine sepccgesteps ( A, b, work, bnorm, rferr, repeat1, &
                                ifiter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print first line of iteration output
!     Compute first estimate of accuracy  eps
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(in) :: b(A%neq)
      double precision, intent(in) :: bnorm
      double complex, intent(inout) :: work(A%neq)
      double precision, intent(in) :: rferr
      integer, intent(in) :: ifiter
      logical, intent(inout) ::  repeat1

!     A             i/o   Matrix A
!     b              i    Right-hand side
!     bnorm          i    Norm of the right-hand side (b,b)
!     ifiter         i    Starting value of parameter iter
!     repeat1       i/o   Logical variable indicating if the accuracy check
!                         must be carried out twice (true) or once (false), &
!                         because the cnorm of the solution could not be computed
!     rferr          i    Maximal accuracy possible for this matrix, due to
!                         round-off
!     work                Work array to store a temporary vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: b0norm

!     b0norm         cnorm of start residual
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgesteps' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgesteps'
         write(irefwr,1) 'ifiter', ifiter
         write(irefwr,2) 'bnorm', bnorm
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifiter==1 ) then

!     --- iprint>=1,  Print information

         if ( A%iprint>=1 ) write ( irefwr, 125 ) bnorm
125      format ( ' The 2-norm of the initial residual', ss,1pe17.2e2 )

!        --- iprint>=2,  More output

         if ( A%iprint>=2 ) write ( irefwr, 130 ) 0, bnorm
130      format ( ' Iteration number  residual'  // i10, ss,1pe15.2e2 )

      end if

!     --- First estimate of eps  ( eps = eps1 + ||b|| eps2 )

      if ( ifiter==1 ) then

         b0norm = bnorm
         if ( abs(A%eps2)<=0d0 ) then

            A%eps = A%eps1
            repeat1 = .false.

         else

            if ( A%iabserr==0 ) then

               A%eps = A%eps1 + cnorm(b,A%neq)*A%eps2
               repeat1 = .true.

            else if ( A%irelerr>0 ) then

               A%eps = A%eps1 + b0norm * A%eps2
               repeat1 = .false.

            else

               if ( A%eistat ) then

!              --- Use cnorm of L^(-1) * b,  work(*) is a scratch array

                  work = b/A
                  A%eps = A%eps1 + cnorm(work,A%neq) * A%eps2

               else

                  A%eps = A%eps1 + cnorm(b,A%neq) * A%eps2

               end if

               repeat1 = .false.

            end if

         end if

      end if

!     --- Check error level.

      if ( A%eps<rferr ) then

!     --- eps is smaller than maximal accuracy possible
!         It is useless to end do the process after bnorm
!         is less than rferr

         A%eps = rferr

         if ( A%iprint>=0 ) then
            call erreal ( rferr, 1 )
            call errwar ( 677, 0, 1, 0 )
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepccgesteps'

      end if  ! ( debug )
      call erclos ( 'sepccgesteps' )

      end subroutine sepccgesteps
