      subroutine sepccgs ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Apply the preconditioned method of conjugate gradients squared to solve
!     the linear system of equations A x = b
!     where the matrix A and the preconditioning matrix M may be unsymmetric
!     This subroutine contains the following algorithms:
!     -  Standard conjugate gradient squared method
!     -  Preconditioned conjugate gradient squared method
!     -  Eisenstat implementation of the preconditioned conjugate gradient
!        squared method
!     Both in the case of the preconditioned conjugate gradients squared method, &
!     as in the case of the Eisenstat implementation, the matrix and right-hand
!     side are supposed to be premultiplied by the inverse of the diagonal D
!     of the triangular factors from the preconditioning matrix in the
!     following way:  ( A x = b )
!               -1        -1
!          A = D   A     D
!                   orig
!               -1
!          b = D   b
!                   orig
!     If the initial estimate x is given it must also be premultiplied by D:
!          x = D  x
!                  orig
!                                                        -1
!     At output the solution x must be premultiplied by D   in order to get
!     the solution of the original problem:
!                  -1
!         x     = D   x
!          orig
!     In the case of Eisenstat the matrix amat is supposed to contain the
!     matrix K = D  - 2I in the first n positions, where D  is the diagonal
!                 A                                       A
!     of A after the premultiplication
!     Original subroutine: Rik Kaaschieter and Henk van der Vorst
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), b(A%neq)

!     A             i/o   Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

      integer :: iter, ifiter, nmax, j
      double precision :: rferr, rnorm, bnorm
      double complex sigma, alpha, beta, rho, rhosta
      double complex, allocatable :: res(:), p(:), work(:), rbar(:), &
                                       q(:), u(:), diag(:)
      logical :: repeat1, restart, finished

!     alpha          Factor alpha in the CG-process ( (r,r) / (p,Ap) )
!     beta           Factor beta in the CG-process ( (r_i,r_i) / (r_i-1,r_i-1) )
!     bnorm          Norm of the right-hand side (b,b)
!                    they are not
!     diag           Diagonal of the matrix
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     ifiter         Starting value of parameter iter
!     iter           Loop variable
!     j              General loop variable
!     nmax           The maximum number of iterations that may be performed
!     p              Work array of length n to store the direction vector
!     q              Work array of length n to store the vector q
!     rbar           Work array of length n to store the vector r (bar)
!     repeat1        Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    cnorm of the solution could not be computed
!     res            Work array of length n to store the residual vector
!     restart        Logical variable indicating the process may be restarted if
!                    rho* too small
!     rferr          Maximal accuracy possible for this matrix, due to round-off
!     rho            Inner product between M^(-1) r and r at old level, i.e.
!                    M^(-1) cnorm of residual
!     rhosta         rho at new level
!     rnorm          cnorm of residual
!     sigma          Factor sigma in the CG-process (inner product of p and Ap)
!     u              Work array of length n
!     work           Work array of size n to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCCGESTEPS   Print first line of iteration output
!                    Compute first estimate of accuracy  eps
!     SEPCCGSTART    converged
!     SEPCMAXACC     Compute maximum accuracy possible in iterative solver
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
!     700   CGS-process has terminated because sigma = 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If no preconditioning is used or the Eisenstat version then M = I
!     If not the Eisenstat implementation is used then L = U = I
!     The CGS- process can be written as:
!     r_1 = L^(-1) ( b - A x_1 )
!     rbar  = U^(-T) ( b - A x_1 )
!    Iteration: k = 1(1) nmax  while ||r||>eps
!       z_k = M^(-1)  r_k
!       rho* = ( rbar, z_k )
!       If (k = multiple of sqrt(A%neq))
!          check error level
!       End
!       If ( k=1 )
!          u_1 = z_1      p_1 = z_1
!       Else if (k>1)
!          beta = rho*  / rho
!          u_k = z_k + beta_k q
!          p_k = u_k + beta_k ( q + beta_(k-1) p )
!       End
!       rho = rho* &
!       AP = Ap_k  (Eisenstat: AP = U^(-T) p_k + L^(-1) ( p_k + K U^(-T) p_k )
!       sigma = (rbar , M^(-1) AP )
!       alpha = rho /sigma
!       q_k = u_k - alpha M^(-1) AP
!       x_(k+1) = x_k + alpha U^(-T) (u_k + q_k )
!       r_(k+1) = r_k - alpha A ( u_k + q_k )
!       Eisenstat:  A(u_k+q_k)=U^(-T)(u_k+q_k)+L^(-1)(u_k+q_k+K U^T (u_k+q_k))
!    End
!    Check error level:
!    If ||r||^2<exp(k/n)^2|| Ax -b -r ||^2   then maximal accuracy has
!    been reached.
!  Remark 1: it seems  "q" and "work" can share memory
!  Remark 2: prprec is called to solve  M * "new work" = "old work", &
!            thus it must be written to allow this sharing of memory
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgs'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      bnorm = cnorm(b,A%neq)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x = 0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      allocate(res(A%neq), p(A%neq), work(A%neq), rbar(A%neq), q(A%neq), &
               u(A%neq), stat = error)
      if ( error/=0 ) call eralloc ( error, 5*A%neq, 'res' )
      if ( ierror/=0 ) go to 1000

!     --- Set some constants

      nmax   = A%nmax       ! maximum number of iterations
      rho    = 1d0
      finished = .false.

      if ( nmax==0 ) nmax=10*A%neq

!     --- Compute maximum accuracy possible

      call sepcmaxacc ( A, b, work, bnorm, rferr )
      if ( debug ) write(irefwr,2) 'rferr', rferr

      if ( A%eistat ) then

!     --- Eisenstat preconditioning, fill diag

         allocate ( diag(A%neq) )

         do j = 1, A%neq
            diag(j) = A%S(A%diag(j))
         end do  ! j = 1, A%neq

      end if  ! ( A%eistat )

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr,100 )
100   format(//' Output for the CGS method', &
               ' (complex non-symmetric matrix)'//)

      ifiter = 1
      restart = .false.

!     --- Set start vector and initial residual
!         This part may be repeated

105   call sepccgstart ( A, x, b, res, rbar )

!     --- Set constants

      rnorm = cnorm ( res, A%neq )
      if ( debug ) write(irefwr,2) 'rnorm', rnorm

!     --- Print first line of iteration output
!         Compute first estimate of accuracy  eps

      iter = ifiter
      call sepccgesteps ( A, b, work, rnorm, rferr, repeat1, iter )

      do while ( .not. finished .and. iter<=nmax+1 )

         if ( debug ) write(irefwr,1) 'iter', iter

!         --- Check on accuracy reached in iter-1

140      if ( iter>ifiter .and. rnorm<=A%eps .or. rnorm<=0d0 ) then

!        --- The accuracy of the process seems to be reached
!             Check if eps must be recomputed

            if ( debug ) write(irefwr,*) 'repeat1 ', repeat1
            if ( repeat1 ) then

!           --- repeat1 is true, recompute A%eps

               repeat1 = .false.
               A%eps = A%eps1 + cnorm ( x,A%neq ) * A%eps2

!              --- Check error level.

               if ( A%eps<rferr ) then

!              --- eps is smaller than maximal accuracy possible
!                  It is useless to end do the process after rnorm
!                  is less than rferr

                  A%eps = rferr

                  if ( A%iprint>=0 ) then
                     call erreal ( rferr, 1 )
                     call errwar ( 677, 0, 1, 0 )
                  end if

               end if

               goto 140

            end if

            if ( A%iprint>=1 ) then

!           --- iprint>=1  Print information

               write ( irefwr,150 ) iter-1, rnorm
150            format(' The number of executed iterations in the', &
                      ' CGS-process is', i5/ &
                      ' The 2-norm of the residual is:', ss,1pe12.2e2 )

            end if

            A%numiter = iter-1
            A%itererr = 0
            finished = .true.
            exit

         end if

!        --- Accuracy not yet reached

         if ( rnorm>1d20 ) then

!        --- better: use rinfin
!            rnorm  overflow,  Process terminates

            write ( irefwr, '( 24x, a, ss,1pe10.2e2 )' ) &
                    ' warning: CGS looks divergent'
         end if

!        --- do not start (nmax+1)-th iteration

         if ( iter>nmax ) then
            finished = .true.
            exit
         end if  ! ( iter>nmax )

!        --- Compute z_k = M^(-1) r_k   and store in u
!            Compute rho*  = ( r, z_k )

         if ( A%precon ) then

            u = res/A
            rhosta = dot_product ( rbar, u )

         else

            rhosta = dot_product ( rbar, res )

         end if
         if ( debug ) write(irefwr,2) 'rhosta', rhosta

         if ( restart .and. abs(rho)*epsmac*1d2>abs(rhosta) ) then

!        --- rhosta too small, restart the process

            ifiter = iter
            restart = .false.
            go to 105

         end if

         if ( iter==1 ) then

!        --- k = 1, Copy u in p  (i.e. u_1 = z_1,  p_1 = z_1 )

            if ( A%precon ) then

               p = u

            else

               p = res
               u = res

            end if

         else

!        --- k>1
!            beta = rho* / rho
!            u_k = z_k + beta q_k
!            p_k = u_k + beta ( q_k + beta p_(k-1) )

            beta = rhosta / rho
            restart = .true.
            if ( A%precon ) then

               u = u + beta * q

            else

               u = res + beta * q

            end if
            if ( debug ) write(irefwr,2) 'beta', beta

            p = u + beta * ( q + beta * p )

         end if

         rho = rhosta
         if ( debug ) write(irefwr,2) 'rho', rho

!        --- Compute M^(-1) Ap, store in work

         if ( A%eistat ) then
            q = p/A
            work = p + diag * q
            work = A/work
            work = work+q
         else
            q = A*p
            work = q/A
         end if  ! ( A%eistat )

!        --- Compute sigma

         sigma = dot_product( rbar, work )
         if ( debug ) write(irefwr,2) 'sigma', sigma

         if ( abs(sigma)<=0d0 ) then

!        --- Sigma = 0, Process terminates

            call errsub ( 700, 0, 0, 0 )

            A%numiter = iter
            A%itererr = 2
            finished = .true.
            exit

         end if

         alpha = rho / sigma

!        --- Update x and r
!            q_k = u_k - alpha M^(-1)  AP

         q = u - alpha * work

!        --- Store alpha * ( u_k + q_k ) into u

         u = alpha * ( u + q )

         if ( A%eistat ) then

!        --- Eisenstat
!            Store U^(-1) * alpha * ( u_k + q_k ) into work

            work = u/A
            x = x + work

!           --- Store  alpha A ( u_k + q_k ) into work, use u as work array

            u = u + diag * work
            u = A/u
            work = work + u

         else

!        --- Normal case

            x = x + u
            work = A*u

         end if

         res = res - work

!        --- Compute cnorm of residual

         rnorm = cnorm ( res,A%neq )

         if ( A%iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            write ( irefwr, 350 ) iter, rnorm
350         format( i10, ss,1pe15.2e2 )

         end if
         iter = iter+1

      end do  ! while ( .not. finished .and. iter<=nmax+1 )

      if ( .not. finished ) then

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

         if ( A%iprint>=1 ) then

!        --- iprint>=1  Print information

            write ( irefwr,150 ) nmax, rnorm

         end if

         call errint ( nmax, 1 )
         if ( A%aterror==1) then
            A%numiter=nmax
            call errwar ( 676, 1, 0, 0 )
         else
            call errsub ( 676, 1, 0, 0 )
         end if

      end if  ! ( .not. finished )

      deallocate ( res, p, work, rbar, q, u, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'res' )
      if ( A%eistat ) deallocate ( diag )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepccgs'

      end if  ! ( debug )
      call erclos ( 'sepccgs' )

      end subroutine sepccgs
