      subroutine sepcglamda ( iter, alpha, beta, eigvl, &
                              lambda, A, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update lambda for cg if iter<=ntreigv
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      integer, intent(in) :: iter
      double precision, intent(in) :: alpha, beta, lambda
      double precision, intent(inout) :: eigvl(*)
      logical, intent(inout) :: finished

!     A              i    Matrix A
!     alpha          i    Factor alpha in the CG-process ( (r,r) / (p,Ap) )
!     beta           i    Factor beta in the CG-process
!                         ( (r_i,r_i) / (r_i-1,r_i-1) )
!     eigvl         i/o   Real 10 x 2 array to store the factors alpha and beta
!                         of the first 10 iterations in eigvl(i,1)
!                         resp. eigvl(i,2)
!     finished      i/o   Indicates if the process has finished (true) or not
!                         (false)
!     iter           i    Iteration sequence number
!     lambda         i    Approximation of smallest eigenvalue
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nwarnsave

!     nwarnsave      Variable to store the value of nwarn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRUPEV         compute the smallest eigenvalue of tridiagonal matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2139   The estimated value of lambda is larger than the required lambda
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcglamda' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcglamda'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iter<=A%ntreigv ) then

!     --- iter <= ntreigv, store lambda

         eigvl(iter) = alpha
         eigvl(A%ntreigv+iter) = beta
         call prupev ( iter, lambda, eigvl, A%ntreigv )
         if ( A%alammin<=0d0 ) A%alammin = lambda

      else if ( A%alammin<lambda ) then

!     --- alammin<lambda, produce error or warning

         call erreal ( lambda, 1 )
         call erreal ( A%alammin, 2 )
         call errint ( A%ntreigv, 1 )

         if ( A%aterror==1) then

            if ( A%iprint>=0 ) then
               if ( A%iprint>=2 ) then
                  nwarnsave = nwarn
                  nwarn = -100
                  call errwar ( 2139, 1, 2, 0 )
                  nwarn = nwarnsave + 1
               else
                  call errwar ( 2139, 1, 2, 0 )
               end if
            end if

         else

            call errsub ( 2139, 1, 2, 0 )

            A%numiter = iter
            A%itererr = 2
            finished = .true.

         end if

      end if  ! ( iter<=A%ntreigv )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcglamda'

      end if  ! ( debug )
      call erclos ( 'sepcglamda' )

      end subroutine sepcglamda
