      subroutine sepcompreac1 ( solutvec, reaction, rhsvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Actual computing of reaction force
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulematr
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: solutvec(nusol), reaction(nusol), rhsvec(nusol)

!     reaction      i/o   reaction vector
!     rhsvec         i    right-hand-side vector
!     solutvec       i    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ih, i

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERPERIODREAC   Incorporate the effect of periodicity to reaction forces
!     SEPLOCTRNRLNOR Compute (back-)transformation in case of local
!                    transformations ( real case no renumbering )
!     SEPLOCTRNRLREN Compute (back-)transformation in case of local
!                    transformations ( real case and renumbering )
!     SEPMULTREAC1   Compute reaction forces for real matrix
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcompreac1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcompreac1'
         write(irefwr,1) 'npltra', npltra
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 .or. nbound==0 ) go to 1000

      indprab = kprobloc(58)
      kprobab => kprobpart%kprobab

      allocate( xri(nusol), yri(nusol) )
      xri = solutvec(kprobhinv(1:nusol))
      yri = 0d0
      if ( npltra>0 ) call seploctrnrlnor ( 1, xri )

!     --- Compute reaction force:
!         y_p = S_pf x_f + S_pp x_p. y_f is left unchanged

      call  sepmultreac1 ( Smat, xri, yri )

!     --- Add contributions of all cpu's to common nodes

      if ( parallel ) call sepparglob ( Smat%S_i, yri, nusol )

      reaction(kprobhinv(1:nusol)) = yri(1:nusol)
      deallocate(xri, yri)

      if ( iseqreac>0 ) then

!     --- Subtract part of right-hand side from reaction force
!         In fact this is the real reaction force

         do i = nusol-nbound+1, nusol
            ih = kprobhinv(i)
            reaction(ih) = reaction(ih) - rhsvec(ih)
         end do

      end if

!     --- (Back) transformation on reaction force and solution

      if ( npltra>0 ) then
         call seploctrnrlren ( 2, reaction )
         call seploctrnrlren ( 2, solutvec )
      end if  ! ( npltra>0 )

      if ( indprab>0 ) then

!     --- indprab>0, hence there are periodical boundary conditions
!         Correct the reaction forces for these periodical boundary
!         conditions

         call erperiodreac ( reaction )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcompreac1'

      end if  ! ( debug )
      call erclos ( 'sepcompreac1' )

      end subroutine sepcompreac1
