      subroutine sepcsolvemeth ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronicechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations
!     Complex, serial case
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec, itypesol, i
      double complex, pointer :: diag(:)

!     diag           diagonal of preconditioning matrix
!     i              general loop variable
!     iprec          Type of preconditioner, see inpsol(4)
!     itypesol       Type of linear solver, see inpsol(3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCBICGSTAB   Solve linear system by Bi-CGstab
!     SEPCBICGSTABL  Solve linear system by Bi-CGstab(l)
!     SEPCCG         Solve linear system by conjugate gradients
!     SEPCCGS        Solve linear system by CGS
!     SEPCGCR        Solve linear system by GCR
!     SEPCGMRES      Solve linear system by GMRES
!     SEPCGMRESR     Solve linear system by GMRESR
!     SEPCIDR        Solve linear system by IDR
!     SEPCSCALMAT    Scale the original large sparse matrix
!     SEPCSCALMATBCK Rescale the original large sparse matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2987   Iterative solver not available for real matrices
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcsolvemeth' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcsolvemeth'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find type of preconditioning

      iprec = A%type_prec
      itypesol = A%typesolv
      if ( A%scaling==0 ) iprec = 0  ! No scaling of matrix

      if ( itypesol==17 .and. A%ninner==1 ) then

!     --- Bicgstab (1): use original bicgstab

         itypesol = 1
         A%typesolv = 1

      end if  ! ( itypesol==17 .and. inpsol(14)==1 )

      if ( debug ) &
         write(irefwr,1) 'iprec, itypesol, scaling', &
                          iprec, itypesol, A%scaling

!     --- Scale matrix and right-hand side with preconditioning

      if ( iprec>0 ) then
         allocate ( diag(A%neq) )
         if ( A%type_prec==3 .or.A%type_prec==4 ) then
            do i = 1, A%neq
               diag(i) = A%MD(A%diag(i))
            end do  ! i = 1, A%neq
         else
            diag = A%MD
         end if  ! ( .not. A%symmetric )

         call sepcscalmat ( A, diag, x, b )

      end if  ! ( A%scaling>0 .and. iprec>0 )

!     --- Solve system of equations

      select case ( itypesol )
      case(1)
         if ( A%symmetric )  then

!        --- Symmetric case: cg

            call sepccg ( A, x, b )

         else

!        --- Unsymmetric case: bicgstab

            call sepcbicgstab ( A, x, b )

         end if  ! ( A%symmetric )

      case(2)
         call sepccgs ( A, x, b )
      case(3)
         call sepcgmres ( A, x, b )
      case(4)
         call sepcgmresr ( A, x, b )
      case(10)
         call sepcidr ( A, x, b )
      case(17)
         call sepcbicgstabl ( A, x, b )
      case(31)
         call sepcgcr ( A, x, b )
      case default

!     --- Option not available

         call errint ( itypesol, 1 )
         call errsub ( 2987, 1, 0, 0 )
         go to 1000

      end select  ! case ( itypesol )

!     --- Scale matrix and right-hand side back with preconditioning

      if ( iprec>0 ) then
         call sepcscalmatbck ( A, diag, x, b )
         deallocate(diag)
      end if  ! ( A%scaling>0 .and. iprec>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcsolvemeth'

      end if  ! ( debug )
      call erclos ( 'sepcsolvemeth' )

      end subroutine sepcsolvemeth
