      subroutine sepdecgg ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the decomposition of a symmetric positive definite
!     profile matrix
!     Only the rows rowmin to rowmax are decomposed
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     positive_definite
!     real
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: p, ii, i, j, jj, ji, ih1, k, pj
      double precision  :: am, som

!     am             Help parameter to store the diagonal of the matrix before
!                    computing the square root
!                    they are not
!     i              Counting variable
!     ih1            Help variable to store a constant temporarily
!     ii             Position of i th diagonal element in array A%MD
!     j              Counting variable
!     ji             Position of element ji in array A%MD
!     jj             Position of j th diagonal element in array A%MD
!     k              Counting variable
!     p              Max (pi, pj)
!     pj             Column number of at most left non-zero element in row j
!     som            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: dasum, ddot

!     DASUM          Blas subroutine computing the sum of absolute values of
!                    vector
!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     104   matrix has small pivots
!    1535   matrix is singular
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdecgg' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdecgg'
         write(irefwr,1) 'A%rowmin, A%rowmax', A%rowmin, A%rowmax
         call sepprinmat2 ( A )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do j = A%rowmin, A%rowmax

         jj = A%Mrows(j)
         am = A%MD(jj)
         pj = A%Mcols(j)

         if ( pj<j ) then

!        --- pj<j

            ih1 = jj-j
            do i = pj,j-1
               p = max(pj,A%Mcols(i))
               ji = ih1+i
               ii = A%Mrows(i)
               som = ddot(i-p,A%MD(ih1+p),1,A%MD(ii-i+p),1)
               A%MD(ji) = (A%MD(ji)-som)/A%MD(ii)
            end do

!           --- g(jj)

            am = am-sum(A%MD(ih1+pj:ih1+j-1)**2)

         end if  ! ( pj<j )

!        --- Check pivot element:

         if ( skipzeros ) then

!        --- skipzeros = true, hence if the diagonal element is too small
!            it is replaced by the machine accuracy (scaled by rowum)

            if ( am<A%rowsum ) am = A%rowsum

         else

!        --- skipzeros = false, check singular matrix

            if ( am<=0d0 ) then

!           --- matrix is singular, error

               call errint ( j, 1 )
               call errsub ( 1535, 1, 0, 0 )
               go to 1000

            else if ( am<A%rowsum ) then

!           --- matrix is probably singular, warning

               call errint ( j, 1 )
               call erreal ( am, 1 )
               call erreal ( A%rowsum, 2 )
               call errwar ( 104, 1, 2, 0 )

            end if

         end if  ! ( skipzeros )
         A%MD(jj) = sqrt(am)

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'G G^t decomposition'
         call sepprinmat2 ( A )
         write(irefwr,*) 'End sepdecgg'

      end if  ! ( debug )
      call erclos ( 'sepdecgg' )

      end subroutine sepdecgg
