      subroutine sepdivmat ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) x, where A is a preconditioning matrix
!     a symmetric compact storage is used, hence only D and L
!     The preconditioning matrix has been created before
!     In other words a part of the ILU is executed (The U part)
!     This routine is meant for eistenstat preconditioning
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr ! definition of matrix structure
      use sepmodulemat    ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision, intent(out) :: y(A%neq)

!     A              i    Matrix A
!     x              i    Solution vector
!     y              o    Result  y = A^(-1) x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iprec          type of preconditioning
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPRCMATDIVEL  Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a row-compact storage is used, hence only D and L
!                    Eisenstat preconditioning
!     SEPSCMATDIVL   Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!                    Eisenstat preconditioning
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2684   Value of iprec not implemented
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdivmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdivmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprec = A%type_prec
      IM => intmt(A%iseqim)

      if ( iprec==2 .or. iprec==4 .or. iprec==7 ) then

!     --- Eisenstat preconditioning, only L y = x

         if ( IM%symmetric ) then

!        --- symmetric compact matrix

            call sepscmatdivL ( A, x, y )

         else

!        --- row-compact matrix

            call seprcmatdiveL ( A, x, y )

         end if  ! ( IM%symmetric )

      else

!     --- Not available

         call errint ( iprec, 1 )
         call errsub ( 2684, 1, 0, 0 )

      end if  ! ( iprec<=1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y' )
         write(irefwr,*) 'End sepdivmat'

      end if  ! ( debug )
      call erclos ( 'sepdivmat' )

      end subroutine sepdivmat
