      subroutine sepfilloldsols ( islold, jsubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array istold with information of old solution vectors.
!     Reserve space for the necessary arrays
!     Help subroutine for TO0050
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      use sepmodulesol
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: islold(*)
      integer, intent(in) :: jsubr

!     islold         i    This array is used if ichois<0, or numold>0
!                         When ichois<0, it is supposed to consist of one
!                         vector only, which must be of the same structure as
!                         the solution  vector
!                         When numold>0, islold contains numold vectors which
!                         may be solution vectors of several problems or vectors
!                         of special structure (type 115) resulting of several
!                         problems
!     jsubr          i    Indicates which subroutine calls sepfilloldsols
!                         possible values:
!                         1:  call by BUILD
!                         2:  call by DERIV
!                         3:  call by INTEGR
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: itypvc, j, iproblc, k, kcopy
      logical :: warning1, warning2

!     iproblc          Problem number corresponding to specific vector
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     j              Counting variable
!     k              Counting variable
!     kcopy          Actual value of j that indicates the vector that must be
!                    copied
!     warning1       If true already a warning has been printed
!     warning2       If true already a warning has been printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRININLONG     print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     720   Array ISLOLD has not been filled.
!    1077   Vector in ISLOLD does not correspond to KPROB
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data warning1 /.false./, warning2 /.false./
! ======================================================================
!
      call eropen ( 'sepfilloldsols' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilloldsols'
         write(irefwr,1) 'numold, jsubr', numold, jsubr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      isubr  = jsubr

!     --- Initialize array ISTOLD

      istold1 = 0
      istold = 0
      istold(2,1:numold) = 1

      if ( isubr==1 ) then

!     --- call by build

         numtypesols = 1  ! number of types of different vectors in islold
                          ! The first one refers to the solution vector

      else

!     --- call by other subroutine

         numtypesols = 0

      end if  ! ( isubr==1)

      if ( debug ) write(irefwr,1) 'numtypesols', numtypesols

!     --- Loop over all solution vectors

      iproblc = 0
      do k = 1, numold

         if ( islold(k)<=0 ) then
            itypvc = 0
         else if ( solarr(islold(k))==0 ) then

!        --- Array does not exist

            islold(k) = 0
            istold1(1,k) = -1
            go to 200

         else if ( probarr(ks(islold(k))%iseqkp)==0 ) then

!        --- Array kprob does not exist

            call seprmsolinfo ( islold(k) )
            istold1(1,k) = -1
            go to 200

         else

            itypvc = ks(islold(k))%typevec

         end if  ! ( islold(k)<=0 )

         if ( itypvc/=110 .and. itypvc/=115 .and. &
              itypvc/=122 .and. itypvc/=116 .and. &
              itypvc/=126 .and. itypvc/=129 .and. &
              itypvc/=801  ) then

!        --- itypvc # 110, 115, 116, 122, 126, 129, 801:
!            this type of array is not supported

            call errint ( k, 1 )
            call errint ( itypvc, 2 )
            if ( .not. warning1 ) then
               call errwar ( 720, 2, 0, 0 )
               warning1 = .true.
            end if
!            istold(1,k) = 0
            istold1(1,k) = -1
            go to 200

         end if

         if ( debug ) &
            write(irefwr,1) 'k, itypvc, isubr, islold(k)', &
                             k, itypvc, isubr, islold(k)

!        --- Find starting position of array islold

         istold1(6,k) = islold(k)
         if ( islold(k)>0 ) then
            call sepactsolbf2 ( islold(k) )
            iproblc  = ks(islold(k))%iprob
         end if   ! ( islold(k)>0 )
         istold1(4,k) = iproblc
         istold1(3,k) = iprob
         if ( debug ) then
            write(irefwr,1) 'iprob, iproblc', iprob, iproblc
            write(irefwr,1) 'indprfi, nunkpi, nphysi', &
                             indprfi, nunkpi, nphysi
         end if  ! ( debug )

         oldsols(k)%nunkp = nunkpi

!        --- Test if array has a new structure or an old one

         if ( isubr<=2 .and. iproblc==iprob .and. itypvc==110 ) then

!        --- iproblc = iprob  and itypvc = 110, so index3 = index2

            istold1(2,k) = indprfi
            istold1(5,k) = 1
            if ( debug ) write(irefwr,1) 'indprfi', indprfi
            if ( .not. warning2 .and. ks(islold(k))%nusol/=nusoli ) then

!           --- Error solution array does not correspond to array KPROB

               call errint ( k, 1 )
               call errint ( nusoli, 2 )
               call errint ( ks(islold(k))%nusol, 3 )
               call errwar ( 1077, 3, 0, 0 )
               warning2 = .true.

            end if
            if ( isubr==2 ) then
               numtypesols = numtypesols+1
               istold1(5,k) = numtypesols
            end if  ! ( isubr==2 )

         else

!        --- iproblc # iprob  check if new problem number is found or that old
!                           must be copied

            kcopy = 0
            do j = 1, k-1
               if ( istold1(4,j)==iproblc ) then

!              --- Problem numbers are identical, now check type of vector

                  if ( ks(islold(k))%typevec==ks(islold(j))%typevec .and. &
                       ks(islold(k))%ivec==ks(islold(j))%ivec ) then
                     kcopy = j
                     istold1(5,k) = istold1(5,j)
                     goto 110
                  end if
               end if
            end do
            if ( debug ) &
               write(irefwr,1) 'iprob, iproblc, kcopy', &
                                iprob, iproblc, kcopy

110         if ( kcopy==0 ) then

!           --- kcopy = 0, hence a new array has been found

               istold1(1,k) = k

!              --- Fill array istold and compute starting positions of
!                  corresponding arrays kprobf and kprobh

               if ( debug ) write(irefwr,1) 'itypvc', itypvc
               numtypesols = numtypesols+1  ! new type found: raise
               if ( itypvc==110 .or. itypvc==122 ) then

!              --- itypvc = 110, 122 standard situation

                   istold1(2,k) = indprfi

               else if ( itypvc==115 ) then

!              --- itypvc = 115, Array of special structure 115

                  if ( debug ) write(irefwr,1) 'itypvc', itypvc
                  istold1(2,k) = indprfi

               end if
               istold1(5,k) = numtypesols
            else

!           --- kcopy # 0, copy information from array kcopy

               istold(2,k) = istold(2,kcopy )
               istold1(1:5,k) = istold1(1:5,kcopy )

            end if
!            if ( debug ) then
!               call prininlong ( istold(1,k), 2, 'istold' )
!               call prinin ( istold1(1,k), 6, 'istold1' )
!            end if  ! ( debug )

         end if

200   end do

1000  if ( debug ) then

!     --- Debug information

!         do k = 1, numold
!            write(irefwr,1) 'k', k
!            call prininlong ( istold(1,k), 3, 'istold' )
!            call prinin ( istold1(1,k), 6, 'istold1' )
!         end do  ! k = 1, numold
         write(irefwr,1) 'numtypesols', numtypesols
         write(irefwr,*) 'End sepfilloldsols'

      end if  ! ( debug )
      call erclos ( 'sepfilloldsols' )

      end subroutine sepfilloldsols
