      subroutine sepgcrcycle ( svec, vvec, ngcr, kiter, rnorm, rup, up, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one step of the gcr process.
!     The matrix vector multiplication corresponds to block methods
!     It is assumed that the preconditioning step has been carried out
!     before
!     Old subroutine prgcrcycle
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ngcr, kiter
      double precision, intent(inout) :: svec(nrusol,0:ngcr+1), rup(nrusol)
      double precision, intent(inout) :: vvec(nrusol,0:ngcr+1), up(nrusol)
      double precision, intent(out) :: rnorm
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     kiter          i    Iteration sequence number
!     ngcr           i    Maximum number of GCR inner iterations
!     rnorm          o    norm of residual
!     rup           i/o   residual
!     svec          i/o   Work array of size ngcr x n_u to store the vectors
!                         s during the iteration
!     up            i/o   array with velocities and pressures
!     vvec                Work array of size ngcr x n_u to store the vectors
!                         v during the iteration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, imin
      double precision :: alphamin

!     alphamin       Minimum value of alpha
!                    they are not
!     i              Counting variable
!     imin           Index for which the vector that has minimal orthogonality
!                    is stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Solve a system of equation by the GCR method
!     In this case the system of equations is a block system
!     of Navier-Stokes type having u and p as unknowns
!     The algorithm for one step reads
!
!        s^(k+1) := (du,dp), r = (ru,r_p)  (! input )
!        v^(k+1) := A s^(k+1)
!        For i := 1 (1) k do
!           alpha := ( v^(k+1), v^i)
!           v^(k+1) := v^(k+1)- alpha v^i
!           s^(k+1) := s^(k+1)- alpha s^i
!        v^(k+1) := v^(k+1)/||v^(k+1)||
!        s^(k+1) := s^(k+1)/||v^(k+1)||
!        alpha := ( r, v^(k+1))
!        (u,p) := (u,p) + alpha s^(k+1)
!        r := r - alpha v^(k+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgcrcycle' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgcrcycle'
         write(irefwr,1) 'gcr iteration :', kiter
         write(irefwr,1) 'nu, np', nu, np
         call prinrl ( up, nrusol, 'ustart' )
         call prinrl ( rup, nrusol, 'rustart' )
         call prinrl ( svec(1,0), nrusol, 'svec' )
         call prinrl ( vvec(1,0), nrusol, 'vvec' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Perform all other steps
!         For i := 1 (1) kiter do

      alphamin = rinfin
      imin = min(kiter+1,ngcr)
      do i = 1, min(kiter,ngcr)

!     --- alpha := ( v^(k+1), v^i)

         alpha = dotprod ( vvec(:,0), vvec(:,i), A )

         if ( kiter>ngcr .and. abs(alpha)<alphamin ) then

!        --- alpha<alphamin, set alpha = alphamin and imin = i
!            Only if the number of iterations exceeds ngcr we compute
!            the minimum value of alpha
!            We apply the truncated GCR process by putting the new vector in
!            position imin

            alphamin = abs(alpha)
            imin = i

         end if

         if ( debug ) &
            write(irefwr,*) 'kiter, i, alpha ', kiter, i, alpha

!        --- v^(k+1) := v^(k+1)- alpha v^i

         vvec(:,0) = vvec(:,0) - alpha * vvec(:,i)

!        --- s^(k+1) := s^(k+1)- alpha s^i

         svec(:,0) = svec(:,0) - alpha * svec(:,i)

      end do  ! i = 1, k

!     --- v^(k+1) := v^(k+1)/||v^(k+1)||

      rnorm =  norm ( vvec(:,0), A )
      if ( debug ) then
         write(irefwr,1) 'kiter', kiter
         write(irefwr,2) 'rnorm', rnorm
      end if  ! ( debug )

      vvec(:,0) = vvec(:,0)/rnorm

!     --- s^(k+1) := s^(k+1)/||v^(k+1)||

      svec(:,0) = svec(:,0)/rnorm

!     --- Copy result to new vector

      vvec(:,imin) = vvec(:,0)
      svec(:,imin) = svec(:,0)

!     --- alpha := ( r, v^(k+1))

      alpha = dotprod ( rup, vvec(:,imin), A )
      if ( debug ) write(irefwr,2) 'alpha', alpha

!     --- (u,p) := (u,p) + alpha s^(k+1)

      up = up + alpha*svec(:,imin)

!     --- r := r - alpha v^(k+1)

      rup = rup - alpha*vvec(:,imin)

!     --- Compute norm of residual and check convergence

      rnorm = norm ( rup, A )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( up, nrusol, 'u' )
         call prinrl ( rup, nrusol, 'ru' )
         write(irefwr,*) 'End sepgcrcycle'

      end if  ! ( debug )
      call erclos ( 'sepgcrcycle' )

      end
