      subroutine sepidr ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Apply the IDR preconditioned method to solve
!     the linear system of equations A x = b
!     matrix A and the preconditioning matrix K = L U  may be unsymmetric.
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     parallel
!     idr
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ifiter, sparam
      double precision :: rnorm, rferr, bnorm
      double precision, allocatable :: xstart(:), res(:), work(:), &
                                       pshadow(:,:)
      logical :: repeat1, extradebug

!     bnorm          Norm of the right-hand side (b,b)
!     extradebug     If true extra debugging statements are carried out
!     ifiter         Starting value of parameter iter
!     pshadow        Array of size n x s
!                    Contains all shadow vectors of size n each
!                    So pshadow(i,j) contains i^th component of p_j
!     repeat1        Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    norm of the solution could not be computed
!     res            Residual
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rnorm          Norm of residual
!     sparam         Defines the parameters s in IDR(s)
!     work           Work array of size n to store an intermediate array
!     xstart         Array of size n to store the start vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPCGESTEPS    Print first line of iteration output
!                    Compute first estimate of accuracy  eps
!     SEPCGSTART     Fill starting value in array x and compute initial residual
!     SEPIDRITER     Perform main loop for idr iteration
!     SEPIDRSHADOWVC Compute shadow vectors p for IDR process
!                    Store in pshadow
!     SEPMAXACC      Compute maximum accuracy possible in iterative solver
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2894   eisenstat has not yet been implemented for idr
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepidr' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepidr'
         if ( extradebug ) call prinrl ( b, A%neq, 'b' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      bnorm = norm(b, A)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x = 0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )
      sparam = A%ninner

      if ( A%eistat ) then

!     ---  Eisenstat not yet implemented

         call errsub ( 2894, 0, 0, 0 )
         go to 1000

      end if  ! ( A%eistat )

      allocate( xstart(A%neq), res(A%neq), work(A%neq), &
                pshadow(A%neq,sparam), stat = error)
      if ( error/=0 ) call eralloc ( error, A%neq*sparam, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Compute maximum accuracy possible

      call sepmaxacc ( A, b, work, bnorm, rferr )
      if ( debug ) write(irefwr,2) 'rferr', rferr

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr,100 ) sparam
100   format(//' Output for the IDR(', i2, ' ) method', &
               ' (non-symmetric matrix)'//)

!     --- compute start vector and residual

      call sepcgstart ( A, x, b, res, work )

      if ( A%startvec/=0 ) then

!     --- copy start vector to xstart and clear x

         xstart = x
         x = 0d0

      end if  ! ( A%startvec/=0 )

      if ( extradebug ) then
         call prinrl ( xstart, A%neq, 'xstart' )
         call prinrl ( res, A%neq, 'start residual' )
      end if  ! ( debug )

!     --- Define shadow space and store in matrix pshadow

      call sepidrshadowvc ( A, sparam, pshadow )
      if ( extradebug ) call prinrl1 ( pshadow, sparam, A%neq, 'pshadow' )

!     --- Compute start residual

      rnorm = norm ( res, A )

!     --- Print first line of iteration output
!         Compute first estimate of accuracy  eps

      ifiter = 1
      call sepcgesteps ( A, b, work, rnorm, rferr, repeat1, ifiter )
      if ( debug ) write(irefwr,2) 'eps', A%eps

!     --- Main iteration loop

      call sepidriter ( A, x, sparam, res, rnorm, pshadow )

!     --- Compute backward substitution M y = x in case of preconditioning

      if ( A%precon ) x = x/A

      if ( ifiter/=1 .or. A%startvec/=0 ) then

!     --- copy start vector to xstart and clear x

         x = x+xstart

      end if  ! ( ifiter/=1 .or. A%startvec/=0 )

!     --- Fill information in citerinfo
!         This is only used for performance checking

      niterout = A%numiter

      deallocate ( xstart, res, work, pshadow, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'sepidr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepidr'

      end if  ! ( debug )

      end subroutine sepidr
