      subroutine seplinsol ( matr, isol, irhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 23-02-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations by either a direct or an iterative
!     method
!     Depending on the parameter ireadinpsol input is given by iinsol or
!     read from the standard input file
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduledebug
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulematr
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: matr(*), isol(*), irhsd(*)

!     irhsd        i    Standard SEPRAN array, containing information of the
!                         large vector.
!                         If defect correction is applied, irhsd must contain
!                         two vectors and hence must be declared as irhsd(5,2)
!                         irhsd(.,1) corresponds to matr(.,1) including the
!                                   effect of boundary conditions
!                         irhsd(.,2) corresponds to matr(.,2) including the
!                                   effect of boundary conditions
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icpmat

!     icpmat         Standard SEPRAN array of length 5 containing information of
!                    the copy of the matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRMATLAB       Write matrix and right-hand side to files in matlab format
!     PRMATLABVECTOR Print a sepran vector in a form suitable for reading
!                    in matlab
!     SEPACTSOLVE    Activate arrays for subroutine seplinsol
!     SEPCOPYMT      Copy matrix
!     SEPFILLSOLMOD  Fill several modules for subroutine seplinsol
!     SEPLINSOLC     Solve system of linear equations (complex matrix)
!     SEPLINSOLR     Solve system of linear equations (real matrix)
!     SEPPRINTMAT    Print matrix
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplinsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplinsol'

      end if
      if ( ierror/=0 .or. irhsd(1)==0 ) go to 1000

!     --- In case of a direct solver (typesolv<=0) and if
!         we have to compute the residual (irescomp>0) or if
!         iterative refinement must be applied (nmax>0)
!         Copy the matrix. Information is stored in icpmat

      icpmat = 0
      if ( typesolv<=0 .and. (irescomp>0 .or. nmax>0) ) then
         call sepcopymt ( matr, icpmat )
         iseqmt2 = icpmat
         iseqmt = matr(1)
      end if  ! ( typesolv<=0 .and. (irescomp>0 .or. nmax>0) )
      if ( ierror/=0 ) go to 1000
      if ( numdef_corr>0 ) iseqmt2 = matr(2)

!     --- Activate arrays

      call sepactsolve ( isol, ks(irhsd(1))%compl )
      if ( ierror/=0 ) go to 1000

!     --- Fill parameters in sepmodulekprob
!         Create arrays usolbuf and rhsdbuf.
!         Also usol and rhsd in reordered sequence

      call sepfillsolmod ( iseqmt, irhsd, isol )

      if ( ierror/=0 .or. nrusol==0 ) go to 1000

      if ( printlev>=4 ) then

!     --- printlev>=4, print matrix and right-hand side

         call sepprintmat ( matr(1) )
         call prinrl ( rhsd, ks(irhsd(1))%nusol, 'rhsd' )

      end if  ! ( printlev>=4 )

      if ( write_matlab_mat ) call prmatlab

!     --- Solve system

      if ( realmt(matr(1))%complexmat ) then
         call seplinsolc ( matr, isol, irhsd )
      else
         call seplinsolr ( matr, isol, irhsd )
      end if  ! ( realmt(matr(1))%complmat )
      realmt(matr(1))%decomp = .false.

      if ( icpmat>0 ) call seprmmatrix ( icpmat )

      if ( write_matlab_mat ) then

!     --- Write solution in matlab format

         call prmatlabvector ( isol(iseqsol), 'sol.dat' )

      end if  ! ( write_matlab_mat )

      if ( jcart>1 ) ks(isol(iseqsol))%jcart = jcart-1

1000  if ( debug ) then

!     --- Debug information

      end if
      call erclos ( 'seplinsol' )

      end subroutine seplinsol
