      subroutine seplinsolcompl ( matr, isol, irhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations by either a direct or an iterative
!     method
!     Complex matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulecmat
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer matr(*), isol(*), irhsd(*)

!     irhsd          i    Standard SEPRAN array, containing information of the
!                         large vector.
!                         If defect correction is applied, irhsd must contain
!                         two vectors and hence must be declared as irhsd(5,2)
!                         irhsd(.,1) corresponds to matr(.,1) including the
!                                   effect of boundary conditions
!                         irhsd(.,2) corresponds to matr(.,2) including the
!                                   effect of boundary conditions
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
!                         If defect correction is applied, matr must contain two
!                         matrices and hence must be declared as matr(5,2)
!                         matr(.,1) corresponds to the matrix to be inverted,
!                                   for example the upwind matrix
!                         matr(.,2) corresponds to the matrix to be used for the
!                                   correction, for example the central matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (cmatrix), pointer :: A
      logical :: new

!     A              Structure of type matrix containing the necessary
!                    information of the matrix
!     new            Temporary parameter indicating that the solvers are
!                    used.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCCOPYMATCS  Copy symmetric compact matrix in type matrix
!     SEPZSOLVE      Solve linear system of equations
!                    Complex matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2140   ISEQ_RES_EXACT has been given, but not ISEQ_EXACT
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplinsolcompl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplinsolcompl'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store matrix in type matrix

      A => realmt(iseqmt)%Sc_i
      call sepccopymatcs ( A )
      if ( debug ) write(irefwr,1) 'typesolv',  A%typesolv

      iseqmt = matr(1)
      if ( A%typesolv==0 ) A%decomp = realmt(iseqmt)%decomp

      if ( iseqres_exact>0 ) then

!     --- iseqres_exact>0, compute residual of exact solution

         if ( iseq_exact<=0 ) then

!        --- iseq_res_exact given, but not iseq_exact

            call errint ( iseqres_exact, 1 )
            call errsub ( 2140, 1, 0, 0 )

         else

!        --- Compute residual

            zresusolex = cresidual ( A, zusolex, zrhsd )

         end if  ! ( iseqexact<=0 )

      end if  ! ( iseqres_exact>0 )

      if ( iseq_start_res>0 ) zstartres = cresidual ( A, zusol, zrhsd )

      call sepzsolve ( A )
      inpsol(1,1) = A%numiter
      if ( A%typesolv==0 ) realmt(iseqmt)%decomp = A%decomp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplinsolcompl'

      end if  ! ( debug )
      call erclos ( 'seplinsolcompl' )

      end subroutine seplinsolcompl
