      subroutine seploctrncm ( itrans, usol )
! ======================================================================
!
!        programmer     Guus Segal
!        version  1.0  date 26-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute (back-)transformation when local transformations are
!     performed
!     complex version
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: itrans
      double complex, intent(inout) :: usol(*)

!     itrans     i    Indicates if transformation from standard vector (i.e.
!                     usual degrees of freedom, for example Cartesian) to
!                     transformed vector (i.e. with transformed degrees of
!                     freedom in points with local transform, for example normal
!                     and tangential components) is defined (1) or that the
!                     backwards transformation to standard vector must be
!                     computed (2)
!     usol      i/o   solution array
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, inus1, inus2, inus3, k
      double precision :: anx, any, det, v1(3), v2(3), v3(3), &
                          r11, r12, r13, r21, r22, r23, r31, r32, r33
      double complex :: u, v, w, uvec(3)

!     anx        x-component of normal vector
!     any        y-component of normal vector
!     det        determinant
!     i          Counting variable
!     inus1      Position in usol corresponding to first unknown in point
!     inus2      Position in usol corresponding to second unknown in point
!     inus3      Position in usol corresponding to third unknown in point
!     k          Counting variable
!     r11        R(1,1) component of matrix R
!     r12        R(1,2) component of matrix R
!     r13        R(1,3) component of matrix R
!     r21        R(2,1) component of matrix R
!     r22        R(2,2) component of matrix R
!     r23        R(2,3) component of matrix R
!     r31        R(3,1) component of matrix R
!     r32        R(3,2) component of matrix R
!     r33        R(3,3) component of matrix R
!     u          first unknown in point
!     uvec       Contains the vector (u,v,w)
!     v          second unknown in point
!     v1         Contains 1-th column of matrix R
!     v2         Contains 2-th column of matrix R
!     v3         Contains 3-th column of matrix R
!     w          third unknown in point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       The tranformation is given by u = R un,
!
!       with R depending on maxtrn:
!
!
!       In R^2:
!                | nx   -ny |              | r11   r12 |
!       2:   R = |          |   4,8:   R = |           |
!                | ny    nx |              | r21   r22 |
!
!
!       this means:
!
!       2:
!            |  nx un - ny ut |             | nx u1 + ny u2 |
!       u  = |                |       un =  |               |
!            |  ny un + nx ut |             |-ny u1 + nx u2 |
!
!       4,8:
!            |  r11 un + r12 ut |            1 | r22 u1 - r12 u2 |
!       u  = |                  |       un = - |                 |
!            |  r21 un + r22 ut |            d |-r21 u1 + r11 u2 |
!
!                                       d = r11 r22 - r12 r21
!
!       In R^3:
!
!       See 4:, but now extended to three dimensions
!
!    if ( itrans=1 ) then
!       perform forward transformation
!    else
!       perform backtransformation
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ndim==2 ) then

!     --- ndim = 2

         if ( itrans==1 ) then

!        --- itrans = 1, forward transformation

            if ( maxtrn==2 ) then

!           --- Standard transformation

               do i = 1, npltra
                  if ( indprh==0 ) then
                     inus1 = kprobn(1,i)
                     inus2 = kprobn(2,i)
                  else
                     inus1 = kprobhinv(kprobn(1,i))
                     inus2 = kprobhinv(kprobn(2,i))
                  end if
                  anx = coortr(1,i)
                  any = coortr(2,i)
                  u = usol(inus1)
                  v = usol(inus2)
                  usol(inus1) =  anx*u+any*v
                  usol(inus2) = -any*u+anx*v
               end do

            else

               do i = 1, npltra
                  if ( indprh==0 ) then
                     inus1 = kprobn(1,i)
                     inus2 = kprobn(2,i)
                  else
                     inus1 = kprobhinv(kprobn(1,i))
                     inus2 = kprobhinv(kprobn(2,i))
                  end if
                  r11 = coortr(1,i)
                  r12 = coortr(2,i)
                  r21 = coortr(3,i)
                  r22 = coortr(4,i)
                  det = r11*r22-r12*r21
                  u = usol(inus1)
                  v = usol(inus2)
                  usol(inus1) = (r22*u-r12*v)/det
                  usol(inus2) = (-r21*u+r11*v)/det
               end do

            end if

         else

!        --- itrans = 2, back transformation

            if ( maxtrn==2 ) then

!           --- Standard transformation

               do i = 1, npltra
                  if ( indprh==0 ) then
                     inus1 = kprobn(1,i)
                     inus2 = kprobn(2,i)
                  else
                     inus1 = kprobhinv(kprobn(1,i))
                     inus2 = kprobhinv(kprobn(2,i))
                  end if
                  anx = coortr(1,i)
                  any = coortr(2,i)
                  u = usol(inus1)
                  v = usol(inus2)
                  usol(inus1) = anx*u-any*v
                  usol(inus2) = any*u+anx*v
               end do

            else

               do i = 1, npltra
                  if ( indprh==0 ) then
                     inus1 = kprobn(1,i)
                     inus2 = kprobn(2,i)
                  else
                     inus1 = kprobhinv(kprobn(1,i))
                     inus2 = kprobhinv(kprobn(2,i))
                  end if
                  r11 = coortr(1,i)
                  r12 = coortr(2,i)
                  r21 = coortr(3,i)
                  r22 = coortr(4,i)
                  u = usol(inus1)
                  v = usol(inus2)
                  usol(inus1) = r11*u+r12*v
                  usol(inus2) = r21*u+r22*v
               end do

            end if

         end if

      else

!     --- ndim = 3

         if ( itrans==1 ) then

!        --- itrans = 1, forward transformation

            do i = 1, npltra
               if ( indprh==0 ) then
                  inus1 = kprobn(1,i)
                  inus2 = kprobn(2,i)
                  inus3 = kprobn(3,i)
               else
                  inus1 = kprobhinv(kprobn(1,i))
                  inus2 = kprobhinv(kprobn(2,i))
                  inus3 = kprobhinv(kprobn(3,i))
               end if
               do k = 1, 3
                  v1(k) = coortr(3*k-2,i)
                  v2(k) = coortr(3*k-1,i)
                  v3(k) = coortr(3*k  ,i)
               end do
               det = v1(1) * ( v2(2)*v3(3) - v3(2)*v2(3) ) + &
                     v2(1) * ( v3(2)*v1(3) - v1(2)*v3(3) ) + &
                     v3(1) * ( v1(2)*v2(3) - v2(2)*v1(3) )
               uvec(1) = usol(inus1)
               uvec(2) = usol(inus2)
               uvec(3) = usol(inus3)
               usol(inus1) = uvec(1) * ( v2(2)*v3(3) - v3(2)*v2(3) ) + &
                             v2(1) * ( v3(2)*uvec(3) - uvec(2)*v3(3) ) + &
                             v3(1) * ( uvec(2)*v2(3) - v2(2)*uvec(3) )
               usol(inus2) = v1(1) * ( uvec(2)*v3(3) - v3(2)*uvec(3) ) + &
                             uvec(1) * ( v3(2)*v1(3) - v1(2)*v3(3) ) + &
                             v3(1) * ( v1(2)*uvec(3) - uvec(2)*v1(3) )
               usol(inus3) = v1(1) * ( v2(2)*uvec(3) - uvec(2)*v2(3) ) + &
                             v2(1) * ( uvec(2)*v1(3) - v1(2)*uvec(3) ) + &
                             uvec(1) * ( v1(2)*v2(3) - v2(2)*v1(3) )
            end do

         else

!        --- itrans = 2, back transformation

            do i = 1, npltra
               if ( indprh==0 ) then
                  inus1 = kprobn(1,i)
                  inus2 = kprobn(2,i)
                  inus3 = kprobn(3,i)
               else
                  inus1 = kprobhinv(kprobn(1,i))
                  inus2 = kprobhinv(kprobn(2,i))
                  inus3 = kprobhinv(kprobn(3,i))
               end if
               r11 = coortr(1,i)
               r12 = coortr(2,i)
               r13 = coortr(3,i)
               r21 = coortr(4,i)
               r22 = coortr(5,i)
               r23 = coortr(6,i)
               r31 = coortr(7,i)
               r32 = coortr(8,i)
               r33 = coortr(9,i)
               u = usol(inus1)
               v = usol(inus2)
               w = usol(inus3)
               usol(inus1) = r11*u+r12*v+r13*w
               usol(inus2) = r21*u+r22*v+r23*w
               usol(inus3) = r31*u+r32*v+r33*w
            end do

         end if

      end if

      end
