      subroutine sepnestlu ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the preconditioning matrix in case of row-compact storage
!     ipreco = 3, Incomplete LU-decomposition
!     Real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, jp, jk, idiag, ii
      integer, pointer :: point(:), diag(:), row(:), col(:)
      double precision, pointer :: mat(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     col            Column numbers of matrix
!     diag           Positions of diagonal elements of the large matrix
!     i              Counting variable
!     idiag          Position of diagonal element in matrix
!     ii             Position of diagonal element i in matrix
!     j              Counting variable
!     jk             Position of element j,k in matrix
!     jp             position of element i,j in matrix
!     k              Counting variable
!     mat            elements of matrix
!     point          Array with positions of specific matrix elements
!     row            row numbers of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnestlu' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnestlu'
         write(irefwr,1) 'n', A%neq
         write(irefwr,1) 'iseqpre, iseqintmat', iseqpre, iseqintmat
         write(irefwr,1) 'prlump', A%prlump
         write(irefwr,2) 'lapshift, adapt', lapshift, A%adapt
         call sepprinmat6p ( A )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      diag => A%Mdiag
      row => A%Mrows
      col => A%Mcols
      mat => A%MD

      IM => intmt(intmat)

      allocate ( point(A%neq) )
      point = 0

!     --- Loop over rows

      do i = 2, A%neq

!     --- Loop over columns to fill start in point

         do j = row(i)+1, row(i+1)
            point(col(j)) = j
         end do  ! j = row(i)+1, row(i+1)
         ii = diag(i)

!        --- Loop over columns in lower triangular part

         do jp = row(i)+1, diag(i)-1

            j = col(jp)  ! column number
            mat(jp) = mat(jp)/mat(diag(j))

!           --- Loop over columns in upper triangular part

            do jk = diag(j)+1, row(j+1)

               k = point(col(jk))
               if ( k>0 ) then

!              --- k> 0, update element k

                  mat(k) = mat(k)-mat(jp)*mat(jk)

               end if  ! ( k>0 )

            end do  ! k = diag(j)+1, row(j+1)-1

         end do  ! j = row(i)+1, diag(i)-1

!        --- Reset point

         do j = row(i)+1, row(i+1)
            point(col(j)) = 0
         end do  ! j = row(i)+1, row(i+1)-1

!        --- If |mat(ii)| is zero give an error message
!            and stop

         if ( abs(mat(ii))<=0d0 ) then
            call errint ( i, 1 )
            call erreal ( mat(ii), 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

      end do  ! i = 2, A%neq

      deallocate ( point )

1000  if ( debug ) then

!     --- Debug information

         call sepprinmat6p ( A )
         write(irefwr,*) 'End sepnestlu'

      end if  ! ( debug )
      call erclos ( 'sepnestlu' )

      end subroutine sepnestlu
