      subroutine sepnestsol ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of real linear equations by Gaussian elimination
!     The matrix is supposed to be decomposed before
!     nested dissection
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     direct_method
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmoduleintmat
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Matrix A
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPNESTSOLLLT  Solve system of real linear equations decomposed into LDL^T
!                    (nested dissection)
!     SEPNESTSOLLU   Compute y = A^(-1) y, where A is decomposed into LU
!                    a row-compact storage is used
!     SEPSCMATDIV    Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and U
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnestsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnestsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( IM%symmetric ) then

!     --- symmetric matrix

         if ( ipos==1 ) then

!        --- Matrix positive definite

            usol = rhsd
            call sepscmatdiv ( A, usol )

         else

!        --- General symmetric matrix

            call sepnestsolllt ( A, usol, rhsd )

         end if  ! ( ipos==1 )

      else

!     --- asymmetric matrix

         call sepnestsollu ( A, usol, rhsd )

      end if  ! ( IM%symmetric )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnestsol'

      end if  ! ( debug )
      call erclos ( 'sepnestsol' )

      end subroutine sepnestsol
