      function sepnorm ( ichois, ichnrm, jdegfd, isol1, isol2, ielhlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 29-05-2014 Extension of timebefore
!        version  1.4  date 17-05-2008 New call to to0035
!        version  1.3  date 30-11-2002 New call to to0022
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute norm of vector or norm of difference of two vectors u1-u2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, ichnrm, jdegfd, isol1, isol2, ielhlp(*)
      double precision sepnorm

!     ichnrm  i    choice parameter indicating the type of norm to be
!                  computed.  Possibilities:
!                   1   l1-norm
!                   2   l2-norm
!                   3   max-norm
!                   4   l1-norm / number of degrees of freedom
!                   5   l2-norm / number of degrees of freedom
!                  10   max-norm  the nodal point number corresponding to the
!                       maximum is stored in ielhlp(1)
!     ichois  i    choice parameter. Possibilities:
!                  0   ||u1 -u2|| is computed
!                  1   ||u1|| is computed
!     ielhlp  o    this array is only used when ichnrm=10
!     isol1   i    in this array information concerning the vector u1
!                  must be stored
!     isol2   i    in this array information concerning the vector u2
!                  must be stored. isol2 is only used when ichois=0
!     jdegfd  i    indication whether all degrees of freedom of the arrays
!                  are used (jdegfd=0) or only the jdegfd th
!     sepnorm o    Computed norm
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, ichvec, iconst(2), idegfd(1), indelgrp(1)
      double precision const(3)
      double precision :: timebefore(2)

!    const        Constant array for subroutine VECMAN
!    ichvec       Parameter ICHOIS for VECMAN
!    iconst       Information array for subroutine VECMAN
!    idegfd       Help array to store jdegfd
!    indelgrp     dummy
!    itypvc       Indication of the type of solution vector:
!                 110: solution vector
!                 115, 116: vectors of special structure
!    timebefore   Time at entrance of subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN     Resets old name of previous subroutine of higher level
!     EROPEN     Produces concatenated name of local subroutine
!     ERRINT     Put integer in error message
!     ERRSUB     Error messages
!     ERSETTIME  Get the CPU time at start of subroutine
!     PRININ         print 1d integer array
!     TO0022     test if vectors are of equal type
!     TO0035     Algebraic manipulations for vectors of type 110, 115 and 116
!     TO0110     Algebraic manipulations (special possibilities)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     217   IDEGFD has wrong value
!     314   ICONST(1)/ICHNRM has wrong value, whereas ICHOIS = 15, &
!           16 or ANORM/DIFFVC has been called
!    1153   vector INVEC1 corresponds to a wrong type of vector, &
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- tests

      call eropen ( 'sepnorm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnorm'
         write(irefwr,1) 'ichois, ichnrm, jdegfd', ichois, ichnrm, jdegfd
         write(irefwr,1) 'isol1, isol2', isol1, isol2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( itime>0 ) call ersettime ( timebefore )
      if ( jdegfd<-1 ) then
         call errint ( jdegfd, 1 )
         call errsub ( 217, 1, 0, 0 )
      end if
      if ( ichois<0 .or. ichois>1 ) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      itypvc = ks(isol1)%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 .and. &
           itypvc/=127 ) then
         call errint ( itypvc, 1 )
         call errsub ( 1153, 1, 0, 0 )
      end if
      if ( ichnrm<0 .or. ichnrm>5 .and. ichnrm/=10 ) then
         call errint ( ichnrm, 1 )
         call errsub ( 314, 1, 0, 0 )
      end if
      if ( ichois==0 ) then

!     --- ichois = 0 hence array isol2 is used
!         Check if arrays isol1 and isol2 are identical

         call to0022 ( isol1, isol2, 1, 2 )

      end if

      if ( debug ) write(irefwr,1) 'ierror', ierror

!     --- Actual call

      if ( ierror==0 ) then
         ichvec = 15+ichois
         iconst(1) = ichnrm
         idegfd(1) = jdegfd
         if ( ichnrm==10 ) then
            call to0110 ( ichvec, idegfd, isol1, isol2, const, iconst )
            ielhlp(1) = iconst(2)
         else
            call to0035 ( ichvec, idegfd, isol1, isol2, &
                          isol1, const, iconst, &
                          0, indelgrp, .true. )
         end if
         sepnorm = const(1)
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnorm'

      end if  ! ( debug )
      call erclmn ( 'sepnorm', timebefore )
      end function sepnorm
