      subroutine seppargcr ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system of equation by the preconditioned CGR method
!     Parallel case (block jacobi)
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: x(A%neq), y(A%neq)

!     A              i    Matrix A
!     x              o    Solution vector
!     y              i    Right-hand side
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: res(:), v(:), s(:)
      double precision, allocatable, target :: svec(:,:), vvec(:,:)
      integer :: nmax, iter, n, ngcr, imin, i, &
                 ireler, numsubiter
      double precision :: bnorm, alphamin, alpha, vnorm, rnorm, eps, &
                          rnormprev
      logical :: finished, extradebug

!     alpha          variable used in the iteration
!     alphamin       Minimum value of alpha
!     bnorm          Norm of the right-hand side (b,b)
!     eps            Accuracy
!     extradebug     If true extra debugging statements are carried out
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     imin           Position in array for which minimum is reached
!     ireler         Defines how the error is measured, see inpsol(11)
!     iter           Iteration sequence number
!     n              number of equations (size of matrix)
!     ngcr           Maximum number of GCR iterations
!     nmax           maximum number of iterations that may be performed by
!                    the CG-process. In any way no more than the number of
!                    degrees of freedom iterations are carried out.
!     numsubiter     Number of subiterations
!     res            residual
!     rnorm          Norm of residual
!     rnormprev      Norm of prior residual
!     s              active vector s during iteration
!     svec           Work array of size ngcr x n_u to store the vectors
!                    s during the iteration
!     v              active vector v during iteration
!     vnorm          L2 norm of v
!     vvec           Work array of size ngcr x n_u to store the vectors
!                    v during the iteration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPSOLVEMETH   Solve system of linear equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The algorithm for one step reads
!
!        s^(k+1) := dx, r = res  (! input )
!        v^(k+1) := A s^(k+1)
!        For i := 1 (1) k do
!           alpha := ( v^(k+1), v^i)
!           v^(k+1) := v^(k+1)- alpha v^i
!           s^(k+1) := s^(k+1)- alpha s^i
!        v^(k+1) := v^(k+1)/||v^(k+1)||
!        s^(k+1) := s^(k+1)/||v^(k+1)||
!        alpha := ( r, v^(k+1))
!        x := x + alpha s^(k+1)
!        r := r - alpha v^(k+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppargcr' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.

!     --- Get parameters

      n = A%neq             ! number of free degrees of freedom in cpu
      nglob = A%nglob       ! number of global degrees of freedom

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppargcr'
         write(irefwr,1) 'n', n

         if ( extradebug ) call prinrl ( x, n, 'x' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      bnorm = norm(y, A)
      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x =0d0
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      nmax = A%nmax                  ! maximum number of iterations
      if ( nmax==0 ) nmax=10*n
      iprint = A%iprint              ! print level
      ngcr = A%ntrunk                ! (maximum) number of gcr iterations
      ireler = A%irelerr
      eps = A%eps

      if ( A%typeparsolv==2 ) then

!     --- Use subsolver, hence
!         Set parameters for subsolver

         A%irelerr = 1  ! relative error with respect to residual
         A%iaccuracy = 1  ! relative error with respect to residual
         A%iabserr = 0
         if ( iprint<2 ) A%iprint = -1  ! no printing for subsolver
         numsubiter = 0

      end if  ! ( A%typeparsolv==2 )

      if ( debug ) &
         write(irefwr,1) 'nmax, iprec, iprint, ngcr', &
                          nmax, A%type_prec, iprint, ngcr

!     --- Allocate svec and vvec

      allocate( svec(n,0:ngcr+1), vvec(n,0:ngcr+1), res(n), &
                stat = error)
      if ( error/=0 ) call eralloc ( error, 2*n*(ngcr+2), 'svec' )
      if ( ierror/=0 ) go to 1000

      v => vvec(:,0)
      s => svec(:,0)

!     --- Set start vector and initial residu

      v = A*x
      res = y-v
      s = res

      if ( extradebug ) then
         call prinrl ( v, n, 'v' )
         call prinrl ( res, n, 'res' )
      end if  ! ( extradebug )

      finished = .false.

      rnorm = norm(res, A)

      if ( ireler==1 ) eps = eps*rnorm
      if ( A%typeparsolv==2 ) A%scaling = 0  ! no scaling is applied

      iter = 0

      if ( iprint>1 ) write(irefwr,100) iter, rnorm, 1d0
100   format( 'iteration   norm    speed'/ i6, ":", 2d12.3 )

      do while ( .not. finished .and. iter<nmax )

!     --- Perform preconditioning step

         if ( A%precon .and. A%typeparsolv==1 ) then

!        --- Standard block Jacobi

            s = res/A

         else if ( A%typeparsolv==2 ) then

!        --- preconditioning by subsolver

            A%eps = A%subsolve*rnorm
            A%eps2 = A%eps
            A%eps1 = 0
            parallel = .false.  ! use a serial subsolver
            call sepsolvemeth ( A, s, res )
            parallel = .true.  ! reset to parallel solver
            numsubiter = numsubiter+A%numiter  ! number of sub iterations

         else

!        --- No preconditioning

            s = res

         end if  ! ( A%precon )

         rnormprev = rnorm

         if ( extradebug ) then
            call prinrl ( s, n, 's' )
         end if  ! ( extradebug )

!        --- Matrix vector multiplication

         v = A*s

         imin = min(iter+1,ngcr)
         alphamin = rinfin

         do i = 1, min(iter,ngcr)

            alpha = dotprod(vvec(:,0), vvec(:,i), A) ! ( v^(k+1), v^i)

            if ( iter>ngcr .and. abs(alpha)<alphamin ) then

!           --- alpha<alphamin, set alpha = alphamin and imin = i
!               Only if the number of iterations exceeds ngcr we compute
!               the minimum value of alpha
!               We apply the truncated GCR process by putting the new vector in
!               position imin

               alphamin = abs(alpha)
               imin = i

            end if

            v = v-alpha * vvec(:,i) !v^(k+1)=v^(k+1)-alpha v^i
            s = s-alpha * svec(:,i) !s^(k+1)=s^(k+1)-alpha s^i

         end do  ! i = 1, min(iter,ngcr)

         vnorm = norm(v, A)

         if ( debug) write(irefwr,2) 'vnorm', vnorm

         v = v/vnorm
         s = s/vnorm

!        --- Copy result to new vector

         vvec(:,imin) = v
         svec(:,imin) = s

         alpha = dotprod(v,res, A)  ! v*res
         x = x+alpha*s
         res = res-alpha*v

         if ( debug) write(irefwr,2) 'alpha, alphamin', alpha, alphamin

         if ( extradebug ) then
            call prinrl ( v, n, 'v' )
            call prinrl ( s, n, 's' )
            call prinrl ( x, n, 'x' )
            call prinrl ( res, n, 'res' )
         end if  ! ( extradebug )

         iter = iter+1  ! raise iter

         rnorm = norm(res, A)

         if ( iprint>1 ) &
            write(irefwr,110) iter, rnorm, rnorm/rnormprev
110      format( i6, ":", 2d12.3 )

         finished = rnorm<eps

      end do  ! while ( .not. finished .and. iter<nmax )

      A%numiter = iter-1

      if ( iprint==1 ) then

         write(irefwr,120) iter
120      format ( 'The number of iterations carried out in the GCR', &
                  ' process is ', i5 )

      end if  ! ( iprint==1 )

      if ( iprint>=1 .and. A%typeparsolv==2 ) &
         write(irefwr,130) numsubiter
130   format ( 'Number of sub-iterations is ', i6 )

!     --- Deallocate arrays

      deallocate ( svec, vvec, res, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'svec' )

1000  call erclos ( 'seppargcr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppargcr'

      end if  ! ( debug )

      end subroutine seppargcr
