      subroutine sepprecilu10 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the preconditioning matrix in case of row-compact storage
!     ipreco = 3, Incomplete LU-decomposition
!     Real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, jp, kp, idiag
      integer, pointer :: point(:), diag(:), row(:), col(:)
      double complex, pointer :: mat(:)
      double complex :: shift
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     col            Column numbers of matrix
!     diag           Positions of diagonal elements of the large matrix
!     i              Counting variable
!     idiag          Position of diagonal element in matrix
!     j              Counting variable
!     jp             position of element i,j in matrix
!     k              Counting variable
!     kp             Position of element j,k in matrix
!     mat            elements of matrix
!     point          Array with positions of specific matrix elements
!     row            row numbers of matrix
!     shift          Laplace shift
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INSTOP         Stop the program
!     PRINMTROWCOMP  Print matrix in case of amg row compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecilu10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecilu10'
         write(irefwr,1) 'n', A%neq
         write(irefwr,2) 'lapshift, adapt', lapshift, A%adapt
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- ipreco = 3,  Incomplete LU-decomposition
!         Copy diag into A%MD

      if ( iseqpre/=iseqintmat ) then

!     --- the structure of the preconditioning
!         matrix and the standard matrix is different

     print *, 'option iseqpre/=iseqintmat not yet implemented'
     call ertrace
     call instop
    !     call preccopy ( A%neq, A%rows, A%Mrows, A%L, A%U, &
    !                     A%ML, A%MU, A%cols, A%Mcols, A%MD, A%S, 6 )

      else

!     --- both matrices have the same structure
!         Copy matrix into premat

         A%MD = A%S
         A%Mrows => A%rows
         A%Mcols => A%cols
         A%Mdiag => A%diag
         if ( debug )  call prinmtrowcomp ( A%MD )

      end if  ! ( inpsol(40)>0 )

      diag => A%Mdiag
      row => A%Mrows
      col => A%Mcols
      mat => A%MD

      IM => intmt(intmat)

!     --- Adapt diagonal in case of shifted laplace

      if ( IM%lapshift ) then
         shift = clapshift+(1d0,0d0)
         do i = 1, A%neq
            idiag = diag(i)
            mat(idiag) = mat(idiag)-shift*A%DLshift(i)
         end do  ! i = 1, A%neq
      end if  ! ( IM%lapshift )

      allocate ( point(A%neq) )
      point = 0

!     --- Loop over rows

      do i = 1, A%neq

!     --- Loop over columns to fill start in point

         do j = row(i)+1, row(i+1)
            point(col(j)) = j
         end do  ! j = row(i)+1, row(i+1)

!        --- Loop over columns in lower triangular part

         do jp = row(i)+1, diag(i)-1

            j = col(jp)  ! column number

            mat(jp) = mat(jp)/mat(diag(j))

!           --- Loop over columns in upper triangular part

            do kp = diag(j)+1, row(j+1)

               k = point(col(kp))
               if ( k>0 ) then

!              --- k>0, update element k

                  mat(k) = mat(k)-mat(jp)*mat(kp)

               end if  ! ( k>0 )

            end do  ! k = diag(j)+1, row(j+1)-1

         end do  ! j = row(i)+1, diag(i)-1

!        --- Reset point

         do j = row(i)+1, row(i+1)
            point(col(j)) = 0
         end do  ! j = row(i)+1, row(i+1)-1

      end do  ! i = 2, A%neq

!     --- Scale ILU matrix

      do  i = 1, A%neq
         do k = diag(i)+1, A%Mrows(i+1)
            mat(k) = mat(k) / mat(diag(col(k)))
         end do
      end do

      deallocate ( point )

1000  if ( debug ) then

!     --- Debug information

         call prinmtrowcomp ( A%MD )
         write(irefwr,*) 'End sepprecilu10'

      end if  ! ( debug )
      call erclos ( 'sepprecilu10' )

      end subroutine sepprecilu10
