      subroutine sepprecmeissym ( amax, A, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the preconditioning matrix for the CG-process
!     iprec = 7,  Modified Eisenstat preconditioning
!     Old subroutine prcg01prec7
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(in) :: amax
      double precision, intent(inout) :: work(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     amax           i    maximal element of a matrix
!     work                Work array that is used to store the row sum of the
!                         matrix U in the eisentat MILU preconditioning
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, j, kpo, k1
      double precision :: sum1
      integer, pointer :: point(:), rowhor(:), colhor(:)

!     colhor         contains column numbers per horizontal row
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     k1             column number
!     kpo            position of element k1 in preconditioning matrix
!     point          pointer array referring to next element in a row
!     rowhor         contains starting addresses (-1) of horizontal rows in
!                    colhor
!     sum1           Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
!    2236   The desired preconditioning matrix does not exist, lumping is tried
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecmeissym' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecmeissym'

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( point(A%neq+1), rowhor(A%neq+1), colhor(size(A%cols)) )

!     --- Store row structure of matrix in row/colhor

      call sepcommatrev ( rowhor, colhor, A%rows, A%cols, A%neq )

!     --- First fill array with row sum of matrix U

700   work = 0d0

      do i = 1, A%neq
         do k = A%rows(i)+2, A%rows(i+1)
            work(i) = work(i) + A%S(k)
         end do
      end do

      if ( A%adapt>0d0 ) then

!     --- adapt diagonal

         A%MD = max(A%S(A%diag),A%adapt)

      else

         A%MD = A%S(A%diag)

      end if

      if ( A%prlump==1) then

!     --- Positive off-diagonal elements are lumped to the main
!         diagonal

         do i = 1, A%neq
            do k = A%rows(i)+2, A%rows(i+1)
               if ( A%S(k)>0d0 ) then
                  A%MD(i) = A%MD(i) + A%S(k)
                  A%MD(A%cols(k)) = A%MD(A%cols(k)) + A%S(k)
               end if
            end do
         end do

      end if

!     --- end of lumping

      point = A%rows+2

      do i = 1, A%neq
         sum1 = A%MD(i)
         do k = rowhor(i)+1, rowhor(i+1)-1
            k1 = colhor(k)
            kpo = point(k1)

!           --- The sum is not adapted when prlump = 1 and the
!               off-diagonal is positive

            if ( A%S(kpo)<=0d0 .or. A%prlump/=1 ) then
               sum1 = sum1 - A%S(kpo)* ( (1d0-A%factor)*A%S(kpo) + &
                     A%factor*work(k1) ) / A%MD(k1) ** 2
            end if
            point(k1) = point(k1)+1

         end do  ! k = rowhor(i)+1, rowhor(i+1)-1

         if ( A%prlump/=1 .and. &
              ( sum1<=0d0 .or. sum1>=amax ) ) then

!        --- If sum becomes too small or too big make prlump = 1
!            and start again.

             call errwar ( 2236, 0, 0, 0)
             A%prlump = 1
             goto 700

         end if

!        --- If sum becomes too small and prlump = 1 give an error
!            message and stop

         if ( sum1<=0d0 ) then
            call errint ( i, 1 )
            call erreal ( sum1, 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

         A%MD (i) = sqrt ( sum1 )
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprecmeissym'

      end if
      call erclos ( 'sepprecmeissym' )

      end subroutine sepprecmeissym
