      subroutine sepprecmodeis9 ( A, amax, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the preconditioning matrix for the CGS-process
!     ipreco = 7,  Eisenstat preconditioning (MILU)
!     Old subroutine prcg02_7
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(in) :: amax
      double precision, intent(inout) :: work(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     amax           i    maximal element of a matrix
!     work                Work array that is used to store the row sum of the
!                         matrix U in the eisentat MILU preconditioning
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, idiag, point(A%neq), iposu, j, jp
      double precision :: sum1
      integer, pointer :: diag(:), row(:), col(:)
      double precision, pointer :: mat(:)

!     col            Column numbers of matrix
!     diag           positions of diagonal elements of the matrix
!     i              Counting variable
!     idiag          Position of diagonal element in matrix
!     iposu          point(j)
!     j              column number
!     jp             position in matrix
!     mat            elements of matrix
!     point          pointer to irowsii in icolsii
!     row            row numbers of matrix
!     sum1           Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINMTROWCOMP  Print matrix in case of amg row compact storage
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
!    2236   The desired preconditioning matrix does not exist, lumping is tried
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecmodeis9' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecmodeis9'
         call prinmtrowcomp ( A%S )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- ipreco = 7,  Eisenstat preconditioning (MILU)

      diag => A%diag
      row => A%rows
      col => A%cols
      mat => A%MD

!         First fill array with row sum of matrix U

100   work = 0d0

      do i = 1, A%neq
         do j = diag(i)+1, row(i+1)
            work(i) = work(i) + A%S(j)
         end do
      end do

      if ( debug ) call prinrl ( work, A%neq, 'work' )

      if ( skipzeros ) then

!     --- Zero diagonal elements are allowed

         A%adapt = max(A%adapt,epsmac)

      end if  ! ( skipzeros )

!     --- ipreco = 2,  Eisenstat preconditioning

      if ( A%adapt>0d0 ) then

!     --- adapt diagonal

         do i = 1, A%neq
            idiag = diag(i)
            mat(i) = max(A%S(idiag),A%adapt)
         end do

      else

         do i = 1, A%neq
            idiag = diag(i)
            mat(i) = A%S(idiag)
         end do

      end if  ! ( A%adapt>0d0 )
      if ( debug ) call prinrl ( mat, A%neq, 'mat/1' )

!     --- Initialize array point, with next position referring to U

      do i = 1, A%neq
         point(i) = diag(i)+1
      end do


      if ( A%prlump==1 ) then

!     --- Off-diagonal elements with the same sign as the diagonal
!         element are lumped to the main diagonal, except when the
!         diagonal element in the same row or column is equal to zero.

         do i = 1, A%neq

!        --- Loop over columns in lower triangular part

            do jp = row(i)+1, diag(i)-1

               j = col(jp)  ! column number
               if ( A%S(jp)*mat(i)>0d0 .and. &
                    abs(A%S(diag(j)))>0d0 ) then
                  mat(i) = mat(i)+A%S(jp)
               end if  ! ( A%S(jp)*mat(i) .and. .. )

            end do  ! jp = row(i)+1, diag(i)-1

!           --- Loop over columns in upper triangular part

            do jp = diag(i)+1, row(i+1)

               j = col(jp)
               if ( A%S(jp)*mat(i)>0d0 .and. &
                    abs(A%S(diag(j)))>0d0 ) then
                  mat(i) = mat(i)+A%S(jp)
               end if  ! ( mat(jp)*mat(ii)>0d0 .and. .. )

            end do  ! k = diag(j)+1, row(j+1)-1

         end do  ! i = 1, A%neq

      end if

      if ( debug ) call prinrl ( mat, A%neq, 'mat/2' )

!     --- end of lumping

      do i = 1, A%neq

         idiag = diag(i)
         sum1 = mat(i)
         do jp = row(i)+1, idiag-1

!           --- The sum is not adapted when prlump = 1 and the
!               off-diagonals have the wrong sign

            j = col(jp)  ! column number
            iposu = point(j)
            point(j) = point(j)+1

            if ( A%S(jp)*A%S(idiag)<=0d0 .and. &
                 A%S(iposu)*A%S(diag(j))<=0d0 .or. &
                 A%prlump/=1 ) then

               sum1 = sum1 - A%S(jp) * ( (1d0-A%factor)*A%S(iposu) + &
                     A%factor*work(j) ) / mat(j) ** 2

            end if
         end do  ! jp = row(i)+1, idiag-1

         if ( A%prlump/=1 .and. &
              ( sum1<=0d0 .or. sum1>=amax ) ) then

!        --- If sum becomes too small or too big make prlump = 1
!            and start again.

             call errwar ( 2236, 0, 0, 0)
             A%prlump = 1
             goto 100

         end if

!        --- If sum becomes too small and prlump = 1 give an error
!            message and stop

         if ( sum1<=0d0 ) then
            call errint ( i, 1 )
            call erreal ( sum1, 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

         mat (i) = sqrt ( sum1 )
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( mat, A%neq, 'mat/e' )
         write(irefwr,*) 'End sepprecmodeis9'

      end if  ! ( debug )
      call erclos ( 'sepprecmodeis9' )

      end subroutine sepprecmodeis9
