      subroutine sepprojU ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-07-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = U x as part of the projection algorithm
!     U is the matrix of projection vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%nproj)
      double precision, intent(out) :: y(A%neq)

!     A             i/o   Matrix A
!     x              i    Vector to be multiplied
!     y              o    result of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h
      integer :: i, j, idegfd, iblock, numneigh

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iblock         Block sequence number
!     idegfd         Degree of freedom sequence number
!     j              Counting variable
!     numneigh       number of neighbors of a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprojU' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprojU'
         call prinin ( A%projrows, nproj+1, 'A%projrows' )
         call prinin ( A%projcols, A%projrows(nproj+1), 'A%projcols' )
         call prinrl ( A%projU, A%projrows(nproj+1), 'A%projU' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Parallel case
!         y = U x

         numneigh = blockunkinfo(1)
         y = x(inodenr)*A%projU(1:A%neq)

!        --- Now the contributions of the neighboring nodes

         do i = 1, numneigh

            iblock =  blockunkinfo(2*i)

            do j = A%projrows(i)+1, A%projrows(i+1)

               idegfd = A%projcols(j)
               y(idegfd) = y(idegfd)+x(iblock)*A%projU(A%neq+j)

            end do

         end do

      else

!     --- Serial case
!         y = U x

         y = 0d0
         do i = 1, A%nproj

            h = x(i)

            do j = A%projrows(i)+1, A%projrows(i+1)
               idegfd = A%projcols(j)
               y(idegfd) = y(idegfd) + A%projU(j)*h
            end do

         end do

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprojU'

      end if  ! ( debug )
      call erclos ( 'sepprojU' )

      end subroutine sepprojU
