      subroutine sepschur ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations of special form using
!     schur type methods
!     Old subroutine prschur
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     schur_method
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: normrhsd

!     normrhsd       Norm of right-hand side vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
!     ERTRACE        Prints trace back of subroutines
!     SEPCOPYMATPM   Copy real PM matrix in type matrix (simple)
!     SEPSCHURMETHOD Solve a system of linear equations of special form using
!                    the schur complement method
!     SEPSIMPLEDEL   Delete all temporary created arrays in the simple process
!     SEPSIMPLEMATPM Create the preconditioning matrix for PM depending on iprec
!     SEPSIMPLEMATQ  Create the preconditioning matrix for Q depending on iprec
!     SEPSIMPLERES   Compute the residuals r_u and r_p to be used at the start
!                    of the simple algorithm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2446   Right-hand side identical 0
!    2447   Norm of right-hand side is small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The system of equations to be solved reads:
!
!     | Q    G | |u|  = |f_u|
!     | D    0 | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p
!     output: du, dp
!
!     In the Schur method this system is rewritten as:
!
!     | Q      0    | | I  Q^-1G | |u|  = |f_u|
!     | D   -DQ^-1G | | 0    I   | |p|    |f_p|, &
!
!     where the matrix -DQ^-1G is called the Schur complement matrix
!
!     Define    | u* |  =  | I  Q^-1G | |u|
!               | p  |     | 0    I   | |p|
!
!     Then the system can be solved in the following 3 steps:
!
!           Q u* = f
!      -DQ^-1G p = f_p = D u* &
!              u = u* - Q^-1 G p
!
!     Here we solve each of the substeps inaccurately, with the following
!     algorithm
!
!     Start:  given u^0 and p^0
!             ru = f_u - Qu^0 - Gp^0;  r_p = f_p - Du^0
!
!     For k := 0 to maxiter do
!           Q u_f  = ru           inaccurately
!       -DQ^-1G p* = rp - D uf    inaccurately by GCR
!            Q ul  = G p*         inaccurately
!        u^k+1 = u^k + uf-ul
!        p^k+1 = p^k + p* &
!        ru = f_u - Qu^k+1 - Gp^k+1;   r_p = f_p - Du^k+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepschur' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepschur'
         call ertrace

      end if

!     --- Compute the residual r_u = f_u - Q u - G p
!         Compute the residual r_p = f_p - D u
!         Also the arrays for parallel computation are activated
!         as wel as amat and intmat

      call sepsimpleres ( normrhsd, A, usol, rhsd )
      if ( ierror/=0 ) go to 1000

      if ( normrhsd<=0d0 ) then

!     --- Right-hand side is identical 0, return

         call errwar ( 2446, 0, 0, 0 )
         go to 500

      else if ( normrhsd<=1d-10 ) then

!     --- Right-hand side is small: give warning

         call erreal ( normrhsd, 1 )
         call errwar ( 2447, 0, 1, 0 )

      end if

!     --- Make relation between R and pressure mass matrix

      if ( debug ) write(irefwr,*)' before sepcopymatpm'
      call sepcopymatpm ( 4 )

!     --- Create the the preconditioning matrices

      if ( debug ) write(irefwr,*)' before sepsimplematq'
      call sepsimplematq
      if ( debug ) write(irefwr,*)' before sepsimplematpm'
      call sepsimplematpm ( .false., 4 )
      if ( ierror/=0 ) go to 1000

!     --- Perform the GCRsimple algorithm

      if ( debug ) write(irefwr,*)' before sepschurmethod'
      call sepschurmethod ( usol, ru, A )
      if ( ierror/=0 ) go to 1000

!     --- Fill the output values into inpsol(2)

      if ( inpsol(22,1)==1 .and. ierror/=0 ) inpsol(2,1) = 1

!     --- Delete temporary created arrays

      if ( debug ) write(irefwr,*)' before sepsimpledel'
500   call sepsimpledel ( A )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepschur'

      end if  ! ( debug )
      call erclos ( 'sepschur' )

      end
