      subroutine sepschurgcrcycl ( svec, vvec, ngcr, kiter, rnorm, &
                                   rhspress, u, A )
! ======================================================================
!
!        programmer    Mehfooz ur Rehman
!        version  1.0  date 28-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one step of the gcr process wrt schur complement method
!     The matrix vector multiplication corresponds to block methods
!     It is assumed that the preconditioning step has been carried out
!     before
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ngcr, kiter
      double precision, intent(inout) :: svec(np,0:ngcr+1), rhspress(np), &
                                         vvec(np,0:ngcr+1), u(np)
      double precision, intent(out) :: rnorm
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     kiter          i    Iteration sequence number
!     ngcr           i    Maximum number of GCR inner iterations
!     rhspress      i/o   Pressure correction right-hand side
!     rnorm          o    norm of residual
!     svec          i/o   Work array of size ngcr x n_u to store the vectors
!                         s during the iteration
!     u             i/o   Solution vector
!     vvec          i/o   Work array of size ngcr x n_u to store the vectors
!                         v during the iteration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: work(:), duloc(:)
      integer :: scalsav

!                    they are not
!     duloc          Array containing the correction for (u,p)
!     scalsav        Help parameter to save the value of Q%scaling
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPGCRCYCLE    Perform one step of the gcr process
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepschurgcrcycl' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepschurgcrcycl'
         write(irefwr,1) 'gcr iteration :', kiter
         call prinrl ( svec(1,0), np, 'svec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( duloc(nu), work(nu), stat = error)
      if ( error/=0 ) call eralloc ( error, nu, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- v^(k+1) := A s^(k+1), where A = -D Q^-1 G
!         First step G s^0 result in work

      work = G*svec(:,0)

!     --- Next Q^-1 work result in du
!         Multiplication by solving the system

      scalsav = Q%scaling  ! save value
      Q%scaling = 2        ! temporary, seems to work. Why?
      duloc = Q.div.work
      Q%scaling = scalsav  ! reset value

!     --- Finally D du, result in v^0

      vvec(:,0) = D*duloc
      if ( ierror/=0 ) go to 1000

!     --- Perform all other steps

      nrusol = np  ! temporary since we are considering a pressure step
      call sepgcrcycle ( svec, vvec, ngcr, kiter, rnorm, rhspress, u, R )
      nrusol = np+nu ! reset

1000  deallocate ( duloc, work, stat = error)
      if ( error/=0 ) call erdealloc ( error, 'work' )

      call erclos ( 'sepschurgcrcycl' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepschurgcrcycl'

      end if  ! ( debug )

      end
