      subroutine sepschurmethod ( u, normrhsd, A )
! ======================================================================
!
!        programmer    Mehfooz ur Rehman
!        version  1.0  date 11-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using
!     the schur complement method
!     The method is accelerated by the GCR process
!     Old subroutine prschurmethod
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
!     schur_method
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpu
      use sepmodulesimple
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: normrhsd
      double precision, intent(inout) :: u(nrusol)
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     normrhsd       i    Norm of right-hand side vector
!     u             i/o   Solution vector
!                         v during the iteration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, iter, itimesav, niteruacc, niterpacc, &
                 niteracc, niterutot, niterptot, &
                 iterations(4), niteru
      double precision, allocatable :: rua(:), ucop(:)
      double precision, pointer :: dup(:)
      double precision :: rnorm, rferr, normprec
      logical :: converged
      double precision :: timebefore(2)

!     converged      If true the process is converged
!                    they are not
!     dup            correction vector
!     iter           Iteration sequence number
!     iterations     ?
!     itimesav       Help parameter to save the parameter itime
!     k              Counting variable
!     niteracc       Accumulated number of GCR iterations
!     niterpacc      Accumulated number of iterations to solve the p-equations
!     niterptot      Total number of p iterations in all GCR steps
!     niteru         Number of iterations to solve the u-equations
!     niteruacc      Accumulated number of iterations to solve the u-equations
!     niterutot      Total number of u iterations in all GCR steps
!     normprec       Norm of preconditioned residual
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rnorm          norm of residual
!     rua            Actual residual at input
!     ucop           Copy of start vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKCLK         print elapsed time
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRINRL         Print 1d real vector
!     SEPSCHURITER   schur complement iteration as part of some outer loop
!     SEPSIMPLECONVG Compute norm of residual and adapt accuracy if necessary
!     SEPSIMPLESTART Compute initial norm, adapted eps and print start
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The system of equations to be solved reads:
!
!     | Q    G | |u|  = |f_u|
!     | D    C | |p|    |f_p|
!
!     input:  Q, G, D, f_u, f_p
!     output: du, dp
!
!     In the Schur method this system is rewritten as:
!
!     | Q      0    | | I  Q^-1G | |u|  = |f_u|
!     | D  C-DQ^-1G | | 0    I   | |p|    |f_p|, &
!
!     where the matrix -DQ^-1G is called the Schur complement matrix
!
!     Define    | u* |  =  | I  Q^-1G | |u|
!               | p  |     | 0    I   | |p|
!
!     Then the system can be solved in the following 3 steps:
!
!           Q u* = f
!      C-DQ^-1G p = f_p - D u* &
!              u = u* - Q^-1 G p
!
!     Here we solve each of the substeps inaccurately, with the following
!     algorithm
!
!     Start:  given u^0 and p^0
!             ru = f_u - Qu^0 - Gp^0;  r_p = f_p - Du^0
!
!     For k := 0 to maxiter do
!           Q u_f  = ru           inaccurately
!       -DQ^-1G p* = rp - D uf    inaccurately by GCR
!            Q ul  = G p*         inaccurately
!        u^k+1 = u^k + uf-ul
!        p^k+1 = p^k + p* &
!        ru = f_u - Qu^k+1 - Gp^k+1;   r_p = f_p - Du^k+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data niteruacc /0/, niterpacc /0/, niteracc /0/
! ======================================================================
!
      call eropen ( 'sepschurmethod' )
      call ersettime ( timebefore )

      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepschurmethod'
         write(irefwr,1) 'nu, np, A%iprint', nu, np, A%iprint
         call prinrl ( resup, nu+np, 'ru' )
         call prinrl ( R%MD, np, 'rmatpre' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      allocate(rua(nu+np), dup(nu+np), ucop(nu+np), &
               stat = error)
      if ( error/=0 ) call eralloc ( error, nu+np, 'rua' )
      if ( ierror/=0 ) go to 1000

      du => dup(1:nu)
      dp => dup(nu+1:nu+np)

      converged = .false.
      itimesav = itime
      itime = 0
      niterutot = 0
      niterptot = 0

!     --- Print heading if A%iprint>0

      if ( A%iprint>0 ) then
         write ( irefwr, 100 )   ! simple
100      format(//' Output for the Schur Method'//)
      end if  ! ( A%iprint>0 )

!     --- Compute initial norm, adapted eps and print start
!         Produce matlab output if required

      call sepsimplestart ( rnorm, rferr, normrhsd, A )

      if ( A%irelerr/=0 .and. rnorm<=A%eps ) then

!     --- Initial residual already small

         go to 1000

      end if

!     --- rua is actual right hand side, dup = contains update of u and p

      rua(1:nu) = ru
      rua(nu+1:nu+np) = rp
      ucop = u(1:nu+np)      ! start vector
      u(1:nu+np) = 0d0       ! make start vector zero

!     --- For k := 0 (1) maxiter-1 do

      do k = 0, A%nmax-1

         iter = k
         dup = 0d0

!        --- Carry out the three steps in the algorithm
!            The result is stored in dup

         call sepschuriter ( iterations, A )
         niteru = iterations(1)+iterations(2)+iterations(4)

         niterutot = niterutot+niteru
         niterptot = niterptot+iterations(3)

         normprec = norm ( dup, A )
         if ( iter==0 .and. A%irelerr==1 ) then

!        --- A%irelerr = 1 and iter=0, we have to adapt the accuracy

            A%eps = A%eps*normprec

         end if  ! ( iter==0 .and. A%irelerr==1 )

         if ( ierror/=0 ) go to 1000

!        --- Update u

         u(1:nu+np) = dup + u(1:nu+np)
         if ( debug ) call prinrl ( dup, nu+np, 'dup' )

!        --- Multiply the complete matrix by u
!            The result is stored in dup

         dup = A*u

!        --- Update the residual

         resup = rua - dup
         rnorm = norm ( ru, A )

         call sepsimpleconvg ( iter, rferr, converged, niteru, &
                               iterations(3), rnorm, normprec, A )

         if ( converged ) go to 500

      end do  ! k = 1, A%nmax

!     --- If the subroutine reaches this stage, to many iterations have
!         been carried out

      call errint ( iter, 1 )
      if ( inpsol(22,1)==1) then
         inpsol(1,1) = iter
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

500   niteracc = niteracc + iter
      niteruacc = niteruacc + niterutot
      niterpacc = niterpacc + niterptot
      u(1:nu+np) = ucop + u(1:nu+np)      ! add start vector to u

      if ( A%iprint>=1 ) then

!     --- A%iprint = 1  Print information

         write ( irefwr, 510 ) iter, rnorm, niterutot, niterptot, &
                               niteracc, niteruacc, niterpacc
510      format('The number of executed iterations in the', &
                ' Schur Method is', i5/ &
                'The 2-norm of the residual is:', ss,1pe12.2e2/ &
                'The number of u iterations is ', i6/ &
                'The number of p iterations is ', i6/ &
                'The accumulated number of GCR-iterations is', i8/ &
                'The accumulated number of u-iterations is', i8/ &
                'The accumulated number of p-iterations is', i8 )

      end if

!     --- Fill information in citerinfo
!         This is only used for performance checking

      niterout = niteracc
      niteruout = niteruacc
      niterpout = niterpacc
      call askclk
      timesubr = tim3-timebefore(1)

1000  nullify ( du, dp )
      deallocate ( rua, dup, ucop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'rua' )

      if ( debug ) write(irefwr,*) 'End sepschurmethod'
      itime = itimesav
      call erclos ( 'sepschurmethod' )

      end
