      subroutine sepsoloverout ( A, minlevel, maxlevel, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the actual overrelaxation process
!     Old subroutine prov01
! **********************************************************************
!
!                       KEYWORDS
!
!     constraint
!     linear_solver
!     overrelaxation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulesolve
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: minlevel(nusol), maxlevel(nusol)
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     maxlevel       i    constraint b(x)
!     minlevel       i    constraint a(x)
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, levsub, k, iter, j
      double precision :: eps1, eps2, rn, sum1, olddif, differ, omopt, rmu, &
                          omega1, anorm
      logical :: upper, lower, relative, check, estom0, estom1, &
                 eachst, finished
      double precision, pointer :: unew(:)

!     anorm          max-norm of vector
!     check          indicates if the norm of the difference must be computed
!                    (true) or not (false)
!     differ         max-norm of unew-uold
!     eachst         indicates whether lambda must be computed in each step
!                    (true) or not (false)
!     eps1           Value of epsilon corrected for the "1"-norm
!     eps2           Value of eps1 corrected for the computed eigenvalue
!     estom0         indicates whether omega_0 must be computed (true) or
!                    not (false)
!     estom1         indicates whether omega_1 must be computed (true) or
!                    not (false)
!     finished       indicates whether the iteration is finished (true) or
!                    not (false)
!     i              General loop variable
!     iter           iteration level
!     j              General loop variable
!     k              n**(1/ndim)
!     levsub         Sub iteration level. For this level a new condition must
!                    be checked or carried out
!     lower          Indicates if there is a constraint of the type u(x)>=a(x)
!     olddif         "1"-norm of u at preceding iteration - u at two
!                    iterations back
!     omega1         Help variable to store the computed value of omega
!                    temporarily
!     omopt          Estimation of optimal omega
!     relative       If true a relative accuracy is required
!     rmu            Parameter mu
!     rn             Help variable to make n real
!     sum1           Help variable to compute a sum
!     unew           Work array to store new solution
!     upper          Indicates if there is a constraint of the type u(x)<=b(x)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     479   Lambda >= 1
!     480   Diagonal of matrix contains zeros
!     481   Accuracy not reached in nmax steps
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The method performed is described int the Programmer's Guide
!   unew := usol
!   check diagonal of matrix
!   eps1 := eps (1+sqrt(n))
!   number_of_iterations := 0
!   check := false
!   estimate_omega_0 := true
!   estimate_omega_1 := true
!   each_step := false
!   difference := eps_machine
!   old_difference := 0
!   if ( printlev>=1 ) then
!      check := true
!   if ( niter1=-2 ) then
!      estimate_omega_0 := false
!      estimate_omega_1 := false
!      eps2 := (1-lambda)*eps1
!   else ( niter1#-2 )
!      lambda := 1000
!      if ( niter1=-1 ) then
!         niter1 := n^(1/ndim)
!      if ( niter2=-1 ) then
!         niter2 := n^(1/ndim)
!      if ( niter1=0 and niter2=0 ) then
!         estimate_omega_0 := false
!         estimate_omega_1 := false
!         each_step := true
!         old_difference := eps_machine
!      else if ( niter1=0 ) then
!         estimate_omega_0 := false
!         sub_iteration_level := niter2-1
!      else ( niter1#0 )
!         sub_iteration_level := niter1-1
!   finished := false
!   For iter := 1(1) nmax while ( not finished ) do
!      unew := one_step_overrelaxation ( usol )
!      if ( check ) then
!         difference := || unew - usol ||
!      unew := usol
!      number_of_iterations := number_of_iterations + 1
!      if ( difference = 0 ) then
!         finished = true
!      else if ( difference<eps1 ) then
!         finished := true
!      if ( not finished and number_of_iterations>=sub_iteration_level ) then
!         if ( estimate_omega_0 ) then
!            if ( not check and old_difference=0 ) then
!               check := true
!            else if ( old_difference=0 ) then
!               old_difference := difference
!            else
!               lambda := difference / old_difference
!               if ( lambda>=1 ) then
!                  error
!                  omega_opt := 1
!                  lambda := 1
!               else
!                  mu := 1/lambda ( (lambda-1+omega)/omega )^2
!                  omega_opt := 2/(1+sqrt(1-mu))
!               omega := omega_opt
!               omega1 := 1+alpha (omega_opt - 1)
!               if ( difference<(1-lambda)*eps1 ) then
!                  finished := true
!               else
!                  omega := omega1
!                  estimate_omega_0 := false
!                  sub_iteration_level := sub_iteration_level + niter2 - 1
!                  if ( estimate_omega_1 ) then
!                     old_difference := 0
!                  if ( printlev>=1 ) then
!                     sub_iteration_level := sub_iteration_level + 1
!                  else
!                     check := false
!         else if ( estimate_omega_1 ) then
!            if ( not check and old_difference=0 ) then
!               check := true
!            else if ( old_difference=0 ) then
!               old_difference := difference
!            else
!               lambda := difference / old_difference
!               if ( lambda>=1 ) then
!                  error
!                  omega_opt := 1
!                  lambda := 1
!               else
!                  mu := 1/lambda ( (lambda-1+omega)/omega )^2
!                  omega_opt := 2/(1+sqrt(1-mu))
!               omega := omega_opt
!               omega1 := 2+beta (omega_opt - 1)
!               if ( difference<(1-lambda)*eps1 ) then
!                  finished := true
!               else
!                  omega := omega1
!                  estimate_omega_1 := false
!               eps2 := (1-lambda) eps1
!         else if ( each_step ) then
!            lambda := difference / old_difference
!            old_difference := difference
!            if ( difference<(1-lambda)eps1 ) then
!               finished := true
!         else ( lambda has been computed )
!            if ( difference<eps2 ) then
!               finished := true
!   if ( not finished ) then
!      error
!      finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsoloverout' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsoloverout'
         write(irefwr,1) 'nmax', nmax
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the parameters lower and upper

      lower = A%iconstrain==1 .or. A%iconstrain==3
      upper = A%iconstrain==2 .or. A%iconstrain==3
      anorm = 1d0

      if ( debug ) write(irefwr,*) 'lower, upper ', lower, upper

      allocate ( unew(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'unew' )
      if ( ierror/=0 ) go to 1000

      unew = usol(1:nrusol)  ! copy usol in unew

!     --- check diagonal of matrix

      do i = 1, nrusol
         if ( abs(A%S(A%diag(i)))<=0d0 ) then
            call errint ( i, 1 )
            call errsub ( 480, 1, 0, 0)
            go to 1000
         end if
      end do

!     --- Initializations

      rn = nrusol
      relative = epsacc<0d0

      eps1 = abs(epsacc)
      niter = 0
      levsub = 1
      estom0 = .true.
      estom1 = .true.
      eachst = .false.
      olddif = 0d0
      differ = epsmac

      if ( debug ) printlev = 1

      check = printlev>=1

      if ( nmax==0 ) nmax = nrusol
      if ( debug ) &
         write(irefwr,1) 'nmax, niter1, niter2', nmax, niter1, niter2

      if ( niter1==-2 ) then

!     --- niter1 = -2, lambda and omega have been computed before

         estom0 = .false.
         estom1 = .false.
         eps2 = (1d0-alam) * eps1
         check = .true.

      else

!     --- niter1 # -2

         alam = 1000d0
         k =  int(nrusol**(1d0/ndim))
         if ( niter1==-1 ) niter1 = k
         if ( niter2==-1 ) niter2 = k
         if ( niter1==0 .and. niter2==0 ) then

!        --- niter1=niter2=0, the first steps are skipped
!            lambda is computed each step

            estom0 = .false.
            estom1 = .false.
            eachst = .true.
            olddif = epsmac

         else if ( niter1==0 ) then

!        --- niter1 = 0, skip first part

            estom0 = .false.
            levsub = niter2-1

         else

!        --- niter1 # 0

            levsub = niter1-1
            if ( check ) levsub = levsub + 1
         end if

      end if

      if ( printlev>=1 ) write ( irefwr,150) niter1, niter2, levsub
150   format(' niter1, niter2, subiteration level', 3i6)
      finished = .false.

      do iter = 1, nmax

         if ( finished ) go to 1000

!        --- actual overrelaxation process

         do i = 1, nrusol
            sum1 = rhsd(i)
            do j = A%rows(i)+1, A%diag(i)-1
               sum1 = sum1 - A%S(j) * unew(A%cols(j))
            end do
            do j = A%diag(i)+1, A%rows(i+1)
               sum1 = sum1 - A%S(j) * unew(A%cols(j))
            end do
            sum1 = sum1/A%S(A%diag(i))
            unew(i) = usol(i)+omega*(sum1-usol(i))
            if ( lower ) unew(i) = max(unew(i),minlevel(i))
            if ( upper ) unew(i) = min(unew(i),maxlevel(i))
         end do
         if ( check ) then

!        --- compute max-norm of difference vector

            differ = maxval(abs(unew-usol(1:nrusol)))
            if ( differ>0d0 .and. relative ) anorm = maxval(unew)

         end if  ! ( check )

         if ( printlev>=1 ) write ( irefwr, 220 ) niter, differ/anorm, &
                            alam, omega
220      format(' iteration', i5, '  differ:', d10.3, '  alam:', d12.5, &
                '  omega:', d10.3 )

!        --- copy unew in usol

         usol(1:nrusol) = unew
         niter = niter+1
         if ( differ<=0d0 ) finished = .true.

         if ( .not. finished .and. niter>=levsub ) then

!        --- Now some actions have to be carried out

            if ( estom0 ) then

!           --- omega 0 must be estimated

               if ( .not. check .and. olddif<=0d0 ) then

!              --- first time, set check

                  check = .true.

               else if ( olddif<=0d0 ) then

!              --- second time, set olddif

                  olddif = differ

               else

!              --- third time, estimate lambda and omega

                  alam = differ / olddif
                  if ( alam>=1d0 ) then

!                 --- Error 479: lambda too large

                     call errint ( niter, 1 )
                     call erreal ( alam, 1 )
                     call errsub ( 479, 1, 1, 0 )
                     omopt = 1d0
                     alam = 1d0
                  else
                     rmu = ( 1d0/alam)*((alam-1d0+omega)/omega)**2
                     omopt = 2d0/(1d0+sqrt(1d0-rmu))
                     omega = omopt
                     omega1 = 1d0 + alpha *  ( omopt - 1d0 )
                  end if
                  if ( differ<(1d0-alam)*eps1*anorm ) then
                     finished = .true.
                  else
                     omega = omega1
                     estom0 = .false.
                     levsub = niter + niter2 - 1
                  end if
                  if ( estom1 ) olddif = 0
                  if ( printlev>=1 ) then
                     levsub = levsub + 1
                  else
                     check = .false.
                  end if
               end if

            else if ( estom1 ) then

!           --- omega 1 must be estimated

               if ( .not. check .and. olddif<=0d0 ) then

!              --- first time, set check

                  check = .true.

               else if ( olddif<=0d0 ) then

!              --- second time, set olddif

                  olddif = differ

               else

!              --- third time, estimate lambda and omega

                  alam = differ / olddif
                  if ( alam>=1d0 ) then

!                 --- Error 479: lambda too large

                     call errint ( niter, 1 )
                     call erreal ( alam, 1 )
                     call errsub ( 479, 1, 1, 0 )
                      omopt = 1d0
                     alam = 1d0
                  else
                     rmu = (1d0/alam)*( (alam-1d0+omega)/omega)**2
                     omopt = 2d0/(1d0+sqrt(1d0-rmu))
                  end if
                  omega = omopt
                  omega1 = 2d0 + beta * ( omopt - 2d0 )
                  if ( differ<(1d0-alam)*eps1*anorm ) then
                     finished = .true.
                  else
                     omega = omega1
                     estom1 = .false.
                     eps2 = (1d0-alam) * eps1
                  end if
               end if

            else if ( eachst ) then

!           --- The eigenvalue must be computed in each step

               alam = differ / olddif
               olddif = differ
               if ( differ<(1d0-alam)*eps1*anorm ) finished = .true.

            else

!           --- alam has alfinished been computed

               if ( differ<eps2*anorm ) finished = .true.

            end if

         end if

      end do
      if ( .not. finished ) then

!     --- Error 481: the accuracy has not been reached yet

         call errint ( niter, 1 )
         call erreal ( alam, 1 )
         call erreal ( differ/((sqrt(rn)+1d0)*anorm), 2 )
         if ( A%aterror==0 ) then

!        --- aterror = 0, give error message

            call errsub ( 481, 1, 2, 0 )

         else

!        --- aterror = 1, give warning

            call errwar ( 481, 1, 2, 0 )

         end if  ! ( A%aterror==0 )

      end if

      deallocate ( unew, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'unew' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsoloverout'

      end if  ! ( debug )
      call erclos ( 'sepsoloverout' )

      end subroutine sepsoloverout
