      subroutine sepsolvemeth ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronicechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations
!     Real, serial case
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec, itypesol, i
      double precision, pointer :: diag(:)

!     diag           diagonal of preconditioning matrix
!     i              general loop variable
!     iprec          Type of preconditioner, see inpsol(4)
!     itypesol       Type of linear solver, see inpsol(3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPBICGSTAB    Solve linear system by Bi-CGstab
!     SEPBICGSTABL   Solve linear system by Bi-CGstab(l)
!     SEPCG          Solve linear system by conjugate gradients
!     SEPCGS         Solve linear system by CGS
!     SEPGCR         Solve linear system by GCR
!     SEPGMRES       Solve linear system by GMRES
!     SEPGMRESR      Solve linear system by GMRESR
!     SEPIDR         Solve linear system by IDR
!     SEPSCALMAT     Scale the original large sparse matrix
!     SEPSCALMATBCK  Rescale the original large sparse matrix
!     SEPSYMMLQ      Solve linear system by SYMMLQ
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2987   Iterative solver not available for real matrices
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvemeth' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolvemeth'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find type of preconditioning

      iprec = A%type_prec
      itypesol = A%typesolv
      if ( debug ) write(irefwr,1) 'iprec, itypesol', iprec, itypesol

      if ( itypesol==17 .and. A%ninner==1 ) then

!     --- Bicgstab (1): use original bicgstab

         itypesol = 1
         A%typesolv = 1

      end if  ! ( itypesol==17 .and. inpsol(14)==1 )

      if ( debug ) &
         write(irefwr,1) 'iprec, itypesol, scaling', &
                          iprec, itypesol, A%scaling

!     --- Scale matrix and right-hand side with preconditioning

      if ( A%scaling>0 .and. iprec>0 ) then
         allocate ( diag(A%neq) )
         if ( A%type_prec==3 .or.A%type_prec==4 ) then
            diag = A%MD(A%diag)
         else
            diag = A%MD
         end if  ! ( .not. A%symmetric )

         call sepscalmat ( A, diag, x, b )

      end if  ! ( A%scaling>0 .and. iprec>0 )

!     --- Solve system of equations

      select case ( itypesol )
      case(1)
         if ( A%symmetric )  then

!        --- Symmetric case: cg

            call sepcg ( A, x, b )

         else

!        --- Unsymmetric case: bicgstab

            call sepbicgstab ( A, x, b )

         end if  ! ( A%symmetric )

      case(2)
         call sepcgs ( A, x, b )
      case(3)
         call sepgmres ( A, x, b )
      case(4)
         call sepgmresr ( A, x, b )
      case(7)
         call sepsymmlq ( A, x, b )
      case(10)
         call sepidr ( A, x, b )
      case(17)
         call sepbicgstabl ( A, x, b )
      case(31)
         call sepgcr ( A, x, b )
      case default

!     --- Option not available

         call errint ( itypesol, 1 )
         call errsub ( 2987, 1, 0, 0 )
         go to 1000

      end select  ! case ( itypesol )

!     --- Scale matrix and right-hand side back with preconditioning

      if ( A%scaling>0 .and. iprec>0 ) then
         call sepscalmatbck ( A, diag, x, b )
         deallocate(diag)
      end if  ! ( A%scaling>0 .and. iprec>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsolvemeth'

      end if  ! ( debug )
      call erclos ( 'sepsolvemeth' )

      end subroutine sepsolvemeth
