      subroutine sepzcreaprec ( A )
! ======================================================================
!
!        programmer    Guus Segal, Zdenek Vlasek
!        version  9.0  date 04-04-2015 New parameter list
!        version  8.0  date 07-10-2014 New parameter list
!        version  7.1  date 29-05-2014 Extension of timebefore
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The routine sepzcreaprec computes the preconditioning matrix for the CG-
!     CGS- and Bi-CGSTAB process, for both complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulecmat
      use sepmodulematr
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: prlumpsav, n, imin, imax, i
      double precision, allocatable :: work(:)
      double precision :: timebefore(2)

!     i              General loop variable
!     imax           index of the maximal element of a matrix
!     imin           index of the minimal element of a matrix
!     n              number of rows in matrix
!     prlumpsav      Help variable to save the value of prlump
!     work           Work array of length 2n to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRINTTIME      Print the CPU time at end of subroutine
!     SEPCREAPRECZSM Computes the actual decomposition for a symmetric matrix
!     SEPCREAPRECZUS Computes the actual decomposition for a non-symmetric
!                    matrix
!     SEPPRECPRIN2   Print minimum and maximum element of diagonal
!                    of preconditioning matrix
!     SEPPRINMAT6    Print matrix that is stored as row-compact real matrix
!     SEPPRINMATDC6  Print preconditioning matrix that is stored as
!                    row-compact real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2234   For this preconditioner (ipreco ), lumping of the positive
!           off-diagonal elements has not been implemented.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('sepzcreaprec')

      if ( itime>0 ) call ersettime ( timebefore )

      debug = .false.

      n = A%neq

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepzcreaprec'
         write(irefwr,1) 'n', n
         call sepprinmat6 ( A )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%type_prec==0 ) go to 1000  ! jump in case of no preconditioning
      if ( debug ) write(irefwr,1) 'ipreco', A%type_prec

      length = 2*n

      allocate(work(length), stat = error)
      if ( error/=0 ) call eralloc ( error, length, 'work' )
      if ( ierror/=0 ) go to 1000

      prlumpsav = A%prlump
      if ( A%prlump==1 .and. &
         ( ( A%type_prec/=2 .and. A%type_prec/=3 .and. A%type_prec/=7 ) .or. &
             A%symmetric ) ) then
         call errint ( A%type_prec, 1 )
         call errwar ( 2234, 1, 0, 0 )
         A%prlump = 0
      end if

      if ( printlev>=2 ) then

!     --- print the maximal and minimal diagonal element of the matrix

         imin = minloc(abs(A%S(A%diag)), dim=1)
         imax = maxloc(abs(A%S(A%diag)), dim=1)
         write ( irefwr,120 ) abs(A%S(A%diag(imin))), imin
120      format (   ' The minimal abs(diag) element of the matrix is ', &
                  1pe10.2e2, ' in row ', i7 )
         write ( irefwr,130 ) abs(A%S(A%diag(imax))), imax
130      format (   ' The maximal abs(diag) element of the matrix is ', &
                  1pe10.2e2, ' in row ', i7  / )

      end if

      if ( A%symmetric) then

!     --- Complex symmetric matrix,

         call sepcreapreczsm ( A )

      else

         call sepcreapreczus ( A )

      end if  ! ( a%symmetric)
      if ( ierror/=0 ) go to 1000

!     --- prlump is filled again, because it can be changed in other
!         subroutines

      A%prlump = prlumpsav

      if ( printlev>=2 ) then

!     --- printlev>=2, extra output

         call sepprecprin2 ( A )

      end if

      if ( printlev>=4 ) then

!     --- printlev>=4, print diagonal of preco matrix

         write(irefwr,400) (i,A%MD(A%Mdiag(i)), i = 1, A%neq )
400      format ( 'Diagonals of preconditioning matrix'/ &
                  (2(1x,i4,': (',ss,1pe10.3e2, ',',ss,1pe10.3e2, ')') ))

      end if

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )
      if ( ierror/=0 ) go to 1000

1000  if ( itime==1 ) call printtime ( 'sepzcreaprec')

      if ( debug ) then

!     --- Debug information

         call sepprinmatdc6 ( A )
         write(irefwr,*) 'End sepzcreaprec'

      end if  ! ( debug )
      call erclos ('sepzcreaprec')

      end
