      subroutine to0107 ( ichois, idegfd, ivec1, const )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 05-12-2012 Long integers
!        version  2.0  date 06-04-1999 Extra parameters ibuffr/buffer
!        version  1.1  date 20-02-1998 New call to to0031
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    TO0107 is a vector manipulation subroutine
!    It is used for ichois = 11, 12 only
!    It is called by the SEPRAN subroutines MANVEC, VECMAN, ALGEBR, ANORM,
!    DIFFVC and MANIVC
!    TO0107 is able to manipulate vectors of common type 110, 115, 116
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision const(*)
      integer ichois, idegfd(*), ivec1

!    const    i/o  In- and output real array containing constants for the
!                  manipulation depending on ichois
!    ichois    i   Choice parameter indicating what type of function
!                  is chosen by the user.  Possible values:
!                  11:    The minimum and maximum values of the array
!                         represented by IVEC1 are computed.
!                         Besides that the co-ordinates for which these
!                         values are reached are stored in const
!                         Restriction: Only one unknown may be considered
!                                      per point
!                         At output CONST(1) contains the minimum, CONST(2)
!                         the maximum value, CONST(3) - (5), contain the
!                         co-ordinates for which the minimum is reached and
!                         CONST(6) - (8), contain the co-ordinates for which
!                         the maximum is reached. Real vectors only.
!                   12:   See 11, however, the minimum and maximum values of the
!                         absolute value of the array are used
!                         (Real and complex)
!    idegfd    i   In this array given by the user, he must store the degrees
!                  of freedom that are requested for the special purpose.
!                  In most applications only one degree of freedom per point
!                  is required, and the array may be replaced by an integer
!                  variable but not by a constant.
!                  If the variable has value 0, then all degrees of freedom
!                  are considered.
!                  If the variable has value -1, then all degrees of freedom
!                  are considered, except those corresponding to essential
!                  boundary conditions
!                  Length of the array: 1
!    ivec1     i   First input vector to be manipulated
!                  ivec1 must be a SEPRAN array of length 5, which may be
!                  of type 110, 115, 116
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, nnodes
      logical compli
      double precision amin, amax, coorlc(6)

!     amax    Maximum value in solution array
!     amin    Minimum value in solution array
!     compli  Indication if IVEC1 corresponds to a complex array (true) or to
!             a real one (false)
!     coorlc    array of length ndim x npoint containing the co-ordinates
!             of the nodal points with respect to the global numbering
!     i       Counting variable
!     itypvc  Indication of the type of solution vector:
!             110: solution vector
!             115, 116: vectors of special structure
!     nnodes  Number of nodal points or number of elements in the case of
!             an array of special structure defined per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     TO0031         Perform some vector manipulations for real vectors
!     TO0032         Perform some vector manipulations for complex vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data coorlc /6*0d0/
! ======================================================================
!
      call eropen ( 'to0107' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0107'
         call prinin ( ivec1, 5, 'ivec1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check type of ivec1

      compli = ks(ivec1)%compl/=0

!     --- Reserve space

      itypvc = ks(ivec1)%typevec

      if ( debug ) write(irefwr,1) 'itypvc', itypvc

      if ( itypvc ==116 ) then
         nphysi = ks(ivec1)%ivec
      else
         call sepactsolbf2 ( ivec1 )
      end if
      if ( itypvc==116 ) then

!     --- Special case, vector defined per element, set npoint equal
!         to number of elements

         nnodes = nelem

      else

!     --- Standard case

         nnodes = npoint

      end if
      usoltemp => ks(ivec1)%sol
      nusoli  = size ( usoltemp )
      if ( compli ) nusoli = nusoli / 2
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'ndim, indprfi, nnodes, indprpi, nusoli', &
                          ndim, indprfi, nnodes, indprpi, nusoli

      nphysi = max(nphysi,1)
      if ( compli ) then

         call to0032 ( ichois, usoltemp, coor, &
                       coorlc, amin, amin, amax, ndim, idegfd(1), nnodes )
      else

         call to0031 ( ichois, usoltemp, coor, &
                       coorlc, amin, amax, ndim, idegfd(1), nnodes )
      end if
      const(1) = amin
      const(2) = amax
      call dcopy ( 6, coorlc, 1, const(3), 1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( const, 2, 'const' )
         write(irefwr,*) 'End to0107'

      end if  ! ( debug )
      call erclos ( 'to0107' )
      end subroutine to0107
