      subroutine to0109 ( idegfd, ivec1, ivec3, const, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 09-03-2013 Correction ipgath
!        version  2.3  date 21-11-2012 Long integers
!        version  2.2  date 07-09-2001 Correction for old array of different
!                                      type
!
!   copyright (c) 1995-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    TO0109 is a vector manipulation subroutine
!    It is only used for ichois = 17
!    It is called by the SEPRAN subroutines MANVEC, VECMAN, ALGEBR, ANORM,
!    DIFFVC and MANIVC
!    TO0109 is able to manipulate vectors of common type 110, 115, 116
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision const(*)
      integer idegfd(*), ivec1(*), ivec3, iconst(*)

!    const    i/o  In- and output real array containing constants for the
!                  manipulation depending on ichois
!    iconst   i/o  In- and output integer array containing constants for the
!                  manipulation depending on ichois
!    idegfd    i   In this array given by the user, he must store the degrees
!                  of freedom that are requested for the special purpose.
!                  In most applications only one degree of freedom per point
!                  variable but not by a constant.
!                  If the variable has value 0, then all degrees of freedom
!                  are considered.
!                  If the variable has value -1, then all degrees of freedom
!                  are considered, except those corresponding to essential
!                  boundary conditions
!                  The length of the array depends on ichois:
!                  If ichois = 1-13:  Length 1
!                                14:         2 or 3 (See iconst(1))
!                             15-16:         1
!                             17,18:     ?
!                                19:         1
!    ivec1     i   First input vector to be manipulated
!                  ivec1 must be a SEPRAN array of length 5, which may be
!                  of type 110, 115, 116
!                  If ichois = 17, IVEC1 is supposed to be a more dimensional
!                  array of size 5 x NVEC, otherwise IVEC1 contains only
!                  one array
!    ivec3     i   Third input vector to be manipulated (if necessary)
!                  If only one or two arrays are needed, ivec3 may be replaced
!                  by a dummy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, lenwor, idum, &
              nbnout, npointlc, nusout, nvec, lenvc1, lenvc3
      logical compli, new, gather
      integer, allocatable :: iwork(:), igathr(:)
      double precision, pointer :: vec3(:)

!     compli         Indication if IVEC1 corresponds to a complex array
!                    (true) or to a real one (false)
!     gather         indication whether gathering has been carried out
!                    (true) or not (false)
!     i              Counting variable
!     idum           General dummy variable
!     igathr         Integer array containing the indices of the positions
!                    in USOL corresponding to IDEGFD
!                    If nboundlc>0 the prescribed degrees of freedom are
!                    positioned at the rear. Periodical boundary conditions
!                    are counted for only once
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     iwork          Array IWORK is filled by subroutine ER0063 as follows:
!                    IWORK(i)>0, gives start position of unknown in point i
!                    IWORK(i) = 0, unknown is not available in point i
!                    IWORK(i)<0, point is second periodical point
!                    -IWORK(i) refers to first point
!                    Array IWORK is only filled if gather is true
!     lenvc1         Length of vector 1 in words
!     lenvc3         Length of vector 2 in words
!     lenwor         Length of the real part in the work space in words
!     nbnout         Number of prescribed boundary conditions in IGATHR
!     new            Indication if the output vector is new (true) and hence
!                    should be cleared or not (false)
!     npointlc       Number of nodal points
!     nusout         Number of degrees of freedom in a solution vector
!                    after the  compress operation
!     nvec           number of vectors in ivectr(1:5,1:nvec)
!     vec3           output vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0063         Fill index array for gather operation
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TO0043         Perform vector manipulations through the subroutine FUNVEC
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0109' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0109'
         write(irefwr,*) 'ivec1, ivec3', ivec1(1), ivec3
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check type of ivec1

      compli = ks(ivec1(1))%compl/=0

!     --- Reserve space

      nvec = iconst(1)

      itypvc = ks(ivec1(1))%typevec

      if ( itypvc ==116 ) then
         nphysi = ks(ivec1(1))%nphys
      else
         call sepactsolbf1 ( ivec1 )
      end if

      if ( debug ) &
         write(irefwr,1) 'nvec, itypvc, nphysi',nvec, itypvc, nphysi

      if ( itypvc==116 ) then

!     --- Special case, vector defined per element, set npointlc equal
!         to number of elements

         npointlc = nelem

      else

!     --- Standard case

         npointlc = npoint

      end if
      if ( itypvc==110 ) then
         npbouni = 0
      else
         nboundi = 0
         npbouni = 0
      end if
      nusoli = ks(ivec1(1))%nusol
      if ( debug ) write(irefwr,1) 'nusoli', nusoli

!     --- Third vector must be created
!         Check if ivec3 has been filled before (new=false) or not (true)

      new = ivec3<=0 .or. ivec3>lastsol
      if ( new ) ivec3 = 0

      if ( debug ) write(irefwr,*) 'new', new

      call sepcreasolbf1 ( ivec3, itypvc, ks(ivec1(1))%ivec, &
                           'ivec3', ks(ivec1(1))%compl )
      vec3 => usoltemp

      if ( new ) vec3 = 0d0   ! Clear ivec3 if it is a new vector

!     --- Create temporary space

      gather = .false.
      if ( idegfd(1)>0 .and. nphysi>1 .or. indprfi>0 ) then

!     --- idegfd(1)>0;
!         For these values a gather operation is required

         gather = .true.

      end if

      length = max(nusoli,npointlc)
      allocate ( iwork(length), igathr(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( ierror/=0 ) go to 1000

!     --- Set some constants depending on the type of vector

      if ( debug ) &
         write(irefwr,1) 'indprfi, nphysi, idegfd(1)', &
                          indprfi, nphysi, idegfd(1)

      if ( indprfi>0 .or. nphysi>1 .and. idegfd(1)>0 ) then

!     --- idegfd(1)>0, only one degree of freedom
!         First apply gather operation

         gather = .true.
         call er0063 ( npointlc, idegfd(1), iwork, &
                       igathr, nusout, nbnout, 0, idum )

      else

         gather = .false.

      end if

      call to0043 ( const, nvec, ivec1, &
                    coor, vec3, npointlc, &
                    iwork, gather, ndim )

      deallocate ( iwork, igathr, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'ivec1, ivec3', ivec1(1), ivec3
         write(irefwr,*) 'End to0109'

      end if  ! ( debug )
      call erclos ( 'to0109' )

      end subroutine to0109
