      subroutine to0110 ( ichois, idegfd, ivec1, ivec2, const, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 03-12-2013 Debug commands
!        version  2.2  date 05-12-2012 Long integers
!        version  2.1  date 15-11-2000 Allow user points for extract
!
!   copyright (c) 1995-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    TO0110 is a vector manipulation subroutine
!    It is used to in case ichois = 4,5 or 15,16 (max norm)
!    It is called by the SEPRAN subroutines MANVEC, VECMAN, ALGEBR, ANORM, &
!    DIFFVC and MANIVC
!    TO0035 is able to manipulate vectors of common type 110, 115, 116
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision const(*)
      integer ichois, idegfd(*), ivec1, ivec2, iconst(*)

!    const    i/o  In- and output real array containing constants for the
!                  manipulation depending on ichois
!    ichois    i   Choice parameter indicating what type of function
!                  is chosen by the user.  Possible values:
!                   4:    VEC1(IDEGFD) in IPOINT is stored in CONST(1)
!                   5:    VEC1(IDEGFD) in IPOINT = CONST(1)
!                   15:   CONST(1) = || VEC1 - VEC 2 ||
!                         The type of norm must be given in ICONST(1).
!                         Possible values:
!                         1       l1-norm
!                         2       l2-norm
!                         3       max-norm
!                         4       l1-norm / number of degrees of freedom
!                         5       l2-norm / number of degrees of freedom
!                        10       max-norm  the nodal point number
!                                 corresponding to the maximum is stored in
!                                 iconst(2)
!                   16:   CONST(1) = || VEC1 ||
!                         For the type of norms available: see ICHOIS = 15
!    iconst   i/o  In- and output integer array containing constants for the
!                  manipulation depending on ichois
!    idegfd    i   In this array given by the user, he must store the degrees
!                  of freedom that are requested for the special purpose.
!                  In most applications only one degree of freedom per point
!                  variable but not by a constant.
!                  If the variable has value 0, then all degrees of freedom
!                  are considered.
!                  If the variable has value -1, then all degrees of freedom
!                  are considered, except those corresponding to essential
!                  boundary conditions
!                  The length of the array depends on ichois:
!                  If ichois = 1-13:  Length 1
!                                14:         2 or 3 (See iconst(1))
!                             15-16:         1
!                             17,18:     ?
!                                19:         1
!    ivec1     i   First input vector to be manipulated
!                  ivec1 must be a SEPRAN array of length 5, which may be
!                  of type 110, 115, 116
!                  If ichois = 17, IVEC1 is supposed to be a more dimensional
!                  array of size 5 x NVEC, otherwise IVEC1 contains only
!                  one array
!    ivec2     i   Second input vector to be manipulated (if necessary)
!                  If only one array is needed, ivec2 may be replaced by a
!                  dummy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipoint, itypvc, nnodes
      logical compli, gather, usevc2

!     compli         Indication if IVEC1 corresponds to a complex array (true)
!                    or to a real one (false)
!     gather         indication whether gathering has been carried out (true) or
!                    not (false)
!     i              Counting variable
!     ipoint         Nodal point sequence number
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     nnodes         Number of nodal points
!     usevc2         If true vector 2 is used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     TO0031         Perform some vector manipulations for real vectors
!     TO0032         Perform some vector manipulations for complex vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0110' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0110'
         write(irefwr,1) 'idegfd, ichois', idegfd(1), ichois
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check type of ivec1

      compli = ks(ivec1)%compl/=0

!     --- Reserve space

      itypvc = ks(ivec1)%typevec

      if ( itypvc ==116 ) then
         nphysi = ks(ivec1)%ivec
      else
         call sepactsolbf2 ( ivec1 )
      end if
      
      if ( debug ) write(irefwr,*) 'compli, itypvc ',  compli, itypvc
      if ( itypvc==116 ) then

!     --- Special case, vector defined per element, set nnodes equal
!         to number of elements

         nnodes = nelem

      else

!     --- Standard case

         nnodes = npoint

      end if
      vectin => ks(ivec1)%sol
      nusoli  = size (vectin  )
      if ( compli ) nusoli = nusoli / 2
      usevc2 = ichois==15
      
      if ( debug ) write(irefwr,*) 'usevc2, nusoli ', usevc2, nusoli

!     --- Create temporary space

      gather = idegfd(1)>0 .and. (indprfi>0 .or. nphysi>1) .and. &
               ichois<4 .or. ichois>5
       
      if ( debug ) write(irefwr,*) 'gather ', gather
      if ( gather ) then

!     --- idegfd(1)>0;
!         ichois # 4 or 5
!         For these values of ichois a gather operation is required

      end if

      if ( ierror/=0 ) go to 1000

!     --- Set some constants depending on the type of vector

      if ( usevc2 ) vectin2 => ks(ivec2)%sol
      
      if ( debug ) &
         write(irefwr,1) 'size(ivec1/2)', &
                          size(ks(ivec1)%sol), size(ks(ivec2)%sol)

      if ( ichois<15 ) then
         ipoint = iconst(1)
         if ( ichois==4 .or. ichois==5 ) then

!        --- extract constant

            if ( iconst(2)==1 ) then

!           --- Use user points instead of node

               ipoint = kmeshl(ipoint)

            end if

         end if

      else

         ipoint = iconst(2)

      end if
      nphysi = max(nphysi,1)
      
      if ( debug ) write(irefwr,1) 'ipoint, nphysi', ipoint, nphysi
      if ( compli ) then

!     --- Complex case (itypvc = 110, 115 or 116)

         call to0032 ( ichois, vectin, vectin2, &
                       vectin2, const(1), const(1), const(2), &
                       ipoint, idegfd(1), nnodes )
      else

!     --- Real case (itypvc = 110, 115 or 116)

         call to0031 ( ichois, vectin, vectin2, vectin2, const(1), const(2), &
                       ipoint, idegfd(1), nnodes )

      end if
      if ( ichois<15 ) then

         iconst(1) = ipoint

      else

         iconst(2) = ipoint

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0110'

      end if  ! ( debug )
      call erclos ( 'to0110' )

      end subroutine to0110
