      subroutine tomanvec44 ( ivec1, ivec2, iconst, ichois, vector )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 26-09-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Perform vector manipulations for subroutine manvec in case of
!    ichois = 44/45
!    For ichois = 44, a vector is contracted per point with another vector
!    If ichois = 45 a vector is expanded per point by multiplying by a vector
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, ivec2, iconst(*), ichois
      double precision vector(*)

!     ichois         i    Choice parameter
!                         Possible values:
!                         44: uout(i) is vector with 1 component in the
!                             points where vec1(i,.) is defined
!                             vec1 must be a vector with ndim components
!                             uout(i) = a . vec1(i)
!                             The vector a must be stored in rinvec(1..ndim)
!                             The type of output vector must be defined in
!                             ISPEC (0=solution vector, >0 vector of special
!                             structure)
!                         45: uout(i) is vector with ndim components in the
!                             points where vec1(i) is defined
!                             vec1 must be a vector with 1 component
!                             uout(i) = vec1(i) a
!                             The vector a must be stored in rinvec(1..ndim)
!                             The type of output vector must be defined in
!                             ISPEC (0=solution vector, >0 vector of special
!                             structure)
!     iconst         i    Contains some constants required to compute the vector
!                         1:  ivec
!                         2:  iprob
!     ivec1          i    input vector to be manipulated
!                         ivec1 must be a SEPRAN array of length 5, which may be
!                         of type 110, 115, 116 or 119
!     ivec2         i/o   Output vector
!     vector         i    Contains the ndim components of the vector to be used
!                         in the manipulation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, ivec
      logical compli

!     compli         Indication if IVEC1 corresponds to a complex array (true)
!                    or to a real one (false)
!     i              Counting variable
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TOMANVEC44_1   Perform vector manipulations for subroutine manvec in case
!                    ichois = 44/45
!     TOMANVEC45_1   Perform vector manipulations for subroutine manvec in case
!                    ichois = 45
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     222   input vector is complex:  not yet allowed.
!     699   Problem number incorrect
!    1731   The number of unknowns per point in the input vector is not 1
!    2275   nphys in input vector<ndim
!    2276   nphys in output vector<ndim
!    2277   nphys in output vector # 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec44' )

!     --- Check type of ivec1

      if ( ks(ivec1)%compl==0 ) then
         compli = .false.
      else
         compli = .true.
         call errsub ( 222, 0, 0, 0 )
         go to 1000
      end if

!     --- Reserve space

      itypvc = ks(ivec1)%typevec
      call sepactsolbf2 ( ivec1 )
      if ( ichois==44 .and. nphysi<ndim ) then

!     --- Number of degrees of freedom too small

         call errint ( nphysi, 1 )
         call errint ( ndim, 2 )
         call errsub ( 2275, 2, 0, 0 )

      end if
      if ( ichois==45 .and. nphysi/=1 ) then

!     --- Number of degrees of freedom not 1

         call errint ( nphysi, 1 )
         call errsub ( 1731, 1, 0, 0 )

      end if
      ivec  = iconst(1)
      iprob = iconst(2)
      if ( ivec==0 ) then
         itypvc = 110
      else
         itypvc = 115
      end if
      if ( iprob<1 .or. iprob>nprob )  then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0)
      end if

!     --- First reserve space for the output vector and activate the
!         corresponding arrays KPROBF, KPROBH and KPROBP

      call sepcreasolbf1 ( ivec2, itypvc, ivec, 'vector', ks(ivec1)%compl )

      if ( ichois==45 .and. nphyso/=ndim ) then

!     --- Number of degrees of freedom too small

         call errint ( nphyso, 1 )
         call errint ( ndim, 2 )
         call errsub ( 2276, 2, 0, 0 )

      end if
      if ( ichois==44 .and. nphyso/=1 ) then

!     --- Number of degrees of freedom not 1

         call errint ( nphyso, 1 )
         call errsub ( 2277, 1, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Carry out the actual mapping

      vectin => ks(ivec1)%sol
      vectin2 => ks(ivec2)%sol
      if ( compli ) then

!     --- Complex case

      else if ( ichois==44 ) then

!     --- Real case, ichois = 44

         call tomanvec44_1 ( vectin, vectin2, indprfi, &
                             kprobfi, indprfo, kprobfo, &
                             npoint, vector, ndim, nphysi )

      else

!     --- Real case, ichois = 45

         call tomanvec45_1 ( vectin, vectin2, indprfi, &
                             kprobfi, indprfo, kprobfo, &
                             npoint, vector, ndim, nphyso )

      end if

1000  call erclos ( 'tomanvec44' )
      end
