      subroutine inineighmark1 ( npoint, nset, set, iwork, icrossobst, &
                                 nmeshc, ninterf, iobst, numbndobst, &
                                 ipointobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all marked neighbors of the interface and store in
!     array neighbors
!     It concerns the end points of the intersected edges
! **********************************************************************
!
!                       KEYWORDS
!
!     neighbour
!     interface
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nset, ninterf, iobst, numbndobst, &
              set(nset), iwork(npoint), icrossobst(*), nmeshc(*), &
              ipointobst(*)

!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     iobst          i    Obstacle sequence number
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     iwork          i    integer work array containing the marked nodes
!                         iwork(i) = 0: node is not marked
!                                   >0: node is marked
!     ninterf        i    Number of nodes in interface
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     npoint         i    Number of nodal points in mesh
!     nset          i/o   at output: Number of neighbors of nodes
!                         at input: declared length of set
!     numbndobst     o    Number of nodal points on the boundary of the obstacle
!     set            o    Set of neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nneigh, ineigh, i, iedge, istart, istarticross, &
              node1, node2, numnodes
      integer, allocatable :: neighs(:), imark(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     iedge          edge sequence number
!     imark          Work array to indicate if a node has been stored (1)
!                    or not (0)
!     ineigh         neighbor
!     istart         Starting address
!     istarticross   Starting address in array icrossobst
!     neighs         set of neighbors
!     nneigh         Number of neighbors in neighs
!     node1          Node of edge
!     node2          Node of edge
!     numnodes       Number of nodal points in edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inineighmark1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inineighmark1'
         write(irefwr,1) 'ninterf, npoint', ninterf, npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( neighs(npoint), imark(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'neighs' )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays

      nneigh = 0
      imark = 0

!     --- Mark all end nodes of the intersected edges

      istarticross = icrossobst(iobst)  ! starting address iobst

!     --- For all edges in interface do

      do i = 1, ninterf

         iedge = icrossobst(istarticross+1)  ! global edge number
         istart = nmeshc(iedge)              ! starting address
         numnodes = nmeshc(istart)           ! number of nodes in edge
         node1 = nmeshc(istart+1)            ! first node
         node2 = nmeshc(istart+numnodes)     ! last node
         istarticross = istarticross+2

         if ( imark(node1)==0 ) then

!        --- imark(node1) = 0, hence node not yet stored
!            So new node found

            nneigh = nneigh+1
            neighs(nneigh) = node1
            imark(node1) = 1        ! mark node as used

         end if  ! ( imark(node1)==0 )

         if ( imark(node2)==0 ) then

!        --- imark(node2) = 0, hence node not yet stored
!            So new node found

            nneigh = nneigh+1
            neighs(nneigh) = node2
            imark(node2) = 1        ! mark node as used

         end if  ! ( imark(node2)==0 )

      end do  ! i = 1, numintersect

      if ( debug ) call prinin ( neighs, nneigh, 'neighs' )

!     --- Put all nodes of neighs satisfying iwork(node)>0 into set

      nset = 0
      do i = 1, nneigh

         ineigh = neighs(i)
         if ( iwork(ineigh)==1 .or. iwork(ineigh)==10 ) then

!        --- node is marked, add to set

            nset = nset+1
            set(nset) = ineigh

         end if  ! ( iwork(ineigh)==1 )

      end do  ! i = 1, nneigh

!     --- Extend set with nodal points on the interface
!         Get number of nodes at bounday of interface

      call inizerolev ( ipointobst, istart, numbndobst, iobst )

      if ( numbndobst>0 ) then

!     --- numbndobst > 0, there are interface nodes

         set(nset+1:nset+numbndobst) = &
            ipointobst(istart+1:istart+numbndobst)

      end if  ! ( numbndobst>0 )

      deallocate ( neighs, imark, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'neighs' )

1000  call erclos ( 'inineighmark1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inineighmark1'

      end if  ! ( debug )

      end
