      subroutine prfilllevset ( inpsep, iphi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 24-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill level set function phi as signed distance function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     obstacle
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), iphi(5)

!     inpsep         i    The integer input read is stored in array INPSEP
!     iphi           i    Standard SEPRAN solution array containing information
!                         of the levelset function phi
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nx(ndim), ntot, iobst, ninterf, isig
      integer, allocatable :: interf(:), grid(:), iwork(:)
      double precision origin(ndim), dx(ndim), xmaxar(ndim), h

!     dx             stepsize of grid in all directions
!     grid           Work array of size at most 101 x 101 to store
!                    information about the boundary in relation with
!                    arrays gridinfo and rgridinfo
!                    The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                    to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                    ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                    if grid(ix,iy) = 0, no boundary node is stored in cell
!                    (ix,iy), if>0 information about the boundary cells
!                    is stored in array grdinfo from position
!                    grid(ix,iy)
!     h              representative mesh size
!     interf         Contains node numbers of interface
!     iobst          Obstacle sequence number
!     isig           Sign of level set inside obstacle
!     iwork          integer work array of length npoint used by several
!                    subroutines
!                     -1:  node with phi<0
!                      0:  node on interface
!                      1:  node with phi>0
!     ninterf        Number of nodes in interf
!     ntot           number of cells in grid
!     nx             number of points in ndim directions of the grid
!     origin         Contains the coordinates of the origin of the region
!     xmaxar         Contains the coordinates xmax, ymax ..
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLGRIDNOD  Fill information of a set of nodes in rectangular 2d or 3d
!                    grid
!     PRGRIDSIZE     Compute nx, dx and ntot from information in kmeshquant
!                    and h
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRMAKEDISTEX2  Make phi a distance function for all nodes
!     PRMARKLEV      Fill arrays iwork and interf with interface points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfilllevset' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfilllevset'
         call prinin ( iphi, 5, 'iphi' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      isig = inpsep(5)
      iobst = inpsep(6)

      allocate ( interf(npoint), iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'interf' )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays iwork and interf with interface points

      call prmarklev ( iobst, interf, ninterf, iwork, isig )
      if ( ierror/=0 ) go to 1000

!     --- Make a rectangular grid over the whole region and store the number
!         of nodes of the interface per cell
!         First compute number of cells and define hx etc.

      h = 0d0 ! compute h in subroutine
      call prgridsize ( kmeshquant, ndim, origin, h, dx, nx, &
                        npoint, ntot, xmaxar )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinin ( nx, ndim, 'nx' )
         call prinrl ( origin, ndim, 'origin' )
         call prinrl ( dx, ndim, 'dx' )
         call prinrl ( xmaxar, ndim, 'xmaxar' )
         write(irefwr,1) 'ntot, lengrid', ntot, ntot+1+ninterf
      end if  ! ( debug )

      allocate ( grid(ntot+1+ninterf), stat = error )
      if ( error/=0 ) call eralloc ( error, ntot+1+ninterf, 'grid' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array grid

      call prfillgridnod ( interf, ninterf, grid, nx, origin, xmaxar )
      if ( ierror/=0 ) go to 1000

!     --- Fill array phi

      call prmakedistex2 ( iwork, grid, iphi, origin, nx, dx )

!     --- Fill set of points with phi>0

      deallocate ( grid, interf, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'grid' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iphi, 5, 'iphi' )
         write(irefwr,*) 'End prfilllevset'

      end if  ! ( debug )
      call erclos ( 'prfilllevset' )

      end
