      subroutine prmakedisterr ( phi, phiold, nodes, nnodes, nusol, eps, &
                                 iprint, iter, finished, maxiter, &
                                 istop, err, err0, gradphi, typenorm, &
                                 ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-08-2012 Change in finished
!        version  1.1  date 11-07-2012 Extension with typenorm
!        version  1.0  date 26-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute difference between phi and phiold and check accuracy
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     level_set
!     distance
!     accuracy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), nusol, iprint, iter, maxiter, &
              istop, typenorm, ndim
      double precision phi(nusol), phiold(nusol), eps, err, err0, &
                       gradphi(ndim,nusol)
      logical finished

!     eps            i    Defines accuracy for distance function
!     err            o    Computed difference
!     err0          i/o   Computed difference at first iteration
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     gradphi        i    Contains the gradient of phi
!     iprint         i    Defines amount of print output
!     istop          i    Indicates if program must stop in case of error (0,1)
!                         or must continue (2,3)
!     iter           i    Iteration sequence number
!     maxiter        i    maximum number of iterations
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that are used to check the
!                         accuracy
!     nusol          i    Total number of unknowns
!     phi            i    Contains the distance function in all nodes
!     phiold         i    Contains the distance function in the prior iteration
!     typenorm       i    Defines the type of norm that is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision err1, errstart, normphi
      logical first

!     err1           Computed difference
!     errstart       Computed difference at first iteration of first time step
!     first          If true it concerns the first step
!     i              Counting variable
!     normphi        Norm of gradient phi
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3094   signed distance function not converged
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      save first, errstart
      data first /.true./
      data errstart /0d0/
! ======================================================================
!
      call eropen ( 'prmakedisterr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedisterr'
         write(irefwr,1) 'typenorm', typenorm
         write(irefwr,2) 'eps, errstart', eps, errstart
         call prinin ( nodes, nnodes, 'nodes' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute maximum error for nodes corresponding to |phi|<phibarrier

      if ( typenorm==1 ) then

!     --- norm based on L2-distance between grad phi and 1

         err1 = 0d0
         do i = 1, nnodes
            normphi = dot_product(gradphi(1:ndim,i),gradphi(1:ndim,i))
            normphi = sqrt(normphi)
            err1 = err1+(normphi-1d0)**2
         end do  ! i = 1, nnodes
         err1 = sqrt(err1/nnodes)

      else if ( typenorm==2 ) then

!     --- norm based on max-distance between grad phi and 1

         err1 = 0d0
         do i = 1, nnodes
            normphi = dot_product(gradphi(1:ndim,i),gradphi(1:ndim,i))
            normphi = sqrt(normphi)
            err1 = max(err1,abs(normphi-1d0))
         end do  ! i = 1, nnodes

      else if ( typenorm==3 ) then

!     --- norm based on L2-distance between phi and phiold

         err1 = 0d0
         do i = 1, nnodes
            err1 = err1+(phi(i)-phiold(i))**2
         end do  ! i = 1, nnodes
         err1 = sqrt(err1/nnodes)

      else if ( typenorm==4 ) then

!     --- norm based on max-distance between phi and phiold

         err1 = maxval(abs(phi(nodes(1:nnodes))- &
                       phiold(nodes(1:nnodes))))

      end if  ! ( typenorm==1 )
      err = err1

      if ( debug ) write(irefwr,2) 'err1', err1

      if ( first ) then

!     --- first is true, hence first call

         finished = .false.
         first = .false.  ! next call
         errstart = err1   ! store error at first step
         finished = err1<eps
         err0 = err1

      else if ( iter<=1 ) then

         err0 = err1
         finished = err1<eps

      else

!     --- Next step, check error

         finished = err1<eps*errstart .or. err1<1d-5 .or. err1<eps*err0

      end if  ! ( first )

      if ( finished .and. iprint==1 ) then

!     --- iprint = 1, print information

         write(irefwr,100) 'Distance function has converged in', &
                           iter, ' iterations. Error is: ', err1
100      format ( a, i4, a, d12.3 )

      end if  ! ( iprint==1 )

      if ( iprint>=2 ) then

!     --- iprint>=2, hence maximal output

         if ( iter==1 .and. typenorm>1 .or. iter==0 .and. typenorm==1 ) &
            write(irefwr,*) 'Iteration  error in distance function'
         write(irefwr,200) iter, err1
200      format ( i4, d13.3 )

      end if  ! ( iprint>=2 )

      if ( iter>=maxiter .and. .not. finished ) then

!     --- No convergence in maxiter steps

         call errint ( iter, 1 )
         call erreal ( err, 1 )
         call erreal ( eps*errstart, 2 )

         if ( istop>1 ) then

!        --- istop = 2 or 3, resume program (warning)

            call errwar ( 3094, 1, 2, 0 )

         else

!        --- istop = 0 or 1, stop program (error)

            call errsub ( 3094, 1, 2, 0 )

         end if  ! ( istop>1 )

      end if  ! ( iter>=maxiter .and. .not. finished ) then

      if ( (iter==1 .or. err1<err) .and. typenorm>2 ) then

!     --- New difference smaller than old one
!         Copy phi to phiold

         phiold = phi
         err = err1

      else if ( (iter==1 .or. err1<err) .and. typenorm<=2 ) then

!     --- New difference smaller than old one

         err = err1

      else if ( iter>1 ) then

!     --- No convergence, the time step is halved

!         tstep = 0.5d0*tstep
!         print *,'new time step', tstep
!         print *,'err, err1, errstart ', err, err1, errstart

      end if  ! ( iter==1 .or. err1<err )

1000  call erclos ( 'prmakedisterr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'err', err
         write(irefwr,*) 'End prmakedisterr'

      end if  ! ( debug )

      end
