      subroutine prmakedistex2 ( iwork, grid, iphi, origin, nx, dx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 26-04-2013 New call to prmarkinterf
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 28-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make phi a distance function for all nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), iphi(5), grid(*), nx(ndim)
      double precision origin(ndim), dx(ndim)

!     dx             i    contains the difference between xmax, xmin etc
!     grid           i    The domain is subdivided in nx(1) x nx(2) cells
!                         of size dx(1) x dx(2)
!                         In grid the number of interface nodes per cell is
!                         stored as well as the node numbers in the folwoing way
!                         Pos. 1 starting address of node numbers
!                         (nx(1)*nx(2)+1)))
!                         The number of nodes in cell ij is defined by
!                         grid(start+1)-grid(start). where
!                         start = (j)-1)*nx(1)+i
!                         The corresponding node numbers are stored in grid
!                         from position start
!     iphi           i    Sepran solution array containing references to
!                         the distance function phi
!     iwork         i/o   integer work array to mark the nodes
!                          -1:  node with phi<0
!                           0:  node on interface
!                           1:  node with phi>0
!     nx             i    number of points in ndim directions of the grid
!     origin         i    Contains the coordinates of the origin of the region
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nset1, nset2, nfaceslc, iseq
      integer, allocatable :: set1(:), set2(:), markintf(:), &
                              markintfedge(:)

!     i              Counting variable
!     iseq           Dummy
!     markintf       Array of size nedges or nfaces to mark the edges or
!                    faces of the interface (1) else 0
!     markintfedge   Array of size nedges to mark the edges
!                    of the interface (1) else 0  (3d only)
!     nfaceslc       Either nfaces (3d) or nedges (2d)
!     nset1          number of nodes in set1
!     nset2          Number of nodes in set 2
!     set1           Contains all neighbors of the interface (one side only)
!     set2           Contains all neighbors of set1 not in the interface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOMPDIST     Compute distance of set of nodes to interface
!                    Store in array phi with sign isigphi
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRMARKINTERF   Mark all edges or faces in interface
!     PRMARKINTFEDGE Mark all edges in 3d interface
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistex2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistex2'
         write(irefwr,1) 'ndim', ndim
         call prinin ( iphi, 5, 'iphi' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      if ( kelme==0 ) call sepfillkmeshe
      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( nmesh(42)==0 ) call sepfillnmeshl
      if ( kelmad==0 ) call sepfillkmeshad ( iseq )
      if ( kmeshad(9)==0 ) call sepfillobfaces

      if ( ndim==3 ) then
         if ( nmesh(31)==0 ) call sepfillnmesha
         if ( nmesh(37)==0 ) call sepfillnmeshg
         if ( nmesh(41)==0 ) call sepfillnmeshk
         nfaceslc = nfaces
      else
         nfaceslc = nedges
      end if  ! ( ndim==3 )

      if ( debug ) &
         write(irefwr,1) 'npoint, nfaces, nedges',npoint, nfaces, nedges

      allocate ( set1(npoint), set2(npoint), markintf(nfaceslc), &
                 markintfedge(nfaceslc), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*npoint+nfaceslc+nedges, 'set1' )
      if ( ierror/=0 ) go to 1000

      markintf = 0  ! clear array
      markintfedge = 0  ! clear array

!     --- Set 1 = all nodes with negative phi
!         Set 2 = all nodes with positive phi

      nset1 = 0
      nset2 = 0
      do i = 1, npoint

         if ( iwork(i)<0 ) then

!        --- phi negative, fill node in set1

            nset1 = nset1+1
            set1(nset1) = i

         else if ( iwork(i)>0 ) then

!        --- phi negative, fill node in set1

            nset2 = nset2+1
            set2(nset2) = i

         end if  ! ( iwork(i)<0 )

      end do  ! i = 1, npoint

      if ( debug ) then
         call prinin ( set1, nset1, 'set1' )
         call prinin ( set2, nset2, 'set2' )
      end if  ! ( debug )

!     --- Mark all edges or faces in interface

      call prmarkinterf ( markintf, obstfaces, nfaceslc )
      if ( ierror/=0 ) go to 1000

      if ( ndim==3 ) then

!     --- ndim = 3, mark edges in interface

         call prmarkintfedge ( obstfaces, markintfedge, nmeshb )

      end if  ! ( ndim==3 )
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes in set 1

      phi2 => ks(iphi(1))%sol

!     --- Clear array phi

      phi2 = 0d0

      if ( nset1>0 ) then

!     --- nset1>0, there are nodes to be adapted

         call prcompdist ( set1, nset1, coor, ndim, nx, &
                           origin, dx, grid, markintf, &
                           nmeshc, nmeshl, npoint, obstfaces, &
                           phi2, -1, markintfedge, &
                           nmeshb, nmeshk, nmeshg, nfaceslc, nfaceslc )

      end if  ! ( nset1>0 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'after prcompdist/1'

!     --- Compute distance for all nodes in set 2

      if ( nset2>0 ) then

!     --- nset2>0, there are nodes to be adapted

         call prcompdist ( set2, nset2, coor, ndim, nx, &
                           origin, dx, grid, markintf, nmeshc, nmeshl, &
                           npoint, obstfaces, &
                           phi2, 1, markintfedge, &
                           nmeshb, nmeshk, nmeshg, nfaceslc, nedges )

      end if  ! ( nset2>0 )
      if ( debug ) write(irefwr,*) 'after prcompdist/2'

      deallocate ( set1, set2, markintf, markintfedge, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'set1' )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( phi2, npoint, 'phi' )
         write(irefwr,*) 'End prmakedistex2'

      end if  ! ( debug )
      call erclos ( 'prmakedistex2' )

      end subroutine prmakedistex2
