      subroutine prmasslevset
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 23-08-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill mass matrix for all nodes that are on the set of points with
!     level set 0
!     The mass matrix is defined for the elements on this set only
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilev
      integer, allocatable :: iwork(:)

!     ilev           Sequence number of level being considered
!     iwork          Work array of length npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLMASSLEV3 Fill mass matrix for all faces with zero level set
!     PRFILLMASSLEVS Fill mass matrix for all edges with zero level set
!     PRMARKLEVSET   Set iwork(i)=1 for all nodes i with level set 0
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHG  Fill array nmeshg
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:npoint)
!     Set iwork(i)=1 for all nodes i with level set 0
!     set (diagonal) mass matrix to zero
!     if ( mesh is 2d ) then
!        for all edges in mesh do
!           if ( both nodes have iwork = 1 ) then
!              create element mass matrix for edge
!              Add to large matrix
!     else
!        for all faces in mesh do
!           if ( all faces have iwork = 1 ) then
!              create element mass matrix for face
!              Add to large matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmasslevset' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmasslevset'

      end if
      if ( ierror/=0 ) go to 1000

      ilev = 1           ! ad-hoc, sequence number of level set function

!     --- Create mass matrix

      allocate ( matrixm(npoint) )
      if ( kelmaa==0 ) call sepfillkmeshaa

      if ( ndim==2 ) then

!     --- ndim = 2, we need the edges

         if ( nmesh(33)==0 ) call sepfillnmeshc

      else

!     --- ndim = 3, we need the faces

         if ( nmesh(37)==0 ) call sepfillnmeshg

      end if  ! ( ndim==2 )

!     --- Define work space

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork(1:npoint)

      iwork = 0

!     --- Set iwork(i)=1 for all nodes i with level set 0

      call prmarklevset ( ipointobst, ilev, iwork )

!     --- set (diagonal) mass matrix to zero

      matrixm = 0d0
      if ( ndim==2 ) then

!     --- mesh is 2d
!         for all edges in mesh do
!            if ( both nodes have iwork = 1 ) then
!               create element mass matrix for edge
!               Add to large matrix

         call prfillmasslevs ( matrixm, coor, &
                               nmeshc, nedges, iwork )

      else

!     --- mesh is 3d
!         for all faces in mesh do
!            if ( all nodes have iwork = 1 ) then
!               create element mass matrix for face
!               Add to large matrix

         call prfillmasslev3 ( matrixm, coor, &
                               nmeshg, nfaces, iwork )

      end if  ! ( ndim==2 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmasslevset'

      end if
      call erclos ( 'prmasslevset' )

      end subroutine prmasslevset
