      subroutine prmindist ( numfound, nodes, ndim, mindist, coor, &
                             node, markintf, nmeshc, nmeshl, &
                             markintfedge, nmeshk, nmeshg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find minimum distance of node to set of nodes in interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interface
!     node
!     distance
!     minimum
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numfound, nodes(numfound), ndim, markintf(*), nmeshc(*), &
              node, nmeshl(*), markintfedge(*), nmeshk(*), nmeshg(*)
      double precision coor(ndim,*), mindist

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     markintf      i/o   Array of size nedges or nfaces to mark the edges or
!                         faces of the interface (1) else 0
!     markintfedge  i/o   Array of size nedges to mark the edges
!                         of the interface (1) else 0  (3d only)
!     mindist       i/o   Minimum distance of node wrt interface
!     ndim           i    Dimension of the space.
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshg         i    nmesh part g, contains node numbers of faces
!     nmeshk         i    nmesh part k
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     node           i    Node number
!     nodes          i    Contains node numbers in interface that are part
!                         of the set of cells under consideration
!     numfound       i    Number of nodes in nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x(ndim), xintf(ndim), dx(ndim)

!     dx             x-xintf
!     i              Counting variable
!     x              co-ordinates of point
!     xintf          co-ordinates of point in interface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRMINDIST2D    Find minimum distance of node to set of nodes in 2d
!                    interface
!     PRMINDIST3D    Find minimum distance of node to set of faces in 3d
!                    interface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmindist' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmindist'
         write(irefwr,1) 'node, numfound', node, numfound
         write(irefwr,2) 'mindist', mindist
         call prinin ( nodes, numfound, 'nodes' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store coordinates of node in x

      x = coor(:,node)

!     --- Find shortest distance from points in array nodes to node

      do i = 1, numfound

         xintf = coor(:,nodes(i))
         dx = x-xintf
         mindist = min(mindist,dot_product(dx,dx))

      end do  ! i = 1, numfound

      if ( debug ) write(irefwr,2) 'mindist', mindist

!     --- Find shortest distance from node to interface

      if ( ndim==2 ) then

!     --- ndim = 2

         call prmindist2d ( markintf, x, coor, numfound, nodes, &
                            nmeshl, mindist, nmeshc, ndim )

      else

!     --- ndim = 3
!         First find shortest distance to all edges in interface

         call prmindist2d ( markintfedge, x, coor, numfound, nodes, &
                            nmeshl, mindist, nmeshc, ndim )
         call prmindist3d ( markintf, x, coor, numfound, nodes, &
                            nmeshk, mindist, nmeshg )

      end if  ! ( ndim==2 )

1000  call erclos ( 'prmindist' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'mindist', mindist
         write(irefwr,*) 'End prmindist'

      end if  ! ( debug )

      end
