      subroutine prmindist2d ( markintf, x, coor, numfound, nodes, &
                               nmeshl, mindist, nmeshc, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find minimum distance of node to set of nodes in edges of interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interface
!     node
!     distance
!     minimum
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer markintf(*), numfound, nodes(numfound), nmeshl(*), &
              nmeshc(*), ndim
      double precision x(ndim), coor(ndim,*), mindist

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     markintf      i/o   Array of size nedges or nfaces to mark the edges or
!                         faces of the interface (1) else 0
!     mindist       i/o   Minimum distance of node wrt interface
!     ndim           i    Dimension of the space.
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     nodes          i    Contains node numbers in interface that are part
!                         of the set of cells under consideration
!     numfound       i    Number of nodes in nodes
!     x              i    co-ordinates of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, ipstart, j, numedges, iedge, node1, node2, &
              jpstart, numnodes
      double precision dist, lambda
      logical inside

!     dist           Distance between two points
!     i              Counting variable
!     iedge          edge sequence number
!     inode          Node sequence number
!     inside         indicates if the perpendicular intersects
!                    between x1 and x2 (true) or not (false)
!     ipstart        Starting address in array nmeshl
!     j              Counting variable
!     jpstart        Starting address in array nmeshc
!     lambda         barycentric coordinate along x1 and x2 of the
!                    intersection point
!     node1          Node of face
!     node2          Node of face
!     numedges       number of edges
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision erdistlinepnt

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDISTLINEPNT  Compute the distance between line x1, x2 and point xp
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmindist2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmindist2d'
         call prinrl ( x, ndim, 'x' )
         write(irefwr,2) 'mindist', mindist
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d14.6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all nodes in set

      do i = 1, numfound

         inode = nodes(i)  ! node in interface

!        --- Find all edges in interface connected to inode that have not
!            been considered yet

         ipstart = nmeshl(inode)    ! starting address in nmeshl
         numedges = nmeshl(ipstart) ! number of edges connected to inode

         if ( debug ) &
            write(irefwr,1) 'i, inode, numedges', i, inode, numedges

         do j = 1, numedges

            iedge = nmeshl(ipstart+j)  ! edge sequence number

            if ( debug ) &
               write(irefwr,1) 'j, iedge, markintf(iedge)', &
                                j, iedge, markintf(iedge)

            if ( markintf(iedge)==1 ) then

!           --- interface edge connected to node that has not been treated yet

               markintf(iedge) = 2  ! mark as treated

               jpstart = nmeshc(iedge)     ! starting address in nmeshc
               numnodes = nmeshc(jpstart)  ! number of nodes in edge
               node1 = nmeshc(jpstart+1)   ! first node
               node2 = nmeshc(jpstart+numnodes)   ! end node

!              --- Compute distance of node with coordinates in x to iedge

               dist = erdistlinepnt ( coor(1,node1), coor(1,node2), x, &
                                      lambda, inside, ndim, .true. )
               mindist = min ( mindist,  dist )

               if ( debug ) then
                  write(irefwr,1) 'node1, node2', node1, node2
                  write(irefwr,2) 'dist, mindist', dist, mindist
               end if  ! ( debug )

            end if  ! ( markintf(iedge)==1 )

         end do  ! j = 1, numedges

      end do  ! i = 1, numfound

1000  call erclos ( 'prmindist2d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmindist2d'

      end if  ! ( debug )

      end
