      subroutine prpartpnlncon1 ( finished, maxvc0, maxprb1, inpsep, &
                                  rinsep, numcomps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     One time step for the integration of a particle is carried out
!     It concerns a system of concentration equations with a nonlinear
!     set of mixed boundary conditions
!     The interface concentration csol is computed in a loop until
!     convergence is reached
!     one dimensional case
! **********************************************************************
!
!                       KEYWORDS
!
!     particle
!     level_set
!     time_dependent
!     non_linear
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxvc0, maxprb1, numcomps, inpsep(*)
      double precision rinsep(*)
      logical finished

!     finished      i/o   Indicates if the loop has been finished (true) or not
!                         (false)
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscpnlnconc for a description
!     maxprb1        i    Maximum number of coupled problems allowed
!     maxvc0         i    Maximum number of vectors per time-step permitted
!     numcomps       i    Number of components
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscpnlnconc for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxiter, iloop, inpseploc(7), inode, i, itermethod, &
              iseqconc, iseqvn, dimsub, substitutional(numcomps), &
              istore, iseqg, numvecsav, iseqiter, minloop, iprint, &
              iseqcs, istart
      double precision diff, bigM, eps, vn, xp(numcomps), k(numcomps), &
                       cboun(numcomps), cpart(numcomps), cs(numcomps), &
                       h, xs(numcomps), factor, vstore(3), &
                       g(numcomps), kinf, diffprev
      logical zeroval

!     bigm           Parameter M in the method
!     cboun          Value of the concentrations at the boundary
!     cpart          Value of the concentrations in the particle
!     cs             mole fractions along the interface
!     diff           Help parameter to compute the difference of two
!                    parameters
!     diffprev       Previous value of diff (prior iteration)
!     dimsub         Dimension of the set of substitutional elements
!     eps            Accuracy with respect to the iteration process
!                    Computation is stopped once |factor-1|<eps
!     factor         Multiplication factor (must go to 1)
!     g              right-hand side for the boundary condition
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iloop          Loop parameter, counts number of iterations
!     inode          Node sequence number
!     inpseploc      Local array inpsep for various subroutines
!     iprint         Defines the amount of output to be printed
!     iseqconc       Sequence number of concentration vectors
!     iseqcs         Sequence number of vector cs
!     iseqg          Sequence number of right-hand side g
!     iseqiter       Sequence number of numiter in which the number of
!                    iterations is stored
!     iseqvn         Sequence number of boundary velocity
!     istart         Dummy
!     istore         Local counter for Aitken's method
!     itermethod     Type of iteration method
!     k              parameter k in the method
!     kinf           parameter kinf in the method
!     maxiter        maximum number of iterations
!     minloop        Minimum number of iterations in loop
!     numvecsav      Help variable to store the value of numvecmn
!     substitutional Set of substitutional elements
!     vn             Normal velocity
!     vstore         Help array to store three consecutive values of vn
!     xp             mole fractions within the precipitate
!     xs             see method
!     zeroval        if true at least one of the xs entries is zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer prnode
      double precision prvalnode, getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRAITKEN       Perform Aitken acceleration to an iteration process
!     PRINRL         Print 1d real vector
!     PRNODE         Find node number as defined by inpsep
!     PRPARTPNLNCONP Extract some information from inpsep with respect
!                    to prpartpnlnconc
!     PRVALNODE      Get the value of the solution vector in a given node
!                    a local array into cons
!     PUTCONVARARRAY Copy the values of a constant or scalar array from
!     PUTSCAL        Put the value of the scalar iseq in the block constants
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPTIMEFREE    Actual body of subroutine TIMEFREE
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In each time step we have to solve a set of convection-diffusion
!     equations for each particle component
!     At the interface the following boundary conditions must be satisfied:
!     D_e dc_e/dn + vn c_e = g_e, &
!     where D_e is the diffusion coefficient in the convection-diffusion e
!           c_e is the concentration of component e
!           vn is the unknown interface velocity and g_e the right-hand side
!     g_e is equal to c_e^p * vn, with c_e^p the particle concentration
!     To compute vn the following relations must be satisfied:
!
!     x_e^p = c_e^p/sum_e(c_e^p) is the mole fraction within the precipitate
!     c_e^s = c_e^p/K_e vn + c_e the mole fraction along the interface
!         with c_e the concentration at the interface and K_e the given
!         interface reaction speed
!     M is the maximum total concentration of substitutional elements
!     M = cinf(1)/xinf(1) = sum(cinf(S))-sum(cinf)
!        with cinf and xinf given and S the set of substitutional elements
!     I is the set of interstitial elements, the complement of S
!     The equations to be satisfied are
!     prod((x_e^s)^x_e^p) = kinf exp ( zeta kappa ),      (1)
!     with
!     x_e^s = c_e^s / (sum(c_I^s) + M)
!     zeta = gamma V_m / (R_g T ) is given
!     kappa is the curvature at the interface
!     convergence is reached if (1) is satisfied accurately enough
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prpartpnlncon1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpartpnlncon1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d14.6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute some parameters

      call prpartpnlnconp ( inpsep, numcomps, xp, bigM, maxiter, &
                            eps, rinsep, iseqconc, iseqvn, cpart, k, &
                            substitutional, dimsub, iseqg, kinf, &
                            itermethod, iseqiter, iprint, iseqcs )

      debug = debug .or. iprint>=3

!     --- Find node at boundary

      inpseploc(6) = 1   ! level set sequence number
      inpseploc(7) = 20  ! zero level set
      inode = prnode ( inpseploc )

      if ( debug ) write(irefwr,1) 'inode', inode

!     --- Copy old concentration into temporary array concold

      numvecsav = numvecmn
      numvecmn = numvecmn+numcomps
      call inichkarlen ( numvecmn, maxvc0, 'isolut' )
      if ( ierror/=0 ) go to 1000

      do i = 1, numcomps
         call sepcopyvc ( isolut2d(iseqconc+i-1), &
                         isolut2d(numvecsav+i) )
      end do  ! i = 1, numcomps
      if ( ierror/=0 ) go to 1000

!     --- loop until convergence

      diff = 1d3
      diffprev = diff
      iloop = 0
      minloop = 2

      vn = getparm ( inpsep, rinsep, 3, 'vnboun' )
      if ( ierror/=0 ) go to 1000

      if ( iprint>=2 ) write(irefwr,100)
100   format ( 'In iteration process for particle non-linear', &
               'concentration'// &
               'iter   |factor-1|  quotient' )

      do while ( diff>eps .and. iloop<maxiter .or. iloop<=minloop )

!     --- Find concentration at boundary

         do i = 1, numcomps
            cboun(i) = prvalnode ( isolut2d(iseqconc+i-1), inode )
         end do  ! i = 1, numcomps

         istore = mod(iloop,3)+1

         if ( debug ) then
            write(irefwr,1) 'iloop, istore', iloop, istore
            write(irefwr,2) 'vn', vn
            write(irefwr,2) 'cboun', cboun
         end if  ! ( debug )

!        --- First compute the mole fractions c_e^s defined by
!            c_e^s = c_e^p/K_e vn + ce, &
!            with ce the concentration along the boundary

         cs = cpart/k * vn + cboun
         if ( debug ) write(irefwr,2) 'cs', cs

!        --- next x_e^s defined by
!            x_e^s =  c_e^s / (sumI(c^s) + M)

         h = bigM+sum(cs(1:numcomps))-sum(cs(substitutional(1:dimsub)))
         xs = cs/h
         if ( debug ) write(irefwr,2) 'xs', xs

!        --- Compute product (x_e^s)^x_e^p

         zeroval = .false.
         i = 0
         do while ( .not. zeroval .and. i<numcomps )

            i = i+1
            if ( abs(xs(i))<=0d0 ) then
               zeroval = .true.
            else
               xs(i) = xs(i)**xp(i)
            end if  ! ( abs(xs(i))<=0d0 )

         end do  ! while ( .not. zeroval )

         if ( zeroval .or. vn==0d0 ) then
            factor = 0d0
            vn = 1d-3
         else
            factor = kinf/product(xs(1:numcomps))
            vn = vn*factor
         end if  ! ( zeroval )
         diff = abs(factor-1d0)

         if ( debug ) &
            write(irefwr,2) 'vn, factor, diff', vn, factor, diff

         if ( itermethod>1 ) then

!        --- itermethod>1, use Aitken's extrapolation

            vstore(istore) = vn

!           --- Apply Aitken's method after each 3 steps
!               However, skip the first three steps to avoid influence
!               of the wrong initial guess

            if ( iloop>3 .and. istore==3 ) then

!           --- 3 values in a row

               if ( debug ) call prinrl ( vstore, 3, 'vstore' )
               call praitken ( 1, vn, vstore, itermethod-1 )
               if ( ierror/=0 ) go to 1000

               if ( debug ) write(irefwr,2) 'vn', vn

            end if  ! ( iloop==3 )

         end if  ! ( itermethod>1 )
         if ( ierror/=0 ) go to 1000

         call putscal ( 'vnboun', iseqvn, vn )
         if ( ierror/=0 ) go to 1000

!        --- Put right-hand side of boundary condition in cons

         g = vn*cpart
         call putconvararray ( inpsep(4), g, 'g', numcomps )
         if ( ierror/=0 ) go to 1000

!        --- Copy old concentration back into concentration

         do i = 1, numcomps
            call sepcopyvc ( isolut2d(numvecsav+i), &
                           isolut2d(iseqconc+i-1) )
         end do  ! i = 1, numcomps
         if ( ierror/=0 ) go to 1000

         inpseploc(1) = inpsep(5)         ! iseqtime
         inpseploc(2) = iseqconc+5000

         istart = istartmn
         istartmn = 0
         call septimefree ( finished, inpseploc, rinsep )
         istartmn = istart

         if ( ierror/=0 ) go to 1000
         maxcalltim = 2  ! set maximum number of calls per timestep>1
                         ! in this way reading of coefficients is triggered
                         ! in the next call

         iloop = iloop+1

         if ( iloop<3 ) diff = 1d0
         if ( debug ) write(irefwr,2) 'g', g

         if ( iprint>=2 ) write(irefwr,120) iloop, diff, diff/diffprev
120      format ( i5, 2d12.3 )

         diffprev = diff

      end do  ! while ( diff>eps and iloop<maxiter )

      if ( iseqcs>0 ) &
         call putconvararray ( inpsep(20), cs, 'cs', numcomps )

      numvecmn = numvecsav

      h = iloop
      if ( iseqiter>0 ) call putscal ( 'num_iter', iseqiter, h )

      if ( iprint>=1 ) write(irefwr,130) iloop, diff
130   format ( 'number of executed iterations = ', i5, &
               '  |factor-1| = ', d12.3 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpartpnlncon1'

      end if  ! ( debug )
      call erclos ( 'prpartpnlncon1' )

      end subroutine prpartpnlncon1
