      subroutine prpartpnlnconc ( finished, inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     One time step for the integration of a particle is carried out
!     It concerns a system of concentration equations with a nonlinear
!     set of mixed boundary conditions
!     The interface concentration csol is computed in a loop until
!     convergence is reached
! **********************************************************************
!
!                       KEYWORDS
!
!     particle
!     level_set
!     time_dependent
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)
      double precision :: rinsep(*)
      logical finished

!     finished      i/o   Indicates if the loop has been finished (true) or not
!                         (false)
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscpnlnconc for a description
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscpnlnconc for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numcomps, ilev, istart, nofacenew

!                    they are not
!     ilev           Sequence number of level being considered
!     istart         dummy
!     nofacenew      Number of nodes in interfacenew
!     numcomps       Number of components
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getintparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     GETINTPARM     Return with real/scalar defined by inpsep/rinsep
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRLEVINTERFST  Compute starting address of interface in array ipointobst
!     PRPARTPNLNCON1 1d time step for the integration of a particle
!     PRPARTPNLNCON2 2/3d time step for the integration of a particle
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3142   new interface is empty
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In each time step we have to solve a set of convection-diffusion
!     equations for each particle component
!     At the interface the following boundary conditions must be satisfied:
!     D_e dc_e/dn + vn c_e = g_e, &
!     where D_e is the diffusion coefficient in the convection-diffusion e
!           c_e is the concentration of component e
!           vn is the unknown interface velocity and g_e the right-hand side
!     g_e is equal to c_e^p * vn, with c_e^p the particle concentration
!     To compute vn the following relations must be satisfied:
!
!     x_e^p = c_e^p/sum_e(c_e^p) is the mole fraction within the precipitate
!     c_e^s = c_e^p/K_e vn + c_e the mole fraction along the interface
!         with c_e the concentration at the interface and K_e the given
!         interface reaction speed
!     M is the maximum total concentration of substitutional elements
!     M = cinf(1)/xinf(1) = sum(cinf(S))-sum(cinf)
!        with cinf and xinf given and S the set of substitutional elements
!     I is the set of interstitial elements, the complement of S
!     The equations to be satisfied are
!     prod((x_e^s)^x_e^p) = kinf exp ( zeta kappa ),      (1)
!     with
!     x_e^s = c_e^s / (sum(c_I^s) + M)
!     zeta = gamma V_m / (R_g T ) is given
!     kappa is the curvature at the interface
!     convergence is reached if (1) is satisfied accurately enough
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prpartpnlnconc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prpartpnlnconc'
         call prininlarge ( inpsep, 19, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First check if there is an interface, if not return

      ilev = 1
      call prlevinterfst ( ipointobst, istart, nofacenew, ilev )

      if ( debug ) write(irefwr,1) 'nofacenew', nofacenew

      if ( nofacenew==0 ) then

!     --- No new interface, warning and return

         call errwar ( 3142, 0, 0, 0 )
         go to 900

      end if  ! ( nofacenew==0 )

!     --- get parameters

      numcomps = getintparm(inpsep(6))         ! Number of components

      if ( ndim==1 ) then

!     --- ndim = 1

         call prpartpnlncon1 ( finished, maxvc1, maxprb, inpsep, &
                               rinsep, numcomps )

      else

!     --- ndim>1

         call prpartpnlncon2 ( finished, inpsep, rinsep, numcomps )

      end if  ! ( ndim==1 )

900   istartmn = istartmn+20

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prpartpnlnconc'

      end if  ! ( debug )
      call erclos ( 'prpartpnlnconc' )

      end
