      subroutine prsolveinv01 ( epsilon, smatrix, rinputinv, cmeasured, &
                                inelem, lengcvec, c0, c1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Solve system of equations for an inverse problem using the formula
!     (S^T S + mu I) eps = S^T C* &
!    with S the matrix, C* the vector with measured quantities, mu a
!    regalurization parameter and eps the unknown quantity
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     inverse_problem
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inelem, lengcvec
      double precision epsilon(inelem), smatrix(inelem,lengcvec), &
                       cmeasured(lengcvec), rinputinv(*), c0(*), c1(*)

!     c0             i    Vector c_0
!     c1             i    Vector c_1
!     cmeasured      i    Array containing the measured capacities
!     epsilon        o    Solution array containing all epsilon value
!     inelem         i    Number of elements in a specific element group
!     lengcvec       i    Length of the capacity vectors
!     rinputinv      i    Double precision input array for subroutine
!                         prcompinv
!                         Contents
!                          1: value of delta_eps
!                          2: value of regularization parameter mu
!                          3: value of eps_ref
!     smatrix        i    Array of size inelem x lengcvec containing
!                         the matrix to be used for the inverse problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision cons1, regulparm, epsilon0, yvector(lengcvec), &
                       amat(lengcvec,lengcvec)

!     amat           work array of size lengcvec x lengcvec to be used
!                    to store a matrix to be inverted
!     cons1          Help variable to store a constant
!     epsilon0       Value of epsilon_0
!     i              Counting variable
!     j              Counting variable
!     regulparm      Regularization parameter mu
!     yvector        Work array of size lengcvec to store a help vector
!                    during inversion
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRFULLGG       Compute the G G^T decomposition of a full symmetric
!                    positive definite matrix
!     PRSFULLGG      Solve system of equations where the matrix has been
!                    decomposed by PRFULLGG
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In order to solve the system of equations:
!
!      eps = ( S^T S + mu I )^-1  S^T C* &
!
!     with C^* of size m and S of size m x nelem  (nelem>>m)
!
!     C* = C_measure - C_0 / (C_1 - C_0)
!
!     we apply the so-called Sherman-Morrison-Woodbury formula:
!     (See Gene H. Golub and Charles F. van Loan;
!      Matrix Computations, page 3)
!
!     (A+UV^T)^-1 = A^-1 - A^-1 * U * (I + V^T *A^-1 * U )^-1 * V^T *A^-1
!
!     where A = mu I and U = V = S^T, hence
!
!     ( mu I + S^T S )^-1 = 1/mu I - 1/mu S^T (I + S (1/mu I) S^T)^-1 S 1/mu I
!
!     So the sequence of computations becomes:
!
!     eps := 1/mu S^T C* &
!     M := I + S (1/mu I) S^T
!     y := S eps
!     y := M^-1 y
!     eps := eps -1/mu S^T y
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsolveinv01' )

!     --- Update cmeasured

      do i = 1, lengcvec
         cmeasured(i) = (cmeasured(i)-c0(i)) / (c1(i)-c0(i))
      end do

      regulparm = rinputinv(2)
      epsilon0 = rinputinv(3)
      cons1 = 1d0/regulparm

!     --- eps := 1/mu S^T C* &

      do i = 1, inelem
         epsilon(i) = cons1 * &
            ddot ( lengcvec, smatrix(i,1), inelem, cmeasured, 1 )
      end do

!     --- M := I + S (1/mu I) S^T
!         First M := 0

      amat = 0d0

!     --- next M := I

      do i = 1, lengcvec
         amat(i,i) = 1d0
      end do

!     --- Finally M := I + S (1/mu I) S^T

      do i = 1, lengcvec
         do j = 1, lengcvec
            amat(i,j) = amat(i,j) + cons1 * &
                ddot ( inelem, smatrix(1,i), 1, smatrix(1,j), 1 )
         end do
      end do

!     --- y := S eps

      do i = 1, lengcvec
         yvector(i) = ddot ( inelem, smatrix(1,i), 1, epsilon, 1 )
      end do

!     --- y := M^-1 y

      call prfullgg ( amat, lengcvec )
      call prsfullgg ( amat, yvector, yvector, lengcvec )

!     --- eps := eps -1/mu S^T y

      do i  = 1, inelem
         epsilon(i) = epsilon0 + epsilon(i) - cons1 * &
            ddot ( lengcvec, smatrix(i,1), inelem, yvector, 1 )
      end do

      call erclos ( 'prsolveinv01' )
      end subroutine prsolveinv01
