      subroutine sepbuildnest ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     replace matrix from row-compact storage to row-compact storage
!     for nested dissection
!     This means that positions corresponding to fill-in are filled
!     by zeros
! **********************************************************************
!
!                       KEYWORDS
!
!     stiffness_matrix
!     copy
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemain
      use sepmodulematr
      use sepmoduleintmat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Matrix A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      integer, pointer ::  point(:), rows(:), cols(:)
      integer :: i, j, numrows

!     IM             Refers to intmt(intmat)
!     cols           Column numbers of matrix
!     i              Counting variable
!     j              Counting variable
!     numrows        number of rows in the matrix
!     point          pointer array to indicate the position of columns
!                    of a row in the matrix
!     rows           starting addresses (-1) of rows in array cols
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildnest' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildnest'
         call sepprinmat6 ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(A%iseqim)
      numrows = A%neq
      rows => IM%rowsiiprec
      cols => IM%colsiiprec
      copyprec = .false.
      skipzeros = .false.
      A%adapt = 0d0
      A%prlump = 0

      if ( debug ) call sepprinmat6 ( A )

!     --- Create new matrix

      allocate ( point(numrows), realmt(iseqmt)%matinv(rows(numrows+1)) )
      A%MD => realmt(iseqmt)%matinv

      point = 0

!     --- Create new matrix

      A%MD = 0d0  ! Clear matrix
      A%Mrows => rows
      A%Mcols => cols
      A%Mdiag => IM%diagprec
      A%scaling = 10

!     --- Copy A%S into A%MD

      do i = 1, numrows

!     --- array point with positions of non-zero columns in row i

         do j = rows(i)+1, rows(i+1)
            point(cols(j)) = j
         end do  ! j = rows(i+1), rows(i+1)

!        --- Copy row from A%S

         do j = IM%rowsii(i)+1, IM%rowsii(i+1)
            A%MD(point(IM%colsii(j))) = A%S(j)
         end do  ! j = IM%rows(i)+1, IM%rows(i+1)

!        --- Clear array point

         do j = rows(i)+1, rows(i+1)
            point(cols(j)) = 0
         end do  ! j = rows(i+1), rows(i+1)
      end do  ! i = 1, numrows

      deallocate ( point )

1000  if ( debug ) then

!     --- Debug information

         call sepprinmat6p ( A )
         write(irefwr,*) 'End sepbuildnest'

      end if  ! ( debug )
      call erclos ( 'sepbuildnest' )

      end subroutine sepbuildnest
