      subroutine seppjnode3
! ======================================================================
!
!        programmer    Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutines determines per node in which set it lies and vice
!     versa. Actually a node can be in two sets, but if this is the case
!     then only the sand set is considered.
!     If a node lies on a DIRICHLET boundary then it gets set number zero.
!     Old subroutine prpjnode3
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielm, inodp, ielgrp, ngelm, idegfd, prevset, prevtype, &
                 i, j, iset, iproj
      integer, allocatable :: iwork1(:), gpelm(:)

!     gpelm          Integer array containing information on element group
!                    numbers for each element.
!                    GPELM( IELM ) = element group element IELM lies in.
!     i              Loop variable
!     idegfd         Number of the degree of freedom
!     ielgrp         Multi purpose variable, element group number
!     ielm           Element number
!     inodp          Loop variable
!     iproj
!     iset           Set number
!     iwork1         Integer help array
!     j              Loop variable
!     ngelm          Number of elements in current element group
!     prevset
!     prevtype
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Collecting all nodes for each set
!
!     Note that if a node lies in more than one set only the sand
!     set is considered.
!
!     Note that if a node lies on a Dirichlet boundary it is
!     assigned to set number zero; that is a non-existing set.
!
!     Note that an interface is a collection of nodes that lie
!     in set iSET, but also in set jSET.
!
!     Semi PSD: prpjDNODE
!
!     Initialization:
!
!     The number of projection vectors is zero: nPROJ := 0.
!     There are no nodes found yet so nSETNODE(1..nSET+1) := 0.
!
!
!     Determine the element group number for each element:
!     Loop over all element groups:
!     DO iELGRP := 1, nELGRP
!        Per element group we know which elements belong to that
!        element group. For each element in the element group
!        iELGRP, the element group number is stored in array
!        GPELM: GPELM(iELM) := iELGRP.
!     END DO
!
!     Determine for each node the set it lies in:
!     Loop over all nodal points:
!     DO iNODP := 1, nPOINT
!
!        The information about elements connected to node
!        iNODP is given. With the array GPELM we can determine
!
!        which element groups the node lies in. Finally the
!        array GROUPSET gives the sets the node lies in.
!        The number of sets can be one (node is inside a set) or
!        two (node lies on an interface between to sets)
!
!        The number of the degree of freedom associated with
!
!        node iNODP is known: iDEGFD.
!        IF this degree of freedom is prescribed
!           This means also that the sets this
!           node lies in have Dirichlet conditions:
!           DIRICHLT(iSET) := 1 for all relevant sets.
!        ELSE
!           If the degree of freedom is not prescribed THEN
!           IF the node lies in one set THEN
!              Assign the set number to the node:
!
!              NODESET(iNODP) := iSET.
!           ELSE
!              Only the sand set is considered.
!              Besides that the set number is marked with a
!              minus sign to indicate that the node lies on an
!              interface: NODESET(iNODP) := -iSET.
!           END IF
!           The number of nodes that belong to the assigned set
!           is increased with one: nSETNODE(iSET+1) := &
!           nSETNODE(iSET+1)+1
!        END IF
!     END DO
!
!     Determine the number of projection vectors and transform
!     array nSETNODE into an index array for array SETNODE:
!     Loop over all sets:
!     DO iSET := 1, nSET
!        Check if area iSET is a sand area (SANDSET(iSET)=1) and
!
!        has no Dirichlet conditions (DIRICHLT(iSET)=0).
!        IF that is the case THEN
!           Increase the number of projection vectors with one:
!           nPROJ := nPROJ + 1. The number of the sand area, iSET, &
!           that belongs to this projection vector is stored in
!           PROJSAND: PROJSAND(nPROJ) := iSET. Besides that we
!           also like to know which projection vector
!           belongs to the current sand area so we set
!           SANDPROJ(iSET) := nPROJ.
!        END IF
!        The array nSETNODE is transformed into an index array:
!        nSETNODE(iSET+1) := nSETNODE(iSET+1)+nSETNODE(iSET).
!     END DO
!
!     Determine the nodes that belong to each set by looping
!     over all entries in array NODESET. This yields array
!     SETNODE.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjnode3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjnode3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork1(nset), gpelm(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nset+nelem, 'iwork1' )
      if ( ierror/=0 ) go to 1000

      if ( print_lv.ge.3 ) then
         write(irefwr,*) ' '
         write(irefwr,*) '   Number of points: ', npoint
         write(irefwr,*) '   Number of sets: ', nset
         write(irefwr,*) ' '
      end if

!     --- Determination of elementgroup per element with KMESHG

      ielm = 0

!     --- loop over all element groups

      do ielgrp = 1, nelgrp

!     --- there are ngelm elements in the current element group

         ngelm = kmeshg( ielgrp )
         do j = 1, ngelm
            ielm = ielm + 1

!           --- store the element group number for each element

            gpelm( ielm ) = ielgrp
         end do

!        --- set the number of nodal points per set equal to zero

         nsetnode( ielgrp + 1 ) = 0

      end do

!     --- Determination of set number for each node with KMESHD

!         --- set the number of nodal points per set equal to zero

      nsetnode( 1 ) = 0

!     --- Initialize array DIRICHLT, SANDPROJ and PROJSAND

      do iset = 1, nset
         dirichlt( iset ) = 0
         sandproj( iset ) = 0
         projsand( iset ) = 0
      end do

!     --- loop over all points

      do inodp = 1, npoint
         idegfd = kprobh( inodp )

!        --- no set has been found yet

         prevset = 0
         prevtype = -1

!        --- loop over all elements which point inodp lies in

         do j = kmeshd( inodp ) + 1, kmeshd( inodp + 1 )
            ielm = kmeshd( npoint + 1 + j )
            ielgrp = gpelm( ielm )
            iset = groupset( ielgrp )

            if ( idegfd.gt.nrusol ) then

!           --- nodal point lies on a dirichlet boundary

               nodeset( inodp ) = 0
               dirichlt( iset ) = 1
               prevtype = 0
               prevset = 0

            else if (( prevtype.eq.0 ).and.( sandset( iset ).eq.1 )) then

!           --- Shale set has previously been assigned to the nodal
!               point. Nodal point lies also in a sand set, so the wrong
!               set has been assigned. This is corrected.

               nsetnode( 1 + prevset ) = nsetnode( 1 + prevset ) - 1
               nsetnode( 1 + iset ) = nsetnode( 1 + iset ) + 1
               prevtype = sandset( iset )
               prevset = iset

!              --- Node lies on an internal border

               nodeset( inodp ) = -iset

            else if (( prevtype.eq.1 ).and.( sandset( iset ).eq.0 )) then

!           --- Node lies on an internal border

               nodeset( inodp ) = - abs( nodeset( inodp ))

            else if ( prevtype.eq.(-1) ) then

!           --- No set has been assigned to the nodal point yet, so
!               the current set is valid.

               nodeset( inodp ) = iset
               nsetnode( 1 + iset ) = nsetnode( 1 + iset ) + 1
               prevtype = sandset( iset )
               prevset = iset

            end if

         end do

      end do

!     --- the number of projection vectors is zero

      nproj = nset

      do iset = 1, nset

!         --- a new projection vector has to be made

            iproj = iset
            projsand( nproj ) = iset
            sandproj( iset ) = iproj

!           --- end if

!               --- accumulating the number of nodes per set

         nsetnode( iset + 1 ) = nsetnode( iset + 1 ) + nsetnode( iset )

!        --- making a copy of nsetnode

         iwork1( iset ) = nsetnode( iset )
      end do

!     --- creating array setnodes from array nodeset

      do inodp = 1, npoint
         iset = abs( nodeset( inodp ))
         if ( iset.gt.0 ) then
            iwork1( iset ) = iwork1( iset ) + 1
            setnode( iwork1( iset )) = inodp
         end if
      end do

      if ( print_lv.ge.4 ) then
         write(irefwr,*) '   [ nodal point : degree of freedom ]'
         do inodp=1,npoint
            idegfd = kprobh( inodp )
            write(irefwr,*) '   ', inodp,' : ',idegfd
         end do
         write(irefwr,*) ' '
         write(irefwr,*) &
                   '   [ set : (number of nodal points) nodal points ]'
         do iset=1,nset
            write(irefwr,*) '   ', iset, ' : (', &
            nsetnode(iset+1)-nsetnode(iset), ') ', &
            (setnode(i),i=nsetnode(iset)+1,nsetnode(iset+1))
         end do
         write(irefwr,*) ' '
      end if

      if ( print_lv.ge.3 ) then
         write(irefwr,*) '   [ sandset : projection vector ]'
         do iset=1,nset

!        --- if (( sandset( iset ).eq.1 ).and.( dirichlt( iset ).eq.0 ))
!            +      then

               write(irefwr,*) '   ', iset, ' : ', sandproj(iset)

!              --- end if

         end do
         write(irefwr,*) ' '
      end if

      deallocate ( iwork1, gpelm, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork1' )

1000  call erclos ( 'seppjnode3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppjnode3'

      end if  ! ( debug )

      end subroutine seppjnode3
