      subroutine septimconv ( isol, iseqsollc, mstep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-02-2016 New call to prsolveinput
!        version  1.0  date 20-11-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Treat the convective terms in the Navier-Stokes equations
!     in an explicit way using substepping and operator splitting
!     Old subroutine prtimconv
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     navier_stokes_equation
!     pressure_correction
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulesol
      use sepmoduleuser
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesolve
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: mstep, iseqsollc
      integer, intent(inout) :: isol(*)

!     iseqsollc      i    Sequence number of solution array in array ISOL
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     mstep          i    Number of substeps per time step
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istep, icopy, isubr, ictime, iprob, iseqcf, &
              iinbldlc(5), irhsdlc, &
              iinvec(3), inpdum(1), mmmassmt, nprob
      double precision rinvec(2), deltatstar, rindum(1)
      logical :: debugext

!     debugext       If true extra debug statements are carried out
!     deltatstar     Local value of delta t, defined by delta t / mstep
!     icopy          Sepran array in which the old solution array is copied
!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 )
!     iinbldlc       User input array for subroutine BUILD
!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     inpdum         dummy
!     iprob          Actual problem number
!     irhsdlc        Contains nformation of the right-hand side
!     iseqcf         Sequence number corresponding to the coefficients
!     istep          Counter for substeps
!     isubr          Indicates the type of calling subroutine
!     mmmassmt       Help variable to store the memory management number of
!                    the mass matrix
!     nprob          Number of various problems
!     rindum         Dummy
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBUILD       Build matrix and right-hand side
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPLINSOL      Body of linear solver
!     SEPMANVEC      Vector manipulation subroutine
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLVEINPUT  Fill arrays inpsol and rinsol
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The time-dependent discretized Navier-Stokes equations can be
!     written as:
!
!     M du/dt + S u + N(u)u + L^t p = f
!     L u = 0
!
!     with M the mass matrix, S the stiffness matrix corresponding
!          S the stiffness matrix corresponding to the viscous part
!          N(u)u the discretization of the convection
!          L^t p the discretization of the gradient of p
!          L u = 0 the discretization of the continuity equation
!
!     These equations are solved by the following type of operator splitting:
!     Let mstep be the number of substeps
!     Then:
!     dt* = dt/mstep
!     For istep := 1 (1) mstep do
!        M (u*-u^n)/dt* + N(u^n)u^n = 0
!        u^n := u* &
!     M  (u^(n+1)-u*)/dt + S u^(n+1) + L^t p^(n+1) = f^(n+1)
!     L u^(n+1) = 0
!
!     In this subroutine we carry out the for loop
!
!     The pseudo code reads:
!     dt* = dt/mstep
!     For istep := 1 (1) mstep do
!        Copy u into ucopy
!        Solve M u /dt* = -N(u) u
!        u := -u + ucopy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimconv' )
      debug = .false.
      debugext = .false.
      iseqsol = iseqsollc
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimconv'
         write(irefwr,1) 'iseqsol, mstep, ireadcf, ireadsl', &
                          iseqsol, mstep, ireadcftim(2), Teq(1)%ireadsl
         write(irefwr,1) 'isol(iseqsol)', isol(iseqsol)
         if ( debugext ) &
            call sepprintv ( isol(iseqsol),'u^n' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      intmat => intmat1d(1)
      irhsdlc = 0

      iprob = ks(isol(iseqsol))%iprob  ! problem number

!     --- Fill coefficients in arrays iuser and user
!         These arrays are stored at the end of ibuffr
!         This coefficients array is only meant for the right-hand side
!         using Newton linearization, but without other input for
!         the right-hand side. The user has to supply this input himself
!         parameters ictime, icofiusr, icofuser are not used

      isubr = 3         ! used by "linprb"
      ictime = 0        ! is only used in case of to0089
      iseqcf = ireadcftim(2)

      if ( ioutp>=0 .and. itimloop<=1 .and. iseqcf>0 .and. &
           nitercom<=1 ) write ( irefwr, 110 ) iseqcf
110   format ( ' Fill coefficients with sequence number ', i3 )

      call erlins ( isubr, iprob, iseqcf )

      if ( associated(iuser1(iprob)%iuser) ) then

!     --- iuser1(iprob)>0, IUSER and USER are filled

         iuser => iuser1(iprob)%iuser
         user => iuser1(iprob)%user

      end if  ! ( iuser1(iprob)>0 )

!     --- dt* = dt/mstep
!         This step is not carried out, since massmat contains M/dt
!         For istep := 1 (1) mstep do

      icopy = 0  ! memory management number of ucopy
      deltatstar = tstep/mstep
      do istep = 1, mstep

!     --- Copy u into ucopy

          call sepcopyvc ( isol(iseqsol), icopy )

!         --- Solve M u /dt* = -N(u) u
!             The matrix we have is M, hence we solve M delta u  = -dt* N(u) u
!             Now we have to compute the convection term
!             This means that we build only a right-hand side vector
!             Furthermore the Newton iteration must be applied without
!             right-hand side.
!             To do this end we need a separate coefficients input, &
!             referring to Newton and no rhs
!             First build right-hand side
!             The right-hand side we get is N(u)u

         iinbldlc(1) = 5     ! Setting first 5 input options for build
         iinbldlc(2) = 2     ! only large vector is build (2)
         iinbldlc(3) = 1     ! BC's taken into account (0) or not (1)
         iinbldlc(4) = 0     ! diag mass mt computed (1) stored as vect
         iinbldlc(5) = 1     ! old sol stored in islold (1) or not (0)
         call sepfillsmf ( iinbldlc, irhsdlc, isol, isol(iseqsol) )

         call sepbuild
         if ( debugext ) call sepprintv ( irhsdlc, 'convective rhs' )

!        --- Next solve u* into isol(iseqsol) (actually - delta u)

         iseqreac = 0
         iseqintmat = 1
         keepmat = 1
         if ( istep==mstep ) keepmat = 0
         iseqsolut = iseqsol

         iseqmt = massmat
         if ( istep==1 ) then
            inpdum(1) = 0
            call sepsolveinput ( Teq(1)%ireadsl, inpdum, rindum )
            skipzeros = .true.
         end if  ! ( istep==1 )
         call seplinsol ( massmat, isol, irhsdlc )
         if ( debugext ) call sepprintv ( isol(iseqsol), 'delta u' )

!        --- u := -u + ucopy
!            or u* = u^n - 1/dt* delta u

         iinvec(1) = 3    ! only 3 positions have been filled
         iinvec(2) = 27   ! linear combination
         iinvec(3) = -1   ! only internal unknowns
         rinvec(1) = 1d0  ! coefficient for icopy
         rinvec(2) = -deltatstar ! coefficient for isol
         call sepmanvec ( iinvec, rinvec, icopy, &
                          isol(iseqsol), isol(iseqsol) )
         if ( debugext ) call sepprintv ( isol(iseqsol), 'u*' )

      end do  ! istep = 1, mstep

!     --- Remove arrays icopy and irhsdlc

      call seprmsolinfo ( icopy )
      call seprmsolinfo ( irhsdlc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimconv'

      end if
      call erclos ( 'septimconv' )

      end
