      subroutine septimefree ( finished, inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 07-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to be used for instationary free surfaces
!     It performs one time-step
!     This is a very special subroutine in the SEPRAN context, since it
!     assumes that the user has made a loop himself
!     septimefree should be called in this loop
!     It returns with a value for finished indicating if the loop may be
!     finished or not.
!     The following tasks are performed:
!     -  In first call: set time integration constants
!     -  Perform one time step
!     -  In last time step: set finished and reset first
!     In contrast to adapmesh input is only possible through the input file
!     and not through the parameter list
!     Warning:  Experimental version
!               Both the parameter list as well as the input may change
!               Check this in each new update of SEPRAN until this
!               subroutine is part of the manuals
!     Old subroutine timefrbf
! **********************************************************************
!
!                       KEYWORDS
!
!     free_boundary
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the loop has been finished (true) or not
!                         (false)
!     inpsep         i    The integer input read is stored in array INPSEP
!                         1: IREAD Defines how the input must be read
!                            Possible values:
!                            0   The input is read from the standard input file
!                            >0  The input as read from the standard input file
!                                by subroutine SEPSTM is used. The value of
!                                iread defines the sequence number
!                         2: IVECTR, sequence number of solution vector with
!                            respect to array isolut
!     rinsep         i    The real input read is stored in array RINSEP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lniincommat=25
      integer :: iseqintmat, isubr, iincommat(lniincommat), ivector, &
                 ivecseq, numarrays
      logical :: first

!     first          If true it concerns the first time step
!     iincommat      Integer input array containing information concerning
!                    the structure of the large matrix
!                    See subroutine read00 for a description
!     iseqintmat     Sequence number with respect to array intmat1d
!     isubr          Indicates the type of calling subroutine
!     ivecseq        Sequence number of vector to be considered with respect
!                    to the solution array (isol or isolut)
!     ivector        Sequence number of first vector
!     lniincommat    Declared length of array iincommat
!     numarrays      Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     INIVECTR       Return with sequence number of vector
!     SEPFILLTIMINP  Fill module sepmoduletime with information stored in
!                    inpsep/rinsep and ibuffr
!     SEPTIMEFR02    Adapt time parameters at end of time step for timefrbf
!     SEPTIMEHIS4    Fill time step for time history
!     SEPTIMPRB      solves time-dependent problems
!     SEPTIMSEQINTM  Fill array intmat1d if necessary and initialize iseqintmat
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimefree' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimefree'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( itimloop<=1 ) then
         first = .true.
      else
         first = .false.
      end if

!     --- Copy input concerning time_integration into module sepmoduletime

      isubr = 3
      itimloop = 2
      ivector = max(1,(inivectr(inpsep(2))))
      call sepfilltiminp ( inpsep, isubr, ivector )
      if ( debug ) then
         write(irefwr,*) 'first ', first
         write(irefwr,1) 'ivector', ivector
         write(irefwr,2) 't, t0, tend, tstep', t, t0, tend, tstep
      end if  ! ( debug )

      numoldtim = max(1,numvecmn)
      maxoldtim = maxvec

      if ( nocomputationtim ) then
         call inireducevec ( ivector, ivecseq, numarrays )
      else
         call inireducevec ( Teq(1)%iseqfr, ivecseq, numarrays )
      end if  ! ( nocomputationtim )
      iprobtim = ks(isolut2d(ivecseq))%iprob

      if ( debug ) write(irefwr,1) 'ivecseq, iprobtim', ivecseq, iprobtim

!     --- Fill iseqintmat and intmat1d if necessary

      intmatar => intmat1d
      call septimseqintm ( iseqintmat )

!     --- Call time subroutine

      length = size(intmat1d)
      intmatar => intmat1d(iseqintmat:length)
      call septimprb ( finished, first, isubr )

      istartmn = istartmn+3  ! raise istartmn

!     --- Adapt time parameters

      if ( debug ) write(irefwr,*) 'before prtimefr02'
      call septimefr02

!     --- write time step to time history

      if ( debug ) write(irefwr,*) 'before prtimehis4'
      call septimehis4

      if ( t>=tend-0.00001*tstep .and. keeptim/=0 .and. &
           itiminterval>=ndt ) then

!     --- t = tend  deallocate arrays

         if ( associated(ireadcftim) ) then

            deallocate ( ireadcftim, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'ireadcftim' )
            if ( ierror/=0 ) go to 1000

            if ( nhist>0 ) deallocate ( coorhist, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'coorhist' )
            if ( ierror/=0 ) go to 1000

            deallocate ( Teq, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'timeequation' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( .not. associated(ireadcftim) )

      end if  ! ( t>=tend-0.00001*tstep )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimefree'

      end if  ! ( debug )
      call erclos ( 'septimefree' )

      end subroutine septimefree
