      subroutine septimeulimpl ( first, isol, irhsdlc, isolprec, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 02-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one step in an Euler implicit scheme for one equation.
!     Coefficients may depend on time and previous solutions
!     It is supposed that the time has been raised outside this subroutine
!     Also that boundary conditions have been filled as well as arrays iuser and
!     user
!     This is the basic subroutine with respect to time integration.
!     All decisions are made outside this subroutine.
!     Old subroutine: prtm00
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isolprec, ieqn
      integer, intent(inout) :: isol(*), irhsdlc
      logical, intent(in) :: first

!     first          i    If true it concerns the first call in a row, &
!                         otherwise it concerns a subsequent call
!     ieqn           i    Equation sequence number
!     irhsdlc          i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol          i/o   SEPRAN array containing information of the solution
!                         array. At input the boundary condition must have been
!                         filled in isol
!     isolprec       i    Array referring to the previous time-level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
!     SEPTIMEULSTEP  Perform one time step for an Euler implicit discretization
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!
!        .
!     M  u   +  S  u   =  f
!
!     Application of Euler
!
!     -Euler part (predictor):
!
!       n+1     n
!      u    -  u            n+1      n
!    M ----------- +  S  u     =  f          (n is actual time-level)
!          dt
!
!
!   Effect of boundary conditions:
!
!   We split the matrix into an internal part and a part due to
!   boundary conditions. The Euler part can then be written as:
!
!                        .
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ii      ir |  |   i |      |   ii      ir |  |   i |      |   i |
!   |              |  |  .  |   +  |              |  |     |  =   |     |
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ri      rr |  |   r |      |   ri      rr |  |   r |      |   r |
!
!   The last row is deleted, hence the discretized equations become:
!
!                    n+1       n+1        n+1            n            n+1   n
!   ( M /dt  + S  ) u    =  ( f    - S   u   )  + M /dt u   - M/dt ( u   - u  )
!      ii       ii   i         i      ir  r        ii    i     ir     r     r
!
!
!
!   Store boundary conditions into usol
!   Build matrix M and S
!   Subdivide M by dt
!   Build right-hand side f without the effect of boundary conditions
!      i.e. the part fi
!   help := M un, i.e. Mii ui + Mir ur
!   help := help + f
!   M_help = M + S
!   help1 := M_helpir ur (n+1)
!   help  := help - help1
!   Solve M_help isol = help
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimeulimpl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimeulimpl'
         write(irefwr,1) 'iseqdt, ieqn', iseqdt, ieqn
         write(irefwr,2) 'dt', dt
         call prinin ( isolprec, 5, 'isolprec' )
         call sepprintv ( isolprec, 'uold' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Build mass matrix, stiffness matrix and right-hand side for
!         one equation and one time level

      call septimbuildmat ( isol, irhsdlc, isolut2dred, first, ieqn )
      if ( ierror/=0 ) go to 1000

      if ( Teq(ieqn)%iseqaddrhs>0 ) then

!     --- iintlc>0 Add extra vector to right-hand side

         rhsdsp => ks(isolut2dred(Teq(ieqn)%iseqaddrhs))%sol
         rhsd => ks(irhsdlc)%sol
         rhsd = rhsd + rhsdsp

      end if  ! ( iintlc(10)>0 )

!     --- Perform one time step for an Euler implicit discretization

      call septimeulstep ( first, isol, isolut2dred, irhsdlc, &
                           isolprec, ieqn )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimeulimpl'

      end if  ! ( debug )
      call erclos ( 'septimeulimpl' )

      end subroutine septimeulimpl
