      subroutine septimeutrss
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine for the time integration of the discretized partial
!    differential equation
!              .
!          M * u + S * u = F
!              -       -   -
!    It has been assumed that the matrices M and S are not dependent
!    of the solution vector u . Hence the system is linear . M and
!    S may of course depend on place and time .
!    Solution method used is a combination of the Implicit Euler
!    method and the Trapezoid rule . The stepsize is selected by
!    the routine itself and saved.
!     Old subroutine septimeutrss
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematstr
      use sepmodulemain
      use sepmodulematr
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      use sepmodulesolve
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ihelp, iresul, istar, isol, irhold, &
              mrhsd, ifail, mdiag, ipoint, idum(1), idummy(1)
      double precision eps, p, q, difmax, told, tnew, alpha2, &
                       beta2, tstep2, epscon, tstepo, a, b
      logical first
      save ihelp, iresul, istar, isol, irhold, mrhsd, tstepo
      double precision, pointer :: STIFF(:), TOTM(:), MASS(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     MASS           Refers to mass matrix
!     STIFF          Refers to stifness matrix
!     TOTM           Refers to combination of mass and stiffness matrix
!     a
!     alpha2  ?
!     b
!     beta2   ?
!     difmax  ?
!     eps            Local accuracy
!     epscon         Accuracy for conjugate gradient matrix
!     first          If true it concerns the first step
!     idum    General dummy variable
!     idummy  Dummy parameter
!     ifail   ?
!     ihelp   ?
!     ipoint
!     iresul  ?
!     irhold  ?
!     isol    Standard SEPRAN array containing information of the solution
!             array
!     istar   ?
!     mdiag   ?
!     mrhsd   ?
!     p       ?
!     q       ?
!     tnew    ?
!     told    ?
!     tstep2  ?
!     tstepo  Old time step, not adjusted for TOUT
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision sepnorm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     SEPALGEBR      algebraical operations on SEPRAN vectors
!     SEPCOPINPSOL   Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!                    into sepmodulesolve
!     SEPCOPYMT      Copy matrix
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPLINSOL      Body of linear solver
!     SEPMAVER       perform matrix vector multiplication
!     SEPNORM        Compute norm of vector
!     SEPRMMATRIX    Remove array with sequence number iseq from realm
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
!     SEPTIMSOLVE    Solve system of linear equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     454            ?
!     623   tout < t
!    1239   rintim(1) too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ihelp/0/,iresul/0/,istar/0/,isol/0/,irhold/0/, &
           mrhsd/0/
! ======================================================================
!
      call eropen ( 'septimeutrss' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimeutrss'
         write(irefwr,2) 't, t0, tstep, tout', t, t0, tstep, tout
         write(irefwr,2) 'epstim', epstim
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      intmat => intmatar(1)
      IM => intmt(intmat)

!     --- check input

      if ( tout<=t ) then
         call erreal ( t, 1 )
         call erreal ( tout, 2 )
         call errsub ( 623, 0, 2, 0 )
      end if
      if ( epstim<100d0*epsmac ) then
         call erreal ( epstim, 1 )
         call erreal ( 100d0*epsmac, 2 )
         call errsub ( 1239, 0, 2, 0 )
      end if
      p = 0d0
      q = 0d0
      ipoint = 0
      iprobtim = intmt(intmat)%iprob
      massmat => massmt(iprobtim)
      stiffmat => stiffm(iprobtim:maxprb)
      totmat => mattot(iprobtim)
      eps=10d0*epsmac
      epscon = max(eps, epstim*1d-3)
      first = .false.
      if ( t<=t0+0.1d0*tstep ) first = .true.

      if ( (t+tstep)>tout ) tstep=tout-t+epsmac
      rinsoltim(1) = epscon

!     --- no errors in input
!         copy isolut2dred in isol

      call sepcopyvc ( isolut2dred, isol )

!     --- compute eps

      eps=1d2*epsmac

!     --- set number of failures

      ifail=0

!     --- build matrices M , S and right hand side F

      call septimbuildmat ( isol, irhsd, isolut2dred, first, 1 )
      if ( ierror/=0 ) go to 1000
      STIFF => realmt(stiffmat(1))%S_ii
      MASS => realmt(massmat)%S_ii

      if ( realmt(massmat)%diagonal ) then
         mdiag=1
      else
         mdiag=2
      end if

!     --- put boundary conditions in istar

      call sepcopyvc ( isol, istar )

!     --- fill irhold

      call sepcopyvc ( irhsd, irhold )

!     --- determine stepsize tstep

      call sepmaver (stiffmat,isolut2dred,ihelp,5 )
      a=-1d0
      b=1d0
      call sepalgebr (3,0,ihelp,irhsd,iresul, &
                      a,b,p,q,ipoint)
      call sepcopymt ( massmat, totmat )
      call septimsolve ( totmat, isol, iresul, 1, irhsd, first )
      difmax = sepnorm ( 1,3,0,isol,idum, idummy)

      if ( debug ) write(irefwr,2) 'difmax', difmax
      call sepcopyvc ( istar,isol )
      if ( abs( t - t0 )<eps ) then

!     --- initial situation

         if ( difmax>eps ) then
            tstep=sqrt(epstim/(4d0*difmax))
         else
            tstep=tout-t+epsmac
         end if
      else
         tstep = tstepo
         if ( t + tstep>tout ) tstep = tout - t + epsmac
      end if
      iflag=2

      if ( debug ) then
         write(irefwr,1) 'mdiag', mdiag
         write(irefwr,2) 'tstep, eps, epscon', tstep, eps, epscon
      end if  ! ( debug )

100   told = t
      tnew = t + tstep

      if ( debug ) &
         write(irefwr,2) 't, told, tnew, tstep', t, told, tnew, tstep

!     --- build matrices M , S and vector F on new time level

      t=tnew
      call septimbuildmat ( isol, irhsd, isolut2dred, first, 1 )
      STIFF => realmt(stiffmat(1))%S_ii

!     --- put boundary conditions in istar

      call sepcopyvc ( isol,istar )

!     --- determine first estimate ( ustar )
!         compute  :  M * u(old)

      if ( mdiag==1 ) then
           call sepmaver (massmat,isolut2dred,iresul, 4 )
      else
           call sepmaver (massmat,isolut2dred,iresul, 5 )
      end if

!     --- add      :  M * u(old) + dt * F(new)

      a=1d0
      call sepalgebr (3,0,iresul,irhsd,ihelp, &
                      a, tstep,p,q,ipoint)

!     --- create   :  M + (dt * S)

      call sepcopymt ( stiffmat, totmat )
      TOTM => realmt(totmat)%S_ii
      TOTM = tstep*TOTM
      if ( realmt(massmat)%diagonal ) then
         TOTM(IM%diag) = TOTM(IM%diag)+MASS
      else
         TOTM = TOTM+MASS
      end if  ! ( realmt(massmat(1))%diagonal )
      ipos = 0

!     --- create and subtract M+dt*S*u(new)boundary

      call sepmaver (totmat,istar,mrhsd,6 )
      a=1d0
      b=-1d0
      call sepalgebr (3,0,ihelp,mrhsd,iresul, &
                      a, b,p,q,ipoint)

!     --- solve system to obtain ustar

      ireadinpsol = -1
      iseqsol = 1
      iseqreac = 0
      iseqintmat = 1
      keepmat = 0
      iseqsolut = Teq(1)%iseqfr
      iseqmt = totmat
      call sepcopinpsol ( inpsoltim, rinsoltim )
      call seplinsol ( totmat, istar, iresul )

!     --- compute second estimate
!         compute  :  (M -  dt/2.* S) * u(old)

      call sepcopymt ( stiffmat, totmat )
      tstep2=tstep/2d0
      TOTM = -tstep2*STIFF
      if ( realmt(massmat)%diagonal ) then
         TOTM(IM%diag) = TOTM(IM%diag)+MASS
      else
         TOTM = TOTM+MASS
      end if  ! ( realmt(massmat)%diagonal )
      call sepmaver (totmat,isolut2dred,iresul,5 )

!     --- add      :  (M - dt/2. * S) * u(old) + dt/2. * F(old)

      a=1d0
      call sepalgebr (3,0,iresul,irhold,ihelp, &
                      a, tstep2,p,q,ipoint)

!     --- add      :  dt/2. * F(new)

      call sepalgebr (3,0,ihelp,irhsd,iresul, &
                      a, tstep2,p,q,ipoint)

!     --- create   :  M + (dt/2. * S)
      call sepcopymt ( stiffmat, totmat )
      TOTM = tstep2*STIFF
      if ( realmt(massmat)%diagonal ) then
         TOTM(IM%diag) = TOTM(IM%diag)+MASS
      else
         TOTM = TOTM+MASS
      end if  ! ( realmt(massmat)%diagonal )

!     --- compute and subtract (M+dt/2.*S)*(u(new)boundary)

      call sepmaver (totmat,isol,ihelp,6 )
      a=1d0
      b=-1d0
      call sepalgebr (3,0,iresul,ihelp,mrhsd, &
                      a, b,p,q,ipoint)

!     --- solve system to obtain usol

      call sepcopinpsol ( inpsoltim, rinsoltim )
      call seplinsol ( totmat, isol, mrhsd )

!     --- determine new stepsize

      difmax = sepnorm ( 0,3,0,istar,isol,idummy)
      if ( debug ) write(irefwr,2) 'difmax', difmax
      alpha=difmax/epstim
      alpha=min(100d0,alpha)
      alpha=max(0.01d0,alpha)

!     --- determine new step dt

      dt=tstep/sqrt(alpha)

      if ( debug ) &
         write(irefwr,2) 'dt, tstep, alpha', dt, tstep, alpha

      if ( alpha<1.0d0 ) then

!     --- accept result

           tstep  = 0.9d0*dt
           call sepcopyvc ( isol,isolut2dred )
           call sepcopyvc ( irhsd,irhold )
           iflag=2
           ifail=0
      else

!     --- result can not be accepted , recompute with smaller step

           tstep=0.5d0*dt
           t=told
           if ( tstep<eps ) tstep=eps
           ifail=ifail+1
           iflag=4
           if ( ifail>5 ) then
              call erreal ( t, 1 )
              call errsub ( 454, 0, 1, 0 )
              go to 1000
           end if
      end if

      if ( debug) &
         write(irefwr,2) 't, tout, tstep, tend/2', t, tout, tstep, tend

      if ( t>=tout-1d-4*tstep ) then

         iflag=3
         call sepcopyvc ( isol,isolut2dred )
         if(t>=tend-0.1*tstep) then

!        --- t = tend   reset all arrays

            call seprmsolinfo ( ihelp  )
            call seprmsolinfo ( iresul  )
            call seprmsolinfo ( istar  )
            call seprmsolinfo ( isol  )
            call seprmsolinfo ( irhold  )
            call seprmmatrix ( massmat )
            call seprmmatrix ( stiffmat )
            call seprmmatrix ( totmat )
            call seprmsolinfo ( irhsd  )
            call seprmsolinfo ( mrhsd  )
            itimloop = 0
         end if
         go to 1000

      end if

!     --- Save old stepsize

      tstepo = tstep
      if ( t+tstep>tout ) tstep=tout-t+eps
      goto 100

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 't, tout, tstep, tend', t, tout, tstep, tend
         write(irefwr,*) 'End septimeutrss'

      end if  ! ( debug )
      call erclos ( 'septimeutrss' )

      end subroutine septimeutrss

