      subroutine septimsecondst ( isol, first, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one step in the solution of the wave equation
!     Old subroutine prtm00_11
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), ieqn
      logical first

!     first         i/o   If true it concerns the first step
!     ieqn           i    Equation sequence number
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichois, iwork

!     ichois         Choice parameter for subroutine MAVER
!     iwork          Standard SEPRAN array of length 5 containing information
!                    concerning a work vector which must of the type of the
!                    solution vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIMULTADD2NBC Compute a1 u1 + a2 u2 and store in ures
!                    The part corresponding to boundary conditions is not filled
!     INIMULTADD3NBC Compute a1 u1 + a2 u2 + a3 u3 and store in ures
!                    The part corresponding to boundary conditions is not filled
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
!     SEPTIMSOLVE    Solve system of linear equations
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The standard central difference scheme reads:
!
!          n+1    n    n-1
!         u   - 2u  + u           n      n
!      M ---------------- +  S u    = f
!            dt**2
!              n+1   -1  2          n        n    n-1
!      Hence  u   = M  dt  ( f - S u  ) + 2 u  - u
!
!      This methods starts with u^0 and du/dt^0 in two consecutive vectors
!      In the first step du/dt^0 is replaced by u^-1 = u^0 - dt*du/dt^0
!      In the next steps we store u^n and u^n-1 in the vectors
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimsecondst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimsecondst'
         call sepprintv ( isol(1), 'u^n' )
         write(irefwr,2) 't', t
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Build matrices at old time level

      t = t-tstep  ! decrease t to use the old time level
      call septimuser ( first, ieqn )

      call septimbuildmat ( isol, irhsd, isolut2dred, first, ieqn )
      t = t+tstep  ! reset t
      if ( ierror/=0 ) go to 1000

!     --- Compute S u^n,  store in iwork (including boundary conditions)

      ichois = 5
      iwork = 0
      call sepmaver ( stiffmat, isolut2dred(iseqfrtim), &
                      iwork, ichois )
      if ( debug ) call sepprintv ( iwork, 'S u^n' )

!     --- Store dt^2 (f - S u^n)  into iwork

      call inimultadd2nbc ( irhsd, iwork, tstep**2, &
                            -tstep**2, iwork )
      if ( debug ) &
         call sepprintv ( iwork, 'dt^2 (f - S u^n)' )

!     --- Multiply by inverse of mass matrix and store in isol

      call septimsolve ( massmat, isol, iwork, ieqn, irhsd, first )
      if ( debug ) &
         call sepprintv ( isol, 'dt^2 M^-1( f^n- Su^n)' )

!     --- Compute u^n+1= dt^2 M^-1( f^n- Su^n) + 2u^n- u^n-1
!         Store in isol

      call  inimultadd3nbc ( isol, isolut2dred(iseqfrtim), &
                             isolut2dred(iseqfrtim+1), 1d0, 2d0, -1d0, &
                             isol )
      if ( debug ) call sepprintv ( isol, 'u^n+1' )

!     --- Remove temporary arrays

      call seprmsolinfo ( iwork )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimsecondst'

      end if  ! ( debug )
      call erclos ( 'septimsecondst' )

      end subroutine septimsecondst
