      subroutine septimtheta ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Time integration of a discretized partial differential equation
!     of first order in time with an implicit modified theta method
!              .
!          M * u + S * u = F
!              -       -   -
!      It has been assumed that the matrices M and S are not dependent
!      of the solution vector u. Hence the system is linear. M and
!      S may of course depend on place and time.
!      Solution method used is the modified theta method. The user
!      has to give the stepsize tstep.
!     Old subroutine pr1105
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, target :: isol(maxprb+1)
      integer :: i, ieqn, isolprec(maxprb), itemp, &
                 inneritr, iinvec(6), iincopy(2), istart, &
                 iaction, istextra, ieqnmat
      double precision :: rinvec(2), tsteps
      character (len=25) :: text
      logical :: converged, finishtm, first, linear

!     converged      If true the non-linear iteration has been converged
!     finishtm       Indicates if the time loop has been finished (true)
!                    or not (false)
!     first          If true it concerns the first time step
!     i              Counting variable
!     iaction        Defines the type of action to be performed
!     ieqn           Equation sequence number
!     ieqnmat        Equation sequence number + icalltim-1
!     iincopy        User input array for subroutine COPYVEC
!     iinvec         User input array for subroutine MANVEC
!     inneritr       Sequence number of inner iteration in case of non-linear
!                    iteration per time-step
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     isolprec       Solution at preceding level
!     istart         First position used at a part
!     istextra       Start position of extra information in iintim
!     itemp          Help array to store a temporary vector
!     linear         If true the linear time-iteration is used, if false an
!                    iteration per time step is performed
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
!     text           text with respect to writing or plotting
!     tsteps         help variable to save the value of tstep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTHIS       Print time history
!     SEPCNVCHK      Check convergence of iteration
!     SEPCOPYVEC     copy Sepran array
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPMANVEC      Vector manipulation subroutine
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBOUNCOND Fill boundary conditions in array isol and check if the
!                    array corresponds to the correct problem
!     SEPTIMEULIMPL  Perform one step in an Euler implicit scheme
!                    for one equation
!     SEPTIMEXTRA    Perform extra actions in time step
!     SEPTIMIINBLD   Fill some parameters in iinbld and initialize
!                    some other parameters for time integration
!     SEPTIMRESET    reset all arrays with respect to time integration that
!                    are no longer used
!     SEPTIMTHETA1   Start the time integration for suboutine septimtheta
!     SEPTIMTHETA2   Set some parameters during a time loop
!     SEPTIMTHETAFRC Extrapolate the solution from old and intermediate time
!                    step to next time step in case of theta method
!     SEPTIMUSER     Fill arrays iuser and user for a specific
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!        .
!     M  u   +  S  u   =  f
!     Application of Euler
!     -Euler part (predictor):
!       n+o     n
!      u    -  u
!       j       j           n+o      n+o
!    M ----------- +  S  u     =  f          (n is actual time-level)
!          dt               j        j         (n+o is time at t+theta dt)
!    - Extrapolation step
!     n+1     1     n+o            n
!    u    = -----( u   - (1-theta)u )
!           theta
!    or. &
!                 n+o        n         n+o
!   ( M + dt S ) u     =  M u    + dt f
!                 j          j         j
!                             n                 n+1
!   M and S may depend on:   u    k>=j  and  u      k<j (Silechi approach)
!                             k                 k
!   If a stationarysoly solution is required, the process is stopped if
!       k+1    k      1-lambda               k+1
!   || u    - u  ||<-------- (eps_abs + ||u   || eps_rel)
!                      lambda
!                     k+1   k
!                 || u   - u ||
!   with lambda = ------------- &
!                     k   k-1
!                 || u - u ||
!   Check input
!   ifrac := 1
!   stationarysoly := false
!   While ( ifrac>1 or t<tout and not stationarysoly ) do
!      if ( nfrac=1 ) then
!         t := t + theta dt
!      else ( nfrac>1 )
!         t := t + theta_(ifrac+1) dt
!         theta := theta_(ifrac+1) / (theta_ifrac + theta_(ifrac+1) )
!      Copy isolut2dred into isol
!      For iprob := 1 (1) ncopeqtim do
!         Build matrices M(iprob) and S(iprob)
!         Store boundary conditions into usol(iprob)
!         Build right-hand side f without the effect of boundary conditions
!            i.e. the part fi
!         help := M un, i.e. Mii ui + Mir ur
!         help := help + dt f
!         if ( icons=0 or t=t0+dt ) then
!            M_help = M + dt S
!         help1 := M_helpir ur (n+1)
!         help  := help - help1
!         Solve M_help isol(iprob) = help
!         Copy isol(iprob) into isolut2dred(iprob)
!      If ( extrapolation required ) then
!         extrapolate solution
!         if ( nfrac=1 ) then
!            t := t + (1-theta) dt
!         else
!            t := t + theta_ifrac dt
!            ifrac := modulo(ifrac+2,nfrac)
!      If ( stationarysoly solution required ) then
!         estimate lambda
!         check stopping criterion
!   If ( t>=tend ) then
!      Delete all arrays created
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimtheta' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimtheta'
         write(irefwr,2) 't, t0, tstep, tout', t, t0, tstep, tout
         write(irefwr,2) 'tend, toutend', tend, toutend
         write(irefwr,*) 'complrun', complrun
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays

      isol = 0
      isolprec = 0
      isol = 0
      itemp = 0

      iinvec = 0

!     --- Start of process
!         Set parameters

      tsteps = tstep
      text = 'solution at iteration '
      call septimtheta1 ( finishtm, first, linear )
      if ( ierror/=0 ) go to 1000
      if ( debug ) &
         write(irefwr,1) 'ncopeqtim, nfrac, nitertim', &
                          ncopeqtim, nfrac, nitertim

!     --- Initialize array isol

      do i = 1, ncopeqtim

!     --- First the vector is copied from isolut2dred

         call sepcopyvec ( isolut2dred(Teq(i)%iseqfr), isol(i), 0 )

!        --- Next it is transformed from Cartesian coordinates to new
!            coordinates in case of local transformations

         call seploctrn ( 1, isol(i) )

      end do
      if ( ierror/=0 ) go to 1000

!     --- While loop over time-steps

      do while ( .not. finishtm )

!     --- t<tout
!         Copy solution at old time level into isolprec

         if ( debug ) write(irefwr,2) 't, tout', t, tout
         do i = 1, ncopeqtim
            call sepcopyvec ( isolut2dred(Teq(i)%iseqfr), isolprec(i), 0 )

!           --- Next it is transformed from Cartesian coordinates to new
!               coordinates in case of local transformations

            call seploctrn ( 1, isolprec(i) )
            if ( ierror/=0 ) go to 1000

!           --- set some parameters, which are used in the time loop

            if ( debug ) write(irefwr,2) 't before septimtheta2', t
            call septimtheta2 ( tsteps, i )
            if ( debug ) write(irefwr,2) 't after septimtheta2', t

         end do
         if ( ierror/=0 ) go to 1000

!        --- Put boundary conditions into isol

         if ( itimbc<2 .and. (first .or. itimbc==0) ) then

!        --- Either first call, or boundary conditions are time-dependent
!            Mark that for the ncopeqtim problems only one
!            call to presbc is used
!            The boundary conditions are always given in the transformed for.
!            They must be filled on the new time level

            if ( debug ) write(irefwr,2) 't before bc', t
            isol2d => isol
            call septimbouncond ( first )

         end if  ! ( itimbc<2 .and. (first .or. itimbc==0) )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,2) 't after bcs', t

         iseqdt = 0
         istextra = 0
         inneritr = 0
         converged = .false.
         do while ( .not. converged .and. inneritr<maxitertim )

!        --- While loop for non-linear iteration
!            Loop over all problems

            inneritr = inneritr+1
            if ( debug ) write(irefwr,*) 'inneritr, converged, ncopeqtim ', &
               inneritr, converged, ncopeqtim

            do ieqn = 1, ncopeqtim

!           --- First set the correct parameters in iinbld

               ieqnmat = ieqn+icalltim-1
               call septimiinbld ( first, ieqn )
               if ( ierror/=0 ) go to 1000

!              --- Compute starting addresses for the arrays iuser and user

               intmat => intmatar(ieqn)
               iprobtim = intmt(intmat)%iprob
               massmat => massmt(ieqnmat)
               stiffmat => stiffm(ieqnmat:maxprb)
               totmat => mattot(ieqnmat)
               if ( ifilltim/=1 ) call septimuser ( first, ieqn )
               if ( ierror/=0 ) go to 1000

               if ( Teq(ieqn)%limit==1 ) then

!              --- limit = 1, compute minimum and maximum value of isol

                  iinvec(1) = 2
                  iinvec(2) = 3
                  call sepmanvec ( iinvec, rinvec, isol(ieqn), isol, isol )

               end if
               if ( ierror/=0 ) go to 1000
               if ( inneritr>1 ) then

!              --- Non-linear iteration, not the first step
!                  Transform to local coordinates if necessary

                  call seploctrn ( 1, isol(ieqn) )

               end if
               if ( ierror/=0 ) go to 1000

!              --- Perform one step Euler implicit

               if ( debug ) write(irefwr,2) 't before septimeulimpl', t

               call septimeulimpl ( first, isol, irhsd(ieqnmat),  &
                                    isolprec(ieqn), ieqn )
               if ( ierror/=0 ) go to 1000

               if ( debug ) then
                  write(irefwr,2) 't after septimeulimpl', t
                  write(irefwr,1) 'ieqn', ieqn
                  call sepprintv ( isol(ieqn), 'solution' )
               end if  ! ( debug )
               if ( ierror/=0 ) go to 1000

               if ( Teq(ieqn)%limit==1 ) then

!              --- limit = 1, limit isol to minimum and maximum values

                  iinvec(1) = 2
                  iinvec(2) = 42
                  call sepmanvec ( iinvec, rinvec, isol(ieqn), isol, isol )

               end if
               if ( ierror/=0 ) go to 1000

!              --- copy isol into isolut2dred
!                  First save isolut2dred in itemp and then copy itemp into isol
!                  In this way isol contains the solution at the old level
!                  and isolut2dred at the new level

               itemp = 0
               call sepcopyvec( isolut2dred(Teq(ieqn)%iseqfr), itemp, 0 )
               call sepcopyvec( isol(ieqn), &
                                isolut2dred(Teq(ieqn)%iseqfr), 0 )
               iincopy(1) = 2
               iincopy(2) = 1
               call sepcopyvec( itemp, isol(ieqn), iincopy )
               call seprmsolinfo ( itemp )

               if ( Teq(ieqn)%lenextra>0 ) then

!               --- Extra actions must be performed

                  call septimextra ( ieqn, first )

               end if  ! ( Teq(ieqn)%lenextra>0 )
               if ( ierror/=0 ) go to 1000

!              --- end of loop over the equations

            end do

            if ( .not. linear ) then

!           --- Non-linear iteration, check on convergence

               call sepcnvchk ( isol, inneritr, converged, text )

            else

!           --- Linear process, no repeating

               converged = .true.

            end if

!           --- end while loop

         end do
         if ( ierror/=0 ) go to 1000

         if ( nfrac>1 .or. abs(1d0-theta)>100d0*epsmac ) then

!        --- extrapolate to final time

            call septimthetafrc ( isolut2dred, isolprec )

         end if  ! ( nfrac>1 .or. abs(1d0-theta)>100d0*epsmac )

         if ( nhist>0 ) then

!        --- print time history

            call printhis ( nhist, nodhist, t, ncopeqtim, &
                            isolut2dred(Teq(1)%iseqfr), first )

         end if
         if ( ifrac==1 .and. stationarysol ) then

!        --- stationary solution required, compute stopping criterion

            call sepcnvchk ( isolprec, nitertim, finishtm, text )
            if ( finishtm ) t = tend

         end if
         if ( ifrac==1 .and. t>=tout-0.0001d0*tstep .or. &
              .not. complrun ) finishtm = .true.
         first = .false.
         if ( debug ) write(irefwr,2) 't, tout', t, tout

      end do

!     --- While loop has been finished, reset quantities

      tstep = tsteps
      iflag = 3

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call septimreset ( irhsd, isol, isolprec, finished )

      end if  ! ( t>=tend-0.00001*tstep )

!     --- reset arrays isol and isolprec

      do i = maxprb+1, 1, -1
         if ( i/=maxprb+1 ) then
            if ( isolprec(i)/=0 ) &
               call seprmsolinfo ( isolprec(i) )
         end if  ! ( i/=maxprb+1 )
         if ( isol(i)/=0 ) call seprmsolinfo ( isol(i) )
      end do  ! i = 1, maxprb

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimtheta'

      end if  ! ( debug )
      call erclos ( 'septimtheta' )

      end subroutine septimtheta
