      subroutine toaddrhspar1 ( work, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add parts of right-hand-side at interface to lower blocks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     right_hand_side
!     real
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
  !    use sepmodulevecs
      use sepmodulebuild
      use sepmodulekprob
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(inout) :: work(*), rhsd(nusol)

!     rhsd          i/o   Right-hand-side vector
!     work          i/o   Work array to be used for transport
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numlow, numupp, icontext, iblock, ilow, &
                 ihigh, ninterf, i
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_s

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     ninterf        Number of unknowns in interf (interface)
!     numlow         Number of neighbor blocks with lower number
!     numupp         Number of neighbor blocks with higher number
!     send_buffer    Buffer for sending messages
!     send_buffer_s  Length of array send_buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddrhspar1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddrhspar1'
         call prinrl ( rhsd, nrusol, 'rhsd' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes,
!         before reading by those other nodes has been done

      call inibarriermpi

      numlow = blockinfor(1)  ! number of neighbors with smaller number
      numupp = blockinfor(2)  ! number of neighbors with larger number

!     --- Compute length of send_buffer

      send_buffer_s = 0

      length = 1

      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)+1   ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)+1 ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block
         length = max(1,2*ninterf)
         send_buffer_s = send_buffer_s+length

      end do  ! i = 1, numlow

      call inichkarlen ( length, 2*nusol, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Multiply by 8 (doubles) and add 100 for safety

      send_buffer_s = send_buffer_s*8+100

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_s, max(2,numlow) )

      allocate(send_buffer(send_buffer_s))

      call inibufattmpi ( send_buffer, send_buffer_s )
      if ( ierror/=0 ) go to 1000

!     --- Send part of rhsd to blocks with smaller number

      icontext = 55
      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)+1   ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)+1 ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block

         if ( ninterf>0 ) then

            if ( debug ) &
               write(irefwr,1) 'i, iblock, ilow, ihigh, ninterf', &
                                i, iblock, ilow, ihigh, ninterf

!           --- in iwork we store the sequence numbers as used in rhsd

            iwork(1:ninterf) = kprobhinv(ilow+nrusol:ihigh+nrusol-1)

            if ( debug ) call prinin ( iwork, ninterf, 'iwork' )

            work(ninterf+1:2*ninterf) = rhsd(iwork(1:ninterf))

!           --- Translate iwork to global numbers

            iwork(1:ninterf) = kprobhglob(ilow+nrusol:ihigh+nrusol-1)

            if ( debug ) call prinin ( iwork, ninterf, 'iwork' )

            work(1:ninterf) = iwork(1:ninterf)

            if ( debug ) &
               call prinrl ( work(ninterf+1), ninterf, 'work' )

!           --- Send work to iblock

            call inibsendmpi ( work, 2*ninterf, SPDOUBLE, iblock, &
                               icontext )

         else

!        --- Send 0 to iblock

            work(1) = 0
            call inibsendmpi ( work, 1, SPDOUBLE, iblock, icontext )

         end if  ! ( ninterf>0 )

      end do  ! i = 1, numlow

      call inibarriermpi

!     --- Receive part of rhsd from blocks with larger number

      do i = 1, numupp

         length = 2*nusol
         call inireceivempi ( work, length, SPDOUBLE, iblock, icontext )

         if ( length>1 ) then

!        --- Length>1, hence information

            ninterf = length/2
            iwork(1:ninterf) = nint(work(1:ninterf))  ! global numbers

            if ( debug ) then
               write(irefwr,1) 'iblock, ninterf', iblock, ninterf
               call prinin ( iwork, ninterf, 'iwork' )
            end if  ! ( debug )

!           --- Translate global numbers to local numbers

            iwork(1:ninterf) = kprobhglobinv(iwork(1:ninterf))

            if ( debug ) call prinin ( iwork, ninterf, 'iwork' )

!           --- in iwork we store the sequence numbers as used in rhsd

            iwork(1:ninterf) = kprobhinv(iwork(1:ninterf))

            if ( debug ) call prinin ( iwork, ninterf, 'iwork' )

            rhsd(iwork(1:ninterf)) = rhsd(iwork(1:ninterf))+ &
               work(ninterf+1:2*ninterf)

         end if  ! ( length>1 )

      end do  ! i = 1, numupp

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_s )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      call inibarriermpi

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( rhsd, nrusol, 'rhsd' )
         write(irefwr,*) 'End toaddrhspar1'

      end if  ! ( debug )
      call erclos ( 'toaddrhspar1' )

      end subroutine toaddrhspar1
