      subroutine toassemblemat ( sym )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 24-07-2015 Use sepmodulebuild
!        version  5.0  date 21-04-2015 New parameter list
!        version  4.1  date 11-04-2015 Remove ibcmat
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble one element matrix in the large matrix
!     real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: sym

!     sym            i    Logical indicating if the element matrix is
!                         symmetrical (true) or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idown, ihigh, icnt, nsum, i
      logical :: shiftedlapl, found
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     found          If true already a possibility has been found
!     i              Counting variable
!     icnt           Number of non-prescribed unknowns in element
!     idown          Lowest position number in USOL with respect to "right"
!                    periodical boundary conditions
!                    idown = nusol-nbound-npboun+1
!     ihigh          Highest position number in USOL with respect to
!                    "right" periodical boundary conditions
!                    ihigh = nusol-nbound
!     nsum           nrusol+ninterface+1, i.e. position of first prescribed
!                    unknown
!     shiftedlapl    If true a shifted laplace preconditioning is applied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
!     TO0207         add element matrix to large matrix part 2, 3, 4
!     TOADDBLOCKMAT  Add block matrices in case of block preconditioners for the
!                    Navier-Stokes equations
!     TOADDMATFI     Assemble one element matrix in the matrix part S_fi
!     TOADDMATIF     Assemble one element matrix in the matrix part S_if
!     TOADDMATSHIFTL Add diagonal element matrix to extra diagonal of matrix
!     TOADDPETSC     Add part of element matrix corresponding to real unknowns
!                    to petsc matrux
!     TOADDROWCOMP   Assemble one element matrix in the large matrix
!                    real unsymmetric matrix with compact row storage
!     TOST01         Add contributions of periodical boundary conditions to
!                    second part of matrix in the case of real arithmetic
!     TOST05         Multiply element matrix and vector by constant for "right"
!                    periodical boundary condition
!     TOST06         Divide element matrix by constant for "right"
!                    periodical boundary condition
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3064   Shifted Laplace preconditioning not available for type number $
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toassemblemat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toassemblemat'
         write(irefwr,1) 'ielem, icount', ielem, icount
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( debug ) &
         write(irefwr,1) 'nrusol, nbound, npboun', &
                          nrusol, nbound, npboun
      shiftedlapl = IM%lapshift

      if ( npboun>0 .and. (matrix .or. vector) ) then

!     --- npboun>0, multiply the rows and columns of the element matrix
!                   and element vector by the multiplication factor

         idown = nusol-nbound-npboun+1
         ihigh = nusol-nbound
         call tost05 ( idown, ihigh, elemmt1, elemvc1 )
         if ( idefcr==1 )  call tost05 ( idown, ihigh, elemmt2, elemvc2  )

      end if  ! ( npboun>0 .and. (matrix .or. vector) )

      if ( stiffmat(1)>0 ) Smat => realmt(stiffmat(1))


      if ( matrix ) then

!     --- Fill matrix

         if ( IM%simple ) then

!        --- special simple structure
!            Build the matrices Q and D separately

            call toaddblockmat ( elemmt, icnt )

         else

!        --- Standard cases

            if ( IM%nosepmat ) then

!           --- building of matrix by petc

               call toaddpetsc ( elemmt, icnt )

            else

!           --- real compact row storage

               call toaddrowcomp ( elemmt1, icnt )

               if ( shiftedlapl .and. ielgrp<=nelgrp ) then

!              --- shiftedlapl = true, store extra diagonal matrix

                  if ( itype>99 .and. itype/=800 .and. itype/=705 ) then

!                 --- Type number not available

                     call errint ( itype, 1 )
                     call errsub ( 3064, 1, 0, 0 )
                     go to 1000

                  end if  ! ( itype/=800 )

                  call toaddmatshiftl ( elemmt2 )

               end if  ! ( shiftedlapl )

            end if  ! ( IM%nosepmat )

         end if  ! ( IM%simple )

!        --- add element matrix to second part of matrix

         if ( debug ) write(irefwr,1) 'icnt, icount', icnt, icount
         if ( icnt<icount ) then

            nrusol = nusol-nbound
            call to0207 ( elemmt1 )
            if ( idefcr==1 ) then
               Smat => realmt(stiffmat(2))
               call to0207 ( elemmt2 )
               Smat => realmt(stiffmat(1))
            end if  ! ( idefcr==1 )
            nrusol = IM%nrusol

         end if  ! ( icnt<icount )

         if ( debug ) write(irefwr,*) 'after to0207'

         if ( npboun>0 ) then

!        --- npboun>0, fill part of second part of large matrix
!            corresponding to periodical boundary conditions
!            First divide the columns of the element matrix by the
!            multiplication factor

            call tost06 ( idown, ihigh, elemmt1 )
            call tost01 ( idown, ihigh, elemmt1 )
            if ( idefcr==1 ) then

               call tost06 ( idown, ihigh, elemmt2 )
               Smat => realmt(stiffmat(2))
               call tost01 ( idown, ihigh, elemmt2 )
               Smat => realmt(stiffmat(1))

            end if  ! ( idefcr==1 )

         end if  ! ( npboun>0 )

         if ( icnt<icount .and. ninterface>0 .and. .not. IM%nosepmat ) then

!        --- Not all unknowns are internal and there are interface unknowns
!            First we check if there are interface unknowns in the element

            nsum = nrusol+ninterface+1
            found = .false.
            do i = 1, icount
               if ( indexren(i)>nrusol .and. indexren(i)<nsum ) then
                  found = .true.
                  exit
               end if  ! ( indexren(i)>nrusol .and. indexren(i)<nsum )
            end do  ! i = 1, icount

            if ( found ) then

!           --- ipicolsfi>0: Fill matrix S_fi and S_if

               call toaddmatfi ( elemmt )
               call toaddmatif ( elemmt )

            end if  ! ( ninterface>0 )

         end if  ! ( icnt<icount .and. ninterface>0 )

      end if  ! ( matrix .and. icnt>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemmt, icount, icount, 'elemmt' )
         write(irefwr,*) 'End toassemblemat'

      end if  ! ( debug )
      call erclos ( 'toassemblemat' )
      end
