      subroutine tocopymatfi
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.0  date 18-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send and receive matrices S_fi to/from other blocks
!     Add matrices S_if and S_if to S_iout and S_ii
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenstore, lenwork, i
      integer, allocatable :: iwork1(:), iwork2(:)
      double precision, allocatable :: work(:), storeinfo1(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     iwork1         integer work array to address column numbers
!     iwork2         integer work array to address column numbers
!     lenstore       Declared length of array istore
!     lenwork        Declared length of array work
!     storeinfo1     Array to store all information that is received
!                    from other blocks
!                    Comparable to istoreinfo in subroutine incommatpar3
!     work           Work array for transportation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     TOADDSIF       Add matrix S_if to matrix S_iout
!     TOADDSIOUT     Add information stored in storeinfo to matrices S_ii
!                    and S_iout
!     TOADDSIOUT55   Add information stored in storeinfo to matrices S_ii and
!                    S_iout. Row compact storage (petsc)
!     TOSENDRECFI    Send and receive matrices S_fi to/from neighbor blocks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tocopymatfi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tocopymatfi'
         write(irefwr,1) 'nrusol, nusolglob', nrusol, nusolglob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      Smat => realmt(stiffmat(1))
      S_if => Smat%S_if
      S_fi => Smat%S_fi
      S_iout => Smat%S_iout
      S_ii => Smat%S_ii

!     --- Get parameters

      lenstore = size(IM%storeinfo)           ! length of array istoreinfo1
      lenwork = max(lenstore,IM%nusol, size(IM%colsfi) )

      if ( allocated(kprobhglobinv) ) deallocate(kprobhglobinv)

      allocate ( iwork(nusolglob), work(lenwork), storeinfo1(lenstore), &
                 kprobhglobinv(nusolglob), iwork1(nrusol), &
                 iwork2(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nusolglob, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array kprobhglobinv with inverse of array kprobhglob

      kprobhglobinv = 0
      do i = 1, nrusol
         kprobhglobinv(kprobhglob(i)) = i
      end do  ! i = 1, nrusol

      if ( size(IM%rowsif)>1 ) then

!     --- Add matrix S_if to S_iout

         call toaddsif ( S_if, S_iout )

      end if  ! ( size(IM%rowsif)>1 )

!     --- Send and receive matrices S_fi to/from neighbor blocks
!         Store in array storeinfo1 with the same storage as in istoreinfo

      call tosendrecfi ( work, storeinfo1, lenwork, lenstore, S_fi )

!     --- Add information from storeinfo1 to S_ii and S_iout

      if ( .not. IM%symmetric ) then

!     --- petsc storage

         call toaddsiout55 ( storeinfo1, iwork1, nrusol, S_ii, S_iout )

      else

!     --- compact sepran symmetric storage

         call toaddsiout ( storeinfo1, iwork1, nrusol, iwork2 )

      end if  ! ( .not. IM%symmetric )

      if ( notmas==0 .and. imas==2 ) then

!     --- Perform the same actions for the mass matrix

         Smat => realmt(massmat)
         M_if => Smat%S_if
         M_fi => Smat%S_fi
         M_iout => Smat%S_iout

         if ( size(IM%rowsif)>1 ) then

!        --- Add matrix M_if to M_iout

            call toaddsif ( M_if, M_iout )

         end if  ! ( size(IM%rowsif)>1 )

!        --- Send and receive matrices S_fi to/from neighbor blocks
!            Store in array storeinfo1 with the same storage as in istoreinfo

         call tosendrecfi ( work, storeinfo1, lenwork, lenstore, M_fi )

!        --- Add information from storeinfo1 to M_ii and M_iout

         if ( IM%petsc ) then

!        --- petsc storage

            call toaddsiout55 ( storeinfo1, iwork1, nrusol, M_ii, M_iout )

         else

!        --- compact sepran symmetric storage

            call toaddsiout ( storeinfo1, iwork1, nrusol, iwork2 )

         end if  ! ( IM%petsc )

      end if  ! ( notmas==0 .and. imas==2 )

      deallocate ( iwork, work, storeinfo1, kprobhglobinv, iwork1, &
                   iwork2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tocopymatfi'

      end if  ! ( debug )
      call erclos ( 'tocopymatfi' )

      end subroutine tocopymatfi
