      subroutine tofillprmassmt ( jwork, nwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-07-2015 Extra parameter nwork
!        version  2.3  date 11-03-2013 Correction declaration
!        version  2.2  date 21-09-2011 Extra debug
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the Pressure mass matrix for the Schur complement algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmoduleelm
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nwork
      integer, intent(inout) :: jwork(nwork)

!     jwork         i/o   integer work array of size nu
!                         In jwork the sequence numbers of the columns of the
!                         non-zero elements in one row of the matrix are stored
!                         At input jwork must be 0
!                         At output jwork is reset to 0
!     nwork          i    Declared length of jwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, inusol, jnusol, iamat, knusol, itel, jtel, &
                 nusolmax, nusolmin

!     i              Counting variable
!     iamat          Position in array AMAT
!     inusol         Position of unknown in array USOL (row number)
!     itel           Counting variable
!     j              Counting variable
!     jnusol         Position of unknown in array USOL (column number)
!     jtel           Counting variable
!     knusol         Position of unknown in array USOL after subtraction
!                    of velocity unknowns
!     nusolmax       Maximum column number in a row
!                    In symmetrical case the row number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2899   The matrix element is not present in the reduced storage, &
!           but the element in the element matrix is not equal to 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) icount do
!        inusol := indexren(i)
!        if ( inusol>nu ) then
!           Fill jwork with sequence numbers
!           Raise S_PM
!           For j := 1 (1) icount do
!              if ( j # i ) then
!                 jnusol := indexren(j)
!                 if ( jnusol = inusol ) then
!                    Add element to S_PM
!                 else if ( jnusol>nu ) then
!                    Add element to S_PML
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofillprmassmt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofillprmassmt'
         write(irefwr,1) 'nrusolvel, nrusolpress, npsi, nrusol', &
                          nrusolvel, nrusolpress, npsi, nrusol
         call prinrl1 ( E%wpressmmat, npsi, npsi, 'E%wpressmmat' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
!call prinrl ( S_PM, 5, 'S_PM/1')

!     --- the matrix is symmetric
!         For i := 1 (1) icount do

      itel = 0
      do i = 1, icount

!     --- inusol := indexren(i)

         inusol = indexren(i)
         if ( debug ) write(irefwr,1) 'i, inusol', i, inusol
         if ( inusol>nrusolvel .and. inusol<=nrusol ) then

!        --- inusol>nrusolvel, i.e. row corresponds to pressure mass matrix
!            Fill jwork with sequence numbers
!            Raise S_PM

            knusol = inusol-nrusolvel
            do j = irowsPM(knusol)+1, irowsPM(knusol+1)
               jwork(icolsPM(j)) = j
            end do  ! j = irowsPM(inusol)+1, irowsPM(inusol+1)
            nusolmax = icolsPM(irowsPM(knusol+1))+nrusolvel
            itel = itel+1

            if ( debug ) then
               write(irefwr,1) 'icount, nusolmax', icount, nusolmax
               call prinin ( indexren, icount, 'indexren' )
            end if  ! ( debug )

            nusolmin = knusol+nrusolvel

!           --- For j := 1 (1) icount do

            jtel = 0
            do j = 1, icount

               jnusol = indexren(j)
               if ( jnusol>nrusolvel .and. jnusol<=nrusol ) then
                  jtel = jtel+1

                  if ( jnusol>=nusolmin .and. jnusol<=nusolmax ) then

!              --- Element inlower triangular matrix, including diagonal

                      knusol = jnusol-nrusolvel
                      iamat = jwork(knusol)
!write(irefwr,1) 'knusol, iamat, jtel, itel', knusol, iamat, jtel, itel

                      if ( iamat>0 ) then

!                     --- iamat>0, standard case

                         S_PM(iamat) = S_PM(iamat) + E%wpressmmat(jtel,itel)

                      else if ( abs(E%wpressmmat(jtel,itel))>0d0 ) then

!                     --- iamat=0, but element in element matrix not equal
!                                 to 0 This is an error

                         call errint ( i, 1 )
                         call errint ( j, 2 )
                         call erreal ( E%wpressmmat(jtel,itel), 1 )
                         call errsub ( 2899, 2, 1, 0 )

                      end if  ! ( iamat>0 )

                   end if  ! ( jnusol==inusol )

               end if  ! ( jnusol<=nusolmax )

            end do  ! j = 1, icount

!           --- Clear jwork

            knusol = inusol-nrusolvel
            do j = irowsPM(knusol)+1, irowsPM(knusol+1)
               jwork(icolsPM(j)) = 0
            end do  ! j = irowsPM(inusol)+1, irowsPM(inusol+1)

         end if  ! ( inusol>nrusolvel )

      end do   ! i = 1, icount
!call prinrl ( S_PM, 5, 'S_PM/e')

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Scaled pressure mass matrix'
         write(irefwr,*) 'End tofillprmassmt'
!call instop

      end if  ! ( debug )
      call erclos ( 'tofillprmassmt' )

      end
