      subroutine chtmc6 ( meshsplt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 28-08-2013 Remove work arrays form parameter list
!        version  2.5  date 28-09-2012 Jump in case of error
!        version  2.4  date 17-11-2008 Fortran 90
!
!   copyright (c) 1996-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if a mesh consists of sub parts and if so give a warning or error
!     The number of nodes per sub part is stored in meshsplt
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshsplt(*)

!     meshsplt   o    In this array the largest node number of the various
!                     disconnected parts of the mesh are stored
!                     Length: number of parts (<npoint)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, icount, mset
      integer, pointer :: level(:)

!     i              Counting variable
!     icount         number of unknowns per element
!     level          Work array of length npoint to store the levels
!     mset           number of nodal points in ineigh
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHTMC6ERR      Print extra information with respect to error 1654
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     552   Mesh consists of unconnected parts
!    1654   Sub parts are not numbered sequentially
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array marknodes
!    nsplit := 1
!    node := 1
!    icount := 1
!    While ( node<=npoint ) do
!       level(1) := node
!       nlevel := 1
!       While ( nlevel>0 ) do
!          Find all neighbours neigh of node with marknodes(neigh) = 0
!          Store these nodes in level
!          nlevel := number of neighbours stored in level
!          icount := icount+nlevel
!          Set marknodes equal to -1 for the nodes in level
!       If ( icount<npoint ) then
!          If ( marknodes(icount+1) ... marknodes(npoint) # 0 ) then
!             error: nodes in separate parts not number sequentially
!          meshsplt(nsplit) := node
!          nsplit := nsplit+1
!          node := icount
!    meshsplt(nsplit) := node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtmc6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtmc6'
  1      format ( a, 1x, (10i6) )

      end if

      allocate ( nextlevel(npoint), level(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'nextlevel' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array marknodes

      marknodes = 0
      nsplit = 1
      node = 1
      icount = 1

      do while ( node<=npoint )

!     --- node<=npoint

         level(1) = node
         if ( debug ) &
            write(irefwr,1) 'node, npoint, nsplit',node, npoint, nsplit
         nlevel = 1
         do while ( nlevel>0 )

!        --- nlevel>0
!            Find all neighbors neigh of node with marknodes(neigh) = 0

            lastlevel => level(1:nlevel)
            call sepnextlevel ( -1, mset )

!           --- Store these nodes in level
!               nlevel = number of neighbours stored in level
!               Set marknodes equal to 1 for the nodes in level

            nlevel = mset
            level(1:nlevel) = nextlevel(1:nlevel)
            marknodes(nextlevel(1:nlevel)) = -1

            if ( debug ) call prinin ( level, nlevel, 'level' )
            icount = icount+nlevel

         end do ! while ( nlevel>0 )

         if ( debug ) write(irefwr,1) 'icount, npoint', icount, npoint

         if ( icount<npoint ) then

!        --- icount<npoint
!            Check if only nodes in the range 1 to icount-1 are used

            meshsplt(nsplit) = icount-1
            do i = icount, npoint
               if ( marknodes(i)<0 ) then

!              --- error: nodes in separate parts are not numbered sequentially

                  call errint ( nsplit, 1 )
                  call errint ( icount, 2 )
                  call errint ( i, 3 )
                  call errsub ( 1654, 3, 0, 0 )
                  call chtmc6err ( marknodes, icount, npoint )
                  meshsplt(nsplit) = i
                  go to 1000
               end if
            end do
            nsplit = nsplit+1
         end if
         node = icount

      end do  ! while ( node<=npoint )

      meshsplt(nsplit) = node-1
      if ( nsplit>1 ) then
         call errint ( nsplit, 1 )
         do i = 1, nsplit
            call errint ( meshsplt(i), i+1 )
         end do
         call errwar ( 552, nsplit+1, 0, 0 )
      end if

1000  deallocate ( nextlevel, level, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nextlevel' )

      if ( debug ) write(irefwr,*) 'End chtmc6'
      call erclos ( 'chtmc6' )

      end
