      subroutine chtslo ( level, istart, lenstr, istmet, ifirst, meshsplt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.4  date 11-11-2014 Extra debug
!        version  6.3  date 13-02-2008 Extra debug
!        version  6.2  date 17-04-2006 Fortran 90
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute renumbering according to the Sloan algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer level(*), istart(*), lenstr, istmet, &
              ifirst, meshsplt(*)

!     ifirst    i/o   The nodes from ifirst must be renumbered
!     istart     i    Contains the start level at input
!     istmet     i    Indicates the method for computing the start for
!                     renumbering
!                     0:  default
!                     1:  user point
!                     2:  curve
!                     3:  surface
!     lenstr     i    Number of elements in istart
!     level           Work array to store a level (length npoint)
!     meshsplt   i   In this array the largest node number of the various
!                    disconnected parts of the mesh are stored
!                    Length: number of parts (<npoint)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, mset, isplit

!     i              Counting variable
!     isplit         Number of parts the mesh is subdivided into
!     mset           Number of points in renumbered set
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer sepstartnode

!     CHSLOA         Renumbering algorithm according to Sloan
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPSTARTNODE   Find start node in a set of nodes
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chtslo' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chtslo'
         write(irefwr,1) 'ifirst, lenstr', ifirst, lenstr
         call prinin ( istart, ifirst+lenstr-1, 'istart' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = ifirst, ifirst+lenstr-1
         kmeshj(i) = istart(i)
      end do

      if ( debug ) write(irefwr,*) 'before chsloa'
      call chsloa ( lenstr, level, mset, istmet, &
                    ifirst, meshsplt(1) )

      if ( debug ) then

!     --- debug statements

         write(irefwr,*) 'mset, npoint ',mset, npoint
         call prinin ( kmeshj, mset, 'kmeshj' )

      end if

      isplit = 1

      do while ( mset<npoint )

!     --- Not all nodes have been renumbered, which means that the
!         region consists of disjunct parts
!         The next part is renumbered

         isplit = isplit+1
         ifirst = mset+1
         do i = ifirst, meshsplt(isplit)
            kmeshj(i) = i
         end do  ! i = ifirst, ilast
         set => kmeshj(ifirst:meshsplt(isplit))
         kmeshj(ifirst) = sepstartnode()
         lenstr = 1

         if ( debug ) then

!        --- debug statements

            write(irefwr,*) 'mset, npoint ',mset, npoint
            write(irefwr,*) 'isplit, ifirst, meshsplt(isplit) ', &
                             isplit, ifirst, meshsplt(isplit)
            call prinin ( kmeshj(ifirst), lenstr, 'kmeshj-start' )

         end if

         call chsloa ( lenstr, level, mset, 0, &
                       ifirst, meshsplt(isplit) )

         if ( debug ) then

!        --- debug statements

            write(irefwr,*) 'lenstr, npoint ',lenstr, npoint
            call prinin ( kmeshj, npoint, 'kmeshj' )

         end if

      end do  ! while ( mset<npoint )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End chtslo'

      end if  ! ( debug )
      call erclos ( 'chtslo' )

      end
