      subroutine ercurvenodtan ( icurvs, tangent )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 22-10-2013 New parameter list
!        version  2.1  date 29-05-2006 Correction for empty curves
!        version  2.0  date 16-03-2005 Raplace icurnr by icurvs
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set iwork(i)>0 for all nodes in a (compound) curve
!     Store the corresponding tangential vectors in array tangent
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     curve
!     tangential_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*)
      double precision tangent(ndim,*)

!     icurvs         i    Input array for subroutine ercurvenodes
!                         icurvs(1) contains number of curves (1)
!                         icurvs(2...n+1) contains curve numbers
!     iwork         i/o   integer work array of length npoint
!                         if node i corresponds to curve iwork(i) gets the
!                         sequence number of the node in the curve and the
!                         tangential vector is stored in array
!                         tangent(.,iwork(i))
!     tangent        o    Contains the tangential vectors corresponding to
!                         icurnr, see iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, ipart, inode1, inode2, inode3, nparts
      integer, allocatable :: ipoint(:), inodes(:)
      double precision t1(ndim)
      logical closed

!     closed         If true the curve part is closed
!     inode1         Previous node
!     inode2         Present node
!     inode3         Next node
!     inodes         Work array to store nodal points
!     ipart          Sequence number of part
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1, &
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     ipstart        Starting address in array
!     nparts         number of parts the boundary consists of
!     t1             tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININNONZERO  Print the non-zero contents of an integer array
!     PRINRL1        Print 2d real vector
!     SEPCURVENODES  Store all nodes in one curves in array inodes and gaps in
!                    these curves in array ipoint
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store all nodes in array inodes and the gaps in array ipoint
!     ipstart := 0
!     For all separate parts in array inodes do
!        ipstart := ipstart+1;  inode1 := inodes(ipstart);
!        t1 := tangential vector in first node of part
!        tangent(ipstart) := t1
!        for i := 2 to number_of_nodes along part minus one do
!           ipstart := ipstart+1
!           inode1 := inodes(ipstart); inode2 := inodes(ipstart+1)
!           t2 := tangential vector from inode1 to inode;
!           tangent(ipstart) := mean value of t1 and t2
!           t1 := t2
!        ipstart := ipstart+1;  inode1 := inodes(ipstart)
!        tangent(ipstart) := t1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercurvenodtan' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercurvenodtan'
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )
         call prinin ( icurvs(2), icurvs(1), 'curves' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      allocate ( ipoint(ncurvs), inodes(kmeshmglob(3)), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs+kmeshmglob(3), 'ipoint' )
      if ( ierror/=0 ) go to 1000

!     --- Store all nodes in array inodes and the gaps in array ipoint

      nparts = 1
      ipstart = 0
      call sepcurvenodes ( icurvs, ipoint, inodes, nparts, kmeshmglob )

!     --- For all separate parts in array inodes do

      do ipart = 1, nparts

!     --- ipstart := ipstart+1;  inode1 := inodes(ipstart);
!         t1 := tangential vector in first node of part
!         tangent(ipstart) := t1

         ipstart = ipoint(ipart)
         inode1 = inodes(ipstart)
         if ( debug ) write(irefwr,100) 'ipart, ipstart, inode1', &
            ipart, ipstart, inode1
         iwork(inode1) = ipstart
         inode2 = inodes(ipstart+1)
         if ( inode1==inodes(ipoint(ipart+1)-1) ) then

!        --- First and last point of part are equal, hence closed curve
!            Set inode3 equal to last point-2

            inode3 = inodes(ipoint(ipart+1)-2)
            closed = .true.

         else

!        --- First and last point of part are not equal, hence open curve
!            Set inode3 equal to last inode1

            inode3 = inode1
            closed = .false.

         end if  ! ( inode1==inodes(ipoint(ipart+1)-1) )

         t1 = coorglob(:,inode2)-coorglob(:,inode3)
         t1 = t1/sqrt(dot_product(t1,t1))
         tangent(:,ipstart) = t1
         if ( debug ) then
            write(irefwr,100) 'inode1, inode2', inode1, inode2
            write(irefwr,110) 't1', t1
         end if  ! ( debug )

!        --- for i := 2 to number_of_nodes along part minus one do

         do ipstart = ipoint(ipart)+1, ipoint(ipart+1)-2

!        --- ipstart := ipstart+1
!            inode1 := inodes(ipstart); inode2 := inodes(ipstart+1)

            inode3 = inodes(ipstart+1)
            iwork(inode2) = ipstart

!           --- t2 := tangential vector from inode1 to inode3;
!               tangent(ipstart) := mean value of t1 and t2
!               t1 := t2

            t1 = coorglob(:,inode3)-coorglob(:,inode1)
            t1 = t1/sqrt(dot_product(t1,t1))
            tangent(:,ipstart) = t1

            if ( debug ) then
               write(irefwr,100) 'inode1, inode2, inode3', &
                                  inode1, inode2, inode3
               write(irefwr,110) 't1', t1
            end if  ! ( debug )
            inode1 = inode2
            inode2 = inode3

         end do  ! ipstart = ipoint(ipart)+1, ipoint(ipart+1)-1

!        --- ipstart := ipstart+1;  inode1 := inodes(ipstart)
!            tangent(ipstart) := t1

         ipstart = ipoint(ipart+1)-1
         iwork(inode2) = ipstart
         if ( closed ) then

!        --- Closed curve first and last tangent identical

            tangent(:,ipstart) = tangent(:,ipoint(ipart))

         else

!        --- Open curve

            t1 = coorglob(:,inode2)-coorglob(:,inode1)
            t1 = t1/sqrt(dot_product(t1,t1))
            tangent(:,ipstart) = t1

         end if  ! ( closed )

         if ( debug ) then
            write(irefwr,100) 'inode1, inode2, ipstart', &
                               inode1, inode2, ipstart
            write(irefwr,110) 't1', t1
         end if  ! ( debug )

      end do  ! ipart = 1, nparts

      deallocate ( ipoint, inodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ipoint' )

1000  if ( debug ) then

!     --- Debug information

         if ( ipstart>0 ) &
            call prinrl1 ( tangent, ipstart, ndim, 'tangent' )
         call prininnonzero ( iwork, npoint, 'iwork' )
         write(irefwr,*) 'End ercurvenodtan'

      end if
      call erclos ( 'ercurvenodtan' )

      end
