      subroutine incm012 ( istore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmodulematstr
!        version  3.0  date 17-12-2015 New parameter list
!        version  2.1  date 14-01-2002 Use direction of subsurfaces
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill part of irowsii corresponding to global unknowns
!   Help subroutine for incm01
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*)

!     istore              work array of length npoint to store the
!                         non-prescribed neighbour nodes of a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inusol, isum, knusol, node, ipkprobx, k, isubsf, &
              icrv1, icrv2, icurnr, iglgrp, inunkpgr, isrf1, isrf2, &
              istart, isurnr, itypegr, nnodes, nrusolgr, inunkp, &
              iend, nusolgr, jsurnr, nsubsf

!     i              Counting variable
!     icrv1          First curve number
!     icrv2          Last curve number
!     icurnr         Curve sequence number
!     iend           Last position used at a part
!     iglgrp         global element group number
!     inunkp         Number of degrees of freedom in point
!     inunkpgr       Number of global unknowns in a group
!     inusol         Position of unknown in array USOL
!     ipkprobx       Address in kprobx
!     isrf1          First surface number
!     isrf2          Last surface number
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isum           Integer variable to compute a sum
!     isurnr         Surface sequence number
!     itypegr        Type of region for a group of global unknowns
!                    1: series of curves
!                    2: series of surfaces
!                    3: series of volumes
!                    4: complete region
!     jsurnr         Surface sequence number
!     k              Counting variable
!     knusol         Sequence number of degree of freedom
!     nnodes         Number of nodes
!     node           Node number
!     nrusolgr       Number of free unknowns minus number of global unknowns
!     nsubsf         Number of subsurfaces
!     nusolgr        Number of unknowns minus number of global unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2002   The possibility global groups corresponding to the complete region
!           or to volumes has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm012' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm012'
         call prinin ( irowsii,nrusol+1,'irowsii')

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- kprob(11)>0, there are global elements
!         For each element group there are extra unknowns
!         These extra unknowns only adapt the corresponding rows

      kprobx => kprobpart%kprobx
      nglgrp = kprobx(1)
      nglunk = kprobx(2)
      nglcoupl = kprobx(3)

      nrusolgr = nrusol-nglunk-nglcoupl
      nusolgr = nusol-nglunk
      ipkprobx = 3

!     --- Loop over global groups

      do iglgrp = 1, nglgrp

!     --- Clear array istore

         do i = 1, nusol
            istore(i) = 0
         end do
         inunkpgr = kprobx(ipkprobx+2)
         itypegr  = kprobx(ipkprobx+3)
         isum = 0
         if ( itypegr==1 ) then

!        --- itypegr = 1, group refer to curves

            icrv1 = kprobx(ipkprobx+5)
            icrv2 = kprobx(ipkprobx+6)

!           --- Find unknowns coupled to curves icrv1 to icrv2

            do icurnr = icrv1, icrv2
               istart = kmeshm(icurnr+4)
               nnodes = kmeshm(icurnr+5)-istart
               istart = istart+kmeshm(2)-2
               do i = 1, nnodes
                  node = kmeshm(istart+i)
                  inusol = sepgetinusol ( node, inunkp )
                  if ( indprr==0 ) then

!                 --- indprr = 0

                     do k = 1, inunkp

                        if ( istore(inusol+k)==0 ) then

!                       --- New unknown found

                           isum = isum+1
                           istore(inusol+k) = 1

                        end if

                     end do

                  else

!                 --- indprr>0

                     do k = 1, inunkp
                        knusol = kprobr(inusol+k)
                        if ( istore(knusol)==0 .and. &
                             knusol<=nrusol ) then

!                       --- New unknown found

                           istore(knusol) = 1
                           if ( knusol<=nrusolgr ) then

!                          --- Standard case, unknown is not put behind
!                              global unknowns

                              isum = isum+1

                           else

!                          --- unknown is put behind global unknowns
!                              due to the symmetrical data structure and the
!                              fact that only the lower triangle is stored
!                              the global unknown belongs to the row
!                              knusol

                              irowsii(knusol+1) = irowsii(knusol+1)+ &
                                                inunkpgr

                           end if

                        end if

                     end do

                  end if

               end do

            end do

         else if ( itypegr==2 ) then

!        --- itypegr = 2, group refer to surfaces

            isrf1 = kprobx(ipkprobx+5)
            isrf2 = kprobx(ipkprobx+6)

!           --- Find smallest unknown coupled to curves isrf1 to isrf2

            if ( kelmn>0 ) then

!           --- kmesh part n does exist

               istart = 2*nsurfs
               do isurnr = 1, isrf1-1
                  istart = istart + &
                           kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
               end do
               do isurnr = isrf1, isrf2
                  nnodes = kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
                  do i = 1, nnodes
                     node = kmeshn(istart+i)
                     inusol = sepgetinusol ( node, inunkp )
                     if ( indprr==0 ) then

!                    --- indprr = 0

                        do k = 1, inunkp

                           if ( istore(inusol+k)==0 ) then

!                          --- New unknown found

                              isum = isum+1
                              istore(inusol+k) = 1
                           end if

                        end do

                     else

!                    --- indprr>0

                        do k = 1, inunkp
                           knusol = kprobr(inusol+k)
                           if ( istore(knusol)==0 .and. &
                                knusol<=nrusol ) then

!                          --- New unknown found

                              istore(knusol) = 1
                              if ( knusol<=nrusolgr ) then

!                             --- Standard case, unknown is not put behind
!                                 global unknowns

                                 isum = isum+1

                              else

!                             --- unknown is put behind global unknowns
!                                 due to the symmetrical data structure and
!                                 the fact that only the lower triangle is
!                                 stored the global unknown belongs to the
!                                 row knusol

                                 irowsii(knusol+1) = irowsii(knusol+1) + &
                                                   inunkpgr

                              end if
                           end if
                        end do

                     end if
                  end do
                  istart = istart+nnodes

               end do

            else

!           --- kmesh part n does not exist, use kmesh part t

               do jsurnr = isrf1, isrf2

                  if ( kelmr>0 ) then

!                 --- kmesh part r exists, there are subsurfaces

                     nsubsf = kmeshr(jsurnr+1) - kmeshr(jsurnr)

                  else

!                 --- kmesh part r does not exist, there are no subsurfaces

                     nsubsf = 0

                  end if
                  do isubsf = 1, max(1,nsubsf)

                     if ( nsubsf>0 ) then
                        isurnr = abs(kmeshr(kmeshr(jsurnr)+isubsf))
                     else
                        isurnr = jsurnr
                     end if

!                    --- Compute starting and end address in kmesh part c

                     istart = sepgetkmeshc ( isurnr, iend )
                     do i = istart, iend
                        node = kmeshc(i)
                        inusol = sepgetinusol ( node, inunkp )
                        if ( indprr==0 ) then

!                       --- indprr = 0

                           do k = 1, inunkp

                              if ( istore(inusol+k)==0 ) then

!                             --- New unknown found

                                 isum = isum+1
                                 istore(inusol+k) = 1

                              end if

                           end do

                        else

!                       --- indprr>0

                           do k = 1, inunkp
                              knusol = kprobr(inusol+k)
                              if ( istore(knusol)==0 .and. &
                                   knusol<=nrusol ) then

!                             --- New unknown found

                                 istore(knusol) = 1
                                 if ( knusol<=nrusolgr ) then

!                                --- Standard case, unknown is not put behind
!                                    global unknowns

                                    isum = isum+1

                                 else

!                                --- unknown is put behind global unknowns
!                                    due to the symmetrical data structure and
!                                    the fact that only the lower triangle is
!                                    stored the global unknown belongs to the
!                                    row knusol

                                    irowsii(knusol+1) = irowsii(knusol+1) + &
                                                      inunkpgr

                                 end if

                              end if
                           end do

                        end if

                     end do

                  end do

               end do

            end if

         else

!        --- This possibility has not yet been implemented

            call eropen ( 'incm012' )
            call errsub ( 2002, 0, 0, 0 )
            call erclos ( 'incm012' )

         end if

!        --- Fill array irowsii for all global unkowns corresponding to iglgrp

         do i = 1, inunkpgr
            knusol = i+nusolgr
            if ( indprr>0 ) knusol = kprobr(knusol)
            irowsii(knusol+1) = isum
         end do

!        --- Raise parameters

         nrusolgr = nrusolgr+inunkpgr
         nusolgr = nusolgr+inunkpgr
         ipkprobx = ipkprobx+6

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsii,nrusol+1,'irowsii')
         write(irefwr,*) 'End incm012'

      end if  ! ( debug )
      call erclos ( 'incm012' )

      end subroutine incm012
