      subroutine incommatpar1 ( leniwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     send information of rowsfi to all neighbor blocks with
!     lower number and receive information rowsfi from all neighbor
!     blocks with higher number
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: leniwork

!     istore         o    Array to store all information that is received
!                         from other blocks
!                         This array is filled as follows:
!                         Pos 1-iacnodes: starting addresses of all blocks
!                                in this array (0 means no information)
!                         This is followed by information received from
!                         all neighbor blocks with higher number
!                         This information consist of:
!                         number of interface unknowns (ninterf)
!                         ninterf global numbers of the interface unknowns
!                         array irowsfi corrected for other blocks, &
!                         i.e. related to this block only
!                         Length ninterf+1 (first position 0)
!     leniwork       o    Computed length of array iwork to be used in
!                         incommatpar3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numlow, numupp, ilow, ihigh, ninterf, i, iblock, &
                 icontext, istart, lenstore ! iwork(3*ninterface+3),
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_s

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     istart         First position used at a part
!     iwork          integer work array to store and receive data to and from
!                    other blocks
!     lenstore       Declared length of array istore
!     ninterf        Number of unknowns at interface
!     numlow         Number of neighbor blocks with lower number
!     numupp         Number of neighbor blocks with higher number
!     send_buffer    Buffer for sending messages
!     send_buffer_s  Length of array send_buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar1'
         write(irefwr,1) 'inodenr, ninterface', inodenr, ninterface
         call prinin ( irowsfi, ninterface+1, 'irowsfi' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi
      lenstore = size(istore)

      numlow = blockinfor(1)  ! number of neighbors with smaller number
      numupp = blockinfor(2)  ! number of neighbors with larger number
      leniwork = 0
      allocate ( iwork(3*ninterface+3) )

      if ( debug ) then
         write(irefwr,1) 'numlow, numupp', numlow, numupp
         write(irefwr,*) 'information sent'
      end if  ! ( debug )

!     --- Compute length of send_buffer

      send_buffer_s = 0

      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)     ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)  ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block
         iwork(1) = ninterf
         length = 1+ninterf
         if ( ninterf>0 ) length = length+ninterf+1
         call inichkarlen ( length, 3*ninterface+3, 'iwork' )
         send_buffer_s = send_buffer_s+length

      end do  ! i = 1, numlow

!     --- Multiply by 4 (integers) and add 100 for safety

      send_buffer_s = send_buffer_s*4+100

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_s, max(2,numlow) )

      allocate(send_buffer(send_buffer_s))

      call inibufattmpi ( send_buffer, send_buffer_s )
      if ( ierror/=0 ) go to 1000

!     --- Send part of irowsfi to blocks with smaller number
!         each row is reset so that it starts with 0

      icontext = 53
      do i = 1, numlow

         iblock = blockinfor(2+i)      ! neighbor block
         ilow = ninterfdeg(iblock)     ! smallest index of interface unknown
         ihigh = ninterfdeg(iblock+1)  ! largest index of interface unknown+1
         ninterf = ihigh-ilow          ! number of interface unknowns in block
         iwork(1) = ninterf
         length = ilow+nrusol+1
         iwork(2:1+ninterf) = kprobhglob(length:length+ninterf-1)
         length = 1+ninterf

         if ( debug ) &
            write(irefwr,1) 'i, iblock, ilow, ihigh, ninterf', &
                             i, iblock, ilow, ihigh, ninterf

         if ( ninterf>0 ) then

!        --- There are interface unknowns
!            Copy array irowsfi into iwork

            iwork(length+1:length+ninterf+1) = &
               irowsfi(ilow:ihigh)-irowsfi(ilow)
            length = length+ninterf+1
            leniwork = max(leniwork,irowsfi(ihigh)-irowsfi(ilow))

         end if  ! ( ninterf>0 )

         if ( debug ) then
            write(irefwr,1) 'i, iblock, length', i, iblock, length
            call prinin ( iwork, length, 'iwork' )
         end if  ! ( debug )

!        --- Send iwork to iblock

         call inibsendmpi ( iwork, length, SPINT, iblock, icontext )

      end do  ! i = 1, numlow

      deallocate ( iwork )

      call inibarriermpi

!     --- Receive part of irowsfi from blocks with larger number

      istore(1:iacnodes) = 0
      istart = iacnodes+1

      if ( debug ) write(irefwr,*) 'information received'
      do i = 1, numupp

         length = lenstore-istart
         call inireceivempi ( istore(istart), length, SPINT, iblock, &
                              icontext )
         istore(iblock) = istart
         istart = istart+length

      end do  ! i = 1, numupp

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_s )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      if ( debug ) call prinin ( istore, istart-1, 'istore' )

      call inibarriermpi

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incommatpar1'

      end if  ! ( debug )
      call erclos ( 'incommatpar1' )

      end subroutine incommatpar1
