      subroutine incommatpar8 ( lensend )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Construct array sendunknowns with the global sequence numbers
!     of the unknowns to be transported to neighbor blocks
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: lensend

!     lensend       i/o   Length of array sendunknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, ncols, icol, iend, iblock, j, istartsend, &
                 help(iacnodes)

!     i              Counting variable
!     iblock         Block sequence number
!     icol           Column number (global numbering)
!     iend           Last position used at a part
!     help           Work array to mark if a block is used or not
!     istart         First position used at a part
!     istartsend     Start address in sendunknowns
!     j              Counting variable
!     ncols          Number of non-zero matrix elements in a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array sendunknowns(0:iacnodes) with number of unknowns to
!     be sent per block:
!       First define the number to be sent to blocks with higher number:
!        i.e. Copy ninterf from storeinfo into sendunknowns
!       Then add number of internal unknowns coupled to each interface unknown
!     Accumulate sendunknowns
!     Fill actual unknown numbers into sendunknowns part 2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar8' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar8'
         call prinin ( irowsout, 1+nrusol, 'irowsout' )
         call prinin ( icolsout, irowsout(nrusol), 'icolsout' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array iwork with "inverse" of nrusollocs

      istart = 1
      do i = 1, iacnodes
         iend = nrusollocs(i)
         iwork(istart:iend) = i
         istart=iend+1
      end do  ! i = 1, iacnodes

!     --- Fill array sendunknowns from arrays irowsout and icolsout
!         with the number of parameters to be send

      sendunknowns = 0

      istart = 0
      do i = 1, nrusol          ! loop over rows

         help = 0
         ncols = irowsout(i)-istart  ! number of outer unknowns coupled

         if ( ncols>0 ) then

!        --- i is coupled to other blocks

            do j = 1, ncols
               icol = icolsout(istart+j)  ! global column number
               iblock = iwork(icol)
               if ( help(iblock)==0 ) then

!              --- Block is not yet used

                  help(iblock) = 1          ! mark block as used
                  sendunknowns(iblock) = sendunknowns(iblock)+1

               end if  ! ( help(iblock)==0 )
            end do  ! j = 1, ncols

            istart = istart+ncols  ! raise istart

         end if  ! ( ncols>0 )

      end do  ! i = 1, nrusol

      if ( debug ) &
         call prinin ( sendunknowns, iacnodes+1, 'sendunknowns' )

!     --- Accumulate sendunknowns

      do i = 1, iacnodes
         sendunknowns(i) = sendunknowns(i)+sendunknowns(i-1)
      end do  ! i = 1, iacnodes

      lensend = sendunknowns(iacnodes)+iacnodes+1

      if ( debug ) &
         call prinin ( sendunknowns, iacnodes+1, 'sendunknowns' )

!     --- Fill actual unknown numbers into sendunknowns part 2
!         First reset sendunknowns(i) to sendunknowns(i-1) and add iacnodes
!         This defines the starting address from position 0

      do i = iacnodes, 1, -1
         sendunknowns(i) = sendunknowns(i-1)
      end do  ! i = 1, iacnodes
      sendunknowns(0:iacnodes) = sendunknowns(0:iacnodes)+iacnodes

      if ( debug ) &
         call prinin ( sendunknowns, iacnodes+1, 'sendunknowns' )

!     --- Fill array sendunknowns with sequence numbers

      istart = 0
      do i = 1, nrusol          ! loop over rows

         help = 0
         ncols = irowsout(i)-istart  ! number of outer unknowns coupled

         if ( ncols>0 ) then

!        --- i is coupled to other blocks

            do j = 1, ncols

               icol = icolsout(istart+j)  ! global column number
               iblock = iwork(icol)

               if ( help(iblock)==0 ) then

!              --- Block is not yet used

                  help(iblock) = 1   ! mark
                  istartsend = sendunknowns(iblock)+1
                  sendunknowns(istartsend) = i     ! store
                  sendunknowns(iblock) = istartsend  ! raise

               end if  ! ( help(iblock)==0 )
            end do  ! j = 1, ncols

            istart = istart+ncols  ! raise istart

         end if  ! ( ncols>0 )

      end do  ! i = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         call prinin ( sendunknowns, lensend, 'sendunknowns' )
         write(irefwr,*) 'End incommatpar8'

      end if  ! ( debug )
      call erclos ( 'incommatpar8' )

      end subroutine incommatpar8
