      subroutine incommatpar9 ( lenrec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send the contents of array sendunknowns in global numbers to
!     other blocks
!     Receive these arrays and create array recunknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: lenrec

!     lenrec        i/o   Length of array recunknowns
!                         At input declared length
!                         At output computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numlow, numupp, icontext, i, iblock, istart, icount
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_s

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     icount         Counter
!     istart         First position used at a part
!     numlow         Number of neighbor blocks with lower number
!     numupp         Number of neighbor blocks with higher number
!     send_buffer    Buffer for sending messages
!     send_buffer_s  Length of array send_buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all neighbor blocks do
!        send contents of array sendunknowns with global numbers
!     For all neighbor blocks do
!        receive contents of array sendunknowns and store in array store
!     Create array recunknowns
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar9' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar9'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call inibarriermpi

!     --- Get parameters

      numlow = blockinfor(1)  ! number of neighbors with smaller number
      numupp = blockinfor(2)  ! number of neighbors with larger number

      if ( debug ) write(irefwr,1) 'numlow, numupp', numlow, numupp

!     --- Compute length of send_buffer

      send_buffer_s = 0
      do i = 1, numlow+numupp

         iblock = blockinfor(2+i)      ! neighbor block
         istart = sendunknowns(iblock-1)
         length = sendunknowns(iblock)-istart
         send_buffer_s = send_buffer_s+length

      end do  ! i = 1, numlow+numupp

      send_buffer_s = send_buffer_s*4 + 100

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_s, max(2,numlow) )

      allocate(send_buffer(send_buffer_s))

      call inibufattmpi ( send_buffer, send_buffer_s )
      if ( ierror/=0 ) go to 1000

!     --- For all neighbor blocks do
!         send contents of array sendunknowns with global numbers

      icontext = 56
      do i = 1, numlow+numupp

         iblock = blockinfor(2+i)      ! neighbor block
         istart = sendunknowns(iblock-1)
         length = sendunknowns(iblock)-istart

         if ( debug ) &
            write(irefwr,1) 'i, iblock, length', i, iblock, length

!        --- Store information to be sent in recunknowns

         if ( length>0 ) then

!        --- Length>0, information to be sent
!            Translate to global numbers

            recunknowns(1:length) = sendunknowns(istart+1:istart+length)
            recunknowns(1:length) = kprobhglob(recunknowns(1:length))

         else

!        --- Length=0, no information to be sent, use 0

            recunknowns(1) = 0
            length = 1

         end if  ! ( length>0 )

         call inibsendmpi ( recunknowns, length, SPINT, iblock, &
                            icontext )

      end do  ! i = 1, numlow+numupp

      call inibarriermpi

!     --- For all neighbor blocks do
!           receive contents of array sendunknowns and store in array store

      recunknowns(1:iacnodes+1) = 0
      istart = iacnodes+2

      icount = 0

      do i = 1, numlow+numupp

         length = lenrec
         call inireceivempi ( recunknowns(istart), length, SPINT, &
                              iblock, icontext )
         if ( length>1 .or. recunknowns(istart)>0 ) then

!        --- Non-zero information received

            recunknowns(iblock+1) = istart-1
            istart = istart+length
            icount = icount+1

         end if  ! ( length>1 .or. recunknowns(istart)>0 )

      end do  ! i = 1, numlow+numupp

      recunknowns(1) = icount

      lenrec = istart-1

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_s )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      call inibarriermpi

1000  if ( debug ) then

!     --- Debug information

         call prinin ( recunknowns, lenrec, 'recunknowns' )
         write(irefwr,*) 'End incommatpar9'

      end if  ! ( debug )
      call erclos ( 'incommatpar9' )

      end subroutine incommatpar9
