      subroutine infil2 ( iwork, infelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.1  date 18-12-2002 Check on ishape
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the extra fill-in in the case of discontinuous
!   pressure elements where all neighbours must be taken into account
! **********************************************************************
!
!                       KEYWORDS
!
!     fill_in
!     structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulematstr
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), infelm(2,*)

!     infelm          Work array of length 2*nelem containing information
!                     concerning the elements in the following way:
!                     infelm(1,i) = starting address of element i in kmeshc -1
!                     infelm(2,i) = number of vertices of element i
!     iwork           integer work array of length nelem
!                     This work array is used to indicate if an element has been
!                     treated already (1) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, iunknown, j, jelem, k, nbr, icount, &
              icentroid, jcentroid, iverts(8), inpelm, &
              ikelmc, irelem, ielgrp, &
              icentr, nverts, jstep, jphys, junknown, iphys

!     i          Counting variable
!     icentr     priority number corresponding to centroid
!     icentroid  Nodal point number of centroid
!     icount     Counter
!     ielem      Element number
!     ielgrp     element group number
!     ikelmc     position in kmesh part c
!     inpelm         number of points per element
!     iphys      Loop variable over the physical degrees of freedom
!     irelem     Relative element sequence number with respect to element group
!     iunknown   Sequence number of unknownn
!     iverts     Array of length 8 containing the relative sequence numbers
!                of the vertices within the element
!     j          Counting variable
!     jcentroid  Nodal point number of centroid
!     jelem      do loop variable with respect to the elements
!     jphys      Loop variable over the physical degrees of freedom
!     jstep      Step in loop
!     junknown   Sequence number of unknownn
!     k          Counting variable
!     nbr        number of neighbours of nodal point
!     nverts     Number of vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork
!    For all element groups ielgrp  with centroid points do
!       For all elements ielem in ielgrp do
!          Find centroid of element
!          iwork(ielem) := 1
!          icentroid := centroid of ielem
!          For all vertices i of element ielem do
!             nbr := number of elements vertex i is lying in
!             for all elements jelem i is lying in do
!                if ( iwork(jelem)=0 ) then
!                   iwork(jelem) := 1
!                   jcentroid := centroid of jelem
!                   For iphys := iphys1 (1) iphys2 do
!                      iunknown := kprobp(icentroid,iphys)
!                      For jphys := iphys1 (1) iphys2 do
!                         junknown := kprobp(jcentroid,iphys)
!                         if ( junknown < iunknown ) then
!                            store junknown in icolsii
!          Clear array iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'infil2' )

!     --- Clear array iwork

      ikelmc = 0
      iwork(1:nelem) = 0

!     --- Fill array infelm with information concerning the elements

      irelem = 0
      do ielgrp = 1, nelgrp
         ishape = sepgetshape(ielgrp)
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         if ( ishape==6 .or. ishape==7 .or. ishape==9 .or. &
              ishape==14 .or. ishape==17 .or. ishape==18 .or. &
              ishape==10 .or. ishape==23 ) then
            if ( ishape==14 ) then
               icentr = 14
            else
               icentr = inpelm
            end if
         else if ( ishape.le.5 .or. ishape.gt.10 .and. ishape.le.13 .or. &
                   ishape==21 ) then
            icentr = 0
         else

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            icentr = 0
         end if
         do ielem = 1+irelem, inelem+irelem
            infelm(1,ielem) = ikelmc+(ielem-1)*inpelm
            infelm(2,ielem) = icentr
         end do
         ikelmc = ikelmc + inelem*inpelm
         irelem = irelem + inelem
      end do

!     --- Loop over element groups

      ikelmc = 0
      do ielgrp = 1, nelgrp
         ishape = sepgetshape(ielgrp)
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         if ( ishape==6 .or. ishape==7 .or. ishape==9 .or. &
              ishape==14 .or. ishape==17 ) then
            if ( ishape==14 ) then
               icentr = 14
               nverts = 8
               iverts(1) = 1
               iverts(2) = 3
               iverts(3) = 7
               iverts(4) = 9
               iverts(5) = 19
               iverts(6) = 21
               iverts(7) = 25
               iverts(8) = 27
            else
               icentr = inpelm
               if ( ishape==6 ) then
                  nverts = 4
                  jstep = 2
               else if ( ishape==7 ) then
                  nverts = 3
                  jstep = 2
               else if ( ishape==9 ) then
                  nverts = 4
                  jstep = 1
               else
                  nverts = 8
                  jstep = 1
               end if
               do i = 1, nverts
                  iverts(i) = (i-1)*jstep+1
               end do
            end if

!           --- element contains centroid points
!               Loop over elements in element group

            do ielem = 1, inelem

!           --- Find centroid of element

               icentroid = kmeshc(ikelmc+icentr)
               iwork(ielem) = 1
               do iphys = iphys1, iphys2
                  iunknown = kprobp(icentroid,iphys)
                  if ( iunknown.gt.0 ) then
                     icount = 0

!                    --- Loop over vertices

                     do i = 1, nverts
                        j = kmeshc(ikelmc+iverts(i))
                        nbr = kmshd1(j+1)-kmshd1(j)

!                       --- for all elements jelem i is lying in do

                        do k = 1, nbr
                           jelem = kmshd2(kmshd1(j)+k)
                           if ( iwork(jelem)==0 ) then

!                          --- element has not been treated yet

                              iwork(jelem) = 1
                              jcentroid = &
                                 kmeshc(infelm(1,jelem)+infelm(2,jelem))
                              do jphys = iphys1, iphys2
                                 junknown = kprobp(jcentroid,iphys)
                                 if ( junknown.gt.0 .and. &
                                      junknown.lt.iunknown ) then
                                    icolsii(irowsii(iunknown+1)-icount) = &
                                       junknown
                                    icount = icount + 1
                                 end if
                              end do
                           end if
                        end do
                     end do
                  end if
               end do

!              --- Clear array iwork

               iwork(ielem) = 0
               do i = 1, nverts
                  j = kmeshc(ikelmc+iverts(i))
                  nbr = kmshd1(j+1)-kmshd1(j)
                  do k = 1, nbr
                     iwork(kmshd2(kmshd1(j)+k)) = 0
                  end do
               end do
               ikelmc = ikelmc + inpelm
            end do
         else

!        --- Not an element group with centroid points

            ikelmc = ikelmc + inpelm*inelem
         end if
      end do
      call erclos ( 'infil2' )
      end
