      subroutine initspfict ( istfl1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-10-2011 New parameter list
!        version  1.2  date 04-01-2002 Do not clear kproby(4)
!        version  1.1  date 29-04-2001 Make inunkp positive
!
!   copyright (c) 2000-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information from iinputkp concerning the element groups corresponding
!     to fictitious unknowns into array kprob part y
! **********************************************************************
!
!                       KEYWORDS
!
!     file_1
!     read
!     problem_definition
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istfl1(*)

!     istfl1        i/o   Array in which information from FILE 1 is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kh1, ipos, kh2, i, iposkproby, &
              loctype, ifictgrp, iinp, iinp11, iinp12, itype, &
              inunkp, numfictcrv, numfictsrf
      integer, allocatable :: iwork(:,:)

!     i              Counting variable iwork(2,*), &
!     ifictgrp       fictitious element group number
!     iinp           last position used in array iinputkp
!     iinp11         first pos. iinputkp definition fictitious group type numbers
!     iinp12         first pos. iinputkp definition fictitious group elements
!     inunkp         Number of degrees of freedom in point
!     ipos           Position in array
!     iposkproby     Position in kprob y
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     iwork          integer work array of length 2* nfictgrp
!                    iwork(1,i) contains the starting address of group i in
!                               iinputkp
!                    iwork(2,i) contains the type of elements to be used
!                               for group i
!                               1 = curves
!                               2 = surfaces
!     kh1            Help parameter to compute a position in KPROB
!     kh2            Help parameter to compute a position in KPROB
!     loctype        mod(itype,10000), i.e. the actual type number
!     numfictcrv     number of parts where curve elements are defined
!                    for fictitious elements
!     numfictsrf     number of parts where surface elements are defined
!                    for fictitious elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    input from file 1 is read
!    File 1 is filled as follows:
! part 1)  records 1 up to krec2-1:  pointers for each standard problem
!          with 99<itype<maxtyp  (maximum type number) to part 2
!          For a certain type the pointer may be to another type
!          only if all the information is identical
! part 2)  records krec2,...,9 information on standard problems
!  for each type:
!  With respect to global unknowns:
!    pos. 1   -inunkp   - number of global degrees of freedom, &
!         2   -1        indicating that it concerns a global unknown
!         3   meshtp    shape number of the element (must be -10)
!         4   0
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
!    2151   ifluidgrp1>ifluidgrp2
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read information concerning type numbers from file 1 if necessary
!     Compute starting address of information concerning the type in array
!     istfl1
!     Read information concerning information corresponding to type numbers
!     from file 1 if necessary
!     Compute starting address of information concerning the type in array
!     istfl1
!     Put information into istore (read information if necessary)
!     kproby is filled as follows
!     Pos.  1    nfictgrp (number of fictitious groups)
!           2    nfictunk total number of lambdas
!           3    nfictpnt total number of nodes with lambdas
!           4    memory management sequence number of kprobfict
!          5-10  not yet used
!         11-11+nfictgrp  starting addresses with information per group
!                The last one refers to the end position plus one
!     Per group the following information is stored:
!     1:  itype       type number of element
!     2:  inunkpgr    number of degrees of freedom per lambda point
!     3:  iregionlam  type of region where the lambda's are defined:
!                     1:  curves
!                     2:  surfaces
!     4:  ishape      Shape of lambda basis functions
!     5:  icurve1     First curve of surface
!     6:  icurve2     Last curve of surface
!     7:  istructgrp  Structural element group
!     8:  ifluidgrp1  First fluid element group
!     9:  ifluidgrp2  Last fluid element group
!    10:  inunkp_elm  Number of lambda's in element
!    11:  nelem       Number of elements for this group
!    12:  ishape_curve  Shape number of "curve" elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'initspfict' )

!     --- Read information concerning type numbers from file 1 if necessary

      iinp11 = intarprb(15)
      iinp12 = intarprb(16)
      if ( iinputkp(15)<=4 ) then
         nfictgrp = 0
      else
         nfictgrp = iinputkp(36)
      end if

      allocate ( iwork(2,nfictgrp),stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*nfictgrp, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear first 10 positions of kprob y, except pos. 4

      kproby(2:3) = 0
      kproby(5:10) = 0

!     --- Find starting addresses of groups in part 11 of iinputkp

      iwork = 0

      numfictcrv = iinputkp(37)
      numfictsrf = iinputkp(38)
      iinp = iinp12
      do i = 1, numfictcrv
         ifictgrp = iinputkp(iinp)
         iwork(1,ifictgrp) = iinp
         iwork(2,ifictgrp) = 1
         iinp = iinp+7
      end do
      do i = 1, numfictsrf
         ifictgrp = iinputkp(iinp)
         iwork(1,ifictgrp) = iinp
         iwork(2,ifictgrp) = 2
         iinp = iinp+7
      end do
      iposkproby = 11+nfictgrp
      do ifictgrp = 1, nfictgrp
         itype = iinputkp(iinp11+2*(ifictgrp-1))
         iinp = iwork(1,ifictgrp)
         if ( itype>=100 ) then

!        --- Standard element

            loctype = mod(itype,10000)
            kh1 = (loctype-100)/nrec1+1
            if ( kh1/=irecp1 ) &
               call readfl ( kh1, 1, nrec1, 1, 1, istfl1 )

!           --- Compute starting address of information concerning the type in
!               array istfl1

            ipos = istfl1(loctype-99-nrec1*(kh1-1))
            if ( ipos==0 ) then

!           --- Error 43: type number not available

               call errint ( loctype, 1 )
               call errint ( ifictgrp, 2 )
               call errsub ( 43, 2, 0, 0)
               go to 1000

            end if

!           --- Read information concerning information corresponding to type
!               numbers from file 1 if necessary

            kh2 = (ipos-1)/nrec1+1
            if ( kh2/=irecp2 ) &
                call readfl ( kh2, nrec1+1, 2*nrec1, 2, 1, istfl1 )

!           --- Compute starting address of information concerning the type in
!               array istfl1

            ipos   = ipos-(kh2-2)*nrec1
            inunkp = istfl1(ipos)
            if ( inunkp==-1001 ) inunkp = ndim
            inunkp = abs(inunkp)

         else

!        --- Number of unknowns stored in iinputkp

            inunkp = iinputkp(iinp11+2*(ifictgrp-1)+1)

         end if

         kproby(10+ifictgrp) = iposkproby+1
         kproby(iposkproby+1) = itype
         kproby(iposkproby+2) = inunkp
         kproby(iposkproby+3) = iwork(2,ifictgrp)
         do i = 1, 6
            kproby(iposkproby+3+i) = iinputkp(iinp+i)
         end do
         ishape = iinputkp(iinp+1)
         if ( ndim==2 ) then

!        --- Curve elements

            nfictunk = inunkp*ishape

         else

!        --- Surface elements

            if ( ishape==3 .or. ishape==5 ) then
               nfictunk = inunkp
            else if ( ishape==4 ) then
               nfictunk = inunkp*3
            else
               nfictunk = inunkp*4
            end if

         end if

!        --- Perform some checks

         if ( kproby(iposkproby+9)<kproby(iposkproby+8) ) then

!        --- Second fluid group smaller than first one

            call errint ( ifictgrp, 1 )
            call errint ( kproby(iposkproby+9), 2 )
            call errint ( kproby(iposkproby+8), 3 )
            call errsub ( 2151, 3, 0, 0 )

         end if
         kproby(iposkproby+10) = nfictunk
         kproby(iposkproby+11) = 0
         kproby(iposkproby+12) = 0
         iposkproby = iposkproby+12

      end do

!     --- Compute number of fictitious degrees of freedom

      kproby(11+nfictgrp) = iposkproby
      kproby(1) = nfictgrp

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'initspfict' )

      end
