      subroutine prob04 ( iskippoint, nodes, ipoint, nesintersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version 18.1  date 17-03-2015 extra debug
!        version 18.0  date 13-10-2013 Complete new parameter list
!        version 17.4  date 20-06-2013 New definition file ref numbers
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions
!     This subroutine is called by PROBDF
!     It replaces a part of the previous subroutines PROB05, PROB06, PROB08, &
!     PROB11, PROB12 and PROB13
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: nesintersect, nodes(npoint,nesintersect), &
                 ipoint(nesintersect), iskippoint(*)

!     ipoint         o    Array of length nesintersect containing the
!                         number of nodes for each intersection
!     iskippoint     i    Array of length nskip+1 containing the points to
!                         be skipped
!                         Position 1 contains nskip (number of points to be
!                                    skipped
!                         The next positions contain the node numbers to be
!                         skipped
!     nesintersect   i    Number of intersections where essential boundary
!                         conditions are given
!     nodes          o    Array of size nesintersect x npoint
!                         nodes(i,1:ipoint(i)) contains the nodal points
!                         corresponding to intersection i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxobst, iinp, ivers, nkbndprev

!     iinp           last position used in array iinputkp
!     ivers          Version sequence number
!     maxobst        maximum number of active obstacles
!     nkbndprev      Value of nkbnd before call of prob04levset
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB04CONT     Compute number of nodes with essential boundary conditions
!                    for contact
!     PROB04CV1      Compute number of nodes with essential boundary conditions
!                    for curve
!     PROB04ELEM     Compute number of nodes with essential boundary conditions
!                    for elements
!     PROB04ELGRP    Compute number of nodes with essential boundary conditions
!                    for element groups
!     PROB04INTERS   Compute number of nodes with essential boundary conditions
!                    for intersections
!     PROB04LEVSET   Compute number of nodes with essential boundary conditions
!                    for level set
!     PROB04NODAL    Compute number of nodes with essential boundary conditions
!                    for nodes
!     PROB04NODP     Compute number of nodes with essential boundary conditions
!                    for  points
!     PROB04OBST     Compute number of nodes with essential boundary conditions
!                    for obstacles
!     PROB04OBSTCV   Compute number of nodes with essential boundary conditions
!                    for obstacle curves
!     PROB04SF       Compute number of nodes with essential boundary conditions
!                    for surface
!     PROB04UP       Compute number of nodes with essential boundary conditions
!                    for user points
!     PROB04VLM      Compute number of nodes with essential boundary conditions
!                    for volumes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04'
         write(irefwr,1) 'npoint', npoint
!         call prinin ( iinputkp, 60, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nkbnd = 0
      iinp  = intarprb(4)-1
      ivers = iinputkp(15)
      if ( debug ) write(irefwr,1) 'iinp, ivers', iinp, ivers

!     --- User points

      if ( iinputkp(6)>0 ) call prob04up ( iinp, iinputkp, nkbnd )
      if ( debug ) &
         write(irefwr,1) 'After prob04up: nkbnd, iinp', nkbnd, iinp

!     --- Curves

      if ( iinputkp(7)>0 ) call prob04cv1 ( iinputkp, iinp, nkbnd )

      if ( debug ) &
         write(irefwr,1) 'After prob04cv: nkbnd, iinp', nkbnd, iinp

!     --- Surfaces

      if ( iinputkp(8)>0 )  call prob04sf ( iinputkp, iinp, nkbnd )
      if ( debug ) &
          write(irefwr,1) 'After prob04sf: nkbnd, iinp', nkbnd, iinp

!     --- Volumes

      if ( iinputkp(39)>0 ) call prob04vlm ( iinputkp, iinp, nkbnd )
      if ( debug ) &
          write(irefwr,1) 'After volumes: nkbnd, iinp', nkbnd, iinp

!     --- Nodal points

      if ( iinputkp(10)>0 ) call prob04nodp ( iinputkp, iinp, nkbnd )
      if ( debug ) &
          write(irefwr,1) 'After Nodal points: nkbnd, iinp', nkbnd, iinp

!     --- Elements

      if ( iinputkp(11)>0 ) call prob04elem ( iinputkp, iinp, nkbnd )
      if ( debug ) &
         write(irefwr,1) 'After Elements: nkbnd, iinp', nkbnd, iinp

!     --- Element groups

      if ( iinputkp(12)>0 ) call prob04elgrp ( iinputkp, iinp, nkbnd )
      if ( debug ) &
          write(irefwr,1) 'After Elementgroup: nkbnd, iinp', nkbnd, iinp

!     --- Obstacles

      if ( ivers<=1 ) go to 1000  ! jump in case of first version
      maxobst = numobst

      if ( iinputkp(16)>0 .and. maxobst>0 .and. kelmh>0 ) &
         call prob04obstcv ( iinputkp, iinp, nkbnd )
      if ( debug ) &
          write(irefwr,1) 'After obstacle: nkbnd, iinp', nkbnd, iinp

      if ( ivers>=6 .and. &
           (iinputkp(45)>0 .or. iinputkp(41)>0 .or. &
            iinputkp(42)>0 .or. iinputkp(43)>0 .or. &
            iinputkp(48)>0 .and. nlevelsets>0) ) then

!     --- points in obstacles are prescribed

         call prob04obst ( iinputkp, iinp, nkbnd )

      end if
      if ( debug ) &
         write(irefwr,1) 'After obstacles: nkbnd, iinp', nkbnd, iinp

!     --- Contact and no_contact

      if ( iinputkp(19)>0 .or. iinputkp(22)>0 ) &
         call prob04cont ( iinputkp, iinp )
      if ( debug ) &
         write(irefwr,1) 'After Contact: nkbnd, iinp', nkbnd, iinp

!     --- Nodal points

      if ( iinputkp(28)>0 ) call prob04nodal ( iinputkp, nkbnd, iinp )
      if ( debug ) &
         write(irefwr,1) 'After Nodal points: nkbnd, iinp', nkbnd, iinp

!     --- Level set functions

      if ( nlevelsets>0 .and. iinputkp(48)>0 ) then

!     --- There are level set functions

         nkbndprev = nkbnd
         call prob04levset ( iinputkp, iinp, nkbnd, intarprb )
         kprobloc(64) = nkbnd-nkbndprev

      end if  ! ( nlevelsets>0 .and. iinputkp(48)>0 )
      if ( debug ) &
          write(irefwr,1) 'After level set: nkbnd, iinp', nkbnd, iinp

!     --- Cavity boundary conditions

      if ( kprobloc(60)>0 .and. iinputkp(49)>0 ) then
         kprobad=> kprobpart%kprobad
         nkbnd = nkbnd+kprobad(1)
         iinp = iinp+2
      end if  ! ( kprobloc(60)>0 .and. iinputkp(49)>0 )
      if ( debug ) &
         write(irefwr,1) 'After cavitation: nkbnd, iinp', nkbnd, iinp

!     --- Intersection boundary conditions

      if ( iinputkp(56)>0 ) &
         call prob04inters ( iinputkp, iinp, nodes, &
                             ipoint, nesintersect )
      if ( debug ) &
         write(irefwr,1) 'After intersection: nkbnd, iinp', nkbnd, iinp

!     --- Skip points

      if ( kprobloc(53)>0 ) then
         nkbnd = nkbnd + iskippoint(1)
         kprobloc(64) = kprobloc(64) + iskippoint(1)  ! number of special nodes
      end if  ! ( kprobloc(53)>0 )
      if ( debug ) &
         write(irefwr,1) 'After skip points: nkbnd, iinp', nkbnd, iinp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nkbnd', nkbnd
         write(irefwr,*) 'End prob04'

      end if  ! ( debug )
      call erclos ( 'prob04' )

      end
