      subroutine prob04sfouter ( iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for the outer surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inodp, istart, ipart, isurnr, k, j, &
              nnodes, nparts, nsurfaces
      integer, allocatable :: nodes(:), iwork(:)

!     inodp          Nodal point number
!     ipart          Sequence number of part
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means that node i has not yet been used
!                    iwork(i)>0,means that node i has been used
!     j              Counting variable
!     k              Counting variable
!     nnodes         Number of nodes
!     nodes          Contains the node numbers that must be filled
!     nparts         number of parts the boundary consists of
!     nsurfaces      Number of surfaces in outer surface part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2462   No outer surfaces found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04sfouter' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04sfouter'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( nodes(npoint), iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'nodes' )
      if ( ierror/=0 ) go to 1000

      if ( mmouter<=0 .or. ndim<3 ) then

!     --- No outer surfaces found

         call errint ( ndim, 1 )
         call errsub ( 2462, 1, 0, 0 )
         go to 1000

      end if
      nparts = outerboun(1)
      if ( debug ) &
         write(irefwr,1) 'nparts, npoint, nkbnd', nparts, npoint, nkbnd

!     --- Clear iwork

      iwork = 0

      do ipart = 1, nparts

         istart = outerboun(ipart+1)
         nsurfaces = outerboun(istart)

         do j = 1, nsurfaces

            isurnr = abs(outerboun(istart+j))
            nnodes = npoint
            if ( debug ) write(irefwr,1) 'j, isurnr', j, isurnr
            call sepsurnod ( isurnr, isurnr, nodes, nnodes )
            if ( debug ) write(irefwr,1) 'nnodes', nnodes
            do k = 1, nnodes

               inodp = nodes(k)
               if ( inodp==0 ) go to 300  ! jump for empty node

               if ( debug ) &
                  write(irefwr,1) 'k, inodp, iwork(inodp)', &
                                   k, inodp, iwork(inodp)
               if ( iwork(inodp)==0 ) then

!              --- iwork(inodp) = 0:  new nodal point

                  iwork(inodp) = 1
                  nkbnd = nkbnd+1
                  if ( debug ) &
                      write(irefwr,1) 'nkbnd, inodp', nkbnd, inodp

               end if  ! ( iwork(inodp)==0 )

300         end do  ! k = 1, nnodes

         end do  ! j = 1, nsurfaces

      end do  ! ipart = 1, nparts

      iinp = iinp+3

      deallocate ( nodes, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04sfouter'

      end if  ! ( debug )
      call erclos ( 'prob04sfouter' )

      end
