      subroutine prob09obst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute npltra and store in kprob with respect to obstacles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jdgfd1, jdgfd2, jstep, iexclu, kstep, n, &
              kstart, kend, k, nodp, j, iinp, jtrans, &
              iinsign, maxobst, nlocobs, iobst, iinp5, inunkp
      integer, allocatable :: iwork(:), iwork1(:)

!     i              Counting variable
!     iexclu         Indicates which points are excluded
!     iinp           last position used in array iinputkp
!     iinp5          Starting address of fifth part in iinputkp
!     iinsign        Sign of curve number
!     inunkp         Number of degrees of freedom in a point
!     iobst          Obstacle sequence number
!     iwork          integer work array of length npoint
!     iwork1         integer work array of length npoint
!     j              Counting variable
!     jdgfd1         Degree of freedom
!     jdgfd2         Degree of freedom
!     jstep          Step in loop
!     jtrans         Type of local transformation
!                    1:  standard transformation
!                    2:  symmetric transformation
!                    3:  non-symmetric transformation
!     k              Counting variable
!     kend           End value of parameter
!     kstart         Start value of parameter
!     kstep          Step in loop
!     maxobst        Maximum number of obstacles
!     n              number of basis functions for each component
!     nlocobs        Number of obstacles with local transforms
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PROB21         Compute the number of nodes with local transformations at
!                    an obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     559   The parameter l for the definition of local transforms has
!           an incorrect value
!    1033   first degree of freedom has wrong value
!    1034   second degree of freedom has wrong value
!    1035   first and second degree of freedom have same value
!    1279   Only ndim =2 permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob09obst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob09obst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinp5 = intarprb(5)
      iinp = iinp5-1
      maxtrn = max(maxtrn,2)

      if ( ndim/=2 ) then

!     --- local transformations are only implemented for two-dimensions

         call errsub ( 1279, 0, 0, 0 )
         go to 1000

      end if

      nlocobs = iinputkp(18)
      iinp = iinp+2

!     --- allocate work space

      allocate ( iwork(npoint), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- obstacles

      maxobst = numobst
      do i = 1, min(maxobst,nlocobs)

         jdgfd1 = iinputkp(iinp+1)
         jdgfd2 = iinputkp(iinp+2)
         jstep = iinputkp(iinp+4)
         if ( jstep>=0 ) then

!        --- jstep>=  0,  compute iexclu

            iexclu = jstep/100
            if ( iexclu>3 ) then
               call errint ( iexclu, 1 )
               call errint ( jstep, 2 )
               call errsub ( 559, 2, 0, 0 )
            end if
            jstep = jstep-100*iexclu

         else

            iexclu = 0

         end if

         iobst = abs(iinputkp(iinp+5))
         jtrans = iinputkp(iinp+6)
         iinp = iinp+6
         if ( jtrans==2 ) then
            iinp = iinp+4
            maxtrn = max(maxtrn,4)
         else if ( jtrans==3 ) then
            iinp = iinp+8
            maxtrn = max(maxtrn,8)
         end if
         if ( jdgfd1==0 ) jdgfd1 = 1
         if ( jdgfd2==0 ) jdgfd2 = 2
         iinputkp(iinp+1) = jdgfd1
         iinputkp(iinp+2) = jdgfd2
         if ( jdgfd1<0 .or. jdgfd1>nunkp ) then
            call errint ( jdgfd1, 1 )
            call errint ( nunkp, 2 )
            call errint ( iobst, 3 )
            call errsub ( 1033, 3, 0, 0 )
         end if
         if ( jdgfd2<0 .or. jdgfd2>nunkp ) then
            call errint ( jdgfd2, 1 )
            call errint ( nunkp, 2 )
            call errint ( iobst, 3 )
            call errsub ( 1034, 3, 0, 0 )
         end if
         if ( jdgfd1==jdgfd2 ) then
            call errint ( iobst, 1 )
            call errsub ( 1035, 1, 0, 0 )
         end if
         iinsign = sign(1,iinputkp(iinp+5))
         if ( iobst<=maxobst ) then
            kstep = abs(jstep)+1
            call prob21 ( n, iobst, kstep, kmeshm, icurvsobst2d, ncurvs, &
                          iwork1 )
            if ( iinsign==-1 ) then

!           --- Reverse sequence in iwork1

               do k = 1, n
                  iwork(k) = iwork1(n+1-k)
               end do
               do k = 1, n
                  iwork1(k) = iwork(k)
               end do

            end if
            iwork = 0
            if ( jstep>=0 ) then

!           --- jstep>=  0   end points of intervals
!               Correct for initial and end point in the case that iexclude>0

               if ( iexclu==0) then

!              --- iexclu = 0  No excluding of nodal points

                  kstart = 1
                  kend = n

               else if ( iexclu==1) then

!              --- iexclu = 1 Exclude initial point

                  kstart = 2
                  kend = n

               else if ( iexclu==2) then

!              --- iexclu = 2 Exclude end point

                  kstart = 1
                  kend = n-1

               else if ( iexclu==3) then

!              --- iexclu = 3 Exclude initial and end point

                  kstart = 2
                  kend = n-1

               end if

               do k = kstart, kend, kstep
                  nodp = iwork1(k)
                  if ( iwork(nodp)==0 ) then
                     if ( indprf>0 ) then

!                    --- indprf>0  kprob part f contains the number of
!                        unknowns

                        inunkp = kprobf(nodp+1)-kprobf(nodp)
                        if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 100

                     end if
                     iwork(nodp) = 1
                     npltra = npltra+1
                  end if
100            end do

            else

!           --- jstep <0      points between intervals

               n = n-1
               do k = 1, n, kstep
                  do j = 1, kstep-1
                     nodp = iwork1(k+j)
                     if ( iwork(nodp)==0 ) then
                        if ( indprf>0) then

!                       --- indprf>0  kprob part f contains the number of
!                           unknowns

                           inunkp = kprobf(nodp+1)-kprobf(nodp)
                           if ( jdgfd1>inunkp .or. jdgfd2>inunkp ) goto 170
                        end if
                        iwork(nodp) = 1
                        npltra = npltra+1
                     end if

170               end do

               end do

            end if

         end if

      end do

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob09obst'

      end if  ! ( debug )
      call erclos ( 'prob09obst' )

      end
