      subroutine prob10tr ( maxtrn, jtrans, coortr, npltra, ndim, &
                            coor, iprec, inext, nodp, k, &
                            ifirst, ilast )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-09-2007 Remove array curves
!        version  1.1  date 26-05-2003 Debug statements
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute local transformation for points on a curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxtrn, jtrans, npltra, ndim, iprec, inext, &
              nodp, k, ifirst, ilast
      double precision coortr(maxtrn,*), coor(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coortr        i/o   kprob part n (local transformations)
!     ifirst         i    First curve number
!     ilast          i    Last curve number
!     inext          i    Next point in curve
!     iprec          i    Prior point in curve
!     jtrans         i    Type of local transformation
!                         1:  standard transformation
!                         2:  symmetric transformation
!                         3:  non-symmetric transformation
!     k              i    Sequence number along curve (only used for error
!                         message)
!     maxtrn         i    Maximum number of real variables per point with
!                         respect to the local transformations
!     ndim           i    Dimension of the space.
!     nodp           i    Nodal point sequence number
!     npltra         i    Number of local transformations
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indrecurs(5), imin, i
      double precision anx, any, anz, h, &
                       normal1(3), normal2(3), tang(3), hinv, amin

!     amin           Minimum value in an array
!     anx            x-component of normal vector
!     any            y-component of normal vector
!     anz            z-component of normal vector
!     h              Length of vector
!     hinv           1/h
!     i              Counting variable
!     imin           Position in array for which minimum is reached
!     indrecurs      Recursion array
!     normal1        First normal vector
!     normal2        Second normal vector
!     tang           In this array the tangential vector is stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB10TR01     Compute the components anx, any and anz of the tangential
!                    vector along a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute nx, ny and nz from tangential vector
!     Compute and test length h of normal vector
!     Normalize components
!     if ( ndim = 2 ) then
!        Compute coortr from the type of transformation and nx and ny
!     else ( ndim = 3 )
!        Find minimum absolute component of tangential vector ( nx, ny, nz)
!        Define first normal vector by 0 in place of minimum and orthogonal
!        to both other components
!        Normalize first normal vector
!        Compute second normal vector as outer product of tangential vector
!        and first normal vector
!        Normalize second normal vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indrecurs /1, 2, 3, 1, 2/
! ======================================================================
!
      call eropen ( 'prob10tr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10tr'
         write(irefwr,*) 'jtrans, ndim, maxtrn, npltra ', &
                          jtrans, ndim, maxtrn, npltra

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute nx and ny from tangential vector

      call prob10tr01 ( ndim, coor, anx, any, anz, iprec, inext, &
                        nodp, ifirst, ilast, h, k )
      if ( ierror/=0 ) go to 1000

!     --- Fill array coortr

      if ( ndim==2 ) then

!     --- ndim = 2, standard case

         if ( maxtrn==2 ) then

!        --- Standard situation

            coortr(1,npltra) = anx/h
            coortr(2,npltra) = -any/h
            if ( debug ) write(irefwr,*) 'coortr ', &
               coortr(1,npltra), coortr(2,npltra)

         else if ( jtrans==1 ) then

!        --- standard situation, however with (un)symmetric matrix

            coortr(1,npltra) = anx/h
            coortr(2,npltra) = any/h
            coortr(3,npltra) = -any/h
            coortr(4,npltra) = anx/h
            if ( maxtrn==8 ) then

!           --- unsymmetric storage

               coortr(5,npltra) = coortr(1,npltra)
               coortr(6,npltra) = coortr(3,npltra)
               coortr(7,npltra) = coortr(2,npltra)
               coortr(8,npltra) = coortr(4,npltra)

            end if

         else

!        --- jtrans = 4, (-un,ut)

            coortr(1,npltra) = -anx/h
            coortr(2,npltra) = -any/h
            coortr(3,npltra) = -any/h
            coortr(4,npltra) = anx/h
            if ( maxtrn==8 ) then

!           --- unsymmetric storage

               coortr(5,npltra) = coortr(1,npltra)
               coortr(6,npltra) = coortr(3,npltra)
               coortr(7,npltra) = coortr(2,npltra)
               coortr(8,npltra) = coortr(4,npltra)
            end if

         end if

      else

!     --- ndim = 3, curve in 3D
!         First determine two normal vectors
!         The tangential vector is anx, any, anz

         imin = 1
         amin = abs(anx)
         if ( abs(any)<amin ) then
            imin = 2
            amin = abs(any)
         end if
         if ( abs(anz)<amin ) then
            imin = 3
            amin = abs(anz)
         end if
         hinv = 1d0/h
         tang(1) = anx*hinv
         tang(2) = any*hinv
         tang(3) = anz*hinv
         hinv = 1d0/(tang(indrecurs(imin+1))**2+ &
                     tang(indrecurs(imin+2))**2)

!        --- Define first normal vector by 0 in place of minimum and orthogonal
!            to both other components
!            Normalize first normal vector

         normal1(imin) = 0d0
         normal1(indrecurs(imin+1)) = -tang(indrecurs(imin+2))*hinv
         normal1(indrecurs(imin+2)) =  tang(indrecurs(imin+1))*hinv

!        --- Compute second normal vector as outer product of tangential vector
!            and first normal vector

         call eloutr ( tang, normal1, normal2 )

!        --- Normalize second normal vector

         hinv = normal2(1)**2+normal2(2)**2+normal2(3)**2
         hinv = 1d0/hinv
         normal2(1) = normal2(1)*hinv
         normal2(2) = normal2(2)*hinv
         normal2(3) = normal2(3)*hinv

         do i = 1, 3
            coortr(3*i-2,npltra) = normal1(i)
            coortr(3*i-1,npltra) = normal2(i)
            coortr(3*i  ,npltra) = tang(i)
         end do

      end if

1000  call erclos ( 'prob10tr' )
      if ( debug ) write(irefwr,*) 'End prob10tr'

      end
