      subroutine probdfinput ( ichprb, iseqprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 06-08-2014 Extra debug
!        version  1.2  date 15-11-2012 Long integers
!        version  1.1  date 30-05-2012 Extension with default problem
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read arrays iinput and rinput for probdf
!     Store in ibuffr and buffer
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulekprob
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichprb, iseqprob

!     ichprb         i    Choice parameter consisting of two parts:
!                         ichoice and kread according to
!                         ichprb = ichoice + kread*100
!                         with
!                         kread     Indication if first record has been read
!                                   (>0) or not (0)
!                                   Possible values:
!                                   0:  first record has not been read
!                                   1:  first record has been read but not the
!                                       problem number
!                                   2:  The default problem number is used
!                                   >10:  first record has been read as well as
!                                       the problem number
!                                       The problem number is equal to kread-10
!                         ichoice    choice parameter.  possibilities:
!                         0    all input is read from the standard input file
!                         1    the input is stored by the user in array iinput
!                         2    the input is read from the standard input file
!                              and stored in array iinput
!                         3    previously read input is reapplied to changed
!                              mesh
!                         4    special case for seppost
!                              iinput(1) must have been filled with the
!                              memory management sequence number of the
!                              previous iinput.
!                              Boundary conditions, local transforms etc. are
!                              skipped
!                         5    special case for seppost
!                              iinput(1) must have been filled with the
!                              memory management sequence number of the previous
!                              iinput.
!                              iinput(2) must have been filled with the
!                              memory management sequence number of the previous
!                              rinput.
!                              Boundary conditions, local transforms etc. are
!                              skipped
!                         10   the input is read from the standard input file
!                              and stored in array iinput.
!                              No further actions are performed, so the only
!                              task is to fill iinput.
!     iseqprob       i    Sequence number of problem input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipiinp, iprinp, maxlen, maxrl, ichoice, kread, lengin, i, &
              lengrl

!     i              Loop variable
!     ichoice        See ichprb
!     ipiinp         Starting address of array IINPUT
!     iprinp         Starting address of array rinput
!     kread          See ichoice
!     lengin         Length of integer array iinput
!     lengrl         Length of real array rinput
!                    (in double precision variables)
!     maxlen         Declared length of input array
!     maxrl          Maximum number of reals that are allowed in the input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PROREA         Read input from standard input file
!     PROREADEFAULT  Defines array iinput for probdf in case of default problem
!     SEPCRPROBINFO  create a new array in kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probdfinput' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probdfinput'
         call ertrace
         write(irefwr,1) 'ichprb, iseqprob', ichprb, iseqprob
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ichoice = mod(ichprb,100)
      kread = ichprb/100
      lengrl = 0

      if ( debug ) write(irefwr,1) 'ichoice', ichoice

      if ( ichoice==1 .or. ichoice==5 ) then

         lengin = size(iinputtmp)
         if ( associated(rinputtmp) ) lengrl = size(rinputtmp)

      else if ( ichoice==3 ) then

!     --- Copy iinput/rinput into iinputtmp/rinputtmp
!         Store information into kprob
!         Reserve space for iinput

         iseqkp = kprob
         nprob = kp(iseqkp)%nprob
         iinputkp => kp(iseqkp)%iinputkp
         rinputkp => kp(iseqkp)%rinputkp
         lengin = size(iinputkp)
         lengrl = size(rinputkp)
         go to 1000

      else if ( ichoice==4 ) then

!     --- Copy iinput/rinput into iinputtmp/rinputtmp
!         Store information into kprob
!         Reserve space for iinput

         iseqkp = kprob2d(1)
         nprob = 1
         iinputkp => kp(iseqkp)%iinputkp
         rinputkp => kp(iseqkp)%rinputkp
         lengin = size(iinputkp)
         lengrl = size(rinputkp)

         allocate ( iinputtmp(lengin), rinputtmp(lengrl) )
         iinputtmp = iinputkp
         rinputtmp = rinputkp

      else

!     --- ichoice = 0,2,10 read from inputfile
!         Reserve temporary positions and check free space
!         20000 positions for integers, 1000 positions for reals

         maxrl = 1000
         maxlen = 20000

         allocate ( iinputtmp(maxlen), rinputtmp(maxrl), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, maxlen+maxrl, 'iinputtmp' )
         if ( ierror/=0 ) go to 1000

!        --- Read information into iinputtmp and rinputtmp

         if ( debug ) write(irefwr,1) 'before prorea: kread', kread

         if ( kread==2 ) then

!        --- kread = 2, default problem definition

            call proreadefault ( iinputtmp, ndim, lengin, lengrl )
            nprob = 1
            iseqprob = 1

         else

!        --- kread # 2, standard case

            call prorea ( iinputtmp, lengin, nprob, kread, &
                          rinputtmp, lengrl, maxlen, maxrl, iseqprob )

            if ( lengrl==0 .and. iinputtmp(52)>0 ) then

!           --- Periodical boundary conditions without jump

               rinputtmp(1:2*nelgrp:2) = 0d0
               rinputtmp(2:2*nelgrp:2) = 1d0
               lengrl = nelgrp*2

            end if  ! ( lengrl<=1 .and. iinputtmp(52)>0 )

         end if  ! ( kread==2 )
         iseqprob = max(1,iseqprob)
         if ( debug ) then

!        --- debug is true, print iinput and rinput

            write(irefwr,1) 'iseqprob, nprob', iseqprob, nprob
            write(irefwr,1) 'lengin, lengrl', lengin, lengrl
            call prinin ( iinputtmp, lengin, 'iinput' )
            call prinrl ( rinputtmp, lengrl, 'rinput' )

         end if

         if ( ierror/=0 ) go to 1000

      end if
      if ( ierror/=0 ) go to 1000

      call sepcrprobinfo ( kprob, nprob )

!     --- move iinput and rinput to final position

      lengrl = max(1,lengrl)
      allocate ( kp(iseqkp)%rinputkp(lengrl) )
      kp(iseqkp)%rinputkp = rinputtmp(1:lengrl)

!     --- We need an extra position to prevent an error in prob14

      allocate ( kp(iseqkp)%iinputkp(lengin+1) )
      kp(iseqkp)%iinputkp(1:lengin) = iinputtmp(1:lengin)
      kp(iseqkp)%iinputkp(lengin+1) = 0
      if ( debug ) write(irefwr,*) 'After filling iinput'

      if ( ichoice/=1 ) then
         deallocate ( iinputtmp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iinputtmp' )
         if ( associated(rinputtmp)) deallocate ( rinputtmp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'rinputtmp' )
      end if  ! ( ichoice/=1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lengin, lengrl', lengin, lengrl
         write(irefwr,1) 'iseqprob', iseqprob
         write(irefwr,*) 'End probdfinput'

      end if  ! ( debug )
      call erclos ( 'probdfinput' )

      end subroutine probdfinput
