      subroutine probfict00
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-10-2000 Allow shape 9
!        version  1.0  date 14-02-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find number of structural elements in which the fictitious unknowns
!    are stored and detect total number of fictitious unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, icurnr, icurve1, icurve2, igrp, &
              inunkp_elm, iposkproby, iregionlam, ishape_curve, &
              istructgrp, isurnr, nelemlc, &
              nfictpnt, inunkpgr

!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     icurve1        First curve number of a set
!     icurve2        Last curve number of a set
!     igrp           Boundary element group number read
!     inunkp_elm     Number of unknowns per point
!     inunkpgr       Number of global unknowns in a group
!     iposkproby     Position in kprob y
!     iregionlam     type of region where the lambda's are defined:
!                     1:  curves
!                     2:  surfaces
!     ishape_curve   Shape of lambda basis functions
!     istructgrp     Structural element group
!     isurnr         Surface sequence number
!     nelemlc        Local number of elements
!     nfictpnt       Total number of points with fictitious unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     152   last curve number has incorrect value
!     800   Shape number not available
!    1050   number of nodal points along the curves not correct for the
!           creation of line elements in fictitious element group
! **********************************************************************
!
!                       PSEUDO CODE
!
!    nfictgrp := kproby(1)
!    nfictunk := 0
!    For igrp := 1 (1) nfictgrp do
!       istructgrp := structural group corresponding to igrp
!       inunkp_elm := number of lambda's in element
!       icurve1 := first curve number for this group
!       icurve2 := last curve number for this group
!       iregionlam := type of region where the lambda's are defined
!
!       if ( region is of type curve ) then
!          ishape := shape number of structural element
!          if ( ishape = 3 or 5 ) then
!             ishape_curve := 1
!          else if ( ishape = 4 or 6 ) then
!             ishape_curve := 2
!          else
!             error: not yet implemented
!          nelem := number of elements along the curves
!          nfictunk := nfictunk+nelem*inunkp_elm
!       else if ( region is of type surface ) then
!          ishape := shape number of structural element
!          if ( ishape = 11 ) then
!             ishape_curve := 3
!          else if ( ishape = 12 ) then
!             ishape_curve := 4
!          else if ( ishape = 13 ) then
!             ishape_curve := 5
!          else if ( ishape = 14 ) then
!             ishape_curve := 6
!          else
!             error: not yet implemented
!          nelem := number of elements along the surfaces
!          nfictunk := nfictunk+nelem*inunkp_elm
!    Store information in kproby
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probfict00' )

!     --- Make some initializations

      nfictgrp = kproby(1)
      nfictunk = 0
      nfictpnt = 0

!     --- Loop over fictitious groups

      do igrp = 1, nfictgrp

!     --- Set some relevant parameters

         iposkproby = kproby(10+igrp)-1
         inunkpgr   = kproby(iposkproby+2)
         iregionlam = kproby(iposkproby+3)
         icurve1    = kproby(iposkproby+5)
         icurve2    = kproby(iposkproby+6)
         istructgrp = kproby(iposkproby+7)
         inunkp_elm = kproby(iposkproby+10)

         if ( iregionlam==1 ) then

!        --- region is of type curve
!            ishape := shape number of structural element

            if ( icurve2<icurve1 .or. icurve2>ncurvs ) then
               call errint ( icurve2, 1 )
               call errint ( icurve1, 2 )
               call errint ( ncurvs, 3 )
               call errsub ( 152, 3, 0, 0 )
            end if
            ishape = kmeshf(istructgrp)
            if ( ishape==3 .or. ishape.eq.5 .or. ishape.eq.9 ) then

!           --- ishape = 3 or 5

               ishape_curve = 1

            else if ( ishape==4 .or. ishape.eq.6 ) then

!           --- ishape = 4 or 6

               ishape_curve = 2

            else

!           --- error: shape number not yet implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

!           --- nelem := number of elements along the curves
!               nfictunk := nfictunk+nelem*inunkp_elm

            nelemlc = 0
            do icurnr = icurve1, icurve2
               i1 = kmeshm(4+icurnr)
               i2 = kmeshm(5+icurnr)-1
               if ( ishape_curve==1 ) then
                  nelemlc = nelemlc+i2-i1
               else
                  nelemlc = nelemlc+(i2-i1)/2
                  if ( nelemlc*2/=i2-i1 ) then
                     call errint ( igrp, 1 )
                     call errint ( nelemlc, 2 )
                     call errint ( i2-i1+1, 3 )
                     call errint ( nelemlc*2, 4 )
                     call errint ( icurnr, 5 )
                     call errsub ( 1050, 5, 0, 0 )
                  end if
               end if
            end do
            nfictunk = nfictunk+nelemlc*inunkp_elm
            nfictpnt = nfictpnt+nelemlc*inunkp_elm/inunkpgr

         else

!        --- region is of type surface
!            ishape := shape number of structural element

            ishape = kmeshf(istructgrp)
            if ( ishape==11 ) then

!           --- ishape = 11

               ishape_curve = 3

            else if ( ishape==12 ) then

!           --- ishape = 12

               ishape_curve = 4

            else if ( ishape==13 ) then

!           --- ishape = 13

               ishape_curve = 5

            else if ( ishape==14 ) then

!           --- ishape = 14

               ishape_curve = 6

            else

!           --- error: shape number not yet implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

!           --- nelem := number of elements along the surfaces
!               nfictunk := nfictunk+nelem*inunkp_elm

            nelemlc = 0
            do isurnr = icurve1, icurve2
               nelemlc = nelemlc+kmeshn(nsurfs+isurnr)
            end do
            nfictunk = nfictunk+nelemlc*inunkp_elm
            nfictpnt = nfictpnt+nelemlc*inunkp_elm/inunkpgr

         end if
         kproby(iposkproby+11) = nelemlc
         kproby(iposkproby+12) = ishape_curve

      end do

!     --- Store information in kproby

      kproby(2) = nfictunk
      kproby(3) = nfictpnt
1000  call erclos ( 'probfict00' )
      end
