      subroutine probparglob1 ( lenkprobhsend )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 03-05-2015 Minor change
!        version  1.1  date 13-08-2014 Correction ipkprflc
!        version  1.0  date 26-03-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute global quantities with respect to problem definition
!     Parallel case (node 1)
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: lenkprobhsend

!     lenkprobhsend i/o   Length of array kprobhsend
!                         At input declared length
!                         At output actual length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iblock, inode, indprfloc, ipkprflc, ipkprhlc, &
                 npointloc, icontext, leniwork, &
                 nrusolglob, lenkprobh, istart, nusolloc
      integer, allocatable :: numunknowns(:), iwork(:), iworkstore(:,:), &
                              kprobhloc(:)
      integer, pointer :: kprobfglob(:)

!     i              Counting variable
!     iblock         Block sequence number, ipstkprobpar
!     icontext       Tag for sending and receiving, is used as check
!     indprfloc      indprf for local node
!     inode          Node sequence number
!     ipkprflc       Starting address of local array KPROB part f
!     ipkprhlc       Starting address of local array KPROB part h
!     istart         First position used at a part
!     iwork          integer work array to receive data from other nodes
!     iworkstore     Contains the following information per node (.,:)
!                    1:  npoint (local)
!                    2:  nunkp (local)
!                    3:  nrusol (local)
!                    4:  lenkprobf (local)
!                    5:  lenkprobh (local)
!                    followed by array map (length npoint)
!                    followed by array kprobf (length lenkprobf)
!                    followed by array kprobh (length lenkprobh)
!     kprobfglob     Global array kprobf
!     kprobhloc      Local array kprobh
!     leniwork       Computed length of array iwork
!     lenkprobh      Length of array kprobh
!     npointloc      Number of points in local mesh
!     nrusolglob     Global number of unknowns
!     numunknowns    Array of length npointglob to store the number of
!                    degrees of freedom in all nodes
!     nusolloc       Local number of degrees of freedom (in block)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
!     PROBPARGLOB10  Fill global array of arrays kprobfglob for parallel case
!                    Only parts referring to vectors of special structure
!     PROBPARGLOB3   Fill array numunknowns for one block
!     PROBPARGLOB4   Fill positions in kprobhglob with a -1 if they correspond
!                    to prescribed boundary conditions
!     PROBPARGLOB5   Compute mapping from global kprobh to local kprobh for
!                    all nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3123   Number of unkowns in node larger than nunkp
! **********************************************************************
!
!                       PSEUDO CODE
!
!     create and clear array numunknowns(1:npoint_global)
!     Fill array numunknowns for all local nodes
!     For blocks := 2, number_of_blocks do
!        Read mapping array as well as kprobf and kprobh from block
!        Fill array numunknowns for all local nodes
!     Compute nusol_glob and fill array kprobf_glob if necessary
!     if ( kprobh must be filled ) then
!        create and clear array kprobh_glob
!        For all blocks do
!           Fill array kprobh from local kprobf/h and local mapping
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob1'
         if ( kprobloc(50)>0 ) call prinin ( kprobh, kprobloc(5), 'kprobh' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      nrusolloc = kprobloc(66)

      if ( debug ) &
         write(irefwr,1) 'npointglob, npoint, nunkp, nrusolloc, nusol', &
                          npointglob, npoint, nunkp, nrusolloc, kprobloc(5)

      leniwork = 5+(npointglob+1)*(nunkp+2)

!     --- create and clear array numunknowns(1:npoint_global)

      allocate ( numunknowns(npointglob), iwork(leniwork), &
                 iworkstore(leniwork,iacnodes), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npointglob+leniwork*(iacnodes+1), &
                        'numunknowns' )
      if ( ierror/=0 ) go to 1000

      numunknowns = 0
      iworkstore = 0

!     --- Fill array numunknowns for all local nodes

      call probparglob3 ( kprobloc(19), npoint, nunkp, numunknowns, &
                          mappingpar, kprobf )
      iworkstore(1,1) = npoint
      iworkstore(2,1) = nunkp
      iworkstore(3,1) = nrusolloc
      iworkstore(4,1) = 0                            ! indprf
      iworkstore(6:5+npoint,1) = mappingpar(1:npoint)       ! map
      ipkprflc = 6+npoint
      if ( kprobloc(19)>0 ) then
         iworkstore(4,1) = npoint+1  ! indprf
         iworkstore(ipkprflc:ipkprflc+npoint,1) = kprobf(1:npoint+1)
         ipkprhlc = ipkprflc+npoint+1
         lenkprobh = kprobf(npoint+1)*2
      else
         lenkprobh = nunkp*npoint*2
         ipkprhlc = ipkprflc
      end if  ! ( kprobloc(19)>0 )

      if ( kprobloc(50)>0 ) then
         iworkstore(5,1) = lenkprobh  ! indprh
         lenkprobh = size(kprobh)
         iworkstore(ipkprhlc:ipkprhlc+lenkprobh-1,1) = kprobh
         ipkprhlc = ipkprhlc+lenkprobh
         iworkstore(ipkprhlc:ipkprhlc+lenkprobh-1,1) = kprobhinv
      end if  ! ( kprobloc(19)>0 )

      if ( debug ) then
         call prinin ( numunknowns, npoint, 'numunknowns' )
         write(irefwr,1) 'leniwork', leniwork
      end if  ! ( debug )

!     --- For blocks := 2, number_of_blocks do
!            Read mapping array as well as kprobf and kprobh from block

      icontext = 42
      do inode = 2, iacnodes

         length = leniwork
         call inireceivempi ( iwork, length, SPINT, iblock, icontext )
         if ( debug ) write(irefwr,1) 'iblock, length', iblock, length
         iworkstore(1:length,iblock) = iwork(1:length)

!        --- Fill array numunknowns for all local nodes

         ipkprflc = 1
         if ( iwork(4)>0 ) ipkprflc = 6+iwork(1)
         call probparglob3 ( iwork(4), iwork(1), iwork(2), numunknowns, &
                             iwork(6), iwork(ipkprflc) )

      end do  ! inode = 2, iacnodes

!     --- Compute nusol_glob and fill array kprobf_glob if necessary

      nusolglob = 0
      do i = 1, npointglob

         nusolglob = nusolglob+numunknowns(i)
         if ( numunknowns(i)>nunkp ) then

!        --- Internal error:, number of unknowns larger than nunkp

            call errint ( i, 1 )
            call errint ( numunknowns(i), 2 )
            call errint ( nunkp, 3 )
            call errsub ( 3123, 3, 0, 0 )
            go to 1000

         end if  ! ( numunknowns(i)<nunkp )

      end do  ! i = 1, npointglob

      if ( debug ) write(irefwr,1) 'nusolglob', nusolglob

      if ( nusolglob==npointglob*nunkp ) then

!     --- number of unknowns per point constant

         indprf = 0
         kprobfglob => kprobf

      else

!     --- number of unknowns per point not constant, fill kprobf

         allocate ( kprobfglobal(0)%arr(npointglob+1), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, npointglob+1, 'kprobfglob' )
         if ( ierror/=0 ) go to 1000
         kprobfglob => kprobfglobal(0)%arr

         indprf = 1
         kprobfglob(1) = 0
         do i = 1, npointglob
            kprobfglob(i+1) = kprobfglob(i)+numunknowns(i)
         end do  ! i = 1, npointglob

         if ( debug ) &
            call prinin ( kprobfglob, npointglob+1, 'kprobfglob' )

      end if  ! ( nusolglob==npointglob*nunkp )

!     --- create array kprobglob

      allocate ( kprobhglob(nusolglob), kprobhloc(nusolglob), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 3*nusolglob, 'kprobhglob' )
      if ( ierror/=0 ) go to 1000

      kprobhglob = 0

!     --- Loop over nodes

      nrusolglob = 0
      do iblock = 1, iacnodes

         if ( debug ) &
            write(irefwr,1) 'iblock, indprhloc', &
                             iblock, iworkstore(5,iblock)
         if ( iworkstore(5,iblock)>0 ) then

!        --- iworkstore(5,iblock)>0, hence kprobhloc exists
!            Compute nrusolglob and fill kprobhglob with -1 for all
!            prescribed degrees of freedom

            npointloc = iworkstore(1,iblock)
            indprfloc = iworkstore(4,iblock)
            ipkprflc = 6+npointloc
            ipkprhlc = ipkprflc+indprfloc
            nusolloc = iworkstore(5,iblock)/2

            if ( debug ) &
               write(irefwr,1) 'indprfloc, ipkprflc, ipkprhlc, npointloc', &
                                indprfloc, ipkprflc, ipkprhlc, npointloc

            call probparglob4 ( nunkp, npointloc, iworkstore(6,iblock), &
                                indprf, indprfloc, &
                                iworkstore(ipkprflc,iblock), kprobfglob, &
                                iworkstore(3,iblock), &
                                iworkstore(ipkprhlc,iblock), kprobhglob, &
                                nusolloc, nrusolglob, iwork )

         end if  ! ( iworkstore(5,iblock)>0 )

      end do  ! iblock = 2, iacnodes

      if ( kprobloc(50)==0 ) then

!     --- No renumbering: fill kprobhglob for internal unknowns

         nrusolglob = 0
         do i = 1, nusolglob

            if ( kprobhglob(i)==0 ) then

!           --- internal unknowns

               nrusolglob = nrusolglob+1
               kprobhglob(i) = nrusolglob

            end if  ! ( kprobhglob(i)==0 )

         end do  ! i = 1, nusolglob

      end if  ! ( kprobloc(50)==0 )

      if ( debug ) call prinin ( kprobhglob, nusolglob, 'kprobhglob' )

!     --- Fill kprobhglob for internal unknowns

      if ( debug ) call prinin ( kprobhglob, nusolglob, 'kprobhglob' )

!     --- Fill mapping from global to local kprobh

      istart = iacnodes+1
      do iblock = 1, iacnodes

         npointloc = iworkstore(1,iblock)
         indprfloc = iworkstore(4,iblock)
         ipkprflc = 6+npointloc
         ipkprhlc = ipkprflc+indprfloc
         nusolloc = iworkstore(5,iblock)/2

         if ( debug ) then
            write(irefwr,1) 'iblock, npointloc, indprfloc, ipkprflc', &
                             iblock, npointloc, indprfloc, ipkprflc
            write(irefwr,1) 'ipkprhlc, nusolloc', ipkprhlc, nusolloc
         end if  ! ( debug )

         call probparglob5 ( nunkp, npointloc, kprobfglob, &
                             iworkstore(6,iblock), &
                             indprfloc, indprf, kprobhglob, &
                             nrusolloc, kprobhloc, &
                             iworkstore(ipkprflc,iblock), &
                             iworkstore(ipkprhlc,iblock), nusolloc )
         if ( debug ) then
            write(irefwr,*) 'iblock', iblock
            call prinin ( kprobhloc, nrusolloc, 'kprobhloc' )
         end if  ! ( debug )

!        --- Check array length

         call inichkarlen ( istart+nrusolloc, lenkprobhsend, &
                           'kprobhsend' )
         if ( ierror/=0 ) go to 1000

!        --- Store kprobhloc into array kprobhsend

         kprobhsend(iblock) = istart
         kprobhsend(istart) = nrusolloc
         kprobhsend(istart+1:istart+nrusolloc) = kprobhloc(1:nrusolloc)
         istart = istart+nrusolloc+1

      end do  ! iblock = 1, iacnodes

      lenkprobhsend = istart-1

      call probparglob10 ( numunknowns )

      deallocate ( kprobhglob, kprobhloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kprobhglob' )
      deallocate ( numunknowns, iwork, iworkstore, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'numunknowns' )

      kprobpar(1) = nusolglob
      kprobpar(2) = nrusolglob

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobhsend, lenkprobhsend, 'kprobhsend' )
         write(irefwr,*) 'End probparglob1'

      end if  ! ( debug )
      call erclos ( 'probparglob1' )

      end
