      subroutine probparinfo ( lensend2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 31-03-2015 Extra parameters
!        version  4.2  date 07-10-2014 Extra debug
!        version  4.1  date 24-03-2013 Use inibsendmpi
!
!   copyright (c) 2001-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Prepare arrays for solver with respect to parallel solution
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     problem_definition
! **********************************************************************
!
!                       MODULES USEDiseqlcol
!
      use sepmodulekmesh
      use sepmodulecpack
      use sepmodulekprob
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lensend2

!     lensend2       o    Length of first part of sendinfo
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, iblock, icontext, numneigh, inunkp, jnunkp, &
              lensendinfo, icount, inodp, &
              inusol, isendinfo, iseqlc, istart, istartinfo, k, nneigh, &
              istarttrans, l, m_i, n_ij, isend(npoint), &
              seqblocks(iacnodes), leninnercontr
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_size
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     icount         Counter
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     isend          Work array to send the number of degrees of freedom
!                    for each boundary point
!                    Maximum length: largest neighbor part in transinfo
!                    (< npoint)
!     isendinfo      Position in array sendinfo
!     iseqlc           Sequence number
!     istart         First position used at a part
!     istartinfo     Starting address to put information in array
!                    blockinfo
!     istarttrans    Starting address in array transinfo
!     iwork          integer work array of length nrusol
!                    iwork(i) contains the number of blocks unknown i
!                    is lying in
!     j              Counting variable
!     jnunkp         Number of degrees of freedom in second point
!     k              Counting variable
!     l              General loop variable
!     leninnercontr  Length of array innercontr
!     lensendinfo    Length of array sendinfo
!     m_i            The number of neighbor blocks of block i that are also
!                    neighbors of the present block
!     n_ij           nodes in the interface of iblock and ineigh that are also
!                    neighbor nodes of blocknr
!     nneigh         Number of neighbors of node
!     numneigh       number of neighbors of a point
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
!     seqblocks      Work array of length nblocks (=nsurfs) which
!                    is used to find the sequence of block numbers
!                    in array blockinfo
!                    seqblocks(iblock) = 0: block iblock not present
!                    seqblocks(iblock) = i: block iblock is the i-th block
!                    in blockinfo
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
!     PRINKMESHPAR   Print the contents of the parallel arrays blockinfo and
!                    transinfo
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2274   Incorrect number of integers read in mpi
!    2495   Number of unknowns in common points of two blocks not equal
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize array blockunkinfo
!     Set array innercontr to 1
!     Store sequence of block numbers in array seqblocks
!     For all neighbor blocks do
!        Store number of unknowns to be sent per point into isend
!        Send array isend
!     For i := 1 (1) number of neighbor blocks do
!        Receive next array and store in isend
!        Get corresponding block number
!        Store sequence number of unknowns to be received in sendinfo
!        Adapt array innercontr if necessary
!        Adapt array blockunkinfo
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparinfo' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparinfo'
         write(irefwr,1) 'numneigh', blockinfo(1)
         write(irefwr,1) 'iacnodes', iacnodes
         call prinkmeshpar ( blockinfo, transinfo )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize array blockunkinfo

      leninnercontr = 3*nrusol+1
      lensendinfo = max(nphys,nunkp)*size(transinfo)
      if ( associated(kprobpart%blockunkinfo) ) &
           deallocate(kprobpart%blockunkinfo)
      allocate ( kprobpart%blockunkinfo(size(blockinfo)), &
                 innercontr(leninnercontr), sendinfo(lensendinfo) )
      blockunkinfo => kprobpart%blockunkinfo
      numneigh = blockinfo(1)
      blockunkinfo(1) = numneigh
      inunkp = nunkp
      jnunkp = inunkp
      indprh = kprobloc(50)

      allocate ( iwork(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'iwork' )
      if ( ierror/=0 ) go to 1000

      blockunkinfo(2:1+numneigh*3) = 0
      isendinfo = 1

!     --- Set array innercontr to 1

      innercontr(1:nrusol) = 1

!     --- Clear array iwork

      iwork = 0

!     --- Store sequence of block numbers in array seqblocks

      seqblocks = 0
      do i = 1, numneigh
         iblock =  blockinfo(2*i)
         seqblocks(iblock) = i
      end do
      if ( debug ) call prinin ( seqblocks, iacnodes, 'seqblocks' )

!     --- For all neighbor blocks do

      send_buffer_size = 0
      do i = 1, numneigh
         istartinfo = blockinfo(2*i+1)
         nneigh = transinfo(istartinfo)
         send_buffer_size = send_buffer_size + nneigh*4  ! integers
      end do

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, numneigh )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

      do i = 1, numneigh

         iblock =  blockinfo(2*i)
         istartinfo = blockinfo(2*i+1)
         nneigh = transinfo(istartinfo)
         blockunkinfo(2*(i-1)+2) = iblock

!        --- Store number of unknowns to be sent per point into isend

         do j = 1, nneigh

            inodp = transinfo(istartinfo+j)
            if ( indprf==0 ) then

!           --- Number of unknowns constant per node

               istart = (inodp-1)*inunkp

            else

!           --- Number of unknowns not constant per node

               istart = kprobf(inodp)
               inunkp = kprobf(inodp+1) - istart

            end if
            isend(j) = inunkp

         end do

         if ( debug ) then
            write(irefwr,1) 'i, iblock', i, iblock
            call prinin ( isend, nneigh, 'isend' )
         end if  ! ( debug )

!        --- Send array isend

         icontext = iprob
         call inibsendmpi ( isend, nneigh, SPINT, iblock, icontext )

      end do

!     --- For i := 1 (1) number of neighbor blocks do

      do i = 1, numneigh

!     --- Receive next array and store in isend
!         Get corresponding block number

         icount = npoint
         icontext = iprob
         call inireceivempi ( isend, icount, SPINT, iblock, icontext )
         if ( ierror/=0 ) go to 1000
         iseqlc = seqblocks(iblock)
         istartinfo = blockinfo(2*iseqlc+1)
         nneigh = transinfo(istartinfo)

         if ( debug ) then
            write(irefwr,1) 'i, iblock, iseqlc, nneigh', i, iblock, iseqlc, nneigh
            call prinin ( isend, icount, 'isend_receive' )
         end if  ! ( debug )

         if ( nneigh/=icount ) then

!        --- Number of data read incorrect

            call errint ( icount, 1 )
            call errint ( nneigh, 2 )
            call errint ( inodenr, 3 )
            call errint ( iblock, 4 )
            call errsub ( 2274, 4, 0, 0 )
            go to 1000

         end if

!        --- Store sequence number of unknowns to be sent/received in sendinfo
!            Adapt array blockunkinfo

         blockunkinfo(2*(iseqlc-1)+3) = isendinfo
         icount = isendinfo

         do j = 1, nneigh

            inodp = transinfo(istartinfo+j)
            if ( indprf==0 ) then

!           --- Number of unknowns constant per node

               istart = (inodp-1)*inunkp

            else

!           --- Number of unknowns not constant per node

               istart = kprobf(inodp)
               jnunkp = kprobf(inodp+1) - istart

            end if
            if ( jnunkp/=isend(j) ) then

!           --- jnunkp # isend(j)

               call errint ( inodp, 1 )
               call errint ( inodenr, 2 )
               call errint ( jnunkp, 3 )
               call errint ( iblock, 4 )
               call errint ( isend(j), 5 )
               call errsub ( 2495, 5, 0, 0 )

            end if
            if ( debug ) &
               write(irefwr,1) 'jnunkp, indprh, istart', &
                                jnunkp, indprh, istart

            do k = 1, jnunkp

               inusol = istart+k
               if ( indprh>0 ) inusol = kprobh(inusol)
               if ( inusol<=nrusol ) then

!              --- Free unknown found, store in sendinfo

                  isendinfo = isendinfo+1
                  sendinfo(isendinfo) = inusol
                  iwork(inusol) = iwork(inusol)+1  !raise iwork

                  if ( iblock<inodenr ) then

!                 --- neighboring block has lower block number than present
!                     block. This means that this unknown does not contribute
!                     to the inner product for this block but only for the
!                     neighboring block
!                     Set innercontr to 0

                     innercontr(inusol) = 0

                  end if

               end if

            end do
            call inichkarlen ( isendinfo, size(sendinfo), 'sendinfo' )

         end do
         sendinfo(icount) = isendinfo-icount
         isendinfo = isendinfo+1

      end do

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi
      if ( ierror/=0 ) go to 1000

      lensend2 = isendinfo-1

!     --- Fill part 2 of the information
!         This information is needed because in a block subdivision we
!         must compute Az_i and this array has contributions in the neighbors
!         of each block
!         First loop over the neighbor blocks

      if ( iacnodes>2 ) then

!     --- iacnodes>2, extra information

         do i = 1, numneigh

            istartinfo = blockinfo(2*numneigh+1+i)  ! start of extra information
            m_i = blockinfo(istartinfo)  ! number of neighbor blocks of blocknr
                                         ! and iblock

            blockunkinfo(2*numneigh+1+i) = istartinfo
            blockunkinfo(istartinfo) = m_i

!           --- Loop over common neighbors

            do j = 1, m_i

               blockunkinfo(istartinfo+2*j-1) = &
                  blockinfo(istartinfo+2*j-1) !ineigh
               istarttrans = blockinfo(istartinfo+2*j) ! starting address in
                                                       ! transinfo
               blockunkinfo(istartinfo+2*j) = isendinfo ! start in sendinfo
               n_ij = transinfo(istarttrans)      ! number of nodes on
                           ! common side of blocks that are also neighbor
               icount = 0

!              --- Loop over nodes on common side

               do l = 1, n_ij

                  inodp = transinfo(istarttrans+l)
                  if ( indprf==0 ) then

!                 --- Number of unknowns constant per node

                     istart = (inodp-1)*inunkp

                  else

!                 --- Number of unknowns not constant per node

                     istart = kprobf(inodp)
                     jnunkp = kprobf(inodp+1) - istart

                  end if

!                 --- Loop over unknowns in node

                  do k = 1, jnunkp

                     inusol = istart+k
                     if ( indprh>0 ) inusol = kprobh(inusol)
                     if ( inusol<=nrusol ) then

!                    --- Free unknown found, store in sendinfo

                        icount = icount+1
                        sendinfo(isendinfo+icount) = inusol

                     end if  ! ( inusol<=nrusol )

                  end do  ! k = 1, jnunkp

               end do  ! l = 1, n_ij
               sendinfo(isendinfo) = icount
               isendinfo = isendinfo+icount
               if ( isendinfo>size(sendinfo) ) then

!              --- lensendinfo too small

                  call errint ( isendinfo, 1 )
                  call errint ( size(sendinfo), 2 )
                  call errchr ( 'sendinfo', 1 )
                  call errsub ( 550, 2, 0, 1 )

               end if
               isendinfo = isendinfo+1

            end do  ! j = 1, m_i

         end do  !   i = 1, numneigh

      end if  ! ( iacnodes>2 )

!     --- Fill information of number of blocks per unknown from iwork into
!         innercontr

      icount = 0
      istart = nrusol+1
      do i = 1, nrusol

         if ( iwork(i)/=0 ) then

!        --- iwork(i) # 0, hence an unknown at the boundary found

            icount = icount+1
            innercontr(istart+1) = i
            innercontr(istart+2) = iwork(i)+1
            istart = istart+2

         end if

      end do  ! i = 1, nrusol
      innercontr(nrusol+1) = icount
      if ( istart>leninnercontr ) then

!     --- lensendinfo too small

         call errint ( istart, 1 )
         call errint ( leninnercontr, 2 )
         call errchr ( 'innercontr', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if
      leninnercontr = istart
      if ( associated(kprobpart%innercontr) ) &
           deallocate(kprobpart%innercontr)
      allocate ( kprobpart%innercontr(leninnercontr) )
      kprobpart%innercontr = innercontr(1:istart)
      lensendinfo = isendinfo-1
      if ( associated(kprobpart%sendinfo) ) &
           deallocate(kprobpart%sendinfo)
      allocate ( kprobpart%sendinfo(lensendinfo) )
      kprobpart%sendinfo = sendinfo(1:lensendinfo)
      if ( associated(kprobpart%sendinfopar) ) &
           deallocate(kprobpart%sendinfopar)
      allocate ( kprobpart%sendinfopar(lensendinfo) )
      kprobpart%sendinfopar = sendinfo(1:lensendinfo)

      deallocate ( iwork, innercontr, sendinfo,  stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      sendinfo => kprobpart%sendinfo
      sendinfopar => kprobpart%sendinfopar
      innercontr => kprobpart%innercontr

1000  if ( debug ) then

!     --- Debug information

         call prinin ( sendinfo, isendinfo-1, 'sendinfo' )
         write(irefwr,*) 'End probparinfo'

      end if
      call erclos ( 'probparinfo' )

      end
