      subroutine probparkprobh1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Define interface unknowns and create new kprobhdd
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, inunkp, inusol, j, icpu, nfree, numneighblocks
      integer :: minnode, nodp, jnusol, ninterf, lenblock, istart, &
                 ineighblocks(iacnodes)
      integer, allocatable :: iwork(:), iwork1(:), blocks(:), iseqlc(:)

!     blocks         Work array
!                    is used to define the relation of a nodal point and its cpu
!     i              Counting variable
!     icpu           Loop variable over the various cpu's
!     ineighblocks   Contains neighbor blocks in increasing sequence
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     iseqlc           Sequence number
!     istart         First position used at a part
!     iwork          integer work array
!                    Is used for several purposes but finally to store the
!                    relative new numbering of the interface unknowns
!     iwork1         integer work array
!     j              Counting variable
!     jnusol         Position of unknown in array USOL
!     lenblock       Length of array blocks
!     minnode        Minimum nodal point number in part
!     nfree          Number of free unknowns including interface unknowns
!     ninterf        Number of interface unknowns
!     nodp           Nodal point sequence number
!     numneighblocks Number of neighborblocks with smaller number than present one
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getinusol

!     CHSORT         Sort integer array for increasing sequence
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     GETINUSOL      Compute number of degrees of freedom in node, as
!                    well as last position in usol before this node
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparkprobh1' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparkprobh1'
         call ertrace
         call prinin ( mappingpar, npoint, 'mappingpar')
         call prinin ( globnodes, iacnodes, 'globnodes' )
         call prinin ( kprobhdd, size(kprobhdd), 'kprobhdd' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      inunkp = nunkp

      lenblock = globnodes(inodenr-1)
      ninterfdeg = 0

      allocate ( iwork(nusol), iwork1(nusol), blocks(lenblock), &
                 iseqlc(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*nusol+lenblock, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Store block numbers for all nodes in blocks

      istart = 1
      do i = 1, inodenr-1
         blocks(istart:globnodes(i)) = i
         istart = globnodes(i)+1
      end do  ! i = 1, inodenr-1

!     --- Fill array iwork1 with the node number for all degrees of freedom at
!         the interface and zero in other positions

      iwork = 0

      minnode = globnodes(inodenr-1)+1  ! smallest node in this block
      ninterf = 0

      if ( debug ) write(irefwr,1) 'minnode', minnode
      do i = 1, npoint

         nodp = mappingpar(i)

         if ( debug ) write(irefwr,1) 'i, nodp', i, nodp

         if ( nodp<minnode ) then

!        --- Node at interface

            inusol = getinusol ( i, kprobf, indprf, inunkp )

            if ( debug ) write(irefwr,1) 'inusol', inusol

            do j = 1, inunkp

               jnusol = inusol+j

               if ( kprobhdd(jnusol)<=nrusol ) then

!              --- Free degree of freedom at interface
!                  The node number is stored in iwork1
!                  The degree of freedom in iwork

                  ninterf = ninterf+1
                  iwork1(ninterf) = nodp
                  iwork(ninterf) = jnusol

               end if  ! ( kprobhdd(jnusol)<=nrusol )

            end do  ! j = 1, inunkp

         end if  ! ( nodp<=minnode )

      end do  ! i = 1, npoint

      if ( debug ) then
         call prinin ( iwork1, ninterf, 'iwork1' )
         call prinin ( iwork, ninterf, 'iwork' )
      end if  ! ( debug )

!     --- Replace positions in iwork1 by block number

      iwork1(1:ninterf) = blocks(iwork1(1:ninterf))

      do i = 1, ninterf
         icpu = iwork1(i)
         ninterfdeg(icpu+1) = ninterfdeg(icpu+1)+1
      end do  ! i = 1, ninterf

      if ( debug ) then
         call prinin ( iwork1, ninterf, 'iwork1' )
         call prinin ( ninterfdeg(2), iacnodes, 'ninterfdeg' )
      end if  ! ( debug )

      if ( inodenr==2 ) then

         iwork1(1:ninterf) = iwork(1:ninterf)

      else

!     --- Fill array blocks with neighbor blocks

         numneighblocks = 0
         do i = 1, blockinfo(1)
            icpu = blockinfo(2*i)
            if ( icpu<inodenr) then
               numneighblocks = numneighblocks+1
               blocks(numneighblocks) = icpu
            end if  ! ( icpu<inodenr)

         end do  ! i = 1, blockinfo(1)

!        --- Fill array blocks, result in ineighblocks

         call chsort ( blocks, iseqlc, numneighblocks )
         ineighblocks(1:numneighblocks) = blocks(iseqlc(1:numneighblocks))

!        --- Sort array iwork1, in order to start with the lowest cpu

         istart = 1
         do i = 1, numneighblocks
            icpu = ineighblocks(i)
            do j = 1, ninterf
               if ( iwork1(j)==icpu ) then
                  iseqlc(istart) = j
                  istart = istart+1
               end if  ! ( iwork1(j)==icpu )
            end do  ! j = 1, ninterf
         end do  ! i = 1, blockinfo(1)

         if ( debug ) &
            write(irefwr,1) 'nusol, nrusol, ninterf', &
                             nusol, nrusol, ninterf

         if ( debug )  call prinin ( iseqlc, ninterf, 'iseqlc' )

!        --- Store new sequence of iwork into iwork1

         iwork1(1:ninterf) = iwork(iseqlc(1:ninterf))

         if ( debug )  call prinin ( iwork1, ninterf, 'iwork1' )

      end if  ! ( inodenr==2 )

!     --- Fill array iwork with interface degrees of freedom

      iwork = 0
      do i = 1, ninterf
         iwork(iwork1(i)) = i
      end do  ! i = 1, ninterf

      if ( debug )  call prinin ( iwork, nusol, 'iwork' )

      nfree = nrusol
      nrusol = nrusol-ninterf  ! new number of unknowns

!     --- Fill array kprobhdd
!         First all free unknowns, then the interface degrees of freedom
!         and finally the prescribed unknowns

      jnusol = 0

      do i = 1, nfree

         inusol = kprobhddinv(i)
         if ( iwork(inusol)/=0 ) then

!        --- Interface unknowns

            kprobhdd(inusol) = iwork(inusol)+nrusol

         else

!        --- Internal unknown

            jnusol = jnusol+1
            kprobhdd(inusol) = jnusol

        end if  ! ( iwork(inusol)/=0 )
        iseqlc(i)= kprobhdd(inusol)

        if ( debug ) &
           write(irefwr,1) 'i, inusol, jnusol', i, inusol, jnusol

      end do  ! i = 1, nusol

      if ( debug ) then
         call prinin ( iseqlc, nfree, 'iseqlc' )
         call prinin ( kprobhglob, nrusol+ninterf, 'kprobhglob' )
      end if  ! ( debug )

!     --- Renumber kprobhglob

      iwork(iseqlc(1:nfree)) = kprobhglob(1:nfree)
      kprobhglob(1:nfree) = iwork(1:nfree)

!     --- Fill array kprobhddinv

      do i = 1, nusol
         kprobhddinv(kprobhdd(i)) = i
      end do  ! i = 1, nusol

!     --- Accumulate ninterfdeg

      do i = 1, iacnodes
         ninterfdeg(i+1) = ninterfdeg(i+1)+ninterfdeg(i)
      end do  ! i = 1, ninterf

!     --- Store new values of ninterf and nrusol

      kprobloc(67) = ninterf

      deallocate ( iwork, iwork1, blocks, iseqlc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobhdd, nusol, 'kprobhdd' )
         call prinin ( kprobhddinv, nusol, 'kprobhddinv' )
         call prinin ( ninterfdeg, iacnodes+1, 'ninterfdeg' )
         call prinin ( kprobhglob, nfree, 'kprobhglob' )
         write(irefwr,1) 'ninterf, nfree', ninterf, nfree
         write(irefwr,*) 'End probparkprobh1'

      end if  ! ( debug )
      call erclos ( 'probparkprobh1' )

      end subroutine probparkprobh1
