      subroutine prord3surf ( iinputlc, iinplc, table, tabout, tabmax, &
                              code, nessrf, lensrf, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-07-2014 Extra error message
!        version  1.0  date 30-03-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interprete surface information read by prord3 and store in iinputlc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     surface
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  iinputlc(*), iinplc, nessrf, lensrf, ncodmx(*)
      double precision table(*), tabout(*), tabmax(*)
      character (len=*) code(*)

!     code           i    Table with keywords
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     lensrf         i    Length of surfaces part
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nessrf         i    Number of parts where surface essential boundary
!                         conditions are given
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposut, inum, m, iskipbn, jnpelm, ncurves, iinplcbef, i, &
              inpelm

!     i              Counting variable
!     iinplcbef      Value of iinplc at start
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     jnpelm         Number of nodes in an element
!     m              number of integration points
!     ncurves        Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READC301       Check the numrel numbers stored in array relnods
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3190   number of nodes per element in set of surfaces not constant
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord3surf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3surf'
         write(irefwr,1) 'iinplc, nessrf', iinplc, nessrf
         write(irefwr,2) 'table(5), tabmax(5)', table(5), tabmax(5)
         iinplcbef = iinplc
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d13.5) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nessrf = nessrf+1
      iinputlc(iinplc+2) = 3

      if ( table(5)>0d0 ) then

!     --- Standard case: surfaces ...

         iposut = nint(table(5))
         inum = nint(tabmax(5))
         m = inum/10000          ! number of surfaces
         inum = mod(inum,10000)  ! number of local nodes

         if ( debug ) write(irefwr,1) 'iposut, inum, m', iposut, inum, m

         iinputlc(iinplc+4) = m  ! number of surfaces
         iinputlc(iinplc+5:iinplc+4+m) = &
            nint(tabout(iposut:iposut+m-1))  ! set of surfaces
         jnpelm = sepgetinpelsurf ( iinputlc(iinplc+5), 0 )
         iinputlc(iinplc+3) = jnpelm

         if ( debug ) then
            write(irefwr,1) 'jnpelm', jnpelm
            call prinin ( iinputlc(iinplc+1), 4+m, 'iinputlc' )
         end if  ! ( debug )

         if ( m>1 ) then

!        --- m > 1, check number of nodes per element

            do i = 2, m

               inpelm = sepgetinpelsurf ( iinputlc(iinplc+4+i), 0 )

               if ( inpelm/=jnpelm ) then

!              --- Error: inpelm#jnpelm

                  call errint ( iinputlc(iinplc+5), 1 )
                  call errint ( jnpelm, 2 )
                  call errint ( iinputlc(iinplc+4+I), 3 )
                  call errint ( inpelm, 4 )
                  call errsub ( 3190, 4, 0, 0 )

               end if  ! ( inpelm/=jnpelm )

            end do  ! i = 2, m

         end if  ! ( m>1 )

         iinplc = iinplc+4+m

         if ( inum<=0 ) then

!        --- inum<=0

            iinputlc(iinplc+1:iinplc+jnpelm) = 1

         else

!        --- inum>0,  copy contents from work into iinput

            call readc301 ( inum, jnpelm, tabout(iposut+m), work )

            if ( debug ) then
               call prinrl ( tabout(iposut+m), inum, 'tabout' )
               call prinrl ( work, inum, 'work' )
            end if  ! ( debug )

            iinputlc(iinplc+1:iinplc+jnpelm) = nint(work(1:jnpelm))

         end if
         iinplc = iinplc+jnpelm
         iskipbn = 0
         ncurves = 0
         if ( table(14)>0d0 ) then

!        --- skip boundary found

            if ( table(15)>0d0 ) &
               call errkeywexcl ( code, ncodmx, 15, 14 )
            if ( table(16)>0d0 ) &
               call errkeywexcl ( code, ncodmx, 16, 14 )

         else if ( table(15)>0d0 ) then

!        --- skip curves

            if ( table(16)>0d0 ) &
               call errkeywexcl ( code, ncodmx, 15, 16 )
            iskipbn = 2

         else if ( table(16)>0d0 ) then

!        --- include curves

            iskipbn = 3

         end if

         if ( iskipbn>1 ) then

!        --- iskipbn>1,  copy curves from work into iinput

            ncurves = nint(tabmax(13+iskipbn))
            iposut = nint(table(13+iskipbn))
            iinputlc(iinplc+3:iinplc+2+ncurves) = &
               nint(tabout(iposut:iposut+ncurves-1))

         end if
         iinputlc(iinplc+1) = iskipbn
         iinputlc(iinplc+2) = ncurves
         iinplc = iinplc+2+ncurves
         lensrf = lensrf+jnpelm+5+ncurves+m

      else  ! ( table(25)>0d0 )

!     --- Special case: outer_surfaces ...

         iinputlc(iinplc+3) = -10    ! all nodes are used
         iinplc = iinplc+3
         lensrf = lensrf+3

      end if  ! ( table(5)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinplc, lensrf', iinplc, lensrf
         call prinin ( iinputlc(iinplcbef+1),iinplc-iinplcbef, &
                       'iinputlc' )
         write(irefwr,*) 'End prord3surf'

      end if  ! ( debug )
      call erclos ( 'prord3surf' )

      end
