      subroutine prordperbccrv ( table, code, ncodmx, tabmax, tabout, &
                                 numcurves, maxcurves, iinputcrv, &
                                 tabmin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret records for definition of periodical boundary conditions
!     Part curves
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcurves, ncodmx(*), maxcurves, iinputcrv(3,maxcurves)
      double precision table(*), tabmax(*), tabout(*), tabmin(*)
      character (len=*) code(*)

!     code           i    Table with keywords, see subroutine prordperbc
!     iinputcrv     i/o   Array of size 3 x maxcurves in which the input read
!                         is stored
!                         See subroutine prordperbc for a description
!     maxcurves      i    Size of array iinputcrv
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     numcurves      o    Number of curve pairs stored in iinputcrv
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut, ifirst, ilast

!     ifirst         First curve number read
!     ilast          Last curve number read
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     317   record with pelm contains more than two user points
!    1432   User point connected to itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordperbccrv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordperbccrv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First check if keywords are not mutually exclusive

      if ( table(3)>0d0 ) call errkeywexcl ( code, ncodmx, 2, 3 ) !surfs
      if ( table(4)>0d0 ) call errkeywexcl ( code, ncodmx, 2, 4 ) !excl
      if ( table(5)>0d0 ) call errkeywexcl ( code, ncodmx, 2, 5 ) !all
      if ( table(9)>0d0 ) call errkeywexcl ( code, ncodmx, 2, 9 ) !C
      inum = nint(tabmax(2))  ! number of user curves read (must be 2)
      if ( inum/=2 ) then

!     --- Error, not two user curves read

         call errint ( inum, 1 )
         call errsub ( 317, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

      numcurves = numcurves+1
      call inichkarlen ( numcurves, maxcurves, 'iinputcrv' ) ! check array len
      if ( ierror/=0 ) go to 1000

      iposut = nint(table(2))  ! position of user curves in tabout
      ifirst = nint(tabout(iposut))  ! first user curve
      ilast = nint(tabout(iposut+1)) ! last user curve
      if ( ilast==ifirst ) then

!     --- error, both user curves are identical

         call errint ( ilast, 1 )
         call errsub ( 1432, 1, 0, 0 )
         call errcrd

      end if

!     --- Store curves in iinputcrv

      iinputcrv(1,numcurves) = ifirst           ! first curve number
      iinputcrv(2,numcurves) = ilast            ! last curve number
      iinputcrv(3,numcurves) = nint(tabmin(2))  ! parameter l

1000  call erclos ( 'prordperbccrv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numcurves', numcurves
         call prinin ( iinputcrv(1,numcurves), 3, 'iinputcrv' )
         write(irefwr,*) 'End prordperbccrv'

      end if  ! ( debug )

      end
