      subroutine sepbounlevset1 ( bndedges, lenbndedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array bndedges with edges corresponding to the boundary
!     of the region with elements with levelset>0
!     Old subroutine prbounlevset1
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     edge
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lenbndedges, bndedges(lenbndedges)

!     bndedges       o    contains edge sequence numbers of boundary
!                         of levelset
!                         bndedges(i) contains starting address of level set i
!                         bndedges(istart) contains the number of edges
!                         in boundary
!                         bndedges(istart+j) the edge sequence numbers
!     lenbndedges   i/o   At input: Declared length of array iobstfaces
!                         At output: Computed length of array iobstfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, ielem, ipstart, nedge_elem, node, &
              numinelmobst, numedges, iedge, ilev, istart, kstart, &
              jstart, numpntobst, numinnobst, numbndobst, nnode_edge
      integer, allocatable :: iwork(:), iwork1(:)

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ilev           Sequence number of level being considered
!     ipstart        Starting address in array
!     istart         First position used at a part
!     iwork          integer work array to mark nodes
!                    iwork(i) contains number of nodes found on boundary for
!                    edge i
!     iwork1         integer work array to mark nodes with level set zero
!     j              Counting variable
!     jstart         starting position in array
!     k              Counting variable
!     kstart         Next free position in bndedges
!     nedge_elem     number of edges corresponding to element
!     nnode_edge     number of nodes corresponding to edge
!     node           Node number
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numedges       number of edges
!     numinelmobst   Number of elements that are completely within the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbounlevset1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbounlevset1'
         write(irefwr,1) 'nedges', nedges
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Allocate work space

      allocate ( iwork(nedges), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges+npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork arrays

      iwork = 0
      iwork1 = 0

!     --- Loop over all levels

      kstart = nlevelsets+1

      do ilev = 1, nlevelsets

         jstart = ipointobst(ilev)
         numpntobst = ipointobst(jstart)
         numinnobst = ipointobst(jstart+1)
         numbndobst = ipointobst(jstart+2)       ! number of nodes in ipointobst
         if ( debug ) then
            write(irefwr,1) 'ilev, numpntobst, numinnobst, numbndobst', &
                             ilev, numpntobst, numinnobst, numbndobst
            write(irefwr,1) 'jstart', jstart
            call prinin ( ipointobst(jstart), numbndobst, &
                          'ipointobst/1' )
         end if  ! ( debug )
         jstart = jstart+numpntobst+numinnobst+4

!        --- Mark nodes of boundary in iwork1

         do i = 0, numbndobst-1
            iwork1(ipointobst(jstart+i)) = 1
         end do  ! i = 0, numbndobst-1

         istart = ielemobst(ilev)
         numinelmobst = ielemobst(istart)+ielemobst(istart+1)
         istart = istart+3

!        --- Mark all edges in inner elements inside the levelset function

         do j = istart, numinelmobst-1+istart

            ielem = ielemobst(j)
            ipstart = nmeshd(ielem)  ! starting address of element in nmeshd
            nedge_elem = nmeshd(ipstart)  ! number of edges of ielem

            if ( debug ) then
               write(irefwr,1) 'j, ielem, ipstart, nedge_elem', &
                                j, ielem, ipstart, nedge_elem
               call prinin ( nmeshd(ipstart+1), nedge_elem, 'edges' )
            end if  ! ( debug )

            do k = 1, nedge_elem
               iedge = abs(nmeshd(ipstart+k))
               iwork(iedge) = iwork(iedge)+1  ! raise iwork for edge
            end do  ! k = 1, nedge_elem

         end do  ! j = istart, numinelmobst-1+istart

!        --- Find all edges that appear only once.
!            These are the boundary edges

         bndedges(ilev) = kstart    ! starting address
         numedges = 0
         do j = 1, nedges

            if ( iwork(j)==1 ) then

!           --- iwork(j) = 1, hence boundary edge found
!               Check if both nodes belong to ipointobst

               ipstart = nmeshc(j)
               nnode_edge = nmeshc(ipstart)
               do i = 1, nnode_edge
                  node = nmeshc(ipstart+i)
                  if ( iwork1(node)==0 ) go to 100  ! jump edge not on bound
               end do  ! j = 1, nnode_edge

!              --- If the program reaches this place an edge has been found

               numedges = numedges+1        ! raise number of edges
               bndedges(kstart+numedges) = j     ! store edge

            end if  ! ( iwork(j)==1 )

100      end do  ! j = 1, nedges
         bndedges(kstart) = numedges  ! store number of edges
         kstart = kstart+1+numedges

         if ( ilev<nlevelsets ) then

!        --- ilev<nlevelsets, clear iwork, iwork1

            iwork = 0  ! clear iwork
            do i = 0, numbndobst-1
               iwork1(ipointobst(jstart+i)) = 0   ! clear iwork1
            end do  ! i = 0, numbndobst-1

         end if  ! ( ilev<nlevelsets )

      end do  ! ilev = 1, nlevelsets

!     --- Deallocate work space

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      lenbndedges = kstart-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenbndedges', lenbndedges
         do i = 1, nlevelsets
            write(irefwr,1) 'level set sequence number', i
            kstart = bndedges(i)
            numedges = bndedges(kstart)
            call prinin ( bndedges(kstart+1), numedges, 'bndedges' )
         end do  ! i = 1, nlevelsets
         write(irefwr,*) 'End sepbounlevset1'

      end if  ! ( debug )
      call erclos ( 'sepbounlevset1' )

      end
