      subroutine sepcommatinter
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parts of array intmat with respect to interface unknowns
!     Is only used in case of parallel computations
!     Old subroutine  incommatinterf
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenfi, lenif
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     lenfi          Length of array icolsfi
!     lenif          Length of array icolsif
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMMATINTER1 Fill arrays irowsfi and irowsif for integer information of
!                    matrix corresponding to interface unknowns
!     INCOMMATINTER2 compute integer information of matrix corresponding to
!                    interface unknowns in case of parallel computing
!     SEPCOMMATPAR   Create the structure of the off-diagonal matrix blocks
!                    Parallel case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Interface unknowns are those unknowns that are positioned at
!     blocks with lower block number than the present one
!     The large matrix is split into 9 parts:
!
!     | S_ii S_if S_ip |
!     | S_fi S_ff S_fp |
!     | S_pi S_pf S_pp |
!
!     where i refers to all internal unknowns at this block
!           p refers to the prescribed unknowns
!       and f refers to the interface unknowns
!     The structure of the matrix is symmetric even if the the matrix
!     is asymmetric.
!
!     In this part the structure of the S_fi, S_ff and S_fp is computed
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatinter' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatinter'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Get parameters

      if ( nrusol==0 ) go to 1000  ! No internal unknowns

!     --- Create arrays irowsfi and irowsif

      allocate ( IM%rowsfi(1:ninterface+1), IM%rowsif(1:nrusol+1), stat = error )
      if ( error/=0 ) call eralloc ( error, ninterface+nrusol+2, 'irowsfi' )
      if ( ierror/=0 ) go to 1000

      irowsfi(0:ninterface) => IM%rowsfi
      irowsif(0:nrusol) => IM%rowsif

      call incommatinter1

!     --- Fill array icolsfi

      lenfi = irowsfi(ninterface)
      lenif = irowsif(nrusol)

      if ( lenfi>0 ) then

!     --- lenfi > 0, create icolsfi

         allocate ( IM%colsfi(lenfi), stat = error )
         if ( error/=0 ) call eralloc ( error, lenfi, 'icolsfi' )
         if ( ierror/=0 ) go to 1000
         icolsfi => IM%colsfi

      end if  ! ( lenfi>0 )
      if ( lenif>0 ) then

!     --- lenif > 0, create icolsif

         allocate ( IM%colsif(lenif), stat = error )
         if ( error/=0 ) call eralloc ( error, lenif, 'icolsif' )
         if ( ierror/=0 ) go to 1000
         icolsif => IM%colsif

      end if  ! ( lenif>0 )

      call incommatinter2

!     --- Send information to other nodes and make the structure for the
!         off-diagonal blocks of the matrix

      call sepcommatpar

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcommatinter'

      end if  ! ( debug )
      call erclos ( 'sepcommatinter' )

      end subroutine sepcommatinter
