      subroutine sepcommatnest2 ( irowsnew, icolsnew, rows, cols, lencols )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array cols
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irowsnew(nrusol+1), &
                             icolsnew(irowsnew(nrusol+1)), lencols
      integer, intent(inout) :: rows(nrusol+1)
      integer, intent(out) :: cols(lencols)

!     cols           o    Work array to store the column numbers of the matrix
!                         including fill in upper triangular part
!     icolsnew       i    Work array to store the column numbers of the matrix
!                         upper triangular part
!     irowsnew       i    Work array to store starting addresses (-1) in
!                         icolsnew
!     lencols        i    length of array cols
!     rows          i/o   Work array to store starting addresses (-1) in cols
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icount, j, k, column, columnnew, ih
      integer, allocatable :: iwork(:), istore(:), ihelp(:)

!     column         column number
!     columnnew      new column number
!     i              Counting variable
!     icount         Counter
!     ih             Help variable to store a constant temporarily
!     ihelp          integer work array for sorting
!     istore         contains column numbers in a row
!     iwork          integer work array to mark nodes in a row
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3228   Computed length of array larger than declared length
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatnest2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatnest2'
         write(irefwr,1) 'nrusol', nrusol
         write(irefwr,*) 'irowsii/icolsii'
         call sepprincompstr ( nrusol, irowsii, icolsii )
         write(irefwr,*) 'irowsnew/icolsnew'
         call sepprincompstr ( nrusol, irowsnew, icolsnew )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(nrusol+1), istore(nrusol), ihelp(nrusol) )

!     --- Clear array iwork

      iwork = 0

      rows(1:2) = irowsnew(1:2)    ! Row 1 remains unchanged
      cols(1:rows(2)) = icolsnew(1:rows(2))

!     --- Loop over other rows

      do i = 2, nrusol

         ih = irowsnew(i+1)-irowsnew(i) ! number of unknowns in row of icolsnew
         iwork(icolsnew(irowsnew(i)+1:irowsnew(i+1))) = 1 ! mark all nodes
         icount =  ih                   ! number of unknowns in new row
         istore(1:icount) = icolsnew(irowsnew(i)+1:irowsnew(i+1))

!        --- Loop over all rows in icolsii corresponding to row i
!            only those with lower row number

         do j = irowsii(i)+1, irowsii(i+1)-1 ! skip last one

            column = icolsii(j)

            do k = rows(column)+2, rows(column+1) ! skip first one

               columnnew = cols(k)  ! new column number
               if ( columnnew>i .and. iwork(columnnew)==0 ) then

!              --- New column number has not been stored yet
!                  Store in istore and mark in iwork

                  icount = icount+1
                  istore(icount) = columnnew
                  iwork(columnnew) = 1

               end if  ! ( iwork(columnnew)==0 )

            end do  ! k = irowsnew(column)+2, irowsnew(column+1)

         end do  ! j = irowsii(i)+1, irowsii(i)

         rows(i+1) = rows(i)+icount  ! Store number in rows (accumulated)
         iwork(istore(1:icount)) = 0 ! reset iwork

!        --- Sort istore

         if ( rows(i+1)>lencols ) then

!        --- Declared length too small

            call errint ( rows(i+1), 1 )
            call errint ( lencols, 2 )
            call errchr ( 'cols', 1 )
            call errsub ( 3228, 2, 0, 1 )
            go to 1000

         end if  ! ( rows(i+1)>lencols )

         if ( icount>ih ) then

!        --- Sort istore and copy into cols

            call chsort ( istore, ihelp, icount )
            cols(rows(i)+1:rows(i+1)) = istore(ihelp(1:icount))

         else

!        --- copy into cols

            cols(rows(i)+1:rows(i+1)) = istore(1:icount)

         end if  ! ( icount>ih )

      end do  ! i = 2, nrusol

      deallocate ( iwork, istore, ihelp )

1000  if ( debug ) then

!     --- Debug information

         call sepprincompstr ( nrusol, rows, cols )
         write(irefwr,*) 'End sepcommatnest2'

      end if  ! ( debug )
      call erclos ( 'sepcommatnest2' )

      end subroutine sepcommatnest2
