      subroutine sepcuthill ( numnodes, points )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Reversed Cuthill-McKee for a subset
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numnodes
      integer, target, intent(inout) :: points(numnodes)

!     numnodes       i    Number of nodal points in array points
!     points        i/o   Contains the set of nodal point numbers
!                         At input: the original set
!                         At output: the renumbered set
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, &
                 levellength, startnode, lenlev, nremain, jstart
      integer, allocatable :: copymark(:), copyset(:)

!     copymark       Contains temporary copy of array marknodes
!     copyset        Contains temporary copy of array set
!     i              Counting variable
!     istart         First position used at a part
!     jstart         starting position in array
!     lenlev         Size of array levelstruc
!     levellength    Length of rooted level structure
!     nremain        Numbers of nodes that has not been reordered yet
!     startnode      Starting node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepstartnode

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPROOTLEVEL   Find rooted level structure corresponding to node
!     SEPSTARTNODE   Find start node in a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcuthill' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcuthill'
         call prinin ( points, numnodes, 'points' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( copymark(npoint), copyset(numnodes) )

      copymark = marknodes   ! copy of marknodes

      marknodes = -10        ! all nodes are marked as used
      marknodes(points) = 0  ! nodes in points are marked as not used yet
      set => points
      nremain = numnodes
      istart = 1

      do while ( nremain>0 )

         if ( nremain==1 ) then
            startnode = set(1)
         else
            startnode = sepstartnode()
         end if  ! ( nremain==1 )
         if ( debug ) write(irefwr,1) 'startnode', startnode

!        --- Construct rooted level structure

         call seprootlevel ( startnode, levellength )
         lenlev = pointlevel(levellength+1)-1
         if ( debug ) then
            call prinin ( pointlevel, levellength+1, 'pointlevel' )
            call prinin ( levelstruc, lenlev, 'levelstruc' )
         end if  ! ( debug )

         nremain = size(set)-lenlev

         if ( debug ) write(irefwr,1) 'nremain', nremain
         if ( nremain>0 ) then

!        --- nremain>0, hence another disjoint set must be renumbered

            copyset(1:size(set)) = set   ! copy of set, since points is changed
            marknodes(levelstruc(1:lenlev)) = -10
            jstart = istart+lenlev
            do i = 1, size(set)
               if ( marknodes(copyset(i))>=0 ) then
                  points(jstart) = copyset(i)
                  jstart = jstart+1
               end if  ! ( marknodes(set(i))>=0 )
            end do  ! i = 1, numnodes

            set => points(istart+lenlev:)

            if ( debug ) call prinin ( set, nremain, 'set' )

         end if  ! ( nremain>0 )

!        --- Copy levels in Reversed sequence in points

         do i = lenlev, 1, -1
            points(istart) = levelstruc(i)
            istart = istart+1
         end do  ! i = levellength, 1, -1

      end do  ! while ( nremain>0 )

!     --- Reset marknodes

      marknodes = copymark

      deallocate ( copymark, copyset )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( points, numnodes, 'points' )
         write(irefwr,*) 'End sepcuthill'

      end if  ! ( debug )
      call erclos ( 'sepcuthill' )

      end subroutine sepcuthill
