      subroutine sepfa05 ( melpef, lpnmpf, lowvolelm )
! ======================================================================
!
!        programmer    Bas van Rens/Guus Segal
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates nmesh part f, i,e. elements connected to faces
!     Old subroutine topfa05
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     surface
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: melpef, lpnmpf, lowvolelm

!     lowvolelm      i    Element number of volume element with lowest number
!     lpnmpf        i/o   length of nmeshf
!                         At input: declared length
!                         At output: computed length
!     melpef         o    maximum number of elements connected with an face
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipf, iface, ipa, jfapel, iwarn1(1000), &
              nelem_face, ielem, iwarn2(1000), &
              i, nwarn1, nwarn2, nelsurf, ierrors(1000), nerrors, &
              nelvolm
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ielem          Element number
!     ierrors        Array containing face numbers of faces that are
!                    marked as incorrect
!     iface          Face sequence number
!     ipa            pointer in kmeshaaa
!     ipf            pointer in kmeshaaf
!     iwarn1         Array containing warnings of a special type
!     iwarn2         Array containing warnings of a special type
!     iwork          integer work array of length nfaces that is used to
!                    store the number of elements coupled to a face
!     jfapel         face in an element
!     nelem_face     number of elements corresponding to face
!     nelsurf        Number of surface elements
!     nelvolm        Number of volume elements
!     nerrors        Number of faces stored in ierrors
!     nwarn1         Number of warnings of a special type
!     nwarn2         Number of warnings of a special type
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     SEPFACEINFO    Write extra information concerning faces that are coupled
!                    to more than two internal elements
!     SEPPRINNMESHA  print nmesh part a
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    1939   More than 2 internal elements connected to a face
!    2362   surface element connected to internal face
!    2363   more than 1 surface element connected to the same face
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Fill iwork with number of volume elements coupled to each face
!     Fill first part of nmeshf
!     Clear array iwork
!     Fill rest of nmeshf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfa05' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfa05'
         write(irefwr,*) 'lowvolelm ', lowvolelm
         call sepprinnmesha

      end if

      allocate ( iwork(nfaces), stat = error )
      if ( error/=0 ) call eralloc ( error, nfaces, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- Fill iwork with number of elements coupled to each face

      length = 2*nfaces
      inelem = km(iseqkm)%kmesh(9)
      do ielem = 1, inelem

         ipa = km(iseqkm)%nmesha(ielem)
         nfapel = km(iseqkm)%nmesha(ipa)/2
         do jfapel = 1, nfapel
            iface = abs(km(iseqkm)%nmesha(ipa+jfapel))
            iwork(iface) = iwork(iface)+1
         end do
         length = length+nfapel

      end do

      if ( length>lpnmpf ) then

!     --- error: length of array nmeshf too small

         call errint ( length, 1 )
         call errint ( lpnmpf, 2 )
         call errchr ( 'nmeshf', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      lpnmpf = length

!     --- Fill first part of nmeshf

      ipf = nfaces+1
      melpef = 0
      do iface = 1, nfaces

         nmeshf(iface) = ipf
         nelem_face = iwork(iface)
         nmeshf(ipf) = nelem_face
         ipf = ipf+iwork(iface)+1
         melpef = max(melpef,nelem_face)

      end do

!     --- Clear array iwork

      iwork = 0

!     --- Fill rest of nmeshf

      do ielem = 1, inelem

         ipa = km(iseqkm)%nmesha(ielem)
         nfapel = km(iseqkm)%nmesha(ipa)/2
         do jfapel = 1, nfapel
            iface = abs(km(iseqkm)%nmesha(ipa+jfapel))
            iwork(iface) = iwork(iface)+1
            ipf = nmeshf(iface)
            nmeshf(ipf+iwork(iface)) = ielem
         end do

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'In sepfa05: nmeshf'
         call  prinnmeshx ( nfaces, nmeshf )

      end if

!     --- Check nmeshx

      nwarn1 = 0
      nwarn2 = 0
      nerrors = 0

      do iface = 1, nfaces

         ipf = nmeshf(iface)
         nelem_face = nmeshf(ipf)
         nelsurf = 0
         nelvolm = 0
         do i = 1, nelem_face
            ielem = nmeshf(ipf+i)
            if ( ielem<lowvolelm ) then

!           --- element is surface element

               nelsurf = nelsurf+1

            else

!           --- element is volume element

               nelvolm = nelvolm+1

            end if

         end do

         if ( nelvolm>2 .and. nerrors<1000 ) then

!        --- More than two volume elements connected to face

            nerrors = nerrors+1
            ierrors(nerrors) = iface

         else if ( nelvolm==2 .and. nelsurf>0 .and. &
                   nwarn1<1000 ) then

!        --- surface element is internal

            nwarn1 = nwarn1+1
            iwarn1(nwarn1) = iface

         else if ( nelsurf>1 .and. nwarn2<1000 ) then

!        --- More than one surface element corresponding to face

            nwarn2 = nwarn2+2
            iwarn2(nwarn2) = iface

         end if

      end do
      if ( nwarn1>0 ) then

!     --- nwarn1>0, Print warning

         do i = 1, nwarn1
            call errint ( iwarn1(i), i )
         end do
         call errwar ( 2362, nwarn1, 0, 0 )

      end if
      if ( nwarn2>0 ) then

!     --- nwarn2>0, Print warning

         do i = 1, nwarn2
            call errint ( iwarn2(i), i )
         end do
         call errwar ( 2363, nwarn2, 0, 0 )

      end if
      if ( nerrors>0 ) then

!     --- nerrors>0, Print error

         do i = 1, nerrors
            call errint ( ierrors(i), i )
         end do
         call errsub ( 1939, nerrors, 0, 0 )

!        --- Print extra information of these elements

         call sepfaceinfo ( ierrors, nerrors )

      end if

1000  if ( debug ) write(irefwr,*) 'End sepfa05'
      call erclos ( 'sepfa05' )

      end
