      subroutine sepfillkmeshmn
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part m with information of all the curves that are
!     created
!     It concerns all outer curves and inner curves defined as the boundary
!     between two element groups
!     Old subroutine topfillkmeshm
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
!     curve
!     topology
!     element
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenedgeelems, lenedgelist, &
              lennodelist, lenkmeshm
      integer, allocatable :: edgeelems(:), nodelist(:), edgenodes(:)
      integer, allocatable :: edgelist(:), kmeshm2(:), kmeshm3(:)
      integer, allocatable :: kmeshm21(:), kmeshm31(:)

!     edgeelems      Contains the element numbers an edge is connected with
!                    in the following way:
!                    Position 1 to nedges contains starting addresses
!                    for each edge
!                    For each edge from the starting position the number
!                    of elements is stored followed by the element numbers
!     edgelist       List of edges that are used
!     edgenodes      Array of size 2 x lennodelist containing the edge
!                    numbers connected to the nodes stored in nodelist
!                    in the sequence of nodelist
!                    Only the edges on the boundary are taken into
!                    account, so each node has at most 2 edge numbers
!                    if node with sequence number j has only one edge
!                    connected to it edgenodes(2,j) = 0
!     kmeshm2        kmesh part m for mesh 2
!     kmeshm21       temporary array kmeshm2
!     kmeshm3        Part 3 of array kmesh part m
!     kmeshm31       temporary array kmeshm3
!     lenedgeelems   Length of array edgeelems
!     lenedgelist    Number of elements in array edgelist
!     lenkmeshm      Length of array kmeshm
!     lennodelist    Number of nodes in nodelist
!     nodelist       Array of size lennodelist containing node numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELGRPEDGES  Make a list of all edges corresponding to 2 element groups
!     SEPELMEDGES    Store all elements corresponding to the edges in array
!                    edgeelems
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPOUTEREDGES  Make a list of all edges corresponding to 1 element only
!     SEPPRINKMESH   Print array KMESH
!     TOPEDGENODES   Make a list of all nodes on the edge stored in edgelist
!                    Make a list of all edges stored in edgelist corresponding
!                    to nodes
!     TOPFILLKMESHM2 Fill arrays kmeshm2 and kmeshm3 for outer curves
!     TOPFILLKMESHM3 Fill arrays kmeshm2 and kmeshm3 for inner curves
!     TOPFILLKMESHMP Create array kmesh part m from separate arrays kmeshm part 2 and
!                    part 3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     create nmesh part c, d, i.e. edges corresponding to elements
!     Next create array edgeelems of elements corresponding to edges
!     Make a list of all edges corresponding to 1 element only (edgelist)
!     Fill array nodelist with all nodes on the outer boundary
!     Make a list of all edges stored in edgelist corresponding to nodes
!     Fill arrays kmeshm2 and kmeshm3 for outer curves
!     if ( number of element groups>1 ) then
!        Fill array edgelist with all edges corresponding to two
!        element groups
!        Fill array nodelist with all nodes on the inner boundaries
!        Make a list of all edges stored in edgelist corresponding to nodes
!        Fill arrays kmeshm2 and kmeshm3 for inner curves
!        Fill arrays kmeshm2 and kmeshm3 for inner curves
!     Create array kmesh part m from separate arrays kmeshm part 2 and part 3
!     Remove superfluous arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshmn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshmn'
         call sepprinkmesh ( 1 )

      end if

!     --- Fill kmesh part m
!         First create nmesh part c, d, i.e. edges corresponding to elements

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         call sepprinkmesh ( 39 )
         call sepprinkmesh ( 40 )

      end if

!     --- Next create array edgeelems of elements corresponding to edges

      lenedgeelems = 4*nedges   ! overestimation of length of edgeelems

!     --- reserve temporary space for array edgeelems

      allocate ( edgeelems(lenedgeelems), stat = error )
      if ( error/=0 ) call eralloc ( error, lenedgeelems, 'edgeelems' )
      if ( ierror/=0 ) go to 1000

!     --- create array edgeelems

      call sepelmedges ( edgeelems, lenedgeelems )
      if ( ierror/=0 ) go to 1000

!     --- Make a list of all edges corresponding to 1 surface element only
!         reserve temporary space for array edgelist

      lenedgelist = nedges   ! overestimation of length edgelist

      allocate ( edgelist(lenedgelist), stat = error )
      if ( error/=0 ) call eralloc ( error, lenedgelist, 'edgelist' )
      if ( ierror/=0 ) go to 1000

!     --- Make a list of all edges corresponding to 1 surface element only

      call sepouteredges ( edgeelems, edgelist, lenedgelist )
      if ( ierror/=0 ) go to 1000

!     --- Make a list of all nodes on the edges stored in edgelist
!         Make a list of all edges stored in edgelist corresponding to nodes
!         reserve temporary space for arrays nodelist and edgenodes

      lennodelist = npoint   ! overestimation of length list

      allocate ( nodelist(lennodelist), edgenodes(2*lennodelist), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, lennodelist, 'nodelist' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array nodelist with all nodes on the outer boundary
!         Make a list of all edges stored in edgelist corresponding to nodes

      call topedgenodes ( npoint, nodelist, lennodelist, &
                          edgenodes, nmeshc, edgelist, lenedgelist )
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays kmeshm2 and kmeshm3 for outer curves
!         reserve semi permanent temporary space for arrays kmeshm2 and kmeshm3

      allocate ( kmeshm2(npoint), kmeshm3(npoint), &
                 kmeshm21(npoint), kmeshm31(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'kmeshm21' )
      if ( ierror/=0 ) go to 1000


      if ( ierror/=0 ) go to 1000

!     --- Fill arrays kmeshm2 and kmeshm3 for outer curves

      call topfillkmeshm2 ( npoint, nedges, ncurvs, kmeshm3, kmeshm2, &
                            nodelist, lennodelist, edgenodes, &
                            nmeshc )
      if ( ierror/=0 ) go to 1000

!     --- Make a list of all edges corresponding to 2 element groups

      ninner = 0
      if (nelgrp>1 ) then

!     --- nelgrp>1, there are several element groups
!         reserve temporary space for array edgelist

         lenedgelist = nedges   ! overestimation of length edgelist

!        --- Fill array edgelist with all edges corresponding to two
!            element groups

         call sepelgrpedges ( edgeelems, edgelist, lenedgelist )
         if ( ierror/=0 ) go to 1000

!        --- Make array edgelist semi permanent

         if ( lenedgelist==0 ) go to 100

!        --- Fill array nodelist with all nodes on the inner boundaries
!            Make a list of all edges stored in edgelist corresponding to nodes

         lennodelist = npoint   ! overestimation of length list

!        --- Fill array nodelist with all nodes on the inner boundaries
!            Make a list of all edges stored in edgelist corresponding to nodes

         call topedgenodes ( npoint, nodelist, lennodelist, &
                             edgenodes, nmeshc, edgelist, lenedgelist )
         if ( ierror/=0 ) go to 1000

!        --- Fill arrays kmeshm2 and kmeshm3 for inner curves

         call topfillkmeshm3 ( npoint, nedges, ninner, &
                               kmeshm31, kmeshm21, &
                               nodelist, lennodelist, &
                               edgenodes, nmeshc )

      end if
100   if ( ierror/=0 ) go to 1000

!     --- Reserve permanent space for array kmeshm

      lenkmeshm = 4+ncurvs+1+kmeshm2(1+ncurvs)-1

      if ( ninner>0 ) lenkmeshm = lenkmeshm+2*ninner+kmeshm21(1+ninner)-1
      allocate ( km(iseqkm)%kmeshm(lenkmeshm) )
      kmeshm => km(iseqkm)%kmeshm
      kmeshmglob => kmeshm

!     --- Create array kmesh part m from separate arrays kmeshm part 2 and
!         part 3

      call topfillkmeshmp ( kmeshm, ncurvs, kmeshm2, &
                            kmeshm3, ninner, kmeshm21, kmeshm31 )
      if ( ierror/=0 ) go to 1000

!     --- Remove superfluous arrays

      deallocate ( edgeelems, nodelist, edgenodes, edgelist, &
                   kmeshm2, kmeshm3, kmeshm21, kmeshm31, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'edgeelems' )
      if ( nmesh(33)>0 ) deallocate ( km(iseqkm)%nmeshc )
      if ( nmesh(34)>0 ) deallocate ( km(iseqkm)%nmeshd )
      deallocate ( km(iseqkm)%nmesh )
      km(iseqkm)%kmesh(48) = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmeshmn'

      end if
      call erclos ( 'sepfillkmeshmn' )

      end
