      subroutine sepflpointobs3
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ipointobst (3d)
!     Old subroutine mshfillkmesha3
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, lenipointobst, &
              lenicrossobst, lenrcrossobst, &
              lenin, lenrl, nelemobst, &
              leniobgrid, lenobstfaces, ipipoin
      integer, allocatable :: iobgrid(:,:,:,:)
      integer, allocatable :: ipointobstlc(:)

!     i              Counting variable
!     iobgrid        position i,j contains:
!                    -1:  cell inside obstacle
!                     0:  cell outside obstacle
!                    >0:  cell contains part of the obstacle
!                    The value contains a starting address in iobgrid1
!     iobst          Obstacle sequence number
!     ipipoin        Starting address in array ipointobst
!     ipointobstlc   Locall array ipointobst
!     lenicrossobst  Computed length of array icrossobst
!     lenin          Number of integers in one row of kmeshx
!     leniobgrid     Declared length of last dimension of iobgrid
!     lenipointobst  length of array ipointobst
!     lenobstfaces   Length of array obstfaces
!     lenrcrossobst  Computed length of array rcrossobst
!     lenrl          Number of reals in one row of rmeshx
!     nelemobst      Number of elements in obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHOB3DMARKPT  Mark all nodal points in the mesh with information if they
!                    are within an obstacle or outside an obstacle
!                    3d version
!     MSHOBSFL3D     Fill array iobgrid with grid information of one obstacle
!                    (R^3 only)
!     MSHOBSTMARKBT1 Fill ipointobst part 4, i.e. the set of boundary nodes of
!                    an obstacle that are only inside obstacle elements
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPPRINIPOBST  Print the contents of array ipointobst
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2206   KMESH part x has not been filled
!    2207   There are no obstacles
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for iobst := 1 (1) numobst do
!        Mark all nodes of mesh in obstacle
!        Mark all elements to be in/out or partly in obstacle
!        Mark all nodes that are in obstacle and only in elements that are
!        completely in obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepflpointobs3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepflpointobs3'
  1      format ( a, 1x, (10i6) )
         call sepprinkmesh ( 1 )
         call sepprinkmesh ( 2 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- initializations

      lenrl = 9
      lenin = 12
      if ( numobst==0 ) then

!     --- No obstacles

         call errwar ( 2207, 0, 0, 0 )
         go to 1000

      end if

      if ( debug ) &
         write(irefwr,1) 'numobst, npoint, nelem',numobst, npoint, nelem

      if ( kelmd==0 ) call sepfillkmeshd

!     --- Reserve temporary space for array ipointobst

      lenipointobst = 5*numobst+npoint

      allocate ( ipointobstlc(lenipointobst), stat = error )
      if ( error/=0 ) call eralloc ( error, lenipointobst, 'ipointobstlc' )
      if ( ierror/=0 ) go to 1000

!     --- Set starting address for first obstacle

      ipipoin = numobst+1

!     --- for iobst := 1 (1) numobst do

      do iobst = 1, numobst

!     --- Mark all nodes of mesh in obstacle

         ipointobstlc(iobst) = ipipoin
         if ( kelmx==0 ) then

!        --- error, kmesh part x has not been filled

            call errsub ( 2206, 0, 0, 0 )
            go to 1000

         end if
         nx = kmeshx(3,iobst)
         ny = kmeshx(4,iobst)
         nz = kmeshx(5,iobst)
         kmobst => km(iseqkm)%kmobst
         if ( ierror/=0 ) go to 1000
         if ( kmeshx(6,iobst)==0 ) then

!        --- Array has not yet been filled

            kmeshcobst => kmobst(iobst)%kmeshcobst
            coorobst => kmobst(iobst)%coorobst
            inpelm = kmeshx(10,iobst)
            nelemobst = kmeshx(11,iobst)

!           --- Create temporary space to store iobgrid;
!               The actual length is computed later on
!               The estimated length is 50 nx ny nz

            leniobgrid = 50

            allocate ( iobgrid(nx,ny,nz,leniobgrid), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, nx*ny*nz*leniobgrid, 'iobgrid' )
            if ( ierror/=0 ) go to 1000

            call mshobsfl3d ( rmeshx, iobgrid, &
                              kmeshcobst, coor, &
                              nx, ny, nz, iobst, inpelm, nelemobst, &
                              leniobgrid )

            allocate ( kmobst(iobst)% &
                       iobgrid3d(0:nx-1,0:ny-1,0:nz-1,leniobgrid) )
            iobgrid3d => kmobst(iobst)%iobgrid3d
            iobgrid3d = iobgrid(:,:,:,1:leniobgrid)

            deallocate ( iobgrid, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'iobgrid' )
            if ( ierror/=0 ) go to 1000

            kmeshx(6,iobst) = iseqkm
            kmeshx(7,iobst) = leniobgrid

         end if
         leniobgrid = kmeshx(7,iobst)
         inpelm = kmeshx(10,iobst)
         iobgrid3d => kmobst(iobst)%iobgrid3d
         kmeshcobst => kmobst(iobst)%kmeshcobst
         coorobst => kmobst(iobst)%coorobst

         call mshob3dmarkpt ( iobst, nx, ny, nz, iobgrid3d, &
                              ipointobstlc(ipipoin), inpelm, &
                              leniobgrid )

!        --- Fill part 4 of ipointsobst of nodes on the boundary of the
!            obstacle which are only in elements completely inside
!            the obstacle

         call mshobstmarkbt1 ( ipointobstlc(ipipoin), &
                               nelem, npoint, kmshd1, &
                               kmshd2, kmeshc )

!        --- raise ipipointobst

         ipipoin = ipipoin+4+ipointobstlc(ipipoin)+ipointobstlc(ipipoin+2)+ &
                   ipointobstlc(ipipoin+3)

      end do

      if ( debug ) then
         write(irefwr,*) 'ipointobstlc after mshobstmarkbt'
         call sepprinipobst ( ipointobstlc )
      end if

!     --- Check array length

      call inichkarlen ( ipipoin, lenipointobst, 'ipointobstlc' )
      if ( ierror/=0 ) go to 1000

!     --- Make array ipointobst permanent

      if ( associated(km(iseqkm)%ipointobst) ) &
         deallocate(km(iseqkm)%ipointobst)
      allocate(km(iseqkm)%ipointobst(lenipointobst))
      kmeshad(1) = lenipointobst
      kmeshad(6) = iseqobst

!     --- Copy contents of temporary array into permanent array

      ipointobst => km(iseqkm)%ipointobst
      ipointobst = ipointobstlc(1:lenipointobst)

      deallocate ( ipointobstlc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ipointobstlc' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepflpointobs3'

      end if
      call erclos ( 'sepflpointobs3' )

      end subroutine sepflpointobs3
