      subroutine sepintmat1 ( nsol, irowsnew, icolsnew, lencolsnew, &
                              rows, cols, lencols )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform symmetric compact storage scheme to row-compact storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nsol, lencolsnew, lencols
      integer, intent(in) :: rows(nsol+1), cols(lencols)
      integer, intent(out) :: irowsnew(nsol+1), icolsnew(lencolsnew)

!     cols           i    array with column numbers
!     icolsnew       o    New array icolsii
!     irowsnew       o    New array irowsii
!     lencols        i    size of array cols
!     lencolsnew     i    size of array icolsnew
!     nsol           i    number of unknowns with respect to these arrays
!     rows           i    Number of non-zero entries of matrix per row
!                         accumulated (rows(1)=0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: point(nsol), ih, ih1, i, j, m

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     j              Counting variable
!     m              Help variable to store a constant temporarily
!     point          pointer to irowsnew in icolsnew
!     rows           Refers to array irowsnew
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmat1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmat1'
         write(irefwr,1) 'nsol', nsol
         call sepprincompstr ( nsol, rows, cols )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Refill array irowsnew

      irowsnew = 0

!     --- First lower part

      do i = 1, nsol

         do j = rows(i)+1, rows(i+1)
            irowsnew(cols(j)) = irowsnew(cols(j))+1
            irowsnew(i) = irowsnew(i)+1
         end do

      end do

!     --- Next upper part + diagonal

      ih = 0
      do i = 1, nsol
         ih1 = ih+irowsnew(i)+1
         irowsnew(i) = ih
         ih = ih1
      end do
      irowsnew(nsol+1) = ih

      point = irowsnew(1:nsol)  ! pointer to irowsnew in icolsnew
      icolsnew(1) = 1
      do i = 1, nsol
         do j = rows(i)+1, rows(i+1)
            m = cols(j)
            point(i) = point(i)+1
            icolsnew(point(i)) = m     ! lower part of matrix
            point(m) = point(m)+1
            icolsnew(point(m)) = i     ! upper part of matrix
         end do
         point(i) = point(i)+1
         icolsnew(point(i)) = i        ! diagonal element
      end do

1000  if ( debug ) then

!     --- Debug information

         call sepprincompstr ( nsol, irowsnew, icolsnew )
         write(irefwr,*) 'End sepintmat1'

      end if  ! ( debug )
      call erclos ( 'sepintmat1' )

      end subroutine sepintmat1
