      subroutine sepnodeelgrps
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 16-07-2012 Remove iwork from parameter list
!        version  1.0  date 29-11-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part af with all element groups corresponding to nodes
!     Old subroutine topnodeelgrps
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element_group
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, istart_ielgrp, nnodes, nodp, istart, lennodeelgrps
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ielgrp         element group number
!     istart         First position used at a part
!     istart_ielgrp  Starting position of nodes for element group
!     iwork          integer work array of length npoint
!                    In first instance iwork is used to store the
!                    number of element groups corresponding to a node
!                    Later on it is used to store pointers in node_elgrps
!                    for some nodes
!     lennodeelgrps  Length of array node_elgrps
!     nnodes         Number of nodes
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear first npoint positions of node_elgrps
!     Fill array iwork with number of element groups per node
!     Fill array node_elgrps with pointers and number of element groups for all
!        nodes that are in more than one element group
!     Fill array node_elgrps completely using array iwork as pointer
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnodeelgrps' )

!     --- Clear first npoint positions of node_elgrps and array iwork

      lennodeelgrps = 4*npoint

      allocate ( iwork(npoint), node_elgrps(lennodeelgrps), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      node_elgrps(1:npoint) = 0
      iwork = 0

!     --- Fill array iwork with number of element groups per node

      do ielgrp = 1, nelgrp
         istart_ielgrp = ielgrp_nodes(ielgrp)
         nnodes =  ielgrp_nodes(istart_ielgrp)
         do i = istart_ielgrp+1, istart_ielgrp+nnodes
            nodp = ielgrp_nodes(i)
            iwork(nodp) = iwork(nodp)+1
         end do
      end do

!     --- Fill array node_elgrps with pointers and number of element groups for
!         all nodes that are in more than one element group

      istart = npoint+1
      do i = 1, npoint
         if ( iwork(i)>1 ) then

!        --- iwork(i)>1, hence node i is in more than one element group

            node_elgrps(i) = -istart
            node_elgrps(istart) = iwork(i)
            istart = istart+1+iwork(i)
            iwork(i) = -node_elgrps(i)

         end if

      end do
      istart = istart-1
      if ( istart>lennodeelgrps ) then

!     --- lenielgrpnods too small

         call errint ( istart, 1 )
         call errint ( lennodeelgrps, 2 )
         call errchr ( 'node_elgrps', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      lennodeelgrps = istart

!     --- Fill array node_elgrps completely using array iwork as pointer

      do ielgrp = 1, nelgrp
         istart_ielgrp = ielgrp_nodes(ielgrp)
         nnodes =  ielgrp_nodes(istart_ielgrp)
         do i = istart_ielgrp+1, istart_ielgrp+nnodes
            nodp = ielgrp_nodes(i)
            if ( node_elgrps(nodp)==0 ) then

!           --- node_elgrps(nodp)=0, hence node in only one element group

               node_elgrps(nodp) = ielgrp

            else

!           --- node_elgrps(nodp)<0, hence node in more element group

               iwork(nodp) =  iwork(nodp)+1
               istart =  iwork(nodp)
               node_elgrps(istart) = ielgrp

            end if

         end do
      end do
      if ( km(iseqkm)%kmesh(56)>0 ) deallocate(km(iseqkm)%node_elgrps)
      allocate ( km(iseqkm)%node_elgrps(lennodeelgrps) )
      km(iseqkm)%kmesh(56) = lennodeelgrps
      kelmaf => km(iseqkm)%kmesh(56)
      km(iseqkm)%node_elgrps = node_elgrps(1:lennodeelgrps)

      deallocate ( iwork, node_elgrps, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      node_elgrps => km(iseqkm)%node_elgrps

1000  call erclos ( 'sepnodeelgrps' )

      end
