      subroutine sepprobloop1 ( ichoice, leniinp, lenrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     information with respect to the problem is computed
!     and filled in array kprob
!     Performs all actions for probdfext
!     Old subroutine probdfloop
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     fictitious_domain
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulecpack
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lenrinp, ichoice, leniinp

!     ichoice        i    Choice parameter, see probdfext
!     leniinp        i/o   Last position of array iinputkp
!     lenrinp       i/o   Length of array rinputkp corresponding to 1 problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer np
      integer, allocatable :: istore(:), kprobptmp(:), ielshape(:)
      logical type922found

!     ielshape       Array of length 2*(nelgrp+numnat) containing:
!                    (1,i) number of nodes in element group i
!                    (2,i) shape number of elements in element group i
!                    if i>nelgrp: i-nelgrp = boundary group number
!                    otherwise it is the element group number
!     istore         Contains the last level set
!     kprobptmp      Temporaray array to store kprob part p
!     np             Help variable to store the maximum value of nphys and
!                    nunkp
!     type922found   If true elements of type 922 have been found, which
!                    means that Lagrangian multipliers are defined by means
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIT02         Print information concerning the type of elements
!     INIT06         information is stored in array kprob
!     INIT07         compute information for boundary elements
!     INITSP         Read and compute information of standard problems
!     PRININ         print 1d integer array
!     PROB00         Fill array KPROB parts f and e
!     PROB01         Fill information concerning the natural boundary conditions
!                    in KPROB
!     PROB14         Computes starting positions in array IINPUT with respect to
!                    various parts
!     PROB16         Fill physical degrees of freedom in KPROB part p
!     PROBFLKPROBAA  Fill kprob part aa with information concerning the elements
!                    to be skipped
!     PROBFLKPROBG   Fill information with respect to essential boundary
!                    conditions in KPROB
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprobloop1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprobloop1'
         write(irefwr,1) 'iprob', iprob
         call prinin ( iinputkp, 60, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ioutp>=1 .and. ichoice/=3 ) then

!     --- writes are required

         write(irefwr,*) '***************************'
         write(irefwr,*) 'Problem number ', iprob
         write(irefwr,*) '***************************'

      end if
      if ( debug ) write(irefwr,1) 'Problem number ', iprob

      irecp1 = 0
      irecp2 = 0
      indprf = 0
      indprh = 0
      lenrinp = 0   ! jump in array rinputkp

!     --- Compute starting positions of parts in array IINPUT

      call prob14 ( leniinp )

      if ( debug ) write(irefwr,1) 'leniinp', leniinp
      if ( ierror/=0 ) go to 1000

!     --- Compute number of nodes per element group as well as
!         corresponding shape numbers (also for boundary elements)
!         Store at the end of IBUFFR from position ipelshap

      numnat = iinputkp(2)+ iinputkp(13) ! numnat + nbcons
      allocate ( ielshape((numnat+nelgrp)*2), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, (numnat+nelgrp)*2, 'ielshape' )
      if ( ierror/=0 ) go to 1000

      call init07 ( ielshape )
      if ( ierror/=0 ) go to 1000

!     --- Fill information concerning standard problems into array KPROB

      allocate ( istore(5000*(nelgrp+numnat)), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 5000*(nelgrp+numnat), 'istore' )
      if ( ierror/=0 ) go to 1000

      kprobloc(80) = 0  ! mark as new
      kprobf => kprobloc

      call initsp ( ielshape, istore, type922found )
      if ( iinputkp(59)==1 ) kprobloc(69) = 1  ! temporary

!     --- reserve space for kprob part p when nphys>0

      nphys = intarprb(14)

      allocate ( kprobptmp(max(1,nphys*kelgrp)),stat = error )
      if ( error/=0 ) &
         call eralloc ( error, max(1,nphys*kelgrp), 'kprobptmp' )
      if ( ierror/=0 ) go to 1000

!     --- Put information in kprob and initialize a part of kprob p

      if ( debug ) write(irefwr,*) 'before init06'
      call init06 ( istore, kprobptmp, ielshape, type922found )

      deallocate ( istore, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istore' )
      if ( debug ) write(irefwr,1) 'ierror', ierror

      if ( ierror/=0 ) go to 1000

      if ( kprobloc(12)>0 ) then

!     --- Fill information concerning the natural boundary conditions
!         PROB01 uses memory management subroutines

         call prob01

      else if ( ichoice/=4 .and. ichoice/=5 .and. kprobloc(22)>0 ) then

         if ( associated(kprobpart%kprobi) ) &
            deallocate(kprobpart%kprobi)
         nullify(kprobi)
         kprobloc(22) = 0

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays KPROB part f and e
!         PROB00 uses memory management subroutines
!         In part f the number of degrees of freedom per point is
!         stored if not constant, in part e information concerning
!         vectors of special structure is stored

      if ( debug ) write(irefwr,*) 'before prob00'
      call prob00 ( ielshape )
      if ( ierror/=0 ) go to 1000

      nusol = kprobloc(5)
      kprobloc(66) = nusol
      if ( debug ) write(irefwr,1) 'nusol', nusol

!     --- Fill kprob part aa if necessary

      if ( ichoice/=4 .and. ichoice/=5 .and. ( intarprb(20)>0 .or. &
           nlevelsets>0 .and. iinputkp(48)>0) ) then

!     --- iskipelem>0, fill kprob aa, i.e. skip elements
!         Also in case of level set functions (nlevelsets0 and iinputkp(48)>0)

         if ( debug ) write(irefwr,*) 'before probflkprobaa'
         call probflkprobaa

      end if  ! ( ichoice/=4 .and. ichoice/=5 .and. ... )
      if ( ierror/=0 ) go to 1000

!     --- Fill kprob part g with essential boundary conditions

      if ( debug ) write(irefwr,*) 'before probflkprobg'
      if ( ichoice/=4 .and. ichoice/=5 ) call probflkprobg
      if ( ierror/=0 ) go to 1000

!     --- Fill kprob part p with positions of unknowns

      if ( debug ) &
         write(irefwr,1) 'nphys, nunkp, kprobloc(14)', &
                          nphys, nunkp, kprobloc(14)

      kprobloc(2) = 101  ! mark as kprob array

      if ( nphys>0 .or. kprobloc(14)/=0 .or. nunkp>1 ) then

!     --- nphys>0, kproba # 0, or nunkp>1:  fill kprob part p

         np = max ( nphys,nunkp )
         if ( associated(kprobpart%kprobp) ) &
            deallocate(kprobpart%kprobp)
         allocate ( kprobpart%kprobp(npoint,np))
         kprobloc(35) = np*npoint
         kprobp => kprobpart%kprobp
         if ( ierror/=0 ) go to 1000
         call prob16 ( kprobptmp, ielshape )

      else if ( kprobloc(35)>0 ) then

         if ( associated(kprobpart%kprobp) ) &
            deallocate(kprobpart%kprobp)

      end if
      indprp = kprobloc(35)

!     --- Print some information if required
!         At this point we know for sure that kprob part p is filled
!         (this is done in prob16)

      if ( ioutp>=0 .and. ichoice/=3 ) call init02 ( kprobptmp, ielshape )
      if ( ierror/=0 ) go to 1000

      deallocate ( kprobptmp, ielshape, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kprobptmp' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobloc, kprobloc(3), 'kprob' )
         write(irefwr,*) 'End sepprobloop1'

      end if  ! ( debug )
      call erclos ( 'sepprobloop1' )

      end subroutine sepprobloop1
