      subroutine seprootlevel ( startnode, levellength )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!     Find rooted level structure corresponding to node
!     Return with length, width and all levels
!     Old subroutine chrootlevel
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: startnode
      integer, intent(out) :: levellength

!     levellength    o    Length of rooted level structure
!     startnode      i    Starting node number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, numnodes, numlastlev, nnodes, j

!     i              Counting variable
!     j              Counting variable
!     nnodes         number of nodes in set
!     numlastlev     number of nodes in last level
!     numnodes       Number of nodes in last level
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPNEXTLEVEL   Find neighbors corresponding to a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprootlevel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprootlevel'
         write(irefwr,1) 'size(levelstruc), size(pointlevel)', &
                          size(levelstruc), size(pointlevel)
         write(irefwr,1) 'startnode', startnode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Mark all nodes that are in set

      nnodes = size(set)
      do i = 1, nnodes
         j = set(i)
         if ( marknodes(j)>=0 ) marknodes(j) = nnodes+1
      end do  ! i = 1, nnodes

!     --- marknodes(startnode) := 1
!         levellength := 0
!         levelstruc(1) := startnode; numlastlev := 1

      nextlevel => levelstruc(2:)
      marknodes(startnode) = 1
      levellength = 0
      levelstruc(1) = startnode
      numlastlev = 1

      pointlevel(1) = 1

!     --- Store all neighbors of lastlevel in work array levelstruc
!         finished := levelstruc is empty

      lastlevel => levelstruc(1:numlastlev)
      call sepnextlevel ( levellength+1, numnodes )
      lastlevel => nextlevel(1:numnodes)

      if ( debug ) then
         write(irefwr,1) 'numnodes', numnodes
         call prinin ( nextlevel, numnodes, 'nextlevel' )
      end if  ! ( debug )

!     --- While ( not finished ) do

      do while ( numnodes>0 )

!     --- numlastlev := number of nodes in levelstruc
!         levellength := levellength+1;
!         lastlevel(1:numlastlev) := levelstruc(1:numlastlev)
!         marknodes(lastlevel(1:numlastlev)) := levellength+1

         levellength = levellength+1
         pointlevel(levellength+1) = pointlevel(levellength)+numlastlev
         numlastlev = numnodes
         marknodes(lastlevel(1:numlastlev)) = levellength+1

!        --- Store all neighbors of lastlevel in work array levelstruc

         nextlevel => levelstruc(pointlevel(levellength+1)+numnodes:)
         if ( debug ) &
            write(irefwr,1) 'sizes', size(lastlevel), size(nextlevel)
         call sepnextlevel ( levellength+1, numnodes )
         if ( debug ) &
            write(irefwr,1) 'numnodes, levellength',numnodes, levellength
         if ( numnodes>0 )  then
            lastlevel => nextlevel(1:numnodes)
            if ( debug ) call prinin ( nextlevel, numnodes, 'nextlevel' )
         end if  ! ( debug )

      end do  ! while ( numnodes>0 )
      levellength = levellength+1
      pointlevel(levellength+1) = pointlevel(levellength)+numlastlev

1000  if ( debug ) then

!     --- Debug information

         call prinin ( pointlevel, levellength+1, 'pointlevel' )
         call prinin ( levelstruc, pointlevel(levellength+1)-1, 'levelstruc' )
         write(irefwr,*) 'End seprootlevel'

      end if  ! ( debug )
      call erclos ( 'seprootlevel' )

      end subroutine seprootlevel
