      subroutine topbv3free1 ( maxren, irenum, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprobh for free unknowns
!     Global renumbering over all nodes or levels
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     renumbering
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: maxren, irenum(*), iwork(*)

!     irenum         i    This array contains the levels of the renumbering of
!                         all unknowns per point
!                         So if iren(i)=j then unknown i is numbered in the j-th
!                         level
!                         Unknowns with sequence number larger than maxren
!                         correspond always to the largest level
!     iwork         i/o   Output of subroutine TOPBV2
!                         iwork(i)>0, means essential boundary condition for
!                                     unknown i
!                         iwork(i)=0, means unknown i is free
!                         iwork(i)<0, means unknown i corresponds to
!                         periodical bc.
!     maxren         i    Maximum number of unknowns that have a special
!                         numbering
!                         If <0, the unknowns must be renumbered per level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, iend, icount, inunkp, ilevel, maxrna, i, j, &
                 nrenlv, inodp, inusol, iren, nlevellc, ihelp(2*nphys)

!     i              Counting variable
!     icount         number of unknowns per element
!     iend           Last position used at a part
!     ihelp          work array
!     ilevel         Present level in array of levels
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     iren           Loop variable with respect to unknowns to be renumbered
!     istart         First position used at a part
!     j              Counting variable
!     maxrna         absolute value of maxren
!     nlevellc         Number of levels stored in kmeshk
!     nrenlv         Number of unknowns that must be renumbered for this level
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv3free1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv3free1'
         call prinin ( iwork, nusol, 'iwork' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nlevellc = nlevel

      if ( debug ) &
         write(irefwr,1) 'nlevel, kelmj, npoint', nlevel, kelmj, npoint

      inunkp = nunkp

      maxrna = abs(maxren)
      if ( maxren>0 ) nlevellc = 1
      if ( debug ) call prinin ( irenum, maxrna, 'irenum' )

!        --- Initialize kprob part ac

      if ( associated(kprobpart%kprobac) ) &
         deallocate(kprobpart%kprobac)
      allocate(kprobpart%kprobac(maxrna+2))
      kprobac => kprobpart%kprobac
      kprobac = 0
      kprobac(1) = kprobloc(72)  ! old maxren
      kprobloc(59) = maxrna+2
      indprac = kprobloc(59)

      istart = 1
      iend = npoint
      icount = 1
      inunkp = nphys

      if ( debug ) &
         write(irefwr,1) 'inunkp, indprp, indprac', &
                          inunkp, indprp, indprac
      do ilevel = 1, nlevellc
         if ( maxren<0 ) iend = kmeshk(ilevel)
         if ( debug ) write(irefwr,1) 'ilevel, maxrna', ilevel, maxrna
         do iren = 1, maxrna+1
            nrenlv = 0
            ihelp(1:inunkp) = 0

            if ( iren<=maxrna ) then

!           --- iren<=maxrna

               do i = 1, maxrna
                  if ( irenum(i)==iren ) then
                     nrenlv = nrenlv+1
                     ihelp(i) = 1
                  end if
               end do

            else

!           --- iren>maxrna

               do i = maxrna+1, inunkp
                  nrenlv = nrenlv+1
                  ihelp(i) = 1
               end do

            end if
            if ( debug ) call prinin ( ihelp, inunkp, 'ihelp' )

            if ( nrenlv>0 ) then

!           --- nrenlv>0

               do i = istart, iend
                  if ( kelmj>0 ) then
                     inodp = kmeshj(i)
                  else
                     inodp = i
                  end if

                  do j = 1, inunkp
                     if ( ihelp(j)>0 ) then

!                    --- ihelp(j) = 1, unknown must be used

                        if ( indprp==0 ) then

!                       --- indprp =  0

                           inusol = (inodp-1)*nunkp+j

                        else

!                       --- indprp>0

                           inusol = kprobp(inodp,j)

                        end if
                        if ( inusol>0 ) then
                           if ( iwork(inusol)==0 ) then

!                          --- iwork (j+inusol)  =  0

                              kprobh(inusol) = icount
                              icount = icount+1
                           end if
                        end if

                     end if

                  end do

               end do

            end if
            if ( indprac>0 ) kprobac(iren+1) = icount-1

         end do  ! iren = 1, maxrna+1
         if ( debug ) &
            write(irefwr,1) 'ilevel, icount', ilevel, icount
         istart = iend+1

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv3free1'

      end if  ! ( debug )
      call erclos ( 'topbv3free1' )

      end subroutine topbv3free1
