      subroutine topbv6 ( iwork, factors, jumps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 14-01-2013 debug information
!        version  5.0  date 19-06-2002 Complete revision
!        version  4.0  date 16-01-1999 Allow more degrees of freedom
!                                      Extra parameter iinput
!
!   copyright (c) 1989-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array kprobh with references of unknowns to array usol.
!    Kprobh should only be filled if npboun>0, otherwise kprobh
!    and kprobr are identical.
!    Furthermore array kprobs (integer part of kprob part s) and
!    rprobs (real part of kprob part s) are filled.
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(nusol)
      double precision factors(*), jumps(*)

!     factors        i    Array of size nusol containing the multiplication
!                         factors for the periodical boundary conditions
!     iwork          i    Semi-permanent work array of length nusol in which
!                         information about essential boundary conditions and
!                         periodical boundary conditions is stored
!                         iwork is filled as follows:
!                         iwork(i) = 0: means unknown i is free
!                         iwork(i)>0: means unknown i is prescribed
!                                       (essential bc)
!                         iwork(i)<0: means unknown i refers to unknown
!                                       iwork(i), which means that unknown i
!                                       has the same value, possibly
!                                       extended with a jump
!     jumps          i    Array of size nusol containing the jumps
!                         for the periodical boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, icount, inusol, istkprobh
      double precision rjump, rmult

!     icount         Counter
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     rjump          Contains the jump constant
!     rmult          Contains the multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Copy KPROBR into KPROBH
!    for i := 1 (1) nusol do
!       if ( iwork(i)<0 ) then
!          fill kprobs, rprobs and kprobh for unknowns in iwork(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv6'
         call prinin ( kprobh, nusol, 'kprobh' )
         call prinin ( iwork, nusol, 'iwork' )
         call prinrl ( jumps, nusol, 'jumps' )
         call prinrl ( factors, nusol, 'factors' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy array kprobr into kprobh

      kprobr = kprobh(1:nusol)

!     --- Set constants, see subroutine topbv5

      icount = 0
      istkprobh = nusol-nbound-npboun

      do j = 1, nusol

         inusol = iwork(j)

         if ( inusol<0 ) then

!        --- Periodical unknown found

            rjump = jumps(j)
            rmult = factors(j)

            icount = icount+1
            kprobs(icount) = kprobr(-inusol)
            rprobs(icount,1) = rjump
            rprobs(icount,2) = rmult
            kprobh(j) = istkprobh+icount

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobh, nusol, 'kprobh' )
         call prinin ( kprobs, icount, 'kprobs' )
         write(irefwr,1) 'npboun', npboun
         call prinrl1 ( rprobs, 2, icount, 'rprobs' )
         write(irefwr,*) 'End topbv6'

      end if  ! ( debug )
      call erclos ( 'topbv6' )

      end
