      subroutine topbv7 ( iwork, jumps, factors, istoreleft, &
                          lenistore, nodepairs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Nodal points of which the unknowns are connected through periodical
!     boundary conditions (possibly with jump) are marked
!     New version without the use of connection elements
!       degrees of freedom corresponding to one of the nodal points
!       are filled in iwork with a negative number referring to the
!       other one
!     Warning: the algoritm used in this subroutine is also used in
!              subroutine incm01
!              If this subroutine is changed, also change incm01
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), istoreleft(*), lenistore, nodepairs(2,*)
      double precision :: jumps(*), factors(*)

!     factors        o    Array of size nusol containing the multiplication
!                         factors for the periodical boundary conditions
!     istoreleft     o    Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!     iwork         i/o   Semi-permanent work array of length nusol in which
!                         information about essential boundary conditions and
!                         periodical boundary conditions is stored
!                         iwork is filled as follows:
!                         iwork(i) = 0: means unknown i is free
!                         iwork(i)>0: means unknown i is prescribed
!                                       (essential bc)
!                         iwork(i)<0: means unknown i refers to unknown
!                                       iwork(i), &
!                         which means that unknown i has the same value, &
!                         possibly extended with a jump
!                         Subroutine TOPBV2 has already initialised iwork with
!                         non-negative values
!     jumps          o    Array of size nusol containing the jumps
!                         for the periodical boundary conditions
!     lenistore      i    Estimated length of arrays istoreleft and istoreright
!     nodepairs      o    Work array of size 2*npoint in which the pairs of node
!                         numbers corresponding to the periodical boundary conditions
!                         are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprinp
      integer, allocatable :: itemp(:), istoreright(:)
      double precision, allocatable :: rjumpleft(:), rmultright(:), &
                                       rjumpright(:), rmultleft(:)

!     iprinp         Starting address in array rinputkp
!     istoreright    Contains all periodical left neighbor unknowns
!                    for all right periodical neighbor unknowns
!                    Pos. 1 to nusol+1 contain pointers to the second part
!                    istoreright(1) contains nusol+1
!                    The number of neighbors of unknown i is stored in
!                    istoreright(i+1)-istoreright(i)
!                    The second part contains the neighbors
!     itemp          Temporaray work array of length nusol to indicate
!                    if an unknown corresponds to a "left" periodical
!                    boundary condition (1) or not (0)
!     rjumpleft      Array wih the same length as the second part of
!                    istoreleft containing the jumps w.r.t the right part
!     rjumpright     Array wih the same length as the second part of
!                    istoreright containing the jumps w.r.t the left part
!     rmultleft      Array wih the same length as the second part of
!                    istoreleft containing the factors w.r.t the right part
!     rmultright     Array wih the same length as the second part of
!                    istoreright containing the factors w.r.t the left part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOPBV701       Fill number of right neighbor unknowns in istoreleft and
!                    number of left neighbor unknowns in istoreright
!     TOPBV702       Fill rest of arrays istoreleft and istoreright
!                    Fill jumps in array jumpleft and jumpright and
!                    multipication factors in rmultleft and rmultfact
!     TOPBV703       Nodal points of which the unknowns are connected through
!                    periodical boundary conditions (possibly with jump) are
!                    marked
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2343   Internal error: nunkp>32
! **********************************************************************
!
!                       PSEUDO CODE
!
!     clear arrays istoreleft, istoreright,ihelp, jumps and factors
!     Fill number of rigth unknowns in istoreleft and
!          number of left unknowns in istoreright
!     Fill all neighbors in ileft and iright and jumps and factors in work1/2
!     Fill array ihelp
!     nhelp := number of non-zero entries in ihelp
!     While ( nhelp>0 ) do
!        istart := smallest unknown with ihelp(istart) = 1
!        ihelp(istart) := 0, nhelp:=nhelp-1
!        Store all neighbors of istart in istore
!        nstore := number of entries in istore
!        Fill iwork, jumps, factors for all entries in istore
!        Remove all neighbors from ihelp and decrease nhelp
!        While ( nstore>0 ) do
!           inext := first unknown in istore
!           remove inext from istore and decrease nstore
!           For all neighbors ineigh of inext do
!              if ( ihelp(neigh) = 0 ) then
!                 debug jump and factor
!              else
!                 add neighbor to iwork, jumps, factors and istore
!                 nstore := nstore+1
!                 ihelp(ineigh) := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv7' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv7'
         write(irefwr,1) 'nusol, lenistore', nusol, lenistore
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nunkp>320 ) then

!     --- nunkp too large

         call errint ( nunkp, 1 )
         call errint ( 320, 2 )
         call errsub ( 2343, 2, 0, 0 )
         nunkp = 320

      end if
      allocate ( itemp(nusol), istoreright(lenistore), &
                 rjumpleft(nusol), rmultright(nusol), &
                 rjumpright(nusol), rmultleft(nusol), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 10*nusol+lenistore, 'itemp' )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays itemp, jumps and factors

      itemp = 0
      jumps(1:nusol) = 0d0
      factors(1:nusol) = 1d0

!     --- Fill number of rigth unknowns in istoreleft and
!              number of left unknowns in istoreright

      if ( debug ) write(irefwr,*) 'before topbv701'
      call topbv701 ( istoreleft, istoreright, iprinp, nodepairs )
      if ( ierror/=0 ) go to 1000

!     --- Fill all neighbors in istoreleft and istoreright and
!         jumps and factors in rjumpleft, rjumpright rmultleft, rmultright

      call topbv702 ( istoreleft, istoreright, iprinp, &
                      itemp, iwork, rjumpleft, rjumpright, &
                      rmultleft, rmultright)

!     --- Fill final arrays

      call topbv703 ( iwork, itemp, jumps, factors, istoreleft, &
                      nusol, rjumpleft, rmultright, rjumpright, &
                      rmultleft, istoreright )

      deallocate ( itemp, istoreright, rjumpleft, rmultright, &
                   rjumpright, rmultleft, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'itemp' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv7'

      end if  ! ( debug )
      call erclos ( 'topbv7' )

      end
