      subroutine topedgenodes ( npoint, nodelist, lennodelist, &
                                edgenodes, nmeshc, edgelist, lenedgelist )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 09-09-2005 Extra parameter nodes
!        version  1.0  date 22-07-2003
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a list of all nodes on the edge stored in edgelist
!     Make a list of all edges stored in edgelist corresponding to nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     node
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nodelist(*), lennodelist, edgenodes(2,*), &
              nmeshc(*), edgelist(*), lenedgelist

!     edgelist       i    List of edges that are used
!     edgenodes      o    Array of size 2 x lennodelist containing the edge
!                         numbers connected to the nodes stored in nodelist
!                         in the sequence of nodelist
!                         Only the edges on the boundary are taken into
!                         account, so each node has at most 2 edge numbers
!                         if node with sequence number j has only one edge
!                         connected to it edgenodes(2,j) = 0
!     lenedgelist    i    Number of elements in array edgelist
!     lennodelist    o    Number of nodes in nodelist
!     nmeshc         i    nmesh part c
!     nodelist       o    Array of size lennodelist containing node numbers
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icount, i, iedge, j, istart, numnodes, &
              inode, jstart, nodes(npoint)

!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     inode          Node sequence number
!     istart         First position used at a part
!     j              Counting variable
!     jstart         starting position
!     nodes          Work array of length npoint that is filled in the
!                    following way:
!                    nodes(i) = 0, means node i has not yet been stored in
!                                  nodelist
!                    nodes(i)>0, means node i has been stored in nodelist
!     numnodes       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINEDGENODES  Print edge numbers corresponding to nodes in nodelist
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array nodes(1:npoint)
!     Fill array nodelist with all nodes on the edges in edgelist:
!
!        icount := 0
!        For all edges iedge in edgelist do
!           istart := nmeshc(iedge); numnodes := nmeshc(istart)
!           for i := 1 (1) numnodes do
!              inode := nmeshc(istart+i)
!              if ( nodes(inode)=0 ) then
!                 icount := icount+1;  nodes(inode) := icount
!                 nodelist(icount) := inode
!     Fill array edgenodes with all edges from edgelist corresponding to nodes
!        Clear array edgenodes(1:2,1:icount)
!        For all edges iedge in edgelist do
!           istart := nmeshc(iedge); numnodes := nmeshc(istart)
!           for i := 1 (1) numnodes do
!              inode := nmeshc(istart+i)
!              jstart := nodes(inode)
!              if ( edgenodes(1,jstart) = 0 ) then
!                 edgenodes(1,jstart) := iedge
!              else ( edgenodes(1,jstart) # 0 )
!                 edgenodes(2,jstart) := iedge
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topedgenodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topedgenodes'
         call prinin ( edgelist, lenedgelist, 'edgelist' )

      end if

!     --- Clear array nodes(1:npoint)

      do i = 1, npoint
         nodes(i) = 0
      end do

!     --- Fill array nodelist with all nodes on the edges in edgelist:
!         icount := 0

      icount = 0

!     --- For all edges iedge in edgelist do

      do j = 1, lenedgelist

         iedge = edgelist(j)

!        --- istart := nmeshc(iedge); numnodes := nmeshc(istart)

         istart = nmeshc(iedge)
         numnodes = nmeshc(istart)

!        --- for i := 1 (1) numnodes do

         do i = 1, numnodes

!        --- inode := nmeshc(istart+i)

            inode = nmeshc(istart+i)
            if ( nodes(inode)==0 ) then

!           --- nodes(inode)=0: node has not been stored yet
!               icount := icount+1;  nodes(inode) := icount
!               nodelist(icount) := inode

               icount = icount+1
               if ( icount>lennodelist ) then

!              --- Array nodelist too small

                  call errint ( icount, 1 )
                  call errint ( lennodelist, 2 )
                  call errchr ( 'nodelist', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  go to 1000

               end if
               nodes(inode) = icount
               nodelist(icount) = inode

            end if

         end do  ! i = 1, numnodes

      end do  ! j = 1, lenedgelist

      lennodelist = icount

!     --- Fill array edgenodes with all edges from edgelist corresponding
!         to nodes
!         Clear array edgenodes(1:2,1:icount)

      do i = 1, icount
         edgenodes(1,i) = 0
         edgenodes(2,i) = 0
      end do

!     --- For all edges iedge in edgelist do

      do j = 1, lenedgelist

         iedge = edgelist(j)

!        --- istart := nmeshc(iedge); numnodes := nmeshc(istart)

         istart = nmeshc(iedge)
         numnodes = nmeshc(istart)

!        --- for i := 1 (1) numnodes do

         do i = 1, numnodes

!        --- inode := nmeshc(istart+i)

            inode = nmeshc(istart+i)

!           --- jstart := nodes(inode)

            jstart = nodes(inode)
            if ( edgenodes(1,jstart)==0 ) then

!           --- edgenodes(1,jstart) = 0, i.e. first node in edge
!               edgenodes(1,jstart) := iedge

                edgenodes(1,jstart) = iedge

            else  ! not ( edgenodes(1,jstart) = 0 )

!           --- edgenodes(1,jstart) # 0, i.e. second node in edge
!               edgenodes(2,jstart) := iedge

                edgenodes(2,jstart) = iedge

            end if  ! ( edgenodes(1,jstart) = 0 )

         end do  ! i = 1, numnodes

      end do  ! j = 1, lenedgelist

1000  if ( debug ) then

!     --- Debug information

         call prinedgenodes ( edgenodes, nodelist, lennodelist )
         write(irefwr,*) 'End topedgenodes'

      end if
      call erclos ( 'topedgenodes' )

      end
