      subroutine toplev ( levels )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 05-03-2000 Correction for disjoint meshes
!        version  3.1  date 19-01-1997 Correction for disjoint meshes
!        version  3.0  date 06-01-1995 remove linear
!
!   copyright (c) 1994-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the levels in the mesh taking into account a possible numbering
!     of nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     level
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer levels(*)

!     levels         o    Contains the levels in the mesh
!                         Level 1 consists of all nodes from 1 to levels(1)
!                         Level i consists of all nodes from levels(i-1) to
!                         levels(i)  (i>1)
!                         If the mesh is renumbered then
!                         Level 1 consists of all nodes from kmeshj(1) to
!                         kmeshj(levels(1))
!                         Level i consists of all nodes from kmeshj(levels(i-1))
!                         to kmeshj(levels(i))  (i>1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ineigh, inodp, j, neigmx, nmax, nodp, kfree(npoint)
      logical ready, empty, join
      integer, allocatable :: invers(:)

!     empty          Indicates if the set of remaining neighbors is empty
!                    (true) or not (false)
!     i              Counting variable
!     ineigh         Nodal point number of neighbor
!     inodp          Nodal point number after renumbering
!     invers         Work array to store the inverse of kmesh part j
!     j              Counting variable
!     join           if true the first and second level may be joined
!     kfree          Contains both the last_level and the
!                    list_of_free_nodes as described in the pseudo code
!                    The organization of kfree is as follows
!                    kfree(i) = 0, means node i is free
!                    kfree(i) # 0, means that node i has been treated
!                    before
!     neigmx         neighbor number of neighbor with maximum (renumbered)
!                    nodal point number
!     nmax           See neigmx, however, now with respect to a new set
!     nodp           Nodal point sequence number
!     ready          Indicates if the loop has been finished (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   List_of_free_nodes := all nodes
!   ready := false
!   nlevel := 0
!   if ( not renum ) then
!      while ( not ready ) do
!         nodp := first node in List_of_free_nodes
!         nlevel := nlevel + 1
!         remove nodp from List_of_free_nodes
!         if ( nodp has neighbors in List_of_free_nodes ) then
!            neigh_max := largest free neighbor number of nodp
!            last_level := nodp to neigh_max
!            remove all nodes in last_level from List_of_free_nodes
!            levels(nlevel) := neigh_max
!            if ( neigh_max = npoint ) then
!               ready := true
!            else ( neigh_max < npoint )
!               empty := false
!               while ( not empty ) do
!                  neigh_max := largest free neighbor number of last_level
!                  if ( no neighbors found ) then
!                     empty := true
!                  else ( neighbors found )
!                     nlevel := nlevel + 1
!                     levels(nlevel) := neigh_max
!                     last_level := all nodes in List_of_free_nodes
!                                   <= neigh_max
!                     if ( neigh_max = npoint ) then
!                        ready := true
!                        empty := true
!         else ( nodp has no neighbors in List_of_free_nodes )
!            levels(nlevel) := nodp
!            if ( nodp = npoint ) then
!               ready := true
!   else ( renum )
!      See not renum, however, now with respect to the numbering
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ready = .false.
      nlevel = 0
      nodp = 0
      nsplit = 1
      join = .true.

      allocate ( invers(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'invers' )
      if ( ierror/=0 ) go to 1000

      if ( kelmj==0 ) then

!     --- No renumbering

200      if ( .not. ready ) then

!        --- while loop ready
!            Find first free node

            nlevel = nlevel + 1
            nodp = nodp+1
            neigmx = 0
            do i = kmshe1(nodp)+1, kmshe1(nodp+1)
               ineigh = kmshe2(i)
               if ( ineigh>nodp ) neigmx = max(neigmx,ineigh)
            end do
            if ( neigmx>0 ) then

!           --- There are neighbors

               levels(nlevel) = neigmx
               if ( neigmx==npoint ) then

!              --- last point has been found

                  ready = .true.
               else

!              --- last point has not yet been found

                  empty = .false.
250               if ( .not. empty ) then

!                 --- while loop empty

                     nmax = 0
                     do j = nodp, neigmx
                        do i = kmshe1(j)+1, kmshe1(j+1)
                           ineigh = kmshe2(i)
                           if ( ineigh>neigmx ) nmax = max(nmax,ineigh)
                        end do
                     end do
                     if ( nmax==0 ) then

!                    --- No neighbors found

                        empty = .true.
                        nsplit = nsplit + 1
                        nodp = max(nodp,neigmx)
                        kfree(nsplit-1) = nodp
                        if ( nlevel==1 ) join = .false.

                     else

!                    --- There are neighbors

                        nlevel = nlevel + 1
                        nodp = neigmx+1
                        neigmx = nmax
                        levels(nlevel) = nmax
                        if ( neigmx==npoint ) then
                           ready = .true.
                           empty = .true.
                        end if
                     end if
                     go to 250
                  end if

!                 --- end of while loop empty

               end if

            else

!           --- nodp has no neighbors in List_of_free_nodes

               nodp = nodp+1
               levels(nlevel) = nodp
               if ( nodp==npoint ) ready = .true.
            end if
            go to 200
         end if

!        --- end of while loop ready

      else

!     --- Renumbering is applied
!         List_of_free_nodes := all nodes

         do i = 1, npoint
            kfree(i) = 0
         end do

!        --- Store inverse of kmeshj in invers

         do i = 1, npoint
            invers(kmeshj(i)) = i
         end do

300      if ( .not. ready ) then

!        --- while loop ready
!            Find first free node

            do i = 1, npoint
               nodp = kmeshj(i)
               if ( kfree(nodp)==0 ) go to 320
            end do
320         nlevel = nlevel + 1
            kfree(nodp) = 1
            neigmx = 0
            inodp = invers(nodp)
            do i = kmshe1(nodp)+1, kmshe1(nodp+1)
               ineigh = kmshe2(i)
               if ( kfree(ineigh)==0 ) &
                   neigmx = max(neigmx,invers(ineigh))
            end do

            if ( neigmx>0 ) then

!           --- There are neighbors

               levels(nlevel) = neigmx
               if ( neigmx==npoint ) then

!              --- last point has been found

                  ready = .true.

               else

!              --- last point has not yet been found
!                  Construct last_level

                  do i = inodp, neigmx
                     kfree(kmeshj(i)) = 1
                  end do
                  empty = .false.
350               if ( .not. empty ) then

!                 --- while loop empty
!                     Compute largest free neighbor number of last_level

                     nmax = 0
                     do j = inodp, neigmx
                        nodp = kmeshj(j)
                        do i = kmshe1(nodp)+1, kmshe1(nodp+1)
                           ineigh = kmshe2(i)
                           if ( kfree(ineigh)==0 ) &
                               nmax = max(nmax,invers(ineigh))
                        end do
                     end do

                     if ( nmax==0 ) then

!                    --- No neighbors found

                        empty = .true.
                        nsplit = nsplit + 1
                        if ( nlevel==1 ) join = .false.

                     else

!                    --- There are neighbors

                        nlevel = nlevel + 1
                        do i = neigmx+1, nmax
                           kfree(kmeshj(i)) = 1
                        end do
                        inodp = neigmx+1
                        neigmx = nmax
                        levels(nlevel) = nmax
                        if ( neigmx==npoint ) then
                           ready = .true.
                           empty = .true.
                        end if

                     end if
                     go to 350

                  end if

!                 --- end of while loop empty

               end if
            else

!           --- nodp has no neighbors in List_of_free_nodes

               levels(nlevel) = inodp
               if ( inodp==npoint ) ready = .true.
            end if
            go to 300
         end if

!        --- end of while loop ready

      end if
      if ( join .and. nlevel>1 ) then

!     --- in case of linear elements the levels 1 and 2 are made to one level

         do i = 1, nlevel-1
            levels(i) = levels(i+1)
         end do
         nlevel = nlevel-1

      end if

      if ( nsplit>1 ) then

!     --- Mesh consists of subparts

         if ( kelmj==0 ) then

!        --- No renumbering, fill last part of levels

            do i = 1, nsplit-1
               levels(nlevel+i) = kfree(i)
            end do
            levels(nlevel+nsplit) = npoint

         end if

      end if

      deallocate ( invers, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'invers' )
1000  end
