      subroutine topol ( jrenum )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.11 date 11-11-2014 Extra debug
!        version 10.10 date 29-05-2014 remove call to time subroutine
!        version 10.9  date 28-08-2013 Remove call to checkspl
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    integer information with respect to the mesh is computed
!    and filled in array kmesh
!    the mesh must have been created before
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: jrenum(*)

!   jrenum   i  Array of length 10 with information about the
!               renumbering.
!               Array must be filled as follows:
!                1: if>=0, only position 1 is filled.
!                   This position contains the parameter method according
!                   to the old SEPRAN definition, i.e.
!                   jrenum = 1000: Skip filling of levels
!                   jrenum = 9: Use default depending on mesh
!                   else:
!                   jrenum = ispecial*100 + method + 3 * ipref
!                   ispecial = 0: standard
!                              1: ivertx=1, imidp=1, icentr=3
!                   If<0, -jrenum(1) contains the sequence number
!                   of the last entry filled by the user
!                   All other values are filled with defaults
!                2: method Indication of the renumbering method used
!                          The algorithms are compared to the original
!                          numbering:
!                          0  No renumbering
!                          1  Cuthill-Mckee algorithm
!                          2  Sloan algorithm
!                          3  Both algorithms,  the best is used
!                          5  Nested dissection
!                          9  Choose depending on mesh
!                   Default: 9
!                3: ipref  Indication of the criterion to be used to find
!                          the best algorithm:
!                          0  Choose the smallest profile
!                          1  Choose the smallest band width
!                          2  Use only the renumbered algorithm (method=1 or 2)
!                   Default: 0
!                4: iprint Indicates if information about the renumbering
!                          must always be printed (1) or only depending on
!                          ioutp (0)
!                   Default: 0
!                5: ivertx Priority for vertex nodes (1-3)
!                   Default: 3
!                6: imidp  Priority for mid point nodes (1-3)
!                   Default: 3
!                7: icentr Priority for centroids (1-3)
!                   Default: 3
!                8: ilevel Indicates if renumbering of special nodes must be
!                          performed per level (1) or not (0)
!                   Default: 0
!                9: istmet Indicates how the start of the renumbering
!                          algorithm must be performed
!                          0  Automatic
!                          1  Start user point given
!                          2  Start curve given
!                          3  Start surface given
!                          4  Start point given
!                   Default: 0
!               10: istval Value of start point, curve f surface
!                   Default: 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical renumber
      integer, allocatable :: kfree(:), levels(:), meshsplt(:)

!     kfree          Contains both the last_level and the
!                    list_of_free_nodes as described in the pseudo code
!                    The organization of kfree is as follows
!                    kfree(i) = 0, means node i is free
!                    kfree(i) # 0, means that node i has been treated
!                    before
!     levels         Contains the levels in the mesh
!                    Level 1 consists of all nodes from 1 to levels(1)
!                    Level i consists of all nodes from levels(i-1) to
!                    levels(i)  (i>1)
!                    If the mesh is renumbered then
!                    Level 1 consists of all nodes from kmeshj(1) to
!                    kmeshj(levels(1))
!                    Level i consists of all nodes from kmeshj(levels(i-1))
!                    to kmeshj(levels(i))  (i>1)
!     meshsplt       In this array the largest node number of the various
!                    disconnected parts of the mesh are stored
!                    Length: number of parts (<npoint)
!     renumber       If true standard renumbering must be applied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHE  Fill array kmesh part e
!     TOPLEV         Fill array levels
!     TOPOL00        Fill array kmesh part v with information about element
!                    groups coupled with element groups
!     TOPREN         Renumber nodal points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topol'

      end if
      if ( ierror/=0 ) go to 1000

      call sepfillkmesh ( kmesh )

!     --- Create temporary space

      allocate ( kfree(npoint), levels(2*npoint), meshsplt(npoint), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'kfree' )
      if ( ierror/=0 ) go to 1000

!     --- Create kmesh parts e if necessary

      if ( debug ) write(irefwr,*) 'Before sepfillkmeshe'
      if ( kelme==0 ) call sepfillkmeshe

!     --- Skip rest in case jrenum(1) = 1000

      if ( jrenum(1)==1000 ) go to 1000

      nsplit = 0
      renumber = .false.
      if ( debug ) write(irefwr,*) 'Before renumbering'
      if ( jrenum(1)/=0 ) then

!     --- Perform renumbering and compute number of parts the mesh consists of

         if ( jrenum(1)>0 ) then
            renumber = .true.
         else if ( jrenum(2)/=4 ) then
            renumber = .true.
         end if

         if ( renumber ) then

!        --- Standard renumbering

            if ( debug ) write(irefwr,*) 'Before topren'

            call topren ( jrenum, meshsplt )

         else

!        --- Renumbering with respect to element groups

            call topol00

         end if
         if ( ierror/=0 ) go to 1000

      end if
      if ( debug ) write(irefwr,*) 'After renumbering'

!     --- Compute levels in the mesh (Fill array levels)

      if ( debug ) write(irefwr,*) 'Before toplev'
      call toplev ( levels, kfree )
      if ( nsplit==1 ) nsplit = 0
      if ( nsplit>0 ) then

!     --- nsplit> 0, Copy information of parts after levels

         levels(nlevel+1:nlevel+nsplit) =  meshsplt(1:nsplit)

      end if

!     --- make array kmeshk permanent

      kelmk = nlevel+nsplit
      allocate ( km(iseqkm)%kmeshk(kelmk) )
      kmeshk => km(iseqkm)%kmeshk
      kmeshk = levels(1:kelmk)

      deallocate ( kfree, levels, meshsplt, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kfree' )

1000  if ( debug ) write(irefwr,*) 'End topol'
      call erclos ( 'topol' )

      end subroutine topol
