      subroutine fnmateri( ichoice, s, se, eps, detf, matpar, makese )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.1    date   21-01-1998
!        version 1.0    date   25-07-1997
!
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute 2nd-Piola-Kirchoff stress from given Green-Lagrange
!     strains. The determinant of the deformation gradient, can be
!     used in this relationship
! **********************************************************************
!
!                       KEYWORDS
!
!      elasticity_equation
!      solid
!      material_behaviour
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      logical          makese
      double precision s(6), eps(6), detf(*), matpar(21), &
                       se(6,6)
      integer ichoice

!     detf       i    determinant of deformation gradient
!     eps        i    Green-Lagrange strains
!     ichoice    i    material model number ( icoef 4 )
!                     NOTE: ichoice = imateri + 10*icomprs + 100*iadd
!                     Choices for imateri:
!                       0 : hookean
!                            coef6 = E
!                            coef7 = nu
!                       1 : orthotropic hookean
!                            coef6 = E1
!                            coef7 = E2
!                            coef8 = nu1
!                            coef9 = nu2
!                            coef10= G1
!                       2 : exponential
!                       3 : Mooney Rivlin
!                            coef6 = G
!                            coef7 = K
!                            if K==0, the model switches to neo-Hookean
!                       4 : exponential variant
!                     if icomprs > 0, coef8 and coef9 contain info about
!                     the type of compressibility function
!     makese    i     se must be computed (true)
!     matpar    i     material parameters (User defines!) for every
!                     integration point
!     s         o     2nd-PK-stresses: You only have to store symmetric
!                     components!
!     se        o     tangential matrix.
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer          i,j, imateri, icomprs

!     i         counter
!     icomprs   compression model number
!     imateri   material model number
!     j         counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     FNCOMPRS  Compute reduced integrated incompressibility term
!     FNEXPON1  fill estimate of 2nd Piola Kirchoff stress plus tangential
!               stiffness matrix for one integration point, exponential
!     FNEXPON2  fill estimate of 2nd Piola Kirchoff stress plus tangential
!               stiffness matrix for one integration point, exponential
!     FNHOOKE   fill estimate of 2nd Piola Kirchoff stress plus tangential
!               stiffness matrix for one integration point (hookean material)
!     FNMOONEY  fill estimate of 2nd Piola Kirchoff stress plus tangential
!               stiffness matrix for one integration point (mooney-Rivlin)
!     FNORTHOT  fill estimate of 2nd Piola Kirchoff stress plus tangential
!               stiffness matrix for one integration point (orthotropic)
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1028   The required material model has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      icomprs = ichoice/10
      imateri = ichoice - icomprs*10 + 1

!     --- reset s and se (mandatory, especially for non-3D)

      do i = 1,6
         s(i) = 0d0
         do j = 1,6
            se(i,j) = 0d0
         end do
      end do

      goto ( 100, 110, 120, 130, 140 ) imateri

      call eropen('fnmateri')
      call errint ( imateri, 1 )
      call errsub ( 1028, 1, 0, 0 )
      call erclos('fnmateri')
      call instop

!     --- isotropic linear (Hookean) material: imateri = 0

100   call fnhooke ( eps, matpar, s, se )
      goto 1000

!     --- orthotropic: imateri = 1

110   call fnorthot ( eps, matpar, s, se )
      goto 1000

!     --- exponential, variant 1: imateri = 2

120   call fnexpon1 ( eps, matpar, s, se, makese )
      goto 1000

!     --- Mooney Rivlin behaviour: imateri = 3
!         S = c0 * E + c1* tr(E)( C - C**2)

130   call fnmooney ( eps, matpar, s, se )
      goto 1000

!     --- exponential, variant 2: imateri = 4

140   call fnexpon2 ( eps, matpar, s, se, makese )
      goto 1000

1000  if ( ichoice.ge.100 ) then

!     --- add incompressibility part

         call fncomprs( ichoice, s, se, eps, detf(1), matpar, makese )

      end if
      end
