      subroutine funcmindist ( coorlc, gamma, index1, g, ni, &
                               dni, vecold, index3 )
! ======================================================================
!
!        programmer    Martijn Cox and Niels Driessen
!        version  1.0  date 04-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute distance and derived quantities between deformable and rigid
!   body in a contact problem, needed to build the element matrix
!   At this moment, only for a rigid body consisting of (combinations of)
!   spheres, cylinders, and/or plates. Only implemented for Newton-Cotes
!   integration. In this subroutine, the quantities are stored in the nodal
!   points. However, when calculating the element matrices and right hand
!   side it will be assumed that the quantities are stored in the integration
!   points. This is allowed because Newton-Cotes integration is applied.
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleconsts
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(n), index3(numold,*)
      double precision coorlc(ndim,n), gamma(n), g(n), &
                       ni(n,ndim), dni(n,ndim,ndim), &
                       vecold(*)

!     coorlc           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dni            o    array containing information (change in normal of
!                         indenter) needed (in nodal points)
!     g              o    array of length n in which the values of the contact
!                         distance g in the nodal points are stored.
!     gamma          i    array of length n containing the contact pressures
!                         in the nodal points
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     index3         i    Two-dimensional integer array of length NUMOLD x
!                         NINDEX containing the positions of the "old"
!                         solutions in array
!                         VECOLD with respect to the present element
!                         For example VECOLD(INDEX3(i,j)) contains the j th
!                         unknown with respect to the i th old solution vector.
!                         The number i refers to the i th vector corresponding
!                         to IVCOLD in the call of SYSTM2 or DERIVA
!     ni             o    array containing normal of indenter (in nodal points)
!     vecold         i    In this array all preceding solutions are stored, i.e.
!                         all solutions that have been computed before and
!                         have been carried to system or deriva by the parameter
!                         islold in the parameter list of these main subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision normi, matrI(ndim,ndim), p0(ndim), r0, theta1, &
                       theta2
      integer i, j, k, igeom, icont(n), icheck(n)

!     i              Counting variable
!     icheck         Help array for contact
!     icont          Help array for contact which contains information on the
!                    contact status of nodal points in the current element
!                    0: no contact; 1: contact; the values have to stored in
!                    vecold
!     igeom          Choice parameter for indentor geometry
!     j              Counting variable
!     k              Counting variable
!     matri          ndim * ndim unity matrix
!     normi          norm of distance
!     p0             array of length ndim containing coorlcdinates of center
!                    of indenter
!     r0             radius of indentor geometry (sphere, cylinder)
!     theta1         angle of indentor geometry with coorlcdinate axes
!     theta2         angle of indentor geometry with coorlcdinate axes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Note: the nodal points and integration points coincide because
!          Newton-Cotes integration is used
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- unity matrix

      do i = 1, ndim
         do j = 1, ndim
            matrI(i,j) = 0d0
            if(i.eq.j) then
               matrI(i,j) = 1d0
            end if
         end do
      end do

!     --- define geometry that is used (igeom):
!         1: plate (3D)
!         2: plate under angle theta1 with y-axis (3D)
!         3: plate under angle theta1 with y-axis and theta2 with x-axis(3D)
!         4: sphere(3D)
!         5: cylinder along z-axis(3D)
!         6: plate (2D)

      igeom = 5

!     --- loop over nodal/integration points (Newton-Cotes)

      do i = 1, n

!     --- derived quantities for a plate at z=1

         if( igeom.eq.1 ) then

!        --- Height of plate (z=1)

            p0(3) = 1d0

!           --- Contact distance

            g(i)  = p0(3) - coorlc(3,i)

!           --- Normal of indenter

            ni(i,1) = 0d0
            ni(i,2) = 0d0
            ni(i,3) = -1d0

!           --- Linearization of indenter normal

            do j = 1, ndim
               do k = 1, ndim
                  dni(i,j,k) = 0d0
               end do
            end do

!           --- derived quantities for a plate under angle theta1 with y-axis

         else if( igeom.eq.2 ) then

!        --- Angle with y-axis

            theta1 = 45d0 * (pi/180d0)

!           --- Position of plate

            p0(1) = 1.5d0 * dcos(theta1)
            p0(2) = 1.5d0 * dsin(theta1)

!           --- Normal of indenter

            ni(i,1) = -dcos(theta1)
            ni(i,2) = -dsin(theta1)
            ni(i,3) = 0d0

!           --- Linearization of indenter normal

            do j = 1, ndim
               do k = 1, ndim
                  dni(i,j,k) = 0d0
               end do
            end do

!           --- Contact distance: dot( ni, (pb-pi) )

            g(i)  = ( coorlc(1,i) - p0(1) ) * ni(i,1) + &
                    ( coorlc(2,i) - p0(2) ) * ni(i,2)

!           --- derived quantities for a plate under angles

         else if( igeom.eq.3 ) then

!        --- Height of plate

            p0(3) = 1d0

!           --- Angle with y-axis

            theta1 = 1d0/8d0*pi

!           --- Angle with x-axis

            theta2 = 1d0/8d0*pi

!           --- Normal of indenter

            ni(i,1) = dsin(theta1) * dcos(theta2)
            ni(i,2) = sin(theta2)
            ni(i,3) = -dcos(theta1) * dcos(theta2)

!           --- Linearization of indenter normal

            do j = 1, ndim
               do k = 1, ndim
                  dni(i,j,k) = 0d0
               end do
            end do

!           --- Contact distance

            g(i)  = ( coorlc(1,i) - 0d0  ) * dsin(theta1)* dcos(theta2) + &
                    ( coorlc(2,i) - 0d0  ) * dsin(theta2) - &
                    ( coorlc(3,i) - p0(3)) * dcos(theta1) * dcos(theta2)

!           --- derived quantities for a sphere at z=1 with r=1

         else if( igeom.eq.4 ) then

!        --- Radius of sphere

            r0 = 1.0d0

!           --- Center of sphere

            p0(1) = 0d0
            p0(2) = 0d0
            p0(3) = 1d0 + r0

!           --- Contact distance
!               normal of indenter, for a sphere

            normi = dsqrt( ( p0(1) - coorlc(1,i) )**2d0 + &
                           ( p0(2) - coorlc(2,i) )**2d0 + &
                           ( p0(3) - coorlc(3,i) )**2d0 )
            ni(i,1) = ( coorlc(1,i) - p0(1) )/normi
            ni(i,2) = ( coorlc(2,i) - p0(2) )/normi
            ni(i,3) = ( coorlc(3,i) - p0(3) )/normi

!           --- Contact distance

            g(i) = normi - r0

!           --- linearization of indenter normal

            do j = 1, ndim
               do k = 1, ndim
                  dni(i,j,k) = 1d0/normi*(matrI(j,k) - ni(i,j)*ni(i,k))
               end do
            end do

!           --- derived quantities for a cylinder

         else if( igeom.eq.5 ) then

!        --- Radius of cylinder

            r0 = 1d0

!           --- Central axis of cylinder

            p0(1) = 2.5d0 * dcos(45d0*pi/180d0)
            p0(2) = 2.5d0 * dsin(45d0*pi/180d0)

!           --- Contact distance
!               normal of indenter, for a cylinder

            normi = dsqrt( ( p0(1) - coorlc(1,i) )**2d0 + &
                           ( p0(2) - coorlc(2,i) )**2d0 + &
                           (      0d0          )**2d0 )
            ni(i,1) = ( coorlc(1,i) - p0(1) )/normi
            ni(i,2) = ( coorlc(2,i) - p0(2) )/normi
            ni(i,3) = (          0d0      )/normi

!           --- Contact distance

            g(i)  = normi - r0

!           --- linearization of indenter normal

            do j = 1, ndim
               do k = 1, ndim
                  dni(i,j,k) = 1d0/normi*(matrI(j,k) - ni(i,j)*ni(i,k))
               end do
               dni(i,3,3) = 0d0
            end do

         else if ( igeom.eq.6 ) then

!        --- Height of plate (y=1.1)

            p0(2) = 1.1d0

!           --- Contact distance

            g(i)  = p0(2) - coorlc(2,i)

!           --- Normal of indenter

            ni(i,1) = 0d0
            ni(i,2) = -1d0

!           --- Linearization of indenter normal

            do j = 1, ndim
               do k = 1, ndim
                  dni(i,j,k) = 0d0
               end do
            end do

         end if

!        --- check for contact

         icont(i)  = int(vecold(index3(4,2*i-1)))
         icheck(i) = int(vecold(index3(4,2*i)))
         if (icheck(i).eq.0) then
            icheck(i) = 1
            if ( (icont(i).eq.0) .and. (g(i).lt.0d0) ) then
               icont(i) = 1
            write(*,*) 'node added:   ', index1(i), &
                       ' with g and gamma: ', g(i), ', ', gamma(i)
            else if ( (icont(i).eq.1) .and. (gamma(i).gt.0d0) ) then
               icont(i) = 0
            write(*,*) 'node removed: ', index1(i), &
                       ' with g and gamma: ', g(i), ', ', gamma(i)
            end if
            vecold(index3(4,2*i-1)) = icont(i)
            vecold(index3(4,2*i))   = icheck(i)
         end if
      end do

      end
