      subroutine el0600 ( iuser, user, jelp, kfirst, jheat, isub, &
                          isup, array, iseq, itypln )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.7  date 03-09-2000 Extension for scalars
!        version  1.6  date 29-05-1999 Adiitin for icheld=39
!
!   copyright (c) 1995-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM600 or
!     ELD600 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, kfirst, jheat, isub, isup, &
              iseq, itypln

!     array   o      Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                  2D            3D
!                    1:   alpha 11         alpha 11       alpha 11
!                    2:                    alpha 12       alpha 12
!                    3:                    alpha 22       alpha 13
!                    4:                                   alpha 22
!                    5:                                   alpha 23
!                    6:                                   alpha 33
!                    7:   u 1              u 1            u 1
!                    8:                    u 2            u 2
!                    9:                                   u 3
!                   10:   beta             beta           beta
!                   11:   f                f              f
!                   12:   rho cp           rho cp         rho cp
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     iseq     i     Sequence number of vector in VECOLD/ISLOLD
!                    In case of a call by ELI600 it is the parameter JDEGFD
!     isub     o     Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup     o     Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     itypln   o     Indicates the type of lineariation to be performed
!                    Possible values:
!                    1: Picard
!                    2: Newton
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jelp     o     Indication of the type of basis function subroutine must
!                    be called by ELM600. Possible vaues:
!                    1:  Subroutine ELP610  (Linear line element)
!                    2:  Subroutine ELP611  (Quadratic line element)
!                    3:  Subroutine ELP620  (Linear triangle)
!                    4:  Subroutine ELP622  (Quadratic triangle)
!                    5:  Subroutine ELP621  (Bilinear quadrilateral)
!                    6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                   11:  Subroutine ELP630  (Linear tetrahedron)
!                   12:  Subroutine ELP631  (Quadratic tetrahedron)
!                   13:  Subroutine ELP632  (Trilinear brick)
!                   14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat    o     Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed
!                    Possible values:
!                    0:  no mass matrix
!                    1:  diagonal mass matrix with constant coefficient
!                    2:  diagonal mass matrix with variable coefficient
!                   11:  full mass matrix with constant coefficient
!                   12:  full mass matrix with variable coefficient
!     kfirst   o     This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     user     i     User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, icoef, kchois, mrule, &
              ichtrn, metup, ishift
      double precision rn
      logical linear, funchk

!     funchk     Indicates whether there are coefficients that must be
!                evaluated by funcc3 (true) or not (false)
!     i          Counting variable
!     ichtrn     Defines the direction of the computation of an integral
!     icoef      Coefficient sequence number
!     ishift     Shift in coefficient sequence number
!     isubcf     Under bound for coefficients check
!     j          Counting variable
!     jwcons     Indication if the weight function is constant (1) or not (2)
!     kchois     Indication what type of parameters must be computed
!                Possible values:
!                0:  Standard method, the calling subroutine is ELM600
!                    It is supposed that an element matrix must be computed
!                1:  Subroutine ELD600 calls this subroutine, only
!                    derivatives must be computed
!                2:  Subroutine ELD600 or ELI600 calls this subroutine, &
!                    a user function defined as first parameter in
!                    IUSER/USER must be filled in elemvc (icheld=9)
!                3:  Subroutine ELD600 calls this subroutine, &
!                    alpha times the gradient must be stored in ELEMVC
!                    The first 3 parameters of IUSER/USER are used
!     linear     Indicates whether the element is a linear simplex (true)
!                or not (false)
!     metup      Upwind parameter
!     mrule      Type of integration rule as given by the user
!     ncoef      Number of coefficients in the equation
!     rn         Double precision value of inpelm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL1002         Compute JSECND for diffusion terms
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  This parameter is no longer in use.
!         The parameter imas in BUILD defines the type of mass matrix
!         Now the parameter is reserved for future purposes
!     2:  Type of upwinding
!         0: no upwinding
!                                                        - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates (not yet available)
!         2: Polar co-orindates (not yet available)
!     5:  not yet available
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!     In the case of a call by INTEGR:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
!     In the case of a call by DERIV:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2-5:  Not yet used
!         In the cases ICHELD = 6, 16 the positions 6-11 contain
!         information about the matrix A:
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!         In the case ICHELD = 21 position 6 contains information about the
!         function f to be filled:
!     6:  Function f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     264   jdegfd has wrong value
!     341   Icheli has wrong value
!     406   Value of icheld not allowed
!     616   Fatal error: METUPW has wrong value
!     771   Rho is non-positive
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1768   Coefficient is filled that is not used
!    1771   Shape number not yet implemented
!    1785   Upwinding has only been implemented for cartesian and axisymmetric
!           co-ordinates systems
!    1787   iseq has incorrect value
!    1863   Value of icheli has not yet been implemented for this element shape
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0600' )

!     --- set parameters depending on the type numbers

      ishift = 0
      if (itype.eq.600) then

!     --- itype = 600, 1-D, 2-D and 3-D spectral element for Elliptic
!         (convection-diffusion) equations)

         n = inpelm
         m = inpelm
         rn = n
         mone = int(rn**(1d0/ndim)+.5d0)
         jdercn = 1
         nvert = 2**ndim
         ndimlc = ndim

      end if
      if ( icheld.lt.0 ) then
         icheli = mod(-icheld,10)
         ichtrn = -icheld/10
      else
         icheli = 0
         ichtrn = 0
      end if
      if ( icheli.ne.7 ) then
         call elcomm
      else
         iweigh = 1
         ideriv = 1
         irule  = 1
         igausp = 0
         iphi   = 0
         jcart  = 1
         icoor  = 1
         ncomp  = 1
         jdiag  = 1
         idim   = 1
         jderiv = 0
         jtrans = 0
         jind   = 1
         kstep  = 1
      end if
      jcart = 1
      metupw = 0
      metup  = 0
      jheat  = 0
      jtrans = 0

      kchois = 1

      if ( icheld.ge.0 ) iseq = 1

!     --- Compute kchois

      if ( icheld.eq.0 ) then

!     --- Call by elm600, all coefficients required

         kchois = 0

      else if ( icheli.gt.0 .and. icheli.ne.7 ) then

!     --- call by eli600, one (new) coefficient required

         kchois = 2

      end if

!     --- if residual have to be computed

      if (icheld.eq.25 .or. icheld.ge.30) kchois = 0
      if (icheld.eq.26 .or. icheld.ge.30) kchois = 0
      if (icheld.eq.38 .or. icheld.eq.39 .or. icheld.eq.40 ) &
          kchois = 1

      if ( icheld.ge.41  ) then

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      else if ( icheli.gt.7 .or. ichtrn.gt.ndim ) then

         call errint ( -icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 341, 3, 0, 0 )

      end if
      if ( icheli.ge.3 .and. icheli.le.5 ) then

         ix = icheli-2
         if ( ix.gt.ndim ) then
            call errint ( -icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 341, 3, 0, 0 )
         end if

      end if
      if ( icheli.eq.6 .and. iseq.le.0 .or. iseq.gt.ndim ) then
         call errint ( iseq, 1 )
         call errint ( ndim, 2 )
         call errsub ( 264, 2, 0, 0 )
      end if

      if ( icheli.eq.6 ) ncomp = ndim

!     --- Set parameters depending on the shape numbers

      n = inpelm
      m = n
      mn = m * n
      jelp = ishape
      if ( jelp.eq.1 .or. jelp.eq.5 .or. jelp.eq.13 ) then

!     --- linear element

         linear = .true.
         mn = m
         jdercn = 0

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

      if ( kchois.eq.0 ) then

!     --- standard situation

         ncoef = 12

      else if ( kchois.eq.1 ) then

!     --- kchois = 1, no coefficients required

         ncoef = 0

      else if ( kchois.eq.2 .or. kchois.eq.4 ) then

!     --- kchois = 2 or 4, one coefficient required

         ncoef = 1

      else if ( kchois.eq.3 ) then

!     --- kchois = 3, six coefficients required

         ncoef = 6

      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      if ( ncoef.gt.0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos.lt.7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if

         nuser = 0

         if ( kchois.eq.0 ) then

!        --- Standard situation, Compute the parameters jheat, metupw

            jheat = imas
            ipos = ipos + 1
            ishift = ishift+1
            if ( jheat.eq.0 ) notmas = 1

            metupw = inirepscal ( iuser(ipos) )
            ipos = ipos + 1
            ishift = ishift+1
            metup  = mod(mod(metupw,1000),100)
            if ( metup.gt.1 ) then
               call errint ( metupw, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 616, 4, 0, 0 )
            end if

         else if ( icheld.gt.0 .and. icheld.ne.25  .and. &
                  icheld.ne.26 ) then

!        --- icheld > 0, Compute iseq

            iseq = inirepscal ( iuser(ipos) )
            ipos = ipos + 4
            ishift = ishift+4
            if ( iseq.lt.0 .or. iseq.gt.numold ) then
               call errint ( iseq, 1 )
               call errint ( numold, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1787, 5, 0, 0 )
            end if
            if ( iseq.eq.0 ) iseq = 1

         end if

         if ( kchois.eq.0 .or. kchois.eq.2 ) then

!        --- Call by BUILD or INTEGR, Compute the parameters jcart and irule

            mrule = inirepscal ( iuser(ipos) )
            if ( mrule.eq.0 ) then
               if  (itype.eq.600) irule = 1
            else
               irule = mrule
            end if
            ipos = ipos + 1
            ishift = ishift+1
            if ( mrule.lt.0 .or. mrule.gt.40 ) then
               call errint ( mrule, 1 )
               if ( kchois.eq.0 ) then
                  call errint ( 3, 2 )
               else
                  call errint ( 1, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1766, 5, 0, 0 )
            end if

            ipos = ipos + 1
            ishift = ishift+1
            if ( jcart.lt.0 .or. jcart.gt.2 ) then
               call errint ( jcart, 1 )
               if ( kchois.eq.0 ) then
                  call errint ( 4, 2 )
               else
                  call errint ( 2, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1767, 5, 0, 0 )
            end if

         end if
         if ( metup.gt.0 .and. jcart.gt.2 ) then
            call errint ( jcart-1, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 1785, 4, 0, 0 )
         end if

         if ( mrule.eq.0 .and. metup.gt.0 ) then

!        --- mrule = 0, and upwind
!            A different integration rule is applied

            if (jelp.eq.3 .or. jelp.eq.4 ) then
               irule = 3
            else if (jelp.eq.11 ) then
               irule = 2
            end if

         end if

         if ( linear ) then
            jwcons = min(jcart,2)
         else
            jwcons = 2
         end if
         if ( icheld.gt.0 .and. mrule.eq.0 ) irule = 1
         if ( jheat.eq.2 .and. (jelp.le.3 .or. jelp.eq.5) .and. &
              mrule.eq.0 ) then
            if ( jelp.eq.2 ) then
               irule = 4
            else
               irule = max ( irule, 3 )
            end if
         else if ( jheat.eq.2 .and. jelp.le.6 .and. mrule.eq.0 ) then
            irule = max ( irule, 2 )
         end if
         if ( icheli.gt.2 .and. icheli.le.5 ) irule = 1

         if ( irule.ne.1 ) then

!        --- irule > 1, Gauss rule

            igausp = 1
            iphi = 1
            jdiag = 2
            jwcons = 2
            if ( jelp.le.2 ) then
               if ( jelp.eq.2 .and. irule.eq.2 ) irule = 3
               m = irule - 1
            else if ( jelp.eq.3 .or. jelp.eq.5 ) then
               if ( irule.eq.2 ) then
                  m = 1
               else
                  m = 4
               end if
            else if ( jelp.eq.4 ) then
               if ( irule.eq.2 ) then
                  m = 4
               else
                  m = 7
               end if
            else if ( jelp.eq.6 ) then
               if ( irule.eq.2 ) then
                  m = 4
               else
                  m = 9
               end if
            else if ( jelp.eq.7 ) then
               if ( irule.eq.2 ) m = 4
            else if ( jelp.eq.11 ) then
               if ( irule.eq.2 ) then
                  m = 1
               else if ( irule.eq.3 ) then
                  m = 4
               else
                  m = 5
               end if
            else if ( jelp.eq.12 ) then
               irule = 1
            else if ( jelp.eq.13 ) then
               if ( irule.eq.2 ) then
                  m = 1
               else
                  m = 8
               end if
            else if ( jelp.eq.14 ) then
               if ( irule.eq.2 ) then
                  m = 8
               else
                  m = 27
               end if
            end if
            mn = m * n

         else if ( metup.gt.0 ) then

!        --- upwinding, fill array phi

            iphi = 1

         end if
         isub = 1
         ipos = ipos + 1
         ishift = ishift+1

         do i = isub, ncoef
            call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
         end do

!        --- Test array lengths, number of nodes and number of degrees
!            of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )

      end if

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      if ( kchois.eq.0 .and. (matrix .or. metup.gt.0 .or. icheld.eq.25 &
            .or. icheld.eq.26) &
            .or. kchois.eq.3) &
           then

!     --- matrix = true  The element matrix must be computed.
!         or metupw > 0:  Upwinding
!                        Compute jsecnd, idim and jzero
!         First consider the diffusion tensor:

         if ( itype.eq.600  ) then

            call el1002 ( 0, 5, 1, jelp, 0, array, ndim )

         end if
         if ( kchois.ne.3 ) then

!        --- Next consider the Convective terms:
!            compute idim and first component idim

            kfirst = 1
            if ( jelp.le.2 ) then

!           --- ndim = 1

               do i = 8, 9
                  if ( ind(i).ne.0 ) then
                     call errint ( i+5, 1 )
                     call errint ( ielem, 2 )
                     call errint ( itype, 3 )
                     call errint ( ielgrp, 4 )
                     call errwar ( 1768, 4, 0, 0 )
                     ind(i) = 0
                  end if
               end do

               if ( ind(7).eq.0 ) then

!              --- u1 = 0

                  idim = 0

               end if

            else if ( jelp.gt.2 .and. jelp.le.9 ) then

!           --- ndim = 2

               if ( ind(9).ne.0 ) then
                  call errint ( 14, 1 )
                  call errint ( ielem, 2 )
                  call errint ( itype, 3 )
                  call errint ( ielgrp, 4 )
                  call errwar ( 1768, 4, 0, 0 )
                  ind(9) = 0
               end if
               if ( ind(7).eq.0 ) then

!              --- u1 = 0

                  if ( ind(8).eq.0 ) then

!                 --- u1 = 0 and u2 = 0

                     idim = 0

                  else

!                 --- u1 = 0, u2 # 0

                     idim = 1
                     kfirst = 2

                  end if

               else if ( ind(8).eq.0 ) then

!              --- u2 = 0 and u1 # 0

                  idim = 1

               else

!              --- General

                  idim = 2

               end if

            else

!           --- ndim = 3

               if ( ind(7).eq.0 ) then

!              --- u1 = 0

                  if ( ind(8).eq.0 ) then

!                 --- u1 = 0 and u2 = 0

                     if ( ind(9).eq.0 ) then

!                    --- u1 = 0, u2 = 0 and u3 = 0

                        idim = 0

                     else

!                    --- u1 = 0 and u2 = 0

                        idim = 1
                        kfirst = 3

                     end if

                  else

!                 --- u1 = 0

                     idim = 2
                     kfirst = 2

                  end if

               else if ( ind(8).eq.0 .and. ind(9).eq.0 ) then

!              --- u2 = 0 and u3 = 0

                  idim = 1

               else

!              --- General

                  idim = 3

               end if

            end if

!           --- Constant part of the equation:
!               --- compute jzero

            if ( ind(10).eq.0 ) then

!           --- beta = 0

               jzero = 0

            else if ( ind(10).lt.0 .or. ind(10).eq.2002 ) then

!           --- beta is constant (per element)

               jzero = jwcons

            else

!           --- General

               jzero = 2

            end if

         end if

      end if

      if ( kchois.eq.0 .and. vector .or. kchois.eq.2 ) then

!     --- Vector = true, the element vector must be computed
!                       or kchois = 2, the first parameter must be computed
!         Compute jconsf

         if ( kchois.eq.0 ) then

            icoef = 11

         else

            icoef = 1

         end if
         if ( ind(icoef).eq.0 ) then

!        --- F = 0, no right-hand side

            if ( itype.eq.600 .or. itypln.eq.1 ) notvec = 0

         else if ( ind(icoef).lt.0 .or. ind(icoef).eq.2002 ) then

!        --- F is constant (per element)

            jconsf = jwcons

         else

!        --- General

            jconsf = 2

         end if

      end if

      if ( jheat.gt.0 ) then

!     --- jheat > 0, i.e. compute mass matrix

         if ( ind(12).eq.0 .or. ind(12).lt.0 .and. array(12*m).le.0 ) &
              then

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 771, 3, 0, 0 )

         else if ( ind(12).lt.0 .or. ind(12).eq.2002 ) then
            jheat = jwcons + 10*(jheat-1)
         else
            jheat = 2 + 10*(jheat-1)
         end if

      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( matrix .and. kchois.eq.0 .or. kchois.ge.2 ) then
         isubcf = isub
      else
         isubcf = 11
      end if

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i) > 0 then paramater i corresponds to a function

         if ( ind(i).gt.0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i).gt.0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
         if ( ind(i).gt.10000 ) funchk = .true.
      end do

      if ( icheld.eq.0 .and. itype.eq.600 .and. &
           (metup.eq.7 .or. metup.eq.8) ) then
         jsol = 1
         jderiv = 1
      end if
      if ( icheli.gt.1 .and. icheli.ne.7 .or. icheld.gt.0 .and. &
           icheld.ne.21 ) ind(lengthmst) = 1
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( jelp.eq.4 .and. jcart.eq.1 .and. metup.eq.0 .and. &
           icheld.le.0 .and. irule.eq.1 ) kstep = 2

      if ( ichtrn.gt.0 ) then

!     --- ichtrn > 0, special integral

         if ( jelp.ne.1 ) then
            call errint ( -icheld, 1 )
            call errint ( jelp, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1863, 5, 0, 0 )
         else
            jtrans = jtrans + 100*ichtrn
         end if

      end if
      if ( ierror.ne.0 ) call instop

      call erclos ( 'el0600' )

      end
