      subroutine el0800inmat ( iuser, metup, mrule, linear, &
                               jelp, ishift, interpol, iqmat, iqmat1, &
                               jheat, jwcons )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.12 date 18-12-2012 Extension with 815
!        version  2.11 date 12-08-2011 extra debug
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of integration points for this element group
!     using array iuser
!     Elliptic second order equations
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), metup, mrule, jelp, ishift, &
              interpol, iqmat, iqmat1, jheat, jwcons
      logical linear

!     interpol      i/o   If 0 the standard interpolation is applied, &
!                         if 1 the interpolation is restricted to linear
!                         subelements
!     iqmat         i/o   Indication whether a transformation has to be
!                         performed if the elements are of a lower
!                         dimension than the dimension of the coordinates.
!                         If 0 no tranformation is performed (standard
!                         case) if 1 then the velocities are mapped to the
!                         dimension of the elment
!     iqmat1         o    Indication whether the right-hand side has to be
!                         transformed
!     ishift        i/o   Shift in coefficient sequence number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELel0800inmx.
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
!     jwcons         o    Indication if the weight function is constant
!                         (1) or not (2)
!     linear         o    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     metup          o    Upwind parameter
!     mrule          o    Type of integration rule as given by the user
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer massparm

!     massparm       Indicates the largest value of jheat during the
!                    time integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800CHK      Check combination of upwind and jcart
!                    Adapt irule and jwcons if necessary
!     EL0800ELP      Set parameters depending on shape of element
!     EL0800HEAT     Compute parameters with respect to the mass matrix
!     EL0800INM      Compute the parameter m for an element
!     EL0800RULE     Fill the parameters irule and jcart from array iuser
!     EL0800UPW      Compute parameter metup defining the type of upwinding
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERROR42        Print error message 42
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800inmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800inmat'
         write(irefwr,1) 'imas', imas
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set defaults

      call elcomm

!     --- Set parameters depending on the shape numbers

      call el0800elp ( linear, jelp )

!     --- Get information of first 4 coefficients
!         Fill the arrays IND, MST and ARRAY with information of the first
!         coefficient

      ipos = iuser ( ielgrp + 5 )
      ishift = 0
      if ( ipos<7 ) call error42 ( ielgrp, ipos, ielem, itype )

!     --- Standard situation, Compute the parameters jheat, metupw
!         Set jheat equal to imas
!         ishift and ipos are adapted

      call el0800heat ( iuser, ishift, iqmat, iqmat1, jheat, massparm )

!     --- Compute upwind parameters
!         ishift and ipos are adapted

      call el0800upw ( iuser, metup, jelp, ishift )
      if ( ierror/=0 ) go to 1000

!     --- Compute the parameters jcart and irule

      call el0800rule ( 0, interpol, ishift, iuser, mrule )

!     --- Check combination of upwind and jcart
!         Adapt irule and jwcons if necessary

      call el0800chk ( metup, mrule, linear, jelp, jheat, &
                       jwcons, massparm )

      if ( irule/=1 ) then

!     --- irule>1, Gauss rule

         call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                          iphi )
         mn = m * n

      else if ( metup>0 ) then

!     --- upwinding, fill array phi

         iphi = 1

      end if

      if ( metup>0 ) jpsi = 1

      if ( jheat==0 ) notmas = 1

1000  call erclos ( 'el0800inmat' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'n, m', n, m
         write(irefwr,1) 'metup, mrule, jelp', &
                          metup, mrule, jelp
         write(irefwr,1) 'ishift, interpol, iqmat, iqmat1', &
                          ishift, interpol, iqmat, iqmat1
         write(irefwr,1) 'jheat, jwcons, jsecnd', &
                          jheat, jwcons, jsecnd
         write(irefwr,1) 'irule, imas, jdiag, jheat, jpsi', &
                          irule, imas, jdiag, jheat, jpsi
         write(irefwr,*) 'End el0800inmat'

      end if  ! ( debug )

      end
