      subroutine el0850 ( iuser, user, jelp, kfirst, isub, &
                          isup, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Correction declaration
!        version  1.0  date 20-11-2009
!
!   copyright (c) 2009-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM850
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     stochastic
!     investment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(7,*)
      integer iuser(*), jelp, kfirst, isub, isup

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                              1D                  2D            3D
!                         1:   alpha 11         alpha 11       alpha 11
!                         2:                    alpha 12       alpha 12
!                         3:                    alpha 22       alpha 13
!                         4:                                   alpha 22
!                         5:                                   alpha 23
!                         6:                                   alpha 33
!                         7:   u 1              u 1            u 1
!                         8:                    u 2            u 2
!                         9:                                   u 3
!                        10:   beta             beta           beta
!                        11:   f                f              f
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    User array in which integer information of the
!                         coefficients is stored (See Standard Problems)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM800. Possible vaues:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic  brick)
!     kfirst         o    This parameter indicates the smallest velocity
!                         component that is not equal to zero in the convective
!                         terms
!                         For example kfirst = 2, means that the y- &
!                         component of the velocity is unequal to zero
!     user           i    User array in which real information of the
!                         coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, ishift, mrule
      logical linear, funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ishift         Shift in coefficient sequence number
!     j              Counting variable
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0850IN       Compute the number of integration points for
!                    stochastic investment problem
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  - &
!     2:  -                           - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  - &
!     5:  - &
!     6:  f  (rhside)
!     7-10: - &
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0850' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0850'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set a number of defaults

      iweigh = 1
      ideriv = 1
      irule  = 1
      igausp = 1
      iphi   = 0
      jcart  = 1
      icoor  = 1
      ncomp  = 1
      jdiag  = 1
      idim   = 1
      jderiv = 0
      jtrans = 0
      jind   = 1
      kstep  = 1
      ishift = 0

!     --- Set parameters depending on the shape numbers

      call el0850in ( iuser, mrule, ncoef, linear, &
                      jelp, ishift, jwcons )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'mrule, ncoef, jelp, ishift, jwcons', &
                          mrule, ncoef, jelp, ishift, jwcons

      end if  ! ( debug )

      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      if ( debug ) &
         write(irefwr,1) 'n, m, ncoef, ipos, ishift', &
                          n, m, ncoef, ipos, ishift

      nuser = 0

      isub = 1

!     --- ipos = ipos + 1

      ishift = ishift+2-isub

      if ( debug ) &
         write(irefwr,1) 'isub, ncoef, ipos', isub, ncoef, ipos

!     --- Extract real coefficients

      do i = isub, ncoef
         call elusr3 ( i, array(1,i), iuser, user, i+ishift )
      end do
      if ( debug ) then
         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
      end if  ! ( debug )

!     --- Test array lengths, number of nodes and number of degrees
!         of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )
      if ( ierror/=0 ) go to 1000

!     --- Next consider all terms carefully in order to get the cheapest
!         computation
!         First consider the diffusion tensor:

      jsecnd = 4    ! general case

!     --- Next consider the Convective terms:
!         compute idim and first component idim

      if ( jelp>2 ) idim = 2   ! all ndim terms are used
      kfirst = 1    ! start with first term

!     --- Constant part of the equation:
!         compute jzero

      jzero = jwcons  ! beta is constant
      if ( debug ) &
         write(irefwr,1) 'jsecnd, kfirst, jzero, idim', &
                          jsecnd, kfirst, jzero, idim

!     --- Vector = true, the element vector must be computed
!         Compute jconsf

      jconsf = 2  ! general case

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      do i = 1, ncoef

         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   ist(isub:isup) = 0
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( jelp==4 .and. jcart==1 .and. irule==1 ) kstep = 2

      if ( igausp==1 .and. mod(iweigh,10)==0 ) iweigh = iweigh+1

1000  call erclos ( 'el0850' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         write(irefwr,1) 'isub, isup, igausp, m, n, iweigh, iphi', &
                          isub, isup, igausp, m, n, iweigh, iphi
         write(irefwr,*) 'End el0850'

      end if  ! ( debug )

      end
