      subroutine el1012 ( elemvc, p, gradp, dphidx, phi, w, array, &
                          ibear, work, coeff )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 07-11-2006 New call to el1009
!        version  1.2  date 07-08-2005 New call to el1009
!        version  1.1  date 27-05-1998 Add u21, u22, ndim=3 (RvO)
!        version  1.0  date 24-12-1993
!
!   copyright (c) 1993-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for flow computation of Reynolds equation
!     Both compressible and incompressible bearing
!     One and two dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision array(m,*), p(m), gradp(m,ndimlc), work(m), w(m), &
                       elemvc(*), dphidx(*), coeff(m,*), phi(*)
      integer ibear

!     array          array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                2D
!                    1:   h                h
!                    2:   mu               mu
!                    3:   ht               ht
!                    4:   k                k
!                    5:   p0               p0
!                    6:   u_1 1            u_1 1
!                    7:                    u_1 2
!                    8:
!                    9:   rho              rho
!                   10:   RT               RT
!                   11:   f1               f1
!                   12:                    f2
!                   13:   u_2 1            u_2 1
!                   14:                    u_2 2
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     coeff          Work array to store the vector g for el1009
!     dphidx   i     Array of length n x m x 3  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear tetrahedron (n=4), then the
!                    derivatives are constant and only k = 1 is filled.
!     elemvc   o     Contains the flow vector for subroutine DERIV
!     gradp    i     Real m x ndim array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = gradp ( k, j )
!                           j   k
!     ibear    i     Indicates type of bearing.
!                    Possible values:
!                    0:  incompressible bearing
!                    1:  compressible bearing (adiabatic)
!     p        i     Contains the pressure p
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points if a Gauss rule is
!                    used.
!                    phi  (xg ) = phi (i,k)
!                       i    k
!     w        i     array of length m containing the weights for the
!                    numerical integration
!     work           Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer l, k, jaddsv

!     jaddsv  Help variable to store the value of jadd temporarily
!     k       Counting variable
!     l       Summation variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL1009  Compute integrals of the shape / u(x).grad phi dx
!                                            e -            i
!     EL3003  Compute integrals of the shape / f(x) phi (x) dx
!                                            e         i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!    The flow is given by:
!    Incompressible bearing:
!    / (h^3/(12 mu) (grad p - rho f) + h/2 u ) grad phi  dx  + &
!                                 -        -           i
!    / (ht + k(p-p0) ) phi  dx
!                         i
!    Compressible bearing (adiabatic)
!    / (p h^3/(12 mu) (grad p - p/RT f) + ph/2 u) grad phi  dx + &
!                                    -         -          i
!                2   2
!    / (pht + k(p -p0 ) ) phi  dx
!                            i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Store H^3/12mu in work

      do k = 1, m
         work(k) = array(k,1)**3/(12d0*array(k,2))
      end do

      jaddsv = jadd
      if ( ibear==0 ) then

!     --- incompressible bearing
!         part / (h^3/(12 mu) (grad p - rho f) - h/2 u ) grad phi_i dx

         do l = 1, min(2,ndimlc)
            do k = 1, m
               coeff(k,l) = work(k)*gradp(k,l) - 0.5d0*array(k,1)* &
                            (array(k,5+l)+array(k,12+l))
            end do
            if ( ind(9)/=0 .and. ind(10+l)/=0 ) then
               do k = 1, m
                  coeff(k,l) = coeff(k,l) - &
                               work(k)*array(k,9)*array(k,10+l)
               end do
            end if
         end do
         jadd = 0
         call el1009 ( coeff, elemvc, w, dphidx, min(2,ndimlc), 1d0 )
         jadd = 1
         if ( ind(3)/=0 .or. ind(4)/=0 ) then

!        --- part / (ht + k(p-p0) ) phi_i dx

            if ( ind(3)/=0 ) then
               do k = 1, m
                  coeff(k,1) = array(k,3)
               end do
            else
               do k = 1, m
                  coeff(k,1) = 0d0
               end do
            end if
            if ( ind(4)/=0 ) then
               do k = 1, m
                  coeff(k,1) = coeff(k,1) + array(k,4)*(p(k)-array(k,5))
               end do
            end if
            call el3003 ( coeff, elemvc, w, phi, m ,n )

         end if

      else

!     --- compressible bearing
!         part / (p h^3/(12 mu) (grad p - p/RT f) - ph/2 u) grad phi_i dx

         do k = 1, m
            work(k) = p(k)*work(k)
         end do
         do l = 1, min(2,ndimlc)
            do k = 1, m
               coeff(k,l) = work(k)*gradp(k,l) - 0.5d0*array(k,1)*p(k)* &
                            (array(k,5+l)+array(k,12+l))
            end do
            if ( ind(9)/=0 .and. ind(10+l)/=0 ) then
               do k = 1, m
                  coeff(k,l) = coeff(k,l) - &
                               work(k)*p(k)/array(10,k)*array(10+l,k)
               end do
            end if
         end do
         jadd = 0
         call el1009 ( coeff, elemvc, w, dphidx, min(2,ndimlc), 1d0 )
         jadd = 1
         if ( ind(3)/=0 .or. ind(4)/=0 ) then

!        --- part / (pht + k(p^2-p0^2) ) phi_i dx

            if ( ind(3)/=0 ) then
               do k = 1, m
                  coeff(k,1) = array(k,3)
               end do
            else
               do k = 1, m
                  coeff(k,1) = 0d0
               end do
            end if
            if ( ind(4)/=0 ) then
               do k = 1, m
                  coeff(k,1) = coeff(k,1) + array(k,4)* &
                               (p(k)**2-array(k,5)**2)
               end do
            end if
            call el3003 ( coeff, elemvc, w, phi, m ,n )

         end if

      end if
      jadd = jaddsv
      end
