      subroutine el1017800 ( icheld, elemvc, n, ndim, alpha, p, gradp, &
                             gamma, height, rho, frhsd, f0, u1, u2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-10-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for flow vector and shear stress of Reynolds equation
!     incompressible bearing
!     Special case where element 800 is used
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, ndim, n
      double precision elemvc(ndim,*), alpha(n,*), p(*), gradp(n,*), &
                       gamma(n,*), height(*), rho(*), frhsd(n,*), f0(*), &
                       u1(n,*), u2(n,*)

!     alpha          i    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     elemvc         o    Element vector
!     f0             i    Factor F0 in generalized Reynolds equation
!     frhsd          i    Extra right-hand side vector
!     gamma          i    Vector in divergence operator following gradient
!     gradp          i    Gradient of the pressure
!     height         i    Heightof the film
!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of this subroutine
!     n              i    number of basis functions for each component
!     ndim           i    Dimension of the space.
!     p              i    Array containing the pressure
!     rho            i    density rho
!     u1             i    Contains the velocity vector at the bottom face
!     u2             i    Contains the velocity vector at the top face
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, j, i
      double precision mult
      logical cavstart, cavitat

!     cavitat        If true we have cavitation in the element
!     cavstart       Starting value for cavitat
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     mult           multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The flow vector is given by:
!    q = (A grap p + gamma) . grad phi_i
!    The shear stress at the bottom:
!    tau = -h/2(grad p -rho f) + (u2-u1) /F0
!    The shear stress at the top:
!    tau =  h/2(grad p -rho f) + (u2-u1) /F0
!    F0 = / from 0 to h dz/eta, which for constant eta reduces to h/eta
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1017800' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1017800'
         write(irefwr,1) 'icheld, n, ndim', icheld, n, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( icheld==23 ) then

!     --- Compute flow vector, depending on dimension

         select case ( ndim )

            case(1)

!           --- ndim = 1

               do k = 1, n
                  elemvc(1,k) = -alpha(k,1)*gradp(k,1)-gamma(k,1)
               end do  ! k = 1, n

            case(2)

!           --- ndim = 2

               do k = 1, n
                  elemvc(1,k) = -(alpha(k,1)*gradp(k,1)+ &
                                  alpha(k,2)*gradp(k,2))-gamma(k,1)
                  elemvc(2,k) = -(alpha(k,2)*gradp(k,1)+ &
                                  alpha(k,3)*gradp(k,2))-gamma(k,2)
               end do  ! k = 1, n

            case(3)

!           --- ndim = 3

               do k = 1, n
                  elemvc(1,k) = -(alpha(k,1)*gradp(k,1)+ &
                                  alpha(k,2)*gradp(k,2)+ &
                                  alpha(k,4)*gradp(k,3))-gamma(k,1)
                  elemvc(2,k) = -(alpha(k,2)*gradp(k,1)+ &
                                  alpha(k,3)*gradp(k,2)+ &
                                  alpha(k,5)*gradp(k,3))-gamma(k,2)
                  elemvc(3,k) = -(alpha(k,4)*gradp(k,1)+ &
                                  alpha(k,5)*gradp(k,2)+ &
                                  alpha(k,6)*gradp(k,3))-gamma(k,3)
               end do  ! k = 1, n

         end select  ! case ( ndim )

      else if ( icheld==24 .or. icheld==25 ) then

!     --- Compute lower or upper shear stress
!         if pressure==0d0 for all elements asume no cavitation
!         else if pressure==0d0 AND dpdx==0d0 AND dpdy==0d0 then
!           assume cavitation in point and thus no shear stress
!         SHOULD BE REPLACED BY A CORRECT MASS-CONSERVATION ALGORITHM

         if ( icheld==24) then
            mult = -0.5d0
         else
            mult = 0.5d0
         end if  ! ( icheld==24)
         cavstart = .false.
         do k = 1, n
            if ( abs(p(k))>1.0d-8 ) then
               cavstart = .true.
            end if
         end do

         if ( debug ) then
            call prinrl ( height, n, 'height' )
            call prinrl ( rho, n, 'rho' )
            call prinrl ( f0, n, 'f0' )
            call prinrl1 ( frhsd, ndim, n, 'frhsd' )
            call prinrl1 ( u1, 2, n, 'u1' )
            call prinrl1 ( u2, 2, n, 'u2' )
         end if  ! ( debug )

         do j = 1, ndim
            do k = 1, n
               elemvc(j,k) = mult*height(k)* &
                             (gradp(k,j)-rho(k)*frhsd(k,j))+ &
                             (u2(k,j)-u1(k,j))/f0(k)
               if ( abs(p(k))<=0d0 ) then
                  cavitat = cavstart
               else
                  cavitat = .false.
               end if
               do i = 1, ndim
                  if ( abs(gradp(k,i))>0d0 ) cavitat = .false.
               end do
               if ( cavitat ) elemvc(j,k) = 0d0
            end do
         end do

      end if

1000  call erclos ( 'el1017800' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemvc, n, ndim, 'elemvc' )
         write(irefwr,*) 'End el1017800'

      end if  ! ( debug )

      end
