      subroutine el2000 ( trans, sp, mdiv, rhsdivtr, rhsdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 27-02-2010 Debug statements
!        version  2.0  date 18-10-1998 Extra parameters mdiv and rhsdivtr
!        version  1.1  date 20-08-1994 Extension for 3D
!        version  1.0  date 11-02-1994
!
!   copyright (c) 1994-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the transformation matrix to be used in the elimination
!     process of the centroid point for incompressible Navier-Stokes
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision sp(ndimlc*n,ndimlc+1), rhsdiv(*), rhsdivtr(*), &
                       trans(ndimlc*(n-1),ndimlc)
      integer mdiv

!     mdiv     i    Defines if divergence right-hand side is used (1) or not (0)
!     rhsdiv   i    Divergence right-hand side (if mdiv=1)
!     rhsdivtr o    Contains values of transformed divergence right-hand side
!                   (if mdiv=1)
!     sp       i    Divergence matrix
!                   SP(i,k) corresponds to the k-th row and i-th column
!                   of the matrix, The k refers to the basis functions of the
!                   pressure, i to the basis functions of the velocity in the
!                   sequence of the solution vector
!     trans    o    Contains the transformation matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, n12
      double precision rinmat(3,3), det, sum

!     det            Determinant of L22
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     n12            Number of nt to be eliminated velocity unknowns
!     rinmat         Inverse of L22, see method
!     sum            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix can be written as:
!
!            |                T     T  |  |   |     |    |
!            |   S     S    -L    -L   |  | u |     | f  |
!            |    11    12    11    21 |  |  1|     |  1 |
!            |                         |  |   |     |    |
!            |                T     T  |  |   |     |    |
!            |   S     S    -L    -L   |  | u |     | f  |
!            |    21    22    12    22 |  |  2|  =  |  2 |
!            |                         |  |   |     |    |
!            |   D     D     0     0   |  | p |     | fd |
!            |    11    12             |  |  1|     |   1|
!            |                         |  |   |     |    |
!            |   D     D     0     0   |  | p |     | fd |
!            |    21    22             |  |  2|     |   2|
!
!     Here the u1 refers to the velocities outside the centroid, &
!     u2 to the velocities in the centroid p1 to the pressure in the
!     centroid and p2 to the gradient of the pressure.
!     All quantities with index 2 must be eliminated.
!
!     It follows immediately that:
!
!            -1           -1
!     u  = -D   D   u  + D   fd  = R  u  - fd_tr
!      2     22  21  1    22   2    0  1
!
!     In this subroutine the matrix R0 is computed as well as the vector fd_tr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- MODULES USED

      call eropen ( 'el2000' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el2000'
         write(irefwr,1) 'ndimlc, n', ndimlc, n
         call prinrl1 ( sp, ndimlc+1, ndimlc*n, 'sp' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 ) then

!     --- ndim = 2

         n12 = ndimlc*(n-1)
         det = sp(n12+1,2)*sp(n12+2,3)-sp(n12+2,2)*sp(n12+1,3)
         if ( debug ) write(irefwr,2) 'det', det
         rinmat(1,1) = sp(n12+2,3)/det
         rinmat(2,2) = sp(n12+1,2)/det
         rinmat(1,2) = -sp(n12+1,3)/det
         rinmat(2,1) = -sp(n12+2,2)/det
         do i = 1, n12
            do j = 1, ndimlc
               sum = 0d0
               do k = 1, ndimlc
                  sum = sum - rinmat(k,j)*sp(i,k+1)
               end do
               trans(i,j) = sum
            end do
         end do

         if ( mdiv==1 ) then

!        --- mdiv = 1, compute right-hand side

            rhsdivtr(1) = -rinmat(1,1)*rhsdiv(2)-rinmat(1,2)*rhsdiv(3)
            rhsdivtr(2) = -rinmat(2,1)*rhsdiv(2)-rinmat(2,2)*rhsdiv(3)

         end if

      else

!     --- ndim = 3

         n12 = ndimlc*13
         det = sp(n12+1,2)*sp(n12+2,3)-sp(n12+2,2)*sp(n12+1,3)
         rinmat(1,1) = sp(n12+2,3)*sp(n12+3,4)-sp(n12+2,4)*sp(n12+3,3)
         rinmat(2,1) = sp(n12+2,4)*sp(n12+3,2)-sp(n12+2,2)*sp(n12+3,4)
         rinmat(3,1) = sp(n12+2,2)*sp(n12+3,3)-sp(n12+2,3)*sp(n12+3,2)
         rinmat(1,2) = sp(n12+3,3)*sp(n12+1,4)-sp(n12+3,4)*sp(n12+1,3)
         rinmat(2,2) = sp(n12+3,4)*sp(n12+1,2)-sp(n12+3,2)*sp(n12+1,4)
         rinmat(3,2) = sp(n12+3,2)*sp(n12+1,3)-sp(n12+3,3)*sp(n12+1,2)
         rinmat(1,3) = sp(n12+1,3)*sp(n12+2,4)-sp(n12+1,4)*sp(n12+2,3)
         rinmat(2,3) = sp(n12+1,4)*sp(n12+2,2)-sp(n12+1,2)*sp(n12+2,4)
         rinmat(3,3) = sp(n12+1,2)*sp(n12+2,3)-sp(n12+1,3)*sp(n12+2,2)
         det = sp(n12+1,2)*rinmat(1,1)+ &
               sp(n12+1,3)*rinmat(2,1)+ &
               sp(n12+1,4)*rinmat(3,1)
         det = 1d0/det
         do i = 1, 3
            do j = 1, 3
               rinmat(i,j) = det*rinmat(i,j)
            end do
         end do
         do i = 1, n12
            do j = 1, ndimlc
               sum = 0d0
               do k = 1, ndimlc
                  sum = sum - rinmat(k,j)*sp(i,k+1)
               end do
               trans(i,j) = sum
            end do
         end do
         do i = 43, 81
            do j = 1, ndimlc
               sum = 0d0
               do k = 1, ndimlc
                  sum = sum - rinmat(k,j)*sp(i,k+1)
               end do
               trans(i,j) = sum
            end do
         end do

         if ( mdiv==1 ) then

!        --- mdiv = 1, compute right-hand side

            rhsdivtr(1) = -rinmat(1,1)*rhsdiv(2)- &
                           rinmat(1,2)*rhsdiv(3)- &
                           rinmat(1,3)*rhsdiv(4)
            rhsdivtr(2) = -rinmat(2,1)*rhsdiv(2)- &
                           rinmat(2,2)*rhsdiv(3)- &
                           rinmat(2,3)*rhsdiv(4)
            rhsdivtr(3) = -rinmat(3,1)*rhsdiv(2)- &
                           rinmat(3,2)*rhsdiv(3)- &
                           rinmat(3,3)*rhsdiv(4)

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el2000'

      end if  ! ( debug )
      call erclos ( 'el2000' )

      end
