      subroutine el2005 ( u, dudx, phi, dphidx, indlc, ugauss, number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.13 date 13-11-2014 Use eluderiv
!        version  1.12 date 12-12-2012 Long integers
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute a local vector and its derivatives from a known vector.
!    the values are evaluated in the integration points or in the nodal points
!    If a transformation is required the value in the extra point(s) is
!    computed by interpolation and the derivatives are not yet computed
!    For that purpose subroutine EL3000 should be used
!     EL2005 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
!     extract_unknown
!     fill
!     fill_solution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indlc(ncomp,n), number
      double precision  u(n,ncomp), phi(n,m), &
                        dudx(m,ndim,ncomp), dphidx(n,m,ndim), &
                        ugauss(m,ncomp)

!     dphidx         i    Array of length n x m x 3  containing the derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           o    Real m x ndim x ncomp array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j, i )
!                           i    j   k
!                         dudx is only filled if jderiv>0
!     indlc          i    Index array that is used if jind<=0, to store the
!                         way the vector u must be extracted from usol. See jind
!     number         i    The value of number indicates which of the solution
!                         vectors must be used (1<=number<=numold)
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                                k
!                         phi (xg ) = phi (i,k)
!                            i
!                         array phi is only filled when gauss integration is
!                         applied
!     u              o    Real n x ncomp array in which the values of the
!                         solution vector in the nodal points of the element are
!                         stored in the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)n, i = 1 (1) ncomp
!                          i   j
!                         Array u is always filled
!     ugauss         o    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
!                         ugauss is only filled if jdiag>1 in common celint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, lstep, lsub, lsup, kderiv, ktrans, iposi
      double precision, pointer :: vecoldlc(:)
      save lstep, lsub, lsup

!     i              Counting variable
!     iposi          Help parameter to compute a position in index3
!     j              Counting variable
!     kderiv
!     ktrans         Help variable, ktrans=1 when jtrans =1, 2 and 0 when
!                    jtrans=0
!     lstep          Step variable for vertices. Usually lstep = 1, &
!                    but when the solution must be computed in the vertices
!                    and the element is quadratic then lstep = 2
!     lsub           Under index for the direction of the derivative to be
!                    computed
!     lsup           Upper index for the direction of the derivative to be
!                    computed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ELUDERIV       Compute the derivative of a given function in one element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el2005' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el2005'
         call ertrace
         write(irefwr,1) 'jind, jderiv, jtrans', jind, jderiv, jtrans
         write(irefwr,1) 'n, m, ndimlc, ncomp', n, m, ndimlc, ncomp
         write(irefwr,1) 'number, numold', number, numold
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lsub=1

!     --- first set some constants

      kderiv = abs(jderiv)
      ktrans = min(1,jtrans)
      if ( kderiv==1 .or. kderiv==5 ) then

!     --- kderiv = 1 or 5:  The gradient must be computed
!                           Components 1 to ndim

         lsub=1
         lsup=ndimlc

      else if ( kderiv>1 ) then

!     --- kderiv 2,3,4, 6, 7 or 8:  Component kderiv-1 must be computed

         lsub=kderiv-1
         if ( kderiv>5 ) lsub = lsub-4
         lsup=lsub

      end if
      if ( kderiv>1 .and. kderiv<6 .and. &
           ndimlc==2 .and. n>=6 ) then

!     --- kderiv>1, hence computation in vertices
!                     Furthermore quadratic 2D element
!                     Skip points between

         lstep=2

      else

         lstep=1

      end if

      if ( kderiv>1 .and. ndim==3 .and. &
           (n/=4 .and. n/=8 .and. n/=6) ) lstep=2

      iseqtype = oldsols(number)%iseqtype

      index3n => indoldsols(iseqtype)%index3
      index4n => indoldsols(iseqtype)%index4
      vecoldlc => oldsols(number)%oldvecs


!     --- Fill array u with solution in nodes

      if ( jind==0 ) then

!     --- jind = 0, use array indlc to find unknowns

         if ( debug ) call prinin1 ( indlc, ncomp, n, 'ind' )
         do j = 1, ncomp
            do i = 1, n-ktrans
               u(i,j) = vecoldlc(index3n(indlc(j,i)))
            end do
         end do
         if ( debug ) call prinrl1 ( u, ncomp, n, 'u' )

      else if ( jind==-1 ) then

!     --- jind = -1, use array indlc partly to find unknowns

         if ( ncomp==1 ) then

!        --- ncomp = 1

            if ( numold==1 ) then

!           --- Numold = 1, index2 is used

               do i = 1, n-ktrans
                  u(i,1) = vecoldlc(index3n(indlc(1,1)+i-1))
               end do

            else

!           --- Numold>1, index3 is used

               iposi = indlc(1,1)
               do i = 1, n-ktrans
                  u(i,1) = vecoldlc(index3n(iposi))
                  iposi = indlc(1,1)+index4n(i)
               end do

            end if

         else

!        --- ncomp>1

            if ( numold==1 ) then

!           --- Numold = 1, index2 is used

               do j = 1, ncomp
                  do i = 1, n-ktrans
                     u(i,j) = &
                        vecoldlc(index3n(indlc(j,1)-ncomp+i*ncomp))
                  end do
               end do

            else

!           --- Numold>1, index3 is used

               do j = 1, ncomp
                  iposi = indlc(j,1)
                  do i = 1, n-ktrans
                     u(i,j) = vecoldlc(index3n(iposi))
                  end do
                  iposi = indlc(j,1)+index4n(i)
               end do

            end if

         end if

      else if ( jind==-2 ) then

!     --- jind = -2, Special case for 27-point Navier-Stokes element with
!                    4 pressure unknowns in centroid (=14)

         if ( numold==1 ) then

!        --- Numold = 1, index2 is used

            do j = 1, ncomp
               do i = 1, 14
                  u(i,j) = vecoldlc(index3n(j+(i-1)*ncomp))
               end do
               do i = 15, n
                  u(i,j) = vecoldlc(index3n(j+(i-1)*ncomp+4))
               end do
            end do

         else

!        --- Numold>1, index3 is used

            do j = 1, ncomp
               iposi = 0
               do i = 1, 14
                  u(i,j) = vecoldlc(index3n(j+iposi))
                  iposi = index4n(i)
               end do
               do i = 15, n
                  u(i,j) = vecoldlc(index3n(j+iposi))
                  iposi = index4n(i)
               end do
            end do

         end if

      else

!     --- jind = >0, use jind to find unknowns

         if ( ncomp==1 ) then

!        --- ncomp = 1

            if ( numold==1 ) then

!           --- Numold = 1, index2 is used

               do i = 1, n-ktrans
                  u(i,1) = vecoldlc(index3n(i))
               end do

            else

!           --- Numold>1, index3 is used

               do i = 1, n-ktrans
                  u(i,1) = vecoldlc(index3n(index4n(i)))
               end do
               if ( debug ) call prinrl ( u, n, 'u/2' )
            end if

         else

!        --- ncomp>1

            if ( numold==1 ) then

!           --- Numold = 1, index2 is used

               do j = 1, ncomp
                  do i = 1, n-ktrans

                     u(i,j) = &
                        vecoldlc(index3n(jind-1+j-ncomp+i*ncomp))
                  end do
               end do

            else

!           --- Numold>1, index3 is used

               do j = 1, ncomp
                  iposi = jind-1
                  do i = 1, n-ktrans
                     u(i,j) = vecoldlc(index3n(iposi+j))
                     iposi = index4n(i)+jind-1
                  end do
               end do
               if ( debug ) call prinrl1 ( u, ncomp, n, 'u/2' )

            end if

         end if

      end if

      if ( jtrans/=0 ) then

!     --- jtrans = 1  Compute extra unknowns in centroid by interpolation

         if ( n==7 ) then
            do j = 1, ncomp
               u(n,j) = (-u(1,j)-u(3,j)-u(5,j) + &
                          4d0 * (u(2,j)+u(4,j)+u(6,j))) / 9d0
            end do
         else if ( n==4 ) then
            do j = 1, ncomp
               u(n,j) = (u(1,j)+u(2,j)+u(3,j))/3d0
            end do
         else if ( n==5 ) then
            do j = 1, ncomp
               u(n,j) = (u(1,j)+u(2,j)+u(3,j)+u(4,j))*0.25d0
            end do
         end if

      end if

      if ( jsol==1 .and. jdiag/=1 ) then

!     --- Compute solution in Gauss points
!                   n
!         ug (k) = sum   u (i) phi (k)
!           l      i=1    l       i

         do j = 1, ncomp
            do i = 1, m
               ugauss(i,j) = ddot ( n, u(1,j), 1, phi(1,i), 1 )
            end do
         end do

      else if ( jsol/=0 ) then

!     --- jsol = 1,  Copy u into ugauss

         do j = 1, ncomp
            do i = 1, n
               ugauss(i,j) = u(i,j)
            end do
         end do

      end if

      if ( jderiv>0 .and. (jtrans/=1 .or. ishape==3 .or. &
           ishape==5 .or. ishape==7 .or. ishape==11 .or. &
           ishape==12 .or. ishape==21) ) then

!     --- jderiv>0,  compute derivatives

         call eluderiv ( u, dudx, dphidx, lsub, lsup, lstep )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( ugauss, ncomp, m, 'ugauss' )
         write(irefwr,*) 'End el2005'

      end if  ! ( debug )
      call erclos ( 'el2005' )

      end
