      subroutine el2203 ( elemmt, s, sp, div, pp, eps, mcont, &
                          w, x, xgauss, signdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 17-09-2008 Extension with pressure mass matrix
!        version  4.0  date 23-02-2004 Extra parameters w, x, xgauss
!        version  3.0  date 17-08-2001 Extra parameter mcont
!                                      Incorporate pressure term in
!                                      continuity equation (Gerard Haagh)
!        version  2.0  date 06-08-1998 Extra parameter eps
!        version  1.0  date 28-01-1995 Renamed subroutine el2203
!
!   copyright (c) 1995-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Put stiffness matrix and part due to gradient and divergence
!    into element matrix for 27-point Navier-Stokes equation
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemmt(icount,icount), s(3*n,3*n), eps, &
                       sp(ndimlc*n,ndimlc+1), div(ndimlc*n,ndimlc+1), &
                       w(*), xgauss(m,*), pp(ndimlc+1,ndimlc+1), x(n,*)
      integer mcont, signdiv

!     div            i    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     elemmt         o    Element matrix to be filled
!                         Only the part corresponding to div and grad is filled
!     eps            i    Penalty parameter.
!                         The pressure mass matrix multiplies by eps is added
!                         to the matrix
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                         2: Goertler equations: No dp/dx, but complete div u
!                         4: the continuity equation contains a pressure term
!     pp             i    Pressure matrix
!     s              i    Element matrix without the gradient and divergence
!                         part
!     signdiv        i    Multiplication sign of "continuity equations"
!                         if -1 the stokes matrix is symmetric
!                         if 1 all eigenvalues are in the positive half plane
!     sp             i    Gradient matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    contains the coordinates of the nodes
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, icent, npres, isigndiv
      double precision diag(4)

!     diag           Diagonal of the pressure mass matrix
!     i              Counting variable
!     icent          Index of centroid
!     isigndiv       signdiv except for mcont = 5, then 1
!     j              Counting variable
!     k              Counting variable
!     npres          Number of pressure parameters per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The stiffness matrix has the following structure
!     Saa Sab Sac       where a refers to the nodes 1 to 13
!     Sba Sbb Sbc             b refers to node 14
!     Sca Scb Scc             c refers to the nodes 15 to 27
!    The element matrix gets the form:
!     Saa Sab Ga Sac     with G the gradient matrix
!     Sba Sbb Gb Sbc     D the divergence matrix
!     Da  Db  P  Dc      and P the pressure matrix
!     Sca Scb Gc Scc
!   Remember matrices are stored in transposed form
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First put Stiffness matrix in correct positions

      do i = 1, 14*3
         do j = 1, 14*3
            elemmt(i,j) = s(i,j)
         end do
      end do

      do i = 14*3+1, 27*3
         do j = 14*3+1, 27*3
            elemmt(i+4,j+4) = s(i,j)
         end do
      end do

      do i = 1, 14*3
         do j = 14*3+1, 27*3
            elemmt(i,j+4) = s(i,j)
         end do
      end do

      do i = 14*3+1, 27*3
         do j = 1, 14*3
            elemmt(i+4,j) = s(i,j)
         end do
      end do

!     --- Next put gradient and divergence matrix in the correct positions

      do j = 1, 4
         do i = 1, 14*3
            elemmt(i,14*3+j) = signdiv*div(i,j)
         end do
         do i = 14*3+1, 27*3
            elemmt(i+4,14*3+j) = signdiv*div(i,j)
         end do
      end do

!     --- gradient matrix

      do j = 1, 4
         do i = 1, 14*3
            elemmt(14*3+j,i) = -sp(i,j)
         end do
         do i = 14*3+1, 27*3
            elemmt(14*3+j,i+4) = -sp(i,j)
         end do
      end do
      do j = 1, 4
         elemmt(14*3+j,14*3+j) = 0d0
      end do

!     --- Next put pressure matrix in the correct positions

      if ( mcont==4 .or. mcont==5 ) then

!     --- mcont = 4 or 5, add pressure mass matrix

         isigndiv = signdiv
         if ( mcont==5 ) isigndiv = 1

         do j = 1, 4
            do i = 1, 4
               elemmt(14*3+i,14*3+j) = isigndiv*pp(i,j)
            end do
         end do

      end if

!     --- Add penalty parameter to diagonal of pressure part

      if ( eps>0d0 ) then

!     --- eps>0, there is a penalty contribution

         icent = 14
         npres = 4

         do i = 1, npres
            diag(i) = 0d0
         end do
         do k = 1, m
            diag(1) = diag(1) + w(k)
         end do

         do i = 1, npres-1
            do k = 1, m
               diag(i+1) = diag(i+1) + w(k)*(xgauss(k,i)-x(icent,i))**2
            end do
         end do

!        --- store the diagonal multiplied by eps in diag

         do i = 1, npres
            diag(i) = diag(i) * eps
         end do

         do j = 1, npres
            elemmt(14*3+j,14*3+j) = &
               elemmt(14*3+j,14*3+j) + signdiv*diag(j)
         end do

      end if

      end
