      subroutine el3001 ( u, s, w, phi, dphidx, ncompd, nsrow, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 11-02-2009 Debug statements
!        version  2.1  date 04-07-2003 Remove common celwrk
!        version  2.0  date 16-12-1994 Removal of mdecl and ndecl;
!                                      Possibility of compressible velocity
!                                      field)
!
!   copyright (c) 1988-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element stiffness matrix corresponding to the
!     first derivative part of a one-, two- or three-dimensional equation
!     i.e.
!        s  = s   + / (u . grad phi )  phi dx
!         ij   ij   e              j      i
!                               k                k         k
!           = s   + sum  w   u(x ) . grad phi (x )  phi (x )
!              ij    k    k                  j          i
!     or in case of compressible velocity field (metupw>=1000):
!        s  = s   - / (u . grad phi )  phi dx
!         ij   ij   e              i      j
!                               k                k         k
!           = s   - sum  w   u(x ) . grad phi (x )  phi (x )
!              ij    k    k                  i         j
!     EL3001 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
!     convection
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncompd, nsrow
      double precision u(m,ncompd), s(nsrow,nsrow), w(m), phi(n,m), &
                       dphidx(n,m,*), work(m,ncompd)

!     dphidx   i     Array of length n x m x ncomp  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear symplex, then the
!                    derivatives are constant and only k = 1 is filled.
!     ncompd   i     Number of components of the vector u  (ncomp)
!     nsrow    i     Size of the element matrix S
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points if a Gauss rule is
!                    used.
!                    phi  (xg ) = phi (i,k)
!                       i    k
!     s       i/o    Element stiffness matrix to be filled
!                    At input S may already have been filled partly depending
!                    on the value of jadd in common celint
!                    Since SEPRAN assumes an one-dimensional storage, S
!                    must be stored transposed.
!                    S has size nsrow x nsrow, but only the part (1:n,1:n)
!                    is filled.
!     u        i     array of length m x ncomp in which the values of
!                    u  are stored in the sequence:
!                     i
!                                    k
!                    u ( k,i ) = u (x )   i = 1(1)3, k=1(1)m
!                                 i
!     w        i     array of length m containing the weights for the
!                    numerical integration
!     work           Work array of length m x ncomp, to store products of
!                    u  x w temporarily
!                     i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h1, h2, h3
      integer i, j, k, ni

!    h1     Help variable to compute a sum or to store a quantity
!    h2     Help variable to compute a sum or to store a quantity
!    h3     Help variable to compute a sum or to store a quantity
!    i      Counting variable
!    j      Counting variable
!    k      Counting variable
!    ni     Lower bound for i variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3001' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3001'
         write(irefwr,1) 'jdiag, kstep, istarw', jdiag, kstep, istarw
         write(irefwr,1) 'jadd, metupw, jdercn', jadd, metupw, jdercn
         write(irefwr,1) 'jtrans', jtrans
         call prinrl1 ( u, idim, m, 'u' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set ni

      if ( jdiag==1 ) then
         ni = max ( kstep,istarw )
      else
         ni = max ( 1,istarw )
      end if

!     --- Check if matrix must be added to an existing matrix or not
!         If not set matrix S equal to zero for the  n x n part

      if ( jadd==0 ) then

!     --- jadd = 0, set matrix equal to zero

         do i = 1, n
            do j = 1, n
               s(i,j) = 0d0
            end do
         end do
      end if

!     --- Add matrix to S

      if ( metupw<1000 ) then

!     --- Standard convection

         if ( jdercn==1 ) then

!        --- jdercn = 1   derivatives of basis functions are variable

            if ( jdiag==1 ) then

!           --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, m, kstep
                     h1 = w(i) * u(i,1)
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,i,1)
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, m, kstep
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,i,1) &
                                        + h2 * dphidx(j,i,2)
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, m, kstep
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     h3 = w(i) * u(i,3)
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,i,1) &
                                        + h2 * dphidx(j,i,2) &
                                        + h3 * dphidx(j,i,3)
                     end do
                  end do
               end if
            else

!           --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij
!               Store w * u into work

               do j = 1, idim
                  do k = kstep, m, kstep
                     work(k,j) = w(k) * u(k,j)
                  end do
               end do
               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + work(k,1) * dphidx(j,k,1) * &
                                     phi(i,k)
                        end do
                        s(j,i) = s(j,i) + h1
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + ( work(k,1) * dphidx(j,k,1) + &
                                       work(k,2) * dphidx(j,k,2) ) &
                                                 * phi(i,k)
                        end do
                        s(j,i) = s(j,i) + h1
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + ( work(k,1) * dphidx(j,k,1) + &
                                       work(k,2) * dphidx(j,k,2) + &
                                       work(k,3) * dphidx(j,k,3) ) &
                                                 * phi(i,k)
                        end do
                        s(j,i) = s(j,i) + h1
                     end do
                  end do
               end if
            end if
         else

!        --- jdercn = 0   derivatives of basis functions are constant

            if ( jdiag==1 ) then

!           --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, m
                     h1 = w(i) * u(i,1)
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, m
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,1,1) &
                                        + h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, m
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     h3 = w(i) * u(i,3)
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,1,1) &
                                        + h2 * dphidx(j,1,2) &
                                        + h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            else

!           --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij
!               --- Store w * u into work

               do j = 1, idim
                  do k = 1, m
                     work(k,j) = w(k) * u(k,j)
                  end do
               end do
               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, n
                     h1 = 0d0
                     do k = 1, m
                        h1 = h1 + work(k,1) * phi(i,k)
                     end do
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, n
                     h1 = 0d0
                     h2 = 0d0
                     do k = 1, m
                        h1 = h1 +  work(k,1) * phi(i,k)
                        h2 = h2 +  work(k,2) * phi(i,k)
                     end do
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,1,1) &
                                        + h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, n
                     h1 = 0d0
                     h2 = 0d0
                     h3 = 0d0
                     do k = 1, m
                        h1 = h1 + work(k,1) * phi(i,k)
                        h2 = h2 + work(k,2) * phi(i,k)
                        h3 = h3 + work(k,3) * phi(i,k)
                     end do
                     do j = 1, n
                        s(j,i) = s(j,i) + h1 * dphidx(j,1,1) &
                                        + h2 * dphidx(j,1,2) &
                                        + h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            end if
         end if
      else

!     --- Compressible convection

         if ( jdercn==1 ) then

!        --- jdercn = 1   derivatives of basis functions are variable

            if ( jdiag==1 ) then

!           --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, m, kstep
                     h1 = w(i) * u(i,1)
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,i,1)
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, m, kstep
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,i,1) &
                                        - h2 * dphidx(j,i,2)
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, m, kstep
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     h3 = w(i) * u(i,3)
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,i,1) &
                                        - h2 * dphidx(j,i,2) &
                                        - h3 * dphidx(j,i,3)
                     end do
                  end do
               end if
            else

!           --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij
!               Store w * u into work

               do j = 1, idim
                  do k = kstep, m, kstep
                     work(k,j) = w(k) * u(k,j)
                  end do
               end do
               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + work(k,1) * dphidx(j,k,1) * &
                                     phi(i,k)
                        end do
                        s(i,j) = s(i,j) - h1
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + ( work(k,1) * dphidx(j,k,1) + &
                                       work(k,2) * dphidx(j,k,2) ) &
                                                 * phi(i,k)
                        end do
                        s(i,j) = s(i,j) - h1
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + ( work(k,1) * dphidx(j,k,1) + &
                                       work(k,2) * dphidx(j,k,2) + &
                                       work(k,3) * dphidx(j,k,3) ) &
                                                 * phi(i,k)
                        end do
                        s(i,j) = s(i,j) - h1
                     end do
                  end do
               end if
            end if
         else

!        --- jdercn = 0   derivatives of basis functions are constant

            if ( jdiag==1 ) then

!           --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, m
                     h1 = w(i) * u(i,1)
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, m
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, m
                     h1 = w(i) * u(i,1)
                     h2 = w(i) * u(i,2)
                     h3 = w(i) * u(i,3)
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2) &
                                        - h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            else

!           --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij
!               --- Store w * u into work

               do j = 1, idim
                  do k = 1, m
                     work(k,j) = w(k) * u(k,j)
                  end do
               end do
               if ( idim==1 .or. jtrans==31 ) then

!              --- idim = 1, only one component of the velocity vector

                  do i = ni, n
                     h1 = 0d0
                     do k = 1, m
                        h1 = h1 + work(k,1) * phi(i,k)
                     end do
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( idim==2 .or. jtrans==32 ) then

!              --- idim = 2, two components of the velocity vector

                  do i = ni, n
                     h1 = 0d0
                     h2 = 0d0
                     do k = 1, m
                        h1 = h1 + work(k,1) * phi(i,k)
                        h2 = h2 + work(k,2) * phi(i,k)
                     end do
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- idim = 3, three components of the velocity vector

                  do i = ni, n
                     h1 = 0d0
                     h2 = 0d0
                     h3 = 0d0
                     do k = 1, m
                        h1 = h1 + work(k,1) * phi(i,k)
                        h2 = h2 + work(k,2) * phi(i,k)
                        h3 = h3 + work(k,3) * phi(i,k)
                     end do
                     do j = 1, n
                        s(i,j) = s(i,j) - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2) &
                                        - h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            end if
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el3001'

      end if  ! ( debug )
      call erclos ( 'el3001' )

      end
