      subroutine el3002( b, s, w, phi, mdecl, ndecl, nsrow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-10-2010 Remove work
!        version  1.2  date 04-08-2006 Debug statements
!        version  1.1  date 03-03-1994 Extension with jadd=-1
!        version  1.0  date 09-05-1988 Developed at Toshiba PC
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element stiffness matrix corresponding to the
!     "constant" part of an elliptic equation, i.e.
!        s  = s   + / b  phi  phi dx
!         ij   ij   e      j     i
!                               k        k         k
!           = s   + sum  w   b(x ) phi (x )  phi (x )
!              ij    k    k           j         i
!     EL3002 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, ndecl, nsrow
      double precision b(m), s(nsrow,nsrow), w(m), &
                       phi(ndecl,mdecl)

!     b        i     array of length m in which the values of
!                    b in the integration points are stored
!     mdecl    i     Number of integration points (m)
!     ndecl    i     Number of basis functions (n)
!     nsrow    i     Size of the element matrix S
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points if a Gauss rule is
!                    used.
!     s       i/o    Element stiffness matrix to be filled
!                    At input S may already have been filled partly depending
!                    on the value of jadd in common celint
!                    Since SEPRAN assumes an one-dimensional storage, S
!                    must be stored transposed.
!                    S has size nsrow x nsrow, but only the part (1:n,1:n)
!                    is filled.
!     w        i     array of length m containing the weights for the
!                    numerical integration
!                    phi  (xg ) = phi (i,k)
!                       i    k
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, sum, work(m)
      integer i, j, k, nstart

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     nstart         Under bound for i variable
!     sum            Help parameter to compute a sum
!     work           Work array of length m, to store products of
!                    b  x w temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3002' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3002'
         write(irefwr,1) 'n, m, ielem, jadd', n, m, ielem, jadd
         write(irefwr,1) 'jzero, jdiag', jzero, jdiag
         call prinrl ( w, m, 'w' )
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set nstart

      nstart = max ( kstep, istarw )

!     --- Check if matrix must be added to an existing matrix or not
!         If not set matrix S equal to zero for the  n x n part

      if ( jadd==0 ) s(1:n,1:n) = 0d0  ! clear matrix

      if ( jadd>=0 ) then

!     --- Add matrix to S

         if ( jdiag==1 ) then

!        --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                             i  j         ij

            if ( jzero==1 ) then

!           --- jzero = 1  b and w constant

               h = b(1) * w(1)
               do i = nstart, m, kstep
                  s(i,i) = s(i,i) + h
               end do

            else

!           --- jzero = 2 or 3  b or w variable

               do i = nstart, m, kstep
                  s(i,i) = s(i,i) + w(i) * b(i)
               end do

            end if

         else

!        --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                      i  j         ij

            if ( jzero==1 ) then

!           --- jzero = 1  b and w constant

               h = b(1) * w(1)
               do i = nstart, n
                  do j = 1, i
                     sum = 0d0
                     do k = 1, m
                        sum = sum + phi(i,k) * phi(j,k)
                     end do
                     sum = h * sum
                     if ( j/=i ) s(j,i) = s(j,i) + sum
                     s(i,j) = s(i,j) + sum
                  end do
               end do
            else

!           --- jzero = 2 or 3  b or w variable
!               Store b * w in work

               work = b * w
               do i = nstart, n
                  do j = 1, i
                     sum = 0d0
                     do k = 1, m
                        sum = sum + work(k) * phi(i,k) * phi(j,k)
                     end do
                     if ( j/=i ) s(j,i) = s(j,i) + sum
                     s(i,j) = s(i,j) + sum
                  end do
               end do
            end if
         end if
      else

!     --- Subtract matrix from S

         if ( jdiag==1 ) then

!        --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                             i  j         ij

            if ( jzero==1 ) then

!           --- jzero = 1  b and w constant

               h = b(1) * w(1)
               do i = nstart, m, kstep
                  s(i,i) = s(i,i) - h
               end do
            else

!           --- jzero = 2 or 3  b or w variable

               do i = nstart, m, kstep
                  s(i,i) = s(i,i) - w(i) * b(i)
               end do
            end if
         else

!        --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                      i  j         ij

            if ( jzero==1 ) then

!           --- jzero = 1  b and w constant

               h = b(1) * w(1)
               do i = nstart, n
                  do j = 1, i
                     sum = 0d0
                     do k = 1, m
                        sum = sum + phi(i,k) * phi(j,k)
                     end do
                     sum = h * sum
                     if ( j/=i ) s(j,i) = s(j,i) - sum
                     s(i,j) = s(i,j) - sum
                  end do
               end do
            else

!           --- jzero = 2 or 3  b or w variable
!               Store b * w in work

               work = b * w
               do i = nstart, n
                  do j = 1, i
                     sum = 0d0
                     do k = 1, m
                        sum = sum + work(k) * phi(i,k) * phi(j,k)
                     end do
                     if ( j/=i ) s(j,i) = s(j,i) - sum
                     s(i,j) = s(i,j) - sum
                  end do
               end do
            end if
         end if
      end if
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el3002'
      end if  ! ( debug )
      call erclos ( 'el3002' )
      end
