      subroutine el3003 ( g, f, w, phi, mdecl, ndecl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 10-02-2015 Extra debug
!        version  1.2  date 30-09-2010 Debug statements
!        version  1.1  date 05-12-1989 Correction for constant quadratic
!                                      triangle
!
!   copyright (c) 1988-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector, i.e.
!        f  = f   + / g  phi dx
!         i    i    e       i
!                              k         k
!           = f  + sum  w   g(x )  phi (x )
!              i    k    k            i
!     EL3003 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, ndecl
      double precision g(mdecl), f(ndecl), w(mdecl), phi(ndecl,mdecl)

!     f       i/o    Element vector to be filled
!                    At input F may already have been filled partly depending
!                    on the value of jadd in common celint
!     g        i     array of length m in which the values of
!                    g in the integration points are stored
!     mdecl    i     Number of integration points (m)
!     ndecl    i     Number of basis functions (n)
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points if a Gauss rule is
!                    used.
!                    phi  (xg ) = phi (i,k)
!                       i    k
!     w        i     array of length m containing the weights for the
!                    numerical integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, som, work(m)
      integer i, k

!    h               Help variables to compute a sum or to store a quantity
!    i               Counting variables
!    k               Counting variables
!    som             Help variables to compute a sum or to store a quantity
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3003' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3003'
         write(irefwr,1) 'n, m, kstep, jadd, jdiag, jconsf', &
                          n, m, kstep, jadd, jdiag, jconsf
         call prinrl ( g, m, 'g' )
         call prinrl ( w, m, 'w' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if vector must be added to an existing vector or not
!         If not set vector F equal to zero

      if ( jadd==0 ) then

!     --- jadd = 0, set vector equal to zero

         do i = 1,n
            f ( i ) = 0d0
         end do
      end if

!     --- Add vector to F

      if ( jdiag==1 ) then

!     --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                          i  j         ij

         if ( jconsf==1 ) then

!        --- jconsf = 1  g and w constant

            h = g(kstep) * w(kstep)
            do i = kstep, m, kstep
               f ( i ) = f ( i ) + h
            end do

         else

!        --- jconsf = 2  g or w variable

            do i = kstep, m, kstep
               f ( i ) = f ( i ) + w ( i ) * g ( i )
            end do

         end if

      else

!     --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                   i  j         ij

         if ( jconsf==1 ) then

!        --- jconsf = 1  g and w constant

            h = g(1) * w(1)
            do i = 1, n
               som = 0d0
               do k = 1, m
                  som = som + phi ( i,k )
               end do
               som = h * som
               f ( i ) = f ( i ) + som
            end do

         else

!        --- jconsf = 2  g or w variable
!            Store g * w in work

            do k = 1, m
               work ( k ) = g ( k ) * w ( k )
            end do
            do i = 1, n
               som = 0d0
               do k = 1, m
                  som = som + work ( k ) * phi ( i,k )
               end do
               f ( i ) = f ( i ) + som
            end do

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el3003'

      end if  ! ( debug )
      call erclos ( 'el3003' )

      end
