      subroutine el3005 ( alpha, s, w, dphidx, mdecl, ndecl, nsrow, &
                          work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-08-2014 Extra debug
!        version  1.0  date 09-05-1988
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element stiffness matrix corresponding to the
!     second derivative part of a three-dimensional equation, i.e.
!        s  = s   + / (alpha grad phi ) . grad phi dx
!         ij   ij   e                j            i
!                                   k             k               k
!           = s   + sum  w  (alpha(x ) grad phi (x ) . grad phi (x )
!              ij    k    k                    j               i
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, ndecl, nsrow
      double precision alpha(mdecl,6), s(nsrow,nsrow), w(mdecl), &
                       dphidx(ndecl,mdecl,3), work(mdecl,6)

!     alpha    i     array of length m x 6 in which the values of
!                    alpha  , alpha  , alpha  , alpha  , alpha  , alpha
!                         11       12       13       22       23       33
!                    are stored in the sequence:
!                                            k
!                    alpha ( k,i ) = alpha (x )   i = 1(1)6, k=1(1)m
!                                         i
!     s        o     Element stiffness matrix to be filled
!                    Since SEPRAN assumes an one-dimensional storage, S
!                    must be stored transposed.
!                    S has size nsrow x nsrow, but only the part (1:n,1:n)
!                    is filled.
!     w        i     array of length m containing the weights for the
!                    numerical integration
!     dphidx   i     Array of length n x m x 3  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear tetrahedron (n=4), then the
!                    derivatives are constant and only k = 1 is filled.
!     mdecl    i     Number of integration points (m)
!     ndecl    i     Number of basis functions (n)
!     nsrow    i     Size of the element matrix S
!     work           Work array of length m x 6, to store products of
!                    alpha   x w temporarily
!                         ij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a11con, a12con, a13con, a22con, a23con, a33con
      double precision sum
      integer i, j, k

!    a11con, a12con, a22con, a13con, a23con, a33con   Constant value of
!                  alpha
!                       ij
!    sum           Help variable to store a sum
!    i,k,j         Counting variables
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3005' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3005'
         write(irefwr,1) 'jsecnd, n, m', jsecnd, n, m
         write(irefwr,1) 'ndecl, mdecl', ndecl, mdecl
         call prinrl1 ( dphidx(1,1,1), mdecl, ndecl, 'dphi/dx' )
         call prinrl1 ( dphidx(1,1,2), mdecl, ndecl, 'dphi/dy' )
         call prinrl1 ( dphidx(1,1,3), mdecl, ndecl, 'dphi/dz' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( jsecnd==1 ) then

!     --- jsecnd = 1   a12=a13=a23=0; a11=a22=a33  a11 and w constant

         a11con = w(1) * alpha(1,1) * m
         if ( n==4 ) then

!        --- n = 4;  Linear tetrahedron  ( derivatives constant )

            do i = 1,n
            do j = 1,i
               s ( i,j ) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) + &
                                      dphidx(i,1,2) * dphidx(j,1,2) + &
                                      dphidx(i,1,3) * dphidx(j,1,3) )
            end do  ! i = 1,n
            end do  ! j = 1,i
         else

!        --- n # 4  other cases,  summation over integration points

            do i = 1,n
            do j = 1,i
               sum = 0d0
               do k = 1,m
                  sum = sum + ( dphidx(i,k,1) * dphidx(j,k,1) + &
                                dphidx(i,k,2) * dphidx(j,k,2) + &
                                dphidx(i,k,3) * dphidx(j,k,3) )
               end do  ! k = 1,m
               s ( i,j ) = a11con * sum
            end do  ! i = 1,n
            end do  ! j = 1,i
         end if
      else if ( jsecnd==2 ) then

!     --- jsecnd = 2   a12=a13=a23=0; a11=a22=a33

          if(n==4) then

!         --- n = 4;  Linear tetrahedron  ( derivatives constant )

            a11con = dot_product(w(1:m),alpha(1:m,1))
            if ( debug ) write(irefwr,2) 'a11con', a11con
            do i=1,n
            do j=1,i
               s ( i,j ) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) + &
                                      dphidx(i,1,2) * dphidx(j,1,2) + &
                                      dphidx(i,1,3) * dphidx(j,1,3) )
            end do  ! i = 1,n
            end do  ! j = 1,i
         else

!        --- n # 4  other cases,  summation over integration points
!              Store w  x alpha  (k) in work(k,1)
!                   k        11

            work(1:m,1) = w(1:m) * alpha(1:m,1)
            if ( debug ) call prinrl ( work, m, 'work' )
            do i = 1, n
               do j = 1, i
               sum = 0d0
               do k = 1,m
                  sum = sum + work(k,1) * ( dphidx(i,k,1)*dphidx(j,k,1)+ &
                                            dphidx(i,k,2)*dphidx(j,k,2)+ &
                                            dphidx(i,k,3)*dphidx(j,k,3))
               end do  ! k = 1,m
               s ( i,j ) = sum
               end do  ! j = 1, i
            end do  ! i = 1, n
         end if
      else if ( jsecnd==3 ) then

!     --- jsecnd = 3   a12#0 or a13#0 or a23#0; a11=a22=a33

         if(n==4) then

!        --- n = 4;  Linear tetrahedron  ( derivatives constant )

            a11con = 0d0
            a12con = 0d0
            a13con = 0d0
            a23con = 0d0
            do k = 1,m
               a11con = a11con + w(k) * alpha(k,1)
               a12con = a12con + w(k) * alpha(k,2)
               a13con = a13con + w(k) * alpha(k,3)
               a23con = a23con + w(k) * alpha(k,5)
            end do  ! k = 1,m
            do i=1,n
            do j=1,i
               s ( i,j ) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) + &
                                      dphidx(i,1,2) * dphidx(j,1,2) + &
                                      dphidx(i,1,3) * dphidx(j,1,3) ) &
                         + a12con * ( dphidx(i,1,1) * dphidx(j,1,2) + &
                                      dphidx(i,1,2) * dphidx(j,1,1) ) &
                         + a13con * ( dphidx(i,1,1) * dphidx(j,1,3) + &
                                      dphidx(i,1,3) * dphidx(j,1,1) ) &
                         + a23con * ( dphidx(i,1,2) * dphidx(j,1,3) + &
                                      dphidx(i,1,3) * dphidx(j,1,2) )
            end do  ! i = 1,n
            end do  ! j = 1,i
         else

!        --- n # 4  other cases,  summation over integration points
!              Store w(k)  x alpha  (k) in work(k,ij)
!                               ij

            do k = 1,m
               work(k,1) = w(k) * alpha(k,1)
               work(k,2) = w(k) * alpha(k,2)
               work(k,3) = w(k) * alpha(k,3)
               work(k,5) = w(k) * alpha(k,5)
            end do  ! k = 1,m
            do i = 1,n
            do j = 1,i
               sum = 0d0
               do k = 1,m
                  sum = sum + work(k,1)*(dphidx(i,k,1)*dphidx(j,k,1)+ &
                                         dphidx(i,k,2)*dphidx(j,k,2)+ &
                                         dphidx(i,k,3)*dphidx(j,k,3)) &
                            + work(k,2)*(dphidx(i,k,1)*dphidx(j,k,2)+ &
                                         dphidx(i,k,2)*dphidx(j,k,1)) &
                            + work(k,3)*(dphidx(i,k,1)*dphidx(j,k,3)+ &
                                         dphidx(i,k,3)*dphidx(j,k,1)) &
                            + work(k,5)*(dphidx(i,k,2)*dphidx(j,k,3)+ &
                                         dphidx(i,k,3)*dphidx(j,k,2))
               end do  ! k = 1,m
               s ( i,j ) = sum
            end do  ! i = 1,n
            end do  ! j = 1,i
         end if
      else if ( jsecnd==4 ) then

!     --- jsecnd = 4   general

         if(n==4) then

!        --- n = 4;  Linear tetrahedron  ( derivatives constant )

            a11con = 0d0
            a12con = 0d0
            a13con = 0d0
            a22con = 0d0
            a23con = 0d0
            a33con = 0d0
            do k = 1,m
               a11con = a11con + w(k) * alpha(k,1)
               a12con = a12con + w(k) * alpha(k,2)
               a13con = a13con + w(k) * alpha(k,3)
               a22con = a22con + w(k) * alpha(k,4)
               a23con = a23con + w(k) * alpha(k,5)
               a33con = a33con + w(k) * alpha(k,6)
            end do  ! k = 1,m
            do i=1,n
            do j=1,i
               s ( i,j ) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) ) &
                         + a22con * ( dphidx(i,1,2) * dphidx(j,1,2) ) &
                         + a33con * ( dphidx(i,1,3) * dphidx(j,1,3) ) &
                         + a12con * ( dphidx(i,1,1) * dphidx(j,1,2) + &
                                      dphidx(i,1,2) * dphidx(j,1,1) ) &
                         + a13con * ( dphidx(i,1,1) * dphidx(j,1,3) + &
                                      dphidx(i,1,3) * dphidx(j,1,1) ) &
                         + a23con * ( dphidx(i,1,2) * dphidx(j,1,3) + &
                                      dphidx(i,1,3) * dphidx(j,1,2) )
            end do  ! j = 1,i
            end do  ! i = 1,n
         else

!        --- n # 4  other cases,  summation over integration points
!              Store w(k)  x alpha  (k) in work(k,ij)
!                               ij

            do j = 1,6
            do k = 1,m
               work(k,j) = w(k) * alpha(k,j)
            end do  ! k = 1,m
            end do  ! j = 1,6
            do i = 1,n
            do j = 1,i
               sum = 0d0
               do k = 1,m
                  sum = sum + work(k,1)*(dphidx(i,k,1)*dphidx(j,k,1)) &
                            + work(k,4)*(dphidx(i,k,2)*dphidx(j,k,2)) &
                            + work(k,6)*(dphidx(i,k,3)*dphidx(j,k,3)) &
                            + work(k,2)*(dphidx(i,k,1)*dphidx(j,k,2)+ &
                                         dphidx(i,k,2)*dphidx(j,k,1)) &
                            + work(k,3)*(dphidx(i,k,1)*dphidx(j,k,3)+ &
                                         dphidx(i,k,3)*dphidx(j,k,1)) &
                            + work(k,5)*(dphidx(i,k,2)*dphidx(j,k,3)+ &
                                         dphidx(i,k,3)*dphidx(j,k,2))
               end do  ! k = 1,m
               s ( i,j ) = sum
            end do  ! j = 1,i
            end do  ! i = 1,n
         end if
      end if

!     --- the upper triangle of s is computed by tranposition

      do i = 1,n
         do j = 1,i-1
            s ( j,i ) = s ( i,j )
         end do  ! j = 1,i-1
      end do  ! i = 1,n
1000  call erclos ( 'el3005' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( s, n, n, 's' )
         write(irefwr,*) 'End el3005'
      end if  ! ( debug )
      end subroutine el3005
