      subroutine el3006 ( b, s, w, phi, psi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 20-10-2010 Remove some parameters from list
!        version  2.0  date 16-12-1994 Removal of mdecl and ndecl
!        version  1.1  date 18-06-1991 Error correction i,j
!
!   copyright (c) 1989-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!     Compute contribution to element stiffness matrix corresponding to the
!     "constant" part of an elliptic equation, where the test functions
!     may differ from the basis functions i.e.
!        s  = s   + / b  phi  psi dx
!         ij   ij   e      j     i
!                               k        k         k
!           = s   + sum  w   b(x ) phi (x )  psi (x )
!              ij    k    k           j         i
!        It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
!     convection
!     element_matrix
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision b(m), s(n,n), w(m), &
                       phi(n,m), psi(n,m)

!     b        i     array of length m in which the values of
!                    b in the integration points are stored
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points if a Gauss rule is
!                    used.
!                    phi  (xg ) = phi (i,k)
!                       i    k
!     psi      i     Array of length n x m containing the values of the test
!                    functions in the integration points if a Gauss rule is
!                    used.
!                    psi  (xg ) = psi (i,k)
!                       i    k
!     s       i/o    Element stiffness matrix to be filled
!                    At input S may already have been filled partly depending
!                    on the value of jadd in common celint
!                    Since SEPRAN assumes an one-dimensional storage, S
!                    must be stored transposed.
!                    S has size nsrow x nsrow, but only the part (1:n,1:n)
!                    is filled.
!     w        i     array of length m containing the weights for the
!                    numerical integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, sum, work(m)
      integer i, j, k, ni

!    h           Help variable to compute a sum or to store a quantity
!    i           Counting variable
!    j           Counting variable
!    k           Counting variable
!    ni          Lower bound for i variable
!    sum         Help variable to compute a sum or to store a quantity
!    work        Work array of length m, to store products of
!                b  x w temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set ni

      ni = max ( kstep, istarw )

!     --- Check if matrix must be added to an existing matrix or not
!         If not set matrix S equal to zero for the  n x n part

      if ( jadd==0 ) s = 0d0  ! set matrix equal to zero

!     --- Add matrix to S

      if ( jdiag==1 ) then

!     --- jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                          i  j         ij

         if ( jzero==1 ) then

!        --- jzero = 1  b and w constant

            h = b(1) * w(1)
            do i = ni, n, kstep
               do j = 1, n
                  s(j,i) = s(j,i) + h *  psi(i,j)
               end do
            end do
         else

!        --- jzero = 2 or 3  b or w variable
!            Store b * w in work

            do k = kstep, m, kstep
               work(k) = b(k) * w(k)
            end do
            do i = ni, n, kstep
               do j = 1, n
                  s(j,i) = s(j,i) + work(j) * psi(i,j)
               end do
            end do
         end if
      else

!     --- jdiag = 2, Gauss rule  phi (x ) # delta
!                                   i  j         ij

         if ( jzero==1 ) then

!        --- jzero = 1  b and w constant

            h = b(1) * w(1)
            do i = ni, n
               do j = 1, n
                  sum = 0d0
                  do k = 1, m
                     sum = sum + phi(j,k) * psi(i,k)
                  end do
                  s(j,i) = s(j,i) + h * sum
               end do
            end do
         else

!        --- jzero = 2 or 3  b or w variable
!            Store b * w in work

           work = b * w
            do i = ni, n
               do j = 1, n
                  sum = 0d0
                  do k = 1, m
                     sum = sum + work(k) * phi(j,k) * psi(i,k)
                  end do
                  s(j,i) = s(j,i) + sum
               end do
            end do
         end if
      end if
      end
