      subroutine el3007 ( usolloc, dudx, phi, dphidx, mdecl, &
                          ndecl, nddim, ndcomp, indlc, trans, &
                          usolgauss, jdtran, numld1, iseqinlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.15 date 29-11-2013 Extra debug
!        version  2.14 date 12-12-2012 Long integers
!
!   copyright (c) 1988-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute a local vector and its derivatives from a known vector.
!    the values are evaluated in the integration points or in the nodal points
!     EL3007 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, ndecl, nddim, ndcomp, numld1, &
              indlc(ndcomp,ndecl), jdtran, iseqinlc
      double precision  usolloc(ndecl,ndcomp), &
                        dudx(mdecl,nddim,ndcomp), &
                        dphidx(ndecl,mdecl,*), &
                        trans(ndecl-jdtran,ndcomp*ndcomp), &
                        usolgauss(mdecl,ndcomp), phi(ndecl,mdecl)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           o    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     indlc          i    integer array that is used if jind<=0, see jind
!     iseqinlc       i    Sequence number of vector in vecoldlc/ISLOLD
!     jdtran         i    Value of jtrans in calling subroutine
!     mdecl          i    number of integration points in element, &
!                         same as  m  in common /celint/
!     ndcomp         i    Indicates the number of quantities that must be filled
!                         in elemvc and elemwg per nodal point
!     nddim          i    Dimension of the space. Indicates the number of
!                         derivatives of the basis functions implicitly
!     ndecl          i    number of nodal points in element, &
!                         same as  n  in common /celint/
!     numld1         i    Formal parameter NUMOLD
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                                k
!                         phi (xg ) = phi (i,k)
!                            i
!                         array phi is only filled when gauss integration is
!                         applied
!     trans          i    array containing the inverse of the transformation
!     usolgauss      o    array of length m x ncomp in which the values of
!                         usolloc in the integration points are stored the
!                         sequence:
!                         u_i(x_j) = usolgauss(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                         usolgauss is only filled if jdiag>1 in common celint
!     usolloc        o    Real n x ncomp array in which the values of the
!                         solution vector in the nodal points of the element are
!                         stored in the sequence:
!                         u_i(x_j) = usolloc(j,i)  j = 1(1)n, i = 1 (1) ncomp
!                         Array usolloc is always filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, lstep, lsub, lsup, kderiv, iposi
      double precision sum
      double precision, pointer :: vecoldlc(:)
      save lstep, lsub, lsup

!     i              Counting variable
!     iposi          Help parameter to compute a position in index3
!     j              Counting variable
!     k              Counting variable
!     kderiv
!     l              General loop variable
!     lstep          Step variable for vertices. Usually lstep = 1,
!                    but when the solution must be computed in the vertices
!                    and the element is quadratic then lstep = 2
!     lsub           Under index for the direction of the derivative to be
!                    computed
!     lsup           Upper index for the direction of the derivative to be
!                    computed
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3007' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3007'
         write(irefwr,1) 'jderiv, ndimlc, n, m, jind, jtrans, ndcomp', &
                          jderiv, ndimlc, n, m, jind, jtrans, ndcomp
!         call prinin1 ( index4, n, numld1, 'index4' )
  1      format ( a, 1x, (10i6) )

      end if

      lsub=1

!     --- first set some constants

      kderiv = abs(jderiv)
      if ( kderiv==1 .or. kderiv==5 ) then

!     --- kderiv = 1 or 5:  The gradient must be computed
!                           Components 1 to ndim

         lsub=1
         lsup=ndimlc

      else if ( kderiv>1 ) then

!     --- kderiv 2,3,4, 6, 7 or 8:  Component kderiv-1 must be computed

         lsub=kderiv-1
         if ( kderiv>5 ) lsub = lsub-4
         lsup=lsub

      end if
      if ( kderiv>1 .and. kderiv<6 .and. &
           ndimlc==2 .and. n>=6 ) then

!     --- kderiv>1, hence computation in vertices
!                     Furthermore quadratic 2D element
!                     Skip points between

         lstep=2

      else

         lstep=1

      end if
      if ( kderiv>1 .and. ndim==3 .and. (n/=4 .and. n/=8) ) &
         lstep=2

      iseqtype = oldsols(iseqinlc)%iseqtype

      index3n => indoldsols(iseqtype)%index3
      index4n => indoldsols(iseqtype)%index4
      vecoldlc => oldsols(iseqinlc)%oldvecs

!     --- Fill array u with solution in nodes

      if ( debug ) &
         write(irefwr,1) 'lsub, lsup, lstep, ncomp',lsub, lsup, lstep, ncomp
      if ( jind==0 ) then

!     --- jind = 0, use array ind to find unknowns

         do j = 1, ncomp
            do i = 1, n - jtrans

               usolloc(i,j) = vecoldlc(index3n( indlc(j,i) ) )
            end do
         end do

      else if ( jind==-1 ) then

!     --- jind = -1, use array indlc partly to find unknowns

         if ( ndcomp==1 ) then

!        --- ndcomp = 1

            iposi = indlc(1,1)
            do i = 1, n - jtrans
               usolloc(i,1) = &
                  vecoldlc(index3n(iposi))
               iposi = indlc(1,1)+index4n(i)
            end do

         else

!        --- ndcomp>1

            do j = 1, ncomp
               iposi = indlc(j,1)
               do i = 1, n - jtrans
                  usolloc(i,j) = &
                     vecoldlc(index3n(iposi))
                  iposi = indlc(j,1)+index4n(i)
               end do
            end do

         end if

      else if ( jind==-2 ) then

!     --- Navier-Stokes

         do j = 1, ndcomp
            iposi = 0
            do i = 1, n - jtrans
               usolloc(i,j) = vecoldlc(index3n(j+iposi))
               iposi = index4n(i)
            end do
         end do

      else

!     --- jind = >0, use jind to find unknowns

         if ( debug ) &
            write(irefwr,1) 'iseqinlc, ndcomp, jtrans', &
                             iseqinlc, ndcomp, jtrans

         if ( ndcomp==1 ) then

!        --- ndcomp = 1

            do i = 1, n - jtrans
               usolloc(i,1) = vecoldlc(index3n(index4n(i)))
            end do

         else

!        --- ndcomp>1

            if ( debug ) write(irefwr,1) 'ncomp', ncomp
            do j = 1, ncomp
               iposi = jind-1
               do i = 1, n - jtrans
                  usolloc(i,j) = vecoldlc(index3n(iposi+j))
                  iposi = index4n(i)+jind-1
               end do
            end do
            if ( debug ) &
               call prinrl1 ( usolloc, ncomp, n - jtrans, 'usolloc' )

         end if

      end if

      if ( jtrans==1 ) then

!     --- jtrans = 1  Compute extra unknowns by transformation
!                  n-1
!         u (n) =  sum  trans (l)  u (l)
!          k       l=1       k      k

         do j = 1, ncomp
            usolloc(n,j) = 0d0
            do i = 1, ncomp
               usolloc(n,j) = usolloc(n,j) + ddot ( n-jtrans, &
                        trans(1,i+(j-1)*ncomp), 1, usolloc(1,i), 1 )
            end do
         end do

      end if
      if ( debug ) write(irefwr,1) 'jsol, jdiag', jsol, jdiag

      if ( jsol==1 .and. jdiag/=1 ) then

!     --- Compute solution in Gauss points
!                   n
!         ug (k) = sum   u (i) phi (k)
!           l      i=1    l       i

         do j = 1, ndcomp
            do i = 1, m
               usolgauss(i,j) = ddot ( n, usolloc(1,j), 1, phi(1,i), 1 )
            end do
         end do

      else if ( jsol==-1 .and. jdiag/=1 ) then

!     --- Compute solution in Gauss points
!         Temporary situation, especcially for old versions of ELP subroutines, &
!         where the sequence of derivatives of basis functions and basis
!         functions itself differs from the new storage
!                   n
!         ug (k) = sum   u (i) phi (k)
!           l      i=1    l       i

         do j = 1, ncomp
            do i = 1, m
               usolgauss(i,j) = ddot ( n, usolloc(1,j), 1, phi(i,1), m )
            end do
         end do

      else if ( jsol/=0 ) then

!     --- jsol = 1,  Copy u into usolgauss

         do j = 1, ndcomp
            do i = 1, n
               usolgauss(i,j) = usolloc(i,j)
            end do
         end do

      end if

      if ( debug ) then
         call prinrl1 ( usolloc, ncomp, n-jtrans, 'usolloc' )
         write(irefwr,1) 'jderiv, jdercn', jderiv, jdercn
         write(irefwr,1) 'n, m, lstep', n, m, lstep
      end if  ! ( debug )

      if ( jderiv>0 ) then

!     --- jderiv>0,  compute derivatives

         if ( jdercn==0 ) then

!        --- Derivatives of basis functions are constant, hence dudx is constant
!                      n
!            du/dx =  sum  u   d phi  / dx
!                 k   i=1   i       i     k

            do j = 1, ncomp
               do k = lsub, lsup
                  sum = ddot ( n, usolloc(1,j), 1, dphidx(1,1,k), 1 )
                  do l = 1, m, lstep
                     dudx ( l, k+1-lsub, j ) = sum
                  end do
               end do
            end do

         else if ( kstep==1 ) then

!        --- dudx is variable

            do j = 1, ncomp
               do k = lsub, lsup
                  do l = 1, m, lstep
                     dudx ( l, k+1-lsub, j ) = ddot ( n, usolloc(1,j), &
                                               1, dphidx(1,l,k), 1 )
                  end do
               end do
            end do

         else

!        --- dudx is variable and kstep=2

            do j = 1, ncomp
               do k = lsub, lsup
                  do l = kstep, m, kstep
                     dudx ( l, k+1-lsub, j ) = ddot ( n, usolloc(1,j), &
                                               1, dphidx(1,l,k), 1 )
                  end do
               end do
            end do

         end if

      else if ( jderiv<0 ) then

!     --- jderiv<0,  compute derivatives in a special way
!         This possibility is temporarily and is only meant to be used in
!         combination with old basis functions subroutines elp...
!         In these subroutines the sequence of dphidx is different from the
!         new one in the sense that we have:
!         dphidx ( k,i,l );  k=1(1)m, i=1(1)n, l=1(1)ndim
!         This possibility works only in the case m=n!!!

         do j = 1, ncomp
            do k = lsub, lsup
               do l = 1, m, lstep
                  dudx ( l, k+1-lsub, j ) = ddot ( n, usolloc(1,j), 1, &
                                            dphidx(l,1,k), m )
               end do
            end do
         end do

      end if

      if ( debug ) write(irefwr,*) 'End el3007'
      call erclos ( 'el3007' )

      end
