      subroutine el3012 ( u, s, w, phi, psi, dphidx, ncompd, nsrow, &
                          work, x )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   16-12-1994
!
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     Compute crrection to element stiffness matrix corresponding to the
!     first derivative part of a one-, two- or three-dimensional equation
!     in case of upwinding and compressible convection
!
!        s  = s   + / div (u phi ) ( psi - phi ) dx
!         ij   ij   e           j       i     i
!
!
!     EL3012 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
!
! ********************************************************************
!
!                      KEYWORDS
!
!     convection
!     element_matrix
!     upwinding
! ********************************************************************
!

      use sepmodulecactl
      use sepmodulecactl
      implicit none
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!

      integer ncompd, nsrow
      double precision u(m,ncompd), s(nsrow,nsrow), w(m), phi(n,m), &
                       dphidx(n,m,*), work(m), psi(n,m), x(m)

!     dphidx   i     Array of length n x m x ncomp  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear symplex, then the
!                    derivatives are constant and only k = 1 is filled.
!     ncompd   i     Number of components of the vector u  (ncomp)
!     nsrow    i     Size of the element matrix S
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points if a Gauss rule is
!                    used.
!                    phi  (xg ) = phi (i,k)
!                       i    k
!     psi      i     Array of length n x m containing the values of the
!                    upwind basis functions in the integration points
!     s       i/o    Element stiffness matrix to be filled
!                    At input S may already have been filled partly depending
!                    Since SEPRAN assumes an one-dimensional storage, S
!                    must be stored transposed.
!                    S has size nsrow x nsrow, but only the part (1:n,1:n)
!                    is filled.
!     u        i     array of length m x ncomp in which the values of
!                    u  are stored in the sequence:
!                     i
!                                    k
!                    u ( k,i ) = u (x )   i = 1(1)3, k=1(1)m
!                                 i
!     w        i     array of length m containing the weights for the
!                    numerical integration
!     work           Work array of length m, to store help products
!                    temporarily
!     x              Array containing the r-co-ordinates
! ********************************************************************
!
!                    LOCAL PARAMETERS
!

      double precision sum
      integer i, j, k

!    i      Counting variable
!    j      Counting variable
!    k      Counting variable
!    sum    Help variable to compute a sum or to store a quantity
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   Define phi u = sum phi (x ) u(x ) phi
!             j     l     j  l     l     l
!
!   Cartesian co-ordinates:
!
!   Contribution becomes: s_ij := s_ij + &
!
!   sum ( ( sum phi_j (x_l) (u_l dphi_l/dx + v_l dphi_l/dy + w_l dphi_l/dz) )
!    k       l
!
!       * (psi_i - phi_i) (x_k) )
!
!   This reduces to:
!
!   sum ( u_j dphi_j/dx + v_j dphi_j/dy + w_j dphi_j/dz
!    k
!
!       * (psi_i - phi_i) (x_k) )
!
!   Cylinder co-ordinates:
!
!   The divergence is equal to du/dr + dw/dz + u/r
!   Hence the Cartesian formulas must be extended with
!
!   u_l phi_l / x_l, whenever x_l # 0
! ======================================================================
!
!     --- Add matrix to S

      if ( jdercn.eq.1 ) then

!     --- jdercn = 1   derivatives of basis functions are variable

         do 200 j = 1, n

!        --- First fill work with sum w_k (u_j dphi_j(k)+...)

            if ( idim.eq.1 ) then

!           --- idim = 1, only one component of the velocity vector

               do 100 k = 1, m
                  work(k) = w(k)*u(j,1)*dphidx(j,k,1)
100            continue

            else if ( idim.eq.2 ) then

!           --- idim = 2, two components of the velocity vector

               do 110 k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,k,1)+ &
                                     u(j,2)*dphidx(j,k,2) )
110            continue
               if ( jcart.eq.2 .and. x(j).ne.0d0 ) then

!              --- Extra term for cylinder co-ordinates

                  work(j) = work(j) + w(j)*u(j,1)/x(j)

               end if

            else

!           --- idim = 3, three components of the velocity vector

               do 120 k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,k,1)+ &
                                     u(j,2)*dphidx(j,k,2)+ &
                                     u(j,3)*dphidx(j,k,3) )
120            continue

            end if
            do 140 i = 1, n
               sum = 0d0
               do 130 k = 1, m
                  sum = sum + work(k)*(psi(i,k)-phi(i,k))
130            continue
               s(j,i) = s(j,i) + sum
140         continue

200      continue

      else

!     --- jdercn = 0   derivatives of basis functions are constant

         do 500 j = 1, n

!        --- First fill work with sum w_k (u_j dphi_j(k)+...)

            if ( idim.eq.1 ) then

!           --- idim = 1, only one component of the velocity vector

               do 410 k = 1, m
                  work(k) = w(k)*u(j,1)*dphidx(j,1,1)
410            continue

            else if ( idim.eq.2 ) then

!           --- idim = 2, two components of the velocity vector

               do 420 k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,1,1)+ &
                                     u(j,2)*dphidx(j,1,2) )
420            continue
               if ( jcart.eq.2 .and. x(j).ne.0d0 ) then

!              --- Extra term for cylinder co-ordinates

                  work(j) = work(j) + w(j)*u(j,1)/x(j)

               end if

            else

!           --- idim = 3, three components of the velocity vector

               do 430 k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,1,1)+ &
                                     u(j,2)*dphidx(j,1,2)+ &
                                     u(j,3)*dphidx(j,1,3) )
430            continue

            end if
            do 450 i = 1, n
               sum = 0d0
               do 440 k = 1, m
                  sum = sum + work(k)*(psi(i,k)-phi(i,k))
440            continue
               s(j,i) = s(j,i) + sum
450         continue

500      continue

      end if

      end
