      subroutine el4921 ( elemmt, sp, div, pp, nveloc, npres, eps, &
                          mcont, w, x, xgauss, signdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.2  date 17-09-2008 Extension with pressure mass matrix
!        version  5.1  date 20-01-2006 Use min div u, to make matrix symmetrical
!        version  5.0  date 23-02-2004 Extra parameters w, x, xgauss
!        version  4.0  date 17-08-2001 Extra parameter mcont
!                                      Incorporate pressure term in continuity
!                                      equation (Gerard Haagh)
!        version  3.1  date 06-08-1998 Include commons
!        version  3.0  date 01-03-1996 Extra parameter eps
!        version  2.0  date 20-03-1995 Extra parameter npres
!        version  1.0  date 20-08-1994
!
!   copyright (c) 1994-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put gradient and divergence matrix into large matrix for the
!     Navier-Stokes equations
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nveloc, npres, signdiv
      double precision sp(ndimlc*n,npres), eps, w(*), xgauss(m,*), &
                       x(n,*), elemmt(icount,icount), &
                       div(ndimlc*n,npres)
      integer mcont
      double precision pp(npres,npres)

!     div            i    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     elemmt        i/o   Element matrix to be filled
!                         Only the part corresponding to div and grad is filled
!     eps            i    Penalty parameter.
!                         The pressure mass matrix multiplies by eps is added
!                         to the matrix
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                         2: Goertler equations: No dp/dx, but complete div u
!                         4: the continuity equation contains a pressure term
!     npres          i    Number of pressure parameters per element
!     nveloc         i    Number of velocity parameters
!     pp             i    Pressure matrix
!     signdiv        i    Multiplication sign of "continuity equations"
!                         if -1 the stokes matrix is symmetric
!                         if 1 all eigenvalues are in the positive half plane
!     sp             i    Gradient matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    contains the coordinates of the nodes
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, isigndiv
      double precision diag(npres)

!     diag           Diagonal of the pressure mass matrix
!     i              Counting variable
!     isigndiv       signdiv except for mcont = 5, then 1
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix has the shape:
!            |               T   |
!            |   S        -SP    |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV       PP    |    p
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize isigndiv
!         Must be 1 in case of mcont = 5

      isigndiv = signdiv
      if ( mcont==5 ) isigndiv = 1

!     --- divergence matrix, mark that the matrix elemmt must be used
!         in transposed form

      if ( ndimlc==2 ) then

!     --- ndim = 2

         do j = 1, npres
            do i = 1, nveloc
               elemmt(i,icount-npres+j) = signdiv*div(i,j)
            end do
         end do

!        --- gradient matrix

         do j = 1, npres
            do i = 1, nveloc
               elemmt(icount-npres+j,i) = -sp(i,j)
            end do
         end do

!        --- pressure matrix

         if( mcont==4 .or. mcont==5 )then

!        --- mcont = 4 or 5, add pressure mass matrix

            do j = 1, npres
               do i = 1, npres
                  elemmt(icount-npres+j,icount-npres+i) = &
                     isigndiv*pp(i,j)
               end do
            end do
         end if

      else

!     --- ndim = 3

         do j = 1, npres
            do i = 1, nveloc
               elemmt(i,icount-npres+j) = signdiv*div(i,j)
            end do
         end do

!        --- gradient matrix

         do j = 1, npres
            do i = 1, nveloc
               elemmt(icount-npres+j,i) = -sp(i,j)
            end do
         end do

!        --- pressure matrix

         if( mcont==4 .or. mcont==5 )then

!        --- mcont = 4 or 5, add pressure mass matrix

            do j = 1, npres
               do i = 1, npres
                  elemmt(icount-npres+j,icount-npres+i) = &
                     isigndiv*pp(i,j)
               end do
            end do

         end if

      end if

!     --- Add penalty parameter to diagonal of pressure part

      if ( eps>0d0 ) then

!     --- eps>0, there is a penalty contribution

         do i = 1, npres
            diag(i) = 0d0
         end do
         do k = 1, m
            diag(1) = diag(1) + w(k)
         end do

         do i = 1, npres-1
            do k = 1, m
               diag(i+1) = diag(i+1) + w(k)*(xgauss(k,i)-x(n,i))**2
            end do
         end do

!        --- store the diagonal multiplied by eps in diag

         do i = 1, npres
            diag(i) = diag(i) * eps
         end do

         do j = 1, npres
            elemmt(icount-npres+j,icount-npres+j) = &
               elemmt(icount-npres+j,icount-npres+j) + diag(j)*signdiv
         end do

      end if

      end
