      subroutine el6074 ( f, old, wxp, dxp, p1p, p2p, p3p, p4p, &
                          work, mvone, mv, numr )
! ======================================================================
!
!        programmer     Peter Minev/ Luc Timmermans
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  30-03-1995
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     Compute the weak gradient of a scalar function (pressure)
!     for righthandside/residual of the time-dependent Navier-Stokes
!     equations; 2-D case.
!     Every component is computed separately depending on numr (1,2)
!
!     EL6074 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640, ELP646
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer mvone, mv, numr
      double precision f(mv), old(mv), wxp(mvone), &
                       dxp(mvone,mvone), &
                       p1p(mvone,mvone), p2p(mvone,mvone), &
                       p3p(mvone,mvone), p4p(mvone,mvone), work(mv)

!     dxp      i   array of length mvone x mvone contaning the values of the
!                  derivatives of the basisfunctions for the pressure in the
!                  pressure points in the x-direction
!     f       i/o  array of length 2*mv containing the righthandside
!                  or the residual
!     mv       i   number of velocity points in the element
!     mvone    i   number of velocity points in the one dimension
!     numr     i   number of the gradient component to be computed
!     old      i   array of length mv containing the old pressure
!     p1p      i   arrays of length mvone x mvone containing the partial
!                  derivatives of the coordinates due to transformation for
!                  the pressure
!     p2p      i   See p1
!     p3p      i   See p1
!     p4p      i   See p1
!     work     -   work array, length mv
!     wxp      i   array of length mvone containing the pressure weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
! ***********************************************************************
!
!                        COMMON BLOCKS
!
!***********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, j, k, ij, ik, kj
      double precision sum

!     i              counting variable
!     ij             position in the vector f
!     ik             position in the vector f
!     j              counting variable
!     k              counting variable
!     kj             position in the vector f
!     sum            help variable to compute a sum
!***********************************************************************
!
!                     SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! ********************************************************************
!
!                    I/O
!
!    none
!***********************************************************************
!
!                     ERROR MESSAGES
!
!    1619   Component of gradient can not be computed
!***********************************************************************
!
!                    PSEUDO CODE
!
! ======================================================================
!
      call eropen ( 'el6074' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el6074'
         write(irefwr,1) 'jadd, numr, mvone, mv', jadd, numr, mvone, mv
         call prinrl ( f, mv, 'f' )
         call prinrl1 ( dxp, mvone, mvone, 'dxp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      if (jadd==0) f(1:mv) = 0d0

      if ( numr<1 .or. numr>2 ) then
         call errint ( numr, 1 )
         call errsub ( 1619, 1, 0, 0 )
         go to 5000
      end if
      go to ( 10, 20) numr

!     --- add to f the divergence part
!         store first help factor in work

10    do 300 i = 1, mvone

         ij = i
         do 200 j = 1, mvone

            work(ij) = wxp(i)*wxp(j)*p4p(i,j)
            ij = ij + mvone
200      continue
300   continue

!     --- add first factor of derivative part to f

      do 600 i = 1, mvone

         ij = i
         do 500 j = 1, mvone

            sum = 0d0
            do 400 k = 1, mvone

               kj = k + (j - 1)*mvone
               sum = sum + dxp(i,k)*old(kj)
400         continue

            f(ij) = f(ij)  + work(ij)*sum
            ij = ij + mvone
500      continue
600   continue

!     --- store second help factor in work

      do 800 i = 1, mvone

         ij = i
         do 700 j = 1, mvone

            work(ij) = - wxp(i)*wxp(j)*p2p(i,j)
            ij = ij + mvone
700      continue
800   continue

!     --- add second factor of divergence part to f

      do 1100 i = 1, mvone

         ij = i
         do 1000 j = 1, mvone

            sum  = 0d0
            do 900 k = 1, mvone

               ik = i + (k - 1)*mvone
               sum = sum + dxp(j,k)*old(ik)
900         continue

            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mvone
1000     continue
1100  continue
      go to 5000

!     --- second component of the gradient
!         store first help factor in work

20    do 1300 i = 1, mvone

         ij = i
         do 1200 j = 1, mvone

            work(ij) = - wxp(i)*wxp(j)*p3p(i,j)
            ij = ij + mvone
1200     continue
1300  continue

!     --- add first factor of derivative part to f

      do 1600 i = 1, mvone

         ij = i
         do 1500 j = 1, mvone

            sum = 0d0
            do 1400 k = 1, mvone

               kj =  k + (j - 1)*mvone
               sum = sum + dxp(i,k)*old(kj)
1400        continue

            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mvone
1500     continue
1600  continue
!     --- store second help factor in work

      do 1800 i = 1, mvone

         ij = i
         do 1700 j = 1, mvone

            work(ij) = wxp(i)*wxp(j)*p1p(i,j)
            ij = ij + mvone
1700     continue
1800  continue

!     --- add second factor of divergence part to f

      do 2100 i = 1, mvone

         ij = i
         do 2000 j = 1, mvone

            sum  = 0d0
            do 1900 k = 1, mvone

               ik = i + (k - 1)*mvone
               sum = sum + dxp(j,k)*old(ik)
1900        continue

            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mvone
2000     continue
2100  continue

5000  continue

      call erclos ( 'el6074' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( f, mv, 'f' )
         write(irefwr,*) 'End el6074'

      end if  ! ( debug )

      end subroutine el6074
