      subroutine eldiscint1m(m, n, intrule, rho, c, kappa, latentheat, &
                             phi, dphidx, delta, temperature, massmt, &
                             stiffmt, dldtmt, latentvc)
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 30-05-2006
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the mushy case. Mushy region.
!   The code differs from the isothermal case, since the dLdT matrix and
!   L-vector are computed differently.
!   This is the 1D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          m, n, intrule
      double precision c, kappa, rho, latentheat, temperature(*), &
                       dphidx(n,m), massmt(n,n), delta, phi(n,m), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     c              i    Specific heat
!     delta          i    Volume of element
!     dldtmt        i/o   Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intrule       i     Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     kappa          i    Diffusivity
!     latentheat    i     latent heat
!     latentvc      i/o   Element latent heat vector
!     m              i    number of integration points
!     massmt        i/o   Element mass matrix
!     n              i    number of basis functions
!     phi            i    basis functions
!     rho            i    density
!     stiffmt       i/o   Element stiffness matrix
!     temperature    i    temperature vector in integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum, phicut(n,m+1), tempcut(m+1)

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     phicut         phi matrix for intersected element
!     sum            Help parameter to compute a sum
!     tempcut        Temperature in integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint1m' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint1m'
      end if

      tempcut(1) = temperature(1)
      tempcut(2) = temperature(2)
      if (intrule==2) then
         tempcut(3) = 0.5d0*(tempcut(1) + tempcut(2))
      end if

      do i = 1, n
         phicut(i, 1) = phi(i, 1)
         phicut(i, 2) = phi(i, 2)
         if (intrule==2) then

!        --- quadratic, in order to get the factor 4, we have multiplied each
!            of the following terms by a factor 2

            phicut(i, 3) = phicut(i,1) + phicut(i,2)
         end if
      end do

!     --- fill element mass matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            if (intrule==1) then
               do k = 1, m      ! integration points
                  sum = sum + rho*c*0.5d0*delta*phi(i,k)*phi(j,k)
               end do
            else if (intrule==2) then
               do k = 1, m+1      ! integration points
                  sum = sum + rho*c*delta/6d0* &
                                     phicut(i,k)*phicut(j,k)
               end do
            end if
            massmt(i,j) = massmt(i,j) + sum
         end do
      end do

!     --- fill element stiffness matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            do k = 1, m      ! integration points
               sum = sum + kappa*0.5d0*delta
            end do
            stiffmt(i,j) = stiffmt(i,j) + sum*dphidx(i,1)*dphidx(j,1)
         end do
      end do

!     --- fill element dldt matrix

      do i = 1, n
         do j = 1, n
            sum = 0d0
            if (intrule==1) then
               do k = 1, m      ! integration points
                  sum = sum + rho*latentheat*0.5d0*delta* &
                       phicut(i,k)*phicut(j,k)/(tliq - tsol)
               end do
            else if (intrule==2) then
               do k = 1, m+1      ! integration points
                  sum = sum + rho*latentheat*delta/6d0* &
                       phicut(i,k)*phicut(j,k)/(tliq - tsol)
               end do
            end if
            dldtmt(i,j) = dldtmt(i,j) + sum
         end do
      end do

!     --- fill latent heat element vector

      do i = 1, n     ! basis functions
         sum = 0d0
         if (intrule==1) then
            do k = 1, m      ! integration points
               sum = sum + rho*latentheat*0.5d0*delta*phicut(i,k)* &
                     (tempcut(k) - tsol)/(tliq-tsol)
            end do
         else if (intrule==2) then
            do k = 1, m+1      ! integration points
               if ( k==m+1) then

!              --- add *2d0 to compensate for not having phicut^2

                  sum = sum + rho*latentheat*delta/3d0* &
                    phicut(i,k)*(tempcut(k) - tsol)/(tliq-tsol)
               else
                  sum = sum + rho*latentheat*delta/6d0* &
                    phicut(i,k)*(tempcut(k) - tsol)/(tliq-tsol)
               end if
            end do
         end if
         latentvc(i) = latentvc(i) + sum
      end do

      call erclos ( 'eldiscint1m' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint1m'

      end if

      end
