      subroutine elfillqmat ( iqmat, dxdxi, m, itang, inorm, qmat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-06-2013 New definition qmat
!        version  1.0  date 24-01-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the transformation matrix qmat for the mapping of a 3d surface
!     onto a 2d surface depending on the parameter iqmat
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element
!     transformation
!     2d
!     3d
!     basis_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iqmat, m, itang, inorm
      double precision dxdxi(m,3,*), qmat(3,3,m)

!     dxdxi          i    work array of length mx3x3 to store the values of
!                         dx/dxsi, dx/deta, dy/dxsi and dy/deta according to:
!                         dx/dxsi(x_k) = dxdxi(k,1,1)
!                         dx/deta(x_k) = dxdxi(k,1,2)
!                         dy/dxsi(x_k) = dxdxi(k,2,1)
!                         dy/deta(x_k) = dxdxi(k,2,2) etc.
!     inorm          i    Indicates which vector in qmat is equal to the normal
!                         vector (iqmat = 2: 3;iqmat = 3: 1)
!     iqmat          i    Defines how the transformation matrix must be
!                         filled
!                           0: do not fill
!                           1:
!                           2:
!                           3:
!                         4-6: the first basis vector is in the e_i direction
!                              along the surface with i = iqmat-3
!     itang          i    Indicates which vector in qmat is equal to the first
!                         tangential vector (iqmat = 2: 1;iqmat = 3: 3)
!     m              i    number of integration points
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, j, idir
      double precision norm, exi(3), eeta(3), eh(3), eidir(3), ezeta(3)

!     eeta           base vector in eta direction
!     eh             help vector
!     eidir          vector in e_idir direction
!     exi            base vector in xi direction
!     ezeta          base vector in zeta direction
!     idir           Direction of base vector i
!     j              Counting variable
!     k              Counting variable
!     norm           norm of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     To compute qmat, we use the lamina coordinate system as introduced by
!     Hughes: (iqmat<6)
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      e_3(l) = e_xi x e_eta / ||e_xi x e_eta||
!      The rest is the lamina coordinate system:
!      (This part is skipped since it does not add anything usefull)
!      e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||
!      e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||
!      e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!      e_2(l) = srqt(2)/2 ( e_alpha + e_beta )
!
!     or in case iqmat>5:
!
!      idir = iqmat-5
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      n = e_xi x e_eta / ||e_xi x e_eta||
!      e_zeta is projection of e_idir onto surface:
!         e_zeta = e_idir - n(idir) n
!         e_xi = e_zeta x n / ||e_zeta x n||
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillqmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillqmat'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( iqmat<6 ) then

!     --- Standard case

         do k = 1, m

!        --- Define e_xi = x,xi / ||x,xi||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,1),dxdxi(k,1:3,1)))
            exi(:) = dxdxi(k,:,1)*norm

!           --- Define e_eta = x,eta / ||x,eta||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,2),dxdxi(k,1:3,2)))
            eeta(:) = dxdxi(k,:,2)*norm

!           --- e_3(l) = e_xi x e_eta / ||e_xi x e_eta||

            call eloutr ( exi, eeta, eh )
            norm = 1d0/sqrt(dot_product(eh,eh))
            eh = eh*norm

            if ( debug ) then
               write(irefwr,1) 'k', k
               call prinrl ( exi, 3, 'exi' )
               call prinrl ( eeta, 3, 'eeta' )
               call prinrl ( eh, 3, 'eh' )
            end if  ! ( debug )

            qmat(itang,:,k) = exi   ! t1
            qmat(2,:,k) = eeta      ! t2
            qmat(inorm,:,k) = eh    ! n

            if ( debug ) then
               write(irefwr,2) 't1', qmat(itang,1:3,k)
               write(irefwr,2) 't2', qmat(2,1:3,k)
               write(irefwr,2) 'n', qmat(3,1:3,k)
            end if  ! ( debug )

         end do  ! k= 1, m

      else

!     --- iqmat>5, special case, map onto Cartesian direction

         idir = iqmat-5
         eidir(1) = 0d0
         eidir(2) = 0d0
         eidir(3) = 0d0
         eidir(idir) = 1d0

!        --- Loop over integration points

         do k = 1, m

!        --- Define e_xi = x,xi / ||x,xi||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,1),dxdxi(k,1:3,1)))
            exi(:) = dxdxi(k,:,1)*norm

!           --- Define e_eta = x,eta / ||x,eta||

            norm = 1d0/sqrt(dot_product(dxdxi(k,1:3,2),dxdxi(k,1:3,2)))
            eeta(:) = dxdxi(k,:,2)*norm

!           --- e_3(l) = e_xi x e_eta / ||e_xi x e_eta||

            call eloutr ( exi, eeta, eh )
            norm = 1d0/sqrt(dot_product(eh,eh))
            eh = eh*norm

!           --- e_zeta = e_idir - n(idir) n

            ezeta(1) = eidir(1) - eh(idir) * eh(1)
            ezeta(2) = eidir(2) - eh(idir) * eh(2)
            ezeta(3) = eidir(3) - eh(idir) * eh(3)
            norm = 1d0/sqrt(ezeta(1)**2+ezeta(2)**2+ezeta(3)**2)
            ezeta(1) = ezeta(1)*norm
            ezeta(2) = ezeta(2)*norm
            ezeta(3) = ezeta(3)*norm

!           --- e_xi = e_zeta x n / ||e_zeta x n||

            call eloutr ( ezeta, eh, exi )
            norm = 1d0/sqrt(exi(1)**2+exi(2)**2+exi(3)**2)
            exi(1) = exi(1)*norm
            exi(2) = exi(2)*norm
            exi(3) = exi(3)*norm

!           --- Store in qmat

            do j = 1, 3
               qmat(2,j,k) = ezeta(j)  ! t1
               qmat(3,j,k) = exi(j)    ! t2
               qmat(1,j,k) = eh(j)     ! n
            end do

         end do  ! k= 1, m

      end if  ! iqmat<6

      call erclos ( 'elfillqmat' )
      if ( debug ) then

         write(irefwr,*) 'End elfillqmat'

      end if  ! ( debug )

      end
