      subroutine elflr71 ( number, user, coeff, x, xgauss, &
                           mdecl, ndecl, phi, ichois, &
                           index, iuser, ugauss, vecloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine fills function values in array coeff for the
!     integration points
!     Number corresponds to the number th coefficient in common block celiar
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     element
!     fill
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, mdecl, ndecl, ichois, index(ndecl), &
              iuser(*)
      double precision user(*), coeff(m,*), x(mdecl,*), xgauss(mdecl,*), &
                       phi(ndecl,*), ugauss(*), &
                       vecloc(numold,maxunk,*)

!     coeff   o      Double precision output array of length m in which the
!                    values of the coefficient in the integration points are
!                    stored
!     ichois  i      Choice parameter to be used when 1000<= ind(number)<2000
!                    Possible values:
!                    1:   In each node there are exactly ncomp unknowns
!                    2:   The number of unkowns per point may vary
!                         Array index contains the number of nodes per point
!                         accumulated. Hence
!                         number of unkowns in node 1 is index(1)
!                         number of unkowns in node i is index(i)-index(i-1)
!     index   i      Index array to be used if 1000<=ind(number)<2000 and
!                    ichois = 2
!     iuser   i      Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     mdecl   i      Number of integration points
!     ndecl   i      Number of nodal points
!     number  i      Parameter indicating which coefficient in common block
!                    celiar must be considered
!     phi     i      array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    array phi is only filled when gauss integration is applied
!     ugauss  i      Preceding solution in Gauss points
!     user    i      Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     vecloc  i      In this array the values of the function in the integration
!                    points is stored.
!                    vecloc(i,j,k) corresponds to the kth integration point, &
!                    the ith vector and the jth unkwon corresponding to that
!                    vector
!     x       i      Double precision array of length ndecl x ndim in which the
!                    co-ordinates of the local nodal points are stored
!                    according to:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                     i              i              i
!     xgauss  i      Double precision array of length mdecl x ndim in which the
!                    co-ordinates of the integration points are stored
!                    according to:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                     i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, istart, ifunc, ndegfd, ivec
      double precision :: work((n+1)*m)

!     i              Counting variable
!     ifunc          Output parameter of elflr8 to be used if ind(number)>4000
!                    It indicates if the parameter which with the function
!                    is multiplied is a function (>0) or a constant (<0)
!     istart         Starting address in array vecold
!     ivec           Sequence number of vector in array vecold.
!                    Hence array vecold(IVEC) is considered
!     j              Counting variable
!     ndegfd         Number of degrees of freedom per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivec
      double precision funccf, funcc1, ddot, funcc3

!     DDOT           (CDOT)     Inner product of two vectors
!     ELFLR8         Fill coefficients in array ARRAY that are equal to a
!                    preceding coefficient
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCC1         User function subroutine to compute the value of a
!                    coefficient in a point depending on a preceding solution
!     FUNCC3         User function subroutine to compute the value of a
!                    coefficient in a point depending on a series of preceding
!                    solutions
!     FUNCCF         User function subroutine to compute the value of a
!                    coefficient in a point
!     INIVEC         Return with vector number corresponding to ivecin
!     PRININLONG     print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2048   Number of integration points in old solution not equal to that in
!           element
!    2050   Vector of type 129 multiplied by function not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elflr71' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elflr71'
         write(irefwr,1) 'ifirst, number', ifirst, number
         write(irefwr,1) 'ich(number), ind(number)', &
                          ich(number), ind(number)
  1      format ( a, 1x, (10i6) )

      end if

      if ( ifirst==0 ) then

!     --- ifirst = 0, first call for this element group

         if ( ich(number)>2000 ) then

!        --- Vector of type 129, check m

            ivec = inivec ( ivecnums(number) )

            iseqtype = oldsols(ivec)%iseqtype

            index4n => indoldsols(iseqtype)%index4
            if ( index4n(2)/=m ) then
               call errint ( ivec, 1 )
               call errint ( index4n(2), 2 )
               call errint ( itype, 3 )
               call errint ( m, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 2048, 5, 0, 0 )
               go to 1000
            end if  ! ( index4n(2)/=m )

         end if  ! ( ich(number)>2000 )

      end if  ! ( ifirst==0 )

      if ( ind(number)>=10000 ) then

!     --- ind(number)>10000, coefficients are computed by funcc3

         if ( jdiag==1 ) then

!        --- jdiag = 1,  Nodal points

            do i = kstep, m, kstep
               coeff(i,number) = funcc3 ( ind(number)-10000, x(i,1), &
                        x(i,2), x(i,3), numold, maxunk, vecloc(1,1,i) )
            end do
         else

!        --- jdiag = 2,  Integration points

            do i = 1, m
               coeff(i,number) = funcc3 ( ind(number)-10000, &
                          xgauss(i,1), xgauss(i,2), xgauss(i,3), numold, &
                          maxunk, vecloc(1,1,i) )
            end do
         end if

      else if ( ind(number)==2003 .or. ind(number)==2004 ) then

!     --- ind(number)>2003, special treatment

         call elflr8 ( number, coeff(1,number), &
                       work, phi, ndecl, iuser, ifunc )
         if ( ierror/=0 ) go to 1000
         if ( ind(number)==2004 ) then

!        --- ind(number)=2004, coefficient must be multiplied

            if ( ich(number)<1000 ) then

!           --- Standard situation, one coefficient at a time

               if ( ifunc<0 ) then

!              --- ifunc<0, multiplication factor is constant

                  do i = kstep, m, kstep
                     coeff(i,number) = coeff(i,number) * user(-ifunc)
                  end do
               else

!              --- ifunc>0, multiplication factor is a function

                  if ( jdiag==1 ) then

!                 --- jdiag = 1,  Nodal points

                     do i = kstep, m, kstep
                        coeff(i,number) = coeff(i,number) * &
                               funccf ( ifunc, x(i,1), x(i,2), x(i,3) )
                     end do
                  else

!                 --- jdiag = 2,  Integration points

                     do i = 1, m
                        coeff(i,number) = coeff(i,number) * &
                             funccf ( ifunc, xgauss(i,1), xgauss(i,2), &
                             xgauss(i,3) )
                     end do
                  end if
               end if
            else if ( ich(number)<2000 ) then

!           --- Vectors of type 126, more coefficients at a time

               ndegfd = ich(number)-1000
               do j = number-1, number+ndegfd-1
                  if ( ifunc<0 ) then

!                 --- ifunc<0, multiplication factor is constant

                     do i = kstep, m, kstep
                        coeff(i,j) = coeff(i,j) * user(-ifunc)
                     end do
                  else

!                 --- ifunc>0, multiplication factor is a function

                     if ( jdiag==1 ) then

!                    --- jdiag = 1,  Nodal points

                        do i = kstep, m, kstep
                           coeff(i,j) = coeff(i,j) * funccf ( ifunc, &
                                         x(i,1), x(i,2), x(i,3) )
                        end do
                     else

!                    --- jdiag = 2,  Integration points

                        do i = 1, m
                           coeff(i,j) = coeff(i,j) * funccf ( ifunc, &
                                         xgauss(i,1), xgauss(i,2), &
                                         xgauss(i,3) )
                        end do
                     end if
                  end if
               end do
            else

!           --- The option multiply vector of type 129 by constant or function
!               has not yet been implemented

               call errsub ( 2050, 0, 0, 0 )
               go to 1000
            end if
         end if
      else if ( ist(number)>0 ) then

!     --- ist(i)>0, copy coefficient from preceding one

         do i = kstep, m, kstep
            coeff(i,number) = coeff(i,ist(number))
         end do
      else if ( ind(number)<=1000 ) then

!     --- ind(number)<1000,  coefficient is a function of space alone

         if ( debug ) write(irefwr,1) 'jdiag, m, kstep', jdiag, m, kstep
         if ( jdiag==1 ) then

!        --- jdiag = 1,  Nodal points

            do i = kstep, m, kstep
               coeff(i,number) = &
                  funccf ( ind(number), x(i,1), x(i,2), x(i,3) )
            end do
         else

!        --- jdiag = 2,  Integration points

            if ( debug ) then
               call prinrl ( xgauss(1,1), m, 'xgauss' )
               call prinrl ( xgauss(1,2), m, 'ygauss' )
            end if  ! ( debug )
            do i = 1, m
               coeff(i,number) = funccf ( ind(number), xgauss(i,1), &
                                   xgauss(i,2), xgauss(i,3) )
            end do
         end if
      else if ( ind(number)<=2000 ) then

!     --- ind(number)<2000,  coefficient is a function of space and a
!                              preceding solution

         if ( ichois==1 ) then

!        --- ichois = 1,  Constant number of degrees of freedom per point
!                         Store unknowns per point in work

            if ( jdiag==1 ) then

!           --- jdiag = 1,  Nodal points

               do i = kstep, m, kstep
                  do j = 1, ncomp
                     work(j) = ugauss(i+(j-1)*m)
                  end do
                  coeff(i,number) = funcc1 ( ind(number)-1000, x(i,1), &
                                     x(i,2), x(i,3), work )
               end do
            else

!           --- jdiag = 2,  Integration points

               do i = 1, m
                  do j = 1, ncomp
                     work(j) = ugauss(i+(j-1)*m)
                  end do
                  coeff(i,number) = funcc1 ( ind(number)-1000, &
                          xgauss(i,1), xgauss(i,2), xgauss(i,3), work )
               end do
            end if
         else

!        --- ichois = 2,  Variable number of degrees of freedom per point
!                         Store unknowns per point in work

            istart = 1
            if ( jdiag==1 ) then

!           --- jdiag = 1,  Nodal points

               do i = kstep, m, kstep
                  do j = istart, index(i)
                     work(j-istart+1) = ugauss(j)
                  end do
                  istart = istart + index(i)+1
                  coeff(i,number) = funcc1 ( ind(number)-1000, x(i,1), &
                                      x(i,2), x(i,3), work )
               end do
            else

!           --- jdiag = 2,  Integration points

               do i = kstep, m, kstep
                  do j = istart, index(i)
                     work(j-istart+1) = ugauss(j)
                  end do
                  istart = istart + index(i)+1
                  coeff(i,number) = funcc1 ( ind(number)-1000, &
                           xgauss(i,1), xgauss(i,2), xgauss(i,3), work )
               end do
            end if
         end if
      else if ( ind(number)==2001 ) then

!     --- ind(number) = 2001,  coefficient is a function of space alone
!                              it is given in all nodes

         if ( jdiag==1 ) then

!        --- jdiag = 1

            do i = kstep, m, kstep
               coeff(i,number) = user ( mst(number) + index1(i) )
            end do
            if ( inpelm/=n ) then

!           --- n # inpelm, i.e. 6-point triangle with unkowns in centroid

               coeff(7,number) = &
                  ( -coeff(1,number)-coeff(3,number)-coeff(5,number) &
                 +4d0*(coeff(2,number)+coeff(4,number)+coeff(6,number))) &
                  /9d0
            end if
         else

!        --- jdiag = 2

            do i = 1, n
               work(i) = user ( mst(number) + index1(i) )
            end do
            if ( inpelm/=n ) then

!           --- n # inpelm, i.e. 6-point triangle with unkowns in centroid

               work(7) = ( -work(1)-work(3)-work(5) &
                            +4d0 * (work(2)+work(4)+work(6)) )/9d0
            end if
            do i = 1, m
               coeff(i,number) = ddot ( n, work, 1, phi(1,i), 1 )
            end do
         end if
      else

!     --- ind(number) = 2002,  coefficient is a function of space alone
!                              it is constant per element

         do i = kstep, m, kstep
            coeff(i,number) = user ( mst(number) + irelem )
         end do
      end if
1000  if ( debug ) then
         call prinrl ( coeff(1,number), m, 'coeff' )
         write(irefwr,*) 'End elflr71'
      end if  ! ( debug )
      call erclos ( 'elflr71' )
      end
