      function eli325int ( p, w, x, diam )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 21-12-2010
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute actual special integral for type number 325
!
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     element_integral
!     integral
!     compressible
!     incompressible
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eli325int, p(m), w(m), x(m,ndim), diam

!     diam           i    Diameter of bearing
!     eli325int      o    Computed integral
!     p              i    Array containing the pressure
!                         p(1) contains the pressure in the centroid
!                         p(2-4) contains the gradient of the pressure in the
!                         centroid
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    x co-ordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision f(m), addsum

!     addsum         Help parameter to compute a sum
!     f              array of length m containing the function to be integrated
!                    in the integration points
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     Depending on icheli the following integrals are computed
!
!     40:  f_h = -/ p sin(2x/Diam)
!     41:  f_h = -/ p cos(2x/Diam)
!     42:  f_h = -/ p sin(2x/Diam) (p>=0)
!     43:  f_h = -/ p cos(2x/Diam) (p>=0)
!     44:  f_h = / x p
!     45:  f_h = / y p
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     341   Icheli has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli325int' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eli325int'
         write(irefwr,1) 'ielem, icheli', ielem, icheli
         write(irefwr,2) 'diam', diam
         call prinrl1 ( x, ndim, m, 'x' )
         call prinrl ( p, m, 'p' )
         call prinrl ( w, m, 'w' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( icheli )

         case ( 40:43 )

            if ( icheli>=42 ) then

!           --- icheli = 42, 43, make p non-negative

               p = max(p,0d0)

            end if  ! ( icheli>=42 )

            if ( icheli==40 .or. icheli==42 ) then

!           --- icheli=40 or 42:  f = sin(2x/diam)

               f = cos(2d0*x(:,1)/diam)

            else

!           --- icheli=41 or 43:  f = cos(2x/diam)

               f = sin(2d0*x(:,1)/diam)

            end if  ! ( icheli==40 .or. icheli==42 )
            if ( debug ) call prinrl ( f, m, 'f' )

         case ( 44 )

!        --- icheli=44:  f = x

            f = x(:,1)

         case ( 45 )

!        --- icheli=45:  f = y

            f = x(:,2)

         case default

!        --- Type number not available

            call errint ( icheli, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 341, 3, 0, 0 )
            go to 1000

      end select  ! case ( icheli )

      addsum = 0d0
      do i = 1, m
         addsum = addsum + w(i)*f(i)*p(i)
      end do  ! i = 1, m
      eli325int = -addsum

1000  call erclos ( 'eli325int' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'eli325int', eli325int
         write(irefwr,*) 'End eli325int'

      end if  ! ( debug )

      end
