      function eli800 ( coor2d, iuser, user, index1, islold, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.16 date 28-12-2014 Extra debug
!        version  4.15 date 22-05-2014 Extra debug
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order ellipitic equations
!     Two dimensional elements
!     ELI800 is a help subroutine for subroutine INTEGR, &
!     it is called through the intermediate subroutine eli000
!     So:
!     INTEGR
!       -  Loop over elements
!          -  ELI000
!             - ELI800
!    The subroutine is able to handle the following element shapes and cases:
!    linear line element (1D)
!    quadratic line element (1D)
!    linear triangle (2D and 3D)
!    quadratic triangle (2D and 3D)
!    bilinear quadrilateral (2D)
!    biquadratic quadrilateral (2D):
!    linear pyramid (3D)
!    trilinear brick (3D)
!    cartesian co-ordinates
!    axi-symmetric co-ordinates (2D only)
!    polar co-ordinates (1D only)
! **********************************************************************
!
!                       KEYWORDS
!
!     element_integral
!     integral
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eli800, coor2d(*), user(*), array(*)
      integer iuser(*), index1(*), islold(numold)

!     array          Real work array
!                    array consists of the following parts
!                    Starting address    length   Name:
!                    1                   27*27*3  DPHIDX
!                    ipdudx              27*3     DUDX
!                    ipphix              27*27*3  PHIKSI for elp633
!                    ipwork              27*6     Work space
!                    Total length: 4617
!                    The subarrays contain the folowing contents:
!                    DPHIDX:
!                    Array of length inpelm x m x ndim containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                    d phi / dz = dphidx (i,k,3)
!                         i
!                      in node k
!                    DUDX
!                    Contains the derivatives of the solution in the integration
!                    nodes
!                    WORK
!                    General work space
!     coor2d     i     array of length ndim x npointcac containing the co-ordinates
!                    of the nodal points with respect to the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i);
!                     i                 i                 i
!     eli800   o     Computed integral
!     index1   i     Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, kfirst, jheat, ipdudx, ipphix, &
              ipwork, itypln, jtrnsv, ipphi, lenuold, imass_mat
      parameter (lenuold=30)
      double precision qmat(3,27), x(81), w(27), xgauss(81), &
                       uold(lenuold), ugauss(lenuold), dphidx(3*27*27), &
                       tang(3*9), psi(8*27), dpsidx(3*8*27), &
                       psiksi(8*27)
      save isub, isup, jelp, kfirst, jheat, ipwork, ipdudx, &
           ipphix, ipphi

!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     i              Counting variable
!     imass_mat      dummy
!     ipdudx         Starting address in array work of dudx
!     ipphi          Starting address of array phi in array
!     ipphix         Starting address in array work of phiksi
!     ipwork         Starting address in array work of work array
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     itypln         workmy parameter
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed
!                    Possible values:
!                     0:  no mass matrix
!                     1:  diagonal mass matrix with constant coefficient
!                     2:  diagonal mass matrix with variable coefficient
!                    11:  full mass matrix with constant coefficient
!                    12:  full mass matrix with variable coefficient
!     jtrnsv         Save variable to store jtrans
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     lenuold        Declared length of arrays uold and ugauss
!     psi            Array of length n x m containing the values of the
!                    modified basis functions in the integration points
!     psiksi         Array of length 8 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision elic00, elic01

!     EL0800         Initialize parameters for subroutine ELD800
!     EL3007         Fills old solution and derivatives
!     ELCHECKCENTR   Check if the centroids in elements of shape 15 and 16
!                    are created in the right way
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIC00         Compute integrals of the shape / f(x) dx
!                                                   e
!     ELIC01         Compute integrals of the shape / f(x) dx
!                                                   e
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the parameter icheli and
!     the arrays IUSER and USER in the usual way
!     The sequence is:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Depending on icheli the following integrals are computed
!       2-5:  The input is a vector of the structure of the solution
!             i.e. 1 unknown per point
!         6:  The input is a vector of the structure of the derivative
!             vector, i.e. ndim unknowns per point
!          1:   / f(x) dx
!          2:   / f(x) u(x) dx
!          3-5: / f(x) du(x)/dx  dx   i=icheld-2
!                              i
!          6: / f(x) du      (x) dx
!                      jdegfd
!          7: / dx (i.e. compute the volume)
!          8: / f(x) u^2(x) dx
!        11-16:  See 1-6, however, the integration is performed in the
!                positive x-direction only
!        21-26:  See 1-6, however, the integration is performed in the
!                positive y-direction only
!        31-36:  See 1-6, however, the integration is performed in the
!                positive z-direction only
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli800' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eli800'
         write(irefwr,1) 'icheli, inpelm, ifirst, ielem', &
                          icheli, inpelm, ifirst, ielem
         call prinin ( index1, inpelm, 'index1' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         ix = 1
         icheld = -icheli
         call el0800 ( iuser, user, jelp, kfirst, jheat, isub, isup, &
                       array, jdegfd, itypln, imass_mat )

         if ( debug ) write(irefwr,1) 'irule, jcart', irule, jcart

!        --- Fill default value for first coefficient if not given

         if ( ind(1)==0 ) array(1:m) = 1d0

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

         if ( icheli==7 ) then
            ipdudx = 1+m
            ipphix = ipdudx
            ipphi = ipphix + 3*m*m
            ipwork = ipphi+n*m
         else
            ipdudx = 1 + 3*m*m
            ipphix = ipdudx + 3*m
            ipphi = ipphix + 3*m*m
            ipwork = ipphi+n*m
         end if

!        --- Check array length of some arrays

         call inichkarlen ( max(n,m)*ncomp, lenuold, 'uold' )

      end if  ! ( ifirst==0 )  then
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      if ( debug ) then
         write(irefwr,1) 'n, m', n, m
         write(irefwr,1) 'ipphi, ipphix', ipphi, ipphix
      end if  ! ( debug )

      w(1:m) = 0d0
      call elm800basefn ( coor2d, x, w, dphidx, xgauss, array(ipphi), &
                          index1, qmat, tang, jelp, array(ipphix), psi, &
                          dpsidx, psiksi )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( debug ) call prinin ( ind, lengthmst, 'ind' )
      if ( ind(lengthmst)==1 ) then

         jtrnsv = jtrans
         jtrans = 0
         call el3007 ( uold, array(ipdudx), array(ipphi), &
                       dphidx, m, n, ndimlc, ncomp, &
                       index1, array(ipwork), ugauss, jtrans, numold, &
                       iseqin )
         jtrans = jtrnsv

      end if

      if ( debug ) write(irefwr,*) 'after filling old solution'

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, user, array, x, xgauss, &
                          m, n, array(ipphi), 1, index1, &
                          iuser, ugauss, array(ipwork) )

         end if

      end do

!     --- Compute integral depending on icheli

      if ( mod(icheli,10)/=6 ) then

!     --- icheli = 1-5, 7, 8

         eli800 = elic00 ( mod(icheli,10), array, w, ugauss, &
                           array(ipdudx), array(ipwork) )

      else

!     --- icheli = 6

         eli800 = elic01 ( 2, jdegfd, array, w, ugauss, array(ipdudx), &
                           array(ipwork) )

      end if
      if ( debug .and. jelp>=15 .and. jelp<=16 ) then

!     --- Debug statements, check if all centroids are positioned correctly

         call elcheckcentr ( jelp, index1, coor2d )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'eli800', eli800
         write(irefwr,*) 'End eli800'

      end if  ! ( debug )
      call erclos ( 'eli800' )

      end
