      subroutine elloccoor ( x, xloc, ezeta, exi, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute local coordinates of a surface element in 3d given the
!     transformation by base vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     element
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm
      double precision x(inpelm,3), xloc(inpelm,2), ezeta(3), exi(3)

!     exi            i    base vector in xi direction
!     ezeta          i    base vector in zeta direction
!     inpelm         i    number of points per element
!     x              i    co-ordinates of vertices in 3D Cartesian system
!     xloc           o    co-ordinates of vertices in local (xi-zeta) coordinate
!                         system
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision s11, s12, s21, s22, f1, f2, det, sinv11, sinv12, &
                       sinv21, sinv22

!     det            determinant of matrix S
!     f1             first component of right-hand side vector
!     f2             second component of right-hand side vector
!     i              Counting variable
!     s11            11 element of matrix
!     s12            12 element of matrix
!     s21            21 element of matrix
!     s22            22 element of matrix
!     sinv11         11 element of inverse of matrix
!     sinv12         12 element of inverse of matrix
!     sinv21         21 element of inverse of matrix
!     sinv22         22 element of inverse of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The local coordinates are defined by:
!     x = x_1 + alpha e_zeta  beta e_xi
!     Hence x_i = x_1 _ alpha_i e_zeta  beta_i e_xi
!     and alpha_i, beta_i are the solution of
!     (e_zeta e_xi)^T (e_zeta e_xi) (alpha_i beta_i)^T = &
!        (e_zeta e_xi)^T (x_i - x_1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elloccoor' )

!     --- Compute least squares matrix

      s11 = ezeta(1)*ezeta(1)+ezeta(2)*ezeta(2)+ezeta(3)*ezeta(3)
      s12 = ezeta(1)*exi(1)+ezeta(2)*exi(2)+ezeta(3)*exi(3)
      s21 = s12
      s22 = exi(1)*exi(1)+exi(2)*exi(2)+exi(3)*exi(3)

!     --- Compute determinant

      det = 1d0/(s11*s22-s12*s21)

!     --- Compute inverse matrix

      sinv11 = s22*det
      sinv12 = -s12*det
      sinv21 = sinv12
      sinv22 = s11*det

!     --- Compute least squares right-hand side and solve system of equations

      xloc(1,1) = 0d0
      xloc(1,2) = 0d0

      do i = 2, inpelm
         f1 = ezeta(1)*(x(i,1)-x(1,1))+ &
              ezeta(2)*(x(i,2)-x(1,2))+ &
              ezeta(3)*(x(i,3)-x(1,3))
         f2 = exi(1)*(x(i,1)-x(1,1))+ &
              exi(2)*(x(i,2)-x(1,2))+ &
              exi(3)*(x(i,3)-x(1,3))
         xloc(i,1) = sinv11*f1+sinv12*f2
         xloc(i,2) = sinv21*f1+sinv22*f2
      end do

      call erclos ( 'elloccoor' )
      end
