      subroutine elm304 ( vecloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.8  date 12-01-2015 Allow string variables
!        version  2.7  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 1990-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for special point element for
!     hydrostatic bearing
!     This element represents the oil supply and the restriction for
!     a pocket with constant pressure
!     itype=304 :    point element consisting of contribution of point for
!                    pocket
!     This element is both meant for a linear relation between oil flow and
!     pressure in the pocket, or a non-linear relation
!     Linear relation:   Q = gamma ( p  - p      )
!                                     s    pcoket
!     Non-linear relation:  Q = gamma sqrt ( p  - p      )
!                                             s    pocket
!     where Q is the oil flow in the pocket,
!           gamma is the given capillary restriction coefficient,
!           p  the supply pressure and p
!            s                          pocket
!     the constant pressure in the pocket
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     element_matrix
!     element_vector
!     point_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision array, ugauss, xgauss, phi, dphidx, uold, &
                       work, dum, x
      common /celwrk/ array(3), x(2), ugauss, xgauss(2), phi, &
                      dphidx, uold, work(989), dum(500)
      save /celwrk/

!    dphidx   Not used (value of derivative of basis function)
!    dum      Dummy array
!    array    Contains the coefficients:
!             array(2): gamma
!             array(3): ps
!    phi      Not used (value of basis function)
!    uold     Solution at preceding level
!    x        Contains the co-ordinates of the point
!    xgauss   Not used (co-ordinates of Gauss points)
!    work     Work space
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*)

!     vecloc         Work array in which all old solution vectors for the
!                    integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nncode
      parameter (nncode=2)
      integer :: ncode(nncode)
      integer model, i, ishift
      double precision a, ps, gamma
      logical funchk
      character (len=10) :: code (nncode)
      save  model

!     a              Help variable to store quantity
!     code           Table with subkeywords for string parameters
!                    corresponding to type_of_bearing
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     gamma          capillary restriction coefficient
!     i              Counting variable
!     ishift         dummy
!     model          See the description
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     nncode         Number of entries in code
!     ps             the oil supply pressure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer elgetintcoef

!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELUSR3         Extracts information concerning the coefficients from iuser
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     in the usual way
!     The sequence is:
!     1:  model
!     2:  gamma  capillary restriction coefficient, &
!     3:  ps (oil supply pressure)
!     model has the following possibilities:
!     model = 0,  linear relation Q = gamma ( ps - ppocket )
!     model = 1,  non-linear relation Q = gamma sqrt ( ps - ppocket )
!                 Newton linearization
!     model = 2,  non-linear relation Q = gamma sqrt ( ps - ppocket )
!                 Picard linearization
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3   Element has incorrect number of nodal points for this type
!      42   Array E%iuser(.) has value which is too small
!    1394   The coefficient gamma is negative
!    1395   The oil pressure in the pocket at the preceding iteration level
!    1396   The oil power supply pressure is negative
!    1397   The parameter model has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Differential equation:
!         linear case:  Q = gamma ( ps - ppocket )
!                       S = gamma ps
!                       f = gamma
!        non linear case:  Q = gamma sqrt ( ps - ppocket )
!                      S = gamma/2 / sqrt ( ps - p old )
!                      f = gamma ( sqrt(ps - pold) + pold/(s sqrt(ps-pold))
!                      with pold the value at the preceding iteration level
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code  / 'LINEAR',  'NON_LINEAR' /
      data ncode  / 6, 10 /
! ======================================================================
!
      call eropen ( 'elm304' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm304'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call elcomm
         n = 1
         m = 1
         mn = 1
         ipos = E%iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
            go to 1000
         end if

!        --- Find method

         ishift = 0
         model = elgetintcoef ( E%iuser, ipos, ishift, code, nncode, &
                                ncode, 1, 'restriction'  )
         if ( model<0 .or. model>2 ) then
            call errint ( model, 1 )
            call errint ( 0, 2 )
            call errint ( 2, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1397, 5, 0, 0 )
         end if
         nuser = 0
         do i = 2, 3
            call elusr3 ( i, array(i), E%iuser, E%user, i )
         end do

!        --- Test array lengths, number of nodes and
!            number of degrees of freedom

         call prinlg ( 'E%iuser', ipos-1, E%user, E%iuser )
         call prinlg ( 'user', -nuser, E%user, E%iuser )

         if ( inpelm/=n ) then
            call errint ( inpelm, 1 )
            call errint ( n, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 3, 5, 0, 0 )
            go to 1000
         end if

!        --- icount is not used, only checked in this subroutine

         if ( icount/=1 ) then
            call errint ( icount, 1 )
            call errint ( 1, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 3, 5, 0, 0 )
            go to 1000
         end if
         if ( model>0 ) then
            ind(lengthmst) = 1
         else
            funchk = .false.
            do i = 2, 3
               if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
               if ( ind(i)>10000 ) funchk = .true.
            end do
            if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2
         end if

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, 1, 1, E%iuser )

      end if

      x(1) = coor ( 1, index1(1) )

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( uold, work, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, dum, ugauss, jtrans, &
                       numold, 1 )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, phi, n )

      do i = 2, 3
         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, array, x, xgauss, &
                          m, n, phi, 1, index1, &
                          E%iuser, ugauss, vecloc )

         end if
      end do

!     --- Compute element matrix and vector, depending on model
!         Next compute gamma from p1 and beta

      gamma = array(2)
      ps    = array(3)

      if ( debug ) then
         write(irefwr,1)'model', model
         write(irefwr,2)'gamma, ps', gamma, ps
      end if  ! ( debug )

!     --- Check the oil pressure supply and gamma

      if ( ps<=0d0 ) then
         call erreal ( ps, 1 )
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 1396, 3, 1, 0 )
         go to 1000
      end if
      if ( gamma<0d0 ) then
         call erreal ( gamma, 1 )
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 1394, 3, 1, 0 )
         go to 1000
      end if
      if ( model==0 ) then

!     --- linear case

         if ( matrix ) elemmt(1,1) = gamma
         if ( vector ) elemvc(1) = ps * gamma

      else

!     --- model = 1, 2, non-linear relation

         if ( uold>ps ) then
            call erreal ( uold, 1 )
            call erreal ( ps, 2 )
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errwar ( 1395, 3, 2, 0 )
            uold = 0.99d0*ps
         end if
         a =  sqrt ( ps - uold )

         if ( model==1 ) then

!        --- model = 1: Newton

            if ( matrix ) elemmt(1,1) = 0.5d0 * gamma / a
            if ( vector ) elemvc(1) = gamma * ( a + 0.5d0*uold/a )

         else

!        --- model = 2: Picard

            elemvc(1) = gamma*a
            elemmt(1,1) = 0d0

         end if  ! ( model==1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'elemmt', elemmt(1,1)
         write(irefwr,2) 'elemvc', elemvc(1)
         write(irefwr,*) 'End elm304'

      end if  ! ( debug )
      call erclos ( 'elm304' )

      end
