      subroutine elm601 ( swork )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.3  date 09-03-2013 Replace elflr7 by elflr71
!        version  3.2  date 12-12-2012 Long integers
!
!   copyright (c) 1995-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for boundary conditions for
!     second order elliptic equations
!     One, two and three dimensional elements
!     ELM601 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine el0000
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  EL0000
!             - ELM601
!    The subroutine is able to handle the following element shapes and cases:
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision swork(*)

!     swork          Spectral work array
!                    total length is: 2*m*m + m*(5 + ndim)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, jboun, iparry, ipxref, ipx, ipwref, &
              ipw, ipold, ipwork, ipd
      double precision rdum(1), transcalc
      save iparry, ipxref, ipx, ipwref, ipw, ipold, ipwork, ipd, isub, &
           isup, jelp, jboun, transcalc

!     i          Counting variable
!     iparry     indicates array in which the coefficients are stored
!                length m
!     ipd        indicates the derivative of the basic functions, m*m
!     ipold      indicates the old solution in the nodal points, length
!                m x 1
!     ipw        indicates the corresponding weights in the element
!                length m
!     ipwork     work array, lenght m*m
!     ipwref     indicates the Legendre Gauss-Lobatto weights for
!                numerical integration corresponding to the reference
!                element, length m
!     ipx        indicates the corresponding points in the element, &
!                length m x ndim
!     ipxref     indicates the Legendre Gauss-Lobatto points corresponding
!                to the reference element [-1,1], length m
!     isub       Integer help parameter for do-loops to indicate the
!                smallest coefficient number that is dependent on space
!     isup       Integer help parameter for do-loops to indicate the
!                largest coefficient number that is dependent on space
!     jboun      Defines the type of boundary condition.
!                Possible values:
!                2:  Standard mixed (Robbins) boundary condition
!                3:  Boundary condition of type 3
!                4:  Boundary condition of type 4 (not yet available)
!     jelp       Choice parameter to distinguish between the types of
!                element subroutines to be called. Possible values:
!                 0:  Point element, no special basis functions
!                 1:  Subroutine ELP615  (Linear line element)
!                 2:  Subroutine ELP616  (Quadratic line element)
!                 3:  Subroutine ELP625  (Linear triangle)
!                 4:  Subroutine ELP627  (Quadratic triangle)
!                 5:  Subroutine ELP625  (Bilinear quadrilateral)
!                 6:  Subroutine ELP628  (Biquadratic quadrilateral)
!     rdum       dummy variable
!     transcalc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0601    Initialise parameters for subroutine ELM601
!     EL3007    Fills old solution and derivatives
!     EL6000    Performs initialization for subroutines ELx600
!     EL6002    compute the spectral element Helmholtz matrix
!     ELFLR71   Fill variable coefficients in array ARRAY
!     ELIND0    Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP644    compute the integration points, weights and derivatives
!               (element dependent)
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Type of natural boundary condition
!         Possible values:
!         0,2: natural boundary condition of type 2
!         3: natural boundary condition of type 3
!         4: natural boundary condition of type 4
!         5: natural boundary condition of type 5
!     2:  Not yet in use:  set equal to 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  Not yet in use:  set equal to 0
!     6:  sigma
!     7:  h
!     8:  alpha 11  (1D/2D/3D)  (The parameters 8-13 are only used for
!     9:  alpha 12  (2D/3D)      boundary conditions of type 4)
!    10:  alpha 13  (3D)
!    11:  alpha 22  (2D/3D)
!    12:  alpha 23  (3D)
!    13:  alpha 33  (3D)
!    14-15: Not yet in use (must be zero)
!    In case of boundary conditions of type 5:
!     6:  u1
!     7:  u2
!     8:  u3
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1682   Option not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Boundary conditions:
!       Type 2:   sum  sum alpha   dc/dx n  + sigma c = h
!                  i    j       ij      j i
!       Type 3:   sum  sum alpha   dc/dx n  + sigma dc/dt = h
!                  i    j       ij      j i
!       Type 4:   dc/dn + sigma dc/dt = h
!       Type 5:   special for convective terms of compressible type
!                 See 2: h=0, sigma = u.n
!        The following formulae are used:
!       Type 2:  S(i,j) = /  sigma phi  phi  de
!                         e           i    j
!                f(i)   = /  h phi  de
!                         e       i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm601' )
      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         iparry = 1
         call  el0601 ( E%iuser, E%user, jelp, jboun, isub, isup, &
                        swork(iparry) )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

!        --- initialize pointers in the work array

         ipxref   = iparry + 8*m
         ipx      = ipxref + m
         ipwref   = ipx    + m*ndim
         ipw      = ipwref + m
         ipold    = ipw    + m
         ipwork   = ipold  + m*m
         ipd      = ipwork + m*m

!        --- compute points and weights for numerical integration
!            corresponding to the reference element

         call elp640(1, swork(ipxref), swork(ipwref), swork(ipd), &
                     swork(ipwork), m)

      end if

!     --- compute basis functions and weights for numerical integration

      call elp644( coor, index1, swork(ipx), swork(ipwref), &
                  swork(ipw), swork(ipd), m)

!     --- compute transformation parameter

      transcalc = swork(ipw+1)/swork(ipwref+1)

      if ( jelp==0 ) then

!     --- Special case of point element

          call errchr ( 'Special case of point element', 1 )
          call errsub ( 1682, 0, 0, 1 )
          go to 1000

      end if

      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( swork(ipold), swork(ipwork), rdum, &
                       rdum, m, m, ndim, ncomp, index1, &
                       rdum, swork(ipold), jtrans, numold, 1 )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71(i, E%user, swork(iparry), &
                        swork(ipx), swork(ipx), m, m, rdum, &
                         1, index1, E%iuser, swork(ipold), swork )
         end if

      end do

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         jadd = 0
         if ( jboun==2 ) then

!        --- Boundary conditions of type 2

            call el6002(swork(iparry), E%mat, swork(ipwref), m, &
                 transcalc)

         else if ( jboun==3 ) then

!        --- Boundary conditions of type 3

            call errchr ( 'Boundary conditions of type 3', 1 )
            call errsub ( 1682, 0, 0, 1 )
            go to 1000

         else if ( jboun==5 ) then

!        --- Boundary conditions of type 5

            call errchr ( 'Boundary conditions of type 5', 1 )
            call errsub ( 1682, 0, 0, 1 )
            go to 1000

         end if

      end if

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         jadd = 0
         call el6000 ( swork(iparry+1*m), E%vec, &
                       swork(ipwref), transcalc, m )
      end if

1000  call erclos ( 'elm601' )

      end
