      subroutine elm650
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.2   date   03-07-1992 replace btamin1b by btamin
!        version 1.1   date   20-03-1991 adjusted for NDIM = 3
!        version 1.0.  date   01-11-1990
!
!
!   copyright (c) 1990,1992  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!     Fill element matrix and vector for potential flow equations
!     Two dimensional mixed-hybrid elements
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      implicit none
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS
!
! ********************************************************************
!
!               LOCAL PARAMETERS

      double precision A(100), det, Amin1(100), Bvec(10), Amin1B(10), &
                       btamin(1), F2(10), vhlp, x(56)
      integer i

!     Meaning of local parameters
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     A           real matrix: A  = / (C v ) . v  dx, C is the inverse of the
!                               ij  S     i     j
!                 hydraulic conductivity tensor, v are the basis functions
!
!                                         -1
!     Amin1       real matrix containing A
!
!                                         -1
!     Amin1B      real vector containing A  B
!
!     Bvec        real vector: B = - / nabla . v dx = -1
!                               i    S          i
!                                T -1
!     btamin      real equal to B A  B
!
!     det         real, determinant of the mapping to a reference triangle
!
!     F2          real vector: F  = - / f dx = f * opp(S)
!                               2     S
!
!     i           loop variables
!
!     vhlp        temporary variable for putting the elements in the correct
!                 order
!
!     x           real array containing the coordinates of the midpoints of the
!                 edges of the element
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!        the matrix D is stored in E%mat and the rhs F is stored in E%vec
!
! ********************************************************************
!
!                  USER INPUT
!
!     The user provides the coefficients through the arrays E%iuser and USER
!     in the usual way
!     The sequence is:
!
!     ndim = 2                           ndim = 3
!     1:  alpha 11                       1:  alpha 11
!     2:  alpha 12                       2:  alpha 12
!     3:  alpha 22                       3:  alpha 31
!                                        4:  alpha 22
!                                        5:  alpha 23
!                                        6:  alpha 33
!     4:  u 1  ( = 0 )                   7:  u 1  ( = 0 )
!     5:  u 2  ( = 0 )                   8:  u 2  ( = 0 )
!     6:  beta ( = 0 )                   9:  beta ( = 0 )
!     7:  f                             10:  f
!     8:  rho cp ( not used )
!     8/9:  method for upwinding (8 if no heat equation must be computed, &
!                                 otherwise 9 )
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!   ELM651   fills x
!   ELM652   computes A
!   ELM653   computes the inverse of A
!   ELM654   fills B (with -1s)
!   ELM655   performs matrix multiplications
!   ELM656   computes the element matrix
!   ELM657   computes F2
!   ELM658   computes the element vector
!
! ********************************************************************
!
      call eropen ( 'elm650' )

      call elm651 ( coor, inpelm, index1, x, ndim )

!     *  First compute A

      call elm652 ( ielgrp, E%user, E%iuser, x, inpelm, A, det, ndim )

!     *  Next compute inverse(A)

      call elm653 ( A, icount, Amin1 )

!                 -1
!     *  Compute A  B

      call elm654 ( icount, Bvec )
      call elm655 ( Amin1, icount, icount, Bvec, icount, 1, Amin1B )

!     *  Fill parts of matrix if necessary

      if ( matrix ) then

!        matrix = true    compute element matrix

!                    -1   -1   T -1  -1  -1  T
!        *  Compute A  - A  B(B A  B)  (A  B)

         call elm655 ( Bvec, 1, icount, Amin1B, icount, 1, btamin )
         call elm656 ( icount, Amin1, Amin1B, btamin(1), E%mat )

         if ( ndim.eq.3 ) then
            do 10 i = 1, 4
               vhlp = E%mat(1,i)
               E%mat(1,i) = E%mat(4,i)
               E%mat(4,i) = E%mat(2,i)
               E%mat(2,i) = E%mat(3,i)
               E%mat(3,i) = vhlp
10          continue
            do 20 i = 1, 4
               vhlp = E%mat(i,1)
               E%mat(i,1) = E%mat(i,4)
               E%mat(i,4) = E%mat(i,2)
               E%mat(i,2) = E%mat(i,3)
               E%mat(i,3) = vhlp
20          continue
         end if

      end if

!    *   Fill element vector if necessary

      if ( vector) then

!      *  vector = true, compute element vector
!
!                  -1   T -1  -1
!      *  Compute A  B(B A  B)  F
!                                2
!
!      *  F  = - rhsf ( x, y ) * opp ( e )
!          2

         call elm657 ( ielgrp, E%iuser, E%user, det, icount, x, F2 )
         call elm658 ( icount, Amin1B, btamin(1), F2, E%vec )

         if ( ndim.eq.3 ) then
            vhlp = E%vec(1)
            E%vec(1) = E%vec(4)
            E%vec(4) = E%vec(2)
            E%vec(2) = E%vec(3)
            E%vec(3) = vhlp
         end if

      end if

      call erclos ( 'elm650' )

      end
