      function elm662 ( i, k, j, l, ndim )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-20 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the basis function is taken to be of the form: (v ) = a x + b y + cz + d
!                                                    i j
!   the integral of v v  over a reference element is computed
!                    i j
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer i, j, k, l, ndim
      double precision elm662

!
!     elm662   o     the returned function value
!
!     ndim     i     the dimension of space
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     i        i     number of basisfunction
!
!     j        i     number of basisfunction
!                                                              th
!     k        i     number of element of basis function, the k   element of
!                         th
!                    the i   basis function is taken
!                                                              th
!     l        i     number of element of basis function, the l   element of
!                         th
!                    the j   basis function is taken
!
! *********************************************************************
!
!               LOCAL PARAMETERS
!
      double precision a, b, c, d, e, f, g, h

!
!     a        real, coefficient of x
!
!     b        real, coefficient of y
!
!     c        real, coefficient of z
!
!                         th                 th
!     d        real, the k   element of the i   basis function is
!              equal to : (v )  = ax + by + cz + d
!                           i k
!
!     e        real, coefficient of x
!
!     f        real, coefficient of y
!
!     g        real, coefficient of z
!
!                         th                 th
!     h        real, the l   element of the j   basis function is
!              equal to : (v )  = ex + fy + gz + h
!                           j l
!
! ********************************************************************
!
!                              METHOD
!    ndim = 2
!          1    1 - x
!          /      /    ( ax + by + d ) ( ex + fy + h ) dy dx = &
!        x = 0  y = 0
!
!          = a (e/12 + f/24 + h/6) + b (e/24 + f/12 + h/6) + d (e/6 + f/6 + h/2)
!
!    ndim = 3
!     1     1-x   1-x-y
!     /      /      /   ( ax + by + cz + d ) ( ex + fy + gz + h ) dz dy dx = &
!   x = 0  y = 0  z = 0
!
!     = a ( e/60  + f/120 + g/120 + h/24 ) + &
!       a ( e/120 + f/60  + g/120 + h/24 ) + &
!       a ( e/120 + f/120 + g/180 + h/24 ) + &
!       a ( e/24  + f/24  + g/24  + h/6 )
!
! *******************************************************************
!
!               SUBROUTINES CALLED
!
!     ELM663   get the coefficients of the basis function
!
! ********************************************************************

      call eropen ( 'elm662' )

      call elm663 ( i, k, a, b, c, d, ndim )
      call elm663 ( j, l, e, f, g, h, ndim )
      if ( ndim.eq.2 ) then
         elm662 = a * ( e/12d0 + f/24d0 + h/6d0 ) + &
                  b * ( e/24d0 + f/12d0 + h/6d0 ) + &
                  d * ( e/6d0  + f/6d0  + h/2d0 )
      else
         elm662 = a * ( e/60d0  + f/120d0 + g/120d0 + h/24d0 ) + &
                  b * ( e/120d0 + f/60d0  + g/120d0 + h/24d0 ) + &
                  c * ( e/120d0 + f/120d0 + g/60d0  + h/24d0 ) + &
                  d * ( e/24d0  + f/24d0  + g/24d0  + h/6d0 )
      end if

      call erclos ( 'elm662' )

      end
