      subroutine elm800conv ( elemmt, w, dphidx, psiupw, phi, xgauss, &
                              symm, metup, uvel, kfirst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.1  date 26-08-2011 Extra debug
!        version  2.0  date 04-12-2009 Remove two parameters
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element matrix for convective terms
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer metup, kfirst
      double precision elemmt(icount**2,*), w(*), uvel(m,*), &
                       dphidx(n,m,*), psiupw(*), &
                       xgauss(*),  phi(*)
      logical symm

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemmt         o    Element matrix
!     kfirst         i    First user point of a curve
!     metup          i    Upwind parameter
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     symm           i    Indication whether the resulting matrix is symmetrical
!                         (true) or not (false)
!     uvel           i    Contains the velocity vector
!     w              i    array of length m containing the weights for
!                         integration
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih
      double precision wrk1(m,ndim)

!     ih             Help variable to store a constant temporarily
!     wrk1           work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3001         Compute integrals of the shape / u(x).grad phi phi (x) dx
!                                                   e -            j   i
!     EL3012         Compute correction to convective terms in case of
!                    upwinding and compressible flow
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800conv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800conv'
         write(irefwr,1) 'metup, metupw, kfirst, idim, idefcr', &
                          metup, metupw, kfirst, idim, idefcr
         call prinrl1 ( uvel, ndim, m, 'uvel' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      symm = .false.
      if ( metup==0 .or. metupw>=1000 ) then

         call el3001 ( uvel(1,kfirst), elemmt, w, &
                       phi, dphidx(1,1,kfirst), idim, n, wrk1 )
         if ( metup>0 ) &
             call el3012 ( uvel(1,kfirst), elemmt, w, phi, psiupw, &
                           dphidx(1,1,kfirst), idim, n, wrk1, &
                           xgauss )

      else

         ih = jdiag
         jdiag = 2
         call el3001 ( uvel(1,kfirst), elemmt, w, psiupw, &
                       dphidx(1,1,kfirst), idim, n, wrk1 )
         jdiag = ih

      end if
      if ( idefcr==1 ) then

!     --- Build central difference matrix in elemmt(.,2)

         call el3001 ( uvel(1,kfirst), elemmt(1,2), w, phi, &
                       dphidx(1,1,kfirst), idim, n, wrk1 )

      end if
      jadd = 1

1000  call erclos ( 'elm800conv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm800conv'

      end if  ! ( debug )

      end
