      subroutine elm800matrix ( jelp, alpha, elemmt, w, dphidx, &
                                psiupw, phi, coor, index1, &
                                iuser, user, vecloc, &
                                second, phixi, x, xgauss, symm, &
                                metup, uvel, kfirst, metupc, &
                                beta, itypln, dudx, idefcr, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 12-12-2012 Long integers
!        version  2.1  date 02-11-2011 Extra debug
!        version  2.0  date 20-10-2010 Remove some parameters from list
!
!   copyright (c) 2006-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for elements of type 800, i.e.
!     general second order elliptic equation with one degree of freedom
!     per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp, index1(*), &
              iuser(*), metup, kfirst, metupc, &
              itypln, idefcr, ndim
      double precision elemmt(*), w(*), uvel(m,*), beta(*), &
                       dphidx(n,m,*), psiupw(*), coor(*), user(*), &
                       x(*), xgauss(*), vecloc(*), &
                       dudx(*), alpha(m,*), phi(*), phixi(*)
      logical second, symm

!     alpha          i    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     beta           i    array of length m containing the values of the
!                         third coefficient in the integration points
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     elemmt         o    Element matrix
!     idefcr         i    If 1 defect correction is applied, if 0 not
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     itypln         i    Indicates the type of lineariation to be performed
!                         Possible values:
!                         Type 800:
!                               1: Standard equation
!                               2: convective term is |ux| + |uy| + |uz|
!                               3: convective term is g(grad(c)), &
!                                  Newton linearization
!                               4-6: special convection equation for free
!                                  surfaces
!                         Type 803:
!                               1: Picard
!                               2: Newton
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     kfirst         i    First user point of a curve
!     metup          i    Upwind parameter
!     metupc         i    If 0 the complete upwinding must be taken into account
!                         otherwise (1) only the convective terms
!     ndim           i    Dimension of the space.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i    array of length n x m to store the values of dphi/dxsi
!                         according to:
!                                     k
!                         dphi /dxsi(x ) = phixi(i,k)
!                             i
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     second         i    Indicates if second derivative term is important in
!                         case of upwinding
!     symm           i    Indication whether the resulting matrix is symmetrical
!                         (true) or not (false)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     uvel           i    Contains the velocity vector
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    co-ordinates of points in element
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL1011         Compute extra terms for element matrix in case of Newton
!                    linearization of special non-linear equation
!     ELM800CONS     Fill element matrix for zeroth order part of PDE
!     ELM800CONV     Compute element matrix due to convection term
!     ELM800DIF      Compute element matrix due to diffusion term
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800matrix' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800matrix'
         write(irefwr,1) 'jsecnd, idim, jzero', jsecnd, idim, jzero
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jadd = 0
      if ( metupc==1 ) second = .false.

      if ( jsecnd>0 ) then

!     --- First second derivative part (Diffusive terms)

         call elm800dif ( jelp, alpha, elemmt, w, dphidx, &
                          psiupw, phi, coor, index1, &
                          iuser, user, vecloc, &
                          second, phixi, x, xgauss )
         if ( debug ) &
            call prinrl1 ( elemmt, icount, icount, 'elemmt/dif' )

      end if
      if ( ierror/=0 ) go to 1000

      if ( idim>0 ) then

!     --- Next first derivative part  (convective terms)

         call elm800conv ( elemmt, w, dphidx, psiupw, phi, xgauss, &
                           symm, metup, uvel, kfirst )
         if ( debug ) &
            call prinrl1 ( elemmt, icount, icount, 'elemmt/conv' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Finally constant part

      if ( jzero>0 ) then

         call elm800cons ( metup, metupc, w, phi, elemmt, &
                           elemmt(n**2+1), idefcr, beta, &
                           psiupw, elemmt(n**2+1) )

      end if

      if ( itype==803 .and. itypln==2 ) then

!     --- itype = 803, Newton linearization, compute extra term

         call el1011 ( elemmt, w, alpha(1,2), alpha(1,3), dudx, dphidx, &
                       ndim )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm800matrix'

      end if  ! ( debug )
      call erclos ( 'elm800matrix' )

      end
