      subroutine elm800rhsd ( itypln, wrk1, dudx, uvel, &
                              gextra, frhsd, ndim, elemvc, metupc, &
                              metup, phi, alpha, idefcr, dphidx, &
                              psiupw, ishape, qmat, w, gamma )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 10-02-2015 Extra debug
!        version  1.4  date 09-03-2013 Correction declaration
!        version  1.3  date 07-11-2006 New call to el1009
!
!   copyright (c) 2006-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for elements of type 800, i.e.
!     general second order elliptic equation with one degree of freedom
!     per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypln, ndim, metupc, metup, idefcr, ishape
      double precision wrk1(m,*), dudx(*), &
                       elemvc(icount,*), dphidx(*), psiupw(*), &
                       qmat(*), w(*), frhsd(*), gamma(m,*), &
                       gextra(*), uvel(*), alpha(m,*), phi(*)

!     alpha          i    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     elemvc         o    Element vector
!     frhsd          i    Contains f in integration points
!     gamma          i    Array of size m x ndim containing the coefficients
!                         gamma_i in the integration points
!     gextra         i    Contains gextra in integration points
!     idefcr         i    If 1 defect correction is applied, if 0 not
!     ishape         i    Shape number of elements
!     itypln         i    Indicates the type of lineariation to be performed
!                         Possible values:
!                         Type 800:
!                            1: Standard equation
!                            2: convective term is |ux| + |uy| + |uz|
!                            3: convective term is g(grad(c)), &
!                               Newton linearization
!                            4-6: special convection equation for free surfaces
!                         Type 803:
!                            1: Picard
!                            2: Newton
!     metup          i    Upwind parameter
!     metupc         i    If 0 the complete upwinding must be taken into account
!                         otherwise (1) only the convective terms
!     ndim           i    Dimension of the space.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psiupw         i    Array of size n*m containing the adapted basis
!                         functions for upwind
!     qmat           i    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     uvel           i    Contains the velocity vector
!     w              i    array of length m containing the weights for
!                         integration
!     wrk1                work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih
      logical extra_rhs

!     extra_rhs      if true the extra right-hand side due to div(gamma)
!                    must be added
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     EL1003         Compute new right-hand side function in case of non-linear
!                    convection with Newton
!     EL1009         Compute integrals of the shape / u(x).grad phi dx
!                                                   e -            i
!     EL1020         Compute extra terms for element vector in case of Newton
!                    linearization of special non-linear equation
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     ELFMULTTANG    Multiply the right-hand side by (e_zeta,e_dim)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800rhsd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800rhsd'
         write(irefwr,1) 'itypln, n, m', itypln, n, m
         write(irefwr,1) 'metup, metupc', metup, metupc
         call prinrl ( frhsd, m, 'frhsd' )
         call prinrl ( w, m, 'w' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jadd = 0
      if ( itypln==3 ) then

!     --- itype = 800, Non-linear convection
!                      Newton linearization, add extra term to rhs
!                      First copy f into wrk1

         do i = 1, m
            wrk1(i,1) = frhsd(i)
         end do
         call el1003 ( dudx, uvel, wrk1, gextra, frhsd, ndim )

      else if ( itypln>3 ) then

!     --- Special case, multiply f by (e_zeta,e_z)

         call elfmulttang ( frhsd, frhsd, qmat, itypln-3, m, ishape )

      end if

      if ( metup==0 .or. metupc==1 ) then

!     --- No upwind required

         call el3003 ( frhsd, elemvc, w, phi, m, n )

      else

!     --- Upwinding, multiply by psiupw instead of phi

         ih = jdiag
         jdiag = 2
         call el3003 ( frhsd, elemvc, w, psiupw, m, n )
         jdiag = ih

      end if

      if ( itype==803 .and. itypln==2 ) then

!     --- itype = 803, Newton linearization, compute extra term

         call el1020 ( elemvc, w, alpha(1,2), alpha(1,3), dudx, &
                       dphidx, ndim )

      else if ( itypln==3 ) then

!     --- itypln = 3, reset f

         do i = 1, m
            frhsd(i) = wrk1(i,1)
         end do

      end if
      extra_rhs = ind(13)/=0 .or. ind(14)/=0 .or. ind(15)/=0
      if ( itype==800 .and. extra_rhs ) then

!     --- type 800, with extra term defined by gamma
!         Subtract extra function from elemvc

         jadd = 1
         call el1009 ( gamma, elemvc, w, dphidx, ndim, -1d0 )

      end if  ! ( itype==800 .and. extra_rhs )

      if ( idefcr==1 ) &
         call dcopy ( icount, elemvc(1,1), 1, elemvc(1,2), 1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc, icount, 'elemvc' )
         write(irefwr,*) 'End elm800rhsd'

      end if  ! ( debug )
      call erclos ( 'elm800rhsd' )

      end
