      subroutine elm807 ( vecloc, coeffarrlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 12-08-2013 Extra parameters
!        version  2.5  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for enthalpy equation (Stefan problem)
!     solved by the Newton method of Fachinotti et al
!     One, two and three dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduletime
      use sepmoduleelem
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), coeffarrlc(*)

!     coeffarrlc            Real work array which must be kept for each element
!                         group
!                         work consists of the following parts
!                         Starting address    length   Name:
!                         1                   13*8     array containing the
!                                                      coefficients
!                         ipdphx              8*8*3    DPHIDX
!                         ippsi               8*8      PSI
!                         ipphixi            3*8*8    phixi
!                         ipphixicp          3*8*8    copy of phixi
!                         array contains the folowing contents:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                          1:   k_solid
!                          2:   k_fluid
!                          3:   rho_solid
!                          4:   c_solid
!                          5:   c_fluid
!                          6:   Q
!                          7:   L
!                          8:   T_melt
!                          9:   rho_liquid
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, iseq, ipalpha, &
              ktrans, iprho, iprhocp, ipphixicp, &
              ipend, intdiscrule, iprho_fluid, &
              ipcp, ipc_fluid, ipc_solid, ipkappa_fluid, ipkappa_solid, &
              iplatent_heat, ipmeltemp, iprhs, j, neg, pos, iprho_solid
      double precision eps, &
                       psiupw(inpelm*icount), &
                       dudx(3*27*icount), &
                       wrk1(icount*inpelm+inpelm*7), &
                       tempold(icount), ugauss(27), &
                       latent_heat, tmelt, lvector(icount), &
                       dldt(icount,icount), stiffness(icount,icount), &
                       massmt(icount,icount), fvec(icount)
      logical second, fluid, solid
      save isub, isup, jelp, ipalpha, latent_heat, &
           ipphixicp, iprhocp, &
           ipcp, ipc_fluid, ipc_solid, ipkappa_fluid, ipkappa_solid, &
           iplatent_heat, ipmeltemp, iprhs, intdiscrule, iprho_solid, &
           iprho_fluid

!     dldt           Contains the matrix of the derivative of latent heat with
!                    respect to temperature
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     eps            Local accuracy
!     fluid          If true some nodes are above melting temperature
!     fvec           Contains the load vector per element
!     i              Counting variable
!     intdiscrule    Numerical integration rule to be used for
!                    discontinuous integration
!                    See eldiscint
!     ipalpha        Starting position of diffusion tensor in coeffarrlc
!     ipc_fluid      Starting position of cp in fluid part in coeffarrlc
!     ipc_solid      Starting position of cp in solid part in coeffarrlc
!     ipcp           Starting position of cp in coeffarrlc
!     ipend          Last position in array
!     ipkappa_fluid  Starting position of kappa in fluid part in coeffarrlc
!     ipkappa_solid  Starting position of kappa in solid part in coeffarrlc
!     iplatent_heat  Starting position of latent_heat in coeffarrlc
!     ipmeltemp      Starting position of melting temperature in coeffarrlc
!     ipphixicp      Starting position of parameter phixicp in coeffarrlc
!     iprho          Starting position of parameter rho in coeffarrlc
!     iprhocp        Starting position of rhocp in coeffarrlc
!     iprho_fluid    Starting position of rhofluid in coeffarrlc
!     iprho_solid    Starting position of rhosolid in coeffarrlc
!     iprhs          Starting position of rhight-hand side in coeffarrlc
!     iseq           Dummy not used
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called.
!     ktrans         Help variable to save the value of jtrans
!     latent_heat    value of latent heat
!     lvector        used to store the conribution of the latent heat to the
!                    element vector
!     massmt         Contains the element mass matrix
!     neg            Number of nodes smaller then Tm/Tsol
!     pos            Number of nodes larger then Tm/Tliq
!     psiupw         Array of size n*m containing the adapted basis functions
!                    for upwind
!     second         Indicates if second derivative term is important in case of
!                    upwinding
!     solid          If true there nodes with temperature below melting
!                    temperature
!     stiffness      Is used to store the diffusion part of the element matrix
!     tempold        Array containing the previous iteration (temperature)
!     tmelt          Melting temperature
!     ugauss         Preceding solution in Gauss points
!     wrk1           Work array to be used by some under subroutines
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0807         performs the initializations for subroutine ELM807
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3009         Fill series of old solutions in array vecloc
!     EL807RHS       Compute right-hand side vectors for enthalpy equation
!                    solved by Newton method according to Fachinotti et al
!     ELDISCINT      Computes the element matrices (mass, stiffness and dLdT)
!                    for elements that are intersected by the free boundary
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ELM800DIF      Compute element matrix due to diffusion term
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Not yet used 0
!     2:  Not yet used 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         jcart:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-orindates
!     5:  0  not yet used
!     6:  kappa_solid
!     7:  kappa_fluid
!     8:  rho_solid
!     9:  c_solid
!    10:  c_fuid
!    11:  Q              (source term)
!    12:  L              (latent heat)
!    13:  T_sol          (melting temperature)
!    14:  rho_solid, if zero rho_solid is used
!    15:  Not yet used (must be 0)
!    16:  Not yet used (must be 0)
!    17:  Not yet used (must be 0)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm807' )

      debug = .false.

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         ix = 0
         call  el0807 ( E%iuser, E%user, jelp, isub, isup, coeffarrlc, iseq, &
                        intdiscrule )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

         jderiv = 1

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

         ipkappa_solid = 1
         ipkappa_fluid = ipkappa_solid+m
         iprho_solid = ipkappa_fluid+m
         ipc_solid =  iprho_solid+m
         ipc_fluid =  ipc_solid+m
         iprhs = ipc_fluid+m
         iplatent_heat = iprhs+m
         ipmeltemp = iplatent_heat+m
         iprho_fluid = ipmeltemp+m
         iprhocp = iprho_fluid+m
         ipphixicp = iprhocp+m  ! work array to store copy of phixi
                                         ! of length 3*8*8
         ipend =  ipphixicp+ndim*n*m
         if ( ipend>12285 ) then

!        --- array wrk1 too small

            call errint ( ipend, 1 )
            call errint ( 12285, 2 )
            call errchr ( 'coeffarrlc', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if
         latent_heat = coeffarrlc(iplatent_heat)
         if ( ind(9)==0 ) iprho_fluid = iprho_solid  ! rho is independent of
                                                     ! state

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!     --- Fill variable coefficients if necessary
!         First fill tempold and dudx if necessary

         ktrans = jtrans
         jtrans = 0
         call el2005 ( tempold, dudx, B%phi, B%phider, &
                       E%nodes, ugauss, iseqin )
         jtrans = ktrans

      end if

!     --- Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, B%phi, n )

!     --- Fill variable coefficients if necessary

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, coeffarrlc, B%xyz, &
                          B%xyzg, m, n, B%phi, &
                          1, E%nodes, E%iuser, ugauss, vecloc )

         end if

      end do
      if ( ierror/=0 ) go to 1000

!     --- Detect if element is solid, fluid or both

      tmelt = coeffarrlc(1+7*m)
      fluid = .false.
      solid = .false.
      neg = 0
      pos = 0
      eps = 1d-6

      if ( mushy ) then
         if ( abs(tmelt)<eps) then
            tsol = -tdelta
            tliq = tdelta
         else
            tsol = (1d0 - tdelta)*tmelt
            tliq = (1d0 + tdelta)*tmelt
         end if

         do i = 1, n
            if (debug) print *, 'temp', tempold(i)
            if ( tempold(i)< tsol ) then

!           --- temperature below solidification temperature

               neg = neg + 1

            else if ( tempold(i)>tliq ) then

!           --- temperature above liquification temperature

               pos = pos + 1

            end if  ! ( tempold(i)<tmelt )

         end do  ! i = 1, n

      else

         do i = 1, n

            if ( tempold(i)<tmelt ) then

!           --- temperature below melting temperature

               solid = .true.

            else

!           --- temperature above melting temperature

               fluid = .true.

            end if  ! ( tempold(i)<tmelt )

         end do  ! i = 1, n

      end if ! mushy

      if ( mushy ) then

         if ( neg==n ) then

            solid = .true.

         else if ( pos==n ) then

            fluid = .true.

         else

            solid = .true.
            fluid = .true.

         end if ! ( neg==n )

      end if ! ( mushy )

!     --- Fill parts of matrix if necessary

      if (debug) print *, 'fluid, solid', fluid, solid
      if (debug) call prinrl(tempold, n, 'tempold')
      jadd = 0
      second = .false.
      if ( fluid .and. solid ) then

!     --- Intersection of element with freezing front, special treatment

         call eldiscint ( massmt, stiffness, dldt, lvector, B%phider, &
                          tempold, coeffarrlc, B%xyz, intdiscrule )

      else

         if ( fluid ) then

!        --- Fluid element, use kappa_fluid and c_fluid

            ipalpha = ipkappa_fluid
            ipcp = ipc_fluid
            iprho = iprho_fluid

!           --- Create L vector = rho L / phi_i d Omega

            do i = 1, n
               lvector(i) = B%w(i)*coeffarrlc(iprho+i-1)*latent_heat
            end do  ! i = 1, n

          else

!         --- Solid element, use kappa_solid and c_solid

            ipalpha = ipkappa_solid
            ipcp = ipc_solid
            iprho = iprho_solid

!           --- Set L vector to 0

            do i = 1, n
               lvector(i) = 0d0
            end do  ! i = 1, n

         end if ! ( fluid ) then

!        --- First second derivative part (Diffusive terms)

         call elm800dif ( B%type_basisfunc, coeffarrlc(ipalpha), stiffness, B%w, B%phider, &
                          psiupw, B%phi, coor, E%nodes, &
                          E%iuser, E%user, vecloc, &
                          second, coeffarrlc(ipphixicp), B%xyz, B%xyzg )
         if ( ierror/=0 ) go to 1000

!        --- Next compute mass matrix
!            Put rho*cp in rhocp

         do i = 0, m-1
            coeffarrlc(iprhocp+i) = coeffarrlc(iprho+i)*coeffarrlc(ipcp+i)
         end do  ! i = 1, m
         jadd = 0

         call el3002 ( coeffarrlc(iprhocp), massmt, B%w, &
                       B%phi, m, n, n )

!        --- Make matrix dL/dT equal to zero

         do i = 1, n
            do j = 1, n
               dldt(i,j) = 0d0
            end do  ! j = 1, n
         end do  ! i = 1, n

      end if  ! ( fluid .and. solid )

      if ( matrix ) then

!     --- Create element matrix: G = M/dt + S + (dL/dT)/dt
!         Due to the FORTRAN storage the element matrix is a transposed
!         matrix, hence i and j are interchanged

         do i = 1, n
            do j = 1, n
               E%mat(j,i) = stiffness(i,j)+(massmt(i,j)+dldt(i,j))/ &
                             tstep
            end do  ! j = 1, n
         end do  ! i = 1, n

      end if  ! ( matrix )

!     --- Compute the various terms in the right-hand side
!         RHS1 = -(M T + L)/dt + S T
!         RHS2 = (M T + L)/dt + F  (only if numrhs = 2)

      if ( numrhs==2 ) then

!     --- numrhs = 2, compute the right-hand side vector F

         if ( ind(6)/=0 ) then

!        --- ind(6) # 0, hence Q # 0

            call el3003 ( coeffarrlc(iprhs), fvec, B%w, B%phi, m, n )

         else

!        --- ind(6) = 0, hence Q = 0

            do i = 1, n
               fvec(i) = 0d0
            end do  ! i = 1, n

         end if  ! ( ind(6)/=0 )

      end if  ! ( numrhs==2 )

!     --- Assemble right-hand side element vectors

      call el807rhs ( elemvc2d, massmt, stiffness, lvector, fvec, &
                      tempold, numrhs )

      if ( debug ) then
         do i = 1, n
            if ( lvector(i)<-1d-6 ) then
               print *, 'solid, fluid', solid, fluid
               call prinrl(lvector,icount,'lvector')
               write(irefwr,*) 'element ', ielem, '  itype ',itype
               call prinrl(tempold,icount,'tempold')
            end if
            do j = 1, n
               if ( dldt(i, j)<-1d-6 ) then
                  print *, 'solid, fluid', solid, fluid
                  call prinrl1(dldt,icount,icount,'dldt')
                  write(irefwr,*) 'element ', ielem, '  itype ',itype
                  call prinrl(tempold,icount,'tempold')
               end if
            end do
         end do
      end if

1000  symm = .true.

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

      if ( debug ) then
         write(irefwr,*) 'numrhs', numrhs
         write(irefwr,*) 'element ', ielem, '  itype ',itype
         write(irefwr,*) 'tstep, t ', tstep, t
         call prinrl(elemvc2d(1,1),icount,'elemvc2d_1')
         call prinrl(elemvc2d(1,2),icount,'elemvc2d_2')
         call prinrl1(E%mat,icount,icount,'E%mat')
         call prinrl1(massmt,icount,icount,'massmt')
         call prinrl1(stiffness,icount,icount,'stiffness')
         call prinrl1(dldt,icount,icount,'dldt')
         call prinrl(lvector,icount,'lvector')
         call prinrl(tempold,icount,'tempold')
         call prinrl(fvec,icount,'fvec')

      end if
      call erclos ( 'elm807' )

      end
