      subroutine elm808 ( vecloc, coeffarrlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 12-08-2013 Extra parameters
!        version  2.1  date 12-12-2012 Long integers
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order elliptic equations
!     and related heat equation with two unknowns per point
!     One, two and three dimensional elements
!     In case of defect correction, the first matrix contains the upwind
!     element matrix and the second one the standard Galerkin matrix
!     The element vectors are identical for both
!     ELM808 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine el0000
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  EL0000
!             - ELM808
!    The subroutine is able to handle the following element shapes and cases:
!    linear line element (1D)
!    quadratic line element (1D)
!    linear triangle (2D and 3D)
!    quadratic triangle (2D and 3D)
!    bilinear quadrilateral (2D)
!    biquadratic quadrilateral (2D):
!    linear pyramid (3D)
!    quadratic pyramid (3D)
!    trilinear brick (3D)
!    triquadratic brick (3D)
!    cartesian co-ordinates
!    axi-symmetric co-ordinates (2D only)
!    polar co-ordinates (1D only)
!    diagonal mass matrix
!    non-diagonal mass matrix
!    Type 808: standard case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), coeffarrlc(*)

!     coeffarrlc     i    Array with coefficents per integration point
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, jelp, kfirst, jheat, &
              iseq, ipalpha1, ipalpha2, &
              ipbeta1, ipbeta2, ipend, ipu1, ipu2, ipf1, &
              ipf2, iprho1, iprho2, ipphixicp, &
              ipalpha12, ipu12, ipbeta12, ipalpha21, ipu21, ipbeta21
      double precision u(6), &
                       dudx(3*3*27), &
                       wrk1(27*inpelm+inpelm*7), &
                       uold(27), ugauss(27)
      save isub, isup, jelp, kfirst, jheat, u, &
           ipalpha1, ipalpha2, ipbeta1, ipbeta2, ipu1, &
           ipu2, ipf1, ipf2, iprho1, iprho2, ipphixicp, &
           ipalpha12, ipu12, ipbeta12, ipalpha21, ipu21, ipbeta21

!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     ipalpha1       starting address of array alpha11
!     ipalpha12      starting address of array alpha12
!     ipalpha2       starting address of array alpha22
!     ipalpha21      starting address of array alpha21
!     ipbeta1        starting address of array beta11
!     ipbeta12       starting address of array beta12
!     ipbeta2        starting address of array beta22
!     ipbeta21       starting address of array beta21
!     ipend          Last position in array
!     ipf1           starting address of array f1
!     ipf2           starting address of array f2
!     ipphixicp      Starting position of parameter phixicp in coeffarrlc
!     iprho1         Starting address of array rho1
!     iprho2         Starting address of array rho2
!     ipu1           Starting address of array u11
!     ipu12          Starting address of array u12
!     ipu2           Starting address of array u22
!     ipu21          Starting address of array u21
!     iseq           Sequence number
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx.
!     jheat          Integer parameter indicating if the mass matrix for
!                    the heat equation must be computed (>0) or not ( = 0)
!                    If jheat = 1, then the coefficient for the mass matrix
!                    is constant, otherwise (jheat = 2) it is variable
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     u              array of length m containing the values of the
!                    second coefficient in the integration points
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     wrk1           work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0808         Initialize parameters for subroutine ELM808
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM805MASSMAT  Fill element mass matrix for elements of type 805, i.e.
!                    general second order elliptic equation with two degrees of
!                    freedom per point
!     ELM805RHSD     Fill element vector for elements of type 805, i.e
!                    general second order elliptic equation with two degrees of
!                    freedom per point
!     ELM808MATRIX   Fill element matrix for elements of type 808, i.e.
!                    general second order elliptic equation with two degrees of
!                    freedom per point
!     ELMFILLCOEFF   Fill the space dependent coefficients for an element
!                    subroutine
!                    Fill the old vectors in the nodes of the element
!     ELMMULTURHO    Multiply the velocity u by rho in case both are constant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
! **********************************************************************
!
!                       I/O
!
!     The system of equations has the shape
!     D_11 u_1 + D_12 u_2 = f_1
!     D_21 u_1 + D_22 u_2 = f_2, &
!     where D_ij refers to a second order equation for one unknown
!     as in type 800
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  This parameter is no longer in use.
!         The parameter imas in BUILD defines the type of mass matrix
!         Now the parameter is reserved for future purposes
!     2:  At this moment only central discretization is available
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
!     5:  This parameter depends on the type number to be used
!         In case of type number 803 it indicates the type of linearization:
!         Possible values:
!         0,1: Picard
!         2:   Newton
!     Parameters 6-20 corresponding to first unknown: (D_11)
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!    Parameters: 21-35 corresponding to second unknown have exactly
!    the same meaning (D_22)
!    Parameters: 36-50 corresponding to first row second unknown
!    have exactly the same meaning (D_12)
!    Parameters: 51-65 corresponding to second row first unknown
!    have exactly the same meaning (D_21)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Differential equation:
!    rho cp d phi/d t - div alpha grad phi +rho cp u . grad phi + beta phi = f
!                                                  - &
!        The following formulae are used:
!           S   =  S      +  S   +  S
!                   alpha     u      beta
!           S     (i,j) = / alpha grad phi  . grad phi   de
!            alpha        e               j           i
!           S     (i,j) = / u . grad phi   phi  de
!            u            e -           j     i
!           S     (i,j) = / beta phi   phi  de
!            beta        e          j     i
!           M (i,j) = / rho cp phi  phi  de
!                     e           i    j
!           F(i)    =  /  f phi  de
!                      e       i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm808' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm808'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         ix = 0
         call  el0808 ( E%iuser, E%user, jelp, kfirst, jheat, isub, isup, &
                        coeffarrlc, iseq )
         if ( debug ) write(irefwr,1) 'jelp, n, m', jelp, n, m
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

         ipalpha1 = 1
         ipu1     = 1+6*m
         ipbeta1  = 1+9*m
         ipf1     = 1+10*m
         iprho1   = 1+11*m
         ipalpha2 = 1+12*m
         ipu2     = ipalpha2+6*m
         ipbeta2  = ipalpha2+9*m
         ipf2     = ipalpha2+10*m
         iprho2   = ipalpha2+11*m
         ipalpha12= 1+24*m
         ipu12    = ipalpha12+6*m
         ipbeta12 = ipalpha12+9*m
         ipalpha21= 1+36*m
         ipu21    = ipalpha21+6*m
         ipbeta21 = ipalpha21+9*m
         ipphixicp = 1+48*m
         ipend =  ipphixicp+ndim*n*m

         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'ipalpha1, ipu1, ipbeta1, ipf1, iprho1', &
                             ipalpha1, ipu1, ipbeta1, ipf1, iprho1
            write(irefwr,1) 'ipalpha2, ipu2, ipbeta2, ipf2, iprho2', &
                             ipalpha2, ipu2, ipbeta2, ipf2, iprho2
            write(irefwr,1) 'ipalpha12, ipu12, ipbeta12', &
                             ipalpha12, ipu12, ipbeta12
            write(irefwr,1) 'ipalpha21, ipu21, ipbeta21', &
                             ipalpha21, ipu21, ipbeta21

         end if  ! ( debug )

         if ( ipend>19000 ) then

!        --- array wrk1 too small, see subroutine elmgrinf

            call errint ( ipend, 1 )
            call errint ( 8910, 2 )
            call errchr ( 'coeffarrlc', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

!        --- Store value of velocity in u and multiply u in coeffarrlc by rho
!            This is only done for constant velocity components

         call elmmulturho ( coeffarrlc(ipu1), coeffarrlc(iprho1), ind(7), &
                            kstep, ndim, m, u, 1, ind(12) )
         call elmmulturho ( coeffarrlc(ipu2), coeffarrlc(iprho2), ind(19), &
                            kstep, ndim, m, u, 1, ind(24) )

      end if

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold and dudx if necessary
!         Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      call elmfillcoeff ( uold, dudx, B%phi, B%phider, E%nodes, ugauss, iseqin, &
                          vecloc, wrk1, isub, isup, E%user, coeffarrlc, &
                          numold, E%iuser, maxunk, B%xyz, B%xyzg )
      if ( ierror/=0 ) go to 1000

      if ( ind(12)/=0 ) then

!     --- rho cp has been given, multiply velocity by rho cp

         call elmmulturho ( coeffarrlc(ipu1), coeffarrlc(iprho1), ind(7), &
                            kstep, ndim, m, u, 2, ind(12) )

      end if  ! ( ind(12)/=0 )

      if ( ind(24)/=0 ) then

!     --- rho cp has been given, multiply velocity by rho cp

         call elmmulturho ( coeffarrlc(ipu2), coeffarrlc(iprho2), ind(19), &
                            kstep, ndim, m, u, 2, ind(24) )

      end if  ! ( ind(12)/=0 )

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         call elm808matrix ( B%type_basisfunc, coeffarrlc(ipalpha1), E%mat, B%w, &
                             B%phider, B%phi, coor, &
                             E%nodes, E%iuser, E%user, vecloc, &
                             coeffarrlc(ipphixicp), B%xyz, B%xyzg, &
                             symm, coeffarrlc(ipu1), kfirst, &
                             coeffarrlc(ipbeta1), dudx, &
                             idefcr, ndim, coeffarrlc(ipalpha2), &
                             coeffarrlc(ipu2), coeffarrlc(ipbeta2), &
                             icount/2, coeffarrlc(ipalpha12), &
                             coeffarrlc(ipu12), coeffarrlc(ipbeta12), &
                             coeffarrlc(ipalpha21), &
                             coeffarrlc(ipu21), coeffarrlc(ipbeta21) )

      end if

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         call elm805rhsd ( wrk1, coeffarrlc(ipf1), coeffarrlc(ipf2), &
                           E%vec, B%phi, B%w, icount/2 )

      end if

!     --- Fill mass matrix if necessary

      if ( notmas==0 ) then

!     --- notmas = 0,  compute mass matrix

         call elm805massmat ( jheat, coeffarrlc(iprho1), coeffarrlc(iprho2), &
                              B%phi, E%mmat, wrk1, B%w, &
                              icount/2 )

      end if

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,1) 'element, itype', ielem, itype
         write(irefwr,1) 'jheat', jheat
         call prinrl(E%vec,icount,'E%vec')
         call prinrl1(E%mat,icount,icount,'E%mat')
         if ( notmas==0 ) then
            if ( jheat==1 ) then
               call prinrl(E%mmat,icount,'E%mmat')
            else
               call prinrl1(E%mmat,icount,icount,'E%mmat')
            end if  ! ( jheat==1 )
         end if  ! ( notmas==0 )
         write(irefwr,*) 'End elm808'

      end if
      call erclos ( 'elm808' )

      end
