      subroutine elm810 ( coeffarrlc, vecloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 12-08-2013 Extra parameters
!        version  2.3  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for enthalpy equation using the
!     Nedjar quasi-newton linearization
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coeffarrlc(*), vecloc(*)

!     coeffarrlc            Real work array which must be kept for each element
!                         group
!                         work consists of the following parts
!                         Starting address    length   Name:
!                         1                   13*27    array containing the
!                                                      coefficients
!                         ipdphx              27*27*3  DPHIDX
!                         ippsi               27*27    PSI
!                         ipphixi            3*27*27  phixi
!                         ipphixicp          3*27*27  copy of phixi
!                         array contains the folowing contents:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                               1D                  2D            3D
!                          1:   kappa_S
!                          2:   kappa_L
!                          3:   Capacity_S
!                          4:   Capacity_L
!                          5:   Latent_Heat
!                          6:   Melt_Temp
!                          7:   u 1              u 1            u 1
!                          8:                    u 2            u 2
!                          9:                                   u 3
!                         10:   beta             beta           beta
!                         11:   f                f              f
!                         12:   rho cp           rho cp         rho cp
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, jelp, jheat, ih, iseq, &
              ktrans, k, ipf, iprho, i, ipgextra, ipphixicp, &
              ipend, itypeeq
      double precision dudx(3*3*27),  &
                       wrk1(27*inpelm+inpelm*7),&
                       uold(27), temperature(27), kappa_s, kappa_l, &
                       tmelt, kappa(27)
      save isub, isup, jheat, ipf, ipgextra, iprho, &
           ipphixicp, kappa_s, kappa_l, tmelt, itypeeq

!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ipend          Last position in array
!     ipf            Starting position of parameter f in coeffarrlc
!     ipgextra       Starting position of parameter gextra in coeffarrlc
!     ipphixicp     Starting position of parameter phixicp in coeffarrlc
!     iprho          Starting position of parameter rho in coeffarrlc
!     iseq           Dummy not used
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     itypeeq        if 0 the standard enthalpy equation is solved
!                    if 1 the equation is restricted to a solid temperature
!                    equation
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed
!                    Possible values:
!                     0:  no mass matrix
!                     1:  diagonal mass matrix with constant coefficient
!                     2:  diagonal mass matrix with variable coefficient
!                    11:  full mass matrix with constant coefficient
!                    12:  full mass matrix with variable coefficient
!     k              Counting variable
!     kappa          Diffusion coefficient per integration point
!     kappa_l        diffusion (liquid)
!     kappa_s        diffusion (solid)
!     ktrans         Help variable to save the value of jtrans
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     temperature    temperature vector per integrtion point
!     tmelt          Melting temperature
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     wrk1           Work array to be used by some under subroutines
!     x              array of length n x ndim containing the coordinates of
!                    the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0810         performs the initializations for subroutine ELM810
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3009         Fill series of old solutions in array vecloc
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ELMDIFFUSION   Compute the element diffusion matrix in case there is one
!                    diffusion coefficient
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Not used
!     2:  Upwind: not used
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         See type 800
!     5:  Type of equation
!         0: standard enthalpy with phase change
!         1: temperature equation, no phase change
!     6:  kappa_S
!     7:  kappa_L
!     8:  Capacity_S
!     9:  Capacity_L
!    10:  Latent_Heat
!    11:  Melt_Temp
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho       (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!    21:  iseqtemp
!    22:  iseqenth
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm810' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm810'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         ix = 0
         call  el0810 ( E%iuser, E%user, jelp, itypeeq, jheat, isub, isup, &
                        coeffarrlc, iseq )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )
         kappa_s = coeffarrlc(1)
         kappa_l = coeffarrlc(1+m)
         tmelt = coeffarrlc(1+5*m)

         ipf      = 1+10*m
         iprho    = 1+11*m
         ipgextra = 1+12*m
         ipphixicp = ipgextra+m  ! work array to store copy of phixi
                                  ! of length 3*27*27
         ipend =  ipphixicp+ndim*n*m
         if ( ipend>12285 ) then

!        --- array wrk1 too small

            call errint ( ipend, 1 )
            call errint ( 12285, 2 )
            call errchr ( 'coeffarrlc', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if
         if ( debug ) &
            write(irefwr,1) 'isub, isup, ind(lengthmst)', &
                             isub, isup, ind(lengthmst)

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!     --- Fill variable coefficients if necessary
!         First fill uold and dudx if necessary

         ktrans = jtrans
         jtrans = 0
         call el2005 ( uold, dudx, B%phi, B%phider, &
                       E%nodes, temperature, iseqin )
         jtrans = ktrans

      end if

      if ( ierror/=0 ) go to 1000

!     --- Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, B%phi, n )

!     --- Fill variable coefficients if necessary

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, coeffarrlc, B%xyz, &
                          B%xyzg, m, n, B%phi, &
                          1, E%nodes, E%iuser, temperature, vecloc )

         end if

      end do
      if ( ierror/=0 ) go to 1000

!     --- Fill array kappa depending on the temperature

      if ( itypeeq==0 ) then

!     --- Enthalpy equation

         do k = 1, m

            if ( temperature(k)>=tmelt ) then

!           --- temperature above melting temperature use kappa_l

               kappa(k) = kappa_l

            else

!           --- temperature below melting temperature use kappa_s

               kappa(k) = kappa_s

            end if  ! ( temperature(k)>=tmelt )

         end do  ! k = 1, m

      else

!     --- Temperature equation

         do k = 1, m
            kappa(k) = kappa_s
         end do  ! k = 1, m

      end if  ! ( itypeeq==0 )

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         jadd = 0
         call elmdiffusion ( B%type_basisfunc, kappa, E%mat, B%w, B%phider )

         if ( ierror/=0 ) go to 1000

      end if  ! ( matrix )
      if ( ierror/=0 ) go to 1000

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         jadd = 0

         call el3003 ( coeffarrlc(ipf), E%vec, B%w, B%phi, m, n )

      end if

      if ( ierror/=0 ) go to 1000

!     --- Fill mass matrix if necessary

      if ( notmas==0 ) then

!     --- notmas = 0,  compute mass matrix

         if ( jheat>10 ) then

!        --- jheat = 11, 12:  Non-diagonal mass matrix

            jadd = 0
            ih = jzero
            jzero = jheat-10

            call el3002 ( coeffarrlc(iprho), E%mmat, B%w, &
                          B%phi, m, n, n )
            jzero = ih

         else

!        --- jheat = 1, 2, i.e. diagonal mass matrix

            jadd = 0
            ih = jconsf
            jconsf = jheat
            call el3003 ( coeffarrlc(iprho), E%mmat, B%w, B%phi, &
                          m ,n )
            jconsf = ih

         end if

      end if

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         write(irefwr,*) 'jheat ', jheat
         call prinrl(E%vec,icount,'E%vec')
         call prinrl1(E%mat,icount,icount,'E%mat')
         if ( notmas==0 ) &
            call prinrl1(E%mmat,icount,icount,'E%mmat')
         write(irefwr,*) 'End elm810'

      end if
      call erclos ( 'elm810' )

      end
