      subroutine elm814matrix ( itypln, dudx, elemmt, w, dphidx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 15-12-2010
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for type 814
!     minimal surface problem
! **********************************************************************
!
!                       KEYWORDS
!
!     minimal_surface
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypln
      double precision dudx(m,ndim), elemmt(n,n), w(m), dphidx(n,m,ndim)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     elemmt         o    Element matrix
!     itypln         i    Indicates the type of linearization to be performed
!                         Possible values:
!                         0: Picard
!                         1: Newton
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision alpha(m), factor, addsum, factor1

!     addsum         Help parameter to compute a sum
!     alpha          work array to store coefficients
!     factor         Multiplication factor
!     factor1        Multiplication factor
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The minimal surface problem reads:
!
!     minimize int sqrt(1+u_x^2+u_y^2) d Omega
!     Applying the Ritz method gives the non-linear system
!     int (u_x phi_i_x + u_y phi_i_y ) / sqrt(1+u_x^2+u_y^2) d Omega = 0
!
!     In case of Picard iteration we take the sqrt at the previous iteration
!     step, so that a linear system follows:
!     sum_j u_j int_Omega alpha (phi_i_x phi_j_x + phi_i_y phi_j_y) d Omega = 0
!     alpha = (1+u_x^2+u_y^2)^(-1/2); u = uold
!
!     In case of Newton we have to solve
!     dF/du  Delta u = - f(u)
!     with f_i(u) = int (u_x phi_i_x + u_y phi_i_y ) beta^(-1/2) d Omega
!          beta = 1+u_x^2+u_y^2
!     dF_i/du_j =int (phi_j_x_x phi_i_x + phi_j_y_y phi_i_y ) beta^(-1/2) - &
!       (u_x phi_i_x + u_y phi_i_y ) (u_x phi_j_x + u_y phi_j_y ) beta^(-3/2)
!        d Omega
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm814matrix' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm814matrix'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill coefficient alpha

      if ( jdercn==0 ) then

!     --- Derivatives constant

         alpha(1) = 1d0/sqrt(1d0+dudx(1,1)**2+dudx(1,2)**2)

      else

!     --- Derivatives variable

         alpha(1:m) = 1d0/sqrt(1d0+dudx(1:m,1)**2+dudx(1:m,2)**2)

      end if  ! ( jdercn==0 )

      if ( itypln==0 ) then

!     --- itypeln = 0: Picard iteration

         if ( jdercn==0 ) then

!        --- Derivatives constant

            factor = sum(w)*alpha(1)

            do i = 1, n

               do j = 1, n

                  elemmt(i,j) = factor*(dphidx(i,1,1)*dphidx(j,1,1) + &
                                        dphidx(i,1,2)*dphidx(j,1,2) )

               end do  ! j = 1, n

            end do  ! i = 1, n

         else

!        --- Derivatives variable

            do i = 1, n

               do j = 1, n

                  addsum = 0d0

                  do k = 1, m
                     addsum = addsum+w(k)*alpha(k)* &
                              (dphidx(i,k,1)*dphidx(j,k,1) + &
                               dphidx(i,k,2)*dphidx(j,k,2) )
                  end do  ! k = 1, m

                  elemmt(i,j) = addsum

               end do  ! j = 1, n

            end do  ! i = 1, n

         end if  ! ( jdercn==0 )

      else

!     --- typeln = 1: Newton iteration

         if ( jdercn==0 ) then

!        --- Derivatives constant

            factor = sum(w)*alpha(1)
            factor1 = factor*alpha(1)**2

            do i = 1, n

               do j = 1, n

                  elemmt(i,j) = factor*(dphidx(i,1,1)*dphidx(j,1,1) + &
                                        dphidx(i,1,2)*dphidx(j,1,2) ) - &
                                factor1*(dudx(1,1)*dphidx(i,1,1)+ &
                                         dudx(1,2)*dphidx(i,1,2))* &
                                        (dudx(1,1)*dphidx(j,1,1)+ &
                                         dudx(1,2)*dphidx(j,1,2))

               end do  ! j = 1, n

            end do  ! i = 1, n

         else

!        --- Derivatives variable

            do i = 1, n

               do j = 1, n

                  addsum = 0d0

                  do k = 1, m
                     addsum = addsum+w(k)*alpha(k)* &
                              (dphidx(i,k,1)*dphidx(j,k,1) + &
                               dphidx(i,k,2)*dphidx(j,k,2)  - &
                              alpha(k)**2*(dudx(k,1)*dphidx(i,k,1)+ &
                                           dudx(k,2)*dphidx(i,k,2))* &
                                          (dudx(k,1)*dphidx(j,k,1)+ &
                                           dudx(k,2)*dphidx(j,k,2)))
                  end do  ! k = 1, m

                  elemmt(i,j) = addsum

               end do  ! j = 1, n

            end do  ! i = 1, n

         end if  ! ( jdercn==0 )

      end if  ! ( typln==0 )

1000  call erclos ( 'elm814matrix' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm814matrix'

      end if  ! ( debug )

      end
