      subroutine elm816 ( vecloc, coeffarrlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 13-11-2014
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for special right-hand side
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(inout) ::  vecloc(*), coeffarrlc(*)

!     coeffarrlc            Real work array which must be kept for each element
!                         group
!                         work consists of the following parts
!                         Starting address    length   Name:
!                         1                   13*27    array containing the
!                                                      coefficients
!                         ipdphx              27*27*3  B%phider
!                         ippsi               27*27    PSI
!                         ipphiksi            3*27*27  phiksi
!                         ipphiksicp          3*27*27  copy of phiksi
!                         array contains the folowing contents:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                               1D                  2D            3D
!                          1:   alpha 11         alpha 11       alpha 11
!                          2:                    alpha 12       alpha 12
!                          3:                    alpha 22       alpha 13
!                          4:                                   alpha 22
!                          5:                                   alpha 23
!                          6:                                   alpha 33
!                          7:   u 1              u 1            u 1
!                          8:                    u 2            u 2
!                          9:                                   u 3
!                         10:   beta             beta           beta
!                         11:   f                f              f
!                         12:   rho cp           rho cp         rho cp
!                         13:   gamma_1          gamma_1        gamma_1
!                         14:                    gamma_2        gamma_2
!                         15:                                   gamma_3
!                         16:   g_extra          g_extra        g_extra
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: isub, isup, jelp
      double precision :: dudx(3*3*27), &
                          wrk1(27*inpelm+inpelm*7),&
                          uold(27), ugauss(27)
      save isub, isup

!     B%phider         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = B%phider (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = B%phider (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = B%phider (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic  brick)
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     wrk1           Work array to be used by some under subroutines
!     x              array of length n x ndim containing the coordinates of
!                    the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0816         performs the initializations for subroutine ELM816
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ELM816RHSD     Fill element vector for elements of type 816
!     ELMFILLCOEFF   Fill the space dependent coefficients for an element
!                    subroutine
!                    Fill the old vectors in the nodes of the element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  - &
!     2:  - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
!     5:  - &
!     6:  vector u
!     7:  vector w
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The right-hand side vector to be computed is
!
!     f_i = /_e  u grad w . grad phi_i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm816' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm816'
         call prinin ( E%nodes, inpelm, 'index1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         ix = 0
         call el0816 ( E%iuser, E%user, jelp, isub, isup, coeffarrlc )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc

      if ( debug ) write(irefwr,1) 'n, m', n, m

      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold and dudx if necessary
!         Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      call elmfillcoeff ( uold, dudx, B%phi, B%phider, E%nodes, ugauss, iseqin, &
                          vecloc, wrk1, isub, isup, E%user, coeffarrlc, &
                          numold, E%iuser, maxunk, B%xyz, B%xyzg )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinrl1 ( coeffarrlc, 2, m, 'coeffarrlc' )

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         call elm816rhsd ( E%vec, coeffarrlc(1), coeffarrlc(1+m), &
                           B%phider, B%w )

      end if  ! ( vector .and. notvec/=1 )

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         call prinrl(E%vec,icount,'elemvc')

      end if
      call erclos ( 'elm816' )

      end
