      subroutine elm850 ( vecloc, coeffarrlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 12-08-2013 Extra parameters
!        version  1.1  date 12-12-2012 Long integers
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for special
!     stochastic investement equation
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
!     1d
!     2d
!     element_matrix
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), &
                       coeffarrlc(*)

!     coeffarrlc      i/o   Array with coefficents per integration point
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     B%phiderc         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, jelp, kfirst, ipend
      double precision, allocatable :: ugauss(:,:), dudx(:,:,:), uold(:,:)
      save isub, isup, kfirst

!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     ipend          Last position in array
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx.
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              array of length n x ndim containing the coordinates of
!                    the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0850         performs the initializations for subroutine ELM850
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ELM850MATRIX   Fill element matrix for stochastic investment problem
!     ELMFILLCOEFF   Fill the space dependent coefficients for an element
!                    subroutine
!                    Fill the old vectors in the nodes of the element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm850' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm850'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         ix = 0
         call  el0850 ( E%iuser, E%user, jelp, kfirst, isub, &
                        isup, coeffarrlc )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

!        --- Set starting addresses in coeffarrlc

         ipend = 1+m   ! end of coeffarrlc
         call inichkarlen ( ipend, 4401, 'coeffarrlc' )

      end if
      if ( ierror/=0 ) go to 1000
      allocate ( ugauss(m,ndim), &
                 dudx(ndim,ndim,m), uold(n,ndim), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( 30*m, 1 )
         call errint ( error, 2 )
         call errchr ( 'elgrpw0', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold and dudx if necessary
!         Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      call elmfillcoeff ( uold, dudx, B%phi, B%phider, E%nodes, ugauss, iseqin, &
                          vecloc, work, isub, isup, E%user, coeffarrlc, &
                          numold, E%iuser, maxunk, B%xyz, B%xyzg )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinrl ( coeffarrlc, m, 'f' )

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         call elm850matrix ( B%type_basisfunc, E%mat, B%w, B%phider, B%phi, &
                             coor, B%phiderc, B%xyz, B%xyzg )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         call el3003 ( coeffarrlc, E%vec, B%w, B%phi, m, n )

      end if

      if ( ierror/=0 ) go to 1000

900   deallocate ( ugauss, dudx, uold, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'x', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         call prinrl(E%vec,icount,'elemvc')
         call prinrl1(E%mat,icount,icount,'elemmt')
         write(irefwr,*) 'End elm850'

      end if  ! ( debug )
      call erclos ( 'elm850' )

      end
