      subroutine elm995
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 09-03-2013 Correction declaration
!        version  1.1  date 12-12-2012 Long integers
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Special element to add part of reaction force along curves
!     or surfaces to right-hand side vector (1 degree of freeedom)
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_vector
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecons
      use sepmoduleelem
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kpos, iseqreac, i, istart, ndegfd, isign1
      double precision sign
      double precision, pointer :: vecoldlc(:)

!     i              Counting variable
!     iseqreac       Sequence number of reaction vector in array ISOL
!     isign1         Defines sign of reaction force, see coefficient 2
!     istart         Starting position
!     kpos           Position in array
!     ndegfd         Number of degrees of freedom per point
!     sign           Multplication parameter for the vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ2ROW     Print one row of a 2d array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The number of coefficients reequired is 10
!     The first 5 are integer
!     1:   Sequence number of reaction force vector in set of solution vectors
!     2:   Defines sign of reaction force
!          0:  minus sign (standard)
!          1:  positive sign
!     3-10:  Not yet defines
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    2901   Sequence number of vector in element subroutine out of range
!    2904   index4(.,inpelm+1) incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm995' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm995'
         write(irefwr,1) 'ndim, inpelm, icount', &
                          ndim, inpelm, icount
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      notmat = 1     ! element matrix is zero
      notmas = 1     ! element matrix is zero

!     --- Get sequence number of reaction force

      kpos = E%iuser ( ielgrp + 5 )
      if ( kpos<7 ) then

!     --- kpos<7, hence E%iuser has not been filled correctly

         call errint ( ielgrp+5, 1 )
         call errint ( kpos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )

      end if  ! ( kpos<7 )

      iseqreac = E%iuser(kpos)     ! sequence number of reaction force
      if ( iseqreac<1 .or. iseqreac>numold ) then

!     --- iseqreac out of bounds

         call errint ( iseqreac, 1 )
         call errint ( numold, 2 )
         call errint ( 1, 3 )
         call errint ( ielem, 4 )
         call errint ( itype, 5 )
         call errint ( ielgrp, 6 )
         call errchr ( 'iseqreac', 1 )
         call errsub ( 2901, 6, 0, 1 )

      end if  ! ( iseqreac<1 .or. iseqreac>numold )
      isign1 = E%iuser(kpos+1)     ! sign of reaction force
      if ( ierror/=0 ) go to 1000

      if ( isign1==0 ) then
         sign = -1d0   ! we need the negative reaction force
      else
         sign = 1d0   ! we need the positive reaction force
      end if  ! ( isign1==0 )

      if ( debug ) &
         call prinin2row ( index4, numold, inpelm+1, iseqreac, &
                           'index4' )

      iseqtype = oldsols(iseqreac)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      index4n => indoldsols(iseqtype)%index4
      vecoldlc => oldsols(iseqreac)%oldvecs

      ndegfd = index4n(inpelm+1)

      if ( ndegfd<=0 ) then

!     --- ndegfd has incorrect value

         call errint ( ndegfd, 1 )
         call errsub ( 2904, 1, 0, 0 )

      end if  ! ( ndegfd>0 .and. ndegfd<ndim )
      if ( ierror/=0 ) go to 1000

!     --- Copy reaction force to element vector

      if ( ndegfd>0 ) then

!     --- ndegfd>0, hence constant number of degrees of freedom

         do i = 1, inpelm
            E%vec((i-1)*ndim+1) = sign* &
               vecoldlc(index3n((i-1)*ndegfd+1))
         end do  ! i = 1, inpelm

      else

!     --- ndegfd = -1, hence variable number of degrees of freedom

         istart = 0
         do i = 1, inpelm
            E%vec((i-1)*ndim+1) = sign* &
               vecoldlc(index3n(istart+1))
            istart = index4n(i)
         end do  ! i = 1, inpelm

      end if  ! ( ndegfd>0 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%vec, inpelm, ndim, 'elemvc' )
         write(irefwr,*) 'End elm995'

      end if  ! ( debug )
      call erclos ( 'elm995' )

      end
