      subroutine elmfillcoeff ( uold, dudx, phi, dphidx, &
                                index1, ugauss, iseqin, vecloc, &
                                wrk1, isub, isup, user, coeffarr, &
                                numold, iuser, maxunk, x, xgauss )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 10-02-2015 Extra debug
!        version  1.4  date 09-03-2013 Replace elflr7 by elflr71
!        version  1.3  date 12-12-2012 Long integers
!
!   copyright (c) 2006-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the space dependent coefficients for an element subroutine
!     Fill the old vectors in the nodes of the element
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     fill
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), iseqin, &
              isub, isup, numold, iuser(*), maxunk
      double precision uold(*), dudx(*), phi(*), dphidx(*), &
                       ugauss(*), vecloc(*), wrk1(*), user(*), &
                       coeffarr(m,*), x(*), xgauss(*)

!     coeffarr      i/o   Array with coefficents per integration point
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           o    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iseqin         i    Sequence number of vector in VECOLD/ISLOLD
!     isub           i    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           i    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     maxunk         i    Maximum number of unknowns per point
!     numold         i    Number of vectors that are stored in VECOLD
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     ugauss         i    Preceding solution in Gauss points
!     uold           i    Array containing the previous iteration (old solution)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc         o    Work array in which all old solution vectors for the
!                         integration points are stored
!     wrk1                Work array
!     x              i    co-ordinates of points in element
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ktrans

!     i              Counting variable
!     ktrans         Indicates if a transformation to a space of lower
!                    degree must be performed for the basis functions
!                    Possible values:
!                    0:  No transformation
!                    21: Tranform from 2D to 1D
!                    31: Tranform from 3D to 1D
!                    32: Tranform from 3D to 2D
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL3009         Fill series of old solutions in array vecloc
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmfillcoeff' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmfillcoeff'
         write(irefwr,1) 'isub, isup', isub, isup
         write(irefwr,1) 'm, n, lengthmst', m, n, lengthmst
         write(irefwr,1) 'ind(lengthmst)', ind(lengthmst)
         call prinrl2 ( x, n, ndimlc, 'x' )
         call prinrl2 ( xgauss, m, ndimlc, 'xgauss' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!     --- Fill variable coefficients if necessary
!         First fill uold and dudx if necessary

         ktrans = jtrans
         jtrans = 0
         call el2005 ( uold, dudx, phi, dphidx, &
                       index1, ugauss, iseqin )
         jtrans = ktrans

      end if

!     --- Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, phi, n )

!     --- Fill variable coefficients if necessary

      do i = isub, isup

         if ( debug ) &
            write(irefwr,1) 'i, ind(i), ist(i)', i, ind(i), ist(i)
         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, user, coeffarr, x, &
                           xgauss, m, n, phi, &
                           1, index1, iuser, ugauss, vecloc )

         end if  ! ( ind(i)>0 .or. ist(i)>0 )

      end do  ! i = isub, isup

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coeffarr, isup, m, 'coefficients' )
         write(irefwr,*) 'End elmfillcoeff'

      end if
      call erclos ( 'elmfillcoeff' )

      end
