      subroutine elp610first ( wh, lambda, phi, jinteg, iweigo, iqmat, &
                               itang, jcartsav, icooro )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.10 date 12-02-2010 Fill dphidx completely
!        version  3.9  date 06-12-2009 Debug information
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a linear line element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jinteg, iweigo, iqmat, itang, jcartsav, icooro
      double precision wh(m), lambda(m), phi(1:2,1:m)

!     icooro         o    if 1 the coordinates must be filled
!     iqmat          o    if >=1 the qmat matrix must be filled
!     itang          o    Defines how the first tangential vector must be
!                         defined
!                         0: arbitrary
!                         1: By a line through points p1 and p2
!     iweigo         o    if 1 the weights must be filled
!     jcartsav       o    Help variable to store jcart temporarily
!     jinteg         o    jtrans/100, indicates the direction of integration
!                         0: usual direction
!                         >0: x_jinteg direction
!     lambda         o    Array containing the linear basis functions in the
!                         integration points
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     wh             o    Contains the integration weights for a unit element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=22) text

!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELPLAMBDA1D    Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!        h   =  x2 - x1
!        d phi_1/dx = -1/h        d phi_2/dx = 1/h
!
!        Newton Cotes rule:  /_e f(x) dx =|h|/2 [ f(x^1)+f(x^2) ]
!
!        The matrix qmat is filled as follows:
!        qmat(.,3) = (x2-x1)/||x2-x1||   t vector
!        ej is unity vector in direction of mallest component of |qmat(.,3)|
!        qmat(.,2) = (qmat(.,3) x ej) / ||qmat(.,3) x ej||
!        qmat(.,1) = qmat(.,2) x qmat(.,3) ! normal vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Linear 1D line element' /
! ======================================================================
!
      call eropen ( 'elp610first' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp610first'

      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      jinteg = jtrans/100
      iweigo = mod(iweigh,10)
      icooro = mod(icoor,10)
      iqmat  = iweigh/10
      itang  = icoor/10
      jcartsav = jcart
      if ( jcart==2 .or. jcart==3 ) then

!     --- jcart = 2, 3  (Polar co-ordinates)

         jcartsav = 3

      else if ( jcart/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( irule<1 .or. irule>5 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( irule, 1770, text, 1 )
         irule = 1

      end if
      if ( irule==1 .and. m/=2 .or. irule==2 .and. m/=1 .or. &
           irule==3 .and. m/=2 .or. irule==4 .and. m/=3 .or. &
           irule==5 .and. m/=3 )then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, irule, 1424, text, 0 )

      end if

!     --- Fill array lambda and wh

      if ( irule==1 ) then

!     --- irule = 1  Newton Cotes rule

         lambda(1) = 0d0
         lambda(2) = 1d0
         wh(1:2) = 0.5d0

      else

!     --- irule>2  one-point Gauss rule

         call elplambda1d ( lambda, wh, text )

      end if  ! ( irule==1 )

      if ( iphi==1 ) then

!     --- Fill array phi

         phi(1,1:m) = 1d0 - lambda(1:m)
         phi(2,1:m) = lambda(1:m)

      end if  ! ( iphi==1 )

1000  call erclos ( 'elp610first' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp610first'

      end if  ! ( debug )

      end
