      subroutine elp616 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, tang, phider )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.2  date 18-08-2004 Adaptation for solids
!        version  3.1  date 04-07-2003 Remove common celwrk
!                                      forcheck warnings
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for
!     quadratic line element in R2
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP616 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecpack
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(3)
      double precision coor2d(2,*), x(3,2), w(m), dphidx(3,m)
      double precision xgauss(m,2), phi(3,m), tang(m,2), qmat(2,2,3), &
                       phider(3,3)

!     coor2d           i    array of length ndim x npointcac containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         o    Array of length n x m x ndim  containing the
!     derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     qmat           o    transformation matrix global -> local coordinates
!     tang           o    In this array the tangential vector is stored
!                         tang(j,i) contains the j th component of the
!                         tangential vector in the i-th integration point
!     w              o    array of length m containing the weights for
!                         integration
!     x             i/o   array of length 2 x 3 containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2)
!                          i              i
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx, dy, delta, t1, t2
      integer i, j, k,l
      logical strait, compqmat
      integer ne, iqmat
      double precision xorg(3), exi(2), eeta(2), norm, &
                       dxdxi(3,2,2), ealpha(2), ebeta(2), h, alpha(3), &
                       wh(3), xksi(3), yksi(3)
      save wh, alpha

!     alpha          Contains the values of the basis functions in the
!                    integration points of the reference element
!     compqmat       if true qmat must be evaluated
!     delta          Jacobian of the element in an integration point
!     dx             "distance in x-direction" between two nodes
!     dxdxi          Work array of length 3x2x2 to store the values of dx/dxi, &
!                    dx/deta, dy/dxi and dy/deta according to:
!                            k
!                    dx/dxi(x ) = dxdxi(k,1,1)
!                             k
!                    dx/deta(x ) = dxdxi(k,1,2)
!                            k
!                    dy/dxi(x ) = dxdxi(k,2,1)
!                             k
!                    dy/deta(x ) = dxdxi(k,2,2)
!     dy             "distance in y-direction" between two nodes
!     ealpha         Work array for the computation of qmat
!     ebeta          Work array for the computation of qmat
!     eeta           Work array for the computation of qmat
!     exi            base vector in xi direction
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iqmat          if 1 the qmat matrix must be filled
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     ne
!     norm           Array of length ndim * npointcac containing the normal
!                    in the nodal points
!     phider         Contains the derivatives of the basis functions in the
!                    reference element
!                                                j
!                    phider(i,j) = d phi  / ds (x )
!                                       i
!     strait         Indicates whether the element is straight (true) or not
!     t1             First component to compute the tangential vector
!     t2             Second component to compute the tangential vector
!     wh             Contains the weights in the reference element
!     xksi           Contains dx / d ksi in the integration points
!     xorg           Contains the original coordinates of the first node
!     yksi           Contains dy / d ksi in the integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INSTOP         Stop the program
!     PRINRL1        Print 2d real vector
!     QLCSYS         Perform transformation from 3D co-ordinates to local (2D)
!                    co-ordinates
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!    1424   m does not correspond to irule
!    1769   Type of co-ordinate system incorrect
!    1770   Type of integration rule incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp616' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp616'

      end if

      iqmat  = iweigh/10

      if ( iqmat.eq.3 ) then

!     --- iqmat = 3

         compqmat = .true.

!        --- reset iqmat!!

         iqmat = 0

      else

!     --- iqmat # 3

         compqmat = .false.

      end if
      if ( debug ) write(irefwr,*) 'iqmat, compqmat ', iqmat, compqmat

      if ( ifirst.eq.0 ) then

!     --- ifirst = 0  First time element group is called
!                     Compute some initial values

         if ( jcart.eq.2 .or. jcart.eq.3 ) then

!        --- jcart = 2, 3  (Polar co-ordinates)

         else if ( jcart.ne.1 ) then

!        --- Error 1769: Co-ordinate system not available

            call errint ( jcart-1, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errchr ( 'Quadratic 2D line element', 1 )
            call errsub ( 1769, 4, 0, 1 )

         end if
         if ( irule.lt.1 .or. irule.gt.4 ) then

!        --- Error 1770: Integration rule not available

            call errint ( irule, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errchr ( 'Quadratic 2D line element', 1 )
            call errwar ( 1770, 4, 0, 1 )
            irule = 1

         end if
         if ( irule.eq.1 .and. m.ne.3 .or. irule.eq.2 .and. m.ne.1 .or. &
              irule.eq.3 .and. m.ne.2 .or. irule.eq.4 .and. m.ne.3 )then

!        --- Error 1424: m and integration rule do not match

            call errint ( m, 1 )
            call errint ( irule, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errchr ( 'Quadratic 2D line element', 1 )
            call errsub ( 1424, 5, 0, 1 )

         end if
         if ( irule.eq.2 ) then

!        --- 1-point Gauss integration

            wh(1) = 1d0
            alpha(1) = 0.5d0

         else if ( irule.eq.3 ) then

!        --- 2-point Gauss integration

            wh(1) = 0.5d0
            wh(2) = 0.5d0
            alpha(1) = alam4(1)
            alpha(2) = alam4(2)

         else if ( irule.eq.4 ) then

!        --- 3-point Gauss integration

            do i = 1, 3
               wh(i) = wgs5(i)
               alpha(i) = alam5(i)
            end do

         end if
         if ( irule.gt.1 ) then

!        --- irule > 1, Gauss integration

            if ( iphi.gt.0 ) then

!           --- Compute values of basis functions phi in integration points

               do j = 1, m
                  phi(1,j) = (1d0-alpha(j)) * ( 2d0*(1d0-alpha(j))-1d0 )
                  phi(2,j) = 4d0 * alpha(j) * (1d0-alpha(j))
                  phi(3,j) = alpha(j) * (2d0*alpha(j)-1d0)
               end do

            end if

         else

!        --- irule = 1  Newton Cotes integration

            wh(1) = 1d0/6d0
            wh(2) = 4d0*wh(1)
            wh(3) = wh(1)
            alpha(1) = 0d0
            alpha(2)  = .5d0
            alpha(3) = 1d0

            do j = 1, m
               phider(1,j) = -(4d0*(1d0-alpha(j))-1d0)
               phider(2,j) = 4d0*(1d0-2d0*alpha(j))
               phider(3,j) = 4d0*alpha(j)-1d0
            end do

            if ( iphi.gt.0 ) then

!           --- Compute values of basis functions phi in integration points

               do j = 1, 3
                  do i = 1, 3
                     phi(i,j) = 0d0
                  end do
                  phi(j,j) = 1d0
               end do

            end if

         end if
         if ( irule.gt.1 .or. ideriv.eq.1 ) then

!        --- ideriv = 1,  compute derivatives

            do j = 1, m
               phider(1,j) = -(4d0*(1d0-alpha(j))-1d0)
               phider(2,j) = 4d0*(1d0-2d0*alpha(j))
               phider(3,j) = 4d0*alpha(j)-1d0
            end do

         end if

      end if
      if ( debug ) write(irefwr,*) 'irule, ideriv, iphi, icoor ', &
         irule, ideriv, iphi, icoor

!     --- Variable part

      if ( mod(icoor,10).ge.1 ) then

!     --- icoor = 1   Fill co-ordinates of nodes in array x

         if ( icoor.ne.2 ) then

!        --- icoor # 2, standard

            do k = 1, 2
               do i = 1, 3
                   x(i,k) = coor2d(k,index1(i))
               end do
            end do

         else

!        --- icoor = 2, special

            do k = 1, 2
               do i = 1, 3
                  x(i,k) = coor2d(k,i)
               end do
            end do

         end if

      end if

      if ( itype.ge.210 .and.itype.le.212 ) then

!     --- Boundary elements for solids, elements are considered to be curved

         strait=.false.

      else

!     --- Standard case, check if element is curved

         strait = abs( 2d0*x(2,1)-x(1,1)-x(3,1) ) .lt. &
                  sqreps * ( abs(x(1,1) ) + abs ( x(3,1) ) ) .and. &
                  abs( 2d0*x(2,2)-x(1,2)-x(3,2) ) .lt. &
                  sqreps * ( abs(x(1,2) ) + abs ( x(3,2) ) )

      end if  ! ( itype.ge.210 .and.itype.le.212 ) then

      if ( debug ) write(irefwr,*) 'strait ', strait

      if ( irule.eq.1 ) then

!     --- irule = 1, Newton cotes rule

         if ( strait ) then

!        --- straight element

            dx = x(3,1)-x(1,1)
            dy = x(3,2)-x(1,2)
            delta = sqrt ( dx*dx+dy*dy )
            if ( delta.le.0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
               go to 1000
            end if
            w(1) = delta
            w(2) = delta
            w(3) = delta
            if ( icoor.ge.10 ) then

!           --- compute tangential vector

               t1 = dx/delta
               t2 = dy/delta
               do i = 1, 3
                  tang(i,1) = t1
                  tang(i,2) = t2
               end do

            end if

            do j = 1, m
               do k = 1, 2
                  do l = 1, 2
                     dxdxi(j,k,l) = 0d0
                     do i = 1, 3
                        dxdxi(j,k,l) = dxdxi(j,k,l)+x(i,k)*phider(i,j)
                     end do
                  end do
               end do
            end do

         else

!        --- curved element

            if ( debug ) call prinrl1 ( x, 2, 3, 'x')
            dx = -1.5d0*x(1,1)+2d0*x(2,1)-0.5d0*x(3,1)
            dy = -1.5d0*x(1,2)+2d0*x(2,2)-0.5d0*x(3,2)
            delta = sqrt(dx*dx+dy*dy)
            if ( debug ) write(irefwr,*) 'dx, dy, delta ',dx, dy, delta
            if ( delta.le.0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
               go to 1000
            end if
            if ( icoor.ge.10 ) then
               tang(1,1) = dx/delta
               tang(1,2) = dy/delta
            end if
            w(1) = 2d0*delta
            dx = 0.5d0 * ( x(3,1)-x(1,1) )
            dy = 0.5d0 * ( x(3,2)-x(1,2) )
            delta = sqrt(dx*dx+dy*dy)
            if ( delta.le.0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
               go to 1000
            end if
            if ( icoor.ge.10 ) then
               tang(2,1) = dx/delta
               tang(2,2) = dy/delta
            end if
            w(2) = 2d0 * delta
            dx = 0.5d0 * x(1,1)-2d0*x(2,1)+1.5d0*x(3,1)
            dy = 0.5d0 * x(1,2)-2d0*x(2,2)+1.5d0*x(3,2)
            delta = sqrt(dx*dx+dy*dy)
            if ( delta.le.0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
               go to 1000
            end if
            if ( icoor.ge.10 ) then
               tang(3,1) = dx/delta
               tang(3,2) = dy/delta
            end if
            w(3) = 2d0 * delta

            do j = 1, m
               do k = 1, 2
                  do l = 1, 2
                     dxdxi(j,k,l) = 0d0
                     do i = 1, 3
                        dxdxi(j,k,l) = dxdxi(j,k,l)+x(i,k)*phider(i,j)
                     end do
                  end do
               end do
            end do

         end if
         if ( igausp.eq.1 ) then

            do i = 1, 3
               xgauss(i,1) = x(i,1)
               xgauss(i,2) = x(i,2)
            end do

         end if

      else

!     --- Gauss integration
!          compute dx/dksi, dy/dksi

         do j = 1, m
            xksi(j) = 0d0
            do i = 1, 3
               xksi(j) = xksi(j)+x(i,1)*phider(i,j)
            end do
            yksi(j) = 0d0
            do i = 1, 3
               yksi(j) = yksi(j)+x(i,2)*phider(i,j)
            end do

         end do

!        --- Compute Jacobian, store temporarily in w

         do i = 1, m
            w(i) = sqrt(xksi(i)**2+yksi(i)**2)
            end do
         do i = 1, m
            if ( w(i).le.0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
               go to 1000
            end if

         end do
         if ( icoor.ge.10 ) then

!        --- compute tangential vector

            do i = 1, m
               tang(i,1) = xksi(i)/w(i)
               tang(i,2) = yksi(i)/w(i)
            end do

         end if

!        --- Fill array xgauss

         do j = 1, m
            xgauss(j,1) = 0d0
            do i = 1, 3
               xgauss(j,1) = xgauss(j,1)+x(i,1)*phi(i,j)
            end do
            xgauss(j,2) = 0d0
            do i = 1, 3
               xgauss(j,2) = xgauss(j,2)+x(i,2)*phi(i,j)
            end do

         end do

      end if
      if ( ideriv.eq.1 ) then

!     --- Compute derivatives

         do i = 1, 3
            do j = 1, m
               dphidx(i,j) = phider(i,j) / w(j)
            end do
         end do

      end if

!     --- Compute weights

      do i = 1, m
         w(i) = w(i)*wh(i)
      end do
      if ( jcart.gt.1 ) then

!     --- axisymmetric co-ordinates  correct weights

         if ( irule.gt.1 ) then
            do i = 1, m
               w(i) = w(i)*xgauss(i,1)*pi2
            end do
         else
            do i = 1, m
               w(i) = w(i)*x(i,1)*pi2
            end do
         end if

      end if

      if ( compqmat ) then

!     --- Fill array qmat
!         Loop over integration points

         do k = 1, m

!        --- Define e_xi = x,xi / ||x,xi||

            norm = 1d0/sqrt(dxdxi(k,1,1)**2+dxdxi(k,2,1)**2)
            exi(1) = dxdxi(k,1,1)*norm
            exi(2) = dxdxi(k,2,1)*norm

!           --- Define e_eta = e_3 x e_xi / ||e_3 x e_xi||

            norm = 1d0/sqrt(exi(1)**2+exi(2)**2)
            eeta(1) = -exi(2)*norm
            eeta(2) = exi(1)*norm

!           --- e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||

            ealpha(1) = 5d-1*(exi(1)+eeta(1))
            ealpha(2) = 5d-1*(exi(2)+eeta(2))
            norm = 1d0/sqrt(ealpha(1)**2+ealpha(2)**2)
            ealpha(1) = ealpha(1)*norm
            ealpha(2) = ealpha(2)*norm

!           --- e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||

            norm = 1d0/sqrt(ealpha(1)**2+ealpha(2)**2)
            ebeta(1) =-ealpha(2)*norm
            ebeta(2) = ealpha(1)*norm

!           --- e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!               e_2(l) = srqt(2)/2 ( e_alpha + e_beta )

            h = 0.5d0*sqrt(2d0)
            do j = 1,2
               qmat(2,j,k) = h*(ealpha(j)-ebeta(j))
               qmat(1,j,k) = h*(ealpha(j)+ebeta(j))
            end do

        end do

      end if

1000  if ( ierror.ne.0 ) call instop
      call erclos ( 'elp616' )
      if ( debug ) write(irefwr,*) 'End elp616'
      end
