      subroutine elp620xloc ( x, ndim, iqmat, qmat, xloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-06-2013 New definition qmat
!        version  1.0  date 02-02-2006
!
!   copyright (c) 2006-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill local coordinates for triangle in 3d
!     Depending on iqmat, also qmat is filled
! **********************************************************************
!
!                       KEYWORDS
!
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, iqmat
      double precision x(3,ndim), qmat(3,ndim,m), xloc(3,2)

!     iqmat          i    Defines if the the qmat matrix must be filled
!                         0: qmat must not be filled
!                         1: qmat must be filled by subroutine qclsys
!                         2: qmat must be filled assuming a linear
!                            transformation i.e. the surface elements
!                            are in a plane
!                         3: qmat must be filled assuming a general
!                            transformation
!     ndim           i    Dimension of the space.
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     x              i    Conains the coordinates of the element
!     xloc           o    co-ordinates of vertices in local (xi-zeta) coordinate
!                         system
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision et1(ndim), et2(ndim), &
                       normal(ndim), ezeta(ndim), exi(ndim)

!     et1            First tangential vector
!     et2            Second tangential vector
!     exi            base vector in xi direction
!     ezeta          base vector in zeta direction
!     i              Counting variable
!     normal         Vector containg the normal on the element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELLOCCOOR      Compute local coordinates of a surface element in 3d given
!                    the transformation by base vectors
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp620xloc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp620xloc'
         write(irefwr,1) 'ndim, iqmat', ndim, iqmat
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute e_t1 and e_t2

      et1(:) = x(2,:)-x(1,:)
      et2(:) = x(3,:)-x(1,:)
      et1 = et1/sqrt(dot_product(et1,et1))
      et2 = et2/sqrt(dot_product(et2,et2))

!     --- Compute normal n

      call eloutr ( et1, et2, normal )
      normal = normal/sqrt(dot_product(normal,normal))

!     --- Compute e_zeta

      if ( iqmat<=5 ) then

!     --- iqmat = 5, e_zeta = e_t1

         ezeta = et1

      else

!     --- iqmat>5, e_zeta = projection of e_i onto surface

         i = iqmat-5
         et1 = 0d0
         et1(i) = 1d0
         ezeta = et1-normal(i)*normal
         ezeta = ezeta/sqrt(dot_product(ezeta,ezeta))

      end if

!     --- Compute  e_xi = (e_zeta x n) / ||e_zeta x n||

      call eloutr ( normal, ezeta, exi )
      exi = exi/sqrt(dot_product(exi,exi))

!     --- Compute local coordinates

      call elloccoor ( x, xloc, ezeta, exi, 3 )
      if ( iqmat/=0 ) then

!     --- iqmat # 0, fill array qmat with e_zeta, e_xi and n

         do i = 1, m
            qmat(1,1:ndim,i) = normal(1:ndim) ! n
            qmat(2,1:ndim,i) = ezeta(1:ndim)  ! t1
            qmat(3,1:ndim,i) = exi(1:ndim)    ! t2
         end do  ! i = 1, m

      end if

1000  call erclos ( 'elp620xloc' )
      if ( debug ) then

!     --- Debug information

         call prinrl2 ( x, 3, 3, 'coordinates' )
         call prinrl2 ( xloc, 3, 2, 'local-coordinates' )
         call prinrl ( et1, 3, 'et1' )
         call prinrl ( et2, 3, 'et2' )
         call prinrl ( ezeta, 3, 'ezeta' )
         call prinrl ( exi, 3, 'exi' )
         call prinrl ( normal, 3, 'normal' )
         if ( iqmat/=0 ) call prinrl2( qmat, 3, 3, 'qmat' )
         write(irefwr,*) 'End elp620xloc'

      end if

      end
