      subroutine elp621first ( xg, phi, phiksi, intrule, jcartorg, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-06-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a bilinear quadrilateral
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     linear
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intrule, jcartorg
      double precision xg(m,2), phi(n,m), phiksi(n,m,2), weight(m)

!     intrule        i    Actual integration rule
!     jcartorg       i    Parameter jcart_orig as defined in jcart
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phiksi         o    Array of length 4 x m x 2 containing the values of the
!                         derivatives of the basis functions in the Gauss points
!                         in the reference element.
!     weight         o    Contains the weights for the integration rule in the
!                         reference element
!     xg             o    Array of length 4x2 containing the co-ordinates of the
!                         Gauss points in the reference element for a four point
!                         Gauss rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      character (len=22) text

!     k              Counting variable
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Bilinear quadrilateral' /
! ======================================================================
!
      call eropen ( 'elp621first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp621first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jcartorg==2 ) then

!     --- jcartorg = 2  (Axi-symmetric co-ordinates)

      else if ( jcartorg/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcartorg-1, 1769, text, 0 )

      end if

      if ( intrule<1 .or. intrule>3 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      if ( intrule==1 .and. m/=4 .or. &
           intrule==2 .and. m/=1 .or. &
           intrule==3 .and. m/=4 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

!     --- Fill array xsi and eta for reference element (xg)

      xg = 0d0
      select case ( intrule )

         case(1)

!        --- intrule = 1  Four point Newton Cotes rule

            xg(2:3,1) = 1d0
            xg(3:4,2) = 1d0
            weight(1:m) = 0.25d0

         case(2)

!        --- intrule = 2  One point Gauss rule

            xg(1,1:2) = 0.5d0
            weight(1) = 1d0

         case(3)

!        --- intrule = 3  Four point Gauss rule

            xg(1:2,1) = alam4(1:2)
            xg(3,1) = alam4(2)
            xg(4,1) = alam4(1)
            xg(1:2,2) = alam4(1)
            xg(3:4,2) = alam4(2)
            weight(1:m) = 0.25d0

      end select  ! case ( intrule )

!     --- Compute phi

      do k = 1, m
         phi(1,k) = ( 1d0-xg(k,1) ) * ( 1d0-xg(k,2) )  ! lambda1(xi) lambda1(eta)
         phi(2,k) = xg(k,1) * ( 1d0-xg(k,2) )          ! lambda2(xi) lambda1(eta)
         phi(3,k) = xg(k,1) * xg(k,2)                  ! lambda2(xi) lambda2(eta)
         phi(4,k) = ( 1d0-xg(k,1) ) * xg(k,2)          ! lambda1(xi) lambda2(eta)
      end do

!     --- Compute dphi/dxi

      do k = 1, m
         phiksi(1,k,1) = -(1d0-xg(k,2))
         phiksi(2,k,1) =   1d0-xg(k,2)
         phiksi(3,k,1) =       xg(k,2)
         phiksi(4,k,1) =      -xg(k,2)
         phiksi(1,k,2) = -(1d0-xg(k,1))
         phiksi(2,k,2) =      -xg(k,1)
         phiksi(3,k,2) =       xg(k,1)
         phiksi(4,k,2) =   1d0-xg(k,1)
      end do

1000  call erclos ( 'elp621first' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( phiksi(1,1,1), m, 4, 'phiksi' )
         call prinrl1 ( phiksi(1,1,2), m, 4, 'phieta' )
         write(irefwr,*) 'End elp621first'

      end if  ! ( debug )

      end
