      subroutine elp621intpol ( coor, index1, phi, xcoord, ycoord, &
                                xsi0, eta0, computephi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-08-2000 Adaptation error message
!        version  1.0  date 05-04-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the values of the basis function in a specific point
!    Bilinear quadrilateral
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*), phi(*), xcoord, ycoord, xsi0, eta0
      integer index1(*)
      logical computephi

!     computephi     i    if true the basis functions in the point must be
!                         computed
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eta0           i    Value of eta in point
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 4 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     xsi0           i    Value of xsi in point
!     ycoord         i    y-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(4), y(4), determinant, b0, b1, b2, b3, xsi1, &
                       gradfmin1(2,2), distance, fnewton(2), gradf(2,2), &
                       eta1, a0, a1, a2, a3
      integer j, iter

!     a0             constant
!     a1             constant
!     a2             constant
!     a3             constant
!     b0             constant
!     b1             constant
!     b2             constant
!     b3             constant
!     determinant    jacobian of element
!     distance       distance between two quantities
!     eta1           New value of eta
!     fnewton        Contains the values of the funcion on which the Newton
!                    process is carried out
!     gradf          Gradient of function in Newton process
!     gradfmin1      Inverse of gradf
!     iter           Iteration sequence number
!     j              Counting variable
!     x              array containing the x-coordinates of the vertices
!     xsi1           New value of xsi
!     y              array containing the y-coordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-diemsional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The four vertices define four bi-linear basis functions phi_i(x,y)
!    In order to compute these basis functions a transformation to
!    a standard square is performed (see elp621)
!
!    The bilinear interpolation is defined by:
!
!    x = x_i (1-xsi)(1-eta) + x_j xsi(1-eta) + x_k xsi eta + x_l (1-xsi)eta
!    y = y_i (1-xsi)(1-eta) + y_j xsi(1-eta) + y_k xsi eta + y_l (1-xsi)eta
!
!    Hence in point (x,y) we have
!         f(1) =  a0 + a1 xsi + a2 eta + a3 xsi eta = 0
!         f(2) =  b0 + b1 xsi + b2 eta + b3 xsi eta = 0
!    a0 = x_i-x; a1 = x_j-x_i; a2 = x_l-x_i; a3 = x_i-x_j+x_k-x_l
!    b0 = x_i-x; b1 = x_j-x_i; b2 = x_l-x_i; b3 = x_i-x_j+x_k-x_l
!    This is a non-linear system of equations that can be solved by Newton:
!
!    F' ^(-1) (xsi^k+1 - xsi^k) = -f(1)
!             (eta^k+1 - eta^k) = -f(2)
!
!    With F' = ( df(1)/dxsi, df(2)/dxsi )  =  (a1 + a3 eta, b1 + b3 eta )
!              ( df(1)/deta, df(2)/deta )  =  (a2 + a3 xsi, b2 + b3 xsi )
!
!         F' ^(-1) = 1/det ( df(2)/deta,-df(2)/dxsi )
!                          (-df(1)/deta, df(1)/dxsi )
!
!    The starting value is (xsi,eta) = (0.5,0.5)
!
!    The basis functions are defined by:
!
!    phi_1 = (1-xsi) (1-eta)
!    phi_2 =    xsi  (1-eta)
!    phi_3 =    xsi     eta
!    phi_4 = (1-xsi)    eta
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do j = 1, 4
         x(j) = coor(1,index1(j))
         y(j) = coor(2,index1(j))
      end do

!     --- Compute factors for bilinear interpolation
!         x = a0 + a1 xsi + a2 eta + a3 xsi eta

      a0 = x(1)-xcoord
      b0 = y(1)-ycoord
      a1 = x(2)-x(1)
      b1 = y(2)-y(1)
      a2 = x(4)-x(1)
      b2 = y(4)-y(1)
      a3 = x(1)-x(2)+x(3)-x(4)
      b3 = y(1)-y(2)+y(3)-y(4)

!     --- Perform Newton iteration

      xsi0 = 0.5d0
      eta0 = 0.5d0
      distance = 1d0
      iter = 0
      do while ( distance.gt.1d-6 .and. iter.le.5 )

!     --- Stop if the distance is less than 1d-3 or number of
!         iterations is 5

         iter = iter+1
         fnewton(1) = a0+a1*xsi0+a2*eta0+a3*xsi0*eta0
         fnewton(2) = b0+b1*xsi0+b2*eta0+b3*xsi0*eta0
         gradf(1,1) = a1+a3*eta0
         gradf(1,2) = b1+b3*eta0
         gradf(2,1) = a2+a3*xsi0
         gradf(2,2) = b2+b3*xsi0
         determinant = (gradf(1,1)*gradf(2,2)- &
                        gradf(1,2)*gradf(2,1))
         determinant = 1d0/determinant
         if ( abs(determinant).le.0d0 ) then
            call eropen ( 'elp621intpol' )
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
            call prinin(index1,4,'node numbers')
            call prinrl2(x,4,2,'co-ordinates')
            call erclos ( 'elp621intpol' )
            go to 1000
         end if
         gradfmin1(1,1) = determinant*gradf(2,2)
         gradfmin1(1,2) =-determinant*gradf(1,2)
         gradfmin1(2,1) =-determinant*gradf(2,1)
         gradfmin1(2,2) = determinant*gradf(1,1)
         xsi1 = xsi0 - gradfmin1(1,1)*fnewton(1) - &
                       gradfmin1(2,1)*fnewton(2)
         eta1 = eta0 - gradfmin1(1,2)*fnewton(1) - &
                       gradfmin1(2,2)*fnewton(2)
         distance = (xsi1-xsi0)**2 + (eta1-eta0)**2
         xsi0 = xsi1
         eta0 = eta1

      end do
      if ( computephi ) then

!     --- computephi = true, evaluate the basis functions

         phi(1) = (1d0-xsi0)*(1d0-eta0)
         phi(2) =      xsi0 *(1d0-eta0)
         phi(3) =      xsi0 *     eta0
         phi(4) = (1d0-xsi0)*     eta0

      end if

1000  end
