      subroutine elp623curved ( iweigo, x, index1, xgauss, w, &
                                dxdxi, dphidx, dphidxi, dpsidx, &
                                psixi, comppsi, iqmat, dxidx, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 12-08-2013 New parameter list
!        version  1.0  date 25-02-2009
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for biquadratic
!     quadrilateral (curved element)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, index1(9), iqmat
      double precision x(9,ndim), xgauss(m,ndim), w(m), &
                       dxdxi(m,3,3), dphidx(9,m,ndim), dphidxi(9,9,2), &
                       dpsidx(4,9,2), psixi(4,9,2), &
                       dxidx(9,2,3), weight(m)
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dphidxi       i/o   Array containing dphi_i/dxi in a point
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     dxdxi          o    work array of length mx3x3 to store the values of
!                         dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                         dx/dxi(x_k) = dxdxi(k,1,1)
!                         dx/deta(x_k) = dxdxi(k,1,2)
!                         dy/dxi(x_k) = dxdxi(k,2,1)
!                         dy/deta(x_k) = dxdxi(k,2,2) etc.
!     dxidx          o    work array of length 9x2x3 to store the values of
!                         dxi/dx, dxi/dy, dxi/dz and deta/dx ... according to:
!                         dxi/dx(x^k) = dxdxi(k,1,1)  dxi/dy(x^k) = dxdxi(k,1,2)
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iqmat          i    if >0 the qmat matrix must be filled
!     iweigo         i    if 1 the weights must be filled
!     psixi          i    Array of length 9 x m x 2 containing the values of the
!                         derivatives of the basis functions in the Gauss
!                         points in the reference element
!     w              o    array of length m containing the weights for
!                         integration
!     weight         i    Contains the weights for the integration rule in the
!                         reference element
!     x              o    array of length 9 x ndim containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x_i = x (i,1);  y_i = x (i,2);
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, j, jsign, ksign
      double precision delta(m)

!     delta          Work array of length 9 to store the values of the Jacobian
!                    in the nodes
!     i              Counting variable
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     ksign          Help parameter to compute sign of the Jacobian
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!       To compute the derivatives of the basis functions, the jacobian
!       of the element and the weights, the following definitions are used:
!
!            9
!       x = sum   x  phi
!           ij=1   ij   ij
!
!       phi  (xi,eta) = theta (xi) theta (eta)
!          ij                 i           j
!
!
!       theta (s) = 2 ( s-1/2 ) ( s-1 )
!            1
!
!       theta (s) = -4s ( s-1 )
!            2
!
!       theta (s) = 2s ( s-1/2 )
!            3
!
!       d theta /ds = 4s-3
!              1
!
!       d theta /ds = -8s+4
!              2
!
!       d theta /ds = 4s-1
!              3
!
!            |  x          y    |
!            |   xi        xi |
!        J  =|                  |
!            |  x          y    |
!            |   eta        eta |
!
!                         9    k
!        d x / dxi   =  sum  x    d phi / d xi
!           i      j     k=1   i        k       j
!
!
!                          -1
!        grad    phi   =  J   grad        phi
!            xy     i             xi eta    i
!
!
!   In case of 3D elements the following formulas are used:
!                   a
!   x = sum phi (xi ) x
!        i     i        i
!
!          a                   a
!   dx/dxi = sum d phi / d xi  x
!              i       i          i
!
!   Jacobian = sqrt(d1^2+d2^2+d3^2)
!
!          i+1       1    i+2       2      i+2       1   i+1       2
!   di = dx   / d xi   dx   / d xi  -  dx   / d xi  dx   / d xi
!
!     In case of 3D elements we use the following formulae:
!
!     x = sum phi (xi,eta) x    dx/dxi = sum dphi /dxi x   dx/deta=...
!     -    i     i          -i    -         i      i      -i
!
!             a                      a                         a
!     dphi /dx = dphi / dxi  dxi/dx   +  dphi / deta  deta/dx
!         i          i                         i
!                                                                        -1
!     | dxi/dx   deta/dx   dzeta/dx  |   | dx/dxi   dy/dxi   dz/dxi |
!     | dxi/dy   deta/dy   dzeta/dy  | = | dx/deta   dy/deta   dz/deta |
!     | dxi/dz   deta/dz   dzeta/dz  |   | dx/dzeta  dy/dzeta  dz/dzeta|
!
!     zeta is choosen perpendicular to the xi-eta direction, which implies
!     that we may choose:
!
!     dx / dzeta = dx / dxi times  dx / deta / || dx/dxi x dx/deta ||
!      -            -                -              -         - &
!
!     where times or x is the outer product
!
!     The Jacobian of the dx/dxi matrix is given by:
!
!     J = dx/dzeta ( dy/dxi dz/deta - dy/deta dz/dxi )
!       + dy/dzeta ( dz/dxi dx/deta - dz/deta dx/dxi )
!       + dz/dzeta ( dx/dxi dy/deta - dx/deta dy/dxi )
!
!     and since dx/dzeta before normalization is given by
!
!         dx/dzeta = dy/dxi dz/deta - dy/deta dz/dxi
!         dy/dzeta = dz/dxi dx/deta - dz/deta dx/dxi
!         dz/dzeta = dx/dxi dy/deta - dx/deta dy/dxi
!
!     we see that the normalization factor is equal to
!
!        sqrt(     ( dy/dxi dz/deta - dy/deta dz/dxi )^2
!                + ( dz/dxi dx/deta - dz/deta dx/dxi )^2
!                + ( dx/dxi dy/deta - dx/deta dy/dxi )^2 ) = J
!
!      The derivatives dxi/dx and so on follow from the inverse, &
!      which in this 3x3 case leads to:
!
!      dxi/dx = ( dy/deta  dz/dzeta - dy/dzeta dz/deta  ) / J
!      dxi/dy = ( dy/dzeta dz/dxi  - dy/dxi  dz/dzeta ) / J
!      dxi/dz = ( dy/dxi  dz/deta  - dy/deta  dz/dxi  ) / J
!      deta/dx = ( dz/deta  dx/dzeta - dz/dzeta dx/deta  ) / J
!      deta/dy = ( dz/dzeta dx/dxi  - dz/dxi  dx/dzeta ) / J
!      deta/dz = ( dz/dxi  dx/deta  - dz/deta  dx/dxi  ) / J
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp623curved' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp623curved'
         call prinrl1 ( dphidxi(1,1,1), m, 9, 'dpdxi' )
         call prinrl1 ( dphidxi(1,1,2), m, 9, 'dpdeta' )
         call prinrl1 ( x, ndim, 9, 'x' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- weights, or derivatives must be computed
!         Compute Jacobian in the Gauss points

         do j = 1, ndim
            do i = 1, m
               dxdxi(i,j,1) = ddot(9, x(1,j), 1, dphidxi(1,i,1), 1)
               dxdxi(i,j,2) = ddot(9, x(1,j), 1, dphidxi(1,i,2), 1)
            end do
         end do
         if ( debug ) then
            call prinrl1 ( dxdxi(1,1,1), ndim, m, 'dxdxi' )
            call prinrl1 ( dxdxi(1,1,2), ndim, m, 'dydxi' )
         end if  ! ( debug )

!        --- Fill Jacobian in delta

         if ( ndim==3 .and. iqmat==0 ) then

!        --- Standard 3D case

            do i = 1, m
               dxdxi(i,1,3) = dxdxi(i,2,1)*dxdxi(i,3,2)- &
                              dxdxi(i,3,1)*dxdxi(i,2,2)
               dxdxi(i,2,3) = dxdxi(i,3,1)*dxdxi(i,1,2)- &
                              dxdxi(i,1,1)*dxdxi(i,3,2)
               dxdxi(i,3,3) = dxdxi(i,1,1)*dxdxi(i,2,2)- &
                              dxdxi(i,2,1)*dxdxi(i,1,2)
            end do
            do k = 1, m
               delta(k) = sqrt( dxdxi(k,1,3)**2 + &
                                dxdxi(k,2,3)**2 + &
                                dxdxi(k,3,3)**2 )
               if ( delta(k)<=0d0 ) then
                  if ( isubr==0 ) then
                     call errint ( ielem, 1 )
                     call errint ( itype, 2 )
                     call errint ( ielgrp, 3 )
                     call errsub ( 230, 3, 0, 0 )
                  else
                     call errint ( ielgrp, 1 )
                     call errint ( ielem, 2 )
                     call errchr ( 'surface', 1 )
                     call errsub ( 1540, 2, 0, 1 )
                  end if
                  call prinin(index1,9,'node numbers')
                  call prinrl2(x,9,3,'co-ordinates')
                  go to 1000
               end if
            end do

         else

!        --- Standard 2D case

            do k = 1, m
               delta(k) = dxdxi(k,1,1)*dxdxi(k,2,2) - &
                          dxdxi(k,1,2)*dxdxi(k,2,1)
            end do

         end if

!        --- Check sign of Jacobian

         ksign = nint ( sign ( 1d0,delta(1) ) )
         do k = 2, m
            jsign = nint ( sign ( 1d0, delta(k) ) )
            if ( ksign/=jsign ) then
               if ( isubr==0 ) then
                  call errint ( ielem, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 230, 3, 0, 0 )
               else
                  call errint ( ielgrp, 1 )
                  call errint ( ielem, 2 )
                  call errchr ( 'surface', 1 )
                  call errsub ( 1540, 2, 0, 1 )
               end if
               call prinin(index1,9,'node numbers')
               call prinrl2(x,9,2,'co-ordinates')
               go to 1000
            end if

         end do
         if( ierror/=0 ) go to 1000

      end if

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         w(1:m) = weight(1:m)*abs(delta(1:m))

         if ( jcart==2 .or. jcart==3 ) then

!        --- Axi-symmetric co-ordinates

            w = w * xgauss(1:m,1)
            if ( jcart==3 ) w = w*pi2

         end if  ! ( jcart==2 .or. jcart==3 )

      end if  ! ( iweigo==1 )

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         if ( debug ) &
            write(irefwr,1) 'ideriv, ndim, iqmat', ideriv, ndim, iqmat
         if ( ndim==3 .and. iqmat==0 ) then

!        --- 3D case

            do k = 1, m
               dxdxi(k,1,3) = dxdxi(k,1,3)/delta(k)
               dxdxi(k,2,3) = dxdxi(k,2,3)/delta(k)
               dxdxi(k,3,3) = dxdxi(k,3,3)/delta(k)
            end do
            do k = 1, m
               dxidx(k,1,1) = (dxdxi(k,2,2)*dxdxi(k,3,3) - &
                               dxdxi(k,3,2)*dxdxi(k,2,3))/delta(k)
               dxidx(k,1,2) = (dxdxi(k,3,2)*dxdxi(k,1,3) - &
                               dxdxi(k,1,2)*dxdxi(k,3,3))/delta(k)
               dxidx(k,1,3) = (dxdxi(k,1,2)*dxdxi(k,2,3) - &
                               dxdxi(k,2,2)*dxdxi(k,1,3))/delta(k)
               dxidx(k,2,1) = (dxdxi(k,2,3)*dxdxi(k,3,1) - &
                               dxdxi(k,3,3)*dxdxi(k,2,1))/delta(k)
               dxidx(k,2,2) = (dxdxi(k,3,3)*dxdxi(k,1,1) - &
                               dxdxi(k,1,3)*dxdxi(k,3,1))/delta(k)
               dxidx(k,2,3) = (dxdxi(k,1,3)*dxdxi(k,2,1) - &
                               dxdxi(k,2,3)*dxdxi(k,1,1))/delta(k)
            end do
            do j = 1, 3
               do k = 1, m
                  do i = 1, 9
                     dphidx(i,k,j) = dphidxi(i,k,1)*dxidx(k,1,j) + &
                                     dphidxi(i,k,2)*dxidx(k,2,j)
                  end do
               end do
            end do

         else

!        --- curved = true, curved element

            do k = 1, m
               do i = 1, 9
                  dphidx(i,k,1) = ( dphidxi(i,k,1)*dxdxi(k,2,2) - &
                            dphidxi(i,k,2)*dxdxi(k,2,1) ) / delta(k)
                  dphidx(i,k,2) = (-dphidxi(i,k,1)*dxdxi(k,1,2) + &
                            dphidxi(i,k,2)*dxdxi(k,1,1) ) / delta(k)
               end do
            end do

            if ( comppsi ) then

!           --- Compute psi derivatives

               do k = 1, m
                  do i = 1, 4
                     dpsidx(i,k,1) = ( psixi(i,k,1)*dxdxi(k,2,2) - &
                                       psixi(i,k,2)*dxdxi(k,2,1)) &
                                       /delta(k)
                     dpsidx(i,k,2) = ( -psixi(i,k,1)*dxdxi(k,1,2) + &
                                        psixi(i,k,2)*dxdxi(k,1,1)) &
                                       /delta(k)
                  end do
               end do

            end if  ! ( comppsi )

         end if

      end if  ! ( ideriv==1 )

1000  call erclos ( 'elp623curved' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp623curved'

      end if  ! ( debug )

      end
