      subroutine elp624first ( intrule, dpdxi, phi, psi, &
                               comppsi, weight, lambda )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-08-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for extended quadratic triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intrule
      double precision lambda(3,m), weight(m), &
                       dpdxi(7,m,2), phi(7,m), psi(3,m)
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     dpdxi          o    contains derivatives in reference element for gauss
!                         rule.
!                         dpdxi(i,k,l) = d phi_i  / d xi_l (x^k)
!     intrule        i    Actual integration rule
!     lambda         o    Array containing the linear basis functions in the
!                         integration points
!     phi            o    array of length 6 x m containing the values of the
!                         basis functions in the gauss points in the sequence:
!                                k
!                         phi (xg ) = phi (i,k)
!                            i
!                         array phi is only filled when ifirst = 0 and gauss
!                         integration is applied
!     psi            o    array of length 3 x m containing the values of the
!                         linear basis functions in the gauss points
!     weight         o    Contains the weights for the integration rule in the
!                         reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i
      character (len=27) text
      double precision h, phicen, dlamde, dlamdk

!     dlamde         eta derivative of lamc
!     dlamdk         xi derivative of lamc
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     k              Counting variable
!     phicen         Value of alam1*alam2*alam3 in integration point
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Extended quadratic triangle' /
! ======================================================================
!
      call eropen ( 'elp624first' )
      debug = .false. .and. ioutp>= 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp624first'

      end if  ! ( debug )
      if ( ierror/= 0 ) go to 1000

      if ( jcart<1 .or. jcart>3 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>4 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if

      if ( intrule==1 .and. m/=7 .or. &
           intrule==2 .and. m/=4 .or. &
           intrule==3 .and. m/=7 .or. &
           intrule==4 .and. m/=7 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

      select case (intrule)

      case(1)

!     --- intrule = 1 : 6 pts Newton Cotes
!         fill d phi / d xsi and  d phi / d eta

         lambda = 0d0
         do i = 1, 3
            lambda(i,2*i-1) = 1d0
         end do  ! i = 1, 3
         lambda(1:2,2) = 0.5d0
         lambda(1,6) = 0.5d0
         lambda(2:3,4) = 0.5d0
         lambda(3,6) = 0.5d0
         h = 1d0 / 3d0
         lambda(1:3,7) = h
         h = 1d0/15d0
         do i = 2, 6, 2
            weight(i) = h
         end do
         h = h*0.025d0
         do i = 1, 5, 2
            weight(i) = h
         end do
         weight(7) = 9d0 * h

      case(2)

!     --- intrule = 2 : 4 pts gauss
!         fill d phi / d xsi and  d phi / d eta

         lambda(1:3,1:m) = lambda(1:3,1:m)
         weight(1:m) = wgs2(1:m)

      case(3, 4)

!     --- intrule = 3: 7pts gauss, fill d phi / d xsi and  d phi / d eta

         lambda(1:3,1:m) = alam3(1:3,1:m)
         weight(1:m) = wgs3(1:m)

      end select  ! case (intrule)

!     --- Compute phi

      do k = 1, m
         phicen = lambda(1,k)*lambda(2,k)*lambda(3,k)
         phi(7,k) = 27d0 * phicen
         do i = 1, 3
            phi(2*i-1,k) = lambda(i,k) * (2d0*lambda(i,k)-1d0) + &
                           3d0*phicen
         end do
         phi(2,k) = 4d0 * lambda(1,k) * lambda(2,k) - 12d0*phicen
         phi(4,k) = 4d0 * lambda(2,k) * lambda(3,k) - 12d0*phicen
         phi(6,k) = 4d0 * lambda(3,k) * lambda(1,k) - 12d0*phicen
      end do  ! k = 1, m

!     --- fill d phi / d xsi and  d phi / d eta

      do k = 1, m
         dlamdk = lambda(3,k) * (lambda(1,k)-lambda(2,k))
         dpdxi(1,k,1) = -4d0*lambda(1,k)+1d0+3d0*dlamdk
         dpdxi(3,k,1) =  4d0*lambda(2,k)-1d0+3d0*dlamdk
         dpdxi(5,k,1) =  3d0*dlamdk
         dpdxi(2,k,1) =  4d0*(lambda(1,k)-lambda(2,k))-12d0*dlamdk
         dpdxi(4,k,1) =  4d0* lambda(3,k)-12d0*dlamdk
         dpdxi(6,k,1) = -4d0* lambda(3,k)-12d0*dlamdk
         dpdxi(7,k,1) =  27d0*dlamdk
         dlamde = lambda(2,k) * (lambda(1,k)-lambda(3,k))
         dpdxi(1,k,2) = -4d0*lambda(1,k)+1d0+3d0*dlamde
         dpdxi(3,k,2) =  3d0*dlamde
         dpdxi(5,k,2) =  4d0*lambda(3,k)-1d0+3d0*dlamde
         dpdxi(4,k,2) =  4d0* lambda(2,k)-12d0*dlamde
         dpdxi(2,k,2) = -4d0* lambda(2,k)-12d0*dlamde
         dpdxi(6,k,2) =  4d0*(lambda(1,k)-lambda(3,k))-12d0*dlamde
         dpdxi(7,k,2) =  27d0*dlamde
      end do  ! k = 1, m

      if ( comppsi ) then

!     --- comppsi true, hence fill array psi

         psi(1:3,1:m) = lambda(1:3,1:m)

      end if  ! ( comppsi )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp624first'

      end if  ! ( debug )
      call erclos ( 'elp624first' )

      end
