      subroutine elp627 ( coor2d, x, w, dphidx, xgauss, phi, index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 03-02-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for linear triangle
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP627 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(3)
      double precision coor2d(ndim,*), x(6,ndim), w(*), &
                       dphidx(3,m,ndim), xgauss(m,ndim), phi(3,m)

!     coor2d     i     array of length 2 x number of nodal points containing
!                    the co-ordinates of the nodal points with respect to
!                    the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);
!                     i                 i
!     dphidx   o     Array of length 3 x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,1,1);  d phi / dy = dphidx (i,1,2);
!                         i                             i
!     index1   i     Array of length 3 containing the nodal point numbers
!                    of the element
!     phi      o     Array of length 3 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     w        o     array of length m containing the weights for the
!                    numerical integration
!     x       i/o    array of length 3 x 2 containing the x- and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision h, rc, xmid(3,ndim), &
                       delta, e(3,2), wh(4), dlamdx(3,2)
      integer i, k, iweigo, iqmat, intrule, jtransfo, &
              jcartorg, ndimloc
      save iweigo, iqmat, jtransfo, jcartorg, ndimloc, wh

!     delta          Jacobian of the element
!     dlamdx         Contains derivatives of linear basis functions
!     e              Work array of length 3 x 2, to store the factors
!                    e^ij See method
!     h              Help variable to store part of a weight
!     i              Counting variable
!     intrule        Actual intergration rule
!     iqmat          if 1 the qmat matrix must be filled
!     iweigo         if 1 the weights must be filled
!     jcartorg       Parameter jcart_orig as defined in jcart
!     jtransfo       Parameter jtransform as defined in jcart
!     k              Counting variable
!     ndimloc        Local value of ndim (usually ndim, except is jtransfo>0: 2)
!     rc             Help variable to store part of a weight for axisymmetric
!                    co-ordinates
!     wh             Contains the weights in the reference element
!     xmid           coordinates of mid-side points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP627FIRST    Fill element independent quantities for a non-conforming
!                    linear triangle
!     ELP627JACOB    Compute the Jacobian of a non-conforming linear triangle
!     ELPCOOR        fill co-ordinates of nodes in array x
!     ELPGAUSS       Fill Gauss coordinates
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij        i+1    i+2    i+1
!        e   =  (-1)    ( x    - x    )   cyclic: i=1,2,3 ;   j=1,2
!                          j      j
!                    32  11    12 31
!        delta   =  e   e   - e  e
!                          ij
!        d phi  / dx   =  e   /  delta
!             i     j
!                                        |delta|       1     2     3
!        Newton Cotes rule:  / f(x) dx = -------- [ f(x )+f(x )+f(x ) ]
!                            e              6
!     In case of 3D elements we transform the 3d element to a 2d element
!     according to the following rules
!
!     In case iqmat<6 there is no special xi or zeta direction
!     if iqmat>5 the zeta (=1) direction is the projection of the
!     iqmat-5 axis to the surface
!     The mapping to local coordinates is as follows:
!     e_t1 = (x2-x1)/||x2-x1||,   e_t2 = (x1-x3)/||x1-x3||
!     n = ( e_t1 x e_t2 ) / ||e_t1 x e_t2||
!     if iqmat<6 then: e_zeta = e_t1
!     else e_zeta is projection of e_i onto surface with i = iqmat-5:
!     e_zeta = ( e_i - (e_i,n) n ) / ||.||
!     e_xi = (e_zeta x n) / ||e_zeta x n||
!     The local coordinates are defined by:
!     x = x_1 + alpha e_zeta  beta e_xi
!     Hence x_i = x_1 _ alpha_i e_zeta  beta_i e_xi
!     and alpha_i, beta_i are the solution of
!     (e_zeta e_xi)^T (e_zeta e_xi) (alpha_i beta_i)^T = &
!        (e_zeta e_xi)^T (x_i - x_1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp627' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp627'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      intrule = mod(irule,100)
      n = 3
      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         call elp627first ( iweigo, iqmat, jcartorg, jtransfo, &
                            ndimloc, phi, intrule, wh )

      end if  ! ( ifirst==0 )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ielem', ielem
         write(irefwr,1) 'icoor, ndim, iqmat, ndimloc, m, n, intrule', &
                          icoor, ndim, iqmat, ndimloc, m, n, intrule
         write(irefwr,1) 'jtransfo, igausp, ideriv, iweigo, ndimloc', &
                          jtransfo, igausp, ideriv, iweigo, ndimloc
         call prinin ( index1, 3, 'index1' )

      end if

!     --- Store coordinates in x

      call elpcoor ( icoor, coor2d, index1, x, 2*n, ndim, 2*n )
      xmid(1,:) = x(2,:)
      xmid(2,:) = x(4,:)
      xmid(3,:) = x(6,:)

      if ( debug ) call prinrl2 ( x, 6, ndim, 'coordinates' )

!     --- gauss points must be filled

      call elpgauss ( ndimloc, intrule, xmid, xgauss, phi )
      if ( debug ) call prinrl2 ( xgauss, m, 2, 'x-gauss' )

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  Compute derivatives or iweigo = 1: weights
!         Compute the quantities e^ij and delta

         call elp627jacob ( e, x, delta, 0, index1, ielem, &
                            ielgrp, itype )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,2) 'delta', delta

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         i = 2
         do k = 1,2
            dlamdx(1,k) = e(1,i) / delta
            dlamdx(3,k) = e(3,i) / delta
            dlamdx(2,k) = -dlamdx(1,k) - dlamdx(3,k)
            i = 1
         end do

         dphidx(1,1,1:2) = -2d0*dlamdx(3,1:2)
         dphidx(2,1,1:2) = -2d0*dlamdx(1,1:2)
         dphidx(3,1,1:2) = -2d0*dlamdx(2,1:2)

         do i = 1, 3
            do k = 1, 2
               dphidx(i,2:m,k) = dphidx(i,1,k)
            end do  ! k = 1, 2
         end do  ! i = 1, 3
         if ( debug ) then

!        --- Debug information

            write(irefwr,2) 'dphidx', dphidx(1,1,1), dphidx(2,1,1), &
                                      dphidx(3,1,1)
            write(irefwr,2) 'dphidy', dphidx(1,1,2), dphidx(2,1,2), &
                                      dphidx(3,1,2)

         end if

      end if

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         if ( intrule==1 .and. jcartorg==2 ) then

!        --- jcartorg = 2  Axi-symmetric co-ordinates

            h = abs ( delta ) / 24d0
            rc = x(1,1)+x(2,1)+x(3,1)
            do k = 1, 3
               w(k) = h*(x(k,1)+rc) * pi2
            end do

         else

!        --- Standard case

            w(1:m) = abs(delta)*wh(1:m)

            if ( jcartorg==2 ) then

!           --- axisymmetric co-ordinates

               w(1:m) = w(1:m) * xgauss(1:m,1) * pi2

            end if  ! ( intrule>1 .and. jcartorg==2 ) then

         end if  ! ( intrule==1 .and. jcartorg==2 )

         if ( debug ) call prinrl ( w, m, 'weights' )

      end if

      if ( ndim>2 .and. iqmat/=1 .and. jtransfo==0 ) then

!     --- Reset coordinates

         do k = 1, ndim
            do i = 1, 6
               x(i,k) = coor2d(k,index1(i))
            end do
         end do

      end if

1000  call erclos ( 'elp627' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp627'

      end if  ! ( debug )

      end
