      subroutine elp630intpol ( coor, index1, phi, xcoord, ycoord, &
                                zcoord )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-12-2001 Correction array x
!        version  1.0  date 20-05-2000
!
!   copyright (c) 2000-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for linear tetrahedron
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP630 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(4)
      double precision coor(3,*), phi(*), xcoord, ycoord, zcoord

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 4 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     ycoord         i    y-coordinate of point
!     zcoord         i    z-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision djacob, x(4,3), e(4,0:3), delta(4,3)
      integer i, j, k, iven, indloc(5), ind1(4)

!     delta          Work array of length 4 x 3, to store the factors delta^ij
!                    see elp630
!     djacob         jacobian of element
!     e              Work array of length 4 x 3, to store the factors e^ij
!                    see elp630
!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of 1 to 5
!     ind1           Array of length 4 to store the cyclic permutation of 1 - 3
!     iven           help parameter to compute (-1)
!     j              Counting variable
!     k              Counting variable
!     x              array containing the co-ordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The four vertices define four linear basis functions phi_i(x,y)
!    with:
!    phi_i(x,y) = a_0 + a_1 x + a_2 y + a_3 z
!    a_1 = dphi_i/dx;  a_2 = dphi_i/dy;  a_3 = dphi_i/dz
!    The computation of these derivatives is copied from elp630
!    a_0 = 1 - a_1 x_i - a_2 y_i - a_3 z_i, because phi_i(x_i,y_i) = 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indloc / 2,3,4,1,2 /, ind1 / 2,3,1,2 /
! ======================================================================
!
      do k = 1,3
         do i = 1,4
            x(i,k) = coor(k,index1(i))
         end do
      end do

      do k = 1,3
         do i = 1,4
            delta(i,k) = x(indloc(i),k) - x(i,k)
         end do
      end do

      iven = -1
      do i = 1, 4
         do k = 1, 3
            e (i,k) = iven * ( delta ( indloc(i), ind1(k) ) * &
                      delta ( indloc(i+1), ind1(k+1) ) - &
                      delta ( indloc(i), ind1(k+1) ) * &
                      delta ( indloc(i+1), ind1(k) ) )
         end do
         iven = - iven
      end do

      djacob = - delta(1,1) * e(1,1) - delta(1,2) * e(1,2) &
               - delta(1,3) * e(1,3)

!     --- Check if djacob is non-zero

      if ( abs(djacob).le.0d0 ) then
         call eropen ( 'elp630intpol' )
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 105, 3, 0, 0 )
         write ( irefwr, 100 ) ((x(k,j),j=1,3), k=1,4)
100      format ( ( 1x, ss, 3(1pe14.5e2) ) )
         call erclos ( 'elp630intpol' )
         go to 1000
      end if

      do j = 1, 4
         e(j,1) = e(j,1)/djacob
         e(j,2) = e(j,2)/djacob
         e(j,3) = e(j,3)/djacob
         e(j,0) = 1d0-e(j,1)*x(j,1)-e(j,2)*x(j,2)-e(j,3)*x(j,3)
         phi(j) = e(j,0)+e(j,1)*xcoord+e(j,2)*ycoord+e(j,3)*zcoord
      end do

1000  end
