      subroutine elp632intpol ( coor, index1, phi, xcoord, ycoord, &
                                zcoord, xsi0, eta0, zeta0 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-09-2013 Use prsolmat3
!        version  1.0  date 25-05-2000
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the values of the basis function in a specific point
!    Trilinear hexahedron
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), phi(*), xcoord, ycoord, zcoord, &
                       xsi0, eta0, zeta0
      integer index1(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eta0           i    Value of eta in point
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 4 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     xsi0           i    Value of xsi in point
!     ycoord         i    y-coordinate of point
!     zcoord         i    z-coordinate of point
!     zeta0          i    Value of zeta in point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(8), y(8), z(8), xsi1, eta1, zeta1, &
                       distance, fnewton(3), gradf(3,3), phixsi(8), &
                       phieta(8), phizeta(8), dxsi(3)
      integer j, iter
      logical singular

!     distance       distance between two quantities
!     dxsi           Difference in xsi values
!     eta1           New value of eta
!     fnewton        Contains the values of the funcion on which the Newton
!                    process is carried out
!     gradf          Gradient of function in Newton process
!     iter           Iteration sequence number
!     j              Counting variable
!     phieta         array of length 8 to store the values of dphi/deta
!     phixsi         array of length 8 to store the values of dphi/dxsi
!     phizeta        array of length 8 to store the values of dphi/dzeta
!     singular       if true the matrix is singular
!     x              x co-ordinate of point
!     xsi1           New value of xsi
!     y              y co-ordinate of point
!     z              z co-ordinate of point
!     zeta1          New value of zeta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRSOLMAT3      Solve system of 3 linear equations S x = f
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2186   The matrix to be inverted is singular
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The four vertices define four bi-linear basis functions phi_i(x,y)
!    In order to compute these basis functions a transformation to
!    a standard square is performed (see elp632)
!
!    The trilinear interpolation is defined by:
!
!    x = sum x_i phi_i(xsi,eta,zeta)
!    y = sum y_i phi_i(xsi,eta,zeta)
!    z = sum z_i phi_i(xsi,eta,zeta)
!
!    Hence in point (x,y,z) we have
!         f(1) = sum x_i phi_i(xsi,eta,zeta)-x = 0
!         f(2) = sum y_i phi_i(xsi,eta,zeta)-y = 0
!         f(3) = sum z_i phi_i(xsi,eta,zeta)-z = 0
!
!    This is a non-linear system of equations that can be solved by Newton:
!
!    F' ^(-1) (xsi^k+1 - xsi^k) = -f(1)
!             (eta^k+1 - eta^k) = -f(2)
!             (zeta^k+1 - zeta^k) = -f(3)
!
!    With F' = ( df(1)/dxsi, df(2)/dxsi, df(3)/dxsi )
!              ( df(1)/deta, df(2)/deta, df(3)/deta )
!              ( df(1)/dzeta, df(2)/dzeta, df(3)/dzeta )
!
!    The starting value is (xsi,eta,zeta) = (0.5,0.5,0.5)
!
!    The basis functions are defined by:
!
!    phi_1 = (1-xsi) (1-eta) (1-zeta)  phi_5 = (1-xsi) (1-eta) zeta
!    phi_2 =    xsi  (1-eta) (1-zeta)  phi_6 =   xsi  (1-eta) zeta
!    phi_3 =    xsi     eta  (1-zeta)  phi_7 =   xsi     eta  zeta
!    phi_4 = (1-xsi)    eta  (1-zeta)  phi_8 = (1-xsi)   eta  zeta
!
!
!    df(1)/dxsi = sum x_i d phi_i/dxsi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp632intpol' )
      do j = 1, 8
         x(j) = coor(1,index1(j))
         y(j) = coor(2,index1(j))
         z(j) = coor(3,index1(j))
      end do

!     --- Perform Newton iteration

      xsi0 = 0.5d0
      eta0 = 0.5d0
      zeta0 = 0.5d0
      distance = 1d0
      iter = 0
      do while ( distance>1d-6 .and. iter<=5 )

!     --- Stop if the distance is less than 1d-3 or number of
!         iterations is 5

         iter = iter+1
         phi(1) = (1d0-xsi0)*(1d0-eta0)*(1d0-zeta0)
         phi(2) =      xsi0 *(1d0-eta0)*(1d0-zeta0)
         phi(3) =      xsi0 *     eta0 *(1d0-zeta0)
         phi(4) = (1d0-xsi0)*     eta0 *(1d0-zeta0)
         phi(5) = (1d0-xsi0)*(1d0-eta0)*     zeta0
         phi(6) =      xsi0 *(1d0-eta0)*     zeta0
         phi(7) =      xsi0 *     eta0 *     zeta0
         phi(8) = (1d0-xsi0)*     eta0 *     zeta0
         phixsi(1) = -(1d0-eta0)*(1d0-zeta0)
         phixsi(2) =  (1d0-eta0)*(1d0-zeta0)
         phixsi(3) =       eta0 *(1d0-zeta0)
         phixsi(4) = -     eta0 *(1d0-zeta0)
         phixsi(5) = -(1d0-eta0)*     zeta0
         phixsi(6) =  (1d0-eta0)*     zeta0
         phixsi(7) =       eta0 *     zeta0
         phixsi(8) = -     eta0 *     zeta0
         phieta(1) = -(1d0-xsi0)*(1d0-zeta0)
         phieta(2) = -     xsi0 *(1d0-zeta0)
         phieta(3) =      xsi0  *(1d0-zeta0)
         phieta(4) = (1d0-xsi0) *(1d0-zeta0)
         phieta(5) = -(1d0-xsi0)*     zeta0
         phieta(6) = -     xsi0 *     zeta0
         phieta(7) =      xsi0  *     zeta0
         phieta(8) = (1d0-xsi0) *     zeta0
         phizeta(1) =-(1d0-xsi0)*(1d0-eta0)
         phizeta(2) =-     xsi0 *(1d0-eta0)
         phizeta(3) =-     xsi0 *     eta0
         phizeta(4) =-(1d0-xsi0)*     eta0
         phizeta(5) = (1d0-xsi0)*(1d0-eta0)
         phizeta(6) =      xsi0 *(1d0-eta0)
         phizeta(7) =      xsi0 *     eta0
         phizeta(8) = (1d0-xsi0)*     eta0
         fnewton(1) = ddot ( 8, x, 1, phi, 1 )-xcoord
         fnewton(2) = ddot ( 8, y, 1, phi, 1 )-ycoord
         fnewton(3) = ddot ( 8, z, 1, phi, 1 )-zcoord

!        --- Store the transpose of gradf in order to use the fortan standard

         gradf(1,1) = ddot ( 8, x, 1, phixsi, 1 )
         gradf(2,1) = ddot ( 8, y, 1, phixsi, 1 )
         gradf(3,1) = ddot ( 8, z, 1, phixsi, 1 )
         gradf(1,2) = ddot ( 8, x, 1, phieta, 1 )
         gradf(2,2) = ddot ( 8, y, 1, phieta, 1 )
         gradf(3,2) = ddot ( 8, z, 1, phieta, 1 )
         gradf(1,3) = ddot ( 8, x, 1, phizeta, 1 )
         gradf(2,3) = ddot ( 8, y, 1, phizeta, 1 )
         gradf(3,3) = ddot ( 8, z, 1, phizeta, 1 )

         call prsolmat3 ( gradf, fnewton, dxsi, singular )
         if ( singular ) then

!        --- determinant is 0

            call errsub ( 2186, 0, 0, 0 )
            call prinin ( index1, 8, 'nodal points' )
            go to 1000

         end if
         xsi1 = xsi0 - dxsi(1)
         eta1 = eta0 - dxsi(2)
         zeta1 = zeta0 - dxsi(3)
         distance = (xsi1-xsi0)**2 + (eta1-eta0)**2 + (zeta1-zeta0)**2
         xsi0 = xsi1
         eta0 = eta1
         zeta0 = zeta1

      end do
1000  call erclos ( 'elp632intpol' )

      end
