      subroutine elp633intpol ( coor, index1, phi, xcoord, ycoord, &
                                zcoord )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-09-2013 Use prsolmat3
!        version  1.0  date 25-05-2000
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the values of the basis function in a specific point
!    Trilinear hexahedron
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(3,*), phi(*), xcoord, ycoord, zcoord
      integer index1(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 4 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     ycoord         i    y-coordinate of point
!     zcoord         i    z-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(27), y(27), z(27), xsi1, eta1, zeta1, &
                       distance, fnewton(3), gradf(3,3), phixsi(27), &
                       phieta(27), phizeta(27), dxsi(3), &
                       xsi0, eta0, zeta0, thetaxsi(3), thetaeta(3), &
                       thetazeta(3), dthetadxsi(3), dthetadeta(3), &
                       dthetadzeta(3)
      integer j, iter, index2(8), indxsi(27), indeta(27), indzeta(27), &
              i, k, ih, ih1
      logical singular

!     distance       distance between two quantities
!     dthetadeta     Derivative of one-dimensional basis function in eta
!                    direction
!     dthetadxsi     Derivative of one-dimensional basis function in xsi
!                    direction
!     dthetadzeta    Derivative of one-dimensional basis function in zeta
!                    direction
!     dxsi           Difference in xsi values
!     eta0           Value of eta in previous iteration
!     eta1           New value of eta
!     fnewton        Contains the values of the funcion on which the Newton
!                    process is carried out
!     gradf          Gradient of function in Newton process
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     indeta         eta index for node
!     index2         Array of length icount containing the positions of the
!                    degrees of freedom in the element with respect to usol
!     indxsi         xsi index for node
!     indzeta        zeta index for node
!     iter           Iteration sequence number
!     j              Counting variable
!     k              Counting variable
!     phieta         array of length 8 to store the values of dphi/deta
!     phixsi         array of length 8 to store the values of dphi/dxsi
!     phizeta        array of length 8 to store the values of dphi/dzeta
!     singular       if true the matrix is singular
!     thetaeta       one-dimensional basis function in eta direction
!     thetaxsi       one-dimensional basis function in xsi direction
!     thetazeta      one-dimensional basis function in zeta direction
!     x              x co-ordinate of point
!     xsi0           Value of xsi in previous iteration
!     xsi1           New value of xsi
!     y              y co-ordinate of point
!     z              z co-ordinate of point
!     zeta0          Value of zeta in previous iteration
!     zeta1          New value of zeta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ELP632INTPOL   Compute the values of the basis functions in a given
!                    point for a tri-linear hexahedron
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRSOLMAT3      Solve system of 3 linear equations S x = f
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2186   The matrix to be inverted is singular
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The four vertices define four bi-linear basis functions phi_i(x,y)
!    In order to compute these basis functions a transformation to
!    a standard square is performed (see elp633)
!
!    The trilinear interpolation is defined by:
!
!    x = sum x_i phi_i(xsi,eta,zeta)
!    y = sum y_i phi_i(xsi,eta,zeta)
!    z = sum z_i phi_i(xsi,eta,zeta)
!
!    Hence in point (x,y,z) we have
!         f(1) = sum x_i phi_i(xsi,eta,zeta)-x = 0
!         f(2) = sum y_i phi_i(xsi,eta,zeta)-y = 0
!         f(3) = sum z_i phi_i(xsi,eta,zeta)-z = 0
!
!    This is a non-linear system of equations that can be solved by Newton:
!
!    F' ^(-1) (xsi^k+1 - xsi^k) = -f(1)
!             (eta^k+1 - eta^k) = -f(2)
!             (zeta^k+1 - zeta^k) = -f(3)
!
!    With F' = ( df(1)/dxsi, df(2)/dxsi, df(3)/dxsi )
!              ( df(1)/deta, df(2)/deta, df(3)/deta )
!              ( df(1)/dzeta, df(2)/dzeta, df(3)/dzeta )
!
!    The starting value is (xsi,eta,zeta) = (0.5,0.5,0.5)
!
!    The basis functions are defined by:
!
!    phi_1 = (1-xsi) (1-eta) (1-zeta)  phi_5 = (1-xsi) (1-eta) zeta
!    phi_2 =    xsi  (1-eta) (1-zeta)  phi_6 =   xsi  (1-eta) zeta
!    phi_3 =    xsi     eta  (1-zeta)  phi_7 =   xsi     eta  zeta
!    phi_4 = (1-xsi)    eta  (1-zeta)  phi_8 = (1-xsi)   eta  zeta
!
!
!    df(1)/dxsi = sum x_i d phi_i/dxsi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp633intpol' )
      do j = 1, 27
         x(j) = coor(1,index1(j))
         y(j) = coor(2,index1(j))
         z(j) = coor(3,index1(j))
      end do
      do i = 1, 8
         index2(i) = index1(ivertx14(i))
      end do
      do k = 1, 3
         ih = (k-1)*9
         do j = 1, 3
            ih1 = ih+(j-1)*3
            do i = 1, 3
               indxsi(i+ih1) = i
               indeta(i+ih1) = j
               indzeta(i+ih1) = k
            end do
         end do
      end do

!     --- First estimate for xsi and eta, using the tri-linear interpolation

      call elp632intpol ( coor, index2, phi, xcoord, ycoord, &
                          zcoord, xsi0, eta0, zeta0 )

!     --- Perform Newton iteration

      distance = 1d0
      iter = 0
      do while ( distance>1d-6 .and. iter<=5 )

!     --- Stop if the distance is less than 1d-6 or number of
!         iterations is 5

         iter = iter+1
         thetaxsi(1) = (1d0-xsi0)*(2d0*(1d0-xsi0)-1d0)
         thetaxsi(2) = 4d0*(1d0-xsi0)*xsi0
         thetaxsi(3) = xsi0*(2d0*xsi0-1d0)
         thetaeta(1) = (1d0-eta0)*(2d0*(1d0-eta0)-1d0)
         thetaeta(2) = 4d0*(1d0-eta0)*eta0
         thetaeta(3) = eta0*(2d0*eta0-1d0)
         thetazeta(1) = (1d0-zeta0)*(2d0*(1d0-zeta0)-1d0)
         thetazeta(2) = 4d0*(1d0-zeta0)*zeta0
         thetazeta(3) = zeta0*(2d0*zeta0-1d0)
         dthetadxsi(1) = -(4d0*(1d0-xsi0)-1d0)
         dthetadxsi(2) = 4d0*(1d0-2d0*xsi0)
         dthetadxsi(3) = 4d0*xsi0-1d0
         dthetadeta(1) = -(4d0*(1d0-eta0)-1d0)
         dthetadeta(2) = 4d0*(1d0-2d0*eta0)
         dthetadeta(3) = 4d0*eta0-1d0
         dthetadzeta(1) = -(4d0*(1d0-zeta0)-1d0)
         dthetadzeta(2) = 4d0*(1d0-2d0*zeta0)
         dthetadzeta(3) = 4d0*zeta0-1d0
         do i = 1, 27
            phi(i) = thetaxsi(indxsi(i))*thetaeta(indeta(i))* &
                     thetazeta(indzeta(i))
            phixsi(i) = dthetadxsi(indxsi(i))*thetaeta(indeta(i))* &
                     thetazeta(indzeta(i))
            phieta(i) = thetaxsi(indxsi(i))*dthetadeta(indeta(i))* &
                     thetazeta(indzeta(i))
            phizeta(i) = thetaxsi(indxsi(i))*thetaeta(indeta(i))* &
                     dthetadzeta(indzeta(i))
         end do
         fnewton(1) = ddot ( 27, x, 1, phi, 1 )-xcoord
         fnewton(2) = ddot ( 27, y, 1, phi, 1 )-ycoord
         fnewton(3) = ddot ( 27, z, 1, phi, 1 )-zcoord

!        --- Store the transpose of gradf in order to use the fortan standard

         gradf(1,1) = ddot ( 27, x, 1, phixsi, 1 )
         gradf(2,1) = ddot ( 27, y, 1, phixsi, 1 )
         gradf(3,1) = ddot ( 27, z, 1, phixsi, 1 )
         gradf(1,2) = ddot ( 27, x, 1, phieta, 1 )
         gradf(2,2) = ddot ( 27, y, 1, phieta, 1 )
         gradf(3,2) = ddot ( 27, z, 1, phieta, 1 )
         gradf(1,3) = ddot ( 27, x, 1, phizeta, 1 )
         gradf(2,3) = ddot ( 27, y, 1, phizeta, 1 )
         gradf(3,3) = ddot ( 27, z, 1, phizeta, 1 )

         call prsolmat3 ( gradf, fnewton, dxsi, singular )
         if ( singular ) then

!        --- determinant is 0

            call errsub ( 2186, 0, 0, 0 )
            go to 1000

         end if
         xsi1 = xsi0 - dxsi(1)
         eta1 = eta0 - dxsi(2)
         zeta1 = zeta0 - dxsi(3)
         distance = (xsi1-xsi0)**2 + (eta1-eta0)**2 + (zeta1-zeta0)**2
         xsi0 = xsi1
         eta0 = eta1
         zeta0 = zeta1

      end do
1000  call erclos ( 'elp633intpol' )

      end
