      subroutine elp637 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          psi )
! ======================================================================
!
!        programmer    Tijmen Gunther & David Bessems
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 10-06-2013 Use subroutines
!        version  1.4  date 09-03-2013 Correction declaration w
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for extended
!     quadratic tetrahedron compared to the standard SEPRAN numbering.
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP637 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduledebug
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(15)
      double precision coor2d(3,*), x(15,3), w(*), dphidx(15,m,3), &
                       xgauss(m,3), phi(15,m), psi(4,m)

!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!                         x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i)
!                          i                 i                 i
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     index1        i     Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi           o     array of lenght n * m, values of shape
!                         functions in integration points
!     psi           o     array of length 4 x m containing the values of the
!                         linear basis functions in the gauss points
!     w             o     array of length m containing the weights for
!                         integration
!     x            i/o    array of length 3 x 10 containing the x- y- and
!                         z-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                          i              i              i
!     xgauss        o     Double precision array of length m x ndim in which the
!                         co-ordinates of the integration points are stored
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision weight, alamnp(4,15), lamc, g(m), alam(4,m), &
                       l123, l124, l134, l234, lam, som, &
                       dpdksi(15,m), dpdeta(15,m), dpdzeta(15,m), &
                       detinv(m,3,3), dxdksi(15,3,3)
      integer i, k, j, intern(15), imidp(6,2), &
              iweigo, intrule, isign, jsign
      logical comppsi, check
      character (len=30) text
      save imidp, intern, alamnp, iweigo, lam, lamc, &
           l123, l124, l134, l234

!     alam           Array containing values of Lambda in integration points
!     alamnp         Array containing the values of lambda in the nodal points
!                    alambd(i,k) = lambda_i(np_k)
!     check
!     comppsi        if true the linear basis functions must be evaluated
!     detinv         inverse of determinant of dxdksi
!     dpdeta         contains derivatives in reference element for gauss
!                    rule.
!                    dpdeta(i,k) = d phi_i/ d eta (x^k)
!     dpdksi         dpksi(i,k) = d phi_i/ d xi (x^k)
!     dpdzeta        dpdzeta(i,k) = d phi_i/ d zeta (x^k)
!     dxdksi         Derivatives d_x/d_ksi, d_x/d_eta, d_x/d_zeta
!     g              Jacobian of element
!     i              Counting variable
!     imidp          Contains the i,j numbering of the mid side points in the
!                    sequence of intern
!     intern         Internal numbering of the nodes, vertices first
!     intrule        Actual integration rule
!     isign          Help parameters to check the sign of the Jacobian in
!                    the integration points
!     iweigo         if 1 the weights must be filled
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     l123           Help variable corresponding to baricentric coordinate 123
!     l124           Help variable corresponding to baricentric coordinate 124
!     l134           Help variable corresponding to baricentric coordinate 134
!     l234           Help variable corresponding to baricentric coordinate 234
!     lam            Help variable.
!     lamc           Help variable. See pseudo code.
!     som            Help parameter to compute a sum
!     text           String variable containing some text
!     weight         Help variable to compute the weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+2,j+1
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                             ij
!        d lambda  / dx   =  e   /  djacob
!                i     j
!        lambda ( x,y,z ) is linear basis function corresponding to
!                         4-point tetrahedron
!
!        lamc = lambda lambda lambda lambda
!                     1      2      3      4
!
!        --- for vertices i = 1,2,3,4 : by permutation
!
!        phi    = lambda  ( 2 lambda  - 1 ) +3 (lambda lambda lambda + &
!           i           i           i                 i      j      k
!       +         lambda lambda lambda + lambda lambda lambda ) - &
!                       k      l      i        l      i      j
!       +         4 lamc
!
!        --- instead of lambda  ( 2 lambda  - 1 ) also the following
!                              i           i
!            expression can be used:
!            (--- indices by cyclic permutation ---)
!
!            lambda - 2 lambda (lambda +lambda +lambda )
!                  i          i       j       k       l
!
!            because of the relationship:
!
!            lambda + lambda + lambda + lambda = 1
!                  1        2        3        4
!
!        --- for midside points : 1<=i<j<=4
!
!        phi   = 4 lambda  lambda  - 12 ( lambda lambda lambda + &
!           ij           i       j              i      j      k
!       +        lambda lambda lamdba ) + 32 lamc
!                      i      j      l
!
!        --- for midface points : 1<=i<j<k<l<=4
!
!        phi   = 27 lambda lambda lambda  - 108 lamc
!           ijk           i      j      k
!
!        --- centroid tetrahedron
!
!        phi    = 256 lamc
!           c
!
!        --- Integration-rule: Gauss quadrature
!
!        * Gauss rule: / f(x) dx = |djacob| * SUM [ w  f(xg )]
!                      e                             i     i
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data intern / 1, 3, 5, 10, 2, 6, 7, 4, 8, &
                    9, 11, 12, 13, 14, 15/, &
           imidp  / 1, 1, 1, 2, 2, 3, &
                    2, 3, 4, 3, 4, 4 /
      data text / 'Extended quadratic tetrahedron' /
! ======================================================================
!
      intrule = mod(irule,100)
      call eropen ( 'elp637' )

         iweigo = mod(iweigh,10)
         comppsi = jpsi==1
         if ( jcart/=1 ) then

!        --- Error 1769: Co-ordinate system not available

            call errorelvar ( jcart-1, 1769, text, 0 )

         end if
         if ( intrule<1 .or. intrule>3 ) then

!        --- Error 1770: Integration rule not available

            call errorelvar ( intrule, 1770, text, 1 )
            intrule = 1

         end if

         if ( intrule==1 .and. m/=15 .or. &
              intrule==2 .and. m/=11 .or. &
              intrule==3 .and. m/=15 ) then

!        --- Error 1424: m and integration rule do not match

            call errorelvar2 ( m, intrule, 1424, text, 0 )

         end if

!        --- Fill array alamnp in the integration points for newton cotes

         do i = 1, 4
            do k = 1, 10
               alamnp(i,k) = 0d0
            end do
            do j = 11, 14
               alamnp(i,j) = 1d0/3d0
            end do
            alamnp(i,15) = 0.25d0
            alamnp(i,intern(i)) = 1d0
            do k = 1, 6
               if ( imidp(k,1)==i .or. imidp(k,2)==i ) then
                  alamnp(i,intern(k+4)) = 0.5d0
               end if
            end do
         end do
         alamnp(1,13) = 0d0
         alamnp(2,14) = 0d0
         alamnp(3,12) = 0d0
         alamnp(4,11) = 0d0

         if ( intrule==1 ) then

!        --- intrule = 1, Newton Cotes

            do k = 1, 4
               do j = 1, 15
                  alam(k,j) = alamnp(k,j)
               end do
            end do

         else if ( intrule==2 ) then

!        --- intrule = 2, 11 pts gauss rule

            do k = 1, 4
               do j = 1, 11
                  alam(k,j) = alam9(k,j)
               end do
            end do

         else if ( intrule==3 ) then

!        --- intrule = 3, 15 pts gauss rule

            do k = 1, 4
               do j = 1, 15
                  alam(k,j) = alam10(k,j)
               end do
            end do

         end if

!        --- fill derivatives d_phi / d_ksi etc. using alam's

         do k = 1, m
            lam = alam(1,k)*alam(3,k)*alam(4,k)- &
               alam(2,k)*alam(3,k)*alam(4,k)

            dpdksi(1,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(3,k) + &
               alam(1,k)*alam(4,k) - alam(2,k)*alam(3,k) - &
               alam(2,k)*alam(4,k) - alam(3,k)*alam(4,k)) - &
               4d0*alam(1,k) + 1d0
            dpdksi(3,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(3,k) + &
               alam(1,k)*alam(4,k) + alam(3,k)*alam(4,k) - &
               alam(2,k)*alam(4,k) - alam(2,k)*alam(3,k)) + &
               4d0*alam(2,k) - 1d0
            dpdksi(5,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(3,k) - &
               alam(2,k)*alam(3,k))
            dpdksi(10,k) = -4d0*lam + 3d0*(alam(1,k)*alam(4,k) - &
               alam(2,k)*alam(4,k))
            dpdksi(2,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(4,k) + &
               alam(2,k)*alam(3,k) - alam(1,k)*alam(4,k) - &
               alam(1,k)*alam(3,k)) + 4d0*(alam(1,k)-alam(2,k))
            dpdksi(4,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(3,k) - &
               alam(1,k)*alam(3,k) - alam(3,k)*alam(4,k)) + &
               4d0*alam(3,k)
            dpdksi(6,k)  = 32d0*lam + 12*(alam(2,k)*alam(3,k) + &
               alam(3,k)*alam(4,k) - alam(1,k)*alam(3,k)) - &
               4d0*alam(3,k)
            dpdksi(7,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(4,k) + &
               alam(3,k)*alam(4,k) - alam(1,k)*alam(4,k)) - &
               4d0*alam(4,k)
            dpdksi(8,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(4,k) - &
               alam(1,k)*alam(4,k) - alam(3,k)*alam(4,k)) + &
               4d0*alam(4,k)
            dpdksi(9,k)  = 32d0*lam
            dpdksi(11,k) = -108d0*lam + 27d0*(alam(1,k)*alam(3,k) - &
               alam(2,k)*alam(3,k))
            dpdksi(12,k) = -108d0*lam + 27d0*(alam(1,k)*alam(4,k) - &
               alam(2,k)*alam(4,k))
            dpdksi(13,k) = -108d0*lam + 27d0*alam(3,k)*alam(4,k)
            dpdksi(14,k) = -108d0*lam - 27d0*alam(3,k)*alam(4,k)
            dpdksi(15,k) = 256d0*lam

            lam = alam(1,k)*alam(2,k)*alam(4,k)- &
               alam(2,k)*alam(3,k)*alam(4,k)

            dpdeta(1,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(2,k) + &
               alam(1,k)*alam(4,k) - alam(2,k)*alam(3,k) - &
               alam(2,k)*alam(4,k) - alam(3,k)*alam(4,k)) - &
               4d0*alam(1,k) + 1d0
            dpdeta(3,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(2,k) - &
               alam(2,k)*alam(3,k))
            dpdeta(5,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(2,k) + &
               alam(1,k)*alam(4,k) + alam(2,k)*alam(4,k) - &
               alam(3,k)*alam(4,k) - alam(2,k)*alam(3,k)) + &
               4d0*alam(3,k) - 1d0
            dpdeta(10,k) = -4d0*lam + 3d0*(alam(1,k)*alam(4,k) - &
               alam(3,k)*alam(4,k))
            dpdeta(2,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(3,k) + &
               alam(2,k)*alam(4,k) - alam(1,k)*alam(2,k)) - &
               4d0*alam(2,k)
            dpdeta(4,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(3,k) - &
               alam(1,k)*alam(2,k) - alam(2,k)*alam(4,k)) + &
               4d0*alam(2,k)
            dpdeta(6,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(3,k) + &
               alam(3,k)*alam(4,k) - alam(1,k)*alam(2,k) - &
               alam(1,k)*alam(4,k)) + 4d0*(alam(1,k)-alam(3,k))
            dpdeta(7,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(4,k) + &
               alam(3,k)*alam(4,k) - alam(1,k)*alam(4,k)) - &
               4d0*alam(4,k)
            dpdeta(8,k)  = 32d0*lam
            dpdeta(9,k)  = 32d0*lam + 12d0*(alam(3,k)*alam(4,k) - &
               alam(2,k)*alam(4,k) - alam(1,k)*alam(4,k)) + &
               4d0*alam(4,k)
            dpdeta(11,k) = -108d0*lam + 27d0*(alam(1,k)*alam(2,k) - &
               alam(2,k)*alam(3,k))
            dpdeta(12,k) = -108d0*lam - 27d0*alam(2,k)*alam(4,k)
            dpdeta(13,k) = -108d0*lam + 27d0*alam(2,k)*alam(4,k)
            dpdeta(14,k) = -108d0*lam + 27d0*(alam(1,k)*alam(4,k) - &
               alam(3,k)*alam(4,k))
            dpdeta(15,k) = 256d0*lam

            lam = alam(1,k)*alam(2,k)*alam(3,k)- &
               alam(2,k)*alam(3,k)*alam(4,k)

            dpdzeta(1,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(2,k) + &
               alam(1,k)*alam(3,k) - alam(2,k)*alam(3,k) - &
               alam(2,k)*alam(4,k) - alam(3,k)*alam(4,k)) - &
               4d0*alam(1,k) + 1d0
            dpdzeta(3,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(2,k) - &
               alam(2,k)*alam(4,k))
            dpdzeta(5,k)  = -4d0*lam + 3d0*(alam(1,k)*alam(3,k) - &
               alam(3,k)*alam(4,k))
            dpdzeta(10,k) = -4d0*lam + 3d0*(alam(1,k)*alam(2,k) + &
               alam(2,k)*alam(3,k) + alam(1,k)*alam(3,k) - &
               alam(2,k)*alam(4,k) - alam(3,k)*alam(4,k)) + &
               4d0*alam(4,k) - 1d0
            dpdzeta(2,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(3,k) + &
               alam(2,k)*alam(4,k) - alam(1,k)*alam(2,k)) - &
               4d0*alam(2,k)
            dpdzeta(4,k)  = 32d0*lam
            dpdzeta(6,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(3,k) + &
               alam(3,k)*alam(4,k) - alam(1,k)*alam(3,k)) - &
               4d0*alam(3,k)
            dpdzeta(7,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(4,k) + &
               alam(3,k)*alam(4,k) - alam(1,k)*alam(2,k) - &
               alam(1,k)*alam(3,k)) + 4d0*(alam(1,k)-alam(4,k))
            dpdzeta(8,k)  = 32d0*lam + 12d0*(alam(2,k)*alam(4,k) - &
               alam(1,k)*alam(2,k) - alam(2,k)*alam(3,k)) + &
               4d0*alam(2,k)
            dpdzeta(9,k)  = 32d0*lam + 12d0*(alam(3,k)*alam(4,k) - &
               alam(1,k)*alam(3,k) - alam(2,k)*alam(3,k)) + &
               4d0*alam(3,k)
            dpdzeta(11,k) = -108d0*lam - 27d0*alam(2,k)*alam(3,k)
            dpdzeta(12,k) = -108d0*lam + 27d0*(alam(1,k)*alam(2,k) - &
               alam(2,k)*alam(4,k))
            dpdzeta(13,k) = -108d0*lam + 27d0*alam(2,k)*alam(3,k)
            dpdzeta(14,k) = -108d0*lam + 27d0*(alam(1,k)*alam(3,k) - &
               alam(3,k)*alam(4,k))
            dpdzeta(15,k) = 256d0*lam
         end do

      if ( icoor>=1 ) then

!     --- icoor = 1   Fill co-ordinates of nodes in array x

         do k = 1, 3
            do i = 1, 15
               if ( icoor==1 ) x(i,k) = coor2d(k,index1(i))
               if ( icoor==2 ) x(i,k) = coor2d(k,i)
            end do
         end do

      end if

      if ( iphi==1 ) then

!     --- ifirst = 0  first call of element subroutine, and iphi = 1
!         fill array phi

         if ( intrule==1 ) then

!        --- newton cotes

            do k = 1, 15
               do i = 1, 15
                  phi(k,i) = 0d0
               end do
               phi(k,k) = 1d0
            end do

         else
            do i = 1, m
               lamc = alam(1,i)*alam(2,i)*alam(3,i)*alam(4,i)
               L123 = alam(1,i)*alam(2,i)*alam(3,i)
               L124 = alam(1,i)*alam(2,i)*alam(4,i)
               L134 = alam(1,i)*alam(3,i)*alam(4,i)
               L234 = alam(2,i)*alam(3,i)*alam(4,i)

               phi(1,i) = alam(1,i)-2d0*alam(1,i)*(alam(2,i) &
                          +alam(3,i)+alam(4,i)) &
                          +3d0*(L123 + L124 + L134) - 4d0*lamc
               phi(2,i) = 4d0*alam(1,i)*alam(2,i) &
                          -12d0*(L123 + L124) + 32d0*lamc
               phi(3,i) = alam(2,i)-2d0*alam(2,i)*(alam(1,i) &
                          +alam(3,i)+alam(4,i)) &
                          +3d0*(L123 + L124 + L234) - 4d0*lamc
               phi(4,i) = 4d0*alam(2,i)*alam(3,i) &
                          -12d0*(L123 + L234) + 32d0*lamc
               phi(5,i) = alam(3,i)-2d0*alam(3,i)*(alam(1,i) &
                          +alam(2,i)+alam(4,i)) &
                          +3d0*(L123 + L134 + L234) - 4d0*lamc
               phi(6,i) = 4d0*alam(1,i)*alam(3,i) &
                          -12d0*(L123 + L134) + 32d0*lamc
               phi(7,i) = 4d0*alam(1,i)*alam(4,i) &
                          -12d0*(L124 + L134) + 32d0*lamc
               phi(8,i) = 4d0*alam(2,i)*alam(4,i) &
                          -12d0*(L124 + L234) + 32d0*lamc
               phi(9,i) = 4d0*alam(3,i)*alam(4,i) &
                          -12d0*(L134 + L234) + 32d0*lamc
               phi(10,i)= alam(4,i)-2d0*alam(4,i)*(alam(1,i) &
                          +alam(2,i)+alam(3,i)) &
                          +3d0*(L234 + L124 + L134) - 4d0*lamc
               phi(11,i)= 27d0*L123 - 108d0*lamc
               phi(12,i)= 27d0*L124 - 108d0*lamc
               phi(13,i)= 27d0*L234 - 108d0*lamc
               phi(14,i)= 27d0*L134 - 108d0*lamc
               phi(15,i)= 256d0*lamc
            end do

         end if

         if ( comppsi ) then

!        --- Compute linear basis functions

            do i = 1, 4
               do j = 1, m
                  psi(i,j) = alam(i,j)
               end do
            end do

         end if

      end if

      if ( ideriv==1 .or. iweigo==1 ) then

!     --- ideriv = 1  Compute derivatives or iweigo = 1: weights

         do j = 1, 3
            do i = 1, m
               dxdksi(i,1,j) = ddot(15, x(1,j), 1, dpdksi(1,i), 1)
               dxdksi(i,2,j) = ddot(15, x(1,j), 1, dpdeta(1,i), 1)
               dxdksi(i,3,j) = ddot(15, x(1,j), 1, dpdzeta(1,i),1)
            end do
         end do

!        --- fill jacobian in djacob

         do k = 1, m
            detinv(k,1,1) = dxdksi(k,2,2)*dxdksi(k,3,3)- &
                            dxdksi(k,3,2)*dxdksi(k,2,3)
            detinv(k,2,1) = dxdksi(k,3,2)*dxdksi(k,1,3)- &
                            dxdksi(k,1,2)*dxdksi(k,3,3)
            detinv(k,3,1) = dxdksi(k,1,2)*dxdksi(k,2,3)- &
                            dxdksi(k,2,2)*dxdksi(k,1,3)
            g(k) = dxdksi(k,1,1)*detinv(k,1,1)+dxdksi(k,2,1)* &
                   detinv(k,2,1)+dxdksi(k,3,1)*detinv(k,3,1)
         end do

         if ( ideriv==1 ) then

!        --- ideriv = 1,  compute all factors of inverse

            do k = 1,m
               detinv(k,1,2) = dxdksi(k,2,3)*dxdksi(k,3,1)- &
                               dxdksi(k,3,3)*dxdksi(k,2,1)
               detinv(k,2,2) = dxdksi(k,3,3)*dxdksi(k,1,1)- &
                               dxdksi(k,1,3)*dxdksi(k,3,1)
               detinv(k,3,2) = dxdksi(k,1,3)*dxdksi(k,2,1)- &
                               dxdksi(k,2,3)*dxdksi(k,1,1)
               detinv(k,1,3) = dxdksi(k,2,1)*dxdksi(k,3,2)- &
                               dxdksi(k,3,1)*dxdksi(k,2,2)
               detinv(k,2,3) = dxdksi(k,3,1)*dxdksi(k,1,2)- &
                               dxdksi(k,1,1)*dxdksi(k,3,2)
               detinv(k,3,3) = dxdksi(k,1,1)*dxdksi(k,2,2)- &
                               dxdksi(k,2,1)*dxdksi(k,1,2)
            end do

         end if

!        --- check sign of jacobian

         isign = nint( sign (1d0, g(1)) )
         if ( checksign .and. isign<0 ) then
            write(irefwr,*) 'area of element ', ielem, ' is negative'
         end if
         check = .false.
         do i = 1, m
            if ( abs(g(i))<=0d0 ) check = .true.
            jsign = nint(sign(1d0, g(i)))
            if( jsign/=isign ) then
               call eropen ( 'elp637' )
               if ( isubr==0 ) then
                  call errint ( ielem, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 230, 3, 0, 0 )
               else
                  call errint ( ielgrp, 1 )
                  call errint ( ielem, 2 )
                  call errchr ( 'volume', 1 )
                  call errsub ( 1540, 2, 0, 1 )
               end if
               call prinin(index1,15,'node numbers')
               call prinrl2(x,15,3,'co-ordinates')
               call erclos ( 'elp637' )
               go to 1000
            end if
         end do
         if ( check ) then
            call eropen ( 'elp637' )
            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'volume', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(index1,15,'node numbers')
            call prinrl2(x,15,3,'co-ordinates')
            call erclos ( 'elp637' )
            go to 1000
         end if

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         do i = 1, 15
            do k = 1, m
               dphidx(i,k,1) = (dpdksi(i,k)*detinv(k,1,1)+ &
                                dpdeta(i,k)*detinv(k,2,1)+ &
                                dpdzeta(i,k)*detinv(k,3,1))/g(k)
               dphidx(i,k,2) = (dpdksi(i,k)*detinv(k,1,2)+ &
                                dpdeta(i,k)*detinv(k,2,2)+ &
                                dpdzeta(i,k)*detinv(k,3,2))/g(k)
               dphidx(i,k,3) = (dpdksi(i,k)*detinv(k,1,3)+ &
                                dpdeta(i,k)*detinv(k,2,3)+ &
                                dpdzeta(i,k)*detinv(k,3,3))/g(k)
            end do
         end do

      end if

      if ( igausp==1 ) then

!     --- gauss points must be filled

         if ( intrule==1 ) then

!        --- newton cotes rule

            do k = 1, 3
               do i = 1, 15
                  xgauss(i,k)=x(i,k)
               end do
            end do

         else

!        --- gauss rules

            do j = 1, 3
               do k = 1, m
                  som = 0d0
                  do i = 1, 15
                     som = som + phi(i,k) * x(i,j)
                  end do
                  xgauss(k,j) = som
               end do
            end do

         end if

      end if

      if ( iweigo==1 ) then

!     --- fill weights

         if ( intrule==1 ) then

!        --- Newton cotes rule

            weight = 1d0 / 5040d0
            do k = 1, 4
               w(intern(k)) = 17d0*abs(g(intern(k)))*weight
            end do
            do k = 5, 10
               w(intern(k)) = 32d0*abs(g(intern(k)))*weight
            end do
            do k = 11, 14
               w(intern(k)) = 81d0*abs(g(intern(k)))*weight
            end do
            w(15) = 256d0*abs(g(15))*weight

         else if ( intrule==2 ) then

!        --- 11-pts Gauss rule

            do k =1, 11
               w(k) = abs(g(k)) * wgs9(k)
            end do

         else if ( intrule==3 ) then

!        --- 15-pts Gauss rule

            do k =1, 15
               w(k) = abs(g(k)) * wgs10(k)
            end do

         end if

      end if

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            som = 0d0
            do k = 1, m
               som = som+w(k)*phi(i,k)**2
            end do  ! k = 1, m
            w(m+i)= som
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'elp637' )

      end
