      subroutine elp639first ( phi, phiksi, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for triquadratic prism
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     elp639 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     prism
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phi(18,m), phiksi(18,m,3), weight(m)

!     phi      o     Array of length 18 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                    phi (xg ) = phi (i,k)
!                       i   k
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     phiksi   o     Array of length 18 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element.
!                    Array phi is only filled when ifirst = 0 and must be kept
!                    by the program
!     weight   o     Contains weights in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, intrule, m1, m2, indm
      double precision, allocatable :: dphitridxi(:,:,:), phitri(:,:), &
                                       weighttri(:), lambdatri(:,:)
      double precision :: h, phiz(3,3), phiderz(3,3), lam1(3), &
                          lambdaz(3), wz(3)
      character (len=18) :: text

!     dphitridxi     Contains the derivatives of the basis functions in a
!                    quadratic reference triangle
!     error          Return value of allocate or deallocate
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     indm           Index in 3d basis function
!     intrule        Actual integration rule
!     j              Counting variable
!     lam1           1-lambdaz
!     lambdatri      Contains the values of the linear 2d basis function
!                    in a triangle reference element
!     lambdaz        Contains the values of the linear 1d basis function
!                    in a reference element
!     m1             Number of integration points in xi,eta direction
!     m2             Number of integration points in zeta direction
!     phiderz        Contains the derivatives of the basis functions in a
!                    1d quadratic reference element
!     phitri         Contains the values of the quadratic 2d basis function
!                    in a triangle reference element
!     phiz           Contains the values of the quadratic 1d basis function
!                    in a line reference element
!     text           String variable containing some text
!     weighttri      Contains the weights in the reference triangle
!     wz             Contains the weights in the reference line element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP622FIRST1   Fill some element independent quantities for a quadratic
!                    triangle
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Triquadratic prism' /
! ======================================================================
!
      call eropen ( 'elp639first' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp639first'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some constants

      intrule = mod(irule,100)
      if ( jcart/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>3 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      if ( intrule==1 .and. m/=18 .or. &
           intrule==2 .and. m/=8 .or. &
           intrule==3 .and. m/=14 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

!     --- First we fill information for quadratic triangle

      if ( intrule==1) then
         m2 = 3
      else
         m2 = 2
      end if  ! ( intrule==1)
      m1 = m/m2

      if ( debug ) &
         write(irefwr,1) 'm, m1, m2, intrule', m, m1, m2, intrule

      allocate ( dphitridxi(6,m1,2), phitri(6,m1), weighttri(m1), &
                 lambdatri(3,m1), stat = error )
      if ( error/=0 ) call eralloc ( error, 22*m1, 'dphitridxi' )
      if ( ierror/=0 ) go to 1000

      call elp622first1 ( intrule, dphitridxi, phitri, weighttri, &
                          lambdatri, m1 )

      if ( debug ) then
         call prinrl1 ( phitri, 6, m1, 'phitri' )
         call prinrl1 ( lambdatri, 3, m1, 'lambdatri')
      end if  ! ( debug )

!     --- Next we fill information for quadratic line element

      if ( intrule==1 ) then

!     --- Newton Cotes rule

         lambdaz(1) = 0d0
         lambdaz(2) = 0.5d0
         lambdaz(3) = 1d0
         h = 1d0/6d0
         wz(1) = h
         wz(2) = 4d0*h
         wz(3) = h

      else

!     --- two-point Gauss rule

         lambdaz(1:2) = alam4(1:2)
         wz(1:2) = 0.5d0

      end if  ! ( intrule==1 )

!     --- Fill array phiz

      lam1(1:m2) = 1d0-lambdaz(1:m2)
      phiz(1,1:m2) = lam1(1:m2)*(2d0*lam1(1:m2)-1d0)
      phiz(2,1:m2) = 4d0*lambdaz(1:m2)*lam1(1:m2)
      phiz(3,1:m2) = lambdaz(1:m2)*(2d0*lambdaz(1:m2)-1d0)

      phiderz(1,1:m2) = -(4d0*lam1(1:m2)-1d0)
      phiderz(2,1:m2) = 4d0*(lam1(1:m2)-lambdaz(1:m2))
      phiderz(3,1:m2) = 4d0*lambdaz(1:m2)-1d0

!     --- Fill arrays phi, phiksi and weight

      do i = 1, m1

         do j = 1, m2

            indm = i+(j-1)*m1
            phi(1:6,indm) = phitri(1:6,i)*phiz(1,j)
            phi(7:12,indm) = phitri(1:6,i)*phiz(2,j)
            phi(13:18,indm) = phitri(1:6,i)*phiz(3,j)

            weight(indm) = weighttri(i)*wz(j)

            phiksi(1:6,indm,1) = dphitridxi(1:6,i,1)*phiz(1,j)
            phiksi(1:6,indm,2) = dphitridxi(1:6,i,2)*phiz(1,j)
            phiksi(1:6,indm,3) = phitri(1:6,i)*phiderz(1,j)

            phiksi(7:12,indm,1) = dphitridxi(1:6,i,1)*phiz(2,j)
            phiksi(7:12,indm,2) = dphitridxi(1:6,i,2)*phiz(2,j)
            phiksi(7:12,indm,3) = phitri(1:6,i)*phiderz(2,j)

            phiksi(13:18,indm,1) = dphitridxi(1:6,i,1)*phiz(3,j)
            phiksi(13:18,indm,2) = dphitridxi(1:6,i,2)*phiz(3,j)
            phiksi(13:18,indm,3) = phitri(1:6,i)*phiderz(3,j)

         end do  ! j = 1, m2

      end do  ! i = 1, m1

      deallocate ( dphitridxi, phitri, weighttri, lambdatri, &
                   stat = error )
      if ( error/=0 ) call erdealloc ( error, 'dphitridxi' )

1000  call erclos ( 'elp639first' )
      if ( debug ) then
         call prinrl1 ( phi, m, 18, 'phi' )
         call prinrl ( weight, m, 'weight' )
         write(irefwr,*) 'End elp639first'
      end if  ! ( debug )

      end
