      subroutine elpup0 ( alpha, u, phi, psi, dphidx, second, &
                          dcdx, dlamdx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.3  date 12-08-2011 comment
!        version  5.2  date 08-05-2007 Correction quadratic Newton Cotes
!        version  5.1  date 15-11-2004 Change to solve error in HP-compiler
!
!   copyright (c) 1989-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute modified basis functions for upwind elements
!    Two dimensional elements (triangles)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision alpha(m,3), u(m,2), phi(n,m), psi(n,m), &
                       dphidx(n,m,2), dcdx(m,ndimlc), dlamdx(3,m,2)
      logical second

!     alpha    i     Array of length m x 3 containing the factors alpha
!                                                                      ij
!                    in the integration points
!     dcdx     i     Contains the values of dc/dx in the integration points
!                    This is used in case of non-linear upwinding
!     dlamdx   i     Contains the derivatives of the linear basis funtions
!     dphidx   i     Array of length n x m x 2 containing the values of the
!                    derivatives of the basis functions in the integration
!                    nodes
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points
!     psi      o     Array of length n x m containing the values of the modified
!                    basis functions in the integration points
!     second   o     Indicates if the second order term is significant or may
!                    be neglected in the upwinding part
!     u        i     Array of length m x 2 containing the velocity in the
!                    integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, jstep, metupl
      double precision xi, phimax, beta, cgnorm, tau1(9), &
                       u1(9,2), tau(9), h(9), unorm(9), eps(9), two

!     beta           Parameter beta in method
!     cgnorm         Norm of dphidx in integration point for specific value of i
!     eps            Array of length 9 to store the value of the tensor alpha
!                    in the direction of the velocity in the vertices
!     h              Array of length 9 to store the values of the diameter h
!                    in the vertices
!     i              Counting variable
!     jstep          Step in loop
!     k              Counting variable
!     metupl         mod(metupw,100): defines type of upwinding
!     phimax         Max ( |u  + grad phi | )
!                     j      i           j
!     tau            Array of length 9 to store the values of tau in the
!                    integration points
!     tau1           Array of length 9 to store the values of tau parallel in
!                    the integration points
!     two            Double precision number 2
!     u1             Parallel velocity
!     unorm          array of length 9 to store the norm of the velocity in the
!                    vertex
!     xi             Factor xi, see method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!       p  = tau u . grad phi
!        i       -           i
!                   h xi          ||u|| h
!       with tau = ------ beta =  -------- &
!                   2||u||         2 eps
!                      - &
!       xi depends on metupw according to:
!       metupw = 1:  Classical upwind scheme  xi = 1
!                2:  Il'in    xi = coth(beta) - 1/beta
!                                             beta/3     -3<beta<3
!                3:  Doubly assymptotic: xi = &
!                                             sign(beta)  |beta|>3
!                                         -1-1/beta     beta<1
!                4:  critical approx: xi = 0            -1<beta<1
!                                          1-1/beta     beta>1
!       eps is either alpha (if a11 = a22) or alpha in the direction of
!       the velocity
!              T
!       eps = u  alpha  u        (tensor alpha)
!             -         - &
!       h = max distance in element in the direction of u
!                                                       - &
!       For a triangle:  h  = ||u || / max (|u + grad lambda ) in the vertices
!                         i      i      j     i             j
!       First tau and h are computed in the vertices and afterwards tau is
!       interpolated to the integration points
!       In the special case of a quadratic triangle with a three point
!       integration rule, values of the coefficients etc are only known in
!       the midside points. In order to interpolate them to the vertices, &
!       the following formulae are used:
!       c(vertex ) = -c +c +c
!               1      2  3  1
!       c(vertex ) = -c +c +c
!               2      3  1  2
!       c(vertex ) = -c +c +c
!               3      1  2  3
!    In case of the discontinuity capturing methods we define u   as
!                                                              ||
!                                            2
!    u   = u . grad(c) grad(c) / || grad(c)||
!     ||
!    In that case the upwind basis functions become
!       p  = ( tau u  + tau   u  ) . grad phi
!        i        1-       2  -||            i
!    with tau  the tau as described above and tau  is tau   (DC1) or
!            1                                   2       ||
!    max(0,tau  -tau) (DC2)      tau   is the tau based upon u
!             ||                    ||                        ||
!    In this case we limit ourselves to the doubly assymptotic approximation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpup0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpup0'
         write(irefwr,1) 'jsecnd', jsecnd
         call prinrl(u(1,1),m,'u')
         call prinrl(u(1,2),m,'v')
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill norm of velocity in vertices into array unorm

      metupl = mod ( metupw, 20 )
      do i = 1, m
         unorm(i) = sqrt ( u(i,1)**2 + u(i,2)**2 )
      end do
      if ( debug ) call prinrl(unorm,m,'unorm')

!     --- Store component of alpha tensor in u-direction into eps

      if ( jsecnd==0 ) then

!     --- jsecnd = 0,   eps = 0

         do i = 1, m
            eps(i) = 0d0
         end do

      else if ( jsecnd==1 ) then

!     --- jsecnd = 1,   eps = alpha = a22  (constant)  a12 = 0

         do i = 1, m
            eps(i) = alpha(1,1)
         end do

      else if ( jsecnd==2 ) then

!     --- jsecnd = 2,   eps = alpha = a22    a12 = 0

         if ( jdiag==1 ) then

!        --- newton cotes rules

            if ( n==3 ) then

!           --- n = 3, linear triangle

               do i = 1, 3
                  eps(i) = alpha(i,1)
               end do
            else

!           --- n = 6, quadratic triangle

               do i = 1, 5, 2
                  eps(i) = alpha(i,1)
               end do
               eps(2) = 0.5d0*(eps(1)+eps(3))
               eps(4) = 0.5d0*(eps(3)+eps(5))
               eps(6) = 0.5d0*(eps(5)+eps(1))
            end if

         else

!        --- gauss integration

            do i = 1, m
               eps(i) = alpha(i,1)
            end do
         end if

      else

!     --- jsecnd>2,   General case                         T
!                      eps is defined by the formula  eps = u  alpha u
!                                                           -        -

         if ( jdiag==1 ) then

!        --- newton cotes rules

            if ( n==3 ) then

!           --- n = 3, linear triangle

               do i = 1, 3
                  if ( unorm(i)>0d0 ) then
                      eps(i) = ( u(i,1)**2*alpha(i,1)+2d0*u(i,1)* &
                                 alpha(i,2)*u(i,2) + u(i,2)**2* &
                                 alpha(i,3) ) / unorm(i)
                  else
                     eps(i) = alpha(i,1)
                  end if
               end do

            else

!           --- n = 6, quadratic triangle

               do i = 1, 5, 2

                  if ( unorm(i)>0d0 ) then
                     eps(i) = ( u(i,1)**2*alpha(i,1)+2d0*u(i,1)* &
                                alpha(i,2)* u(i,2) + u(i,2)**2* &
                                alpha(i,3) ) / unorm(i)
                  else
                     eps(i) = alpha(i,1)
                  end if

               end do

            end if

         else

!        --- gauss integration rules

            do i = 1, m
              if ( unorm(i)>0d0 ) then
                  eps(i) = ( u(i,1)**2*alpha(i,1)+2d0*u(i,1)* &
                             alpha(i,2)* u(i,2) + u(i,2)**2* &
                             alpha(i,3) ) / unorm(i)
               else
                  eps(i) = alpha(i,1)
               end if
            end do
         end if
      end if

      if ( debug ) then
         write(irefwr,1) 'n, m, jdiag', n, m, jdiag
         call prinrl ( eps,m,'eps' )
      end if

!     --- Compute the distance h in the vertices

      if ( jdiag==1 ) then

!     --- newton cotes integration rules

         if ( n==3 ) then

!        --- n = 3,  linear element

            do i = 1, 3
               phimax = max ( abs(u(i,1)*dphidx(1,1,1)+ &
                                  u(i,2)*dphidx(1,1,2)), &
                        abs(u(i,1)*dphidx(2,1,1)+u(i,2)*dphidx(2,1,2)), &
                        abs(u(i,1)*dphidx(3,1,1)+u(i,2)*dphidx(3,1,2)) )
               if ( phimax>sqreps ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         else

!        --- n = 6,  quadratic element

            do i = 1, 5, 2
               phimax = max(abs(u(i,1)*dlamdx(1,1,2)+ &
                                u(i,2)*dlamdx(1,1,1)), &
                            abs(u(i,1)*dlamdx(2,1,2)+ &
                                u(i,2)*dlamdx(2,1,1)), &
                            abs(u(i,1)*dlamdx(3,1,2)+ &
                                u(i,2)*dlamdx(3,1,1)))
               if ( phimax>sqreps ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         end if

      else

!     --- gauss integration rules

         if ( n==3 ) then

!        --- n = 3,  linear element

            do i = 1, m
               phimax = max ( abs(u(i,1)*dphidx(1,1,1)+ &
                                  u(i,2)*dphidx(1,1,2)), &
                        abs(u(i,1)*dphidx(2,1,1)+u(i,2)*dphidx(2,1,2)), &
                        abs(u(i,1)*dphidx(3,1,1)+u(i,2)*dphidx(3,1,2)) )
               if ( phimax>epsmac ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         else

            do i = 1, m
               phimax = max(abs(u(i,1)*dlamdx(1,1,1)+ &
                                u(i,2)*dlamdx(1,1,2)), &
                            abs(u(i,1)*dlamdx(2,1,1)+ &
                                u(i,2)*dlamdx(2,1,2)), &
                            abs(u(i,1)*dlamdx(3,1,1)+ &
                                u(i,2)*dlamdx(3,1,2)))
               if ( phimax>epsmac ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         end if

      end if
      if ( debug ) then
         call prinrl ( h, m, 'h' )
      end if

!     --- Compute tau in the vertices
!         for quadratic element with newton cotes integration use linear
!         tau field

      jstep = 1
      if ( jdiag==1 .and. n==6 ) jstep = 2

      do i = 1, m, jstep

         if ( eps(i)>1d-15 .and. metupl<=8 .and. metupl/=6 ) then

!        --- eps(i)>1d-10,  hence second order term plays a role
!            Due to an error in the HP compiler we have to replace 2d0
!            by a variable

            two = 2d0
            beta = unorm(i)*h(i)/(two*eps(i))
            if ( beta<1d-15 ) then
               xi = 0d0
            else if ( metupl==1 ) then

!           --- metupl = 1   classical upwind scheme

               xi = 1d0

            else if ( metupl==2 ) then

!           --- metupl = 2   il'in scheme

               xi = 1d0/tanh(beta)-1d0/beta

            else if ( metupl==3 .or. metupl==7 .or. metupl==8 ) then

!           --- metupl = 3, 7 or 8    doubly assymptotic approximation

               if ( abs(beta)>3d0 ) then
                  xi=sign(1d0,beta)
               else
                  xi = beta / 3d0
               end if

            else if ( metupl==4 ) then

!           --- metupl = 4    critical approximation

               if ( beta<-1d0 ) then
                  xi = -1d0 - 1d0 / beta
               else if ( beta>1d0 ) then
                  xi = 1d0 - 1d0 / beta
               else
                  xi = 0d0
               end if

            else if ( metupl==5 ) then

!           --- metupl = 5

               if ( unorm(i)<=1d-5*eps(i) ) then
                  xi = 0d0
               else
                  beta = beta*beta
                  xi = sqrt(beta/(9d0+beta))
               end if
            end if

         else if (metupl==6 ) then

!        --- metupl = 6

            if ( h(i)>epsmac ) then
               beta = (2d0*unorm(i)/h(i))**2 + &
                      (4d0*eps(i)/h(i)**2)**2
            else
               beta = 0d0
            end if
            if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
            tau(i) = 1d0/sqrt(beta)

         else
            xi = 1d0

         end if

         if ( metupl<=8 .and. metupl/=6 ) then
            if ( unorm(i)>epsmac ) then
               tau(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau(i) = 0d0
            end if
         end if

      end do

!     --- interpolate tau in integration points for quadratic element

      if ( jstep==2 ) then

!     --- n = 6   Quadratic element with Newton Cotes integration

         tau(2) = 0.5d0 * (tau(1) + tau(3) )
         tau(4) = 0.5d0 * (tau(3) + tau(5) )
         tau(6) = 0.5d0 * (tau(5) + tau(1) )

      end if
      if ( debug ) call prinrl ( tau, m, 'tau' )

!     --- compute new basis functions, it has been assumed that the old
!         basis functions are already stored in array phi

      if ( jdercn==0 ) then

!     --- jdercn = 0,  constant derivatives

         do k = 1, m
            do i = 1, n
               psi(i,k) = tau(k) * ( u(k,1)*dphidx(i,1,1)+ &
                                     u(k,2)*dphidx(i,1,2) ) + phi(i,k)
            end do
         end do

      else

!     --- jdercn = 1,  variable derivatives

         do k = 1,m
            do i = 1, n
               psi(i,k) = tau(k) * ( u(k,1)*dphidx(i,k,1)+ &
                                     u(k,2)*dphidx(i,k,2) ) + phi(i,k)
            end do
         end do
      end if
      if ( jcart==1 ) then
         do k = 1, m
            if ( abs(unorm(k))>0d0 .and. &
                 abs(eps(k))*1d2>unorm(k) ) second = .true.
         end do
      end if
      if ( metupl==7 .or. metupl==8 ) then

!     --- Discontinuity capturing
!         First compute u
!                        ||

         do k = 1, m
            cgnorm = dcdx(k,1)**2 + dcdx(k,2)**2
            if ( cgnorm<epsmac ) then
               u1(k,1) = 0d0
               u1(k,2) = 0d0
            else
               xi = (u(k,1)*dcdx(k,1) + u(k,2)*dcdx(k,2))/cgnorm
               u1(k,1) = xi*dcdx(k,1)
               u1(k,2) = xi*dcdx(k,2)
            end if
            unorm(k) = sqrt ( u1(k,1)**2 + u1(k,2)**2 )
         end do
         if ( jsecnd>2 ) then

!        --- jsecnd>2,   General case                         T
!                         eps is defined by the formula  eps = u  alpha u
!                                                              -        - &

            if ( jdiag==1 ) then

!           --- newton cotes rules

               if ( n==3 ) then

!              --- n = 3, linear triangle

                  do i = 1, 3
                     if ( unorm(i)>0d0 ) then
                         eps(i) = ( u1(i,1)**2*alpha(i,1)+2d0*u1(i,1)* &
                                    alpha(i,2)*u1(i,2) + u1(i,2)**2* &
                                    alpha(i,3) ) / unorm(i)
                     else
                        eps(i) = alpha(i,1)
                     end if
                  end do

               else

!              --- n = 6, quadratic triangle

                  do i = 1, 5, 2
                     if ( unorm(i)>0d0 ) then
                        eps(i) = ( u1(i,1)**2*alpha(i,1)+2d0*u1(i,1)* &
                                   alpha(i,2)* u1(i,2) + u1(i,2)**2* &
                                   alpha(i,3) ) / unorm(i)
                     else
                        eps(i) = alpha(i,1)
                     end if
                  end do
               end if
            else

!           --- gauss integration rules

               do i = 1, m
                 if ( unorm(i)>0d0 ) then
                     eps(i) = ( u1(i,1)**2*alpha(i,1)+2d0*u1(i,1)* &
                                alpha(i,2)* u1(i,2) + u1(i,2)**2* &
                                alpha(i,3) ) / unorm(i)
                  else
                     eps(i) = alpha(i,1)
                  end if
               end do
            end if
         end if

!        --- Compute the distance h in the vertices

         if ( jdiag==1 ) then

!        --- newton cotes integration rules

            if ( n==3 ) then

!           --- n = 3,  linear element

               do i = 1, 3
                  phimax = max ( abs(u1(i,1)*dphidx(1,1,1)+ &
                                     u1(i,2)*dphidx(1,1,2)), &
                                 abs(u1(i,1)*dphidx(2,1,1)+ &
                                     u1(i,2)*dphidx(2,1,2)), &
                                 abs(u1(i,1)*dphidx(3,1,1)+ &
                                     u1(i,2)*dphidx(3,1,2)) )
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do

            else

!           --- n = 6,  quadratic element

               do i = 1, 5, 2
                  phimax = max(abs(u1(i,1)*dlamdx(1,1,1)+ &
                                   u1(i,2)*dlamdx(1,1,2)), &
                               abs(u1(i,1)*dlamdx(2,1,1)+ &
                                   u1(i,2)*dlamdx(2,1,2)), &
                               abs(u1(i,1)*dlamdx(3,1,1)+ &
                                   u1(i,2)*dlamdx(3,1,2)))
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do

            end if

         else

!        --- gauss integration rules

            if ( n==3 ) then

!           --- n = 3,  linear element

               do i = 1, m
                  phimax = max ( abs(u1(i,1)*dphidx(1,1,1)+ &
                                     u1(i,2)*dphidx(1,1,2)), &
                                 abs(u1(i,1)*dphidx(2,1,1)+ &
                                     u1(i,2)*dphidx(2,1,2)), &
                                 abs(u1(i,1)*dphidx(3,1,1)+ &
                                     u1(i,2)*dphidx(3,1,2)) )
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do
            else
               do i = 1, m
                  phimax = max(abs(u1(i,1)*dlamdx(1,1,1)+ &
                                   u1(i,2)*dlamdx(1,1,2)), &
                               abs(u1(i,1)*dlamdx(2,1,1)+ &
                                   u1(i,2)*dlamdx(2,1,2)), &
                               abs(u1(i,1)*dlamdx(3,1,1)+ &
                                   u1(i,2)*dlamdx(3,1,2)))
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do
            end if
         end if

!        --- Compute tau in the vertices
!            for quadratic element with newton cotes integration use linear
!            tau field

         jstep = 1
         if ( jdiag==1 .and. n==6 ) jstep = 2

         do i = 1, m, jstep
            if ( eps(i)>1d-15 ) then

!           --- eps(i)>1d-15,  hence second order term plays a role

               beta = unorm(i)*h(i)/(2d0*eps(i))
               if ( beta<1d-15 ) then
                  xi = 0d0
               else

!              --- metupl = 3, 7 or 8    doubly assymptotic approximation

                  if ( abs(beta)>3d0 ) then
                     xi=sign(1d0,beta)
                  else
                     xi = beta / 3d0
                  end if
               end if
               if ( unorm(i)>epsmac ) then
                  tau1(i) = h(i)*xi/(2d0*unorm(i))
               else
                  tau1(i) = 0d0
               end if

            end if  ! ( eps(i)>1d-15 )

         end do  ! i = 1, m, jstep

!        --- interpolate tau in integration points for quadratic element

         if ( jstep==2 ) then

!        --- n = 6   Quadratic element with Newton Cotes integration

            tau(2) = 0.5d0 * (tau(1) + tau(3) )
            tau(4) = 0.5d0 * (tau(3) + tau(5) )
            tau(6) = 0.5d0 * (tau(5) + tau(1) )

         end if
         if ( debug ) call prinrl ( tau, m, 'tau' )

         if ( metupl==8 ) then
            do k = 1, m
               tau1(k) = max(0d0,tau1(k)-tau(k))
            end do
         end if
         do i = 1, n
            do k = 1, m
               psi(i,k) = tau1(k) * ( u1(k,1)*dphidx(i,k,1)+ &
                                      u1(k,2)*dphidx(i,k,2) ) + psi(i,k)
            end do
         end do
      end if

1000  call erclos ( 'elpup0' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elpup0'

      end if  ! ( debug )

      end
