      subroutine elpup5 ( u, h, x, unorm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Correction declaration
!        version  1.0  date 15-01-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute distance h for modified basis functions for upwind elements
!    Three dimensional elements, linear prisms
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwinding
!     prism
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision h(m), u(m,3), x(6,3), unorm(m)

!     h              o    Contains the approximation of the distance
!                         h in the integration points
!     u              i    Array of length m x 3 containing the velocity in the
!                         integration points
!     unorm          i    Length of velocity vector in the integration points
!     x              i    Contains the coordinates of the vertices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, j
      double precision xcent(2,3), uxi, ueta, uzeta, xtria(3,3), &
                       c, et1(3), et2(3), e(3,2), phimax, &
                       length1, length2, normal(3), exi(3), xloc(3,2), &
                       delta, dlamdx(3,2)

!     c              help variable to store a constant
!     delta          jacobian of element
!     dlamdx         contains derivatives of linear basis functions in triangle
!     e              See subroutine elp620
!     et1            First tangential vector
!     et2            Second tangential vector
!     exi            base vector in xi direction
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     length1        Help parameter to compute the length of a vector
!     length2        Help parameter to compute the length of a vector
!     normal         Normal on trianfle
!     phimax         Max j ( |u_i. grad phi_j| )
!     ueta           Velocity component in eta direction
!     uxi            Velocity component in ksi direction
!     uzeta          Velocity component in zeta direction
!     xcent          Contains the coordinates of the centroids of lower face
!                    and upper face
!     xloc           co-ordinates of vertices in local (xi-zeta) coordinate
!                    system
!     xtria          Contains the coordinates of an average triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELLOCCOOR      Compute local coordinates of a surface element in 3d given
!                    the transformation by base vectors
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpup5' )

!     --- In xcent we store the centroids of the bottom (4) and top
!         triangle(5)
!         In xcent 1 to 3 the mean triangle

      c = 1d0/3d0
      do j = 1, 3
         xcent(1,j) = c * ( x(1,j)+x(2,j)+x(3,j) )
         xcent(2,j) = c * ( x(4,j)+x(5,j)+x(6,j) )
         xtria(1,j) = 0.5d0 * ( x(1,j)+x(4,j) )
         xtria(2,j) = 0.5d0 * ( x(2,j)+x(5,j) )
         xtria(3,j) = 0.5d0 * ( x(3,j)+x(6,j) )
      end do

!     --- We also need the derivatives of the linear basis functions
!         We use the method of elp620
!         Compute e_t1 and e_t2

      do k = 1, 3
         et1(k) = xtria(2,k)-xtria(1,k)
         et2(k) = xtria(3,k)-xtria(1,k)
      end do
      length1 = sqrt(et1(1)**2+et1(2)**2+et1(3)**2)
      length2 = sqrt(et2(1)**2+et2(2)**2+et2(3)**2)
      do k = 1, 3
         et1(k) = et1(k)/length1
         et2(k) = et2(k)/length2
      end do

!     --- Compute normal n

      call eloutr ( et1, et2, normal )
      length1 = sqrt(normal(1)**2+normal(2)**2+normal(3)**2)
      do k = 1, 3
         normal(k) = normal(k)/length1
      end do

!     --- Compute  e_xi = (e_zeta x n) / ||e_zeta x n||
!         e_zeta = et1

      call eloutr ( normal, et1, exi )
      length1 = sqrt(exi(1)**2+exi(2)**2+exi(3)**2)
      do k = 1, 3
         exi(k) = exi(k)/length1
      end do

!     --- Compute local coordinates

      call elloccoor ( xtria, xloc, et1, exi, 3 )

!     --- Compute the quantities  e and delta

      do k = 1, 2
         e(1,k) = xloc(3,k) - xloc(2,k)
         e(3,k) = xloc(2,k) - xloc(1,k)
      end do

      e(1,2) = -e(1,2)
      e(3,2) = -e(3,2)
      delta = e(3,2)*e(1,1) - e(1,2)*e(3,1)

      if ( abs(delta).gt.0d0 ) then

!     --- Standard case

         i = 2
         do k = 1, 2
            dlamdx(1,k) = e(1,i) / delta
            dlamdx(3,k) = e(3,i) / delta
            dlamdx(2,k) = -dlamdx(1,k) - dlamdx(3,k)
            i = 1
         end do

      else

!     --- delta = 0, should not happen

         do k = 1, 2
            dlamdx(1,k) = 0d0
            dlamdx(3,k) = 0d0
            dlamdx(2,k) = 0d0
         end do

      end if

      do i = 1, m
         if ( unorm(i).gt.epsmac ) then

!        --- Velocity unequal to zero: upwinding makes sense
!            First we look in the zeta direction like in quads

            uzeta = (u(i,1) * ( xcent(2,1) - xcent(1,1) ) + &
                     u(i,2) * ( xcent(2,2) - xcent(1,2) ) + &
                     u(i,3) * ( xcent(2,3) - xcent(1,3) ) ) / unorm(i)

!           --- Next we consider the "plane direction" line in triangles

            uxi = u(i,1)*et1(1) + u(i,2)*et1(2) + u(i,3)*et1(3)
            ueta = u(i,1)*exi(1) + u(i,2)*exi(2) + u(i,3)*exi(3)
            phimax = max(abs(uxi*dlamdx(1,1)+ueta*dlamdx(1,2)), &
                         abs(uxi*dlamdx(2,1)+ueta*dlamdx(2,2)), &
                         abs(uxi*dlamdx(3,1)+ueta*dlamdx(3,2)))
            if ( phimax.gt.epsmac ) then
               h(i) = unorm(i)/phimax
            else
               h(i) = 0d0
            end if
            h(i) = sqrt( h(i)**2 + uzeta**2 )

         else

            h(i) = 0d0

         end if

      end do

      call erclos ( 'elpup5' )
      end
