      subroutine elpuptimdep3d ( jelp, u, phi, psi, dphidx, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute modified basis functions psi in case of upwind of type 12
!     Only linear tetrahedrons are implemented
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     upwinding
!     basis_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp
      double precision phi(n,m), dphidx(n,m,3), u(m,3), psi(n,m), w(m)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     u              i    array of length m containing the values of the
!                         second coefficient in the integration points
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision normu(m), beta(m), delta

!     beta           array of length m containing the values of the
!                    multiplication factor
!     delta          jacobian of element
!     k              Counting variable
!     normu          norm of u-vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3082   This type of upwind has not been implemented for this shape
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The upwind basis function is given by
!
!     psi_i = phi_i + beta u . grad phi_i
!
!     with beta = 1 / 2 sqrt(Delta t^-2 + h^-2 ||u||^2)
!     instead of h^2 we take the Jacobian of the element |delta|^2/3
!     which is equal to 6 times the volume of the tetrahedron
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpuptimdep3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpuptimdep3d'
         write(irefwr,2) 'tstep', tstep
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jelp/=11 ) then

!     --- Error: shape # 11

         call errint ( 12, 1 )
         call errint ( jelp, 2 )
         call errsub ( 3082, 2, 0, 0 )
         go to 1000

      end if  ! ( jelp/=11 )

!     --- Compute delta

      delta = 6d0*sum(w)
      delta = delta**(2d0/3d0)  ! to make it h^2
      if ( debug ) write(irefwr,2) 'delta', delta

!     --- Compute beta
!         beta = 1 / 2 sqrt(Delta t^-2 + h^-2 ||u||^2)

      normu = u(:,1)**2+u(:,2)**2+u(:,3)**2
      beta = 1d0 / (2d0*sqrt(1d0/tstep**2+normu/delta))
      if ( debug ) write(irefwr,2) 'normu, beta', normu, beta

!     --- Compute psi
!         psi_i = phi_i + beta u . grad phi_i

      do k = 1, m
         psi(1:n,k) = phi(1:n,k) + beta(k) * &
                      ( u(k,1)*dphidx(1:n,k,1)+u(k,2)*dphidx(1:n,k,2)+ &
                        u(k,3)*dphidx(1:n,k,3) )
      end do  ! k = 1, m

1000  call erclos ( 'elpuptimdep3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elpuptimdep3d'

      end if  ! ( debug )

      end
