      subroutine elup02 ( alpha, dphidx, eps, user, index1, &
                          work, coor2d, jelp, vecloc, &
                          qmat, iuser, a1, a2, phi, phixi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 12-08-2013 Extra parameters
!        version  3.2  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute help arrays for second derivative part in case of upwinding
!   Bilinear and quadratic two-dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), iuser(*), jelp
      double precision alpha(m,3), dphidx(n,n,2), phi(*), phixi(*), &
                       eps(n,3), user(*), vecloc(*), work(*), &
                       coor2d(*), qmat(*), a1(9,9), a2(9,9)

!     a1                  Work array to store parts of the computation of the
!                         second derivative term
!     a2                  Work array to store parts of the computation of the
!                         second derivative term
!     alpha          i    Array of length m x 3 containing the factors alpha
!                                                                           ij
!                         in the integration points
!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     eps                 Work array in which the values of alpha ij in the
!                         nodal points are stored
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     qmat                transformation matrix global -> local coordinates
!                         in case of a 3D element
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
!     work                Real work array of length n
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, ihcoor2d, ihweig, ihrule, ihiphi, ihgaus, ih, msave, &
              ifirsv, ksteps
      double precision x(81), w(27), xgauss(81), ugauss(27), dum(100)

!     dum            General double precision dummy variable
!     i              Counting variable
!     ifirsv         Help variable to store the value of ifirst temporarily
!     ih             Help variable to store a constant temporarily
!     ihcoor2d         Help variable to store the value of icoor temporarily
!     ihgaus         Help variable to store the value of igauss temporarily
!     ihiphi         Help variable to store the value of iphi temporarily
!     ihrule         Help variable to store the value of irule temporarily
!     ihweig         Help variable to store the value of iweigh temporarily
!     k              Counting variable
!     ksteps         Help variable to store the value of kstep temporarily
!     msave          Help variable to store the value of m temporarily
!     ugauss         Preceding solution in Gauss points
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Compute the coefficients of the second derivative term (eps) in the
!    nodal points
!    Compute dphidx in the nodal points
!    Compute a1 and a2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elup02' )

!     --- Compute eps in the nodal points

      do i = 1, 3

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            ih = jdiag
            jdiag = 1
            call elflr71 ( i, user, eps, x, xgauss, &
                          m, n, phi, 1, index1, &
                          iuser, ugauss, vecloc )
            jdiag = ih

         else

!        --- Coefficient is constant

            do k = 1, n
               eps(k,i) = alpha(1,i)
            end do

         end if

      end do

!     --- Compute dphidx in the basis points

      ihcoor2d = icoor
      ihiphi = iphi
      ihrule = irule
      ihweig = iweigh
      ihgaus = igausp
      ksteps = kstep
      ifirsv = ifirst
      icoor  = 1
      iphi   = 0
      irule  = 1
      iweigh = 0
      igausp = 0
      msave  = m
      kstep  = 1
      m      = n
      ifirst = 1
      call elm800basefn ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, dum, jelp, phixi, dum, dum, dum )
      icoor  = ihcoor2d
      iphi   = ihiphi
      irule  = ihrule
      iweigh = ihweig
      igausp = ihgaus
      kstep  = ksteps
      ifirst = ifirsv
      m      = msave

!     --- Fill arrays a1 and a2

      do i = 1, n
         do k = 1, n
            a1(k,i) = eps(k,1)*dphidx(i,k,1)+eps(k,2)*dphidx(i,k,2)
            a2(k,i) = eps(k,2)*dphidx(i,k,1)+eps(k,3)*dphidx(i,k,2)
         end do
      end do

      call erclos ( 'elup02' )

      end
