      subroutine sepel0251
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 251
!     boundary conditions for linear elastic problem
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulestrings
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPADAPRULE251 Adapt integration rule in case of boundary elements
!                    for linear elasticity
!     SEPEL0251T     Test coefficients for type 251
!     SEPEL0801CF    Fill real coefficients for elements of type 801
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELILOAD     Fill and check parameter iload for elasticity problems
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPGETRULE     Get the integration rule as stored in iuser
!     SEPGETSTRESSRL Read stress-strain relations and store information in E
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0251' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0251'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Get iload

      call sepeliload
      if ( debug ) write(irefwr,1) 'iload', E%iload

!     --- Get stress-strain relation

      call sepgetstressrl
      if ( debug ) write(irefwr,1) 'igprbo', E%stress_strain-1
      if ( ierror/=0 ) go to 1000

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      if ( ierror/=0 ) go to 1000
      B%cart = 1
      if ( E%stress_strain==2 .or. E%stress_strain==5 .or. &
           E%stress_strain==7 ) B%cart = 2
      E%ipos = E%ipos+2          ! skip two positions

!     --- Adapt integration rule for specific cases

      call sepadaprule251

!     --- Get information of basis functions

      call sepelbasefnch
      if ( ierror/=0 ) go to 1000
      E%plane = E%ndim==2 .and. B%cart==1

!     --- Fill all real coefficients

      E%ncoef = 10
      call sepel0801cf

!     --- Test if given coefficients correspond to problem

      call sepel0251t

      if ( E%iload==3 ) then

!     --- iload = 3, compute cos(theta) and sin(theta)

         if ( E%ind(2)==0 ) then
            E%costet = 1d0
            E%sintet = 0d0
         else if ( E%ind(2)<0 ) then
            E%costet = cos(E%coeffs(1,2))
            E%sintet = sin(E%coeffs(1,2))
         end if

      end if  ! ( E%iload==3 )

      if ( E%iload==4 ) then

!     --- iload = 4, tangent vector and possibily qmat required

         if ( E%ndim==2 ) then
            E%ktang = 1
         else
            E%iqmat = 3
            B%fillqmat = .true.
         end if  ! ( E%ndim==2 )

      end if  ! ( E%iload==4 )

!     --- Check if matrix and vector must be filled

      if ( E%ind(5)==0 .and. E%ind(6)==0 .and. E%ind(7)==0 .and. &
           E%ind(8)==0 .and. E%ind(9)==0 .and. E%ind(10)==0 ) notmat = 1
      if ( E%ind(1)==0 .and. E%ind(2)==0 .and. E%ind(3)==0 ) notvec = 1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( E%ind, 10, 'ind' )
         write(irefwr,*) 'End sepel0251'

      end if  ! ( debug )
      call erclos ( 'sepel0251' )

      end subroutine sepel0251
