      subroutine sepel0805
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine SEPELM805 or
!     ELD805 in the case that IFIRST = 0
!     Old subroutine el0805
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPELTYPEDIFF1  ?
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  This parameter is no longer in use.
!         The parameter imas in BUILD defines the type of mass matrix
!         Now the parameter is reserved for future purposes
!     2:  -
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  -
!     Parameters with respect to the first unknown
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!    Parameters: 21-35 corresponding to second unknown have exactly
!    the same meaning
!     In the case of a call by INTEGR:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
!     In the case of a call by DERIV:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2-5:  Not yet used
!         In the cases ICHELD = 6, 16 the positions 6-11 contain
!         information about the matrix A:
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0805' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0805'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Compute the parameters jcart and irule

      E%ipos = E%ipos + 2
      E%ishift = E%ishift+2

      call sepgetrule     ! quadrature rule
      call sepgetcoorsys  ! coordinate system

      if ( E%imas==0 ) notmas = 1

!     --- Adapt integration rule for specific cases

      if ( imas==2 .and. (E%ishape<=3 .or. E%ishape==5) .and. &
           B%rule==0 ) then
         if ( E%ishape==2 ) then
            B%rule = 4
         else
            B%rule = max ( B%rule, 3 )
         end if
      else if ( imas==2 .and. E%ishape<=6 .and. B%rule==0 ) then
         B%rule = max ( B%rule, 2 )
      end if

!     --- Get information of basis functions

      call sepelbasefnch
      n = B%n

      E%ipos = E%ipos + 1
      E%ishift = E%ishift+1

!     --- Fill all real coefficients

      E%ncoef = 30
      call sepelalloccf ( 0, 0 ) ! allocate coefficient arrays

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      call sepeltypediff1 ( E%ind, E%typediff1, E%diffusion1 )
      call sepeltypediff1 ( E%ind(16), E%typediff2, E%diffusion2 )

      E%convection1 = E%ind(7)/=0 .or. B%nd>1 .and. &
                      E%ind(8)/=0 .or. B%nd>2 .and. E%ind(9)/=0
      E%convection2 = E%ind(22)/=0 .or. B%nd>1 .and. &
                      E%ind(23)/=0 .or. B%nd>2 .and. E%ind(24)/=0
      E%zeroterm1 = E%ind(10)/=0
      E%zeroterm2 = E%ind(25)/=0

      E%sourceterm1 = E%ind(11)/=0
      E%sourceterm2 = E%ind(26)/=0

      if ( .not. E%sourceterm1 .and. .not. E%sourceterm2 ) notvec = 1

      E%rhoone = E%ind(12)==0 .and. E%ind(27)==0

!     --- Find lower and upper bound for space dependent coefficients

      call sepellowupp

      allocate ( E%subvec(B%n), E%submat(B%n,B%n) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0805'

      end if  ! ( debug )
      call erclos ( 'sepel0805' )

      end subroutine sepel0805
