      subroutine sepel705map
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map coefficients of general elliptic equation from present dimension
!     onto space of lower dimension
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     diffusion
!     convection
!     map
!     right_hand_side
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k, m
      integer, pointer :: ind(:)
      double precision, pointer :: coeffs(:,:), qmat(:,:,:)
      double precision :: a1(E%ndim,E%ndim), a2(E%ndim,E%ndim)

!     a1             Is used to store the diffusion tensor per quadrature
!     a2             Contains the transpose of qmat
!                    point
!     coeffs         Contains the coefficients of the equation
!     ind            Indicates if coefficient is zero (0) or not
!     j              general loop variable
!     k              general loop variable
!     m              number of quadrature points
!     qmat           contains transformation matrix of global to local
!                    coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel705map' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel705map'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Associate pointers with arrays in modules

      coeffs => E%coeffs
      qmat => B%qmat
      ind => E%ind

      m = B%m

!     --- Next map coefficients
!         First diffusion

      if ( E%ndim==2 )  then

!     --- ndim = 2

         do k = 1, m

            if ( maxval(abs(ind(2:4)))>0 ) then

!           --- Store diffusion tensor per quadrature point in  a1

               a1(1,1) = coeffs(k,1)
               a1(1,2) = coeffs(k,2)
               a1(2,1) = a1(1,2)
               a1(2,2) = coeffs(k,4)

!              --- Store transpose of qmat in a2

               a2(1,1) = qmat(1,1,k)
               a2(1,2) = qmat(2,1,k)
               a2(2,1) = qmat(1,2,k)
               a2(2,2) = qmat(2,2,k)

!              --- help = Q^T a1

               a1 = matmul (a2, a1 )

!              --- a1 = Q^T a1 Q

               a1 = matmul ( a1, qmat(1:2,1:2,k) )

!              --- Store new diffusion tensor

               coeffs(k,1) = a1(1,1)
               coeffs(k,2) = a1(1,2)
               coeffs(k,4) = a1(2,2)

            end if  ! ( maxval(abs(ind(2:4)))>0 )

            if ( abs(ind(8))>0 ) then

!           --- Velocity vector

               coeffs(k,7:8) = matmul ( qmat(1:2,1:2,k), coeffs(k,7:8) )

            end if  ! ( maxval(abs(ind(7:9)))>0 )

         end do  ! k = 1, m

      else

!     --- ndim = 3

         do k = 1, m

            if ( maxval(abs(ind(1:6)))>0 ) then

!           --- Store diffusion tensor per quadrature point in  a1

               a1(1,1) = coeffs(k,1)
               a1(1,2) = coeffs(k,2)
               a1(1,3) = coeffs(k,3)
               a1(2,2) = coeffs(k,4)
               a1(2,3) = coeffs(k,5)
               a1(3,3) = coeffs(k,6)
               a1(2,1) = a1(1,2)
               a1(3,1) = a1(1,3)
               a1(3,2) = a1(2,3)

!              --- Store transpose of qmat in a2

               a2(1,1:3) = qmat(1:3,1,k)
               a2(2,1:3) = qmat(1:3,2,k)
               a2(3,1:3) = qmat(1:3,3,k)

!              --- help = Q^T a1

               a1 = matmul (a2, a1 )

!              --- a1 = Q^T a1 Q

               a1 = matmul ( a1, qmat(1:3,1:3,k) )

!              --- Store new diffusion tensor

               coeffs(k,1) = a1(1,1)
               coeffs(k,2) = a1(1,2)
               coeffs(k,3) = a1(1,3)
               coeffs(k,4) = a1(2,2)
               coeffs(k,5) = a1(2,3)
               coeffs(k,6) = a1(3,3)

            end if  ! ( maxval(abs(ind(1:6)))>0 )

            if ( maxval(abs(ind(7:9)))>0 ) then

!           --- Velocity vector

               coeffs(k,7:9) = matmul ( qmat(1:3,1:3,k), coeffs(k,7:9) )

            end if  ! ( maxval(abs(ind(7:9)))>0 )

         end do  ! k = 1, m

      end if  ! ( E%ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel705map'

      end if  ! ( debug )
      call erclos ( 'sepel705map' )

      end subroutine sepel705map
