      subroutine sepelbearrhs
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute right-hand side part of Reynolds equation
!     Old subroutine elbearrhs
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: coeff(:,:), p0(:), dpdx(:), dpdy(:)
      double precision, pointer :: rho(:), RT(:), cons(:), pold(:), h(:)
      double precision, pointer :: arrmu(:), f(:,:), u1(:,:), u2(:,:), ht(:)
      integer, pointer :: ind(:)
      integer :: m, n
      logical :: seconf

!     RT             Contains the constant RT from the ideal gas law
!     arrmu          Contains the parameter h^3/(12 mu)
!     coeff          Help array to store the computed coefficient
!     cons           Contains the variable k in the equations
!     dpdx           Contains dpdx
!     dpdy           Contains dpdy
!     f              Contains the function f in the equation (source)
!     h              Contains the function h in the equation (film thickness)
!     ht             Contains the function ht in the equation
!     ind            Refers to E%ind
!     m              number of quadrature points
!     n              number of basis functions for each component
!     p0             Contains the function p0 in the equation (source)
!     pold           Contains the pressure in the prior iteration
!     rho            density rho
!     seconf         Indicates if second part of F is zero (false) or not (true)
!     u1             velocity of lower surface
!     u2             velocity of upper surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPELLOAD      Fill the element vector with a load
!     SEPELVECGAMMA  Add term div( gamma) to element vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbearrhs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbearrhs'
         write(irefwr,1) 'E%typebear, E%typelinear', E%typebear, E%typelinear
         call prinin ( E%ind, 14, 'ind' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rho => E%coeffs(:,E%indrho)
      rt => E%coeffs(:,E%indrt)
      cons => E%coeffs(:,E%indconst)
      p0 => E%coeffs(:,E%indpress)
      pold => E%ugauss(:,1)
      arrmu => E%coeffs(:,E%inddiff)
      h => E%coeffs(:,E%indh)
      ht => E%coeffs(:,E%indsqueeze)
      f => E%coeffs(:,E%indsource:E%indsource+1)
      u1 => E%coeffs(:,E%indvel:E%indvel+1)
      u2 => E%coeffs(:,E%induppvel:E%induppvel+1)
      if ( E%typelinear==1 ) dpdx => gradoldvec(:,1,1)
      if ( E%ndim>1 ) dpdy => gradoldvec(:,2,1)
      coeff => E%gamma
      ind => E%ind

      m = B%m
      n = B%n

      if ( debug ) then
         call prinrl ( rho, m, 'rho' )
         call prinrl ( rt, m, 'rt' )
         call prinrl ( cons, m, 'cons' )
         call prinrl ( p0, m, 'p0' )
         call prinrl ( h, m, 'h' )
         call prinrl ( arrmu, m, 'arrmu' )
         call prinrl ( ht, m, 'ht' )
         call prinrl ( f(1,1), m, 'f1' )
         call prinrl ( f(1,2), m, 'f2' )
         call prinrl ( u1(1,1), m, 'u1/1' )
         call prinrl ( u2(1,1), m, 'u2/1' )
         call prinrl ( pold, m, 'pold' )
         if ( E%typelinear==1 ) call prinrl ( dpdx, m, 'dpdx' )
         if ( E%ndim>1 ) then
            call prinrl ( dpdy, m, 'dpdy' )
            call prinrl ( u1(1,2), m, 'u1/2' )
            call prinrl ( u2(1,2), m, 'u2/2' )
         end if  ! ( E%ndim>1 )
      end if  ! ( debug )

!     --- Part -ht (sqeeze term)

      if ( E%typebear<2 .and. ind(E%indsqueeze)/=0 ) then

!     --- Bearing without rho

         coeff(:,1) = -ht

      else if ( ind(E%indsqueeze)/=0 ) then

!     --- Bearing with mass conservation
!         state variable is rho

         coeff(:,1) = -ht*rho

      else

         coeff(:,1) = 0d0

      end if
      if ( debug ) call prinrl ( coeff, m, 'ht' )

!     --- rest of part corresponding to F1
!         not implemented in case of mass conservation

      select case (E%typebear)
      case ( 0 )

!     --- Incompressible bearing, no mass conservation

         if ( ind(E%indconst)/=0 .and. ind(E%indpress)/=0 ) &
            coeff(:,1) = coeff(:,1)+cons*p0

      case ( 1 )

!     --- compressible bearing, no mass conservation

         if ( ind(E%indconst)/=0 .and. ind(E%indpress)/=0 ) &
            coeff(:,1) = coeff(:,1)+cons*p0**2
         if ( E%typelinear==1 .and. ind(E%indconst)/=0 ) &
            coeff(:,1) = coeff(:,1)+cons*pold**2

      end select  ! case (E%typebear)
      if ( debug ) call prinrl ( coeff, m, 'ht+cp' )

      call sepelload ( coeff ) ! store contribution in right-hand side
      if ( debug ) call prinrl ( E%vec, n, 'F1' )

!     --- Finally part F2
!         not implemented in case of mass conservation

      seconf = .false.
      coeff = 0d0

      if ( E%typebear==0 ) then

!     --- incompressibe bearing

         if ( ind(E%indrho)/=0 .and. (ind(E%indsource)/=0 .or. &
              ind(E%indsource+1)/=0) .or. ind(E%indvel)/=0 .or. &
              ind(E%indvel+1)/=0 .or. ind(E%induppvel)/=0 .or. &
              ind(E%induppvel+1)/=0 ) then
            seconf = .true.
            if ( ind(E%indrho)/=0 ) then
               if ( ind(E%indsource)/=0 ) coeff(:,1) = arrmu*rho*f(:,1)
               if ( ind(E%indsource+1)/=0 ) coeff(:,2) = arrmu*rho*f(:,2)
            end if

!           --- Convective part, no upwind

            if ( ind(E%indvel)/=0 .or. ind(E%induppvel)/=0 ) &
               coeff(:,1) = coeff(:,1)+0.5d0*h*(u1(:,1)+u2(:,1))

            if ( debug ) then
               call prinrl ( h, m, 'h' )
               call prinrl ( u1, m, 'u1' )
               call prinrl ( u2, m, 'u2' )
               call prinrl ( coeff, m, 'coeff' )
            end if  ! ( debug )

            if ( E%ndim>1 .and. &
                 (ind(E%indvel+1)/=0 .or. ind(E%induppvel+1)/=0) ) &
               coeff(:,2) = coeff(:,2)+0.5d0*h*(u1(:,2)+u2(:,2))

            if ( debug ) then
               call prinrl ( h, m, 'h' )
               if ( E%ndim>1 ) then
                  call prinrl ( u1(1,2), m, 'u1/1' )
                  call prinrl ( u2(1,2), m, 'u2/2' )
               end if  ! ( E%ndim>1 )
               call prinrl ( coeff(1,2), m, 'coeff' )
            end if  ! ( debug )

         end if

      else if ( E%typebear==2 ) then

!     --- incompressibe bearing, mass conserving
!         The terms with f have not yet been implemented
!         State varibale is rho
!         Convective part, no upwinding

         seconf = .true.
         if ( ind(E%indvel)/=0 .or. ind(E%induppvel)/=0 ) &
            coeff(:,1) = 0.5d0*h*(u1(:,1)+u2(:,1))*rho

         if ( ind(E%indvel+1)/=0 .or. ind(E%induppvel+1)/=0 ) &
            coeff(:,2) = 0.5d0*h*(u1(:,2)+u2(:,2))*rho

      else if ( E%typelinear==1 ) then

!     --- compressibe bearing, newton method

         seconf = .true.

         if ( debug ) call prinrl (coeff, m, 'coeff/0')
         coeff(:,1) = coeff(:,1) + arrmu*dpdx*pold
         if ( debug ) call prinrl (coeff, m, 'coeff/1')

         if ( ind(E%indsource)/=0 ) &
            coeff(:,1) = coeff(:,1) - arrmu*f(:,1)*pold/RT
         if ( debug ) call prinrl (coeff, m, 'coeff/2')

         if ( E%ndim>=2 ) then
            coeff(:,2) = coeff(:,2) + arrmu*dpdy*pold
            if ( ind(E%indsource+1)/=0 ) &
               coeff(:,2) = coeff(:,2) - arrmu*f(:,2)*pold/RT
         end if
         if ( debug ) call prinrl (coeff, m, 'coeff/3')

      end if  ! ( E%typebear==0 )

      if ( seconf ) then
         coeff = -coeff
         call sepelvecgamma
      end if  ! ( seconf )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, n, 'F1+F2' )
         write(irefwr,*) 'End sepelbearrhs'

      end if  ! ( debug )
      call erclos ( 'sepelbearrhs' )

      end subroutine sepelbearrhs
