      subroutine sepelconvect5
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute correction to element stiffness matrix corresponding to the
!     first derivative part of a one-, two- or three-dimensional equation
!     in case of upwind and compressible convection
!
!        s  = s   + / div (u phi ) ( psi - phi ) dx
!         ij   ij   e           j       i     i
!     Old subroutine el3012
! **********************************************************************
!
!                       KEYWORDS
!
!     convection
!     element_matrix
!     upwind
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, n, nd, k, m, i
      double precision, pointer :: dphidx(:,:,:), u(:,:), x(:), psi(:,:), &
                                   phi(:,:)
      double precision :: w(B%m), work(B%m), som

!     dphidx         derivatives of basis functions phi
!     i              General loop variable
!     j              General loop variable
!     k              Loop variable (quadrature points)
!     m              number of quadrature points
!     n              number of basis functions
!     nd             dimension of elements
!     phi            basis functions
!     psi            upwind basis functions
!     som            Help varaible to compute a sum
!     u              velocity components in all quadrature points
!     w              weights multiplied by rho cp
!     work           work array
!     x              x-coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!
!   Define phi u = som phi (x ) u(x ) phi
!             j     l     j  l     l     l
!
!   Cartesian co-ordinates:
!
!   Contribution becomes: s_ij := s_ij + &
!
!   som ( ( som phi_j (x_l) (u_l dphi_l/dx + v_l dphi_l/dy + w_l dphi_l/dz) )
!    k       l
!
!       * (psi_i - phi_i) (x_k) )
!
!   This reduces to:
!
!   som ( u_j dphi_j/dx + v_j dphi_j/dy + w_j dphi_j/dz
!    k
!
!       * (psi_i - phi_i) (x_k) )
!
!   Cylinder co-ordinates:
!
!   The divergence is equal to du/dr + dw/dz + u/r
!   Hence the Cartesian formulas must be extended with
!
!   u_l phi_l / x_l, whenever x_l # 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvect5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvect5'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider
      phi => B%phi
      psi => B%psi
      u => E%coeffs(:,7:9)
      x => B%xyz(:,1)

      m = B%m
      n = B%n
      nd = B%nd

      w = B%w * E%coeffs(:,E%indrhocp)

      if ( .not. B%derivconst ) then

!     --- derivatives of basis functions are variable

         do j = 1, n

!        --- First fill work with som w_k (u_j dphi_j(k)+...)

            select case ( nd )

            case(1)

!           --- nd = 1, only one component of the velocity vector

               do k = 1, m
                  work(k) = w(k)*u(j,1)*dphidx(j,k,1)
               end do

            case(2)

!           --- nd = 2, two components of the velocity vector

               do k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,k,1)+ &
                                     u(j,2)*dphidx(j,k,2) )
               end do

               if ( B%cart==2 .and. x(j)/=0d0 ) then

!              --- Extra term for cylinder co-ordinates

                  work(j) = work(j) + w(j)*u(j,1)/x(j)

               end if

            case(3)

!           --- nd = 3, three components of the velocity vector

               do k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,k,1)+ &
                                     u(j,2)*dphidx(j,k,2)+ &
                                     u(j,3)*dphidx(j,k,3) )
               end do

            end select  ! case ( nd )

            do i = 1, n
               som = 0d0
               do k = 1, m
                  som = som + work(k)*(psi(i,k)-phi(i,k))
               end do
               E%mat(j,i) = E%mat(j,i) + som
            end do

         end do

      else

!     --- derivatives of basis functions are constant

         do j = 1, n

!        --- First fill work with som w_k (u_j dphi_j(k)+...)

            select case ( nd )

            case(1)

!           --- nd = 1, only one component of the velocity vector

               do k = 1, m
                  work(k) = w(k)*u(j,1)*dphidx(j,1,1)
               end do

            case(2)

!           --- nd = 2, two components of the velocity vector

               do k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,1,1)+ &
                                     u(j,2)*dphidx(j,1,2) )
               end do
               if ( B%cart==2 .and. x(j)/=0d0 ) then

!              --- Extra term for cylinder co-ordinates

                  work(j) = work(j) + w(j)*u(j,1)/x(j)

               end if

            case(3)

!           --- nd = 3, three components of the velocity vector

               do k = 1, m
                  work(k) = w(k) * ( u(j,1)*dphidx(j,1,1)+ &
                                     u(j,2)*dphidx(j,1,2)+ &
                                     u(j,3)*dphidx(j,1,3) )
               end do

            end select  ! case ( nd )

            do i = 1, n
               som = 0d0
               do k = 1, m
                  som = som + work(k)*(psi(i,k)-phi(i,k))
               end do
               E%mat(j,i) = E%mat(j,i) + som
            end do

         end do

      end if  ! ( .not. B%derivconst )

1000  call erclos ( 'sepelconvect5' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelconvect5'

      end if  ! ( debug )

      end subroutine sepelconvect5
