      subroutine sepeldiffdiag ( coeffs, ind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for diffusion equation
!     The diffusion is a diagonal tensor
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: coeffs(B%m,6)
      integer, intent(in) :: ind(6)

!     coeffs         i    contains the diffusion coefficients in the quadrature
!                         points:
!                         (.,1): D_xx
!                         (.,2): D_xy
!                         (.,3): D_xz
!                         (.,4): D_yy
!                         (.,5): D_yz
!                         (.,6): D_zz
!     ind            i    Indicates if coefficients are 0, constant or variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h, h1, h2, h3, work(B%m), work1(B%m), work2(B%m)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:), mat(:,:)
      integer :: i, j, k

!     h              help parameter to store w*diff_x
!     h1             help parameter to store element of diffusion matrix
!     h2             help parameter to store w*diff_y
!     h3             help parameter to store w*diff_z
!     i              General loop variable
!     j              General loop variable
!     k              Loop variable (quadrature points)
!     mat            sub element matrix
!     phix           x derivative of basis function phi
!     phiy           y derivative of basis function phi
!     phiz           z derivative of basis function phi
!     work           help array to store w*diff_x
!     work1          help array to store w*diff_y
!     work2          help array to store w*diff_z
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The diffusion matrix is given by
!
!     / D grad(phi_i) grad(phi_j) with D a matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffdiag' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffdiag'
         write(irefwr,*) 'B%derivconst ', B%derivconst
         write(irefwr,1) 'B%nd ', B%nd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)
      mat => E%submat

      if ( B%derivconst ) then

!     --- Derivatives of basis functions constant

         h = sum ( B%w*coeffs(:,1) )
         select case ( B%nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, B%n
               do j = 1, i
                  h1 = h*phix(i,1)*phix(j,1)
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         case(2)

!        --- 2d dimensional element

            if ( ind(4)/=0 ) h2 = sum ( B%w*coeffs(:,4) )
            do i = 1, B%n
               do j = 1, i
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+h*phix(i,1)*phix(j,1)
                  if ( ind(4)/=0 ) h1 = h1+h2*phiy(i,1)*phiy(j,1)
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         case(3)

!        --- 3d dimensional element

            if ( ind(4)/=0 ) h2 = sum ( B%w*coeffs(:,4) )
            if ( ind(6)/=0 ) h3 = sum ( B%w*coeffs(:,6) )

            do i = 1, B%n
               do j = 1, i
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+h*phix(i,1)*phix(j,1)
                  if ( ind(4)/=0 ) h1 = h1+h2*phiy(i,1)*phiy(j,1)
                  if ( ind(6)/=0 ) h1 = h1+h3*phiz(i,1)*phiz(j,1)
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         end select  ! case ( B%nd )

      else

!     --- Derivatives of basis functions are variable

         if ( ind(1)/=0 ) then
            do k = B%kstep, B%m, B%kstep
               work(k) = B%w(k)*coeffs(k,1)
            end do  ! k = B%kstep, B%m, B%kstep
         else
            work = 0d0
         end if  ! ( ind(1)/=0 )
         select case ( B%nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, B%n
               do j = 1, i
                  h1 = sum ( work(:)*phix(i,:)*phix(j,:) )
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         case(2)

!        --- 2d dimensional element

            if ( B%kstep==1 ) then

!           --- Standard case, all quadrature points are used

               if ( ind(4)/=0 ) work1 = B%w*coeffs(:,4)
               do i = 1, B%n
                  do j = 1, i
                     h1 = 0d0
                     if ( ind(1)/=0 ) &
                        h1 = h1+sum ( work(:)*phix(i,:)*phix(j,:) )
                     if ( ind(4)/=0 ) &
                        h1 = h1+sum ( work1(:)*phiy(i,:)*phiy(j,:) )
                     mat(i,j) = mat(i,j)+h1
                     if ( j/=i ) mat(j,i) = mat(j,i)+h1
                  end do  ! j = 1, i
               end do  ! i = 1, B%n

            else

!           --- Special case of quadratic triangle, where only the mid-points
!               are used

               if ( ind(4)/=0 ) then
                  do k = B%kstep, B%m, B%kstep
                     work1(k) = B%w(k)*coeffs(k,4)
                  end do  ! k = B%kstep, B%m, B%kstep
               end if  ! ( ind(1)/=0 )
               do i = 1, B%n
                  do j = 1, i
                     h1 = 0d0
                     if ( ind(1)/=0 ) then
                        do k = B%kstep, B%m, B%kstep
                           h1 = h1+work(k)*phix(i,k)*phix(j,k)
                        end do  ! k = B%kstep, B%m, B%kstep
                     end if  ! ( ind(4)/=0 )
                     if ( ind(4)/=0 ) then
                        do k = B%kstep, B%m, B%kstep
                           h1 = h1+work1(k)*phiy(i,k)*phiy(j,k)
                        end do  ! k = B%kstep, B%m, B%kstep
                     end if  ! ( ind(4)/=0 )
                     mat(i,j) = mat(i,j)+h1
                     if ( j/=i ) mat(j,i) = mat(j,i)+h1
                  end do  ! j = 1, i
               end do  ! i = 1, B%n

            end if  ! ( B%kstep==1 )

         case(3)

!        --- 3d dimensional element

            if ( ind(4)/=0 ) work1 = B%w*coeffs(:,4)
            if ( ind(6)/=0 ) work2 = B%w*coeffs(:,6)
            do i = 1, B%n
               do j = 1, i
                  h1 = 0d0
                  if ( ind(1)/=0 ) &
                     h1 = h1+sum ( work(:)*phix(i,:)*phix(j,:) )
                  if ( ind(4)/=0 ) &
                     h1 = h1+sum ( work1(:)*phiy(i,:)*phiy(j,:) )
                  if ( ind(6)/=0 ) &
                     h1 = h1+sum ( work2(:)*phiz(i,:)*phiz(j,:) )
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         end select  ! case ( B%nd )

      end if  ! ( B%derivconst )

1000  call erclos ( 'sepeldiffdiag' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( mat, B%n, B%n, 'element_matrix/diffusion' )
         write(irefwr,*) 'End sepeldiffdiag'

      end if  ! ( debug )

      end subroutine sepeldiffdiag
