      subroutine sepeldiffup04
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add contribution of second derivative part to matrix in the case of
!     upwind (3D)
!     Old subroutines elup04 and elup05
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elliptic_equation
!     diffusion
!     upwind
!     convection_diffusion
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, i, j, n, k, l
      double precision :: a1(B%n,B%n), a2(B%n,B%n), a3(B%n,B%n), som, som1
      double precision, pointer :: dphidx(:,:,:)
      double precision, pointer :: eps(:,:), w(:)
      double precision, pointer :: phi(:,:), psi(:,:)

!     a1             Work array to store parts of the computation of the
!                    second derivative term
!     a2             Work array to store parts of the computation of the
!                    second derivative term
!     a3             Work array to store parts of the computation of the
!                    second derivative term
!     dphidx         Array of length n x m x 2 containing the values of the
!                    derivatives of the basis functions in the integration
!                    node
!     eps            Work array in which the values of alpha ij in the
!                    nodal points are stored
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              Counting variable
!     m              Number of quadrature points
!     n              Number of basis functions
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phi_i(xg^k) = phi (i,k)
!     psi            upwind basis functions
!     som            Help variable to compute a sum
!     som1           Help variable to compute a sum
!     w              weights for integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        The contribution of the second derivative upwind terms is:
!
!        sum  d/dx  alpha   dphi / dx   (psi -phi )
!         kl      k      kl     j    l      i    i
!
!
!        In order to avoid the computation of second derivatives of basis
!        functions we apply the following approximation:
!
!          klj                             n    klj
!         T    = alpha   dphi / dx     ~  sum  T     phi
!                     kl     j    l    ~  m=1   m       m
!
!
!                   klj     n    klj
!        So d/dx   T    ~  sum  T     d phi / dx
!               k       ~  m=1   m         m    k
!
!
!     For i := 1 (1) n do
!        For j := 1 (1) n do
!            For l := 1 (1) n do
!               a1(l) := alpha11 dphi j / dx1 + alpha12 dphi j / dx2   (xl)
!               a2(l) := alpha12 dphi j / dx1 + alpha22 dphi j / dx2   (xl)
!            For k := 1 (1) m do
!               sec_der jk := sum a1(l) dphi l / dx1 + a2(l) dphi l /dx2  (xk)
!                              l
!               sij := sij - wk sec_der jk ( psi i (xk) - phi i (xk) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiffup04' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiffup04'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      m = B%m
      n = B%n

      dphidx => B%phidern
      eps => E%coeffsnode(:,1:6)
      phi => B%phi
      psi => B%psi
      w => B%w

!     --- Fill arrays a1 and a2

      do i = 1, n
         do k = 1, n
            a1(k,i) = eps(k,1)*dphidx(i,k,1)+eps(k,2)*dphidx(i,k,2)+ &
                      eps(k,3)*dphidx(i,k,3)
            a2(k,i) = eps(k,2)*dphidx(i,k,1)+eps(k,4)*dphidx(i,k,2)+ &
                      eps(k,5)*dphidx(i,k,3)
            a3(k,i) = eps(k,3)*dphidx(i,k,1)+eps(k,5)*dphidx(i,k,2)+ &
                      eps(k,6)*dphidx(i,k,3)
         end do
      end do

      dphidx => B%phider

      do i = 1, n

!     --- second derivative part

         do j = 1, n
            som = 0d0
            do k = 1, m
               som1 = 0d0
               do l = 1, n
                  som1 = som1 + a1(l,j)*dphidx(l,k,1)+ &
                                a2(l,j)*dphidx(l,k,2)+ &
                                a3(l,j)*dphidx(l,k,3)
               end do
               som = som - (psi(i,k)-phi(i,k)) * som1 * w(k)
            end do
            E%mat(j,i) = E%mat(j,i) + som
         end do

      end do

1000  call erclos ( 'sepeldiffup04' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldiffup04'

      end if  ! ( debug )

      end subroutine sepeldiffup04
