      subroutine sepelfillvar2 ( number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fills array coeff with a coefficient, where the coefficient is
!     equal to a solution or derivative of a preceding computation.
!     The value is only stored in all nodal points.
!     If the old solution is not available for all nodes, then it is
!     computed in the missing points by interpolation.
!     At this moment it is assumed that the old solution is at least
!     available in all vertex points
!     Number corresponds to the number th coefficient in common block celiar
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     element
!     fill
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      use sepmoduletypes
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: number

!     number         i    Parameter indicating which coefficient
!                         must be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivec, idegfd, i, j, interpol, ndegfd
      double precision :: som, work(B%m)
      double precision, pointer :: vecloc(:), veclocnode(:)
      integer, pointer :: index4(:)

!     i              General loop variable
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     index4         Array of length inpelm+1 containing the
!                    number of degrees of freedom in the old solution
!                    vectors per point, accumulated with respect to the
!                    element to be considered (positions 1-inpelm)
!                    Pos inpelm+1 contains the following extra
!                    information:
!                     -2:  Array i not filled
!                     -1:  non-constant number of degrees of freedom
!                      1:  Array i has exactly one degree of freedom per
!                          point
!                      2:  Array i has a constant number of degrees of
!                          freedom per point
!                    116:  Array i is array of type 116
!                    126:  Array i is array of type 126
!                    129:  Array i is array of type 129
!     interpol        Indicates how the coefficients must be interpolated
!                     Possibilities:
!                      0:  Interpolation by the basis functions
!                      1:  Linear interpolation defined in integration points
!     ivec           Sequence number of vector in array oldsols.
!                    Hence array oldsols(IVEC) is considered
!     j              General loop variable
!     ndegfd         Number of degrees of freedom per point in array
!                    oldsols(IVEC)
!     som            Help variable to compute a sum
!     vecloc         Contains solution in integration points
!     veclocnode     Contains solution in nodal points
!     work           work array to store a part of the coefficient
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivec

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIVEC         Return with vector number corresponding to ivecin
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2049   unknown not present in all nodes in 3D not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelfillvar2' )
      debug = .false. .and. ioutp>=0

      if ( B%type_basisfunc==30 ) n = E%npelm

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelfillvar2'
         write(irefwr,1) 'number,inpelm', number, E%npelm
         write(irefwr,1) 'kstep, m, nvert, irule', kstep, m, nvert, B%rule
         write(irefwr,1) 'n, jdiag', n, B%jdiag
         write(irefwr,1) 'E%ind(number)', E%ind(number)
         write(irefwr,1) 'E%mst(number)', E%mst(number)
         write(irefwr,1) 'E%ich(number)', E%ich(number)
         write(irefwr,1) 'ivecnums(number)', E%ivecnums(number)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      interpol = irule/100
      ipos = E%ivecnums(number)
      ivec = inivec ( E%iuser(ipos) )
      if ( oldsols(ivec)%iseqtype==0 ) go to 1000
      itype  = E%itype

      idegfd = max(1,E%mst(number))

      if ( debug ) write(irefwr,1) 'inpelm, ivec, idegfd', inpelm, ivec, idegfd

      vecloc => oldsols(ivec)%locvecs(1:m,idegfd)
      veclocnode => oldsols(ivec)%locvecsnode(1:n,idegfd)
      iseqtype = oldsols(ivec)%iseqtype
      index4 => indoldsols(iseqtype)%index4(1:E%npelm+1)

      if ( E%ich(number)==0 ) then

!     --- E%ich(number) = 0, constant value per element

         som = vecloc(1)
         do i = kstep, m, kstep
            E%coeffs(i,number) = som
         end do  ! i = kstep, m, kstep
         E%coeffsnode(1:n,number) = som
         go to 1000

      else if ( E%ich(number)==1 ) then

!     --- in each point there is exactly one unknown

         E%coeffs(1:m,number) = vecloc(1:m)
         E%coeffsnode(1:n,number) = veclocnode(1:n)
         go to 1000

      else if ( E%ich(number)==2 ) then

!     --- in each point the number of unknowns is constant

         E%coeffs(1:m,number) = vecloc(1:m)
         E%coeffsnode(1:n,number) = veclocnode(1:n)
         go to 1000

      else if ( E%ich(number)==3 ) then

!     --- There is exactly one unknown in the vertices only

         if ( n==27 ) then

!        --- quadratic hexahedron

            work(ivertx14(1:8)) = vecloc(1:8)

         else

!        --- standard situation

            do i = 1, B%nvert*2, 2
               work(i) = vecloc((i+1)/2)
            end do

         end if
         E%coeffsnode(1:n,number) = veclocnode(1:n)

      else if ( E%ich(number)==4 ) then

!     --- in each vertex the number of unknowns is constant
!         Outside the vertices there are no unknowns

         do i = 1, B%nvert*2 ,2
            work(i) = vecloc(i)
         end do
         E%coeffsnode(1:n,number) = veclocnode(1:n)

      else if ( E%ich(number)==5 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes

         if ( debug ) then
            write(irefwr,1) 'idegfd', idegfd
         end if
         work(1:m) = vecloc(1:m)
         E%coeffsnode(1:n,number) = veclocnode(1:n)

      else if ( E%ich(number)==7 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes except centroid

         work(1) = veclocnode(1)
         do i = 2, inpelm-1
            work(i) = veclocnode(i)
         end do  ! i = 1, inpelm

         if ( inpelm==7 ) then

!        --- inpelm = 7

            work(7) = (4d0*(work(2)+work(4)+work(6))- &
                       work(1)-work(3)-work(5))/9d0

         else

!        --- inpelm # 7

            som = work(1)
            do i = 2, inpelm-1
               som = som + work(i)
            end do
            work(inpelm) = som/inpelm

         end if  ! ( inpelm==7 )
         E%coeffsnode(1:n,number) = work(1:n)

      else if ( E%ich(number)==8 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes except centroids
!         of tetrahedrons
!         First fill vertices and mid-points

         work(1) = vecloc(1)
         do i = 2, 10
            work(i) = vecloc(index4(i-1)+1)
         end do  ! i = 1, inpelm

!        --- Next fill the 4 centroids of the faces

         work(11) = (4d0*(work(2)+work(4)+work(6))-work(1)-work(3)-work(5))/9d0
         work(12) = (4d0*(work(2)+work(8)+work(7))-work(1)-work(3)-work(10))/9d0
         work(13) = (4d0*(work(4)+work(8)+work(9))-work(3)-work(5)-work(10))/9d0
         work(14) = (4d0*(work(6)+work(7)+work(9))-work(1)-work(5)-work(10))/9d0

         if ( inpelm==15 ) then

!        --- inpelm = 15, centroid of element

            work(15) = 0.25d0*(work(11)+work(12)+work(13)+work(14))

         end if  ! ( inpelm==15 )
         E%coeffsnode(1:n,number) = work(1:n)

      else if ( E%ich(number)>2000 ) then

!     --- Special situation of vector of type 129 with E%ich(number)-2000
!         degrees of freedom per element

         do i = 1, (E%ich(number)-2000)*m
            E%coeffs(i,number) = vecloc(i)
         end do
         E%coeffsnode(1:n,number) = 0d0

      else if ( E%ich(number)>1000 ) then

!     --- Special situation of vector of type 126 with E%ich(number)-1000
!         degrees of freedom per element

         if ( B%jdiag==1 ) then

!        --- The nodes are also integration points

            ndegfd = E%ich(number)-1000
            do j = 1, ndegfd
               do i = 1, n
                  E%coeffs(i,number+j-1) =  vecloc(j)
               end do
            end do

         else

!        --- Gauss integration points, store in work and then compute coeff

            ndegfd = E%ich(number)-1000
            do j = 1, ndegfd
               do i = 1, n
                  work(i) = vecloc(j)
               end do
               do i = 1, m
                  E%coeffs(i,number+j-1) = &
                     dot_product ( work(1:n), B%phi(1:n,i) )
               end do
            end do

         end if
         E%coeffsnode(1:n,number) = 0d0

      else if ( E%ich(number)>100 ) then

!     --- Special situation of vector of type 116 with E%ich(number)-100
!         degrees of freedom per element

         do i = 1, E%ich(number)-100
            som = oldsols(ivec)%locvecs(1,i)
            E%coeffs(i,number+i-1) = som
            E%coeffsnode(1:n,number+i-1) = som
         end do
         go to 1000

      else

!     --- The number of unknowns is variable
!         The unknown is present in all vertices, but not in all nodes

         work(1) = vecloc(1)

         if ( inpelm==3 ) then

!        --- inpelm = 3:  quadratic one-dimensional elements

            work(3)=vecloc(index4(4)+1)

         else if ( inpelm==6 .or. inpelm==7 .or. inpelm==9 ) then

!        --- inpelm = 6, 7 or 9:  quadratic two-dimensional elements

            do i = 3, inpelm-1, 2
               work(i) = vecloc(index4(2*i-2)+1)
            end do

         else if ( inpelm==27 ) then

!        --- inpelm = 27:  quadratic hexahedron

            work(1) = vecloc(index4(1))
            do i = 1, 8
               work(i) = vecloc(index4(i-1)+1)
            end do

         else

!        --- This possibility has not yet been implemented in ELFLR8

            call errsub ( 2049, 0, 0, 0 )
            go to 1000

         end if
         E%coeffsnode(1:n,number) = work(1:n)

      end if

      if ( E%ich(number)==3 .or. E%ich(number)==4 .or. E%ich(number)==6 ) then

!     --- The coefficients have only been computed in the vertices
!         Interpolate to the other nodes

         if ( n==3 ) then

!        --- n = 3:  quadratic one-dimensional elements

            work(2) = 0.5d0* ( work(1) + work(3) )

         else if ( n==6 .or. n==7 .or. n==9 ) then

!        --- n = 6, 7 or 9:  quadratic two-dimensional elements

            work(2*B%nvert+1) = work(1)
            do i = 1, B%nvert
               work(2*i) = 0.5d0 * ( work(2*i-1) + work(2*i+1) )
            end do

            if ( n==7 ) then

!           --- n = 7, fill 7 th point

               work(n) = ( work(1) + work(3) + work(5) ) / 3d0

            else if ( n==9 ) then

!           --- n = 9, fill 9 th point

               work(n) = ( work(1) + work(3) + work(5) + work(7) )*.25d0

            end if

         else if ( n==27 ) then

!        --- n = 27, fill midside points and centroids

            work(10) = 0.5d0*(work(1)+work(19))
            work(12) = 0.5d0*(work(3)+work(21))
            work(16) = 0.5d0*(work(7)+work(25))
            work(18) = 0.5d0*(work(9)+work(27))
            do i = 0, 18, 9
               work(2+i) = 0.5d0*(work(1+i)+work(3+i))
               work(4+i) = 0.5d0*(work(1+i)+work(7+i))
               work(6+i) = 0.5d0*(work(3+i)+work(9+i))
               work(8+i) = 0.5d0*(work(7+i)+work(9+i))
               work(5+i) = 0.25d0*(work(1+i)+work(3+i)+work(7+i)+work(9+i))
               work(2+i) = 0.5d0*(work(1+i)+work(3+i))
               work(4+i) = 0.5d0*(work(1+i)+work(7+i))
               work(6+i) = 0.5d0*(work(3+i)+work(9+i))
               work(8+i) = 0.5d0*(work(7+i)+work(9+i))
               work(5+i) = 0.25d0*(work(1+i)+work(3+i)+work(7+i)+work(9+i))
            end do

         else

!        --- This possibility has not yet been implemented in ELFLR8

            call errsub ( 2049, 0, 0, 0 )
            go to 1000

         end if
         E%coeffsnode(1:n,number) = work(1:n)

      end if

      if ( itype/=220 .and. inpelm<n .and. E%ich(number)<100 ) then

!     --- inpelm<n, special situation
!                   At this moment only 6 point triangles with dummy
!                   centroid are allowed
!                   Elements for the biharmonic equation are excluded,
!                   since n=2*inpelm due to extra degrees of freedom

         if ( n==7 ) then
            if ( interpol==0 ) then
               work(n) = ( -work(1)-work(3)-work(5) + &
                         4d0 * ( work(2)+work(4)+work(6) ) ) / 9d0
            else
               work(n) = ( work(2)+work(4)+work(6) ) /3d0
            end if
         else if ( n==4 ) then
            work(n) = ( work(1)+work(2)+work(3) ) /3d0
         else if ( n==5 ) then
            work(n) = ( work(1)+work(2)+work(3)+work(4) ) * 0.25d0
         end if
         E%coeffsnode(1:n,number) = work(1:n)

      end if

      if ( inpelm==6 .and. n==3 ) then

!     --- Special case of non-conforming triangle

         do i = 1, 3
            work(i) = work(2*i)
         end do  ! i = 1, 3
         E%coeffsnode(1:n,number) = work(1:n)

      end if  ! ( inpelm==6 .and. n==3 )

      if ( debug ) call prinrl ( work, m, 'work' )

      if ( B%jdiag==1 ) then
         do i = kstep, m, kstep
            E%coeffs(i,number) = work(i)
         end do
      else
         do i = 1, m
            E%coeffs(i,number) = &
               dot_product ( E%coeffsnode(:,number), B%phi(:,i) )
         end do  ! i = 1, m
      end if  ! ( B%jdiag==1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%coeffsnode(1,number), n, 'coeffsnode' )
         call prinrl ( E%coeffs(1,number), m, 'coeff' )
         write(irefwr,*) 'End sepelfillvar2'

      end if  ! ( debug )
      call erclos ( 'sepelfillvar2' )

      end subroutine sepelfillvar2
