      subroutine sepellinquad ( index1, inpelm, vecinp, vecout, nvert )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     copy the input vector, which is a vector
!     corresponding to the vertices of quadratic elements onto the output
!     vector. All midside points are filled by linear interpolation.
!     Restriction:  At this moment only the shape numbers: 2, 4, 6, 14
!                   are available
!                   Both the input vector and output vector must contain
!                   one unknown per point (in vertices or all points)
!     Old subroutine eld010
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletypes
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, index1(inpelm), nvert
      double precision vecinp(*), vecout(*)

!     index1     i    Array of length inpelm containing the nodal point numbers
!                     of the element
!     inpelm     i    Number of nodes in the element
!     nvert      i    Number of vertices
!     vecinp     i    Input vector containing the solution in the vertices
!     vecout     o    Output vector containing the solution in all points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer index2(8), index3(27), i
      double precision phi(8)

!     i          General loop variable
!     index2     This array contains the positions of the vertices in the
!                input vector (2D)
!     index3     This array contains the positions of all points in the
!                output vector
!     phi        contains the values in the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store positions of input vector in array index2
!     Store positions of output vector in array index3
!     Copy input vector into output array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepellinquad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepellinquad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Put positions of input vector into index2

      if ( ishape<10 ) then

!     --- 1d or 2d elements

         do i = 1, nvert
            index2(i) = index1(2*i-1)
         end do ! i = 1, nvert

      else if ( ishape==12 )then

!     --- quadratic tetrahedron

         index2(1:4) = index1(ivertx12(1:4))

      else if ( ishape==14 )then

!     --- quadratic hexahedron

         index2(1:8) = index1(ivertx14(1:8))

      end if  ! ( ishape<10 )

      if ( indprf>0 ) index2(1:nvert) = kprobf(index2(1:nvert))+1

!     --- Put positions of output vector into index3

      index3(1:inpelm) = index1(1:inpelm)

      if ( indprfo>0 ) index3(1:inpelm) = kprobfo(index3(1:inpelm))+1

!     --- An extra value index3(inpelm+1) = index3(1) is added to array index3
!         to be able to calculate vecout(index3(inpelm)) at a later stage.

      if ( ishape<10 ) index3(inpelm+1) = index3(1)

!     --- Copy input vector into output vector

      if( ishape<10 )then

!     --- 1d or 2d elements

         do i = 1, nvert
            vecout(index3(2*i-1)) = vecinp(index2(i))
         end do

      else if ( ishape==12 )then

!     --- quadratic tetrahedron

         vecout(index3(ivertx12(1:4))) = vecinp(index2(1:4))

      else if ( ishape==14 )then

!     --- quadratic hexahedron

         vecout(index3(ivertx14(1:8))) = vecinp(index2(1:8))

      end if  ! ( ishape<10 )

!     --- Fill mid points

      if ( ishape<10 ) then
         do i = 2, inpelm, 2
            vecout(index3(i)) = 0.5d0 * ( vecout(index3(i-1)) + &
                                          vecout(index3(i+1)) )
         end do

         if ( inpelm==6 .or. inpelm==7 ) then

!        --- quadratic triangles

            vecout(index3(6)) = 0.5d0 * ( vecout(index3(1)) + &
                                          vecout(index3(5)) )
            if( inpelm==7 )vecout(index3(7)) = ( vecout(index3(1)) + &
               vecout(index3(5)) + vecout(index3(3)) ) /3d0

         else if ( inpelm==9 ) then

!        --- quadratic rectangles

            vecout(index3(8)) = 0.5d0 * ( vecout(index3(7)) + &
                                          vecout(index3(1)) )
            vecout(index3(9)) = 0.25d0 * ( vecout(index3(1)) + &
                 vecout(index3(3)) + vecout(index3(5)) + &
                 vecout(index3(7)) )

         end if

      else if ( ishape==14 ) then

         phi(1)= vecout(index3(1))
         phi(2)= vecout(index3(3))
         phi(3)= vecout(index3(7))
         phi(4)= vecout(index3(9))
         phi(5)= vecout(index3(19))
         phi(6)= vecout(index3(21))
         phi(7)= vecout(index3(25))
         phi(8)= vecout(index3(27))

         vecout(index3(2))=0.5d0*phi(1) + 0.5d0*phi(2)
         vecout(index3(4))=0.5d0*phi(1) + 0.5d0*phi(3)
         vecout(index3(5))=0.25d0*phi(1) + 0.25d0*phi(2) &
                           + 0.25d0*phi(3) + 0.25d0*phi(4)
         vecout(index3(6))=0.5d0*phi(2) + 0.5d0*phi(4)
         vecout(index3(8))=0.5d0*phi(3) + 0.5d0*phi(4)
         vecout(index3(10))=0.5d0*phi(1) + 0.5d0*phi(5)
         vecout(index3(11))=0.25d0*phi(1) + 0.25d0*phi(2) &
                            + 0.25d0*phi(5) + 0.25d0*phi(6)
         vecout(index3(12))=0.5d0*phi(2) + 0.5d0*phi(6)
         vecout(index3(13))=0.25d0*phi(1) + 0.25d0*phi(3) &
                            + 0.25d0*phi(5) + 0.25d0*phi(7)
         vecout(index3(14))=0.125d0*phi(1) + 0.125d0*phi(2) &
                            + 0.125d0*phi(3) + 0.125d0*phi(4) &
                            + 0.125d0*phi(5) + 0.125d0*phi(6) &
                            + 0.125d0*phi(7) + 0.125d0*phi(8)
         vecout(index3(15))=0.25d0*phi(2) + 0.25d0*phi(4) &
                            + 0.25d0*phi(6) + 0.25d0*phi(8)
         vecout(index3(16))=0.5d0*phi(3) + 0.5d0*phi(7)
         vecout(index3(17))=0.25d0*phi(3) + 0.25d0*phi(4) &
                            + 0.25d0*phi(7) + 0.25d0*phi(8)
         vecout(index3(18))=0.5d0*phi(4) + 0.5d0*phi(8)
         vecout(index3(20))=0.5d0*phi(5) + 0.5d0*phi(6)
         vecout(index3(22))=0.5d0*phi(5) + 0.5d0*phi(7)
         vecout(index3(23))=0.25d0*phi(5) + 0.25d0*phi(6) &
                            + 0.25d0*phi(7) + 0.25d0*phi(8)
         vecout(index3(24))=0.5d0*phi(6) + 0.5d0*phi(8)
         vecout(index3(26))=0.5d0*phi(7) + 0.5d0*phi(8)

      end if

1000  call erclos ( 'sepellinquad' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepellinquad'

      end if  ! ( debug )

      end subroutine sepellinquad
