      subroutine sepelm804
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for discontinuous boundary conditions for
!     second order elliptic equations
!     Old subroutine elm804
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelmsav
      double precision, pointer :: sigma(:), hrhs(:)

!     hrhs           Contains right-hand side h in quadrature points
!     i              Counting variable
!     inpelmsav      Help parameter to store inpelm
!     sigma          Contains sigma in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0804      Initialize parameters for subroutine SEPELM804
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELLOAD      Fill the element vector with a load
!     SEPELMASSMAT1  Fill the element mass matrix; one unknown per node
!                    Consistent mass matrix
!     SEPELOLDVECS   Fill old vectors for element
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Not yet in use:  set equal to 0
!     2:  Not yet in use:  set equal to 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  Not yet in use:  set equal to 0
!     6:  sigma
!     7:  h
!     8-15: Not yet in use (must be zero)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Boundary conditions:
!       sum  sum alpha   dc/dx n  + sigma c - c = h
!        i    j       ij      j i          u   l
!        The following formulae are used:
!       Type 2:  S(i,j) = /  sigma phi  phi  de
!                         e           i    j
!                f(i)   = /  h phi  de
!                         e       i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
!        The element matrix becomes:   |  S   -S |   and the vector | f |
!                                      |  S   -S |                  | f |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm804' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm804'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepel0804

!     --- Divide E%npelm temporary by 2, in order to fake the elp subroutines

      E%npelm = E%npelm/2
      inpelmsav = inpelm

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill all coefficients

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

      sigma => E%coeffs(:,1)
      hrhs => E%coeffs(:,2)

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix
!         First the sub element matrix is computed and stored in submat

         E%submat = 0d0
         call sepelmassmat1 ( sigma ) ! sigma phi_i phi_j

!        --- Next the subelement matrix is copied four times into E%mat

         E%mat(1:E%npelm,1:E%npelm) = E%submat
         E%mat(1:E%npelm,E%npelm+1:) = E%submat
         E%mat(E%npelm+1:,1:E%npelm) = -E%submat
         E%mat(E%npelm+1:,E%npelm+1:) = -E%submat

      end if

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         E%vec = 0d0
         call sepelload ( hrhs ) ! h phi_i

!        --- Copy element vector again in last part

         E%vec(1:E%npelm) = E%subvec
         E%vec(E%npelm+1:) = E%subvec

      end if

!     --- Reset E%npelm

      E%npelm = E%npelm*2
      inpelm = inpelmsav

      if ( E%last==1 ) then
         call sepeldealloc
         deallocate ( E%submat, E%subvec )
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm804'

      end if  ! ( debug )
      call erclos ( 'sepelm804' )

      end subroutine sepelm804
