      subroutine sepelp02curv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill dxdxi and dphidx for a curved 1d element
!     Old subroutine elp611curved
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: x(:,:), phider(:,:), w(:), dphidx(:,:)
      double precision, pointer :: dxdxi(:,:)
      integer :: m, k, j
      double precision :: dx, dy, delta

!     delta          Jacobian of the element in an integration point
!     dphidx         derivatives of basis functions
!     dx             "distance in x-direction" between two nodes
!     dxdxi          dx/dxi, dx/deta, dy/dxi and dy/deta
!     dy             "distance in y-direction" between two nodes
!     j              Counting variable
!     k              Counting variable
!     m              number of quadrature points
!     phider         Contains the derivatives of the basis functions in the
!                    reference element
!                    phider(i,j) = d phi_i/ ds (x^j)
!     w              array of length m containing the weights for
!                    quadrature
!     x              co-ordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR0    Print error message errornr for element
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp02curv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp02curv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider(:,:,1)
      phider => B%phiderc(:,:,1)
      w => B%w
      x => B%xyz
      dxdxi => B%dxdxig(:,:,1)
      m = B%m

!     --- Compute dx/dxi = sum_i x_i d phi_i / dxi

      do k = 1, E%ndim
         do j = 1, m
            dxdxi(j,k) = sum(x(1:3,k)*phider(1:3,j))
         end do  ! j = 1, m
      end do  ! k = 1, ndim

      if ( debug ) call prinrl1 ( dxdxi, E%ndim, m, 'dxdxi' )

!     --- Compute weights

      if ( B%rule==1 ) then

!     --- Newton Cotes rule

         dx = -1.5d0*x(1,1)+2d0*x(2,1)-0.5d0*x(3,1)
         dy = -1.5d0*x(1,2)+2d0*x(2,2)-0.5d0*x(3,2)
         delta = sqrt(dx*dx+dy*dy)
         if ( E%ktang>0 ) then
            B%tang(1,1) = dx/delta
            B%tang(1,2) = dy/delta
         end if  ! ( E%ktang>0 )
         w(1) = 2d0*delta
         dx = 0.5d0 * ( x(3,1)-x(1,1) )
         dy = 0.5d0 * ( x(3,2)-x(1,2) )
         delta = sqrt(dx*dx+dy*dy)
         if ( E%ktang>0 ) then
            B%tang(2,1) = dx/delta
            B%tang(2,2) = dy/delta
         end if  ! ( E%ktang>0 )
         w(2) = 2d0*delta
         dx = 0.5d0 * x(1,1)-2d0*x(2,1)+1.5d0*x(3,1)
         dy = 0.5d0 * x(1,2)-2d0*x(2,2)+1.5d0*x(3,2)
         delta = sqrt(dx*dx+dy*dy)
         w(3) = 2d0 * delta
         if ( E%ktang>0 ) then
            B%tang(3,1) = dx/delta
            B%tang(3,2) = dy/delta
         end if  ! ( E%ktang>0 )


      else

!     --- Gauss rule

         do k = 1, m
            w(k) = sqrt(dot_product(dxdxi(k,:),dxdxi(k,:)))
         end do  ! k = 1, m

         if ( E%ktang>0 ) then
            do k = 1, E%ndim
               B%tang(:,k) = dxdxi(:,k)/w(:)
            end do  ! k = 1, E%ndim
         end if  ! ( E%ktang>0 )

      end if  ! ( B%rule==1 )

      do k = 1, m
         if ( abs(w(k))<=0d0 ) then

!        --- Error 105: h is zero

            call errorelvar0 ( 105 )
            call prinrl2(B%xyz,2,E%ndim,'co-ordinates')
            go to 1000

         end if  ! ( abs(w(k))<=0d0 )
      end do  ! k = 1, m

      if ( debug ) call prinrl ( w, m, 'w/1' )

      if ( B%fillderiv ) then

!     --- Compute derivatives

         do k = 1, 3
            dphidx(k,1:m) = phider(k,1:m) / w(1:m)
         end do

      end if

      w = w*B%wc

      if ( debug ) call prinrl ( w, m, 'w/2' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp02curv'

      end if  ! ( debug )
      call erclos ( 'sepelp02curv' )

      end subroutine sepelp02curv
