      subroutine sepelp03jacob ( e1, delta )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Linear triangle (3 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     jacobian
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(out) :: e1(3,2), delta

!     delta          o    Jacobian of the element
!     e1             o    Work array of length 3 x 2, to store the factors
!                         e^ij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: n
      double precision :: h
      double precision, pointer :: x(:), y(:)

!     h              Help variable to store part of a weight
!     n              number of basis functions
!     x              Contains x co-ordinates of nodes
!     y              Contains y co-ordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp03jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp03jacob'
         write(irefwr,1) 'n', B%n
         call prinrl1 ( B%xyz, E%ndim, 3, 'xyz' )
  1      format ( a, 1x, (10i8) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = B%n
      x => B%xloc(:,1)
      y => B%xloc(:,2)

      if ( n<6 ) then

!     --- n = 3, 4 linear triangle

         e1(1,1) = y(2) - y(3)
         e1(3,1) = y(1) - y(2)
         e1(1,2) = x(3) - x(2)
         e1(3,2) = x(2) - x(1)

      else

!     --- n = 6, quadratic triangle

         e1(1,1) = y(3) - y(5)
         e1(3,1) = y(1) - y(3)
         e1(1,2) = x(5) - x(3)
         e1(3,2) = x(3) - x(1)

      end if  ! ( n==3 )

      h = max(abs(e1(1,1)),abs(e1(1,2)),abs(e1(3,1)),abs(e1(3,2)))
      h = h**2*1d-7

      e1(2,:) = -e1(1,:)-e1(3,:)
      delta = e1(3,1)*e1(1,2) - e1(1,1)*e1(3,2)

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( e1, 2, 3, 'e1' )
         write(irefwr,2) 'delta ', delta
         write(irefwr,2) 'h', h

      end if   ! ( debug )

      if ( checksign .and. delta<h ) then

         write(irefwr,*) 'area of element ', E%ielem, ' is negative '
         write(irefwr,*) 'area = ', delta
         call prinin ( E%nodes, n,  'nodes' )
         call prinrl1 ( B%xyz, E%ndim, n, 'coordinates' )
         call ertrace
         go to 1000

      end if

!     --- Check if delta is non-zero

      if ( abs(delta)<=h ) then

!     --- delta<=0

         call errint ( E%ielem, 1 )
         call errint ( E%itype, 2 )
         call errint ( E%ielgrp, 3 )
         call errsub ( 105, 3, 0, 0 )
         call prinin(E%nodes,n,'node numbers')
         call prinrl2(B%xyz,n,2,'co-ordinates')
                  go to 1000

      end if

      e1 = e1/delta

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp03jacob'

      end if  ! ( debug )
      call erclos ( 'sepelp03jacob' )

      end subroutine sepelp03jacob
