      subroutine sepelp07det ( lambda, text, irule, m, weight, phi, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Extended quadratic triangle (7 nodes)
!     Old subroutine elp624first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irule, m
      double precision, intent(out) :: lambda(3,m), weight(m), phi(7,m), &
                                       dphidxi(7,m,2)
      character (len=*) text

!     dphidxi        o    Contains values of dphi/dxi in quadrature points
!     irule          i    type of quadrature rule
!     lambda         o    Contains values of lambda in quadrature points
!     m              i    number of quadrature points
!     phi            o    Contains values of basis functions in quadrature points
!     text           i    String variable containing some text
!     weight         o    weights in reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: h, phicen(m), dlamdk(m)

!     dlamdk         eta/xi derivative of lambda
!     h              Help parameter to compute a constant
!     i              Counting variable
!     phicen         Value of alam1*alam2*alam3 in quadrature point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELPLAMBDA2D Fill the two-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp07det' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp07det'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (irule)

      case (1)

!     --- Newton Cotes rule

         lambda = 0d0
         do i = 1, 3
            lambda(i,2*i-1) = 1d0
         end do  ! i = 1, 3
         lambda(1:2,2) = 0.5d0
         lambda(1,6) = 0.5d0
         lambda(2:3,4) = 0.5d0
         lambda(3,6) = 0.5d0
         h = 1d0 / 3d0
         lambda(1:3,7) = h
         h = 1d0/15d0
         weight(2:6:2) = h
         h = h*0.025d0
         weight(1:5:2) = h
         weight(7) = 9d0 * h

      case(2, 3)

!     --- intrule = 2, 3 : Gauss integration rules

         call sepelplambda2d ( lambda, text, irule+1, m, weight )

      case default

!     --- Quadrature not available

         call errint ( irule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (intrule)

!     --- Fill basis functions in quadrature points

      phicen = lambda(1,:)*lambda(2,:)*lambda(3,:)
      phi(7,:) = 27d0 * phicen
      do i = 1, 3
         phi(2*i-1,:) = lambda(i,:)*(2d0*lambda(i,:)-1d0) + 3d0*phicen
      end do  ! i = 1, 3
      phicen = 12d0*phicen
      phi(2,:) = 4d0 * lambda(1,:) * lambda(2,:) - phicen
      phi(4,:) = 4d0 * lambda(2,:) * lambda(3,:) - phicen
      phi(6,:) = 4d0 * lambda(3,:) * lambda(1,:) - phicen

!     --- fill d phi / d xsi and  d phi / d eta

      dlamdk = lambda(3,:) * (lambda(1,:)-lambda(2,:))
      dphidxi(1,:,1) = -4d0*lambda(1,:)+1d0+3d0*dlamdk
      dphidxi(3,:,1) =  4d0*lambda(2,:)-1d0+3d0*dlamdk
      dphidxi(5,:,1) =  3d0*dlamdk
      dphidxi(2,:,1) =  4d0*(lambda(1,:)-lambda(2,:))-12d0*dlamdk
      dphidxi(4,:,1) =  4d0* lambda(3,:)-12d0*dlamdk
      dphidxi(6,:,1) = -4d0* lambda(3,:)-12d0*dlamdk
      dphidxi(7,:,1) =  27d0*dlamdk
      dlamdk = lambda(2,:) * (lambda(1,:)-lambda(3,:))
      dphidxi(1,:,2) =  -4d0*lambda(1,:)+1d0+3d0*dlamdk
      dphidxi(3,:,2) =  3d0*dlamdk
      dphidxi(5,:,2) =  4d0*lambda(3,:)-1d0+3d0*dlamdk
      dphidxi(4,:,2) =  4d0* lambda(2,:)-12d0*dlamdk
      dphidxi(2,:,2) = -4d0* lambda(2,:)-12d0*dlamdk
      dphidxi(6,:,2) =  4d0*(lambda(1,:)-lambda(3,:))-12d0*dlamdk
      dphidxi(7,:,2) =  27d0*dlamdk

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp07det'

      end if  ! ( debug )
      call erclos ( 'sepelp07det' )

      end subroutine sepelp07det
