      subroutine sepelp07jacob
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Quadratic triangle (6 nodes)
!     Old subroutine elp624
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: x(:), y(:), z(:,:)
      double precision :: h
      logical :: curved

!     curved         indication if the element is curved (true) or not (false)
!     h              help variable to store a constant
!     x              Contains x co-ordinates of nodes
!     y              Contains y co-ordinates of nodes
!     z              Contains z and y co-ordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELP04JACCUR Compute basis functions and related quantities for
!                    quadratic triangle (curved element)
!     SEPELP07JACSTR Compute basis functions and related quantities for
!                    extended quadratic triangle (straight element)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp07jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp07jacob'
         write(irefwr,1) 'kstep, ndim, iqmat', B%kstep, E%ndim, E%iqmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz(:,1)
      y => B%xyz(:,2)
      z => B%xyz

!     --- check if element is straight or curved

      if ( E%ndim>2 .and. (E%iqmat==0 .or. E%iqmat==3) ) then

!     --- Standard 3D case. Treat as curved

         curved = .true.

      else

!     --- 2D case, compute curved

         curved = .false.
         if ( abs(x(2)+x(4)+x(6)-x(1)-x(3)-x(5)) > sqreps ) then
            curved = .true.
         else if ( abs(y(2)+y(4)+y(6)-y(1)-y(3)-y(5)) > sqreps ) then
            curved = .true.
         end if

      end if
      if ( debug ) write(irefwr,*) 'curved ', curved

      if ( E%npelm==6 ) then

!     --- 6 node element, fill seventh point

         if ( curved ) then

!        --- curved element

            z(7,:)=(4d0*(z(2,:)+z(4,:)+z(6,:))-z(1,:)-z(3,:)-z(5,:))/9d0

         else

!        --- straight element

            h = 1d0/3d0
            z(7,:)=(z(1,:)+z(3,:)+z(5,:))*h

         end if

      end if  ! ( E%npelm==6 )

      if ( .not. curved ) then

!     --- curved = false,  straight triangle

         call sepelp07jacstr

      else

!     --- curved = true,  curved triangle

         call sepelp04jaccur

      end if  ! ( .not. curved )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phider(1,1,1), B%m, 6, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), B%m, 6, 'dphidy' )
         write(irefwr,*) 'End sepelp07jacob'

      end if  ! ( debug )
      call erclos ( 'sepelp07jacob' )

      end subroutine sepelp07jacob
