      subroutine sepelp14first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     3d Tri-Quadratic hexahedron (27 nodes)
!     Old subroutine: elp633
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     hexahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indi(27,3), inds(B%m,3), m, i, j, k, m2, mrule, ih, ih1, n
      integer :: indpsi(8,3)
      double precision :: theta(3,3), dtetds(3,3), w(3), lambda(3), s
      double precision :: lam(3,3), dlamds(3,3)
      double precision, pointer :: weight(:), phi(:,:), phixi(:,:,:)
      double precision, pointer :: psi(:,:), psixi(:,:,:)
      character (len=24), parameter :: text = 'Tri-quadratic hexahedron'

!     dlamds         Derivatives of 1-dimensional linear basis functions
!     dtetds         Derivatives of theta in the integration points
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     indi           Array in which the triples indi, indj and indk are stored
!     inds           Array in which the triples inds1, inds2 and inds3 are
!                    stored
!     j              Counting variable
!     k              Counting variable
!     lam            1-dimensional linear basis functions in quadrature points
!                    Both
!     lambda         1-dimensional linear basis functions in quadrature points
!                    (only the first one)
!     m2             Contains the number of quadrature points in one direction
!     mrule          Type of integration rule as given by the user
!     n              number of basis functions
!     phi            contains basis functions
!     phixi          Array of length 8 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element.
!     psi            contains linear basis functions
!     psixi          Contains the values of the derivatives of the
!                    linear basis functions in the Gauss points in
!                    the reference element.
!     s              Parameter along edge
!     text           String variable containing some text
!     theta          1D basis funtion in integration points
!     w              weights for 1-dimensional reference element
!     weight         Contains weights for reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp14first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp14first'
         write(irefwr,1) 'intrule, m, n', B%rule, B%m, B%n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      weight => B%wc
      phi => B%phi
      phixi => B%phiderc
      psi => B%lam
      psixi => B%lamderc

      m = B%m
      n = B%n

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

!        --- Fill arrays theta and dtetds in integration points

      select case (B%rule)

      case (1)

!     --- intrule = 1  Newton Cotes rule

         theta = 0d0
         do i = 1, 3
            theta(i,i) = 1d0
            s = (i-1) * 0.5d0
            dtetds(1,i) = 4d0 * s - 3d0
            dtetds(2,i) = -8d0 * s + 4d0
            dtetds(3,i) = 4d0 * s - 1d0
         end do
         w(1) = 1d0/6d0
         w(2) = 4d0*w(1)
         w(3) = w(1)
         mrule = 3
         lambda(1) = 1d0
         lambda(2) = 0.5d0
         lambda(3) = 0d0

      case (2, 3)

!     --- intrule = 2  Eight point Gauss rule

         mrule = B%rule
         call sepelplambda1d ( lambda, text, B%rule+1, B%rule, w )
         do k = 1, mrule
            theta(1,k) = 2d0 * lambda(k) * (lambda(k)-0.5d0)
            theta(2,k) = -4d0 * lambda(k) * (lambda(k)-1d0)
            theta(3,k) = 2d0 * (lambda(k)-0.5d0) * (lambda(k)-1d0)
            dtetds(1,k) = -4d0 * lambda(k) + 1d0
            dtetds(2,k) =  8d0 * lambda(k) - 4d0
            dtetds(3,k) = -4d0 * lambda(k) + 3d0
         end do

      end select  ! case (intrule)
      m2 = mrule**2

!     --- Fill arrays lam and dlamds

      lam(1,1:mrule) = lambda(1:mrule)
      lam(2,1:mrule) = 1d0-lambda(1:mrule)
      dlamds(1,1:mrule) = -1d0
      dlamds(2,1:mrule) = 1d0

!     --- fill array indi

      do k = 1, 3
         ih = (k-1)*9
         do j = 1, 3
            ih1 = ih+(j-1)*3
            do i = 1, 3
               indi(i+ih1,1) = i
               indi(i+ih1,2) = j
               indi(i+ih1,3) = k
            end do
         end do
      end do

!     --- fill array inds

      do k = 1, mrule
         ih = (k-1)*m2
         do j = 1, mrule
            ih1 = ih+(j-1)*mrule
            do i = 1, mrule
               inds(i+ih1,1) = i
               inds(i+ih1,2) = j
               inds(i+ih1,3) = k
            end do
         end do
      end do

!     --- Compute linear basis functions

      do k = 1, 2
         indpsi(1+4*(k-1),1) = 1
         indpsi(1+4*(k-1),2) = 1
         indpsi(1+4*(k-1),3) = k
         indpsi(2+4*(k-1),1) = 2
         indpsi(2+4*(k-1),2) = 1
         indpsi(2+4*(k-1),3) = k
         indpsi(3+4*(k-1),1) = 1
         indpsi(3+4*(k-1),2) = 2
         indpsi(3+4*(k-1),3) = k
         indpsi(4+4*(k-1),1) = 2
         indpsi(4+4*(k-1),2) = 2
         indpsi(4+4*(k-1),3) = k
      end do
      if ( debug ) call prinin1 ( indpsi, 3, 8, 'indpsi' )

!     --- Fill array weight

      weight = w(inds(:,1))*w(inds(:,2))*w(inds(:,3))

      do i = 1, n
         phi(i,:) = theta(indi(i,1),inds(:,1))* &
                    theta(indi(i,2),inds(:,2))* &
                    theta(indi(i,3),inds(:,3))
      end do

      do i = 1, 8
         do k = 1, m
            psi(i,k) = lam(indpsi(i,1),inds(k,1))* &
                       lam(indpsi(i,2),inds(k,2))* &
                       lam(indpsi(i,3),inds(k,3))
         end do
      end do

!     --- fill phixi

      call sepelp13phixi ( phixi, n, m, 3, theta, dtetds, indi, inds )

      if ( B%fillderivn ) then

!     --- Fill also phidercn

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            B%phidercn = phixi

         else

!        --- Gauss rule, use formulas from Newton Cotes

            theta = 0d0
            do i = 1, 3
               theta(i,i) = 1d0
               s = (i-1) * 0.5d0
               dtetds(1,i) = 4d0 * s - 3d0
               dtetds(2,i) = -8d0 * s + 4d0
               dtetds(3,i) = 4d0 * s - 1d0
            end do

            phixi => B%phidercn

            call sepelp13phixi ( phixi, n, n, 3, theta, dtetds, indi, indi )

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderivn )

!     --- Fill dpsi/dxi

      do i = 1, 8
         do k = 1, m
            psixi(i,k,1) = dlamds(indpsi(i,1),inds(k,1))* &
                           lam(indpsi(i,2),inds(k,2))* &
                           lam(indpsi(i,3),inds(k,3))
            psixi(i,k,2) = lam(indpsi(i,1),inds(k,1))* &
                           dlamds(indpsi(i,2),inds(k,2))* &
                           lam(indpsi(i,3),inds(k,3))
            psixi(i,k,3) = lam(indpsi(i,1),inds(k,1))* &
                           lam(indpsi(i,2),inds(k,2))* &
                           dlamds(indpsi(i,3),inds(k,3))
         end do
      end do

1000  call erclos ( 'sepelp14first' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( weight, m, 'weight' )
         call prinrl1 ( phi, m, n, 'phi' )
         call prinrl1 ( phixi(1,1,1), m, n, 'dphidxi_1' )
         call prinrl1 ( phixi(1,1,2), m, n, 'dphidxi_2' )
         call prinrl1 ( phixi(1,1,3), m, n, 'dphidxi_3' )
         write(irefwr,*) 'End sepelp14first'

      end if  ! ( debug )

      end subroutine sepelp14first
