      subroutine sepelp17first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     Extended tri-linear hexahedron (9 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indi(8,3), inds(B%m,3), m, i, j, k, m2, mrule, ih, ih1, n
      double precision theta(2,2), dtetds(2,2), w(2), lambda(2), h, phic
      double precision, pointer :: weight(:), phi(:,:), phixi(:,:,:)
      character (len=30), parameter :: text = 'Extended tri-linear hexahedron'

!     dtetds         Derivatives of theta in the integration points
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     indi           Array in which the triples indi, indj and indk are stored
!     inds           Array in which the triples inds1, inds2 and inds3 are
!                    stored
!     j              Counting variable
!     k              Counting variable
!     lambda         1-dimensional basis functions in quadrature points
!     m              number of quadrature points
!     m2             Contains the number of quadrature points in one direction
!     mrule          Type of integration rule as given by the user
!     n              number of basis functions
!     phi            contains basis functions
!     phic           Value of phi_c
!     phixi          Array of length 8 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element.
!     text           String variable containing some text
!     theta          1D basis funtion in integration points
!     w              weights for 1-dimensional reference element
!     weight         Contains weights for reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRSUB         Error messages
!     SEPELP17PHIXI  Compute dphi/dxi in quadrature points
!                    3d Tri-linear hexahedron (9 nodes)
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp17first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp17first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      weight => B%wc
      phi => B%phi
      phixi => B%phiderc

      m = B%m
      n = B%n

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      select case (B%rule)

      case (3)

!     --- Gauss rules

         mrule = 2
         call sepelplambda1d ( lambda, text, 3, mrule, w )
         dtetds(1,:) = -1d0
         dtetds(2,:) =  1d0
         do k = 1, 2
            theta(1,k) = 1d0 - lambda(k)
            theta(2,k) = lambda(k)
         end do
         mrule = 2
         m2 = mrule**2

!        --- fill array indi

         indi(1,1) = 1
         indi(2,1) = 2
         indi(3,1) = 2
         indi(4,1) = 1
         indi(1,2) = 1
         indi(2,2) = 1
         indi(3,2) = 2
         indi(4,2) = 2
         do k = 1, 4
            indi(k,3) = 1
            indi(k+4,1) = indi(k,1)
            indi(k+4,2) = indi(k,2)
            indi(k+4,3) = 2
         end do

!        --- fill array inds

         do k = 1, mrule
            ih = (k-1)*m2
            do j = 1, mrule
               ih1 = ih+(j-1)*mrule
               do i = 1, mrule
                  inds(i+ih1,1) = i
                  inds(i+ih1,2) = j
                  inds(i+ih1,3) = k
               end do
            end do
         end do

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

!     --- Fill array weight

      weight = w(inds(:,1))*w(inds(:,2))*w(inds(:,3))

!     --- fill phi

      do k = 1, m
         phic = theta(1,inds(k,1))*theta(2,inds(k,1))* &
                theta(1,inds(k,2))*theta(2,inds(k,2))* &
                theta(1,inds(k,3))*theta(2,inds(k,3))
         h = 8d0*phic
         do i = 1, 8
            phi(i,k) = theta(indi(i,1),inds(k,1))* &
                       theta(indi(i,2),inds(k,2))* &
                       theta(indi(i,3),inds(k,3))-h
         end do
         phi(9,k) = 8d0*h
      end do

!     --- fill phixi

      call sepelp17phixi ( phixi, n, m, 2, theta, dtetds, indi, inds )

      if ( B%fillderivn ) then

!     --- Fill also phidercn

         phixi => B%phidercn

         call sepelp17phixi ( phixi, n, n, 2, theta, dtetds, indi, indi )

!     --- Fill basis functions in quadrature points

      end if  ! ( B%fillderivn )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp17first'

      end if  ! ( debug )
      call erclos ( 'sepelp17first' )

      end subroutine sepelp17first
