      subroutine sepelp18
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Extended linear tetrahedron (5 nodes)
!     Old subroutine elp635
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: e1(4,3), djacob, h, phicen, gradc
      double precision, pointer :: lambda(:,:), dphidx(:,:,:)
      integer :: k, i, j

!     djacob         Jacobian of the element
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     e1             Work array of length 3 x 2, to store the factors
!                    e^ij See method
!     gradc          Gradient of lambda_1 * lambda_2 * lambda_3
!     h              Help variable to store part of a weight
!     i              Counting variable
!     j              Counting variable
!     k              Loop variable
!     lambda         Array containing the linear basis functions in a point
!     phicen         Value of alam1*alam2*alam3 in quadrature point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELP11JACOB  Compute Jacobian and derivatives of basis functions
!                    Linear tetrahedrons
!     SEPELP18FIRST  compute some element independent quantities
!                    3d linear tetrahedron (5 nodes)
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+1,j+2
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                             ij
!        d lambda  / dx   =  e   /  djacob
!                i     j
!        lamc = lambda lambda lambda lambda
!                     1      2      3      4
!        phi  =  lambda  - 64 lamc  i=1,2,3,4   (basis functions:1,2,3,4)
!           i          i
!        phi  = 256 lamc
!           5
!        grad phi  = grad lambda  - 64 grad lamc
!                i              i
!        grad phi    = 256 grad lamc
!                5
!        grad lamc = lambda_2 lambda_3 lambda_4 grad lambda_1 + &
!                    lambda_3 lambda_4 lambda_1 grad lambda_2 + &
!                    lambda_4 lambda_1 lambda_2 grad lambda_3 + &
!                    lambda_1 lambda_2 lambda_3 grad lambda_4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp18' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp18'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider
      lambda => B%lam
      m = B%m

      if ( E%first==0 ) call sepelp18first
      if ( ierror/=0 ) go to 1000

      if ( E%npelm==4 ) &
         B%xyz(5,:) = ( B%xyz(1,:)+B%xyz(2,:)+B%xyz(3,:)+B%xyz(4,:) ) * 0.25d0

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of linear basis functions

      call sepelp11jacob ( e1, djacob )
      if ( ierror/=0 ) go to 1000

!     --- Fill weights

      if ( B%rule==1 ) then

!     --- Newton Cotes rule

         h = abs(djacob)*B%wc(1)
         phicen = abs(djacob)/5040d0
         B%w(1:4) = h-64d0*phicen
         B%w(5) = 256d0*phicen

      else

!     --- Gauss rule

         B%w = abs(djacob)*B%wc

      end if  ! ( B%rule==1 )
      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            do i = 1, 5
               do k = 1, 3
                  dphidx(1:4,i,k) = e1(1:4,k)
               end do
               dphidx(5,i,1:3) = 0d0
            end do  !  i = 1, 5

         else

!        --- Gauss rule

            do j = 1, 3
               do k = 1, m
                  gradc = lambda(2,k)*lambda(3,k)*lambda(4,k)*e1(1,j) + &
                          lambda(3,k)*lambda(4,k)*lambda(1,k)*e1(2,j) + &
                          lambda(4,k)*lambda(1,k)*lambda(2,k)*e1(3,j) + &
                          lambda(1,k)*lambda(2,k)*lambda(3,k)*e1(4,j)
                  do i = 1, 4
                     dphidx(i,k,j) = e1(i,j)-64d0*gradc
                  end do
                  dphidx(5,k,j) = 256d0*gradc
               end do  ! k = 1, m
            end do  ! j = 1, 3

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderiv )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp18'

      end if  ! ( debug )
      call erclos ( 'sepelp18' )

      end subroutine sepelp18
