      subroutine sepelp21
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     linear wedge element
!     Old subroutine elp638
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: jac(B%m)
      integer :: k, i

!     i              Counting variable
!     jac            Array of length m containing the values of the
!                    Jacobian in the vertices or for the Gauss integration
!                    rules work array to store some help variables
!     k              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP13JACOB  Compute Jacobian and derivatives of basis functions
!                    3d Tri-linear hexahedron (8 nodes)
!     SEPELP21FIRST  compute some element independent quantities
!                    linear wedge element
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     To compute the derivatives of the basis functions, the jacobian
!     of the element and the weights, the following definitions are used:
!     The lower triangle is defined by the points 1,2,3, the upper
!     triangle by 4,5,6
!     The basis functions are mapped on a unit prism in the (xi, eta ,zeta )
!     space with basis functions:
!
!     phi_1 = lambda_1(xi,eta) lambda_1(zeta)
!     phi_2 = lambda_2(xi,eta) lambda_1(zeta)
!     phi_3 = lambda_3(xi,eta) lambda_1(zeta)
!     phi_4 = lambda_1(xi,eta) lambda_2(zeta)
!     phi_5 = lambda_2(xi,eta) lambda_2(zeta)
!     phi_6 = lambda_3(xi,eta) lambda_2(zeta)
!
!     lambda_1(xi,eta) = 1 -xi - eta
!     lambda_2(xi,eta) = xi
!     lambda_3(xi,eta) = eta
!
!     lambda_1(zeta) = 1-zeta
!     lambda_2(zeta) = zeta
!
!     Hence the triangles are in the xi,eta planes and the upwards direction
!     is defined as zeta
!
!     The weights for the numerical integration rule are given as follows:
!     Let J be the Jacobian in the integration points then:
!
!     intrule = 1: (Newton Cotes rule)  w = 1/12 J(x_k)
!     intrule = 2: (One point Gauss rule)  w = 1/2 J(x_k)
!     intrule = 3: (Eight point Gauss rule)  w = 1/2 wgs2(k3) J(x_k)
!        (k3 is k modulo 3)
!
!     The integration points for the numerical integration rule are given as
!     follows:
!
!     intrule = 1: (Newton Cotes rule)  6 vertices
!     intrule = 2: (One point Gauss rule)  1 centroid
!     intrule = 3: (Eight point Gauss rule)
!       (xi_k,eta_k,zeta_k) = alam2(k3) wgs2(kz)
!       (kz = (k-1)/3+1)
!
!     dphi_dxi_j can be computed immediately from the standard basis functions:
!
!     dphi_i/dxi_j        xi              eta            zeta
!         1           -(1-zeta)        -(1-zeta)        -(1-xi-eta)
!         2             1-zeta             0               -xi
!         3                0             1-zeta            -eta
!         4              -zeta            zeta            1-xi-eta
!         5               zeta             0                xi
!         6                0              zeta              eta
!
!     dx/dxi_j = sum_k x_k dphi_k/dxi_j
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp21' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp21'
         write(irefwr,1) 'irule, n, m', B%rule, B%n, B%m
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepelp21first

      if ( debug ) then
         call prinrl1 ( B%phi, B%m, B%n, 'phi' )
         call prinrl1 ( B%phiderc(1,1,1), B%m, B%n, 'dphi/dxi1' )
         call prinrl1 ( B%phiderc(1,1,2), B%m, B%n, 'dphi/dxi2' )
         call prinrl1 ( B%phiderc(1,1,3), B%m, B%n, 'dphi/dxi3' )
         call prinrl ( B%wc, B%m, 'weight')
      end if  ! ( debug )

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp13jacob ( jac )

      if ( debug ) call prinrl ( jac, B%m, 'jacobian' )

!     --- Fill weights

      B%w = abs(jac)*B%wc
      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phider(1,1,1), B%m, B%n, 'dphi/dx' )
         call prinrl1 ( B%phider(1,1,2), B%m, B%n, 'dphi/dy' )
         call prinrl1 ( B%phider(1,1,3), B%m, B%n, 'dphi/dz' )
         call prinrl ( B%w, B%m, 'w')
         write(irefwr,*) 'End sepelp21'

      end if  ! ( debug )
      call erclos ( 'sepelp21' )

      end subroutine sepelp21
