      subroutine sepelpupw11
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute modified basis functions psi in case of upwind
!     Three dimensional elements (tetrahedrons)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
!     tetrahedron
!     linear
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, methodupw, m, n
      double precision :: xi, beta, cgnorm, unorm(B%m), eps(B%m), h(B%m), &
                          tau(B%m), tau1(B%m)
      double precision, target :: uloc(B%m,3)
      double precision, pointer :: uvel(:), vvel(:), wvel(:)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)
      double precision, pointer :: dcdx(:,:), psi(:,:), phi(:,:)
      double precision, pointer :: u1(:), u2(:), u3(:)

!     beta           Parameter beta in method
!     cgnorm         Norm of dphidx in integration point for specific value of i
!     dcdx           Gradient of old solution
!     eps            Contains the diffusion
!     h              Distances in direction of velocity in integration points
!     i              Counting variable
!     k              Counting variable
!     m              Number of quadrature points
!     methodupw      defines type of upwind
!     n              Number of basis functions
!     phi            Contains values of the basis functions in the
!                    integration points
!     phix           Contains x-derivatives of basis functions in quadrature
!                    points
!     phiy           Contains y-derivatives of basis functions in quadrature
!                    points
!     phiz           Contains z-derivatives of basis functions in quadrature
!                    points
!     psi            Contains values of the modified basis functions in the
!                    integration points
!     tau            Contains parameter tau in integration points
!     tau1           Contains parameter tau parallel in integration points
!     u1             Parallel velocity first component
!     u2             Parallel velocity second component
!     u3             Parallel velocity third component
!     uloc           Parallel velocity
!     unorm          length of velocity vector in integration points
!     uvel           u-component velocity
!     vvel           v-component velocity
!     wvel           w-component velocity
!     xi             Factor xi, see method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELPUPWDIST     Compute distance h for modified basis functions for upwind
!                    elements. Three dimensional elements
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELPUPW2     Fill array eps with values of the tensor alpha
!                    in the direction of the velocity in the vertices (3D)
!     SEPELPUPWDIST  Compute distance h for modified basis functions for upwind
!                    elements
!                    Three dimensional element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!       p_i = tau u . grad phi_i
!
!                   h xi
!       with tau = ------ beta = ||u|| h / 2 eps
!                   2||u||
!                      -
!       xi depends on metupw according to:
!       metupw = 1:  Classical upwind scheme  xi = 1
!                2:  Il'in    xi = coth(beta) - 1/beta
!                                             beta/3     -3<beta<3
!                3:  Doubly assymptotic: xi =
!                                             sign(beta)  |beta|>3
!                                         -1-1/beta     beta<1
!                4:  critical approx: xi = 0            -1<beta<1
!                                          1-1/beta     beta>1
!       eps is either alpha (if a11 = a22) or alpha in the direction of
!       the velocity
!              T
!       eps = u  alpha  u        (tensor alpha)
!             -         -
!       h = max distance in element in the direction of u
!                                                       -
!       For a tetrahedron:  h  = ||u || / max (|u . grad lambda|) in the vert.
!                            i      i      j     i             j
!       First tau and h are computed in the vertices and afterwards tau is
!       interpolated to the integration points
!    In case of the discontinuity capturing methods we define u   as
!                                                              ||
!                                            2
!    u   = u . grad(c) grad(c) / || grad(c)||
!     ||
!    In that case the upwind basis functions become
!       p  = ( tau u  + tau   u  ) . grad phi
!        i        1-       2  -||            i
!    with tau  the tau as described above and tau  is tau   (DC1) or
!            1                                   2       ||
!    max(0,tau  -tau) (DC2)      tau   is the tau based upon u
!             ||                    ||                        ||
!    In this case we limit ourselves to the doubly assymptotic approximation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpupw11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw11'
         write(irefwr,1) 'jdiag, m, n, irule', B%jdiag, B%m, B%n, B%rule
         write(irefwr,1) 'methodupw', E%methodupw
         call prinrl1 ( B%phi, B%m, B%n, 'phi' )
         call prinrl1 ( B%phider(1,1,1), B%m, B%n, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), B%m, B%n, 'dphidy' )
         call prinrl1 ( B%phider(1,1,3), B%m, B%n, 'dphidz' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      uvel => E%veloc(:,1)
      vvel => E%veloc(:,2)
      wvel => E%veloc(:,3)
      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)
      psi => B%psi
      phi => B%phi
      if ( E%filloldderiv ) dcdx => gradoldvec(1:B%m,1:E%ndim,1)

      u1 => uloc(:,1)
      u2 => uloc(:,2)
      u3 => uloc(:,3)

      m = B%m
      n = B%n

      if ( debug ) then
         call prinrl(uvel,B%m,'u')
         call prinrl(vvel,B%m,'v')
         call prinrl(wvel,B%m,'w')
      end if  ! ( debug )

      methodupw = E%methodupw

      if ( debug ) write(irefwr,1) 'methodupw', methodupw

!     --- Store component of alpha tensor in u-direction into eps

      call sepelpupw2 ( eps, unorm, uvel, vvel, wvel )

!     --- Compute the distance h in the vertices

      call sepelpupwdist ( h, unorm )

      if ( debug ) then
         call prinrl ( h, m, 'h' )
         call prinrl ( eps, m, 'eps' )
      end if  ! ( debug )

!     --- Compute tau in the vertices

      do i = 1, m
         if ( eps(i)>1d-10 .and. methodupw<=5 ) then

!        --- eps(i)>1d-10,  hence second order term plays a role

            beta = unorm(i)*h(i)/(2d0*eps(i))
            if ( beta<1d-10 ) then
               xi = 0d0
            else if ( methodupw==1 ) then

!           --- methodupw = 1   classical upwind scheme

               xi = 1d0
            else if ( methodupw==2 ) then

!           --- methodupw = 2   il'in scheme

               xi = 1d0/tanh(beta)-1d0/beta
            else if ( methodupw==3 .or. methodupw==7 .or. methodupw==8 ) then

!           --- methodupw = 3, 7 or 8    doubly assymptotic approximation

               if ( abs(beta)>3d0 ) then
                  xi=sign(1d0,beta)
               else
                  xi = beta / 3d0
               end if
            else if ( methodupw==4 ) then

!           --- methodupw = 4    critical approximation

               if ( beta<-1d0 ) then
                  xi = -1d0 - 1d0 / beta
               else if ( beta>1d0 ) then
                  xi = 1d0 - 1d0 / beta
               else
                  xi = 0d0
               end if
            else if ( methodupw==5 ) then

!           --- methodupw = 5

               if ( unorm(i)<=1d-5*eps(i) ) then
                  xi = 0d0
               else
                  beta = beta*beta
                  xi = sqrt(beta/(9d0+beta))
               end if
            end if

         else if ( methodupw==6 ) then

!        --- methodupw = 6

            if ( h(i)>epsmac ) then
               beta = (2d0*unorm(i)/h(i))**2 + (4d0*eps(i)/h(i)**2)**2
            else
               beta = 0d0
            end if
            if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
            tau(i) = 1d0/sqrt(beta)

         else
            xi = 1d0

         end if

         if ( methodupw<=8 .and. methodupw/=6 ) then
            if ( unorm(i)>epsmac ) then
               tau(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau(i) = 0d0
            end if
         end if

      end do

      if ( debug ) then
         call prinrl ( tau, m, 'tau' )
         write(irefwr,*) 'jdercn', B%derivconst
      end if  ! ( debug )

!     --- compute new basis functions, it has been assumed that the old
!         basis functions are already stored in array phi

      if ( B%derivconst ) then

!     --- jdercn = 0,  constant derivatives

         do k = 1, m
            do i = 1, n
               psi(i,k) = tau(k) * ( uvel(k)*phix(i,1)+ &
                          vvel(k)*phiy(i,1)+wvel(k)*phiz(i,1)) + phi(i,k)
            end do
         end do

      else

!     --- jdercn = 1,  variable derivatives

         do k = 1,m
            do i = 1, n
               psi(i,k) = tau(k) * ( uvel(k)*phix(i,k)+ &
                          vvel(k)*phiy(i,k)+wvel(k)*phiz(i,k) ) + phi(i,k)
            end do
         end do
      end if

      do k = 1, m
         if ( abs(eps(k))*1d2>unorm(k) ) B%secondupw = .true.
      end do

      if ( methodupw==7 .or. methodupw==8 ) then

!     --- Discontinuity capturing
!         First compute u
!                        ||

         do k = 1, m
            cgnorm = dcdx(k,1)**2 + dcdx(k,2)**2 + dcdx(k,3)**2
            if ( cgnorm<epsmac ) then
               u1(k) = 0d0
               u2(k) = 0d0
               u3(k) = 0d0
            else
               xi = ( uvel(k)*dcdx(k,1) + vvel(k)*dcdx(k,2) + &
                      wvel(k)*dcdx(k,3) )/cgnorm
               u1(k) = xi*dcdx(k,1)
               u2(k) = xi*dcdx(k,2)
               u3(k) = xi*dcdx(k,3)
            end if
            unorm(k) = sqrt ( u1(k)**2 + u2(k)**2 + u3(k)**2 )

         end do

!        --- Store component of alpha tensor in u1-direction into eps

         call sepelpupw2 ( eps, unorm, u1, u2, u3 )

!        --- Compute the distance h in the vertices


         call elpupwdist ( uloc, B%phider, h, unorm, B%type_basisfunc, &
                           B%xyz, B%nvert, B%lamder )

!        --- Compute tau in the vertices

         do i = 1, m
            if ( eps(i)>1d-10 .and. methodupw<=5 ) then

!           --- eps(i)>1d-10,  hence second order term plays a role

               beta = unorm(i)*h(i)/(2d0*eps(i))
               if ( beta<1d-10 ) then
                  xi = 0d0
               else

!              --- methodupw = 3, 7 or 8    doubly assymptotic approximation

                  if ( abs(beta)>3d0 ) then
                     xi=sign(1d0,beta)
                  else
                     xi = beta / 3d0
                  end if
               end if
            else
               xi = 1d0
            end if
            if ( unorm(i)>epsmac ) then
               tau1(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau1(i) = 0d0
            end if
         end do
         if ( methodupw==8 ) then
            do k = 1, m
               tau1(k) = max(0d0,tau1(k)-tau(k))
            end do
         end if
         do i = 1, n
            do k = 1, m
               psi(i,k) = tau1(k) * ( uvel(k)*phix(i,k)+ &
                                      vvel(k)*phiy(i,k)+ &
                                      wvel(k)*phiz(i,k) ) + psi(i,k)
            end do
         end do
      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( psi, m, n, 'psi' )
         write(irefwr,*) 'End sepelpupw11'

      end if  ! ( debug )
      call erclos ( 'sepelpupw11' )

      end subroutine sepelpupw11
