      subroutine sepeltr3dquad
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map coordinates from a bilinear quadrilateral surface in R^3 onto a
!     two-dimensional representation
!     Only meant for 4 point quads
!     The new 2d coordinates overwrite the old 3d ones
!     Old subroutine eltrans3dquad
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     2d
!     transformation
!     element
!     element
!     basis_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, j
      double precision, pointer :: x(:,:), dxdxi(:,:,:), dxdxig(:,:,:)

!                    they are not
!     dxdxi          work array of length 4x3x3 to store the values of
!                    dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                    dx/dxi(x_k) = dxdxi(k,1,1)
!                    dx/deta(x_k) = dxdxi(k,1,2)
!                    dy/dxi(x_k) = dxdxi(k,2,1)
!                    dy/deta(x_k) = dxdxi(k,2,2) etc.
!     dxdxig         See dxdxi but now for the integration points
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     x              coordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELFILLQMAT  Fill the transformation matrix qmat for the mapping of a
!                    3d surface onto a 2d surface
!     SEPELLOCCOORGN Compute local coordinates of a surface element in 3d
!                    given the transformation by base vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeltr3dquad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeltr3dquad'

      end if

      x => B%xyz
      dxdxi => B%dxdxi
      dxdxig => B%dxdxig

!     --- First fill arrays dxdxi with dx/dxi and dx/deta
!         in the vertices

      do k = 1, 3
         dxdxi(1:2,k,1) = x(2,k)-x(1,k)
         dxdxi(3:4,k,1) = x(3,k)-x(4,k)
         dxdxi(1,k,2) = x(4,k)-x(1,k)
         dxdxi(2,k,2) = x(3,k)-x(2,k)
         dxdxi(3,k,2) = x(3,k)-x(2,k)
         dxdxi(4,k,2) = dxdxi(1,k,2)
      end do  ! k = 1, 3

      B%itang = 1
      B%inorm = 2

!     --- Next compute lamina system and store in qmat

      if ( B%rule/=1 ) then

!     --- Gauss integration rule,
!         compute dx/dxi and dx/deta in integration points
!         in order to store qmat

         do j = 1, 3
            do i = 1, B%m
               do k = 1, 2
                  dxdxig(i,j,k) = &
                     dot_product ( x(:,j), B%phiderc(:,i,k) )
               end do  ! k = 1, 2
            end do  ! i = 1, m
         end do  ! j = 1, 3

      end if  ! ( B%rule/=1 )

!     --- Compute lamina system and store in qmat

      call sepelfillqmat

!     --- Finally compute coordinates in local system and store in x

      call sepelloccoorgn

      call erclos ( 'sepeltr3dquad' )
      if ( debug ) write(irefwr,*) 'End sepeltr3dquad'

      end
