      subroutine sepelucderiv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the derivative of a given function in one element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k, l, nunkp, ndim, n, m, kstep
      double complex :: som
      double complex, pointer :: u(:,:), dudx(:,:,:)

!     dudx           Gradient of u
!     j              Counting variable
!     k              Counting variable
!     kstep          step in integration points loop
!     l              General loop variable
!     m              Number of quadrature points
!     n              Number of basis functions
!     ndim           dimension of space
!     nunkp          Number of unknowns per point
!     som            Help parameter to compute a sum
!     u              Contains function to be differentiated in quadrature
!                    points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPL      Print 1d complex vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelucderiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelucderiv'
         write(irefwr,*) 'B%derivconst ', B%derivconst
         write(irefwr,1) 'iseqprevsol', iseqprevsol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nunkp = oldsols(iseqprevsol)%nunkp
      ndim = B%nd
      m = B%m
      n = B%n
      kstep = B%kstep
      ucprev => oldsols(iseqprevsol)%loccvecsnode(1:B%n,1:nunkp)
      dudx => gradoldcvec
      u => ucprev

      if ( debug ) then
         write(irefwr,1) 'n, m, ndim, kstep, nunkp', &
                          n, m, ndim, kstep, nunkp
         call princompl ( u, n, 'u' )
      end if  ! ( debug )

      if ( B%derivconst ) then

!     --- Derivatives of basis functions are constant, hence dudx is constant
!                   n
!         du/dx =  som  u   d phi  / dx
!              k   i=1   i       i     k

         do j = 1, nunkp
            do k = 1, ndim
               som = sum ( u(1:n,j)*B%phider(1:n,1,k) )
               do l = 1, m, kstep
                  dudx(l,k,j) = som
               end do
            end do
         end do

      else

!     --- dudx is variable

         do j = 1, nunkp
            do k = 1, ndim
               do l = kstep, m, kstep
                  dudx(l,k,j) = &
                     sum ( u(1:n,j)*B%phider(1:n,l,k) )
               end do
            end do
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         call princompl ( dudx(1,1,1), m, 'dudx' )
         if ( ndim>1 ) call princompl ( dudx(1,2,1), m, 'dudy' )
         write(irefwr,*) 'End sepelucderiv'

      end if  ! ( debug )
      call erclos ( 'sepelucderiv' )

      end subroutine sepelucderiv
