      subroutine sepelvec302
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for type 302
!
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     elasticity_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: dphidx(:), w(:), u(:)
      double precision, pointer :: eta(:), k0(:), p0(:), elemvc(:)
      double precision :: c

!     c              variable to store a constant temporarily
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!     elemvc         Element vector
!     eta            viscosity in quadrature points
!     k0             Constant k in the equations
!     p0             reference pressure in quadrature points
!     u              velocity in the quadrature points
!     w              quadrature weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvec302' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvec302'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider(:,1,1)
      w => B%w
      eta => E%coeffs(:,2)
      k0 => E%coeffs(:,3)
      p0 => E%coeffs(:,4)
      u => E%coeffs(:,5)

!     --- Picard part
!               F_w (i) = -eta_i U_i /h_i   w_i

      if ( .not. E%newton .and. E%tangstress==0 ) &
         E%vec(2:5:3) = -eta*u/E%hn*E%waxi

!        --- 0
!            F (i) = sum[w /2 U  h  dphi i/dz ]  + kappa p0  w
!            p       k   k    k  k                     i  i  i

      elemvc => E%vec(3:6:3)
      c=0.5d0*(w(1)*u(1)*E%h0(1)+w(2)*u(2)*E%h0(2))
      elemvc=c*dphidx
      if ( E%ind(3)/=0 .and. E%ind(4)/=0 ) &
         elemvc=elemvc+k0*p0*w
      if ( E%newton ) then

!     --- E%newton = true    add terms for Newton linearisation
!         e            n     0       n     n              n 2  0
!        F (i) = w [ dp /dz h /2 - dp /dz h /2 + eta U /(h )  h
!         w       i          i             i        i i   i    i
!                             n
!                -2 eta  U / h ]
!                      i  i   i

         if ( E%tangstress==0 ) then
            elemvc => E%vec(2:5:3)
            elemvc=E%waxi*(E%h0*(E%dpndz*.5d0+ (eta*u)/E%hn**2) &
                      -E%hn*.5d0*E%dpndz-2d0*u* eta/E%hn)
            elemvc => E%vec(3:6:3)
         end if

!        --- e                          n      n 2        n  0
!            --- F (i) = sum [ w dphi i/dz dp /dz (h ) /4eta (h -h ))
!            p       k     k                   k       k  k  k

         c=E%dpndz*0.25d0*(w(1)*E%hn(1)**2/eta(1)* &
                        (E%hn(1)-E%h0(1))+ &
                        w(2)*E%hn(2)**2/eta(2)* &
                        (E%hn(2)-E%h0(2)))
         elemvc=elemvc+c*dphidx

      end if  ! ( E%newton )
      if(E%timedep) then

!     --- E%timedep = true    add terms for Time dependent part
!          e       n
!         F (i) = v  w / theta delta t
!          p       i  i

         elemvc=elemvc+w*E%ugauss(:,1)/(tstep*theta)

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelvec302'

      end if  ! ( debug )
      call erclos ( 'sepelvec302' )

      end subroutine sepelvec302
