      subroutine sepfillbmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill B-matrix for linear elasticity problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: igprob, i
      double precision, pointer :: dphidx(:,:,:), bmat(:,:,:,:)

!     bmat           B matrix
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     i              Counting variable
!     igprob         E%stress_strain+1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillbmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillbmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      igprob = E%stress_strain+1

      dphidx => B%phider
      bmat => E%Bmat

      if ( igprob==3 .or. igprob.eq.6 .or. igprob.eq.8 ) then

!     --- Axi-symmetric stress analysis

         if ( B%derivconst ) then

!        --- Linear triangle   derivatives are constant

            do i = 1, 3
               bmat(1,1,i,:) = dphidx(i,1,1)
               bmat(2,2,i,:) = dphidx(i,1,2)
               bmat(3,1,i,:) = B%phi(i,:)/B%xyzg(:,1)
               bmat(4,1,i,:) = dphidx(i,1,2)
               bmat(4,2,i,:) = dphidx(i,1,1)
            end do

         else

!        --- General element

            do i = 1, n
               bmat(1,1,i,:) = dphidx(i,:,1)
               bmat(2,2,i,:) = dphidx(i,:,2)
               bmat(4,1,i,:) = dphidx(i,:,2)
               bmat(4,2,i,:) = dphidx(i,:,1)
            end do

            if ( B%jdiag/=1 ) then

!           --- jdiag = 2, Gauss integration rule

               do i = 1, n
                  bmat(3,1,i,:) = B%phi(i,:)/B%xyzg(:,1)
               end do

            else

!           --- jdiag = 1, Newton Cotes integration rule

               do i = 1, n
                  bmat(3,1,i,i) = 1d0/B%xyzg(i,1)
               end do

            end if

         end if

      else

!     --- Plane stress or plane strain

         if ( B%derivconst ) then

!        --- Linear triangle, derivatives are constant
!            Only k = 1 is filled

            do i = 1, 3
               bmat(1,1,i,1) = dphidx(i,1,1)
               bmat(2,2,i,1) = dphidx(i,1,2)
               bmat(3,1,i,1) = dphidx(i,1,2)
               bmat(3,2,i,1) = dphidx(i,1,1)
            end do

         else

!        --- General element

            do i = 1, n
               bmat(1,1,i,:) = dphidx(i,:,1)
               bmat(2,2,i,:) = dphidx(i,:,2)
               bmat(3,1,i,:) = dphidx(i,:,2)
               bmat(3,2,i,:) = dphidx(i,:,1)
            end do

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillbmat'

      end if  ! ( debug )
      call erclos ( 'sepfillbmat' )

      end subroutine sepfillbmat
