      subroutine el0200 ( iuser, user, jelp, imatlaw, iusrvc, &
                          iusrflg, nd, isub, isup, array, npres, &
                          ndisp, isequ, iseqp, ncoef, ifsi, &
                          nsig, maxusr, ichfib )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 07-07-2007 Extra parameters
!        version  3.0  date 13-04-2005 Add integer nsig for axi-symmetry (RvL)
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM200
!     or ELD200 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value.
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     initialization
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulevecs
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, imatlaw, iusrvc, iusrflg, &
              isub, isup, maxusr, ichfib
      integer isequ(inpelm*ndim), iseqp(4), nd, npres, ndisp, ncoef, &
              ifsi, nsig

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!     ichfib         o    Indicates which fiber subroutine has to be used
!                         0: fnfib...
!                         1: fnfib...1
!     ifsi           o    Sequence number of old solution vector for
!                         fully coupled fluid-structure interaction
!     imatlaw        o    Type of material constitutive equation to be used
!     iseqp          o    Sequence for pressure dof's
!     isequ          o    Sequence for displacement dof's
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     iusrflg        i    User flag
!     iusrvc         i    if>0, a user defined vector is used
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM100. Possible values:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic brick)
!                        16:  Subroutine ELP637  (Extended Quadr. tetrah.)
!     maxusr         i    Maximum number of degrees of freedom in usrvec
!     ncoef          i    Number of coefficients in the equation
!     nd             o    Dimension of tangent stiffness matrix
!     ndisp          o    Number of displacement dof's
!     npres          o    Number of pressure dof's
!     nsig           i    ?
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, isubcf, kchois, mrule, mrule1, mrule2, ishift
      logical funchk, allowed

!     allowed        If true it concerns an allowed possibility
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     j              Counting variable
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM200
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD200 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELI200 calls this subroutine, only
!                        integrals must be computed
!     mrule          Type of integration rule as given by the user
!     mrule1         mrule for call by BUILD
!     mrule2         mrule for call by DERIV
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0201         Fills sequence of displacement and pressure dof's
!     ELCHKINTCOEF   Check if integer coefficient is between two limits
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELERRINTCOEF   Give error message if integer coefficient out of range
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!
!    +-------+-------------------------------------------------------------+
!    | coef  | description                                                 |
!    +-------+-------------------------------------------------------------+
!    |   1   | ifsi_solution (integer refers to sequence number of solution|
!    |       |                vector for fully coupled fluid-solid inter.) |
!    +-------+---------+----+----------------------------------------------+
!    |   2   | type of coordinate system                                   |
!    |       +---------+----+----------------------------------------------+
!    |                 |  0 | cartesian coordinates                        |
!    |                 |  1 | axisymmetric coordinates                     |
!    +-------+---------+----+----------------------------------------------+
!    |   3   | type of numerical integration                               |
!    |       +---------+----+----------------------------------------------+
!    |                 |  0 | default value, defined by element            |
!    |                 |  1 | Newton-Cotes                                 |
!    |                 |  3 | Gauss                                        |
!    +-------+---------+----+----------------------------------------------+
!    |   4   | constitutive law  coef = imatlaw + 1000 * inf_coef5         |
!    |       +---------+----+----------------------------------------------+
!    |         imatlaw |  1 | compressible elastic solid                   |
!    |                 |  2 | incompressible elastic solid                 |
!    |                 |  4 | incompressible Mooney-Rivlin                 |
!    |                 | 10 | composite material                           |
!    |                 | 11 | composite material                           |
!    |                 | 99 | user-defined material                        |
!    |        inf_coef5|    | defines how coefficient 5 is defined         |
!    |                 |    | It also defines which fnfib.. routs are used |
!    +-------+---------+----+----------------------------------------------+
!    |   5   | user flags    The definition depends on inf_coef5           |
!    |       | if inf_coef5 = 0 then coef = iusrvc + 100*iusrflg           |
!    |       |                1 then coef = iusrvc + 1000*iusrflg          |
!    |       +---------+----+----------------------------------------------+
!    |         iusrvc  |  0 | user vector is not filled                    |
!    |                 |  * | user is vector is filled and vector has      |
!    |                 |    | iusrvec degrees of freedom                   |
!    |         iusrflg |  * | user defined flags                           |
!    +-------+---------+----+----------------------------------------------+
!    |   6   | density                                                     |
!    +-------+-------------------------------------------------------------+
!    |   7   | ilinJ   |  0 | volumeratio is not linearised for            |
!    |                 |    | incompressible materials (default)           |
!    |                 |  1 | volumeratio is linearised for incompressible |
!    |                 |    | materials (non-symmetric matrix)             |
!    +-------+-------------------------------------------------------------+
!    |   8   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!    |   9   | not used                                                    |
!    +-------+-------------------------------------------------------------+
!
!    +---------------------------------------------------------------------+
!    |    VECTORS NEEDED                                                   |
!    +---------------------------------------------------------------------+
!    | The following vectors are needed (in this order)                    |
!    +----+-----------+----------------------------------------------------+
!    |  1 |  u        |  incremental solution                              |
!    |  2 |  un       |  old solution at increment n                       |
!    |  3 |  usrvec   |  user defined vector  (if iusrvec>=1)            |
!    +----+-----------+----------------------------------------------------+
!
!
!    +---------------------------------------------------------------------+
!    |    CONSTITUTIVE EQUATION PARAMETERS                                 |
!    +---------------------------------------------------------------------+
!
!    +---------------------------------------------------------------------+
!    |    compressible Neo-Hookean (COEF4 = 1)                             |
!    +-------+-------------------------------------------------------------+
!    |  10   | shear modulus G                                             |
!    |  11   | bulk modulus K                                              |
!    +-------+-------------------------------------------------------------+
!
!
!    +---------------------------------------------------------------------+
!    |    incompressible Neo-Hookean (COEF4 = 2)                           |
!    +-------+-------------------------------------------------------------+
!    |  10   | shear modulus G                                             |
!    +-------+-------+-----------------------------------------------------+
!    |    incompressible Mooney-Rivlin (COEF4 = 4)                         |
!    +-------+-------+-----------------------------------------------------+
!            |  10   | material parameter c0                               |
!            |  11   | material parameter c1                               |
!    +-------+-------+-----------------------------------------------------+
!    |    composite (COEF4 = 10, 11)                                       |
!    +-------+-------+-----------------------------------------------------+
!            | 10-29 | see above for matrix material                       |
!            +-------+-----------------------------------------------------+
!            | 30-34 | composite and fiber type                            |
!            |       +---+----+--------------------------------------------+
!            |           | 30 | type of constitutive equation for matrix   |
!            |           | 31 | not used                                   |
!            |           | 32 | number of layers                           |
!            |           | 33 | not used                                   |
!            |           | 34 | theta (only for COEF4 = 11)                |
!            +-------+---+----+--------------------------------------------+
!            | 35-45 | fiber material parameters                           |
!    +-------+-------+-----------------------------------------------------+
!    |    user defined material (COEF4 = 99)                               |
!    +-------+-------+-----------------------------------------------------+
!            | 10-29 | user material parameters                            |
!            +-------+-----------------------------------------------------+
!
!
!    +---------------------------------------------------------------------+
!    |    2-PHASE MATERIAL MODEL                                           |
!    +---------------------------------------------------------------------+
!
!    +-------+-------------------------------------------------------------+
!    | 10-19 | see above for solid phase                                   |
!    +-------+-------------------------------------------------------------+
!    | 20-25 | permeability                                                |
!    |       +---+----+----------------------------------------------------+
!    |           | 20 | permeability coefficient K11                       |
!    |           | 21 | permeability coefficient K22                       |
!    |           | 22 | permeability coefficient K33                       |
!    |           | 23 | permeability coefficient K12                       |
!    |           | 24 | permeability coefficient K23                       |
!    |           | 25 | permeability coefficient K31                       |
!    +-------+---+----+----------------------------------------------------+
!    | 26-27 | time integration                                            |
!    |       +---+----+----------------------------------------------------+
!    |           | 26 | delta t                                            |
!    |           | 27 | theta (0=explicit, 1=implicit)                     |
!    +-------+---+----+----------------------------------------------------+
!    | 28    | misc                                                        |
!    |       +---+----+----------------------------------------------------+
!    |           | 28 | fluid volume fraction                              |
!    +-----------+----+----------------------------------------------------+
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     262   ix has wrong value
!     264   jdegfd has wrong value
!     406   Value of icheld not allowed
!    1014   Sequence number of mesh velocity out of range
!    1016   Structure of mesh velocity vector and velocity vector different
!    1636   NUMOLD<2, whereas at least two old solutions are required
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ('el0200')
      call elcomm
      ncomp  = ndim
      jcart = 1
      icoor = 2
      ishift = 0
      kchois = 0
      ncoef = 0

!     --- Compute kchois

      if (icheld==0) then

!     --- Call by elm200, all coefficients required

         ncoef = 35

      else if (icheld>0) then

!     --- Call by eld200, all coefficients required

         ncoef = 35
         allowed = (icheld<=30)

         if (.not. allowed) then

!        --- icheld out of range

            call errint (icheld, 1)
            call errint (itype, 2)
            call errint (ielgrp, 3)
            call errsub (406, 3, 0, 0)
         end if

         if (icheld==1 .or. icheld==21) then
            if (ix<=0 .or. ix>ndim) then
               call errint (ix, 1)
               call errint (ndim, 2)
               call errint (itype, 3)
               call errint (ielgrp, 4)
               call errsub (262, 4, 0, 0)
            end if
            if (jdegfd<=0 .or. jdegfd>ndim) then
               call errint (jdegfd, 1)
               call errint (ndim, 2)
               call errsub (264, 2, 0, 0)
            end if
         end if
         ncomp  = ndim
         jderiv = 1

      else if (icheld<0) then

!     --- Call by eli200, one coefficient required

         kchois = 2
         ncoef  = 1
         ncomp = icount/inpelm
         if (jdegfd<1 .or. jdegfd>ncomp) then
            call errint (jdegfd, 1)
            call errint (ncomp, 2)
            call errsub (264, 2, 0, 0)
         end if

      end if

!     --- Set some variables depending on the element type.
!         Number of displacement unknowns is constant. Number of
!         pressure unknowns depends on element type.

      ndisp = ndim*n
      goto (200, 201, 202, 202) itype-199

!     --- Compressible element (ITYPE=200)

 200  npres = 0
      goto 299

!     --- Incompressible element Crouzeix-Raviart (ITYPE=201)

 201  if (ishape==6.or.ishape==7) then
       npres = 3
       npsi = 3
      else if (ishape==14.or.ishape==16) then
       npres = 4
       npsi = 4
      end if
      goto 299

!     --- Taylor-Hood element (ITYPE=202)

 202  if (ishape==6) then
       npres = 4
       npsi = 4
       jpsi = 1
      else if (ishape==4.or.ishape==7) then
       npres = 3
       npsi = 3
       jpsi = 1
      else if (ishape==14.or.ishape==16) then
       npres = 8
       npsi = 8
       jpsi = 1
      end if
      jdiag = 1

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

299   if (ncoef>0 .or. icheld>0) then

         ipos = iuser (ielgrp + 5)
         if (ipos<7) then
            call errint (ielgrp+5, 1)
            call errint (ipos, 2)
            call errint (ielem, 3)
            call errint (itype, 4)
            call errint (ielgrp, 5)
            call errsub (42, 5, 0, 0)
         end if

         nuser = 0

!        --- Fill flag ifsi

         ifsi = inirepscal(iuser(ipos))

         if (kchois==0) then

!        --- Standard situation, compute the parameter notmas

            ipos = ipos + 1
            ishift = ishift+1
            if (imas==0) notmas = 1
         else if (kchois==1) then
            ipos = ipos + 1
            ishift = ishift+1
         end if

         if (ifsi<0 .or. ifsi>numold) then

!        --- ifsi out of range

            call errint ( ifsi, 1 )
            call errint ( numold, 2 )
            call errsub ( 1014, 2, 0, 0 )

         end if
         if ( ifsi>0 ) then

!        --- Check if ifsi and iseqin have the same number of unknowns

            iseqtype = oldsols(ifsi)%iseqtype
            index4n => indoldsols(iseqtype)%index4

            do j = 1,2
               iseqtype = oldsols(iseqin+j-1)%iseqtype
               do i = 1, inpelm
                  if ( indoldsols(iseqtype)%index4(i)/=index4n(i) ) then
                     call errint ( ifsi, 1 )
                     call errint ( index4n(i), 2 )
                     call errint ( i, 3 )
                     call errint ( ndim, 4 )
                     call errchr ( namesol(ifsi), 1 )
                     call errsub ( 1016, 4, 0, 1 )
                  end if
                end do
            end do

         end if

         jcart = inirepscal ( iuser(ipos) )
         ipos = ipos + 1
         if ( jcart<0 .or. jcart>1 ) then
            call errint ( jcart, 1 )
            call errint ( 2, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1767, 5, 0, 0 )
         end if
         jcart = jcart + 1

         if (kchois>=0 .and. kchois<=2) then

!        --- Call by BUILD or INTEGR, compute the parameters irule and
!            material behaviour

            mrule = inirepscal(iuser(ipos))
            mrule1 = mod(mrule,10)
            mrule2 = int(mrule/10)
            ipos = ipos + 1
            ishift = ishift+1
            if (mrule1<0 .or. mrule1>4) then
               call errint (mrule1, 1)
               call errint (3, 2)
               call errint (ielem, 3)
               call errint (itype, 4)
               call errint (ielgrp, 5)
               call errsub (1766, 5, 0, 0)
            end if
            if (mrule2<0 .or. mrule2>4) then
               call errint (mrule2, 1)
               call errint (3, 2)
               call errint (ielem, 3)
               call errint (itype, 4)
               call errint (ielgrp, 5)
               call errsub (1766, 5, 0, 0)
            end if

!           --- Material behaviour (passed to el8300)

            imatlaw = inirepscal(iuser(ipos)) ! complete definition
            ichfib = imatlaw/1000             ! information coef 5
            imatlaw = mod(imatlaw,1000)       ! actual material law
            ipos = ipos + 1
            ishift = ishift+1
            if ( imatlaw<1 .or. imatlaw>4 .and. imatlaw/=10 .and. &
                 imatlaw/=11 .and. imatlaw/=99 ) then

!           --- error: imatlaw has wrong value

               call elerrintcoef ( 4, itype, ielgrp, ielem, imatlaw, &
                                   'imatlaw' )

            end if  ! ( imatlaw<1 .or. imatlaw>4 .and. imatlaw/=10 ... )

!           --- Check ichfib ( between 0 and 1 )

            if ( icheld>=0 ) then

!           --- icheld>=0, hence call by elmxxx or eldxxx

               call elchkintcoef ( 4, itype, ielgrp, ielem, ichfib, &
                                   'inf_coef5', 0, 1 )

            else

!           --- ichfib = 1 is only implemented for elm200

               call elchkintcoef ( 4, itype, ielgrp, ielem, ichfib, &
                                   'inf_coef5', 0, 0 )

            end if  ! ( icheld==0 )

!           --- User vector and user flags

            iusrvc = inirepscal(iuser(ipos))
            if ( ichfib==0 ) then
               iusrflg = iusrvc/100
               iusrvc = mod(iusrvc,100)
            else
               iusrflg = iusrvc/1000
               iusrvc = mod(iusrvc,1000)
            end if  ! ( ichfib==0 )
            ipos = ipos + 1
            ishift = ishift+1

!           --- Check iusrvc ( between 0 and maxusr )

            call elchkintcoef ( 5, itype, ielgrp, ielem, ichfib, &
                                'iusrvc', 0, maxusr )

         end if

      end if

!     --- Set integration rule

      if (icheld==0) then

!     --- Called by BUILD

         if (mrule1>0) then

!        --- integration rule was given by the user

            irule = mrule1

         else

!        --- default is Gauss integration

            irule = 3
         end if

      else if (icheld>0) then

!     --- Called by DERIV

         if (mrule2>0) then

!        --- integration rule was given by the user

            irule = mrule2

         else

!        --- default is Newton Cotes

               irule = 1

         end if

      else if (icheld<0) then

!     --- Called by INTEGR

      end if

!     --- Set jdiag = 2 when Gauss integration

      if (irule/=1) jdiag = 2

      if (ndim==2) nsig=3
      if (ndim==3) nsig=6
      if (jcart==2) nsig=4

!     --- Set parameters depending on the shape numbers

      goto (198, 198, 198, 104, 198, 106, 107, 198, 198, 198, &
            198, 198, 198, 114, 198, 116, 198, 198), ishape

!     --- Quadratic triangle

 104     jtrans = 0
         jelp = ishape
         nd = 4
         n = 6
         m = n
         if (irule==2) m = 3
         call el0201 (isequ, iseqp)
         goto 199

!        --- Biquadratic quadrilateral

 106     jtrans = 0
         jelp = ishape
         if (jcart==2) then
            nd = 5
         else
            nd = 4
         end if
         n = 9
         m = n
         if (irule==2) m = 4
         if (irule==4) m = 5
         call el0201 (isequ, iseqp)
         goto 199

!        --- Extended quadratic triangle

 107     jtrans = 0
         jelp = ishape
         if (jcart==2) then
            nd = 5
         else
            nd = 4
         end if
         n = 7
         m = n
         if (irule==2) m = 3
         call el0201 (isequ, iseqp)
         goto 199

!        --- Triquadratic brick

 114     jtrans = 0
         jelp = ishape
         nd = 9
         n = 27
         m = n
         if (irule==2) m = 8
         if (irule==4) m = 14

!        --- fill arrays isequ and iseqp for ishape = 14

         call el0201 (isequ, iseqp)
         goto 199

!        --- Ext. Quadratic Tetrahedron

 116     jtrans = 0
         jelp = ishape
         nd = 9
         n = 15
         m = n
         if (irule==2) m = 11
         if (irule==3) m = 15

!        --- fill arrays isequ and iseqp for ishape = 16

         call el0201 (isequ,iseqp)
         goto 199

!        --- Shape not available

 198     call errint (ishape, 1)
         call errint (ielem, 2)
         call errint (itype, 3)
         call errint (ielgrp, 4)
         call errsub (1771, 4, 0, 0)

 199  if (ierror/=0) goto 1000

!     --- Fill some element constants

      mn = m * n
      iphi = 1
      igausp = 1
      if (inpelm==27) then
         nvert  = 8
      else if (inpelm==14 .or. inpelm==15) then
         nvert  = 4
      else if (inpelm==6 .or. inpelm==7) then
         nvert = 3
      else if (ishape==6) then
         nvert = 4
      else if (ishape==7) then
         nvert = 3
      else
         nvert = n
      end if

!     --- Check for at least 2 old solutions when called by BUILD or DERIV

      if (numold<2 .and. icheld>=0) then
         call errint (ielem, 1)
         call errint (itype, 2)
         call errint (ielgrp, 3)
         call errsub (1636, 3, 0, 0)
      end if

!     --- Clear array ARRAY

      do i = 1, ncoef*m
         array(i) = 0d0
      end do

!     --- And fill ARRAY with coefficients

      do i = 1, ncoef
         call elusr3 (i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do

!     --- Test coefficients

      if (ind(1)==0) then

!     --- rho has not been given a value, set rho equal to 1

         do i = 1, m
            array(i) = 1d0
         end do
      end if

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         compute isub and isup

      isubcf = 1
      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then parameter i corresponds to a function

         if (ind(i)>0) goto 170
      end do
      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do

      funchk = .false.
      do i = isub, isup
         if (ind(i)>0 .and. ind(i)<2000) then

!        --- Only the function calls are checked

            do j = i+1,isup
               if (ind(j)==ind(i)) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
            igausp = 1
         end if
         if (ind(i)>1000 .and. ind(i)<=2000) ind(lengthmst) = 1
         if (ind(i)>10000) funchk = .true.
         if (ind(i)>1000 .and. ind(i)<=2000 .or. ind(i)>10000) &
              igausp=1
      end do

!     --- Old solutions need to be computed

      ind(lengthmst) = 3

      if (funchk) ind(lengthmst) = ind(lengthmst) + 2

 1000 call erclos ( 'el0200' )

      end
