      subroutine el1004 ( sp, div, psi, phix, phi, w, xgauss, mcont, &
                          rho, pp, kappa, rhsdiv, fdiv, mdiv, rhonodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 03-02-2010 Correction compressible div
!        version  2.3  date 21-12-2008 Remove mcont = 5
!        version  2.2  date 17-09-2008 Extension with pressure mass matrix
!        version  2.1  date 08-05-2007 Fortran 90/debug
!        version  2.0  date 08-11-2002 Extra parameters kappa ... mdiv
!
!   copyright (c) 1996-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the divergence and the pressure gradient matrix for the
!     incompressible Navier-Stokes equations in case of Taylor-Hood
!     The divergence matrix is stored in array DIV
!     The transpose of the pressure gradient matrix in SP
!     The pressure matrix is stored in array PP
!     If mcont=1, only the pressure gradient matrix is stored
!     If mcont=4, the continuity equation contains a pressure term
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mcont, mdiv
      double precision phix(n,m,ndimlc), w(m), sp(ndimlc*n,npsi), &
                       xgauss(m,*), phi(n,m), rho(m), &
                       div(ndimlc*n,npsi), psi(npsi,m), pp(npsi,npsi), &
                       kappa(m), rhsdiv(npsi), fdiv(m), rhonodes(n)

!     div            o    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     fdiv           i    Contains values of divergence right-hand side to be
!                         integrated (if mdiv=1)
!     kappa          i    Factor for pressure term
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                            Not applicable with penalty function method
!                         2: Goertler equations: No dp/dx, but complete div u
!                         4: the continuity equation contains a pressure term
!     mdiv           i    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     pp             o    Pressure matrix
!     psi            i    array of length npsi x m containing the values of the
!                         psi basis functions for the pressure
!     rho            i    density rho in integration points
!     rhonodes       i    density rho in nodal  points
!     rhsdiv         o    Divergence right-hand side (if mdiv=1)
!     sp             o    Gradient of pressure matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the
!                         velocity in the sequence of the solution vector
!     w              i    array of length m containing the weights for
!                         integration
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     sum            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLDIV      Fill div matrix, in case of slightly compressible flow
!                    with respect to Crouzeix Raviart elements
!     ELFILLPRESMMT  Fill pressure mass matrix weighted by coefficient kappa
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The gradient matrix is given by:
!     SP = |  SPu  SPv  SPw  |
!     with:
!     SPu(i,k) = / psi(i) dphidx(k) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k) dx
!                x
!     SPw(i,k) = / psi(i) dphidz(k) dx
!                x
!     The divergence matrix is the same but each term multiplied by rho
!     The matrix SP is organized in the sequence of the velocity vector
!     i.e. u1, v1, w1, u2, v2 ....
!     In case of axi-symmetric co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k) dx
!                x
!     In case of polar co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k)/x dx
!                x
!     The right-hand side is defined by:
!     rhsdiv(i) = / psi(i) f dx
!                 x
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1004' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1004'
         write(irefwr,1) 'jcart, ndimlc, n, m, npsi, mcont, mdiv', &
                          jcart, ndimlc, n, m, npsi, mcont, mdiv
         call prinrl1 ( psi, m, npsi, 'psi' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jcart==1 ) then

!     --- Cartesian co-ordinates

         do idim = 1, ndimlc
            do j = 1, n
               do i = 1, npsi
                  sum = 0d0
                  do k = 1, m
                     sum = sum + w(k)*psi(i,k)*phix(j,k,idim)
                  end do
                  sp(ndimlc*(j-1)+idim,i) = sum
               end do
            end do
         end do
         if ( mdiv==1 ) then

!        --- mdiv = 1, divergence right-hand side
!            compute  rhsdiv(i) = / psi(i) f dx
!                                 x

            do i = 1, npsi
               sum = 0.d0
               do k = 1, m
                  sum = sum + w(k)*fdiv(k)*psi(i,k)
               end do
               rhsdiv(i) = sum
            end do

         end if

      else if ( jcart==3 ) then

!     --- Polar co-ordinates

         do j = 1, n
            do i = 1, npsi
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k)*psi(i,k)* &
                            ( phix(j,k,1)+phi(j,k)/xgauss(k,1) )
               end do
               sp(ndimlc*(j-1)+1,i) = sum
            end do
         end do
         do j = 1, n
            do i = 1, npsi
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k)*psi(i,k)* &
                              phix(j,k,2)/xgauss(k,1)
               end do
               sp(ndimlc*(j-1)+2,i) = sum
            end do
         end do

         if ( mdiv==1 ) then

!        --- mdiv = 1, divergence right-hand side
!            compute  rhsdiv(i) = / psi(i) f dx
!                                 x

            do i = 1, npsi
               sum = 0.d0
               do k = 1, m
                  sum = sum + w(k)*fdiv(k)*psi(i,k)
               end do
               rhsdiv(i) = sum
            end do

         end if

      else

!     --- Axi-symmetric co-ordinates

         do j = 1, n
            do i = 1, npsi
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k)*psi(i,k)* &
                            ( phix(j,k,1)+phi(j,k)/xgauss(k,1) )
               end do
               sp(ndimlc*(j-1)+1,i) = sum
            end do
         end do
         do j = 1, n
            do i = 1, npsi
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k)*psi(i,k)*phix(j,k,2)
               end do
               sp(ndimlc*(j-1)+2,i) = sum
            end do
         end do
         if ( mdiv==1 ) then

!        --- mdiv = 1, divergence right-hand side
!            compute  rhsdiv(i) = / psi(i) f dx
!                                 x

            do i = 1, npsi
               sum = 0.d0
               do k = 1, m
                  sum = sum + w(k)*fdiv(k)*psi(i,k)
               end do
               rhsdiv(i) = sum
            end do

         end if

      end if
      if ( debug ) call prinrl1 ( sp, npsi, ndimlc*n, 'sp' )

!     --- Perform some extra actions depending on mcont

      select case ( mcont )

      case(0)

!     --- mcont = 0, divergence matrix = pressure matrix

         div = sp

      case(1)

!     --- mcont = 1, fill also the divergence matrix

         call elfilldiv ( rho, phix, psi, npsi, div, phi, w, &
                          xgauss, rhonodes )

      case(4)

!     --- mcont = 4, fill also the divergence matrix and the
!                    pressure matrix
!         Fill divergence matrix similar to above procedure
!         for MCONT = 1; do NOT multiply by density

         do j = 1, npsi
            do i = 1, ndimlc
               do k = 1, n
                  div(ndimlc*(k-1)+i,j) = sp(ndimlc*(k-1)+i,j)
               end do
            end do
         end do

!        --- Fill the pressure matrix according to
!            PP(i,j) = / kappa * psi(i) * psi(j) dx
!                      x

         call elfillpresmmt ( psi, pp, kappa, npsi, w )

      end select  ! case ( mcont )

      if ( metupw>0 ) then

!     --- Upwinding, copy gradient matrix into divergence matrix

         do j = 1, npsi
            do i = 1, ndimlc
               do k = 1, n
                  div(ndimlc*(k-1)+i,j) = sp(ndimlc*(k-1)+i,j)
               end do  ! k = 1, n
            end do  ! i = 1, ndimlc
         end do  ! 1, npsi

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el1004'

      end if  ! ( debug )
      call erclos ( 'el1004' )

      end
