      subroutine el2010sp ( elemmt, phix, phi, rho, w, mconv, u, &
                            dudx, elmtsub )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 20-10-2010 new call to el3002
!        version  2.0  date 27-02-2010 Change of parameter list
!        version  1.0  date 15-04-2001
!
!   copyright (c) 2001-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element matrix for navier-stokes
!     and store in matrix s for 2-d and 3-d elements
!     In this case the special form of the Navier-Stokes equations is used
!     based on the given coefficients for the various terms
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mconv
      double precision phix(n,m,ndimlc), rho(m,ndimlc,ndimlc), w(m), &
                       elemmt(ndimlc*n,ndimlc*n), elmtsub(n,n), &
                       u(m,*), &
                       dudx(m,ndimlc,*), phi(n,m)

!     dudx    i    Real m x ndim x ncomp array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  du / dx  (x ) = dudx ( k, j, i )
!                    i    j   k
!     elemmt  i    Element matrix
!     elmtsub      Work array in which a sub element matrix is stored
!     mconv   i    Defines the treatment of the convective terms
!                  Possible values:
!                  0: convective terms are skipped (Stokes flow)
!                  1: convective terms are linearized by Picard iteration
!                  2: convective terms are linearized by Newton iteration
!                  3: convective terms are linearized by the incorrect Picard
!                     iteration
!     phi     i    array of lenght n * m, values of shape
!                  functions in integration points
!     phix    i    Array of length n x m x 3  containing the derivatives
!                  of the basis functions in the sequence:
!                  d phi / dx (xg ) = phix (i,k,1);
!                       i        k
!                  d phi / dy (xg ) = phix (i,k,2);
!                       i        k
!                  d phi / dz (xg ) = phix (i,k,3);
!                       i        k
!                  If the element is a linear tetrahedron (n=4), then the
!                  derivatives are constant and only k = 1 is filled.
!     rho     i    Array with coefficients c_conv
!                  c_conv_1 = rho(.,1,1)
!                  c_conv_2 = rho(.,2,1)
!                  c_conv_3 = rho(.,1,2)
!                  c_conv_4 = rho(.,2,2)
!     u       i    array of length m x ncomp in which the values of
!                  u in the integration points are stored the sequence:
!                  u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                   i   j
!     w       i    array of length m containing the weights for integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rhow(m), work(m), rhou(m,ndimlc)
      integer i, j

!     i              Counting variable
!     j              Counting variable
!     rhou           Contains u rho
!     rhow           Contains rho w
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3001  Compute integrals of the shape / u(x).grad phi phi (x) dx
!                                            e -            j   i
!     EL3002  Compute integrals of the shape / beta(x) phi (x) dx
!                                            e            i
!     EL3010  Fill small matrix elemmt into larger matrix s
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices depend on the parameter mconv.
!     The non-linear term has the shape:
!            | u ux     v uy      w uz |
!            |                         |
!       rho  | u vx     v vy      w vz | = rho u . grad u
!            |                         |
!            | u wx     v wy      w wz |
!     If ( mconv=1 ) then
!      rho u . grad u (n+1)  =  rho u(n) . grad u (n+1)
!     If ( mconv=2 ) then
!      rho u . grad u (n+1)  =  rho u(n) . grad u (n+1) + &
!                               rho u(n+1) . grad u (n) - &
!                               rho u(n) . grad u (n)
!     If ( mconv=3 ) then
!      rho u . grad u (n+1)  =  rho u(n+1) . grad u (n)
!     The submatrices rho u(n) . grad u (n+1) are defined by:
!     S uu =  rho u(n) . gradphi(j) phi(i) = Svv = Sww
!     The submatrices rho u(n+1) . grad u (n) are defined by:
!     S uu =  rho ux(n) phi(j) phi(i)
!     S uv =  rho uy(n) phi(j) phi(i)
!     S uw =  rho uz(n) phi(j) phi(i)
!     S vu =  rho vx(n) phi(j) phi(i)
!     S vv =  rho vy(n) phi(j) phi(i)
!     S vw =  rho vz(n) phi(j) phi(i)
!     S wu =  rho wx(n) phi(j) phi(i)
!     S wv =  rho wy(n) phi(j) phi(i)
!     S ww =  rho wz(n) phi(j) phi(i)
!    In case of cylinder co-ordinates the convective terms become
!    ur dur/dr   + uphi/r dur/dphi   + uz dur/dz   - uphi**2/r
!    ur duphi/dr + uphi/r duphi/dphi + uz duphi/dz + ur uphi/r
!    ur duz/dr   + uphi/r duz/dphi   + uz duz/dz
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      jzero = 2
      if ( ndimlc>2 ) then
         write(irefwr,*) 'Array work too small in el2010sp'
         stop
      end if

      if ( mconv/=1 ) then

!     --- mconv = 2 or 3, successive substitution part, i.e.
!         rho u(n+1) . grad u (n)

         if ( jcart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            do i = 1, ndimlc
               do j = 1, ndimlc

!              --- multiply the weight by rho and store in rhow

                  rhow(1:m) = w(1:m)*rho(1:m,j,i)
                  jadd = 0
                  call el3002 ( dudx(1,j,i), elmtsub, rhow, phi, m, n, &
                                n )
                  jadd = 1
                  call el3010 ( elemmt, elmtsub, ndimlc*n, n, j, i, &
                                ndimlc, .false., 1d0 )
               end do
            end do

         end if

      end if

      if ( mconv<3 ) then

!     --- mconv = 1 or 2, Picard part, i.e.
!         rho u(n) . grad u (n+1)

         if ( jcart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            idim = ndimlc
            jadd = 0

            do i = 1, ndimlc

!           --- multiply u by rho and store in rhou

               do j = 1, ndimlc
                  rhou(1:m,j) = u(1:m,j)*rho(1:m,j,i)
               end do
               call el3001 ( rhou, elmtsub, w, phi, phix, idim, n, &
                             work )

               jadd = 1
               call el3010 ( elemmt, elmtsub, ndimlc*n, n, i, i, ndimlc, &
                             .false., 1d0 )

            end do

         end if

      end if

      end
