      subroutine el2011sp ( acof, elemvc, phi, rho, w, u, dudx, &
                            elvcsub )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-02-2010 Change of parameter list
!        version  1.0  date 15-04-2001
!
!   copyright (c) 2001-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element vector for navier-stokes
!     and add in consisting element vector for a 3-d element
!     In this case the special form of the Navier-Stokes equations is used
!     based on the given coefficients for the various terms
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision acof, rho(m,ndimlc,ndimlc), w(m), &
                       elemvc(ndimlc*n), elvcsub(n), u(m,*), &
                       dudx(m,ndimlc,*), phi(n,m)

!     acof    i    Multiplication factor Adams-Bashfort
!     dudx    i    Real m x ndimlc  array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  du / dx  (x ) = dudx ( k, j )
!                    j     k
!                  If jdercn = 0 in common block celint, then only
!                  dudx (1,*) is filled because dudx is then a constant
!                  per element
!                  dudx is only filled if jderiv>0
!     elemvc  i    Element vector
!     elvcsub      Work array to store a sub element vector
!     phi     i    array of lenght n * m, values of shape
!                  functions in integration points
!     rho     i    Array with coefficients c_conv
!                  c_conv_1 = rho(.,1,1)
!                  c_conv_2 = rho(.,2,1)
!                  c_conv_3 = rho(.,1,2)
!                  c_conv_4 = rho(.,2,2)
!     u       i    array of length m x ncomp in which the values of
!                  u in the integration points are stored the sequence:
!                  u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                   i   j
!     w       i    array of length m containing the weights for integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision coeff(m), work(m)
      integer i, j, k

!     coeff          Work array to store coefficient
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003  Compute integrals of the shape / f(x) phi (x) dx
!                                            e         i
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective vector results in a 3x1 block vector, each
!     consisting of nx1 vectors of the following shape:
!            |   F    |
!            |    u   |
!            |        |
!            |   F    |
!     F =    |    v   |
!            |        |
!            |   f    |
!            |    w   |
!    The sub vectors are defined by:
!    Fu(i) = rho ( u du/dx + v du/dy + w du/dz ) phi (i)
!    Fv(i) = rho ( u dv/dx + v dv/dy + w dv/dz ) phi (i)
!    Fw(i) = rho ( u dw/dx + v dw/dy + w dw/dz ) phi (i)
!    For the expression in cylinder co-ordinates see subroutine el2010
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      jconsf = 2

      jadd = 0
      if ( jcart<=2 ) then

!     --- jcart = 1, Cartesian co-ordinates
!         or jcart = 2, Axi-symmetrical co-ordinates without swirl

         do i = 1, ndimlc

!        --- Put u dui/dx + v dui/dy + w dui/dz in coeff

            do k = 1, m
               coeff(k) = rho(k,1,i) * u(k,1) * dudx(k,1,i) + &
                          rho(k,2,i) * u(k,2) * dudx(k,2,i)
            end do

            if ( ndimlc==3 ) then

!           --- Put third part in coeff

               do k = 1, m
                  coeff(k) = coeff(k) + rho(k,3,i)*u(k,3)*dudx(k,3,i)
               end do

            end if

!           --- Compute contribution to one element vector and store in elvcsub

            call el3003 ( coeff, elvcsub, w, phi, m, n )

!           --- Add contribution to element vector

            do j = 1, n
               elemvc((j-1)*ndimlc+i) = elemvc((j-1)*ndimlc+i) + &
                                        acof*elvcsub(j)
            end do

         end do

      end if

      end
