      subroutine el2614 ( Ni, Nn, ci, cn, Jd, Jn, z, work, ncoef, &
                          npres )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 19-05-2003 Complete revision (RvL)
!        version  1.0  date 08-12-2000
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Add displacement field to element coordinates in buffer
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, npres
      double precision ni(3,*), nn(3,*), ci(3,*), cn(3,*), jd(*), jn(*), &
                       z(*), work(*)

!     ci             i    Array of 3 x m long that contains the concentrations.
!                         ci(1,i) fixed charge concentration
!                         ci(2,i) cation concentration
!                         ci(3,i) anion concentration
!     cn             i    Array of 3 x m long that contains the old
!                         concentrations.
!                         ci(1,i) fixed charge concentration
!                         ci(2,i) cation concentration
!                         ci(3,i) anion concentration
!     jd             i    Determinant of deformation gradient
!     jn             i    Determinant of deformation gradient at incr. n
!     ncoef          i    Number of coefficients in the equation
!     ni             o    Volume fractions in the nodal points
!                         ni(1,i) Volume fraction of the fluid
!                         ni(2,i) Volume fraction of the cations
!                         ni(3,i) Volume fraction of the anions
!     nn             o    Old volume fractions in the nodal points
!                         nn(1,i) Volume fraction of the fluid
!                         nn(2,i) Volume fraction of the cations
!                         nn(3,i) Volume fraction of the anions
!     npres          i    Number of pressure parameters per element
!     work           i    Work array
!     z              o    Help array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcoef
      parameter (maxcoef = 50)
      integer i, k, idx(8)
      double precision coef(maxcoef), Vp, Vm, nf, V(3), val(3), F, np, &
                       nm, lambda

!     coef           Array containing the coefficients
!     f              Faraday's constant
!     i              Counting variable
!     idx            Array containing the vertex indices
!     k              Counting variable
!     lambda         Deformation at the initial state
!     maxcoef        Number of coefficients
!     nf             Initial volume fraction of the fluid
!     nm             Initial volume fraction of the anions
!     np             Initial volume fraction of the cationsd
!     v              Molar volumes
!     val            Electric charge of fluid (0), cations (1) and anions (-1)
!     vm             Molar volume of anions
!     vp             Molar volume of cations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1, 3, 7, 9, 19, 21, 25, 27 /
! ======================================================================
!
      do i = 1, m
         do k = 1, ncoef
            coef(k) = work(i+m*(k-1))
         end do
         Vp = coef(31-5)
         Vm = coef(32-5)
         nf = coef(37-5)
         np = coef(38-5)
         nm = coef(39-5)
         lambda = coef(45-5)

!        --- fluid volume fraction: Nf = J * nf

         Ni(1,i)= (((nf+np+nm-1d0)/((Jd(i)*Jn(i)/lambda)))+1d0)/ &
                  (1d0+ci(2,i)*Vp+ci(3,i)*Vm)
         Nn(1,i)= (((nf+np+nm-1d0)/(Jn(i)/lambda))+1d0)/ &
                    (1d0+cn(2,i)*Vp+cn(3,i)*Vm)

!        --- ion volume fractions

         Ni(2,i) = Ni(1,i)*ci(2,i)*Vp
         Ni(3,i) = Ni(1,i)*ci(3,i)*Vm
         Nn(2,i) = Nn(1,i)*cn(2,i)*Vp
         Nn(3,i) = Nn(1,i)*cn(3,i)*Vm
      end do

!     --- 9 noded quad

      if (ishape.eq.4.or.ishape.eq.6.or.ishape.eq.7) then
         idx(1)=1
         idx(2)=3
         idx(3)=5
         idx(4)=7
      end if

!     --- compute some constants

      do i = 1, npres
         val(1) = 0d0
         val(2) = work(idx(i)+m*(33-6))
         val(3) = work(idx(i)+m*(34-6))
         F = work(idx(i)+m*(44-6))
         V(1) = 1d0
         V(2) = work(idx(i)+m*(31-6))
         V(3) = work(idx(i)+m*(32-6))
         do k = 1, 3
            z(k) = (val(k)*F)/V(k)
         end do
      end do

!     --- write(*,*) (cn(2,i), i=1,8)
!         write(*,*) (cn(3,i), i=1,8)
!         write(*,*) (Ni(1,i), i=1,8)
!         write(*,*) (Ni(2,i), i=1,8)
!         write(*,*) (Ni(3,i), i=1,8)

      end
