      subroutine el8205 ( s, w, dpsidx, cn, work, ncoef, npres )
! ======================================================================
!
!        programmer    Raoul van Loon/Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 19-05-2003  Some adjustments
!        version  1.0  date 29-05-2000
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the part of the element matrix, that is correlated to
!     the fluid flux out and into the element and its right hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, npres
      double precision s(npres,npres,6), w(*), dpsidx(npres,m,ndim), &
                       cn(3,*), work(*)

!     cn             i    Array of 3 x m long that contains the old
!                         concentrations.
!                         ci(1,i) fixed charge concentration
!                         ci(2,i) cation concentration
!                         ci(3,i) anion concentration
!     dpsidx         i    Array of length 8 x m x 3  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                                 i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                                 i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                                 i        k
!     ncoef          i    Number of coefficients in the equation
!     npres          i    Number of pressure parameters per element
!     s              o    Element matrix to be filled
!     w              i    array of length m containing the weights for
!                         integration
!     work           i    Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, q, maxcoef
      parameter (maxcoef=50)
      double precision coef(maxcoef), K11, theta, delta_t, Dp, Dm, hind, &
                       Temp, R, nf, np, nm, K_wrk(6), K_inv(6)

!     coef           Element coefficients
!     delta_t        Timestep
!     dm             Diffusion coefficient of anions
!     dp             Diffusion coefficient of cations
!     hind           Hindrance factor
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     k11            Permeability
!     k_inv          Inverse of friction matrix
!     k_wrk          Friction matrix
!     l              General loop variable
!     maxcoef        Number of coefficients
!     nf             Initial volume fraction of the fluid
!     nm             Initial volume fraction of the anions
!     np             Initial volume fraction of the cations
!     q              General loop variable
!     r              Gas constant
!     temp           Temperature
!     theta          Constant, responsible for the extent to which
!                    the old solution is taken into account
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELSYMINV       Compute the inverse of a symmetric stored 3 x 3 matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear the matrix

      do k = 1, 6
         do i = 1, npres
            do j = 1, npres
               S(i,j,k) = 0d0
            end do
         end do
      end do

!     --- Loop over the integration points

      do k = 1, m

!     --- get coefficients

         do i = 1, ncoef
            coef(i) = work(1+m*(i-1))
         end do
         K11 = coef(20-5)
         delta_t = coef(26-5)
         theta = coef(27-5)
         Dp = coef(28-5)
         Dm = coef(29-5)
         hind = coef(30-5)
         Temp = coef(42-5)
         R = coef(43-5)
         nf= coef(37-5)
         np= coef(38-5)
         nm= coef(39-5)

!        --- compute inverse P-matrix

         K_inv(4) = -(nf*R*Temp*cn(2,k))/Dp
         K_inv(5) = 0d0
         K_inv(6) = -(nf*R*Temp*cn(3,k))/Dm
         K_inv(1) = (nf**2d0)/K11 - (1d0-hind)*K_inv(4) - &
                    (1d0-hind)*K_inv(6)
         K_inv(2) = -K_inv(4)/(1d0-hind)
         K_inv(3) = -K_inv(6)/(1d0-hind)

!        --- invert it and compute Kab

         call elsyminv(K_inv, K_wrk)
         K_wrk(1) = K_wrk(1)*nf*nf
         K_wrk(2) = K_wrk(2)*np*np
         K_wrk(3) = K_wrk(3)*nm*nm
         K_wrk(4) = K_wrk(4)*nf*np
         K_wrk(5) = K_wrk(5)*np*nm
         K_wrk(6) = K_wrk(6)*nm*nf

!        --- T
!            --- assemble Bp * K * Bp

         do q = 1, 6
            do i = 1, npres
               do j = 1, npres
                  do l = 1, ndim
                     S(i,j,q) = S(i,j,q) + theta*w(k)*dpsidx(j,k,l)* &
                                           K_wrk(q)* &
                                           dpsidx(i,k,l)*delta_t
                  end do
               end do
            end do
         end do

      end do

!     --- do k = 1, 6
!            do i =1 , npres
!               write(*,*) (s(i,j,k), j=1,npres)
!            end do
!            write(*,*)
!         end do

      end
