      subroutine el8208 (s, B, D, w, nd, ilinJ)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.2  date 29-05-2014 remove call to time subroutine
!        version  2.1  date 13-04-2005 Extension for axi-symmetry (RvL)
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element matrix for nonlinear elasticity element
!     3D case from B' * D * B
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nd, ilinJ
      double precision B(nd,ndim,n,*), D(nd,nd,*), &
                       w(*), s(ndim*n,ndim*n)

!     b              i    B matrix
!     d              i    Array containing the Elasticity matrix D
!                                         k
!                         D(i,j,k) = d  (x )
!                                     ij
!     ilinJ          i    indicate whether D_j has to be computed
!                         (for incomp. mat.), &
!                         resulting in non-symmetric element stiffness matrix
!     nd             i    ?
!     s              o    Element (stiffness) matrix
!     w              i    Weight for numerical integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l1, l2, ip, iq, idx(2,14), p, q, &
              idx1, idx2, istart
      double precision sum

!     i              Counting variable
!     idx            ?
!     idx1           Array for renumbering of the linear basisfunctions
!     idx2           ?
!     ip
!     iq             ?
!     istart         beginposition of 2D part of array idx
!     j              Counting variable
!     k              Counting variable
!     l1             ?
!     l2             ?
!     p
!     q              ?
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     METHOD
!                                               T
!     The element stiffness matrix is given by B  D  B, &
!     with B the matrix:
!     |   phi       0          0       |
!     |      x                         |
!     |                                |
!     |    0       phi         0       |
!     |               y                |
!     |                                |
!     |    0        0         phi      |
!     |                          z     |
!     |                                |
!     |   phi       0          0       |
!     |      y                         |
!     |                                |
!     |    0       phi         0       |
!     |               z                |
!     |                                |
!     |    0        0         phi      |
!     |                          x     |
!     |    0       phi         0       |
!     |               x                |
!     |                                |
!     |    0        0         phi      |
!     |                          y     |
!     |                                |
!     |   phi       0          0       |
!     |      z                         |
!     Each phi stands for a series of basis functions
!     The matrix D is a 9x9 matrix with elements d
!                                                 ij
!     Computation of the stiffness matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices are defined by:
!     S    =  S
!      xy      yx
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1,1, 2,2, 3,3, 4,1, 5,2, 6,3, 7,2, 8,3, 9,1, &
                 1,1, 2,2, 3,1, 4,2, 5,1 /
! ======================================================================
!
      call eropen ('el8208')

      if (ndim==2) istart=9
      if (ndim==3) istart=0

      do k = 1, ndim*n
         do i = 1, ndim*n
            s(i,k) = 0d0
         end do
      end do

      do p = 1, nd
         ip = idx(1,istart+p)
         l1 = idx(2,istart+p)
         do q = 1, nd
            iq = idx(1,istart+q)
            l2 = idx(2,istart+q)

            do i = 1, n
               do j = 1, n
                  sum = 0d0
                  idx1 = (i-1)*ndim + l1
                  idx2 = (j-1)*ndim + l2

                  if (ilinJ/=1) then

!                 --- use symmetry

                     if (idx2>idx1) then
                        goto 100
                     end if
                  end if

                  do k = 1, m
                     sum = sum+w(k)*b(ip,l1,i,k)*D(ip,iq,k)* &
                                    b(iq,l2,j,k)
                  end do

!                 --- add to stiffness matrix

                  s(idx1,idx2) =  s(idx1,idx2) + sum

                  if (ilinJ/=1) then

!                 --- use symmetry

                     if (idx1/=idx2) then
                        s(idx2,idx1) = s(idx2,idx1) + sum
                     end if
                  end if
 100           end do
            end do
         end do
      end do

      call erclos ('el8208')

      end
