      subroutine el8250 ( fibdir, xg, x0, Fd, Fn, imatlaw, usrvec, &
                          iusrvc, iusrflg, work, ncoef, index1 )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 07-07-2007 Fortran 90
!        version  1.1  date 01-11-2002 extra parameter for elmul03
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the fiber direction in case of composite elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imatlaw, iusrvc, iusrflg, ncoef, index1(*)
      double precision fibdir(ndim,m), &
                       xg(m,ndim), x0(ndim,m), work(*), Fd(m,ndim,ndim), &
                       Fn(m,ndim,ndim), norm, usrvec(m,*)

!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fibdir         i    ?
!     fn             i    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     imatlaw        i    Type of material constitutive equation to be used
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iusrflg        i    ?
!     iusrvc         i    if>0, a user defined vector is used
!     ncoef          i    Number of coefficients in the equation
!     usrvec         i    ?
!     work           i    Work array
!     x0             i    ?
!     xg             i    Array of length 4x2 containing the co-ordinates of the
!                         Gauss points in the reference element for a four point
!                         Gauss rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision Fd_wrk(3,3), Fn_wrk(3,3), corr(3), e0(3,3), e(3), &
                       F(3,3), corr0(3), usr_wrk(100)

!     corr           array of length ndim containing the
!                    co-ordinates of the nodal points
!     corr0          coordinates in undeformed configuration
!     e              real, coefficient of x
!     e0             ?
!     f              ?
!     fd_wrk         ?
!     fn_wrk         ?
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     norm           ?
!     usr_wrk        ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DGEMM          BLAS routine for matrix multiplication general.
!                    (NAG: f06yaf)
!     ELMUL03        ?
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FNFIBDIR       ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2244   The value of imatlaw is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- check if we have composite material

      if (imatlaw<10) then

!     --- no composite material law choosen

         call eropen ('el8250')
         call errint (imatlaw, 1)
         call errint (10, 2)
         call errint (20, 3)
         call errsub (2244, 3, 0, 0)
         call erclos ('el8250')
      end if

!     --- Loop over the integration points

      do k = 1, m

!     --- compute coordinates of current integration point

         do i = 1, ndim
            corr0(i) = x0(i,k)
            corr(i) = xg(k,i)
         end do

!        --- compute deformation gradient in current integration point

         do j = 1, ndim
            do i = 1, ndim
               Fd_wrk(i,j) = Fd(k,j,i)
               Fn_wrk(i,j) = Fn(k,j,i)
            end do
         end do

!        --- compute user vector in current integration point

         if (iusrvc>=1) then
            do i = 1, iusrvc
               usr_wrk(i) = usrvec(k,i)
            end do
         end if

!        --- compute F = Fd * Fn

         call elmul03 (Fd_wrk, Fn_wrk, F, 0, ndim)

!        --- fill in fiber direction in the undeformed configuration (e0)

         call fnfibdir ( e0, ix, ielgrp, index1(k), corr0, corr, &
                         iusrvc, usr_wrk, iusrflg, 2 )

!        --- compute current local (fiber) corrdinate system
!            lambda e = F * e0

         call dgemm ('n', 'n', 3, 1, 3, 1d0, F, 3, e0, 3, 0d0, e, 3)

         norm = sqrt(e(1)**2 + e(2)**2 + e(3)**2)
         e(1) = e(1)/norm
         e(2) = e(2)/norm
         e(3) = e(3)/norm

!        --- store fiber directions

         do i = 1, 3
            fibdir(i,k) = e(i)
         end do

      end do

      end
