      subroutine elfilldiv ( rho, gradphi, psipres, npres, div, phi, w, &
                             xgauss, rhonodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill div matrix, in case of slightly compressible flow
!     with respect to Crouzeix Raviart elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     divergence
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres
      double precision rho(m), gradphi(n,m,ndimlc), psipres(npres,m), &
                       div(ndimlc,n,npres), phi(n,m), w(m), xgauss(m,*), &
                       rhonodes(n)

!     div            i    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     gradphi        i    Contains the gradient of phi
!                         gradphi(i,j,k) refers to basis function phi_i, &
!                         in integration point j, k^th derivative
!     npres          i    Number of pressure parameters per element
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psipres        i    Contains the pressure basis functions
!                         psipres(i,k) refers to i-th basis function in
!                         integration point k
!     rho            i    density rho
!     rhonodes       i    density rho in nodal  points
!     w              i    array of length m containing the weights for
!                         integration
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, jdim
      double precision gradrho(m,ndimlc), sum

!     gradrho        contains gradient of rho
!     i              Counting variable
!     j              Counting variable
!     jdim           loop variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELGRAD         Compute the gradient of a function in an integration point
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3043   Value of jcart not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The weakly compressible continuity equation is given by
!
!     div ( rho u ) = 0
!
!     The discretization becomes
!
!     / psi_i div ( rho u ) d Omega = 0
!
!     or:
!
!     / psi_i ( rho div u + u . grad rho ) d Omega = 0
!
!     Discrtetized:
!
!     Sum_j ( u_j DijU + v_j DijV + w_j DijW )
!
!     In Cartesian coordinates:
!
!     DijU = / psi_i ( rho dphi_j/dx + phi_j d rho/dx )
!
!     Hence:
!
!     DijU = sum_k w(k) psi_i(k) ( rho(k) dphi_j(k)/dx + phi_j(k) rho/dx(k)
!
!     d rho/dx = sum_l rho(l) dphi_l/dx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfilldiv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfilldiv'
         write(irefwr,1) 'npres, ndim', npres, ndimlc
         call prinrl ( rho, m, 'rho in integration points' )
         call prinrl ( rhonodes, n, 'rho in nodes' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill grad(rho) in integration points
!         d rho/dx = sum_l rho(l) dphi_l/dx

      call elgrad ( rhonodes, gradphi, gradrho, xgauss )
      if ( debug ) call prinrl1 ( gradrho, ndimlc, m, 'gradrho' )
      if ( ierror/=0 ) go to 1000

      select case ( jcart )

      case(1)

!     --- Case 1: cartesian coordinates
!         DijU = / psi_i ( rho dphi_j/dx + phi_j d rho/dx )

         if ( debug ) then
            write(irefwr,1) 'ndimlc, m, npres', ndimlc, m, npres
            call prinrl1 ( psipres, m, npres, 'psipres' )
         end if  ! ( debug )

         do jdim = 1, ndimlc
            do i = 1, npres
               do j = 1, n
                  sum = 0d0
                  do k = 1, m
                     sum = sum + w(k) * psipres(i,k) * &
                           ( rho(k) * gradphi(j,k,jdim) + &
                             phi(j,k) * gradrho(k,jdim) )
                  end do  ! k = 1, m
                  div(jdim,j,i) = sum
               end do  ! j = 1, n
            end do  ! i = 1, npres
         end do  ! jdim = 1, ndimlc

      case(2)

!     --- Case 2: cylinder coordinates
!         Dij_r = / psi_i ( rho d r phi_j/(rdr) + phi_j d rho/dr )

         do i = 1, npres
            do j = 1, n
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k) * psipres(i,k) * &
                        ( rho(k)*(gradphi(j,k,1)+phi(j,k)/xgauss(k,1))+ &
                          phi(j,k) * gradrho(k,1) )
               end do  ! k = 1, m
               div(1,j,i) = sum
            end do  ! j = 1, n
         end do  ! i = 1, npres

!        --- Dij_z = / psi_i ( rho dphi_j/dz + phi_j d rho/dz )

         do i = 1, npres
            do j = 1, n
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k) * psipres(i,k) * &
                        ( rho(k) * gradphi(j,k,2) + &
                          phi(j,k) * gradrho(k,2) )
               end do  ! k = 1, m
               div(2,j,i) = sum
            end do  ! j = 1, n
         end do  ! i = 1, npres

      case(3)

!     --- case 3: polar coordinates
!         Dij_r = / psi_i ( rho d r phi_j/(rdr) + phi_j d rho/dr )

         do i = 1, npres
            do j = 1, n
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k) * psipres(i,k) * &
                        ( rho(k)*(gradphi(j,k,1)+phi(j,k)/xgauss(k,1))+ &
                          phi(j,k) * gradrho(k,1) )
               end do  ! k = 1, m
               div(1,j,i) = sum
            end do  ! j = 1, n
         end do  ! i = 1, npres

!        --- Dij_z = / psi_i ( rho dphi_j/( r dphi) + phi_j d rho/(r dphi) )

         do i = 1, npres
            do j = 1, n
               sum = 0d0
               do k = 1, m
                  sum = sum + w(k) * psipres(i,k) * &
                        ( rho(k) * gradphi(j,k,2)/xgauss(k,1) + &
                          phi(j,k) * gradrho(k,2) )
               end do  ! k = 1, m
               div(2,j,i) = sum
            end do  ! j = 1, n
         end do  ! i = 1, npres

      case default

!     --- not available

         call errint ( jcart, 1 )
         call errsub ( 3043, 1, 0, 0 )

      end select  ! case ( jcart )

1000  call erclos ( 'elfilldiv' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( div, npsi, ndimlc*n, 'div' )
         write(irefwr,*) 'End elfilldiv'

      end if  ! ( debug )

      end
