      function eli900 ( coor2d, iuser, user, index1, &
                        numld1, islold, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.14 date 09-03-2013 Replace elflr7 by elflr71
!        version  2.13 date 12-12-2012 Long integers
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order ellipitic equations
!     Two dimensional elements
!     ELI900 is a help subroutine for subroutine INTEGR, &
!     it is called through the intermediate subroutine eli000
!     So:
!     INTEGR
!       -  Loop over elements
!          -  ELINS2
!             - ELI900
!    The subroutine is able to handle the following element shapes and cases:
!    triquadratic brick (3D)
!    cartesian co-ordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     element_integral
!     integral
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eli900, coor2d(*), user(*), array(*)
      integer numld1, iuser(*), index1(*), &
              islold(numld1)

!     array          Real work array
!                    work consists of the following parts
!                    Starting address    length   Name:
!                    1                   9*27     ARRAY
!                    ipugs               3*27     UGAUSS
!                    ipdudx              9*27     DUDX
!                    ipdpdx              27*27*3  DPHIDX
!                    ipphix              27*27*3  PHIKSI for elp633
!                    ipwork              27*31    Work space
!                    Total length: 5778
!                    The subarrays contain the following contents:
!                    ARRAY:
!                    array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         2D                  3D
!                    1:   rho              rho
!                    2:   omega            omega
!                    3:   f1               f1
!                    4:   f2               f2
!                    5:                    f3
!                    6:   eta              eta
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!                    DPHIDX:
!                    Array of length inpelm x m x ndim containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                    d phi / dz = dphidx (i,k,3)
!                         i
!                      in node k
!                    WORK
!                    General work space
!                    1                   27*27*3  DPHIDX
!                    ipdudx              27*9     DUDX
!                    ipphix              27*27*3  PHIKSI for elp633
!                    ipwork              27*6     Work space
!                    Total length: 4779
!                    The subarrays contain the folowing contents:
!                    DPHIDX:
!                    Array of length inpelm x m x ndim containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                    d phi / dz = dphidx (i,k,3)
!                         i
!                      in node k
!                    DUDX
!                    Contains the derivatives of the solution in the integration
!                    nodes
!                    WORK
!                    General work space
!     coor2d     i     array of length ndim x npoint containing the co-ordinates
!                    of the nodal points with respect to the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i);
!                     i                 i                 i
!     eli900   o     Computed integral
!     index1   i     Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     numld1   i     Number of old vectors that are stored in VECOLD
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, jtime, ipdpdx, ipugs, ioldm, &
              ipwork, ipphix, mconv, modelv, ipdudx, mcont, ipphi, &
              nveloc, npres, imesh, mdiv, mcoefconv, modelrhsd
      double precision eps, cn, clamb, thetdt, qmat(81), x(81), &
                       w(54), xgauss(81), uold(81), dum(1)
      save isub, isup, jelp, jtime, ipdpdx, ipwork, ipphix, ipphi, &
           ipdudx, ipugs, nveloc

!     clamb          Dummy parameter
!     cn             Dummy parameter
!     dum            General double precision dummy variable
!     eps            Dummy parameter
!     i              Counting variable
!     imesh          Sequence number of mesh velocity in input vector
!                    If 0 no mesh velocity is present
!     ioldm          Dummy parameter
!     ipdpdx         Starting address in array work of dphidx
!     ipdudx         Starting address in array work of dudx
!     ipphi          Starting address of array phi
!     ipphix         Starting address in array work of phiksi
!     ipugs          Starting address in array work of ugauss
!     ipwork         Starting address in array work of work array
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Indication of the type of basis function subroutine must
!                    be called by ELM100. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     jtime          Dummy parameter
!     mcoefconv      Defines how the convetive term is treated:
!                    0: standard case
!                    1: special case: the convective term is defined as
!                          c_conv_1 u du/dx + c_conv_2 v du/dy
!                          c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mcont          Dummy parameter
!     mconv          Dummy parameter
!     mdiv           Defines if divergence right-hand side is used (1) or
!                    not (0)
!     modelrhsd      Dummy parameter
!     modelv         Dummy parameter
!     npres          Number of pressure parameters
!     nveloc         Number of velocity parameters
!     qmat           dummy
!     thetdt         Dummy parameter
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              array of length n x ndim containing the co-ordinates
!                    of the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision elic01

!     EL0900GEN      Performs the initializations for subroutine ELM900, ELI900
!                    or ELD900 in the case that IFIRST = 0
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIC01         Compute integrals of the shape / f(x) dx
!                                                   e
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the parameter icheli and
!     the arrays IUSER and USER in the usual way
!     The sequence is:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Depending on icheli the following integrals are computed
!          1:   / f(x) dx
!          2:   / f(x) u      (x) dx
!                       jdegfd
!          3-5: / f(x) du      (x)/dx  dx   i=icheld-2
!                        jdegfd      i
!          6:   / f(x) |u      (x)| dx
!                        jdegfd
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli900' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eli900'
         write(irefwr,1) 'icheli, jdegfd, iseqin', &
                          icheli, jdegfd, iseqin
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = -icheli
         call el0900gen ( iuser, user, jelp, jtime, eps, cn, &
                          clamb, mconv, mcont, modelv, isub, isup, &
                          array, thetdt, nveloc, npres, imesh, &
                          mdiv, mcoefconv, modelrhsd, ioldm, &
                          work )
         if ( jtrans==1 .and. ks(islold(1))%typevec/=110 ) jtrans = 0

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

         ipphi  = 1      + 9*m
         ipugs  = ipphi  + m*n
         ipdudx = ipphi  + n*m
         ipdpdx = ipdudx + ndim**2*m
         ipphix = ipdpdx + ndim*m*n
         ipwork = ipphix + ndim*n*m

      end if

!     --- compute basis functions and weights for numerical integration

      if ( debug ) write(irefwr,1) 'jtrans, jelp', jtrans, jelp

      call elm800basefn ( coor2d, x, w, array(ipdpdx), xgauss, &
                          array(ipphi), index1, qmat, dum, jelp, &
                          array(ipphix), dum, dum, dum )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( debug ) &
          write(irefwr,1) 'lengthmst, ind(lengthmst)', &
                           lengthmst, ind(lengthmst)

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el2005 ( uold, array(ipdudx), array(ipphi), &
                       array(ipdpdx), index1, array(ipugs), iseqin )

      if ( debug ) write(irefwr,1) 'isub, isup', isub, isup
      do 150 i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            if ( ind(i)>10000 ) then
               call errint ( i, 1 )
               call errsub ( 1781, 1, 0, 0 )
            end if
            call elflr71 ( i, user, array, x, xgauss, &
                          m, n, array(ipphi), 1, &
                          index1, iuser, array(ipugs), array(ipwork) )

         end if

150   continue

!     --- Compute integral depending on icheli

      eli900 = elic01 ( icheli, jdegfd, array, w, array(ipugs), &
                        array(ipdudx), array(ipwork) )

1000  call erclos ( 'eli900' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'eli900', eli900
         write(irefwr,*) 'End eli900'

      end if  ! ( debug )

      end function eli900
