      function eli903 ( coor2d, iuser, user, islold, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.10 date 09-03-2013 Replace elflr7 by elflr71
!        version  2.9  date 12-12-2012 Long integers
!
!   copyright (c) 1998-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute integral over elements of Taylor-Hood type
!     ELI903 is a help subroutine for subroutine INTEGR, &
!     it is called through the intermediate subroutine eli000
!     So:
!     INTEGR
!       -  Loop over elements
!          -  ELINS2
!             - ELI903
!    The subroutine is able to handle the following element shapes and cases:
!    triquadratic brick (3D)
!    cartesian co-ordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     element_integral
!     integral
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eli903, coor2d(*), user(*), array(*)
      integer iuser(*), islold(numold)

!     array          Real work array
!                    work consists of the following parts
!                    Starting address    length   Name:
!                    1                   1*27     ARRAY
!                    ipugs               3*27     UGAUSS
!                    ipdudx              9*27     DUDX
!                    ipdpdx              27*27*3  DPHIDX
!                    ipphix              27*27*3  PHIKSI for elp633
!                    ippsi               8*27     Linear basis function psi
!                    ipwork              27*31    Work space
!                    Total length: 5778
!                    The subarrays contain the following contents:
!                    ARRAY:
!                    array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         2D                  3D
!                    1:   rho              rho
!                    2:   omega            omega
!                    3:   f1               f1
!                    4:   f2               f2
!                    5:                    f3
!                    6:   eta              eta
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!                    DPHIDX:
!                    Array of length inpelm x m x ndim containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                    d phi / dz = dphidx (i,k,3)
!                         i
!                      in node k
!                    WORK
!                    General work space
!                    1                   27*27*3  DPHIDX
!                    ipdudx              27*9     DUDX
!                    ipphix              27*27*3  PHIKSI for elp633
!                    ipwork              27*6     Work space
!                    Total length: 4779
!                    The subarrays contain the folowing contents:
!                    DPHIDX:
!                    Array of length inpelm x m x ndim containing the
!                    derivatives of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                    d phi / dz = dphidx (i,k,3)
!                         i
!                      in node k
!                    DUDX
!                    Contains the derivatives of the solution in the integration
!                    nodes
!                    WORK
!                    General work space
!     coor2d     i     array of length ndim x npointcac containing the co-ordinates
!                    of the nodal points with respect to the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i);
!                     i                 i                 i
!     eli903   o     Computed integral
!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, jtime, ipdpdx, ipugs, ipphi, &
              ipwork, ipphix, mconv, modelv, ipdudx, mcont, &
              nveloc, iseqvel(81), iseqpres(8), ippsi, mdiv, modelrhsd
      double precision cn, clamb, thetdt, qmat(3,27), tang(3*3*6), &
                       dpsidx(4*9*2), x(ndim*inpelm), &
                       w(54), xgauss(81), uold(81)
      double precision, pointer :: vecoldlc(:)
      logical velocity
      save isub, isup, jelp, jtime, ipdpdx, ipwork, ipphix, ippsi, &
           ipdudx, ipugs, nveloc, iseqvel, iseqpres, ipphi

!     clamb          Dummy parameter
!     cn             Dummy parameter
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     i              Counting variable
!     ipdpdx         Starting address in array work of dphidx
!     ipdudx         Starting address in array work of dudx
!     ipphi          Starting address of array phi
!     ipphix         Starting address in array work of phiksi
!     ippsi          Starting address in array work of psi
!     ipugs          Starting address in array work of ugauss
!     ipwork         Starting address in array work of work array
!     iseqpres       Contains the sequence numbers of the pressure in the
!                    element vector
!     iseqvel        Array containing the positions of the velocities in
!                    the element vector in the sequence required by el2005
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Indication of the type of basis function subroutine must
!                    be called by ELM100. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     jtime          Dummy parameter
!     mcont          Dummy parameter
!     mconv          Dummy parameter
!     mdiv           Defines if divergence right-hand side is used (1) or
!                    not (0)
!     modelrhsd      Dummy parameter
!     modelv         Dummy parameter
!     nveloc         Number of velocity parameters
!     qmat           dummy
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     thetdt         Dummy parameter
!     uold           Array containing the previous iteration (old solution)
!     velocity       If true the vector to be integrated is of type of the
!                    velocity vector
!     w              array of length m containing the weights for
!                    integration
!     x              array of length n x ndim containing the co-ordinates
!                    of the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision elic01

!     EL0903         Initialize parameters for subroutines ELD/M/I903
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIC01         Compute integrals of the shape / f(x) dx
!                                                   e
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the parameter icheli and
!     the arrays IUSER and USER in the usual way
!     The sequence is:
!     1:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     2:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     3:  Not yet used (must be 0)
!     4:  f
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Depending on icheli the following integrals are computed
!          1:   / f(x) dx
!          2:   / f(x) u      (x) dx
!                       jdegfd
!          3-5: / f(x) du      (x)/dx  dx   i=icheld-2
!                        jdegfd      i
!          6:   / f(x) |u      (x)| dx
!                        jdegfd
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli903' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eli903'
         write(irefwr,1) 'iseqin, jdegfd', iseqin, jdegfd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = -icheli
         call el0903 ( iuser, user, jelp, jtime, cn, clamb, &
                       mconv, mcont, modelv, isub, isup, array, &
                       thetdt, nveloc, iseqvel, iseqpres, mdiv, &
                       modelrhsd, work )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

         ipphi  = 1      + m
         ipugs  = ipphi  + m*n
         ipdudx = ipugs  + ndim*m
         ipdpdx = ipdudx + ndim**2*m
         ipphix = ipdpdx + ndim*m*n
         ippsi  = ipphix + ndim*n*m
         ipwork = ippsi  + 8*m

!        --- Check if vector with sequence number iseqin is a velocity
!            vector, or a standard vector

         iseqtype = oldsols(iseqin)%iseqtype
         index4n => indoldsols(iseqtype)%index4
         velocity = index4n(1)==ndim+1  ! first component contains
                                              ! pressure
         if ( velocity ) then

!        --- Vector might be a velocity vector
!            In that case it is necessary that iseqvel(end) = index4(last)
!            or minus 1

            if ( iseqvel(inpelm*ndim)/=index4n(inpelm) .and. &
                 iseqvel(inpelm*ndim)/=index4n(inpelm)-1 ) &
               velocity = .false.

         end if  ! ( velocity )

         if ( velocity ) then

!        --- velocity vector

            jind = 0

         else

!        --- Not velocity vector

            ncomp = max(1,index4n(inpelm)/inpelm)
            jind = max(1,jdegfd)

         end if  ! ( .not. velocity )

         if ( debug ) then
            call prinin ( iseqvel, inpelm*ndim, 'iseqvel' )
            write(irefwr,*) 'velocity ', velocity
         end if  ! ( debug )

      end if

!     --- compute basis functions and weights for numerical integration

      if ( jelp<=6 .or. jelp==12 .or. jelp==14 ) jpsi = 1
      call  elm800basefn ( coor2d, x, w, array(ipdpdx), xgauss, &
                           array(ipphi), index1, qmat, tang, jelp, &
                           array(ipphix), array(ippsi), dpsidx, array )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!     --- Fill uold

         call el2005 ( uold, array(ipdudx), array(ipphi), &
                       array(ipdpdx), iseqvel, array(ipugs), iseqin )

      end if  ! ( ind(lengthmst)==1 .or. ind(lengthmst)==3 )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            if ( ind(i)>10000 ) then
               call errint ( i, 1 )
               call errsub ( 1781, 1, 0, 0 )
            end if
            call elflr71 ( i, user, array, x, xgauss, &
                          m, n, array(ipphi), 1, &
                          index1, iuser, array(ipugs), array(ipwork) )

         end if

      end do

!     --- Compute integral depending on icheli

      eli903 = elic01 ( icheli, jdegfd, array, w, array(ipugs), &
                        array(ipdudx), array(ipwork) )

1000  call erclos ( 'eli903' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eli903'

      end if  ! ( debug )

      end
