      subroutine elm900divgrad ( sp, div, dphidx, pp, phi, w, x, xgauss, &
                                 mcont, rho, kappa, rhsdiv, fdiv, &
                                 mdiv, ishape, coeffgrad, coeffdiv, &
                                 psipres, npres, icent, rhonodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-02-2010 Change of parameter list
!        version  1.0  date 13-02-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the divergence and gradient matrix for Navier-Stokes
!     Incompressible equations
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mcont, mdiv, ishape, npres, icent
      double precision dphidx(n,m,ndimlc), w(m), sp(ndimlc*n,*), &
                       xgauss(m,*), x(n,*), phi(n,m), rho(m), &
                       div(ndimlc*n,*), rhsdiv(*), fdiv(m), &
                       coeffgrad(m), coeffdiv(m,ndimlc), rhonodes(n)
      double precision pp(ndimlc+1,ndimlc+1), kappa(m), psipres(npres,m)

!     coeffdiv       i    Array of size m*ndim containing the coefficients
!                         for the partial derivatives in the pseudo divergence
!                         operators for the m integration points
!                         Hence coeffdiv(k,i) is the coefficient for du/dx_i
!                         in integration point x^k
!     coeffgrad      i    Array of size m containing the multiplication factor
!                         for the gradient in the integration points
!     div            o    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     fdiv           i    Contains values of divergence right-hand side to be
!                         integrated (if mdiv=1)
!     icent          i    Index of centroid
!     ishape         i    Shape number of elements
!     kappa          i    Factor for pressure term
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                         2: Goertler equations: No dp/dx, but complete div u
!                         4: the continuity equation contains a pressure term
!     mdiv           i    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     npres          i    Number of pressure parameters per element
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     pp             o    Pressure matrix
!     psipres        i    Contains the pressure basis functions
!                         psipres(i,k) refers to i-th basis function in
!                         integration point k
!     rho            i    density rho
!     rhonodes       i    density rho in nodal  points
!     rhsdiv         o    Divergence right-hand side (if mdiv=1)
!     sp             o    Gradient matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL1000         Compute the divergence matrix for Navier-Stokes
!     EL1000SP       Compute the divergence matrix for Navier-Stokes
!                    (special form)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900divgrad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900divgrad'
         write(irefwr,1) 'mcont, npsi', mcont, npsi
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( mcont==3 ) then

!     --- mcont = 3, special divergence and gradient matrix

         call el1000sp ( sp, div, dphidx, w, x, xgauss, &
                         rhsdiv, fdiv, mdiv, ishape, &
                         coeffgrad, coeffdiv )

      else

!     --- Standard divergence and gradient matrix

         call el1000 ( sp, div, dphidx, pp, phi, w, x, xgauss, &
                       mcont, rho, kappa, rhsdiv, fdiv, &
                       mdiv, ishape, psipres, icent, rhonodes )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( sp, npsi, ndimlc*n, 'sp' )
         call prinrl1 ( div, npsi, ndimlc*n, 'div' )
         write(irefwr,*) 'End elm900divgrad'

      end if
      call erclos ( 'elm900divgrad' )

      end
