      subroutine elm900massmat ( elemms, rho, w, phi, workmass, &
                                 workelem, uoldm, workelvec, thetdt, &
                                 uold, jtime, mconv, dphidx, dudx, &
                                 goertler, kappa, index1, mcont, ugauss, &
                                 xgauss, ioldm, iseqvel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.2  date 12-12-2012 Long integers
!        version  2.1  date 20-10-2010 new call to el3002
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element mass matrix for the incompressible Navier-Stokes equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jtime, mconv, index1(*),  mcont, &
              ioldm, iseqvel(*)
      double precision elemms(*), rho(*), w(*), phi(*), &
                       workmass((n*ndim)**2), workelem(*), uoldm(n,*), &
                       thetdt, dphidx(*), dudx(*), &
                       goertler(*), kappa(*), uold(n,*), ugauss(*), &
                       xgauss(*), workelvec(ndim*n)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     elemms         o    Element mass matrix
!     goertler       i    Goertler number (array)
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     ioldm          i    Sequence number of velocity at end of previous time
!                         step in input vector.
!                         If 0 no iterative procedure is adopted for dynamical
!                         analyses.
!     iseqvel        i    Array containing the positions of the velocities in
!                         the element vector in the sequence required by el2005
!     jtime          i    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not ( = 0)
!                         Possible values:
!                           0:  no mass matrix
!                           1:  diagonal mass matrix with constant coefficient
!                           2:  diagonal mass matrix with variable coefficient
!                          11:  full mass matrix with constant coefficient
!                          12:  full mass matrix with variable coefficient
!                         101:  refers to the special possibility of the theta
!                               method directly applied
!                         102:  refers to the special possibility of the theta
!                               method directly applied with variable rho
!     kappa          i    coefficient for Goertler equations
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                            Not applicable with penalty function method
!     mconv          i    see USER INPUT in subroutine ELM600
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     rho            i    density rho
!     thetdt         i    1/(theta dt) for the case of the theta method
!                         immediately applied
!     ugauss         i    Preceding solution in Gauss points
!     uold           i    Array containing the previous iteration (old solution)
!     uoldm          i    Velocity vector at the end of the previous time
!                         step. The rhs contribution of the fluid mass is
!                         computed with this vector for iterative dynamical
!                         analyses. Standard case (no iterations): uold is used.
!     w              i    array of length m containing the weights for
!                         integration
!     workelem            Work array
!     workelvec           Work array
!     workmass            Work array
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision adams, work(m+n+n**2)

!     adams          Multiplication factor for the Adams-Bashfort method
!     i              Counting variable
!     work           work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL1010         Compute matrix vector multiplication
!     EL2011         compute convective terms of Navier-stokes
!                    for newton iteration   (k-d)  (right-hand side)
!     EL2011G        compute convective terms of Goertler equations
!                    for newton iteration   (k-d)  (right-hand side)
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3007         Fills old solution and derivatives
!     EL3010         Fill small matrix elemmt into larger matrix s
!     EL3013         Fill small matrix elemmtloc of size nesmall x nesmall
!                    into larger matrix elemmt of size nelarge x nelarge
!     ELMASS27       Fill the element mass matrix for the special case of 27
!                    node brick with 3 unknowns per point except for the
!                    centroid, where there are 7 unknowns due to the presence
!                    of the pressure
!     ELMASSDIAG     Copy components-wise defined diagonal element mass matrix
!                    into diagonal element mass matrix
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900massmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900massmat'
         write(irefwr,1) 'imas, m, n, ndim, jtrans, itype', &
                          imas, m, n, ndim, jtrans, itype
         call prinrl ( rho, m, 'rho' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jtrans==1 ) workmass = 0

      jadd = 0
      if ( imas==1 ) then

!     --- diagonal mass matrix

         jconsf = jtime
         call el3003 ( rho, work(1+n*n), w, phi, m ,n )

      else

!     --- non-diagonal mass matrix

         jzero = 2
         call el3002 ( rho, work, w, phi, m, n, n )
         if ( debug ) call prinrl1 ( work, n, n, 'work' )

      end if  ! ( imas==1 )

      if ( imas==1 ) then

!     --- imas = 1, build diagonal mass matrix

         call elmassdiag ( elemms, work(1+n*n), iseqvel )

      else if ( imas==2 ) then

!     --- imas = 2, build non-diagonal mass matrix

         jadd = 0
         if ( ndim==3 .and. inpelm==27 .and. &
              (itype==902 .or. itype==201) ) then

!        --- Special case, because centroid is not the last point

            call elmass27 ( elemms, work )

         else

!        --- Standard case

            do i = 1, ndim

               if ( jtrans==1 ) then

!              --- jtrans = 1, unknowns are eliminated by static condensation

                  call el3010 ( workmass, work, ndim*n, &
                                n, i, i, ndim, .false., 1d0 )

               else if ( itype==903 .or. itype==202 ) then

!              --- Type 202, 903, (Taylor-Hood element) use iseqvel

                  call el3013 ( elemms, work, icount, n, &
                                iseqvel, i, .false., 1d0 )

               else

!              --- Standard case

                  call el3010 ( elemms, work, icount, n, i, &
                                i, ndim, .false., 1d0 )

               end if  ! ( jtrans==1 )

            end do

         end if

      end if

      if ( jtime>100 ) then

!     --- jtime>100, explicit theta method
!         Update matrix, i.e. add 1/(theta dt) M to the stiffness matrix

         jadd = 1
         do i = 1,ndim
            call el3010 ( workelem, work, ndim*n, n, &
                          i, i, ndim, .false., thetdt )
         end do

!        --- Compute 1/(theta dt) M u(n) and add to right-hand side

         jadd = 1
         do i = 1,ndim

            if (ioldm/=0) then

!           --- iterative procedure adopted for dynamic analysis:
!               use velocity vector at end of previous time step

               call el1010 ( work, uoldm(1,i), &
                             workelvec, thetdt, i, ndim )

            else

!           --- standard case, no iterations: uold = uoldm

               call el1010 ( work, uold(1,i), &
                             workelvec, thetdt, i, ndim )

            end if

         end do

      end if

      if ( mconv==4 ) then

!     --- successive substitution

         call el3007 ( uold, dudx, phi, &
                       dphidx, m, n, ndimlc, ncomp, index1, &
                       work, ugauss, jtrans, numold, 2 )
         adams = 16d0/12d0

         if ( mcont==2 ) then

!        --- compute convective terms in case of Goertler equations

            call el2011g ( adams, workelvec, phi, rho, w, &
                           ugauss, dudx, xgauss, &
                           work, goertler, kappa )
         else

!        --- standard Navier Stokes equation

            call el2011 ( adams, workelvec, phi, rho, w, &
                          ugauss, dudx, xgauss, &
                          work )
         end if

         call el3007 ( uold, dudx, phi, &
                       dphidx, m, n, ndimlc, ncomp, index1, &
                       work, ugauss, jtrans, numold, 34 )
         adams = -5d0/12d0

         if ( mcont==2 ) then

!        --- compute convective terms in case of Goertler equations

            call el2011g ( adams, workelvec, phi, rho, w, &
                           ugauss, dudx, xgauss, &
                           work, goertler, kappa )
         else

!        --- standard Navier Stokes equation

            call el2011 ( adams, workelvec, phi, rho, w, &
                          ugauss, dudx, xgauss, &
                          work )
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         if ( jtrans==1 .and. imas==2 ) then

!        --- Special case, information stored in workmass

            call prinrl1(workmass,icount,icount,'workmass')

         else if ( imas==1 ) then

!        --- imas = 1, diagonal mass matrix

           call prinrl(elemms,icount,'elemms')

         else

!        --- imas = 2, non-diagonal mass matrix

            call prinrl1(elemms,icount,icount,'elemms')

         end if
         write(irefwr,*) 'End elm900massmat'

      end if  ! ( debug )
      call erclos ( 'elm900massmat' )

      end
