      subroutine elm900matal ( npres, viscos, iseqvel, psi, cr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 27-02-2010 Change of parameter list
!        version  3.1  date 04-09-2009 Correction nvel
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store extra element matrices which are necessary for
!     schur complement type methods
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, iseqvel(*)
      double precision :: viscos(m), psi(npres,m)
      logical cr

!     cr             i    if true it concerns Crouzeix-Raviart type elements, &
!                         otherwise Taylor-Hood elements
!     iseqvel        i    Array containing the positions of the velocities in
!                         the element vector in the sequence required by el2005
!     npres          i    Number of pressure parameters per element
!     psi            i    Array of length npsi x m containing the values of the
!                         basis functions for the pressure in the integration
!                         points
!     viscos         i    Array of size m containing the viscosity in the
!                         integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, nvel, nsave
      double precision kappa(m)

!     i              Counting variable
!     k              Counting variable
!     kappa          Cis used to store the inverse of the viscosity
!     nsave          Help variable to store old value of n
!     nvel           Number of velocity points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLPRESMMT  Fill pressure mass matrix weighted by coefficient kappa
!     ELMASSDIAG     Copy components-wise defined diagonal element mass matrix
!                    into diagonal element mass matrix
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900matal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900matal'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store weighted pressure mass matrix, diagonal of pressure mass
!         matrix and diagonal of velocity mass matri
!         Set nvel, i.e. number of velocity points

      if ( .not. cr .and. n==ndim+2 ) then
         nvel = n-1
      else
         nvel = n
      end if  ! ( .not. cr .and. n==ndim+2 )
      if ( debug ) then
         write(irefwr,1) 'nvel', nvel
         call prinrl ( B%w2, nvel, 'w part 2' )
      end if  ! ( debug )

!     --- Fill diagonal of velocity mass matrix

      nsave = n    ! save value of n
      n = nvel
      call elmassdiag ( E%dvelmmat, B%w2, iseqvel )
      n = nsave    ! reset n
      if ( debug ) then
         write(irefwr,1) 'n, nvel, icount, npres', &
                          n, nvel, icount, npres
         call prinin ( iseqvel, nvel*ndim, 'iseqvel' )
         call prinrl ( E%dvelmmat, icount-npres, 'vel_mass_mat' )
      end if  ! ( debug )

!     --- Fill weighted pressure mass matrix

      kappa = 1d0/viscos
      call elfillpresmmt ( psi, E%wpressmmat, kappa, npres, B%w )

!     --- Fill diagonal of pressure mass matrix

      do i = 1, npres
         E%dpressmmat(i) = sum(B%w*psi(i,:)**2)
      end do  ! i = 1, npres
      if ( debug ) &
         call prinrl ( E%dpressmmat, npres, 'press_mass_mat' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm900matal'

      end if  ! ( debug )
      call erclos ( 'elm900matal' )

      end
