      subroutine elm905 ( vecloc, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 12-08-2013 Extra parameters
!        version  2.3  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for the  navier-stokes equations
!     incompressible flow, using the integrated method (Taylor-Hood)
!     in combination with pressure correction
!     Two and three-dimensional elements
!     elm905 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine elmns2
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMMEC
!             - elm905
!    The subroutine is able to handle the following element shapes and cases:
!    2D:  3 node triangle
!         4 node triangle
!         6 node triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), array(*)

!     array               array array of length 13473
!                         contents:
!                         Starting address    length   Name:
!                         1                   32*27    ARRAY
!                         ipugs               3*27     UGAUSS
!                         ipdudx              9*27     DUDX
!                         ipdpdx              27*27*3  DPHIDX
!                         ipphix              27*27*3  PHIKSI for elp633
!                         ipetef              27       ETHA_EFFECTIVE
!                         ipseci              27       SECOND_INVARIANT
!                         ipdetd              27       DERIVATIVE of
!                                                      SECOND_INVARIANT
!                         ipsp                81*4     SP_matrix (Gradient P)
!                         ipdiv               81*4     DIV_matrix
!                         ippp                8*8      Pressure matrix
!                         ipwork              27*31    array space
!                         ipelem              81*81    Copy of part of el matrix
!                         ippsi               8*27     Linear basis function psi
!                         Total length: 14185
!                         The subarrays contain the following contents:
!                         ARRAY:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                              2D                  3D
!                         1:   rho              rho
!                         2:   omega            omega
!                         3:   f1               f1
!                         4:   f2               f2
!                         5:                    f3
!                         6:   eta/E            eta/E
!                         7:   rho in nodal points (mcont=1 only)
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!                         DPHIDX:
!                         Array of length inpelm x m x ndim containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                              i                             i
!                         d phi / dz = dphidx (i,k,3)
!                              i
!                           in node k
!                         array
!                         General array space
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     vecloc              array array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, jtime, ipdpdx, ipugs, jdiags, &
              ipwork, ipphix, mconv, modelv, ipdudx, ipsp, ipetef, &
              ipseci, ipdetd, ipelem, mcont, ipdiv, ippp, &
              iprho, j, nveloc, ipmass, iprhoc, mdiv, modelrhsd, &
              idummy(1), iseqpress, &
              ipkappa, ipfdiv, ippsix
      double precision cn, clamb, thetdt, adams, &
                       rhsdiv(8), pold(8), a1(9,m)
      double precision uold(81)
      save isub, isup, jtime, ipdpdx, ipwork, ipphix, ipugs, &
           mconv, modelv, cn, clamb, ipdudx, ipsp, ipetef, &
           ipseci, ipdetd, thetdt, ipelem, mcont, ipdiv, &
           iprho, nveloc, ipmass, iprhoc, mdiv, iseqpress, &
           modelrhsd, ippp, ipkappa, ipfdiv, ippsix

!     a1             A1 tensor defined by gradv + gradv^T
!     adams          Multiplication factor for the Adams-Bashfort method
!     clamb          Parameter lambda with respect to viscosity model
!     cn             Power in power law model
!     i              Counting variable
!     idummy         Dummy parameter
!     ipdetd         Starting address in array array of detdsc
!     ipdiv          Starting address in array array of div
!     ipdpdx         Starting address in array array of dphidx
!     ipdudx         Starting address in array array of dudx
!     ipelem         Starting address in array array of copy of element matrix
!     ipetef         Starting address in array array of etheff
!     ipfdiv         Starting address in array array of fdiv
!     ipkappa        Starting address in array array of kappa
!     ipmass         Starting address in array array of mass matrix before
!                    transformation
!     ipphix         Starting address in array array of phixi
!     ippp           Starting address in array array of pressure matrix
!     ippsix         Pointer to dpsi/dx
!     iprho          Starting address in array array of rho
!     iprhoc         Starting address in array array of rho multiplied by w
!     ipseci         Starting address in array array of secinv
!     ipsp           Starting address in array array of sp
!     ipugs          Starting address in array array of ugauss
!     ipwork         Starting address in array array of array array
!     iseqpress      Sequence number of pressure in set of solution arrays
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     jdiags         Help variable to save jdiag
!     jelp           Indication of the type of basis function subroutine must
!                    be called by ELM100.
!     jtime          Integer parameter indicating if the mass matrix for the
!                    time-dependent equation must be computed (>0) or not ( = 0)
!                    Possible values:
!                      0:  no mass matrix
!                      1:  diagonal mass matrix with constant coefficient
!                      2:  diagonal mass matrix with variable coefficient
!                     11:  full mass matrix with constant coefficient
!                     12:  full mass matrix with variable coefficient
!                    101:  refers to the special possibility of the theta
!                          method directly applied
!                    102:  refers to the special possibility of the theta
!                          method directly applied with variable rho
!     mcont          Defines the type of continuity equation
!                    Possible values:
!                    0: incompressible flow
!                    1: compressible flow: div (rho u ) = 0
!                       Not applicable with penalty function method
!                    4: the continuity equation contains a pressure term.
!     mconv          Defines the treatment of the convective terms
!                    Possible values:
!                    0: convective terms are skipped (Stokes flow)
!                    1: convective terms are linearized by Picard iteration
!                    2: convective terms are linearized by Newton iteration
!                    3: convective terms are linearized by the incorrect Picard
!                       iteration
!                    4: convective terms are linearized by successive
!                       substitution
!     mdiv           Defines if divergence right-hand side is used (1) or
!                    not (0)
!     modelrhsd      Defines how the stress tensor is treated in the rhs
!                    Possible values:
!                    0:    if there is a given stress tensor the reference
!                          to this tensor is stored in position 19
!                    1:    It is supposed that there are 6 given stress
!                          tensor components stored in positions 19-24
!     modelv         Defines the type of viscosity model
!                    Possible values:
!                      1:  newtonian liquid
!                      2:  power law liquid
!                      3:  carreau liquid
!                      4:  plastico viscous liquid
!                    100:  user provided model depending on grad u
!                    101:  user provided model depending on the second invariant
!                    102:  user provided model depending on the second invariant
!                          the co-ordinates and the velocity
!     nveloc         Number of velocity parameters
!     pold           Array containing the old pressure in this element
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rhsdiv         Divergence right-hand side (if mdiv=1)
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     thetdt         1/(theta dt) for the case of the theta method
!                    immediately applied
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              array of length n x ndim containing the co-ordinates
!                    of the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     EL0905         Perform initialization for element subroutine type 905
!     EL1004         Compute gradient and divergence matrix (Taylor-Hood)
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL2010         compute  convective terms of Navier-stokes
!                    for Picard or newton iteration   (k-d)
!     EL2011         compute convective terms of Navier-stokes
!                    for newton iteration   (k-d)  (right-hand side)
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3009         Fill series of old solutions in array vecloc
!     EL3010         Fill small matrix E%mat into larger matrix s
!     EL4029         Fill element vector for newton linearization of power law
!                    viscosity terms
!     EL4914         Compute contribution of Coriolis term
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELGETUNKNOWN   Get unknown vector from all vectors stored vecold and
!                    store in uold
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ELMATMULTRECT  Multiply rectangular matrix sp of size (nrow x ncol) by
!                    vector of size ncol to get vector of size nrow
!     ELSTRRHS       Compute contribution of initial stress to right-hand side
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELNSA1      Compute the A1 tensor, defined by grad v + grad v^T
!     SEPELNSGRAD    Compute gradient tensor in case of Navier-Stokes
!     SEPELNSSECINV  Compute second inverse in case of Navier-Stokes
!     SEPELNSSTRESS  compute the viscous matrix for 3-dim Navier stokes
!     SEPELVISC      Compute effective viscosity depending on the viscosity
!                    model
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  information about the time-integration
!         Possible values:
!          0: if imas in BUILD=0 then time-independent flow
!             else the mass matrix is built according to imas
!          1: The theta method is applied directly in the following way:
!             the mass matrix divided by theta delta is added to the stiffness
!             matrix
!             the mass matrix divided by theta delta and multiplied by u(n) is
!             added to the right-hand-side vector
!     2:  type of viscosity model used (modelv) as well as given stress
!         tensor as right-hand side accordinag to
!         modelv + 1000*model_rhsd
!         Possible values for modelv
!         0,1:  newtonian liquid
!         2:    power law liquid
!         3:    carreau liquid
!         4:    plastico viscous liquid
!       100:    user provided model depending on grad u
!       101:    user provided model depending on the second invariant
!       102:    user provided model depending on the second invariant, the
!               co-ordinates and the velocity
!         Possible values for model_rhsd:
!         0:    if there is a given stress tensor the reference to this
!               tensor is stored in position 19
!         1:    It is supposed that there are 6 given stress tensor components
!               stored in positions 19-24
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
!     5:  Information about the convective terms (mconv)
!         and continuity equation (mcont) according to mconv+10*mcont
!         Possible values for mconv:
!         0: convective terms are skipped (Stokes flow)
!         1: convective terms are linearized by Picard iteration
!         2: convective terms are linearized by Newton iteration
!         3: convective terms are linearized by the incorrect Picard
!            iteration
!         Possible values for mcont:
!         0: Standard situation, the continuity equation is div u = 0
!         1: A special continuity equation div ( rho u ) = 0, which rho
!            variable is solved. This is not the standard continuity equation
!            for incompressible flow. In fact we have a time-independent
!            compressible flow description.
!            This possiblility can not be applied with the penalty function
!            approach
!         4: the continuity equation contains a pressure term.
!     6-20: Information about the equations and solution method
!     6:  - &
!     7:  density rho
!     8:  angular velocity
!     9:  F1  (Body force in 1-direction)
!    10:  F2  (Body force in 2-direction)
!    11:  F3  (Body force in 3-direction)
!    12:  eta (if modelv = 1, the dynamic viscosity
!                          2, the parameter eta_n
!                          3, the parameter eta_c
!                          4, the parameter eta_pv)
!    13:  n (if modelv = 2-4, the parameter n in the power of the model)
!    14:  lambda (if modelv = 3, the parameter lambda in the viscosity model
!                 if modelv = 4, the parameter s in the viscosity model)
!    15-16: not yet used
!    17:  kappa  Parameter for the pressure term in the continuity equation
!                (MCONT=4)
!    18:  fdiv   Right-hand side for continuity equation
!    19:  stress If model_rhsd = 0 then
!                Given stress tensor used in the right-hand side vector
!                At this moment it is supposed that the stress is defined
!                per element and that in each element all components are
!                constant
!                If model_rhsd = 1 then
!                The six components of the stress tensors are stored in
!                positions 19 to 24 in the order: xx, yy, zz, xy, yz, zx
!                This option can not be used in cooporation with the
!                dimensionless coefficients stored in positions 20-41
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2786   iseqpress = 0, but the matrix must be built
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( first call ) then
!       initialize standard parameters
!    compute basis functions
!    fill old solutions if necessary
!    if ( matrix ) then
!       fill the viscosity in the corresponding array
!       Compute the viscous terms
!       if ( mconv>0 ) then
!          compute convective terms
!       Compute the divergence matrix
!       Compute the pressure matrix
!       Compute the penalty contribution
!    if ( vector ) then
!       Compute contribution of body forces
!       if ( mconv=2 ) then
!          compute contribution of convective terms
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm905' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm905'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0905 ( E%iuser, E%user, jelp, jtime, cn, clamb, &
                       mconv, mcont, modelv, isub, isup, array, &
                       thetdt, nveloc, mdiv, modelrhsd, iseqpress )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch
         if ( ierror/=0 ) go to 1000

         if ( jelp<9 .or. jelp>10 .and. jelp<17 ) B%comppsi = .true.
         if ( debug ) then
            write(irefwr,100) 'mconv, mcont, modelv, isub, isup', &
                               mconv, mcont, modelv, isub, isup
            write(irefwr,100) 'nveloc, mdiv, modelrhsd, iseqpress ', &
                               nveloc, mdiv, modelrhsd, iseqpress
         end if  ! ( debug )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

         ipugs = 1 +32*m
         iprho = 1
         ipkappa = 1+7*m
         ipfdiv = 1+8*m
         ipdudx = ipugs  + ncomp*m
         ipdpdx = ipdudx + ncomp**2*m
         ipphix = ipdpdx + ndim*m*n
         ipetef = ipphix + ndim*n*m
         ipseci = ipetef + m
         ipdetd = ipseci + m
         ipsp   = ipdetd + m
         if ( mcont==0 ) then
            ipdiv = ipsp
         else
            ipdiv = ipsp + n*npsi*ndim
         end if

!        --- set pointer for pressure matrix

         if ( mcont==4 ) then
            ippp = ipdiv + n*npsi*ndim
            ipwork = ippp + (npsi)**2
         else
            ippp = ipdiv
            ipwork = ippp + n*npsi*ndim
         end if

         ipelem = ipwork + n*n+max(n*(1+ndim),8*m)
         iprhoc = ipelem + (n*ndim)**2
         ipmass = iprhoc+m
         ippsix = ipmass+(ndim*n)**2
         if ( imas==2 ) then
            do i = 1, (ndim*n)**2
               array(ipmass+i-1) = 0d0
            end do
         end if

!        --- Check array length as defined in elmgrinf

         call inichkarlen (  ippsix+m*npsi*n, 27316, 'array' )
         if ( ierror/=0 ) go to 1000

      end if
      do i = 1, 8
         rhsdiv(i) = 0d0
      end do

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc

      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el2005 ( uold, array(ipdudx), B%phi, B%phider, &
                       idummy, array(ipugs), iseqin )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, B%phi, n )

!     --- Copy pressure into array pold

      if ( iseqpress>0 ) &
         call elgetunknown ( iseqpress, npsi, pold, 'pressure' )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinrl ( pold, npsi, 'pold' )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            jdiags = jdiag
            if ( mcont==1 .and. i==1 .and. jdiag/=1 ) jdiag = 1
            call elflr71 ( i, E%user, array, B%xyz, B%xyzg, &
                          m, n, B%phi, 1, E%nodes, &
                          E%iuser, array(ipugs), vecloc )
            if ( mcont==1 .and. i==1 .and. jdiags/=1 ) then

!           --- rho requires special attention in case of compressible flow
!               Copy rho to part 7 and compute rho in Gauss points

               do j = 1, n
                  array(iprho-1+j) = array(j)
               end do
               do j = 1, m
                  array(j) = ddot(n, array(iprho), 1, B%phi(1,j), 1)
               end do
               jdiag = jdiags

            end if

         end if

      end do

!     --- Compute the divergence and gradient matrix

      if ( iseqpress>0 ) then

!     --- Only in case the pressure is given, we compute the matrix

         call el1004 ( array(ipsp), array(ipdiv), B%lam, &
                       B%phider, B%phi, B%w, B%xyzg, mcont, &
                       array(iprho), array(ippp), array(ipkappa), &
                       rhsdiv, array(ipfdiv), mdiv, array )

      end if  ! ( iseqpress>0 )

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix
!         fill the viscosity in array etaeff

         if ( iseqpress==0 ) then

!        --- iseqpres = 0, hence matrix must not be computed

            call errint ( 1, ielgrp )
            call errint ( 2, iseqpress )
            call errint ( 3, itype )
            call errsub ( 2786,  3, 0, 0 )
            go to 1000

         end if  ! ( iseqpress=0 )

         if ( modelv/=1 .and. modelv/=5 .and. modelv/=10 ) then
            call sepelnsgrad ( array(ipwork), B%xyzg, array(ipugs), &
                               array(ipdudx), m )
            if ( modelv/=100 ) then
               call sepelnsa1 ( array(ipwork), a1 )
               call sepelnssecinv ( a1,array(ipseci)  )
            end if  ! ( modelv/=100 ) then
         end if  ! ( modelv/=1 .and. modelv/=5 .and. modelv/=10 )

         call sepelvisc ( modelv, array(1+5*m), array(ipetef), cn, &
                          clamb, B%xyzg, array(ipugs), array(ipdudx), &
                          array(ipseci), array(ipdetd), array(ipwork), &
                          vecloc, maxunk, numold )

!        --- Compute the viscous terms

         call sepelnsstress ( E%mat, B%phider, B%phi, B%xyzg, &
                              array(ipetef), B%w, mcont, modelv )
         if ( ind(2)/=0 ) then

!        --- Coriolis accelaration

            call el4914 ( E%mat, B%phi, array, B%w, array(m+1), &
                          array(ipwork), array(ipwork+n*n) )

         end if
         if ( mconv>0 .and. mconv/=4 ) then

!        --- compute convective terms

            symm = .false.
            call el2010 ( E%mat, B%phider, B%phi, array, B%w, &
                          mconv, array(ipugs), array(ipdudx), B%xyzg, &
                          array(ipwork), B%lam )

         end if

      end if

      if ( debug ) then
         write(irefwr,*) 'matrix, vector ', matrix, vector
         write(irefwr,100) 'notvec', notvec
      end if  ! ( debug )

      if ( vector .and. notvec==0 ) then

!     --- vector = true
!         First set element vector equal to zero

         E%vec=0d0

!        --- In case of iseqpress = 0, we jump over the computation of the rhs
!            Only the convective term is computed

         if ( iseqpress==0 ) go to 200

!        --- Next compute contribution due to external forces
!            Multiply weights by density rho and store in array from
!            position iprhoc

         if ( ind(3)/=0 .or. ind(4)/=0 .or. ind(5)/=0 ) then

!        --- At least one body force vector available

            do i = 1, m
               array(iprhoc+i-1) = B%w(i) * array(i)
            end do

         end if

!        --- First component

         if ( ind(3)/=0 ) then

!        --- f1 # 0  store contribution temporary in array array

            jadd = 0
            call el3003 ( array(2*m+1), array(ipwork), array(iprhoc), &
                          B%phi, m, n )
            do i = 1, n
               E%vec(ndim*(i-1)+1) = array(ipwork+i-1)
            end do

         end if

!        --- Second component

         if ( ind(4)/=0 ) then

!        --- f2 # 0  store contribution temporary in array array

            jadd = 0
            call el3003 ( array(3*m+1), array(ipwork), array(iprhoc), &
                          B%phi, m, n )
            do i = 1, n
               E%vec(ndim*(i-1)+2) = array(ipwork+i-1)
            end do

         end if

!        --- Third component

         if ( ind(5)/=0 ) then

!        --- f3 # 0  store contribution temporary in array array

            jadd = 0
            call el3003 ( array(4*m+1), array(ipwork), array(iprhoc), &
                          B%phi, m, n )
            do i = 1, n
               E%vec(ndim*(i-1)+3) = array(ipwork+i-1)
            end do

         end if

         if ( ind(10)/=0 .or. ind(11)/=0 .or. ind(12)/=0 .or. &
              ind(13)/=0 .or. ind(14)/=0 .or. ind(15)/=0 ) then

!        --- Stress term corresponding to right-hand side

            call elstrrhs ( array(9*m+1), B%phider, E%vec, &
                            B%w, m, n, ndimlc, B%phi, B%xyzg, modelrhsd )

         end if

200      if ( mconv==2 .or. mconv ==4 ) then

!        --- compute contribution of convective terms

            if ( mconv==4 ) then
               if ( jtime>0 ) then
                  adams = -23d0/12d0
               else
                  adams = -1d0
               end if
            else
               adams = 1d0
            end if
            call el2011 ( adams, E%vec, B%phi, array, B%w, &
                          array(ipugs), array(ipdudx), B%xyzg, &
                          array(ipwork) )

         end if

!        --- Jump if iseqpress = 0, since only the convective contribution
!            is computed

         if ( iseqpress==0 ) go to 1000

         if ( modelv==2 .and. cn>1d0 ) then

!        --- modelv = 2, and cn>1:  Newton linearization

            call el4029 ( E%vec, B%xyzg, array(ipugs), &
                          array(ipdudx), array(ipseci), array(ipdetd), &
                          array(ipwork), B%phider, B%w )
         end if

!        --- Add Part Grad p^n in right-hand side vector

         jadd = 1
         call elmatmultrect ( array(ipsp), pold, E%vec, icount, &
                              npsi, jadd )
         if ( debug ) call prinrl ( E%vec, icount, &
            'E%vec after L^T P' )

      end if

      if ( jtime>0 ) then

!     --- jtime>0, i.e. the mass matrix must be computed
!         First compute the mass matrix for one velocity component
!         The non-diagonal mass matrix for this component is stored in
!         array positions ipwork - ipwork+n*n-1
!         The diagonal matrix in array pos. ipwork+n*n - ipwork+n*n+n-1

         jadd = 0

!        --- non-diagonal mass matrix

         jzero = 2
         call el3002 ( array, array(ipwork), B%w, B%phi, m, n, n )
         jadd = 0
         do i = 1, ndim
            call el3010 ( E%mmat, array(ipwork), ndim*n, n, &
                          i, i, ndim, .false., 1d0 )
         end do

      end if

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,100) 'element, itype', ielem, itype
         call prinrl ( E%vec, icount, 'elemvc' )
         call prinrl1 ( E%mat, icount, icount, 'elemmt' )
         if ( jtime>0 ) &
            call prinrl1 ( E%mmat, icount, icount, 'elemms' )
         write(irefwr,*) 'End el0m05'

      end if
      call erclos ( 'elm905' )

      end
