      subroutine elm907 ( vecloc, workar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.4  date 12-08-2013 Combine dphi/dxi and dphideta
!        version  1.3  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for boundary conditions for
!     pressure equation in pressure correction method
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), workar(*)

!     vecloc         Work array in which all old solution vectors for the
!                    integration points are stored
!     workar         Real work array
!                    work consists of the following parts
!                    Starting address    length   Name:
!                    1                   8*9    ARRAY
!                    Total length: 72
!                    The subarrays contain the folowing contents:
!                    ARRAY:
!                    array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                  2D            3D
!                    1:   sigma            sigma          sigma
!                    2:   h                h              h
!                    3:   alpha 11         alpha 11       alpha 11
!                    4:                    alpha 12       alpha 12
!                    5:                    alpha 22       alpha 13
!                    6:                                   alpha 22
!                    7:                                   alpha 23
!                    8:                                   alpha 33
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(27), w(9), xgauss(27), dphidx(243), &
                       uold(9), ugauss(9), tang(27), qmat(12), &
                       dpsidx(36*3)
      integer i, isub, isup, jelp, jboun, ipphi, ippsi, &
              ipdphidxi, ipdphideta, ipend, indexv(4), ind1(4)
      save isub, isup, jelp, jboun, ipphi, ippsi, ipdphidxi, &
           ipdphideta

!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     i              Counting variable
!     ind1           Array containing the relative position of the vertices
!                    in index1
!     indexv         Contains the nodal points of the vertices
!     ipdphideta     Starting address of array dphideta
!     ipdphidxi      Starting address of array dphidxi
!     ipend          Last position in array
!     ipphi          Starting address of array phi
!     ippsi          Starting address in array work of psi
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jboun          Defines the type of boundary condition.
!                    Possible values:
!                    2:  Standard mixed (Robbins) boundary condition
!                    3:  Boundary condition of type 3
!                    4:  Boundary condition of type 4 (not yet available)
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     0:  Point element, no special basis functions
!                     1:  Subroutine ELP615  (Linear line element)
!                     2:  Subroutine ELP616  (Quadratic line element)
!                     3:  Subroutine ELP625  (Linear triangle)
!                     4:  Subroutine ELP627  (Quadratic triangle)
!                     5:  Subroutine ELP625  (Bilinear quadrilateral)
!                     6:  Subroutine ELP628  (Biquadratic quadrilateral)
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              contains the coordinates of the nodes
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0907         Initialize some constants with respect to subroutine elm907
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM910BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!                    (boundary elements)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm907' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm907'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call el0907 ( E%iuser, E%user, jelp, jboun, isub, isup, workar, &
                       ind1 )
         if ( notvec==1 .and. notmat==1 ) go to 1000

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )
         ipphi = 1+m
         ippsi = ipphi+n*m
         ipdphidxi = ippsi+n*m
         ipdphideta = ipdphidxi+n*m*ndim
         ipend = ipdphideta+n*m*ndim

         if ( ipend>1500 ) then

!        --- Array work too small

            call errint ( ipend, 1 )
            call errint ( 1500, 2 )
            call errchr ( 'work', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end if

!     --- Put vertex numbers in indexv

      do i = 1, n
         indexv(i) = index1(ind1(i))
      end do  ! i = 1, n

!     --- compute basis functions and weights for numerical integration

      call elm910basefn ( coor, x, w, dphidx, xgauss, work(ipphi), &
                          indexv, qmat, tang, jelp, work(ippsi), dpsidx, &
                          work(ipdphidxi) )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( uold, workar, work(ipphi), dphidx, m, n, &
                       ndimlc, ncomp, indexv, workar, ugauss, &
                       jtrans, numold, 1 )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, work(ipphi), n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, workar, x, xgauss, &
                          m, n, work(ipphi), 1, indexv, &
                          E%iuser, ugauss, vecloc )

         end if

      end do

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix
!         Boundary conditions of type 2

         jadd = 0
         call el3002 ( workar(1), E%mat, w, work(ipphi), m, n, n )

      end if

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         jadd = 0
         call el3003 ( workar(1+m), E%vec, w, work(ipphi), m ,n )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'element, itype', ielem, itype
         write(irefwr,1) 'notvec, notmat, notmas', &
                          notvec, notmat, notmas
         write(irefwr,*) 'matrix, vector', matrix, vector
         call prinrl ( E%vec, icount, 'elemvc' )
         call prinrl1 ( E%mat, icount, icount, 'elemmt' )
         write(irefwr,*) 'End elm907'

      end if  ! ( debug )
      call erclos ( 'elm907' )

      end
