      subroutine elm910basefn ( coor, x, w, dphidx, xgauss, phi, index1, &
                                qmat, tang, jelp, psi, dpsidx, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 12-08-2013 Extra parameters
!        version  4.0  date 12-08-2013 Remove dphideta
!        version  3.1  date 29-07-2013 New call to elp621
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the basis functions phi, its derivatives and other related
!     quantities for various element shapes (boundary elements)
!     Also the weights for numerical integration are computed
!     Help subroutine for elm910
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), jelp
      double precision coor(ndimlc,*), x(*), w(*), dphidx(*)
      double precision xgauss(*), phi(*), tang(*), qmat(*), psi(*), &
                       dpsidx(*), dphidxi(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dphidxi       i/o   Array containing dphi_i/dxi in a point
!                         At first call output otherwise input
!     dpsidx         o    Array of length n/2 x m x ndim  containing the
!                         derivatives of the linear basis functions
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx. Possible values:
!                          1:  Subroutine ELP610  (Linear line element)
!                          2:  Subroutine ELP611  (Quadratic line element)
!                          3:  Subroutine ELP620  (Linear triangle)
!                          4:  Subroutine ELP624  (Extended quadratic triangle)
!                          5:  Subroutine ELP621  (Bilinear quadrilateral)
!                          6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                         11:  Subroutine ELP630  (Linear tetrahedron)
!                         12:  Subroutine ELP631  (Quadratic tetrahedron)
!                         13:  Subroutine ELP632  (Trilinear brick)
!                         14:  Subroutine ELP633  (Triquadratic brick)
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     tang           o    In this array the tangential vector is stored
!                         tang(j,i) contains the j th component of the
!                         tangential vector in the i-th integration point
!     w              o    array of length m containing the weights for
!                         integration
!     x              o    x co-ordinate of point
!     xgauss         o    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP610         Compute basis functions and weights for linear line element
!     ELP616         Compute basis functions and weights for quadratic line
!                    element R2
!     ELP620         Compute basis functions and weights for linear triangle
!     ELP621         Compute basis functions and weights for bilinear
!                    quadrilateral
!     ELP622         Compute basis functions and weights for quadratic triangle
!     ELP623         Compute basis functions and weights for biquadratic
!                    quadrilateral
!     ELP624         Compute basis functions and weights for extended quadratic
!                    triangle
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm910basefn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm910basefn'

      end if
      if ( ierror/=0 ) go to 1000

      select case ( jelp )

         case(-10,0)

!        --- ishape = -10 or 0, Point element, fill array x

            do i = 1, ndimlc
               x(i) = coor(i,index1(1))
            end do
            w(1) = 1d0
            if ( jcart==2) w(1) = 2d0*pi*x(1)

         case(:-11,-9:-1,8:)

!        --- Shape number not available (<0 but not -10 or>8)

            call errint ( jelp, 1 )
            call errsub ( 800, 1, 0, 0 )

         case(1)

!        --- Shape number 1

            call elp610 ( coor, x, w, dphidx, xgauss, phi, index1, &
                          qmat, tang )

         case(2)

!        --- Shape number 2

            call elp616 ( coor, x, w, dphidx, xgauss, phi, index1, &
                          qmat, tang, dphidxi )

         case(3)

!        --- Shape number 3

            call elp620 ( coor, x, w, dphidx, xgauss, phi, index1, &
                          qmat )

         case(4)

!        --- Shape number 4

            call elp622 ( coor, x, w, dphidx, xgauss, phi, index1, qmat, &
                          psi, dpsidx, dphidxi )

         case(5)

!        --- Shape number 5

            call elp621 ( coor, x, w, dphidx, xgauss, phi, index1, &
                          qmat, dphidxi )

         case(6)

!        --- Shape number 6

            call elp623 ( coor, x, w, dphidx, xgauss, phi, index1, qmat, &
                          psi, dpsidx, dphidxi  )

         case(7)

!        --- Shape number 7

            call elp624 ( coor, x, w, dphidx, xgauss, phi, index1, qmat, &
                          psi, dpsidx, dphidxi )

      end  select  ! case ( jelp )

1000  if ( debug ) write(irefwr,*) 'End elm910basefn'
      call erclos ( 'elm910basefn' )

      end
