      subroutine elm912 ( array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 05-03-2015 Use defaults
!        version  1.2  date 23-08-2001 New call to elusr3
!        version  1.1  date 14-09-1998 Extension for axi-symmetric coordinate
!
!   copyright (c) 1998-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill large line element with equation for given mass flow
!    The penalty function approach is used
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_flow
!     boundary_element
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleconsts
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision array(*)

!     array           Work array for the element subroutines
!                     Length: inpelm
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idegfd, j, mrule, ishift
      double precision h, massflux, penalp

!     h          Length of element
!     i          Counting variable
!     idegfd     Degree of freedom
!     ishift     Shift in coefficient sequence number
!     j          Counting variable
!     massflux   Given mass flux
!     mrule      Type of integration rule as given by the user
!     penalp     Penalty parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is for the given mass-flow element is:
!
!     1:  - &
!     2:  - &
!     3:  Type of numerical integration
!         Possible values:
!         1 = linear
!         2 = quadratic
!     4:  Type of co-ordinate system
!         0 = Cartesian
!         1 = axi-symmetric
!     5:  Sequence number of degree of freedom that corresponds to normal
!         component
!         If u is the normal component: 1
!         If v is the normal component: 2
!         Use local co-ordinates if the boundary is not in one of these
!         directions
!     6:  Amount of mass through boundary, i.e. / u.n ds
!     7:  Penalty parameter (usually 10^6)
!     8-10:  - &
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     838   Penalty parameter is not constant
!    1070   Degree of freedom incorrect
!    1766   Type of integration rule incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm912' )

!     --- Get information from arrays E%iuser and user

      ishift = 0
      ipos = E%iuser ( ielgrp + 5 )
      if ( ipos.lt.7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )
      end if
      ipos = ipos + 2
      ishift = ishift+2
      mrule = E%iuser(ipos)
      ipos = ipos + 1
      ishift = ishift+1
      if ( mrule.lt.1 .or. mrule.gt.2 ) then
         call errint ( mrule, 1 )
         call errint ( 5, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if
      jcart = E%iuser(ipos)
      ipos = ipos + 1
      ishift = ishift+1
      if ( jcart.lt.0 .or. jcart.gt.1 ) then
         call errint ( jcart, 1 )
         call errint ( 4, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if
      idegfd = max(1,E%iuser(ipos))
      ipos = ipos + 1
      ishift = ishift+1
      if ( idegfd.lt.1 .or. idegfd.gt.2 ) then
         call errint ( idegfd, 1 )
         call errint ( 4, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1070, 5, 0, 0 )
      end if

!     --- Get mass flux

      m = 1
      ishift = ishift+1
      call elusr3 ( 1, array, E%iuser, E%user, ishift )
      if ( ind(1).gt.0 ) then
         call errint ( itype, 1 )
         call errint ( 6, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errsub ( 838, 4, 0, 0 )
      end if
      massflux = array(1)

!     --- penalty parameter

      ishift = ishift+1
      call elusr3 ( 2, array, E%iuser, E%user, ishift )

      if ( ind(2).gt.0 ) then
         call errint ( itype, 1 )
         call errint ( 7, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errsub ( 838, 4, 0, 0 )
      end if
      penalp = array(1)
      if ( penalp.le.0d0 ) penalp = 1d6

!     --- Clear the element matrix and element vector

      E%vec = 0d0
      E%mat = 0d0
      array(1:inpelm) = 0d0

!     --- Compute the element matrix and vector

      if ( mrule.eq.1 ) then

!     --- Linear elements
!         Store the factors for the trapezoid rule in array

         do i = 1, inpelm-1
            h = sqrt((coor(1,index1(i+1))-coor(1,index1(i)))**2+ &
                     (coor(2,index1(i+1))-coor(2,index1(i)))**2)*0.5d0
            array(i) = array(i)+h
            array(i+1) = array(i+1)+h
         end do

      else

!     --- Quadratic elements
!         Store the factors for the Simpson rule in array

         do i = 1, inpelm-2, 2
            h = sqrt((coor(1,index1(i+2))-coor(1,index1(i)))**2+ &
                     (coor(2,index1(i+2))-coor(2,index1(i)))**2)/6d0
            array(i) = array(i)+h
            array(i+1) = array(i+1)+h*4d0
            array(i+2) = array(i+2)+h
         end do

      end if

!     --- Next store the correct values in E%mat and E%vec

      h = penalp * massflux

      if ( jcart.eq.1 ) then

!     --- axi-symmetric coordinates

         do i = 1, inpelm
            array(i) = array(i) * coor(1,i) * 2d0 * pi
         end do

      end if
      do i = 1, inpelm
         E%vec(2*i-2+idegfd) = h*array(i)
         do j = 1, inpelm
            E%mat(2*i-2+idegfd,2*j-2+idegfd) = &
               penalp*array(i)*array(j)
         end do
      end do

      call erclos ( 'elm912' )
      end
