      subroutine elm914 ( vecloc, workar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.9  date 12-08-2013 Combine dphi/dxi and dphideta
!        version  1.8  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for discontinuous boundary conditions for
!     Navier-Stokes equations
!     Two and three dimensional elements
!    The subroutine is able to handle the following element shapes and cases:
!    linear line element (2D)
!    quadratic line element (2D)
!    linear triangle (3D)
!    quadratic triangle (3D)
!    bilinear quadrilateral (3D)
!    biquadratic quadrilateral (3D):
!    cartesian co-ordinates
!    axi-symmetric co-ordinates (2D only)
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), workar(*)

!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
!     workar              Real work array
!                         work consists of the following parts
!                         Starting address    length   Name:
!                         1                   10*9    ARRAY
!                         Total length: 90
!                         The subarrays contain the folowing contents:
!                         ARRAY:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                                2D             3D
!                         1:     Tn             Tx
!                         2:     Tt             Ty
!                         3:     -              Tz
!                         4:     cn             cx
!                         5:     ct             cy
!                         6:     -              cz
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, j, iload, induv(12), ipelem, ipend, &
              icountd2, inpelmsav
      double precision uold(9), ugauss(9)
      save isub, isup, jelp, iload, induv, ipelem

!     i              Counting variable
!     icountd2       icount/2
!     iload          Indicates the type of boundary condition.
!                    Possible values:
!                    0: Prescribed load in co-ordinate directions
!                    1: Prescribed load given in normal and tangential
!                       direction
!                    2: Surface tension only in 2D
!     induv          Index array of size ncomp x inpelm that contains
!                    the positions of the degrees of freedom 1 to ncomp
!                    in the element
!                    Hence induv(k,i) contains the position of the k^th
!                    degree of freedom in the i^th node of the element
!                    with respect to the element vector
!     ipdphideta     Starting address of array dphideta
!     ipdphidxi      Starting address of array dphidxi
!     ipelem         Starting address of subelement matrix in work
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELwork. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0910         Initialize parameters for subroutine ELM910
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     EL3010         Fill small matrix E%mat into larger matrix s
!     EL3013         Fill small matrix E%matloc of size nesmall x nesmall
!                    into larger matrix E%mat of size nelarge x nelarge
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM910BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!                    (boundary elements)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!     1:  Type of boundary condition
!         Possible values:
!         0: Prescribed load (in co-ordinate directions)
!         1: Prescribed load (in normal and tangential direction (2d))
!     2:
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!     5:  Not yet in use:  set equal to 0
!     6:  Tx (1-component in local co-ordinate system)
!         or if iload=2: sigma is coefficient of surface tension
!     7:  Ty (2-component in local co-ordinate system)
!     8:  Tz (z component of load or 3-component in local co-ordinate system)
!     9:  cx (1-component in local co-ordinate system of coefficient before ux)
!    10:  cy (2-component in local co-ordinate system of coefficient before uy)
!    11:  cz (z component of load or 3-component in local co-ordinate system
!             of coefficient before uz)
!    12-15: Not yet in use (must be zero)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!
!
!        The following formulae are used:
!
!       Type 1:  f_l(i)   = /_e  T_l phi_i  de
!
!                s_l(i,j) = /_e  c_l phi_i phi_j de
!
!        Each integral is approximated by a numerical rule of the shape:
!
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!
!
!        the factors w  form the weights for the numerical integration
!                     k
!
!        In the axi-symmetric case they include the factor 2 pi r
!
!
!        The element matrix becomes:
!           |  S   -S |                  S = |  S_n   0  |
!           |  S   -S |                      |   0   S_t |
!        and the vector
!           | f_n |
!           | f_l |
!           | f_n |
!           | f_l |
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm914' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm914'
         write(irefwr,1) 'ielem, itype, icount, inpelm', &
                          ielem, itype, icount, inpelm
         call prinin ( index1, inpelm, 'index1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0910 ( E%iuser, E%user, jelp, isub, isup, workar, iload, induv )
         ipelem = 6*m+1

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )
         E%vec = 0d0
         E%mat = 0d0
         do i = 1, icount*icount/4
            workar(ipelem+i) = 0d0
         end do
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

      end if
      icountd2 = icount/2

!     --- Divide inpelm temporary by 2, in order to fake the elp subroutines

      E%npelm = E%npelm/2
      inpelmsav = inpelm

!     --- compute basis functions and weights for numerical integration

!      call elm910basefn ( coor, B%xyz, B%w,B%phider B%xyzg, B%phi, &
!                          index1, qmat, tang, jelp, work(ippsi), dpsidx, &
!                          work(ipdphidxi) )
      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Reset inpelm

      E%npelm = E%npelm*2
      inpelm = inpelmsav

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( uold, workar, B%phi,B%phider, m, n, &
                       ndimlc, ncomp, induv, workar, ugauss, &
                       jtrans, numold, iseqin )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, B%phi, n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, workar, B%xyz, B%xyzg, &
                          m, n, B%phi, 1, index1, &
                          E%iuser, ugauss, vecloc )

         end if

      end do

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         jadd = 0
         do i = 1, ndim

            if ( ind(i)/=0 ) then

!           --- Component is unequal to zero

               call el3003 ( workar(1+(i-1)*m), work, B%w, B%phi, &
                             m, n )
               do j = 1, n
                  E%vec(induv(ndim*(j-1)+i))=-work(j)
               end do

            end if

         end do

!        --- Copy element vector again in last part

         do i = 1, icountd2
            E%vec(i+icountd2) = -elemvc(i)
         end do

      end if

!     --- Fill element matrix if necessary

      if ( matrix .and. notmat/=1 ) then

!     --- matrix = true, compute element matrix
!         Loop over the ndim components

         do i = 1, ndim

            if ( ind(i+3)/=0 ) then

!           --- Component is unequal to zero

               jadd = 0

               call el3002 ( workar(1+(i+2)*m), work, B%w, B%phi, &
                             m, n, n )
               jadd = 0
               call el3013 ( workar(ipelem), work, icountd2, n, induv, &
                             i, .false., 1d0 )

            end if

         end do

!        --- Next the subelement matrix is copied four times into E%mat

         call el3010 ( E%mat, workar(ipelem), icount, icountd2, 1, 1, &
                       1, .false., 1d0 )
         call el3010 ( E%mat, workar(ipelem), icount, icountd2, 1, &
                       icountd2+1, 1, .false., -1d0 )
         call el3010 ( E%mat, workar(ipelem), icount, icountd2, &
                       icountd2+1, 1, 1, .false., -1d0 )
         call el3010 ( E%mat, workar(ipelem), icount, icountd2, &
                       icountd2+1, icountd2+1, 1, .false., 1d0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ielem, itype', ielem, itype
         call prinrl(E%vec,icount,'elemvc')
         call prinrl1(E%mat,icount,icount,'elemmt')
         write(irefwr,*) 'End elm914'

      end if  ! ( debug )
      call erclos ( 'elm914' )

      end subroutine elm914
