      subroutine elm915 ( vecloc, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.6  date 09-03-2013 Replace elflr7 by elflr71
!        version  1.5  date 12-12-2012 Long integers
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element matrix and vector for quadratic line element corresponding
!     to type 915: element for update of free boundary according to the
!     method described in
!     N.P. Kruyt, C. Cuvelier, A. Segal, J. van der Zanden, &
!     A total linearisation method for solving viscous free boundary flow
!     problems by the finite element method,<BR>
!     Int. J. for Num. Methods in Fluids, Vol. 8, pp. 351-363, 1988
!     ELM915 is a help subroutine for subroutine BUILD (TO0050)
!     it is called through the intermediate subroutine el0000
!     So:
!     BUILD
!     TO0050
!       -  Loop over elements
!          -  ELMMEC
!             - ELM915
!    The subroutine is able to handle the following element shapes and cases:
!    linear line element (2D)
!    quadratic line element (2D)
!    linear triangle (3D)
!    quadratic triangle (3D)
!    bilinear quadrilateral (3D)
!    biquadratic quadrilateral (3D):
!    cartesian co-ordinates
!    axi-symmetric co-ordinates (2D only)
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), array(*)

!     array               Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                                2D
!                         1:     rho
!                         2:     mu
!                         3:     sigma
!                         4:     p
!                         5:     basis functions phi (nxm)
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, induv(27), ipphider, ipend, ipphi
      double precision x(3,2), w(3), dphidx(3,3), u(3,2), ugauss(3,2), &
                       xgauss(3,2), tang(3,2), &
                       dudx(3,2), work(12), thead(2), qmat(12)
      save isub, isup, jelp, induv, ipphider, ipphi

!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     i              Counting variable
!     induv          Index array of size ncomp x inpelm that contains
!                    the positions of the degrees of freedom 1 to ncomp
!                    in the element
!                    Hence induv(k,i) contains the position of the k^th
!                    degree of freedom in the i^th node of the element
!                    with respect to the element vector
!     ipend          Last position in array
!     ipphi          Starting address of array phi
!     ipphider       Starting address of array phider
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELelm915x. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     thead          Vector defining the tangential direction in the point
!                    element
!     u              array of length m containing the values of the
!                    second coefficient in the integration points
!     ugauss         Preceding solution in Gauss points
!     w              array of length m containing the weights for
!                    integration
!     work           Work array
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0915         Initialize parameters for subroutine ELM915
!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     EL915MAT       Compute element matrix for elm915
!     EL915VEC       Compute element vector for elm915
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP616         Compute basis functions and weights for quadratic line
!                    element R2
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is:
!
!     1:  Type of boundary condition
!         Possible values:
!         0: Prescribed load (in co-ordinate directions)
!         1: Prescribed load (in normal and tangential direction (2d))
!         2: Surface tension only in 2D
!         3: Boundary condition of the type c_t v_t + sigma_t = u . t
!            c_t must be given in position 9 and the Cartesian vector u in
!            positions 6 to 8
!            Only in R^3 and in combination with local transforms
!            The first component of the local transformation must correspond to
!            the normal vector and the other 2 to the tangential vectors
!            The tangential vectors are extracted from the local transform
!     2:
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!     5:  Not yet in use:  set equal to 0
!     6:  rho
!     7:  mu
!     8:  gamma
!     9:  t1
!    10:  t2
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm915' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm915'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call el0915 ( E%iuser, E%user, jelp, isub, isup, array, induv )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )
         E%vec = 0d0
         E%mat = 0d0
         if ( jelp==-10 ) then

!        --- jelp = -10, point element:  fill tangential direction in thead

            thead(1) = array(4)
            thead(2) = array(5)

         end if
         ipphi = 19
         ipphider = ipphi+n*m
         ipend = ipphider+9
         if ( ipend>50 ) then

!        --- array wrk1 too small

            call errint ( ipend, 1 )
            call errint ( 28, 2 )
            call errchr ( 'array', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

         if ( debug ) write(irefwr,1) 'jelp, isub, isup', jelp, isub, isup

      end if

!     --- compute basis functions and weights for numerical integration

      if ( jelp==2 ) then
         call elp616 ( coor, x, w, dphidx, xgauss, array(ipphi), &
                       index1, qmat, tang, array(ipphider) )
      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill vecold if necessary

      if ( ierror/=0 ) go to 1000
      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el3007 ( u, dudx, array(ipphi), dphidx, m, 3, &
                       ndimlc, 2, induv, work, ugauss, &
                       jtrans, numold, iseqin )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, array(ipphi), n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, array, x, xgauss, &
                          m, n, array(ipphi), 1, index1, &
                          E%iuser, ugauss, vecloc )

         end if

      end do

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         if ( jelp==2 ) then

!        --- quadratic line element

            call el915vec ( m, n, tang, E%vec, dphidx, &
                            array(1+2*m), w )

         else if ( jelp<=0 ) then

!        --- point element

            E%vec(1) = array(3)*thead(1)
            E%vec(2) = array(3)*thead(2)
            E%vec(3) = 0d0

         end if

      end if

!     --- Fill element matrix if necessary

      if ( matrix .and. notmat/=1 ) then

!     --- matrix = true, compute element matrix

         if ( jelp==2 ) then

!        --- quadratic line element

            call el915mat ( ugauss, array(1+3*m), m, n, tang, E%mat, &
                            array(ipphi), dphidx, array(1), array(1+m), &
                            array(1+2*m), w, dudx )

         else

!        --- point element

            E%mat(3,3) = -(thead(1)*u(1,1)+thead(2)*u(1,2))

         end if

      end if

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         call prinrl(E%vec,icount,'elemvc')
         call prinrl1(E%mat,icount,icount,'elemmt')

      end if
      call erclos ( 'elm915' )

      end subroutine elm915
