      subroutine elm998
! ======================================================================
!
!        programmer    Bas van Rens
!        version  3.2  date 12-12-2012 Long integers
!        version  3.1  date 12-08-2005 Debug statements
!        version  3.0  date 04-07-2005 Replace body by call to subroutines
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the element stiffness matrix of a friction element
!     use the Brekelmans approach
! **********************************************************************
!
!                       KEYWORDS
!
!     friction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ndimloc, istar, stat, prstat
      double precision u(3), f(3), normalvc(3), mu, &
             sumf, usc(3), sumt
      double precision, pointer :: vecoldlc(:)

!     f              Reaction force vector
!     i              Counting variable
!     istar          Starting address of the reaction force
!     j              Counting variable
!     mu             viscosity
!     normalvc       Normal vector
!     ndimloc        Dimension of the space.
!     prstat         Previous status
!     stat           Current status
!     sumf           Inner product
!     sumt           Length of a vector
!     u              Velocity vector or tangential force
!     usc            Vector used to compute the tangential directions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMFRICT2D     compute the element stiffness matrix of a
!                    friction element (2D)
!     ELMFRICT3D     compute the element stiffness matrix of a
!                    friction element (3D)
!     ELMFRICT3DTR   compute the element stiffness matrix of a
!                    friction element (3D)
!                    Special case with local transformations
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INSTOP         Stop the program
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm998' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm998'
         write(irefwr,*) 'itype, icount, inpelm ', &
                          itype, icount, inpelm

      end if

      if ( icount==5 ) then
         istar = 3
         ndimloc = 2
      else if ( icount==7 ) then
         istar = 4
         ndimloc = 3
      else
         write(*,*)'ERROR: Element not used correctly'
         call instop
      end if

      do i=1,icount
         do j=1,icount
            elemmt(j,i)= 0d0
         end do
         elemvc(i) = 0d0
      end do

      iseqtype = oldsols(1)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(1)%oldvecs
      u(1:ndimloc) = vecoldlc(index3n(1:ndimloc))
      f(1:ndimloc) = vecoldlc(index3n(istar:istar+ndimloc))
      iseqtype = oldsols(4)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(4)%oldvecs
      normalvc(1:ndimloc) = vecoldlc(index3n(1:ndimloc))
      usc(1:ndimloc) = 0d0
      iseqtype = oldsols(5)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(5)%oldvecs

      stat   = int(vecoldlc(index3n(1)))
      prstat = int(vecoldlc(index3n(2)))
      mu     = vecoldlc(index3n(3))
      prstat = mod(prstat,10)

      if (prstat>1) then

!     --- previous situation sliding, use velocity direction

         sumt = 0d0
         do i=1,ndimloc
            sumt = sumt + u(i)**2
         end do

         if ( sumt>1d-20 ) then
            sumt = 1d0/sqrt(sumt)
            do i=1,ndimloc
               usc(i) = u(i) * sumt
            end do
         end if

      end if

      if (prstat<=2) then

!     --- previous situation stick, or slip,  use negative tangential
!         force as well for the velocity direction. For prstat = 2
!         this may lead to flip-flop behavior. For this reason we
!         set usc = 0 when veldir = 1

!         --- Cartesian coordinates

         sumf = 0d0
         do i=1,ndimloc
            sumf = sumf + normalvc(i)*f(i)
         end do

         do i=1,ndimloc
            u(i) = -(f(i) - normalvc(i)*sumf)
         end do

         sumt = 0d0
         do i=1,ndimloc
            sumt = sumt + u(i)**2
         end do
         if (sumt>1d-20) then
            sumt = 10d0/sqrt(sumt)

!           --- scale the direction of the force

            do i=1,ndimloc
               usc(i) = usc(i) + u(i) * sumt
            end do
         end if
      end if

      if (ndimloc==2) then

!     --- Two-dimensional case

         call elmfrict2d ( elemmt, icount, istar, normalvc, stat, usc, mu )

      else

!     --- Three-dimensional case without local transformations

         call elmfrict3d ( elemmt, icount, istar, normalvc, stat, usc, mu )

      end if  ! (ndimloc==2)

!     --- fill the f,f part of element matrix diagonals in order to
!         avoid zero pivots

      do i=istar+1,icount
         elemmt(i,i)=elemmt(i,i)+1d-14
      end do

      if ( debug ) then

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         write(irefwr,*) 'stat, prstat ', stat, prstat
         call prinrl ( normalvc, 3, 'normalvc' )
         call prinrl(elemvc,icount,'elemvc')
         call prinrl1(elemmt,icount,icount,'elemmt')
         write(irefwr,*) 'End elm998'
      end if  ! ( debug )
      call erclos ( 'elm998' )

      end
