      subroutine elmfrict3d ( elemmt, icount, ist, n, stat, usc, mu )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-07-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the element stiffness matrix of a friction element (3D)
!     use the Brekelmans approach
! **********************************************************************
!
!                       KEYWORDS
!
!     friction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icount, ist, stat
      double precision elemmt(icount,icount), n(3), usc(3), mu

!     elemmt        i/o   Element matrix
!     icount         i    Number of unknowns in element
!     ist            i    Start of first Lagrange multiplier in element
!     mu             i    friction coefficient
!     n              i    normal vector
!     stat           i    Status of point
!     usc            i    Vector used to compute the tangential directions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t1(3), t2(3), ut1, ut2, sumt, lnu

!     lnu            Length of a vector
!     sumt           Length of a vector
!     t1             First tangential vector
!     t2             Second tangential vector
!     ut1            Component in first tangential direction
!     ut2            Component in second tangential direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FR0020         Subroutine to calculate tangential vectors
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmfrict3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmfrict3d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- This gives the effect of the reaction forces at the momentum
!         equations.

      elemmt(ist+1,1) = -1d0
      elemmt(ist+2,2) = -1d0
      elemmt(ist+3,3) = -1d0

!     --- compute two tangential vectors that are orthogonal to n

      call fr0020(n, t1, t2)

      if (stat==1 .or. stat==0) then

!     --- stick, so the extra equations model that all velocities
!         are zero

         elemmt(    1,ist+1) = 1d0
         elemmt(    2,ist+1) = 0d0
         elemmt(    3,ist+1) = 0d0

         elemmt(    1,ist+2) = 0d0
         elemmt(    2,ist+2) = 1d0
         elemmt(    3,ist+2) = 0d0

         elemmt(    1,ist+3) = 0d0
         elemmt(    2,ist+3) = 0d0
         elemmt(    3,ist+3) = 1d0

      else if (stat==2 .or. stat==5) then

!     --- slip, the tangential force is a constant times the normal
!         force. The normal velocity is zero.
!         slip : - ft  = mu ut/||ut|| |fn|

         ut1 = usc(1)*t1(1) + usc(2)*t1(2) + usc(3)*t1(3)
         ut2 = usc(1)*t2(1) + usc(2)*t2(2) + usc(3)*t2(3)

         sumt = sqrt((ut1*t1(1) + ut2*t2(1))**2 + &
                          (ut1*t1(2) + ut2*t2(2))**2 + &
                          (ut1*t1(3) + ut2*t2(3))**2)

         if ( sumt>1d-20 ) then

            lnu = 1d0 / sumt
            ut1 = ut1 * lnu
            ut2 = ut2 * lnu

         else

            ut1 = 1d0
            ut2 = 0d0

         end if

         elemmt(    1,ist+1) = n(1)
         elemmt(    2,ist+1) = n(2)
         elemmt(    3,ist+1) = n(3)

         elemmt(ist+1,ist+2) = -t1(1) + ut1*mu*n(1)
         elemmt(ist+2,ist+2) = -t1(2) + ut1*mu*n(2)
         elemmt(ist+3,ist+2) = -t1(3) + ut1*mu*n(3)

         elemmt(ist+1,ist+3) = -t2(1) + ut2*mu*n(1)
         elemmt(ist+2,ist+3) = -t2(2) + ut2*mu*n(2)
         elemmt(ist+3,ist+3) = -t2(3) + ut2*mu*n(3)

      else if (stat==3) then

!     --- release, the tangential force is zero and the normal
!         velocity is zero.

         elemmt(    1,ist+1) = n(1)
         elemmt(    2,ist+1) = n(2)
         elemmt(    3,ist+1) = n(3)

         elemmt(ist+1,ist+2) = -t1(1)
         elemmt(ist+2,ist+2) = -t1(2)
         elemmt(ist+3,ist+2) = -t1(3)

         elemmt(ist+1,ist+3) = -t2(1)
         elemmt(ist+2,ist+3) = -t2(2)
         elemmt(ist+3,ist+3) = -t2(3)

      else if (stat==4) then

!     --- release, the tangential forces are related to the tangential
!         velocity. The normalvelocity is zero. Is this value ever used?

         elemmt(    1,ist+1) = -t1(1)*mu
         elemmt(    2,ist+1) = -t1(2)*mu
         elemmt(    3,ist+1) = -t1(3)*mu
         elemmt(ist+1,ist+1) = -t1(1)
         elemmt(ist+2,ist+1) = -t1(2)
         elemmt(ist+3,ist+1) = -t1(3)

         elemmt(    1,ist+2) = -t2(1)*mu
         elemmt(    2,ist+2) = -t2(2)*mu
         elemmt(    3,ist+2) = -t2(3)*mu
         elemmt(ist+1,ist+2) = -t2(1)
         elemmt(ist+2,ist+2) = -t2(2)
         elemmt(ist+3,ist+2) = -t2(3)

         elemmt(    1,ist+3) = n(1)
         elemmt(    2,ist+3) = n(2)
         elemmt(    3,ist+3) = n(3)

      else

         write(*,*)'ERROR: this value of stat is not implemented'
         call instop

      end if

1000  call erclos ( 'elmfrict3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elmfrict3d'

      end if

      end
