      subroutine sepel900visc2d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!     Cartesian coordinates (2d)
!     Old subroutine sepelnsstrc2d
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: dmult(2), work(m), sw(n,n)
      double precision, pointer :: S(:,:), phix(:,:), phiy(:,:)

!     S              sub matrix
!     dmult          Array containing multiplication factors
!     i              Counting variable
!     j              Counting variable
!     phix           dphi/dx
!     phiy           dphi/dy
!     sw             sub matrix
!     work           Work array to store w eta
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 2 x 2 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S    |
!            |    uu      uv  |
!     S =    |                |
!            |   S       S    |
!            |    vu      vv  |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     S uu =  2  eta phix phix + eta phiy phiy + eta phiz phiz
!     S uv =     eta phix(j) phiy(i)
!     S vu =     eta phiy(j) phix(i)
!     S vv =  2  eta phiy phiy + eta phix phix + eta phiz phiz
!     In case of E%modelvisc = 10:
!     S uu =  eta phix phix + eta phiy phiy + eta phiz phiz = S vv
!
!    In case of compressible flow the stress term is changed from
!
!    T = eta ( grad v + grad v^T ) into
!
!    T = eta ( grad v + grad v^T ) - 2/3 eta div v I
!
!    with v the velocity
!
!    This means in the Cartesian case that the following extra terms are added:
!
!             |   phi_jx phi_ix   phi_jy phi_ix   |
!   -2/3 eta  |                                   |
!             |   phi_jx phi_iy   phi_jy phi_iy   |
!    The diagonal terms can be treated by multiplying by 4/3 (=2-2/3)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900visc2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900visc2d'
         write(irefwr,1) 'typecontinuity, modelvisc', &
                          E%typecontinuity, E%modelvisc
         call prinrl ( E%etaeff, m, 'etaeff' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      s => E%submat

      dmult(2) = 0d0
      if ( E%modelvisc==10 ) then

!     --- Pure incompressible flow, apply divergence freedom
!         simplified expression

         dmult(1) = 1d0

      else

!     --- Pure incompressible flow, apply divergence freedom

         dmult(1) = 2d0

      end if  ! ( E%modelvisc==10 )
      if ( E%typecontinuity==1 .or. E%typecontinuity==4 ) then

!     --- compressible flow

         dmult(1) = 4d0/3d0
         dmult(2) = -0.5d0*dmult(1)

      end if  ! ( E%typecontinuity==1 .or. E%typecontinuity==4 )

      if ( debug ) call prinrl ( dmult, 2, 'dmult' )

!     --- eta phix phix

      work = B%w*E%etaeff

      if ( debug ) call prinrl ( work, m, 'work' )

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phix(i,:)*phix(j,:))
         end do
      end do

!     --- Put in S_uu and S_vv

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = dmult(1)*sw(i,j)
            s(2*i,2*j) = sw(i,j)
         end do
      end do

!     --- eta phiy phiy

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phiy(i,:)*phiy(j,:))
         end do
      end do

!     --- Add to S_uu and S_vv

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(i,j)
            s(2*i,2*j) = s(2*i,2*j)+dmult(1)*sw(i,j)
         end do
      end do

      if ( E%modelvisc/=10 ) then

!     --- S vu =  eta phi_jx phi_iy

         do i = 1, n
            do j = 1, n
               sw(j,i) = sum(work*phix(i,:)*phiy(j,:))
            end do
         end do

!        --- Put in S_uv

         do i = 1, n
            do j = 1, i
               s(2*i,2*j-1) = sw(j,i)
               s(2*i-1,2*j) = sw(i,j)
            end do
         end do

         if ( E%typecontinuity==1 .or. E%typecontinuity==4 ) then

!        --- compressible flow -2/3 eta phi_ix phi_jy

            do i = 1, n
               do j = 1, n
                  sw(j,i) = sum(work*phiy(i,:)*phix(j,:))
               end do
            end do

            do i = 1, n
               do j = 1, i
                  s(2*i,2*j-1) = s(2*i,2*j-1)+sw(j,i)*dmult(2)
                  s(2*i-1,2*j) = s(2*i-1,2*j)+sw(i,j)*dmult(2)
               end do
            end do

         end if  ! ( E%typecontinuity==1 .or. E%typecontinuity==4 )

      end if  ! ( E%modelvisc/=10 )

!     --- Fill upper half of matrix

      do i = 1, 2*n
         do j = 1, i
            s(j,i) = s(i,j)
         end do  ! j = 1, i
      end do  ! i = 1, 2*n

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900visc2d'

      end if  ! ( debug )
      call erclos ( 'sepel900visc2d' )

      end subroutine sepel900visc2d
