      subroutine sepel900viscax
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!     Axisymmetric coordinates
!     Old subroutine sepelnsstraxi
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: work(m), sw(n,n), work1(m), h
      double precision, pointer :: S(:,:), phir(:,:), phiz(:,:), &
                                   xgauss(:), phi(:,:)

!     S              sub matrix
!     h              Help variable to store a constant
!     i              Counting variable
!     j              Counting variable
!     phi            values of shape functions in quadrature points
!     phir           dphi/dr
!     phiz           dphi/dz
!     sw             sub matrix
!     work           Work array to store w eta
!     work1          Work array
!     xgauss         gauss co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 2 x 2 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S     |
!            |    uu      uv   |
!     S =    |                 |
!            |   S       S     |
!            |    vu      vv   |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     2D Cylinder co-ordinates (r, z) without swirl
!     S uu =  2 eta phir phir + eta phiz phiz + 2 eta phi phi /r**2
!     S uv =    eta phir(j) phiz(i)
!     S vu =    eta phiz(j) phir(i)
!     S vv =  2 eta phiz phiz + eta phir phir
!
!    In case of compressible flow the stress term is changed from
!
!    T = eta ( grad v + grad v^T ) into
!
!    T = eta ( grad v + grad v^T ) - 2/3 eta div v I
!
!    with v the velocity
!
!    This means that the following extra terms are added:
!
!    -2/3 eta div(phi(j)) div(phi(i))
!
!    div (v) = d(r v_r)/rdr + dv_z/dz = 1/r*dv_r/dr + v_r + dv_z/dz
!
!
!             | G_j G_i      phi_jy G_i     |
!   -2/3 eta  |                             |
!             | G_j phi_iy   phi_jy phi_iy  |
!
!   with G = phi_ix + phi_i/r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900viscax' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900viscax'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phi => B%phi(:,:)
      phir => B%phider(:,:,1)
      phiz => B%phider(:,:,2)
      xgauss => B%xyzg(:,1)
      s => E%submat

      work = B%w*E%etaeff

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phir(i,:)*phir(j,:))
         end do
      end do

!     --- Put in S_rr and S_zz

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = 2d0*sw(i,j)
            s(2*i,2*j) = sw(i,j)
         end do
      end do

!     --- 2 eta phi phi / r**2

      work1 = 2d0*work/xgauss**2
      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work1*phi(i,:)*phi(j,:))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(i,j)
         end do
      end do

!     --- eta phiz phiz

      if ( E%typecontinuity==1 ) then
         h = 4d0/3d0
      else
         h = 2d0
      end if  ! ( E%typecontinuity==1 )

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phiz(i,:)*phiz(j,:))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(i,j)
            s(2*i,2*j) = s(2*i,2*j)+h*sw(i,j)
         end do
      end do

!     --- S uv =  eta phir phiz

      do i = 1, n
         do j = 1, n
            sw(j,i) = sum(work*phir(i,:)*phiz(j,:))
         end do
      end do

!     --- Put in S_uv

      do i = 1, n
         do j = 1, i
            s(2*i,2*j-1) = sw(j,i)
            s(2*i-1,2*j) = sw(i,j)
         end do
      end do

      if ( E%typecontinuity==1 ) then

!     --- compressible flow -2/3 eta div U div V
!
!             | G_j G_i      phi_jy G_i     |
!   -2/3 eta  |                             |
!             | G_j phi_iy   phi_jy phi_iy  |
!
!   with G = phi_ix + phi_i/r

         work = -2d0/3d0*work
         do i = 1, n
            do j = 1, i
               sw(j,i) = sum(work*(phir(j,:)+phi(j,:)/xgauss)*&
                                  (phir(i,:)+phi(i,:)/xgauss) )
            end do
         end do
         do i = 1, n
            do j = 1, i
               s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(j,i)
            end do
         end do

         do i = 1, n
            do j = 1, n
               sw(j,i) = sum(work*(phir(j,:)+phi(j,:)/xgauss)*&
                                   phiz(i,:) )
            end do
         end do

         do i = 1, n
            do j = 1, i
               s(2*i,2*j-1) = s(2*i,2*j-1)+sw(j,i)
               s(2*i-1,2*j) = s(2*i-1,2*j)+sw(i,j)
            end do
         end do

      end if  ! ( E%typecontinuity==1 )

!     --- Fill upper half of matrix

      do i = 1, 2*n
         do j = 1, i
            s(j,i) = s(i,j)
         end do  ! j = 1, i
      end do  ! i = 1, 2*n

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900viscax'

      end if  ! ( debug )
      call erclos ( 'sepel900viscax' )

      end subroutine sepel900viscax
