      subroutine sepel900viscpl
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!     Polar coordinates
!     Old subroutine sepelnsstrpol
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: work1(m), sw(n,n), work(m)
      double precision, pointer :: S(:,:), phir(:,:), phiz(:,:), &
                                   xgauss(:), phi(:,:)

!     S              sub matrix
!     i              Counting variable
!     j              Counting variable
!     phi            values of shape functions in quadrature points
!     phir           dphi/dr
!     phiz           dphi/dz
!     sw             Work array in which a sub element matrix is stored
!     work           Work array to store w eta
!     work1          Work array
!     xgauss         gauss co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2985   combination of compressible flow and non-Cartesian coordinates
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 2 x 2 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S     |
!            |    uu      uv   |
!     S =    |                 |
!            |   S       S     |
!            |    vu      vv   |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     2D Polar co-ordinates (r, z)   (in this case z is actually the angle)
!     S uu = eta ( 2 phir phir + phiz phiz/r**2 + 2 phi phi /r**2 )
!     S uv =     eta (phir(j) phiz(i)/r + 2 phiz(j) phi(i)/r**2 - &
!                      phi(j) phiz(i)/r**2)
!     S vu =     eta (phiz(j)/r phir(i) + 2 phi(j) phiz(i)/r**2 - &
!                      phiz(j) phi(i)/r**2)
!     S vv =  eta ( 2 phiz phiz/r**2 + phir phir + phi phi /r**2 - &
!                    phi(i) phir(j)/r - phi(j) phir(i)/r )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900viscpl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900viscpl'
         write(irefwr,1) 'mcont', E%typecontinuity
         call prinrl ( E%etaeff, m, 'etaeff' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phi => B%phi(:,:)
      phir => B%phider(:,:,1)
      phiz => B%phider(:,:,2)
      xgauss => B%xyzg(:,1)
      s => E%submat

      if ( E%typecontinuity==1 .or. E%typecontinuity==4 ) then

!     --- The combination compressible flow and polar coordinates is not
!         implemented

         call errint ( E%typecontinuity, 1 )
         call errint ( B%cart, 2 )
         call errsub ( 2985, 2, 0, 0 )

      end if  ! ( E%typecontinuity==1 .or. E%typecontinuity==4 )

!     --- 2D polar co-ordinates
!         eta phir phir

      work = B%w*E%etaeff

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phir(i,:)*phir(j,:))
         end do
      end do

!     --- Put in S_rr and S_zz

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = 2d0*sw(i,j)
            s(2*i,2*j) = sw(i,j)
         end do
      end do

!     --- eta ( - phi(i) phir(j) - phi(j) phir(i) )/ r

      work1 = work/xgauss

      do i = 1, n
         do j = 1, n
            sw(i,j) = sum(work1*phi(i,:)*phir(j,:))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i,2*j) = s(2*i,2*j)-sw(i,j)
            s(2*i,2*j) = s(2*i,2*j)-sw(j,i)
         end do
      end do

!     --- S uv
!         eta phir phiz/r

      do i = 1, n
         do j = 1, n
            sw(j,i) = sum(work1*phir(i,:)*phiz(j,:))
         end do
      end do

!     --- Put in S_uv

      do i = 1, n
         do j = 1, i
            s(2*i,2*j-1) = s(2*i,2*j-1)+sw(j,i)
            s(2*i-1,2*j) = s(2*i-1,2*j)+sw(i,j)
         end do
      end do

!     --- eta phi phi / r**2

      work1 = work/xgauss**2
      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work1*phi(i,:)*phi(j,:))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+2d0*sw(i,j)
            s(2*i,2*j) = s(2*i,2*j)+sw(i,j)
         end do
      end do

!     --- eta phiz phiz / r**2

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work1*phiz(i,:)*phiz(j,:))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(i,j)
            s(2*i,2*j) = s(2*i,2*j)+2d0*sw(i,j)
         end do
      end do

!     --- 2 eta phi(i) phiz(j)/r**2

      do i = 1, n
         do j = 1, n
            sw(j,i) = sum(work1*phi(i,:)*phiz(j,:))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i,2*j-1) = s(2*i,2*j-1)+2d0*sw(i,j)
            s(2*i-1,2*j) = s(2*i-1,2*j)+2d0*sw(j,i)
            s(2*i,2*j-1) = s(2*i,2*j-1)-sw(j,i)
            s(2*i-1,2*j) = s(2*i-1,2*j)-sw(i,j)
         end do
      end do

!     --- Fill upper half of matrix

      do i = 1, 2*n
         do j = 1, i
            s(j,i) = s(i,j)
         end do  ! j = 1, i
      end do  ! i = 1, 2*n

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900viscpl'

      end if  ! ( debug )
      call erclos ( 'sepel900viscpl' )

      end subroutine sepel900viscpl
