      subroutine sepelgradv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute gradient tensor in case of Navier-Stokes
!     Old subroutine sepelnsgrad
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     gradient
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, nunkp
      double precision, pointer :: gradv(:,:), dudx(:,:,:), x(:), u(:,:)

!     dudx           derivatives of the solution vector in the quadrature
!                    point
!     gradv          Array containing the gradient vector in
!                    the quadrature points
!     k              Counting variable
!     nunkp          Maximum number of unknowns in a point
!     u              velocity
!     x              x co-ordinates of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3043   Value of jcart not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The gradient is filled as follows
!     Cartesian coordinates:
!       2d: du/dx, dv/dx, du/dy, dv/dy
!       3d: du/dx, dv/dx, dw/dx, du/dy, dv/dy, dw/dy, du/dz, dv/dz, dw/dz
!     Axisymmetric coordinates without swirl:
!           du/dr, dv/dr, du/dz, dv/dz, u/r; with u = u_r and v = u_z
!     3d cylinder coordinates:
!           du/dx, dv/dr, dw/dr, du/dz, dv/dz, dw/dz,
!           (du/dphi-v)/r, (dv/dphi+u)/r, (dw/dphi)/r
!           with u = u_r, v= u_phi and w = u_z
!     Polar coordinates:
!           du/dr, dv/dr, (du/dphi-v)/r, (dv/dphi+u)/r
!           with u = u_r and v = u_phi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelgradv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelgradv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nunkp = oldsols(iseqprevsol)%nunkp
      gradv => E%gradv
      dudx => gradoldvec
      x => B%xyzg(:,1)
      u => E%ugauss

      select case ( B%cart )

      case(1)

!     --- Cartesian co-ordinates

         if ( E%ndim==2 ) then

!        --- E%ndim = 2

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,2,1)
            gradv(4,:) = dudx(1:m,2,2)
            if ( debug ) call prinrl1 ( gradv, m, 4, 'gradv' )

         else

!        --- E%ndim = 3

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,1,3)
            gradv(4,:) = dudx(1:m,2,1)
            gradv(5,:) = dudx(1:m,2,2)
            gradv(6,:) = dudx(1:m,2,3)
            gradv(7,:) = dudx(1:m,3,1)
            gradv(8,:) = dudx(1:m,3,2)
            gradv(9,:) = dudx(1:m,3,3)

         end if  ! ( E%ndim==2 )

      case(2)

!     --- Cylinder co-ordinates

         if ( E%ndim==2 ) then

!        --- E%ndim = 2

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,2,1)
            gradv(4,:) = dudx(1:m,2,2)
            do k = 1, m
               if ( abs(x(k))>epsmac ) then
                  gradv(5,k) = u(k,1)/x(k)
               else
                  gradv(5,k) = dudx(k,1,1)
               end if
            end do
            if ( debug ) call prinrl1 ( gradv, m, 9, 'gradv' )

         else

!        --- E%ndim = 3, store in sequence r, z, phi
!                      It is assumed that the solution is stored in the
!                      sequence: ur, uz, uphi
!                      Remark: in the old version it is stored in the
!                              sequence r, phi, z
!                      the co-ordinate system should be in the same sequence

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,1,3)
            gradv(4,:) = dudx(1:m,2,1)
            gradv(5,:) = dudx(1:m,2,2)
            gradv(6,:) = dudx(1:m,2,3)
            do k = 1, m
               if ( abs(x(k))>epsmac ) then
                  gradv(7,k) = (dudx(k,3,1)-u(k,3))/x(k)
                  gradv(8,k) = dudx(k,3,2)/x(k)
                  gradv(9,k) = (dudx(k,3,3)+u(k,1))/x(k)
               else
                  gradv(7,k) = -dudx(k,1,3)
                  gradv(8,k) = 0d0
                  gradv(9,k) = dudx(k,1,1)
               end if
            end do

         end if  ! ( E%ndim==2 )

      case(3)

!     --- Polar co-ordinates

         do k = 1, m
            gradv(1,k) = dudx(k,1,1)
            gradv(2,k) = dudx(k,1,2)
            if ( abs(x(k))>epsmac ) then
               gradv(3,k) = (dudx(k,2,1)-u(k,2))/x(k)
               gradv(4,k) = (dudx(k,2,2)+u(k,1))/x(k)
            else
               gradv(3,k) = -dudx(k,1,2)
               gradv(4,k) = dudx(k,1,1)
            end if
         end do

      case(4)

!     --- Cylindrical co-ordinates with swirl (2D),
!         store in sequence r, phi, z
!         It is assumed that the solution is stored in the sequence:
!         ur, uz, uphi

         gradv(1,:) = dudx(1:m,1,1)
         gradv(2,:) = dudx(1:m,1,2)
         gradv(3,:) = dudx(1:m,1,3)
         gradv(4,:) = dudx(1:m,2,1)
         gradv(5,:) = dudx(1:m,2,2)
         gradv(6,:) = dudx(1:m,2,3)
         do k = 1, m
            if ( abs(x(k))>epsmac ) then
               gradv(7,k) = -u(k,3)/x(k)
               gradv(9,k) = u(k,1)/x(k)
            else
               gradv(7,k) = -dudx(k,1,3)
               gradv(9,k) = dudx(k,1,1)
            end if
         end do

      case default

!     --- Value of B%cart not available for this subroutine

         call errint ( B%cart, 1 )
         call errsub ( 3043, 1, 0, 0 )

      end select  ! case ( B%cart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelgradv'

      end if  ! ( debug )
      call erclos ( 'sepelgradv' )

      end subroutine sepelgradv
