      subroutine sepelm900
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for the Navier-Stokes equations
!     penalty function approach
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     INSTOP         Stop the program
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPEL0900      Fill element independent quantities for element type 90
!     SEPEL900FINISH Deallocate arrays with respect to type 900
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDIVGRAD   Compute the divergence and gradient matrix
!                    for Navier-Stokes
!     SEPELEFFVISC   compute the effective viscosity depending on the
!                    viscosity model used
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELM900MASS  Fill element mass matrix for the incompressible
!                    Navier-Stokes equations
!     SEPELM900MAT   Fill element matrix for the incompressible
!                    Navier-Stokes equations
!     SEPELM900MATA  Assemble element matrix using submatrices created before
!     SEPELM900MATAL Store extra element matrices which are necessary for
!                    schur complement type methods
!     SEPELM900TRANS Compute the transformation matrix
!     SEPELM900VEC   Compute element vector for Navier-Stokes element
!     SEPELM900VECF  Create element vector for Navier-Stokes equations
!                    using separate subvectors
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELPHIPRESS  Fill array phipress with the pressure basis functions
!                    Navier-Stokes Crouzeix-Raviart elements
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     icoef 1 = type_time (0 standard; 1 integrated theta method)
!         This position consists of 2 parts:  jtime_old + 10*ioldm
!         Possible values for jtime_old
!         0: if imas in BUILD=0 then time-independent flow
!            else the mass matrix is built according to imas
!         1: The theta method is applied directly in the following way:
!            the mass matrix divided by theta delta is added to the stiffness
!            matrix
!            the mass matrix divided by theta delta and multiplied by u(n) is
!            added to the right-hand-side vector
!         ioldm indicates the sequence number of velocity at end of previous
!               time step in input vector.
!               If 0 no iterative procedure is adopted for dynamical
!               analyses.
!     icoef 2 = modelvisc  viscosity model as well as given stress
!                          tensor as right-hand side according to
!         modelv + 1000*model_rhsd
!         Possible values for modelv
!         0,1:  newtonian liquid
!         2:    power law liquid
!         3:    carreau liquid
!         4:    plastico viscous liquid
!         5:    special case: the viscous term is defined as
!               c_visc_1 d^2 u /dx^2 + c_visc_2 d^2 u /dy^2
!               c_visc_3 d^2 v /dx^2 + c_visc_4 d^2 v /dy^2
!        10:    special case: Newtonian fluid, with constant viscosity
!               incompressible
!               The viscous part is written as eta laplace u, &
!               hence the continuity equation is used to simplify
!               the expression
!               The natural boundary conditions are different from the
!               standard ones.
!       100:    user provided model depending on grad u
!       101:    user provided model depending on the second invariant
!       102:    user provided model depending on the second invariant, the
!               co-ordinates and the velocity
!         Possible values for model_rhsd:
!         0:    if there is a given stress tensor the reference to this
!               tensor is stored in position 19
!         1:    It is supposed that there are 6 given stress tensor components
!               stored in positions 19-24!     icoef 3 = irule (numerical quadrature)
!     icoef 4 = jcart (type of coordinate system)
!     icoef 5 = Information about the convective terms (mconv)
!         and continuity equation (mcont) according to
!         mconv+10*mcont+100*mcoef_conv
!         Possible values for mconv:
!         0: convective terms are skipped (Stokes flow)
!         1: convective terms are linearized by Picard iteration
!         2: convective terms are linearized by Newton iteration
!         3: convective terms are linearized by the incorrect Picard
!            iteration
!         4: convective terms are linearized by successive substitution
!         Possible values for mcont:
!         0: Standard situation, the continuity equation is div u = 0
!         1: A special continuity equation div ( rho u ) = 0, which rho
!            variable is solved. This is not the standard continuity equation
!            for incompressible flow. In fact we have a time-independent
!            compressible flow description.
!            This possibility can not be applied with the penalty function
!            approach
!         2: The so-called Goertler equations are solved
!         3: The continuity equation is equal to:
!            c_div_1 du/dx +  c_div_2 dv/dy = 0
!            The gradient of the pressure is multiplied by c_grad
!         4: the continuity equation contains a pressure term.
!         Possible values for mcoef_conv:
!         0: standard convective terms
!         1: special case: convective terms are defined by:
!            c_conv_1 u du/dx + c_conv_2 v du/dy
!            c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     coef  6 = penalty parameter eps
!     coef  7 = density rho
!     coef  8 = angular velocity
!     coef  9 = F1  (Body force in 1-direction)
!     coef 10 = F2  (Body force in 2-direction)
!     coef 11 = F3  (Body force in 3-direction)
!     coef 12 = viscosity parameter eta
!     coef 13 = parameter n (power in viscosity model)
!     coef 14 = parameter lambda in viscosity model
!     icoef 15 = imesh_velocity   (integer, refers to sequence number of
!                mesh-velocity)
!                If unequal to zero the convective term is adapted by
!                -u_mesh * grad(u)
!                In case of the Goertler equations imesh_velocity refers to
!                the velocity field U_0
!     coef 16 = G_L    Parameter for the Goertler equations
!     coef 17 = Parameter for the Goertler equations
!               Parameter for the pressure term in the continuity equation
!               (MCONT=4)
!     coef 18 = Right-hand side for continuity equation
!     coef 19 = stress: If model_rhsd = 0 then
!               Given stress tensor used in the right-hand side vector
!               At this moment it is supposed that the stress is defined
!               per element and that in each element all components are
!               constant
!               If model_rhsd = 1 then
!               The six components of the stress tensors are stored in
!               positions 19 to 24 in the order: xx, yy, zz, xy, yz, zx
!               This option can not be used in cooporation with the
!               dimensionless coefficients stored in positions 20-41
!     coef 20 = c_div_1
!     coef 21 = c_div_2
!     coef 22 = -
!     coef 23 = c_grad
!     coef 24-32 = c_conv 1 ... 9
!     coef 33-41 = c_visc 1 ... 9
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2989   Block preconditioners only available for type 902
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) then

!     --- First element

         call sepel0900

      else

!     --- Next element, restore coeffs

         E%coeffs = E%coeffcp

      end if  ! ( E%first==0 )
      if ( ierror/=0 ) go to 1000

!     --- Fill basis functions for the velocity

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill array phipress with basis functions for the pressure

      if ( npsi>1 ) call sepelphipress

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill variable coefficients if necessary

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

      if ( debug ) then
         call prinrl1 ( E%coeffsnode(:,2), 7, n, 'coeffnodes')
         call prinrl1 ( E%coeffs(:,2), 7, m, 'coeffs')
      end if  ! ( debug )

!     --- Fill divergence matrix, gradient matrix and pressure matrix

      call sepeldivgrad
      if ( debug ) then
         write(irefwr,1) 'npres, npsi', E%npress, npsi
         call prinrl2 ( E%gradmat, E%ndim*n, E%ndim+1, 'sp' )
         call prinrl2 ( E%divmat, E%ndim*n, E%ndim+1, 'div' )
      end if  ! ( debug )

!     --- Compute the transformation matrix
!         Compute the velocity in the centroid and the derivative of u

      if ( jtrans==1 ) call sepelm900trans

!     --- Compute effective viscosity in case of modelv # 1, 5, 10

      if ( E%useetaeff ) call sepeleffvisc

      if ( E%methodupw>0 ) then

!     --- Upwind method, compute upwind basis functions

         print *,'E%methodupw ', E%methodupw
         call ertrace
         call instop
   !       second = .true.
   !       call elm800upwbase ( metupw, jelp, E%coeffs(:indetha), &
   !                            ugauss, phi, psiupw, x, second, &
   !                            dphidx, dudx, iflipflop, dpsidx, w )

      end if  ! ( metupw>0 )
      if ( matrix .or. E%matelim ) then
         call sepelm900mat
         if ( debug ) &
            call prinrl2 ( E%submat, E%ndim*n, E%ndim*n, 'elemmtmp' )
      end if  ! ( matrix .or. E%matelim )
      if ( vector .and. notvec==0 ) call sepelm900vec
      if ( debug ) call prinrl ( E%subvec, E%ndim*n, 'elemvctmp' )

      if ( E%typetime>0 ) call sepelm900mass  ! Compute mass matrix

      if ( matrix ) call sepelm900mata

      if ( vector .and. notvec==0 ) call sepelm900vecf

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last

      if ( IM%simple ) then

!     --- Special case for Schur type elements
!         itype # 902, not permitted

         if ( itype/=902 ) then

!        --- itype # 902, not permitted

            call errint ( itype, 1 )
            call errsub ( 2989, 1, 0, 0 )
            go to 1000

         end if  ! ( itype/=902 )

         call sepelm900matal  ! Extra element matrices

      end if  ! ( itype/=902 )

!     --- Last call

      if ( E%last==1 ) call sepel900finish

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat ( E )
         call prinrl1(E%mmat,icount,icount,'elemms')
         write(irefwr,*) 'End sepelm900'

      end if  ! ( debug )
      call erclos ( 'sepelm900' )

      end subroutine sepelm900
