      subroutine sepelm900spv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the viscosity part of the element matrix for navier stokes
!     and store in matrix s for 2d elements
!     In this case the special form of the Navier-Stokes equations is used
!     based on the given coefficients for the various terms
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: work(m)
      double precision, pointer :: S(:,:), phix(:,:), phiy(:,:)

!     S              element submatrix
!     i              Counting variable
!     j              Counting variable
!     phix           dphi/dx
!     phiy           dphi/dy
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELPHIXPHIXA Fill part of element matrix
!                    / C dphi_i/dx_k dphi_j/dx_l dOmega
!                    Add to S-matrux
!                    Symmetric matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     Cartesian co-ordinates:
!     S uu =  eta_1 phix(j) phix(i) + eta_2 phiy(j) phiy(i)
!     S vv =  eta_3 phix(j) phix(i) + eta_4 phiy(j) phiy(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900spv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900spv'
         write(irefwr,1) 'B%cart, E%ndim, n, m', B%cart, E%ndim, n, m
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute matrices eta phix phix, eta phiy phiy and eta phiz phiz
!         and store in Suu, Svv and Sww

      if ( B%cart==1 .and. E%ndim==2 ) then

!     --- 2D Cartesian co-ordinates
!         eta_1 phix phix

         phix => B%phider(:,:,1)
         phiy => B%phider(:,:,2)
         S => E%submat(1:2*n:2,1:2*n:2)
         work = B%w*E%coeffs(:,E%indcvisc)
         call sepelphixphixa ( phix, phix, work, S )

!        --- eta_2 phiy phiy

         work = B%w*E%coeffs(:,E%indcvisc+1)
         call sepelphixphixa ( phiy, phiy, work, S )
         do i = 1, n
            do j = i+1, n
               S(i,j) = S(j,i)
            end do  ! j = i+1, n
         end do  ! i = 1, n

!        --- eta_3 phix phix

         S => E%submat(2:2*n:2,2:2*n:2)
         work = B%w*E%coeffs(:,E%indcvisc+2)
         call sepelphixphixa ( phix, phix, work, S )

!        --- eta_4 phiy phiy

         work = B%w*E%coeffs(:,E%indcvisc+3)
         call sepelphixphixa ( phiy, phiy, work, S )
         do i = 1, n
            do j = i+1, n
               S(i,j) = S(j,i)
            end do  ! j = i+1, n
         end do  ! i = 1, n

      end if  ! ( B%cart==1 .and. E%ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm900spv'

      end if  ! ( debug )
      call erclos ( 'sepelm900spv' )

      end subroutine sepelm900spv
