      subroutine sepelm900vecel
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Eliminate the centroid point for the incompressible Navier-Stokes
!     equations in case of an extended quadratic triangle
!     Add contribution of divergence right-hand side
!     Old subroutine el2201
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, n12, ncent
      double precision :: rhs(3)
      double precision, pointer :: s(:,:), rhsdivtr(:), trans(:,:), f(:)

!     f              sub element vector
!     i              Counting variable
!     k              Counting variable
!     n12            Number of nt to be eliminated velocity unknowns
!     ncent          Number of nt to be eliminated velocity unknowns
!     rhs            Work array to store the factor s22 rhsdivtr
!     rhsdivtr       Transformed divergence right-hand side (if mdiv=1)
!     s              Element matrix to be filled
!     trans          array containing the inverse of the transformation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                T
!      F = f  + R  f
!           1    1  2
!
!             new   1     T   2
!     Hence: f   = f   + R1  f
!             i     i      ik k
!
!     In case of mdiv = 1, the right-hand side must be updated by the following
!     term:
!
!
!               T             T
!     F = f  + R  f + ( S  + R   S   ) fd_tr ,
!          1    1  2     12   1   22
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900vecel' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900vecel'
         call prinrl1 ( E%subvec, n, E%ndim, 'elemvctmp' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      trans => E%transpress
      s => E%submat
      rhsdivtr => E%rhsdivtr
      f => E%subvec

      n12 = E%ndim*(n-1)

      if ( jtrans==1 ) then

!     --- Eliminate centroid in case of 2D quadratic elements
!         The resulting vector is stored in elemvc

         do i = 1, n12
            E%vec(i) = f(i) + sum(trans(i,:)*f(n12+1:n12+E%ndim))
         end do

      else if ( itype==902 .and. inpelm==27 ) then

!        --- 27-point element, integrated method

            E%vec(1:42) = f(1:42)
            E%vec(43:46) = 0d0
            E%vec(47:85) =f(43:81)

      end if

      if ( E%mdiv==1 ) then

!     --- mdiv = 1, divergence right-hand side

         if ( E%ndim==2 ) then

!        --- R2

            ncent = 2*(n-1)

!           --- First the term S22 * fd_tr is stored in rhs

            do k = 1, 2
               rhs(k) = sum(s(k+ncent,ncent+1:ncent+2)*rhsdivtr(1:2))
            end do  ! k = 1, 2

!           --- Next the complete term is added

            do i = 1, n12
               E%vec(i) = E%vec(i) + &
                          sum(s(i,ncent+1:ncent+2)*rhsdivtr(1:2)+&
                                trans(i,1:2)*rhs(1:2))
            end do

         else

!        --- R3

            ncent = 3*13
            do k = 1, 3
               rhs(k) = sum(s(ncent+1:ncent+3,k+ncent)*rhsdivtr)
            end do

!           --- Next the complete term is added

            do i = 1, ncent
               E%vec(i) = E%vec(i) + &
                          sum(s(i,ncent+1:ncent+3)*rhsdivtr+trans(i,:)*rhs)
            end do

            do i = 43, 81
               E%vec(i) = E%vec(i) + &
                          sum(s(i,ncent+1:ncent+3)*rhsdivtr+trans(i,:)*rhs)
            end do

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm900vecel'

      end if  ! ( debug )
      call erclos ( 'sepelm900vecel' )

      end subroutine sepelm900vecel
