      subroutine sepelnsgrad ( gradv, x, u, dudx, mdecl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute gradient tensor in case of Navier-Stokes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     gradient
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: mdecl
      double precision, intent(out) :: gradv(9,m)
      double precision, intent(in) :: x(mdecl,*), u(mdecl,*), &
                                      dudx(mdecl,ndimlc,*)

!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!     gradv          o    Array containing the gradient vector in
!                         the integration points
!     mdecl          i    Formal declaration parameter for the arrays (=m)
!     u              i    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3043   Value of jcart not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The gradient is filled as follows
!     Cartesian coordinates:
!       2d: du/dx, dv/dx, du/dy, dv/dy
!       3d: du/dx, dv/dx, dw/dx, du/dy, dv/dy, dw/dy, du/dz, dv/dz, dw/dz
!     Axisymmetric coordinates without swirl:
!           du/dr, dv/dr, du/dz, dv/dz, u/r; with u = u_r and v = u_z
!     3d cylinder coordinates:
!           du/dx, dv/dr, dw/dr, du/dz, dv/dz, dw/dz,
!           (du/dphi-v)/r, (dv/dphi+u)/r, (dw/dphi)/r
!           with u = u_r, v= u_phi and w = u_z
!     Polar coordinates:
!           du/dr, dv/dr, (du/dphi-v)/r, (dv/dphi+u)/r
!           with u = u_r and v = u_phi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsgrad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsgrad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      gradv = 0d0

      select case ( jcart )

      case(1)

!     --- Cartesian co-ordinates

         if ( ndimlc==2 ) then

!        --- ndimlc = 2

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,2,1)
            gradv(4,:) = dudx(1:m,2,2)
            if ( debug ) call prinrl1 ( gradv, m, 4, 'gradv' )

         else

!        --- ndimlc = 3

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,1,3)
            gradv(4,:) = dudx(1:m,2,1)
            gradv(5,:) = dudx(1:m,2,2)
            gradv(6,:) = dudx(1:m,2,3)
            gradv(7,:) = dudx(1:m,3,1)
            gradv(8,:) = dudx(1:m,3,2)
            gradv(9,:) = dudx(1:m,3,3)

         end if  ! ( ndimlc==2 )

      case(2)

!     --- Cylinder co-ordinates

         if ( ndimlc==2 ) then

!        --- ndimlc = 2

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,2,1)
            gradv(4,:) = dudx(1:m,2,2)
            do k = 1, m
               if ( abs(x(k,1))>epsmac ) then
                  gradv(5,k) = u(k,1)/x(k,1)
               else
                  gradv(5,k) = dudx(k,1,1)
               end if
            end do
            if ( debug ) call prinrl1 ( gradv, m, 9, 'gradv' )

         else

!        --- ndimlc = 3, store in sequence r, z, phi
!                      It is assumed that the solution is stored in the
!                      sequence: ur, uz, uphi
!                      Remark: in the old version it is stored in the
!                              sequence r, phi, z
!                      the co-ordinate system should be in the same sequence

            gradv(1,:) = dudx(1:m,1,1)
            gradv(2,:) = dudx(1:m,1,2)
            gradv(3,:) = dudx(1:m,1,3)
            gradv(4,:) = dudx(1:m,2,1)
            gradv(5,:) = dudx(1:m,2,2)
            gradv(6,:) = dudx(1:m,2,3)
            do k = 1, m
               if ( abs(x(k,1))>epsmac ) then
                  gradv(7,k) = (dudx(k,3,1)-u(k,3))/x(k,1)
                  gradv(8,k) = dudx(k,3,2)/x(k,1)
                  gradv(9,k) = (dudx(k,3,3)+u(k,1))/x(k,1)
               else
                  gradv(7,k) = -dudx(k,1,3)
                  gradv(8,k) = 0d0
                  gradv(9,k) = dudx(k,1,1)
               end if
            end do

         end if  ! ( ndimlc==2 )

      case(3)

!     --- Polar co-ordinates

         do k = 1, m
            gradv(1,k) = dudx(k,1,1)
            gradv(2,k) = dudx(k,1,2)
            if ( abs(x(k,1))>epsmac ) then
               gradv(3,k) = (dudx(k,2,1)-u(k,2))/x(k,1)
               gradv(4,k) = (dudx(k,2,2)+u(k,1))/x(k,1)
            else
               gradv(3,k) = -dudx(k,1,2)
               gradv(4,k) = dudx(k,1,1)
            end if
         end do

      case(4)

!     --- Cylindrical co-ordinates with swirl (2D),
!         store in sequence r, phi, z
!         It is assumed that the solution is stored in the sequence:
!         ur, uz, uphi

         gradv(1,:) = dudx(1:m,1,1)
         gradv(2,:) = dudx(1:m,1,2)
         gradv(3,:) = dudx(1:m,1,3)
         gradv(4,:) = dudx(1:m,2,1)
         gradv(5,:) = dudx(1:m,2,2)
         gradv(6,:) = dudx(1:m,2,3)
         do k = 1, m
            if ( abs(x(k,1))>epsmac ) then
               gradv(7,k) = -u(k,3)/x(k,1)
               gradv(9,k) = u(k,1)/x(k,1)
            else
               gradv(7,k) = -dudx(k,1,3)
               gradv(9,k) = dudx(k,1,1)
            end if
         end do

      case default

!     --- Value of jcart not available for this subroutine

         call errint ( jcart, 1 )
         call errsub ( 3043, 1, 0, 0 )

      end select  ! case ( jcart )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsgrad'

      end if  ! ( debug )
      call erclos ( 'sepelnsgrad' )

      end subroutine sepelnsgrad
