      subroutine el3015 ( g, gamma, rhsx, rhsg, w, phi, ni )
! ======================================================================
!
!        programmer    Martijn Cox & Niels Driessen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 12-12-2012 Long integers
!        version  1.0  date 07-04-2004
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute contribution to element right hand side, resulting from
!   contact conditions for general nonlinear solids.
!   At this moment, the subroutine has been tested only for
!   quadrilaterals and extended triangles.
!   Here it is assumed that the quantities (e.g. gamma, g, ni and dni) are
!   stored in the integration points. However, actually, they are
!   stored in the nodal points. This will not cause inconsistencies
!   because Newton-Cotes integration is used.
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision g(m), &
                       gamma(m), rhsx(n*ndim), rhsg(n), &
                       w(m), phi(n,m), ni(m,ndim)

!     g              i    array of length m in which the values of
!                         g in the integration points are stored
!     gamma          i    array with the contact pressures
!     ni             i    Normal of indentor
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     rhsg          i/o   Element vector to be filled
!     rhsx          i/o   Element vector to be filled
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision matrC(ndim,n*ndim), Cn(n*ndim)
      integer i, j, k, icont(m)
      double precision, pointer :: vecoldlc(:)

!     cn             Matrix for product C and normal
!     i              Counting variable
!     icont          Help array for contact
!     j              Counting variable
!     k              Counting variable
!     matrc          Matrix containing the shape functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Note: the nodal points and integration points coincide because
!          Newton-Cotes integration is used
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set rhs-vectors to zero and define icont based on vecold (i.e.
!         --- isolo(4))

      do i = 1, n*ndim
         rhsx(i) = 0d0
      end do
      iseqtype = oldsols(4)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(4)%oldvecs
      do i = 1, n
         rhsg(i) = 0d0
         icont(i) = int(vecoldlc(index3n(2*i-1)))
      end do

!     --- compute the integral

      do i = 1, m

!     --- clear matrices

         do j = 1, ndim
            do k = 1, n*ndim
               matrC(j,k) = 0d0
            end do
         end do

!        --- fill in the C matrix with shape functions in current integration
!            --- point

         do j = 1, n
            do k = 1, ndim
               matrC(k,(j-1)*ndim + k) = phi(j,i)
            end do
         end do

!        --- calculate C' * n

         do j = 1, n*ndim
            Cn(j) = 0d0
            do k = 1, ndim
               Cn(j) = Cn(j) + matrC(k,j)*ni(i,k)
            end do
         end do

!        --- add to rhsx using weight factors for current integration point

         do j = 1, n*ndim
            rhsx(j) = rhsx(j) + w(i) * gamma(i) * Cn(j)
         end do

!        --- add to rhsg using weight factors for current integration point

         do j = 1, n

!        --- contact

            if (icont(i)==1) then
               rhsg(j) = rhsg(j) + phi(j,i) * w(i) * g(i)

!              --- no contact

            else if (icont(i)==0) then
               rhsg(j) = rhsg(j) + phi(j,i) * w(i) * gamma(i)
            end if
         end do

      end do

      end
