      subroutine el8001 ( s, phix, phi, xgauss, elast, w, swork, work, &
                          anu )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-10-2010 new call to el3002
!        version  1.1  date 16-12-1994 New call to el3001
!        version  1.0  date 02-08-1994 Derived from el4915
!
!   copyright (c) 1994-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the stress part of the element matrix for rubber elements
!     and store in matrix s for 2d and 3-d elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     rubber_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phix(n,m,ndimlc), phi(n,m), elast(m), w(m), &
                       s(ndimlc*n,ndimlc*n), swork(n,n), work(*), &
                       xgauss(m,ndimlc), anu(m)

!     anu    i/o   Array containing the Poisson parameter nu in the integration
!     node
!     elast   i    Array containing the Elasticity modulus E in the integration
!     node
!     phi     i    array of length n * m, values of shape
!                  functions in integration points in the sequence:
!                         k
!                  phi (xg ) = phi (i,k)
!                     i
!                  array phi is only filled when gauss integration is applied
!     phix    i    Array of length n x m x 3  containing the derivatives
!                  of the basis functions in the sequence:
!                  d phi / dx (xg ) = phix (i,k,1);
!                       i        k
!                  d phi / dy (xg ) = phix (i,k,2);
!                       i        k
!                  d phi / dz (xg ) = phix (i,k,3);
!                       i        k
!                  If the element is a linear tetrahedron (n=4), then the
!                  derivatives are constant and only k = 1 is filled.
!     s      i/o   Element matrix to be filled
!     swork        Work array in which a sub element matrix is stored
!     w       i    array of length m containing the weights for integration
!     work    i    Work array
!     xgauss  i    array of length m x ndimlc containing the co-ordinates of
!                  the gauss points.  array xgauss is only filled when gauss
!                  integration is applied.
!                  xg  = xgauss (i,1);  yg  = xgauss (i,2);  zg  = xgauss (i,3)
!                    i                    i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical symm, incomp
      integer jzeros, idimsv, k
      double precision dmult(3), param(27)
      save incomp, dmult
      data incomp /.true./

!     dmult   Array containing multiplication factors
!     idimsv  help variable to save idim
!     incomp  If true the element is incompressible otherwise compressible
!     jzeros  help variable to save jzero
!     k       Counting variable
!     param   Array to store a parameter
!     symm    Indication whether the resulting matrix is symmetrical
!             (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3001  Compute integrals of the shape / u(x).grad phi phi (x) dx
!                                            e -            j   i
!     EL3002  Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                            e            i       j
!     EL3010  Fill small matrix elemmt into larger matrix s
!     EL3011  Compute contribution to element stiffness matrix corresponding to
!             a part of the second derivative part of an ellipticl equation
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     555   Poisson ratio nu (or nu1) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     Cartesian co-ordinates:
!     S uu =  2  elast phix phix + elast phiy phiy + elast phiz phiz
!     S uv =     elast phix(j) phiy(i)
!     S uw =     elast phix(j) phiz(i)
!     S vu =     elast phiy(j) phix(i)
!     S vv =  2  elast phiy phiy + elast phix phix + elast phiz phiz
!     S vw =     elast phiy(j) phiz(i)
!     S wu =     elast phiz(j) phix(i)
!     S wv =     elast phiz(j) phiy(i)
!     S ww =  2  elast phiz phiz + elast phix phix + elast phiy phiy
!     2D Cylinder co-ordinates (r, z) without swirl
!     S uu =  2 elast phir phir + elast phiz phiz + 2 elast phi phi /r**2
!     S uv =     elast phir(j) phiz(i)
!     S vu =     elast phiz(j) phir(i)
!     S vv =  2  elast phiz phiz + elast phir phir
!     2D Cylinder co-ordinates (r, z) with swirl
!     See without swirl, however with extra term:
!     S ww = elast ( psiz psiz + (psir(j) - psi(j)/r) (psir(i)-psi(i)/r) )
!     2D Polar co-ordinates (r, z)   (in this case z is actually the angle)
!     S uu = elast ( 2 phir phir + phiz phiz/r**2 + 2 phi phi /r**2 )
!     S uv =     elast (phir(j) phiz(i)/r + 2 phiz(j) phi(i)/r**2 - &
!                      phi(j) phiz(i)/r**2)
!     S vu =     elast (phiz(j)/r phir(i) + 2 phi(j) phiz(i)/r**2 - &
!                      phiz(j) phi(i)/r**2)
!     S vv =  elast ( 2 phiz phiz/r**2 + phir phir + phi phi /r**2 - &
!                    phi(i) phir(j)/r - phi(j) phir(i)/r )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute matrices elast phix phix, elast phiy phiy and elast phiz phiz
!         and store in Suu, Svv and Sww

      if ( ifirst==0 .or. ind(2)>0 ) then

!     --- First call or nu variable
!         Check nu and set incomp

         do 100 k = 1, m
            if ( anu(k)<=0d0 .or. anu(k)>0.5d0 ) then
               call erreal ( anu(k), 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 555, 3, 1, 0 )
            end if
            anu(k) = min ( anu(k), 0.49999d0 )
100      continue
         if ( ind(2)>0 ) then
            incomp = .false.
         else if ( anu(1)<0.499989d0 ) then
            incomp = .false.
         end if
      end if
      if ( jcart==1 .and. ndimlc==2 ) then

!     --- 2D Cartesian co-ordinates

         if ( incomp ) then

!        --- Pure incompressible, apply divergence freedom

            dmult(1) = 2d0/3d0
            dmult(2) = 1d0/3d0
            dmult(3) = dmult(2)
            do 110 k = 1, m
               param(k) = elast(k)
110         continue
         else

!        --- slightly compressible

            dmult(1) = 4d0/3d0
            dmult(2) = 1d0
            dmult(3) = 1d0/3d0
            do 120 k = 1, m
               param(k) = elast(k)/(2d0*(1d0+anu(k)))
120         continue
         end if

!        --- elast phix phix

         jadd = 0
         symm = .true.
         call el3011 ( param, swork, w, phix, m, n, 1, 1, work, &
                       symm )
         call el3010 ( s, swork, 2*n, n, 1, 1, 2, .false., dmult(1) )
         call el3010 ( s, swork, 2*n, n, 2, 2, 2, .false., dmult(2) )

!        --- elast phiy phiy

         jadd = 0
         symm = .true.
         call el3011 ( param, swork, w, phix, m, n, 2, 2, work, &
                       symm )
         jadd = 1
         call el3010 ( s, swork, 2*n, n, 1, 1, 2, .false., dmult(2) )
         call el3010 ( s, swork, 2*n, n, 2, 2, 2, .false., dmult(1) )

!        --- S uv =  elast phix phiy

         jadd = 0
         symm = .false.
         call el3011 ( param, swork, w, phix, m, n, 1, 2, work, &
                       symm )
         call el3010 ( s, swork, 2*n, n, 2, 1, 2, .true., dmult(3) )
         call el3010 ( s, swork, 2*n, n, 1, 2, 2, .false., dmult(3) )
      else if ( jcart==1 ) then

!     --- 3D Cartesian co-ordinates
!         --- elast phix phix

         jadd = 0
         symm = .true.
         call el3011 ( elast, swork, w, phix, m, n, 1, 1, work, &
                       symm )
         call el3010 ( s, swork, 3*n, n, 1, 1, 3, .false., 2d0 )
         call el3010 ( s, swork, 3*n, n, 2, 2, 3, .false., 1d0 )
         call el3010 ( s, swork, 3*n, n, 3, 3, 3, .false., 1d0 )

!        --- elast phiy phiy

         jadd = 0
         symm = .true.
         call el3011 ( elast, swork, w, phix, m, n, 2, 2, work, &
                       symm )
         jadd = 1
         call el3010 ( s, swork, 3*n, n, 1, 1, 3, .false., 1d0 )
         call el3010 ( s, swork, 3*n, n, 2, 2, 3, .false., 2d0 )
         call el3010 ( s, swork, 3*n, n, 3, 3, 3, .false., 1d0 )

!        --- elast phiz phiz

         jadd = 0
         symm = .true.
         call el3011 ( elast, swork, w, phix, m, n, 3, 3, work, &
                       symm )
         jadd = 1
         call el3010 ( s, swork, 3*n, n, 1, 1, 3, .false., 1d0 )
         call el3010 ( s, swork, 3*n, n, 2, 2, 3, .false., 1d0 )
         call el3010 ( s, swork, 3*n, n, 3, 3, 3, .false., 2d0 )

!        --- S uv =  elast phix phiy

         jadd = 0
         symm = .false.
         call el3011 ( elast, swork, w, phix, m, n, 1, 2, work, &
                       symm )
         call el3010 ( s, swork, 3*n, n, 2, 1, 3, .true., 1d0 )
         call el3010 ( s, swork, 3*n, n, 1, 2, 3, .false., 1d0 )

!        --- S uw =  elast phix phiz

         symm = .false.
         call el3011 ( elast, swork, w, phix, m, n, 1, 3, work, &
                       symm )
         call el3010 ( s, swork, 3*n, n, 3, 1, 3, .true., 1d0 )
         call el3010 ( s, swork, 3*n, n, 1, 3, 3, .false., 1d0 )

!        --- S vw =  elast phiy phiz

         symm = .false.
         call el3011 ( elast, swork, w, phix, m, n, 2, 3, work, &
                       symm )
         call el3010 ( s, swork, 3*n, n, 3, 2, 3, .true., 1d0 )
         call el3010 ( s, swork, 3*n, n, 2, 3, 3, .false., 1d0 )
      else if ( jcart==2 .and. ndimlc==2 .or. jcart==4 ) then

!     --- 2D Cylinder co-ordinates
!         --- elast phir phir

         jadd = 0
         symm = .true.
         call el3011 ( elast, swork, w, phix, m, n, 1, 1, work, &
                       symm )
         call el3010 ( s, swork, ncomp*n, n, 1, 1, ncomp, .false., 2d0 )
         call el3010 ( s, swork, ncomp*n, n, 2, 2, ncomp, .false., 1d0 )

!        --- 2 elast phi phi / r**2

         do 200 k = 1, m
            work(k) = 2d0*elast(k)/xgauss(k,1)**2
200      continue
         jadd = 0
         jzeros = jzero
         jzero  = 2
         call el3002 ( work, swork, w, phi, m, n, n )
         jzero  = jzeros
         jadd = 1
         call el3010 ( s, swork, ncomp*n, n, 1, 1, ncomp, .false., 1d0 )

!        --- elast phiz phiz

         jadd = 0
         symm = .true.
         call el3011 ( elast, swork, w, phix, m, n, 2, 2, work, &
                       symm )
         jadd = 1
         call el3010 ( s, swork, ncomp*n, n, 1, 1, ncomp, .false., 1d0 )
         call el3010 ( s, swork, ncomp*n, n, 2, 2, ncomp, .false., 2d0 )

!        --- S uv =  elast phir phiz

         jadd = 0
         symm = .false.
         call el3011 ( elast, swork, w, phix, m, n, 1, 2, work, &
                       symm )
         call el3010 ( s, swork, ncomp*n, n, 2, 1, ncomp, .true., 1d0 )
         call el3010 ( s, swork, ncomp*n, n, 1, 2, ncomp, .false., 1d0 )
      else

!     --- 2D polar co-ordinates
!         --- elast phir phir

         jadd = 0
         symm = .true.
         call el3011 ( elast, swork, w, phix, m, n, 1, 1, work, &
                       symm )
         call el3010 ( s, swork, ncomp*n, n, 1, 1, ncomp, .false., 2d0 )
         call el3010 ( s, swork, ncomp*n, n, 2, 2, ncomp, .false., 1d0 )

!        --- elast phi phi / r**2

         do 300 k = 1, m
            work(k) = elast(k)/xgauss(k,1)**2
300      continue
         jadd = 0
         jzeros = jzero
         jzero  = 2
         call el3002 ( work, swork, w, phi, m, n, n )
         jzero  = jzeros
         jadd = 1
         call el3010 ( s, swork, ncomp*n, n, 1, 1, ncomp, .false., 2d0 )
         call el3010 ( s, swork, ncomp*n, n, 2, 2, ncomp, .false., 1d0 )

!        --- elast phiz phiz / r**2

         do 310 k = 1, m
            work(k) = elast(k)/xgauss(k,1)**2
310      continue
         jadd = 0
         symm = .true.
         call el3011 ( work, swork, w, phix, m, n, 2, 2, work(m+1), &
                       symm )
         jadd = 1
         call el3010 ( s, swork, ncomp*n, n, 1, 1, ncomp, .false., 1d0 )
         call el3010 ( s, swork, ncomp*n, n, 2, 2, ncomp, .false., 2d0 )

!        --- elast ( - phi(i) phir(j) - phi(j) phir(i) )/ r

         do 320 k = 1, m
            work(k) = -elast(k)/xgauss(k,1)
320      continue
         jadd = 0
         idimsv = idim
         idim   = 1
         call el3001 ( work, swork, w, phi, phix, 1, n, work(m+1))
         jadd = 1
         call el3010 ( s, swork, ncomp*n, n, 2, 2, ncomp, .false., 1d0 )
         call el3010 ( s, swork, ncomp*n, n, 2, 2, ncomp, .true., 1d0 )

!        --- S uv
!            elast phir phiz/r

         do 330 k = 1, m
            work(k) = -work(k)
330      continue
         jadd = 0
         symm = .false.
         call el3011 ( work, swork, w, phix, m, n, 1, 2, work(m+1), &
                       symm )
         call el3010 ( s, swork, ncomp*n, n, 2, 1, ncomp, .true., 1d0 )
         call el3010 ( s, swork, ncomp*n, n, 1, 2, ncomp, .false., 1d0 )

!        --- 2 elast phi(i) phiz(j)/r**2

         do 340 k = 1, m
            work(k) = work(k)/xgauss(k,1)
340      continue
         jadd = 0
         call el3001 ( work, swork, w, phi, phix(1,1,2), 1, n, &
                       work(m+1) )
         jadd = 1
         call el3010 ( s, swork, ncomp*n, n, 2, 1, ncomp, .false., 2d0 )
         call el3010 ( s, swork, ncomp*n, n, 1, 2, ncomp, .true., 2d0 )

!        --- elast phi(j) phiz(i)/r**2

         call el3010 ( s, swork, ncomp*n, n, 2, 1, ncomp, .true., -1d0 )
         call el3010 ( s, swork, ncomp*n, n, 1, 2, ncomp, .false., -1d0)
         idim = idimsv
      end if
      end
