      subroutine el8100 ( e, anu, igprob, nd, mdecl, d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 19-11-2014 Debug statements
!        version  1.6  date 01-07-2014 Extension with igprob = 8
!        version  1.4  date 28-03-1996 Implementation of igprob = 6 and 8
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill elasticity matrix D for elasticity problem, depending on the
!     values of the elasticity E and the poisson ratio nu (anu)
!     plane stress, plane strain or axi-symmetric stress analysis
!     two and three-dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, nd, igprob
      double precision e(mdecl,*), anu(mdecl,*), d(nd,nd,mdecl)

!     anu     i   Array containing the values of the poisson ratio nu in
!                 the integration points, depending on IGPROB
!                 IGPROB = 1-3:  ANU(1,k) contains nu
!                 IGPROB = 4-6:  ANU(2,k) contains nu1
!                                ANU(3,k) contains nu2
!                                ANU(4,k) contains G2
!     d       o   Array containing the Elasticity matrix D
!                                 k
!                 D(i,j,k) = d  (x )
!                             ij
!     e       i   Array containing the values of the elasticity modulus E in
!                 the integration points, depending on IGPROB
!                 IGPROB = 1-3:  E(1,k) contains E
!                 IGPROB = 4-6:  E(1,k) contains E1
!                                E(2,k) contains E2
!                 IGPROB = 7:    E(1,k) contains D11
!                                E(2,k) contains D21
!                                E(3,k) contains D22
!                                E(4,k) contains D31
!                                E(5,k) contains D32
!                                E(6,k) contains D33
!                 IGPROB = 8:    E(1,k) contains D11
!                                E(2,k) contains D21
!                                E(3,k) contains D22
!                                E(4,k) contains D31
!                                E(5,k) contains D32
!                                E(6,k) contains D33
!                                E(7,k) contains D41
!                                E(8,k) contains D42
!                                E(9,k) contains D43
!                               E(10,k) contains D44
!     igprob  i   Type of stress-strain relations
!                 Possible values:
!                 1: plane stress, isotropic material (cartesian)
!                 2: plane strain, isotropic material (cartesian)
!                 3: axi-symmetric stress, isotropic material (axi-symmetric)
!                 4: plane stress, orthotropic material (cartesian)
!                 5: plane strain, orthotropic material (cartesian)
!                 6: axi-symmetric stress, orthotropic material
!                    (axi-symmetric)
!                 7: User defined elasticity matrix (3x3) (cartesian)
!                 8: User defined elasticity matrix (4x4) (axi-symmetric)
!     mdecl   i   Number of integration points in the element
!     nd      i   Dimension of elasticity matrix D
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision factor, fac1, fac2, facn

!     fac1           Help variable to compute a constant
!     fac2           Help variable to compute a constant
!     facn           Factor n defined by Zienkiewicz (G2/E2)
!     factor         Help variable to compute a constant
!     i              General loop variable
!     j              General loop variable
!     k              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The D matrix is computed
!   D depends on IGPROB:
!   igprob = 1:
!                       |   1   nu    0      |
!     D^b = E /(1-nu^2) |  nu    1    0      |
!                       |   0    0  (1-nu)/2 |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el8100' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el8100'
         write(irefwr,1) 'nd, mdecl, m, n, igprob', &
                          nd, mdecl, m, n, igprob
         call prinrl ( e, m, 'e' )
         call prinrl ( anu, m, 'anu' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 ) then

!     --- ifirst = 0  Set matrix D equal to zero

         do i = 1, nd
            do j = 1, nd
               do k = 1, m
                  d(i,j,k) = 0d0
               end do
            end do
         end do

      end if

      select case (igprob)
      case(1)

!     --- igprob = 1  plane stress isotropic

         if ( nd==3 ) then

!        --- ndim = 2

            do k = 1, m
               factor = e(k,1)/(1d0-anu(k,1)**2)
               fac1 = anu(k,1)*factor
               d(1,1,k) = factor
               d(2,2,k) = factor
               d(1,2,k) = fac1
               d(3,3,k) = 0.5d0*(1d0-anu(k,1))*factor
               d(2,1,k) = fac1
            end do

         else

!        --- ndim = 3

            do k = 1, m
               factor = e(k,1)*(1d0-anu(k,1))/ &
                        ((1d0+anu(k,1))*(1d0-2d0*anu(k,1)))
               fac1 = anu(k,1)*factor/(1d0-anu(k,1))
               fac2 = (1d0-2d0*anu(k,1))*factor/(2d0*(1d0-anu(k,1)))
               d(1,1,k) = factor
               d(2,2,k) = factor
               d(3,3,k) = factor
               d(1,2,k) = fac1
               d(1,3,k) = fac1
               d(2,1,k) = fac1
               d(2,3,k) = fac1
               d(3,1,k) = fac1
               d(3,2,k) = fac1
               d(4,4,k) = fac2
               d(5,5,k) = fac2
               d(6,6,k) = fac2
            end do

         end if
         case(2)

!        --- igprob = 2  plane strain isotropic

         do k = 1, m
            factor = e(k,1)*(1d0-anu(k,1))/((1d0+anu(k,1))* &
                     (1d0-2d0*anu(k,1)))
            fac1 = anu(k,1)*factor/(1d0-anu(k,1))
            d(1,1,k) = factor
            d(2,2,k) = factor
            d(1,2,k) = fac1
            d(3,3,k) = e(k,1)/(2d0*(1d0+anu(k,1)))
            d(2,1,k) = fac1
         end do
         case(3)

!        --- igprob = 3  axi-symmetric stress analysis   isotropic

         do k = 1, m
            factor = e(k,1)/((1d0+anu(k,1))*(1d0-2d0*anu(k,1)))
            fac1 = factor*(1d0-anu(k,1))
            fac2 = anu(k,1)*factor
            do i = 1, 3
               d(i,i,k) = fac1
               do j = 1, i-1
                  d(i,j,k) = fac2
                  d(j,i,k) = fac2
               end do
               d(4,4,k) = e(k,1)/(2d0*(1d0+anu(k,1)))
            end do
         end do
         case(4)

!        --- igprob = 4  plane stress orthotropic

         do k = 1, m
            facn = e(k,1)/e(k,2)
            factor = e(k,2)/(1d0-facn*anu(k,3)**2)
            d(1,1,k) = factor*facn
            d(2,2,k) = factor
            d(1,2,k) = factor*facn*anu(k,3)
            d(3,3,k) = anu(k,4)
            d(2,1,k) = d(1,2,k)
         end do
         case(5)

!        --- igprob = 5  plane strain orthotropic

         do k = 1, m
            facn = e(k,1)/e(k,2)
            factor = e(k,2)/((1d0+anu(k,2))* &
                             (1d0-anu(k,2)-2d0*facn*anu(k,3)**2))
            d(1,1,k) = factor*facn*(1d0-facn*anu(k,3)**2)
            d(2,2,k) = factor*(1d0-anu(k,2)**2)
            d(1,2,k) = factor*facn*anu(k,3)*(1d0+anu(k,2))
            d(2,1,k) = d(1,2,k)
            d(3,3,k) = anu(k,4)
         end do
         case(6)

!        --- igprob = 6  axi-symmetric stress analysis   stratified material

         do k = 1, m
            facn = e(k,1)/e(k,2)
            factor = e(k,2)/((1d0+anu(k,2))* &
                             (1d0-anu(k,2)-2d0*facn*anu(k,3)**2))
            d(1,1,k) = factor*(1d0-anu(k,2)**2)
            d(1,2,k) = factor*facn*anu(k,3)*(1d0+anu(k,2))
            d(1,3,k) = d(1,2,k)
            d(2,1,k) = d(1,2,k)
            d(3,1,k) = d(1,2,k)
            d(2,2,k) = factor*facn*(1d0-facn*anu(k,3)**2)
            d(3,3,k) = d(2,2,k)
            d(2,3,k) = factor*facn*(anu(k,2)+facn*anu(k,3)**2)
            d(3,2,k) = d(2,3,k)
            d(4,4,k) = anu(k,4)
         end do
         case(7)

!        --- igprob = 7  plane stress/strain User defined matrix

         do k = 1, m
            d(1,1,k) = e(k,1)
            d(1,2,k) = e(k,2)
            d(1,3,k) = e(k,4)
            d(2,1,k) = e(k,2)
            d(2,2,k) = e(k,3)
            d(2,3,k) = e(k,5)
            d(3,1,k) = e(k,4)
            d(3,2,k) = e(k,5)
            d(3,3,k) = e(k,6)
         end do
         case(8)

!        --- igprob = 8  axisymmetric User defined matrix

         do k = 1, m
            d(1,1,k) = e(k,1)
            d(1,2,k) = e(k,2)
            d(1,3,k) = e(k,3)
            d(1,4,k) = e(k,4)
            d(2,1,k) = e(k,2)
            d(2,2,k) = e(k,5)
            d(2,3,k) = e(k,6)
            d(2,4,k) = e(k,7)
            d(3,1,k) = e(k,3)
            d(3,2,k) = e(k,6)
            d(3,3,k) = e(k,8)
            d(3,4,k) = e(k,9)
            d(4,1,k) = e(k,4)
            d(4,2,k) = e(k,7)
            d(4,3,k) = e(k,9)
            d(4,4,k) = e(k,10)
         end do
         case(9)

!        --- igprob = 9  3d user defined matrix

         d(1,1,1:m) = e(1:m,1)
         d(1,2,1:m) = e(1:m,2)
         d(1,3,1:m) = e(1:m,3)
         d(1,4,1:m) = e(1:m,4)
         d(1,5,1:m) = e(1:m,5)
         d(1,6,1:m) = e(1:m,6)
         d(2,2,1:m) = e(1:m,7)
         d(2,3,1:m) = e(1:m,8)
         d(2,4,1:m) = e(1:m,9)
         d(2,5,1:m) = e(1:m,10)
         d(2,6,1:m) = e(1:m,11)
         d(3,3,1:m) = e(1:m,12)
         d(3,4,1:m) = e(1:m,13)
         d(3,5,1:m) = e(1:m,14)
         d(3,6,1:m) = e(1:m,15)
         d(4,4,1:m) = e(1:m,16)
         d(4,5,1:m) = e(1:m,17)
         d(4,6,1:m) = e(1:m,18)
         d(5,5,1:m) = e(1:m,19)
         d(5,6,1:m) = e(1:m,20)
         d(6,6,1:m) = e(1:m,21)
         do i = 2, 6
            do j = 1, i-1
               d(i,j,1:m) =  d(j,i,1:m)
            end do  ! j = 1, i-1
         end do  ! i = 2, 6

      end select  ! case (igprob)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el8100'

      end if  ! ( debug )
      call erclos ( 'el8100' )

      end subroutine el8100
