      subroutine eld250 ( iuser, user, coor2d, islold, vecloc, array, &
                          phi, phixi, psi, psixi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 06-01-2016 Use sepmoduleelem
!        version  5.1  date 26-06-2014 New calls to el8106/7
!        version  5.0  date 21-08-2013 Extra parameters
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill element derivative vector for linear elacticity element
!     plane stress, plane strain or axi-symmetric stress analysis
!     two-dimensional elements
!     ELD250 is a help subroutine for subroutine DERIV (TO0047)
!     it is called through the intermediate subroutine eldmec
!     So:
!     DERIV
!     TO0047
!       -  Loop over elements
!          -  ELDMEC
!             - ELD250
!    The subroutine is able to handle the following element shapes and cases:
!    linear triangle (2D and 3D)
!    bilinear quadrilateral (2D)
!    plane stress
!    plane strain
!    axi-symmetric stress
!    isotropic material
!    anisotropic material
!    user defined elasticity matrix
!    Type numbers:
!                 206  Plane stress, plane strain, axi-symmetric stress, &
!                      linear triangle
!                 207  Plane stress, plane strain, axi-symmetric stress, &
!                      bilinear quadrilateral
!                 250  General case
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elasticity_equation
!     element_vector
!     mechanics
!     strain
!     stress
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      integer mip
      parameter ( mip = 27 )
      double precision w, dum, d
      common /celwrk/ d(972), w(27), dum(501)
      save /celwrk/

!          The following parameters are used in CELWRK
!     D              Array containing the Elasticity matrix D
!                                    k
!                    D(i, j, k) = d  (x )
!                                  ij
!     dum            500 positions reserved for the ELP subroutines
!     w              array of length m containing the weights for the
!                    numerical integration
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), islold(*)
      double precision user(*), coor2d(*), vecloc(*), array(*), &
                       phi(*), phixi(*), psi(*), psixi(*)

!     array          Work array of length 8939 in which the values of the
!                    coefficients in the integration points are stored in
!                    the sequence:
!                     1-21:  Information about the elasticity matrix D
!                            or user defined parameters for IGPROB>=10
!                       22:  Thickness h
!                       23:  External Force in 1-direction
!                       24:  External Force in 2-direction
!                       25:  External Force in 3-direction
!                    26-31:  Initial strain
!                    32-37:  Initial stress
!                       38:  Density rho
!                     The storage of the various components depends on the
!                     type of model to be used.
!                     For the elasticity matrix we have:
!                     Isotropic material:
!                                    1:  E
!                                    2:  nu
!                                 3-21: Not used
!                     Orthotropic material:
!                                    1:  E1
!                                    2:  E2
!                                    3:  nu1
!                                    4:  nu2
!                                    5:  G2
!                                 6-21: Not used
!                     User defined 3*3 elasticity matrix:
!                                    1:  d11
!                                    2:  d12
!                                    3:  d22
!                                    4:  d31
!                                    5:  d32
!                                    6:  d33
!                                 7-21: Not used
!                     For the initial strain we have:
!                    26:  eps x             eps r
!                    27:  eps y             eps z
!                    28:  eps z             eps theta
!                    29:  gamma xy          gamma rz
!                    30:  gamma xz          - &
!                    31:  gamma yz          - &
!                     For the initial stress we have:
!                    32:  sigmalc x           sigmalc r
!                    33:  sigmalc y           sigmalc z
!                    34:  sigmalc z           sigmalc theta
!                    35:  tau xy            tau rz
!                    36:  tau xz
!                    37:  tau yz
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!                        The positions after 40*m = (1080) are used to store
!                        other large arrays, i.e.:  (mip=27)
!                        array name       starting address  length
!                        detf             ipdetf              mip
!                        qmat             ipqmat            3*12
!                        blwork           ipblwork          6*3*8*8=1152
!                        blwrkr           ipblwrkr          6*3*8=144
!                        xgauss           ipxgauss          81
!                        ugauss           ipugauss          81
!                        uold             ipuold            81
!                        dphidx           ipdphidx          2187
!                        b                ipb               128
!                        work             ipwork            783
!                        These subarrays have the following meaning:
!                        b          Array containing the B matrix
!                                   (See for example Zienkiewicz)
!                                   B(i,j,(i-1)*m+k) = b_ij phi_i(x^k)
!                        blwork     blmatrix: blwork(i,j,k,l)
!                                   i: corresponding stress/strain vector
!                                      postion
!                                   j: direction
!                                   k: node
!                                   l: integration point
!                        blwrkr     blmatrix for RI point
!                        detf       to store the determinant of dfgrad (for
!                                   incompressibility computations)
!                        dphidx     Array of length n x m x ndim  containing the
!                                   derivatives of the basis functions in the
!                                   sequence:
!                                   d phi / dx (xg ) = dphidx (i,k,1);
!                                        i        k
!                                   d phi / dy (xg ) = dphidx (i,k,2);
!                                        i        k
!                                   d phi / dz (xg ) = dphidx (i,k,3);
!                                        i        k
!                        glstrn     to store green-lagrange strains (reduced
!                                   storage due to symmetry )
!                                                      | e1 e4 e6 |    j
!                                   glstrn(i=1..6,j) = | e4 e2 e5 | ( x )
!                                                      | e6 e5 e3 |
!                        pksest     to store piola-kirchoff stresses of
!                                   estimated situation same format as: glstrn
!                        qmat       transformation matrix global ->
!                                   local coordinates
!                                   in case of a 3D element
!                        ugauss     Preceding solution in Gauss points
!                        xgauss     array of length m x ndim containing the
!                                   co-ordinates of the gauss points.
!                                   array xgauss is only filled when gauss
!                                   integration is applied.
!                                   xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                                   zg_i = xgauss (i,3)
!     coor2d           i    array of length ndim x npointcac containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, isub, isup, jelp, igprob, istres, istrai, &
              nd, jheat, ncounw, jcheld, indexi, k, ioption(10), &
              ipdetf, ipblwork, ipblwrkr, ipend, ipb, ipwork
      logical compd, elemdef
      double precision xpos(3), rotat(9,mip), matpar(21), &
                       xgausr(2), ugausr(2), glstrr(6), dfgrdr(9), &
                       detfr(1), pksr(6), wr(1), dphdxr(8*3), x(81), &
                       dudx(243), eps(162), sigmalc(162), dfgrad(3,3), &
                       dphidx(27*27*3), xgauss(3*27), &
                       qmat(3*3*27), tang(1), uold(3*27), &
                       ugauss(3*27)
      save isub, isup, jelp, igprob, istres, istrai, compd, nd, jcheld, &
           ioption, ncounw

!     compd          Indication if the matrix D must be computed in each call
!                    (true) or if it is constant and hence has to be computed
!                    once (false)
!     detfr          determinant of at centre
!     dfgrad         to store the deformation gradient
!                    dfgrad(k,j,i) = f_ij(x^k)
!                    this way of storage used to be compatible with:
!                    EL3007
!     dfgrdr         ?
!     dphdxr         ?
!     dphidx         Array of length n x m x ndim  containing the
!                    derivative
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemdef        If true the vector is defined per element
!     eps            Strain tensor
!     glstrr         ?
!     i              Counting variable
!     igprob         Type of stress-strain relations
!                    Possible values:
!                     1: plane stress, isotropic material
!                        or 3D, isotropic material (Cartesian)
!                     2: plane strain, isotropic material (cartesian)
!                     3: axi-symmetric stress, isotropic material(axi-symmetric)
!                     4: plane stress, anisotropic material (cartesian)
!                     5: plane strain, anisotropic material (cartesian)
!                     6: axi-symmetric stress, anisotropic material
!                        (axi-symmetric)
!                     7: User defined elasticity matrix (3x3) (cartesian)
!                     8: User defined elasticity matrix (4x4) (axi-symmetric)
!                     9: User defined elasticity matrix (6x6) (3d)
!                    11: Nonlinear
!                    12: Nonlinear with local fibre direction
!                    13: 12 + reduced integration of compressibility
!                    21: Muscle contraction (=12)
!     indexi         help variable used to remap the computed strains
!     ioption        Integer array to with options of this element:
!                    ioption(1) = inonlin
!                    ioption(2) = ispecial
!                    ioption(3) = user coefficient 4 -> passed to fnmateri
!                    ioption(4) = user coefficient 5
!                    ioption(5-10) unused
!     ipb            Starting address of array b in array
!     ipblwork       Starting address of array blwork in array
!     ipblwrkr       Starting address of array blwrkr in array
!     ipdetf         Starting address of array detf in array
!     ipend          Last position in array
!     ipwork         Starting address of array work in array
!     istrai         Indicates if initial strain must be taken into account (1)
!                    or not (0)
!     istres         Indicates if initial stress must be taken into account (1)
!                    or not (0)
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     j              Counting variable
!     jcheld         Adapted value of icheld
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                    3:  Subroutine ELP620  (Linear triangle)
!                    5:  Subroutine ELP621  (Bilinear quadrilateral)
!     jheat          Dummy parameter
!     k              Counting variable
!     matpar         Help array to store the material properties
!     ncounw         Defines number of unknowns per point in element vector
!     nd             dimension of elaticity matrix d
!     pksr           to store piola-kirchoff stresses of estimated situation
!                    for RI point
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rotat          Help array to store the local rotation
!     sigmalc          stress tensor
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     ugausr         ?
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     wr             weight of RI point
!     x              x co-ordinate of point
!     xgausr         ?
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
!     xpos           Help array to store position of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision fndetf

!     EL0250         Initialize parameters for subroutine ELM250
!     EL1005         fill of derivatives and weights in elemvc1 and elemwg
!                    with respect to subroutine ELD250
!     EL3007         Fills old solution and derivatives
!     EL3009         Fill series of old solutions in array vecloc
!     EL8100         Fill elasticity matrix D for elasticity problem, depending
!                    on the values of the elasticity E and the Poisson ratio nu
!                    (anu)
!     EL8105         Compute eps = B u in integration points
!     EL8106         Compute the tensor sigmalc
!     EL8107         Fill array eps into elemvc1
!     ELDF2GLS       Compute Green-Lagrange strains from deformation gradient
!     ELDU2DFG       Make deformation gradient from displacement gradient
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ELMKBL         Setup the linear strain-displacement matrix for
!                    nonlinear analysis.
!     ELNLAV         Assemble vector for nonlinear solid analysis
!     ELRI01         make reduced integration data for linear element
!     ELROTSYM       Rotate symmetrical matrix
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FNDETF         Function to fill element vector with f(detf)
!     FNLOCDIR       Determines rotation matrix
!     FNMATERI       Compute material behaviour
!     PRINRL1        Print 2d real vector
!     PRINRL3        Print 3d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  - &
!     2:  Type of stress-strain relations. This parameter is equal to IGPROB-1
!         Possible values:
!                    igprob = irelat + 10*inonlin + 100*ispecial
!                    Possible values:
!            ------------------------------------------------------------ &
!            inonlin = 0
!            ------------------------------------------------------------ &
!            irelat  0: plane stress, isotropic material
!                       or 3D stress, isotropic material (Cartesian)
!                    1: plane strain, isotropic material (Cartesian)
!                    2: axi-symmetric stress, isotropic material
!                                                        (axi-symmetric)
!                    3: plane stress, orthotropic material (Cartesian)
!                    4: plane strain, orthotropic material (Cartesian)
!                    5: axi-symmetric stress, orthotropic material
!                       (axi-symmetric)
!                    6: User defined elasticity matrix (3x3) (Cartesian)
!                    7: User defined elasticity matrix (4x4) (axi-symmetric)
!                    8: User defined elasticity matrix (6x6) (Cartesian)
!            ------------------------------------------------------------- &
!            inonlin>0
!            ------------------------------------------------------------- &
!            inonlin 1: Geometrical and physical nonlinear behaviour (3D)
!                    2: 1 + reduced integration of compressibility term
!                    3: nonlin + variable anisotropy
!                    4: 2 + reduced integration of compressibility term
!            irelat  0: 3D
!                    2: 2D axi symmetric
!            ------------------------------------------------------------- &
!            ispecial = 1: muscle contraction, requires ELMUSC
!                          contact TUE/WFW for info
!            ------------------------------------------------------------- &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type material model
!             ichoice parameter of fnmateri
!     5:  Extra options
!             should be zero (0)
!     6-26: Information about the elasticity matrix depending on the
!           value of IGPROB
!         IGPROB = 1, 2 or 3:
!            6:  E
!            7:  nu
!         IGPROB = 4 or 5:
!            6:  E1
!            7:  E2
!            8:  nu1
!            9:  nu2
!           10:  G2
!         IGPROB = 6:
!            6:  D11
!            7:  D21
!            8:  D22
!            9:  D31
!           10:  D32
!           11:  D33
!         IGPROB = 7:
!            6:  D11
!            7:  D21
!            8:  D22
!            9:  D31
!           10:  D32
!           11:  D33
!           12:  D41
!           13:  D42
!           14:  D43
!           15:  D44
!         IGPROB = 11,12,21:
!         6 t/m 15: user defined parameters
!    27:  h (thickness of the plate: 2D cartesian)
!    28:  F1  (Body force in 1-direction)
!    29:  F2  (Body force in 2-direction)
!    29:  F3  (Body force in 3-direction)
!    31:  epsx 0  (or epsr 0)             initial strain
!    32:  epsy 0  (or epsz 0)
!    33:  epsz 0  (or epstheta 0)
!    34:  gammaxy 0  (or gammarz 0)
!    35:  gammayz 0
!    36:  gammazx 0
!    37:  sigmalcx 0  (or sigmalcr 0)         initial stress
!    38:  sigmalcy 0  (or sigmalcz 0)
!    39:  sigmalcz 0  (or sigmalctheta 0)
!    40:  tauxy 0   (or taurz 0)
!    41:  tauyz 0
!    42:  tauzx 0
!    43:  rho (parameter for time-derivative)
!    44:  Not yet used (must be 0)
!    45:  Not yet used (must be 0)
!     Depending on the parameter icheld the following quantities are computed:
!     icheld          plane stress/strain            Axi-symmetric stress
!       1                        du        / d
!                                  jdegfd     ix
!       2                        grad u
!       3                        - grad u
!       4                        div u
!       5                        curl u
!                             | sigmalc x |                    | sigmalc r     |
!       6                     | sigmalc y |                    | sigmalc z     |
!                     sigmalc = | sigmalc z |            sigmalc = | sigmalc theta |
!                             | tau xy  |                    | tau rz      |
!                             | tau yz  |                    |   -         |
!                             | tau zx  |                    |   -         |
!       incase on NONLINEAR: function FNMATERI must be supplied
!                             | eps x    |                   | eps r     |
!       7                     | eps y    |                   | eps z     |
!                       eps = | eps z    |             eps = | eps theta |
!                             | gamma xy |                   | gamma rz  |
!                             | gamma yz |                   |   -       |
!                             | gamma zx |                   |   -       |
!       8               val =  f(detf)
!                              the user must supply user function FNDETF
!       9               RHS vector
!      12-16      See 2 to 6, now defined per element (type 116)
!      32-37      See 2 to 7, now defined per element per node (type 126)
!      42-47      See 2 to 7, now defined per element per integration point
!                 (type 129)
!     Structure of output arrays:
!     Solution array:  ndim unknowns per point (IVEC=0)
!     Arrays of special structure:
!     IVEC = 1:   1 unknowns per point
!     IVEC = 2:   2 unknowns per point
!     IVEC = 3:   3 unknowns per point
!     IVEC = 4:   6 unknowns per point
!     IVEC = 5:   ndim unknowns per point
!     IVEC = 6:   1 unknowns per vertex
!     IVEC = 7:   2 unknowns per vertex
!     IVEC = 8:   3 unknowns per vertex
!     IVEC = 9:   6 unknowns per vertex
!     IVEC =10:   ndim unknowns per vertex
!     IVEC =11:   1/3/6 unknowns per point
!     IVEC =12:   0/1/3 unknowns per point
!     IVEC =13:   1/4/9 unknowns per point
!     IVEC =14:   1/3/6 unknowns per vertex
!     IVEC =15:   0/1/3 unknowns per vertex
!     IVEC =16:   1/4/9 unknowns per vertex
!     ICHELD                 IVEC
!       1                      1
!       2                     13
!       3                     13
!       4                      1
!       5                     12
!       6                      4
!       7                      4
!       8                      1
!       9                      5
!      21                      6
!      22                     16
!      23                     16
!      24                      6
!      25                     16
!      26                      9
!      27                      9
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld250' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eld250'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         jcheld = icheld
         call el0250 ( iuser, user, jelp, jheat, igprob, istres, &
                       istrai, nd, compd, isub, isup, array, &
                       ioption )

!        --- Fill the D-matrix if this matrix is constant

         if ( igprob<=9 .and. .not. compd ) &
            call el8100 ( array, array(1+m), igprob, nd, m, d )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

!        --- Fill array elemwg

         if ( jcheld==1 .or. jcheld==4 .or. jcheld==8 ) then
            ncounw = 1
         else if ( jcheld==2 .or. jcheld==3 ) then
            ncounw = ndim**2
         else if ( jcheld==5 ) then
            if ( ndim==2 ) then
               ncounw = 1
            else
               ncounw = 3
            end if
         else if ( icheld==6  .or. icheld==7  .or. icheld==36 .or. &
                   icheld==37 .or. icheld==46 .or. icheld==47 ) &
           then
            ncounw = 6
         else if ( jcheld==6 ) then
            ncounw = 4
         else if ( jcheld==7 ) then
            ncounw = 3
         end if
         elemdef = (icheld>=12 .and. icheld<=16) .or. &
                   (icheld>=32 .and. icheld<=37) .or. &
                   (icheld>=42 .and. icheld<=47)
         if ( elemdef ) jcheld = mod(icheld,10)
         do i = 1, ncounw*m
            eps(i) = 0d0
            sigmalc(i) = 0d0
         end do
         if ( icheld<=40 ) then
            do i = 1, inpelm*ncounw
               elemvc1(i) = 0d0
            end do
            if ( .not. elemdef ) then
               do i = 1, inpelm*ncounw
                  elemwg(i) = 1d0
               end do
            end if
         end if
         if ( debug ) write(irefwr,1) 'm,n,jelp', m,n,jelp

      end if

!     --- Set pointers in arrays

      ipdetf   = 40*m
      ipblwork = ipdetf   + mip
      ipblwrkr = ipblwork + 1152
      ipb      = ipblwrkr + 144
      ipwork   = ipb + 128
      ipend = ipwork+6*n
      if ( ipend>5618 ) then

!     --- Array array as computed in elmgrinf too small

         call errint ( ipend, 1 )
         call errint ( 5906, 1 )
         call errchr ( 'array', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- compute basis functions and weights for numerical integration

      call elm800basefn ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, tang, jelp, phixi, psi, &
                          array, psixi )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!     --- Fill uold ( in nodal points ), it is needed inside el3007
!         Fill dudx ( in integration points ) if necessary
!                   ( when jdercn=0 only in the first point )

         call el3007 ( uold, dudx, phi, dphidx, m, &
                       n, ndimlc, ncomp, index1, dum, ugauss, &
                       jtrans, numold, iseqi1 )
         if ( debug ) then
            call prinrl1 ( uold, ncomp, m, 'uold' )
            call prinrl3 ( dudx, m, ndimlc , ncomp, 'dudx' )
         end if

      end if
      maxunk = ndim
      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, phi, n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, user, array, x, &
                          xgauss, m, n, phi, &
                          1, index1,iuser, ugauss, vecloc )

         end if

      end do
      if ( jcheld>=1 .and. jcheld<=5 ) then

!     --- jcheld = 1, compute derivatives

         call el1005 ( jcheld, ix, jdegfd, uold, dudx, x, &
                       ncounw, elemvc1, m )

      else

!     --- Compute array eps = B u

         if ( igprob<=9 ) then

!        --- Linear case

            call el8105 ( igprob, eps, ugauss, dudx, &
                          xgauss, m )

         else

!        --- Non-linear case
!            collect all basic nonlin data

            call eldu2dfg ( dudx, array(ipdetf), xgauss, &
                            ugauss, m, ndim, jcart )
            call eldf2gls ( eps, dudx, ugauss, xgauss, &
                            m, ndim, jcart )

            if ( ioption(1)>=3 ) then

!           --- rotate for every datapoint

               do i = 1, m
                  xpos(1) = x(i)
                  xpos(2) = x(i+m)
                  xpos(3) = x(i+m+m)
                  call fnlocdir ( xpos, rotat(1,i), &
                                  ielgrp )
                  call elrotsym ( eps(1+6*(i-1)), rotat(1,i), 0 )
               end do
            end if

         end if

         if ( jcheld==6 ) then

!        --- jcheld = 6,  Store the stress into elemvc1
!            Compute matrix D if necessary

            if ( igprob<=9 ) then

!           --- Linear case

               if ( compd ) call el8100 ( array, array(1+m), igprob, nd, &
                   m, d )

               if ( icheld==6 .or. icheld==36 .or. icheld==46 ) then

!              --- Type number 250

                  call el8106 ( igprob, elemvc1, array(25*m+1), &
                                array(31*m+1), array(m+1), d, sigmalc, &
                                eps, m, nd, icheld )

               else

!              --- Type numbers 206/207, store in work

                  call el8106 ( igprob, array(ipwork), array(25*m+1), &
                                array(31*m+1), array(m+1), d, sigmalc, &
                                eps, m, nd, icheld )

!                 --- Copy work to elemvc1

                  do i = 1, 4
                     do k = 1, n
                        elemvc1((k-1)*4+i) = array(ipwork-1+(k-1)*6+i)
                     end do
                  end do

               end if

            else

!           --- igprob>9
!               nonlinear -> stress via call to fnmateri
!               loop over data points

               do i = 1, m
                  ix = 1+6*(i-1)

!                 --- get material parameters

                  do j = 1, 21
                     matpar(j) = array(i+m*(j-1))
                  end do

                  call fnmateri ( ioption(3), sigmalc(ix), array(ipwork), &
                                  eps(ix), array(ipdetf-1+i), matpar, &
                                 .false. )

!                 --- rotate sigmalc back to global

                  if ( ioption(1)>=3 ) then
                     call elrotsym ( sigmalc(ix), rotat(1,i), 1 )
                  end if

!                 --- from 2nd Piola-Kirchoff stress to Cauchy
!                     C = 1/J F * P * F'
!                     first reorder deformation gradient
!                     Still needs some work here for axisymmetry

                  do j = 1, ndim
                     do k = 1, ndim
                        dfgrad(k,j)=dudx(i+m*(j-1)+ndim*m*(k-1))
                     end do
                  end do

!                 --- sigmalc = dfgrad * sigmalc * dfgrad'
!                     routine elrotsym can perform this type of
!                     multiplication

                  call elrotsym ( sigmalc(ix), dfgrad, 1 )

!                 --- local axes?

                  if ( ioption(1)>=3 ) then
                     call elrotsym( sigmalc(ix), rotat(1,i), 0 )
                  end if

!                 --- copy sigmalc to elemvc1 and divide by detf

                  do j=1,6
                     elemvc1(j+6*(i-1))= &
                        sigmalc(j+6*(i-1))/array(ipdetf-1+i)
                  end do
               end do

            end if

         else if ( jcheld==7 ) then

!        --- jcheld = 7,  Store the strain into elemvc1

            if ( igprob<=9 ) then

!           --- Linear case

               if ( icheld==7 .or. icheld==37 .or. icheld==47 ) then

!              --- Type number 250

                  call el8107 ( elemvc1, eps, m, nd, icheld )

               else

!              --- Type numbers 206/207, store in work

                  call el8107 ( array(ipwork), eps, m, nd, icheld )

!                 --- Copy work to elemvc1

                  do i = 1, 3
                     indexi = i
                     if ( indexi==3 ) indexi = 4
                     do k = 1, n
                        elemvc1((k-1)*3+i) = &
                           array(ipwork-1+(k-1)*6+indexi)
                     end do
                  end do

               end if

            else

!           --- nonlinear strains to elemvc1

               do i = 1,6
                  do k = 1,n
                     elemvc1((k-1)*6+i)=eps((k-1)*6+i)
                  end do
               end do

            end if

         else if ( jcheld==8 ) then

!        --- jcheld = 8 ; f(detf) to elemvc1

            do k = 1,n

!           --- get matpar from array

               do j = 1, 21
                  matpar(j) = array(k+n*(j-1))
               end do

!              --- extra argument

               elemvc1(k) = fndetf( ioption(3), eps((k-1)*6+1), &
                                   array(ipdetf-1+k), matpar )

            end do

         else

!        --- jcheld = 9 ; build RHS force vector
!            build blwork

            call elmkbl( array(ipblwork), dudx, dphidx, &
                         phi, xgauss, ugauss, &
                         ndim, m, n )

!           --- reduced integration ?

            if ( ioption(1)==2 .or. ioption(1)==4 ) then

!           --- compute RI point: dfgrdr, and dphdxr

               call elri01 ( dphdxr, dfgrdr, detfr(1), wr(1), &
                             dphidx, dudx, w, ugausr, xgausr, &
                             ugauss, xgauss, &
                             ndim, jcart, m, n )

!              --- compute strain in RI point

               call eldf2gls( glstrr, dfgrdr, ugausr, xgausr, 1, &
                              ndim, jcart )

!              --- bl matrix for RI point

               call elmkbl( array(ipblwrkr), dfgrdr, dphdxr, &
                            phi, xgausr, ugausr,  ndim, 1, n )

            end if

!           --- vector for nonlinear analysis

            call elnlav ( elemvc1, array(ipblwork), sigmalc, w, ndim, &
                          icount, n, m, nd )

!           --- reduced integration part

            if ( ioption(1)==2 .or. ioption(1)==4 ) then

               call elnlav ( elemvc1, array(ipblwrkr), pksr, wr, ndim, &
                             icount, n, 1, nd )

            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemvc1, n, 6, 'elemvc1' )
         write(irefwr,*) 'End eld250'

      end if
      call erclos ( 'eld250' )

      end
