      subroutine eld260 ( coor2d,  iuser, user, islold, vecloc, work, &
                          numld1, phi, phixi, psi, psixi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 06-01-2016 Use sepmoduleelem
!        version  5.0  date 21-08-2013 Extra parameters
!        version  4.0  date 07-08-2013 Remove phi
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector and weights for rubber elements
!     Two and three dimensional elements
!     ELD260 is a help subroutine for subroutine DERIV (TO0047)
!     it is called through the intermediate subroutine eld000
!     So:
!     DERIV
!     TO0047
!       -  Loop over elements
!          -  ELD000
!             - ELD260
!    The subroutine is able to handle the following element shapes and cases:
!    2D:  4 node bilinear quadilateral (Type 260)
!         5 node bilinear quadilateral (Type 261)
!         6 node extended triangle (Type 260)
!         7 node extended triangle (Type 261)
!         9 node biquadratic quadilateral (Types 260 and 261)
!    3D: 27 node brick (Types 260 and 262)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
!     rubber_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor2d(*), user(*), vecloc(*), work(*), &
                       phi(*), phixi(*), psi(*), psixi(*)
      integer numld1, iuser(*), islold(*)

!     coor2d           i    array of length ndim x npointcac containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     numld1         i    Formal parameter NUMOLD
!     phi           i/o   array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phixi         i/o   Array of length n x m x ndim containing the values of
!                         the derivatives of the basis functions in the Gauss
!                         points in the reference element.
!                         Array phi is only filled when ifirst = 0 and must be
!                         kept by the program
!     psi           i/o   Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi         i/o   Array of length nvert x m x ndim containing the values
!                         of the derivatives of the basis functions in the Gauss
!                         points in the reference element
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     vecold         i    In this array all preceding solutions are stored, i.e.
!                         all solutions that have been computed before and
!                         have been carried to system or deriva by the parameter
!                         islold in the parameter list of these main subroutines
!     work                work array of length 163 + 238m**3 + 6m + 15n**2 + 10n
!                         contents:
!                         Starting address length   Name:
!                         1                7*27     ARRAY
!                         ipugs            3*27     UGAUSS
!                         ipdudx           9*27     DUDX
!                         ipdpdx           27*27*3  DPHIDX
!                         ipphix           27*27*3  PHIKSI for elp633
!                         ipdiv            81*4     DIV_matrix (Divergence
!                                                   matrix)
!                         ipwork           27*31    Work space
!                         ipgrad           9*7      Gradient (2D)
!                         ipsigm           6*7      Stress tensor (2D)
!                         iptran           12*2     Transformation matrix based
!                                                   on div
!                                                   (2D only, does not
!                                                    contribute to total
!                                                    length, since this
!                                                    length is maximized for 3d)
!                         Total length: 6558
!                         The subarrays contain the following contents:
!                         ARRAY:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                              2D                  3D
!                         1:   rho              rho
!                         2:   omega            omega
!                         3:   f1               f1
!                         4:   f2               f2
!                         5:                    f3
!                         6:   eta              eta
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!                         DPHIDX:
!                         Array of length inpelm x m x ndim containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi_i / dx = dphidx (i,k,1);
!                         d phi_i / dy = dphidx (i,k,2);
!                         d phi_i / dz = dphidx (i,k,3);
!                           in node k
!                         WORK
!                         General work space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, ipdpdx, ipugs, icounw, ipwork, &
              ipphix, ipdudx, ipdiv, jcheld, mult, ipgrad, ipsigm, &
              iptran, iprho, nveloc, npres, lstep, lsub, lsup, icent, &
              ipend
      double precision qmat(3,27), p(4), rhsdiv(27), rhsdivtr(27), &
                       tang(4*9*2), &
                       psipres(4*27), x(81), w(27), xgauss(81), uold(81)
      double precision, pointer :: vecoldlc(:)
      save isub, isup, jelp, ipdpdx, ipwork, ipphix, ipugs, ipdudx, &
           ipdiv, icounw, mult, ipgrad, ipsigm, iptran, iprho, &
           nveloc, icent, npres

!     i              Counting variable
!     icent          Index of centroid
!     icounw         length of elemwg(*)
!     ipdiv          Starting address in array work of div
!     ipdpdx         Starting address in array work of dphidx
!     ipdudx         Starting address in array work of dudx
!     ipend          Last position in array
!     ipgrad         Starting address in array work of gradv (2D only)
!     ipphix         Starting address in array work of dphpi/dx
!     iprho          Starting address in array work of rho
!     ipsigm         Starting address in array work of the stress tensor sigma
!                    (2D only)
!     iptran         Starting address in array work of trans with respect to div
!     ipugs          Starting address in array work of ugauss
!     ipwork         Starting address in array work of work array
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jcheld         if ( icheld<= 10 ) then icheld else icheld-20
!     jelp           Indication of the type of basis function subroutine must
!                    be called by ELxxxx. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     lstep          Step variable for vertices. Usually lstep = 1, &
!                    but when the solution must be computed in the vertices
!                    and the element is quadratic then lstep = 2
!     lsub           Under index for the direction of the derivative to be
!                    computed
!     lsup           Upper index for the direction of the derivative to be
!                    computed
!     mult           ?
!     npres          Number of pressure parameters
!     nveloc         Number of velocity parameters
!     p              Array containing the pressure
!                    p(1) contains the pressure in the centroid
!                    p(2-4) contains the gradient of the pressure in the
!                    centroid
!     psipres        Contains the pressure basis functions
!                    psipres(i,k) refers to i-th basis function in
!                    integration point k
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rhsdiv         Dummy
!     rhsdivtr       Dummy
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     uold           Array containing the previous iteration (old solution)
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0260         Initialize parameters for subroutines ELD260/ELI260/ELM260
!     EL1000         Compute the divergence matrix for Navier-Stokes
!     EL1005         fill of derivatives and weights in elemvc1 and elemwg
!                    with respect to subroutine ELD250
!     EL1007         Fill element vector for vertices only
!     EL2000         Compute the transformation matrix to be used in the
!                    elimination process of the centroid point for
!                    incompressible Navier-Stokes
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL3000         Compute a local vector and its derivatives from a known
!                    vector
!     EL3009         Fill series of old solutions in array vecloc
!     EL8003         compute the pressure for the rubber elements in the
!                    centroids
!     EL8004         computes the stress tensor for rubber elements
!     ELFILLDERIV    Fill the derivatives of the old solution in nodal points
!     ELFILLPSIPRES  Fill the pressure basis functions in the integration points
!                    Crouzeix Raviart elements
!     ELFILLUGAUSS   Fill the old solution in Gauss points
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  ISEQ:  Sequence number of vector in VECOLD/ISLOLD from which UOLD and
!         derivatives must be computed.
!         If 0 or 1 the first vector is used
!         If more vectors are needed, these vectors are supposed to be stored
!         sequentially, where the first one is indicated by ISEQ
!     2:  - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!     5:  - &
!     6-20: Information about the equations and solution method
!     6:  density rho
!     7:  nu  (Poisson ratio)
!     8:  E   (Elasticity modulus)
!     9:  F1  (Body force in 1-direction)
!    10:  F2  (Body force in 2-direction)
!    11:  F3  (Body force in 3-direction)
!    12-20: not yet used
!     Remark:  the coefficients 2-20 are only used in the case of icheld = 6
!              or 11, i.e. computation of the stress tensor
!              In the case icheld = 7 and an extended quadratic triangle
!              also all parameters are used
!     icheld                    Cartesian            Cylindrical/Polar
!       1                        du        / d
!                                  jdegfd     ix
!       2                        grad u               2D/Cyl: (r,z)
!                                i.e. du1/dx1,        dur/dr, duz/dr, &
!                                du2/dx1, ...         dur/dz, duz/dz
!                                                     3D/Cyl: (r,z,phi)
!                                                     dur/dr, duz/dr, duphi/dr
!                                                     dur/dr, duz/dz, duphi/dz
!                                                     dur/rdphi-uphi/r, &
!                                                     duz/rdphi,duphi/rdphi+ur/r
!                                                     2D/Pol: (r,phi)
!                                                     dur/dr, duz/dr, &
!                                                     dur/rdphi-uphi/r, &
!                                                     duphi/rdphi+ur/r
!       3                        - grad u
!       4                        div u
!       5                        curl u                  Cylinder co-or
!                     2D:       du2/dx1-du1/dx2      2D:   duz/dr-dur/dz
!                     3D:       du3/dx2-du2/dx3      3D:   duz/rdphi-duphi/dz
!                               du1/dx3-du3/dx1       duphi/dr+(uphi-dur/dphi)/r
!                               du2/dx1-du1/dx2            dur/dz-duz/dr
!                                                         Polar co-or
!                                                     duphi/dr+(uphi-dur/dphi)/r
!                             | sigma x |                    | sigma r     |
!       6                     | sigma y |                    | sigma z     |
!                     sigma = | sigma z |            sigma = | sigma theta |
!                             | tau xy  |                    | tau rz      |
!                             | tau yz  |                    | tau zphi    |
!                             | tau zx  |                    | tau phir    |
!                             | eps x    |                   | eps r     |
!       7                     | eps y    |                   | eps z     |
!                       eps = | eps z    |             eps = | eps theta |
!                             | gamma xy |                   | gamma rz  |
!                             | gamma yz |                   |   -       |
!                             | gamma zx |                   |   -       |
!      21-31:  See 1-11, however, now defined in the vertices instead of the
!              nodal points
!     Structure of output arrays:
!     Solution array:  ndim unknowns per point (IVEC=0)
!     Arrays of special structure:
!     IVEC = 1:   1 unknowns per point
!     IVEC = 2:   2 unknowns per point
!     IVEC = 3:   3 unknowns per point
!     IVEC = 4:   6 unknowns per point
!     IVEC = 5:   ndim unknowns per point
!     IVEC = 6:   1 unknowns per vertex
!     IVEC = 7:   2 unknowns per vertex
!     IVEC = 8:   3 unknowns per vertex
!     IVEC = 9:   6 unknowns per vertex
!     IVEC =10:   ndim unknowns per vertex
!     IVEC =11:   1/3/6 unknowns per point
!     IVEC =12:   0/1/3 unknowns per point
!     IVEC =13:   1/4/9 unknowns per point
!     IVEC =14:   1/3/6 unknowns per vertex
!     IVEC =15:   0/1/3 unknowns per vertex
!     IVEC =16:   1/4/9 unknowns per vertex
!     ICHELD                 IVEC
!       1                      1
!       2                     13
!       3                     13
!       4                      1
!       5                     12
!       6                      4
!       7                      4
!      21                      6
!      22                     16
!      23                     16
!      24                      6
!      25                     16
!      26                      9
!      27                      9
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ifirst=0 ) then
!        Define element independent quantities
!        Compute length of array elemwg
!        Compute pointers in array WORK
!     Compute basis functions and related quantities
!     Fill old solution into array UOLD and if necessary the derivatives
!     Compute variable coefficients if necessary
!     Compute derived quantities
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld260' )

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         call el0260 ( iuser, user, jelp, isub, isup, work, &
                       nveloc, npres )
         if ( inpelm==27 ) then
            icent = 14
         else
            icent = n
         end if

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

!        --- Length of elemwg(*) to be filled

         if ( icheld<=11 ) then
            jcheld = icheld
         else
            jcheld = icheld-20
         end if
         if ( jcheld==1 .or. jcheld==4 ) then

!        --- icheld = 1, 4
!            1 unknown per point

            mult = 1

         else if ( jcheld==2 .or. jcheld==3 ) then

!        --- icheld = 2 or 3
!            ndim**2 unknowns per point

            mult = ndim**2

         else if ( icheld==6 .or. icheld==7 ) then

!        --- icheld = 6 or 7
!            lower triangle of tensor

            mult = 6

         else if ( ndim==2 ) then

!        --- icheld = 5 and ndim = 2, &
!            one unknown per point

            mult = 1

         else

!        --- icheld = 5 and ndim = 3, &
!            three unknowns per point

            mult = 3

         end if

         if ( icheld<=11 ) then

!        --- icheld<=11 unknowns in all nodes

            icounw = inpelm * mult

         else

!        --- icheld>=21, unknowns in the vertices only

            icounw = nvert * mult

         end if

!        --- Initialize weight vector, if any

         do i = 1, icounw
            elemvc1(i) = 0d0
            elemwg(i) = 1d0
         end do

!        --- Fill pointers in array work

         iprho  = 1
         ipugs  = 1 + 6*m+n
         ipdudx = ipugs  + ndim*m
         ipdpdx = ipdudx + ndim**2*m
         ipphix = ipdpdx + ndim*m*n
         ipdiv  = ipphix + ndim*n*m
         ipwork = ipdiv  + m*ndim*(ndim+1)
         ipgrad = ipwork + m*(m+ndim+1)
         ipsigm = ipgrad + 9 * m
         iptran = ipsigm + 4 * m
         ipend = iptran+ndim**2*(n-1)
         call inichkarlen ( ipend, 15000, 'work' )
         if ( ierror/=0 ) go to 1000

      end if

!     --- compute basis functions and weights for numerical integration

      call elm800basefn ( coor2d, x, w, work(ipdpdx), xgauss, phi, &
                          index1, qmat, tang, jelp, phixi, &
                          psi, work, psixi )
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      iseqin = 1
      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then
         call el2005 ( uold, work(ipdudx), phi, &
                       work(ipdpdx), index1, work(ipugs), iseqin )
         if ( itype==261 ) then

!        --- itype = 261, put pressure in p

            iseqtype = oldsols(iseqin)%iseqtype
            vecoldlc => oldsols(iseqin)%oldvecs
            index3n => indoldsols(iseqtype)%index3
            p(1) = vecoldlc(index3n(nveloc+1))

         else if ( itype==262 ) then

!        --- itype = 262, put pressure and pressure gradient in p

            iseqtype = oldsols(iseqin)%iseqtype
            vecoldlc => oldsols(iseqin)%oldvecs
            index3n => indoldsols(iseqtype)%index3
            do i = 1, 4
               p(i) = vecoldlc(index3n(14*3+i))
            end do

         end if

      end if

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, phi, n )

!     --- Fill variable coefficients if necessary

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, user, work, x, xgauss, m, n, &
                           phi, 1, index1, &
                           iuser, work(ipugs), vecloc )

         end if

      end do

!     --- Compute the divergence and gradient matrix

      if ( jtrans==1 .or. icheld==6 .or. icheld==26 ) then
         call elfillpsipres ( psipres, npres, xgauss, x, icent )
         call el1000 ( work(ipdiv), work(ipdiv), work(ipdpdx), work, &
                       phi, w, x, xgauss, 0, work(iprho), &
                       work, rhsdiv, work(1+8*m), 0, ishape, &
                       psipres, icent, work )
      end if  ! ( jtrans==1 .or. icheld==6 .or. icheld==26 )

      if ( jtrans==1 ) then

!     --- Compute the transformation matrix
!         Compute the velocity in the centroid and the derivative of u

         call el2000 ( work(iptran), work(ipdiv), 0, rhsdivtr, rhsdiv )
         if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then

!        --- Fill centroid velocity

            call el3000 ( uold, work(ipdudx), phi, work(ipdpdx), &
                          work(iptran), work(ipugs), 0, rhsdivtr, &
                          lstep, lsub, lsup )

            if ( jtrans>0 ) then

!           --- jtrans>0, transformation, compute derivatives and u in
!               Gauss points

               if ( jsol/=0 ) then

!              --- Compute solution in Gauss points

                  call elfillugauss ( uold, phi, work(ipugs), n, &
                                      ncomp, m, jsol, jdiag )

               end if  ! ( jsol/=0 )

               if ( jderiv>0 ) then

!              --- jderiv>0,  compute derivatives

                  call elfillderiv ( uold, work(ipdpdx), work(ipdudx), &
                                     n, ncomp, m, jdercn, lsub, lsup, &
                                     lstep, ndimlc )

               end if  ! ( jderiv>0 )

            end if  ! ( jtrans>0 )

         end if

      end if

!     --- Fill elemvc1 and elemwg, content depends on icheld

      if ( icheld==6 .or. icheld==26 ) then

!     --- icheld = 6 or 26
!         Compute the stress tensor and store in elemvc1
!         First fill array p if the penalty method is applied

         if ( itype==260 ) call el8003 ( work(ipdiv), w, x, xgauss, &
                             work(1+2*m), work(1+m), uold, p )
         if ( icheld==6 ) then

!        --- icheld = 6, fill in all nodes

            if ( jtrans==1 ) then

!           --- Special case, the centroid has been eliminated

            else

!           --- Standard case

               call el8004 ( x, uold, p, work(ipdudx), work(ipgrad), &
                             mult, elemvc1, work(1+2*m), work(1+m), &
                             work(ipwork) )

            end if

         else

!        --- icheld = 26, fill in vertices
!            First fill in all nodes

            if ( jtrans==1 ) then

            else

!           --- Standard case

               call el8004 ( x, uold, p, work(ipdudx), work(ipgrad), &
                             mult, elemvc1, work(1+2*m), work(1+m), &
                             work(ipwork) )

            end if

!           --- Copy in vertices

            call el1007 ( work(ipwork), elemvc1, mult, ishape )

         end if

      else if ( icheld<=7 ) then

!     --- icheld<=7, fill elemvc1 by derivatives, &
!         elemwg by weights

         call el1005 ( icheld, ix, jdegfd, uold, work(ipdudx), &
                       x, mult, elemvc1, m )

      else if ( icheld<=27 ) then

!     --- 20<icheld<=27 fill elemvc1 by derivatives, &
!         elemwg by weights
!         First fill in all nodes

         call el1005 ( icheld-20, ix, jdegfd, uold, work(ipdudx), &
                       x, mult, work(ipwork), m )

!        --- Copy in vertices

         call el1007 ( work(ipwork), elemvc1, mult, ishape )

      end if

1000  call erclos ( 'eld260' )

      end
