      subroutine sepelpqfib
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the fiber coordinate system
!     Old subroutine el8125
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     shell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleconsts
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k
      double precision u(3), eh(3), eh1(3), a(3), norm
      double precision, pointer :: xhat(:,:), qfib(:,:,:)

!     a              Help vector of size 3
!     eh             Help vector of size 3
!     eh1            Help vector of size 3
!     j              Counting variable
!     k              Counting variable
!     norm           Norm of a vector
!     qfib           Contains the fiber coordinate system
!     u              Contains the coordinates of the Cartesian coordinate system
!                    u(k) is the k^th component of the j^th vector
!     xhat           normal vector in the nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute cross product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!
!     The algorithm of Hughes is applied, i.e.
!
!     1. Let a_i = Xhat_i (i=1,2,3)
!     2. j= 1
!     3. If a_1 > a_3 then a_3 = a_1 and j=2
!     4. if a_2 > a_3, j=3
!     5. ef_3 = Xhat
!     6. ef_2 = (Xhat x e_j) / ||Xhat x e_j||
!     7. ef_1 = ef_2 x Xhat
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpqfib' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpqfib'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      xhat => B%xhat
      qfib => B%qfib

!     --- Loop over the nodes

      u = 0d0
      do k = 1, n

!     --- Let a_i = Xhat_i (i=1,2,3)

         a(1) = abs(xhat(k,1))
         a(2) = abs(xhat(k,2))
         a(3) = abs(xhat(k,3))

         j = 1
         if ( a(1)>a(3) ) then

!        --- a_1 > a_3 then a_3 = a_1 and j=2

            a(3) = a(1)
            j = 2

         end if

         if ( a(2)>a(3) ) j=3

!        --- ef_3 = Xhat

         qfib(3,1,k) = xhat(k,1)
         qfib(3,2,k) = xhat(k,2)
         qfib(3,3,k) = xhat(k,3)

!        --- ef_2 = (Xhat x e_j) / ||Xhat x e_j||

         a(1) = xhat(k,1)
         a(2) = xhat(k,2)
         a(3) = xhat(k,3)
         u(j) = 1d0
         call eloutr ( a, u(j), eh )
         u(j) = 0d0
         norm = 1d0/sqrt(eh(1)**2+eh(2)**2+eh(3)**2)

         eh(1) = eh(1)*norm
         eh(2) = eh(2)*norm
         eh(3) = eh(3)*norm
         qfib(2,1,k) = eh(1)
         qfib(2,2,k) = eh(2)
         qfib(2,3,k) = eh(3)

!        --- ef_1 = ef_2 x Xhat

         call eloutr ( eh, a, eh1 )
         qfib(1,1,k) = eh1(1)
         qfib(1,2,k) = eh1(2)
         qfib(1,3,k) = eh1(3)

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( qfib, 3, 2, 'qfib' )
         write(irefwr,*) 'End sepelpqfib'

      end if  ! ( debug )
      call erclos ( 'sepelpqfib' )

      end subroutine sepelpqfib
