      subroutine sepelsgmd01 ( fdu, nip, nnode )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 1.0    date   14-11-1997
!
!   copyright (c) 1997-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Assemble data for Souza method. The part:
!          -c
!       ( F  : grad (du) ) is computed, resulting in a fdu vector.
! **********************************************************************
!
!                       KEYWORDS
!
!      elasticity_equation
!      solid
!      souza
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nip, nnode
      double precision fdu(3,nnode,nip)

!     dphidx     i    Array of length n x m x ndim  containing the derivatives
!                     of the basis functions in the sequence:
!                     d phi / dx (xg ) = dphidx (i,k,1);
!                          i        k
!                     d phi / dy (xg ) = dphidx (i,k,2);
!                          i        k
!                     d phi / dz (xg ) = dphidx (i,k,3);
!                          i        k
!                     If the element is a linear simplex, the
!                     derivatives are constant and only k = 1 is filled.
!     f          i    deformation gradient
!     fdu        o    contains at output the incremental gradient
!                     for Souza method
!     nip        i    number of integration points
!     nnode      i    number of nodes
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ip, k
      double precision h(3,3), t4, t6, t8, t10, t12, t14, t17
      double precision, pointer :: dphidx(:,:,:), f(:,:,:)

!     h          matrix for inverse of F
!     ip         nodal point numbers of line to be considered
!     k          Counting variable
!     t10        temporary variable
!     t12        temporary variable
!     t14        temporary variable
!     t17        temporary variable
!     t4         temporary variable
!     t6         temporary variable
!     t8         temporary variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!            -c
!       H = F       for every integration point
!
!       fdu = H  dphidx       for every integration point and node
!          i   ji      j
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      dphidx => E%dphidx
      f => E%dfgrad
      do ip = 1,nip

         t4 =  f(ip,1,1)*f(ip,2,2)
         t6 =  f(ip,1,1)*f(ip,3,2)
         t8 =  f(ip,1,2)*f(ip,2,1)
         t10 = f(ip,3,1)*f(ip,1,2)
         t12 = f(ip,2,1)*f(ip,1,3)
         t14 = f(ip,3,1)*f(ip,1,3)
         t17 = 1d0/( t4*f(ip,3,3)-t6*f(ip,2,3)-t8*f(ip,3,3) &
                    +t10*f(ip,2,3) +t12*f(ip,3,2)-t14*f(ip,2,2) )

         h(1,1) = (  f(ip,2,2)*f(ip,3,3) -f(ip,2,3)*f(ip,3,2) )*t17
         h(1,2) = ( -f(ip,1,2)*f(ip,3,3) +f(ip,3,2)*f(ip,1,3) )*t17
         h(1,3) = (  f(ip,1,2)*f(ip,2,3) -f(ip,2,2)*f(ip,1,3) )*t17
         h(2,1) = ( -f(ip,2,1)*f(ip,3,3) +f(ip,3,1)*f(ip,2,3) )*t17
         h(2,2) = (  f(ip,1,1)*f(ip,3,3) -t14 )*t17
         h(2,3) = ( -f(ip,1,1)*f(ip,2,3) +t12 )*t17
         h(3,1) = (  f(ip,2,1)*f(ip,3,2) -f(ip,3,1)*f(ip,2,2) )*t17
         h(3,2) = ( -t6 +t10 )*t17
         h(3,3) = (  t4 -t8 )*t17

         do k = 1,nnode

            fdu( 1,k,ip ) = h(1,1)*dphidx( k,ip,1 ) &
                           +h(2,1)*dphidx( k,ip,2 ) &
                           +h(3,1)*dphidx( k,ip,3 )
            fdu( 2,k,ip ) = h(1,2)*dphidx( k,ip,1 ) &
                           +h(2,2)*dphidx( k,ip,2 ) &
                           +h(3,2)*dphidx( k,ip,3 )
            fdu( 3,k,ip ) = h(1,3)*dphidx( k,ip,1 ) &
                           +h(2,3)*dphidx( k,ip,2 ) &
                           +h(3,3)*dphidx( k,ip,3 )

         end do

      end do

      end
