      subroutine sepfillbmatsh
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the B-matrix for the shell element
!     This matrix contains all curvature effects
!     Old subroutine el8126
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     shell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: dphidxl(4,3), omega(4,3,2), meannorm, za
      double precision, pointer :: dphidx(:,:,:), qmat(:,:,:), qfib(:,:,:)
      double precision, pointer :: bs(:,:,:,:), bsh(:,:,:,:,:), xhat(:,:), h(:)

!     bs             B^s matrix for shell element
!     bsh            B matrix for shell element
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dphidxl        dphi / dx^l, i.e. derivatives of the basis functions in
!                    the lamina directions
!                    dphidxl(i,j) = dphi_i / dx^l_j
!     h              height of shell element
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     meannorm       Mean value of the norm of two vectors
!     omega          Array omega as defined in Hughes
!                    omega(i,j,k) corresponds to node i, component j and
!                    the k^th rotation
!     qfib           Contains the fiber coordinate system
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     xhat           normal vector in the nodes
!     za             za(zeta) = (1+zeta)/2 h/2 - (1-zeta)/2 h/2 = zeta h/2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The B matrix can be written as a matrix consisting of 4 blocks:
!    |  B_uu        B_u theta     |
!    |  B_theta u   B_theta theta |
!    The parts B_uu and B_u theta are as described in Hughes, i.e.
!           | b_u_1 |                 | b_theta_1 |
!    b_uu = | b_u_2 |     b_u theta = | b_theta_2 |
!           | b_u_3 |                 | b_theta_3 |
!    b_u_1 = Q_1 dphi/dx^l_1
!    b_u_2 = Q_2 dphi/dx^l_2
!    b_u_3 = Q_1 dphi/dx^l_2 + Q_2 dphi/dx^l_1
!    b_theta_1 = omega_1 dphi/dx^l_1 za
!    b_theta_2 = omega_2 dphi/dx^l_2 za
!    b_theta_3 = omega_1 dphi/dx^l_2 za + omega_2 dphi/dx^l_1 za
!    with omega_1 = - Q e^f_2
!         omega_2 =   Q e^f_1
!    The theta row is computed by taking the Bs matrix defined for plates, &
!    where the first element is multiplied by the normal to the element
!    This approach is not completely correct, since the curvature of the
!    normal is not taken into account
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillbmatsh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillbmatsh'
         call prinrl1 ( B%qmat, 3, 3, 'qmat' )
         call prinrl1 ( B%phider(1,1,1), n, n, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), n, n, 'dphidy' )
         call prinrl1 ( B%phider(1,1,3), n, n, 'dphidz' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider
      qmat => B%qmat
      qfib => B%qfib
      bs => E%bs
      bsh => E%bsh
      xhat => B%xhat
      h => E%coeffs(:,22)

!     --- Loop over the integration points

      do k = 1, m

!     --- Start with the computation of dphi/dx_l

         do i = 1, n
            do j = 1, 2
               dphidxl(i,j) = qmat(3-j,1,k)*dphidx(i,k,1)+ &
                              qmat(3-j,2,k)*dphidx(i,k,2)+ &
                              qmat(3-j,3,k)*dphidx(i,k,3)
            end do
         end do

!        --- Next the coefficients omega

         do i = 1, n
            do j = 1, 2
               omega(i,j,1) =-qmat(3-j,1,k)*qfib(2,1,i) &
                             -qmat(3-j,2,k)*qfib(2,2,i) &
                             -qmat(3-j,3,k)*qfib(2,3,i)
               omega(i,j,2) = qmat(3-j,1,k)*qfib(1,1,i) &
                             +qmat(3-j,2,k)*qfib(1,2,i) &
                             +qmat(3-j,3,k)*qfib(1,3,i)
            end do
         end do

!        --- First the matrix B_uu

         do i = 1, n
            do j = 1, 3
               bsh(1,j,i,k,1) = qmat(3,j,k)*dphidxl(i,1)
               bsh(2,j,i,k,1) = qmat(2,j,k)*dphidxl(i,2)
               bsh(3,j,i,k,1) = qmat(3,j,k)*dphidxl(i,2)+ &
                              qmat(2,j,k)*dphidxl(i,1)
               bsh(1,j,i,k,2) = bsh(1,j,i,k,1)
               bsh(2,j,i,k,2) = bsh(2,j,i,k,1)
               bsh(3,j,i,k,2) = bsh(3,j,i,k,1)
            end do
         end do

!        --- Next the matrix B_u theta

         za = h(k)*0.5d0
         do i = 1, n
            do j = 1, 2
               bsh(1,j+3,i,k,1) = omega(i,1,j)*dphidxl(i,1)*B%zeta(1)*za
               bsh(2,j+3,i,k,1) = omega(i,2,j)*dphidxl(i,2)*B%zeta(1)*za
               bsh(3,j+3,i,k,1) =(omega(i,1,j)*dphidxl(i,2)+ &
                                omega(i,2,j)*dphidxl(i,1))*B%zeta(1)*za
               bsh(1,j+3,i,k,2) = omega(i,1,j)*dphidxl(i,1)*B%zeta(2)*za
               bsh(2,j+3,i,k,2) = omega(i,2,j)*dphidxl(i,2)*B%zeta(2)*za
               bsh(3,j+3,i,k,2) =(omega(i,1,j)*dphidxl(i,2)+ &
                                omega(i,2,j)*dphidxl(i,1))*B%zeta(2)*za
            end do
         end do

!        --- Next the matrix B_theta u
!            Since we are dealing with the T1 elements, this requires the
!            special adaptation
!            We use the matrix Bs, since this matrix is already defined
!            for the plate element
!            The part Bs(.,1) must be used multiplied by the normal in the
!            midside point
!            We assume that this the mean value of Xhat
!            This is not correct, but a reasonable approximation

         do i = 1, n
            do j = 1, 3
               meannorm = 0.5d0*(xhat(i,j)+xhat(icyc4p(i),j))
               bsh(4,j,i,k,1) = meannorm*bs(1,1,i,k)
               bsh(5,j,i,k,1) = meannorm*bs(2,1,i,k)
               bsh(4,j,i,k,2) = bsh(4,j,i,k,1)
               bsh(5,j,i,k,2) = bsh(5,j,i,k,1)
            end do
         end do

!        --- Finally the matrix B_theta theta

         do i = 1, n
            do j = 1, 2
               bsh(4,j+3,i,k,1) = bs(1,j+1,i,k)
               bsh(5,j+3,i,k,1) = bs(2,j+1,i,k)
               bsh(4,j+3,i,k,2) = bsh(4,j+3,i,k,1)
               bsh(5,j+3,i,k,2) = bsh(5,j+3,i,k,1)
            end do
         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         do i = 1, n
            call prinrl1 ( bsh(1,1,i,1,1), 5, 5, 'bshi1' )
            call prinrl1 ( bsh(1,1,i,1,2), 5, 5, 'bshi2' )
         end do  ! i = 1, n
         write(irefwr,*) 'End sepfillbmatsh'

      end if  ! ( debug )
      call erclos ( 'sepfillbmatsh' )

      end subroutine sepfillbmatsh
