! **********************************************************************
!
! *   Solution of the elastohydrodynamic lubrication of an oil pumping
! *   ring seal by SEPRAN
! *
! *   Time-dependent problem
! *
! *   At t=0 the non-linear stationary equations are solved by a Newton
! *   iteration
! *   For t>0 a modified Crank-Nicolson scheme is used.
! *   The non-negative pressure condition is imposed by subroutine OVERCS
! *
! *
! *   Programmers:  Kees Kassels and Guus Segal
! *   version 2.0   date   05-12-93
! *
! **********************************************************************
!
      program pump

! **********************************************************************
!
!                      DECLARATIONS

      integer lnmesh, lnprob
      parameter ( lnmesh=200, lnprob = 500 )
      integer kmesh(lnmesh), kprob(lnprob), intmat(5), isol(5),
     +        islold(5), iinstr(3), iincrt(1), iinsol(4), istep, nstep,
     +        iinvec(2), jmetod, istop, iinout(1)
      double precision pi, omeg, rinvec(3)
! **********************************************************************
!
!                      COMMON BOCKS

      include 'SPcommon/ctimen'

! **********************************************************************
!
!     --- start sepran

      kmesh = 0
      kprob = 0

      kmesh(1) = lnmesh
      kprob(1) = lnprob
      iinstr(1) = 2
      iinstr(2) = 1
      call sepstn ( kmesh, kprob, intmat, iinstr )

!     --- create start vector

      t0 = 0d0
      t = t0
      iincrt(1) = 0
      call creatn ( iincrt, kmesh, kprob, isol )

!     --- non-linear iteration to find solution at t = 0

      iinsol(1) = 0
      call nlnprb ( kmesh, kprob, intmat, isol, iinsol )
      iinout(1) = 0
      call outsol ( kmesh, kprob, isol, iinout, t )

!     --- Compute time-dependent solution with non-constant velocity
!         Set time parameters ( two strokes will be computed )

      omeg  = 151.84d0
      pi    = 4d0*atan(1d0)
      tend  = 4d0*pi/omeg
      nstep = 80
      theta = 0.5d0
      tstep = (tend-t0)/nstep

!     --- define type of matrix for subroutine OVERCS (jmetod=9)

      jmetod = 9
      call commat ( jmetod, kmesh, kprob, intmat )

!     --- Time iteration by Crank-Nicolson (theta=.5)
!         nstep time steps are carried out

      do 100 istep = 1, nstep

!        --- Copy old solution in islold

         call copyvc ( isol, islold )

!        --- Compute u(t n+1/2)

         t = t+0.5d0*tstep
         iinsol(1) = 4
         iinsol(2) = 0
         iinsol(3) = 2
         iinsol(4) = 2
         call linprb ( kmesh, kprob, intmat, isol, iinsol )

!        --- u(n+1) := 2 u(n+1/2) - u(n)

         iinvec(1) = 2
         iinvec(2) = 39
         rinvec(1) = 2d0
         rinvec(2) = -1d0
         rinvec(3) = 0d0
         call manvec ( iinvec, rinvec, isol, islold, isol, kmesh,
     +                 kprob )
         t = t+0.5d0*tstep

!        --- Output of the solution at 10, 20, 30, ... ,nstep steps

         if ( mod(istep,10).eq.0 ) call outsol ( kmesh, kprob, isol,
     +      iinout, t )

100   continue

!     --- Stop SEPRAN

      istop=0
      call finish ( istop )
      end

!     --- funccf for the computation of the velocity as function of time

      function funccf ( ichois, r, z, dummy )
      double precision funccf, r, z, dummy
      integer ichois

      include 'SPcommon/ctimen'

      funccf = 3.492d6 * sin ( 151.84d0 * t )
      end


!     --- function func, for the computation of the starting pressure

      function func ( ichois, r, z, dummy )
      double precision func, r, z, dummy
      integer ichois
      func=max(0d0,6d0*(3.5d3-z)/3.5d3)
      end
