      subroutine readscstandard ( istart, inpsep, tabout, outfnd, icode,
     +                            nout, strout, nprob, numvec,
     +                            errorcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-07-2011 Extra parameter errorcode
!        version  1.7  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.6  date 30-01-2008 Extension with copy_coor
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following standard keywords with respect
!   to subroutine readsc
!             1:  PRESCRIBE_BOUNDARY_CONDITIONS
!             5:  OUTPUT
!            14:  COPY_COOR
!            19:  SOLVE_TIME_DEPENDENT_PROBLEM
!            41:  COMPUTE_CONTACT_SURFACE
!            48:  CHANGE_COORDINATES
!            50:  CHANGE_PROBLEM
!            51:  COMPUTE_CAPACITY
!            52:  SOLVE_INVERSE_PROBLEM
!            71:  ENTHALPY_INTEGRATION
!            72:  COMPUTE_ENTHALPY
!            77:  PRESSURE_CORRECTION
!            78:  NAVIER_STOKES
!            79:  SOLVE_BEARING
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
!     boundary_condition
!     change_boundary
!     capacity
!     inverse_problem
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cinout'
      include 'SPcommon/comcons1'
      include 'SPcommon/comcons2'
      include 'SPcommon/csepcm'
      include 'SPcommon/csepint'
      include 'SPcommon/cuscons'
      include 'SPcommon/cusname'
      include 'SPcommon/cpost5'
      include 'SPcommon/cscalvec'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical outfnd
      integer istart, inpsep(*), icode, nout, nprob, numvec, errorcode
      double precision tabout(*)
      character (len=*) strout(*)

!     errorcode     i/o   If 0 at input an error message is given if a keyword
!                         is not recognized
!                         If>0, the subroutine does not give an error message,
!                         but instead returns with -1
!     icode          i    Indicates which of the options in readsc is found
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2  ISEQNR Sequence number corresponding to input
!                                   If iseqnr = -1, the next one will be used
!                         3  IPROB  Problem sequence number
!                                   If iprob = -1, the next one will be used
!                         4  IVECTORVector sequence number (first vector)
!                                   If ivector = -1, the same value as iprob
!                                   will be used
!     istart        i/o   Next free position in inpsep
!     nout           i    Number of entries in tabout
!     nprob          i    Number of various problems
!     numvec        i/o   Number of vectors stored in isol
!     outfnd        i/o   Call to output found (true) or not (false)
!     strout        i/o   Contains series of strings
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncod1
      parameter ( nncod1=5 )
      integer ncode1(nncod1), itype1(nncod1), ncod1m(nncod1),
     +        m, ivec, i, iretur, iseqkeyw(nncod1), nseqkeyw
      double precision table1(nncod1), tabmn1(nncod1), tabmx1(nncod1)
      character (len=15) code1(nncod1)
      logical debug

!     code1          Table with keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iretur         Return value of subroutine INTX02
!                    Possible values:
!                    -2:  end of file has been reached
!                    -1:  error has been found
!                     0:  normal return
!     iseqkeyw       Array of size nseqkeyw that contains the sequence in
!                    which keywords are found in a record
!                    isekeyw(i)>0, means i-th keyword found is the
!                                    iseqkeyw(i)-th keyword in the table
!                    isekeyw(i)<0, means keyword refers to number found
!                                    -isekeyw(i) refers to position in tabout
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     ivec           Sequence number of vector with respect to array ISOL
!     m              number of items read
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     nseqkeyw       Number of entries in iseqkeyw
!     table1         Contains values of items read corresponding to code1
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRWAR         Print warnings
!     INTX02EXT1     Interpret a record containing keywords followed by numbers
! **********************************************************************
!
!                       I/O
!
!       PRESCRIBE_BOUNDARY_CONDITIONS, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!       OUTPUT, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default 1)
!               FILE = file_name     (Default sepcomp)
!       SOLVE_TIME_DEPENDENT_PROBLEM, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!       COMPUTE_CONTACT_SURFACE, sequence_number = k (Default 1)
!       CHANGE_COORDINATES, sequence_number = k (Default 1)
!       CHANGE_PROBLEM, vector=i, problem=p (Default 1,1)
!       COMPUTE_CAPACITY, sequence_number = k (Default 1)
!       SOLVE_INVERSE_PROBLEM, sequence_number = k (Default 1)
!       ENTHALPY_INTEGRATION, sequence_number = k (Default 1)
!       COMPUTE_ENTHALPY, sequence_number = k (Default 1)
!       PRESSURE_CORRECTION, sequence_number = k (Default 1)
!       COPY_COOR, vector = i, problem=p (Default 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1886   Vector has not yet been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQUENCE_NUMBER', 'VECTOR', 'V', 'PROBLEM', 'FILE' /
      data ncod1m /15, 6, 1, 7, 4 /
      data ncode1 / 6, 6, 1, 4, 4 /
      data itype1 / 0,38,38, 0, 6 /
      data tabmn1 / 5 * 1d0 /
      data tabmx1 / 5 * 1d2 /

!     code1   Table with sub keywords corresponding to some keywords
!             Values:
!             1:  SEQUENCE
!             2:  VECTOR
!             3:  V
!             4:  PROBLEM
!             5:  FILE
! ======================================================================
!
      call eropen ( 'readscstandard' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscstandard'
         write(irefwr,1) 'icode', icode
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icode==5 .or. icode==19 ) outfnd = .true.
      table1 = -1d0
      tabmx1(1) = 100d0

!     --- tabmx1(2) = dble(maxvectors)
!         tabmx1(3) = tabmx1(2)

      tabmx1(4) = nprob
      iretur = 0
      if ( icode==1 .or. icode==19 .or.
     +     icode==51 .or. icode==52 ) then
         m = 3
      else if ( icode==41 .or. icode==48 .or. icode>=71 .and.
     +          icode/=14 ) then
         m = 1
      else if ( icode== 5 ) then
         m = nncod1
      else
         m = 4
      end if
      nseqkeyw = nncod1
      call intx02ext1 ( code1, ncode1, m, itype1, table1, code1,
     +                  ncode1, 0, iretur, tabmn1, tabmx1, tabout,
     +                  nout, strout, 1, ncod1m,
     +                  iseqkeyw, nseqkeyw, errorcode )
      if ( ierror/=0 ) go to 1000
      if ( table1(2)>0d0 .and. table1(3)>0d0 )
     +   call errkeywexcl ( code1, ncod1m, 3, 2 )

      table1(2) = max(table1(2),table1(3))
      inpsep(istart) = icode
      inpsep(istart+1) = nint(table1(1))

      if ( icode==48 .or. icode>=71 ) then

!     --- CHANGE_COORDINATES, ENTHALPY_INTEGRATION, COMPUTE_ENTHALPY or
!         PRESSURE_CORRECTION found
!         only sequence number required

         istart = istart+2
         go to 1000

      end if

      inpsep(istart+2) = nint(table1(4))
      inpsep(istart+3) = nint(table1(2))
      ivec = inpsep(istart+3)
      if ( ivec>0 ) then
         if ( icode==1 .or. icode==3 .or. icode==4 .or. icode==8 .or.
     +        icode==14 .or. icode==19 .or. icode>=51 ) then

!        --- vector is new

            numvec = min ( max(numvec,ivec), maxvectors )
            if ( ivec<=maxvectors ) ivectors(ivec) = 1

         else if ( ivectors(ivec)==0 ) then

!        --- vector does not exist

            call errint ( ivec, 1 )
            call errchr ( names(ivec), 1 )
            call errwar ( 1886, 1, 0, 1 )
            call errcrd

         end if
      end if
      if ( icode==5 ) then

!     --- OUTPUT consider name of output files

         inpsep(istart+2) = 0
         if ( table1(5)>0 ) then
            inpsep(istart+2) = nextxt
            texts(nextxt) = strout(1)
            nextxt = nextxt+1
         end if

      end if
      istart = istart + 4

1000  call erclos ( 'readscstandard' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscstandard'

      end if  ! ( debug )

      end
