      subroutine solvematlab ( inpsol, rinsol, matr, &
                               isol, irhsd, intmat, kprob, iseqsol, eq )
! ======================================================================
!
!        programmer    Dragan Vidovic
!        version  1.0  date 14-03-2002
!
!   copyright (c) 1996-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Calls Matlab to solve the system of linear equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix
!     matlab
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematlab
      use sepmodulebuf
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsol(*), matr(*), isol(5,*), irhsd(*), intmat(5), &
              kprob(*), iseqsol, eq
      double precision rinsol(*)
!     eq             i    Choice parameter indicating which equation is to be
!                         solved:
!                         eq = 1, ..., ndim                   : momentum
!                         eq = ndim + 1                       : pressure
!                         eq = ndim + 2                       : enthalpy
!                         eq = ndim + 3, ..., ndim+1+ntrnsp   : transport eq.
!                         eq = ndim+2+ntrnsp,...,ndim+1+ntrans: turbulence eq.
!                         eq = ndim+1+ntrans+1                : density
!                         eq = ndim+1+ntrans+2                : velocity
!                         eq = ndim+1+ntrans+3,...,numsol     : additional
!                         ntrnsp  :  number of transport equations excluding
!                                   turbulence quantities
!                         ntrans  = ntrnsp + nturb
!                         numsol  = ndim + 1 + ntrans + nvcextra
!                         nvcextra: number of additional stored quantities
!     inpsol     i/o   Contains integer information concerning the solution
!                      process. This array may be filled by subroutine READ03,
!                      which reads information from the standard input file
!                      Array INPSOL must be filled as follows:
!                      Pos. 1  Contains the number of iterations at output
!                      Pos. 2  If an error has been occurred: 1 else 0
!                      Pos. 3  METHOD  Indicates the type of solver used
!                                    Possible values:
!                                    1: CG (conjugate gradients) or BiCGstab
!                                       In case of a non-symmetric matrix
!                                       Bi-CGSTAB is used
!                                    2: CGS is used
!                                    3: GMRES is used
!                                    4: GMRESR is used
!                      Pos. 4  IPRECO  Is used only for iterative methods
!                                    Control parameter concerning the type of
!                                    preconditioning
!                                    Possible values
!                                    0: No preconditioning.
!                                    1: The matrix is premultiplied by a
!                                       diagonal matrix, such that all diagonal
!                                        elements become 1. (Diagonal scaling)
!                                    2: Incomplete LU-decomposition using the
!                                       diagonal as only changed elements
!                                       Efficient Eisenstat implementation
!                                    3: Incomplete LU-decomposition
!                                    4: Symmetric Gauss-Seidel
!                                    5: Incomplete LU-decomposition using a pre-
!                                       assembled decomposition according to
!                                       a new method of Rik Kaasschieter
!                                    6: a standard (=default) preconditioner,
!                                    7: See 2, however, MILU is applied
!                                    8: Block SSOR
!                                    9: Block ILU
!                      Pos. 5  MAXITR  Maximum number of iterations to be
!                                      performed
!                      Pos. 6  IPRINT  Print level of subroutine
!                                    Possible values:
!                                   -1   No print output
!                                    0   Only error messages are printed
!                                    1   A little amount of information of the
!                                        iteration method is printed
!                                    2   The first and last iterations are
!                                        printed
!                                    3   A maximal amount of information of the
!                                        iteration method is printed
!                                    4   See 3, in this case also the matrix and
!                                        right-hand side are printed
!                      Pos. 7  MATSYM  Is used only when nonsymmetric storage
!                                    scheme jmetod=9,10,11,12 is used.
!                                    It tells that the matrix is symmetric, so
!                                    the CG methods can be used. Possible values
!                                    0: The matrix is supposed to be
!                                       nonsymmetric
!                                    1: The matrix is symmetric.
!                      Pos. 8  MGMRES  Dimension of Krylov space for GMRES
!                      Pos. 9  ISTART  Control parameter concerning the start of
!                                    the conjugate process. (ISTART).
!                                    Possibilities:
!                                    0: The process starts with vector 0
!                                    1: The process starts with an explicitly
!                                           given start vector.
!                                    2: The process starts with a random vector
!                      Pos. 10 KEEP  With this parameter the user may indicate
!                                    whether the preconditioning matrix must be
!                                    destroyed, kept or that an old
!                                    preconditioning matrix must be used.
!                                    Possibilities:
!                                    0 The preconditioning matrix is destroyed.
!                                    1 The preconditioning matrix is kept.
!                                      A reference is written in MATR(4).
!                                    2 The preconditioning matrix as computed
!                                      before is used.
!                                      It is supposed that the reference for
!                                      this matrix has been stored in MATR(4).
!                      Pos. 11 IRELER: Indicates which criterion should be used
!                                    for the termination of the process
!                                    Consists of two parts: IRELER_orig and
!                                    IABS according to
!                                    IRELER = IRELER_orig + 100 x IABS
!                                    Possible values for IRELER:
!                                    -1: The relative error is taken with
!                                        respect to ||b||.
!                                     0: The relative error is taken with
!                                        respect to ||x||.
!                                     1: The relative error is taken with
!                                        respect to ||res0||, where res0=initial
!                                        residual
!                                     2: The absolute error is taken
!                                    10: Use, default value, i.e. 1 except for
!                                        CG where 0 is used
!                                    Possible values for IABS:
!                                     0: No action
!                                     1: Except the error using IRELER_orig it
!                                        is also checked if the absolute error
!                                        is smaller than EPS2 (RINSOL(7))
!                                    Default: 10
!                      Pos. 12 IRESID: Indicates whether the residual must be
!                                    computed and printed after computation of
!                                    the solution. Possible values:
!                                    0:  Do not compute the residual
!                                    1:  Compute and print the 2-norm of the
!                                        residual
!                                    2:  Compute and print the 2-norm of the
!                                        residual divided by the 2-norm of the
!                                        right-hand side
!                      Pos. 13  NTRUNK: This parameter is only used if GMRESR
!                                    or CG is used.
!                                    In combination with GMRESR it indicates the
!                                    maximum number of search
!                                    directions in the GMRESR process. NTRUNK
!                                    should be much smaller than MGMRES
!                                    In combination with CG it indicates the
!                                    maximum number of iterations that are used
!                                    to estimate the smallest eigenvalue
!                      Pos. 14 NINNER: This parameter is only used if GMRESR is
!                                    used. It indicates the maximum number of
!                                    iterations in the inner loop of the GMRESR
!                                    process.
!                      Pos. 15 LAMBDA: indicates if a minimum
!                                    eigenvalue is given that is used in the
!                                    termination test (1) or not (0)
!                      Pos. 16 PROJ_PRINT_LEVEL:
!                                    Print level with respect to projection
!                                    Possible levels: -1, 0, 1, 2, where
!                                    -1 means no output, 0 standard output and
!                                    >0 extra output
!                      Pos. 17 PROJ_KEEP: Indicates if projection vectors must
!                                    be kept, destroyed or reused.
!                                    Possible values:
!                                    0: destroy
!                                    1: keep
!                                    2: use old vectors
!                      Pos. 18 PROJ_METHOD: projection method
!                                    Possible values:
!                                    0: none
!                                    1: projection based on approximate
!                                       eigenvectors
!                      Pos. 19       PROJ_PLOTVECTORS
!                                    indicates if the projection vectors must
!                                    be plotted
!                                    Possible values:
!                                    0: do not plot
!                                    1: make a contour plot
!                      Pos. 20       PROJ_TYPE
!                                    indicates how the projection vectors must
!                                    be computed
!                                    Possible values:
!                                    1: by solving a Poisson equation for the
!                                       shale layers
!                                    2: by setting shale layers equal to 1
!                                    3: by using a 0/1 projection vector for
!                                       each layer
!                      Pos. 21 not used, but also not free
!                      Pos. 22 AT_ERROR: Defines what to do in case of an error:
!                                    Possible values:
!                                    0: stop
!                                    1: return
!                      Pos. 23 ISEQ_EXACT: Sequence number of exact solution in
!                                    ISOL
!                                    If zero the exact solution is not given
!                      Pos. 24 ISEQ_RES_EXACT: Sequence number of the residual
!                                    of exact solution in ISOL
!                                    If zero the residual is not computed.
!                                    This position can only be used in
!                                    combination with ISEQ_EXACT
!                                    Default: 0
!                      Pos. 25 NPAIRS Number of pairs where the special boundary
!                                    conditions for the over-pressures must be
!                                    given
!                                    Default: 0
!                      Pos. 26 IPOSPAIRS Only used if NPAIRS>0
!                                    |IPOSPAIRS| defines the starting address of
!                                    array PAIRS in INPSOL.
!                                    If<0, user points are given, if>0
!                                    node numbers
!                                    This starting address must be at least 41
!                                    Default: 0
!                      Pos. 27 RPOSPAIRS Only used if NPAIRS>0
!                                    Defines the start of the threshold values
!                                    in RINSOL
!                                    Must be at least equal to 11
!                                    Default: 0
!                      Pos. 28 ISEQ_START_RES Contains position of start
!                                    residual in array ISOL
!                                    Default: 0
!                      Pos. 29 ISEQ_END_RES Contains position of end residual
!                                    in array ISOL
!                                    Default: 0
!                      Pos. 30 ISEQ_DIAG Contains position of diagonal of matrix
!                                    in array ISOL
!                                    Default: 0
!                      Pos. 31 ISEQ_PREC DIAG Contains position of diagonal of
!                                    preconditionings matrix in array ISOL
!                                    Default: 0
!                      Pos. 33 PRLUMP if 1 lumping of positive
!                                    off-diagonal elements
!                                    Default: 0
!                      Pos. 34 iscaling:  Choice of diagonal scaling.
!                                    Possible values:
!                                    0: no scaling
!                                    1: Only row scaling
!                                    2: Row and column scaling
!                                    3: Only column scaling
!                                    Default: 0
!                      Pos. IPOSPAIRS ... IPOSPAIRS+ 2 x NPAIRS-1
!                                    Array IPAIRS of size (2,NPAIRS)
!                                    contains the pairs of nodal points that
!                                    are connected. The lowest point with the
!                                    threshold pressure first the other one last
!     intmat      i    Standard SEPRAN array of length 5 containing information
!                      concerning the structure of the matrix. See the
!                      Programmer's Guide 13.5 for a complete description.
!     irhsd       i    Standard SEPRAN array, containing information of the
!                      large vector
!     iseqsol     i    Sequence number of solution array in array ISOL
!     isol       i/o   Standard SEPRAN array containing information concerning
!                      the solution vector. At input essential boundary
!                      conditions must have been filled in the solution vector.
!                      If ICONTR(2) = 1, also the start vector must have been
!                      filled in ISOL.
!                      At output the solution (or the last iteration) has been
!                      stored in ISOL.
!     kprob       i    Standard SEPRAN array, containing information of
!                      the problem
!     matr        i    Standard SEPRAN array, containing information of the
!                      large matrix
!     rinsol      i    Array containing the real user input if iread=-1
!                      The storage of rinsol depends on iinsol.
!                      The following positions are used
!                      1. eps      Required accuracy for the iteration process
!                      2  droptol  adapt diagonal
!                      3. alpha    factor for MILU process
!                      4. alammin  Minimum eigenvalue for termination
!                      5.          Accuracy with respect to the computation of
!                                  the projection vectors
!                      6.          Threshold value. All elements in the
!                                  projection vector with absolute value less
!                                  than this value are ignored
!                      7. eps2     Required accuracy for the iteration process
!                                  See ireler
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipint1, ipamat, iprhsd, ipusol
      integer iprob, intmt1, jmetod, ipkprb, nusol, nbound, npboun, &
              nrusol, lengmt, ishift, icompl, &
              spmat, sr, irs, jcs, nzmax, rhsd, intrcl, &
              inisol, sol, matinpsol, matrinsol
      logical reamat
      character * 80 solvstr
      character * 1000 outbuffer
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     inisol         Initial solution passed to MATLAB linear solver
!     intmt1         first part of array intmat
!     intrcl         Length of a variable ( real resp. complex ) divided
!                    by the length of an integer.
!     ipamat         Starting position of array AMAT in array IBUFFR.
!                    Array AMAT contains the matrix to be "inverted".
!                    AMAT consists of 3 parts:
!                    Part 1 of length NRUSOL, contains the diagonal of the
!                    matrix
!                    Part 2 contains the lower diagonal elements in exactly the
!                    same sequence as is given in imat part 2
!                    Part 3 contains the transpose of the upper diagonal
!                    elements in exactly the same sequence as is given in
!                    imat part 2
!     ipint1         Starting position of first part of array intmat in
!                    array IBUFFR.
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     iprhsd         Starting position of array RHSD in array BUFFER.
!                    Array RHSD contains the right-hand side of the system of
!                    equations
!     iprob          Actual problem number
!     ipusol         Starting address of solution array in BUFFER
!     irs            Pointer to ir array, part of a sparse matrix. See
!                    MATLAB External Interfaces/API manual.
!     ishift         Shift from the lower to the upper triangular part
!                    in amat(*)
!     jcs            Pointer to jc array, part of a sparse matrix. See
!                    MATLAB External Interfaces/API manual.
!     jmetod         Indicates type of storage scheme for the matrix
!                    Possible values:
!                     1:  Symmetric profile matrix
!                     2:  Non-symmetric profile matrix
!                     3:  Symmetric complex profile matrix
!                     4:  Non-symmetric complex profile matrix
!                     5:  Symmetric compact storage (Real matrix)
!                     6:  Non-symmetric compact storage (Real matrix)
!                     7:  Symmetric compact storage (Complex matrix)
!                     8:  Non-symmetric compact storage (Complex matrix)
!                     9:  Non-symmetric structure compact storage
!                         (Real matrix)
!                    10:  Non-symmetric structure compact storage
!                         (Complex matrix)
!                    11:  Compact storage for vector processors
!                         (Real matrix)
!                    12:  Compact storage for vector processors
!                         (Complex matrix)
!                    19:  The matrix is stored as a hermitian complex
!                         profile matrix
!                    20:  The matrix is stored as a symmetric real
!                         full matrix
!                    21:  The matrix is stored as a unsymmetric real
!                         full matrix
!                    22:  The matrix is stored as a symmetric complex
!                         full matrix
!                    23:  The matrix is stored as a unsymmetric complex
!                         full matrix
!     lengmt         Length of matrix
!     matinpsol      MATLAB version of inpsol array
!     matlabstrlen   Lenght of MATLAB solver string
!     matrinsol      MATLAB version of rinsol array
!     nbound         Number of essential boundary conditions
!     nmatlabeq      Maximal number of equations
!     npboun         Number of periodical boundary conditions
!     nrusol         Number of free unknowns
!     nusol          Total number of unknowns
!     nzmax          Size of MATLAB sparse matrix.  See
!                    MATLAB External Interfaces/API manual.
!     outbuffer      MATLAB output buffer
!     reamat         Indicates if the matrix  and vectors are real (true)
!                    or complex
!     rhsd           Right-hand-side MATLAB vector
!     sol            Solution MATLAB vector
!     solvstr        String containing MATLAB command that solves the system
!     spmat          MATLAB sparse matrix
!     sr             Pointer to pr array, part of a sparse matrix. See
!                    MATLAB External Interfaces/API manual.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget, inidgt
      integer inilen, iniprb, &
              engPutArray, engEvalString, engGetArray,engOutputBuffer, &
              mxGetPr, mxCreateSparse, mxGetIr, mxGetJc, &
              mxCreateDoubleMatrix, mxGetM, mxGetN
!     COPYINTTOPTR   Copies integer array to MATLAB array
!     ENGEVALSTRING  Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     ENGGETARRAY    Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     ENGOUTPUTBUFFER Opens MATLAB output buffer
!     ENGPUTARRAY    Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     INI060         Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INIDGT         Compute pointer of array in BUFFER
!     INIGET         Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
!     INIPRB         Get value of ipkprb as function of iprob
!     INSTOP         Stop the program
!     MXCOPYPTRTOREAL8 Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXCOPYREAL8TOPTR Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXCREATEDOUBLEMATRIX Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXCREATESPARSE Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXDESTROYARRAY Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXGETIR        Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXGETJC        Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXGETM         Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXGETN         Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXGETPR        Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     MXSETNAME      Function from MATLAB communication library. See
!                    MATLAB External Interfaces/API manual.
!     POPULATEMAT    Exports SEPRAN large matrix to MATLAB sparse matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2725   MATLAB function returned an error
!    2726   After evaluating MATLAB variable sol is still undefined.
!    2727   MATLAB command produces a solution of invalid size.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!matlab      call eropen ( 'solvematlab' )
!matlab      debug = .false.
!matlab
!matlab      if (ichar(matlabsolvstr(eq)(1:1))==0) then
!matlab         solvstr = 'sol=mat\\rhsd;'
!matlab      else
!matlab         solvstr = matlabsolvstr(eq)
!matlab      end if
!matlab
!matlab      iprob  = intmat(1) / 1000 + 1
!matlab      intmt1 = intmat(1) - 1000 * ( iprob - 1 )
!matlab      jmetod = mod ( intmt1, 100 )
!matlab      ipkprb = iniprb ( iprob, kprob )
!matlab      reamat = jmetod==5 .or. jmetod==6 .or. jmetod==9 .or. &
!matlab               jmetod==11
!matlab      nusol  = kprob(ipkprb+5)
!matlab      nbound = kprob(ipkprb+10)
!matlab      npboun = kprob(ipkprb+34)
!matlab      nrusol = kprob(ipkprb+66)
!matlab
!matlab      if ( reamat ) then
!matlab         icompl = 1
!matlab      else
!matlab         icompl = 2
!matlab      end if
!matlab      intrcl = icompl * intlen
!matlab
!matlab      call ini060 ( ibuffr, -intmat(3) )
!matlab      call ini060 ( ibuffr, -matr(3) )
!matlab      call ini060 ( ibuffr, -irhsd(1) )
!matlab
!matlab      if ( inpsol(9)==1 ) then
!matlab
!matlab!     --- start vector is given
!matlab
!matlab         call ini060 ( ibuffr, isol(1,iseqsol) )
!matlab
!matlab      else
!matlab
!matlab!     --- start vector is not given
!matlab
!matlab         if ( isol(2,iseqsol)/=110 .and. isol(2,iseqsol)/=116 .and. &
!matlab              isol(2,iseqsol)/=127 ) isol(1,iseqsol) = 0
!matlab         call ini081 ( ibuffr, isol(1,iseqsol), nusol*intrcl, 'usol' )
!matlab
!matlab      end if
!matlab
!matlab      lengmt = inilen(matr(3))
!matlab      ipamat = inidgt(matr(3))
!matlab
!matlab      ipusol = inidgt(isol(1,iseqsol))
!matlab      iprhsd = inidgt(irhsd(1))
!matlab
!matlab      ipint1 = iniget(intmat(3))
!matlab
!matlab      if ( jmetod==5 .or. jmetod==7 ) then
!matlab
!matlab!     --- symmetric matrix
!matlab
!matlab         ishift = 0
!matlab
!matlab      else if ( jmetod<30 ) then
!matlab
!matlab!     --- nonsymmetric matrix
!matlab
!matlab         ishift = ibuffr(ipint1+nrusol)
!matlab
!matlab      end if
!matlab
!matlab!     --- Create sparse matrix in MATLAB format
!matlab
!matlab      nzmax = lengmt/2
!matlab      spmat = mxCreateSparse(nrusol,nrusol,nzmax,0)
!matlab      call mxSetName(spmat, 'mat')
!matlab      sr = mxGetPr(spmat)
!matlab      irs = mxGetIr(spmat)
!matlab      jcs = mxGetJc(spmat)
!matlab
!matlab      call populatemat(nrusol, buffer(ipamat), buffer(ipamat+nrusol), &
!matlab                       buffer(ipamat+nrusol+ishift), ibuffr(ipint1), &
!matlab                       ibuffr(ipint1+nrusol+1), %val(sr), %val(irs), &
!matlab                       %val(jcs))
!matlab
!matlab!     --- Place the sparse matrix into the MATLAB workspace
!matlab
!matlab      if (engPutArray(matlabeng, spmat)/=0) then
!matlab         call errchr ( 'engPutArray', 1 )
!matlab         call errsub ( 2725, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab
!matlab!     --- Create rhsd in MATLAB format
!matlab
!matlab      rhsd = mxCreateDoubleMatrix(nrusol, 1, 0)
!matlab      call mxSetName(rhsd, 'rhsd')
!matlab      call mxCopyReal8ToPtr(buffer(iprhsd), mxGetPr(rhsd), nrusol)
!matlab
!matlab!     --- Place rhsd into the MATLAB workspace
!matlab
!matlab      if (engPutArray(matlabeng, rhsd)/=0) then
!matlab         call errchr ( 'engPutArray', 1 )
!matlab         call errsub ( 2725, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab
!matlab!     --- Create the initial solution in MATLAB format
!matlab
!matlab      inisol = mxCreateDoubleMatrix(nrusol, 1, 0)
!matlab      call mxSetName(inisol, 'inisol')
!matlab      call mxCopyReal8ToPtr(buffer(ipusol), mxGetPr(inisol), nrusol)
!matlab
!matlab!     --- Place the initial solution into the MATLAB workspace
!matlab
!matlab      if (engPutArray(matlabeng, inisol)/=0) then
!matlab         call errchr ( 'engPutArray', 1 )
!matlab         call errsub ( 2725, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab
!matlab!     --- Create the inpsol array in MATLAB format
!matlab
!matlab      matinpsol = mxCreateDoubleMatrix(34, 1, 0)
!matlab      call mxSetName(matinpsol, 'inpsol')
!matlab      call copyinttoptr(inpsol, %val(mxGetPr(matinpsol)), 34)
!matlab
!matlab!     --- Place inpsol into the MATLAB workspace
!matlab
!matlab      if (engPutArray(matlabeng, matinpsol)/=0) then
!matlab         call errchr ( 'engPutArray', 1 )
!matlab         call errsub ( 2725, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab
!matlab!     --- Create the rinsol array in MATLAB format
!matlab
!matlab      matrinsol = mxCreateDoubleMatrix(7, 1, 0)
!matlab      call mxSetName(matrinsol, 'rinsol')
!matlab      call mxCopyReal8ToPtr(rinsol, mxGetPr(matrinsol), 7)
!matlab
!matlab!     --- Place rinsol into the MATLAB workspace
!matlab
!matlab      if (engPutArray(matlabeng, matrinsol)/=0) then
!matlab         call errchr ( 'engPutArray', 1 )
!matlab         call errsub ( 2725, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab
!matlab      if (engOutputBuffer(matlabeng, outbuffer)/=0) then
!matlab         call errchr ( 'engOutputBuffer', 1 )
!matlab         call errsub ( 2725, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab
!matlab!     --- Solve the linear system
!matlab
!matlab      if (engEvalString(matlabeng, solvstr)/=0) then
!matlab         call errchr ( 'engEvalString', 1 )
!matlab         call errsub ( 2725, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab
!matlab!     print the beginning of MATLAB output if there was any
!matlab
!matlab      if (outbuffer(1:1)=='>') then
!matlab         print*, '-------------Beginning of MATLAB output--------------'
!matlab         print*, outbuffer
!matlab         print*, '--------The rest of MATLAB output was skipped--------'
!matlab       end if
!matlab
!matlab      if (debug) then
!matlab
!matlab!     --- Save the relevant MATLAB matrices
!matlab
!matlab         if (engEvalString(matlabeng, 'save sol sol;')/=0) then
!matlab            call errchr ( 'engEvalString', 1 )
!matlab            call errsub ( 2725, 0, 0, 1 )
!matlab            call instop
!matlab         end if
!matlab         if (engEvalString(matlabeng, 'save inisol inisol;')/=0) &
!matlab            then
!matlab            call errchr ( 'engEvalString', 1 )
!matlab            call errsub ( 2725, 0, 0, 1 )
!matlab            call instop
!matlab         end if
!matlab         if (engEvalString(matlabeng, 'save inpsol inpsol;')/=0) &
!matlab            then
!matlab            call errchr ( 'engEvalString', 1 )
!matlab            call errsub ( 2725, 0, 0, 1 )
!matlab            call instop
!matlab         end if
!matlab         if (engEvalString(matlabeng, 'save rinsol rinsol;')/=0) &
!matlab            then
!matlab            call errchr ( 'engEvalString', 1 )
!matlab            call errsub ( 2725, 0, 0, 1 )
!matlab            call instop
!matlab         end if
!matlab         if (engEvalString(matlabeng, 'save rhsd rhsd;')/=0) then
!matlab            call errchr ( 'engEvalString', 1 )
!matlab            call errsub ( 2725, 0, 0, 1 )
!matlab            call instop
!matlab         end if
!matlab         if (engEvalString(matlabeng, 'save mat mat;')/=0) then
!matlab            call errchr ( 'engEvalString', 1 )
!matlab            call errsub ( 2725, 0, 0, 1 )
!matlab            call instop
!matlab         end if
!matlab      end if
!matlab
!matlab!     --- Copy the solution back from MATLAB
!matlab
!matlab      sol = engGetArray(matlabeng, 'sol')
!matlab
!matlab      if (sol==0) then
!matlab         call errchr ( solvstr, 1 )
!matlab         call errsub ( 2726, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab      if (mxGetM(sol)*mxGetN(sol)/=nrusol) then
!matlab         call errchr ( solvstr, 1 )
!matlab         call errsub ( 2727, 0, 0, 1 )
!matlab         call instop
!matlab      end if
!matlab      call mxCopyPtrToReal8(mxGetPr(sol), buffer(ipusol), nrusol)
!matlab
!matlab!     --- Free MATLAB arrays
!matlab
!matlab      call mxDestroyArray(spmat)
!matlab      call mxDestroyArray(rhsd)
!matlab      call mxDestroyArray(inisol)
!matlab      call mxDestroyArray(sol)
!matlab      call mxDestroyArray(matrinsol)
!matlab      call mxDestroyArray(matinpsol)
!matlab
!matlab      call erclos ( 'solvematlab' )
!matlab
      end
