/*
 * Copyright (c) 1999 Kees Lemmens, TU Delft
 *
 *
 * Date : June , 1999
 *
 * Modified : December 2005 : hack for 64 bit little endian binaries , KL
 *
 * Function : Replacement functions for the Fortran-Sepran routines that
 *            read a binary sepplot file. This is the reason why we use
 *            "call by reference" : in that case they can be seamlessly
 *            replaced by the Fortran routines if anyone would prefer to
 *            use these instead.
 *
 *            This file makes it possible to build a sepviewer without 
 *            having Sepran itself installed.

 * NOTE: Currently there is only support for Linux-type little-endian 
 *       binary plotfiles (both 32 and 64 bit), but adding other formats
 *       should be straightforward.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "plotfun.h"

#define ASCII    0
#define BINARY32 1  // Intel (little endian)
#define BINARY64 2  // AMD Athlon64 (little endian)

static FILE* input;
static char mode;

void initmd()
{
  /* just a dummy */
}

void start(int *start,int *irotat,int *ioutp,int *itime)
{
  /* just a dummy */
}

/* Next is used by sepview to obtain the path to the Motif UIL file.
 * If your UIL resides somewhere else you should change this function
 * accordingly or else Motif will complain.
 */

void gethom(char *pathname, int *pathlen)
{
   /*
    char tmp[MAXFILENAME];
    if((tmp = getcwd(),MAXFILENAME-1) != NULL)
    */
   
   char *tmp;
   if((tmp = getenv("SPHOME") ) != NULL)
   {
      *pathlen=strlen(tmp);
      strncpy(pathname,tmp,MAXFILENAME-1);
   }
}

void plrdcl(int *ios)
{
  *ios = fclose(input);
}

void plrdop(char *fileid, int *fillen, int *ios)
{
   int firstbyte,secondbyte;

   input = fopen(fileid,"r");

   if(input == NULL)
   {
      *ios = -1;
      return;
   }
   else
     *ios = 0;
   
   fread(&firstbyte,sizeof(int),1,input);
   
   switch(firstbyte)
   {
    case 0x00000018 :
      fread(&secondbyte,sizeof(int),1,input);
      switch(secondbyte)
      {
	 /* According to Guus Segal the first instruction can NEVER be a zero, so
	  * if this byte is zero it MUST be a 64-bit plotfile.
	  */
       case 0x00000000 :
	 mode = BINARY64;
	 /* fprintf(stderr,"64-bit binary little endian fileformat detected\n"); */
	 break;
       default :
	 mode = BINARY32;
	 /* fprintf(stderr,"32-bit binary little endian fileformat detected\n"); */
	 break;
      }
      break;
    case 0x31202020 :
      mode = ASCII;
      /* fprintf(stderr,"little endian ASCII fileformat detected\n"); */
      break;
    case 0x20202031 :
      mode = ASCII;
      /* fprintf(stderr,"big endian ASCII fileformat detected\n"); */
      break;
    default: 
      fprintf(stderr,"Unknown fileformat !\n");
      *ios = -1;
   }
   rewind(input);
}

void plrd01(int *command, float *x, float *y,float *a,float *h,
	     int *nchar, int *ios)
{
   int dummy[2];
   
   switch(mode)
   {
    case ASCII:
      fscanf(input,"%d%f%f%f%f%d",command,x,y,a,h,nchar);
      
      break;
    case BINARY32:
      fread(dummy,sizeof(int),1,input);   /* skip record start (0x18) */
      
      fread(command,sizeof(int),1,input);
      
      fread(x,sizeof(float),1,input);
      fread(y,sizeof(float),1,input);
      fread(a,sizeof(float),1,input);
      fread(h,sizeof(float),1,input);
      
      fread(nchar,sizeof(int),1,input);
      
      fread(dummy,sizeof(int),1,input);   /* skip record end (0x18) */
      break;
    case BINARY64:
      fread(dummy,sizeof(int),2,input);   /* skip record start (0x18 + 0x00) */
      
      fread(command,sizeof(int),1,input);
      
      fread(x,sizeof(float),1,input);
      fread(y,sizeof(float),1,input);
      fread(a,sizeof(float),1,input);
      fread(h,sizeof(float),1,input);
      
      fread(nchar,sizeof(int),1,input);
      
      fread(dummy,sizeof(int),2,input);   /* skip record end (0x18 + 0x00) */
      break;
    default :
      *ios = -1;
    return;
   }
   
   if(feof(input) || ferror(input))
     *ios = -1;
   else
     *ios = 0;
   
    /*fprintf(stderr,"%02d: %f %f %f %f , %d\n",*command,*x,*y,*a,*h,*nchar);*/
   
}

void plrd06(int *nchar,char * string, int *ios)
{
   int len[2];

   switch(mode)
   {
    case ASCII:
      while(fgetc(input)!='\n'); /* find begin of next line */
      fgets(string,*nchar+1,input);
      while(fgetc(input)!='\n'); /* find begin of next line */
      break;
    case BINARY32:
      fread(len,sizeof(int),1,input);
      fread(string,len[0],1,input);
      fread(len,sizeof(int),1,input);
      break;
    case BINARY64:
      fread(len,sizeof(int),2,input); /* len is now a 64-bit integer */
      fread(string,len[0],1,input);
      fread(len,sizeof(int),2,input);
      break;
    default :
      *ios = -1;
      return;
   }
   
   /* fprintf(stderr,"%s\n",string);
    */
   if(feof(input) || ferror(input))
     *ios = -1;
   else
     *ios = 0;
}

void plrd07(int *nchar,float polyg[], int *ios)
{
   int x, dummy[2];
   
   switch(mode)
   {
    case ASCII:
      for(x=0;x<*nchar;x++)
	fscanf(input,"%f%f",
	       &polyg[2*x  ],
	       &polyg[2*x+1]);
      break;
    case BINARY32:
      fread(dummy,sizeof(int),1,input);   /* skip array start (0x20) */

      for(x=0;x<*nchar;x++)
      {
	 fread(&polyg[2*x  ],sizeof(float),1,input);
	 fread(&polyg[2*x+1],sizeof(float),1,input);
      }
      
      fread(dummy,sizeof(int),1,input);   /* skip array end (0x20) */
      break;
    case BINARY64:
      
      fread(dummy,sizeof(int),2,input);   /* skip array start (0x20 + 0x00) */

      for(x=0;x<*nchar;x++)
      {
	 fread(&polyg[2*x  ],sizeof(float),1,input);
	 fread(&polyg[2*x+1],sizeof(float),1,input);
      }
      
      fread(dummy,sizeof(int),2,input);   /* skip array end (0x20 + 0x00) */
      break;
    default :
      *ios = -1;
      return;
   }

   if(feof(input) || ferror(input))
     *ios = -1;
   else
     *ios = 0;
}

