      subroutine nonlin ( itertp, maxitr, eps, iprint, iwrkit, workit, &
                          intmatlc, ichsys, iuser, user, kmeshlc, &
                          kproblc, isol, ielhlp )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  4.19 date 11-12-2017 New storage of intmat
!        version  4.18 date 29-05-2014 Extension of timebefore
!        version  4.17 date 05-12-2012 Long integers
!
!   copyright (c) 1984-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve set of non-linear equations
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eps
      integer itertp, maxitr, iprint, ichsys
      integer iwrkit(*), iuser(*), isol, ielhlp(*)
      integer, target :: intmatlc, kmeshlc, kproblc
      double precision user(*), workit(*)
!     eps     i    Local accuracy
!     ichsys  i    choice parameter for call of system
!     ielhlp  i    parameter for subroutine system
!     intmatlc i   Standard SEPRAN array of length 5 containing information
!                  concerning the structure of the matrix. See the Programmer's
!                  Guide 13.5 for a complete description.
!     iprint  i    iprint=0  only warnings/errors will be printed
!                  iprint=-1,1  succesful completion will be reported
!                  iprint=-2,2  difference with preceding iteration will
!                               be printed too
!                  iprint=-3,3  ratio of differences will be printed too
!                  (on error).and.(iprint<=0) stop
!     isol   i/o   Standard SEPRAN array containing information of the solution
!                  array
!     itertp  i    itertp=jtertp+irelat*10+iresid*100+isepar*1000
!                  jtertp=0 no iteration, only one iteration is carried out
!                  jtertp=1 standard iteration process with maxitr iterations
!                           and relaxation, absolute error criterion
!                  irelat=0  absolute difference with previous iteration
!                  irelat=1  relative difference with previous iteration,
!                            which must be<=eps
!                  iresid=0  no residual computation
!                  iresid=1  residual computation for previous iteration
!                            ||A isolol - irhsdlc || / || isolol ||<=eps
!                            without effect of boundary conditions
!                  isepar=0  no separate test of differences
!                  isepar=1  separate test of difference with previous
!                            iteration for each degree of freedom
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     iwrkit       Not yet used
!     kmeshlc i    Standard SEPRAN array containing information of the mesh
!     kproblc i    Standard SEPRAN array, containing information of the problem
!     maxitr  i    Maximum number of iterations
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     workit       Not yet used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i    , ichini, idum, iposlv, &
              jtime, jtertp, &
              kiter, lngth, iter, icompl
      double precision difnew, difn1, difn2, difold, dif0 , dum  , &
                       difhlp, a, b, p, q, &
                       resnew, resold, res0 , snorm, omegac
      integer isolit, isolol, matrix, irhsdlc, &
              iresid, ihelp, inorm
      logical compl, relatv, residu, separ, new
      type (intmatrix), pointer :: IM
      double precision :: timebefore(2)

!     IM             Refers to intmt(intmat)
!     a
!     b
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     dif0
!     difhlp
!     difn1
!     difn2
!     difnew
!     difold
!     dum            General double precision dummy variable
!     i              Counting variable
!     ichini
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     idum           General dummy variable
!     ihelp
!     inorm
!     iposlv
!     iresid
!     irhsdlc          Standard SEPRAN array, containing information of the large
!                    vector
!     isolit         Corresponds to the iteration vector.
!                    In the iteration vector the solution of
!                    one iteration for one equation is stored
!     isolol         Corresponds to the solution of the preceding iteration
!     iter          Iteration sequence number
!     jtertp
!     jtime
!     kiter
!     lngth
!     matrix
!     new
!     omegac        Relaxation parameter
!     p
!     q
!     relatv
!     res0
!     residu
!     resnew
!     resold
!     separ
!     snorm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision sepnorm

!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     FILNLN         ?
!     INSTOP         Stop the program
!     PRINTTIME      Print the CPU time at end of subroutine
!     PRINTV         Print solution vector or vector of special structure
!     SEPALGEBR      algebraical operations on SEPRAN vectors
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
!     SEPLINSOL      Body of linear solver
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPMAVER       perform matrix vector multiplication
!     SEPNORM        compute norm of vector or norm of difference of two vectors
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMSOLINFO   Remove array with sequence number ivectlc from ks
!     SEPSOLDIR      Actual body of subroutine SOLVE
!     SYSTM0         assemble a system of equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     239   Type of iteration is not available
!     240   Parameter eps is not positive
!     241   Relaxation parameter has wrong value
!    1562   Divergence of iteration
!    1563   Divergence of iteration
!    1564   Too many iterations
!    1565   Too many iterations
!    1566   Too many iterations
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data omegac/1.0d0/
      data dif0, difold, res0, resold / 2d16, 1d16, 2d16, 1d16 /
! ======================================================================
!
      call eropen ( 'nonlin' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from nonlin'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dum = 0d0
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      kprob2d(1) = kproblc
      kprob => kprob2d(1)

!     --- analyze itertp

      jtertp=itertp
      separ=.false.
      residu=.false.
      relatv=.false.
      if (jtertp>10000) then
         kiter=jtertp/10000
         jtertp=jtertp-kiter*10000
      else
         kiter=5
      end if
      if (jtertp>1000) then
         separ=.true.
         jtertp=jtertp-1000
      end if
      if (jtertp>100) then
         residu=.true.
         jtertp=jtertp-100
      end if
      if (jtertp>10) then
         relatv=.true.
         jtertp=jtertp-10
      end if
      if ((jtertp<0).or.(jtertp>1)) then
         call errsub ( 239, 0, 0, 0 )
         call instop
      end if
      if ( eps<=0 .and. jtertp/=0 ) then
         call erreal ( eps, 1 )
         call errsub ( 240, 0, 1, 0 )
         call instop
      end if

      intmat => intmatlc

      call sepfillmatstr

!     --- initialize extra arrays

      isolol=0
      isolit=0
      matrix=0
      irhsdlc=0
      iresid=0
      ihelp=0
      inorm=0
      jtime=itime
      itime=0
      IM => intmt(intmat)
      inpsol(1,1) = 2
      if (IM%symmetric) then
         iposlv=1
      else
         iposlv=0
      end if
      iter=0
      compl=IM%complexm

!     --- step 1 : (re)fill iuser and user with coefficients and parameters
!             (and essential boundary conditions in isol)

100   call filnln(iter,iuser,user,kmesh,kprob,isol,omegac,difold)

!     --- step 2 : form isolit for iteration purposes (from isol)

      if (iter==0) then

!     --- if isol did not exist create it

         new = isol<=0 .or. isol>lastsol
         if ( .not. new ) new = solarr(isol)==0

         if ( new ) then
            if (.not.compl) then
               ichini=2
               icompl=0
            else
               ichini=3
               icompl=1
            end if
            call sepcreasol110 ( isol, 'isolit', icompl )
            usoltemp = 0d0

         end if

!        --- create zero'th iterand

         call sepcopyvc ( isol,isolit )

      else

!     --- create next iterand by relaxation, if required

         if ((abs(omegac)<epsmac).or.(abs(omegac-1d0)<epsmac)) then
            call sepcopyvc ( isol,isolit )
         else
            if ((omegac<0d0).or.(omegac>2d0)) then
               call errsub ( 241, 0, 0, 0 )
               call instop
            end if
            if (.not.compl) then
               b=1d0-omegac
               call sepalgebr (3,0,isol,isolol,isolit, &
                               omegac,b,dum,dum,idum)
            else
               a=1d0-omegac
               b=1d0-omegac
               call sepalgebr (3,0,isol,isolol,isolit, &
                               omegac,omegac,a, b, idum)
            end if
         end if
      end if

!     --- copy (new) iterand into solold and transform iterand back

      call sepcopyvc ( isolit,isolol )
      inpsol(2,1) = iposlv+20
      call sepsoldir ( matrix, isolit, irhsdlc )

      if ( debug ) call printv ( isolit, kmesh, kprob, 'isolit' )

!     --- step 3 : build system and solve

      call systm0( ichsys,matrix,intmatlc,kmesh,kprob, &
                   irhsdlc,isol, iuser,user,isolit,ielhlp)
      if ((iter==0).and.(iprint/=0).and.(jtime==1)) &
         call printtime('system', timebefore)

!     --- step 3a : determine residual for previous iteration

      if ( debug ) then
         write(irefwr,*) 'residu ', residu
         write(irefwr,1) 'iter', iter
      end if  ! ( debug )

      if (residu .and. iter>0) then
         call sepmaver ( matrix,isolol,ihelp,2 )

!        --- substract irhsdlc from ihelp

         if (.not.compl) then
            a=1d0
            b=-1d0
            call sepalgebr (3,0,ihelp,irhsdlc,iresid, &
                            a, b, dum,dum,idum)
         else
            a=1d0
            b=1d0
            p=-1d0
            q=-1d0
            call sepalgebr (3,0,ihelp,irhsdlc,iresid, &
                            a, b, p, q, idum)
         end if

!        --- determine the norm of the residual

         call sepalgebr (7,0,isolol,isolol,isolol, &
                         snorm,dum,dum,dum,idum)
         snorm=sqrt(snorm)

!        --- for the residual exclude the boundary conditions

         call sepalgebr (10,0,iresid,iresid,iresid, &
                         resnew,dum,dum,dum,idum)
         resnew=sqrt(resnew)
         resnew=resnew/snorm
         if ((iter==0).and.(iprint/=0).and.(jtime==1)) &
            call printtime('maver', timebefore)
      end if

!     --- Solve system of equations without backtransformation

      inpsol(2,1) = iposlv+10
      if ( debug ) call printv ( irhsdlc, kmesh, kprob, 'irhsdlc' )
      iseqsol = 1
      call seplinsol ( matrix, isol, irhsdlc )
      if ((iter==0).and.(iprint/=0).and.(jtime==1)) &
         call printtime('solve ', timebefore)
      if (jtertp<=0) goto 900
      if ( debug ) call printv ( isol, kmesh, kprob, 'isol' )

!     --- step 4 : determine difference with old solution

      iter=iter+1
      if (.not.separ) then
         difnew = sepnorm ( 0, 3, 0, isolol, isol, idum )
         if (relatv) then

!        --- determine minimum and maximum

            if (.not.compl) then
               call sepalgebr (6,0,isol,isol,isol, &
                               difn1,difn2,dum,dum,idum)
            else
               call sepalgebr (8,0,isol,inorm,inorm, &
                               1.0d0,dum,dum,dum,idum)
               call sepalgebr (6,0,inorm,inorm,inorm, &
                               difn1,difn2,dum,dum,idum)
            end if
            difnew=difnew/max(abs(difn1),difn2)
         end if
      else if (separ) then

!     --- determine difference vector and difference per degree of freedom

         if (.not.compl) then
            a=1d0
            b=-1d0
            call sepalgebr (3,0,isol,isolol,ihelp, &
                            a, b, dum,dum,idum)
         else
            a=1d0
            b=1d0
            p=-1d0
            q=-1d0
            call sepalgebr (3,0,isol,isolol,ihelp, &
                            a, b, p, q, idum)
         end if
         difnew=0.0d0
         do i=1,max(kprobloc(4),kprobloc(33))
            if (.not.compl) then
               call sepalgebr (6,i,ihelp,ihelp,ihelp, &
                               difn1,difn2,dum,dum,idum)
            else
               a=1d0
               call sepalgebr (8,i,ihelp,inorm,inorm, &
                               a,dum,dum,dum,idum)
               call sepalgebr (6,i,inorm,inorm,inorm, &
                               difn1,difn2,dum,dum,idum)
            end if
            difhlp=max(abs(difn1),difn2)
            if (relatv) then

!           --- determine minimum and maximum

               if (.not.compl) then
                  call sepalgebr (6,i,isol,isol,isol, &
                                  difn1,difn2,dum,dum,idum)
               else
                  a=1d0
                  call sepalgebr (8,i,isol,inorm,inorm, &
                                  a,dum,dum,dum,idum)
                  call sepalgebr (6,i,inorm,inorm,inorm, &
                                  difn1,difn2,dum,dum,idum)
               end if
               dum=max(abs(difn1),difn2)
               if (dum>sqreps) difhlp=difhlp/dum
            end if
            if (difnew<difhlp) difnew=difhlp
         end do
      end if
      if (abs(iprint)>=2) then
         write(irefwr,200) iter,difnew
200      format(' nonlin : iter=',i5,'  new difference =',ss,1pe12.3e2)
         if (iter>1) then
            if ( difnew<epsmac .and. difold<epsmac ) then
               difn2=1.0d0
            else if (difold>epsmac) then
               difn2=difnew/difold
            else
               difn2=difnew
            end if
            if (abs(iprint)>=3) write(irefwr,210) difn2
210         format(21x,'   difnew/difold =',ss,1pe12.3e2)
            if (residu) then
               if (iter>1) then
                  write(irefwr,220) resnew
220               format(21x,'  (old) residual =',ss,1pe12.3e2)
               end if
               if ( resnew<epsmac .and. resold<epsmac ) then
                  difn2=1.0d0
               else if (resold>epsmac) then
                  difn2=resnew/resold
               else
                  difn2=resnew
               end if
               if (abs(iprint)>=3 .and. iter>2) &
                  write(irefwr,230) difn2
230               format(21x,'   resnew/resold =',ss,1pe12.3e2)
            end if
         end if
      end if
      if (iter==1) then
         difold=difnew
         goto 100
      end if

!     --- step 5 : test whether iteration process diverges or converges
!                   for residu BOTH tests must be satisfied together

      if ((.not.residu .and. difnew>=difold) .or. &
          ( residu .and.(difnew>=difold .and. resnew>=resold))) &
          then

!     --- prevent that superfluous iteration is performed

         if (iter<kiter) then
            if ((.not.residu .and. difnew<=eps) .or. &
                ( residu .and. difnew<=eps .and. resnew<=eps)) then
               goto 900
            else

!           --- divergence is allowed for iter<kiter  or one step

               dif0=difold
               difold=difnew
               res0=resold
               resold=resnew
               goto 100
            end if
         else if (iter<maxitr) then
            if ((.not.residu .and. difold<dif0 ).or. &
                ( residu .and. difold<dif0 .and. resold<res0)) then

!           --- divergence not repeated for iter>=kiter

               dif0=difold
               difold=difnew
               res0=resold
               resold=resnew
               goto 100
            else

!           --- (repetitive) divergence of solution (iter>=kiter)

               if (iprint<0) then
                  eps=difnew
                  maxitr=-iter
               end if
               call errint ( iter, 1 )
               call erreal ( dif0, 1 )
               call erreal ( difold, 2 )
               call erreal ( difnew, 3 )
               if ( residu ) then
                  call erreal ( res0, 4 )
                  call erreal ( resold, 5 )
                  call erreal ( resnew, 6 )
                  if ( iprint<0 ) then
                     call errwar ( 1563, 1, 6, 0 )
                  else
                     call errsub ( 1563, 1, 6, 0 )
                  end if
               else if ( iprint<0 ) then
                  call errwar ( 1562, 1, 3, 0 )
               else
                  call errsub ( 1562, 1, 3, 0 )
               end if
               if (iprint>=0) call instop
               goto 900
            end if
         else

!        --- too many iterations

            if (iprint<0) then
               maxitr=-iter
               eps=difnew
            end if
            call errint ( abs(iter), 1 )
            call erreal ( difnew, 1 )
            call erreal ( resnew, 2 )
            if ( residu .and. iprint>=0 ) then
               call errsub ( 1566, 1, 2, 0 )
            else if ( residu .and. iprint<0 ) then
               call errwar ( 1566, 1, 2, 0 )
            else if ( iprint>0 ) then
               call errsub ( 1565, 1, 1, 0 )
            else if ( iprint<0 ) then
               call errwar ( 1565, 1, 1, 0 )
            else
               call errsub ( 1564, 1, 0, 0 )
            end if
            if (iprint>=0) call instop
            goto 900
         end if

!        --- step 6 : test whether truncation criterium has been reached or not

      else if ((.not.residu .and. difnew<=eps ).or. &
               ( residu .and. difnew<=eps .and. resnew<=eps)) then
         if (abs(iprint)>=1) &
               write(irefwr,'('' nonlin : criteria have been met'')')
      else

!     --- other cases (not yet completed)

         if (iter>=maxitr) then

!        --- maximum number of iterations not sufficient

            if (iprint<0) then
               maxitr=-iter
               eps=difnew
            end if
            call errint ( abs(iter), 1 )
            call erreal ( difnew, 1 )
            call erreal ( resnew, 2 )
            if ( residu .and. iprint>=0 ) then
               call errsub ( 1566, 1, 2, 0 )
            else if ( residu .and. iprint<0 ) then
               call errwar ( 1566, 1, 2, 0 )
            else if ( iprint>0 ) then
               call errsub ( 1565, 1, 1, 0 )
            else if ( iprint<0 ) then
               call errwar ( 1565, 1, 1, 0 )
            else
               call errsub ( 1564, 1, 0, 0 )
            end if
            if (iprint>=0) call instop
            goto 900
         else
            dif0=difold
            difold=difnew
            res0=resold
            resold=resnew
            goto 100
         end if
      end if
 900  if (abs(iprint)>=1) write(irefwr,1010) iter
1010  format (' nonlin : iteration completed after', i5,'  iterations')

!     --- Carry out the backtransformation process:

  !    inpsol(2,1) = iposlv+20
  !    call sepsoldir ( matrix, isol, irhsdlc )
      call seploctrn ( 2, isol )

!     --- step 7 : remove matrix, isolol, isolit, irhsdlc,inorm by resetting
!                  of ibfree and iinfor

      call seprmsolinfo ( isolit )
      call seprmsolinfo ( isolol )
      call seprmmatrix ( matrix )
      call seprmsolinfo ( irhsdlc )
      call seprmsolinfo ( ihelp )
      if ( residu ) call seprmsolinfo ( iresid )
      if (compl) call seprmsolinfo ( inorm )
      itime=jtime

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End nonlin'

      end if  ! ( debug )
      call erclmn ( 'nonlin', timebefore )

      end subroutine nonlin
