      subroutine sepchancf ( iuserloc, userloc, kproblc, kmeshlc, iinfil )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Change coefficients for the standard element subroutines
!   in the arrays IUSER and USER.
!   It is supposed that the arrays IUSER and USER have been filled before
!   either by hand, by FIL103/4 or FIL153/4 or CHNCOF.
!   This subroutine may only change the coefficients, so the coefficients
!   to be changed must have been filled before!
!    CHANCF replaces one or several calls of the SEPRAN subroutines:
!           FIL103: Fill coefficients for one element group (real case)
!           FIL104: Fill coefficients for one boundary element group
!                   (real case)
!           FIL153: Fill coefficients for one element group (real case)
!           FIL154: Fill coefficients for one boundary element group
!                   (complex case)
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinfil(*)
      integer, target :: kmeshlc, kproblc, iuserloc(*)
      double precision, target :: userloc(*)


!     iinfil  i    Input array defined by the user
!                  With this array the user may indicate which actions
!                  he wants to be performed by this subroutine
!                  IINFIL must be filled as follows:
!                  Pos. 1: Last entry number that has been filled by the
!                          user. If 0 or 1 only defaults are used
!                       2: IREAD Defines how the input must be read
!                                Possible values:
!                                0:  All SEPRAN input has been read by
!                                    subroutine SEPSTN until END_OF_SEPRAN_INPUT
!                                    or end of file has been found
!                                    This input is used
!                                1:  The input is read as described for CHNCOF
!                          Default value: 0
!                       3: ISEQNR, This parameter is only used if IREAD=0
!                          It indicates the sequence number of the input
!                          for essential boundary conditions to be used
!                          Default value: 1
!                       4: IPROB, This parameter is only used if IREAD=1
!                          Problem number
!                          Default value: 1
!     iuserloc i/o Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!                  The first 5 positions of IUSER are filled as follows:
!                  Pos. 1    Filled by the user with the declared length
!                  Pos. 2    Contains the number of element groups stored
!                  Pos. 3    Computed length of IUSER
!                  Pos. 4    Last position used in IUSER
!                  Pos. 5    Indicates if the problem is real (0) or complex (1)
!     kmeshlc i    Standard SEPRAN array containing information of the mesh
!     kproblc i    Standard SEPRAN array, containing information of the problem
!     userloc i/o  Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!                  The first 5 positions of USER are filled as follows:
!                  Pos. 1    Filled by the user with the declared length
!                  Pos. 2    -
!                  Pos. 3    Computed length of USER
!                  Pos. 4    Last position used in USER
!                  Pos. 5    -
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iread, iseqnr, isubr, iprob

!     iprob   Problem number
!     iread   Defines how the input must be read
!             Possible values:
!             0:  All SEPRAN input has been read by
!                 subroutine SEPSTN until END_OF_SEPRAN_INPUT
!                 or end of file has been found this input is used
!             1:  The input is read as described for CHNCOF
!     iseqnr  Sequence number with respect to the storage in array ISEPIN
!     isubr   Indicates the type of calling subroutine
!             Possible values
!             1:  CHNCOF
!             2:  CHNCOF
!             3:  CHANCF
!             4:  CHANCF
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TO0093  Body of subroutines CHNCOF and CHNCOF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepchancf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepchancf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      iuser => iuserloc(1:iuserloc(1))
      user => userloc(1:nint(userloc(1)))

      kprob2d(1) = kproblc
      kprob => kprob2d(1)
      call sepfillkprob
      iread  = 0
      iseqnr = 1
      iprob  = 1
      if ( iinfil(1)>=2 ) iread  = iinfil(2)
      if ( iinfil(1)>=3 ) iseqnr = iinfil(3)
      if ( iinfil(1)>=4 ) iprob  = iinfil(4)
      if ( iread==0 ) then
         isubr = 4
      else
         isubr = 2
      end if
      call to0093 ( isubr, iprob, iseqnr )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepchancf'

      end if  ! ( debug )
      call erclos ( 'sepchancf' )

      end subroutine sepchancf
