      subroutine seppetscvecs ( rhsd, nrusol, kprobhglob, nrusolglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create petsc vector from sepran vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecomio
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nrusol, kprobhglob(*), nrusolglob
      double precision, intent(in) :: rhsd(nrusol)

!     kprobhglob     i    Array kprobh with global numbers of free degrees
!                         of freedom (length nrusol)
!     nrusol         i    Number of free unknowns
!     nrusolglob     i    Global number of unknowns
!     rhsd           i    Right-hand-side vector
!                         Renumbered with all free unknowns from position 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, rowindex(nrusol), nghosts, ghosts(1)

!     ghosts         dummy
!     i              Counting variable
!     nghosts        Number of ghost points on this cpu
!     rowindex       Contains sequence numbers of unknowns to be added
!                    to right-hand side in petsc style, i.e. counted
!                    from position 0 (c style)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     VECASSEMBLYBEGIN  Begins assembling the vector.
!                    This routine should be called after completing
!                    all calls to VecSetValues()
!     VECASSEMBLYEND Completes assembling the vector.
!                    This routine should be called after VecAssemblyBegin()
!     VECCREATE      Creates an empty vector object.
!                    The type can then be set with VecSetType(), &
!                    or VecSetFromOptions()
!     VECCREATEGHOST Creates a parallel vector with ghost padding on
!                    each processor.
!     VECSETSIZES    Sets the local and global sizes, and checks to determine
!                    compatibility
!     VECSETVALUES   Inserts or adds values into certain locations of a vector
!     VECVIEW        Views a vector object.
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscvecs' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscvecs'
         write(irefwr,1) 'nrusolglob', nrusolglob
         write(irefwr,*) 'parallel', parallel
         call prinrl ( rhsd, nrusol, 'rhsd' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create right-hand-side vector and set size

      if ( parallel ) then

!     --- Parallel case
!         In order to find the ghost values in the solution easily we
!         use VecCreateGhost instead of VecCreate

         nghosts = 0
         call VecCreateGhost ( PETSC_COMM_WORLD, nrusol, &
                               nrusolglob, nghosts, ghosts, b, &
                               ierrpetsc )  ! create rhs vector

!        --- Fill array rowindex with petsc numbering

         do i = 1, nrusol
            rowindex(i) = kprobhglob(i)-1
         end do  ! i = 1, nrusol

      else

!     --- Serial case

         call VecCreate ( PETSC_COMM_WORLD, b, ierrpetsc )  ! create rhs
         call VecSetSizes ( b, PETSC_DECIDE, nrusolglob, ierrpetsc )  ! set size
         call VecSetFromOptions ( b, ierrpetsc )   ! defines the standard type

!        --- Fill array rowindex with petsc numbering

         do i = 1, nrusol
            rowindex(i) = i-1
         end do  ! i = 1, nrusol

      end if  ! ( parallel )

      if ( debug ) call prinin ( rowindex, nrusol, 'rowindex' )

!     --- Add vector rhsd to b

      call VecSetValues ( b, nrusol, rowindex, rhsd, ADD_VALUES, &
                          ierrpetsc )

!     --- Actual assembling

      call VecAssemblyBegin ( b, ierrpetsc )
      call VecAssemblyEnd ( b, ierrpetsc )

1000  if ( debug ) then

!     --- Debug information

         call VecView ( b, PETSC_VIEWER_STDOUT_WORLD, ierrpetsc )
         write(irefwr,*) 'End seppetscvecs'

      end if  ! ( debug )
      call erclos ( 'seppetscvecs' )

      end subroutine seppetscvecs
